         TITLE  'AMDPRECT - EXIT CONTROL TABLE'
AMDPRECT CSECT
*A-000000-999999                                                @Y02006
* /* START OF SPECIFICATIONS ****
*
*01*  MODULE-NAME = AMDPRECT
*
*02*     CSECT-NAME = AMDPRECT
*
*01*  DESCRIPTIVE-NAME = AMDPRDMP EXIT CONTROL TABLE
*
*01*  COPYRIGHT = NONE
*01*  STATUS = VS2 RELEASE 02.0
*
*01*  FUNCTION = THE FUNCTION OF THE EXIT CONTROL TABLE IS TO INDICATE
*     TO THE EXIT INTERFACE ROUTINE (AMDPRUIM) WHICH EXIT MODULES ARE
*     TO GET CONTROL UPON EACH OF THE EXITS FROM AMDPRDMP.  THE TABLE
*     ALSO INDICATES WHICH EXIT MODULE IS TO GET CONTROL UPON
*     ENCOUNTERING AN EXIT VERB.
*
*02*     OPERATION = THE EXIT CONTROL TABLE CONTAINS A 20 BYTE ENTRY
*        FOR EACH EXIT MODULE THAT IS TO RECEIVE CONTROL ON ONE OF
*        AMDPRDMP'S EXITS OR UPON AMDPRDMPS ENCOUNTERING OF AN EXIT
*        VERB.  THE TABLE INITIALLY IS SET UP TO CONTAIN 20 ENTRIES,
*        BUT MAY BE EXPANDED TO ANY LENGTH, AS REQUIRED.  EACH ENTRY
*        CONTAINS THE FOLLOWING.
*
*             MODULE NAME - NAME OF THE EXIT MODULE (PADDED ON RIGHT
*                           WITH BLANKS) THAT SHOULD BE LOADED AND
*                           GIVEN CONTROL.
*
*             EXIT FLAG -   A ONE BYTE FIELD INDICATING UPON WHICH
*                           AMDPRDMP EXIT THIS MODULE SHOULD GET
*                           CONTROL.  ANY COMBINATION OF THE
*                           FOLLOWING MAY OCCUR:
*
*                           X'80'  EACH TCB
*                           X'40'  EACH ASID
*                           X'20'  FORMAT STATEMENT
*                           X'10'  PRINT CURRENT/JOBNAME
*                           X'08'  PRINT NUCLEUS
*
*             VERB NAME -   NAME OF THE VERB (PADDED ON RIGHT WITH
*                           BLANKS) UPON WHICH THE EXIT MODULE
*                           SHOULD GET CONTROL.
*
*        THE TABLE IS SEARCHED BY MODULE AMDPRUIM EACH TIME AN
*        AMDPRDMP EXIT OCCURS AND CONTROL IS GIVEN TO THE
*        APPROPRIATE EXIT MODULES IN THE ORDER ENCOUNTERED
*        IN THE TABLE.  BOTH EXIT FLAGS AND A VERB NAME MAY BE
*        SPECIFIED IN THE SAME ECT ENTRY.
*
*01*   NOTES =
*
*02*     DEPENDENCIES = NONE
*
*03*        CHARACTER-CODE-DEPENDENCIES = NONE THAT CANNOT BE RESOLVED
*           BY RE-COMPILATION.
*
*02*     RESTRICTIONS = NONE
*
*02*     REGISTER-CONVENTIONS = N/A
*
*02*     PATCH-LABEL = N/A
*
*01*  MODULE-TYPE = TABLE
*
*02*     PROCESSOR = ASSEMBLER
*
*02*     MODULE-SIZE = 0.4K BYTES
*
*02*     ATTRIBUTES = PROBLEM PROGRAM REGION; NOT EXECUTABLE.
*
*01*  ENTRY-POINT = N/A
*
*01*  EXIT-NORMAL = N/A
*
*01*  EXIT-ERROR = N/A
*
*01*  EXTERNAL-REFERENCES = N/A
*
*01*  TABLES = N/A
*
*01*  MACROS = N/A
*
*01*  CHANGE-ACTIVITY = Y02006
*
*01*  MESSAGES = N/A
*
*01*  ABEND-CODES = N/A
*
**** END OF SPECIFICATIONS ***/
*
***********************************************************************
*                                                                     *
*                        EXIT FLAG DEFINITIONS                        *
*                                                                     *
***********************************************************************
*
ATCB     EQU   X'80'                EXIT GETS CONTROL EACH TIME
*                                   AMDPRDMP PROCESSES TCB ASSOCIATED
*                                   CONTROL BLOCKS
BASID    EQU   X'40'                EXIT GETS CONTROL EACH TIME
*                                   AMDPRDMP PROCESSES ASID ASSOCIATED
*                                   CONTROL BLOCKS
CFORMAT  EQU   X'20'                EXIT GETS CONTROL EACH TIME
*                                   AMDPRDMP PROCESSES FORMAT CONTROL
*                                   STATEMENT
DCURJOB  EQU   X'10'                EXIT GETS CONTROL EACH TIME
*                                   AMDPRDMP PROCESSES PRINT CURRENT OR
*                                   PRINT JOBNAME CONTROL STATEMENT
ENUCLEUS EQU   X'08'                EXIT GETS CONTROL EACH TIME
*                                   AMDPRDMP PROCESSES PRINT NUCLEUS
*                                   CONTROL STATEMENT
*
***********************************************************************
*                                                                     *
*                        EXIT CONTROL TABLE                           *
*                                                                     *
***********************************************************************
*
ECTNTR01 DS    0CL20                EXIT CONTROL TABLE ENTRY 01
*                                   FOR FORMATTING QUEUE CONTROL BLKS
         DC    CL8'AMDPRQCB'        EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8'QCBTRACE'        VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR02 DS    0CL20                EXIT CONTROL TABLE ENTRY 02
*                                   FOR FORMATTING QUEUE CONTROL BLKS
         DC    CL8'AMDPRQCB'        EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8'Q'               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR03 DS    0CL20                EXIT CONTROL TABLE ENTRY 03
*                                   FOR FORMATTING THE CVT
         DC    CL8'AMDPRCVT'        EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8'CVTMAP'          VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR04 DS    0CL20                EXIT CONTROL TABLE ENTRY 04
*                                   FOR FORMATTING ASM CONTROL BLOCKS
         DC    CL8'ILRFMT00'        EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8'ASMDATA'         VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR05 DS    0CL20                EXIT CONTROL TABLE ENTRY 05
*                                   FOR FORMATTING ATCAM CONTROL BLOKS
         DC    CL8'ISTRAFD1'        EXIT MODULE NAME          @ZM32220
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8'VTAMMAP'         VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR06 DS    0CL20                EXIT CONTROL TABLE ENTRY 06
*                                  FOR PROCESSING SUMDUMP VERB @G33AFRM
         DC    CL8'IEAVTFSD'       EXIT MODULE NAME            @G33AFRM
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8'SUMDUMP'        VERB TO INVOKE EXIT         @G33AFRM
*                                   IF ANY
*
*
ECTNTR07 DS    0CL20                EXIT CONTROL TABLE ENTRY 07
*                                  FOR PROCESSING LOGDATA VERB @G33AFRM
         DC    CL8'IFCERFMT'       EXIT MODULE NAME            @G33AFRM
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8'LOGDATA '       VERB TO INVOKE EXIT         @G33AFRM
*                                   IF ANY
*
*
ECTNTR08 DS    0CL20                EXIT CONTROL TABLE ENTRY 08
*                                  FOR DATA MANAGEMENT FORMATER@G33AFRM
         DC    CL8'IECDAFMT'       EXIT MODULE NAME            @G33AFRM
         DC    AL1(128)            TCB EXIT FLAG               @G33AFRM
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR09 DS    0CL20                EXIT CONTROL TABLE ENTRY 09
*                                  FOR IOS DATA FORMATTER      @G33AFRM
         DC    CL8'IECIOFMT'       EXIT MODULE NAME            @G33AFRM
         DC    AL1(128)            TCB EXIT FLAG               @G33AFRM
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR10 DS    0CL20                EXIT CONTROL TABLE ENTRY 10
*                                  FOR RTM FORMATTER           @G33AFRM
         DC    CL8'IEAVTFMT'       EXIT MODULE NAME            @G33AFRM
         DC    AL1(128)            TCB EXIT FLAG               @G33AFRM
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR11 DS    0CL20                EXIT CONTROL TABLE ENTRY 11
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR12 DS    0CL20                EXIT CONTROL TABLE ENTRY 12
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR13 DS    0CL20                EXIT CONTROL TABLE ENTRY 13
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR14 DS    0CL20                EXIT CONTROL TABLE ENTRY 14
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR15 DS    0CL20                EXIT CONTROL TABLE ENTRY 15
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR16 DS    0CL20                EXIT CONTROL TABLE ENTRY 16
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR17 DS    0CL20                EXIT CONTROL TABLE ENTRY 17
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR18 DS    0CL20                EXIT CONTROL TABLE ENTRY 18
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR19 DS    0CL20                EXIT CONTROL TABLE ENTRY 19
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
*
*
ECTNTR20 DS    0CL20                EXIT CONTROL TABLE ENTRY 20
         DC    CL8' '               EXIT MODULE NAME
         DC    AL1(0)               INVOCATION FLAGS FOR THIS EXIT
         DC    3XL1'00'             RESERVED
         DC    CL8' '               VERB NAME TO INVOKE THIS EXIT,
*                                   IF ANY
         END
