***********************************************************************
*                                                                     *
* MODULE NAME = BNGIMENU                                              *
*                                                                     *
* DESCRIPTIVE NAME =  IMS DEMF MENU DISPLAY MODULE                    *
*                                                                     *
* COPYRIGHT = NONE                                                    *
*                                                                     *
* STATUS = RELEASE 1, LEVEL 1                                         *
*                                                                     *
* FUNCTION = 1) DISPLAYS THE FIRST MENU SCREEN THAT OFFERS A CHOICE   *
*               OF ALL, 3270, REMOT OR LOCAL DISPLAYS;                *
*            2) DISPLAYS THE SECOND MENU SCREEN THAT DESCRIBES THE    *
*               SYNTAX OF THE DEMF COMMANDS AND PARAMETERS WHICH USE  *
*               EXPLICIT PHYSICAL ADDRESSES OR SYMBOLIC NAMES.        *
*                                                                     *
* NOTES = SEE BELOW                                                   *
*                                                                     *
*    DEPENDENCIES = NONE                                              *
*                                                                     *
*    RESTRICTIONS = NONE                                              *
*                                                                     *
*    REGISTER CONVENTIONS :                                           *
*    --------------------                                             *
*                                                                     *
*    ALL DEMF RECORDING DISPLAY COMPONENTS USE STANDARD REGISTER      *
*    CONVENTIONS WITH THE EXCEPTION OF THE IMS DISPLAY COMPONENTS.    *
*    CERTAIN IMS MODULES USE ADDITIONAL REGISTERS AND THE LINK MACRO  *
*    FOR PASSING OR RETRIEVING DATA PARAMETER INFORMATION AND ARE     *
*    DETAILED BELOW:                                                  *
*                                                                     *
*     MODULE    REGISTER    CONTENTS.                                 *
*    --------   --------    ------------------------------------      *
*    BNGIDISP     R5        PASSES ADDRESS OF IMS PROGRAM             *
*                           COMMUNICATION BLOCK (PCB) TO              *
*                           BNGI3270, BNGIMENU, BNGILOCL, AND         *
*                           BNGIRMOT.                                 *
*                                                                     *
*                 R11       PASSES TO BNGIREAD THE ADDRESS OF A       *
*                           PARAMETER LIST CONTAINING:                *
*                                                                     *
*                          1) 4-BYTES - ADDRESS OF I/O AREA FROM      *
*                                       BNGIREAD                      *
*                                                                     *
*                          2) 3-BYTES - RELATIVE BLOCK ADDRESS TO     *
*                                       BE READ BY BNGIREAD           *
*                                                                     *
*                          3) 2-BYTES - STATUS OF READ FROM           *
*                                       BNGIREAD                      *
*                                                                     *
*                 R12      FOR BNGI3270, PASSES THE ADDRESS OF A      *
*                          COMMUNICATION AREA CONTAINING              *
*                          UP TO SIX EDITTED PARM FIELDS, SAVE        *
*                          AND WORK AREAS, AND THE INPUT MESSAGE.     *
*                                                                     *
*                          FOR BNGILOCL, BNGIRMOT, AND BNGIMENU;      *
*                          PASSES THE ADDRESS OF THE INPUT MESSAGE.   *
*                                                                     *
*    BNGIREAD    LINK TO BNGIDCBA       RETRIEVES THE ADDRESS OF THE  *
*                (ALIAS FOR BNGISIMS)   ADDRESS OF THE SYS1.DEMFLOG   *
*                                       DCB.                          *
*                                                                     *
*                 R11      RECEIVES ADDRESS OF PARAMETER LIST         *
*                          (SEE BNGIDISP R11 FOR LIST).               *
*                                                                     *
*    BNGI3270     R5      RECEIVES ADDRESS OF IMS PCB FROM BNGIDISP   *
*                                                                     *
*                 R11     PASSES TO BNGIREAD THE ADDRESS OF A         *
*                         PARAMETER LIST (SEE BNGIDISP R11 FOR LIST)  *
*                                                                     *
*                 R12     RECEIVES FROM BNGIDISP THE ADDRESS OF THE   *
*                         COMMUNICATION AREA (SEE BNGIDISP R12 FOR    *
*                         CONTENTS OF AREA).                          *
*                                                                     *
*    BNGIMENU     R5      RECEIVES ADDRESS OF IMS PCB FROM BNGIDISP   *
*                                                                     *
*                 R12     RECEIVES ADDRESS OF INPUT MESSAGE FROM      *
*                         BNGIDISP.                                   *
*                                                                     *
*    BNGILOCL     R5      RECEIVES ADDRESS OF IMS PCB FROM BNGIDISP   *
*                                                                     *
*                 R12     RECEIVES ADDRESS OF INPUT MESSAGE FROM      *
*                         BNGIDISP.                                   *
*                                                                     *
*    BNGIRMOT     R5      RECEIVES ADDRESS OF IMS PCB FROM BNGIDISP   *
*                                                                     *
*                 R12     RECEIVES ADDRESS OF INPUT MESSAGE FROM      *
*                         BNGIDISP.                                   *
*                                                                     *
*    PATCH LABEL = NONE                                               *
*                                                                     *
* MODULE TYPE = IMS DISPLAY                                           *
*                                                                     *
*    PROCESSOR = ASSEMBLER H                                          *
*                                                                     *
*    MODULE SIZE =  319 DECIMAL BYTES                                 *
*                                                                     *
*    ATTRIBUTES = SERIALLY REUSABLE                                   *
*                                                                     *
* ENTRY POINT = BNGIMENU                                              *
*                                                                     *
*    PURPOSE = SEE FUNCTION                                           *
*                                                                     *
*    LINKAGE = R14 LOADED WITH THE RETURN ADDR (USED BY SUBROUTINE)   *
*                                                                     *
* INPUT = PARAM LIST ITEMS USED ARE : INPUT MESSAGE                   *
*                                                                     *
* OUTPUT = PARAM LIST ITEMS USED ARE : PAGE 1 OR 2 OF MENU DISPLAY    *
*                                                                     *
* EXIT-NORMAL = REGISTER 15 RETURN CODE RETURNED TO CALLER :          *
*               00 - OPERATION PERFORMED AS REQUESTED                 *
*                                                                     *
* EXIT-ERROR  = NONE                                                  *
*                                                                     *
* EXTERNAL REFERENCE = NONE                                           *
*                                                                     *
*    ROUTINES = NONE                                                  *
*                                                                     *
*    DATA AREAS =  NONE                                               *
*                                                                     *
*    CONTROL BLOCK = NONE                                             *
*                                                                     *
* TABLES = NONE                                                       *
*                                                                     *
* MACROS = SAVE, RETURN                                               *
*                                                                     *
* CHANGE ACTIVITIES AS FOLLOWS:                                       *
* CHANGE 01   11/15/77  PROLOGUE ADDED                                *
*                                                                     *
***********************************************************************
BNGIMENU TITLE 'DISPLAY EXCEPTION MONITORING FACILITY MENU PROGRAM'
         SPACE
PCB      DSECT                 MAP OF IMS PROGRAM COMMUNICATION BLOCK
         USING *,PCBREG
PCBTERMN DS    CL8                 REQUESTING TERMINAL SYMBOLIC NAME
         DS    CL2                 RESERVED
PCBSTAT  DS    CL2                 DL/I STATUS CODE
PCBDATE  DS    PL4                 TRANSACTION DATE STAMP 00YYDDDF
PCBTIME  DS    PL4                 TRANSACTION TIME STAMP HHMMSSTH
PCBSEQ   DS    F                   TRANSACTION SEQUENCE NUMBER
PCBMODNM DS    CL8                 MFS MESSAGE OUTPUT DESCRIPTOR NAME
MSGIN    DSECT
         USING *,MSGINREG
TIOATDL  DS    H                   MESSAGE LENGTH INCL. LL,Z1,Z2,DATA
MSGZ1    DS    X                   FLAG FIELD - USED BY IMS INTERNALLY
MSGZ2    DS    X                   FLAG FIELD - NON-ZERO IF MFS EDITED
TIOADBA  DS    0F                  START OF MESSAGE DATA
MSGTXCD  DS    CL5                 DEMF
MSGKEYED DS    CL29
MSGCMDCD DS    CL4                 MENU
MSGALL   DS    C
MSG3270  DS    C
MSGRMOT  DS    C
MSGLOCL  DS    C
MSGDPRSS DS    CL7
         PRINT NOGEN
BNGIMENU CSECT
R1       EQU   1
BASE1    EQU   2
PCBREG   EQU   5                   POINTER TO TP-PCB ADDRESS
MSGINREG EQU   12                  POINTER TO MSGIN AREA
R13      EQU   13
         SAVE  (14,12)             SAVE CALLER'S REGISTERS
         BALR  BASE1,0
         USING *,BASE1
         LA    14,SAVE             POINT TO OUR SAVE AREA
         ST    14,8(13)            FORWARD CHAIN FROM OLD
         ST    13,4(14)            BACKWARD CHAIN TO OLD
         LR    13,14               FIRST BASE AND S/A FOR OUR REGS
         B     GO
SAVEID   DC    C'BNGIMENU - DISPLAY EXCEPTION MEASUREMENT FACILITY FOR ,
               IMS'                EYE-CATCHER
SAVE     DS    18F                 SAVE AREA
GO       DS    0H
         CLC   MSGDPRSS,=C'DEPRESS'      IS THIS A PAGE 2 REQUEST?
         BE    A10                        YES.
         CALL  ASMTDLI,(ISRT,(PCBREG),,MENUPG1),VL
         B     CHECKST
A10      CALL  ASMTDLI,(ISRT,(PCBREG),,MENUPG2),VL
CHECKST  CLC   PCBSTAT,DLIOK       DID ISRT GO OK?
         BNE   ABEND1              IF NOT, ABEND.
         L     R13,4(R13)          RESTORE INCOMING SAVE AREA ADDRESS
         RETURN (14,12)            RETURN TO BNGIDISP
ABEND1   ABEND 111,DUMP
***********************************************************************
ISRT     DC    CL4'ISRT'           INSERT FUNCTION - OUTPUT TO TERMINAL
DLIOK    DC    CL2' '              DL/I STATUS SAYS OPERATION PERFORMED
MENUPG1  DC    CL8'BNGIOMN1'       MFS MESSAGE OUTPUT DESCRIPTOR NAME
MENUPG2  DC    CL8'BNGIOMN2'       MFS MESSAGE OUTPUT DESCRIPTOR NAME
         LTORG
         END   BNGIMENU
