* %/*                                                        @G24LB18*/
*/*                                                          @G24LB18*/
*/*  MODULE NAME = ICBSELTM                                  @G24LB18*/
*/*                                                          @G24LB18*/
*/*  DESCRIPTIVE NAME = SELECT/TERMINATE                     @G24LB18*/
*/*                                                          @G24LB18*/
*/*  COPYRIGHT = NONE                                        @G24LB18*/
*/*                                                          @G24LB18*/
*/*  STATUS = OS/MVS                                         @G24LB18*/
*/*                                                          @G24LB18*/
*/*  FUNCTION = PROVIDES A REQUEST BLOCK MAPPING FOR AN MSVC @G24LB18*/
*/*             SELECT OR TERMINATE REQUEST.                 @G24LB18*/
*/*                                                          @G24LB18*/
*/*  NOTES = ICBSELTM IS A BILINGUAL MACRO.                  @G24LB18*/
*/*                                                          @G24LB18*/
*/*    DEPENDENCIES = NONE                                   @G24LB18*/
*/*                                                          @G24LB18*/
*/*    RESTRICTIONS = PREFIXES SEL AND TRM RESERVED          @G24LB18*/
*/*                   FOR USE BY THE MACRO.                  @G24LB18*/
*/*                                                          @G24LB18*/
*/*    REGISTER CONVENTIONS = NORMAL MACRO USAGE OF          @G24LB18*/
*/*                           R0, R1, R14, AND R15.          @G24LB18*/
*/*                                                          @G24LB18*/
*/*  MODULE TYPE = MACRO                                     @G24LB18*/
*/*                                                          @G24LB18*/
*/*    PROCESSOR = PLS2 AND ASSEMBLER                        @G24LB18*/
*/*                                                          @G24LB18*/
*/*    ENTRY POINT = ICBSELTM                                @G24LB18*/
*/*                                                          @G24LB18*/
*/*    PURPOSE = THE SELECT REQUEST IS ISSUED BY ALLOCATION  @G24LB18*/
*/*              FOR EACH 3330V DD TO ALLOW MSVC TO KEEP     @G24LB18*/
*/*              TRACK OF THE VIRTUAL VOLUMES CURRENTLY      @G24LB18*/
*/*              BEING USED AND TO SELECT VOLUMES FOR MSVGP  @G24LB18*/
*/*              REQUESTS. THE TERMINATE REQUEST IS ISSUED   @G24LB18*/
*/*              BY ALLOCATION AT THE END OF ALLOCATION FOR  @G24LB18*/
*/*              EACH STEP CONTAINING 3330V DDS. THIS ALLOWS @G24LB18*/
*/*              MSVC TO FREE UP THE RECORDS IT BUILT AT     @G24LB18*/
*/*              SELECT TIME FOR THE 3330V DDS IN THE STEP.  @G24LB18*/
*/*                                                          @G24LB18*/
*/*  INPUT                                                   @G24LB18*/
*/*                                                          @G24LB18*/
*/*    MACRO SYNTAX: LIST FORM ONLY                          @G24LB18*/
*/*                                                          @G24LB18*/
*/*        VERB       REQUIRED PARAMETERS                    @G24LB18*/
*/*                                                          @G24LB18*/
*/*        ICBSELTM   MF=L,TYPE=SEL/TRM       -- ASSEMBLER   @G24LB18*/
*/*                                                          @G24LB18*/
*/*        ICBSELTM   MF(L) TYPE(SEL/TRM)     -- PLS         @G24LB18*/
*/*                                                          @G24LB18*/
*/*    KEYWORD PARAMETERS:                                   @G24LB18*/
*/*                                                          @G24LB18*/
*/*      MF - MACRO FORMAT: ONLY THE LIST FORM OF THE MACRO  @G24LB18*/
*/*           MAY BE SPECIFIED. THE EXPANSION OF THE LIST    @G24LB18*/
*/*           FORM WILL RESULT IN A MAPPING OF A REQUEST     @G24LB18*/
*/*           BLOCK AND EQUATES FOR THE OPCODE AND LENGTH    @G24LB18*/
*/*           OF THE REQUEST BLOCK.                          @G24LB18*/
*/*                                                          @G24LB18*/
*/*      TYPE - INDICATES WHETHER THE REQUEST IS FOR THE     @G24LB18*/
*/*             SELECT(SEL) FUNCTION OR THE TERMINATE (TRM)  @G24LB18*/
*/*             FUNCTION. THE REQUEST BLOCK AND THE EQUATES  @G24LB18*/
*/*             FOR THE OPCODE AND LENGTH WILL BE GENERATED  @G24LB18*/
*/*             ACCORDING TO THE FUNCTION SPECIFIED.         @G24LB18*/
*/*                                                          @G24LB18*/
*/*  EXIT-NORMAL = VALID INVOCATION                          @G24LB18*/
*/*                                                          @G24LB18*/
*/*    OUTPUT = A BASED DCL STRUCTURE (PLS) OR DSECT         @G24LB18*/
*/*             (BAL) OF THE REQUEST BLOCK FOR THE REQUESTED @G24LB18*/
*/*             FUNCTION, PLUS THE APPROPRIATE DCLS (PLS)    @G24LB18*/
*/*             OR EQUATES (BAL) FOR THE OPCODE AND LENGTH   @G24LB18*/
*/*             OF THE REQUEST BLOCK.                        @G24LB18*/
*/*                                                          @G24LB18*/
*/*    CONDITION = RETURN CODE = 0                           @G24LB18*/
*/*                                                          @G24LB18*/
*/*  EXIT-ERROR = INVALID INVOCATION (RESULTS IN FAILURE OF  @G24LB18*/
*/*               COMPILATION OR ASSEMBLY OF CALLING MODULE) @G24LB18*/
*/*                                                          @G24LB18*/
*/*    OUTPUT = ERROR MESSAGE AS A COMMENT AT POINT OF       @G24LB18*/
*/*             INVOCATION.                                  @G24LB18*/
*/*                                                          @G24LB18*/
*/*    CONDITION = RETURN CODE = 12                          @G24LB18*/
*/*                                                          @G24LB18*/
*/*  EXTERNAL REFERENCES = NONE                              @G24LB18*/
*/*                                                          @G24LB18*/
*/*  TABLES = NONE                                           @G24LB18*/
*/*                                                          @G24LB18*/
*/*  MESSAGES                                                @G24LB18*/
*/*                                                          @G24LB18*/
*/*    ICB600I:  TYPE MUST BE SPECIFIED.                     @G24LB18*/
*/*    ICB601I:  TYPE IS INCORRECTLY SPECIFIED.              @G24LB18*/
*/*    ICB617I:  INVALID MF PARAMETER SPECIFIED; MUST BE L.  @G24LB18*/
*/*                                                          @G24LB18*/
*/*  CONTROL BLOCKS = NONE                                   @G24LB18*/
*/*                                                          @G24LB18*/
*/*  INNER MACROS = NONE                                     @G24LB18*/
*/*                                                          @G24LB18*/
*/*  SYSTEM MACROS = NONE                                    @G24LB18*/
*/*                                                          @G24LB18*/
*/*  CHANGE ACTIVITY =                                       @G24LB18*/
*/*                                                          @G24LB18*/
*/*    G24LB18  ALLOCATE TO MOUNTED VOLUMES        4/5/76    @G24LB18*/
*/*    G24LB04  ALLOCATE TO MOUNTED VOLUMES      07/06/76    @G24LB04*/
*/*                                                          @G24LB18*/
*/********************************************************** @G24LB18*/
*/*
         MACRO
         ICBSELTM &MF=,&TYPE=
.*                                                             @G24LB04
.*       CHECK FOR VALID TYPE SPECIFICATION                    @G24LB04
.*                                                             @G24LB04
.TYPE    AIF   ('&TYPE' EQ '').NUL                             @G24LB04
         AIF   ('&TYPE' EQ 'SEL' OR '&TYPE' EQ 'TRM').START    @G24LB04
         MNOTE 12,'  ICB601I - TYPE IS INCORRECTLY SPECIFIED'  @G24LB04
         MEXIT                                                 @G24LB04
.*                                                             @G24LB04
.NUL     ANOP                                                  @G24LB04
         MNOTE 12,'  ICB600I - TYPE MUST BE SPECIFIED '        @G24LB04
         MEXIT                                                 @G24LB04
.*                                                             @G24LB04
.*       DETERMINATION OF DESIRED MACRO FORM                   @G24LB04
.*                                                             @G24LB04
.START   AIF   ('&MF(1)' EQ 'L').LFORM      LIST FORM?         @G24LB04
         MNOTE 12,'  ICB617I - INVALID MF PARAMETER '          @G24LB04
         MNOTE 12,'  SPECIFIED; MUST BE L.'                    @G24LB04
         MEXIT                                                 @G24LB04
.*                                                             @G24LB04
.*       PROCESSING OF 'LIST' MACRO FORM                       @G24LB04
.*                                                             @G24LB04
.LFORM   ANOP                                                  @G24LB04
         AIF   ('&TYPE' EQ 'TRM').TERMN    SELECT REQUEST?     @G24LB04
.*                                                             @G24LB04
.SELCDS  ANOP                                                  @G24LB04
*  MACMEAN  SELECT MACRO--UNINITIALIZED LIST FORM              @G24LB04
*  MVS VERSION--07/20/76                                       @G24LB04
SELRB    DSECT                                                 @G24LB04
         DS    0F                                              @G24LB04
SELRBLEN DS    XL4                 LENGTH OF SELECT RB         @G24LB04
SELRBOPC DS    XL1                 OPCODE FOR SELECT           @G24LB04
SELFLGS1 DS    XL1                 ALLOCATION FLAGS            @G24LB04
SELPARMT EQU   X'80'               PARALLEL MOUNT REQUESTED    @G24LB04
SELDEFER EQU   X'40'               DEFERRED MOUNT REQUESTED    @G24LB04
SELSEPVL EQU   X'20'               SEPVOL REQUESTED            @G24LB04
SELVENQV EQU   X'10'               VOLUME ENQ VALID            @G24LB04
SELVOLRF EQU   X'08'               VOLUME REFERBACK TO DDNAME  @G24LB04*
                                   REQUESTED                   @G24LB04
SELVRNEW EQU   X'04'               REFERENCED DD IS FOR NEW    @G24LB04*
                                   DATA SET                    @G24LB04
SELVRGRP EQU   X'02'               REFERENCED DD IS FOR GROUP  @G24LB04*
                                   VOLUME (IF NEW)             @G24LB04
SELMXESO EQU   X'01'               MIXED ESOTERIC REQUEST      @G24LB04
SELFLGS2 DS    XL1                 ALLOCATION/MSVC FLAGS       @G24LB04
SELPRSPL EQU   X'80'               RESERVED FOR VS1 USE        @G24LB04
SELSCSPL EQU   X'40'               RESERVED FOR VS1 USE        @G24LB04
SELSUBAL EQU   X'20'               RESERVED FOR VS1 USE        @G24LB04
SELUNSEP EQU   X'10'               RESERVED FOR VS1 USE        @G24LB04
SELCNTWT EQU   X'08'               CAN NOT WAIT REQUEST        @G24LB04
SELINITC EQU   X'04'               INITIAL CALL FOR STEP OR    @G24LB04*
                                   DYNAMIC ALLOCATION          @G24LB04
SELRPRIV EQU   X'02'               MARK REQUEST PRIVATE        @G24LB04
SELNOVSL EQU   X'01'               NO VOLS WERE SELECTED       @G24LB04
SELUNCNT DS    XL1                 UNIT COUNT OF REFERENCED DD @G24LB04*
                                   STATEMENT                   @G24LB04
SELJOBID DS    XL4                 THE ASID                    @G24LB04
SELJFCBA DS    F                   ADDRESS OF JFCB             @G24LB04
SELDEVTP DS    XL4                 4-BYTE INTERNAL DEVICE TYPE @G24LB04
SELUCBAD DS    F                   UCB ADDR (IF SPECIFIC UNIT) @G24LB04
SELJOBNM DS    CL8                 8-BYTE JOB NAME             @G24LB04
SELDDNAM DS    CL8                 DD NAME                     @G24LB04
SELSTNUM DS    XL1                 STEP NUMBER                 @G24LB04
SELUNUS1 DS    CL1                 UNUSED                      @G24LB04
SELDDCAT DS    H                   DD RELATIVE CONCAT. NUMBER  @G24LB04
SELDDNUM DS    H                   CURRENT DD NUMBER,          @G24LB04*
                                   ZERO FOR DYNAMIC ALLOCATION @G24LB04
SELRFDDN DS    H                   REFERENCED DD NO.(SEPVOL)   @G24LB04
SELUNUS2 DS    CL4                 UNUSED                      @G24LB04
SELJFCBX DS    F                   ADDR OF 1ST JFCB EXTN       @G24LB04
SELVLRET DS    F                   ADDR OF VOLSER RETURN AREA  @G24LB04
SELGRPPR DS    F                   ADDR OF GROUP PARAMS AREA   @G24LB04
SELUNUSE DS    XL8                 UNUSED                      @G24LB04
SELLEN   EQU   X'48'               SELECT REQUEST BLOCK LENGTH @G24LB04
SELOPCD  EQU   X'89'               OP CODE FOR SELECT REQUEST  @G24LB04
         AGO   .FINISH                                         @G24LB04
.*                                                             @G24LB04
.TERMN   ANOP                                                  @G24LB04
*  MACMEAN  TERMINATE MACRO--UNINITIALIZED LIST FORM           @G24LB04
*  MVS VERSION--07/06/76                                       @G24LB04
TRMRB    DSECT                                                 @G24LB04
         DS    0F                                              @G24LB04
TRMRBLEN DS    XL4                 LENGTH OF TERMINATE REQ BLK @G24LB04
TRMRBOPC DS    XL1                 OPCODE FOR TERMINATE        @G24LB04
TRMFLAGS DS    XL1                 1-BYTE ALLOCATION FLAGS     @G24LB04
TRMALLOK EQU   X'80'               STEP ALLOCATED SUCCESSFULLY @G24LB04
TRMSCHAB EQU   X'40'               RESERVED FOR VS1 USE        @G24LB04
         DS    XL2                 UNUSED                      @G24LB04
TRMJOBID DS    XL4                 4-BYTE ASID AREA            @G24LB04
TRMSTHDR DS    F                   STEP TABLE HEADER ADDRESS   @G24LB04
TRMLEN   EQU   X'10'               TERMINATE REQ BLOCK LENGTH  @G24LB04
TRMOPCD  EQU   X'8A'               OP CODE FOR TERMINATE REQ   @G24LB04
         AGO   .FINISH                                         @G24LB04
.FINISH  MEND                                                  @G24LB04
**/
*   ICBSELTM:  MACRO KEYS(MF,TYPE);                       /* @G24LB04*/
*/*                                                          @G24LB04*/
*/*                                                          @G24LB04*/
*/*                                                          @G24LB04*/
*/*            INTERNAL DECLARE                              @G24LB04*/
*/*                                                          @G24LB04*/
*   DCL RETCODE FIXED;       /*RETURN CODE AREA              @G24LB04*/
*        RETCODE=0;          /*INITIALIZE RETURN CODE        @G24LB04*/
*/*                                                          @G24LB04*/
*/*                                                          @G24LB04*/
*/*            CHECK FOR VALID TYPE SPECIFICATION            @G24LB04*/
*/*                                                          @G24LB04*/
*   IF TYPE='' THEN          /*IS THE TYPE PARM NULL?        @G24LB04*/
*     DO;                    /*PRINT ERROR MESSAGE           @G24LB04*/
*        ANS MSG('  ICB600I - TYPE MUST BE '||
*                'SPECIFIED ')SKIP;                        /*@G24LB04*/
*        RETCODE=12;         /*SET RETURN CODE TO 12         @G24LB04*/
*        GO TO FINISH;       /*BRANCH TO FINISH              @G24LB04*/
*     END;                   /*                              @G24LB04*/
*   IF TYPE(1)='SEL' THEN    /*IS THE TYPE SELECT?           @G24LB04*/
*        GO TO START;        /*YES, VALID TYPE, BRANCH       @G24LB04*/
*   IF TYPE(1)='TRM' THEN    /*IS THE TYPE FOR TERMINATE?    @G24LB04*/
*        GO TO START;        /*YES, VALID TYPE, BRANCH       @G24LB04*/
*/*                                                          @G24LB04*/
*/*      INVALID 'TYPE' WILL CREATE THE FOLLOWING ERR MSG    @G24LB04*/
*/*                                                          @G24LB04*/
*   ANS MSG('  ICB601I - TYPE IS INCORRECTLY '||
*           'SPECIFIED')SKIP;                              /*@G24LB04*/
*   RETCODE=12;              /*SET RETURN CODE TO 12         @G24LB04*/
*        GO TO FINISH;       /*BRANCH TO FINISH              @G24LB04*/
*/*                                                          @G24LB04*/
*START:
*   IF MF(1) = 'L' THEN      /*IS REQUEST FOR LIST FORM?     @G24LB04*/
*        GO TO BLDLIST;      /*YES, WILL GO BUILD LIST       @G24LB04*/
*   ANS MSG('  ICB617I - INVALID MF PARAMETER '||
*           'SPECIFIED; MUST BE L.')SKIP;                /*  @G24LB04*/
*   RETCODE=12;              /*SET RETURN CODE TO 12         @G24LB04*/
*        GO TO FINISH;       /*BRANCH TO FINISH              @G24LB04*/
*/*                                                          @G24LB04*/
*BLDLIST:
*   IF TYPE(1)='SEL' THEN       /*SELECT REQUEST?            @G24LB04*/
*     DO;                       /*PRINT UNINITIALIZED BLK    @G24LB04*/
*        ANS('/*******************************************'||
*            '************** @G24LB04*/')SKIP;
*        ANS('/*MACMEAN  MSSC SELECT MACRO                '||
*            '               @G24LB04*/')SKIP;
*        ANS('/*MVS VERSION--07/20/76                     '||
*            '               @G24LB04*/')SKIP;
*        ANS('/*THIS IS THE UNINITIALIZED LIST FORM       '||
*            '               @G24LB04*/')SKIP;
*        ANS('/*******************************************'||
*            '************** @G24LB04*/')SKIP;
*        ANS('DCL 1 SELRB BASED,'||
*        ' /*SELECT REQUEST BLOCK @G24LB04*/')SKIP;
*        ANS('    2  SELRBLEN FIXED(31), /*LENGTH'||
*        ' OF BLOCK @G24LB04*/')SKIP;
*        ANS('    2  SELRBOPC FIXED(8), /*SELECT'||
*        ' OPCODE @G24LB04*/')SKIP;
*        ANS('    2  SELFLGS1 BIT(8), /* ALLOCATION FLAGS'||
*        ' @G24LB04*/')SKIP;
*        ANS('    3  SELPARMT BIT(1) , /*PARALLEL MOUNT REQUESTED '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELDEFER BIT(1) , /*DEFERRED MOUNT REQUESTED '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELSEPVL BIT(1) , /*SEPVOL REQUESTED '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELVENQV BIT(1) , /*VOLUME ENQUEUE VALID '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELVOLRF BIT(1) , /*VOLUME REFERBACK TO DDNAME '||
*        ' REQUESTED @G24LB04*/')SKIP;
*        ANS('    3  SELVRNEW BIT(1) , /*REFERENCED DD IS FOR NEW '||
*        'DATA SET @G24LB04*/')SKIP;
*        ANS('    3  SELVRGRP BIT(1) , /*REFERENCED DD IS FOR GROUP '||
*        'VOLUME (IF NEW) @G24LB04*/')SKIP;
*        ANS('    3  SELMXESO BIT(1), /* MIXED ESOTERIC REQUEST'||
*        ' @G24LB04*/')SKIP;
*        ANS('    2  SELFLGS2 BIT(8), /* ALLOCATION/MSVC FLAGS'||
*        ' @G24LB04*/')SKIP;
*        ANS('    3  SELPRSPL BIT(1) , /*RESERVED FOR VS1 USE '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELSCSPL BIT(1) , /*RESERVED FOR VS1 USE '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELSUBAL BIT(1) , /*RESERVED FOR VS1 USE '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELUNSEP BIT(1) , /*RESERVED FOR VS1 USE '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELCNTWT BIT(1) , /*CAN NOT WAIT REQUEST '||
*        '@G24LB04*/')SKIP;
*        ANS('    3  SELINITC BIT(1) , /*INITIAL CALL FOR STEP OR '||
*        ' DYNAMIC ALLOCATION @G24LB04*/')SKIP;
*        ANS('    3  SELRPRIV BIT(1) , /*MARK REQUEST PRIVATE '||
*        ' @G24LB04*/')SKIP;
*        ANS('    3  SELNOVSL BIT(1) , /*NO VOLUMES WERE SELECTED '||
*        ' @G24LB04*/')SKIP;
*        ANS('    2  SELUNCNT FIXED(8),/*UNIT COUNT '||
*        '@G24LB04*/')SKIP;
*        ANS('    2  SELJOBID FIXED(31), /*ASID'||
*        ' @G24LB04*/')SKIP;
*        ANS('    2  SELJFCBA PTR(31),/*ADDRESS OF JFCB '||
*        '@G24LB04*/')SKIP;
*        ANS('    2  SELDEVTP CHAR(4) , /*INTERNAL DEVICE TYPE '||
*        '@G24LB04*/')SKIP;
*        ANS('    2  SELUCBAD PTR(31) , /*ADDRESS OF UCB (IF '||
*        'SPECIFIC UNIT REQUESTED) @G24LB04*/')SKIP;
*        ANS('    2  SELJOBNM CHAR(8) , /*JOBNAME '||
*        ' @G24LB04*/')SKIP;
*        ANS('    2  SELDDNAM CHAR(8) , /*DD NAME '||
*        ' @G24LB04*/')SKIP;
*        ANS('    2  SELSTNUM FIXED(8) , /*STEP NUMBER '||
*        ' @G24LB04*/')SKIP;
*        ANS('    2  SELUNUS1 CHAR(1),/*UNUSED '||
*        '@G24LB04*/')SKIP;
*        ANS('    2  SELDDCAT FIXED(15), /*DD RELATIVE CONCATENATION'||
*        ' NUMBER @G24LB04*/')SKIP;
*        ANS('    2  SELDDNUM FIXED(15) , /*CURRENT DD NUMBER, '||
*        'ZERO FOR DYNAMIC ALLOCATION @G24LB04*/')SKIP;
*        ANS('    2  SELRFDDN FIXED(15) , /*REFERENCED DD NUMBER '||
*        '(FOR SEPVOL) @G24LB04*/')SKIP;
*        ANS('    2  SELUNUS2 CHAR(4),/*UNUSED '||
*        '@G24LB04*/')SKIP;
*        ANS('    2  SELJFCBX PTR(31) , /*ADDRESS OF FIRST JFCB '||
*        'EXTENSION @G24LB04*/')SKIP;
*        ANS('    2  SELVLRET PTR(31),/*ADDRESS OF VOLUME SERIAL '||
*        'RETURN AREA @G24LB04*/')SKIP;
*        ANS('    2  SELGRPPR PTR(31),/*ADDRESS OF GROUP PARAMETERS '||
*        'AREA @G24LB04*/')SKIP;
*        ANS('    2  SELUNUSE CHAR(8);/*UNUSED @G24LB04*/')SKIP;
*        ANS('DCL SELLEN FIXED(8) CONSTANT(72);'||
*        '/*SELECT REQUEST LENGTH @G24LB04*/')SKIP;
*        ANS('DCL SELOPCD BIT(8) CONSTANT(' || QUOTE('89') || 'X);'||
*        '/*SELECT OP CODE @G24LB04*/')SKIP;
*        GO TO FINISH;                                    /* @G24LB04*/
*     END;                                                /* @G24LB04*/
*/*                                                          @G24LB04*/
*  ELSE                      /*PROCESS TERMINATE REQ BLK     @G24LB04*/
*     DO;                    /*PRINT TERMINATE REQ BLK       @G24LB04*/
*        ANS('/*******************************************'||
*            '************** @G24LB04*/')SKIP;
*        ANS('/*MACMEAN  MSSC TERMINATE MACRO             '||
*            '               @G24LB04*/')SKIP;
*        ANS('/*MVS VERSION--06/16/76                     '||
*            '               @G24LB04*/')SKIP;
*        ANS('/*THIS IS THE UNINITIALIZED LIST FORM       '||
*            '               @G24LB04*/')SKIP;
*        ANS('/*******************************************'||
*            '************** @G24LB04*/')SKIP;
*        ANS('DCL 1 TRMRB BASED,'||
*        ' /*TERMINATE REQUEST BLOCK @G24LB04*/')SKIP;
*        ANS('    2  TRMRBLEN FIXED(31),/*LENGTH OF'||
*        ' TERMINATE REQ BLK @G24LB04*/')SKIP;
*        ANS('    2  TRMRBOPC FIXED(8),/*TERMINATE'||
*        ' OPCODE @G24LB04*/')SKIP;
*        ANS('    2  TRMFLAGS BIT(8), /*ALLOCATION FLAGS'||
*        ' @G24LB04*/')SKIP;
*        ANS('    3  TRMALLOK BIT(1), /*STEP ALLOCATED '||
*        'SUCCESSFULLY @G24LB04*/')SKIP;
*        ANS('    3  TRMSCHAB BIT(1), /*RESERVED FOR VS1 USE '||
*        ' @G24LB04*/')SKIP;
*        ANS('    3  * BIT(6), /*UNUSED BITS'||
*        ' @G24LB04*/')SKIP;
*        ANS('    2  * CHAR(2), /* UNUSED @G24LB04*/')SKIP;
*        ANS('    2  TRMJOBID FIXED(31), /*ASID @G24LB04*/')SKIP;
*        ANS('    2  TRMSTHDR PTR(31);/*STEP TABLE HEADER ADDRESS' ||
*        ' @G24LB04*/')SKIP;
*        ANS('DCL TRMLEN FIXED(8) CONSTANT(16);/*TERMINATE REQUEST '
*         || 'LENGTH @G24LB04*/')SKIP;
*        ANS('DCL TRMOPCD BIT(8) CONSTANT(' || QUOTE('8A') || 'X);'||
*        '/*TERMINATE OP CODE @G24LB04*/')SKIP;
*     END;                 /*                                @G24LB04*/
*/*                                                          @G24LB04*/
*FINISH:  RETURN CODE(RETCODE);    /*SET RETURN CODE         @G24LB04*/
*         %END;                                            /*@G24LB04*/
