*/********************************************************************/
*/*                                                                  */
*/*   MACRO NAME = IDABUFC                                           */
*/*                                                                  */
*/*   DESCRIPTIVE NAME = VSAM BUFFER CONTROL BLOCK HEADER (IDABUFDR) */
*/*              VSAM BUFFER CONTROL BLOCK (IDABUFC)                 */
*/*                                                                  */
*/*   FUNCTION = THIS MACRO CONTAINS:                                */
*/*              IDABUFDR = ORIGIN OF THE COMMON BUFFER AND THE      */
*/*               PREFORMAT BUFFER CHAINS                            */
*/*              IDABUFC = DESCRIBES THE BUFFER                      */
*/*                                                                  */
*/*   CONTROL BLOCK STRUCTURE = IDABUFDR IS POINTED TO BY IDAAMB     */
*/*                                                                  */
*/*   INCLUDED MACROS = NONE                                         */
*/*                                                                  */
*/*   METHOD OF ACCESS = PL/S - NO DECLARES NECESSARY                */
*/*                                                                  */
*/*   CREATED BY = VSAM OPEN                                         */
*/*                                                                  */
*/*   STATUS = VS/1 RELEASE 4  (CHANGE FLAG @X04SVXX)                */
*/*                                                                  */
*/*   DATE OF LAST CHANGE = 9 AUG 74                                 */
*/********************************************************************/
*  DCL 1 IDABUFDR BASED BDY(WORD),         /* BUFFER HEADER          */
*        2 BUFDRID CHAR(1),                /* BUFFER HEADER ID      */
*        2 BUFDRNO PTR(8),                 /* NUMBER OF BUFFERS     */
*        2 BUFDRLEN FIXED(15),             /* LENGTH OF CNTL BLK    */
*        2 BUFDRPFB PTR(31),               /* FIRST PREFORMAT BUFC  */
*        2 BUFDRPFN PTR(8),                /* NO. OF PREFORMAT BUFCS*/
*        2 BUFDRCIX PTR(8),                /* NO OF COMMON IX BFRS  */
*          3 BUFDRMAX PTR(8),              /* MAX BFRS PER SEQ PLH  */
*        2 BUFDRTSB CHAR(1),               /* HDR TEST AND SET BYTE */
*        2 BUFDRFLG BIT(8),                /* BFR HEADER FLAGS      */
*          3 BUFDRREL BIT(1),              /* BFR REL FLG           */
*          3 BUFDRAVL BIT(1),              /* BFR AVAILABLE          */
*          3 * BIT(6),                     /* UNUSED                 */
*        2 BUFDBUFC PTR(31),               /* PTR TO 1ST BUFC@X04SVLC*/
*        2 *        CHAR(4);               /* RESERVED              */
*/********************************************************************/
*/* BUFFER CONTROL BLOCK                                             */
*/********************************************************************/
*  DCL 1 IDABUFC  BASED BDY(DWORD),        /* BUFC ENTRY     @X04SVLC*/
*        2 BUFCAVL CHAR(1),                /* BUFFER TEST/SET BYTE  */
*          3 BUFCUCNT FIXED(8),            /* USE COUNT      @X04SVLC*/
*        2 BUFCFLG1 BIT(8),                /* BUFC FLAGS--BYTE ONE  */
*          3 BUFCUPG BIT(1),               /* UPGRADE SET BUFC
*                                                           @X04SVLC*/
*          3 BUFCSEG BIT(1),               /* SEGMENTED REC IN BFR  */
*          3 BUFCINS BIT(1),               /* BUFC FOR INSERTS ONLY */
*          3 BUFCER1 BIT(1),               /* READ ERROR OCCURRED   */
*          3 BUFCER2 BIT(1),               /* WRITE ERROR OCCURRED  */
*          3 BUFCVAL BIT(1),               /* BUFCDDDD IS VALID     */
*          3 BUFCEXC BIT(1),               /* BFR IN EXCL CNTL      */
*          3 BUFCEPT BIT(1),               /* POST BIT               */
*        2 BUFCIOFL BIT(8),                /* I/O MGR COMMUN FLAGS   */
*          3 BUFCMW  BIT(1),               /* MUST WRITE THIS CINV   */
*          3 BUFCFMT BIT(1),               /* FORMAT WRITE BUFC      */
*          3 BUFCRRD BIT(1),               /* READ REQUIRED          */
*          3 BUFCREAL BIT(1),              /* BUFCBAD IS A REAL ADDR */
*          3 BUFCWC  BIT(1),               /* WRITE CHECK OPTION     */
*          3 BUFCXEDB BIT(1),              /* NO EDB FOUND FOR RBA */
*          3 BUFCPFCP BIT(1),              /* PREFORMAT CP COMPLETE  */
*          3 BUFCFIX  BIT(1),              /* BUFFER FIXED           */
*        2 BUFCFLG2 BIT(8),         /* FLAG BYTE--TWO        @X04SVLC*/
*          3 BUFCXDDR BIT(1),       /* SUPRESS DDR ON ERRORS @X04SVLC*/
*          3 BUFCNLAS BIT(1),       /* NON-EXIST PG RD IND   @X04SVLC*/
*          3 BUFCBSYR BIT(1),       /* BUSY READING          @X04SVLC*/
*          3 BUFCBSYW BIT(1),       /* BUSY WRITING          @X04SVLC*/
*          3 * BIT(4),              /* UNUSED                @X04SVLC*/
*        2 BUFCPLH PTR(31),         /* POINTER TO PLH        @X04SVLC*/
*          3 BUFCAMB PTR(31),       /* PTR TO AMB            @X04SVLC*/
*        2 BUFCDDDD PTR(32),        /* INPUT RBA             @X04SVLC*/
*        2 BUFCORBA PTR(32),        /* OUTPUT RBA            @X04SVLC*/
*        2 BUFCCPA PTR(31),         /* CHAN PGM AREA ADDRESS @X04SVLC*/
*        2 BUFCBAD PTR(31),         /* BUFFER ADDRESS        @X04SVLC*/
*        2 BUFCNXT1 PTR(31),        /* NEXT BUFC IN I/O CHN  @X04SVLC*/
*        2 BUFCINV    PTR(31),      /* INVOKERS FLD FOR ASM AND DB/DC
*                                                            @X04SVLC*/
*          3 BUFCWLEN FIXED(15),    /* BUFC DATA LEN FOR VBP @X04SVLC*/
*        2 BUFCDSPC   PTR(31),      /* DSPCT PTR FOR VBP.    @X04SVLC*/
*        2 BUFCIDXL FIXED(8),       /* INDEX LEVEL           @X04SVLC*/
*        2 BUFCNXT2 PTR(24),        /* NEXT LOGICAL BUFFER   @X04SVLC*/
*        2 BUFXIRBA FIXED(32),      /* EXCL INPUT RBA        @X04SVLC*/
*        2 BUFXORBA FIXED(32),      /* EXCL OUTPUT RBA       @X04SVLC*/
*        2 BUFCHAIN PTR(31),        /* NEXT BUFC IN POOL     @X04SVLC*/
*        2 BUFCMDBT FIXED(32),      /* MODIFICATION MASK     @X04SVLC*/
*        2 BUFCUCUP PTR(31),        /* NXT BUFC UP USE CHN   @X04SVLC*/
*        2 BUFCUCDN PTR(31),        /* NXT BUFC DOWN USE CHN @X04SVLC*/
*        2 BUFCEND CHAR(0)BDY(DWORD);/* ROUND LEN TO DWORD   @X04SVLC*/
*        DCL BUFCRSC1 CHAR(8)DEFINED(BUFCAVL);/* CDS RESOURCE
*                                                            @X04SVLC*/
