*/********************************************************************/
*/*                                                                  */
*/*   MACRO NAME = IDACPA                                            */
*/*                                                                  */
*/*   DESCRIPTIVE NAME = CHANNEL PROGRAM AREA                        */
*/*                                                                  */
*/*   FUNCTION =                                                     */
*/*                                                                  */
*/* THE CHANNEL PROGRAM AREA(CPA) CONTAINS A FIXED, HEADER SECTION   */
*/* CONTAINING VARIOUS POINTERS AND PARAMETERS, AND OTHER SECTIONS   */
*/* WHICH CONTAIN CHANNEL PROGRAM SEGMENTS AND ARGUMENTS NEEDED BY   */
*/* THE CCW-S. THESE VARIOUS OTHER SECTIONS ARE LOCATED BY POINTERS  */
*/* FROM THE FIXED SECTION SINCE THE AMOUNT OF SPACE REQUIRED FOR    */
*/* THEM VARIES DEPENDING ON CONTROL INTERVAL SIZE, BLOCKSIZE, OPEN  */
*/* OPTIONS SPECIFIED, ETC.                                          */
*/*                                                                  */
*/*   CONTROL BLOCK STRUCTURE = IDACPA IS POINTED TO BY IDABUFC      */
*/*                                                                  */
*/*   INCLUDED MACROS = NONE                                         */
*/*                                                                  */
*/*   METHOD OF ACCESS = PL/S - NO DECLARES NECESSARY                */
*/*                                                                  */
*/*   CREATED BY = VSAM OPEN                                         */
*/*                                                                  */
*/*   STATUS = VS/1 RELEASE 4  (CHANGE FLAG @X04SVXX)                */
*/*                                                                  */
*/*   DATE OF LAST CHANGE = 13 AUG 74                                */
*/*                                                                  */
*/********************************************************************/
*% GOTO CPAPLS1;
*/*                                                                  */
*/* READ AND WRITE CHANNEL COMMANDS SPECIFY AN INDIRECT DATA ADDRESS */
*/* LIST AND POINT TO AN ADDRESS WITHIN THE AREA POINTED TO BY       */
*/* CPAIDAL. THE REAL ADDRESSES IN CPAIDAL ARE FILLED IN LATER IN    */
*/* CONJUNCTION WITH PAGE FIXING.  THE VIRTUAL ADDRESSES CORRESPONDING*/
*/* TO THE REAL ADDRESSES ARE IN THE AREA POINTED TO BY CPAVPL.      */
*/* THESE VIRTUAL ADDRESSES ARE NORMALLY SUPPLIED BY OPEN.           */
*/* (EXCEPTION WHEN CPAVPLV IS OFF)                                  */
*/* THE FIRST ADDRESS IS THE STARTING ADDRESS OF THE                 */
*/* BUFFER. SUBSEQUENT ADDRESSES ARE OBTAINED BY INCREMENTING THE    */
*/* PREVIOUS ADDRESS BY 2K OR PHYSICAL BLOCK SIZE, WHICHEVER IS LESS.*/
*/*                                                                  */
*/* IT IS ALSO ASSUMED BY THE I/O MANAGER THAT CCW LISTS WILL        */
*/* EXIST, AS REQUIRED, AS SPECIFIED FOR EACH TYPE OF CHANNEL        */
*/* PROGRAM SEGMENT.                                                 */
*/*                                                                  */
*/*                                                                  */
*/* *** IMPORTANT NOTE: THE ENTIRE CPA MUST BE CONTAINED WITHIN      */
*/*                     A SINGLE PAGE.                               */
*/*          ALSO NOTE THAT CHANNEL PROGRAM SEGMENTS MUST BE         */
*/*                    ON DOUBLE WORD BOUNDARIES.                    */
*/*                                                                  */
*/*                                                                  */
*/* IN THE FOLLOWING DESCRIPTIONS,                                   */
*/*     CC = COMMAND CHAIN                                           */
*/*     MT = MULTI-TRACK MODE                                        */
*/*     IDAL = INDIRECT DATA ADDRESS LIST                            */
*/*     LL = PHYSICAL BLOCK SIZE (LPMBLKSZ)                          */
*/*     N = NUMBER OF PHYSICAL BLOCKS PER CINV (CPABCINV)            */
*/*     NT = NO TRANSFER                                             */
*/*                                                                  */
*/*                                                                  */
*/*                                                                  */
*/*                                                                  */
*/******       UPDATE WRITE CHANNEL PROGRAM SEGMENT             ******/
*/*                                                                  */
*/*                                                                  */
*/*                                                                  */
*/* EVERY SEARCH ID CCW AFTER THE FIRST ONE IS A MULTI-TRACK SEARCH. */
*/*                                                                  */
*/*                                                                  */
*/* (CPAWCPS POINTER MINUS 8)     (EXTRA CCW SLOT IF RPS DEVICE)     */
*/* CPAWCPS->                     SEEKH       CPAWSEEK,CC,6          */
*/*                         __  | SRCH ID EQ  SA,CC,5                */
*/*                        |  _ | TIC         *-8                    */
*/*                        |    | WD          DA1,(CC,IDAL),LL       */
*/*                        |                                         */
*/*                        |    | SRCH ID EQ  SA+5,(CC,MT),5         */
*/*                        |   -| TIC         *-8                    */
*/*                        |    | WD          DA2,(CC,IDAL),LL       */
*/*            N GROUPS  --|       .                                 */
*/*            OF 3 CCW-S  |       .                                 */
*/*                                .                                 */
*/*                        |    | SRCH ID EQ  SA+5*(N-1),(CC,MT),5   */
*/*                        |   -| TIC         *-8                    */
*/*                        |__  | WD          DA(N),(CC,IDAL),LL     */
*/*                               NOP                                */
*/*                                                                  */
*/*                               |-------------------------------|  */
*/* CPAIDAL->               DA1   |                               |  */
*/*                               |  IF LL=4K, 2*N 4 BYTE ENTRIES |  */
*/*  IF LL=4K,              DA2   |                               |  */
*/*     DA(N)=DA(N-1)+8           |  IF LL<4K, N 4 BYTE ENTRIES   |  */
*/*  IF LL<4K,              DA(N) |                               |  */
*/*     DA(N)=DA(N-1)+4           |                               |  */
*/*                               |-------------------------------|  */
*/* CPAVPL->                      |                               |  */
*/*                               |                               |  */
*/*                               |                               |  */
*/*                               |     SAME AS ABOVE             |  */
*/*                               |                               |  */
*/*                               |                               |  */
*/*                               |------------------------------ |  */
*/*                                                                  */
*/* CPAWSID->               SA    |-------------------------------|  */
*/*                               |  N 5 BYTE ENTRIES             |  */
*/*                               |------------------------------ |  */
*/*                                                                  */
*/*                                                                  */
*/*  BYTES REQUIRED BY CCW-S = (3*N+2(+1,IF RPS))*8                  */
*/*                                                                  */
*/*  BYTES REQUIRED FOR SID ARGS = 5*N                               */
*/*                                                                  */
*/*  BYTES REQUIRED FOR BOTH,                                        */
*/*            CCW MINIMUM         24*N+16                           */
*/*            SID ARGS            5*N                               */
*/*                                -------                           */
*/*            TOTAL MINIMUM       29*N+16                           */
*/*                                                                  */
*/*            ADDITIONAL IF RPS         8                           */
*/*                                                                  */
*/*                                                                  */
*/*                                                                  */
*/******           READ CHANNEL PROGRAM SEGMENT                 ******/
*/*                                                                  */
*/* EVERY READ CCW AFTER THE FIRST ONE IS A MULTI-TRACK READ.        */
*/*                                                                  */
*/*                                                                  */
*/* (CPARCPS POINTER MINUS 8)      (EXTRA CCW SLOT IF RPS DEVICE)    */
*/* CPARCPS->                      SEEKH      CPARSEEK,CC,6          */
*/*                                SRCH ID EQ CPARSID,CC,5           */
*/*                                TIC        *-8                    */
*/*                            --- RD         DA1,(CC,IDAL),LL       */
*/*                           |    RD         DA2,(CC,IDAL,MT),LL    */
*/*            N READ         |     .                                */
*/*            DATA CCW-S  ---|     .                                */
*/*                           |     .                                */
*/*                           |--- RD         DA(N),(CC,IDAL,MT),LL  */
*/*                                NOP                               */
*/*                                                                  */
*/*                                |------------------------------|  */
*/* CPAIDAL->                DA1   |                              |  */
*/*                                |  IF LL=4K, 2*N 4 BYTE ENTRIES|  */
*/*  IF LL=4K,               DA2   |                              |  */
*/*     DA(N)=DA(N-1)+8            |  IF LL<4K, N 4 BYTE ENTRIES  |  */
*/*  IF LL<4K,               DA(N) |                              |  */
*/*     DA(N)=DA(N-1)+4            |                              |  */
*/*                                |------------------------------|  */
*/* CPAVPL->                       |                              |  */
*/*                                |                              |  */
*/*                                |                              |  */
*/*                                |    SAME AS ABOVE             |  */
*/*                                |                              |  */
*/*                                |                              |  */
*/*                                |------------------------------|  */
*/*                                                                  */
*/*                                                                  */
*/*  BYTES REQUIRED BY CCW-S = (N+4(+1 IF RPS))*8                    */
*/*                          = 8*N+32 (+8, IF RPS)                   */
*/*                                                                  */
*/*                                                                  */
*/*                                                                  */
*/*                                                                  */
*/******       FORMAT WRITE CHANNEL PROGRAM SEGMENT             ******/
*/*                                                                  */
*/*                                                                  */
*/* ONLY THE PART OF THE CHANNEL PROGRAM SEGMENT WHICH WRITES        */
*/* THE FIRST BLOCK CAN BE PREBUILT. THEREFORE OPEN WILL             */
*/* BUILD THE CHANNEL PROGRAM (6 CCW-S) AS IF THERE WERE             */
*/* ONLY ONE PHYSICAL BLOCK PER CONTROL INTERVAL AND THE             */
*/* I/O MANAGER WILL DYNAMICALLY MODIFY THE CCW LIST,                */
*/* WHEN NECESSARY. OPEN WILL LEAVE SPACE FOR ADDITIONAL CCW-S.      */
*/*                                                                  */
*/*                                                                  */
*/*                                                                  */
*/* (CPAWCPS POINTER MINUS 8)     (EXTRA CCW SLOT IF RPS DEVICE)     */
*/* CPAWCPS->                     SEEKH       CPAWSEEK,CC,6          */
*/*                               SRCH ID EQ  SA,CC,5                */
*/*                               TIC        *-8                     */
*/*                               WCKD        FWCA1,DC,8             */
*/*                               WCKD        DA1,(CC,IDAL),LL       */
*/*                               NOP         ,SILI,2                */
*/*                                                                  */
*/*                                                                  */
*/* CPAFWCNT->             FWCA1  |-------------------------------|  */
*/*                               |                               |  */
*/*                               | N 8 BYTE ENTRIES - USED TO    |  */
*/*                               |                               |  */
*/*                               | FORM COUNT FIELDS - CCHHRKDD  |  */
*/*                               |                               |  */
*/*                               |-------------------------------|  */
*/*                                                                  */
*/*                                                                  */
*/* THE REST OF THE ITEMS IN THE ABOVE FORMAT WRITE CHANNEL          */
*/* PROGRAM EXAMPLE ARE THE SAME AS IN THE UPDATE WRITE EXAMPLE      */
*/*                                                                  */
*/* TOTAL SPACE REQUIRED FOR CCW-S = 32*N+16 (+8 IF RPS)             */
*/*                                                                  */
*/* TOTAL BYTES REQUIRED,                                            */
*/*                 CCW     32*N+16                                  */
*/*            SID ARGS      5*N                                     */
*/*           COUNT FLDS     8*N                                     */
*/*                         -------                                  */
*/*                         45*N+16    PLUS CPAIDAL AND CPAVPL SPACE */
*/*           ADDITIONAL                                             */
*/*               IF RPS          8                                  */
*/*                                                                  */
*/*                                                                  */
*/******          WRITE CHECK CHANNEL PROGRAM SEGMENT           ******/
*/*                                                                  */
*/* EVERY READ CCW AFTER THE FIRST ONE IS A MULTI-TRACK READ.        */
*/*                                                                  */
*/*                                                                  */
*/* CPAWCKS->                      SEEKH      CPAWSEEK,CC,6          */
*/*                                SRCH ID EQ SA*,CC,5               */
*/*                                TIC        *-8                    */
*/*                            --- RD         DA1,(CC,NT),LL         */
*/*                           |    RD         DA2,(CC,NT,MT),LL      */
*/*             N READ        |     .                                */
*/*             DATA CCW-S ---|     .                                */
*/*                           |     .                                */
*/*                           |--- RD         DA(N),(CC,NT,MT),LL    */
*/*                                NOP                               */
*/*                                                                  */
*/*   *SA = CPAWCHR FOR UPDATE WRITE                                 */
*/*       = CPAFWCHR (FIRST FIVE BYTES POINTED TO BY CPAFWCNT)       */
*/*            FOR FORMAT WRITE                                      */
*/*                                                                  */
*/* BYTES REQUIRED BY CCW-S = (N+4)*8                                */
*/*                         = 8*N+32                                 */
*/*                                                                  */
*/*                                                                  */
*/********************************************************************/
*/*                                                                  */
*/*      CPA   DECLARATION                                           */
*/*                                                                  */
*%CPAPLS1: ;
* /* CPA (CHANNEL PROGRAM AREA)                                      */
 /*                                                                  */
*  DCL 1 IDACPA BDY(WORD) BASED,    /* CPA                           */
*        2 CPAID        CHAR(1),    /* CONTROL BLOCK ID              */
*        2 *            CHAR(1),    /* RESERVED                      */
*        2 CPALEN       FIXED(15),  /* CONTROL BLOCK SIZE            */
*        2 CPAWREAL     PTR(31),    /* PREVIOUS REAL ADDR WRITE CP   */
*        2 CPAWCPS      PTR(31),    /* PTR TO 1ST WRITE CCW (SEEK)   */
*        2 CPAWCPE      PTR(31),    /* PTR TO LAST WRITE CCW(NOP/TIC)*/
*        2 CPAWCKS      PTR(31),    /* PTR TO 1ST WRITE CHECK CCW    */
*        2 CPAWCKE      PTR(31),    /* PTR TO LAST WRITE CHECK CCW   */
*        2 CPARREAL     PTR(31),    /* PREVIOUS REAL ADDR READ CP    */
*        2 CPARCPS      PTR(31),    /* PTR TO 1ST READ CCW           */
*        2 CPARCPE      PTR(31),    /* PTR TO LAST READ CCW          */
*        2 CPAWPHAD     CHAR(8),    /* WRITE PHYS ADDR - MBBCCHHR    */
*          3 *          CHAR(1),
*          3 CPAWSEEK   CHAR(6),    /* WRITE SEEK ADDR               */
*            4 CPAWBB   CHAR(2),
*            4 CPAWCHR  CHAR(5),
*          3 CPAWPHR    CHAR(1),
*        2 CPAWSID      PTR(31),    /* PTR TO WRITE SRCH ID ARG LIST */
*        2 CPAFWCNT     PTR(31),    /* PTR TO FORMAT WRITE CNT FIELDS*/
*        2 CPARPHAD     CHAR(8),    /* READ PHYS ADDR - MBBCCHHR     */
*          3 *          CHAR(1),
*          3 CPARSEEK   CHAR(6),    /* READ SEEK ADDR                */
*            4 CPARBB   CHAR(2),
*            4 CPARSID  CHAR(5),    /* READ SEARCH ID ARGUMENT       */
*          3 *          CHAR(1),
*        2 CPAIDAL      PTR(31),    /* PTR TO REAL PAGE LIST         */
*        2 CPAVPL       PTR(31),    /* PTR TO VIRT PAGE LIST         */
*        2 CPAWORK1     PTR(31),    /* WORK AREA                     */
*        2 CPAWORK2     PTR(31),    /* WORK AREA                     */
*        2 CPABLKSZ     FIXED(31),  /* PHYS BLOCK SIZE FROM CONVERT  */
*        2 CPABCINV     FIXED(15),  /* NBR OF BLOCKS PER CINV        */
*        2 CPASSECT     PTR(8),     /* SET SECTOR ARG                */
*        2 CPASTAT1     BIT(8),     /* CPA STATUS                    */
*          3 CPAVPLV    BIT(1),     /* SET ON IN VPL VALID           */
*        2 CPAFLAGS     CHAR(2),    /* I/O MANAGER FLAGS             */
*          3 CPAFLAG1   CHAR(1),
*            4 CPAWV    BIT(1),     /* WRITE CHAN PROG SEG VALID     */
*            4 CPAWCV   BIT(1),     /* WRITE CHK CHAN PROG SEG VALID */
*            4 CPARV    BIT(1),     /* READ CHAN PROG SEG VALID      */
*            4 CPAWRPS  BIT(1),     /* WRITE CP SEG FOR RPS DEVICE   */
*            4 CPARRPS  BIT(1),     /* READ CP SEG FOR RPS DEVICE    */
*            4 CPACHNED BIT(1),     /* CHAINING OF CP SEGS COMPLETE  */
*            4 *        BIT(1),     /* RESERVED                      */
*          3 CPAFLAG2   CHAR(1),
*            4 CPAWREPL BIT(1),     /* REPLICATED INDEX ON WRITE     */
*            4 CPARREPL BIT(1),     /* REPLICATED INDEX ON READ      */
*            4 CPAXLRA  BIT(1),     /* LRA INSTRUCTION ERROR         */
*            4 CPAPFENT BIT(1),     /* PAGEFIX APPENDAGE ENTERED     */
*            4 CPATKOFL BIT(1),     /* TRACK OVERFLOW IN MVM @Y30DSJH*/
*        2 CPARSECT     PTR(8),     /* SET SECTOR ARG--READ  @X04SVMR*/
*        2 CPAWSECT     PTR(8),     /* SET SECTOR ARG--WRITE @X04SVMR*/
*        2 CPANXT1      PTR(31),    /* MVM STATIC CPA CHAIN  @Y30DSJH*/
*        2 CPACPCHN     PTR(31);    /* MVM DYN CPA CHAIN FLD @Y30DSJH*/
*/********************************************************************/
*/* COMMON CHANNEL PROGRAM CCWS                                      */
*/********************************************************************/
    DCL 1 CPACCW BDY(DWORD)BASED,   /* FIRST 4 CCWS          @X04SVLC*/
          2 CPACCWSK CHAR(8),       /* SEEK CCW              @X04SVLC*/
          2 CPACCWSS CHAR(8),       /* SET SECTOR            @X04SVLC*/
          2 CPACCWSI CHAR(8),       /* SEARCH ID             @X04SVLC*/
          2 CPACCWTC CHAR(8);       /* TRANSFER IN CHANNEL   @X04SVLC*/
