 /********************************************************************/
 /*                                                                  */
 /*   MACRO NAME = IDAUPT                                            */
 /*                                                                  */
 /*   DESCRIPTIVE NAME = UPGRADE TABLE.                              */
 /*                                                                  */
 /*   FUNCTION = THE UPT CONTAINS INFORMATION RELATING TO THE        */
 /*              IMMEDIATE UPGRADE SET.                              */
 /*                                                                  */
 /*   CONTROL BLOCK STRUCTURE = POINTED TO BY THE BIB (BIBUPT)       */
 /*                             WHENEVER THE IMMEDIATE UPGRADE SET   */
 /*                             IS OPENED.                           */
 /*                                                                  */
 /*   INCLUDED MACROS = NONE                                         */
 /*                                                                  */
 /*   METHOD OF ACCESS = PL/S - RESPECIFY IDAUPT BASED.              */
 /*                                                                  */
 /*   CREATED BY: VSAM OPEN MODULE IDA0192F.                         */
 /*                                                                  */
 /*   STATUS = VS1 RELEASE 4, VS2 RELEASE 3.                         */
 /*                                                                  */
 /*   DATE OF LAST CHANGE = 1 AUGUST 74                              */
 /*                                                                  */
 /********************************************************************/
    DCL
      1 IDAUPT BASED BDY(WORD),     /* UPGRADE TABLE                 */
       2 UPTHDR,                    /* UPT HEADER SECTION            */
        3 UPTID CHAR(1),            /* CONTROL BLOCK ID              */
        3 UPTFLG0 BIT(8),           /* FLAGS                         */
         4 UPTPWS BIT(1),           /* CONTINUE WITH SCAN            */
        3 UPTLEN FIXED(15),         /* LENGTH OF UPT                 */
       2 UPTNEW PTR(31),            /* ADDR OF NEW AIX RECORD        */
       2 UPTOLD PTR(31),            /* ADDR OF OLD AIX RECORD        */
       2 UPTRSC BIT(8),             /* RESOURCE BYTE -- USED TO
                                       SERIALIZE UPDATES             */
       2 UPTNOENT FIXED(8),         /* NUMBER OF AIX ENTRIES         */
       2 UPTLLEN FIXED(15),         /* LARGEST VALUE OF RKP + KEYLEN */
       2 UPTSA CHAR(72) BDY(WORD),  /* SAVE AREA                     */
        3 UPTBRPL PTR(31),          /* PTR TO BASE RPL               */
        3 UPTLSA PTR(31),           /* LAST SAVE AREA                */
        3 UPTBEREG FIXED(8),        /* BASE RPLERREG                 */
        3 UPTBERCD FIXED(8),        /* BASE RPLERRCD                 */
        3 * FIXED(8),               /* UNUSED                        */
        3 UPTSARES BIT(8),          /* UPT SAVE AREA RESOURCE BYTE   */
        3 UPTR14 PTR(31),           /* RETURN ADDR FROM IDA019R4
                                       AFTER I/O INITIATED           */
        3 UPTR15 CHAR(56) BDY(WORD),/* REST OF SAVE AREA             */
       2 UPTAXENT(*),               /* AIX ENTRY--ONE PER AIX        */
        3 UPTRPL PTR(31),           /* ADDR OF UPGRADE RPL           */
         4 UPTF1LOP BIT(8),         /* LAST OPERATION AGAINST UPG ACB*/
        3 UPTFLG1 BIT(16),          /* FLAGS                         */
         4 UPTF1LST BIT(1),         /* LAST ENTRY                    */
         4 UPTF1ATV BIT(1),         /* ENTRY IS ACTIVE FOR UPGRADE   */
         4 UPTF1NUK BIT(1),         /* NON-UNIQUE KEYS               */
         4 UPTF1NOP BIT(1),         /* AIX NOT OPEN                  */
         4 UPTF1NRF BIT(1),         /* NO RECORD FOUND ERR DETECTED  */
         4 UPTF1KEY BIT(1),         /* KEY BEING WORKED UPON
                                       0=OLD KEY 1=NEW KEY           */
         4 UPTF1RTY BIT(1),         /* LAST OPERATION BEING RETRIED  */
         4 UPTF1UPG BIT(1),         /* AIX IS IN UPGRADE             */
         4 UPTF1BKO BIT(1),         /* BACKOUT IS IN PROGRESS        */
         4 UPTF1LOG BIT(1),         /* THIS AIX HAD LOGICAL ERROR    */
         4 UPTF1PHY BIT(1),         /* THIS AIX HAD PHYSICAL ERROR   */
         4 UPTF1ERA BIT(1),         /* BASE FUNCTION WAS ERASE       */
         4 UPTF1PNU BIT(1),         /* BASE FUNCTION WAS PUT INSERT  */
         4 UPTF1PUD BIT(1),         /* BASE FUNCTION WAS PUT UPDATE  */
        3 UPTRKP FIXED(15),         /* RELATIVE KEY POSITION         */
        3 UPTPASS FIXED(8),         /* PROCESSING PASS NUMBER        */
        3 UPTLNCDE FIXED(8),        /* KEY LENGTH - ONE              */
        3 UPTBG FIXED(15);          /* LENGTH OF RPLAREA FIELD       */
    DCL
     (UPTOKEY CONSTANT('0'B),       /* OLD KEY FOR UPTF1KEY          */
      UPTNKEY CONSTANT('1'B)) BIT(1);/* NEW KEY FOR UPTF1KEY         */
    DCL
     (UPTLOPER CONSTANT('10000000'B),/* LAST OPERATION WAS ERASE     */
      UPTLOPGU CONSTANT('01000000'B),/* LAST OPERATION WAS GET UPDATE*/
      UPTLOPPU CONSTANT('00100000'B),/* LAST OPERATION WAS PUT UPDATE*/
      UPTLOPPN CONSTANT('00010000'B),/* LAST OPERATION WAS PUT NUP   */
      UPTLOPEN CONSTANT('00000000'B))/* LAST OPERATION WAS ENDREQ    */
          BIT(8);
