 /********************************************************************/
 /*                                                                  */
 /*   MACRO NAME = IDAVSRT                                           */
 /*                                                                  */
 /*   DESCRIPTIVE NAME = VSAM SHARED RESOURCE TABLE.                 */
 /*                                                                  */
 /*   FUNCTION = THE VSRT CONTAINS POINTERS TO BUFFER AND PLH POOLS  */
 /*              AS WELL AS POINTERS TO VARIOUS BLOCKS BUILT BY THE  */
 /*              BLDVRP ROUTINES.                                    */
 /*                                                                  */
 /*   CONTROL BLOCK STRUCTURE = CHAINED OFF THE VAT (LSR OPTION) OR  */
 /*                             THE AMCBS (GSR OPTION).              */
 /*                                                                  */
 /*   INCLUDED MACROS = NONE                                         */
 /*                                                                  */
 /*   METHOD OF ACCESS = PL/S - RESPECIFY IDAVSRT BASED.             */
 /*                                                                  */
 /*   CREATED BY: VSAM OPEN MODULE IDA0192Y.                         */
 /*                                                                  */
 /*   STATUS = VS2 RELEASE 4.                                @Z40ESSR*/
 /*                                                                  */
 /*   DATE OF LAST CHANGE = 31 JAN. 75.                      @Z40ESSR*/
 /*                                                                  */
 /********************************************************************/
    DCL 1 IDAVSRT BASED BDY (WORD), /* VSAM SHARED RESOURCE POOL     */
        2 VSRTBKID CHAR (1),        /* BLOCK ID                      */
        2 VSRTKEY FIXED(8),         /* KEY OF GSR USER       @Z40ESSR*/
        2 VSRTLEN FIXED (15),       /* VSRT LENGTH IN BYTES          */
        2 VSRTID CHAR (8),          /* BLOCK IDENTIFIER              */
        2 VSRTFLGS,                 /* FLAG BYTES                    */
          3 VSRTGSRF BIT (1),       /* ON INDICATES GSR OPTION       */
          3 VSRTLSRF BIT (1),       /* ON INDICATES LSR OPTION       */
          3 VSRTIOBF BIT (1),       /* ON INDICATES IOB'S FIXED      */
          3 VSRTBFRF BIT (1),       /* ON INDICATES BUFFERS FIXED    */
          3 * BIT (12),             /* RESERVED                      */
        2 VSRTKL FIXED (8),         /* MAX KEY LENGTH                */
        2 VSRTSTRN FIXED (8),       /* STRNO FROM BLDVRP MACRO       */
        2 VSRTPLHH PTR (31),        /* POINTER TO PLH HEADER         */
        2 VSRTBUFH PTR (31),        /* POINTER TO BUFFER HEADER      */
        2 VSRTCPAH PTR (31),        /* POINTER TO CPA HEADER         */
        2 VSRTWAH PTR (31),         /* POINTER TO WORKAREA HEADER    */
        2 VSRTVIOT PTR (31),        /* POINTER TO VIOT (0 IN VS1)    */
        2 VSRTASCB PTR(31),     /* ADDRS OF ORIGINATING ASCB IF GSR. */
        2 VSRTNEXT PTR(31),         /* ADDR OF NEXT VSRT             */
        2 VSRTCSL (*),              /* ENTRIES FOR GOTTEN STORAGE    */
          3 VSRTCSLF,               /* FLAGS                         */
            4 VSRTCSFX BIT (1),     /* ON INDICATES FIXED STORAGE    */
            4 VSRTCSVS BIT (1),     /* INDICATES BLOCK IS VSRT       */
            4 VSRTCSBF BIT (1),     /* INDICATES BLOCK IS BUFFER     */
              5 VSRTCSGT BIT(1), /*GSR KEY(VSRTKEY) CORE IND @Z40ESSR*/
            4 VSRTCSPF BIT (1),     /* INDICATES  PFL BLOCK          */
              5 VSRTCSVG BIT(1),    /* INDIC VGTT PREFIX BLK @Z30SSDP*/
            4 VSRTCSWS BIT (1),     /* INDICATES BLOCK IS WORKING
                                       STORAGE                       */
            4 VSRTCSPL BIT (1),     /* INDICATES BLOCK IS PLH'S      */
            4 VSRTCSIO BIT (1),     /* INDICATES BLOCK IS IOB'S      */
            4 VSRTCSBH BIT (1),     /* INDICATES BLOCK IS BUFFER     */
          3 VSRTCSAD PTR (24),      /* POINTER TO GOTTEN STORAGE     */
          3 VSRTCSSP FIXED (8),     /* SUBPOOL NUMBER                */
          3 VSRTCSLN FIXED (24);    /* LENGTH OF GOTTEN STORAGE      */
