         TITLE 'LOCK INTERFACE TABLE'
* /* START OF SPECIFICATIONS ****
*
*01*  MODULE-NAME = IEAVELIT
*
*02*     CSECT-NAME = IEAVELIT
*
*01*  DESCRIPTIVE-NAME = LOCK INTERFACE TABLE.
*
*01*  COPYRIGHT = NONE
*
*01*  STATUS = VERSION 1.
*
*01*  FUNCTION = MEANS THROUGH WHICH THE SETLOCK MACRO EXPANSION
*     INITIALIZES THE ENTRY REGISTERS AND ENTERS THE APPROPRIATE LOCK
*     ROUTINE.
*
*02*     OPERATION = NONE, MODULE IS READ ONLY.
*
*01*   NOTES = IEAVELIT IS A READ ONLY MODULE. FOR DESCRIPTION OF TABLE
*      ORGANIZATION SEE COMMENTS PRECEDING TABLE.
*
*02*     DEPENDENCIES = NONE.
*
*03*        CHARACTER-CODE-DEPENDENCIES = NONE.
*
*02*     RESTRICTIONS = NONE.
*
*02*     REGISTER-CONVENTIONS = NONE.
*
*02*     PATCH-LABEL = NONE. IEAVELIT IS A FIXED RESIDENT MODULE IN THE
*        NUCLEUS.
*
*01*  MODULE-TYPE = TABLE.
*
*02*     PROCESSOR = ASSEMBLER
*
*02*     MODULE-SIZE = 624 BYTES
*
*02*     ATTRIBUTES = NUCLEUS
*
*01*  ENTRY-POINT = IEAVELIT
*
*02*     PURPOSE = NOT ENTERED, ENTRY POINT USED AS REFERENCE POINT AS
*        BEGINNING OF TABLE.
*
*02*     LINKAGE = IEAVELIT IS NOT CALLED BUT ONLY REFERENCED AS A
*        TABLE.
*
*02*     INPUT = NONE.
*
*02*     OUTPUT = NONE.
*
*01*  EXIT-NORMAL = NONE.
*
*02*     CONDITIONS = NONE.
*
*02*     OUTPUT = NONE.
*
*02*     RETURN-CODES = NONE.
*
*01*  EXIT-ERROR = NONE.
*
*02*     CONDITIONS = NONE.
*
*02*     OUTPUT = NONE.
*
*02*     RETURN-CODES = NONE.
*
*01*  EXTERNAL-REFERENCES = (1) SYSTEM LOCKS (2) IEAVELKS ENTRIES
*  (1)   DISPLOCK - THE DISPATCHER LOCK
*        SALCLOCK - THE SALLOC LOCK
*        OPTLOCK  - THE OPTIMIZER LOCK
*        CMSLOCK  - THE CMS LOCK
*
*  (2)   GSLMCOBT  - THE GLOBAL SPIN LOCK CONDITION OBTAIN ROUTINE FOR
*        LOCKS WHOSE ADDRESS IS SUPPLIED BY THE CALLER
*        GSLSCOBT  - THE GLOBAL SPIN LOCK CONDITION OBTAIN ROUTINE FOR
*        LOCKS WHOSE ADDRESS IS KNOWN BY THE SYSTEM
*        GSLMUOBT  - THE GLOBAL SPIN LOCK UNCONDITION OBTAIN ROUTINE
*        FOR LOCKS WHOSE ADDRESS IS SUPPLIED BY THE CALLER
*        GSLSUOBT  - THE GLOBAL SPIN LOCK UNCONDITION OBTAIN ROUTINE
*        FOR LOCKS WHOSE ADDRESS IS KNOWN BY THE SYSTEM
*        GSLMREL   - THE GLOBAL SPIN LOCK RELEASE ROUTINE FOR LOCKS
*        WHOSE ADDRESS IS SUPPLIED BY THE CALLER
*        GSLSREL   - THE GLOBAL SPIN LOCK RELEASE ROUTINE FOR LOCKS
*        WHOSE ADDRESS IS KNOWN BY THE SYSTEM
*        GSLMRELD  - THE GLOBAL SPIN LOCK RELEASE DISABLE ROUTINE FOR
*        LOCKS WHOSE ADDRESS IS SUPPLIED BY THE CALLER
*        GSLSRELD  - THE GLOBAL SPIN LOCK RELEASE DISABLE ROUTINE FOR
*        LOCKS WHOSE ADDRESS IS KNOWN BY THE SYSTEM
*        CMSCOBT  - THE CMS LOCK CONDITION OBTAIN ROUTINE
*        CMSUOBT  - THE CMS LOCK UNCONDITION OBTAIN ROUTINE
*        CMSREL   - THE CMS LOCK RELEASE ROUTINE
*        LLCOBT   - THE LOCAL LOCK CONDITIONAL OBTAIN ROUTINE
*        LLUOBT   - THE LOCAL LOCK UNCONDITIONAL OBTAIN ROUTINE
*        LLREL    - THE LOCAL LOCK RELEASE ROUTINE
*        RELGS    - THE RELEASE MULTIPLE GLOBAL SPIN LOCKS ROUTINE
*
*        PSACLHT ENTRIES ARE REFERENCED AS ABSOLUTE LOW CORE ADDRESS
*        AND NOT BY EXTERNAL SYMBOLS.
*
*02*     ROUTINES = NONE.
*
*02*     DATA-AREAS = NONE.
*
*02*     CONTROL-BLOCKS = NONE.  ONLY PROVIDES ADDRESSES OF SYSTEM
*        LOCKS AND ALL LOCKS PSACLHT ENTRIES.
*
*01*  TABLES = NONE.
*
*01*  MACROS = NONE.
*
*02*    SERIALIZATION = NONE.
*
*01*  CHANGE-ACTIVITY = Y02715,Y02752
*
*01*  MESSAGES = NONE.
*
*01*  ABEND-CODES = NONE
**** END OF SPECIFICATIONS ***/
         TITLE 'LOCK INTERFACE TABLE'
IEAVELIT CSECT
***********************************************************************
*                      THE LOCK INTERFACE TABLE                       *
*        EACH TYPE OF GLOBAL SPIN LOCK HAS A MAJOR ENTRY IN THE LOCK  *
*        INTERFACE TABLE.  EACH MAJOR ENTRY FOR A GLOBAL SPIN LOCK    *
*        HAS 4 MINOR ENTRIES.  THE MINOR ENTRIES REPRESENT THE        *
*        FOLLOWING FUNCTIONS PERTAINING TO THAT LOCK:                 *
*            1)  CONDITIONAL OBTAIN                                   *
*            2)  UNCONDITIONAL OBTAIN                                 *
*            3)  RELEASE                                              *
*            4)  RELEASE DISABLE                                      *
*        EACH TYPE OF SUSPEND LOCK ALSO HAS A MAJOR ENTRY, BUT ITS    *
*        MAJOR ENTRY ONLY CONSIST OF THE FIRST 3 MINOR ENTRIES LISTED *
*        ABOVE.                                                       *
*                                                                     *
*        EACH MINOR ENTRY CONSISTS OF THREE WORDS. THESE WORDS CONTAIN*
*            1) LOCKWORD ADDRESS OR ZEROS                             *
*            2) LOCK'S ENTRY OFFSET IN CPU LOCK'S HELD TABLE IF THE   *
*               FIRST WORD IS ZERO, OR MASK TO UPDATE CPU LOCKS HELD
*               STRING IF ADDRESS IS IN FIRST WORD.
*            3) LOCK ROUTINE'S ENTRY POINT ADDRESS FOR TYPE OF REQUEST*
*                                                                     *
*        THERE IS ONE SPECIAL ENTRY PROVIDED FOR MULTIPLE RELEASE OF  *
*        SPIN LOCKS, CONTAINING THREE WORDS:
*              A) MASK FOR CHECKING OF SPIN LOCKS HELD: X'FFFFFFFC'
*              B) POINTER TO A TABLE CONTAINING ONE WORD PER LEVEL
*                 IN THE ORDER OF THE HIERARCHY. EACH WORD CONTAINS
*                 THE ADDRESS OF A WORD WHICH ITSELF POINTS TO THE
*                 LOCKWORD HELD AT THIS LEVEL IF THERE IS ONE.
*              C) ENTRY POINT ADDRESS OF RELGS, THE ROUTINE WHICH    *
*                 WILL RELEASE THE SPECIFIED SPIN LOCKS
*                                                                     *
*        THIS IS A READ ONLY STORAGE TABLE                            *
***********************************************************************
         USING FLC,0
***********************************************************************
*                          DISPATCHER LOCK                            *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
ADISP    DC    V(DISPLOCK)              DISPATCHER'S LOCK ADDRESS
         DC    A(DISP)                  MASK FOR DISP LOCK OBTAIN
         DC    V(GSLSCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    V(DISPLOCK)              DISPATCHER'S LOCK ADDRESS
         DC    A(DISP)                  MASK FOR DISP LOCK OBTAIN
         DC    V(GSLSUOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    V(DISPLOCK)              DISPATCHER'S LOCK ADDRESS
         DC    AL2(FF,FF-DISP)          MASK FOR DISP LOCK RELEASE
         DC    V(GSLSREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLED                       *****
         DC    V(DISPLOCK)              DISPACTHER'S LOCK ADDRESS
         DC    AL2(FF,FF-DISP)          MASK FOR DISP LOCK RELEASE
         DC    V(GSLSRELD)              SETLOCK'S RELEASE DISABLE RTN
***********************************************************************
*                            IOSCAT LOCK                              *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSCL-PSACLHT)      IOSCAT LOCK'S OFFSET IN CLHT
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSCL-PSACLHT)      IOSCAT LOCK'S OFFSET IN CLHT
         DC    V(GSLMUOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                          RELEASE
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSCL-PSACLHT)      IOSCAT LOCK'S OFFSET IN CLHT
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLED                       *****
         DC    A(0)                     IOS WILL FURNISH LOOK ADDR
         DC    A(PSAIOSCL-PSACLHT)      IOSCAT LOCK'S OFFSET IN CLHT
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLED RTN
***********************************************************************
*                            IOSUCB LOCK                              *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSUL-PSACLHT)      IOSUCB LOCK'S CLHT OFFSET
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSUL-PSACLHT)      IOSUCB LOCK'S CLHT OFFSET
         DC    V(GSLMUOBT)              SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSUL-PSACLHT)      IOSUCB LOCK'S CLHT OFFSET
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                     RELEASE DISABLED                        *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSUL-PSACLHT)      IOSUCB LOCK'S CLHT ENTRY
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLE RTN
***********************************************************************
*                            IOSLCH LOCK                              *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSLL-PSACLHT)      IOSLCH LOCK'S CLHT OFFSET
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSLL-PSACLHT)      IOSLCH LOCK'S CLHT OFFSET
         DC    V(GSLMUOBT)              SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSLL-PSACLHT)      IOSLCH LOCK'S CLHT OFFSET
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLE                        *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADR
         DC    A(PSAIOSLL-PSACLHT)      IOSLCH LOCK'S CLHT OFFSET
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLE RTN
***********************************************************************
*                           IOSYNCH LOCK                              *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSSL-PSACLHT)      IOSYNCH LOCK'S CLHT OFFSET
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSSL-PSACLHT)      IOSYNCH LOCK'S CLHT OFFSET
         DC    V(GSLMUOBT)              SETLOCK'S UNCOND OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADDR
         DC    A(PSAIOSSL-PSACLHT)      IOSYNCH LOCK'S CLHT OFFSET
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLE                        *****
         DC    A(0)                     IOS WILL FURNISH LOCK ADR
         DC    A(PSAIOSSL-PSACLHT)      IOSYNCH LOCK'S CLHT OFFSET
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLE RTN
***********************************************************************
*                             TPNCB LOCK                              *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPNCL-PSACLHT)      TPNCB LOCK'S CLHT OFFSET
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPNCL-PSACLHT)      TPNCB LOCK'S CLHT OFFSET
         DC    V(GSLMUOBT)              SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPNCL-PSACLHT)      TPNCB LOCK'S CLHT OFFSET
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLED                       *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPNCL-PSACLHT)      TPNCB LOCK'S CLHT OFFSET
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLE RTN
***********************************************************************
*                           TPDNCB LOCK                               *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPDNL-PSACLHT)      TPDNCB LOCK'S CLHT OFFSET
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPDNL-PSACLHT)      TPDNCB LOCK'S CLHT OFFSET
         DC    V(GSLMUOBT)              SETLOCK'S UNCOND. OBTAIN RTNEP
*****                          RELEASE                            *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPDNL-PSACLHT)      TPDNCB LOCK'S CLHT OFFSET
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLED                       *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPDNL-PSACLHT)      TPDNCB LOCK'S CLHT OFFSET
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLE RTN
***********************************************************************
*                           TPACBDEB LOCK                             *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPACL-PSACLHT)      TPACBDEB LOCK'S CLHT OFFSET
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN RTN
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPACL-PSACLHT)      TPACBDEB LOCK'S CLHT OFFSET
         DC    V(GSLMUOBT)              SETLOCK'S UNCOND. OBTAIN RTN
*****                          RELEASE                            *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPACL-PSACLHT)      TPACBDEB LOCK'S CLHT OFFSET
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                 RELEASE DISABLED                            *****
         DC    A(0)                     ATCAM WILL FURNISH LOCK ADDR
         DC    A(PSATPACL-PSACLHT)      TPACBDEB LOCK'S CLHT OFFSET
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLE RTN
***********************************************************************
*                              ASM LOCK                               *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     ASM WILL FURNISH LOCK ADDR
         DC    A(PSAASML-PSACLHT)       ASM LOCK'S CLNT OFFSET
         DC    V(GSLMCOBT)              SETLOCK'S COND. OBTAIN ROUTINE
*****                   UNCONDITIONAL OBTAIN                      *****
         DC    A(0)                     ASM WILL FURNISH LOCK ADDR
         DC    A(PSAASML-PSACLHT)       ASM LOCK'S CLNT OFFSET
         DC    V(GSLMUOBT)              SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    A(0)                     ASM WILL FURNISH LOCK ADDRESS
         DC    A(PSAASML-PSACLHT)       ASM LOCK'S CLNT OFFSET
         DC    V(GSLMREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLED                       *****
         DC    A(0)                     ASM WILL FURNISH LOCK ADDRESS
         DC    A(PSAASML-PSACLHT)       ASM LOCK'S CLNT OFFSET
         DC    V(GSLMRELD)              SETLOCK'S RELEASE DISABLED RTN
***********************************************************************
*                            SALLOC LOCK                              *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
ASALLOC  DC    V(SALCLOCK)              SALLOC LOCKWORD ADDRESS
         DC    A(SALLOC)                MASK FOR SALLOC LOCK OBTAIN
         DC    V(GSLSCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    V(SALCLOCK)              SALLOC LOCKWORD ADDRESS
         DC    A(SALLOC)                MASK FOR SALLOC LOCK OBTAIN
         DC    V(GSLSUOBT)              SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    V(SALCLOCK)              SALLOC LOCKWORD ADDRESS
         DC    AL2(FF,FF-SALLOC)        MASK FOR SALLOC LOCK RELEASE
         DC    V(GSLSREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLED                       *****
         DC    V(SALCLOCK)              SALLOC LOCKWORD ADDRESS
         DC    AL2(FF,FF-SALLOC)        MASK FOR SALLOC LOCK RELEASE
         DC    V(GSLSRELD)              SETLOCK'S RELEASE DISABLED RTN
***********************************************************************
*                              SRM LOCK                               *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
ASRM     DC    V(OPTLOCK)               SRM LOCKWORD ADDRESS
         DC    A(SRM)                   MASK FOR SRM LOCK OBTAIN
         DC    V(GSLSCOBT)              SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    V(OPTLOCK)               SRM LOCKWORD ADDRESS
         DC    A(SRM)                   MASK FOR SRM LOCK OBTAIN
         DC    V(GSLSUOBT)              SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    V(OPTLOCK)               SRM LOCKWORD ADDRESS
         DC    AL2(FF,FF-SRM)           MASK FOR SRM LOCK RELEASE
         DC    V(GSLSREL)               SETLOCK'S RELEASE RTN EP
*****                      RELEASE DISABLED                       *****
         DC    V(OPTLOCK)               SRM LOCKWORD ADDRESS
         DC    AL2(FF,FF-SRM)           MASK FOR SRM LOCK RELEASE
         DC    V(GSLSRELD)              SETLOCK'S RELEASE DISABLED RTN
***********************************************************************
*                              CMS LOCK                               *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
ACMS     DC    V(CMSLOCK)               CMS LOCKWORD ADDRESS
         DC    A(CMS)                   MASK FOR CMS LOCK OBTAIN
         DC    V(CMSCOBT)               SETLOCK'S COND. OBTAIN RTN EP
*****                    UNCONDITIONAL OBTAIN                     *****
         DC    V(CMSLOCK)               CMS LOCKWORD ADDRESS
         DC    A(CMS)                   MASK FOR CMS LOCK OBTAIN
         DC    V(CMSUOBT)               SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    V(CMSLOCK)               CMS LOCKWORD ADDRESS
         DC    AL2(FF,FF-CMS)           MASK FOR CMS LOCK RELEASE
         DC    V(CMSREL)                SETLOCK'S RELEASE RTN EP
***********************************************************************
*                             LOCAL LOCK                              *
***********************************************************************
*****                     CONDITIONAL OBTAIN                      *****
         DC    A(0)                     CURRENT ASCB WILL CONTAIN LOCK
         DC    A(LOCAL)                 MASK FOR LOCAL LOCK OBTAIN
         DC    V(LLCOBT)                SETLOCK'S COND. OBTAIN RTN EP
*****               UNCONDITIONAL OBTAIN                          *****
         DC    A(0)                     CURRENT ASCB WILL CONTAIN LOCK
         DC    A(LOCAL)                 MASK FOR LOCAL LOCK OBTAIN
         DC    V(LLUOBT)                SETLOCK'S UNCOND. OBTAIN RTN EP
*****                          RELEASE                            *****
         DC    A(0)                     CURRENT ASCB WILL CONTAIN LOCK
         DC    AL2(FF,FF-LOCAL)         MASK FOR LOCAL LOCK RELEASE
         DC    V(LLREL)                 SETLOCK'S RELEASE RTN EP
***********************************************************************
*                            RELEASE MULTIPLE SPIN LOCKS              *
***********************************************************************
         DC    X'FFFFFFFC'              MASK TO CHECK SPIN LOCKS HELD
         DC    A(IEAVELKT)              INDIRECT LOCK ADDRESSES TABLE
         DC    V(RELGS)                 RELEASE GLOBAL SPIN LOCKS
***********************************************************************
*                            INDIRECT LOCKWORD ADDRESSES TABLE        *
***********************************************************************
IEAVELKT DC    0H'0'
         ENTRY IEAVELKT
         DC    A(ADISP)           DISPATCHER LOCK ADDRESS
         DC    A(PSAASML-FLC)     SAM LOCK ADDRESS
         DC    A(ASALLOC)         SALLOC LOCK ADDRESS
         DC    A(PSAIOSSL-FLC)    IOSYNCH LOCK ADDRESS
         DC    A(PSAIOSCL-FLC)    IOSCAT LOCK ADDRESS
         DC    A(PSAIOSUL-FLC)    IOSUCB LOCK ADDRESS
         DC    A(PSAIOSLL-FLC)    IOSLCH LOCK ADDRESS
         DC    A(PSATPNCL-FLC)    TPNCB LOCK ADDRESS
         DC    A(PSATPDNL-FLC)    TPDNCB LOCK ADDRESS
         DC    A(PSATPACL-FLC)    TPACBDEB LOCK ADDRESS
         DC    A(ASRM)            SRM LOCK ADDDRESS
LOCAL    EQU   1
CMS      EQU   LOCAL*2
SRM      EQU   CMS*2
TPACBDEB EQU   SRM*2
TPDNCB   EQU   TPACBDEB*2
TPNCB    EQU   TPDNCB*2
IOSLCH   EQU   TPNCB*2
IOSUCB   EQU   IOSLCH*2
IOSCAT   EQU   IOSUCB*2
IOSYNCH  EQU   IOSCAT*2
SALLOC   EQU   IOSYNCH*2
ASM      EQU   SALLOC*2
DISP     EQU   ASM*2
FF       EQU   X'FFFF'
*****                       END OF TABLE                          *****
         TITLE 'LOCK INTERFACE TABLE - PSA EXPANSION'
         IHAPSA
         END   IEAVELIT
