***********************************************************************
*                                                                     *
*  TITLE: IEBCRANL  DATA GENERATOR  CREATE ANALYZE                    *
*                                                                     *
*  STATUS: CHANGE LEVEL 0                                             *
*  FUNCTION - ANALYZES THE CREATE CARD AND BUILDS TABLES TO HOLD      *
*              KEYWORD VALUES.                                        *
*             THIS MODULE IS ALWAYS ENTERED FROM MODULE IEBDG AFTER   *
*              A CREATE CARD IS READ.  CONTROL IS PASSED TO MODULE    *
*              IEBCREAT AFTER A SINGLE CREATE CARD IS PROCESSED OR    *
*              AFTER ALL CREATE CARDS OF A REPEAT GROUP ARE PROCESSED.*
*              IF THE CREATE CARD IS NOT THE LAST CARD OF A REPEAT    *
*              GROUP, CONTROL IS PASSED TO MODULE IEBDG TO READ THE   *
*              NEXT CREATE CARD.                                      *
*             IF AN ERROR IS FOUND IN THE CREATE CARD, CONTROL IS     *
*              PASSED TO IEBDG TO PRINT A MESSAGE, SET THE NOGO SWITCH*
*              TO STOP GENERATING OUTPUT RECORDS, AND READ THE NEXT   *
*              CONTROL CARD TO CONTINUE SYNTAX CHECKING.              *
*                                                                     *
*  ENTRY POINT: IEBCRANL THE CSECT NAME OF THIS MODULE                *
*             LINKED TO BY MODULE IEBDG.                              *
*                                                                     *
*  INPUT -    'INBUFA' - CONTROL CARD READ FROM SYSIN.                *
*             'COMMON' - WORK AREA.                                   *
*             R9       - POINTS TO START OF CREATE KEYWORD. FOR A     *
*                        CREATE CONTINUATION CARD R9 POINTS TO COLUMN *
*                        1.                                           *
*             R5       - POINTS TO 'COMMON' WORK AREA.                *
*             'CRCSW'  - IF ON, THE CONTROL CARD IS A CREATE CONT-    *
*                         UATION CARD.                                *
*                                                                     *
*  OUTPUT -   CREATE TABLE - A CREATE ENTRY IS BUILT FOR EACH CREATE  *
*              CARD.  IF A REPEAT GROUP IS PRESENT THERE WILL BE      *
*              MULTIPLE CREATE ENTRIES CHAINED TOGETHER.  ADDITIONAL  *
*              GETMAINS ARE ISSUED IF NECESSARY TO HOLD ALL CREATE    *
*              ENTRIES.                                               *
*             PICTURE TABLE - HOLDS THE PICTURE LENGTH, START         *
*              LOCATION, AND THE PICTURE VALUE.                       *
*             FD ADDRESS TABLE - HOLDS ADDRESSES OF FD TABLES FOR     *
*              EACH FD NAME SPECIFIED ON THE CREATE CARD.             *
*             EXIT NAME TABLE - HOLDS NAMES OF USER EXIT ROUTINES SO  *
*              THAT USER EXIT ROUTINES MAY BE DELETED AT END OF DSD   *
*              GROUP.                                                 *
*                                                                     *
*  EXTERNAL ROUTINES - GETMAIN MACRO.                                 *
*                      LINK MACRO.                                    *
*                      MODULE IEBCREAT.                               *
*                                                                     *
*  EXITS -    NORMAL - RETURN TO MODULE IEBDG.                        *
*                      LINK TO MODULE IEBCREAT.                       *
*             ERROR - RETURN TO MODULE IEBDG.                         *
*                      LINK TO MODULE IEBCREAT.                       *
*                                                                     *
*                                                                     *
*  TABLES/WORKAREAS                                                   *
*                                                                     *
*    CREATE TABLES - FIRST GETMAIN IS POINTED TO BY 'CRTABPT'.        *
*                    STORAGE IS GOTTEN BY THIS MODULE.  IT IS FREED   *
*                    BY IEBCREAT AFTER ALL RECORDS ARE WRITTEN.       *
*                                                                     *
*     BYTE 1 BYTE 2 BYTE 3 BYTE 4 BYTE 5 BYTE 6 BYTE 7 BYTE 8         *
*    *****************************                                    *
*    * ADDRESS OF NEXT GETMAIN   *---FIRST 4 BYTES OF EACH 512 BYTE   *
*    *****************************    GETMAIN.                        *
*    *********************************************************        *
*  0 *  ADDRESS OF NEXT CREATE   *       QUANTITY            *        *
*    *********************************************************        *
*  8 *   INPUT DCB ADDRESS       *   USER EXIT ADDRESS       *        *
*    *********************************************************        *
* 16 *   PICTURE ADDRESS         * FILL *     NOT USED       *        *
*    *********************************************************        *
* 24 *   FD ADDRESS TABLE        *                                    *
*    *****************************                                    *
*       ANY POINTER OF 0 MEANS NO TABLE OR LAST TABLE.                *
*                                                                     *
*                                                                     *
*                                                                     *
*    PICTURE TABLE - POINTED TO BY 'PICPTR' IN CREATE ENTRY.          *
*                    GETMAIN FOR PICTURE LENGTH + 6. FREED WHEN THE   *
*                    CREATE TABLES ARE FREED.                         *
*                                                                     *
*     BYTE 1 BYTE 2 BYTE 3 BYTE 4 BYTE 5 BYTE 6 BYTE 7 BYTE 8         *
*    *********************************************************        *
*    *  START LOCATION OFFSET    *  LENGTH     *  PICTURE    *        *
*    *********************************************************        *
*    *         STRING                                        *        *
*    *********************************************************        *
*                                                                     *
*                                                                     *
*                                                                     *
*    FD ADDRESS TABLE - POINTED TO BY 'FDADTAB' IN CREATE ENTRY.      *
*                       GETMAIN OF 88 BYTES. FREED WHEN THE CREATE    *
*                       TABLES ARE FREED.                             *
*                                                                     *
*    *****************************                                    *
*  O * ADDRESS OF NEXT GETMAIN   *                                    *
*    *****************************                                    *
*  4 *   FD TABLE ADDRESS        *                                    *
*    *****************************                                    *
*  8 *   FD TABLE ADDRESS        *                                    *
*    *****************************                                    *
*       LAST FD TABLE ADDRESS IN GETMAIN IS FOLLOWED BY 4 BYTES OF 0'S*
*                                                                     *
*                                                                     *
*                                                                     *
*    EXIT NAME TABLE - FIRST GETMAIN IS POINTED TO BY 'EXITTAB'.      *
*                      GETMAIN OF 72 BYTES. FREED BY MODULE IEBDG     *
*                      WHEN AN END CARD IS READ.                      *
*                                                                     *
*     BYTE 1 BYTE 2 BYTE 3 BYTE 4 BYTE 5 BYTE 6 BYTE 7 BYTE 8         *
*    *****************************                                    *
*  0 * ADDRESS OF NEXT GETMAIN   *                                    *
*    *********************************************************        *
*  4 *      USER EXIT ROUTINE NAME                           *        *
*    *********************************************************        *
* 12 *      USER EXIT ROUTINE NAME                           *        *
*    *********************************************************        *
*       LAST EXIT NAME IN GETMAIN IS FOLLOWED BY 4 BYTES OF 0'S.      *
*                                                                     *
*                                                                     *
*                                                                     *
*    DCB TABLES- KEPT FOR DSD GROUP  GETMAIN 272 BYTES FOR EACH UNIQUE*
*                 DDNAME ON CREATE CARD. INPUT DCB'S ARE CHAINED      *
*                 TOGETHER AND POINTED TO BY 'IDCBPT'. OUTPUT DCB IS  *
*                 POINTED TO BY 'ODCBPT'.                             *
*                                                                     *
*     BYTE 1 BYTE 2 BYTE 3 BYTE 4 BYTE 5 BYTE 6 BYTE 7 BYTE 8         *
*    *********************************************************        *
*    *         DCB FOR INPUT OR OUTPUT DATA SET              *        *
*    *                   256 BYTES                           *        *
*    *********************************************************        *
*    * ADDRESS OF NEXT DCB       *     DDNAME---             *        *
*    *********************************************************        *
*    * ---DDNAME                 *EODSW *                    *        *
*    *********************************************************        *
*                                                                     *
*  ATTRIBUTES: REENTRANT,REUSEABLE                                    *
*                                                                     *
***********************************************************************
*    NEW MODULE, RELEASE 17.                                       DG0H
*A818600,876600-876900                                           A38710
*A687600-687998                                                  A38781
*C687500                                                         A38781
*C206000                                                       @ZA04231
*C206000                                                       @ZA07345
         LCLA  &T,&SPN                                            0003
.@001    ANOP                                                     0003
IEBCRANL CSECT ,                                                  0003
         ST    @E,12(0,@D)                                        0003
         STM   @0,@8,20(@D)                                       0003
         STM   @A,@C,60(@D)                                       0003
         BALR  @B,0                                               0003
@PSTART  DS    0H                                                 0003
         USING @PSTART+00000,@B                                   0003
         L     @0,@SIZ001                                         0003
         GETMAIN  R,LV=(0)                                        0003
         LR    @C,@1                                              0003
         USING @DATD+00000,@C                                     0003
         LM    @0,@1,20(@D)                                       0003
         XC    @TEMPS(@L),@TEMPS                                  0003
         ST    @D,@SAV001+4                                       0003
         LA    @F,@SAV001                                         0003
         ST    @F,8(0,@D)                                         0003
         LR    @D,@F                                              0003
*  A6A1:;
*                                   /*SET 'EPSW' TO 2 IN CASE OF SUC- *
*                                   /* CESSFUL RETURN TO IEBDG.       *
*            EPSW = 2;
A6A1     MVI   568(@5),2                                          0050
*                                   /*TEST IF CREATE CARD OR CREATE */
*                                   /*CONTINUATION CARD WAS READ.  */
*         IF CRCSW = ON THEN GO TO A6A21;
         CLC   554(1,@5),ON                                       0051
         BC    08,A6A21                                           0052
*  A6A15: R9 = R9 + 6     /*POINT AT BLANK FOLLOWING CREATE KEYWORD*/;
A6A15    AH    @9,@D1                                             0053
*         IF INBUF(1) ^=' ' THEN DO;
         CLI   0(@9),C' '                                         0054
         BC    08,@9FF                                            0054
*                                   /*NO BLANK FOLLOWING OPERATION FLD.
*                                   /*SET MSG 20.
*            MS = 20;
         LA    @F,20                                              0056
         STH   @F,406(0,@5)                                       0056
*         GO TO ERRORF;
         BC    15,ERRORF                                          0057
*         END;
*  /*******************************************************************
*  /** THIS SECTION WILL SCAN OUT BLANKS ON THE CONTROL CARD.IT IS    *
*  /** LOOKING FOR THE FIRST NON-BLANK                                *
*  /*******************************************************************
*  A6A3:;
@9FF     EQU   *                                                  0059
*         R9 = R9 + 1     /*ADVANCE COLUMN POINTER.*/;
A6A3     AH    @9,@D2                                             0060
*                                   /*TEST IF PAST COLUMN 71. */
*         IF R9 > COUNTER THEN DO;
         C     @9,232(0,@5)                                       0061
         BC    12,@9FE                                            0061
*                              /***************************************
*                              /**CREATE CARD HAS NO OPERANDS. CREATE *
*                              /** ENTRY WILL CONTAIN DEFAULT VALUES  *
*                              /** OF - QUANTITY = 1                  *
*                              /**      FILL = X'00'                  *
*                              /***************************************
*                                   /*TURN ON 'CRTBLK' SWITCH. */
*            CRTBLK = ON;
         MVC   560(1,@5),ON                                       0063
*                                   /*GO TO GET SPACE FOR CREATE ENTRY.
*         GO TO A6A10;
         BC    15,A6A10                                           0064
*         END;
*                                   /*TEST IF COLUMN IS BLANK. */
*         IF INBUF(1) = ' ' THEN GO TO A6A3;
@9FE     CLI   0(@9),C' '                                         0066
         BC    08,A6A3                                            0067
*         GO TO A6A10;
         BC    15,A6A10                                           0068
* /********************************************************************
* /** CREATE CONTINUATION - THIS SECTION WILL TEST THE COMMENT CONT.  *
* /**   SW. BEING ON. IF YES, IT WILL BRANCH TO TEST FOR MORE CONTIN- *
* /**   UATIONS. IT WILL TEST FOR SPECIFIC NAME OR PICTURE SUB-       *
* /**   PARAMETER CONTINUATIONS AND BRANCH TO THE ROUTINE TO CONTINUE *
* /**   PROCESSING THE SUBPARAMETER. IF NONE OF THESE ARE FOUND THEN  *
* /**   THE CONT. CARD  BEGINS WITH A NEW KEYWORD.                    *
* /********************************************************************
*  A6A21:
*                                   /*TURN OFF CREATE CONTINUE SWITCH.*
*         CRCSW = OFF;
A6A21    MVC   554(1,@5),OFF                                      0069
*                                   /*TEST IF COMMENT CONT. SW. ON. */
*         IF COMCSW = ON THEN DO;
         CLC   565(1,@5),ON                                       0070
         BC    07,@9FD                                            0070
*                                   /*TURN OFF COMMENT CONT. SW. */
*            COMCSW = OFF;
         MVC   565(1,@5),OFF                                      0072
*                                   /*GO TO CHECK MORE CONTINUATIONS. *
*         GO TO A6P01;
         BC    15,A6P01                                           0073
*         END;
*                                   /*ADVANCE CARD COL PTR TO COL 4.*/
*            R9 = R9 + 3;
@9FD     AH    @9,@D3                                             0075
*                                   /*TEST IF PICTURE STRING CONT. */
*         IF PICCSW(3) = '1'B THEN DO;
         TM    562(@5),B'00100000'                                0076
         BC    12,@9FC                                            0076
*                                   /*TURN OFF SWITCH.*/
*            PICCSW(3) = '0'B;
         NI    562(@5),B'11011111'                                0078
*                                   /*GO TO CONTINUE PROCESSING */
*                                   /* PICTURE KEYWORD.         */
*         GO TO A6D7;
         BC    15,A6D7                                            0079
*         END;
*                    /*************************************************
*                    /**THIS SECTION WILL SCAN THE CARD BEGINNING AT  *
*                    /** COL. 4 LOOKING FOR A NON-BLANK. A PICTURE    *
*                    /** STRING CONT. IS REQUIRED TO START IN COL. 4. *
*                    /** THE FOLLOWING CONTINUATIONS MAY BEGIN IN COL.*
*                    /** 4 OR ANY COLUMN AFTER 4.                     *
*                    /*************************************************
*                                   /*TEST IF PAST COL. 71. */
*  A6A31: IF R9 > COUNTER THEN GO TO MSG21;
@9FC     EQU   *                                                  0081
A6A31    C     @9,232(0,@5)                                       0081
         BC    02,MSG21                                           0082
*                                   /*TEST IF COLUMN IS BLANK. */
*         IF INBUF(1) = ' ' THEN DO;
         CLI   0(@9),C' '                                         0083
         BC    07,@9FB                                            0083
*                                   /*STEP CARD COL PTR. */
*            R9 = R9 + 1;
         AH    @9,@D2                                             0085
*         GO TO A6A31;
         BC    15,A6A31                                           0086
*         END;
*                                   /*TEST IF NAME SUBPARAMETER CONT.*/
*         IF NAMCSW(1) = '1'B THEN DO;
@9FB     TM    561(@5),B'10000000'                                0088
         BC    12,@9FA                                            0088
*                                   /*TURN OFF SWITCH.*/
*            NAMCSW(1) = '0'B;
         NI    561(@5),B'01111111'                                0090
*                                   /*GO TO CONTINUE PROCESSING */
*                                   /* NAME KEYWORD.            */
*         GO TO A6C5;
         BC    15,A6C5                                            0091
*         END;
*                                   /*TEST IF NAME SUBPARAMETER CONT.*/
*         IF NAMCSW(2) = '1'B THEN DO;
@9FA     TM    561(@5),B'01000000'                                0093
         BC    12,@9F9                                            0093
*                                   /*TURN OFF SWITCH.*/
*            NAMCSW(2) = '0'B;
         NI    561(@5),B'10111111'                                0095
*                                   /*GO TO CONTINUE PROCESSING */
*                                   /* NAME KEYWORD.            */
*         GO TO A6C8;
         BC    15,A6C8                                            0096
*         END;
*                                   /*TEST IF PICTURE SUBPARAM. CONT.*/
*         IF PICCSW(1) = '1'B THEN DO;
@9F9     TM    562(@5),B'10000000'                                0098
         BC    12,@9F8                                            0098
*                                   /*TURN OFF SWITCH.*/
*            PICCSW(1) = '0'B;
         NI    562(@5),B'01111111'                                0100
*                                   /*GO TO CONTINUE PROCESSING */
*                                   /* PICTURE KEYWORD.         */
*         GO TO A6D4;
         BC    15,A6D4                                            0101
*         END;
*                                   /*TEST IF PICTURE SUBPARAM. CONT.*/
*         IF PICCSW(2) = '1'B THEN DO;
@9F8     TM    562(@5),B'01000000'                                0103
         BC    12,@9F7                                            0103
*                                   /*TURN OFF SWITCH.*/
*            PICCSW(2) = '0'B;
         NI    562(@5),B'10111111'                                0105
*                                   /*GO TO CONTINUE PROCESSING */
*                                   /* PICTURE KEYWORD.         */
*         GO TO A6D6;
         BC    15,A6D6                                            0106
*         END;
*                                   /*CONTINUATION IS AFTER A COMPLETE
*                                   /* PARAMETER.
*         GO TO KEYSCAN;
* /********************************************************************
* /** GET SPACE FOR A CREATE ENTRY - THIS SECTION WILL GET SPACE FOR  *
* /**   THE INITIAL CREATE TABLE OR FOR AN ADDITIONAL TABLE IF THE    *
* /**   PREVIOUS ONE IS FULL BECAUSE OF THE NUMBER OF CREATE CARDS IN *
* /**   A REPEAT GROUP. GETMAINS ARE CHAINED TOGETHER. IF A GETMAIN   *
* /**   IS ISSUED THE ENTIRE AREA IS CLEARED TO 0'S.  EACH CREATE     *
* /**   TABLE IS 512 BYTES AND EACH CREATE ENTRY IS 28 BYTES.         *
* /**      ENTRY - 'CREATESW' = OFF IF CREATE CARD ISN'T IN A REPEAT  *
* /**                 GROUP OR IF IT IS THE 1ST CARD IN A REPEAT GROUP*
* /**      EXIT  - 'CURCRGM'  POINTS TO THE CURRENT CREATE TABLE.     *
* /**              'CURCRTE'  POINTS TO THE NEW CREATE ENTRY.         *
* /********************************************************************
*                                   /*TEST IF 1ST CREATE CARD. */
*  A6A10: IF CREATESW = OFF THEN GO TO A6A11;
A6A10    CLC   552(1,@5),OFF                                      0109
         BC    08,A6A11                                           0110
*                                   /*TEST IF ENOUGH ROOM IN GETMAIN  *
*                                   /* FOR FIXED PORTION OF NEW CREATE*
*                                   /* ENTRY.                         *
*         IF CURCRGM + 512 > CURCRTE + 56 THEN DO;
         LA    @F,512                                             0111
         A     @F,320(0,@5)                                       0111
         ST    @F,@T1                                             0111
         LA    @F,56                                              0111
         A     @F,316(0,@5)                                       0111
         C     @F,@T1                                             0111
         BC    10,@9F6                                            0111
*                                   /*CHAIN NEW CREATE ENTRY TO
*                                   /* PREVIOUS ONE.
*            CURCRTE -> NXTCRTE = CURCRTE + 28;
         LA    @F,28                                              0113
         A     @F,316(0,@5)                                       0113
         L     @1,316(0,@5)
         ST    @F,0(0,@1)                                         0113
*                                   /*POINT TO NEW CREATE ENTRY.*/
*            CURCRTE = CURCRTE + 28;
         LA    @F,28                                              0114
         A     @F,316(0,@5)                                       0114
         ST    @F,316(0,@5)                                       0114
*         GO TO A6A12;
         BC    15,A6A12                                           0115
*         END;
*                                   /*ISSUE CONDITIONAL GM 512 BYTES*/
*                                   /* FOR CREATE TABLE. ADR IS     */
*                                   /* RETURNED IN 'GCADDR'.        */
*                                   /*PUT LENGTH IN GM PARAMETER LIST.
*  A6A11:    GLENGTH = 512;
@9F6     EQU   *                                                  0117
A6A11    LA    @F,512                                             0117
         ST    @F,364(0,@5)                                       0117
*                                   /*GO TO GETMAIN ROUTINE. */
*         CALL GETMAIN;
         BAL   @E,GETMAIN                                         0118
*                                   /*BASE POINTER FOR 'AREA'.*/
*            R4 = GCADDR;
         L     @4,376(0,@5)                                       0119
*         AREA(1) = '0'X            /*CLEAR GETMAIN AREA.*/;
         MVI   0(@4),X'00'                                        0120
*         AREA(2:256) = AREA(1:255);
         MVC   1(255,@4),0(@4)                                    0121
*         AREA(257:512) = AREA(256:511);
         MVC   256(256,@4),255(@4)                                0122
*                                   /*TEST IF NON-1ST CREATE CARD.*/
*         IF CREATESW = OFF THEN DO;
         CLC   552(1,@5),OFF                                      0123
         BC    07,@9F5                                            0123
*               CRTABPT = GCADDR    /*GM ADR TO CREATE TABLE PTR.*/;
         MVC   312(4,@5),376(@5)                                  0125
*               CREATESW = ON       /*TURN ON CREATE SWITCH.*/;
         MVC   552(1,@5),ON                                       0126
         BC    15,@9F4                                            0128
*         END;
*         ELSE DO;
*               R4 = CURCRGM        /*POINT TO PREVIOUS GM.*/;
@9F5     L     @4,320(0,@5)                                       0129
*               AREA(1:4) = GCADDR  /*LINK NEW GM TO PREVIOUS ONE.*/;
         MVC   0(4,@4),376(@5)                                    0130
*         END;
*         CURCRTE = GCADDR + 4      /*POINT TO NEW CREATE ENTRY.*/;
@9F4     LA    @F,4                                               0132
         A     @F,376(0,@5)                                       0132
         ST    @F,316(0,@5)                                       0132
*         CURCRGM = GCADDR          /*SAVE ADR OF CURRENT GM.*/;
         MVC   320(4,@5),376(@5)                                  0133
*                                   /*IF 'CRTBLK' SW. IS ON, THERE
*                                   /* ARE NO CREATE OPERANDS.  GO TO
*                                   /* CHECK FOR COMMENTS CONTINUATION.
*  A6A12: IF CRTBLK = ON THEN DO;
A6A12    CLC   560(1,@5),ON                                       0134
         BC    07,@9F3                                            0134
*                                   /*TURN OFF 'CRTBLK' SWITCH. */
*            CRTBLK = OFF;
         MVC   560(1,@5),OFF                                      0136
*                                   /*GO TO CHECK FOR COMMENTS CONTIN-
*                                   /* UATIONS.
*         GO TO A6P01;
         BC    15,A6P01                                           0137
*         END;
* /********************************************************************
* /** KEYWORD SCAN - THIS SECTION SCANS FOR ALL VALID CREATE CARD     *
* /**                 KEYWORDS. IF A KEYWORD IS FOUND INVALID GO TO   *
* /**                 IEBDG TO ISSUE SYNTAX MESSAGE.                  *
* /**      ENTRY - R9 POINTS TO START OF KEYWORD.                     *
* /********************************************************************
*  KEYSCAN:;
@9F3     EQU   *                                                  0139
*                                   /*TEST IF QUANTITY KEYWORD. */
*         IF INBUF(1:9) = 'QUANTITY=' THEN GO TO A6B1;
KEYSCAN  CLC   0(9,@9),@C5                                        0140
         BC    08,A6B1                                            0141
*                                   /*TEST IF NAME KEYWORD. */
*         IF INBUF(1:5) = 'NAME=' THEN GO TO A6C1;
         CLC   0(5,@9),@C6                                        0142
         BC    08,A6C1                                            0143
*                                   /*TEST IF PICTURE KEYWORD. */
*         IF INBUF(1:8) = 'PICTURE=' THEN GO TO A6D1;
         CLC   0(8,@9),@C7                                        0144
         BC    08,A6D1                                            0145
*                                   /*TEST IF FILL KEYWORD. */
*         IF INBUF(1:5) = 'FILL=' THEN GO TO A6E1;
         CLC   0(5,@9),@C8                                        0146
         BC    08,A6E1                                            0147
*                                   /*TEST IF INPUT KEYWORD. */
*         IF INBUF(1:6) = 'INPUT=' THEN GO TO A6F1;
         CLC   0(6,@9),@C9                                        0148
         BC    08,A6F1                                            0149
*                                   /*TEST IF EXIT KEYWORD. */
*         IF INBUF(1:5) = 'EXIT=' THEN GO TO A6G1;
         CLC   0(5,@9),@C10                                       0150
         BC    08,A6G1                                            0151
*                                   /*INVALID KEYWORD. */
*                                   /*SET MSG 5.       */
*            MS = 5;
         LA    @F,5                                               0152
         STH   @F,406(0,@5)                                       0152
*         GO TO ERRORF;
         BC    15,ERRORF                                          0153
* /********************************************************************
* /**     PROCESS QUANTITY KEYWORD                                    *
* /**       ENTRY - R9 POINTS TO START OF KEYWORD.                    *
* /**       EXIT  - R8 POINTS TO DELIMITER FOLLOWING QUANTITY         *
* /**                 PARAMETER.                                      *
* /**       FORMAT - QUANTITY=DD  1 TO 9 DECIMAL DIGITS.              *
* /********************************************************************
*  A6B1:
*                                   /*ADVANCE COLUMN POINTER TO START *
*                                   /* OF PARAMETER.                  *
*            R9 = R9 + 9;
A6B1     AH    @9,@D4                                             0154
*                                   /*GO TO SCAN OUT PARAMETER.*/
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0155
*                                   /*GO TO CONVERT QUANTITY TO BINARY.
*         CALL CONVDB;
         BAL   @E,CONVDB                                          0156
*                                   /*STORE QUANTITY IN CREATE ENTRY.*/
*            CURCRTE -> QUAN = R6;
         L     @1,316(0,@5)
         ST    @6,4(0,@1)                                         0157
*                                   /*QUANTITY KEYWORD HAS BEEN */
*                                   /* PROCESSED. GO TO SCAN OUT NEXT*/
*         GO TO CARDSCAN            /* KEYWORD.                      */
         BC    15,CARDSCAN                                        0158
* /********************************************************************
* /**     PROCESS NAME KEYWORD.                                       *
* /**           AFTER EACH NAME IS SCANNED, THE 'FDSRCH' ROUTINE IS   *
* /**           CALLED TO FIND THE FD TABLE AND STORE ITS ADDRESS IN  *
* /**           THE FD ADDRESS TABLE. AFTER A COPY GROUP IS COMPLETED,*
* /**           ALL THE FD ADDRESSES ASSOCIATED WITH THE COPY GROUP   *
* /**           ARE DUPLICATED IN THE FD ADR. TABLES THE NO. OF TIMES *
* /**           INDICATED BY THE COPY KEYWORD.                        *
* /**        ENTRY - R9 POINTS TO START OF KEYWORD.                   *
* /**        EXIT  - R8 POINTS TO DELIMITER FOLLOWING NAME KEYWORD.   *
* /**        FORMAT - NAME=NAME1                                      *
* /**                 NAME=(NAME1,...(COPY=VALUE,NAME2,...),...)      *
* /********************************************************************
*  A6C1:;
*                                   /*ADVANCE CARD COLUMN PTR TO START
*                                   /* OF PARAMETER.
*            R9 = R9 + 5;
A6C1     AH    @9,@D5                                             0160
*                                   /*TURN ON FIRSTSW. */
*            FIRSTSW = ON;
         MVC   637(1,@5),ON                                       0161
*                                   /*TEST IF COLUMN IS A LEFT PAREND.
*         IF INBUF(1) = '(' THEN GO TO A6C2;
         CLI   0(@9),C'('                                         0162
         BC    08,A6C2                                            0163
*                    /*************************************************
*                    /** ONLY A SINGLE FD NAME PRESENT - NO PARENDS   *
*                    /**   WERE FOUND.                                *
*                    /*************************************************
*                                   /*ONLY ONE FD NAME IS PRESENT.   */
*                                   /*GO TO SPSCAN TO SCAN OUT NAME. */
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0164
*                                   /*GO TO SEARCH FD TABLES. */
*         CALL FDSRCH;
         BAL   @E,FDSRCH                                          0165
*                                   /*NAME KEYWORD HAS BEEN PROCESSED,
*                                   /* GO TO SCAN OUT NEXT KEYWORD.
*         GO TO CARDSCAN;
         BC    15,CARDSCAN                                        0166
*                    /*************************************************
*                    /** SCAN FD NAMES IN AN OUTER PARENTHESES GROUP. *
*                    /**   THIS SECTION WILL CONTINUE TO SCAN NAMES   *
*                    /**   UNTIL A RIGHT PAREND IS FOUND. IF A LEFT   *
*                    /**   PAREND IS FOUND A BRANCH IS TAKEN TO THE   *
*                    /**   SECTION THAT SCANS THE INNER PARENTHESES   *
*                    /**   GROUP.                                     *
*                    /*************************************************
*                                   /*STEP CARD COLUMN PTR. */
*  A6C2:     R9 = R9 + 1;
A6C2     AH    @9,@D2                                             0167
*                                   /*TEST IF COLUMN IS A LEFT PAREND.
*  A6C5:  IF INBUF(1) = '(' THEN GO TO A6C6;
A6C5     CLI   0(@9),C'('                                         0168
         BC    08,A6C6                                            0169
*                                   /*GO TO SPSCAN TO SCAN OUT NAME. */
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0170
*                                   /*GO TO SEARCH FD TABLES. */
*         CALL FDSRCH;
         BAL   @E,FDSRCH                                          0171
*                                   /*SET R9 TO DELIMITER FOLLOWING */
*                                   /* NAME.                        */
*            R9 = R8;
         LR    @9,@8                                              0172
*                                   /*TEST IF COLUMN IS A RIGHT PAREND.
*  A6C55: IF INBUF(1) = ')' THEN DO;
A6C55    CLI   0(@9),C')'                                         0173
         BC    07,@9F2                                            0173
*                                   /*STEP CARD COLUMN PTR. */
*  A6C54:    R8 = R9 + 1;
A6C54    LA    @8,1                                               0175
         AR    @8,@9                                              0175
*                                   /*NAME KEYWORD HAS BEEN PROCESSED,
*                                   /* GO TO SCAN OUT NEXT KEYWORD.
*         GO TO CARDSCAN;
         BC    15,CARDSCAN                                        0176
*         END;
*                              /***************************************
*                              /**THIS SECTION TESTS IF NEXT NAME IS  *
*                              /** ON THIS CARD OR IF IT IS CONTINUED *
*                              /** ONTO NEXT CARD.                    *
*                              /***************************************
*                                   /*TEST IF COLUMN IS A COMMA. */
*         IF INBUF(1) ^= ',' THEN DO;
@9F2     CLI   0(@9),C','                                         0178
         BC    08,@9F1                                            0178
*                                   /*INVALID DELIMITER. */
*                                   /*SET MSG 3.         */
*  MSG3:     MS = 3;
MSG3     LA    @F,3                                               0180
         STH   @F,406(0,@5)                                       0180
*         GO TO ERRORF;
         BC    15,ERRORF                                          0181
*         END;
*                                   /*TEST IF COMMA IS IN COL 71. */
*         IF R9 = COUNTER THEN GO TO A6C52;
@9F1     C     @9,232(0,@5)                                       0183
         BC    08,A6C52                                           0184
*                                   /*STEP CARD COLUMN PTR. */
*            R9 = R9 + 1;
         AH    @9,@D2                                             0185
*                                   /*TEST IF COLUMN PAST COMMA IS */
*                                   /* BLANK.                      */
*         IF INBUF(1) = ' ' THEN DO;
         CLI   0(@9),C' '                                         0186
         BC    07,@9F0                                            0186
*                                   /*CONTINUATION AFTER A NAME IS */
*                                   /* INDICATED.                  */
*                                   /*SET NAMCSW1.                 */
*  A6C52:    NAMCSW(1) = '1'B;
A6C52    OI    561(@5),B'10000000'                                0188
*                                   /*TURN ON CREATE CONT. SW. */
*  A6C53:    CRCSW = ON;
A6C53    MVC   554(1,@5),ON                                       0189
*                                   /*GO TO IEBDG TO READ CONTINUATION
*                                   /* CARD.
*         GO TO RETURN2;
         BC    15,RETURN2                                         0190
*         END;
*                                   /*GO TO SCAN FOR ANOTHER NAME. */
*         GO TO A6C5;
*                    /*************************************************
*                    /** SCAN FD NAMES IN AN INNER PARENTHESES GROUP. *
*                    /**   THIS SECTION WILL SCAN NAMES UNTIL A RIGHT *
*                    /**   PAREND IS FOUND. THE COPY KEYWORD MUST     *
*                    /**   APPEAR FIRST.  WHEN A RIGHT PAREND IS FOUND*
*                    /**   ALL FD ADR'S ASSOCIATED WITH THE GROUP ARE *
*                    /**   DUPLICATED IN THE FD ADDRESS TABLE. IF     *
*                    /**   COPY = 1 NONE ARE DUPLICATED.              *
*                    /*************************************************
*                                   /*STEP CARD COLUMN PTR. */
*  A6C6:     R9 = R9 + 1;
A6C6     AH    @9,@D2                                             0193
*                                   /*TEST IF COPY KEYWORD IS PRESENT.
*         IF INBUF(1:5) ^= 'COPY=' THEN GO TO MSG3;
         CLC   0(5,@9),@C14                                       0194
         BC    07,MSG3                                            0195
*                                   /*SET SWITCH TO INDICATE 1ST NAME *
*                                   /* IN COPY GROUP.                 *
*            FRSTSW = ON;
         MVC   638(1,@5),ON                                       0196
*                                   /*CLEAR NAME COUNTER. */
*            NAMCTR = 0;
         SR    @F,@F                                              0197
         STH   @F,652(0,@5)                                       0197
*                                   /*ADVANCE CARD COL PTR TO COPY NO.*
*            R9 = R9 + 5;
         AH    @9,@D5                                             0198
*                                   /*GO TO SPSCAN TO SCAN OUT COPY */
*                                   /* VALUE.                       */
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0199
*                                   /*GO TO CONVDB TO CONVERT COPY */
*                                   /* VALUE TO BINARY.            */
*         CALL CONVDB;
         BAL   @E,CONVDB                                          0200
*                                   /*SAVE COPY VALUE. */
*            COPYVAL = R6;
         STH   @6,640(0,@5)                                       0201
*                                   /*SET R9 TO DELIMITER FOLLOWING */
*                                   /* COPY VALUE.                  */
*            R9 = R8;
         LR    @9,@8                                              0202
*                              /***************************************
*                              /**THIS SECTION TESTS IF NEXT NAME IS  *
*                              /** ON THIS CARD OR IF IT IS CONTINUED *
*                              /** ONTO NEXT CARD.                    *
*                              /***************************************
*                                   /*TEST IF COLUMN IS A COMMA. */
*  A6C61: IF INBUF(1) ^= ',' THEN GO TO MSG3;
A6C61    CLI   0(@9),C','                                         0203
         BC    07,MSG3                                            0204
*                                   /*TEST IF COMMA IS IN COL 71. */
*         IF R9 = COUNTER THEN GO TO A6C62;
         C     @9,232(0,@5)                                       0205
         BC    08,A6C62                                           0206
*                                   /*STEP CARD COLUMN PTR. */
*            R9 = R9 + 1;
         AH    @9,@D2                                             0207
*                                   /*TEST IF COLUMN PAST COMMA IS */
*                                   /* BLANK.                      */
*         IF INBUF(1) = ' ' THEN DO;
         CLI   0(@9),C' '                                         0208
         BC    07,@9EF                                            0208
*                                   /*CONTINUATION AFTER COPY OR AN */
*                                   /* FD NAME IS INDICATED.        */
*                                   /*SET NAMCSW2.                  */
*  A6C62:    NAMCSW(2) = '1'B;
A6C62    OI    561(@5),B'01000000'                                0210
*         GO TO A6C53;
         BC    15,A6C53                                           0211
*         END;
*                                   /*GO TO SPSCAN TO SCAN OUT FD NAME.
*  A6C8:  CALL SPSCAN;
@9EF     EQU   *                                                  0213
A6C8     BAL   @E,SPSCAN                                          0213
*                                   /*GO TO SEARCH FD TABLES. */
*         IF NAMCTR = 20 THEN
         LA    @F,20                                              0214
         CH    @F,652(0,@5)                                       0214
         BC    07,@9EE                                            0214
*         FRSTSW = ON;
         MVC   638(1,@5),ON                                       0215
*         CALL FDSRCH;
@9EE     BAL   @E,FDSRCH                                          0216
*                                   /*TEST IF 1ST NAME IN COPY GROUP. *
*         IF FRSTSW = ON THEN DO;
         CLC   638(1,@5),ON                                       0217
         BC    07,@9ED                                            0217
*                                   /*TURN SWITCH OFF. */
*            FRSTSW = OFF;
         MVC   638(1,@5),OFF                                      0219
*                                   /*SAVE PTR TO 1ST FD ADR OF COPY */
*                                   /* GROUP.                        */
*            COPYFD = CURFD;
         MVC   644(4,@5),352(@5)                                  0220
*                                   /*SAVE PTR TO GM FOR 1ST FD ADR. */
*            COPYFDGM = CURFDGM;
         MVC   648(4,@5),632(@5)                                  0221
*         END;
*                                   /*INCREMENT NAME CTR. */
*            NAMCTR = NAMCTR + 1;
@9ED     LA    @F,1                                               0223
         AH    @F,652(0,@5)                                       0223
         STH   @F,652(0,@5)                                       0223
*                                   /*SET R9 TO DELIMITER FOLLOWING */
*                                   /* NAME.                        */
*            R9 = R8;
         LR    @9,@8                                              0224
*                                   /*TEST IF COLUMN IS A RIGHT PAREND.
*         IF INBUF(1) ^= ')' THEN GO TO A6C61;
         CLI   0(@9),C')'                                         0225
         BC    07,A6C61                                           0226
*                                   /*DECREMENT COPY VALUE. */
*  A6C82:    COPYVAL = COPYVAL - 1;
A6C82    LH    @F,@D6                                             0227
         AH    @F,640(0,@5)                                       0227
         STH   @F,640(0,@5)                                       0227
*                                   /*TEST IF COPY VALUE IS 0. */
*         IF COPYVAL = 0 THEN DO;
         SR    @F,@F                                              0228
         CH    @F,640(0,@5)                                       0228
         BC    07,@9EC                                            0228
*                                   /*STEP CARD COLUMN PTR. */
*            R9 = R9 + 1;
         AH    @9,@D2                                             0230
*                    /*************************************************
*                    /** AT THIS POINT ALL FD NAMES OF AN INNER PAREN-*
*                    /**   THESES GROUP HAVE BEEN SCANNED. THIS SECT- *
*                    /**   ION DOES GETMAIN OF 88 BYTES FOR A NEW FD  *
*                    /**   ADR. TABLE IF NECESSARY. 'NAMCTR' CONTAINS *
*                    /**   NO. OF FD ADR'S. IN GROUP.                 *
*                    /*************************************************
*                                   /*END OF INNER PARENTHESES GROUP. *
*         GO TO A6C55;
         BC    15,A6C55                                           0231
*         END;
*                                   /*POINT TO 1ST FD ADR IN COPY GROUP
*            R4 = COPYFD;
@9EC     L     @4,644(0,@5)                                       0233
*                                   /*POINT TO GM FOR 1ST COPY NAME. */
*            R6 = COPYFDGM;
         L     @6,648(0,@5)                                       0234
*                                   /*LOAD COUNTER WITH NO. OF FD ADRS.
*                                   /* TO BE COPIED.
*            NAMCTR1 = NAMCTR;
         MVC   654(2,@5),652(@5)                                  0235
*                                   /*TEST IF ROOM IN CURRENT FD ADR */
*                                   /* GM FOR ANOTHER FD ADR ENTRY.  */
*  A6D88: IF CURFD =< CURFDGM + 76 THEN DO;
A6D88    LA    @F,76                                              0236
         A     @F,632(0,@5)                                       0236
         C     @F,352(0,@5)                                       0236
         BC    04,@9EB                                            0236
*                                   /*THERE IS ENOUGH ROOM.    */
*                                   /*UPDATE PTR TO NEW ENTRY. */
*            CURFD = CURFD + 4;
         LA    @F,4                                               0238
         A     @F,352(0,@5)                                       0238
         ST    @F,352(0,@5)                                       0238
*         GO TO A6C81;
         BC    15,A6C81                                           0239
*         END;
*                                   /*GETMAIN 88 BYTES FOR A NEW FD */
*                                   /* ADR. TABLE.                  */
*                                   /*PUT LENGTH IN GM PARAMETER LIST.
*            GLENGTH = 88;
@9EB     LA    @F,88                                              0241
         ST    @F,364(0,@5)                                       0241
*                                   /*GO TO GETMAIN ROUTINE. */
*         CALL GETMAIN;
         BAL   @E,GETMAIN                                         0242
*                                   /*CLEAR GETMAIN AREA. */
*            GCADDR -> AREA(1:88) = GCADDR -> AREA(1:88) &&
*                    GCADDR -> AREA(1:88);
         L     @1,376(0,@5)
         XC    0(88,@1),0(@1)                                     0243
*                                   /*CHAIN NEW FD ADR. TABLE TO */
*                                   /* PREVIOUS ONE.             */
*            CURFDGM -> NXTFDGM = GCADDR;
         L     @2,632(0,@5)
         MVC   0(4,@2),376(@5)                                    0244
*                                   /*UPDATE POINTER TO NEW GM. */
*            CURFDGM = GCADDR;
         MVC   632(4,@5),376(@5)                                  0245
*                                   /*UPDATE POINTER TO NEW ENTRY. */
*            CURFD = GCADDR + 4;
         LA    @F,4                                               0246
         A     @F,376(0,@5)                                       0246
         ST    @F,352(0,@5)                                       0246
*                                   /*COPY FD ADR. TO NEW ENTRY. */
*  A6C81:    CURFD -> FDADR = FDADR;
A6C81    L     @1,352(0,@5)
         MVC   0(4,@1),0(@4)                                      0247
*                                   /*DECREMENT 'NAMECTR1'. */
*            NAMCTR1 = NAMCTR1 - 1;
         LH    @F,@D6                                             0248
         AH    @F,654(0,@5)                                       0248
         STH   @F,654(0,@5)                                       0248
*                                   /*TEST IF 'NAMECTR1' IS 09 */
*         IF NAMCTR1 = 0 THEN GO TO A6C82;
         SR    @F,@F                                              0249
         CH    @F,654(0,@5)                                       0249
         BC    08,A6C82                                           0250
*                                   /*STEP PTR TO NEXT FD ADR. */
*            R4 = R4 + 4;
         AH    @4,@D7                                             0251
*                                   /*TEST IF LAST FD ADR IN THIS GM. *
*         IF FDADR ^= 0 THEN GO TO A6D88;
         C     @F,0(0,@4)                                         0252
         BC    07,A6D88                                           0253
*                                   /*LAST FD ADR. GET ADR OF NEXT GM.*
*            R6 = R6 -> NXTFDGM;
         L     @6,0(0,@6)                                         0254
*                                   /*POINT TO 1ST FD ADR. */
*            R4 = R6 + 4;
         LA    @4,4                                               0255
         AR    @4,@6                                              0255
*         GO TO A6D88;
         BC    15,A6D88                                           0256
* /********************************************************************
* /** FDSRCH - THIS SUBROUTINE WILL SCAN THE FD TABLES FOR AN EQUAL   *
* /**           NAME AND STORE THE ADR OF THE FD TABLE IN THE FD ADR  *
* /**           TABLE POINTED TO BY THE CREATE ENTRY. IF AN EQUAL NAME*
* /**           IS NOT FOUND MSG 7 IS ISSUED.                         *
* /**      ENTRY - R14 IS LINKAGE REGISTER.                           *
* /**              R9 POINTS TO START OF NAME.                        *
* /**              R7 CONTAINS LENGTH OF NAME.                        *
* /**      EXIT  - ENTRY REGISTERS ARE UNCHANGED.                     *
* /********************************************************************
*  FDSRCH:;
*                                   /*SAVE REG 14 IN COMMON AREA. */
*            RECREM = R14;
FDSRCH   ST    @E,348(0,@5)                                       0258
*                                   /*MOVE NAME TO 'Q' FOR COMPARE. */
*            Q(1:8) = INBUF(1:R7);
         LR    @1,@7                                              0259
         BCTR  @1,0                                               0259
         LR    @E,@9                                              0259
         LA    @A,216(0,@5)                                       0259
         MVI   0(@A),C' '                                         0259
         MVC   1(007,@A),0(@A)                                    0259
         EX    @1,@MVC                                            0259
*                    /*************************************************
*                    /** WHILE SCANNING FD TABLES -                   *
*                    /**   R4 POINTS TO CURRENT FD TABLE GETMAIN.     *
*                    /**   R6 POINTS TO CURRENT FD TABLE.             *
*                    /*************************************************
*                                   /*POINT TO 1ST FD TABLE GM. */
*            R4 = FDPTR1;
         L     @4,600(0,@5)                                       0260
*                                   /*TEST IF NO FD TABLES PRESENT. */
*         IF R4 = 0 THEN GO TO A6C94;
         LTR   @4,@4                                              0261
         BC    08,A6C94                                           0262
*                                   /*POINT TO 1ST FD ENTRY. */
*  A6C93:    R6 = R4;
A6C93    LR    @6,@4                                              0263
*                                   /*COMPARE NAME ON CREATE CARD TO */
*                                   /* NAME IN FD TABLE.             */
*  A6C90: IF Q(1:8) = FDNAME THEN GO TO A6C91;
A6C90    CLC   216(8,@5),0(@6)                                    0264
         BC    08,A6C91                                           0265
*                                   /*UPDATE PTR TO NEXT FD ENTRY. */
*            R6 = R6 + 64;
         AH    @6,@D8                                             0266
*                                   /*TEST IF ANOTHER FD ENTRY IN GM. *
*         IF R6 -> AREA(1:4) ^= 0 THEN GO TO A6C90;
         CLC   0(4,@6),@D9                                        0267
         BC    07,A6C90                                           0268
*                                   /*NO MORE FD ENTRIES IN THIS GM. */
*                                   /*TEST IF ANOTHER FD TABLE GM. */
*         IF R4 -> NXTFDTAB = 0 THEN DO;
         SR    @F,@F                                              0269
         C     @F,516(0,@4)                                       0269
         BC    07,@9EA                                            0269
*                                   /*NO FD TABLE FOR FD NAME SPECIFIED
*                                   /* ON CREATE CARD.
*                                   /*SET MSG 7. */
*  A6C94:    MS = 7;
A6C94    LA    @F,7                                               0271
         STH   @F,406(0,@5)                                       0271
*         GO TO ERRORF;
         BC    15,ERRORF                                          0272
*         END;
*                                   /*POINT TO NEXT FD TABLE GM. */
*            R4 = R4 -> NXTFDTAB;
@9EA     L     @4,516(0,@4)                                       0274
*         GO TO A6C93;
         BC    15,A6C93                                           0275
*                                   /*TEST IF FIRSTSW IS ON. */
*  A6C91: IF FIRSTSW = ON THEN GO TO A6C92;
A6C91    CLC   637(1,@5),ON                                       0276
         BC    08,A6C92                                           0277
*                                   /*TEST IF ENOUGH ROOM IN FD ADR */
*                                   /* TABLE FOR NEW ENTRY.         */
*         IF CURFD =< CURFDGM + 76 THEN DO;
         LA    @F,76                                              0278
         A     @F,632(0,@5)                                       0278
         C     @F,352(0,@5)                                       0278
         BC    04,@9E9                                            0278
*                                   /*THERE IS ENOUGH ROOM.        */
*                                   /*UPDATE POINTER TO NEW ENTRY. */
*            CURFD = CURFD + 4;
         LA    @F,4                                               0280
         A     @F,352(0,@5)                                       0280
         ST    @F,352(0,@5)                                       0280
*         GO TO A6C98;
         BC    15,A6C98                                           0281
*         END;
*                                   /*GETMAIN 88 BYTES FOR A NEW FD */
*                                   /* ADR. TABLE.                  */
*                                   /*PUT LENGTH IN GM PARAMETER LIST.
*  A6C92:    GLENGTH = 88;
@9E9     EQU   *                                                  0283
A6C92    LA    @F,88                                              0283
         ST    @F,364(0,@5)                                       0283
*                                   /*GO TO GETMAIN ROUTINE. */
*         CALL GETMAIN;
         BAL   @E,GETMAIN                                         0284
*                                   /*BASE POINTER FOR 'AREA'. */
*            R4 = GCADDR;
         L     @4,376(0,@5)                                       0285
*                                   /*CLEAR GETMAIN AREA. */
*            AREA(1:88) = AREA(1:88) && AREA(1:88);
         XC    0(88,@4),0(@4)                                     0286
*                                   /*TEST IF FIRSTSW IS OFF. */
*         IF FIRSTSW = OFF THEN DO;
         CLC   637(1,@5),OFF                                      0287
         BC    07,@9E8                                            0287
*                                   /*CHAIN NEW FD ADR. TABLE TO */
*                                   /* PREVIOUS ONE.             */
*            CURFDGM -> NXTFDGM = R4;
         L     @1,632(0,@5)
         ST    @4,0(0,@1)                                         0289
         BC    15,@9E7                                            0291
*         END;
*         ELSE DO;
*                                   /*STORE ADR. OF 1ST FD ADR. TABLE *
*                                   /* IN CREATE ENTRY.               *
*            CURCRTE -> FDADTAB = R4;
@9E8     L     @1,316(0,@5)
         ST    @4,24(0,@1)                                        0292
*                                   /*TURN OFF FIRSTSW. */
*            FIRSTSW = OFF;
         MVC   637(1,@5),OFF                                      0293
*         END;
*                                   /*UPDATE POINTER TO NEW GM. */
*            CURFDGM = R4;
@9E7     ST    @4,632(0,@5)                                       0295
*                                   /*UPDATE POINTER TO NEW ENTRY. */
*            CURFD = R4 + 4;
         LA    @F,4                                               0296
         AR    @F,@4                                              0296
         ST    @F,352(0,@5)                                       0296
*                                   /*STORE FD ADR. IN FD ADR. TABLE. *
*  A6C98:    CURFD -> FDADR = R6;
A6C98    L     @1,352(0,@5)
         ST    @6,0(0,@1)                                         0297
*                                   /*RESTORE REG 14. */
*            R14 = RECREM;
         L     @E,348(0,@5)                                       0298
*                                   /*RETURN TO CALLER. */
*         GO TO R14;
         BCR   15,@E                                              0299
* /********************************************************************
* /**     PROCESS PICTURE KEYWORD.                                    *
* /**        ENTRY - R9 POINTS TO START OF KEYWORD.                   *
* /**        EXIT  - R8 POINTS TO DELIMITER FOLLOWING PICTURE         *
* /**                  PARAMETERS.                                    *
* /**        FORMAT - PICTURE=LENGTH,STARTLOC,'EBCDIC STRING' OR      *
* /**                                         P'DECIMAL VALUE' OR     *
* /**                                         B'DECIMAL VALUE'        *
* /**           LENGTH IS THE NO. OF BYTES THE PICTURE FIELD WILL     *
* /**            OCCUPY IN THE LOGICAL RECORD.                        *
* /**           STARTLOC IS THE RELATIVE POSITION TO 0 IN THE LOGICAL *
* /**            RECORD WHERE THE PICTURE FIELD WILL START.           *
* /**           'P' TYPE STRING - THE DECIMAL VALUE WILL BE CONVERTED *
* /**            TO PACKED DECIMAL AND PLACED RIGHT-JUSTIFIED IN THE  *
* /**            PICTURE FIELD.                                       *
* /**           'B' TYPE STRING - THE DECIMAL VALUE WILL BE CONVERTED *
* /**            TO A BINARY NO. AND PLACED RIGHT-JUSTIFIED IN THE    *
* /**            PICTURE FIELD.                                       *
* /********************************************************************
*  A6D1:;
*                                   /*ADVANCE CARD COLUMN PTR TO START
*                                   /* OF PARAMETER.
*            R9 = R9 + 8;
A6D1     AH    @9,@D10                                            0301
*                    /*************************************************
*                    /**  PROCESS PICTURE LENGTH.                     *
*                    /*************************************************
*                                   /*GO TO SCAN OUT FIRST SUBPARAMETER
*                                   /* (PICTURE LENGTH).
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0302
*                                   /*GO TO CONVERT QUANTITY TO BINARY.
*         CALL CONVDB;
         BAL   @E,CONVDB                                          0303
*                                   /*INCREMENT PICUTRE LENGTH + 6 FOR
*                                   /* SIZE OF PICTURE TABLE.
*                                   /*PUT LENGTH IN GM PARAMETER LIST.
*            GLENGTH = R6 + 6;
         LA    @F,6                                               0304
         AR    @F,@6                                              0304
         ST    @F,364(0,@5)                                       0304
*                                   /*GO TO GETMAIN ROUTINE. */
*         CALL GETMAIN;
         BAL   @E,GETMAIN                                         0305
*                                   /*PUT GM ADDRESS IN BASE FOR */
*                                   /* PICTURE TABLE.            */
*            PICBASE = GCADDR;
         MVC   664(4,@5),376(@5)                                  0306
*                                   /*STORE PICTURE LENGTH IN PICTURE *
*                                   /* TABLE.                         *
*            PICLGTH = R6;
         L     @1,664(0,@5)
         STH   @6,4(0,@1)                                         0307
*                             /* STORE ADDR OF PICTURE TABLE */
*                                   /* IN CREATE ENTRY.              */
*            CURCRTE -> PICPTR = ADDR(CRPICT);
         L     @2,316(0,@5)
         ST    @1,16(0,@2)                                        0308
*                                   /*UPDATE CARD COL PTR TO DELIMITER
*                                   /* AFTER PICTURE LENGTH.
*            R9 = R8;
         LR    @9,@8                                              0309
*                              /***************************************
*                              /**THIS SECTION TESTS IF PICTURE START *
*                              /** LOCATION IS ON THIS CARD OR IF IT  *
*                              /** CONTINUED ONTO NEXT CARD.          *
*                              /***************************************
*                                   /*TEST IF COMMA AFTER LENGTH  */
*                                   /* SUBPARAMETER. IF NOT, GO   */
*                                   /* TO ISSUE MESSAGE #3.       */
*         IF INBUF(1) ^= ',' THEN GO TO MSG3;
         CLI   0(@9),C','                                         0310
         BC    07,MSG3                                            0311
*                                   /*TEST IF COMMA IS IN COL 71.*/
*         IF R9 = COUNTER THEN GO TO A6D2;
         C     @9,232(0,@5)                                       0312
         BC    08,A6D2                                            0313
*                                   /*STEP CARD COLUMN PTR.*/
*            R9 = R9 + 1;
         AH    @9,@D2                                             0314
*                                   /*TEST IF COLUMN PAST COMMA IS */
*                                   /* BLANK.                      */
*         IF INBUF(1) = ' ' THEN DO;
         CLI   0(@9),C' '                                         0315
         BC    07,@9E6                                            0315
*                                   /*CONTINUATION AFTER 1ST PICTURE */
*                                   /* SUBPARAMETER IS INDICATED.    */
*                                   /*SET PICCSW1.                   */
*  A6D2:     PICCSW(1) = '1'B;
A6D2     OI    562(@5),B'10000000'                                0317
*                                   /*TURN ON CREATE CONT. SW. */
*  A6D3:     CRCSW = ON;
A6D3     MVC   554(1,@5),ON                                       0318
*                                   /*GO TO IEBDG TO READ CONTINUATION
*                                   /*CARD.
*         GO TO RETURN2;
         BC    15,RETURN2                                         0319
*         END;
*                    /*************************************************
*                    /**  PROCESS PICTURE START LOCATION.             *
*                    /*************************************************
*                                   /*GO TO SCAN OUT 2ND SUBPARAMETER.*
*                                   /* (PICTURE START LOC).           *
*  A6D4:  CALL SPSCAN;
@9E6     EQU   *                                                  0321
A6D4     BAL   @E,SPSCAN                                          0321
*                                   /*GO TO CONVERT QUANTITY TO BINARY.
*         CALL CONVDB;
         BAL   @E,CONVDB                                          0322
*                                   /*STORE PICTURE START LOC. IN */
*                                   /* CREATE ENTRY.              */
*            PICSTLOC = R6 - 1;
         LR    @F,@6                                              0323
         BCTR  @F,0                                               0323
         L     @1,664(0,@5)
         ST    @F,0(0,@1)                                         0323
*                                   /*UPDATE CARD COL PTR TO DELIMITER
*                                   /* AFTER PICTURE LENGTH.
*            R9 = R8;
         LR    @9,@8                                              0324
*                              /***************************************
*                              /**THIS SECTION TESTS IF PICTURE STRING*
*                              /** IS ON THIS CARD OR IF IT IS        *
*                              /** CONTINUED ONTO NEXT CARD.          *
*                              /***************************************
*                                   /*TEST IF COMMA AFTER START LOC */
*                                   /* SUBPARAMETER. IF NOT, GO TO  */
*                                   /* ISSUE MESSAGE #3.            */
*         IF INBUF(1) ^= ',' THEN GO TO MSG3;
         CLI   0(@9),C','                                         0325
         BC    07,MSG3                                            0326
*                                   /*TEST IF COMMA IS IN COL 71.*/
*         IF R9 = COUNTER THEN GO TO A6D5;
         C     @9,232(0,@5)                                       0327
         BC    08,A6D5                                            0328
*                                   /*STEP CARD COLUMN PTR.*/
*            R9 = R9 + 1;
         AH    @9,@D2                                             0329
*                                   /*TEST IF COLUMN PAST COMMA IS */
*                                   /* BLANK.                      */
*         IF INBUF(1) = ' ' THEN DO;
         CLI   0(@9),C' '                                         0330
         BC    07,@9E5                                            0330
*                                   /*CONTINUATION AFTER 2ND PICTURE */
*                                   /* SUBPARAMETER IS INDICATED.    */
*                                   /* SET PICCSW2.                  */
*  A6D5:     PICCSW(2) = '1'B;
A6D5     OI    562(@5),B'01000000'                                0332
*                                   /*GO TO SET CRCSW.*/
*         GO TO A6D3;
         BC    15,A6D3                                            0333
*         END;
*                    /*************************************************
*                    /**  PROCESS EBCDIC PICTURE.                     *
*                    /**    PICTURE LENGTH IS USED TO DETERMINE       *
*                    /**    IF THE STRING TERMINATES ON THIS CARD OR  *
*                    /**    IF A CONTINUATION CARD IS NECESSARY TO    *
*                    /**    SATISFY LENGTH. THE STRING IS MOVED FROM  *
*                    /**    THE CARD TO THE PICTURE TABLE AS EACH CARD*
*                    /**    IS READ IF NECESSARY. 'CURPIC' POINTS TO  *
*                    /**    LOCATION IN PICTURE TABLE WHERE NEXT      *
*                    /**    PORTION OF STRING IS TO BE MOVED.         *
*                    /*************************************************
*                                   /*INITIALIZE PTR TO START OF */
*                                   /* PICTURE STRING.           */
*  A6D6:     CURPIC = ADDR(PICSTRNG);
@9E5     EQU   *                                                  0335
A6D6     L     @1,664(0,@5)
         LA    @F,6(0,@1)                                         0335
         ST    @F,324(0,@5)                                       0335
*                                   /*INITIALIZE PICCTR  TO PICTURE */
*                                   /* LENGTH.                      */
*            PICCTR = PICLGTH;
         L     @1,664(0,@5)
         LH    @F,4(0,@1)                                         0336
         ST    @F,328(0,@5)                                       0336
*                                   /*TEST IF COLUMN IS A QUOTE.*/
*         IF INBUF(1) ^= '''' THEN GO TO A6D65;
         CLI   0(@9),C''''                                        0337
         BC    07,A6D65                                           0338
*         PICCSW(4) = '1'B ;           /* SET SW. BUSY WITH PIC A41807*
         OI    562(@5),B'00010000'                                0339
*                                   /*TEST IF QUOTE IS IN COL 71.*/
*         IF R9 = COUNTER THEN GO TO A6D73;
         C     @9,232(0,@5)                                       0340
         BC    08,A6D73                                           0341
*                                   /*PICTURE STRING IS IN EBCDIC.   */
*                                   /*ADVANCE CARD COLUMN PTR.*/
*            R9 = R9 + 1;
         AH    @9,@D2                                             0342
*                                   /*TEST IF EBCDIC STRING ON THIS  */
*                                   /* CARD IS GT THE PICTURE LENGTH.*/
*         IF PICCTR => COUNTER - R9 + 1 THEN GO TO A6D68;
         LA    @F,1                                               0343
         SR    @F,@9                                              0343
         A     @F,232(0,@5)                                       0343
         C     @F,328(0,@5)                                       0343
         BC    12,A6D68                                           0344
*                                   /*REMAINDER OF PICTURE STRING IS */
*                                   /* CONTAINED ON THIS CARD.       */
*                                   /*MOVE STRING FROM CARD TO PICTURE*
*                                   /* TABLE ACCORDING TO LENGTH      *
*                                   /* REMAINING IN PICCTR.           *
*  A6D71:    CURPIC -> AREA(1:PICCTR) = INBUF;
A6D71    LR    @E,@9                                              0345
         L     @1,328(0,@5)                                       0345
         BCTR  @1,0                                               0345
         L     @2,324(0,@5)
         LR    @A,@2                                              0345
         EX    @1,@MVC                                            0345
*                                   /*ADVANCE CARD COLUMN PTR TO COL */
*                                   /* PAST PICTURE STRING.          */
*            R9 = R9 + PICCTR;
         A     @9,328(0,@5)                                       0346
*                                   /*TEST IF COLUMN IS A QUOTE. IF NOT
*                                   /* GO TO ISSUE MESSAGE #3.
*  A6D72: IF INBUF(1) ^= '''' THEN GO TO MSG3;
A6D72    CLI   0(@9),C''''                                        0347
         BC    07,MSG3                                            0348
*                                   /*SET R8 TO DELIMITER FOLLOWING */
*                                   /* LAST PICTURE PARAMETER.      */
*         PICCSW(4) = '0'B ;           /* READY WITH PICTURE    A41807*
         NI    562(@5),B'11101111'                                0349
*            R8 = R9 + 1;
         LA    @8,1                                               0350
         AR    @8,@9                                              0350
*                                   /*PICTURE KEYWORD HAS BEEN        *
*                                   /* PROCESSED. GO TO SCAN OUT NEXT *
*                                   /* KEYWORD.                       *
*         GO TO CARDSCAN;
         BC    15,CARDSCAN                                        0351
*                                   /*CALCULATE NO. OF BYTES TO BE */
*                                   /* MOVED.                      */
*  A6D68:    R6 = COUNTER - R9 + 1;
A6D68    LA    @6,1                                               0352
         SR    @6,@9                                              0352
         A     @6,232(0,@5)                                       0352
*                                   /*MOVE PICTURE STRING FROM CARD TO
*                                   /* PICTURE TABLE. STRING IS CONT-
*                                   /* AINED IN COLUMNS FROM WHERE R9
*                                   /* POINTS TO COLUMN 71.
*            CURPIC -> AREA(1:R6) = INBUF;
         LR    @E,@9                                              0353
         LR    @1,@6                                              0353
         BCTR  @1,0                                               0353
         L     @2,324(0,@5)
         LR    @A,@2                                              0353
         EX    @1,@MVC                                            0353
*                                   /*DECREMENT PICCTR BY NO. IF BYTES*
*                                   /* MOVED.                         *
*  A6D69:    PICCTR = PICCTR - R6;
A6D69    LCR   @F,@6                                              0354
         A     @F,328(0,@5)                                       0354
         ST    @F,328(0,@5)                                       0354
*                                   /*UPDATE CURPIC TO POINT TO NEXT
*                                   /* POSITION IN PICTURE TABLE STRING
*            CURPIC = CURPIC + R6;
         LR    @F,@6                                              0355
         A     @F,324(0,@5)                                       0355
         ST    @F,324(0,@5)                                       0355
*                                   /*CONTINUATION OF PICTURE STRING IS
*                                   /* INDICATED.
*                                   /*SET PICCSW3.
*  A6D73:    PICCSW(3) = '1'B;
A6D73    OI    562(@5),B'00100000'                                0356
*                                   /*GO TO SET CRCSW.*/
*            GO TO A6D3;
         BC    15,A6D3                                            0357
*                              /***************************************
*                              /** CONTROL IS TRANSFERRED TO THIS     *
*                              /**  POINT IF A CONTINUATION CARD WAS  *
*                              /**  READ.                             *
*                              /***************************************
*                                   /*TEST IF REMAINING LENGTH OF */
*                                   /* STRING IS GT 67.           */
*  A6D7:  IF PICCTR > 67 THEN DO;
A6D7     LA    @F,67                                              0358
         C     @F,328(0,@5)                                       0358
         BC    10,@9E4                                            0358
*                                   /*STRING DOESN'T TERMINATE ON THIS
*                                   /* CARD.
*                                   /*MOVE 68 CHARS FROM COL 4 TO COL
*                                   /* 71 TO PICTURE TABLE STRING.
*            CURPIC -> AREA(1:68) = INBUF;
         L     @1,324(0,@5)
         MVC   0(68,@1),0(@9)                                     0360
*                                   /*PUT NO. CHARS MOVED IN R6. */
*            R6 = 68;
         LA    @6,68                                              0361
*                                   /*LOOP BACK TO PROCESS REMAINDER */
*                                   /* OF PICTURE STRING.            */
*         GO TO A6D69;
         BC    15,A6D69                                           0362
*         END;
*                                   /*TEST IF LENGTH OF PICTURE */
*                                   /* REMAINING IS 0.          */
*         IF PICCTR = 0 THEN GO TO A6D72;
@9E4     SR    @F,@F                                              0364
         C     @F,328(0,@5)                                       0364
         BC    08,A6D72                                           0365
*                                   /*GO TO MOVE REMAINING STRING.*/
*         GO TO A6D71;
         BC    15,A6D71                                           0366
*                    /*************************************************
*                    /**  PROCESS PACKED DECIMAL OR BINARY PICTURE    *
*                    /**    STRING.                                   *
*                    /**    THESE STRINGS MUST BE CONTAINED ON ONE    *
*                    /**    CARD AND MAY NOT BE CONTINUED.            *
*                    /*************************************************
*                                   /*TEST IF START OF PICTURE STRING *
*                                   /* IS P'.                         *
*  A6D65: IF INBUF(1:2) = 'P''' THEN DO;
A6D65    CLC   0(2,@9),@C16                                       0367
         BC    07,@9E3                                            0367
*                                   /*TURN ON PICCSW5.*/
*            PICCSW(5) = '1'B;
         OI    562(@5),B'00001000'                                0369
*         GO TO A6D79;
         BC    15,A6D79                                           0370
*         END;
*                                   /*TEST IF START OF PICTURE STRING *
*                                   /* IS B'. IF NOT, GO TO SET       *
*                                   /* MESSAGE #3.                    *
*         IF INBUF(1:2) ^= 'B''' THEN GO TO MSG3;
@9E3     CLC   0(2,@9),@C17                                       0372
         BC    07,MSG3                                            0373
*                                   /*TURN ON PICCSW6. */
*            PICCSW(6) = '1'B;
         OI    562(@5),B'00000100'                                0374
*                                   /*INITIALIZE PICCTR TO 0.*/
*  A6D79:    PICCTR = 0;
A6D79    SR    @F,@F                                              0375
         ST    @F,328(0,@5)                                       0375
*                              /***************************************
*                              /** THIS SECTION SCANS THE STRING      *
*                              /**  LOOKING FOR AN ENDING QUOTE.      *
*                              /***************************************
*                                   /*ADVANCE CARD COLUMN PTR TO START
*                                   /* OF STRING.
*           R9 = R9 + 2;
         AH    @9,@D11                                            0376
*            R8 = R9;
         LR    @8,@9                                              0377
*                                   /*TEST IF COLUMN IS A QUOTE */
*                                   /* (ENDING QUOTE).          */
*  A6D8: IF R8 -> INBUF(1) = '''' THEN GO TO A6D81;
A6D8     CLI   0(@8),C''''                                        0378
         BC    08,A6D81                                           0379
*                                   /*STEP PICCTR BY 1.*/
*            PICCTR = PICCTR + 1;
         LA    @F,1                                               0380
         A     @F,328(0,@5)                                       0380
         ST    @F,328(0,@5)                                       0380
*                                   /*TEST IF PICCTR IS GT 16.*/
*         IF PICCTR > 16 THEN DO;
         LA    @F,16                                              0381
         C     @F,328(0,@5)                                       0381
         BC    10,@9E2                                            0381
*                                   /*PACKED DECIMAL OR BINARY PICTURE*
*                                   /* TOO LONG.                      *
*                                   /*SET MSG 8. */
*           MS = 8;
         LA    @F,8                                               0383
         STH   @F,406(0,@5)                                       0383
*         GO TO ERRORF;
         BC    15,ERRORF                                          0384
*         RETURN;
*         END;
*                                   /*TEST IF AT COL 71.*/
*        IF R8 => COUNTER THEN DO;
@9E2     C     @8,232(0,@5)                                       0387
         BC    04,@9E1                                            0387
*                                  /*PARAMETER EXTENDS INTO COL 72. */
*                                   /*SET MSG 21. */
*           MS = 21;
         LA    @F,21                                              0389
         STH   @F,406(0,@5)                                       0389
*         GO TO ERRORF;
         BC    15,ERRORF                                          0390
*         END;
*                                   /*ADVANCE CARD COLUMN PTR.*/
*            R8 = R8 + 1;
@9E1     AH    @8,@D2                                             0392
*                                   /*LOOP BACK TO SCAN NEXT COLUMN.*/
*         GO TO A6D8;
         BC    15,A6D8                                            0393
*                                   /*TEST IF BYTE COUNTER IS 0. IF SO,
*                                   /* GO TO SET MESSAGE #6.
*  A6D81: IF PICCTR = 0 THEN DO;
A6D81    SR    @F,@F                                              0394
         C     @F,328(0,@5)                                       0394
         BC    07,@9E0                                            0394
*                                  /*NO STRING SUPPLIED. */
*                                   /*GO TO SET MSG 3. */
*         GO TO MSG3;
         BC    15,MSG3                                            0396
*        END;
*                              /***************************************
*                              /** PICTURE STRING IN PICTURE TABLE    *
*                              /**  MUST BE CLEARED TO ZEROES TO PRO- *
*                              /**  VIDE LEADING ZEROES FOR THE VALUE.*
*                              /***************************************
*                                   /*GET LENGTH OF PICTURE STRING. */
*            R6 = PICLGTH;
@9E0     L     @1,664(0,@5)
         LH    @6,4(0,@1)                                         0398
*                                   /*GET ADDRESS OF PICTURE STRING. */
*            R7 = ADDR(PICSTRNG);
         LA    @7,6(0,@1)                                         0399
*                                   /*TEST IF LENGTH IS GT 256. */
*  A6D83: IF R6 > 256 THEN DO;
A6D83    CH    @6,@D12                                            0400
         BC    12,@9DF                                            0400
*                                   /*CLEAR 256 BYTES. */
*            GEN( XC  0(256,R7),0(R7));
          XC  0(256,R7),0(R7)
         DS    0H
*                                   /*DECREMENT LENGTH BY 256. */
*            R6 = R6 - 256;
         SH    @6,@D12                                            0403
*                                   /*INCREMENT ADDRESS BY 256. */
*            R7 = R7 + 256;
         AH    @7,@D12                                            0404
*                                   /*LOOP BACK TO CLEAR MORE. */
*         GO TO A6D83;
         BC    15,A6D83                                           0405
*         END;
*                                   /*PUT LENGTH - 1 INTO R6. */
*            R6 = R6 - 1;
@9DF     BCTR  @6,0                                               0407
*                                   /*CLEAR REMAINDER OF AREA. */
*            GEN( EX  R6,CLEAR);
          EX  R6,CLEAR
         DS    0H
*         GO TO A6D87;
         BC    15,A6D87                                           0409
*  CLEAR:    GEN( XC  0(1,R7),0(R7));
CLEAR     XC  0(1,R7),0(R7)
         DS    0H
*                                   /*FIND LENGTH OF STRING ON CARD. */
*  A6D87:    R7 = R8 - R9;
A6D87    LCR   @7,@9                                              0411
         AR    @7,@8                                              0411
*                              /***************************************
*                              /** CALL 'CONVDB' ROUTINE.             *
*                              /**   PACKED DECIMAL STRING - NO. IS   *
*                              /**     PACKED INTO 'Q'.               *
*                              /**   BINARY STRING - NO. IS PACKED AND*
*                              /**     CONVERTED TO BINARY IN REG 6.  *
*                              /***************************************
*         CALL CONVDB;
         BAL   @E,CONVDB                                          0412
*                                   /*TEST IF PACKED DECIMAL STRING.*/
*         IF PICCSW(5) = '0'B THEN GO TO A6D84;
         TM    562(@5),B'00001000'                                0413
         BC    08,A6D84                                           0414
*                    /*************************************************
*                    /**  PROCESS PACKED DECIMAL PICTURE.             *
*                    /**    PACKED NO. IS MOVED FROM 'Q' INTO PICTURE *
*                    /**    AND RIGHT-JUSTIFIED.                      *
*                    /*************************************************
*                                   /*PICTURE IS PACKED DECIMAL. */
*                                   /*CLEAR SWITCH.*/
*            PICCSW(5) = '0'B;
         NI    562(@5),B'11110111'                                0415
*                                   /*CALCULATE NO. OF BYTES TO MOVE */
*                                   /* BY, LENGTH OF STRING ON CARD  */
*                                   /* / 2 + 1.                      */
*            R6 = R7 / 2 + 1;
         LR    @E,@7                                              0416
         SRA   @E,1                                               0416
         AH    @E,@D2                                             0416
         LR    @6,@E                                              0416
*                                   /*MOVE PACKED DECIMAL NUMBER FROM *
*                                   /* Q TO PICTURE TABLE.            *
*                                   /*MAKE SIGN OF PACKED DECML NO. A *
*                                   /* STANDARD PLUS SIGN.            *
*            Q(16) = Q(16) & 'FC'X;
         NI    231(@5),X'FC'                                      0417
*                                   /*CALCULATE FROM ADDRESS. */
*            R9 = ADDR(Q) + 16 - R6;
         LCR   @F,@6                                              0418
         AH    @F,@D13                                            0418
         LA    @0,216(0,@5)                                       0418
         AR    @F,@0                                              0418
         LR    @9,@F                                              0418
*                                   /*CALCULATE TO ADDRESS. */
*            R7 = ADDR(PICSTRNG) + PICLGTH - R6;
         LCR   @F,@6                                              0419
         L     @1,664(0,@5)
         AH    @F,4(0,@1)                                         0419
         LA    @0,6(0,@1)                                         0419
         AR    @F,@0                                              0419
         LR    @7,@F                                              0419
*            R7 -> AREA(1:R6) = R9 -> AREA(1:R6);
         LR    @E,@9                                              0420
         LR    @2,@6                                              0420
         BCTR  @2,0                                               0420
         LR    @A,@7                                              0420
         EX    @2,@MVC                                            0420
*                                   /*UPDATE CARD COLUMN PTR TO      */
*                                   /* DELIMITER AFTER STRING QUOTE. */
*  A6D85:    R8 = R8 + 1;
A6D85    AH    @8,@D2                                             0421
*                                   /*PICTURE KEYWORD HAS BEEN        *
*                                   /* PROCESSED. GO TO SCAN OUT NEXT *
*                                   /* KEYWORD.                       *
*         GO TO CARDSCAN;
         BC    15,CARDSCAN                                        0422
*                    /*************************************************
*                    /**  PROCESS BINARY PICTURE.                     *
*                    /**    BINARY VALUE IS PLACED RIGHT JUSTIFIED IN *
*                    /**      THE PICTURE TABLE.                      *
*                    /*************************************************
*                                   /*PICTURE IS BINARY. */
*                                   /*CLEAR SWITCH. */
*  A6D84:    PICCSW(6) = '0'B;
A6D84    NI    562(@5),B'11111011'                                0423
*                                   /*SET R7 TO PICTURE LENGTH. */
*            R7 = PICLGTH;
         L     @1,664(0,@5)
         LH    @7,4(0,@1)                                         0424
*                                   /*TEST IF LENGTH IS EQ OR GT 4. */
*         IF R7 => 4 THEN DO;
         CH    @7,@D7                                             0425
         BC    04,@9DE                                            0425
*                                   /*ADJUST R7 FOR A MOVE OF 4 BYTES.
*            R7 = 4;
         LA    @7,4                                               0427
*         GO TO A6D86;
         BC    15,A6D86                                           0428
*         END;
*                                   /*TEST IF LENGTH IS 3. */
*         IF R7 = 3 THEN DO;
@9DE     CH    @7,@D3                                             0430
         BC    07,@9DD                                            0430
*                                   /*SHIFT NO. LEFT 1 BYTE. */
*            GEN( SLL  R6,8);
          SLL  R6,8
         DS    0H
*         GO TO A6D86;
         BC    15,A6D86                                           0433
*         END;
*                                   /*TEST IF LENGTH IS 2. */
*         IF R7 = 2 THEN DO;
@9DD     CH    @7,@D11                                            0435
         BC    07,@9DC                                            0435
*                                   /*SHIFT NO. LEFT 2 BYTES. */
*            GEN( SLL  R6,16);
          SLL  R6,16
         DS    0H
*         GO TO A6D86;
         BC    15,A6D86                                           0438
*         END;
*                                   /*LENGTH IS 1. SHIFT NO. LEFT 3 */
*                                   /* BYTES.                       */
*            GEN( SLL  R6,24);
@9DC     EQU   *                                                  0440
          SLL  R6,24
         DS    0H
*                                   /*MOVE BINARY VALUE TO Q. */
*  A6D86:    Q(1:4) = R6;
A6D86    ST    @6,@TEMP4                                          0441
         MVC   216(4,@5),@TEMP4                                   0441
*                                   /**********************************
*                                   /** MOVE BINARY VALUE TO PICTURE  *
*                                   /**   TABLE.                      *
*                                   /**********************************
*                                   /*CALCULATE TO ADDRESS. */
*            R6 = ADDR(PICSTRNG) + PICLGTH - R7;
         LCR   @F,@7                                              0442
         L     @1,664(0,@5)
         AH    @F,4(0,@1)                                         0442
         LA    @0,6(0,@1)                                         0442
         AR    @F,@0                                              0442
         LR    @6,@F                                              0442
*            R6 -> AREA(1:R7) = Q(1:R7);
         LA    @E,216(0,@5)                                       0443
         LR    @2,@7                                              0443
         BCTR  @2,0                                               0443
         LR    @A,@6                                              0443
         EX    @2,@MVC                                            0443
*         GO TO A6D85;
         BC    15,A6D85                                           0444
* /********************************************************************
* /**    PROCESS FILL KEYWORD.                                        *
* /**       ENTRY - R9 POINTS TO START OF KEYWORD.                    *
* /**       EXIT  - R8 POINTS TO DELIMITER FOLLOWING FILL PARAMETER.  *
* /**       FORMAT - FILL='C'    1 EBCDIC CHAR.                       *
* /**                FILL=X'XX'  2 HEX DIGITS.                        *
* /********************************************************************
*  A6E1:;
*                                   /*ADVANCE COLUMN POINTER TO START *
*                                   /* OF PARAMETER.                  *
*            R9 = R9 + 5;
A6E1     AH    @9,@D5                                             0446
*                                   /*TEST IF AT OR PAST COL 72.*/
*         IF R9 > COUNTER THEN GO TO A6E4;
         C     @9,232(0,@5)                                       0447
         BC    02,A6E4                                            0448
*                                   /*TEST IF HEX FORMAT.*/
*         IF INBUF(1) = 'X' THEN GO TO A6E5;
         CLI   0(@9),C'X'                                         0449
         BC    08,A6E5                                            0450
*                    /*************************************************
*                    /**  FILL CHAR. IS EBCDIC.                       *
*                    /**    CHARACTER IS MOVED DIRECTLY FROM CARD TO  *
*                    /**    CREATE ENTRY.                             *
*                    /*************************************************
*                                   /*FILL IS EBCDIC CHAR.*/
*                                   /*TEST IF QUOTE.      */
*         IF INBUF(1) = '''' THEN GO TO A6E3;
         CLI   0(@9),C''''                                        0451
*                                   /*IMPROPER SYNTAX. */
*                                   /*GO TO SET MSG 3. */
*         GO TO MSG3;
         BC    07,MSG3                                            0453
*                                   /*TEST IF QUOTE.*/
*  A6E3:  IF INBUF(3) ^= '''' THEN GO TO MSG3;
A6E3     CLI   2(@9),C''''                                        0454
         BC    07,MSG3                                            0455
*                                   /*PUT FILL CHAR IN CREATE ENTRY.*/
*  A6E35:    CURCRTE -> FILLCH = INBUF(2);
A6E35    SR    @F,@F                                              0456
         IC    @F,1(0,@9)                                         0456
         L     @1,316(0,@5)
         STC   @F,20(0,@1)                                        0456
*                                   /*POINT TO END OF PARAMETER.*/
*            R8 = R9 + 3;
         LA    @8,3                                               0457
         AR    @8,@9                                              0457
*                                   /*TEST IF NOT AT COL 72. IF NOT,
*                                   /* FILL KEYWORD HAS BEEN PROCESSED.
*                                   /* GO TO SCAN NEXT KEYWORD.
* A6E36: IF R8 ^> COUNTER+1             /* NOT PAST COLUMN 72   A38781*
*             THEN DO ;                 /* CHECK IF AT COL 72   A38781*
A6E36    LA    @F,1                                               0458
         A     @F,232(0,@5)                                       0458
         CR    @F,@8                                              0458
         BC    04,@9DB                                            0458
*                 IF R8 = COUNTER+1     /* POINTER JUST AT COL72A38781*
*                    THEN DO ;          /* CHECK IF CONTINUATIONA38781*
         LA    @F,1                                               0460
         A     @F,232(0,@5)                                       0460
         CR    @F,@8                                              0460
         BC    07,@9DA                                            0460
*                       IF R8 -> INBUF(1) ^= ' '  /* REQUIRED   A38781*
*                          THEN DO ;    /* COMMENT CONTINUATION A38781*
         CLI   0(@8),C' '                                         0462
         BC    08,@9D9                                            0462
*                             CRCSW = ON ; /* CREATE CONT.SW. ONA38781*
         MVC   554(1,@5),ON                                       0464
*                             COMCSW = ON ; /*CREATE COMMENT SW A38781*
         MVC   565(1,@5),ON                                       0465
*                             GO TO RETURN2 ; /* READ NEXT CARD A38781*
         BC    15,RETURN2                                         0466
*                       END ;           /* END OF CONT. CHECK   A38781*
*                       GO TO CREATE ;  /* NO CONT.SO END CREATEA38781*
*                 END ;                 /* NOT IN COL72,BUT LESSA38781*
*                 GO TO CARDSCAN ;      /* THEREFORE GO ON SCANNING   *
*                                      /* CREATE CARD           A38781*
*           END ;                       /* POINTER PAST COL 72  A38781*
*  A6E4:     MS = 21                /*PARAMETER EXTENDS INTO COL 72.*/;
@9DB     EQU   *                                                  0472
A6E4     LA    @F,21                                              0472
         STH   @F,406(0,@5)                                       0472
*         GO TO ERRORF;
         BC    15,ERRORF                                          0473
*  A6E5:
*                    /*************************************************
*                    /**  FILL CHAR. IS HEX.                          *
*                    /**    EACH CHAR. IS TESTED FOR VALIDITY AND THEN*
*                    /**     IS CONVERTED TO HEX.                     *
*                    /*************************************************
*                                   /*HEX FORMAT. TEST IF QUOTE.*/
*         IF INBUF(2) ^= '''' THEN GO TO MSG3;
A6E5     CLI   1(@9),C''''                                        0474
         BC    07,MSG3                                            0475
*                                   /*TEST IF QUOTE.*/
*         IF INBUF(5) ^= '''' THEN GO TO MSG3;
         CLI   4(@9),C''''                                        0476
         BC    07,MSG3                                            0477
*                                   /*SET FIRSTSW. */
*            FIRSTSW = ON;
         MVC   637(1,@5),ON                                       0478
*                                   /*STEP CARD COL PTR TO 1ST HEX CHAR
*            R9 = R9 + 2;
         AH    @9,@D11                                            0479
*                                   /*TEST IF CHAR IS LT A '0'. */
*  A6E52: IF INBUF(1) < '0' THEN GO TO A6E6;
A6E52    CLI   0(@9),C'0'                                         0480
         BC    04,A6E6                                            0481
*                                   /*TEST IF CHAR IS GT A '9'. */
*         IF INBUF(1) > '9' THEN GO TO MSG3;
         CLI   0(@9),C'9'                                         0482
         BC    02,MSG3                                            0483
*                                   /*SET R6 TO LO-ORDER DIGIT OF CHAR.
*            R6 = INBUF(1) & '0F'X;
         LA    @F,X'0F'                                           0484
         SR    @0,@0                                              0484
         IC    @0,0(0,@9)                                         0484
         NR    @F,@0                                              0484
         LR    @6,@F                                              0484
*                                   /*TEST IF FIRSTSW IS ON. */
*  A6E53: IF FIRSTSW = ON THEN DO;
A6E53    CLC   637(1,@5),ON                                       0485
         BC    07,@9D8                                            0485
*                                   /*TURN OFF FIRSTSW. */
*            FIRSTSW = OFF;
         MVC   637(1,@5),OFF                                      0487
*                                   /*SHIFT DIGIT LEFT 4 BITS. */
*            R6 = R6 * 16;
         SLA   @6,4                                               0488
*                                   /*PUT DIGIT IN CREATE ENTRY. */
*            CURCRTE -> FILLCH = R6;
         L     @1,316(0,@5)
         STC   @6,20(0,@1)                                        0489
*                                   /*STEP CARD COL PTR. */
*            R9 = R9 + 1;
         AH    @9,@D2                                             0490
*                                   /*LOOP BACK TO PROCESS 2ND DIGIT. *
*         GO TO A6E52;
         BC    15,A6E52                                           0491
*         END;
*                                   /*PUT 2ND DIGIT IN CREATE ENTRY. */
*            CURCRTE -> FILLCH = CURCRTE -> FILLCH | R6;
@9D8     LR    @F,@6                                              0493
         L     @1,316(0,@5)
         SR    @0,@0                                              0493
         IC    @0,20(0,@1)                                        0493
         OR    @F,@0                                              0493
         STC   @F,20(0,@1)                                        0493
*                                   /*STEP CARD COL PTR TO DELIMITER */
*                                   /* FOLLOWING FILL PARAMETER.     */
*            R8 = R9 + 2;
         LA    @8,2                                               0494
         AR    @8,@9                                              0494
*                                   /*HEX FILL HAS BEEN PROCESSED. */
*         GO TO A6E36;
         BC    15,A6E36                                           0495
*                                   /*TEST IF CHAR IS LT AN 'A'. */
*  A6E6:  IF INBUF(1) < 'A' THEN GO TO MSG3;
A6E6     CLI   0(@9),C'A'                                         0496
         BC    04,MSG3                                            0497
*                                   /*TEST IF CHAR IS GT AN 'F'. */
*         IF INBUF(1) > 'F' THEN GO TO MSG3;
         CLI   0(@9),C'F'                                         0498
         BC    02,MSG3                                            0499
*                                   /*SET R6 TO LO-ORDER DIGIT OF CHAR.
*            R6 = INBUF(1) & '0F'X;
         LA    @F,X'0F'                                           0500
         SR    @0,@0                                              0500
         IC    @0,0(0,@9)                                         0500
         NR    @F,@0                                              0500
         LR    @6,@F                                              0500
*                                   /*ADD 9 TO OBTAIN HEX EQUIVALENCE.
*            R6 = R6 + 9;
         AH    @6,@D4                                             0501
*         GO TO A6E53;
         BC    15,A6E53                                           0502
* /********************************************************************
* /**     PROCESS INPUT KEYWORD.                                      *
* /**        ENTRY - R9 POINTS TO START OF KEYWORD.                   *
* /**        EXIT  - R8 POINTS TO DELIMITER FOLLOWING DDNAME PARAMETER*
* /**        FORMAT - INPUT=DDNAME OR                                 *
* /**                       =SYSIN IMPLIES A $$$E INPUT DELIMETER OR  *
* /**                      =SYSIN(DDDD) EXPLICIT 1 TO 4 CHAR DELIMITER*
* /********************************************************************
*  A6F1:;
*                                   /*ADVANCE COLUMN POINTER TO START *
*                                   /* OF PARAMETER.                  *
*            R9 = R9 + 6;
A6F1     AH    @9,@D1                                             0504
*                                   /*GO TO SCAN OUT PARAMETER.*/
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0505
*                                   /*TEST IF LENGTH OF DDNAME IS GT */
*                                   /* 8 CHARS.                      */
*         IF R7 > 8 THEN DO;
         CH    @7,@D10                                            0506
         BC    12,@9D7                                            0506
*                                   /*DDNAME IS TOO LONG.*/
*            MS = 12                /*SET MSG 12.        */;
         LA    @F,12                                              0508
         STH   @F,406(0,@5)                                       0508
*         GO TO ERRORF;
         BC    15,ERRORF                                          0509
*         END;
*                                   /*MOVE DDNAME INTO 'Q' FOR COMPARE.
*            Q = INBUF(1:R7);
@9D7     LR    @1,@7                                              0511
         BCTR  @1,0                                               0511
         LR    @E,@9                                              0511
         LA    @A,216(0,@5)                                       0511
         MVI   0(@A),C' '                                         0511
         MVC   1(015,@A),0(@A)                                    0511
         EX    @1,@MVC                                            0511
*                                   /*TEST IF DDNAME IS 'SYSIN'.*/
*         IF Q(1:8) = 'SYSIN   ' THEN GO TO A6F2;
         CLC   216(8,@5),@C24                                     0512
         BC    08,A6F2                                            0513
*                    /*************************************************
*                    /**  DDNAME IS NOT 'SYSIN'. SCAN ALL INPUT DCB'S *
*                    /**    FOR AN EQUAL DDNAME. ADDRESS OF DCB IS    *
*                    /**    THEN STORED IN THE CREATE ENTRY.          *
*                    /*************************************************
*                                   /*SET 'DCBPTR' TO ADDRESS OF FIRST
*                                   /* INPUT DCB.
*            DCBPTR = FIRSTGMI;
         MVC   300(4,@5),392(@5)                                  0514
*                                   /*TEST IF NO INPUT DCB'S.     */
*                                   /*IF NONE, GO TO ISSUE MSG 4. */
*         IF DCBPTR = 0 THEN GO TO A6F25;
         SR    @F,@F                                              0515
         C     @F,300(0,@5)                                       0515
         BC    08,A6F25                                           0516
*                                   /*TEST IF DDNAME ON CREATE CARD IS
*                                   /* EQUAL TO DDNAME IN THIS INPUT
*                                   /* DCB.
*  A6F3:  IF Q(1:8) = DDNAME1(1:8) THEN DO;
A6F3     L     @1,300(0,@5)
         CLC   216(8,@5),260(@1)                                  0517
         BC    07,@9D6                                            0517
*                                   /*PUT ADDRESS OF INPUT DCB IN */
*                                   /* CREATE ENTRY.              */
*            CURCRTE -> IDCBPTR = DCBPTR;
         L     @2,316(0,@5)
         MVC   8(4,@2),300(@5)                                    0519
*                                   /*DDNAME KEYWORD HAS BEEN PROCESSED
*                                   /*GO TO SCAN OUT NEXT KEYWORD.
*         GO TO CARDSCAN;
         BC    15,CARDSCAN                                        0520
*         END;
*                                   /*DDNAMES AREN'T EQUAL.*/
*                                   /*TEST IF PTR TO NEXT INPUT DCB IS
*                                   /* 0.
*         IF NEXTDCB = 0 THEN DO;
@9D6     SR    @F,@F                                              0522
         L     @1,300(0,@5)
         C     @F,256(0,@1)                                       0522
         BC    07,@9D5                                            0522
*                                   /*DDNAME ON CREATE CARD WASN'T */
*                                   /* FOUND IN INPUT DCB'S.       */
*                                   /*SET MSG 4.                   */
*  A6F25:    MS = 4;
A6F25    LA    @F,4                                               0524
         STH   @F,406(0,@5)                                       0524
*         GO TO ERRORF;
         BC    15,ERRORF                                          0525
*         END;
*                                   /*SET 'DCBPTR' TO ADDRESS OF NEXT *
*                                   /* INPUT DCB.                     *
*            DCBPTR = NEXTDCB;
@9D5     L     @1,300(0,@5)
         MVC   300(4,@5),256(@1)                                  0527
*                                   /*LOOP BACK TO TEST NEXT INPUT DCB.
*         GO TO A6F3;
         BC    15,A6F3                                            0528
*                    /*************************************************
*                    /**  DDNAME IS 'SYSIN'.  ADDRESS OF SYSIN DCB    *
*                    /**    WHICH RESIDES IN 'COMMON' AREA IS STORED  *
*                    /**    IN THE CREATE ENTRY.                      *
*                    /*************************************************
*                                   /*STORE ADR OF SYSIN DCB IN CREATE
*                                   /* ENTRY.
*  A6F2:     CURCRTE -> IDCBPTR = ADDR(SYSI);
A6F2     LA    @F,116(0,@5)                                       0529
         L     @1,316(0,@5)
         ST    @F,8(0,@1)                                         0529
*                                   /*TEST IF LEFT  PAREND FOLLOWS */
*                                   /* DDNAME.                     */
*       IF R8 -> INBUF(1) ^= '(' THEN DO;
         CLI   0(@8),C'('                                         0530
         BC    08,@9D4                                            0530
*                              /***************************************
*                              /** NO EXPLICIT SYSIN DELIMITER.       *
*                              /***************************************
*                                   /*SET DELIMITER TO DEFAULT OF  */
*                                   /* '$$$E'                      */
*            DELIM = '$$$E';
         MVC   344(4,@5),@C25                                     0532
         BC    15,@9D3                                            0534
*         END;
*                              /***************************************
*                              /** THERE IS A SYSIN DELIMITER SPEC-   *
*                              /**  IFIED.  DELIMITER IS SCANNED AND  *
*                              /**  SAVED IN 'DELIM'.                 *
*                              /***************************************
*         ELSE DO                   /*SET R9 TO START OF DELIMITER.*/;
*            R9 = R8 + 1;
@9D4     LA    @9,1                                               0535
         AR    @9,@8                                              0535
*                                   /*CALL SPSCAN TO SCAN OUT DELIMITER
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0536
*                                   /*TEST IF LENGTH OF DELIMITER IS */
*                                   /* GT 4 CHARACTERS.              */
*         IF R7 > 4 THEN DO;
         CH    @7,@D7                                             0537
*                                   /*DELIMITER IS TOO LONG.*/
*                                   /*GO TO SET MSG 3. */
*         GO TO MSG3;
         BC    03,MSG3                                            0539
*         END;
*                                   /*MOVE DELIMITER TO 'DELIM'.*/
*            DELIM = INBUF(1:R7);
@9D2     LR    @1,@7                                              0541
         BCTR  @1,0                                               0541
         LR    @E,@9                                              0541
         LA    @A,344(0,@5)                                       0541
         MVI   0(@A),C' '                                         0541
         MVC   1(003,@A),0(@A)                                    0541
         EX    @1,@MVC                                            0541
*                                   /*POINT TO COLUMN FOLLOWING RIGHT *
*                                   /* PAREND.                        *
*            R8 = R8 + 1;
         AH    @8,@D2                                             0542
*         END;
*                                   /*DDNAME KEYWORD HAS BEEN PROCESSED
*                                   /*GO TO SCAN OUT NEXT KEYWORD.
*         GO TO CARDSCAN;
         BC    15,CARDSCAN                                        0544
* /********************************************************************
* /**    PROCESS EXIT KEYWORD.                                        *
* /**     THIS SECTION WILL SCAN OUT THE NAME OF THE USER'S EXIT      *
* /**      ROUTINE AND LOAD THAT ROUTINE INTO STORAGE. THE ADDRESS OF *
* /**      THE ROUTINE WILL BE IN THE CREATE ENTRY.                   *
* /**      IF THE USER'S EXIT ROUTINE IS NOT IN THE LIBRARY, THE LOAD *
* /**      MACRO WILL ABEND.                                          *
* /**       ENTRY - R9 POINTS TO START OF KEYWORD.                    *
* /**       EXIT  - R8 POINTS TO DELIMETER FOLLOWING EXIT NAME.       *
* /**       FORMAT - EXIT=NAME  1 TO 8 A/N CHARACTERS.                *
* /********************************************************************
*  A6G1:;
*                                   /*ADVANCE COLUMN POINTER TO START*/
*                                   /* OF PARAMETER.                 */
*            R9 = R9 + 5;
A6G1     AH    @9,@D5                                             0546
*                                   /*GO TO SCAN OUT PARAMETER.*/
*         CALL SPSCAN;
         BAL   @E,SPSCAN                                          0547
*                                   /*TEST IF EXITNAME IS GT 8 BYTES.*/
*         IF R7 > 8 THEN DO;
         CH    @7,@D10                                            0548
         BC    12,@9D1                                            0548
*                                   /*EXIT NAME IS GT 8 CHARS. */
*                                   /*SET MSG 12.              */
*            MS = 12;
         LA    @F,12                                              0550
         STH   @F,406(0,@5)                                       0550
*         GO TO ERRORF;
         BC    15,ERRORF                                          0551
*         END;
*                    /*************************************************
*                    /**  THIS SECTION WILL GETMAIN 72 BYTES FOR AN   *
*                    /**    INITIAL EXIT NAME TABLE OR FOR AN ADDIT-  *
*                    /**    IONAL TABLE IF NECESSARY.                 *
*                    /*************************************************
*                                   /*TEST IF EXITSW IS OFF. */
*         IF EXITSW = OFF THEN GO TO A6G2;
@9D1     CLC   555(1,@5),OFF                                      0553
         BC    08,A6G2                                            0554
*                                   /*TEST IF ENOUGH ROOM IN EXITNAME *
*                                   /* TABLE FOR NEW ENTRY.           *
*         IF CUREXIT + 8 => EXITGM + 68 THEN GO TO A6G2;
         LA    @F,8                                               0555
         A     @F,340(0,@5)                                       0555
         ST    @F,@T1                                             0555
         LA    @F,68                                              0555
         A     @F,336(0,@5)                                       0555
         C     @F,@T1                                             0555
         BC    12,A6G2                                            0556
*                                   /*POINT TO NEXT EXITNAME ENTRY.*/
*            CUREXIT = CUREXIT + 8;
         LA    @F,8                                               0557
         A     @F,340(0,@5)                                       0557
         ST    @F,340(0,@5)                                       0557
*         GO TO A6G5;
         BC    15,A6G5                                            0558
*                                   /*ISSUE A GM FOR 72 BYTES FOR */
*                                   /* INITIAL EXITNAME TABLE.    */
*                                   /*PUT LENGTH IN GM PARAMETER LIST.
*  A6G2:     GLENGTH = 72;
A6G2     LA    @F,72                                              0559
         ST    @F,364(0,@5)                                       0559
*                                   /*GO TO GETMAIN ROUTINE. */
*         CALL GETMAIN;
         BAL   @E,GETMAIN                                         0560
*                                   /*BASE POINTER FOR 'AREA'.*/
*            R4 = GCADDR;
         L     @4,376(0,@5)                                       0561
*                                   /*CLEAR GETMAIN AREA.*/
*            AREA(1) = '0'X;
         MVI   0(@4),X'00'                                        0562
*            AREA(2:72) = AREA(1:71);
         MVC   1(71,@4),0(@4)                                     0563
*                                   /*TEST IF EXITSW IS OFF. */
*         IF EXITSW = OFF THEN DO;
         CLC   555(1,@5),OFF                                      0564
         BC    07,@9D0                                            0564
*                                   /*EXITNAME TABLE PTR = ADR OF GM.*/
*            EXITTAB = GCADDR;
         MVC   332(4,@5),376(@5)                                  0566
*                                   /*TURN ON EXITSW. */
*            EXITSW = ON;
         MVC   555(1,@5),ON                                       0567
         BC    15,@9CF                                            0569
*         END;
*         ELSE DO;
*                                   /*CHAIN PREVIOUS EXITNAME TABLE TO
*                                   /* NEW ONE.
*            EXITGM -> NXTEXGM = GCADDR;
@9D0     L     @1,336(0,@5)
         MVC   0(4,@1),376(@5)                                    0570
*         END;
*                                   /*SET CURRENT EXITNAME PTR TO ADR *
*                                   /* OF GETMAIN + 4.                *
*            CUREXIT = GCADDR + 4;
@9CF     LA    @F,4                                               0572
         A     @F,376(0,@5)                                       0572
         ST    @F,340(0,@5)                                       0572
*                                   /*SET CURRENT EXITNAME GETMAIN TO *
*                                   /* ADR OF GETMAIN.                *
*            EXITGM = GCADDR;
         MVC   336(4,@5),376(@5)                                  0573
*                    /*************************************************
*                    /**  THIS SECTION WILL MOVE THE USER'S EXIT      *
*                    /**    ROUTINE NAME TO THE EXIT NAME TABLE. IT   *
*                    /**    THEN LOADS THE USER'S ROUTINE INTO STORAGE*
*                    /*************************************************
*                                   /*MOVE USER'S EXIT ROUTINE NAME */
*                                   /* INTO EXITNAME TABLE.         */
*  A6G5:     CUREXIT -> EXITNAME = INBUF(1:R7);
A6G5     LR    @1,@7                                              0574
         BCTR  @1,0                                               0574
         LR    @E,@9                                              0574
         L     @2,340(0,@5)
         LR    @A,@2                                              0574
         MVI   0(@A),C' '                                         0574
         MVC   1(007,@A),0(@A)                                    0574
         EX    @1,@MVC                                            0574
*                                   /*POINT TO 'EXITNAME'.*/
*            R6 = CUREXIT;
         L     @6,340(0,@5)                                       0575
*                                   /*LOAD USER'S EXIT ROUTINE. ADDRESS
*                                   /* OF ROUTINE IS RETURNED IN REG 0.
*         GEN( LOAD  EPLOC=(6));
          LOAD  EPLOC=(6)
         DS    0H
*                                   /*STORE EXIT ADR IN CREATE ENTRY.*/
*            CURCRTE -> EXITADR = R0;
         L     @1,316(0,@5)
         ST    @0,12(0,@1)                                        0577
*                                   /*EXIT KEYWORD HAS BEEN PROCESSED*/
*                                   /* GO TO SCAN OUT NEXT KEYWORD.  */
*         GO TO CARDSCAN;
         BC    15,CARDSCAN                                        0578
* /********************************************************************
* /** SCAN NEXT KEYWORD - PREVIOUS KEYWORD HAS BEEN PROCESSED. THIS   *
* /**           SECTION WILL SCAN FOR DELIMITER FOLLOWING PREVIOUS    *
* /**           KEYWORD. IT WILL CHECK FOR OPERAND CONTINUATION OR    *
* /**           COMMENTS CONTINUATION AND IF SO, GO TO IEBDG TO READ  *
* /**           THE NEXT CARD.                                        *
* /**      ENTRY - R8 POINTS TO DELIMITER FOLLOWING LAST PARAMETER    *
* /**                THAT WAS PROCESSED.                              *
* /**      EXIT - TO KEYWORD SCAN(KEYSCAN)                            *
* /**             R9 POINTS TO START OF NEXT KEYWORD TO BE PROCESSED. *
* /********************************************************************
*  CARDSCAN:;
*                                   /*SET R9 = R8. R8 POINTS TO DELIMIT
*                                   /* ER FOLLOWING PARAMETER.
*            R9 = R8;
CARDSCAN LR    @9,@8                                              0580
*                                   /*TEST IF COLUMN IS BLANK.*/
*         IF INBUF(1) = ' ' THEN GO TO A6P01;
         CLI   0(@9),C' '                                         0581
         BC    08,A6P01                                           0582
*                                   /*TEST IF COLUMN IS A COMMA.*/
*         IF INBUF(1) = ',' THEN GO TO A6P1;
         CLI   0(@9),C','                                         0583
         BC    08,A6P1                                            0584
*                                   /*DELIMETER FOLLOWING A COMPLETE */
*                                   /* PARAMETER IS NOT A BLANK OR   */
*                                   /* COMMA.                        */
*        IF R9 = COUNTER THEN GO TO A6P2 ; /* CONTINUATION     A38710 *
         C     @9,232(0,@5)                                       0585
         BC    08,A6P2                                            0586
*                                   /*GO TO ISSUE MESSAGE NO. 3.     */
*         GO TO MSG3;
         BC    15,MSG3                                            0587
*  A6P1:  IF R9 = COUNTER THEN DO;
A6P1     C     @9,232(0,@5)                                       0588
         BC    07,@9CE                                            0588
*                                   /*TURN ON CREATE CONTINUE SW.*/
*  A6P2:     CRCSW = ON;
A6P2     MVC   554(1,@5),ON                                       0590
*                                   /*GO TO IEBDG TO READ CREATE */
*                                   /* CONTINUATION CARD.        */
*         GO TO RETURN2;
         BC    15,RETURN2                                         0591
*         END;
*                                   /*ADVANCE CARD COLUMN PTR.*/
*            R9 = R9 + 1;
@9CE     AH    @9,@D2                                             0593
*                                   /*TEST IF COLUMN IS BLANK.*/
*         IF INBUF(1) = ' ' THEN GO TO A6P2;
         CLI   0(@9),C' '                                         0594
         BC    08,A6P2                                            0595
*                                   /*READY TO PROCESS NEXT KEYWORD.*/
*         GO TO KEYSCAN;
         BC    15,KEYSCAN                                         0596
*                    /*************************************************
*                    /**  AT THIS POINT ALL OPERAND KEYWORDS HAVE     *
*                    /**    BEEN PROCESSED. THERE ARE COMMENTS CONTIN-*
*                    /**    UATION ON ANOTHER CARD IF COL. 72 IS NOT  *
*                    /**    BLANK.                                    *
*                    /*************************************************
*                                   /*TEST IF COL 72 NOT BLANK.*/
*  A6P01: IF COUNTER -> AREA(2) ^= ' ' THEN DO;
A6P01    L     @1,232(0,@5)
         CLI   1(@1),C' '                                         0597
         BC    08,@9CD                                            0597
*                                   /*TURN ON CREATE CONTINUE SW. */
*            CRCSW = ON;
         MVC   554(1,@5),ON                                       0599
*            COMCSW = ON            /*TURN ON COMMENTS CONT. SW.*/;
         MVC   565(1,@5),ON                                       0600
*                                   /*GO TO IEBDG TO READ COMMENTS */
*                                   /*CONTINUATION CARD.          */
*         GO TO RETURN2;
         BC    15,RETURN2                                         0601
*         END;
*                    /*************************************************
*                    /**  ALL CONTINUATION CARDS THAT ARE PRESENT     *
*                    /**    HAVE BEEN PROCESSED.  SET DEFAULTS FOR    *
*                    /**    QUANTITY AND FILL IF NOT SPECIFIED ON THE *
*                    /**    CREATE CARD.                              *
*                    /*************************************************
*                                   /*TEST IF QUANTITY = 0.*/
*  CREATE:IF CURCRTE -> QUAN = 0 THEN
@9CD     EQU   *                                                  0603
CREATE   L     @1,316(0,@5)
         CLC   4(4,@1),@D9                                        0603
         BC    07,@9CC                                            0603
*                                   /*TEST IF NO INPUT DCB. */
*         IF CURCRTE -> IDCBPTR = 0 THEN
         SR    @F,@F                                              0604
         C     @F,8(0,@1)                                         0604
         BC    07,@9CB                                            0604
*                                   /*SET QUANTITY = 1        */
*            CURCRTE -> QUAN = 1;
         MVC   4(4,@1),@D14                                       0605
*                                   /*DECREMENT NO. OF CREATE CARDS */
*                                   /* REMAINING.                   */
*            CREATENO = CREATENO - 1;
@9CB     EQU   *                                                  0606
@9CC     LH    @F,@D6                                             0606
         AH    @F,18(0,@5)                                        0606
         STH   @F,18(0,@5)                                        0606
*                                   /*TEST IF MORE CARDS IN A REPEAT */
*                                   /* GROUP.                        */
*         IF CREATENO > 0 THEN DO;
         SR    @F,@F                                              0607
         CH    @F,18(0,@5)                                        0607
         BC    10,@9CA                                            0607
*                                   /*GO TO IEBDG TO READ NEXT CREATE*/
*                                   /* CARD.                         */
*         GO TO RETURN2;
         BC    15,RETURN2                                         0609
*         END;
* /********************************************************************
* /**  LINK TO IEBCREAT MODULE -  CONTROL IS PASSED TO IEBCREAT TO    *
* /**           WRITE THE OUTPUT RECORDS FOR A SINGLE CREATE CARD OR  *
* /**           FOR MULTIPLE CREATE CARDS IF A REPEAT GROUP.          *
* /********************************************************************
*                                   /*TURN OFF 'REPEATSW' TO INDICATE *
*                                  /* REPEAT GROUP IS COMPLETED.     */
*           REPEATSW = OFF;
@9CA     MVC   578(1,@5),OFF                                      0611
*  LINK:     GEN( LINK  EP=IEBCREAT);
LINK      LINK  EP=IEBCREAT
         DS    0H
* /********************************************************************
* /**  RETURN FROM IEBCREAT MODULE.                                   *
* /********************************************************************
*         GO TO RETURN2;
* /********************************************************************
* /**  NORMAL RETURN.                                                 *
* /**     RETURN TO MODULE IEBDG.                                     *
* /********************************************************************
*                                   /*RETURN TO IEBDG. */
*  RETURN2: RETURN;
         BC    15,@EL01                                           0614
* /********************************************************************
* /**   ERROR RETURNS. -  CONTROL IS PASSED TO MODULE IEBCREAT TO     *
* /**                      TO FREE TABLES. CONTROL IS RETURNED TO     *
* /**                      MODULE IEBDG TO PRINT THE APPROPRIATE      *
* /**                      MESSAGE.                                   *
* /********************************************************************
*                                   /*TURN ON ERROR FLAG SW. */
*  ERRORF:   FLAGSW(1) = '1'B;
ERRORF   OI    566(@5),B'10000000'                                0615
*                                   /*SET 'EPSW' TO INDICATE ERROR. */
*  ERROR:    EPSW = 1;
ERROR    MVI   568(@5),1                                          0616
*                                   /*SET CONDITION CODE TO 8. */
*            CONCODE = 8;
         LA    @F,8                                               0617
         STH   @F,306(0,@5)                                       0617
*                                   /*TURN ON 'NOGO' SWITCH. */
*            NOGOSW = ON;
         MVC   551(1,@5),ON                                       0618
*                                   /*GO TO LINK TO IEBCREAT. */
*         GO TO LINK;
         BC    15,LINK                                            0619
* /********************************************************************
* /** SPSCAN - THIS SUBROUTINE WILL SCAN OUT A PARAMETER.             *
* /**           IT SCANS FOR A DELIMITER OF COMMA, BLANK, LEFT PAREND *
* /**           OR RIGHT PAREND.                                      *
* /**           IT CHECKS FOR A FIELD LENGTH OF 0, AND IF SO, ISSUES  *
* /**           MESSAGE #6.                                           *
* /**      ENTRY - R14 IS LINKAGE REGISTER.                           *
* /**              R9 POINTS TO START OF PARAMETER.                   *
* /**      EXIT  - R9 NO CHANGE.                                      *
* /**              R8 POINTS TO DELIMETER FOLLOWING PARAMETER.        *
* /**              R7 CONTAINS LENGTH OF PARAMETER.                   *
* /********************************************************************
*  SPSCAN:;
*                                   /*SAVE REG 14 IN COMMON AREA. */
*            SAVE14 = R14;
SPSCAN   ST    @E,360(0,@5)                                       0621
*            R8 = R9                /*INITIALIZE CARD COL PTR.*/;
         LR    @8,@9                                              0622
*  A6M11: IF R8 > COUNTER THEN DO;
A6M11    C     @8,232(0,@5)                                       0623
         BC    12,@9C9                                            0623
*  MSG21:    MS = 21                /*PARAMETER EXTENDS INTO COL 72.*/;
MSG21    LA    @F,21                                              0625
         STH   @F,406(0,@5)                                       0625
*         GO TO ERRORF;
         BC    15,ERRORF                                          0626
*         END;
*                                   /*TEST IF COMMA.*/
*  A6M12: IF PARAM(1) = ',' THEN GO TO A6M13;
@9C9     EQU   *                                                  0628
A6M12    CLI   0(@8),C','                                         0628
         BC    08,A6M13                                           0629
*                                   /*TEST IF BLANK.*/
*         IF PARAM(1) = ' ' THEN GO TO A6M13;
         CLI   0(@8),C' '                                         0630
         BC    08,A6M13                                           0631
*                                   /*TEST IF LEFT PAREND.*/
*         IF PARAM(1) = '(' THEN GO TO A6M13;
         CLI   0(@8),C'('                                         0632
         BC    08,A6M13                                           0633
*                                   /*TEST IF RIGHT PAREND.*/
*         IF PARAM(1) = ')' THEN GO TO A6M13;
         CLI   0(@8),C')'                                         0634
         BC    08,A6M13                                           0635
*            R8 = R8 + 1            /*ADVANCE COLUMN POINTER.*/;
         AH    @8,@D2                                             0636
*        IF R8 = COUNTER + 1 THEN DO ;    /* AT COL 72         A38710 *
         LA    @F,1                                               0637
         A     @F,232(0,@5)                                       0637
         CR    @F,@8                                              0637
         BC    07,@9C8                                            0637
*              IF R8 -> INBUF(1) ^= ' ' THEN  /* NO CONT.CHHAR A38710 *
         CLI   0(@8),C' '                                         0639
*                   GO TO A6P2 ;                            /* A38710 *
         BC    07,A6P2                                            0640
*              END ;                                        /* A38710 *
*         GO TO A6M11;
         BC    15,A6M11                                           0642
*                                   /*CALCULATE LENGTH OF PARAMETER.*/
*  A6M13:    R7 = R8 - R9;
A6M13    LCR   @7,@9                                              0643
         AR    @7,@8                                              0643
*                                   /*TEST IF PARAMETER LENGTH IS 0.*/
*         IF R7 = 0 THEN DO;
         LTR   @7,@7                                              0644
*                                   /*PARAMETER LENGTH IS 0. */
*                                   /*GO TO ISSUE MSG 3.     */
*         GO TO MSG3;
         BC    08,MSG3                                            0646
*         END;
*                                   /*RESTORE REG 14 FROM COMMON AREA.
*            R14 = SAVE14;
@9C7     L     @E,360(0,@5)                                       0648
*                                   /*RETURN TO CALLER.*/
*         GO TO R14;
         BCR   15,@E                                              0649
* /********************************************************************
* /** CONVDB - THIS SUBROUTINE WILL PACK A DECIMAL FIELD FROM THE CARD*
* /**           AND CONVERT IT TO BINARY. IT CHECKS FOR A MAXIMUM     *
* /**           DECIMAL VALUE OF 2,147,483,647 OR A VALUE OF 0, AND   *
* /**           IF SO, ISSUES MESSAGE #6.                             *
* /**      ENTRY - R14 IS LINKAGE REGISTER.                           *
* /**              R9 POINTS TO START OF PARAMETER.                   *
* /**              R7 CONTAINS LENGTH OF PARAMETER.                   *
* /**      EXIT  - R6 CONTAINS CONVERTED BINARY VALUE.                *
* /**              R7 NO CHANGE.                                      *
* /**              R9 NO CHANGE.                                      *
* /********************************************************************
*  CONVDB:;
*  A6N11:
*                                   /*SAVE REG 14 IN COMMON AREA. */
*            SAVE14 = R14;
CONVDB   EQU   *                                                  0651
A6N11    ST    @E,360(0,@5)                                       0651
*                                   /*TEST IF LENGTH OF VALUE ON CARD *
*                                  /* IS GT 16 BYTES.                */
*        IF R7 > 16 THEN GO TO A6N20;
         CH    @7,@D13                                            0652
         BC    02,A6N20                                           0653
*                                  /*PUT LENGTH - 1 IN R6. */
*            R6 = R7 - 1;
         LH    @6,@D6                                             0654
         AR    @6,@7                                              0654
*                                  /*CLEAR 'Q' TO ZEROS. */
*            Q = Q && Q;
         XC    216(16,@5),216(@5)                                 0655
*                                  /*MOVE ZONES ONLY OF CHARACTERS ON *
*                                  /* CARD INTO 'Q'.                  *
*           GEN( EX  R6,MVZON );
          EX  R6,MVZON
         DS    0H
*                              /***************************************
*                              /** CHECK CHARS. ON CARD FOR NUMERIC   *
*                              /**  BY TESTING ALL ZONES FOR AN 'F'.  *
*                              /***************************************
*         IF Q(1:R7) = FOXZEROS(1:R7) THEN GO TO A6N12;
         LA    @E,FOXZEROS                                        0657
         LR    @1,@7                                              0657
         BCTR  @1,0                                               0657
         LA    @A,216(0,@5)                                       0657
         EX    @1,@CLC                                            0657
         BC    08,A6N12                                           0658
*                                  /*IF CHARS ARE INVALID ISSUE MSG 3 *
*         GO TO MSG3;
         BC    15,MSG3                                            0659
*                                   /*PACK QUANTITY INTO Q.*/
*  A6N12:    GEN( EX  R6,PACK);
A6N12     EX  R6,PACK
         DS    0H
*                              /***************************************
*                              /** IF PACKED DECIMAL PICTURE RETURN   *
*                              /**  TO CALLER.  PACKED NO. IS IN 'Q'. *
*                              /***************************************
*         IF PICCSW(5) = '1'B THEN GO TO A6N26;
         TM    562(@5),B'00001000'                                0661
         BC    01,A6N26                                           0662
*                                  /*TEST IF LENGTH GT 10. */
*        IF R7 > 10 THEN GO TO A6N20;
         CH    @7,@D15                                            0663
         BC    02,A6N20                                           0664
*                                   /*TEST IF DECIMAL VALUE GREATER*/
*                                   /* THAN 2,147,483,647.         */
*        IF Q(11:16) > '02147483647F'X THEN DO;
         CLC   226(6,@5),@X26                                     0665
         BC    12,@9C6                                            0665
*  A6N20:   MS = 6;
A6N20    LA    @F,6                                               0667
         STH   @F,406(0,@5)                                       0667
*         GO TO ERRORF;
         BC    15,ERRORF                                          0668
*         END;
*                                   /*CONVERT QUANTITY FROM PACKED DEC.
*                                   /* TO BINARY, RESULT IN R6.
*           GEN( CVB  R6,Q+8(R5));
@9C6     EQU   *                                                  0670
          CVB  R6,Q+8(R5)
         DS    0H
*                                   /*TEST IF NOT BINARY PICTURE. */
*         IF PICCSW(6) = '0'B THEN
         TM    562(@5),B'00000100'                                0671
         BC    05,@9C5                                            0671
*                                   /*TEST IF VALUE IS 0. IF SO, GO TO
*                                   /* ISSUE MSG 3.
*         IF R6 = 0 THEN GO TO MSG3;
         LTR   @6,@6                                              0672
         BC    08,MSG3                                            0673
*                                   /*RESTORE REG 14 FROM COMMON AREA.
*  A6N26:    R14 = SAVE14;
@9C5     EQU   *                                                  0674
A6N26    L     @E,360(0,@5)                                       0674
*                                   /*RETURN TO USER.*/
*         GO TO R14;
         BCR   15,@E                                              0675
*  MVZON:   GEN( MVZ  Q(1,R5),0(R9));
MVZON     MVZ  Q(1,R5),0(R9)
         DS    0H
*  PACK:    GEN( PACK  Q(16,R5),0(1,R9));
PACK      PACK  Q(16,R5),0(1,R9)
         DS    0H
* /********************************************************************
* /** GETMAIN - THIS SUBROUTINE ISSUES A CONDITIONAL GETMAIN MACRO    *
* /**            WHICH IS REENTRANT. IT CHECKS FOR AN ERROR RETURN    *
* /**            CODE FROM THE GETMAIN AND ISSUES MESSAGE 10 IF       *
* /**            UNSUCCESSFUL.                                        *
* /**     ENTRY - 'GLENGTH' CONTAINS THE LENGTH TO BE GOTTEN.         *
* /**            R14 IS LINKAGE REGISTER.                             *
* /**     EXIT  - 'GCADDR' CONTAINS THE ADDRESS OF THE GOTTEN CORE.   *
* /********************************************************************
*  GETMAIN:;
*                                   /*SAVE REG 14 IN COMMON AREA. */
*            SAVE14 = R14;
GETMAIN  ST    @E,360(0,@5)                                       0679
*                                   /*POINT TO GETMAIN PARAMETER LIST.
*            R1 = ADDR(GETMLIST);
         LA    @1,364(0,@5)                                       0680
*                                   /*POINT TO RETURNED GM ADDRESS. */
*            R2 = ADDR(GCADDR);
         LA    @2,376(0,@5)                                       0681
*                                   /*ISSUE GETMAIN MACRO         */
*                                   /*LENGTH VALUE IS ALREADY IN  */
*                                   /* PARAMETER LIST.            */
*            GEN( GETMAIN  EC,A=(2),SP=0,MF=(E,(1)));
          GETMAIN  EC,A=(2),SP=0,MF=(E,(1))
         DS    0H
*                                   /*TEST IF GETMAIN SUCCESSFUL. */
*         IF R15 ^= 0 THEN DO;
         LTR   @F,@F                                              0683
         BC    08,@9C4                                            0683
*                                   /*GETMAIN UNSUCCESSFUL. */
*                                   /*SET MSG 10.           */
*            MS = 10;
         LA    @F,10                                              0685
         STH   @F,406(0,@5)                                       0685
*         GO TO ERROR;
         BC    15,ERROR                                           0686
*         END;
*                                   /*RESTORE REG 14. */
*            R14 = SAVE14;
@9C4     L     @E,360(0,@5)                                       0688
*                                   /*RETURN TO CALLER. */
*         GO TO R14;
         BCR   15,@E                                              0689
*  END IEBCRANL;
@EL01    L     @D,4(0,@D)                                         0690
         LR    @1,@C                                              0690
         L     @0,@SIZ001                                         0690
         FREEMAIN R,LV=(0),A=(1)                                  0690
         L     @E,12(0,@D)                                        0690
         LM    @0,@8,20(@D)                                       0690
         LM    @A,@C,60(@D)                                       0690
         BCR   15,@E                                              0690
@DATA1   EQU   *
@0       EQU   00                  EQUATES FOR REGISTERS 0-15
@1       EQU   01
@2       EQU   02
@3       EQU   03
@4       EQU   04
@5       EQU   05
@6       EQU   06
@7       EQU   07
@8       EQU   08
@9       EQU   09
@A       EQU   10
@B       EQU   11
@C       EQU   12
@D       EQU   13
@E       EQU   14
@F       EQU   15
@D9      DC    F'0'
@D14     DC    F'1'
@D1      DC    H'6'
@D2      DC    H'1'
@D3      DC    H'3'
@D4      DC    H'9'
@D5      DC    H'5'
@D6      DC    H'-1'
@D7      DC    H'4'
@D8      DC    H'64'
@D10     DC    H'8'
@D11     DC    H'2'
@D12     DC    H'256'
@D13     DC    H'16'
@D15     DC    H'10'
@CLC     CLC   0(1,@A),0(@E)
@MVC     MVC   0(1,@A),0(@E)
         DS    0F
@SIZ001  DC    AL1(&SPN)
         DC    AL3(@DATEND-@DATD)
         DS    0F
@C7      DC    C'PICTURE='
@C24     DC    C'SYSIN   '
@C25     DC    C'$$$E'
@C9      DC    C'INPUT='
@C16     DC    C'P'''
@C17     DC    C'B'''
@X26     DC    X'02147483647F'
@C5      DC    C'QUANTITY='
@C6      DC    C'NAME='
@C8      DC    C'FILL='
@C10     DC    C'EXIT='
@C14     DC    C'COPY='
         DS    0D
@DATA    EQU   *
R1       EQU   00000001            FULLWORD INTEGER REGISTER
R2       EQU   00000002            FULLWORD INTEGER REGISTER
R3       EQU   00000003            FULLWORD INTEGER REGISTER
R4       EQU   00000004            FULLWORD POINTER REGISTER
R5       EQU   00000005            FULLWORD POINTER REGISTER
R6       EQU   00000006            FULLWORD POINTER REGISTER
R7       EQU   00000007            FULLWORD POINTER REGISTER
R8       EQU   00000008            FULLWORD POINTER REGISTER
R9       EQU   00000009            FULLWORD POINTER REGISTER
R14      EQU   00000014            FULLWORD POINTER REGISTER
R15      EQU   00000015            FULLWORD INTEGER REGISTER
R0       EQU   00000000            FULLWORD INTEGER REGISTER
COMMON   EQU   00000000            308 BYTE(S) ON DWORD
PAGENO   EQU   COMMON+00000000     4 BYTE(S)
LINECT   EQU   COMMON+00000004     FULLWORD INTEGER
LINECTR  EQU   COMMON+00000008     FULLWORD INTEGER
PARM     EQU   COMMON+00000012     FULLWORD POINTER
REPEATNO EQU   COMMON+00000016     HALFWORD INTEGER
CREATENO EQU   COMMON+00000018     HALFWORD INTEGER
SYSP     EQU   COMMON+00000020     96 BYTE(S)
SYSI     EQU   COMMON+00000116     96 BYTE(S)
Q        EQU   COMMON+00000216     16 BYTE(S) ON DWORD
QFILL    EQU   COMMON+00000216     7 BYTE(S)
QSIGN    EQU   COMMON+00000223     8 BIT(S)
QFILL1   EQU   COMMON+00000224     7 BYTE(S)
QSIGN1   EQU   COMMON+00000231     8 BIT(S)
COUNTER  EQU   COMMON+00000232     FULLWORD POINTER
OPENLIST EQU   COMMON+00000236     8 BYTE(S)
OPTBYTE1 EQU   COMMON+00000236     4 BYTE(S)
OPTBYTE2 EQU   COMMON+00000240     4 BYTE(S)
EXLST    EQU   COMMON+00000244     24 BYTE(S) ON WORD
INHDR    EQU   COMMON+00000244     1 BYTE(S)
INHDR1   EQU   COMMON+00000245     3  BYTE  POINTER ON WORD+1
OUTHDR   EQU   COMMON+00000248     1 BYTE(S)
OUTHDR1  EQU   COMMON+00000249     3  BYTE  POINTER ON WORD+1
INTRL    EQU   COMMON+00000252     1 BYTE(S)
INTRL1   EQU   COMMON+00000253     3  BYTE  POINTER ON WORD+1
OUTTRL   EQU   COMMON+00000256     1 BYTE(S)
OUTTRL1  EQU   COMMON+00000257     3  BYTE  POINTER ON WORD+1
EXITDCB  EQU   COMMON+00000260     1 BYTE(S)
EXITDCB1 EQU   COMMON+00000261     3  BYTE  POINTER ON WORD+1
TOTAL    EQU   COMMON+00000264     1 BYTE(S)
TOTAL1   EQU   COMMON+00000265     3  BYTE  POINTER ON WORD+1
EXLST1   EQU   COMMON+00000268     4 BYTE(S) ON WORD
EDCB1    EQU   COMMON+00000268     1 BYTE(S)
EDCB2    EQU   COMMON+00000269     3  BYTE  POINTER ON WORD+1
EXLST2   EQU   COMMON+00000272     4 BYTE(S) ON WORD
EDCB3    EQU   COMMON+00000272     1 BYTE(S)
EDCB4    EQU   COMMON+00000273     3  BYTE  POINTER ON WORD+1
EXLST3   EQU   COMMON+00000276     4 BYTE(S) ON WORD
EDCB5    EQU   COMMON+00000276     1 BYTE(S)
EDCB6    EQU   COMMON+00000277     3  BYTE  POINTER ON WORD+1
DLRECL   EQU   COMMON+00000280     HALFWORD INTEGER
DBLKSI   EQU   COMMON+00000282     HALFWORD INTEGER
DRECFM   EQU   COMMON+00000284     8 BIT(S)
LEFTOVER EQU   COMMON+00000288     FULLWORD INTEGER
OFFSET   EQU   COMMON+00000292     FULLWORD POINTER
LPTR     EQU   COMMON+00000296     FULLWORD POINTER
DCBPTR   EQU   COMMON+00000300     FULLWORD POINTER
DUMMY    EQU   COMMON+00000304     FULLWORD POINTER
COMMON1  EQU   00000304            236 BYTE(S) ON WORD
SAVEMS   EQU   COMMON1+00000000    HALFWORD INTEGER
CONCODE  EQU   COMMON1+00000002    HALFWORD INTEGER
OUTREC   EQU   COMMON1+00000004    FULLWORD POINTER
CRTABPT  EQU   COMMON1+00000008    FULLWORD POINTER
CURCRTE  EQU   COMMON1+00000012    FULLWORD POINTER
CURCRGM  EQU   COMMON1+00000016    FULLWORD POINTER
CURPIC   EQU   COMMON1+00000020    FULLWORD POINTER
PICCTR   EQU   COMMON1+00000024    FULLWORD INTEGER
EXITTAB  EQU   COMMON1+00000028    FULLWORD POINTER
EXITGM   EQU   COMMON1+00000032    FULLWORD POINTER
CUREXIT  EQU   COMMON1+00000036    FULLWORD POINTER
DELIM    EQU   COMMON1+00000040    4 BYTE(S)
RECREM   EQU   COMMON1+00000044    FULLWORD INTEGER
CURFD    EQU   COMMON1+00000048    FULLWORD POINTER
CUROUT   EQU   COMMON1+00000052    HALFWORD POINTER
SAVE14   EQU   COMMON1+00000056    FULLWORD POINTER
GETMLIST EQU   COMMON1+00000060    16 BYTE(S) ON WORD
GLENGTH  EQU   COMMON1+00000060    FULLWORD INTEGER
ADRLIST  EQU   COMMON1+00000064    FULLWORD POINTER
IND      EQU   COMMON1+00000068    4 BYTE(S)
GCODE    EQU   COMMON1+00000068    1 BYTE(S)
SPOOL    EQU   COMMON1+00000069    1 BYTE(S)
CCODE    EQU   COMMON1+00000070    2 BYTE(S)
GCADDR   EQU   COMMON1+00000072    FULLWORD POINTER
FIRSTGMO EQU   COMMON1+00000076    FULLWORD POINTER
CURRGMO  EQU   COMMON1+00000080    FULLWORD POINTER
LASTGMO  EQU   COMMON1+00000084    FULLWORD POINTER
FIRSTGMI EQU   COMMON1+00000088    FULLWORD POINTER
CURRGMI  EQU   COMMON1+00000092    FULLWORD POINTER
LASTGMI  EQU   COMMON1+00000096    FULLWORD POINTER
CONDCODE EQU   COMMON1+00000100    HALFWORD INTEGER
MS       EQU   COMMON1+00000102    HALFWORD INTEGER
INBUFA1  EQU   COMMON1+00000104    121 BYTE(S)
INFILL   EQU   COMMON1+00000104    10 BYTE(S)
INBUFA   EQU   COMMON1+00000114    111 BYTE(S)
DDPTR    EQU   COMMON1+00000228    FULLWORD POINTER
DUMMY1   EQU   COMMON1+00000232    FULLWORD POINTER
COMMON2  EQU   00000536            76 BYTE(S) ON WORD
SWITCH   EQU   COMMON2+00000000    52 BYTE(S)
FDCSW    EQU   COMMON2+00000000    1 BYTE(S)
FDNAMESW EQU   COMMON2+00000001    1 BYTE(S)
FDPCSW   EQU   COMMON2+00000002    1 BYTE(S)
FDFMTSW  EQU   COMMON2+00000003    1 BYTE(S)
FDPLSW   EQU   COMMON2+00000004    1 BYTE(S)
RANGESW  EQU   COMMON2+00000005    1 BYTE(S)
FILLSW   EQU   COMMON2+00000006    1 BYTE(S)
REPSW    EQU   COMMON2+00000007    1 BYTE(S)
INDEXSW  EQU   COMMON2+00000008    1 BYTE(S)
INDNMSW  EQU   COMMON2+00000009    1 BYTE(S)
BQUOTESW EQU   COMMON2+00000010    1 BYTE(S)
PQUOTESW EQU   COMMON2+00000011    1 BYTE(S)
EQUOTESW EQU   COMMON2+00000012    1 BYTE(S)
FDSW     EQU   COMMON2+00000013    1 BYTE(S)
DSDSW    EQU   COMMON2+00000014    1 BYTE(S)
NOGOSW   EQU   COMMON2+00000015    1 BYTE(S)
CREATESW EQU   COMMON2+00000016    1 BYTE(S)
DSDCSW   EQU   COMMON2+00000017    1 BYTE(S)
CRCSW    EQU   COMMON2+00000018    1 BYTE(S)
EXITSW   EQU   COMMON2+00000019    1 BYTE(S)
EODSTOP  EQU   COMMON2+00000020    1 BYTE(S)
DSDNULSW EQU   COMMON2+00000021    1 BYTE(S)
DSDORGSW EQU   COMMON2+00000022    1 BYTE(S)
DSDDDSW  EQU   COMMON2+00000023    1 BYTE(S)
CRTBLK   EQU   COMMON2+00000024    1 BYTE(S)
NAMCSW   EQU   COMMON2+00000025    8 BIT(S)
PICCSW   EQU   COMMON2+00000026    8 BIT(S)
BUFPSW   EQU   COMMON2+00000027    1 BYTE(S)
ENDSW    EQU   COMMON2+00000028    1 BYTE(S)
COMCSW   EQU   COMMON2+00000029    1 BYTE(S)
FLAGSW   EQU   COMMON2+00000030    8 BIT(S)
PAGESW   EQU   COMMON2+00000031    1 BYTE(S)
EPSW     EQU   COMMON2+00000032    1  BYTE  POINTER
SYSISW   EQU   COMMON2+00000033    1 BYTE(S)
SYSPSW   EQU   COMMON2+00000034    1 BYTE(S)
OLDNEWSW EQU   COMMON2+00000035    1 BYTE(S)
FLUSHSW  EQU   COMMON2+00000036    1 BYTE(S)
FLUSHSW1 EQU   COMMON2+00000037    1 BYTE(S)
DSDOSW   EQU   COMMON2+00000038    1 BYTE(S)
DSDISW   EQU   COMMON2+00000039    1 BYTE(S)
QUANSW   EQU   COMMON2+00000040    1 BYTE(S)
PARENSW  EQU   COMMON2+00000041    1 BYTE(S)
REPEATSW EQU   COMMON2+00000042    1 BYTE(S)
SYSINEOD EQU   COMMON2+00000043    1 BYTE(S)
FDPLGTH  EQU   COMMON2+00000052    HALFWORD INTEGER
SGCADDR  EQU   COMMON2+00000056    FULLWORD POINTER
FDPTR    EQU   COMMON2+00000060    FULLWORD POINTER
FDPTR1   EQU   COMMON2+00000064    FULLWORD POINTER
FDPTR2   EQU   COMMON2+00000068    FULLWORD POINTER
DUMMY2   EQU   COMMON2+00000072    FULLWORD POINTER
COMMON3  EQU   00000608            188 BYTE(S) ON WORD
FDCTR    EQU   COMMON3+00000000    HALFWORD POINTER
LREMAIN  EQU   COMMON3+00000004    FULLWORD POINTER
COMPCTR  EQU   COMMON3+00000008    FULLWORD POINTER
LMOVED   EQU   COMMON3+00000012    HALFWORD POINTER
U        EQU   COMMON3+00000016    FULLWORD POINTER
PICEND   EQU   COMMON3+00000020    HALFWORD POINTER
CURFDGM  EQU   COMMON3+00000024    FULLWORD POINTER
SWTCH    EQU   COMMON3+00000028    4 BYTE(S) ON WORD
SYSINSEL EQU   COMMON3+00000028    1 BYTE(S)
FIRSTSW  EQU   COMMON3+00000029    1 BYTE(S)
FRSTSW   EQU   COMMON3+00000030    1 BYTE(S)
STOPSW   EQU   COMMON3+00000031    1 BYTE(S)
COPYVAL  EQU   COMMON3+00000032    HALFWORD INTEGER
COPYFD   EQU   COMMON3+00000036    FULLWORD POINTER
COPYFDGM EQU   COMMON3+00000040    FULLWORD POINTER
NAMCTR   EQU   COMMON3+00000044    HALFWORD INTEGER
NAMCTR1  EQU   COMMON3+00000046    HALFWORD INTEGER
INRECSZ  EQU   COMMON3+00000048    HALFWORD POINTER
OUTRECSZ EQU   COMMON3+00000050    HALFWORD POINTER
INRECFM  EQU   COMMON3+00000052    1 BYTE(S)
RECOFFST EQU   COMMON3+00000053    1  BYTE  POINTER
OUTRECFM EQU   COMMON3+00000054    1 BYTE(S)
PICBASE  EQU   COMMON3+00000056    FULLWORD POINTER
MESSAGE  EQU   COMMON3+00000060    121 BYTE(S)
DUMMY3   EQU   COMMON3+00000184    FULLWORD POINTER
DCBD     EQU   00000000            280 BYTE(S) ON DWORD
FILL     EQU   DCBD+00000000       26 BYTE(S)
DSORG1   EQU   DCBD+00000026       2 BYTE(S)
DSORG    EQU   DCBD+00000026       8 BIT(S)
FILLER   EQU   DCBD+00000028       8 BYTE(S)
IOBAD    EQU   DCBD+00000028       4 BYTE(S)
BFTEK    EQU   DCBD+00000032       8 BIT(S)
EODAD    EQU   DCBD+00000033       3 BYTE(S)
RECFM    EQU   DCBD+00000036       8 BIT(S)
EXLIST   EQU   DCBD+00000037       3  BYTE  POINTER ON WORD+1
DDNAME   EQU   DCBD+00000040       8 BYTE(S)
DEBAD    EQU   DCBD+00000040       4 BYTE(S)
IFLGS    EQU   DCBD+00000040       8 BIT(S)
GETAD    EQU   DCBD+00000048       4 BYTE(S)
OFLGS    EQU   DCBD+00000048       8 BIT(S)
OFLGS1   EQU   DCBD+00000049       1 BYTE(S)
MACRF    EQU   DCBD+00000050       2 BYTE(S)
FILL2    EQU   DCBD+00000052       10 BYTE(S)
BLKSI    EQU   DCBD+00000062       HALFWORD INTEGER
FILL3    EQU   DCBD+00000064       18 BYTE(S)
LRECL    EQU   DCBD+00000082       HALFWORD INTEGER
FILL4    EQU   DCBD+00000084       172 BYTE(S)
NEXTDCB  EQU   DCBD+00000256       FULLWORD POINTER
DDNAME1  EQU   DCBD+00000260       8 BYTE(S)
EODSW    EQU   DCBD+00000268       1 BYTE(S)
DCBSW1   EQU   DCBD+00000269       1 BYTE(S)
DCBSW2   EQU   DCBD+00000270       1 BYTE(S)
DCBSW3   EQU   DCBD+00000271       8 BIT(S)
INREC    EQU   DCBD+00000272       FULLWORD POINTER
GMLGTH   EQU   DCBD+00000276       HALFWORD INTEGER
FIELDSEL EQU   DCBD+00000278       1 BYTE(S)
SPARE    EQU   DCBD+00000279       1 BYTE(S)
CRPICT   EQU   00000000            7 BYTE(S) ON WORD
PICSTLOC EQU   CRPICT+00000000     FULLWORD POINTER
PICLGTH  EQU   CRPICT+00000004     HALFWORD INTEGER
PICSTRNG EQU   CRPICT+00000006     1 BYTE(S)
CRTAB    EQU   00000000            28 BYTE(S) ON WORD
NXTCRTE  EQU   CRTAB+00000000      FULLWORD POINTER
QUAN     EQU   CRTAB+00000004      4 BYTE(S)
IDCBPTR  EQU   CRTAB+00000008      FULLWORD POINTER
EXITADR  EQU   CRTAB+00000012      FULLWORD POINTER
PICPTR   EQU   CRTAB+00000016      FULLWORD POINTER
FILLCH   EQU   CRTAB+00000020      1  BYTE  POINTER
EMP1     EQU   CRTAB+00000021      3 BYTE(S)
FDADTAB  EQU   CRTAB+00000024      FULLWORD POINTER
ENDFD    EQU   CRTAB+00000024      1 BYTE(S)
FDTBL    EQU   00000000            520 BYTE(S) ON WORD
FDNAME   EQU   FDTBL+00000000      8 BYTE(S)
FDREPNM  EQU   FDTBL+00000008      8 BYTE(S)
FDINDNM  EQU   FDTBL+00000016      8 BYTE(S) ON WORD
FDUMMY   EQU   FDTBL+00000016      4 BYTE(S)
FDINDNUM EQU   FDTBL+00000020      FULLWORD POINTER
FDLGTH   EQU   FDTBL+00000024      HALFWORD POINTER
FDCYCLE  EQU   FDTBL+00000026      HALFWORD POINTER
FDACTION EQU   FDTBL+00000028      2 BYTE(S)
FDFORMAT EQU   FDTBL+00000030      2 BYTE(S)
FDSWITCH EQU   FDTBL+00000032      8 BIT(S)
INDBYNAM EQU   FDTBL+00000032      1 BIT(S)
PASS     EQU   FDTBL+00000032      1 BIT(S)
FXACTION EQU   FDTBL+00000032      1 BIT(S)
RPACTION EQU   FDTBL+00000032      1 BIT(S)
ROACTION EQU   FDTBL+00000032      1 BIT(S)
WVACTION EQU   FDTBL+00000032      1 BIT(S)
STACTION EQU   FDTBL+00000032      1 BIT(S)
NUACTION EQU   FDTBL+00000032      1 BIT(S)
FDFILL   EQU   FDTBL+00000033      1 BYTE(S)
FDSIGN   EQU   FDTBL+00000034      1 BYTE(S)
FDCHAR   EQU   FDTBL+00000035      1 BYTE(S)
FDRANGE  EQU   FDTBL+00000036      4 BYTE(S)
FDOBUF   EQU   FDTBL+00000040      HALFWORD POINTER
FDFRINC  EQU   FDTBL+00000042      HALFWORD POINTER
FDFROMAD EQU   FDTBL+00000044      4 BYTE(S)
FDMLGTH  EQU   FDTBL+00000048      HALFWORD POINTER
FDTOINC  EQU   FDTBL+00000050      HALFWORD POINTER
FDCYCCNT EQU   FDTBL+00000052      HALFWORD POINTER
FDSLGTH  EQU   FDTBL+00000054      HALFWORD POINTER
FDSLGTHR EQU   FDTBL+00000056      HALFWORD POINTER
FDFRINCR EQU   FDTBL+00000058      1 BYTE(S)
FDTOINCR EQU   FDTBL+00000059      1 BYTE(S)
LTOFREE  EQU   FDTBL+00000060      HALFWORD POINTER
FDSW1    EQU   FDTBL+00000062      8 BIT(S)
STRTLOC1 EQU   FDTBL+00000062      1 BIT(S)
NXTFDTAB EQU   00000516            FULLWORD POINTER
INBUF    EQU   00000000            80 BYTE(S)
PARAM    EQU   00000000            1 BYTE(S)
AREA     EQU   00000000            512 BYTE(S)
NXTCRGM  EQU   00000000            FULLWORD POINTER
NXTEXGM  EQU   00000000            FULLWORD POINTER
EXITNAME EQU   00000000            8 BYTE(S)
NXTFDGM  EQU   00000000            FULLWORD POINTER
FDADR    EQU   00000000            FULLWORD POINTER
ONE      EQU   *                   FULLWORD INTEGER
         DC    FL4'0'
YES      EQU   *                   1 BYTE(S)
         DC    X'FF'
NO       EQU   *                   1 BYTE(S)
         DC    X'00'
ON       EQU   *                   1 BYTE(S)
         DC    X'FF'
OFF      EQU   *                   1 BYTE(S)
         DC    X'00'
FOXZEROS EQU   *                   16 BYTE(S)
         DC    X'F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0'
         ORG   @DATA
         DS    00000024C
@L       EQU   1
@DATD    DSECT
@SAV001  EQU   @DATD+00000000      72 BYTE(S) ON WORD
         DS    00000072C
@TEMPS   DS    0F
@TEMP4   DC    F'0'
@T1      DC    F'0'
@DATEND  EQU   *
IEBCRANL CSECT ,
@9F7     EQU   KEYSCAN
@9F0     EQU   A6C5
@9D9     EQU   CREATE
@9DA     EQU   CARDSCAN
@9D3     EQU   CARDSCAN
RETURN2  EQU   @EL01
@9C8     EQU   A6M11
         END   IEBCRANL
