***********************************************************************
*                                                                     *
*  TITLE 'IEBDG'-DATA GENERATOR                                       *
*                                                                     *
*  STATUS: CHANGE LEVEL 0                                             *
*                                                                     *
* FUNCTION- TO GENERATE DATA USING THE PS AND PO ACCESS METHODS.      *
*        RECORDS(FIELDS) MAY BE DEFINED USING SEVEN CHARACTER FORMATS *
*        UPON WHICH NINE ACTIONS MAY BE PERFORMED.A PICTURE CAN BE    *
*        SUPPLIED BY CONTROL CARD OR DYNAMICALLY FROM AN OLD DATA SET.*
*        A USER EXIT IS SUPPLIED SO THAT A RECORD CAN BE INSPECTED    *
*        BEFORE IT IS OUTPUT.                                         *
*                                                                     *
*  ENTRY POINT:  IEBDG , THE CSECT NAME OF THIS MODULE                *
*                                                                     *
*  INPUT:                                                             *
*              BY INVOCATION                                          *
*                                                                     *
*        REGISTER ONE(1) CONTAINS ADDRESS OF LIST                     *
*        NO OPTIONS OR PAGANATION ARE PROVIDED,ONLY ACTIVE ENTRY      *
*        SHOULD BE THE DDPTR.                                         *
*        FORMAT OF LIST SHOULD BE AS FOLLOWS:                         *
*        FIRST FOUR BYTES ZERO INDICATING NO OPTIONS                  *
*        NEXT BYTE HEX 80 INDICATING LAST ENTRY                       *
*        NEXT 3 BYTES CONTAIN ADDRESS OF LIST OF DDNAMES              *
*                                                                     *
*        FORMAT OF DDNAME LIST AS FOLLOWS:                            *
*        BYTE 0-1 CONTAIN LENGTH OF LIST                              *
*        BYTE 2-9   DD CARD NAME DESCRIBING OLD DATA SET FOR INPUT    *
*        BYTE 10-17      ZEROES-NOT ACTIVE                            *
*        BYTE 18-25      ZEROES-NOT ACTIVE                            *
*        BYTE 26-33      ZEROES-NOT ACTIVE                            *
*        BYTE 34-41 DD CARD NAME DESCRIBING DATA SET FOR SYSIN        *
*        BYTE 42-49 DD CARD NAME DESCRIBING DATA SET FOR SYSPRINT     *
*        BYTE 50-57      ZEROES-NOT ACTIVE                            *
*        BYTE 58-65 DD CARD NAME DESCRIBING DATA SET FOR OUTPUT RCDS  *
*        BYTE XX-XX ADDITIONAL ENTRIES AS ABOVE FOR OUTPUT RECORDS    *
*                                                                     *
*                                                                     *
*  INPUT:                                                             *
*              BY CONTROL CARDS                                       *
*        KEYWORDS ARE USED                                            *
*        THE NAME OF THE CONTROL CARD CANNOT START IN COL 1 WHICH IS  *
*        RESERVED FOR A NAME.                                         *
*        CONTROL CARDS ARE DSD    DATA SET DEFINITION                 *
*                          FD     FIELD DEFINITION                    *
*                          CREATE RECORD DEFINITION                   *
*                          REPEAT                                     *
*                          END                                        *
*        DSD KEYWORDS ALLOWED ARE INPUT AND OUTPUT                    *
*        FD KEYWORDS ALLOWED ARE:PICTURE=,FORMAT=,NAME=,LENGTH=,      *
*                                 ACTION=,CHARACTER=,SIGN=,CYCLE=,    *
*                                 INDEX=,REPLACE=,FILL=,RANGE=,       *
*        CREATE KEYWORDS ALLOWED ARE:QUANTITY=,NAME=,PICTURE=,FILL=,  *
*                                    LENGTH=,USER=,STARTLOC=          *
*                                    INPUT                            *
*                                                                     *
*  INPUT:                                                             *
*              BY SYSTEM                                              *
*        THIS PROGRAM DEPENDS UPON THE SYSTEM FOR INPUT TO THE EXTENT *
*        THAT THE CORRECT INFORMATION FROM LABELS AND DDCARDS MUST    *
*        BE CONVEYED TO THE DCBS.A DD CARD DESCRIBING EVERY DATA SET  *
*        MUST BE PRESENT.THIS PROGRAM WILL USE THE FOLLOWING NAMES    *
*        UNLESS REQUIRED TO CHANGE BY INVOCATION.                     *
*                                                                     *
*        SYSIN -DATA SET FOR INPUTTING CONTROL CARDS                  *
*                   CARDS MAY BE BLOCKED 80 CHAR FIXED FORMAT         *
*        SYSPRINT -DATA SET FOR OUTPUTTING MESSAGES                   *
*        ANY   DATA SET FOR OUTPUTING GENERATED RECORDS               *
*        ANY   DATA SET FOR INPUTING OLD RECORDS                      *
*                                                                     *
*  OUTPUT:                                                            *
*        REGISTER FIVE POINTING TO COMMON COMMUNICATION AREA          *
*                                                                     *
*        TO MESSAGE MODULE -                                          *
*              MS --INDICATES MESSAGE NUMBER                          *
*              FLAG SWITCH- INDICATES ERROR FLAG WANTED               *
*        TO FD ANALYSIS MODULE                                        *
*              REGISTER 9 POINTING AT OPERATION FIELD IN INBUF        *
*        TO CREATE ANALYSIS MODULE                                    *
*              REGISTER 9 POINTING AT OPERATION FIELD IN INBUF        *
*                                                                     *
*  EXTERNAL ROUTINES:                                                 *
*                                                                     *
*        IEBDGMSG                                                     *
*        IEBFDANL                                                     *
*        IEBCRANL                                                     *
*        IEBDGCUP                                                     *
*        GET MACRO                                                    *
*        GETMAIN MACRO                                                *
*        LINK MACRO                                                   *
*                                                                     *
*    EXITS--NORMAL                                                    *
*        TO IEBDGMSG BY USE OF LINK MACRO FOR MESSAGES                *
*        TO IEBFDANL BY USE  OF THE LINK MACRO FOR FD ANALYSIS        *
*        TO IEBCRANL BY USE OF THE LINK MACRO FOR CREATE ANALYSIS     *
*        TO IEBDGCUP BY USE OF THE LINK MACRO FOR CLEANUP             *
*                                                                     *
*  EXITS-NORMAL: FROM THIS ROUTINE                                    *
*                CONDITION CODE 0 GIVEN FOR NORMAL CONCLUSION         *
*                                                                     *
*  EXITS-ERROR                                                        *
*        CONTROL IS RETURNED TO THE USER WITH ONE OF THE FOLLOWING    *
*        CONDITION CODES                                              *
*                CONDITION CODE 4  TERMINATION AT USERS REQUEST       *
*                CONDITION CODE 8  ERROR PROCESSING CONTROL CARD      *
*                CONDITION CODE 12 TERMINATION-UNRECOVERABLE ERROR    *
*                                                                     *
*  TABLES/WORKAREAS:DCBS REFERENCED AT DCBOFLG                        *
*              FIELD DEFINITION TABLES                                *
*                                                                     *
*       BYTE 0 BYTE 1 BYTE 2 BYTE 3 BYTE 4 BYTE 5 BYTE 6 BYTE 7       *
*      *********************************************************      *
*      *                     NAME                              *      *
*      *********************************************************      *
*      *               REPLACE NAME                            *      *
*      *********************************************************      *
*      *               INDEX NAME                              *      *
*      *********************************************************      *
*      *   LENGTH    *   CYCLE     *   ACTION    *   FORMAT    *      *
*      *********************************************************      *
*      *SWITCH* FILL * SIGN * CHAR *        RANGE              *      *
*      *********************************************************      *
*      *   STARTLOC IN WORKAREA    *                           *      *
*      *********************************************************      *
*                                                                     *
*  ATTRIBUTES: REENTRANT,REUSEABLE                                    *
*                                                                     *
*                                                                     *
***********************************************************************
*     NEW MODULE, RELEASE 17.                                      DGOH
*                                                                A28655
*A197600-197984,609600-611869,990000-990200                      A37883
*D567500-572500,873500-888700,891600-891700                      A37883
*C540500,542500,777500                                           A38710
*                                                                A41807
*C544100,545500                                                  A38766
*A545600-545900                                                  A38766
*A463520-463960                                                  A41807
*C153000,153100.A638800-639000                                   A45169
*C225600,225700                                                  A45200
*A690800-691000                                                  A48784
*C610296,611595.D610292,611454,611485,611528,611553.A611538      A50927
*C262000                                                        YA01692
*C261500-262000,265500                  @XA12917,@YA12894,(ORG)@ZA07373
*A994300-994700,996700-997100           @XA13603,(ORG)@YA12911,@ZA07394
*D994300,996700                         @XA13603.(ORG)@YA12911,@ZA07394
*A339600,339700                         (ORG)@XA16970,@YA18072,@ZA24201
*A940800-940900                                                @ZA24161
*A339550                                @XA21352,@YA18072,(ORG)@ZA28703
*A1                                              (ORG)@XA20990,@ZA33694
*C1                                     @XA22239,(ORG)@YA19824,@ZA33693
*D171000                                @XA22239,(ORG)@YA19824,@ZA33693
*A5/C2A1C3A6/C1A1/A3                    @XA22239,(ORG)@YA19824,@ZA33693
         LCLA  &T,&SPN                                            0003
.@001    ANOP                                                     0003
IEBDG    CSECT ,                                                  0003
         STM   @E,@C,12(@D)                                       0003
         BALR  @B,0                                               0003
@PSTART  DS    0H                                                 0003
         USING @PSTART+00000,@B                                   0003
         LA    @3,4095(0,@B)                                      0003
         USING @PSTART+04095,@3                                   0003
         L     @0,@SIZ001                                         0003
         GETMAIN  R,LV=(0)                                        0003
         LR    @C,@1                                              0003
         USING @DATD+00000,@C                                     0003
         LM    @0,@1,20(@D)                                       0003
         XC    @TEMPS(@L),@TEMPS                                  0003
         ST    @D,@SAV001+4                                       0003
         LA    @F,@SAV001                                         0003
         ST    @F,8(0,@D)                                         0003
         LR    @D,@F                                              0003
         B     MODULEVL                                    /*@ZA33693*/
         DC    C'IEBDG   -OZ33693'                         /*@ZA33693*/
MODULEVL DS    0H                                          /*@ZA33693*/
         DS    0H
*               Q(1) = '00'X;           /*SET FIRST BYTE TO 0*/
         MVI   COMMON+216,X'00'                                   0074
*               R5 = ADDR(Q)            /* SET UP FROM ADDR*/;
         LA    @5,COMMON+216                                      0075
*                                        /* PICK UP CONSTANT LGTH*/
*                   /* GET ADDRESS OF END OF COMMON */
*         GEN ( LA R7,@DATEND);
          LA R7,@DATEND
         DS    0H
*               /* GET ADDRESS OF Q FOR CLEARING*/
*               R7 = R7 - ADDR(Q);
         LR    @F,@7                                              0077
         LA    @0,COMMON+216                                      0077
         SR    @F,@0                                              0077
         LR    @7,@F                                              0077
*               R7 = R7 - 1;   /** TO CORRECT THE AMOUNT**/
         BCTR  @7,0                                               0078
*  A000:;
*         IF R7 < 256 THEN GO TO A002   /*BR IF LESS THAN 256*/;
A000     CH    @7,@D1                                             0080
         BC    04,A002                                            0081
*         IF R7 = 0 THEN GO TO A003     /*BR IF NONE TO MOVE*/;
         LTR   @7,@7                                              0082
         BC    08,A003                                            0083
*                                       /* MOVES 256 BYTES */
*         GEN ( MVC 1(256,5),0(5));
          MVC 1(256,5),0(5)
         DS    0H
*               R5 = R5 + 256           /* ADD 256 TO POSITION ADDR*/;
         AH    @5,@D1                                             0085
*               R7 = R7 - 256           /*SUBT 256 TO CORRECT LGTH*/;
         SH    @7,@D1                                             0086
*               GO TO A000              /* BR TO REPEAT SEQUENCE */;
         BC    15,A000                                            0087
*  A001:;
*                         /* THIS INST MOVES LESS THAN 256*/
*         GEN ( MVC 1(1,5),0(5));
A001      MVC 1(1,5),0(5)
         DS    0H
*  A002:;
*               R7 = R7 - 1              /*TO CORRECT LENGTH.*/;
A002     BCTR  @7,0                                               0091
*                    /* THIS INST CAUSES MOVE OF < 256*/
*               GEN ( EX R7,A001);
          EX R7,A001
         DS    0H
*                    /* END OF CLEAR CORE INSTRUCTIONS */
*  A003:;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION MOVES COPIES OF THE DATA GENERATOR INPUT AND   *
*  /** OUTPUT DCBS TO THE COMMON AREA FOR REENTRANTIBILITY.           *
*  /**                                                                *
*  /*******************************************************************
*        PARM = R1  /* SAVE PARAMETER POINTER */;
A003     ST    @1,COMMON+12                                       0094
*               /** MOVE DCBS TO COMMON AREA FOR REENTRANCY **/
*  SYSI = MYDCB1A;
         MVC   COMMON+116(96),MYDCB1A                             0095
*  SYSP=MYDCB2A;
         MVC   COMMON+20(96),MYDCB2A                              0096
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION ESTABLISHES THE COMMUNICATION REGISTER REG 5   *
*  /** AND INITILIZES DEFAULTS FOR LINE COUNT,PAGE NUMBER AND THE     *
*  /** INITIAL VALUE FOR THE RANDOM NUMBER GENERATOR.THE DCB EXIT LIST*
*  /** IS ALSO ESTABLISHED FOR DATA GENERATOR INPUT AND OUTPUT DCBS   *
*  /**                                                                *
*  /*******************************************************************
*  /* LOAD PARA REG FOR REENTRANT COMMUNICATION  */
*         R5 = ADDR(COMMON);
         LA    @5,COMMON                                          0097
*               LINECT = 58 /* DEFAULT LINECT = 58 */;
         LA    @F,58                                              0098
         ST    @F,COMMON+4                                        0098
*         GENERATE ;
         TIME  BIN                     TAKE TIME VALUE FOR INIT  A45200
         ST    R0,624(0,R5)            RANDOM NUMBER             A45200
         DS    0H
*               PAGENO = 'F0F0F0F1'X     /*DEFAULT PAGENO = 0001*/;
         MVC   COMMON(4),@X2                                      0100
*               DCBPTR = ADDR(SYSI) /* POINT AT SYSIDCB*/;
         LA    @F,COMMON+116                                      0101
         ST    @F,COMMON+300                                      0101
*         INBUFA1 = '40'X      /**BLANK TO FIRST BYTE OF INBUF **/;
         MVI   COMMON1+104,X'40'                                  0102
         MVI   COMMON1+105,C' '                                   0102
         MVC   COMMON1+106(119),COMMON1+105                       0102
*         INBUFA1(2:121) = INBUFA1(1:120) /** BLANKS TO INBUFA1 **/;
         MVC   COMMON1+105(120),COMMON1+104                       0103
*         EXLIST = ADDR(EXLST1) /* EXIT LIST ADDR TO SYSI DCB */;
         LA    @F,COMMON+268                                      0104
         L     @2,COMMON+300       COMMON                         0104
         ST    @F,@TEMP4                                          0104
         MVC   37(3,@2),@TEMP4+1                                  0104
*        EDCB1 = '85'X /* ACTIVATE DCB EXIT */;
         MVI   COMMON+268,X'85'                                   0105
*        EDCB2 = ADDR(DCBROUT1) /* SYSI EXIT RT ADDR */;
         LA    @F,DCBROUT1                                        0106
         ST    @F,@TEMP4                                          0106
         MVC   COMMON+269(3),@TEMP4+1                             0106
*               DCBPTR = ADDR(SYSP) /* POINT AT SYSP DCB */;
         LA    @F,COMMON+20                                       0107
         ST    @F,COMMON+300                                      0107
*        EXLIST = ADDR(EXLST2) /* EXIT LIST ADDR TO SYSP DCB */;
         LA    @F,COMMON+272                                      0108
         L     @2,COMMON+300       COMMON                         0108
         ST    @F,@TEMP4                                          0108
         MVC   37(3,@2),@TEMP4+1                                  0108
*        EDCB3 = '85'X /* ACTIVATE DCB EXIT */;
         MVI   COMMON+272,X'85'                                   0109
*        EDCB4 = ADDR(DCBROUT2) /* SYSP EXIT RT ADDR */;
         LA    @F,DCBROUT2                                        0110
         ST    @F,@TEMP4                                          0110
         MVC   COMMON+273(3),@TEMP4+1                             0110
*  GEN;
***********************************************************************
*                                                                     *
*                   --------- INVOCATION ---------                    *
*                                                                     *
*        INVOCATION MUST BE TESTED UPON ENTRY FOR OS/360.REMOVE CODE  *
*        FOR DOS SUPPORT. REG 1 POINTS TO PARAMETER LIST UPON ENTRY.  *
*        LIST IS CONSTRUCTED AS FOLLOWS:                              *
*              FIRST WORD ADDRESS OF OPTIONS                          *
*              2ND WORD ADDRESS OF DDNAMES                            *
*              3RD WORD ADDRESS OF PAGE INFO                          *
*        AN '80' X IN BYTE ZERO OF ANY WORD INDICATES END OF LIST.    *
*        THIS WORD WILL CONTAIN AN ADDRESS.ANY LIST POINTED AT STARTS *
*        ON 1/2 WORD BOUNDRY.THAT 1/2 WORD CONTAINS LENGTH OF LIST    *
*        IF WE ARE INVOKED FOR DDNAMES THE ONLY ONES WE ACCEPT ARE    *
*        SYSIN AND SYSPRINT AT CHAR LOCATIONS 35 AND 43. IT IS THE    *
*        USERS RESPONSABILITY TO PLACE LENGTH IN EACH LIST.IF ZERO IS *
*        FOUND THAT LIST IS INACTIVE.USER MAY PLACE UP TO A 4 DIGIT   *
*        DECIMAL NUMBER IN PARM FIELD OF EXEC CARD FOR CONTROL OF     *
*        SYSPRINT LINE COUNT. IF HE WANTS TO PLACE 52 LINES PER PAGE  *
*        HE WILL CODE PARM='LINECNT=0052'                             *
*        AN INCORRECT KEYWORD IN THE PARM FIELD WILL CAUSE A MESSAGE  *
*        UPON SYSPRINT WHEN IT IS OPENED                              *
*                                                                     *
***********************************************************************
         DS    0H
*               OPTPTR = LIST(1:4)       /*INIT OPTION PTR*/;
         L     @4,COMMON+12        COMMON                         0112
         MVC   @TEMP4(4),0(@4)                                    0112
         L     @F,@TEMP4                                          0112
         ST    @F,OPTPTR                                          0112
*  /*IF LENGTH IN OPTION FIELD BR TO PROCESS LINT COUNT*/
*         IF OPTIONS(1:2) ^= C00 THEN GO TO PLINECT;
         LR    @2,@F                                              0113
         CLC   0(2,@2),C00                                        0113
         BC    07,PLINECT                                         0114
*  /*IF OPTIONS ONLY ARE PRESENT INVOCATION IS ENDED*/
*  PLINECTR:;
*         IF LIST(1) = '80'X THEN GO TO AFTER;
PLINECTR L     @2,COMMON+12        COMMON                         0116
         CLI   0(@2),X'80'                                        0116
         BC    08,AFTER                                           0117
*  /* DDNAMES AND PAGEING ARE STILL LEFT TO PROCESS*/
*               DDPTR = LIST(5:8)        /*INIT PTR TO DDNAME LIST*/;
         MVC   @TEMP4(4),4(@2)                                    0118
         L     @F,@TEMP4                                          0118
         ST    @F,COMMON1+228                                     0118
*  /*IF LENGTH IN DDNAMES BR TO PROCESS NAMES*/
*         IF DDLST(1:2) ^= C00 THEN GO TO PDDNAME;
         LR    @4,@F               COMMON1                        0119
         CLC   0(2,@4),C00                                        0119
         BC    07,PDDNAME                                         0120
*  PDDNAMER:;
*  /*IF BYTE 0 OF DDPTR IS '80'X NO MORE TO PROCESS*/
*         IF LIST(5) = '80'X THEN GO TO AFTER;
PDDNAMER L     @2,COMMON+12        COMMON                         0122
         CLI   4(@2),X'80'                                        0122
         BC    08,AFTER                                           0123
*  /*IF WE GET HERE ONLY PAGEING IS LEFT TO PROCESS*/
*               PGPTR = LIST(9:12)       /*INIT PTR TO PAGE LIST*/;
         MVC   @TEMP4(4),8(@2)                                    0124
         L     @F,@TEMP4                                          0124
         ST    @F,PGPTR                                           0124
*  /*IF LGTH PRESENT WE HAVE SOMETHIMG TO PROCESS OTHERWISE WE DONE*/
*         IF PAGE(1:2) = C00 THEN GO TO AFTER /*BR IF NO LENGTH*/;
         LR    @4,@F                                              0125
         CLC   0(2,@4),C00                                        0125
         BC    08,AFTER                                           0126
*                                        /* PROCESS PAGEING HERE*/
*  /* NUMBER WILL BE PRESENT IN BINARY .PLACE NUMBER IN COMMON AREA*/
*  /* FOR MSGMOD*/
*               PAGENO = PAGE(3:6)       /*NUMBER TO COMMON AREA*/;
         MVC   COMMON(4),2(@4)                                    0127
*               R6 = PAGENO              /*BINARY NUMBER TO REGISTER*/;
         L     @6,COMMON                                          0128
*  /* CONVERT TO PACKED DECIMAL*/
*         GEN ( CVD 6,Q);
          CVD 6,Q
         DS    0H
*               PAGENO(1:4) = Q(5:8)     /*PACKED NUMBER TO PAGENO*/;
         MVC   COMMON(4),COMMON+220                               0130
*                                        /* UNPACK PAGE NUMBER*/
*         GEN   ( UNPK Q(8),PAGENO(4));
          UNPK Q(8),PAGENO(4)
         DS    0H
*               QSIGN = 'F0'X | QSIGN  /* OR TO SIGN BIT */;
         OI    COMMON+223,X'F0'                                   0132
*               PAGENO(1:4) = Q(5:8)     /*MAKE NUMBER READY FOR MSG*/;
         MVC   COMMON(4),COMMON+220                               0133
*         GO TO AFTER                    /*BR INVOCATION DONE*/;
         BC    15,AFTER                                           0134
*  /* LINECT IS PROCESSED HERE.NUMBER IS CONVERTED TO DECIMAL AND*/
*  /*PLACED AT LOCATION LINECT.THIS IS USED FOR COMPARISON BY MSG*/
*  PLINECT:;
*                       /* IS LENGTH CORRECT IN OPTION FIELD*/
*         IF OPTIONS(1:2) ^= '000C'X THEN GO TO PLINECT2;   /*@ZA07373*
PLINECT  L     @2,OPTPTR                                          0136
         CLC   0(2,@2),@X6                                        0136
         BC    07,PLINECT2                                        0137
*         IF OPTIONS(3:10) ='LINECNT=' THEN GO TO PLINECT1; /*@ZA07373*
         CLC   2(8,@2),@C7                                        0138
         BC    08,PLINECT1                                        0139
*  /* AT THIS POINT WILL INIT MESSAGE INDICATING INCORRECT KEYWORD AND*
*  /*DEFAULT LINE COUNT.IT WILL BE PRINTED AFTER SYSPRINT IS OPENED*/
*  PLINECT2:;
*               MS = 26     /* INDICATE MESSAGE*/;
PLINECT2 LA    @F,26                                              0141
         STH   @F,COMMON1+102                                     0141
*               GO TO PLINECTR /*BR TO CONTINUE PROCESSING*/;
         BC    15,PLINECTR                                        0142
*  PLINECT1:;
*               COUNTER=OPTIONS(11:14); /* LINECNT TO COUNTER @ZA07373*
PLINECT1 L     @2,OPTPTR                                          0144
         MVC   @TEMP4(4),10(@2)                                   0144
         L     @F,@TEMP4                                          0144
         ST    @F,COMMON+232                                      0144
*                                        /* PACK NUMBER */
*               GEN ( PACK Q(8),COUNTER(4));
          PACK Q(8),COUNTER(4)
         DS    0H
*     /* CONVERT TO BINARY */
*                                        /*CONVERT TO BINARY LINECT*/
*         GEN   ( CVB  6,Q);
          CVB  6,Q
         DS    0H
*                   /* DO NOT ALLOW LINECT OF 1 USE DEFAULT OF 58 */
*         IF R6 = 1 THEN GO TO PLINECTR ;
         CH    @6,@D2                                             0147
         BC    08,PLINECTR                                        0148
*               LINECT = R6              /*NUMBER TO COMMON AREA*/;
         ST    @6,COMMON+4                                        0149
*               LINECTR = 0              /*CLEAR COUNTER FOR MSGMOD*/;
         SR    @F,@F                                              0150
         ST    @F,COMMON+8                                        0150
*         GO TO PLINECTR                 /*BR TO CONTINUE INVOCATION*/;
         BC    15,PLINECTR                                        0151
*  /*******************************************************************
*  /**                                                                *
*  /**    ENTRY HERE INDICATES THAT A REPLACEMENT FOR SYSPRINT OR     *
*  /** SYSIN MAY BE INDICATED.THE FIRST TWO BYTES OF DDLST WILL       *
*  /** CONTAIN THE LENGTH.A LENGTH GT OR EQUAL TO 42 INDICATES THAT   *
*  /** SYSIN AND SYSPRINT BOTH MAY BE PRESENT.A LENGTH GT OR EQUAL TO *
*  /** 34 INDICATES THAT SYSIN ONLY MAY BE PRESENT.ZEROS AT SPECIFIC  *
*  /** LOCATION INDICATE PARAMETER NOT PRESENT.                       *
*  /**                                                                *
*  /*******************************************************************
*  PDDNAME:;
*  /*FIRST TWO BYTES OF DDLIST CONTAINS LENGTH.IF LGTH=> 42 INDICATES*/
*  /*SYSIN AND SYSPRINT BOTH MAY BE PRESENT.IF LGTH => 34 INDICATES*/
*  /*THAT SYSIN ONLY MAY BE PRESENT.ZEROES AT SPECIFIC LOCATION INDICA*
*  /*TES PARAMETER NOT PRESENT*/
*         IF DDLST(1:2) => '0030'X THEN DO /*ARE BOTH DD FIELDS HERE*/;
PDDNAME  L     @2,COMMON1+228      COMMON1                        0153
         CLC   0(2,@2),@X8                                        0153
         BC    04,@9FF                                            0153
*   /* Q- NAME FOR SYSPRINT */
*         IF DDLST(43:50) ^= '0000000000000000'X THEN DO;
         CLC   42(8,@2),@X9                                       0155
         BC    08,@9FE                                            0155
*               DCBPTR = ADDR(SYSP)      /*PICK UP SYSP ADDR*/;
         LA    @F,COMMON+20                                       0157
         ST    @F,COMMON+300                                      0157
*               DDNAME(1:8) = DDLST(43:50) /* NEW NAME TO SYSP DCB*/;
         LR    @4,@F               COMMON                         0158
         MVC   40(8,@4),42(@2)                                    0158
*           END;
*  /* NO SYSPRINT DDNAME AS FIELD CONTAINED 0 */
*         END;
@9FE     EQU   *                                                  0160
*         IF DDLST(1:2) => '0028'X THEN DO /*COULD SYSIN BE PRESENT*/;
@9FF     L     @2,COMMON1+228      COMMON1                        0161
         CLC   0(2,@2),@X10                                       0161
         BC    04,@9FD                                            0161
*         IF DDLST(35:42) ^= '0000000000000000'X THEN DO;
         CLC   34(8,@2),@X9                                       0163
         BC    08,@9FC                                            0163
*               DCBPTR = ADDR(SYSI)      /*PICK UP SYSI ADDR*/;
         LA    @F,COMMON+116                                      0165
         ST    @F,COMMON+300                                      0165
*               DDNAME(1:8) = DDLST(35:42) /*NEW NAME TO SYSI DCB*/;
         LR    @4,@F               COMMON                         0166
         MVC   40(8,@4),34(@2)                                    0166
*           END;
*  /* NO SYSIN DDNAME AS FIELD CONTAINED 0 */
*         END;
*               GO TO PDDNAMER           /*BR TO CONTINUE INVOCATION*/;
         BC    15,PDDNAMER                                        0169
*  /*******************************************************************
*  /**                                                                *
*  /**    SYSIN AND SYSPRINT ARE OPENED HERE.CONDITION CODE 12        *
*  /** RETURNED IF UNABLE TO OPEN SYSIN.THE OPEN LIST IS INITILIZED   *
*  /** AND THE E FORM OF THE OPEN MACRO USGD.                         *
*  /**                                                                *
*  /*******************************************************************
*  /* THIS SECTION INITILIZES OPEN LIST WHICH IS IN DYNAMIC AREA */
*  /* LIST MUST BE IN THIS AREA TO BECOME REENTRANT */
*  /* INPUT DCB ADDRESS  */
*  AFTER:;
*         OPTBYTE1 = ADDR(SYSI);
AFTER    LA    @F,COMMON+116                                      0171
         ST    @F,COMMON+236                                      0171
*  /* OUTPUT DCB ADDRESS   */
*         OPTBYTE2 = ADDR(SYSP);
         LA    @F,COMMON+20                                       0172
         ST    @F,COMMON+240                                      0172
*  /* FIRST ENTRY FOR LIST TAKE DISP FROM DD CARD  */
*         OPTBYTE1(1) = '00'X ;
         MVI   COMMON+236,X'00'                                   0173
*  /* LAST ENTRY T AKE DISP FROM DD CARD  */
*         OPTBYTE2(1) = '8F'X ;
         MVI   COMMON+240,X'8F'                                   0174
*         R1 = ADDR(OPENLIST)  /* LOAD REG FOR OPEN */;
         LA    @1,COMMON+236                                      0175
*         GEN (OPEN MF=(E,(1)));
         OPEN MF=(E,(1))
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /**    SYSIN IS TESTED FOR BEING OPEN,IF OPEN A HEADER MESSAGE IS  *
*  /** INDICATED AND A BRANCH IS TAKEN TO GET THE HEADING PRINTED.    *
*  /** IN CASE ANOTHER MESSAGE IS ALREADY INDICATED ITS NUMBER IS     *
*  /** SAVED.THE OTHER MESSAGE WILL BE PRINTED AFTER THE HEADER MSG.  *
*  /** IF SYSIN IS NOT OPEN AN ERROR IS INDICATED,WE WILL PRINT THE   *
*  /** MSG,SET CONDITION CODE 12 AND RETURN TO THE SYSTEM.            *
*  /**                                                                *
*  /*******************************************************************
*  TEST1  :;
*         DCBPTR = ADDR(SYSI) /**PICK UP SYSI ADDRESS **/;
TEST1    LA    @F,COMMON+116                                      0178
         ST    @F,COMMON+300                                      0178
*         /* IF SYSIN IS OPEN GO TO PROCESS CARD */
*         IF OFLGS(4) = '1'B THEN DO ;
         LR    @2,@F               COMMON                         0179
         TM    48(@2),B'00010000'                                 0179
         BC    12,@9FB                                            0179
*         IF MS ^= 0 THEN DO /*IS THERE A MESS TO PRINT AFTER HDR MSG*/
         SR    @F,@F                                              0181
         CH    @F,COMMON1+102                                     0181
         BC    08,@9FA                                            0181
*               SAVEMS = MS              /*YES-SAVE MESSAGE*/;
         MVC   COMMON1(2),COMMON1+102                             0183
*               MS = 1                   /*INDICATE HDR MESSAGE*/;
         LA    @F,1                                               0184
         STH   @F,COMMON1+102                                     0184
*     /* LINK TO PRINT MESSAGE */
*         GEN ( LINK EP=IEBDGMSG);
          LINK EP=IEBDGMSG
         DS    0H
*               MS = SAVEMS              /*RESTORE MESSAGE*/;
         MVC   COMMON1+102(2),COMMON1                             0186
*               GO TO A1B35              /*BR TO PRINT AND GET CARD*/;
         BC    15,A1B35                                           0187
*         END;
*               MS  =  1 ; /* PRINT HEADING ON SYSPRINT */
@9FA     LA    @F,1                                               0189
         STH   @F,COMMON1+102                                     0189
*               GO TO A1B35;  /* TO PRINT HEAD */
         BC    15,A1B35                                           0190
*         END;
*               MS = 24 ;                 /* MESSAGE  24  */
@9FB     LA    @F,24                                              0192
         STH   @F,COMMON1+102                                     0192
*         GEN ( LINK EP=IEBDGMSG);
          LINK EP=IEBDGMSG
         DS    0H
*               CONCODE = 12;  /**PLACE 12 IN CONCODE**/
         LA    @F,12                                              0194
         STH   @F,COMMON1+2                                       0194
*         GO TO A2J2           /**BR TO END JOB**/;
         BC    15,A2J2                                            0195
*  GO TO A2B3;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION WILL TURN ON FLAGSW(1) FIRST BIT TO INDICATE   *
*  /** THAT SPECIAL ERROR FLAGGING IS WANTED.                         *
*  /**                                                                *
*  /*******************************************************************
*  PRINTERR:;
*               FLAGSW(1) = '1'B /**INDICATE FLAGGING WANTED WITH MSG**
PRINTERR OI    COMMON2+30,B'10000000'                             0198
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION WILL TEST FOR FLUSHING OF REMAINDER OF CONTROL *
*  /** CARDS.IF SWITCH IS ON A BRANCH IS TAKEN TO PERFORM CLEANUP.    *
*  /** THIS SECTION ALSO SETS CONDITION CODE TO 8 AND TURNS ON THE    *
*  /** NOGO SWITCH FOR SYNTAX CHECKING ONLY.                          *
*  /**                                                                *
*  /*******************************************************************
*  PRINTMSG:;
*  A1B3:;
PRINTMSG EQU   *                                                  0200
*         IF CONCODE < 8 THEN DO; /** IS CONCODE LESS THAN EIGHT**/
A1B3     LA    @F,8                                               0201
         CH    @F,COMMON1+2                                       0201
         BC    12,@9F9                                            0201
*               CONCODE = 8    /**CONDITION CODE FOR USER**/;
         STH   @F,COMMON1+2                                       0203
*         END;
*               EPSW = 0       /**TURN OFF MSG INDICATOR SWITCH**/;
@9F9     MVI   COMMON2+32,0                                       0205
*    /* TURN ON NOGO SWITCH SO THAT SYNTAX WILL BE CHECKED*/
*               NOGOSW = ON              /*TURN ON NOGO SWITCH*/;
         MVC   COMMON2+15(1),ON                                   0206
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION PICKS UP THE DATA GENERATOR OUTPUT DATA SET    *
*  /** ADDRESS AND 'LINKS' TO THE MESSAGE MODULE TO PRINT A MESSAGE.  *
*  /** IF CONDITION CODE IS GT OR EQUAL TO 12,UPON RETURN FROM THE    *
*  /** MESSAGE MODULE,CONTROL RETURNS TO THE USER.                    *
*  /**                                                                *
*  /*******************************************************************
*  A1B35:;
*         GEN ( LINK EP=IEBDGMSG);
A1B35     LINK EP=IEBDGMSG
         DS    0H
*         IF FLUSHSW1 = ON THEN GO TO A2J2 /*BR TO CLEANUP USER STOP*/;
         CLC   COMMON2+37(1),ON                                   0209
         BC    08,A2J2                                            0210
*         IF CONCODE => 12 THEN DO /**IF CCODE EQ OR GT 12 DO BELOW*/;
         LA    @F,12                                              0211
         CH    @F,COMMON1+2                                       0211
*         GO TO A2J2           /**BR TO END JOB**/;
         BC    13,A2J2                                            0213
*         END;
*  GEN;
@9F8     EQU   *                                                  0215
***********************************************************************
*        THIS SECTION WILL GET A CARD FROM SYSIN AND PUT IT ON SYSOUT *
*        COUNTER IS INITILIZED TO ADDRESS OF THE INPUT BUFFER + 72    *
*        FOR THE SCANNING OF CONTINUATIONS                            *
***********************************************************************
         DS    0H
*  A2B3:;
*  R9 = ADDR(INBUFA) /*REG 9 POINTS AT INBUF */;
A2B3     LA    @9,COMMON1+114                                     0217
*  /* INITILIZE COUNTER TO INBUF COL 72 */
*               COUNTER = R9 + 70  /* INIT COUNTER TO CONT COLUMN */;
         LA    @F,70                                              0218
         AR    @F,@9                                              0218
         ST    @F,COMMON+232                                      0218
*  /*******************************************************************
*  /**                                                                *
*  /**                                                                *
*  /**    ***** *****  GET A CONTROL CARD  ***** *****                *
*  /**                                                                *
*  /**                                                                *
*  /*******************************************************************
*  A2B33:;
*            DCBPTR = ADDR(SYSI); /*POINT TO SYSIN DCB         @ZA28703
A2B33    LA    @F,COMMON+116                                      0220
         ST    @F,COMMON+300                                      0220
*         IF DCBPTR -> EODSW = ON THEN GO TO CLEANUP; /*EOF ON @ZA24201
         LR    @2,@F               COMMON                         0221
         CLC   268(1,@2),ON                                       0221
         BC    08,CLEANUP                                         0222
*                          /*SYSIN ALREADY, THEN GO TO CLEANUP @ZA24201
*  GEN(GET SYSI,INBUFA);
         GET SYSI,INBUFA
         DS    0H
*  /***************************************************************/
*  /**  FLUSH TO END OF DSD GROUP HERE ****************************/
*  /***************************************************************/
*         IF FLUSHSW = ON THEN DO        /*IF FLUSHSW ON DO BELOW*/;
         CLC   COMMON2+36(1),ON                                   0224
         BC    07,@9F7                                            0224
*  A2B34:;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION WILL FLUSH CARDS ON INPUT UNTIL AN 'END'       *
*  /** CONTROL CARD IS FOUND.THEY ARE NOT PRINTED ON SYSPRINT.WHEN END*
*  /** IS FOUND THE END SWITCH IS TURNED ON AND A BRANCH IS TAKEN TO  *
*  /** CLEANUP.                                                       *
*  /**                                                                *
*  /*******************************************************************
*         IF INBUF(1) ^= ' ' THEN GO TO A2B344 /**BR TO LOOK FOR END**/
A2B34    CLI   0(@9),C' '                                         0227
         BC    07,A2B344                                          0228
*               R9 = R9 + 1    /**INCREMENT SCAN POINTER**/;
         AH    @9,@D2                                             0229
*         IF R9 = COUNTER THEN GO TO A2B33 /*BR AT END OF SCAN**/;
         C     @9,COMMON+232                                      0230
         BC    08,A2B33                                           0231
*               GO TO A2B34    /** LOOP ON SCAN**/;
         BC    15,A2B34                                           0232
*  A2B344:;
*         IF INBUF(1:3) ^= 'END' THEN GO TO A2B33 /**BR TO GET CARD**/;
A2B344   CLC   0(3,@9),@C14                                       0234
         BC    07,A2B33                                           0235
*               ENDSW = ON     /**TURN ON END SWITCH**/;
         MVC   COMMON2+28(1),ON                                   0236
*               GO TO CLEANUP  /**BRANCH TO END THIS DSD GROUP**/;
         BC    15,CLEANUP                                         0237
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION IS ENTERED WHEN NO FLUSHING IS REQUIRED.IT WILL*
*  /** CAUSE THE CARD IMAGE TO BE PLACED ON SYSPRINT.IF PRINTING OF   *
*  /** ALL FLUSHED CARDS IS REQUIRED THIS SECTION SHOULD BE MOVED     *
*  /** AHEAD OF PRIOR SECTION.                                        *
*  /**                                                                *
*  /*******************************************************************
*               MS  =  30                 /* PRINT CARD ON SYSPRINT */;
@9F7     LA    @F,30                                              0239
         STH   @F,COMMON1+102                                     0239
*                GEN ( LINK EP=IEBDGMSG);
          LINK EP=IEBDGMSG
         DS    0H
*  GEN;
***********************************************************************
*        THIS SECTION WILL SCAN CONTROL CARDS AT LOCATION INBUF. R9   *
*        IS USED AS A POINTER WHICH IS UPDATED AS NEEDED BY THE SCAN  *
*        CONTROL CARD NAMES ARE SKIPPED BY IF PRESENT.A NAME IS       *
*        INDICATED TO THE SCAN BY A NON-BLANK IN COL ONE UPON FIRST   *
*        ENTRY WHEN INSPECTING THE CARD. A BLANK SHOULD FOLLOW THE    *
*        NAME.BLANK CARDS ARE FLUSHED. SEVERAL SWITCHES ARE TESTED    *
*        FOR CONTINUATION AND APPROPRIATE BRANCHES GIVEN.CONTINUATION *
*        FOR ALL CONTROL CARDS STARTS IN COL 4                        *
***********************************************************************
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /**    ***** CONTROL CARD SCAN STARTS HERE *****                   *
*  /**                                                                *
*  /*******************************************************************
*  SCAN:;
*  /*******************************************************************
*  /**                                                                *
*  /**    IF THE FD CONTINUATION SWITCH IS ON,'LINK' TO IEBFDANL.     *
*  /**                                                                *
*  /*******************************************************************
*            IF FDCSW = ON THEN DO /* IF FD CONT CARD DO FOLLOWING*/;
SCAN     CLC   COMMON2(1),ON                                      0243
         BC    07,@9F6                                            0243
*  LINKFD:;
*         GEN ( LINK EP=IEBFDANL) ;
LINKFD    LINK EP=IEBFDANL
         DS    0H
*                              /***************************************
*                              /* UPON RETURN FROM THE FD ANALYSIS   **
*                              /* MODULE WE MUST DETERMINE WHETHER A  *
*                              /* MESSAGE IS TO BE PRINTED OR ANOTHER *
*                              /* CONTROL CARD IS NEEDED.A MESSAGE IS *
*                              /* INDICATED BY EPSW BEING A 1.        *
*                              /***************************************
*  LINKFD1:;
*         IF EPSW ^= 1 THEN GO TO A2B3 /**GET A CARD NO MSG REQD**/;
LINKFD1  CLI   COMMON2+32,1                                       0248
         BC    07,A2B3                                            0249
*               GO TO A1B3     /**BRANCH TO ISSUE MESSAGE**/;
         BC    15,A1B3                                            0250
*            END;
*  /*******************************************************************
*  /**                                                                *
*  /**    IF THE CR CONTINUATION SWITCH IS ON,BRANCH TO ISSUE 'LINK'  *
*  /** TO IEBCRANL.                                                   *
*  /**                                                                *
*  /*******************************************************************
*                   /* IF CRCSW ON GO TO LINK TO CRANL*/
*         IF CRCSW = ON THEN GO TO A2G35;
@9F6     EQU   *                                                  0252
         CLC   COMMON2+18(1),ON                                   0252
         BC    08,A2G35                                           0253
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION WILL TEST FOR COMMENTS CONTINUATION FOR THIS   *
*  /** MODULE(IGBDG) AND (IGBFDANL).AS LONG AS A NON-BLANK IS FOUND   *
*  /** IN COLUMN 72 AND THE COMCSW IS ON A BRANCH WILL BE TAKEN TO    *
*  /** GET ANOTHER CARD.THE LAST COMMENT CARD,BLANK IN 72,CAUSES THE  *
*  /** COMCSW TO BE TURNED OFF AND A BRANCH TO GET ANOTHER CARD.      *
*  /**                                                                *
*  /*******************************************************************
*         IF COMCSW = ON THEN DO /*IF COM CONT SW ON DO FOLLOWING*/;
         CLC   COMMON2+29(1),ON                                   0254
         BC    07,@9F5                                            0254
*                   /* IF THIS CARD HAS COMM INDICATOR GET ANOTHER */
*         IF COUNTER -> INBUF(2) ^= ' ' THEN GO TO A2B3;
         L     @2,COMMON+232       COMMON                         0256
         CLI   1(@2),C' '                                         0256
         BC    07,A2B3                                            0257
*               COMCSW = OFF /* TURN OFF COMMENT CONT SW */;
         MVC   COMMON2+29(1),OFF                                  0258
*               GO TO A2B3  /* BR TO GET ANOTHER CARD*/;
         BC    15,A2B3                                            0259
*         END;
*  /*******************************************************************
*  /**    THIS SECTION TESTS FOR A DSD CONTINUATION.IF ON,THE SWITCH  *
*  /** IS TURNED OFF,SCAN POINTED AT COLUMN 4 AND BRANCH TO PROCESS.  *
*  /*******************************************************************
*        IF DSDCSW = ON THEN DO          /*IS DSD CONTINUATION SW ON*/;
@9F5     CLC   COMMON2+17(1),ON                                   0261
         BC    07,@9F4                                            0261
*              DSDCSW = OFF             /* YES TURN IT OFF*/;
         MVC   COMMON2+17(1),OFF                                  0263
*         R9 = R9 + 3          /**POINTER TO CONT COLUMN**/;
         AH    @9,@D3                                             0264
*               GO TO A3C4 /* BR TO CONTINUE SCANNING DSD */;
         BC    15,A3C4                                            0265
*                             END;
*  /*******************************************************************
*  /**                                                                *
*  /** IF THE FD PICTURE CONTINUE SWITCH IS ON A BRANCH IS TAKEN TO   *
*  /** 'LINK' TO IEBFDANL FOR FURTHER PROCESSING.                     *
*  /*******************************************************************
*         IF FDPCSW = ON THEN GO TO LINKFD /* BR TO LINK TO FD */;
@9F4     CLC   COMMON2+2(1),ON                                    0267
         BC    08,LINKFD                                          0268
*  /*******************************************************************
*  /**                                                                *
*  /** IF FIRST COLUMN OF CARD IS NOT BLANK BRANCH TO PROCESS         *
*  /** A CONTROL CARD NAME.                                           *
*         IF INBUF(1) ^= ' ' THEN GO TO LABEL1 /**BR IF COL1 NOT BLANK*
         CLI   0(@9),C' '                                         0269
         BC    07,LABEL1                                          0270
*  SCAN1:;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION WILL SCAN OUT BLANKS LOOKING FOR THE FIRST     *
*  /** NON-BLANK COLUMN IN THE CARD.A NON-BLANK CAUSES A BRANCH TO    *
*  /** PROCESS FOR THE CONTROL CARD NAME.                             *
*  /*******************************************************************
*         /* IF CHAR FOUND GO TO ANALYZE KEYWORD */
*         IF INBUF(1) ^= ' ' THEN GO TO A2D3 /*BR NON-BLANK TO PROCESS*
SCAN1    CLI   0(@9),C' '                                         0272
         BC    07,A2D3                                            0273
*                                       /* THIS WILL SCAN OUT BLANKS */
*         IF R9 = COUNTER THEN GO TO A2B3 /*BR IF AT COLUMN 71        *
         C     @9,COMMON+232                                      0274
         BC    08,A2B3                                            0275
*               R9 = R9 + 1    /** ADD ONE TO SCAN POINTER */;
         AH    @9,@D2                                             0276
*  GO TO SCAN1;
         BC    15,SCAN1                                           0277
*  GEN;
***********************************************************************
*  THIS SECTION SCANS OUT ANY NAME PRESENT UPON A CONTROL CARD. A NAME*
*  IS RECOGNISED AS ANY NON-BLANK STARTING IN COL 1. IT MAY BE 8 CHAR *
*  LONG.THE NAME MUST BE FOLLOWED BY A BLANK.                         *
***********************************************************************
         DS    0H
*  LABEL1:;
*      R9= R9 + ONE;
LABEL1   AH    @9,ONE                                             0280
*  IF INBUF(1) ^=' ' THEN DO;       /* TO SCAN OUT CONTROL CARD LABEL*/
         CLI   0(@9),C' '                                         0281
         BC    08,@9F3                                            0281
*         IF R9 > ADDR(INBUFA) + 8 THEN DO /* IS NAME TO LONG */;
         LA    @F,8                                               0283
         LA    @0,COMMON1+114                                     0283
         AR    @F,@0                                              0283
         CR    @F,@9                                              0283
         BC    10,@9F2                                            0283
*               MS  =  12                 /* MESSAGE 12  */;
         LA    @F,12                                              0285
         STH   @F,COMMON1+102                                     0285
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0286
*                               END;
*         GO TO LABEL1;
*                         END;
*      GO TO SCAN1;
*  GEN;
***********************************************************************
*        THIS SECTION SCANS OUT DSD,FD,CREATE. A DSD CARD MUST BE     *
*        PRESENT FIRST OTHERWISE NOGO IS SET AND SYNTAX CHECKING      *
*        ONLY IS PERFORMED. MULTIPLE DSD CARDS NOT ALLOWED. THE LAST  *
*        ONE WILL BE EFFECTIVE. LABELS OF A3 OPERATE ON DSD.LABELS OF *
*        A4 OPERATE ON FD. LABELS OF A6 OPERATE ON CREATE             *
***********************************************************************
         DS    0H
*  A2D3:;
*  /* LOOK FOR DSD BEING FIRST CARD */
*  /* IF WE HAVE A DSD GO TO LOOK FOR OTHER CONTROL CARDS */
*  IF INBUF(1:3)=DSD THEN GO TO A3B3;
A2D3     CLC   0(3,@9),DSD                                        0293
         BC    08,A3B3                                            0294
*         IF DSDSW = ON THEN GO TO A2G3  /* BR IF DSDSW ON*/;
         CLC   COMMON2+14(1),ON                                   0295
         BC    08,A2G3                                            0296
*         FLUSHSW = ON         /**FLUSH TO END OF DSD GROUP**/;
         MVC   COMMON2+36(1),ON                                   0297
*               MS = 18        /** MSG DSD CARD NOT FIRST **/;
         LA    @F,18                                              0298
         STH   @F,COMMON1+102                                     0298
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0299
*  A2G3:;
*  /*******************************************************************
*  /**                                                                *
*  /**    IF CONTROL CARD IS 'FD' THEN BRANCH TO LINK TO IEBFDANL     *
*  /*******************************************************************
*         IF INBUF(1:2) = 'FD' THEN GO TO LINKFD ;
A2G3     CLC   0(2,@9),@C15                                       0301
         BC    08,LINKFD                                          0302
*  /*******************************************************************
*  /**                                                                *
*  /**    IF CONTROL CARD IS 'CREATE' THEN LINK TO IEBCREAT           *
*  /**                                                                *
*  /*******************************************************************
*         IF INBUF(1:6) = 'CREATE' THEN DO;
         CLC   0(6,@9),@C16                                       0303
         BC    07,@9F1                                            0303
*  A2G35:;
*         GEN( LINK  EP=IEBCRANL);
A2G35     LINK  EP=IEBCRANL
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /**    UPON RETURN FROM IEBCREAT WE TEST FOR NEED FOR MESSAGE      *
*  /** AND FLUSHING TO END OF JOB.FLUSHSW1 EQ END OF JOB.EPSW A ONE   *
*  /** EQ NEED FOR MESSAGE.IF NEITHER ONE IS INDICATED A BRANCH IS    *
*  /** TAKEN TO GET ANOTHER CONTROL CARD.                             *
*  /*******************************************************************
*         IF EPSW  =  1  THEN GO TO A1B3;
         CLI   COMMON2+32,1                                       0307
         BC    08,A1B3                                            0308
*         IF FLUSHSW1 = ON THEN GO TO A2J2 /**BR TO CLEANUP USER STOP**
         CLC   COMMON2+37(1),ON                                   0309
         BC    08,A2J2                                            0310
*               GO TO A2B3;
         BC    15,A2B3                                            0311
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION TESTS FOR 'REPEAT'.IF FOUND A BRANCH IS TAKEN  *
*  /** TO PROCESS THE REPEAT CONTROL CARD.                            *
*  /**                                                                *
*  /*******************************************************************
*                                        /* IF REPEAT CONTROL CARD */
*                                        /* GO TO PROCESS IT */
*         IF INBUF(1:6) = 'REPEAT' THEN GO TO A3C14;
@9F1     CLC   0(6,@9),@C17                                       0313
         BC    08,A3C14                                           0314
*  /**                                                                *
*  /*******************************************************************
*  /**    THIS SECTION TESTS FOR END.IF FOUND THE END SWITCH IS       *
*  /** TURNED ON AND A BRANCH TAKEN TO CLEANUP.                       *
*  /**                                                                *
*  /*******************************************************************
*                                       /* IF END CONTROL CARD GO */
*                                        /* TO PROCESS IT */
*         IF INBUF(1:3) = 'END' THEN DO;
         CLC   0(3,@9),@C14                                       0315
         BC    07,@9F0                                            0315
*               ENDSW = ON  /* TURN ON END SWITCH*/;
         MVC   COMMON2+28(1),ON                                   0317
*         GO TO CLEANUP;
         BC    15,CLEANUP                                         0318
*          END;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION TESTS FOR 'DUMP'.IF FOUND REGISTER 15 IS LOADED*
*  /** WITH ZERO AND A BRANCH TAKEN ON REG 15.THIS FACILITY ALLOWS THE*
*  /** FIELD ENGINEER TO CAUSE AN'ABEND' DUMP TO OCCUR.IT IS INTENDED *
*  /** THAT THIS FACILITY WILL BE EXPANDED SOME TIME IN THE FUTURE TO *
*  /** INCORPORATE MUCH MORE DIAGNOSTIC CAPABILITY.                   *
*  /**                                                                *
*  /*******************************************************************
*         IF INBUF(1:4) ^= 'DUMP' THEN GO TO A2G36;
@9F0     CLC   0(4,@9),@C18                                       0320
         BC    07,A2G36                                           0321
*               R15 = 0 ;
         SR    @F,@F                                              0322
*               GEN ( BR 15 );
          BR 15
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /**    AT THIS POINT A CORRECT CONTROL CARD NAME HAS NOT BEEN FOUND*
*  /** A MESSAGE IS INDICATED AND A BRANCH TAKEN TO ISSUE THE MESSAGE.*
*  /**                                                                *
*  /*******************************************************************
*  A2G36:;
*         IF INBUF(1:2) = '/*' THEN
A2G36    CLC   0(2,@9),@C19                                       0325
*         GO TO CLEANUP;
         BC    08,CLEANUP                                         0326
*               MS  =  2                 /* MESSAGE  2  */;
         LA    @F,2                                               0327
         STH   @F,COMMON1+102                                     0327
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0328
*  GEN;
***********************************************************************
*        THIS SECTION PERFORMS THE CLEANUP FUNCTION IT IS ENTERED FROM*
*        EODAD ON SYSIN WHEN A /* CARD IS FOUND IN THE INPUT STREAM.  *
* ENTRY IS ALSO FROM THE END CONTROL CARD*******                      *
***********************************************************************
         DS    0H
*  CLEANUP:;
*  A2J2:;
CLEANUP  EQU   *                                                  0331
*  /*******************************************************************
*  /**    A MESSAGE WILL BE INDICATED IF THE REPEATSW IS LEFT ON BY   *
*  /** THE CREATE MODULE.CREATE TURNS THIS OFF IF THE REPEAT FUNCTION *
*  /** HAS BEEN FULLY SATISFIED.                                      *
*  /*******************************************************************
*         IF REPEATSW = ON THEN DO  /*IS REPEAT QUANTITY SATISFIED*/;
A2J2     CLC   COMMON2+42(1),ON                                   0332
         BC    07,@9EF                                            0332
*               MS = 14     /* NO-SET MESSAGE NUMBER */;
         LA    @F,14                                              0334
         STH   @F,COMMON1+102                                     0334
*         GEN ( LINK EP=IEBDGMSG);
          LINK EP=IEBDGMSG
         DS    0H
*                           /* REPEAT QUANTITY SATISFIED HERE */
*         END;
* /********************************************************************
* /*                                                                  *
* /* CHECK IF SPECIFIED PICTURE DOES EXCEED END OR /* ,IF SO GIVE MSG *
* /* AND SET CONDITION CODE .                                   A41807*
* /*                                                                  *
* /********************************************************************
*         IF PICCSW(4) = '1'B           /* STILL PICTURE EXPECT A41807*
*              THEN DO ;            /* END OF CONTROL STATEMENT A41807*
@9EF     TM    COMMON2+26,B'00010000'                             0337
         BC    12,@9EE                                            0337
*                   MS = 16 ;       /* SET MESSAGE              A41807*
         LA    @F,16                                              0339
         STH   @F,COMMON1+102                                     0339
*               PICCSW(4) = '0'B ;      /* SET TO ZERO          A41807*
         NI    COMMON2+26,B'11101111'                             0340
*                   FLUSHSW1 = ON ;    /* SET SWITCH TO END JOB A41807*
         MVC   COMMON2+37(1),ON                                   0341
*                   GO TO PRINTERR ;   /* PRINT ERROR MESSAGE   A41807*
         BC    15,PRINTERR                                        0342
*          END ;                    /* END OF CHECK PICTURE LEN A41807*
* /*                                                                  *
*         IF ENDSW = ON THEN GO TO A2J5 /**BR AROUND COMP MSG,END ON **
@9EE     CLC   COMMON2+28(1),ON                                   0344
         BC    08,A2J5                                            0345
*               MS = 29 /**INDICATE SUCCESSFUL COMPLETION MESSAGE**/;
         LA    @F,29                                              0346
         STH   @F,COMMON1+102                                     0346
*  A2J4:;
*                              /** LINK TO MESSAGE MODULE **/
*         GEN ( LINK EP=IEBDGMSG);
A2J4      LINK EP=IEBDGMSG
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /** ******** CLEANUP MODULE ENTERED FROM HERE ********             *
*  /**                                                                *
*  /*******************************************************************
*  A2J5:;
*         GEN ( LINK EP=IEBDGCUP);
A2J5      LINK EP=IEBDGCUP
         DS    0H
*  A2J6:;
*  /*******************************************************************
*  /**    UPON RETURN FROM CLEANUP THE END SWITCH BEING ON INDICATES  *
*  /** THAT THERE MAY BE ANOTHER GROUP OF CONTROL CARDS TO PROCESS.   *
*  /** THE SWITCHES ARE TURNED OFF AND A BRANCH TAKEN TO GET ANOTHER  *
*  /** CONTROL CARD.IF NO MORE CONTROL CARDS ARE PRESENT A SLASH      *
*  /** ASTERICK CARD SHOULD BE PRESENT.CLEANUP WOULD BE ENTERED AGAIN *
*  /** WITH THE END SWITCH OFF.THIS CAUSES DATA GENERATOR SYSIN AND   *
*  /** SYSPRINT DATA SETS TO BE CLOSED.UPON RETURN FROM CLEANUP A     *
*  /** RETURN IS TAKEN FROM THIS MODULE BACK TO THE SYSTEM.           *
*  /*******************************************************************
*         IF ENDSW = ON THEN DO /**DO BELOW IF ENDSW ON **/;
A2J6     CLC   COMMON2+28(1),ON                                   0352
         BC    07,@9ED                                            0352
*               SWITCH(1) = '00'X /**FIRST SW TO OFF **/;
         MVI   COMMON2,X'00'                                      0354
*         SWITCH(2:53) = SWITCH(1:52);
         MVC   COMMON2+1(52),COMMON2                              0355
*               SYSINSEL = OFF /** TURN OFF SYSIN SEL SWITCH**/;
         MVC   COMMON3+28(1),OFF                                  0356
*         GO TO A2B3;
         BC    15,A2B3                                            0357
*         END;
*         GO TO USER           /** RETURN TO USER **/;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION WILL SCAN FOR KEYWORDS ON THE DSD CONTROL CARD.*
*  /** KEYWORDS ALLOWED ARE OUTPUT= AND INPUT=. AN EIGHT CHARACTER    *
*  /** DD NAME IS ALLOWED WITHIN THE PARANTHESIS.ANY NUMBER OF DDNAMES*
*  /** ARE ALLOWED WITHIN THE DELIMITERS,ALTHOUGH THE CURRENT         *
*  /** IMPLEMENTATION WITHIN THE CREATE MODULE CAN ONLY PROCESS ONE   *
*  /** OUTPUT DATA SET.THE SRL ONLY ALLOWS ONE OUTPUT DATA SET.ONE    *
*  /** INPUT DATA SET MAY BE PROCESSED PER CREATE CARD.               *
*  /**                                                                *
*  /*******************************************************************
* A3B3:;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION TESTS AND ALLOWS ONLY ONE DSD CARD PER GROUP.  *
*  /** MESSAGE IS INDICATED IF MORE THAN ONE.                         *
*  /**                                                                *
*  /*******************************************************************
*         IF DSDSW = ON THEN DO /**IS A DSD ALREADY PRESENT**/;
A3B3     CLC   COMMON2+14(1),ON                                   0361
         BC    07,@9EC                                            0361
*               MS = 25        /**INDICATE MESSAGE**/;
         LA    @F,25                                              0363
         STH   @F,COMMON1+102                                     0363
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0364
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION INITILIZES REPEATNO AND CREATEND TO ONE FOR THE*
*  /** CREATE MODULE IF THE NOGO SWITCH ISNT ON.                      *
*  /*******************************************************************
*         IF NOGOSW = ON THEN GO TO A3C3 /* BR IF NOGO SWITCH IS ON*/;
@9EC     CLC   COMMON2+15(1),ON                                   0366
         BC    08,A3C3                                            0367
*               DSDSW = ON               /* TURN ON DSD SWITCH*/;
         MVC   COMMON2+14(1),ON                                   0368
*               REPEATNO = 1       /* INIT REPEATNO FOR CRANL*/;
         LA    @F,1                                               0369
         STH   @F,COMMON+16                                       0369
*               CREATENO = 1      /*INIT CREATENO FOR CRANL */;
         STH   @F,COMMON+18                                       0370
*  /*******************************************************************
*  /**                                                                *
*  /**    TEST FOR A BLANK FOLLOWING 'DSD' AND ISSUE MSG IF NOT THERE.*
*  /*******************************************************************
* A3C3:;
*         R9 = R9 + THREE /* ADVANCE POINTER TO BLANK */;
A3C3     AH    @9,THREE                                           0372
*         IF INBUF(1) = ' ' THEN GO TO A3C4;
         CLI   0(@9),C' '                                         0373
         BC    08,A3C4                                            0374
*               MS  =  20                /* MESSAGE  20 */;
         LA    @F,20                                              0375
         STH   @F,COMMON1+102                                     0375
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0376
* A3C4:;
*  /** THIS SECTION TESTS FOR PROCESSING OF MULTIPLE DDNAMES WITHIN **/
*  /** THE PARENS.IF DSDSW IS ON,IT IS TURNED OFF AND A BRANCH TAKEN**/
*  /** TO A3C45 FOR FURTHER PROCESSING OF A NAME.SCANNING IS STARTED **
*  /** AT COL FOUR **/
*         IF DSDISW = ON THEN DO /** IS INPUT CONT IND **/;
A3C4     CLC   COMMON2+39(1),ON                                   0378
         BC    07,@9EB                                            0378
*               DSDISW = OFF   /** TURN OFF DSDISW**/;
         MVC   COMMON2+39(1),OFF                                  0380
*               GO TO A3C45    /** BRANCH FOR FURTHER SCANNING**/;
         BC    15,A3C45                                           0381
*         END;
*         IF REPEATSW = ON THEN DO /**DO BELOW IF REPEATSW ON**/;
@9EB     CLC   COMMON2+42(1),ON                                   0383
*               GO TO A3C143   /** BRANCH FOR REPEAT CONT**/;
         BC    08,A3C143                                          0385
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION SCANS OUT ANY BLANK COLUMNS FOLLOWING 'DSD'    *
*  /*******************************************************************
*         IF INBUF(1) = ' ' THEN DO ;
@9EA     CLI   0(@9),C' '                                         0387
         BC    07,@9E9                                            0387
*               R9 = R9 + ONE ;          /* ADD ONE TO BUFFER PTR */
         AH    @9,ONE                                             0389
*                                        /* THIS SCANS OUT BLANKS */
*         IF R9 < COUNTER THEN GO TO A3C4;
         C     @9,COMMON+232                                      0390
         BC    04,A3C4                                            0391
*         IF R9 => COUNTER THEN GO TO A3C441 /**BR IF WRONG COLUMN**/;
         C     @9,COMMON+232                                      0392
         BC    10,A3C441                                          0393
*         END;
* /********************************************************************
* /**                                                                **
* /** THIS SECTION SCANS OUT THE KEYWORDS OUTPUT AND INPUT           **
* /** IT WILL ISSUE A GETMAIN FOR 256 BYTES AS A DDNAME IS FOUND     **
* /** A COPY OF  A DCB IS MOVED INTO THIS AREA AND THE DDNAME FOUND  **
* /** PLACED IN THE DCB.THE DCB IS THEN OPENED.IF UNABLE TO OPEN THE **
* /** DCB A MESSAGE IS ISSUED NOGOI SET AND INPUT STREAM FLUSHED THRU**
* /** THE END CARD.ADDRESSES OF MULTIPLE DCBS ARE KEPT AS FOLLOWS:   **
*  /**    FIRSTGMI = FIRST INPUT DCB,FIRSTGMO = FIRST OUTPUT DCB      *
* /**     NEXTDCB = ADDRESS OF NEXT DCB,WORD 65 OF EACH DCB          **
* /**     DDNAME1 = DDNAME FOR THIS DCB,WORD 66-67 OF EACH DCB       **
* /**                                                                **
* /********************************************************************
*  A3C41:;
@9E9     EQU   *                                                  0395
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION TESTS FOR 'OUTPUT',IF FOUND,THE DSDDSW IS      *
*  /** TURNED ON,POINTER ADJUSTED TO DDNAME AND A BRANCH TAKEN TO     *
*  /** SCAN OUT THE OUTPUT DDNAME.                                    *
*  /*******************************************************************
*                                        /* LOOK FOR OUTPUT KEYWORD  */
*         IF INBUF(1:8) = 'OUTPUT=(' THEN DO;
A3C41    CLC   0(8,@9),@C21                                       0396
         BC    07,@9E8                                            0396
*               DSDOSW = ON              /* TURN ON DSDOSW */;
         MVC   COMMON2+38(1),ON                                   0398
*               R9 = R9 + 8              /* ADJUST POINTER */;
         AH    @9,@D4                                             0399
*               GO TO A3C44              /* BRANCH TO SCAN OUT NAME */;
         BC    15,A3C44                                           0400
*         END;                           /*LOOK FOR INPUT KEYWORD*/;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION TESTS FOR 'INPUT' AND BRANCHES TO SCAN OUT     *
*  /** INPUT DDNAME,OTHERWISE AN ERROR MESSAGE IS INDICATED           *
*  /*******************************************************************
*         IF INBUF(1:7) = 'INPUT=(' THEN GO TO A3C43;
@9E8     CLC   0(7,@9),@C22                                       0403
         BC    08,A3C43                                           0404
*               MS = 5                   /*INVALID KEYWORD MESS */;
         LA    @F,5                                               0405
         STH   @F,COMMON1+102                                     0405
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0406
*  A3C43:;
*  /*******************************************************************
*  /**                                                                *
*  /**    ENTRY HERE FOR INPUT DDNAMES ONLY.THE DSDOSW IS TURNED OFF  *
*  /** AND THE SCAN POINTER ADJUSTED TO POINT TO INPUT DDNAME.        *
*  /*******************************************************************
*               DSDOSW = OFF   /** TURN OFF DSD OUTPUT SWITCH**/;
A3C43    MVC   COMMON2+38(1),OFF                                  0408
*               R9 = R9 + 7              /* ADJUST POINTER*/;
         AH    @9,@D5                                             0409
*  A3C44:;
*  /*******************************************************************
*  /**                                                                *
*  /** IF DDNAME IS IN COL 72 INDICATE MSG AND BRANCH.                *
*  /*******************************************************************
*         IF R9 < COUNTER THEN GO TO A3C45 /* IS POINTER AT COL 72*/;
A3C44    C     @9,COMMON+232                                      0411
         BC    04,A3C45                                           0412
*  A3C441:;
*               MS = 21                  /* COL 72 MESSAGE */;
A3C441   LA    @F,21                                              0414
         STH   @F,COMMON1+102                                     0414
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0415
*  /*******************************************************************
*  /**                                                                *
*  /** SET UP FOR SCAN AND BRANCH TO SCAN OUT DDNAME.                 *
*  /*******************************************************************
*  A3C45:;
*               R7 = R9                  /* COPY PTR FOR SCAN */;
A3C45    LR    @7,@9                                              0417
*               GO TO A3C50   /* BRANCH TO SCAN OUT NAME */;
         BC    15,A3C50                                           0418
*  /*******************************************************************
*  /**                                                                *
*  /** INCREMENT POINTER AND TEST TO SEE IF DDNAME EXTENDS INTO COLUMN*
* /* 72. BRANCH TO SET MESSAGE IF SO                           A38710 *
*  /*******************************************************************
*  A3C47:;
*               R7 = R7 + 1              /* ADD 1 TO SCAN PTR*/;
A3C47    AH    @7,@D2                                             0420
*       IF R7 = COUNTER + 1 THEN GO TO A3C441 ; /*BRANCH TO MSGA38710 *
         LA    @F,1                                               0421
         A     @F,COMMON+232                                      0421
         CR    @F,@7                                              0421
         BC    08,A3C441                                          0422
*  /*******************************************************************
*  /*
*  /*    LOOK FOR COMMA OR PAREN FOLLOWING DDNAME
*  /*    FOR OUTPUT ONLY ONE DDNAME IS PERMITTED                 A38766
*  /*
*  /*******************************************************************
*  A3C50:;
*         IF R7 -> INBUF(1) = ',' THEN DO ;  /* CHECK FOR COMMA A38766*
A3C50    CLI   0(@7),C','                                         0424
         BC    07,@9E7                                            0424
*               IF DSDOSW ^= ON THEN GO TO A3C60 ; /*INPUT:CONT A38766*
         CLC   COMMON2+38(1),ON                                   0426
         BC    07,A3C60                                           0427
*                    /* FOR OUTPUT MORE THEN ONE DDNAME SPECIF  A38766*
*                    MS = 3 ;           /* SET MSG INVALID OPER A38766*
         LA    @F,3                                               0428
         STH   @F,COMMON1+102                                     0428
*                    GO TO PRINTERR ;    /* PRINT ERROR MSG     A38766*
         BC    15,PRINTERR                                        0429
*               END ;                                        /* A38766*
*         IF R7 -> INBUF(1) ^= ')' THEN GO TO A3C47 /*BR NOT FOUND*/;
@9E7     CLI   0(@7),C')'                                         0431
         BC    07,A3C47                                           0432
*               PARENSW = ON             /*TURN ON PAREN SW */;
         MVC   COMMON2+41(1),ON                                   0433
* /* END OF DDNAME PARA FOUND AT THIS POINT R9 POINTS AT START OF */
* /* NAME IN INBUF.WE WILL GET LENGTH IN R6,GETMAIN FOR DCB PLACE*/
* /* NAME IN DCB AND AT WORD 66-67 OF DCB */
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION DETERMINES IF A VALID DDNAME WAS FOUND.        *
*  /*******************************************************************
*  A3C60:;
*               R6 = R7 - R9             /* CALCULATE DDNAME LENGTH*/;
A3C60    LCR   @6,@9                                              0435
         AR    @6,@7                                              0435
*         IF R6 = 0 THEN GO TO A3C65 /* BR TO SET MSG */;
         LTR   @6,@6                                              0436
         BC    08,A3C65                                           0437
*     IF R6 > EIGHT THEN DO;            /* NAME TO LONG */
         CH    @6,EIGHT                                           0438
         BC    12,@9E6                                            0438
*  A3C65:;
*               MS = 3                   /*INDICATE MESSAGE 3 */;
A3C65    LA    @F,3                                               0441
         STH   @F,COMMON1+102                                     0441
*               GO TO PRINTMSG /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTMSG                                        0442
*                         END;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION GETS CORE FOR 280 BYTES FOR A DCB.16 BYTES ARE**
*  /** USED FOR SWITCHES AND DDNAME.4 BYTES FOR WORKAREA POINTER      *
*  /*******************************************************************
*         GEN ( GETMAIN EC,LV=280,A=GCADDR,SP=0,MF=(E,GETMLIST));
@9E6     EQU   *                                                  0444
          GETMAIN EC,LV=280,A=GCADDR,SP=0,MF=(E,GETMLIST)
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /**    IF GETMAIN IS SUCCESSFUL BRANCH TO CLEAR GOTTEN CORE,       *
* /* R15 CONTAINS CCODES UPON RETURN FROM GMAIN 0=OK,4=NOTOK  */
*  /** OTHERWISE INDICATE MSG,CONCODE AND BRANCH TO PRINT MESSAGE.    *
*  /*******************************************************************
*         IF R15 = 0 THEN GO TO A3C70 /* BRANCH IF GMAIN SUCESS */;
         LTR   @F,@F                                              0445
         BC    08,A3C70                                           0446
*               MS = 10                  /*NO SUCCESS GMAIN MESSAGE*/;
         LA    @F,10                                              0447
         STH   @F,COMMON1+102                                     0447
*               CONCODE = 12             /*CONDITION CODE FOR USER*/;
         LA    @F,12                                              0448
         STH   @F,COMMON1+2                                       0448
*               GO TO PRINTMSG /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTMSG                                        0449
*  /*******************************************************************
*  /**                                                                *
*  /** CLEAR CORE HERE AND PUT CORE ADDRESS IN DCB POINTER.MOVE COPY  *
*  /** OF DCB TO THIS AREA.                                           *
*  /*******************************************************************
* A3C70:;
*               DCBPTR = GCADDR          /* INIT DCB PTR*/;
A3C70    MVC   COMMON+300(4),COMMON1+72                           0451
*         DCBD(1:256) = MYDCB3A          /* COPY 256 BYTE DCB*/;
         L     @2,COMMON+300       COMMON                         0452
         MVC   0(256,@2),MYDCB3A                                  0452
*        DCBD(257) = '00'X              /* CLEAR CORE*/;
         MVI   256(@2),X'00'                                      0453
*         DCBD(258:280) = DCBD(257:279) /** CLEAR CORE**/;
         MVC   257(23,@2),256(@2)                                 0454
*  /*******************************************************************
*  /**    THIS SECTION INITILIZES TABLES FOR GETMAIN ADDRESSES        *
*  /*******************************************************************
* /*                                                                 */
* /* IF WE ARE PROCESSING OUTPUT DS USE THIS SECTION OTHERWISE GO TO */
* /*                                                           NEXT  */
*         IF DSDOSW = ON THEN DO         /* IS DSD OUTPUT SW ON */;
         CLC   COMMON2+38(1),ON                                   0455
         BC    07,@9E5                                            0455
*               IF FIRSTGMO = 0 THEN DO  /*IS ADDR INITILIZED*/;
         SR    @F,@F                                              0457
         C     @F,COMMON1+76                                      0457
         BC    07,@9E4                                            0457
*                  FIRSTGMO = GCADDR     /*IF NO,INITILIZE IT*/;
         MVC   COMMON1+76(4),COMMON1+72                           0459
*               LASTGMO = GCADDR         /* INIT LAST GM PTR*/;
         MVC   COMMON1+84(4),COMMON1+72                           0460
*         CURRGMO = GCADDR       /* INIT CURRENT GM ADDR*/;
         MVC   COMMON1+80(4),COMMON1+72                           0461
*               GO TO A3C75    /*BR TO OPEN OUTPUTDCB FIRST TIME*/;
         BC    15,A3C75                                           0462
*                  END;
*               DCBPTR = LASTGMO         /*INIT DCBPTR TO PREV DCBAD*/;
@9E4     MVC   COMMON+300(4),COMMON1+84                           0464
*               NEXTDCB = GCADDR         /*INIT NEXTDCB PTR TO NEXT*/;
         L     @2,COMMON+300       COMMON                         0465
         MVC   256(4,@2),COMMON1+72                               0465
*         CURRGMO = GCADDR       /* INIT CURRENT GM ADDR*/;
         MVC   COMMON1+80(4),COMMON1+72                           0466
*               DCBPTR = GCADDR          /*INIT DCBPTR TO CURRENT*/;
         MVC   COMMON+300(4),COMMON1+72                           0467
*               LASTGMO = GCADDR         /*INIT LASTGM PTR*/;
         MVC   COMMON1+84(4),COMMON1+72                           0468
*               GO TO A3C75              /* BR TO OPEN DS */;
         BC    15,A3C75                                           0469
*         END;
* /* THIS POINT IS ENTERED IF INPUT DDNAME IS FOUND LENGTH OF NAME*/
* /* IS IN R6  R9 POINTS AT START                                    */
* /*                                                                 */
*         IF FIRSTGMI = 0 THEN DO        /*Q-FIRST TIME ON INPUT */;
@9E5     SR    @F,@F                                              0471
         C     @F,COMMON1+88                                      0471
         BC    07,@9E3                                            0471
*               FIRSTGMI = GCADDR        /* INIT FIRST GM PTR*/;
         MVC   COMMON1+88(4),COMMON1+72                           0473
*               LASTGMI = GCADDR         /* INIT LAST GM PTR*/;
         MVC   COMMON1+96(4),COMMON1+72                           0474
*               CURRGMI = GCADDR         /* INIT CURRENT GM PTR*/;
         MVC   COMMON1+92(4),COMMON1+72                           0475
*               GO TO A3C73              /*BR THIS IS FOR FIRST TIME*/;
         BC    15,A3C73                                           0476
*         END;
* /* COME HERE WHEN FIRST GM PTR HAS BEEN INITILIZED */
*               DCBPTR = LASTGMI         /* INIT DCBPTR TO PREV DCBAD*/
@9E3     MVC   COMMON+300(4),COMMON1+96                           0478
*               NEXTDCB = GCADDR         /* INIT NEXTDCB PTR TO NEXT*/;
         L     @2,COMMON+300       COMMON                         0479
         MVC   256(4,@2),COMMON1+72                               0479
*               CURRGMI = GCADDR         /* INIT CURRENT GM PTR*/;
         MVC   COMMON1+92(4),COMMON1+72                           0480
*               DCBPTR = GCADDR          /* INIT DCBPTR TO CURRENT*/;
         MVC   COMMON+300(4),COMMON1+72                           0481
*               LASTGMI = GCADDR         /* INIT LAST GM PTR*/;
         MVC   COMMON1+96(4),COMMON1+72                           0482
*  /*******************************************************************
*  /**                                                                *
*  /**    AT THIS POINT WE HAVE GOT SPACE FOR A DCB,CLEARED THIS SPACE*
*  /** AND INIT NEXTDCBPTR IN PREVIOUS DCB.WE WILL NOW PLACE THE      *
*  /** DDNAME INTO THE DCB AND AT END OF DCB FOR THE CREATE MODULE.   *
*  /**                                                                *
*  /*******************************************************************
*  A3C73:;
*  A3C75:;
A3C73    EQU   *                                                  0484
*         DDNAME1 = INBUF(1:R6)          /*MOVE DDNAME TO TABLE*/;
A3C75    LR    @2,@6                                              0485
         BCTR  @2,0                                               0485
         LR    @E,@9                                              0485
         L     @4,COMMON+300       COMMON                         0485
         LA    @A,260(0,@4)                                       0485
         MVI   0(@A),C' '                                         0485
         MVC   1(007,@A),0(@A)                                    0485
         EX    @2,@MVC                                            0485
*  A3C76:;
*         DDNAME = INBUF(1:R6)           /*MOVE DDNAME TO DCB */;
A3C76    LR    @2,@6                                              0487
         BCTR  @2,0                                               0487
         LR    @E,@9                                              0487
         L     @4,COMMON+300       COMMON                         0487
         LA    @A,40(0,@4)                                        0487
         MVI   0(@A),C' '                                         0487
         MVC   1(007,@A),0(@A)                                    0487
         EX    @2,@MVC                                            0487
* /********************************************************************
* /*                                                                  *
* /* TO TAKE CARE THAT THE RIGHT ACCESS METHOD IS USED THERE    A37883*
* /* IS DONE A GETMAIN TO GET STORAGE NECESSARY TO READ A JFCB  A37883*
* /* SO THAT BEFORE OPEN THE DSORG CAN BE CHECKED               A37883*
* /* TO SUPPORT PO MEMBERS THEIR DSORG WILL BE SET TO PS        A37883*
* /*                                                                  *
* /********************************************************************
*         DSORG = '40'X ;               /* SET DEFAULT DSORG=PS A37883*
         MVI   26(@4),X'40'                                       0488
*        OPTBYTE1 = DCBPTR ;            /* GIVE ADDRESS OF DCB  A50927*
         MVC   COMMON+236(4),COMMON+300                           0489
*         GENERATE ;                    /* TO GET CORE FOR JFCB A37883*
*                                                                     *
*   DO GETMAIN FOR JFCB                                        A37883 *
*                                                                     *
         LA    R0,GETAREA              LOAD 176 FOR JFCB       A37883 *
         GETMAIN R,LV=(0)              GETMAIN FOR 176 BYTES   A37883
         DS    0H
* /********************************************************************
* /*                                                            A37883*
* /* IF GETMAIN SUCCESFUL CLEAR GOTTEN CORE, RETURN CODE=0,     A37883*
* /* OTHERWISE GIVE MESS,COND COD & PRINT MESS                  A37883*
* /*                                                            A37883*
* /********************************************************************
*         IF R15 ^= 0                   /* GETMAIN NOT SUCCESFULA37883*
*            THEN DO ;                  /* THEN                 A37883*
         LTR   @F,@F                                              0491
         BC    08,@9E2                                            0491
*                   MS = 10 ;           /* GIVE MESS            A37883*
         LA    @F,10                                              0493
         STH   @F,COMMON1+102                                     0493
*                   CONCODE = 12 ;      /* COND CODE FOR USER   A37883*
         LA    @F,12                                              0494
         STH   @F,COMMON1+2                                       0494
*                   GO TO PRINTMSG ;    /* PRINT MESSAGE        A37883*
         BC    15,PRINTMSG                                        0495
*         END ;
* /*                                                                  *
* /*    BUILD SKELETON DCB FOR RDJFCB                           A37883*
* /*                                                                  *
*        JFCBPTR = R1 ;                 /* SAVE JFCB ADDRESS    A37883*
@9E2     ST    @1,JFCBPTR                                         0497
*        EXLIST = ADDR(EXITLIST) ;      /* PUT EXIT ADDR IN DCB A37883*
         LA    @F,EXITLIST                                        0498
         L     @2,COMMON+300       COMMON                         0498
         ST    @F,@TEMP4                                          0498
         MVC   37(3,@2),@TEMP4+1                                  0498
*        PTRJFCB = ADDR(JFCB) ;         /* POINT DCB EXIT TO JFCB37883*
         L     @4,JFCBPTR                                         0499
         ST    @4,@TEMP4                                          0499
         MVC   EXITLIST+1(3),@TEMP4+1                             0499
*        HEX87 = '87'X ;                /* FILL IN INDIC JFCB   A37883*
         MVI   EXITLIST,X'87'                                     0500
*              JFCB(1) = '00'X ;        /* CLEAR CORE           A37883*
         MVI   0(@4),X'00'                                        0501
*              JFCB(2:176) = JFCB(1:175) ; /* FOR USING JFCB    A38883*
         MVC   1(175,@4),0(@4)                                    0502
*         OPTBYTE1(1) = '80'X ;         /* INDICATE INPUT       A50927*
         MVI   COMMON+236,X'80'                                   0503
*         R1 = ADDR(OPENLIST) ;         /*GIVE PLACE IN SUBPOOL A50927*
         LA    @1,COMMON+236                                      0504
*        GEN( RDJFCB MF=(E,(1)))  ;     /*  READ JFCB           A50927*
          RDJFCB MF=(E,(1))
         DS    0H
* /* TEST DSORG; IF IT IS INDEX SEQ. SET IT SO; ELSE DSORG=P.S  A37883*
*        IF DSORGJ(1) = '1'B            /* IS DSORG INDEX SEQ   A37883*
*            THEN DO ;                  /* YES, THEN            A37883*
         TM    98(@4),B'10000000'                                 0506
         BC    12,@9E1                                            0506
*                   DSORG = '80'X ;     /* RESET DSORG TO I.S.  A37883*
         MVI   26(@2),X'80'                                       0508
*        END ;                          /* END OF CHECKING DSORGA37883*
* /* RESET JFCB-EXIT AND GIVE STORAGE BACK;ALSO RELEASE REG 3.  A37883*
*        HEX87 = '07'X ;                /* RESET JFCBEXIT NOT L A37883*
@9E1     MVI   EXITLIST,X'07'                                     0510
*        R1 = JFCBPTR ;                 /* POINT R1 TO JFCBAREA A37883*
         L     @1,JFCBPTR                                         0511
*        GENERATE ;                     /* FREEMAIN MACRO       A37883*
         LA    R0,GETAREA              SIZE OF JFCBAREA         A37883
         FREEMAIN R,LV=(0),A=(1)       FREE UP JFCBAREA         A37883
         DS    0H
* /*                                                                 */
* /* WE ARE NOW READY TO OPEN THE DCB THE OPEN LIST MUST BE INIT.*/
* /* SEE CONTROL PROGRAM SERVICES FORM C28-6541-1 PAGE 123 FOR  */
* /* DESCRIPTION OF PARAMETER LIST */
* /*                                                                 */
*               OPTBYTE1 = DCBPTR        /* DCB ADDRESS TO LIST*/;
         MVC   COMMON+236(4),COMMON+300                           0513
*         EXLIST  = ADDR(EXLST3) /* INITLZ DCB EXLIST ADDR */;
         LA    @F,COMMON+276                                      0514
         L     @2,COMMON+300       COMMON                         0514
         ST    @F,@TEMP4                                          0514
         MVC   37(3,@2),@TEMP4+1                                  0514
*         EDCB5  = '85'X  /* INITLZ EXIT LIST IND FIELD */;
         MVI   COMMON+276,X'85'                                   0515
*         EDCB6  = ADDR(DCBROUT3) /*INITLZ EXIT LIST ADDR FIELD */;
         LA    @F,DCBROUT3                                        0516
         ST    @F,@TEMP4                                          0516
         MVC   COMMON+277(3),@TEMP4+1                             0516
*         IF DSDOSW = ON THEN DO         /*IF OUTPUT SET BITS */;
         CLC   COMMON2+38(1),ON                                   0517
         BC    07,@9E0                                            0517
*               OPTBYTE1(1) = '8F'X /* INDICATE LAST,DISP,OUTPUT */;
         MVI   COMMON+236,X'8F'                                   0519
*               GO TO A3C77              /* BR TO OPEN DCB */;
         BC    15,A3C77                                           0520
*         END;
*         OPTBYTE1(1) = '80'X  /* INDICATE OPEN FOR INPUT */;
@9E0     MVI   COMMON+236,X'80'                                   0522
*               MACRF = '5000'X          /*SET MACRF TO GET MOVE */;
         L     @2,COMMON+300       COMMON                         0523
         MVC   50(2,@2),@X27                                      0523
* /* READY TO OPEN DCB AS INDICATED IN LIST */
*  A3C77:;
*               R1 = ADDR(OPENLIST)      /*OPEN LIST ADDR FOR OPEN*/;
A3C77    LA    @1,COMMON+236                                      0525
*         GEN   (OPEN MF=(E,(1)));
         OPEN MF=(E,(1))
         DS    0H
*         IF OFLGS(4) ^='1' B THEN GO TO A3C79 /* BR DCB NOT OPEN*/;
         L     @2,COMMON+300       COMMON                         0527
         TM    48(@2),B'00010000'                                 0527
         BC    12,A3C79                                           0528
*         IF FLUSHSW = ON THEN DO /* FALL THROUGH-FLUSH REQUIRED */;
         CLC   COMMON2+36(1),ON                                   0529
         BC    07,@9DF                                            0529
*                                  /* IF FUSHING IS REQD ISSUE MESSAGE*
*               MS = 28            /* MESS FOR INCORRECT DCB PARA */;
         LA    @F,28                                              0531
         STH   @F,COMMON1+102                                     0531
*               GO TO PRINTMSG /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTMSG                                        0532
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION GETS CORE FOR THE LRECL WORKAREA.ITS ADDRESS   *
*  /** IS STORED IN LOCATION OUTREC FOR USE BY THE CREATE MODULE.     *
*  /** BLOCKSIZE LENGTH IS GOTTEN IF RECORD FORMAT IS 'U'.            *
*  /**                                                                *
*  /*******************************************************************
*               R6 = LRECL      /* COPY RCD LGTH FOR GETMAIN */;
@9DF     L     @2,COMMON+300       COMMON                         0534
         LH    @6,82(0,@2)                                        0534
*         IF RECFM = 'C0'X THEN DO /**BRANCH IF NOT 'U' RCD **/;
         CLI   36(@2),X'C0'                                       0535
         BC    07,@9DE                                            0535
*               R6 = BLKSI     /** PICK UP BLOCKSIZE**/;
         LH    @6,62(0,@2)                                        0537
*         END;
*         IF DSORG(1) = '1'B THEN DO ;   /* INDEX SEQUENTIAL    A45169*
@9DE     L     @2,COMMON+300       COMMON                         0539
         TM    26(@2),B'10000000'                                 0539
         BC    12,@9DD                                            0539
*            R6 = R6 + KEYLEN ;          /* ADD KEYLEN TO LRECL A45169*
         SR    @0,@0                                              0541
         IC    @0,16(0,@2)                                        0541
         AR    @6,@0                                              0541
*         END ;                          /* FOR GETTING CORR BUFA45169*
*               R8 = ADDR(GCADDR) /* GET GC ADDRESS */;
@9DD     LA    @8,COMMON1+72                                      0543
*  GEN ( GETMAIN EC,LV=(R6),A=(R8),SP=0,MF=(E,GETMLIST));
          GETMAIN EC,LV=(R6),A=(R8),SP=0,MF=(E,GETMLIST)
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /** IF GETMAIN IS SUCCESSFUL INITILIZE INREC OR OUTREC AND BRANCH  *
*  /**                                                                *
*  /*******************************************************************
*         IF R15 = 0 THEN DO   /* IS GETMAIN SUCESSFUL */;
         LTR   @F,@F                                              0545
         BC    07,@9DC                                            0545
*               GMLGTH = R6    /** SAVE LENGTH FOR CUP**/;
         L     @2,COMMON+300       COMMON                         0547
         STH   @6,276(0,@2)                                       0547
*         IF DSDOSW = ON THEN DO    /**IF OUTPUT DO BELOW**/;
         CLC   COMMON2+38(1),ON                                   0548
         BC    07,@9DB                                            0548
*               OUTREC = GCADDR     /**INIT ADDR FOR OUTPUT WORKAREA**/
         MVC   COMMON1+4(4),COMMON1+72                            0550
*               GO TO A3C80         /** BRANCH FOR PROCESSING**/;
         BC    15,A3C80                                           0551
*         END;
*               INREC = GCADDR      /**INIT ADDR FOR INPUT WORKAREA**/;
@9DB     L     @2,COMMON+300       COMMON                         0553
         MVC   272(4,@2),COMMON1+72                               0553
*               GO TO A3C80         /**BRANCH FOR PROCESSING**/;
         BC    15,A3C80                                           0554
*         END ;
*  /*******************************************************************
*  /**                                                                *
*  /** SET MESSAGE,INDICATE FLUSH TO END CARD AND BRANCH TO PRINT     *
*  /*******************************************************************
*               MS = 10        /* INDICATE MESSAGE*/;
@9DC     LA    @F,10                                              0556
         STH   @F,COMMON1+102                                     0556
*               FLUSHSW = ON  /* TURN ON FLUSH AND ISSUE MESSAGE*/;
         MVC   COMMON2+36(1),ON                                   0557
*               GO TO PRINTMSG /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTMSG                                        0558
*  /*******************************************************************
*  /**                                                                *
*  /**    THIS SECTION INDICATES MSG,CONCODE FOR AN UNSUCCESSFUL      *
*  /** OPEN AND BRANCHES TO PRINT MESSAGE.                            *
*  /**                                                                *
*  /*******************************************************************
*  A3C79:;
*               CONCODE = 12             /*CODE FOR USER */;
A3C79    LA    @F,12                                              0560
         STH   @F,COMMON1+2                                       0560
*               MS = 24  /* SET MSG CANNOT OPEN */;
         LA    @F,24                                              0561
         STH   @F,COMMON1+102                                     0561
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0562
*  /*******************************************************************
*  /**                                                                *
*  /**    CONTINUE HERE AFTER AN OPEN SUCCESS.                        *
*  /** THERE MAY BE MORE DCBS TO OPEN.R9 POINTS AT START OF DDNAME AS *
*  /** OF LAST OPEN.R6 = LENGTH OF NAME.R7 = COMMA OR PAREN AFTER NAME*
*  /** A BLANK AFTER PAREN INDICATES END OF CARD.COMMA INDICATES      *
*  /** MORE DDNAMES ARE PRESENT.THE POINTER WILL BE ADVANCED AND IF   *
*  /** COMMA IS PRESENT A BRANCH IS TAKEN TO SCAN OUT ANOTHER NAME.   *
*  /** IF BLANK A BRANCH IS TAKEN TO GET ANOTHER CARD.                *
*  /**                                                                *
*  /*******************************************************************
*  A3C80:;
*  A3C81:;
A3C80    EQU   *                                                  0564
*               R9 = R7 + 1              /*ADVANCE POINTER TO OPERAND*/
A3C81    LA    @9,1                                               0565
         AR    @9,@7                                              0565
*         IF R7 -> INBUF(1) = ',' THEN DO /**IF COMMA PRESENT WE ARE
         CLI   0(@7),C','                                         0566
         BC    07,@9DA                                            0566
*                                            SCANNING INSIDE PARENS**/;
*         IF INBUF(1) = ' ' THEN GO TO A3C83 /**  IF A BLANK FOLLOWS
         CLI   0(@9),C' '                                         0568
*                                                 THE COMMA WE HAVE
*                                                 INPUT DDNAMES ON THE
*                                                 NEXT CARD**/;
         BC    08,A3C83                                           0569
*         IF R7 = COUNTER THEN DO /** IF COMMA IN COL 71 DO BELOW**/;
         C     @7,COMMON+232                                      0570
         BC    07,@9D9                                            0570
*  A3C83:;
*               DSDCSW = ON    /** TURN ON DSD CONT SW**/;
A3C83    MVC   COMMON2+17(1),ON                                   0573
*             DSDISW = ON      /** GT 1 NAME WITHIN PARENS DSDISW ON**/
         MVC   COMMON2+39(1),ON                                   0574
*               GO TO A2B3     /** BRANCH TO GET ANOTHER CARD**/;
         BC    15,A2B3                                            0575
*         END;
*               GO TO A3C45    /** BRANCH NOT AT COL 71 YET**/;
*         END;
*         IF INBUF(1) = ' ' THEN DO      /*TO CHECK FOR COMMENTS*/;
@9DA     CLI   0(@9),C' '                                         0579
         BC    07,@9D8                                            0579
*               IF COUNTER -> INBUF(2) ^= ' ' THEN DO;
         L     @2,COMMON+232       COMMON                         0581
         CLI   1(@2),C' '                                         0581
         BC    08,@9D7                                            0581
*               COMCSW = ON;             /*SET COMMENTS SW*/
         MVC   COMMON2+29(1),ON                                   0583
*         END;
*               IF FIRSTGMO = 0 THEN DO;  /* NO OUTPUT SPECIFIEDA48784*
@9D7     SR    @F,@F                                              0585
         C     @F,COMMON1+76                                      0585
         BC    07,@9D6                                            0585
*                    MS = 24 ;            /* GIVE MESSAGE       A48784*
         LA    @F,24                                              0587
         STH   @F,COMMON1+102                                     0587
*                    GO TO PRINTERR ;     /*PRINT ERROR MSG     A48784*
         BC    15,PRINTERR                                        0588
*               END;                      /* END OF OUTPUT CHECKA48784*
*         GO TO A2B3                     /* NO COMMENTS-BRANCH*/;
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /** A COMMA AT THIS POINT INDICATES ANOTHER KEYWORD PRESENT,       *
*  /** OTHERWISE A MSG IS INDICATED.                                  *
*  /*******************************************************************
*         IF INBUF(1) = ',' THEN DO      /* IF COMMA DO BELOW*/;
@9D8     CLI   0(@9),C','                                         0592
         BC    07,@9D5                                            0592
*               R9 = R9 + 1              /*STEP PTR PAST COMMA*/;
         AH    @9,@D2                                             0594
*         IF INBUF(1) = ' ' THEN DO /**A BLANK INDICATES CONT **/;
         CLI   0(@9),C' '                                         0595
         BC    07,@9D4                                            0595
*               DSDCSW = ON    /**TURN ON DSD CONT SW**/;
         MVC   COMMON2+17(1),ON                                   0597
*               GO TO A2B3     /**BRANCH TO GET ANOTHER CARD**/;
         BC    15,A2B3                                            0598
*         END;
*               GO TO A3C41              /*BR TO SCAN NEXT KEYWORD*/;
*         END;
*               MS = 3                   /*SET MESSAGE INDICATION*/;
@9D5     LA    @F,3                                               0602
         STH   @F,COMMON1+102                                     0602
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0603
*  /*******************************************************************
*  /**                                                                *
*  /**    ***** REPEAT CARD HERE *****                                *
*  /**                                                                *
*  /** ADVANCE POINTER PAST KEYWORD,MSG IF NO FOLLOWING BLANK         *
*  /**                                                                *
*  /*******************************************************************
*  A3C14:;
*                                       /* POINT TO BLANK FOLLOWING*/
*                                        /* KEYWORD REPEAT */
*               R9 = R9 + SIX;
A3C14    AH    @9,SIX                                             0605
*         IF INBUF(1) = ' ' THEN GO TO A3C141;
         CLI   0(@9),C' '                                         0606
         BC    08,A3C141                                          0607
*               MS  =  20                /* MESSAGE  20 */;
         LA    @F,20                                              0608
         STH   @F,COMMON1+102                                     0608
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0609
*  /*******************************************************************
*  /**                                                                *
*  /** SCAN LOOKING FOR FIRST KEYWORD,IF NO KEY BY COL 71 CAUSE A     *
*  /** MESSAGE TO BE ISSUED.                                          *
*  /**                                                                *
*  /*******************************************************************
*  A3C141:;
*         IF R9 > COUNTER THEN DO /**ARE WE AT COL 72 **/;
A3C141   C     @9,COMMON+232                                      0611
         BC    12,@9D3                                            0611
*  MS21:;
*               MS = 21        /** INDICATE MESSAGE 21 **/;
MS21     LA    @F,21                                              0614
         STH   @F,COMMON1+102                                     0614
*               GO TO PRINTMSG /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTMSG                                        0615
*         END;
*               R9 = R9 + ONE ; /* ADVANCE POINTER */
@9D3     AH    @9,ONE                                             0617
*         IF INBUF(1) = ' ' THEN GO TO A3C141 /*BR IF COLUMN BLANK*/;
         CLI   0(@9),C' '                                         0618
         BC    08,A3C141                                          0619
*  A3C142:;
*  /*******************************************************************
*  /**                                                                *
*  /** THIS SECTION TESTS FOR MORE THAN ONE REPEAT CARD PER DSD GROUP *
*  /*******************************************************************
*         IF REPEATSW = ON THEN DO /* CHECK FOR DUPLICATE REPEAT CARD*/
A3C142   CLC   COMMON2+42(1),ON                                   0621
         BC    07,@9D2                                            0621
*               MS = 14                  /* SET MESSAGE 14 */;
         LA    @F,14                                              0623
         STH   @F,COMMON1+102                                     0623
*               FLUSHSW = ON             /* FLUSH ON */;
         MVC   COMMON2+36(1),ON                                   0624
*               GO TO PRINTMSG /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTMSG                                        0625
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /** THIS SECTION TURNS ON THE REPEAT SWITCH AND PROCESSES FOR THE  *
*  /** KEYWORD 'QUANTITY'.IF FOUND A BRANCH IS TAKEN TO CONVERT THE   *
*  /** DECIMAL NUMBER TO BINARY.                                      *
*  /*******************************************************************
*  A3C143:;
@9D2     EQU   *                                                  0627
*               REPEATSW = ON    /* INDICATE A REPEAT CARD*/;
A3C143   MVC   COMMON2+42(1),ON                                   0628
*        IF INBUF(1:9) = 'QUANTITY=' THEN DO ;
         CLC   0(9,@9),@C29                                       0629
         BC    07,@9D1                                            0629
*               R9 = R9 + NINE ;
         AH    @9,NINE                                            0631
*     IF R9 > COUNTER THEN GO TO MS21 /**PRINT MS21 IF AT COL 72**/;
         C     @9,COMMON+232                                      0632
         BC    02,MS21                                            0633
*               R7 = ADDR(REPEATNO);
         LA    @7,COMMON+16                                       0634
*               QUANSW = ON /* SET SWITCH INDICATING QUANTITY */;
         MVC   COMMON2+40(1),ON                                   0635
*               GO TO CONVERTB ;
         BC    15,CONVERTB                                        0636
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /** THIS SECTION PROCESSES THE CREATE KEYWORD OF THE REPEAT CARD   *
*  /** IF FOUND A BRANCH IS TAKEN TO CONVERT THE DECIMAL NUMBER.      *
*  /*******************************************************************
*        IF INBUF(1:7) = 'CREATE=' THEN DO ;
@9D1     CLC   0(7,@9),@C30                                       0638
         BC    07,@9D0                                            0638
*               R9 = R9 + SEVEN;
         AH    @9,SEVEN                                           0640
*     IF R9 > COUNTER THEN GO TO MS21 /**PRINT MS21 IF AT COL 72**/;
         C     @9,COMMON+232                                      0641
         BC    02,MS21                                            0642
*              R7 = ADDR(CREATENO);
         LA    @7,COMMON+18                                       0643
*               GO TO CONVERTB;
         BC    15,CONVERTB                                        0644
*         END;
*  /*******************************************************************
*  /**                                                                *
*  /** THIS SECTION INDICATES A MESSAGE WHEN THE KEY IS UNRECOGNIZABLE*
*  /*******************************************************************
*               MS  =  5                 /* MESSAGE  5 */;
@9D0     LA    @F,5                                               0646
         STH   @F,COMMON1+102                                     0646
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0647
*   GEN;
***********************************************************************
* AT ENTRY TO THIS CONVERT DECIMAL TO BINARY SUBROUTINE R9 POINTS AT  *
* INBUFA JUST AFTER QUANTITY= OR CREATE=.  R7 CONTAINS ADDRESS OF     *
* REPEATNO OR CREATENO. UPON COMPLETION BINARY NUMBER >HOULD BE      *
* PRESENT IN CORRECT FIELD.R9 WILL BE INIT TO BLANK OR JUST PAST COMMA*
* FOLLOWING FIELD. BLANK INDICATES END OF CARD . CONTROL GOES TO READ *
* ANOTHER. COMMA INDICATES ANOTHER FIELD ,CONTROL TO A3C142 TO LOOK  *
* FOR CORRECT FIELD                                                   *
***********************************************************************
         DS    0H
*                                        /* COPY START ADDR OF FIELD*/
*  CONVERTB:;
*                                        /* CLEAR WORK FIELD */
*               R8 = R9 ;
CONVERTB LR    @8,@9                                              0650
*  CONV1:;
*                                        /* SEARCH FOR END OF FIELD */
*        IF R8 -> INBUF(1) = ' ' THEN GO TO CONV2;
CONV1    CLI   0(@8),C' '                                         0652
         BC    08,CONV2                                           0653
*                                        /* SEARCH FOR END OF FIELD */
*        IF R8 -> INBUF(1) = ',' THEN GO TO CONV2;
         CLI   0(@8),C','                                         0654
         BC    08,CONV2                                           0655
*               R8 = R8 + ONE; /* INC POINTER BY ONE */
         AH    @8,ONE                                             0656
*       IF R8 ^> COUNTER THEN GO TO CONV1 ; /* IF NOT PAST 72  A38710 *
         C     @8,COMMON+232                                      0657
         BC    12,CONV1                                           0658
*               MS = 21        /** INDICATE MESSAGE 21 **/;
         LA    @F,21                                              0659
         STH   @F,COMMON1+102                                     0659
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0660
*  CONV2:;
*               R8 = R8-R9 ; /* OBTAIN FIELD LENGTH */
CONV2    SR    @8,@9                                              0662
*         IF R8 <= 0 THEN GO TO CONV4;
         LTR   @8,@8                                              0663
         BC    12,CONV4                                           0664
*               R8 = R8 -ONE; /* TO CORRECT NUMBER */
         SH    @8,ONE                                             0665
*                                        /* PACK FIELD */
*        GEN (  EX R8,CONV5);
           EX R8,CONV5
         DS    0H
*                                        /* CONVERT TO BINARY*/
*         GEN ( CVB R6,Q);
          CVB R6,Q
         DS    0H
*         IF R6 = 0 THEN DO    /** DO BELOW IF ZERO**/;
         LTR   @6,@6                                              0668
         BC    07,@9CF                                            0668
*               MS = 15        /** SET MESSAGE 15 **/;
         LA    @F,15                                              0670
         STH   @F,COMMON1+102                                     0670
*               GO TO PRINTERR /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTERR                                        0671
*         END;
*         IF R6 > 65535 THEN GO TO CONV4;
@9CF     C     @6,@D6                                             0673
         BC    02,CONV4                                           0674
*         GEN ( STH R6,0(R7)); /* STORE NUMBER */
          STH R6,0(R7)
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /** AT THIS POINT WE ARE THROUGH CONVERTING THE DECIMAL NUMBER.    *
*  /** SCAN CONTINUES LOOKING FOR END OF CARD OR INDICATION OF        *
*  /** ANOTHER KEYWORD.                                               *
*  /**                                                                *
*  /*******************************************************************
*  CONV3:;
*                                        /* POINTER TO COMMA OR */
*                                        /* BLANK FOLLOWING NUMBER*/
*               R9 = R9 + R8 + ONE;
CONV3    LH    @F,ONE                                             0677
         AR    @F,@8                                              0677
         AR    @9,@F                                              0677
*                                        /* IF AT END OF FIELD GO */
*                                        /* TO READ ANOTHER CARD */
*         IF INBUF(1) = ' ' THEN GO TO CONV6 /* BR IF COLUMN BLANK */;
         CLI   0(@9),C' '                                         0678
         BC    08,CONV6                                           0679
*  /** R9 IS NOW POINTING AT COMMA**/
*         IF R9 = COUNTER THEN GO TO CONV34 /** IS COMMA IN COL 71**/;
         C     @9,COMMON+232                                      0680
         BC    08,CONV34                                          0681
*         IF INBUF(2) = ' ' THEN DO /** IF BLANK DO BELOW**/;
         CLI   1(@9),C' '                                         0682
         BC    07,@9CE                                            0682
*  CONV34:;
*               DSDCSW = ON /** TURN ON CONT SW**/;
CONV34   MVC   COMMON2+17(1),ON                                   0685
*               GO TO A2B3     /** BR TO GET ANOTHER CARD**/;
         BC    15,A2B3                                            0686
*         END;
*               R9 = R9 + ONE; /* ADVANCE PTR PAST COMMA */
@9CE     AH    @9,ONE                                             0688
*               GO TO A3C143    /* BR TO LOOK FOR ANOTHER KEYWORD*/;
         BC    15,A3C143                                          0689
*  CONV4:;
*               MS = 14;       /** SET MSG TO 14 **/
CONV4    LA    @F,14                                              0691
         STH   @F,COMMON1+102                                     0691
*               GO TO PRINTERR /** BRANCH TO PRINT MSG AND FLAG **/;
         BC    15,PRINTERR                                        0692
*   /* **** PACK FIELD *** THIS INST EXECUTED FROM ABOVE**********/
*  CONV5:;
*         GEN ( PACK Q(8),0(1,R9));
CONV5     PACK Q(8),0(1,R9)
         DS    0H
*  CONV6:;
*                                   /* BRANCH IF COLUMN BLANK */
*         IF COUNTER -> INBUF(2) = ' ' THEN GO TO CONV7;
CONV6    L     @2,COMMON+232       COMMON                         0696
         CLI   1(@2),C' '                                         0696
         BC    08,CONV7                                           0697
*               COMCSW = ON /* TURN ON COMMENTS CONTINUE SWITCH*/;
         MVC   COMMON2+29(1),ON                                   0698
*  CONV7:;
*         IF QUANSW = ON THEN GO TO A2B3 /*BR IF QUAN WAS FOUND */;
CONV7    CLC   COMMON2+40(1),ON                                   0700
         BC    08,A2B3                                            0701
*               MS = 15 /* INDICATE MESSAGE QUANTITY MISSING */;
         LA    @F,15                                              0702
         STH   @F,COMMON1+102                                     0702
*               GO TO PRINTMSG /** BRANCH TO PRINT MESSAGE**/;
         BC    15,PRINTMSG                                        0703
*  /*******************************************************************
*  /** THIS IS THE   DCB EXIT ROUTINE                                **
*  /** IT WILL TEST FOR BLOCKSIZE AND LRECL BEING PRESENT.LRECL FOR  **
*  /** F RECORDS MUST BE AN EVEN MULTIPLE OF BLOCKSIZE.              **
*  /** FOR SYSIN THE DEFAULT IS:                                     **
*  /**                          RECFM = FB                           **
*  /**                          LRECL = 80                           **
*  /**                          BLKSIZE = 80                         **
*  /** FOR SYSPRINT THE DEFAULT IS:                                  **
*  /**                          RECFM = FBA                          **
*  /**                          LRECL = 121                          **
*  /**                          BLKSIZE = 121                        **
*  /** FOR INPUT OR OUTPUT THE DEFAULT IS:                           **
*  /**                          RECFM = U                            **
*  /**                          LRECL = 1                            **
*  /**                          BLKSIZE = 1                          **
*  /**                                                               **
*  /** SWITCHES ARE SET UPON ENTRY TO INDICATE TYPE OF DATA SET      **
*  /**          SYSISW = SYSIN                                       **
*  /**          SYSPSW = SYSPRINT                                    **
*  /**          OLDNEWSW = INPUT(OLD) OR OUTPUT(NEW)                 **
*  /**                                                               **
*  /** AT ENTRY FROM DCB SEPERATE ENTRY POINTS ARE ENTERED FOR EACH  **
*  /** TYPE OF DATA SET SO THAT DEFAULT VALUS MAY BE ASSIGNED.       **
*  /** IF BLKSIZE OR LRECL IS MISSING UPON ENTRY FROM INPUT OR OUTPUT**
*  /** DATA SET A SWITCH IS SET SO THAT CONTROL CARDS MAY BE FLUSHED **
*  /** TO THE NEXT END OR SLASH ASTERICK CARD.A VALUE OF ONE IS      **
*  /** ASSIGNED SO THAT THE DATA SET MAY BE OPENED WITHOUT ABENDING. **
*  /**                                                               **
*  /*******************************************************************
*  DCBROUT1:;
*  /*******************************************************************
*  /**                                                                *
*  /** ENTRY FOR SYSIN DATA SET TO ASSIGN DEFAULTS                    *
*  /*******************************************************************
*         SAVE14 = R14  /* SAVE OPEN RETURN REG */;
DCBROUT1 ST    @E,COMMON1+56                                      0705
*         DCBPTR = ADDR(SYSI);
         LA    @F,COMMON+116                                      0706
         ST    @F,COMMON+300                                      0706
*               DLRECL = 80 /* DEFAULT LRECL 80 */;
         LA    @F,80                                              0707
         STH   @F,COMMON+280                                      0707
*               DBLKSI = 80 /* DEFAULT BLKSIZE 80 */;
         STH   @F,COMMON+282                                      0708
*               DRECFM = '90'X /*DEFAULT RECFM IS FB */;
         MVI   COMMON+284,X'90'                                   0709
*               SYSISW = ON /*TURN ON SYSIN SW */;
         MVC   COMMON2+33(1),ON                                   0710
*               GO TO DCBEXIT /* BRANCH TO CHECK PARAMETERS */;
         BC    15,DCBEXIT                                         0711
*                                        /* ENTRY FROM SYSPRINT DCB */
*  /*******************************************************************
*  /**                                                                *
*  /** ENTRY FOR SYSPRINT DATA SET TO ASSIGN DEFAULTS                 *
*  /**                                                                *
*  /*******************************************************************
*  DCBROUT2:;
*         SAVE14 = R14  /* SAVE OPEN RETURN REG */;
DCBROUT2 ST    @E,COMMON1+56                                      0713
*               DCBPTR = ADDR(SYSP) /* SYSP ADDR TO DCBPTR */;
         LA    @F,COMMON+20                                       0714
         ST    @F,COMMON+300                                      0714
*               DLRECL = 121 /* DEFAULT LRECL 121*/;
         LA    @F,121                                             0715
         STH   @F,COMMON+280                                      0715
*               DBLKSI = 121 /*DEFAULT BLKSI 121*/;
         STH   @F,COMMON+282                                      0716
*               DRECFM = '94'X           /* DEFAULT RECFM IS FB ASA */;
         MVI   COMMON+284,X'94'                                   0717
*               SYSPSW = ON /* TURN ON SYSPRINT SW */;
         MVC   COMMON2+34(1),ON                                   0718
*               GO TO DCBEXIT /* BRANCH TO CHECK PARAMETERS */;
         BC    15,DCBEXIT                                         0719
*                                        /*ENTRY FROM OLDNEW DS */
*  /*******************************************************************
*  /**                                                                *
*  /** ENTRY FOR USER INPUT OR OUTPUT DATA SET TO ASSIGN DEFAULTS.    *
*  /**                                                                *
*  /*******************************************************************
*  DCBROUT3:;
*         SAVE14 = R14  /* SAVE OPEN RETURN REG */;
DCBROUT3 ST    @E,COMMON1+56                                      0721
*         IF DSDOSW = ON THEN DO  /*DROP THRU IF OUTPUT DCB*/;
         CLC   COMMON2+38(1),ON                                   0722
         BC    07,@9CD                                            0722
*               DCBPTR = CURRGMO  /* PICK UP OUTPUT DCB ADDRESS*/;
         MVC   COMMON+300(4),COMMON1+80                           0724
*               GO TO DCBROUT4 /* BR TO CONTINUE PROC */;
         BC    15,DCBROUT4                                        0725
*         END;
*               DCBPTR = CURRGMI  /*PICK UP INPUT DCB ADDRESS*/;
@9CD     MVC   COMMON+300(4),COMMON1+92                           0727
*  DCBROUT4:;
*               DLRECL = 1 /*DEFAULT LRECL IS ONE*/;
DCBROUT4 LA    @F,1                                               0729
         STH   @F,COMMON+280                                      0729
*               DBLKSI = 1 /* DEFAULT BLKSIZE IS ONE */;
         STH   @F,COMMON+282                                      0730
*               DRECFM = 'C0'X /* DEFAULT RECFM IS U */;
         MVI   COMMON+284,X'C0'                                   0731
*               OLDNEWSW = ON /* TURN ON SWITCH */;
         MVC   COMMON2+35(1),ON                                   0732
*  /*******************************************************************
*  /**                                                                *
*  /** LOOK FOR BLOCKSIZE AND SET DEFAULT IF MISSING.                 *
*  /*******************************************************************
*  DCBEXIT:;
*               /* DCB POINTER WAS INITILIZED AT OPEN TIME */
*               /* IF BLOCKSIZE IS EQUAL TO ZERO ASSIGN DEFAULT*/
*         IF BLKSI ^= 0 THEN GO TO DCBEXIT1;
DCBEXIT  SR    @F,@F                                              0734
         L     @2,COMMON+300       COMMON                         0734
         CH    @F,62(0,@2)                                        0734
         BC    07,DCBEXIT1                                        0735
*               /* WHEN OLDNEW WE FLUSH IF DEFAULTED */
*         IF OLDNEWSW = ON THEN DO;
         CLC   COMMON2+35(1),ON                                   0736
         BC    07,@9CC                                            0736
*               FLUSHSW = ON;
         MVC   COMMON2+36(1),ON                                   0738
*         END;
*               /* ASSIGN DEFAULT */
*         BLKSI = DBLKSI;
@9CC     L     @2,COMMON+300       COMMON                         0740
         MVC   62(2,@2),COMMON+282                                0740
*  /*******************************************************************
*  /**                                                                *
*  /** LOOK FOR LRECL AND SET DEFAULT IF MISSING.                     *
*  /*******************************************************************
*  DCBEXIT1:;
*               /** PASS RECFM 'U', 'F', 'FS', 'V' AND      /*@ZA33693*
*               /** 'VB' THROUGH THIS TEST AS LRECL         /*@ZA33693*
*               /** IS NOT REQUIRED BY DATA MANAGEMENT      /*@ZA33693*
*         IF RECFM(1)='1'B        /*IS RECFM 'U' OR 'F'     /*@ZA33693*
*         THEN IF RECFM(2)='1'B   /*YES, WHICH IS IT        /*@ZA33693*
DCBEXIT1 L     @2,COMMON+300       COMMON                         0742
         TM    36(@2),B'10000000'                                 0742
         BC    12,@9CB                                            0742
*              THEN GOTO DCBEXIT4;     /*'U'.               /*@ZA33693*
         TM    36(@2),B'01000000'                                 0743
         BC    01,DCBEXIT4                                        0744
*              ELSE IF RECFM(4)='0'B   /*'F', IS IT UNBLCKD /*@ZA33693*
*                   THEN GOTO DCBEXIT2;     /*YES.          /*@ZA33693*
         TM    36(@2),B'00010000'                                 0745
         BC    08,DCBEXIT2                                        0746
         BC    15,@9CA                                            0748
*                   ELSE;                                   /*@ZA33693*
*         ELSE IF RECFM(2)='1'B   /*NO,  IS IT 'V'          /*@ZA33693*
*              THEN IF RECFM(5)='0'B   /*YES, IS IT ^SPANNED/*@ZA33693*
@9CB     L     @2,COMMON+300       COMMON                         0748
         TM    36(@2),B'01000000'                                 0748
         BC    12,@9C9                                            0748
*                   THEN GOTO DCBEXIT2;     /*YES.          /*@ZA33693*
         TM    36(@2),B'00001000'                                 0749
         BC    08,DCBEXIT2                                        0750
*
*               /* IF LRECL ZERO DEFAULT */
*         IF LRECL ^= 0 THEN GO TO DCBEXIT2;
@9C9     EQU   *                                                  0751
@9CA     SR    @F,@F                                              0751
         L     @2,COMMON+300       COMMON                         0751
         CH    @F,82(0,@2)                                        0751
         BC    07,DCBEXIT2                                        0752
*         /* FLUSH WHEN OLDNEW */
*         IF OLDNEWSW = ON THEN DO;
         CLC   COMMON2+35(1),ON                                   0753
         BC    07,@9C8                                            0753
*               FLUSHSW = ON;
         MVC   COMMON2+36(1),ON                                   0755
*         END;
*               /* ASSIGN DEFAULT */
*         LRECL = DLRECL;
@9C8     L     @2,COMMON+300       COMMON                         0757
         MVC   82(2,@2),COMMON+280                                0757
*  /*******************************************************************
*  /**                                                                *
*  /** THIS SECTION TESTS FOR 'F' RECORD.IF 'F' BLOCKSIZE MUST BE AN  *
*  /** EVEN MULTIPLE OF LRECL.IF NOT,FLUSHING TO END CARD IS INDICATED*
*  /** DEFAULTS ARE ASSIGNED SO THAT DATA SET WILL OPEN WITHOUT AN    *
*  /** ABEND.A BRANCH IS THEN TAKEN TO RETURN TO OPEN.                *
*  /*******************************************************************
*  DCBEXIT2:;
*                                        /* TEST FOR V OR U RCD */
*         IF RECFM(1) = '0'B THEN GO TO DCBEXIT3;
DCBEXIT2 L     @2,COMMON+300       COMMON                         0759
         TM    36(@2),B'10000000'                                 0759
         BC    08,DCBEXIT3                                        0760
*                                        /* TEST FOR F RCD */
*         IF RECFM(2) = '1'B THEN GO TO DCBEXIT3;
         TM    36(@2),B'01000000'                                 0761
         BC    01,DCBEXIT3                                        0762
*
*         /* F TYPE AT THIS POINT. IF UNBLOCKED AND         /*@ZA33693*
*         /* LRECL=0, SKIP TEST TO AVOID DIVISION BY 0.     /*@ZA33693*
*
*         IF RECFM(4) ='0'B  /*IS IT UNBLOCKED              /*@ZA33693*
*         THEN IF LRECL=0    /*YES, IS LRECL ZERO           /*@ZA33693*
         TM    36(@2),B'00010000'                                 0763
         BC    05,@9C7                                            0763
*              THEN GOTO DCBEXIT3;/*YES, SKIP TEST          /*@ZA33693*
         SR    @F,@F                                              0764
         CH    @F,82(0,@2)                                        0764
         BC    08,DCBEXIT3                                        0765
*
*         /* BLOCKSIZE MUST BE AN EVEN MULTIPLE OF LRECL DIVIDE THE */
*         /* TWO THE REMAINDER IS PLACED IN LEFTOVER */
*               LEFTOVER = BLKSI // LRECL;
@9C7     L     @2,COMMON+300       COMMON                         0766
         LH    @E,62(0,@2)                                        0766
         SRDA  @E,32                                              0766
         LH    @0,82(0,@2)                                        0766
         DR    @E,@0                                              0766
         ST    @E,COMMON+288                                      0766
*                                        /* LOOK FOR REMAINDER */
*         IF LEFTOVER = 0 THEN GO TO DCBEXIT4;
         SR    @F,@F                                              0767
         C     @F,COMMON+288                                      0767
         BC    08,DCBEXIT4                                        0768
*         /* THERE WAS A REMAINDER AT THIS POINT */
*         /* TURN ON FLUSH SW PLACE DEFAULT VALUES IN DCB TO */
*         /* PREVENT OPEN ABEND */
*         /* ON RETURN FROM OPEN FLUSH WILL BE CHECKED */
*         FLUSHSW  =  ON  /* TURN ON FLUSH SW */;
         MVC   COMMON2+36(1),ON                                   0769
*               BLKSI = DBLKSI /* DEFAULT BLOCKSIZE */;
         MVC   62(2,@2),COMMON+282                                0770
*               LRECL = DLRECL /* DEFAULT LRECL */;
         MVC   82(2,@2),COMMON+280                                0771
*               MS = 28  ;     /* PRINT MSG IEB728I            @ZA24161
         LA    @F,28                                              0772
         STH   @F,COMMON1+102                                     0772
*                       /* OZ24161 DID NOT CHANGE IEBDGMSG. /*@ZA33694*
*               CONCODE = 8 ;  /* SET RETURN CODE 8            @ZA24161
         LA    @F,8                                               0773
         STH   @F,COMMON1+2                                       0773
*         GO TO DCBEXIT4 /* BRANCH TO RETURN */;
         BC    15,DCBEXIT4                                        0774
*  /*******************************************************************
*  /**                                                                *
*  /** LOOK FOR RECORD FORMAT AND SET DEFAULT IF MISSING.             *
*  /*******************************************************************
*  DCBEXIT3:;
*                                        /* RECORD IS U OR V AT THIS*/
*                                        /* POINT BUT MUST FIND IF */
*                                        /* NO RECFM WAS SPECIFIED */
*         /* AT THIS POINT NO RECFM WAS SPECIFIED */
*         IF RECFM(1) = '0'B THEN DO;
DCBEXIT3 L     @2,COMMON+300       COMMON                         0776
         TM    36(@2),B'10000000'                                 0776
         BC    05,@9C6                                            0776
*               IF RECFM(2) ^= '0'B THEN GO TO DCBEXIT4;
         TM    36(@2),B'01000000'                                 0778
         BC    05,DCBEXIT4                                        0779
*         /* ASSIGN DEFAULT LRECL */
*         RECFM = DRECFM;
         MVC   36(1,@2),COMMON+284                                0780
*         /* MUST FLUSH IF OLDNEW DS */
*         /* WE CAN LIVE WITH SYSIN SYSPRINT DEFAULTS*/
*         IF OLDNEWSW ^= ON THEN GO TO DCBEXIT4;
         CLC   COMMON2+35(1),ON                                   0781
         BC    07,DCBEXIT4                                        0782
*               FLUSHSW = ON /* TURN ON FLUSH */;
         MVC   COMMON2+36(1),ON                                   0783
*         END;
*  DCBEXIT4:;
@9C6     EQU   *                                                  0785
*  /*******************************************************************
*  /**                                                                *
*  /** RETURN TO OPEN HERE                                            *
*  /*******************************************************************
*  RETDCB:;
DCBEXIT4 EQU   *                                                  0786
*         /* DEACTIVATE DCB EXIT . IT IS NORMALLY ACTIVATED */
*         /* JUST PRIOR TO ISSUEING OPEN */
*  /* SYSI AND SYSP DCB EXITS ARE LEFT ACTIVE */
*  /* THIS IS FOR OLDNEW DATA SET */
*               EXITDCB = '00'X ;
RETDCB   MVI   COMMON+260,X'00'                                   0787
*         R14 = SAVE14 ; /* RESTORE RETURN REG */
         L     @E,COMMON1+56                                      0788
*         GEN ( BR 14 ) ;
          BR 14
         DS    0H
*  /*******************************************************************
*  /**                                                                *
*  /** THIS SECTION PROCESSES FOR I/O ERRORS                          *
*  /*******************************************************************
*         /* THIS IS THE SYNAD ROUTINE AND USES SYNADAF  AND SYNADRLS*/
*         /* MACROS. ERROR INFORMATION IS WRITTEN ON SYSPRINT */
*  ERRORS:;
*               /* ISSUE SYNADAF MACRO*/
*         GEN ( SYNADAF  ACSMETH=QSAM );
ERRORS    SYNADAF  ACSMETH=QSAM
         DS    0H
*         /* MOVE INFO TO INBUF */
*         INBUFA1 = ' IEB729I PERMANENT I/O ERROR'; /*SET MSG ID */
         MVC   COMMON1+104(28),@C33                               0792
         MVI   COMMON1+132,C' '                                   0792
         MVC   COMMON1+133(92),COMMON1+132                        0792
*  GEN ( MVC INBUFA1+28(79),49(1));
          MVC INBUFA1+28(79),49(1)
         DS    0H
*         MS = 30 /* MESSAGE NUMBER FOR MSGMOD */;
         LA    @F,30                                              0794
         STH   @F,COMMON1+102                                     0794
*         /* LINK TO MESSAGE MODULE */
*         GEN ( LINK EP=IEBDGMSG);
          LINK EP=IEBDGMSG
         DS    0H
*         /* RELEASE BY MACRO */
*         GEN ( SYNADRLS);
          SYNADRLS
         DS    0H
*               GO TO A2J2;    /** BRANCH TO CLEANUP**/
         BC    15,A2J2                                            0797
*  USER : ;
*  /*******************************************************************
*  /** THIS SEQUENCE PLACES THE CONDITION CODE INTO THE SAVE AREA     *
*  /** AT THE SPOT THAT REG 15 WILL BE LOADED FROM BY THE RETURN      *
*  /** MECHANISM.THIS PASSES THE RETURN CODE BACK TO THE USER THROUGH *
*  /** REG 15 AS DEMANDED BY SYSTEM CONVENTIONS                       *
*  /*******************************************************************
*               R0 = CONCODE   /** PLACE CONDITION CODE IN REG 0 **/;
USER     LH    @0,COMMON1+2                                       0799
*         GEN ( L R1,4(13));
          L R1,4(13)
         DS    0H
*         GEN ( ST R0,16(R1));
          ST R0,16(R1)
         DS    0H
*                   RETURN;
         BC    15,@EL01                                           0802
*        GENERATE ;                                          /* A37883*
GETAREA  EQU   176                     AREA FOR JFCB           A38883
         DS    0H
*  GEN;
MYDCB1 DCB DDNAME=SYSIN,DSORG=PS,MACRF=GM,                             *
               EODAD=CLEANUP,                                          *
               SYNAD=ERRORS,                                           *
               EXLST=DCBROUT1,                                         *
               EROPT=ACC,                                  /*@ZA07394*/C
               LRECL=80,                                   /*@ZA07394*/C
               RECFM=FB                                    /*@ZA07394*/
MYDCB2 DCB DDNAME=SYSPRINT,DSORG=PS,MACRF=PM,                          *
               EXLST=DCBROUT2,                                         *
               EROPT=ACC,                                  /*@ZA07394*/C
               LRECL=121,                                  /*@ZA07394*/C
               RECFM=FBA                                   /*@ZA07394*/
MYDCB3 DCB DDNAME=SYSUT1,DSORG=IS,MACRF=PM,EXLST=DCBROUT3
         DS    0H
*  END IEBDG;
@EL01    L     @D,4(0,@D)                                         0805
         LR    @1,@C                                              0805
         L     @0,@SIZ001                                         0805
         FREEMAIN R,LV=(0),A=(1)                                  0805
         LM    @E,@C,12(@D)                                       0805
         BCR   15,@E                                              0805
@DATA1   EQU   *
@0       EQU   00                  EQUATES FOR REGISTERS 0-15
@1       EQU   01
@2       EQU   02
@3       EQU   03
@4       EQU   04
@5       EQU   05
@6       EQU   06
@7       EQU   07
@8       EQU   08
@9       EQU   09
@A       EQU   10
@B       EQU   11
@C       EQU   12
@D       EQU   13
@E       EQU   14
@F       EQU   15
@D6      DC    F'65535'
@D1      DC    H'256'
@D2      DC    H'1'
@D3      DC    H'3'
@D4      DC    H'8'
@D5      DC    H'7'
@MVC     MVC   0(1,@A),0(@E)
         DS    0F
@SIZ001  DC    AL1(&SPN)
         DC    AL3(@DATEND-@DATD)
         DS    0F
@X2      DC    X'F0F0F0F1'
@C7      DC    C'LINECNT='
@X9      DC    X'0000000000000000'
@C18     DC    C'DUMP'
@C21     DC    C'OUTPUT=('
@C33     DC    C' IEB729I PERMANENT I/O ERROR'
@X6      DC    X'000C'
@X8      DC    X'0030'
@X10     DC    X'0028'
@C15     DC    C'FD'
@C16     DC    C'CREATE'
@C17     DC    C'REPEAT'
@C19     DC    C'/*'
@X27     DC    X'5000'
@C14     DC    C'END'
@C22     DC    C'INPUT=('
@C29     DC    C'QUANTITY='
@C30     DC    C'CREATE='
         DS    0D
@DATA    EQU   *
DUMMY    EQU   00000000            FULLWORD INTEGER
OPTIONS  EQU   00000000            256 BYTE(S)
PAGE     EQU   00000000            256 BYTE(S)
DDLST    EQU   00000000            256 BYTE(S)
DCBD     EQU   00000000            280 BYTE(S) ON DWORD
FILL     EQU   DCBD+00000000       16 BYTE(S)
KEYLEN   EQU   DCBD+00000016       1 BYTE(S)
DEVT     EQU   DCBD+00000017       8 BIT(S)
FILL1    EQU   DCBD+00000018       8 BYTE(S)
DSORG1   EQU   DCBD+00000026       2 BYTE(S)
DSORG    EQU   DCBD+00000026       8 BIT(S)
FILLER   EQU   DCBD+00000028       8 BYTE(S)
IOBAD    EQU   DCBD+00000028       4 BYTE(S)
BFTEK    EQU   DCBD+00000032       8 BIT(S)
EODAD    EQU   DCBD+00000033       3 BYTE(S)
RECFM    EQU   DCBD+00000036       8 BIT(S)
EXLIST   EQU   DCBD+00000037       3  BYTE  POINTER ON WORD+1
DDNAME   EQU   DCBD+00000040       8 BYTE(S)
DEBAD    EQU   DCBD+00000040       4 BYTE(S)
IFLGS    EQU   DCBD+00000040       8 BIT(S)
GETAD    EQU   DCBD+00000048       4 BYTE(S)
OFLGS    EQU   DCBD+00000048       8 BIT(S)
OFLGS1   EQU   DCBD+00000049       1 BYTE(S)
MACRF    EQU   DCBD+00000050       2 BYTE(S)
FILL2    EQU   DCBD+00000052       4 BYTE(S)
SYNAD    EQU   DCBD+00000056       4 BYTE(S)
CIND     EQU   DCBD+00000060       2 BYTE(S)
BLKSI    EQU   DCBD+00000062       HALFWORD INTEGER
FILL3    EQU   DCBD+00000064       18 BYTE(S)
LRECL    EQU   DCBD+00000082       HALFWORD INTEGER
FILL4    EQU   DCBD+00000084       172 BYTE(S)
NEXTDCB  EQU   DCBD+00000256       FULLWORD POINTER
DDNAME1  EQU   DCBD+00000260       8 BYTE(S)
EODSW    EQU   DCBD+00000268       1 BYTE(S)
DCBSW1   EQU   DCBD+00000269       1 BYTE(S)
DCBSW2   EQU   DCBD+00000270       1 BYTE(S)
DCBSW3   EQU   DCBD+00000271       8 BIT(S)
INREC    EQU   DCBD+00000272       FULLWORD POINTER
GMLGTH   EQU   DCBD+00000276       HALFWORD INTEGER
FIELDSEL EQU   DCBD+00000278       1 BYTE(S)
SPARE    EQU   DCBD+00000279       1 BYTE(S)
R2       EQU   00000002            FULLWORD INTEGER REGISTER
R4       EQU   00000004            FULLWORD POINTER REGISTER
R5       EQU   00000005            FULLWORD POINTER REGISTER
R6       EQU   00000006            FULLWORD POINTER REGISTER
R7       EQU   00000007            FULLWORD POINTER REGISTER
R8       EQU   00000008            FULLWORD POINTER REGISTER
R9       EQU   00000009            FULLWORD POINTER REGISTER
R1       EQU   00000001            FULLWORD INTEGER REGISTER
R14      EQU   00000014            FULLWORD INTEGER REGISTER
R15      EQU   00000015            FULLWORD INTEGER REGISTER
R0       EQU   00000000            FULLWORD INTEGER REGISTER
ONE      EQU   *                   HALFWORD INTEGER
         DC    FL2'1'
TWO      EQU   *                   HALFWORD INTEGER
         DC    FL2'2'
THREE    EQU   *                   HALFWORD INTEGER
         DC    FL2'3'
FOUR     EQU   *                   HALFWORD INTEGER
         DC    FL2'4'
FIVE     EQU   *                   HALFWORD INTEGER
         DC    FL2'5'
SIX      EQU   *                   HALFWORD INTEGER
         DC    FL2'6'
SEVEN    EQU   *                   HALFWORD INTEGER
         DC    FL2'7'
EIGHT    EQU   *                   HALFWORD INTEGER
         DC    FL2'8'
NINE     EQU   *                   HALFWORD INTEGER
         DC    FL2'9'
TEN      EQU   *                   HALFWORD INTEGER
         DC    FL2'10'
ELEVEN   EQU   *                   HALFWORD INTEGER
         DC    FL2'11'
DSD      EQU   *                   3 BYTE(S)
         DC    C'DSD'
CREATE   EQU   *                   6 BYTE(S)
         DC    C'CREATE'
FD       EQU   *                   2 BYTE(S)
         DC    C'FD'
ON       EQU   *                   1 BYTE(S)
         DC    X'FF'
OFF      EQU   *                   1 BYTE(S)
         DC    X'00'
MYDCB1A  EQU   MYDCB1+00000000     96 BYTE(S)
MYDCB2A  EQU   MYDCB2+00000000     96 BYTE(S)
MYDCB3A  EQU   MYDCB3+00000000     256 BYTE(S)
         ORG   @DATA+00000036
C00      EQU   *                   HALFWORD INTEGER
         DC    FL2'0'
C0       EQU   *                   24 BIT(S) ON BYTE
         DC    X'000000'
         ORG   @DATA+00000042
C10      EQU   *                   HALFWORD INTEGER
         DC    FL2'10'
C42      EQU   *                   HALFWORD INTEGER
         DC    FL2'42'
C50      EQU   *                   HALFWORD INTEGER
         DC    FL2'50'
C66      EQU   *                   HALFWORD INTEGER
         DC    FL2'66'
INBUF    EQU   00000000            80 BYTE(S)
LIST     EQU   00000000            12 BYTE(S)
FDTBL    EQU   00000000            40 BYTE(S)
BUFFER   EQU   00000000            32767 BYTE(S)
AREA     EQU   00000000            512 BYTE(S)
JFCB     EQU   00000000            176 BYTE(S) ON WORD
JFILL    EQU   JFCB+00000000       98 BYTE(S)
DSORGD   EQU   JFCB+00000098       2 BYTE(S)
DSORGJ   EQU   JFCB+00000098       8 BIT(S)
KFILL    EQU   JFCB+00000100       76 BYTE(S)
         ORG   @DATA
         DS    00000050C
@L       EQU   1
@DATD    DSECT
@SAV001  EQU   @DATD+00000000      72 BYTE(S) ON WORD
COMMON   EQU   @DATD+00000072      304 BYTE(S) ON DWORD
PAGENO   EQU   COMMON+00000000     4 BYTE(S)
LINECT   EQU   COMMON+00000004     FULLWORD INTEGER
LINECTR  EQU   COMMON+00000008     FULLWORD INTEGER
PARM     EQU   COMMON+00000012     FULLWORD POINTER
REPEATNO EQU   COMMON+00000016     HALFWORD INTEGER
CREATENO EQU   COMMON+00000018     HALFWORD INTEGER
SYSP     EQU   COMMON+00000020     96 BYTE(S)
SYSI     EQU   COMMON+00000116     96 BYTE(S)
Q        EQU   COMMON+00000216     16 BYTE(S) ON DWORD
QFILL    EQU   COMMON+00000216     7 BYTE(S)
QSIGN    EQU   COMMON+00000223     8 BIT(S)
QFILL1   EQU   COMMON+00000224     7 BYTE(S)
QSIGN1   EQU   COMMON+00000231     8 BIT(S)
COUNTER  EQU   COMMON+00000232     FULLWORD POINTER
OPENLIST EQU   COMMON+00000236     8 BYTE(S)
OPTBYTE1 EQU   COMMON+00000236     4 BYTE(S)
OPTBYTE2 EQU   COMMON+00000240     4 BYTE(S)
EXLST    EQU   COMMON+00000244     24 BYTE(S) ON WORD
INHDR    EQU   COMMON+00000244     1 BYTE(S)
INHDR1   EQU   COMMON+00000245     3  BYTE  POINTER ON WORD+1
OUTHDR   EQU   COMMON+00000248     1 BYTE(S)
OUTHDR1  EQU   COMMON+00000249     3  BYTE  POINTER ON WORD+1
INTRL    EQU   COMMON+00000252     1 BYTE(S)
INTRL1   EQU   COMMON+00000253     3  BYTE  POINTER ON WORD+1
OUTTRL   EQU   COMMON+00000256     1 BYTE(S)
OUTTRL1  EQU   COMMON+00000257     3  BYTE  POINTER ON WORD+1
EXITDCB  EQU   COMMON+00000260     1 BYTE(S)
EXITDCB1 EQU   COMMON+00000261     3  BYTE  POINTER ON WORD+1
TOTAL    EQU   COMMON+00000264     1 BYTE(S)
TOTAL1   EQU   COMMON+00000265     3  BYTE  POINTER ON WORD+1
EXLST1   EQU   COMMON+00000268     4 BYTE(S) ON WORD
EDCB1    EQU   COMMON+00000268     1 BYTE(S)
EDCB2    EQU   COMMON+00000269     3  BYTE  POINTER ON WORD+1
EXLST2   EQU   COMMON+00000272     4 BYTE(S) ON WORD
EDCB3    EQU   COMMON+00000272     1 BYTE(S)
EDCB4    EQU   COMMON+00000273     3  BYTE  POINTER ON WORD+1
EXLST3   EQU   COMMON+00000276     4 BYTE(S) ON WORD
EDCB5    EQU   COMMON+00000276     1 BYTE(S)
EDCB6    EQU   COMMON+00000277     3  BYTE  POINTER ON WORD+1
DLRECL   EQU   COMMON+00000280     HALFWORD INTEGER
DBLKSI   EQU   COMMON+00000282     HALFWORD INTEGER
DRECFM   EQU   COMMON+00000284     8 BIT(S)
LEFTOVER EQU   COMMON+00000288     FULLWORD INTEGER
OFFSET   EQU   COMMON+00000292     FULLWORD POINTER
LPTR     EQU   COMMON+00000296     FULLWORD POINTER
DCBPTR   EQU   COMMON+00000300     FULLWORD POINTER
COMMON1  EQU   @DATD+00000376      232 BYTE(S) ON WORD
SAVEMS   EQU   COMMON1+00000000    HALFWORD INTEGER
CONCODE  EQU   COMMON1+00000002    HALFWORD INTEGER
OUTREC   EQU   COMMON1+00000004    FULLWORD POINTER
CRTABPT  EQU   COMMON1+00000008    FULLWORD POINTER
CURCRTE  EQU   COMMON1+00000012    FULLWORD POINTER
CURCRGM  EQU   COMMON1+00000016    FULLWORD POINTER
CURPIC   EQU   COMMON1+00000020    FULLWORD POINTER
PICCTR   EQU   COMMON1+00000024    FULLWORD INTEGER
EXITTAB  EQU   COMMON1+00000028    FULLWORD POINTER
EXITGM   EQU   COMMON1+00000032    FULLWORD POINTER
CUREXIT  EQU   COMMON1+00000036    FULLWORD POINTER
DELIM    EQU   COMMON1+00000040    4 BYTE(S)
RECREM   EQU   COMMON1+00000044    FULLWORD INTEGER
CURFD    EQU   COMMON1+00000048    FULLWORD POINTER
CUROUT   EQU   COMMON1+00000052    FULLWORD POINTER
SAVE14   EQU   COMMON1+00000056    FULLWORD POINTER
GETMLIST EQU   COMMON1+00000060    16 BYTE(S) ON WORD
GLENGTH  EQU   COMMON1+00000060    FULLWORD INTEGER
ADRLIST  EQU   COMMON1+00000064    FULLWORD POINTER
IND      EQU   COMMON1+00000068    4 BYTE(S)
GCODE    EQU   COMMON1+00000068    1 BYTE(S)
SPOOL    EQU   COMMON1+00000069    1 BYTE(S)
CCODE    EQU   COMMON1+00000070    2 BYTE(S)
GCADDR   EQU   COMMON1+00000072    FULLWORD POINTER
FIRSTGMO EQU   COMMON1+00000076    FULLWORD POINTER
CURRGMO  EQU   COMMON1+00000080    FULLWORD POINTER
LASTGMO  EQU   COMMON1+00000084    FULLWORD POINTER
FIRSTGMI EQU   COMMON1+00000088    FULLWORD POINTER
CURRGMI  EQU   COMMON1+00000092    FULLWORD POINTER
LASTGMI  EQU   COMMON1+00000096    FULLWORD POINTER
CONDCODE EQU   COMMON1+00000100    HALFWORD INTEGER
MS       EQU   COMMON1+00000102    HALFWORD INTEGER
INBUFA1  EQU   COMMON1+00000104    121 BYTE(S)
INFILL   EQU   COMMON1+00000104    10 BYTE(S)
INBUFA   EQU   COMMON1+00000114    111 BYTE(S)
DDPTR    EQU   COMMON1+00000228    FULLWORD POINTER
COMMON2  EQU   @DATD+00000608      72 BYTE(S) ON WORD
SWITCH   EQU   COMMON2+00000000    52 BYTE(S)
FDCSW    EQU   COMMON2+00000000    1 BYTE(S)
FDNAMESW EQU   COMMON2+00000001    1 BYTE(S)
FDPCSW   EQU   COMMON2+00000002    1 BYTE(S)
FDFMTSW  EQU   COMMON2+00000003    1 BYTE(S)
FDPLSW   EQU   COMMON2+00000004    1 BYTE(S)
RANGESW  EQU   COMMON2+00000005    1 BYTE(S)
FILLSW   EQU   COMMON2+00000006    1 BYTE(S)
REPSW    EQU   COMMON2+00000007    1 BYTE(S)
INDEXSW  EQU   COMMON2+00000008    1 BYTE(S)
INDNMSW  EQU   COMMON2+00000009    1 BYTE(S)
BQUOTESW EQU   COMMON2+00000010    1 BYTE(S)
PQUOTESW EQU   COMMON2+00000011    1 BYTE(S)
EQUOTESW EQU   COMMON2+00000012    1 BYTE(S)
FDSW     EQU   COMMON2+00000013    1 BYTE(S)
DSDSW    EQU   COMMON2+00000014    1 BYTE(S)
NOGOSW   EQU   COMMON2+00000015    1 BYTE(S)
CREATESW EQU   COMMON2+00000016    1 BYTE(S)
DSDCSW   EQU   COMMON2+00000017    1 BYTE(S)
CRCSW    EQU   COMMON2+00000018    1 BYTE(S)
EXITSW   EQU   COMMON2+00000019    1 BYTE(S)
EODSTOP  EQU   COMMON2+00000020    1 BYTE(S)
DSDNULSW EQU   COMMON2+00000021    1 BYTE(S)
DSDORGSW EQU   COMMON2+00000022    1 BYTE(S)
DSDDDSW  EQU   COMMON2+00000023    1 BYTE(S)
CRTBLK   EQU   COMMON2+00000024    1 BYTE(S)
NAMCSW   EQU   COMMON2+00000025    8 BIT(S)
PICCSW   EQU   COMMON2+00000026    8 BIT(S)
BUFPSW   EQU   COMMON2+00000027    1 BYTE(S)
ENDSW    EQU   COMMON2+00000028    1 BYTE(S)
COMCSW   EQU   COMMON2+00000029    1 BYTE(S)
FLAGSW   EQU   COMMON2+00000030    8 BIT(S)
PAGESW   EQU   COMMON2+00000031    1 BYTE(S)
EPSW     EQU   COMMON2+00000032    1  BYTE  POINTER
SYSISW   EQU   COMMON2+00000033    1 BYTE(S)
SYSPSW   EQU   COMMON2+00000034    1 BYTE(S)
OLDNEWSW EQU   COMMON2+00000035    1 BYTE(S)
FLUSHSW  EQU   COMMON2+00000036    1 BYTE(S)
FLUSHSW1 EQU   COMMON2+00000037    1 BYTE(S)
DSDOSW   EQU   COMMON2+00000038    1 BYTE(S)
DSDISW   EQU   COMMON2+00000039    1 BYTE(S)
QUANSW   EQU   COMMON2+00000040    1 BYTE(S)
PARENSW  EQU   COMMON2+00000041    1 BYTE(S)
REPEATSW EQU   COMMON2+00000042    1 BYTE(S)
FDPLGTH  EQU   COMMON2+00000052    HALFWORD INTEGER
SGCADDR  EQU   COMMON2+00000056    FULLWORD POINTER
FDPTR    EQU   COMMON2+00000060    FULLWORD POINTER
FDPTR1   EQU   COMMON2+00000064    FULLWORD POINTER
FDPTR2   EQU   COMMON2+00000068    FULLWORD POINTER
COMMON3  EQU   @DATD+00000680      128 BYTE(S) ON WORD
FDCTR    EQU   COMMON3+00000000    HALFWORD POINTER
LREMAIN  EQU   COMMON3+00000004    FULLWORD POINTER
COMPCTR  EQU   COMMON3+00000008    FULLWORD POINTER
LMOVED   EQU   COMMON3+00000012    HALFWORD POINTER
U        EQU   COMMON3+00000016    FULLWORD POINTER
PICEND   EQU   COMMON3+00000020    HALFWORD POINTER
CURFDGM  EQU   COMMON3+00000024    FULLWORD POINTER
SWTCH    EQU   COMMON3+00000028    4 BYTE(S) ON WORD
SYSINSEL EQU   COMMON3+00000028    1 BYTE(S)
FIRSTSW  EQU   COMMON3+00000029    1 BYTE(S)
FRSTSW   EQU   COMMON3+00000030    1 BYTE(S)
STOPSW   EQU   COMMON3+00000031    1 BYTE(S)
COPYVAL  EQU   COMMON3+00000032    HALFWORD INTEGER
COPYFD   EQU   COMMON3+00000036    FULLWORD POINTER
COPYFDGM EQU   COMMON3+00000040    FULLWORD POINTER
NAMCTR   EQU   COMMON3+00000044    HALFWORD INTEGER
NAMCTR1  EQU   COMMON3+00000046    HALFWORD INTEGER
INRECSZ  EQU   COMMON3+00000048    HALFWORD POINTER
OUTRECSZ EQU   COMMON3+00000050    HALFWORD POINTER
INRECFM  EQU   COMMON3+00000052    1 BYTE(S)
RECOFFST EQU   COMMON3+00000053    1  BYTE  POINTER
OUTRECFM EQU   COMMON3+00000054    1 BYTE(S)
PICBASE  EQU   COMMON3+00000056    FULLWORD POINTER
MESSAGE  EQU   COMMON3+00000060    121 BYTE(S)
DUMMY3   EQU   COMMON3+00000184    FULLWORD POINTER
OPTPTR   EQU   @DATD+00000868      FULLWORD POINTER
PGPTR    EQU   @DATD+00000872      FULLWORD POINTER
BUFPTR   EQU   @DATD+00000876      FULLWORD POINTER
JFCBPTR  EQU   @DATD+00000880      FULLWORD POINTER
EXITLIST EQU   @DATD+00000884      4 BYTE(S) ON WORD
HEX87    EQU   EXITLIST+00000000   8 BIT(S)
PTRJFCB  EQU   EXITLIST+00000001   3  BYTE  POINTER ON WORD+1
         DS    00000888C
@TEMPS   DS    0F
@TEMP4   DC    F'0'
@DATEND  EQU   *
IEBDG    CSECT ,
@9FD     EQU   PDDNAMER
@9FC     EQU   PDDNAMER
@9F2     EQU   LABEL1
@9F3     EQU   SCAN1
@9ED     EQU   USER
@9D9     EQU   A3C45
@9D6     EQU   A2B3
@9D4     EQU   A3C41
         END   IEBDG
