 /********************************************************************/
 /*                                                                  */
 /* CONTROL BLOCK NAME - SCD                                         */
 /*                                                                  */
 /* DESCRIPTIVE NAME - HOT I/O STATUS COLLECTION DATA                */
 /*                                                                  */
 /* MAPPING MACRO NAME - IECDSCD                                     */
 /*                                                                  */
 /* FUNCTION -                                                       */
 /*   THE SCD IS USED TO MAINTAIN INFORMATION ON UNSOLICITED STATUS  */
 /*   AND INTERRUPTS FOR THE PURPOSE OF DETERMINING IF HOT I/O IS    */
 /*   OCCURRING. THERE IS ONE SCD PER CHANNEL PER PROCESSOR. THE     */
 /*   SCDS ARE INDEXED BY CHANNEL NUMBER.                            */
 /*                                                                  */
 /* CHANGE ACTIVITY - NONE                                           */
 /*   A000000-999999                                         @ZA30350*/
 /*                                                                  */
 /********************************************************************/
 DCL
   1 SCD BASED(SCDPTR),            /*HOT I/O STATUS COLLECTION DATA  */
    3 SCDCPUID FIXED(15),          /*CPU OF HOT CHANNEL OR DEVICE    */
    3 SCDADDR FIXED(15),           /*CHANNEL/DEVICE ADDRESS          */
    3 SCDFLG1 BIT(8),              /*FLAG BYTE 1                     */
     5 SCDRECUR BIT(1),            /*HOT I/O RECURSION INDICATOR     */
     5 * BIT(7),                   /*RESERVED                        */
    3 SCDRSCDE,                    /*HOT I/O DETECTION FLAGS         */
     5 SCDRSC1 BIT(8),             /*DETECTION FLAG BYTE 1           */
      7 SCDTMOUT BIT(1),           /*TIME OUT HOT I/O                */
      7 SCDAVAIL BIT(1),           /*AVAILABILITY INTERRUPT          */
      7 SCDUNSOL BIT(1),           /*UNSOLICITED STATUS              */
      7 SCDNOSYS BIT(1),           /*DEVICE NOT SYSGENED             */
      7 SCDHOTD BIT(1),            /*IF SET-HOT DEVICE--OTHERWISE-HOT
                                     HOT CHANNEL                     */
      7 SCDINVDV BIT(1),           /*CHANNEL ERROR AND INVALID DEVICE
                                     ADDR                            */
      7 SCDCUERR BIT(1),           /*CHANNEL ERROR-CONTROL UNIT
                                     PROBABLE CAUSE                  */
      7 SCDCHERR BIT(1),           /*CHANNEL ERROR-CONTROL UNIT NOT
                                     PROBABLE CAUSE                  */
     5 SCDRSC2 BIT(8),             /*RESERVED                        */
    3 * BIT(8),                    /*RESERVED                        */
    3 SCDCSWST BIT(16),            /*CSW STATUS                      */
    3 * FIXED(15),                 /*RESERVED                        */
    3 SCDCNTS,                     /*HOT I/O COUNTERS                */
     5 SCDRCNT FIXED(15),          /*STATUS REPEAT COUNT             */
     5 SCDTRCNT FIXED(15),         /*TIMEOUT REPEAT COUNT            */
    3 SCDTIME,                     /*TIME OF STATUS                  */
     5 SCDTIME1 FIXED(32),         /*VALUE FOR CALCULATING SECONDS
                                     ELAPSED                         */
     5 SCDTIME2 FIXED(32);         /*2ND WORD OF TIME VALUE          */
