* /* START OF SPECIFICATIONS *****************************************
*
*01*  MODULE-NAME= IECVOPTI - 3886 Optical Character Reader
*
*01*  DESCRIPTIVE-NAME =
*                        3886 Optical Character Reader
*                        Channel Command Word (CCW) translation table
*
* COPYRIGHT =  N/A
*
* STATUS =  OS/VS2  EMI2220
*
*01*  FUNCTION=
*
*         Provides a channel command word (CCW) translation table
*         for indicating how a command code is to be translated for
*         the 3886 Optical Character Reader.
*
*
*02*    OPERATION=
*
*         The operation table is 256 bytes in length, one
*         byte for each possible channel command word.
*
*         The normal handling for the translation of a channel
*         command operation code is to treat the command as a
*         data transfer type command and to translate the data address
*         and to fix the data areas involved with the channel command
*         word.
*
*         Each byte in an operation table is defined as follows:
*            . X'80'  - The CCW provides status modifer (SM) support.
*            . X'40'  - The CCW is a non-data transfer type command.
*                       (No data areas to be fixed).
*            . X'20'  - Always zero, reserved for IECVTCCW.
*                       Used by IECVTCCW to indicate a
*                       NO-OP TIC command.  The bit is set in the
*                       copied byte that is moved into byte 5
*                       of each CCW to be translated.
*            . X'10'  - Reserved
*            . X'08'  - Reserved
*            . X'04'  - Reserved
*            . X'02'  - Reserved
*            . X'01'  - Reserved
*
*         The address to each CCW translation operation table
*         is contained as an entry in the Device Dependent Table
*         (DDT) for that device.  The DDT is pointed to out of
*         the UCB for that device.
*
*02*    RECOVERY OPERATION = None
*
*01*  NOTES=
*
*02*    DEPENDENCIES =
*
*         While this operation table has been provided for the
*         device listed above, it may also  have been
*         selected by other newer devices for their use.
*
*         Any other device support that uses this table must
*         be aware that if this table has to be changed to provide
*         for addititional support, that these
*         changes may impact their use of the table.
*
*02*    CHARACTER-CODE-DEPENDENCIES= None
*
*02*    RESTRICTIONS= None
*
*02*    REGISTER-CONVENTIONS = Not applicable
*
*03*      REGISTERS-SAVED = Not applicable
*
*03*      REGISTER-USAGE = Not applicable
*
*03*      REGISTERS-RESTORED= Not applicable
*
*02*    PATCH-LABEL= Not applicable
*
*01*  MODULE-TYPE=
*
*02*    PROCESSOR= Assembler H
*
*02*    MODULE-SIZE= 280 bytes
*
*02*    ATTRIBUTES=
*
*03*      LOCATION= Nucleus
*
*03*      STATE= Supervisor
*
*03*      AMODE= Not applicable
*
*03*      RMODE= 24
*
*03*      KEY= 0
*
*03*      MODE= Not applicable
*
*03*      SERIALIZATION = Not applicable
*
*03*      TYPE= Not applicable
*
*01*  ENTRY-POINT = None
*
*01*  EXTERNAL-REFERENCES = None
*
*02*    ROUTINES = None
*
*02*    DATA-AREAS = None
*
*02*    CONTROL-BLOCKS = None
*
*01*  TABLES = None
*
*01*  MACROS-EXECUTABLE = None
*
*02*    SERIALIZATION = None
*
*01*  CHANGE-ACTIVITY=
* $L0=MICROCR,EMI2220,85220,PDG7:  MICR/OCR reship
*
*01*  MESSAGES = None
*
*01*  ABEND-CODES = None
*
*01*  LOGIC TOOL DATA: For prologue only.
*
*             OPTIONS: CMTCTL    IOCBT      MOPUB
*                      BF12      ACRO
*
*      CONTROL BLOCKS:
*
*              TABLES:
*
**** END OF SPECIFICATIONS *******************************************/
         TITLE 'IECVOPTI: TRANSLATION OP TABLE FOR 3886 OPT CHAR RDR  '
IECVOPTI CSECT      Translation Operation Table: 3886 Opt Char Reader
         SPACE 3
***********************************************************************
*                I     C S     C T     C R     C                      *
*                N W R T N W R T I W R T D W R T                      *
*                V R D L S R D L C R D L B R D L                      *
************** * -0-1-2-3-4-5-6-7-8-9-A-B-C-D-E-F  ********************
*                . . . . . . . . . . . . . . . .                      *
         DC    X'00000040000000400000004000000000'   0-
         DC    X'00000040000000400000004000000040'   1-
         DC    X'00000040000000400000004000000000'   2-
         DC    X'00000040000000400000004000000040'   3-
         DC    X'00000040000000400000004000000000'   4-
         DC    X'00000000000000400000004000000000'   5-
         DC    X'00000000000000400000004000000000'   6-
         DC    X'00000000000000400000004000000000'   7-
         DC    X'00000040000000400000004000000000'   8-
         DC    X'00000040000000400000004000000000'   9-
         DC    X'00000040000000400000004000000000'   A-
         DC    X'00000040000000400000004000000000'   B-
         DC    X'00000040000000400000004000000000'   C-
         DC    X'00000040000000400000004000000000'   D-
         DC    X'00000040000000400000004000000000'   E-
         DC    X'00000000000000400000000000000000'   F-
         SPACE
         MODID BR=NO
***********************************************************************
******** The following bits are defined to indicate special ***********
*        handling to module IECVTCCW :                                *
*         X'80' - The CCW requires status modifier support.           *
*         X'40' - The CCW is a non-data transfer type command         *
*         X'20' - Reserved for IECVTCCW use                           *
*         X'10' - Reserved                                            *
*         X'08' - Reserved                                            *
*         X'04' - Reserved                                            *
*         X'02' - Reserved                                            *
*         X'01' - Reserved                                            *
***********************************************************************
         END   IECVOPTI
