         TITLE 'IEE3503D - DISPLAY / TRACK ROUTER'
***********************************************************************
*                                                                     *
*  MODULE NAME  =  IEE3503D                                     Y02669*
*                                                                     *
*  DESCRIPTION  =  ROUTER FOR DISPLAY AND TRACK COMMANDS        Y02669*
*                                                                     *
*  COPYRIGHT    =  N/A                                          Y02669*
*                                                                     *
*  STATUS       =  VS/2 - DUMPING IMPROVEMENTS                @G33CPMR*
*                                                                     *
*  FUNCTION     =  THIS MODULE DETERMINES THE DISPLAY COMMAND   Y02669*
*                  OPERANDS: D  A          ACTIVE               Y02669*
*                               C,K        K COMMAND OPERANDS   Y02669*
*                               DMN        DOMAIN             @Z40BPSV*
*                               DUMP       DUMP               @G33CPMR*
*                               CONSOLES(C)CONSOLES             YM1916*
*                               JOBS (J)   JOBS, STARTED TASKS  Y02669*
*                               M          MATRIX               Y02651*
*                               NET        VTAM                 Y02674*
*                               PFK        PGM FUNCTION KEYS    Y02669*
*                               R          REQUESTS             Y02669*
*                               SLIP       SERVICEABILITY     @G17SPTJ*
*                                           LEVEL INDICATOR   @G17SPTJ*
*                                            PROGRAM          @G17SPTJ*
*                               T          TIME                 Y02669*
*                               TP         TCAM                 Y02669*
*                               TS         TIME SHARING USERIDS Y02669*
*                               U          UNITS                Y02669*
*                               3850       MSS COMMAND        @Z40LPTA*
*                                                                     *
*                  AND THE TRACK OPERANDS:                      Y02669*
*                           TR  A          ACTIVE               Y02669*
*                               JOBS (J)   JOBS, STARTED TASKS  Y02669*
*                               TS         TIME SHARING USERIDS Y02669*
*                  (THE TRACK COMMAND IS A TIMER INTERVAL       Y02669*
*                  DRIVEN DISPLAY.)                             Y02669*
*                                                                     *
*                  IT THEN ROUTES SPECIFIC INFORMATION IN THE   Y02669*
*                  XSA (EXTENTED SAVE AREA) TO THE APPROPIATE   Y02669*
*                  PROCESSOR.                                   Y02669*
*                                                                     *
*                  ONLY D T (DISPLAY TIME) IS HANDLED           Y02669*
*                  EXCLUSIVELY IN IEE3503D.                     Y02669*
*                                                                     *
     EJECT
*                                                                     *
*   OPERATION   =  FOR D U; U,; THE VERB CODE IN XAN IS CHANGED Y02669*
*                  TO X'52', IEE3503D BRANCHES TO IEE7503D TO   Y02669*
*                  RESOLVE THE ROUTING OPERAND (L=CCA) OR       Y02669*
*                  DETERMINE THE DEFAULTS.                      Y02669*
*                                                                     *
*                  FOR D M; M,; M= THE VERB CODE IN XAN IS      Y02651*
*                  CHANGED TO X'32', IEE3503D BRANCHES TO       Y02651*
*                  IEE7503D TO RESOLVE THE ROUTING OPERAND      Y02651*
*                  (L=CCA) OR DETERMINE THE DEFAULTS.           Y02651*
*                                                                     *
*                  FOR D TS; TS,; JOBS(J); JOBS,(J,); A; A,;    Y02669*
*                  A BIT IS SET IN XAX (XAXMASK), IEE3503D      Y02669*
*                  BRANCHES TO IEE7503D TO ESTABLISH ROUTING    Y02669*
*                  INFORMATION FROM THE L=CCA OPERAND OR        Y02669*
*                  DETERMINE THE APPROPIATE DEFAULTS            Y02669*
*                  AND CHECK FOR AND PROCESS THE 'LIST'(L)      Y02669*
*                  OPERAND. (SEE OUTPUT FOR BIT SETTINGS)       Y02669*
*                                                                     *
*                  FOR D C,K; C,K,; CONSOLES(C); CONSOLES,(C,); YM1916*
*                  THE VERB CODE IN XAN IS CHANGED TO X'2C'     Y02669*
*                  FOR C,K AND X'3E' FOR CONSOLES; IEE3503D     YM1916*
*                  BRANCHES TO IEE7503D TO RESOLVE THE ROUTING  Y02669*
*                  OPERAND (L=CCA) OR DETERMINE THE DEFAULTS.   Y02669*
*                                                                     *
*                  FOR D R; R,; THE VERB CODE IN XAN IS         Y02669*
*                  CHANGED TO X'42', IEE3503D BRANCHES TO       Y02669*
*                  IEE7503D TO RESOLVE THE ROUTING OPERAND      Y02669*
*                  (L=CCA) OR DETERMINE THE DEFAULTS.           Y02669*
*                                                                     *
*                  FOR D PFK; THE VERB CODE IN XAN IS CHANGED   Y02669*
*                  TO X'36'AND XAS IS SET TO ZEROES, IEE3503D   Y02669*
*                  BRANCHES TO IEE0803D TO CREATE A CSCB.       Y02669*
*                                                                     *
*                  FOR D T; THE FOLLOWING MESSAGE IS WRITTEN    Y02669*
*                  TO THE OPERATOR VIA WTO OR TPUT:             Y02669*
*                    IEE136I    TIME=XX.XX.XX    DATE=XX.XXX    Y02669*
*                  CONTROL IS RETURNED.                         Y02669*
*                                                                     *
*                  FOR D TP,; IEE3503D BRANCHES TO IED1303D.    Y02669*
*                                                                     *
*                  FOR D NET,; IEE3503D BRANCHES TO ISTCFF3D.   Y02674*
*                                                                     *
*                  FOR D 3850; IEE3503D BRANCHES TO IEE7503D  @Z40LPTA*
*                                                                     *
*                  FOR D DMN; IEE3503D BRANCHES TO IEE7503D   @Z40BPSV*
*                                                                     *
         EJECT
*                                                                     *
*                  FOR D SLIP, THE VERB CODE IN XAN IS        @G17SPTJ*
*                  CHANGED TO X'A8'.  IEE3503D BRANCHES TO    @G17SPTJ*
*                  IEE7503D TO RESOLVE THE ROUTING OPERAND    @G17SPTJ*
*                  (L=CCA) OR TO DETERMINE THE DEFAULTS       @G17SPTJ*
*                                                                     *
*                  FOR D DUMP, THE VERB CODE IN XAN IS        @G33CPMR*
*                  CHANGED TO X'B0'.  IEE3503D BRANCHES TO    @G33CPMR*
*                  IEE7503D TO RESOLVE THE ROUTING OPERAND    @G33CPMR*
*                  (L=CCA) OR TO DETERMINE THE DEFAULTS       @G33CPMR*
*                                                                     *
*                  **** NOTE ****                             @G17SPTJ*
*                    FOR 'D DMN', 'D SLIP' AND 'D DUMP':      @G33CPMR*
*                  WHENEVER THE ABOVE-MENTIONED COMMANDS ARE  @G17SPTJ*
*                  ENTERED, THEY ARE CHECKED OUT FURTHER TO   @G17SPTJ*
*                  SEE IF THE SYSTEM SUPPORTS THE SU'S THAT   @G17SPTJ*
*                  ARE ASSOCIATED WITH THE COMMAND.           @G17SPTJ*
*                                                             @G17SPTJ*
*                  FOR TR TS; TS,; JOBS(J); JOBS,(J,); A; A,;   Y02669*
*                  A BIT IS SET IN XAX (XAXMASK), IEE3503D      Y02669*
*                  BRANCHES TO IEE7503D TO ESTABLISH ROUTING    Y02669*
*                  INFORMATION FROM THE L=CCA OPERAND OR        Y02669*
*                  DETERMINE THE APPROPIATE DEFAULTS AND        Y02669*
*                  CHECK FOR AND PROCESS THE 'LIST' (L)         Y02669*
*                  OPERAND.(SEE OUTPUT FOR BIT SETTINGS)        Y02669*
*                                                                     *
         EJECT
*                                                                     *
*  NOTES                                                        Y02669*
*                                                                     *
*  CHARACTER/                                                   Y02669*
*  CODE                                                         Y02669*
*  DEPENDENCIES =  ANY CHARACTER CODE OTHER THAN EBCDIC WILL    Y02669*
*                  REQUIRE REASSEMBLY OF THIS MODULE.           Y02669*
*                                                                     *
*  DEPENDENCIES =  IEE0403D TO RECOGNIZE THE DISPLAY AND        Y02669*
*                  TRACK COMMANDS AND PASS THE OPERAND          Y02669*
*                  INFORMATION IN THE XSA.                      Y02669*
*                                                                     *
*                  SVC 100 WILL REJECT AS INVALID D A AND       Y02669*
*                  THE TRACK COMMAND FROM ANY TIME SHARING      Y02669*
*                  TERMINAL INCLUDING THOSE IN OPERATOR MODE.   Y02669*
*                                                                     *
*                  ISTCFF3D WILL VALIDITY CHECK FOR NET,.       Y02674*
*                                                                     *
*                  MP WILL BE SUPPORTED AS A STANDARD           Y02651*
*                  AND D M (MATRIX) WILL BE INCLUDED.           Y02651*
*                                                                     *
*                  THE MSGRT COMMAND WILL PROVIDE SUPPORT FOR   Y02669*
*                  A ROUTING DEFAULT FOR D TS; JOBS; A; R;      Y02669*
*                  AND TR TS; JOBS; AND A.                      Y02669*
*                                                                     *
*                  JOBS AND J AND JOBS,LIST AND J,L WILL BE     Y02669*
*                  SUPPORTED. L CAN SUBSTITUTE FOR 'LIST'.      Y02669*
*                                                                     *
*                  A SU TABLE (IHASUBIT) REFLECTING THE SU'S  @G17SPTJ*
*                  INSTALLED ON THIS SYSTEM.                  @G17SPTJ*
*                                                                     *
*  RESTRICTIONS =  TIME SHARING TERMINALS INCLUDING THOSE IN    Y02669*
*                  OPERATOR MODE ARE NOT PERMITTED TO ISSUE D A.Y02669*
*                                                                     *
*                  THE 'L' OPERAND FROM A TIME SHARING USER     Y02669*
*                  ON D C,K; CONSOLES(C); JOBS; M; TS; R; U     YM1916*
*                  WILL BE DETECTED AS INVALID IN IEE7503D.     Y02669*
*                                                                     *
*                  NO TIME SHARING TERMINAL IS PERMITTED TO     Y02669*
*                  ISSUE THE TRACK COMMAND.                     Y02669*
*                                                                     *
*  REGISTER/                                                    Y02669*
*  CONVENTIONS  =  NONE                                         Y02669*
*                                                                     *
*  PATCH-LABEL  =  PATCH                                        Y02669*
*                                                                     *
* MODULE TYPE   =  CSECT                                        Y02669*
*                                                                     *
*  PROCESSOR    =  N/A                                          Y02669*
*                                                                     *
*  MODULE SIZE  =  X'3DD'                                     @G33CPMR*
*                                                                     *
*                                                                     *
*   ATTRIBUTES  =  REENTRANT, SUPERVISOR MODE, KEY ZERO,        Y02669*
*                  PAGED LPA                                    Y02669*
*                                                                     *
*  ENTRY POINT  =  IEE3503D FROM IEE0403D                       Y02669*
*  PURPOSE      =  TO ROUTE THE DISPLAY OR TRACK                Y02669*
*                  COMMAND TO THE PROPER PROCESSOR.             Y02669*
*  LINKAGE      =  BRANCH                                       Y02669*
*  INPUT DATA   =  REGISTER 2 POINTS TO XSA                     Y02669*
*                    XAL POINTS TO 1ST OPERAND IN BUFFER        Y02669*
*                    XAU IS 1-BYTE UCMI (X'00' COMMAND ISSUED   Y02669*
*                        INTERNALLY OR FROM THE INPUT STREAM)   Y02669*
*                    XAA CONTAINS THE ASID (X'0000' = NON-TS)   Y02669*
*                    XAN CONTAINS THE VERB CODE X'68' - DISPLAY Y02669*
*                                               X'C4' - TRACK   Y02669*
*                    XAR POINTS TO THE LENGTH FIELD AT THE      Y02669*
*                        BEGINNING OF THE COMMAND BUFFER        Y02669*
*                    XAV CONTAINS THE COMMAND VERB              Y02669*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1   Y02669*
*                                                                     *
*   REGISTERS                                                   Y02669*
*    SAVED      =  NONE  (REGISTER 14 REMAINS UNTOUCHED)        Y02669*
*                                                                     *
*    CONTENTS DURING                                            Y02669*
*     PROCESSING=  R0   WORK REGISTER                           Y02669*
*                  R1   WORK REGISTER                           Y02669*
*                  R2   INPUT REGISTER - POINTER TO XSA         Y02669*
*                  R3   MODULE BASE REGISTER                    Y02669*
*                  R4   USED TO CONTAIN UCMI FOR WTO            Y02669*
*                  R5   PTR TO COMMAND BUFFER AND               Y02669*
*                       CONTAINS ASID FOR TPUT                  Y02669*
*                  R6   USED TO ADDRESS COMPARE TABLE ENTRIES   Y02669*
*                        AND THE SU BIT STRING                @G17SPTJ*
*                  R7   UNUSED                                  Y02669*
*                  R8   UNUSED                                  Y02669*
*                  R9   WORK REGISTER                           Y02669*
*                  R10  CONTAINS SU NUMBER                    @G17SPTJ*
*                  R11  ADDRESSES END OF COMPARE TABLE          Y02669*
*                  R12  UNUSED                                  Y02669*
*                  R13  UNUSED                                  Y02669*
*                  R14  RETURN ADDRESS                          Y02669*
*                  R15  USED FOR BAL, RC FROM TPUT, AND BR EXIT Y02669*
*                                                                     *
*    RESTORED    = NONE                                         Y02669*
*                                                                     *
      EJECT
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - NORMAL = RETURN VIA BR R14 FROM DDTIM                 Y02669*
*   CONDITIONS   = FOR D T                                      Y02669*
*   OUTPUT DATA  = IEE136I  TIME=XX.XX.XX   DATE=XX.XXX VIA WTO Y02669*
*   RETURN CODES = NONE                                         Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - NORMAL = RETURN VIA BR R14 FROM XTPUT                 Y02669*
*   CONDITIONS   = FOR D T                                      Y02669*
*   OUTPUT DATA  = IEE136I  TIME=XX.XX.XX  DATE=XX.XXX VIA TPUT Y02669*
*   RETURN CODES = NONE                                         Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - NORMAL = BRANCH TO IEE0803D FROM DDPFK                Y02669*
*   CONDITIONS   = TO CREATE A CSCB FOR D PFK                   Y02669*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02669*
*                    XAN NOW CONTAINS X'36'                     Y02669*
*                    XAS CLEARED (WILL CONTAIN ZEROES)          Y02669*
*                        CONTAINS VALID INPUT TO IEE0803D.      Y02669*
*                        FOR ALL OTHER DISPLAY FUNCTIONS        Y02669*
*                        THIS FIELD IS FILLED IN IN IEE7503D    Y02699*
*                    XAX CLEARED (WILL CONTAIN ZEROES)          Y02669*
*   RETURN CODES = NONE                                         Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - NORMAL = BRANCH TO IEE7503D FROM DD7503D              Y02669*
* 1 CONDITIONS   = TO VALIDITY CHECK:                           Y02669*
*                    L=CCA,LIST FOR  D A,  AND A                Y02669*
*                                      TS,  AND TS              Y02669*
*                                      JOBS, AND JOBS (J, & J)  Y02669*
*                                    TR A, AND A                Y02669*
*                                       TS, AND TS              Y02669*
*                                       JOBS, & JOBS (J, & J)   Y02669*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02669*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1   Y02669*
*                    XAN STILL CONTAINS X'68' FOR DISPLAY       Y02669*
*                                       X'C4' FOR TRACK         Y02669*
*                    XAXMASK WILL CONTAIN AN INDICATOR          Y02669*
*                      SPECIFYING THE DESIRED DISPLAY OR        Y02669*
*                      TRACK FUNCTION:                          Y02669*
*                      X'80' FOR JOBS                           Y02669*
*                      X'08' FOR TS                             Y02669*
*                      X'88' FOR A                              Y02669*
*   RETURN CODES = NONE                                         Y02669*
* 2 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D U AND U,       Y02669*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02669*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1   Y02669*
*                    XAN NOW CONTAINS X'52'                     Y02669*
*                    XAXMASK WILL CONTAIN ZEROES                Y02669*
*   RETURN CODES = NONE                                         Y02669*
* 3 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D C,K; C,K,;     Y02669*
*                    CONSOLES(C); AND CONSOLES,(C,)             YM1916*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02669*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1   Y02669*
*                    XAN NOW CONTAINS X'2C' FOR C,K             Y02669*
*                                     X'3E' FOR CONSOLES(C)     YM1916*
*                    XAXMASK WILL CONTAIN ZEROES                Y02669*
*   RETURN CODES = NONE                                         Y02669*
* 4 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D M; M=; AND M,  Y02651*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02651*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1   Y02669*
*                    XAN NOW CONTAINS X'32'                     Y02651*
*                    XAXMASK WILL CONTAIN ZEROES                Y02651*
*   RETURN CODES = NONE                                         Y02651*
* 5 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D R AND R,       Y02669*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02669*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1   Y02669*
*                    XAN NOW CONTAINS X'42'                     Y02651*
*                    XAXMASK WILL CONTAIN ZEROES                Y02651*
*   RETURN CODES = NONE                                         Y02669*
* 6 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D 3850 AND     @Z40LPTA*
*                    3850,                                    @Z40LPTA*
*   OUTPUT DATA  = R2 POINTS TO XSA                           @Z40LPTA*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1 @Z40LPTA*
*                    XAN NOW CONTAINS X'F3'                   @Z40LPTA*
*                    XAXMASK WILL CONTAIN ZEROES              @Z40LPTA*
*   RETURN CODES = NONE                                       @Z40LPTA*
* 7 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D DMN; DMN=;   @Z40BPSV*
*                    DMN,                                     @Z40BPSV*
*   OUTPUT DATA  = R2 POINTS TO XSA                           @Z40BPSV*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1 @Z40BPSV*
*                    XAN NOW CONTAINS X'88'                   @Z40BPSV*
*                    XAXMASK WILL CONTAIN ZEROES              @Z40BPSV*
*   RETURN CODES = NONE                                       @Z40BPSV*
* 8 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D SLIP; SLIP=; @G17SPTJ*
*                    SLIP,                                    @G17SPTJ*
*   OUTPUT DATA  = R2 POINTS TO XSA                           @G17SPTJ*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1 @G17SPTJ*
*                    XAN NOW CONTAINS X'A8'                   @G17SPTJ*
*                    XAXMASK WILL CONTAIN ZEROES              @G17SPTJ*
*   RETURN CODES = NONE                                       @Z40BPSV*
* 9 CONDITIONS   = TO VALIDITY CHECK L=CCA FOR D DUMP AND     @G33CPMR*
*                    D DUMP,                                  @G33CPMR*
*   OUTPUT DATA  = R2 POINTS TO XSA                           @G33CPMR*
*                    XAP CONTAINS ADDRESS OF END OF BUFFER +1 @G33CPMR*
*                    XAN NOW CONTAINS X'B0'                   @G33CPMR*
*                    XAXMASK WILL CONTAIN ZEROES              @G33CPMR*
*   RETURN CODES = NONE                                       @G33CPMR*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - NORMAL = BRANCH TO IED1303D FROM DDTP                 Y02669*
*   CONDITIONS   = FOR D TP,                                    Y02669*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02669*
*   RETURN CODES = NONE                                         Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - NORMAL = BRANCH TO ISTCFF3D FROM DDNET                Y02674*
*   CONDITIONS   = FOR D NET,                                   Y02674*
*   OUTPUT DATA  = R2 POINTS TO XSA                             Y02674*
*   RETURN CODES = NONE                                         Y02674*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - ERROR  = RETURN VIA BR R14 FROM XTPUT                 Y02669*
*   CONDITIONS   = FOR D T; TPUT FAILED DUE TO OUTSTANDING TPUT Y02669*
*   OUTPUT DATA  = NONE                                         Y02669*
*   RETURN CODES = REG 15 CONTAINS X'00000014'                  Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - ERROR  = BRANCH TO IEE0503D FROM D0503D               Y02669*
*   CONDITIONS   = TO ISSUE MESSAGE 'IEE305I COMMAND INVALID',  Y02669*
*                  IF AN INVALID DISPLAY OR TRACK OPERAND IS    Y02669*
*                  FOUND.                                       Y02669*
*                  TO ISSUE MESSAGE 'IEE026I COMMAND NOT      @G17SPTJ*
*                  SUPPORTED' WHEN SYSTEM DOES NOT SUPPORT    @G17SPTJ*
*                  THE SU REQUIRED BY THE COMMAND.            @G17SPTJ*
*                  TO ISSUE MESSAGE 'IEE311I PARAMETER MISSING',Y02669*
*                  IF NO OPERAND IS FOUND IN THE COMMAND BUFFER.Y02669*
*                  TO ISSUE MESSAGE 'IEE341I CLOCK NOT ACTIVE', Y02669*
*                  IF THE TIME MACRO RETURNS AN ERROR CODE      Y02669*
*                  FOR DISPLAY TIME (D T).                      Y02669*
*   OUTPUT DATA  = ERROR CODES FOR MESSAGES ISSUED IN IEE0503D  Y02669*
*                  IN XSA POINTED TO BY REG 2                   Y02669*
*                    XAE WILL CONTAIN  X'05' FOR IEE305I        Y02669*
*                                      X'0B' FOR IEE311I        Y02669*
*                                      X'24' FOR IEE341I        Y02669*
*                    XAV STILL CONTAINS THE COMMAND VERB        Y02669*
*                        OR THE WORD 'CLOCK' FOR IEE341I        Y02669*
*                    XAU STILL CONTAINS THE CONSOLE ID          Y02669*
*                    XAA STILL CONTAINS ASID IF REQUESTER       Y02669*
*                        IS TIME SHARING USER ELSE X'0000'.     Y02669*
*   RETURN CODES = NONE                                         Y02669*
*                                                                     *
      EJECT
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*  EXTERNAL                                                     Y02669*
*   REFERENCES                                                  Y02669*
*   ROUTINES     = BRANCHES TO   IEE0803D   IEE0503D            Y02669*
*                                IED1303D   IEE7503D            Y02669*
*                                ISTCFF3D                       Y02674*
*                                                                     *
*   DATA AREAS   = PSEUDO-SACB TO BE USED THROUGHOUT SVC 34     Y02669*
*                    ORG   XAX                                  Y02669*
*                  XAXMASK  DS    C       REQUEST FLAGS         Y02669*
*                  XAXJOBS  EQU   X'80'   JOBS      (J)         Y02669*
*                  XAXJOBSL EQU   X'40'   JOBS,LIST (J,L)       Y02669*
*                  XAXTS    EQU   X'08'   TS                    Y02669*
*                  XAXTSL   EQU   X'04'   TS,LIST   (TS,L)      Y02669*
*                  XAXA     EQU   X'88'   A                     Y02669*
*                  XAXAL    EQU   X'44'   A,LIST    (A,L)       Y02669*
*                  XAXFREE  DS    C       RESERVED              Y02669*
*                  XAXUTME  DS    H       UTME=NNN VALUE        Y02669*
*                  XAXSACB  DS    F       ADDRESS OF SACB       Y02669*
*                                                                     *
*   CONTROL                                                     Y02669*
*    BLOCKS      = IEEXSA     R - INPUT - COMMAND BUFFER ADDR   Y02669*
*                             W - OUTPUT - BITS SET             Y02669*
*                  IHASUBIT   R                               @G17SPTJ*
*                  CVT        R                               @G17SPTJ*
*                                                                     *
         EJECT
*    TABLES      = A COMPARE TABLE IS USED TO  DETERMINE THE    Y02669*
*                  VALIDITY OF THE OPERANDS ON THE DISPLAY AND  Y02669*
*                  TRACK COMMANDS.                              Y02669*
*                  THE FORMAT OF EACH ENTRY IS THE SAME.        Y02669*
*                                                                     *
*                  DLEN   DC   AL1(L'XXX-1) - LENGTH OF         Y02669*
*                                             SPECIFIC OPERAND  Y02669*
*                  XXX    DC   C'YYYYY' - CHARACTER OPERAND     Y02669*
*                                         REPRESENTATION        Y02669*
*                                                                     *
*                  DLEN AND 'XXX' ARE USED IN AN EX OF A CLC.   Y02669*
*                  IF THE OPERANDS MATCH, A BRANCH IS TAKEN TO  Y02669*
*                  THE CODE WHICH IN TURN BRANCHES TO THE       Y02669*
*                  PROPER PROCESSOR OR HANDLES THE COMMAND.     Y02669*
*                  OTHERWISE, THE REGISTER USED TO ADDRESS THE  Y02669*
*                  TABLE IS INCREMENTED BY TWO + THE VALUE OF   Y02669*
*                  DLEN TO REACH THE NEXT TABLE ENTRY.          Y02669*
*                  THE VALID TRACK OPERANDS ARE PLACED AT THE   Y02669*
*                  END OF THE COMPARE TABLE AND ADDRESSING OF   Y02669*
*                  THE TABLE IS ADJUSTED WHEN VALIDITY          Y02669*
*                  CHECKING THE TRACK COMMAND.                  Y02669*
*                                                                     *
*                  A BRANCH TABLE IS USED IN CONNECTION WITH    Y02669*
*                  THE COMPARE TABLE. IT CONSISTS OF BRANCHES   Y02669*
*                  TO THE PROPER OPERAND PROCESSORS. AN INDEX   Y02669*
*                  REGISTER IS MAINTAINED THROUGHOUT THE SCAN   Y02669*
*                  OF THE COMPARE TABLE. THIS IS USED AT        Y02669*
*                  DBRANCH TO GIVE CONTROL TO THE INDICATED     Y02669*
*                  PROCESSOR VIA THE BRANCH TABLE.              Y02669*
*                                                                     *
*   MACROS       = MODID                                        Y02669*
*                  TIME                                         Y02669*
*                  WTO                                          Y02669*
*                  TPUT                                         Y02669*
*                                                                     *
*   ENQUEUE                                                     Y02669*
*     RESOURCES  = NONE                                         Y02669*
*                                                                     *
*   LOCKS HELD   = NONE                                         Y02669*
*                                                                     *
*   CHANGE LEVEL = Y02669                                       Y02669*
*                  Y02674                                       Y02674*
*                  Y02651                                       Y02651*
*                  YM1916                                       YM1916*
*                  Z40LPTA                                    @Z40LPTA*
*                  Z40BPSV                                    @Z40BPSV*
*                  G17SPTJ                                    @G17SPTJ*
*                  G33CPMR                                    @G33CPMR*
*                                                                     *
         EJECT
*   MESSAGES     = THE FOLLOWING MESSAGE IS CONTAINED IN AND    Y02669*
*                  ISSUED FROM THIS MODULE:                     Y02669*
*                                                                     *
*                   IEE136I  TIME=HH.MM.SS  DATE=YY.DDD         Y02669*
*                                                                     *
*                  THE FOLLOWING MESSAGES ARE CONTAINED IN      Y02669*
*                  AND ISSUED FROM IEE0503D:                    Y02669*
*                                                                     *
*                    MSGID    CODE    MESSAGE TEXT              Y02669*
*                    _____    ____    ____________              Y02669*
*                   IEE026I   X'1A'   NOT SUPPORTED           @G17SPTJ*
*                   IEE305I   X'05'   COMMAND INVALID           Y02669*
*                   IEE311I   X'11'   PARAMETER MISSING         Y02669*
*                   IEE341I   X'24'   'CLOCK' NOT ACTIVE        Y02669*
*                                                                     *
*   SYSGEN       = INCLUDED WITH SVC 34 IN LOAD MODULE          Y02669*
*                  IGC0003D, SYSGEN MACRO SGIEF441,             Y02669*
*                  DLIB ASOB3, TARGLIB LPALIB.                  Y02669*
*                                                                     *
*   SYSTEM                                                      Y02669*
*    LIBRARIES   = NONE                                         Y02669*
***********************************************************************
         EJECT
IEE3503D CSECT
***********************************************************************
*                                                                     *
*                         REGISTER    EQUATES                         *
*                                                                     *
***********************************************************************
R0       EQU   0                  WORK REGISTER
R1       EQU   1                  WORK REGISTER
R2       EQU   2                  INPUT - BASE REG FOR XSA
R3       EQU   3                  MODULE BASE REGISTER
R4       EQU   4                  USED FOR UCMI                  Y02669
R5       EQU   5                  PTR TO OPERAND BUFFER AND ASID Y02669
R6       EQU   6                  USED TO ADDRESS TABLE ENTRIES
*                                  AND THE SU BIT STRING       @G17SPTJ
R7       EQU   7    (UNUSED)      UNUSED                         Y02669
R8       EQU   8    (UNUSED)      UNUSED                         Y02669
R9       EQU   9                  WORK REGISTER
R10      EQU   10                 SU NUMBER STORAGE AREA       @G17SPTJ
R11      EQU   11                 ADDRESSES END OF TABLE
R12      EQU   12   (UNUSED)      UNUSED                         Y02669
R13      EQU   13   (UNUSED)      UNUSED                         Y02669
R14      EQU   14                 RETURN ADDRESS
R15      EQU   15                 USED FOR BAL AND BRANCH EXIT
         SPACE
***********************************************************************
*                                                                     *
*                      DISPLACEMENTS AND LENGTHS                      *
*                                                                     *
***********************************************************************
D0       EQU   0                  DISPLACEMENTS
D1       EQU   1
D2       EQU   2
D3       EQU   3
D4       EQU   4
D5       EQU   5
D6       EQU   6
D7       EQU   7
L0       EQU   0                  LENGTHS                        Y02669
L1       EQU   1
L2       EQU   2
L3       EQU   3
L4       EQU   4
L8       EQU   8
         EJECT
***********************************************************************
*                                                                     *
*                           VERB CODES                           Y02669
*                                                                     *
***********************************************************************
TR       EQU   X'C4'              VERB CODE FOR TRACK            Y02669
CK       EQU   X'2E'              VERB CODE FOR DISPLAY C,K      Y02669
DOMAIN   EQU   X'88'              VERB CODE FOR DISPLAY DMN    @Z40BPSV
MATRIX   EQU   X'32'              VERB CODE FOR DISPLAY MATRIX   Y02669
PFK      EQU   X'36'              VERB CODE FOR DISPLAY PFK      Y02669
CONSOLES EQU   X'3E'              VERB CODE FOR DISPLAY CONSOLES Y02669
REQUESTS EQU   X'42'              VERB CODE FOR DISPLAY REQUESTS Y02669
UNITS    EQU   X'52'              VERB CODE FOR DISPLAY UNITS    Y02669
D38      EQU   X'F3'              VERB CODE FOR DISPLAY 3850   @Z40LPTA
SLIP     EQU   X'A8'              VERB CODE FOR DISPLAY SLIP   @G17SPTJ
DUMP     EQU   X'B0'              VERB CODE FOR DISPLAY DUMP   @G33CPMR
         SPACE
***********************************************************************
*                                                                     *
*                           SU TEST BIT MASKS                  @G17SPTJ
*                                                                     *
***********************************************************************
MASK7    EQU   7                  SU 7 BIT MASK                @G17SPTJ
MASK17   EQU   17                 SU 17 BIT MASK               @G17SPTJ
MASK33   EQU   33                 SU 33 BIT MASK               @G33CPMR
         SPACE
***********************************************************************
*                                                                     *
*                           MASKS                                Y02669
*                                                                     *
***********************************************************************
FLAG     EQU   X'80'              FLAG - DENOTES TPUT RETRY       20030
XF0      EQU   X'F0'              MASK FOR OI                    Y02669
X0F      EQU   15                 MASK FOR STORING ALL BYTES     Y02669
*                                 IN A REGISTER                  Y02669
M7       EQU   7                  MASK FOR MOVING ADDRESS IN ICM Y02669
         SPACE
***********************************************************************
*                                                                     *
*                           ERROR CODES                          Y02669
*                                                                     *
***********************************************************************
ERR305   EQU   X'05'              MASK FOR IEE305I-INVALID CMD   Y02669
ERR311   EQU   X'0B'              MASK FOR IEE311I-PARM MISSING  Y02669
ERR341   EQU   X'24'              MASK FOR IEE341I-NOT ACTIVE    Y02669
ERR026   EQU   X'1A'              MASK FOR IEE026I-NOT         @G17SPTJ
*                                                   SUPPORTED  @G17SPTJ
         EJECT
***********************************************************************
*                                                                     *
*            ESTABLISH ADDRESSABILITY FOR MODULE AND XSA         Y02669
*               GET POINTER TO OPERANDS FROM XAL                 Y02669
*  ERR - IEE311I PARAMETER MISSING - IF THERE ARE NO OPERANDS    Y02669
*                                                                     *
***********************************************************************
         BALR  R3,0               ESTABLISH PROGRAM ADDRESSABILITY
         USING *,R3
         MODID BRANCH=YES                                        Y01886
         USING IEEXSA,R2          ADDRESS EXTENDED SAVE AREA
         SR    R5,R5              CLEAR THE REGISTER             Y02669
         ICM   R5,M7,XAL+1        GET 3 BYTE ADDR OF OPERANDS    Y02669
         BZ    DERR311            NONE, ERR - PARAMETER MISSING  Y02669
         SPACE  2
***********************************************************************
*                                                                     *
*                 IS THIS A DISPLAY OR TRACK COMMAND?            Y02669
*                                                                     *
***********************************************************************
         SPACE 2
         SR    R9,R9              CLEAR REGISTER FOR OPERAND LENGTH
         SR    R15,R15            USED TO INDEX BRANCH TABLE
         XC    XAX(L8),XAX        CLEAR DISPLAY/TRACK FIELDS     Y02669
         LA    R11,DENTAB         CALCULATE COMPARE TABLE END ADDRESS
         CLI   XAN,TR             IS THIS A TRACK COMMAND?       Y02669
         BNE   DISPLAY            NO, USE WHOLE COMPARE TABLE    Y02669
         EJECT
***********************************************************************
*                                                                     *
*    LOOP THROUGH COMPARE TABLE LOOKING FOR VALID TRACK OPERAND  Y02669
*                                                                     *
***********************************************************************
         SPACE 2
TRACK    EQU   *                                                 Y02669
         LA    R6,DTRTAB          OTHERWISE, USE ONLY VALID      Y02669
         USING DTRTAB,R6          TRACK COMPARES                 Y02669
TRLOOP   EQU   *                                                 Y02669
         IC    R9,DTRLEN          LENGTH OF COMPARE TABLE ENTRY  Y02669
         EX    R9,COMPARE         CHECK FOR MATCH                Y02669
         BE    TRTABLE(R15)       IF MATCH,PROCESS ACCORDINGLY   Y02669
         LA    R6,D2(R9,R6)       NO MATCH, GO TO NEXT ENTRY     Y02669
         LA    R15,D4(R15)        INCRE INDEX FOR BRANCH TABLE   Y02669
         CR    R6,R11             END OF COMPARE TABLE REACHED?  Y02669
         BL    TRLOOP             NO, GO BACK AND CHECK NEXT     Y02669
*                                 TABLE ENTRY                    Y02669
         DROP  R6                 NECESSARY TO RE-EST BASE REG   Y02669
         B     DERR305            YES, INVALID OPERAND-ERROR     Y02669
         SPACE 2
***********************************************************************
*                                                                     *
* LOOP THROUGH COMPARE TABLE LOOKING FOR VALID DISPLAY OPERAND   Y02669
*                                                                     *
***********************************************************************
         SPACE 2
DISPLAY  EQU   *                                                 Y02669
         LA    R6,DTAB            COMPARE TABLE BEGINNING ADDR   Y02669
         USING DTAB,R6            EST. ADDRESSABILITY TO TABLE
DLOOP    EQU   *                                                 Y02669
         IC    R9,DLEN            LENGTH OF COMPARE TABLE ENTRY
         EX    R9,COMPARE         CHECK FOR MATCH                Y02669
         BE    BRTABLE(R15)       IF MATCH, PROCESS ACCORDINGLY  Y02669
         LA    R6,D2(R9,R6)       NO MATCH, GO TO NEXT ENTRY
         LA    R15,D4(R15)        INCRE INDEX FOR BRANCH TABLE
         CR    R6,R11             END OF COMPARE TABLE REACHED?   M0467
         BL    DLOOP              NO, GO BACK AND CHECK NEXT      20030
*                                 TABLE ENTRY                     20030
         DROP  R6                 NECESSARY TO RE-EST BASE REG   Y02669
         B     DERR305            YES, INVALID OPERAND-ERROR     Y02669
         EJECT
***********************************************************************
*                                                                     *
*                       DISPLAY UNITS                              O111
*                                                                     *
***********************************************************************
DDUNIT   EQU   *                                                 Y02669
         MVI   XAN,UNITS          CHANGE VERB CODE IN XSA        Y02669
         B     DD7503D            BRANCH TO IEE7503D             Y02669
         SPACE  5
***********************************************************************
*                                                                     *
*                DISPLAY CONFIGURATION MATRIX                    Y02651
*                                                                     *
***********************************************************************
DDMATRIX EQU   *                                                 Y02651
         MVI   XAN,MATRIX         CHANGE VERB CODE IN XSA        Y02651
         B     DD7503D            BRANCH TO IEE7503D             Y02651
         SPACE   5
***********************************************************************
*                                                                     *
*            TRACK OR DISPLAY JOBS  (J)                          Y02669
*                             TS                                 Y02669
*                             A                                  Y02669
*                                                                     *
***********************************************************************
DDACTIVE EQU   *                                                 Y02669
         OI    XAXMASK,XAXJOBS+XAXTS  INDICATE ACTIVE OPTION     Y02669
         B     DD7503D             BRANCH TO IEE7503D            Y02669
DDJOBS   EQU   *                                                 Y02669
         OI    XAXMASK,XAXJOBS     INDICATE JOBS OPTION          Y02669
         B     DD7503D             BRANCH TO IEE7503D            Y02669
DDTS     EQU   *                                                 Y02669
         OI    XAXMASK,XAXTS       INDICATE TS OPTION            Y02669
         B     DD7503D             BRANCH TO IEE7503D            Y02669
         EJECT
***********************************************************************
*                                                                     *
*                       DISPLAY CONSOLES (C)                     YM1916
*                                                                     *
***********************************************************************
DDCONS   EQU   *                                                 Y02669
         MVI   XAN,CONSOLES       CHANGE VERB CODE IN XSA        Y02669
         B     DD7503D            BRANCH TO IEE7503D             Y02669
         SPACE   5
***********************************************************************
*                                                                     *
*                      DISPLAY REQUESTS                          Y02669
*                                                                     *
***********************************************************************
DDREQ    EQU   *                                                 Y02669
         MVI   XAN,REQUESTS       CHANGE VERB CODE IN XSA        Y02669
         B     DD7503D            BRANCH TO IEE7503D             Y02669
         SPACE   5
***********************************************************************
*                                                                     *
*                       DISPLAY 3850                           @Z40LPTA
*                                                                     *
***********************************************************************
DD3850   EQU   *                                               @Z40LPTA
         MVI   XAN,D38          RE-SET VERB INDICATOR          @Z40LPTA
         B     DD7503D          BRANCH TO IEE7503D             @Z40LPTA
         EJECT
***********************************************************************
*                                                                     *
*                       DISPLAY DOMAIN                         @Z40BPSV
*                                                                     *
***********************************************************************
DDDOMAIN EQU   *                                               @Z40BPSV
         MVI   XAN,DOMAIN       CHANGE VERB CODE IN XSA        @Z40BPSV
         LA    R10,MASK7        SET SU NUMBER FOR TESTING      @G17SPTJ
         B     TESTSU           BRANCH TO TEST SU NUMBER       @G17SPTJ
         SPACE 5
***********************************************************************
*                                                                     *
*                       DISPLAY SLIP                           @G17SPTJ
*                                                                     *
***********************************************************************
DDSLIP   EQU   *                                               @G17SPTJ
         MVI   XAN,SLIP         CHANGE VERB CODE IN XSA        @G17SPTJ
         LA    R10,MASK17       SET SU NUMBER FOR TESTING      @G17SPTJ
         B     TESTSU           BRANCH TO TEST SU NUMBER       @G17SPTJ
         SPACE 5
***********************************************************************
*                                                                     *
*                       DISPLAY DUMP                           @G33CPMR
*                                                                     *
***********************************************************************
DDDUMP   EQU   *                                               @G33CPMR
         MVI   XAN,DUMP         CHANGE VERB CODE IN XSA        @G33CPMR
         LA    R10,MASK33       SET SU NUMBER FOR TESTING      @G33CPMR
         B     TESTSU           BRANCH TO TEST SU NUMBER       @G33CPMR
         SPACE 5
***********************************************************************
*                                                                     *
*                       DISPLAY C,K                              Y02669
*                                                                     *
***********************************************************************
DDCK     EQU   *                                                 Y02669
         MVI   XAN,CK             CHANGE VERB CODE IN XSA        Y02669
DD7503D  EQU   *                                                 Y02669
         L     R15,VCON7503       GET ADDRESS OF IEE7503D        Y02669
         BR    R15                BRANCH TO CHECK ROUTING INFO   Y02669
         EJECT
***********************************************************************
*                                                                     *
*                       DISPLAY PFK                               21002
*                                                                     *
***********************************************************************
DDPFK    EQU   *                                                 Y02669
         MVI   XAN,PFK            CHANGE VERB CODE IN XSA        Y02669
         XC    XAS(L8),XAS        CLEAR DEFAULT ROUTING FIELD    Y02669
*                                 FOR ALL OTHER DISPLAY FUNCTIONSY02669
*                                 FIELD IS SET IN IEE7503D       Y02669
         L     R15,VCON0803       GET ADDR OF IEE0803D           Y02669
         BR    R15                BRANCH TO CSCB CREATION        Y02669
         SPACE   5
***********************************************************************
*                                                                     *
*                        DISPLAY TP,                              20002
*                                                                     *
***********************************************************************
DDTP     EQU   *                                                 Y02669
         L     R15,VCON1303       GET ADDR OF IED1303D           Y02669
         BR    R15                BRANCH TO D TP MODULE          Y02669
         SPACE  5
***********************************************************************
*                                                                     *
*                       DISPLAY NET,                             Y02674
*                                                                     *
***********************************************************************
DDNET    EQU   *                                                 Y02674
         L     R15,VCONCFF3       GET ADDR OF ISTCFF3D           Y02674
         BR    R15                BRANCH TO D NET MODULE         Y02674
         EJECT
***********************************************************************
*                                                                     *
*                      DISPLAY  TIME                                  *
*                                                                     *
***********************************************************************
DDTIME   EQU   *                                                 Y02669
         SR    R4,R4                 CLEAR FOR IC
         IC    R4,XAU                SAVE UCMI FOR WTO
         LH    R5,XAA                SAVE ASID FOR TPUT          Y02669
         TIME  DEC,ERRET=DERR341                                 Y02669
         MVC   XSA(DDLN),DDMSG       MOVE IN INCOMPLETE WTO MSG  Y02669
         STCM  R0,X0F,XTM            TIME IN XSA AFTER 'TIME='   Y02669
         UNPK  XTM(L8),XTM(L4)       UNPACK TIME
         MVC   XTM(L2),XTM+D1        LEFT ADJUST HOURS
         MVI   XTM+D2,C'.'           INSERT PERIOD
         MVC   XTM+D7(L1),XTM+D6     RIGHT ADJUST  ....
         MVC   XTM+D6(L1),XTM+D5     ....SECONDS
         MVI   XTM+D5,C'.'           INSERT PERIOD
         STCM  R1,X0F,XDATE          DATE IN XSA AFTER 'DATE= '  Y02669
         UNPK  XDATE(L8),XDATE(L4)   UNPACK DATE
         OI    XDATE+D7,XF0          SET LOW ORDER BYTE  EBCDIC  Y02669
         MVC   XDATE(L2),XDATE+D3    LEFT ADJUST YEAR
         MVI   XDATE+D2,C'.'         INSERT PERIOD
         MVC   XDATE+D3(L3),XDATE+D5 LEFT ADJUST DAY
         MVC   XDATE+D6(L2),DDBLNK   BLANK OUT ANY LEFT-OVERS
         LTR   R5,R5                 TERMINAL REQUEST             20030
         BNZ   XTPUT                 YES, PUT TIME MSG TO TERM    20030
         LR    R0,R4                 PUT UCMI INTO REG 0
         WTO   MF=(E,(R2))           WRITE TO OPERATOR - TIME AND DATE
         BR    R14                   RETURN TO SYSTEM
         EJECT
***********************************************************************
*                                                                     *
*      FOR TPUTS REGISTER 1 POINTS TO THE BEGINNING OF THE       Y02669
*      MESSAGE TEXT NOT TO THE PRECEDING WORD OF TEXT LENGTH     Y02669
*      AND MCS FLAGS.  THEREFORE THE HI ORDER BYTE OF THIS WORD  Y02669
*      IS USED AS THE RETRY INDICATOR. IT WILL BE SET TO X'80'   Y02669
*      THE FIRST TIME A RETRY IS ATTEMPTED AND CONTROL WILL BE   Y02669
*      RETURNED TO THE SYSTEM IF THE RETRY FAILS.                Y02669
*                                                                     *
***********************************************************************
         SPACE 2
XTPUT    EQU   *                                                  20030
         LH    R0,XAH                TEXT BY 4 FOR TPUT (MSFLAGS) 20030
         SH    R0,FOUR                                            20030
         LA    R1,XAI                ADDR OF MSG TEXT             20030
         TPUT  (1),(0),TJID=(5)      PUT MSG TO TERMINAL          20030
         CH    R15,ERRCODE           CHECK TPUT RETURN CODE       20030
         BCR   D7,R14                OK, RETURN TO SYSTEM         20030
         SPACE  1
         TM    XAH,FLAG              HAS A RETRY BEEN ATTEMPTED  Y02669
         BCR   D1,R14                YES, RETURN TO SYSTEM        20030
         OI    XAH,FLAG              SET FLAG TO DENOTE RETRY    Y02669
         B     XTPUT                 RETRY TPUT                   20030
         EJECT
***********************************************************************
*                                                                     *
*     FOR SU-DEPENDENT COMMANDS, EXECUTION PASSES HERE TO             *
*     ASCERTAIN IF THE SYSTEM SUPPORTS THE COMMAND IN QUESTION.       *
*     THE TEST IS MADE AGAINST A FOUR WORD SU BIT MASK IN THE         *
*     NUCLEUS WHICH IS POINTED TO BY THE CVTIHASU FIELD.  ON          *
*     ENTRY TO THIS SUBROUTINE, REGISTER 10 CONTAINS THE SU           *
*     NUMBER TO BE USED AS THE TEST AGAINST THE BIT MASK FIELD.       *
*     IF THE SU IS INSTALLED, PROCESSING CONTINUES.  OTHERWISE,       *
*     AN IEE026I ERROR MESSAGE IS ISSUED.                      @G33CPMR
*                                                                     *
***********************************************************************
         SPACE
TESTSU   EQU   *                                               @G17SPTJ
         L     R6,CVTPTR          ESTABLISH CVT ADDRESSABILITY @G17SPTJ
         USING CVT,R6                                          @G17SPTJ
         L     R6,CVTIHASU        ESTABLISH SU ADDRESSABILITY  @G17SPTJ
         USING SUBITS,R6                                       @G17SPTJ
         C     R10,SIXFOUR        IS SU NUMBER BELOW 64?       @G17SPTJ
         BL    BELOW64            YES, BRANCH AROUND REDUCTION @G17SPTJ
         LM    R4,R5,SUBYTE9      OBTAIN 2ND 2 SU WORDS        @G33CPMR
         S     R10,SIXFOUR        REDUCE SU NUMBER BY 64       @G17SPTJ
         B     SHIFT              BRANCH AROUND NEXT INSTR.    @G17SPTJ
BELOW64  EQU   *                                               @G17SPTJ
         LM    R4,R5,SUBYTE1      OBTAIN 1ST 2 SU WORDS        @G33CPMR
SHIFT    EQU   *                                               @G17SPTJ
         LTR   R10,R10            IS IT SU 0 OR SU 64?         @G17SPTJ
         BZ    SHIFTED            YES, AVOID SHIFTING          @G17SPTJ
LOOP     EQU   *                                               @G17SPTJ
         SLDL  R4,D1              SHIFT ACCORDING TO SU NUMBER @G17SPTJ
         BCT   R10,LOOP           CONTINUE SHIFTING UNTIL ZERO @G17SPTJ
SHIFTED  EQU   *                                               @G17SPTJ
         LTR   R4,R4              IS SU INSTALLED ON SYSTEM?   @G17SPTJ
         BM    DD7503D            YES, BRANCH TO IEE7503D      @G17SPTJ
         B     DERR026            NO, SU NOT INSTALLED - ERROR @G33CPMR
         EJECT
***********************************************************************
*                                                                     *
*                         ERROR EXITS                            Y02669
*                                                                     *
***********************************************************************
         SPACE 2
DERR026  EQU   *                                               @G33CPMR
         MVI   XAE,ERR026         ERROR CODE = 26 - NO SUPPORT @G33CPMR
         B     D0503D             BRANCH TO IEE0503D           @G33CPMR
DERR311  EQU   *                                                 Y02669
         MVI   XAE,ERR311         ERROR CODE=11 FOR PARM MISSING Y02669
         B     D0503D             BRANCH TO IEE0503D             Y02669
DERR305  EQU   *                                                 Y02669
         MVI   XAE,ERR305         ERROR CODE=5 FOR INVALID COMD  Y02669
         B     D0503D             BRANCH TO IEE0503D             Y02669
DERR341  EQU   *                                                 Y02669
         MVI   XAE,ERR341         ERROR CODE=36 FOR NOT ACTIVE   Y02669
         MVC   XAV(L8),CLOCKERR   GET KEYWORD 'CLOCK'            Y02669
D0503D   EQU   *                                                 Y02669
         L     R15,VCON0503       GET ADDR OF IEE0503D           Y02669
         BR    R15                BRANCH TO MESSAGE MODULE       Y02669
         EJECT
***********************************************************************
*                                                                     *
*                         CONSTANTS                              Y02669
*                                                                     *
***********************************************************************
         SPACE   5
FOUR     DC    H'4'               USED IN SH INSTRUCTION         Y02669
SIXFOUR  DC    F'00000064'        COMPARAND OF 64              @G17SPTJ
ERRCODE  DC    X'0014'            TPUT RETURN CODE - MSG DID NOT  20030
*                                 GO DUE TO AN OUTSTANDING TPUT   20030
DDBLNK   DC    C'        '        USED FOR PADDING
CLOCKERR DC    C'CLOCK   '        USED FOR D T ERROR MESSAGE     Y02669
VCON0503 DC    V(IEE0503D)        ADDRESS OF MESSAGE MODULE      Y02669
VCON0803 DC    V(IEE0803D)        ADDRESS OF CSCB CREATION MOD   Y02669
VCON1303 DC    V(IED1303D)        ADDRESS OF D TP MODULE         Y02669
VCON7503 DC    V(IEE7503D)        ADDRESS OF SDS MODULE          Y02669
VCONCFF3 DC    V(ISTCFF3D)        ADDRESS OF NET MODULE          Y02674
         SPACE
DDMSG    WTO   'IEE136I  TIME=         DATE=        ',MF=L,            X
               DESC=(5),MCSFLAG=(REG0,RESP)                      Y02669
DDLN     EQU   *-DDMSG
         SPACE
COMPARE  CLC   D1(L0,R6),D0(R5)   LOOK FOR MATCH IN TABLE        Y02669
         SPACE   5
***********************************************************************
*                                                                     *
*                        PATCH AREA                              Y02669
*                                                                     *
***********************************************************************
         SPACE   2
PATCH    DC      CL50'*****PATCH AREA - IEE3503D******'          Y02669
         EJECT
***********************************************************************
*                                                                     *
*                         BRANCH  TABLE                          Y02669
*                                                                     *
*        THIS TABLE MUST BE IN A ONE-TO-ONE CORRESPONDENCE WITH  Y02669
*        THE 'COMPARE TABLE'                                     Y02669
*        THE BRANCH TABLE FOR THE TRACK COMMAND STARTS WITH      Y02669
*        TRTABLE                                                 Y02669
*                                                                     *
***********************************************************************
         SPACE   5
BRTABLE  DS    0H                                                Y02669
         B     DDTIME             BRANCH TO PROCESS D T          Y02669
         B     DDREQ              BRANCH TO PROCESS D R          Y02669
         B     DDREQ              BRANCH TO PROCESS D R,LIST     Y02669
         B     DDUNIT             BRANCH TO PROCESS D U          Y02669
         B     DDUNIT             BRANCH TO PROCESS D U,         Y02669
         B     DDMATRIX           BRANCH TO PROCESS D M          Y02651
         B     DDMATRIX           BRANCH TO PROCESS D M=         Y02651
         B     DDMATRIX           BRANCH TO PROCESS D M,         Y02651
         B     DDTP               BRANCH TO PROCESS DISPLAY TP,  Y02669
         B     DDNET              BRANCH TO PROCESS DISPLAY NET, Y02674
         B     DDCK               BRANCH TO PROCESS DISPLAY C,K  Y02669
         B     DDCK               BRANCH TO PROCESS DISPLAY C,K, Y02669
         B     DDCONS             BRANCH TO PROCESS D CONSOLES   YM1916
         B     DDCONS             BRANCH TO PROCESS D CONSOLES,  YM1916
         B     DDCONS             BRANCH TO PROCESS D C          YM1916
         B     DDCONS             BRANCH TO PROCESS D C,         YM1916
         B     DDPFK              BRANCH TO PROCESS D PFK        Y02669
         B     DD3850             BRANCH TO PROCESS D 3850     @Z40LPTA
         B     DD3850             BRANCH TO PROCESS D 3850,    @Z40LPTA
         B     DDDOMAIN           BRANCH TO PROCESS D DMN      @Z40BPSV
         B     DDDOMAIN           BRANCH TO PROCESS D DMN=     @Z40BPSV
         B     DDDOMAIN           BRANCH TO PROCESS D DMN,     @Z40BPSV
         B     DDSLIP             BRANCH TO PROCESS D SLIP     @G17SPTJ
         B     DDSLIP             BRANCH TO PROCESS D SLIP,    @G17SPTJ
         B     DDSLIP             BRANCH TO PROCESS D SLIP=    @G17SPTJ
         B     DDDUMP             BRANCH TO PROCESS D DUMP     @G33CPMR
         B     DDDUMP             BRANCH TO PROCESS D DUMP,    @G33CPMR
         B     DDDUMP             BRANCH TO PROCESS D D        @G33CPMR
         B     DDDUMP             BRANCH TO PROCESS D D,       @G33CPMR
TRTABLE  DS    0H                                                Y02669
         B     DDACTIVE           BRANCH TO PROCESS D OR TR A    Y02669
         B     DDACTIVE           BRANCH TO PROCESS D OR TR A,   Y02669
         B     DDJOBS             BRANCH TO PROCESS D OR TR J    Y02669
         B     DDJOBS             BRANCH TO PROCESS D OR TR J,   Y02669
         B     DDJOBS             BRANCH TO PROCESS D OR TR JOBS Y02669
         B     DDJOBS             BRANCH TO PROCESS D OR TR JOBS,Y02669
         B     DDTS               BRANCH TO PROCESS D OR TR TS   Y02669
         B     DDTS               BRANCH TO PROCESS D OR TR TS,  Y02669
         EJECT
***********************************************************************
*   NOTE:  THE FOLLOWING TABLE MUST BE THE LAST ITEM IN THIS     Y02669
*          MODULE SINCE A USING FOR THE TABLE IS ISSUED ON       Y02669
*          REGISTER 6 AND THUS CAN REESTABLISH THE MODULE'S      Y02669
*          BASE REGISTER FOR ANY CODE WHICH FOLLOWS THIS TABLE.  Y02669
*                                                                     *
*                        COMPARE TABLE                           Y02669
*                                                                     *
*           THIS TABLE MUST BE IN A ONE-TO-ONE CORRESPONDENCE    Y02669
*           WITH THE BRANCH TABLE                                Y02669
*           TRACK COMMANDS START WITH DTRTAB                     Y02669
***********************************************************************
DTAB     EQU   *
DLEN     EQU   *
         DC    AL1(L'DTIME-1)     LENGTH OF DISPLAY OPERAND       20030
DTIME    DC    C'T '              DISPLAY OPERAND                 20030
         DC    AL1(L'DREQ-1)            *                        Y02669
DREQ     DC    C'R '                    *                        Y02669
         DC    AL1(L'DREQA-1)           *                        Y02669
DREQA    DC    C'R,'                    *                        Y02669
         DC    AL1(L'DUNIT-1)           *                        Y02669
DUNIT    DC    C'U '                    *                        Y02669
         DC    AL1(L'DUNITA-1)          *                        Y02669
DUNITA   DC    C'U,'                    *                        Y02669
         DC    AL1(L'DMATRIX-1)         *                        Y02651
DMATRIX  DC    C'M '                    *                        Y02651
         DC    AL1(L'DMATRIXA-1)        *                        Y02651
DMATRIXA DC    C'M='                    *                        Y02651
         DC    AL1(L'DMATRIXB-1)        *                        Y02651
DMATRIXB DC    C'M,'                    *                        Y02651
         DC    AL1(L'DTP-1)             *                        Y02669
DTP      DC    C'TP,'                   *                        Y02669
         DC    AL1(L'DNET-1)            *                        Y02674
DNET     DC    C'NET,'                  *                        Y02674
         DC    AL1(L'DCK-1)             *                         21002
DCK      DC    C'C,K '                  *                         21002
         DC    AL1(L'DCKA-1)            *                         21002
DCKA     DC    C'C,K,'                  *                         21002
         DC    AL1(L'DCONSOLE-1)        *                         20030
DCONSOLE DC    C'CONSOLES '             *                         20030
         DC    AL1(L'DCONSOLA-1)        *                         21002
DCONSOLA DC    C'CONSOLES,'             *                         21002
         DC    AL1(L'DC-1)              *                        YM1916
DC       DC    C'C '                    *                        YM1916
         DC    AL1(L'DCA-1)             *                        YM1916
DCA      DC    C'C,'                    *                        YM1916
         DC    AL1(L'DPFK-1)            *                        Y02669
DPFK     DC    C'PFK '                  *                        Y02669
         DC    AL1(L'D3850-1)           *                      @Z40LPTA
D3850    DC    C'3850 '                 *                      @Z40LPTA
         DC    AL1(L'D3850C-1)          *                      @Z40LPTA
D3850C   DC    C'3850,'                 *                      @Z40LPTA
         DC    AL1(L'DDOMAIN-1)         *                      @Z40BPSV
DDOMAIN  DC    C'DMN '                  *                      @Z40BPSV
         DC    AL1(L'DDOMAINE-1)        *                      @Z40BPSV
DDOMAINE DC    C'DMN='                  *                      @Z40BPSV
         DC    AL1(L'DDOMAINC-1)        *                      @Z40BPSV
DDOMAINC DC    C'DMN,'                  *                      @Z40BPSV
         DC    AL1(L'DSLIP-1)           *                      @G17SPTJ
DSLIP    DC    C'SLIP '                 *                      @G17SPTJ
         DC    AL1(L'DSLIPA-1)          *                      @G17SPTJ
DSLIPA   DC    C'SLIP,'                 *                      @G17SPTJ
         DC    AL1(L'DSLIPA-1)          *                      @G17SPTJ
DSLIPB   DC    C'SLIP='                 *                      @G17SPTJ
         DC    AL1(L'DDUMP-1)           *                      @G33CPMR
DDUMP    DC    C'DUMP '                 *                      @G33CPMR
         DC    AL1(L'DDUMPA-1)          *                      @G33CPMR
DDUMPA   DC    C'DUMP,'                 *                      @G33CPMR
         DC    AL1(L'DD-1)              *                      @G33CPMR
DD       DC    C'D '                    *                      @G33CPMR
         DC    AL1(L'DDA-1)             *                      @G33CPMR
DDA      DC    C'D,'                    *                      @G33CPMR
         EJECT
DTRTAB   EQU   *                                                 Y02669
DTRLEN   EQU   *                                                 Y02669
         DC    AL1(L'DACTIVE-1)   LENGTH OF DISPLAY/TRACK OPERANDY02669
DACTIVE  DC    C'A '              DISPLAY/TRACK OPERAND          Y02669
         DC    AL1(L'DACTIVEA-1)        *                        Y02669
DACTIVEA DC    C'A,'                    *                        Y02669
         DC    AL1(L'DJ-1)              *                        Y02669
DJ       DC    C'J '                    *                        Y02669
         DC    AL1(L'DJA-1)             *                        Y02669
DJA      DC    C'J,'                    *                        Y02669
         DC    AL1(L'DJOBS-1)           *                        Y02669
DJOBS    DC    C'JOBS '                 *                        Y02669
         DC    AL1(L'DJOBSA-1)          *                        Y02669
DJOBSA   DC    C'JOBS,'                 *                        Y02669
         DC    AL1(L'DTS-1)             *                        Y02669
DTS      DC    C'TS '                   *                        Y02669
         DC    AL1(L'DTSA-1)            *                        Y02669
DTSA     DC    C'TS,'                   *                        Y02669
DENTAB   EQU   *                                                 Y02669
         EJECT
IEEXSA   DSECT
         IEEXSA
XTM      EQU   XAT-D2                                            Y02669
XDATE    EQU   XAS                                               Y02669
         EJECT
         IHASUBIT
         EJECT
CVT      DSECT
         CVT
         END
