          TITLE 'IEE5503D - STOPMN / STOPTR COMMAND PROCESSOR'
***********************************************************************
*                                                                     *
* MODULE NAME      =   IEE5503D                                 Y02669*
*                                                                     *
* DESCRIPTIVE NAME =   STOPMN AND STOPTR COMMAND PROCESSOR      Y02669*
*                                                                     *
* COPYRIGHT        =   N/A                                      Y02669*
*                                                                     *
* STATUS           =   OS/VS2 RELEASE 037                     @ZA30198*
*                                                                     *
* FUNCTION         =   THIS MODULE DETERMINES AND PROCESSES     Y02669*
*                      THE STOPMN COMMAND OPERANDS:             Y02669*
*                      JOBNAMES; STATUS; DSNAME; SPACE; SESS.   Y02669*
*                                                                     *
*                      AND THE STOPTR COMMAND OPERANDS:         Y02669*
*                      JOBS (J); TS; AND A.                     Y02669*
*                                                                     *
*  OPERATION       =   THE STOPMN COMMAND OPERANDS RECOGNIZED   Y02669*
*                      BY IEE5503D AND THEIR VALIDITY BY SOURCE Y02669*
*                      ARE:                                     Y02669*
*                                                                     *
*                                    INPUT   TIME     SUBSYSTEM@Y02668P
*                                    STREAM  SHARING  CONSOLE  @Y02668P
*                                    OR      (OPERATOR  (JES3) @Y02668P
*                                    CONSOLE MODE)             @Y02668P
*                                                              @Y02668P
*                        JOBNAMES(,T)  VALID   VALID   INVALID @Y02668P
*                        STATUS        VALID   VALID   INVALID @Y02668P
*                        SESS(,T)      VALID   VALID   INVALID @Y02668P
*                        SPACE         VALID  INVALID   VALID  @Y02668P
*                        DSNAME        VALID  INVALID   VALID  @Y02668P
*                                                                     *
*                      FOR STOPTR THE VALID OPERANDS BY         Y02669*
*                      SOURCE ARE:                              Y02669*
*                                                                     *
*                         OPERAND    OPERATOR     TIME SHARING  Y02669*
*                                     CONSOLE      (OPERATOR    Y02669*
*                                ISSUED INTERNALLY    MODE)     Y02669*
*                        ---------   -------      ------------  Y02669*
*                         JOBS (J)    VALID         INVALID     Y02669*
*                         TS          VALID         INVALID     Y02669*
*                         A           VALID         INVALID     Y02669*
*                                                                     *
*                      IEE5503D, DEPENDING ON THE COMMAND       Y02669*
*                      SOURCE, TERMINATES MONITOR AND TRACK     Y02669*
*                      REQUESTS BY CLEARING BITS PREVIOUSLY     Y02669*
*                      SET IN THE APPROPIATE MONITOR AND        Y02669*
*                      TRACK CONTROL BLOCKS AND COMMON          Y02669*
*                      DATA AREAS:                              Y02669*
*                                                                     *
         EJECT
*                      FOR STOPMN COMMAND SOURCE:               Y02669*
*                                                                     *
*                      SOURCE       CONTROL BLOCK  COMMON AREA  Y02669*
*                      -------      -------------  -----------  Y02669*
*                      INTERNALLY ISSUED UCME          BASEA    Y02669*
*                      INPUT STREAM      UCME          BASEA    Y02669*
*                      OPERATOR CONSOLE  UCME          BASEA    Y02669*
*                      TIME SHARING                             Y02669*
*                       (OPERATOR MODE)  MQE           BASEA    Y02669*
*                                                                     *
*                      AND STOPTR COMMAND SOURCE:               Y02669*
*                                                                     *
*                      SOURCE             CONTROL BLOCK         Y02669*
*                      -------            -------------         Y02669*
*                      INTERNALLY ISSUED  SACB (SCREEN AREA     Y02669*
*                      OPERATOR CONSOLE    CONTROL BLOCK)       Y02669*
*                                                                     *
*                      IEE5503D INTERFACES WITH COMMUNICATIONS  Y02669*
*                      TASK, IEAVMNTR (MONITOR QUEUE MANAGER),  Y02669*
*                      VIA THE MONITOR PARM LIST (IEZMPL)       Y02669*
*                      TO ADJUST THE APPROPIATE FIELDS IN UCME, Y02669*
*                      BASEA, AND MQE (MONITOR QUEUE ELEMENT)   Y02669*
*                      IN RESPONSE TO A STOP MONITOR JOBNAMES;  Y02669*
*                      STATUS; OR SESS COMMAND. THE VECTOR FOR  Y02669*
*                      IEAVMNTR IS FOUND IN THE UCM (UCMMNTR).  Y02669*
*                                                                     *
*                      THE TIME FIELD IN BASEA IS CLEARED, IN   Y02669*
*                      THIS MODULE, IF THERE ARE NO MORE ACTIVE Y02669*
*                      MONITOR JOBNAMES OR SESS REQUESTS WITH   Y02669*
*                      TIME OPTION.                             Y02669*
*                                                                     *
*                      THE FIELDS IN BASEA REPRESENTING MONITOR Y02669*
*                      SPACE AND DSNAME ARE ALSO CLEARED BY     Y02669*
*                      THIS MODULE.                             Y02669*
*                                                                     *
*                      THE FIELDS IN THE SACB, IEERDCM (SCREEN  Y02669*
*                      AREA CONTROL BLOCK), ARE ADJUSTED IN     Y02669*
*                      THIS MODULE IN RESPONSE TO A STOP TRACK  Y02669*
*                      COMMAND. (SEE EXITS)                     Y02669*
*                                                                     *
* NOTES                IF THE STOPMN COMMAND IS ENTERED THROUGH Y02669*
*                      THE INPUT STREAM OR ISSUED INTERNALLY,   Y02669*
*                      THE CONSOLE ID (XAU) WILL BE X'00'.      Y02669*
*                      THE MONITORING WILL OCCUR ON THE MASTER  Y02669*
*                      CONSOLE.                                 Y02669*
*
*                      WHEN THE ISSUING CONSOLE IS PART OF A    Y02669*
*                      COMPOSITE CONSOLE, XAU WILL CONTAIN THE  Y02669*
*                      CONSOLE ID OF THE INPUT DEVICE.          Y02669*
*                                                                     *
*   CHARACTER/CODE                                              Y02669*
*    DEPENDENCIES  =   ANY CHARACTER CODE OTHER THAN EBCDIC     Y02669*
*                      WILL REQUIRE REASSEMBLY OF THIS MODULE.  Y02669*
*                                                                     *
*    DEPENDENCIES  =   SVC 100 WILL NOT PERMIT ANY TIME         Y02669*
*                      SHARING USERS TO ISSUE STOPTR; OR STOPMN Y02669*
*                      DSNAME; OR SPACE.                        Y02669*
*                                                                     *
*                      IEE0403D TO RECOGNIZE THE STOPMN         Y02669*
*                      COMMAND AND ROUTE THE OPERAND            Y02669*
*                      INFORMATION IN XSA.                      Y02669*
*                                                                     *
*                      MONITOR QUEUE MANGER (IEAVMNTR) TO       Y02669*
*                      ADJUST THE PROPER BITS IN THE UCME,      Y02669*
*                      BASEA, AND MQE FOR STOP MONITOR          Y02669*
*                      JOBNAMES; STATUS; AND SESS.              Y02669*
*                                                                     *
*                      THE STAE EXIT FOR SVC 34 SHOULD EXPECT   Y02669*
*                      AN ABEND 077 FROM COMMUNICATIONS TASK    Y02669*
*                      (IEAVMNTR) IF THE IEZMPL PARM LIST       Y02669*
*                      CONTAINS INVALID INFORMATION.            Y02669*
*                                                                     *
*                      IEEBASEA, IEECUCM AND IEEXSA WILL BE     Y02669*
*                      UPDATED TO REFLECT THE NEW FIELDS.       Y02669*
*                                                                     *
*                      A MAPPING MACRO FOR IEZMPL MUST BE       Y02669*
*                      PROVIDED.                                Y02669*
*                                                                     *
*                      THE UCME UCMDISPK BIT WILL BE CHECKED   @Y02668P
*                      IF THE BIT IS ON THE CONSOLE IS A       @Y02668P
*                      SUBSYSTEM CONSOLE. PM JOBNAMES,STATUS   @Y02668P
*                      AND SESS ARE INVALID COMMANDS ON A      @Y02668P
*                      SUBSYSTEM CONSOLE                       @Y02668P
*                                                                     *
*                      IEE7503D TO RECOGNIZE THE STOPTR         Y02669*
*                      COMMAND AND SEND THE ROUTING             Y02669*
*                      INFORMATION AND THE ADDRESS OF THE       Y02669*
*                      APPROPIATE SACB IN THE XSA.              Y20669*
*                                                                     *
*                      IEE7503D WILL REJECT THE STORTR         @Y02668P
*                      COMMAND IF IT IS ISSUED TO OR ON        @Y02668P
*                      A SUBSYSTEM CONSOLE                     @Y02668P
*                                                                     *
*                      THE READER/INTERPRETER WILL REJECT AS    Y02669*
*                      INVALID THE STOPTR (PT) COMMAND FROM THE Y02669*
*                      THE INPUT STREAM.                        Y02669*
*                                                                     *
*                      TWO NEW WORDS WILL BE ADDED TO THE SACB  Y02669*
*                      (SCREEN AREA CONTROL BLOCK) IEERDCM TO   Y02669*
*                      ACCOMMODATE THE OPTIONS FIELDS FOR THE   Y02669*
*                      TRACK COMMAND OPERANDS AND A TRACK ECB.  Y02669*
*                                                                     *
*    RESTRICTIONS  =   TIME SHARING USERS ARE NOT PERMITTED     Y02669*
*                      TO ISSUE THE STOPTR COMMAND OR STOPMN    Y02669*
*                      DSNAME; OR SPACE. (SEE DEPENDENCIES).    Y02669*
*                                                                     *
*                      THE STOPTR COMMAND IS INVALID FROM THE   Y02669*
*                      INPUT STREAM.                            Y02669*
*                                                                     *
*    REGISTER/                                                  Y02669*
*     CONVENTIONS  =   IEE5503D WILL SUPPLY AN 18 WORD          Y02669*
*                      SAVE AREA POINTED TO BY REGISTER 13      Y02669*
*                      WHEN CALLING IEAVMNTR FOR STOPMN.        Y02669*
*                      THIS SAVE AREA IS PART OF THE XSA.       Y02669*
*                                                                     *
*    PATCH-LABEL   =   PATCH                                    Y02669*
*                                                                     *
* MODULE TYPE      =   CSECT                                    Y02669*
*                                                                     *
*    PROCESSOR     =   N/A                                      Y02669*
*                                                                     *
*    MODULE SIZE   =   X'318'                                 @ZA30198*
*                                                                     *
*    ATTRIBUTES    =   REENTRANT, KEY 0, SUPERVISOR MODE,       Y02669*
*                      PAGED LPA                                Y02669*
*                                                                     *
* ENTRY POINT      =   IEE5503D FROM IEE0403D                   Y02669*
*    PURPOSE       =   TO RESET BITS AS THE RESULT OF A         Y02669*
*                      STOPMN COMMAND.                          Y02669*
*    LINKAGE       =   BRANCH                                   Y02669*
*    INPUT DATA    =   REGISTER TWO ADDRESSES THE XSA           Y02669*
*                      XAL POINTS TO 1ST OPERAND IN BUFFER      Y02669*
*                      XAU CONTAINS A CONSOLE UCMI (X'00' -     Y02669*
*                      ISSUED INTERNALLY OR THRU INPUT STREAM)  Y02669*
*                      XAA CONTAINS AN ASID (X'0000' = NON-TS)  Y02669*
*                      XAR POINTS TO LENGTH FIELD OF BUFFER     Y02669*
*                      XAN CONTAINS VERB CODE X'38' - STOPMN    Y02669*
*                      XAV CONTAINS COMMAND VERB                Y02669*
*                                                                     *
* ENTRY POINT      =   IEE5503D FROM IEE7503D                   Y02669*
*    PURPOSE       =   TO RESET BITS AS THE RESULT OF A         Y02669*
*                      STOPTR COMMAND.                          Y02669*
*    LINKAGE       =   BRANCH                                   Y02669*
*    INPUT DATA    =   REGISTER TWO ADDRESSES THE XSA           Y02669*
*                      XAL POINTS TO 1ST OPERAND IN BUFFER      Y02669*
*                      XAU CONTAINS A CONSOLE IDENTIFIER        Y02669*
*                         (X'00' INDICATES INTERNALLY ISSUED)   Y02669*
*                      XAA CONTAINS X'0000'(INVALID FROM TS)    Y02669*
*                      XAS CONTAINS L=CCA INFORMATION           Y02669*
*                          XASCID - CONSOLE ID OR XAU           Y02669*
*                          XASDID - AREA ID IF CRT TARGET       Y02669*
*                          XASSDS - ON IF 'L=' ON COMMAND       Y02669*
*                          XASSDS5 - ON LOCKS HELD              Y02651*
*                      XAX CONTAINS X'00' EXCEPT THAT           Y02669*
*                          XAXSACB CONTAINS ADDRESS OF SACB     Y02669*
*                      XAR POINTS TO LENGTH FIELD OF BUFFER     Y02669*
*                      XAV CONTAINS COMMAND VERB                Y02669*
*                      XAN CONTAINS VERB CODE X'C8' - STOPTR    Y02669*
*                                                                     *
         EJECT
* REGISTERS USAGE  =                                                  *
*    SAVED         =   REGISTER 14 SAVED IN XSA (XAS) FOR       Y02669*
*                        STOPMN AT COMINTER WHEN CALLING        Y02669*
*                        IEAVMNTR                               Y02669*
*                      REGISTERS 11 - 14 SAVED IN XSA           Y02651*
*                        (XASAVLOC) BEFORE ISSUING SETLOCK      Y02651*
*                        FOR STOPTR AT TRUPDATE & TRD0503D      Y02651*
*                                                                     *
*    CONTAINED     =   R0    0     USED FOR NIL MACRO           Y02651*
*                      R1    1     POINTER TO MONITOR PARM LIST Y02669*
*                                  USED FOR THE NIL MACRO       Y02651*
*                      R2    2     INPUT-BASE REG FOR XSA       Y02669*
*                      R3    3     BASE REG FOR CVT             Y02669*
*                      R4    4     POINTER TO OPERAND BUFFER    Y02669*
*                      R5    5     WORK REG, AND FOR NIL MACRO  Y02651*
*                      R6    6     ADDRESS COMPARE TABLE END    Y02669*
*                      R7    7     ADDRESS COMPARE TABLE ENTRY  Y02669*
*                      R8    8     BASE REG FOR UCM             Y02669*
*                      R9    9     CONTAINS ASID OR UCMI        Y02669*
*                      R10   10    BASE REG FOR BASEA           Y02669*
*                      R11   11    BASE REG FOR SACB            Y02669*
*                      R12   12    BASE REG FOR MODULE          Y02669*
*                      R13   13    SAVE AREA POINTER FOR STOPMN Y02669*
*                      R14   14    BALR AND RETURN REG          Y02669*
*                      R15   15    BALR AND BRANCH REG          Y02669*
*                                                                     *
*    RESTORE       =   REGISTER 14 RESTORED AT LABEL RETURN     Y02669*
*                        FOR STOPMN ONLY WHEN RETURNING FROM    Y02669*
*                        IEAVMNTR (NOT SAVED FOR STOPTR)        Y02669*
*                      REGISTERS 11 - 14 RESTORED FOR STOPTR    Y02651*
*                        AFTER ISSUING SETLOCK AT TRUPDATE &    Y02651*
*                        TRD0503D                               Y02651*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
**EXIT - NORMAL    =   RETURN VIA BR R14 FROM DDSNAME           Y02669*
*    CONDITIONS    =   PM DSNAME REQUEST REGISTERED IN BASEA    Y02669*
*    OUTPUT DATA   =   BIT CLEARED IN BASEA                     Y02669*
*                        BAMONITR - BADSN X'80'                 Y02669*
*    RETURN CODES  =   NONE                                     Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
**EXIT - NORMAL    =   RETURN VIA BR R14 FROM DDSPACE           Y02669*
*    CONDITIONS    =   PM SPACE REQUEST REGISTERED IN BASEA     Y02669*
*    OUTPUT DATA   =   BIT CLEARED IN BASEA                     Y02669*
*                        BAMONITR - BADSPACE X'40'              Y02669*
*    RETURN CODES  =   NONE                                     Y02669*
*                                                                     *
         EJECT
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
**EXIT - NORMAL    =   BALR 14,15 TO IEAVMNTR FROM COMINTER     Y02669*
*    CONDITIONS    =   TO HAVE PM JOBNAMES; SESS; OR STATUS     Y02669*
*                      REQUEST REGISTERED BY COMM TASK          Y02669*
*    OUTPUT DATA   =   FIELDS SET IN MONITOR PARM LIST          Y02669*
*                      (IEZMPL), ORG'ED ON XAX, THE ADDRESS OF  Y02669*
*                      WHICH IS STORED IN XAR, WHICH IN TURN IS Y02669*
*                      POINTED TO BE REGISTER 1                 Y02669*
*                      (SEE DATA AREAS)                         Y02669*
*                      REG 13 POINTS TO 18 WORD SAVE AREA       Y02669*
*    RETURN CODES  =   NONE (HOWEVER, IEAVMNTR WILL ABEND ITS   Y02669*
*                      CALLER WITH AN '077' IF THE PARM LIST    Y02669*
*                      CONTAINS INVALID INFORMATION)            Y02669*
*                      (SEE DEPENDENCIES)                       Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
**EXIT - NORMAL    =   RETURN VIA BR R14 FROM RETURN            Y02669*
*    CONDITIONS    =   PM JOBNAMES; SESS; OR STATUS REQUEST     Y02669*
*                      REGISTERED BY COMM TASK                  Y02669*
*    OUTPUT DATA   =   FIELDS ALTERED IN BASEA AND UCME AND     Y02669*
*                      MQE VIA COMM TASK.                       Y02669*
*                      IF THERE ARE NO ACTIVE MN JOBNAMES OR    Y02669*
*                      SESS REMAINING, THE TIME BIT IN BASEA    Y02669*
*                      IS CLEARED BY THIS MODULE.               Y02669*
*                        MSBTN - MSTN  X'02'                    Y02669*
*    RETURN CODES  =   NONE                                     Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
**EXIT - NORMAL    =   RETURN VIA BR R14 FROM TRUPDATE          Y02669*
*    CONDITIONS    =   FOR STOPTR WHEN THERE IS STILL AN ACTIVE Y02669*
*                      OPTION TO TRACK IN THAT AREA (L=A)       Y02669*
*    OUTPUT DATA   =   SACB UPDATED AND LOCKS THAT WERE         Y02669*
*                        SET IN IEE7503D ARE RELEASED           Y02651*
*                      REG 2 POINTS TO XSA WITH:                Y02651*
*                        XASSDS5 - OFF - LOCKS RELEASED         Y02651*
*    RETURN CODES  =   NONE                                     Y02669*
*
         EJECT
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - NORMAL    =   BRANCH TO IEE6703D FROM D6703D           Y02669*
*    CONDITIONS    =   FOR THE STOPTR COMMAND ONLY IF THE       Y02669*
*                      SCREEN NEEDS TO BE CLEARED (I.E.         Y02669*
*                      A STOPTR A HAS BEEN ISSUED OR AS THE     Y02669*
*                      RESULT OF THIS STOPTR THERE ARE NO       Y02669*
*                      ACTIVE OPTIONS TO TRACK IN THIS AREA.    Y02669*
*    OUTPUT DATA   =   REGISTER TWO ADDRESSES THE XSA           Y02669*
*                        XAL POINTS TO 1ST OPERAND IN BUFFER    Y02669*
*                        XAU CONTAINS A CONSOLE IDENTIFIER      Y02669*
*                           (X'00' INDICATES INTERNALLY ISSUED) Y02669*
*                        XAA CONTAINS X'0000'(INVALID FROM TS)  Y02669*
*                        XAS CONTAINS L=CCA INFORMATION         Y02669*
*                           XASCID - CONSOLE ID OR XAU          Y02669*
*                           XASDID = AREA ID                    Y02669*
*                           XASSDS = ON IF 'L=' ON COMMAND      Y02669*
*                           XASSDS5 = ON - LOCKS HELD           Y02651*
*                        XAR POINTS TO LENGTH FIELD OF BUFFER   Y02669*
*                        XAN CONTAINS VERB CODE X'C8' - STOPTR  Y02669*
*                        XAX CONTAINS ZERO EXCEPT FOR XAXSACB   Y02669*
*                           BITS CLEARED IN SACB FOR THE        Y02669*
*                           ASSOCIATED CONSOLE:                 Y02669*
*                        DCMATJOB & DCMATJBL = X'C0' FOR JOBS(J)Y02669*
*                        DCMATTS & DCMATTSL =  X'0C' FOR TS     Y02669*
*                        DCMATA & DCMATAL   =  X'CC' FOR A      Y02669*
*    RETURN CODES  =   NONE                                     Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - ERROR     =   BRANCH TO IEE0503D FROM D0503D FOR       Y20669*
*                        1 - STOPMN                             Y02669*
*                        2 - STOPTR AFTER ALL LOCKS RELEASED    Y02651*
*    CONDITIONS    =   TO ISSUE MESSAGE 'IEE305I COMMAND        Y02669*
*                      INVALID', IF AN INVALID STOPMN OR        Y02669*
*                      STOPTR OPERAND IS FOUND.                 Y02669*
*                      TO ISSUE MESSAGE 'IEE311I PARAMETER      Y02669*
*                      MISSING', IF NO OPERAND IS FOUND IN      Y02669*
*                      THE COMMAND BUFFER.                      Y02669*
*                      TO ISSUE MESSAGE 'IEE777I INVALID      @Y02668P*
*                      FROM SUBSYSTEM CONSOLE' STOP MONITOR   @Y02668P*
*                      COMMAND ISSUED FROM A SUBSYSTEM        @Y02668P*
*    OUTPUT DATA   =   LOCKS RELEASED FOR STOPTR FUNCTION ONLY  Y02669*
*                      REG 2 POINTS TO XSA IN WHICH:            Y02669*
*                      XAE WILL CONTAIN  X'05' FOR IEE305I      Y02669*
*                                        X'11' FOR IEE311I      Y02669*
*                                        X'15' FOR IEE777I    @Y02668P*
*                      XAV STILL CONTAINS THE COMMAND VERB FOR  Y02669*
*                          THE MESSAGE.                         Y02669*
*                      XAU WILL STILL CONTAIN THE CONSOLE ID    Y02669*
*                      XAA WILL STILL CONTAIN ASID IF REQUESTER Y02669*
*                          IS TIME SHARING USER (STOPMN  ONLY)  Y02669*
*                      XASSDS5 = OFF LOCKS OFF (STOPTR ONLY)    Y02651*
*    RETURN CODES  =   NONE                                     Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXTERNAL                                                      Y02669*
*    REFERENCES                                                 Y02669*
*                                                                     *
*    ROUTINES      =   BALR TO COMMUNICATION TASK TO ADJUST     Y02669*
*                      FIELDS IN BASEA, UCME AND MQE WITH       Y02669*
*                      REG 1 POINTING TO THE ADDRESS OF THE     Y02669*
*                      MONITOR PARM LIST                        Y02669*
*                      REG 13 POINTING TO REG SAVE AREA IN XSA  Y02669*
*                                                                     *
*                      BRANCH TO   IEE6703D    IEE0503D         Y02669*
*                                                                     *
*    DATA AREAS    =   IEZMPL - PARAMTER LIST FOR MONITOR       Y02669*
*                      ( ORG XAX MAPPED BY IEZMPL AND THE       Y02669*
*                        ADDR OF WHICH IS STORED IN XAR AND     Y02669*
*                        POINTED TO BE REGISTER 1)              Y02669*
*                       MPLPROC  DS  H     PROCESSING FLAGS     Y02669*
*                       MPLCONS  EQU X'80'  MPLID = CONSOLE ID  Y02669*
*                       MPLTERM  EQU X'40'  MPLID = ASID        Y02669*
*                       MPLSTRT  EQU X'20'  START MONITOR       Y02669*
*                       MPLSTOP  EQU X'10'  STOP MONITOR        Y02669*
*                       MPLID    DS  H     CONSOLE ID/ASID      Y02669*
*                       MPLTYPE  DS  H     MONITOR TYPE FLAGS   Y02669*
*                       MPLJBNM  EQU X'80'  MN JOBNAMES         Y02669*
*                       MPLSTAT  EQU X'40'  MN STATUS           Y02669*
*                       MPLSESS  EQU X'04'  MN SESSIONS         Y02669*
*                       MPLRSV10 DS  H     RESERVED             Y02669*
*                                                                     *
*   DATA AREAS   = PSEUDO-SACB TO BE USED THROUGHOUT SVC 34     Y02669*
*                    ORG   XAX                                  Y02669*
*                  XAXMASK  DS    C       REQUEST FLAGS         Y02669*
*                  XAXJOBS  EQU   X'80'   JOBS      (J)         Y02669*
*                  XAXJOBSL EQU   X'40'   JOBS,LIST (J,L)       Y02669*
*                  XAXTS    EQU   X'08'   TS        (TS)        Y02669*
*                  XAXTSL   EQU   X'04'   TS,LIST   (TS,L)      Y02669*
*                  XAXA     EQU   X'88'   A         (A)         Y02669*
*                  XAXAL    EQU   X'44'   A,LIST    (A,L)       Y02669*
*                  XAXFREE  DS    C       RESERVED              Y02669*
*                  XAXUTME  DS    H       UTME=NNN VALUE        Y02669*
*                  XAXSACB  DS    F       ADDRESS OF SACB       Y02669*
*                                                                     *
         EJECT
*   DATA AREAS  =  SACB (SCREEN AREA CONTROL BLOCK)             Y02669*
*                  DCMADFLG DS   XL1      DYNAMIC DISPLAY FLAGS Y02669*
*                  DCMADD   EQU  X'80'    TRACK IN AREA         Y02669*
*                  DCMAHOLD EQU  X'40'    TRACK IN HOLD MODE    Y02669*
*                  DCMATRCK DS   X        REQUEST INDICATORS    Y02669*
*                  DCMATJOB EQU  X'80'    JOBS       (J)        Y02669*
*                  DCMATJBL EQU  X'40'    JOBS,LIST  (J,L)      Y02669*
*                  DCMATTS  EQU  X'08'    TS         (TS)       Y02669*
*                  DCMATTSL EQU  X'04'    TS,LIST    (TS,L)     Y02669*
*                  DCMATA   EQU  X'88'    A          (A)        Y02669*
*                  DCMATAL  EQU  X'44'    A,LIST     (A,L)      Y02669*
*                  DCMAUTME DS   XL2      UTME=NNN VALUE        Y02669*
*                  DCMATECB DS   F        TRACK CANCEL ECB ADDR Y02669*
*                                                                     *
*    CONTROL BLOCKS=   IEERDCM                                  Y02669*
*                       (SACB)     W  -  CLEAR BITS FOR STOPTR  Y02669*
*                      CVT         R  -  ADDRESS BASEA AND UCM  Y02669*
*                      IEEBASEA    R  -  # ACTIVE MN JOBS; SESS Y02669*
*                                  W  -  CLEAR TIME/DSNAME/     Y02669*
*                                        SPACE FLAGS            Y02669*
*                      IEEXSA      R  -  INPUT                  Y02669*
*                                  W  -  MONITOR PARM LIST      Y02669*
*                      IEECUCM     R  -  VECTOR FOR IEAVMNTR    Y02669*
*                      IEZMPL      W  -  INPUT TO IEAVMNTR      Y02669*
*                      IHAPSA         -  NECESSARY FOR SETLOCK  Y02651*
*                                                                     *
*    TABLES        =   TWO COMPARE TABLES ARE USED TO DETERMINE Y02669*
*                      THE VALIDITY OF THE OPERAND ON THE       Y02669*
*                      STOPMN AND STOPTR COMMANDS.              Y02669*
*                      THE FORMAT OF EACH ENTRY IS THE SAME     Y02669*
*                                                                     *
*                      DLEN   DC   AL1(L'XXX-1) - LENGTH OF     Y02669*
*                                              SPECIFIC OPERAND Y02669*
*                      XXX    DC   C'YYYYY' - CHARACTER OPERAND Y02669*
*                                             REPRESENTATION    Y02669*
*                                                                     *
*                      DLEN AND 'XXX' ARE USED IN AN EX OF A    Y02669*
*                      CLC. IF THE OPERAND MATCHES WITH THE     Y02669*
*                      ENTERED OPERAND, A BRANCH IS TAKEN TO    Y02669*
*                      EITHER HANDLE THE COMMAND IN THIS MODULE Y02669*
*                      OR ELSE TO BUILD A PARM LIST AND BALR    Y02669*
*                      TO IEAVMNTR(STOPMN) OR IEE6703D(STOPTR). Y02669*
*                      OTHERWISE, THE REGISTER USED TO ADDRESS  Y02669*
*                      THE TABLE IS INCREMENTED BY TWO + THE    Y02669*
*                      VALUE OF DLEN TO REACH THE NEXT TABLE    Y02669*
*                      ENTRY.                                   Y02669*
*                                                                     *
         EJECT
*                      TWO BRANCH TABLES ARE USED IN CONNECTION Y02669*
*                      WITH THE COMPARE TABLES. THEY CONSIST    Y02669*
*                      OF BRANCHES TO THE PROPER OPERAND        Y02669*
*                      PROCESSORS. AN INDEX REGISTER IS         Y02669*
*                      MAINTAINED THROUGHOUT THE SCAN OF THE    Y02669*
*                      COMPARE TABLE. THIS IS USED AT DBRANCH   Y02669*
*                      FOR STOPMN AND MATCHTR FOR STOPTR        Y02669*
*                      TO GIVE CONTROL TO THE INDICATED         Y02669*
*                      PROCESSOR VIA THE BRANCH TABLE.          Y02669*
*                                                                     *
*    MACROS        =   MODID                                    Y02669*
*                      NIL - SERIALIZATION FOR BASEA BITS       Y02651*
*                      SETLOCK - CMS AND LOCAL LOCKS RELEASED   Y02651*
*                                                                     *
*    SERIALIZATION =   BASEA - NIL FOR TIME/DSNAME/SPACE FIELDS Y02651*
*                      SACB - SETLOCK FOR STOPTR                YO2651*
*                                                                     *
*    ENQUEUE                                                    Y02669*
*     RESOURCES    =   NONE                                     Y02669*
*                                                                     *
*    LOCKS HELD    =   SACB  - CMS AND LOCAL LOCKS OBTAINED BY  Y02651*
*                              IEE7503D FOR STOPTR, ARE HELD    Y02651*
*                              THROUGH THIS MODULE AND ARE      Y02651*
*                              RELEASED BY IEE6703D EXCEPT:     Y02651*
*                                 1. FOR SYNTAX ERRORS THEN     Y02651*
*                                    THEY WILL BE RELEASED AT   Y02651*
*                                    TRD0503D BEFORE BRANCHING  Y02651*
*                                    TO IEE0503D.               Y02651*
*                                 2. OR IF SOME TRACK OPTION    Y02651*
*                                    IS STILL ACTIVE ON THAT    Y02651*
*                                    SCREEN THEN THEY WILL BE   Y02651*
*                                    RELEASED BEFORE RETURNING  Y02651*
*                                    AT TRUPDATE.               Y02651*
*                              WHEN RELEASED XASSDS5 WILL BE    Y02561*
*                              TURNED OFF.                      Y02651*
*                                                                     *
*    CHANGE LEVEL  =   Y02669, ZA30198                        @ZA30198*
*                      Y02651                                   Y02651*
*                      YM1911 (PROLOGUE & COMMENT CHANGES ONLY) YM1911*
*                      Y02668                                 @Y02668P*
* MESSAGES         =   THE FOLLOWING MESSAGES ARE CONTAINED IN  Y02669*
*                      AND ISSUED FROM IEE0503D:                Y02669*
*                                                                     *
*                       MSGID    CODE    MESSAGE TEXT           Y02669*
*                       _____    ____    ____________           Y02669*
*                      IEE305I   X'05'   COMMAND INVALID        Y02669*
*                      IEE311I   X'11'   PARAMETER MISSING      Y02669*
*                      IEE777I   X'15'   INVALID FROM SUBSYSTEM       *
*                                        CONSOLE              @Y02668P*
* SYSGEN           =   INCLUDED WITH SVC 34 IN LOAD MODULE      Y02669*
*                      IGC0003D, DLIB AOSB3, MACRO SGIEF441,    Y02669*
*                      TARGLIB LPALIB.                          Y02669*
*                                                                     *
* SYSTEM LIBRARIES =   NONE                                     Y02669*
***********************************************************************
* C MATCH - TEST FOR ZERO CONSOLE ID                          @ZA30198*
         EJECT
IEE5503D CSECT
***********************************************************************
*                                                                     *
*                   SYMBOLIC REGISTER EQUATES                    Y02669
*                                                                     *
***********************************************************************
R0       EQU   0                       USED FOR THE NIL MACRO    Y02651
R1       EQU   1                       ADDRESS OF MN PARM LIST   Y02669
*                                      USED FOR THE NIL MACRO    Y02651
R2       EQU   2                       BASE REG FOR XSA
R3       EQU   3                       BASE REG FOR CVT
R4       EQU   4                       POINTER TO OPERAND BUFFER Y02669
R5       EQU   5                       WORK REGISTER, NIL MACRO  Y02651
R6       EQU   6                       ADDRESS END OF TABLE      Y02669
R7       EQU   7                       USED TO ADDR TAB ENTRIES  Y02669
R8       EQU   8                       BASE REG FOR UCM          Y02669
R9       EQU   9                       CONTAINS ASID OR UCMI     Y02669
R10      EQU   10                      BASE REG FOR BASEA
R11      EQU   11                      BASE REG FOR SACB         Y02669
R12      EQU   12                      BASE REG FOR PROGRAM
R13      EQU   13                      PTR TO SAVE AREA FOR PM   Y02669
R14      EQU   14                      USED FOR BALR AND RETURN  Y02669
R15      EQU   15                      USED FOR BALR AND BRANCH  Y02669
         EJECT
***********************************************************************
*                                                                     *
*                      DISPLACEMENTS AND LENGTHS                 Y02669
*                                                                     *
***********************************************************************
D0       EQU   0                       OFFSETS,LENGTHS,ETC.
D1       EQU   1                        *
D2       EQU   2                        *                         20030
D4       EQU   4                        *                         21002
L8       EQU   8                        *                        Y02669
***********************************************************************
*                                                                     *
*                             MASKS                              Y02669
*                                                                     *
***********************************************************************
OFF      EQU   X'00'                   ALL TRACK OPTIONS OFF     Y02669
FF       EQU   X'FF'                   MASK FOR NI
M7       EQU   7                       MASK FOR ADDRESS IN ICM   Y02669
***********************************************************************
*                                                                     *
*                          VERB CODES                            Y02669
*                                                                     *
***********************************************************************
PT       EQU   X'C8'                   STOPTR VERB CODE          Y02669
***********************************************************************
*                                                                     *
*                          ERROR CODES                           Y02669
*                                                                     *
***********************************************************************
ERR305   EQU   X'05'                   IEE305I-COMMAND INVALID   Y02669
ERR311   EQU   X'0B'                   IEE311I-MISSING PARM      Y02669
ERR777   EQU   X'15'                   IEE777I-SUBSYSTEM CONSL @Y02668P
         EJECT
***********************************************************************
*            ESTABLISH ADDRESSABILITY FOR MODULE                 Y02669
***********************************************************************
         BALR  R12,D0             ESTABLISH CSECT -
         USING *,R12              - ADDRESSABILITY
         MODID BRANCH=YES                                        Y01886
         SPACE 2
***********************************************************************
*            ESTABLISH ADDRESSABILITY FOR XSA                    Y02669
*  IF ENTERED FOR STOP TRACK COMMAND, BRANCH TO THAT SECTION     Y02669
*  OTHERWISE, GET ADDRESS OF STOP MONITOR OPERANDS               Y02669
*  ERR - IEE311I PARAMETER MISSING - IF NO OPERANDS              Y02669
***********************************************************************
         USING XSA,R2             EXTENDED SAVE AREA             Y02669
         CLI   XAN,PT             IF STOPTR BEING PORCESSED      Y02651
         BE    STOPTR             GO PROCESS STOPTR OPERANDS     Y02651
         SPACE
         SR    R4,R4              CLEAR REGISTER                 Y02669
         ICM   R4,M7,XAL+1        GET 3 BYTE ADDR OF OPERANDS    Y02669
         BZ    DERR311            NONE, ERR-IEE311 MISSING PARM  Y02669
         EJECT
***********************************************************************
*                                                                     *
*                            STOPMN                              Y02669
* LOOP THROUGH STOPMN COMPARE TABLE LOOKING FOR VALID OPERAND    Y02669
*                                                                     *
***********************************************************************
         SPACE
         SR    R5,R5              CLEAR REG FOR OPERAND LENGTH   Y02669
         SR    R15,R15            USED TO INDEX BRANCH TABLE     Y02669
         LA    R6,PARMTABN        ADDR LAST COMPARE TABLE ENTRY  Y02669
         LA    R7,PARMTAB         ADDR FIRST COMAPRE TABLE ENTRY Y02669
         USING PARMLEN,R7         ESTABLISH ADDRESSABILITY       Y02669
LOOP     EQU   *                  MATCH CMD OPERAND WITH ENTRIES Y02669
         IC    R5,PARMLEN         LENGTH OF COMPARE TABLE ENTRY  Y02669
         EX    R5,COMPARE         COMPARE OPERAND WITH ENTRY     Y02669
         BE    MATCH              IF MATCH, PROCESS ACCORDINGLY  Y02669
         LA    R7,D2(R5,R7)       ELSE INCREMENT TABLE POINTER   Y02669
         LA    R15,D4(R15)        INCREMENT BRANCH TABLE INDEX   Y02669
         CR    R7,R6              HAS LAST TABLE ENTRY TESTED?   Y02669
         BL    LOOP               NO, TEST NEXT TABLE ENTRY      Y02669
         DROP  R7                 NECESSARY TO RE-EST BASR REG   Y02669
         B     DERR305            ERROR, IEE305I COMMAND INVALID Y02669
         SPACE  2
***********************************************************************
*                                                                     *
* WHEN VALID MATCH IS FOUND ADDRESS CONTROL BLOCKS AND BRANCH    Y02669
* THEN CHECK, IF THE ISSUING CONSOLE IS JES3 THEN THE          @Y02668P
* COMMAND IS INVALID                                           @Y02668P
*                                                                     *
***********************************************************************
         SPACE 2
MATCH    EQU   *                                                 Y02669
         L     R3,CVTPTR          GET CVT ADDR                   Y02669
         USING CVT,R3             CVT ADDRESSABILITY             Y02669
         L     R10,CVTMSER        MASTER RESIDENT CORE ADDRESS   Y02669
         USING BASE,R10           BASEA ADDRESSABILITY           Y02669
         L     R8,CVTCUCB         GET ADDRESS OF UCM FROM CVT    Y02669
         USING UCM,R8             UCM ADDRESSABILITY             Y02669
         DROP  R3                                                Y02669
         USING UCMLIST,R9         EST. UCME ADDRESSABILITY     @Y02668P
         L     R9,UCMVEA          ADDRESS OF THE FIRST UCME    @Y02668P
         SR    R5,R5              ZERO OUT REG 5               @Y02668P
         IC    R5,XAU             CONSOLE ID                   @Y02668P
         LTR   R5,R5              INTERNAL,INPUT CMND SOURCE?  @ZA30198
         BZ    CONTINUE           YES,DEFAULT TO MSTR CONS.    @ZA30198
         BCTR  R5,D0              SUBTRACT ONE FROM CONSOLE ID @Y02668P
         M     R4,UCMVEZ          DISPLACEMENT OF UCME         @Y02668P
         AR    R9,R5              ADD TO FIRST UCME            @Y02668P
         TM    UCMDISP2,UCMDISPK  IS THIS A SUBSYSTEM CONSOLE  @Y02668P
         BZ    CONTINUE           NOT SUBSYSTEM CONSOLE        @Y02668P
         MVI   XAE,ERR777         ERROR CODE = 15              @Y02668P
CONTINUE EQU   *                                               @Y02668P
         DROP  R9                                              @Y02668P
         SR    R9,R9              CLEAR FOR CONSOLE TESTING      Y02669
         LH    R9,XAA             LOAD ASID INTO REG 9           Y02669
         XC    XAX(L8),XAX        CLEAR AREA IN XSA FOR MNTRPL   Y02669
         LA    R1,XAX             GET ITS ADDRESS AND            Y02669
         USING MPL,R1             ORG IEEMPL TO XAX              Y02669
         B     BRTABLE(R15)       BRANCH TO PROPER PROCESSOR     Y02669
         EJECT
***********************************************************************
*                                                                     *
*                    STOP MONITOR JOBNAMES                       Y02669
*          BUILD PARAMETER LIST FOR COMMUNICATIONS TASK          Y02669
*                                                                     *
***********************************************************************
         SPACE
DDJOB    EQU   *                                                 Y02669
         LTR   R9,R9              TEST FOR COMMAND SOURCE        Y02669
         BZ    DDJOB2             BRANCH - NOT TIME SHARING USER Y02669
         OI    MPLPROC1,MPLTERM+MPLSTOP     STOP MN WITH ASID    Y02669
         OI    MPLTYPE1,MPLJBNM   JONAMES OPTION                 Y02669
         B     COMINTER           GO TO COMMUNICATIONS TASK      Y02669
         SPACE
DDJOB2   EQU   *                                                 Y02669
         CLI   XAE,ERR777         ?SUBSYSTEM CONSOLE           @Y02668P
         BE    DERR777            YES, BRANCH COMMAND INVALID  @Y02668P
         OI    MPLPROC1,MPLCONS+MPLSTOP     STOP MN WITH CONS ID Y02669
         OI    MPLTYPE1,MPLJBNM   JOBNAMES OPTION                Y02669
         IC    R9,XAU             GET CONSOLE ID                 Y02669
         B     COMINTER           GO TO COMMUNICATIONS TASK      Y02669
         EJECT
***********************************************************************
*                                                                     *
*                     STOP MONITOR SESS                          Y02669
*          BUILD PARAMETER LIST FOR COMMUNICATIONS TASK          Y02669
*                                                                     *
***********************************************************************
         SPACE
DDSES    EQU   *                                                 Y02669
         LTR   R9,R9              TEST FOR COMMAND SOURCE        Y02669
         BZ    DDSES2             BRANCH - NOT TIME SHARING USER Y02669
         OI    MPLPROC1,MPLTERM+MPLSTOP     STOP MN WITH ASID    Y02669
         OI    MPLTYPE1,MPLSESS   SESSIONS OPTION                Y02669
         B     COMINTER           GO TO COMMUNICATIONS TASK      Y02669
         SPACE
DDSES2   EQU   *                                                 Y02669
         CLI   XAE,ERR777         ?SUBSYSTEM CONSOLE           @Y02668P
         BE    DERR777            YES, BRANCH COMMAND INVALID  @Y02668P
         OI    MPLPROC1,MPLCONS+MPLSTOP     STOP MN WITH CONS ID Y02669
         OI    MPLTYPE1,MPLSESS   SESSIONS OPTION                Y02669
         IC    R9,XAU             GET CONSOLE ID                 Y02669
         B     COMINTER           GO TO COMMUNICATIONS TASK      Y02669
         EJECT
***********************************************************************
*                                                                     *
*                      STOP MONITOR STATUS                       Y02669
*          BUILD PARAMETER LIST FOR COMMUNICATIONS TASK          Y02669
*                                                                     *
***********************************************************************
         SPACE
DDSTAT   EQU   *                                                 Y02669
         LTR   R9,R9              TEST FOR COMMAND SOURCE        Y02669
         BZ    DDSTAT2            BRANCH - NOT TIME SHARING USER Y02669
         OI    MPLPROC1,MPLTERM+MPLSTOP     STOP MN WITH ASID    Y02669
         OI    MPLTYPE1,MPLSTAT   STATUS OPTION                  Y02669
         B     COMINTER           GO TO COMMUNICATIONS TASK      Y02669
         SPACE
DDSTAT2  EQU   *                                                 Y02669
         CLI   XAE,ERR777         ?SUBSYSTEM CONSOLE           @Y02668P
         BE    DERR777            YES, BRANCH COMMAND INVALID  @Y02668P
         OI    MPLPROC1,MPLCONS+MPLSTOP     STOP MN WITH CONS ID Y02669
         OI    MPLTYPE1,MPLSTAT   STATUS OPTION                  Y02669
         IC    R9,XAU             GET CONSOLE ID                 Y02669
         B     COMINTER           GO TO COMMUNICATIONS TASK      Y02669
         EJECT
***********************************************************************
*                                                                     *
*                      STOP MONITOR DSNAME                       Y02669
*           TURN OFF MONITOR DSNAME BIT IN BASEA AND RETURN      Y02669
*                                                                     *
***********************************************************************
         SPACE
DDSNAME  EQU   *                                                 Y02669
         NIL   BAMONITR,FF-BADSN,WREGS=(0,1,5),REF=BASE                *
                                  CLEAR MN DSNAME BIT IN BASEA   Y02651
         BR    R14                RETURN                         Y02669
         EJECT
***********************************************************************
*                                                                     *
*                      STOP MONITOR SPACE                        Y02669
*           TURN OFF MONITOR SPACE BIT IN BASEA AND RETURN       Y02669
*                                                                     *
***********************************************************************
         SPACE
DDSPACE  EQU   *                                                 Y02669
         NIL   BAMONITR,FF-BASPACE,WREGS=(0,1,5),REF=BASE              *
                                  CLEAR MN SPACE BIT IN BASEA    Y02651
         BR    R14                RETURN                         Y02669
         EJECT
***********************************************************************
*                                                                     *
*        GO TO COMM TASK VIA BALR 14,15 TO RESET THE APPROPIATE  Y02669
*        MONITOR FILEDS IN THE APPROPIATE CONTROL BLOCKS.        Y02669
*             R1 CONTAINS ADDRESS OF XAR WHICH IN TURN           Y02669
*                POINTS TO IEZMPL (MONITOR PARM LIST)            Y02669
*             R13 CONTAINS ADRRESS OF THE SAVE AREA IN THE XSA   Y02669
*             R15 CONTAINS ADDRESS OF IEAVMNTR FROM UCM          Y02669
*                                                                     *
***********************************************************************
         SPACE   2
COMINTER EQU   *                                                 Y02669
         STH   R9,MPLID           GET ASID OR CONSOLE ID         Y02669
         ST    R14,XAS            SAVE REG 14 - RETURN ADDR      Y02669
         LA    R13,XASAVSTD       GET ADDRESS OF SAVE AREA       Y02669
         ST    R1,XAR             PUT ADDR OF MNTRPL IN XAR AND  Y02669
         LA    R1,XAR             PASS ADDR OF XAR TO COMM TASK  Y02669
         L     R15,UCMMNTR        GET ADDRESS OF COMM TASK MOD   Y02669
         BALR  R14,R15            HAVE FIELDS SET BY COMM TASK   Y02669
         EJECT
***********************************************************************
*                                                                     *
*        UPON RETURN FROM IEAVMNTR (THERE ARE NO RETURN CODES)   Y02669
*        IF NO CONSOLE OR TS TERMINAL IN OPERATOR MODE IS        Y02669
*        MONITORING JOBNAMES OR SESS, THE SYSTEM TIME BIT IN     Y02669
*        BASEA IS CLEARED. REGARDLESS, AFTER RESTORING REGISTER  Y02669
*        14 FROM THE XAS, CONTROL IS RETURNED VIA REGISTER 14.   Y02669
*                                                                     *
***********************************************************************
         SPACE
         SR    R5,R5                                             Y02669
         CH    R5,BAJNCNT         ANY ACTIVE MN JOBNAMES?        Y02669
         BL    RETURN             IF ONE IS ON, RETURN         @YM4797P
         CH    R5,BASESCT         ANY ACTIVE MN SESSIONS?        Y02669
         BL    RETURN             IF ONE IS ON, RETURN         @YM4797P
         NIL   MSBTN,FF-MSTN,WREGS=(0,1,5),REF=BASE                    *
                              ALL OFF, CLEAR BASEA SYSTEM TIME   Y02651
RETURN   EQU   *                                                 Y02669
         L     R14,XAS            RESTORE RETURN ADDRESS         Y02669
         BR    R14                RETURN TO CALLER               Y02669
         EJECT
***********************************************************************
*                                                                     *
*                             STOPTR                             Y02669
* LOOP THROUGH STOPTR COMPARE TABLE LOOKING FOR VALID OPERAND    Y02669
*    CMS AND LOCAL LOCKS ARE HELD ON ENTRY FROM IEE7503D         Y02651
*                                                                     *
***********************************************************************
         SPACE
STOPTR   EQU   *                                                 Y02669
         SR    R4,R4              CLEAR REGISTER                 Y02651
         ICM   R4,M7,XAL+1        GET 3 BYTE ADDRESS OF OPERANDS Y02651
         BZ    TRERR311           NONE, ERR-IEE311I MISSING PARM Y02651
         SPACE
         SR    R5,R5              CLEAR REG FOR OPERAND LENGTH   Y02669
         SR    R15,R15            CLEAR BR TABLE INDEX REGISTER  Y02669
         LA    R6,PARMTA2N        ADDR LAST COMPARE TABLE ENTRY  Y02669
         LA    R7,PARMTAB2        ADDR FIRST COMPARE TABLE ENTRY Y02669
         USING PARMLEN2,R7        ESTABLISH ADDRESSABILITY       Y02669
LOOPTR   EQU   *                  MATCH CMD OPERAND WITH ENTRIES Y02669
         IC    R5,PARMLEN2        LENGTH OF COMPARE TABLE ENTRY  Y02669
         EX    R5,COMPARE         COMPARE CMD OPERAND WITH ENTRY Y02669
         BE    MATCHTR            IF MATCH, PROCESS ACCORDINGLY  Y02669
         LA    R7,D2(R5,R7)       INCREMENT TABLE POINTER        Y02669
         LA    R15,D4(R15)        INCREMENT BRANCH TABLE INDEX   Y02669
         CR    R7,R6              HAS LAST TABLE ENTRY TESTED?   Y02669
         BL    LOOPTR             NO, TEST NEXT TABLE ENTRY      Y02669
         DROP  R7                 NECESSARY TO RE-EST BASR REG   Y02669
         B     TRERR305           ERROR, IEE305I COMMAND INVALID Y02651
         SPACE  5
***********************************************************************
*                                                                     *
*     GET ADDRESS OF SACB FROM XSA (XAXSACB) INPUT FROM IEE7503D Y02669
*                                                                     *
***********************************************************************
         SPACE
MATCHTR  EQU   *                                                 Y02669
         L     R11,XAXSACB        GET ADDRESS OF SACB PASSED BY  Y02669
         USING DCMACB,R11         IEE7503D                       Y02669
         B     TRKBRTAB(R15)      GO TO PROPER PROCESSOR         Y02669
         EJECT
***********************************************************************
*                                                                     *
*                         STOP TRACK JOBS                        Y02669
*            TURN OFF JOBS AND JOBS,LIST FLAGS IN SACB           Y02669
*                      J          J,L                            Y02669
*                                                                     *
***********************************************************************
         SPACE
TRKJOBS  EQU   *                                                 Y02669
         NI    DCMATRCK,FF-(DCMATJOB+DCMATJBL) TURN OFF JOBS (J) Y02669
         B     TRACKOFF                                          Y02669
         SPACE  5
***********************************************************************
*                                                                     *
*                         STOP TRACK TS                          Y02669
*              TURN OFF TS AND TS,LIST FLAGS IN SACB             Y02669
*                              TS,L                              Y02669
*                                                                     *
***********************************************************************
         SPACE
TRKTS    EQU   *                                                 Y02669
         NI    DCMATRCK,FF-(DCMATTS+DCMATTSL)   TURN OFF TS      Y02669
         B     TRACKOFF                                          Y02669
         SPACE 5
***********************************************************************
*                                                                     *
*                         STOP TRACK ACTIVE                      Y02669
*       TURN OFF JOBS; JOBS,LIST; TS AND TS,LIST FLAGS IN SACB   Y02669
*                  J      J,L            TS,L                    Y02669
*                                                                     *
***********************************************************************
         SPACE
TRKA     EQU   *                  TURN OFF ALL INDICATORS        Y02669
         NI    DCMATRCK,FF-(DCMATJOB+DCMATJBL+DCMATTS+DCMATTSL)  Y02669
         EJECT
***********************************************************************
*                                                                     *
*   IF THIS STOPTR COMMAND WIPES OUT THE TRACK DISPLAY FOR THIS  Y02669
*   AREA, DIDOCS IS GIVEN CONTROL VIA IEE6703D TO CLEAR THE      Y02669
*   SCREEN OF THE DISPLAY. OTHERWISE, THE LOCKS SET IN IEE7503D  Y02669
*   ARE RELEASED AND CONTROL IS RETURNED TO THE SYSTEM.          Y02669
*                                                                     *
***********************************************************************
         SPACE 2
TRACKOFF EQU   *                                                 Y02669
         CLI   DCMATRCK,OFF       ALL TRACK OPTIONS OFF?         Y02669
         BZ    D6703D             YES, HAVE DIDOCS CLEAR SCREEN  Y02669
         SPACE
TRUPDATE EQU   *                                                 Y02669
         BAL   R15,LOCKSOFF       GO RELEASE LOCKS               Y02651
         BR    R14                AND RETURN                     Y02669
         SPACE
D6703D   EQU   *                                                 Y02669
         L     R15,VCON6703       GET ADDRESS OF IEE6703D        Y02669
         BR    R15                                               Y02669
         EJECT
***********************************************************************
*                                                                     *
*                      STOP MONITOR ERROR EXITS                  Y02669
*                                                                     *
***********************************************************************
         SPACE
DERR311  EQU   *                                                 Y02669
         MVI   XAE,ERR311         ERROR CODE=11 FOR PARM MISSING Y02669
         B     D0503D             BRANCH TO MESSAGE MODULE       Y02669
DERR305  EQU   *                                                 Y02669
         MVI   XAE,ERR305         ERROR CODE=5 FOR COMD INVALID  Y02669
         B     D0503D             BRANCH TO MESSAGE MODULE       Y02669
DERR777  EQU   *                                               @Y02668P
         MVI   XAE,ERR777         ERROR CODE= 15 FOR INVALID   @Y02668P
         B     D0503D             FROM SUBSYSTEM CONSOLE       @Y02668P
         SPACE 2
***********************************************************************
*                                                                     *
*                      STOP TRACK ERROR EXITS                    Y02651
*     CMS AND LCOAL LOCKS MUST BR RELEASED BEFORE GOING TO       Y02651
*     IEE0503D SINCE IEE0503D DOES NOT EXPECT LOCKS TO BE SET    Y02651
*                                                                     *
***********************************************************************
         SPACE
TRERR311 EQU   *                                                 Y02651
         MVI   XAE,ERR311         ERROR CODE=11 FOR PARM MISSING Y02651
         B     TRD0503D           BRANCH TO MESSAGE MODULE       Y02651
TRERR305 EQU   *                                                 Y02651
         MVI   XAE,ERR305         ERROR CODE=5 FOR COMD INVALID  Y02651
TRD0503D EQU   *                                                 Y02651
         BAL   R15,LOCKSOFF       GO RELEASE LOCKS               Y02651
D0503D   EQU   *                                                 Y02669
         L     R15,VCON0503       GET ADDR OF IEE0503D           Y02651
         BR    R15                BRANCH TO MESSAGE MODULE       Y02651
         EJECT
***********************************************************************
*                                                                     *
*  CMS AND LOCAL LOCKS MUST BE RELEASED BEFORE EXITING FOR STOP TRACK *
*                                                                     *
***********************************************************************
         SPACE 2
LOCKSOFF EQU   *                                                 Y02651
         STM   R11,R14,XASAVLOC   SAVE REGISTERS ACROSS SETLOCK  Y02651
         SETLOCK RELEASE,TYPE=CMS,RELATED=(STOPTR,IEE7503D(LOCKSET))
         SETLOCK RELEASE,TYPE=LOCAL,RELATED=(STOPTR,IEE7503D(LOCKSET))
         LM    R11,R14,XASAVLOC   RESTORE REGS AFTER SETLOCK     Y02651
         NI    XASSDS,FF-XASSDS5  INDICATE LOCKS OFF             Y02651
         BR    R15                RETURN TO MAIN LINE            Y02651
         EJECT
***********************************************************************
*                                                                     *
*                         CONSTANTS                              Y02669
*                                                                     *
***********************************************************************
         SPACE  2
VCON0503 DC    V(IEE0503D)        ERROR MESSAGE MODULE ADDRESS   Y02669
VCON6703 DC    V(IEE6703D)        STOPTR MODULE ADDRESS          Y02669
COMPARE  CLC   D1(D0,R7),D0(R4)   COMPARE COMMAND OPERAND WITH   Y02669
*                                 COMPARE TABLE ENTRY            Y02669
         SPACE   5
***********************************************************************
*                                                                     *
*                         PATCH AREA                             Y02669
*                                                                     *
***********************************************************************
         SPACE  2
PATCH    DC    2CL25'** IEE5503D PATCH AREA **'               @ZA30198*
         EJECT
***********************************************************************
*                                                                     *
*                      STOPMN  BRANCH  TABLE                     Y02669
*                                                                     *
*        THIS TABLE MUST BE IN A ONE-TO-ONE CORRESPONDENCE WITH  Y02669
*        THE 'STOPMN COMPARE TABLE'                              Y02669
*                                                                     *
***********************************************************************
         SPACE
BRTABLE  DS    0H                                                Y02669
         B     DDJOB              BRANCH TO PROCESS PM JOBNAMES  Y02669
         B     DDSTAT             BRANCH TO PROCESS PM STATUS    Y02669
         B     DDSNAME            BRANCH TO PROCESS PM DSNAME    Y02669
         B     DDSPACE            BRANCH TO PROCESS PM SPACE     Y02669
         B     DDSES              BRANCH TO PROCESS PM SESS      Y02669
         SPACE  2
***********************************************************************
*  NOTE: THE FOLLOWING TABLE AND THE STOPTR OPERAND TABLE MUST   Y02669
*        BE THE LAST ITEMS IN THIS MODULE SINCE A USING FOR THE  Y02669
*        TABLES, DEPENDING ON THE FUNCTION BEING PROCESSED, IS   Y02669
*        ISSUED ON REGISTER 7 AND THUS CAN REESTABLISH THE       Y02669
*        MODULE'S BASE REGISTER FOR ANY CODE WHICH FOLLOWS THESE Y02669
*        TABLES.                                                 Y02669
*                                                                     *
*            S T O P M N   C O M P A R E   T A B L E             Y02669
*                                                                     *
*        THIS TABLE MUST BE IN A ONE-TO-ONE CORRESPONDENCE WITH  Y02669
*        THE 'STOPMN BRANCH TABLE'                               Y02669
***********************************************************************
         SPACE
PARMTAB  EQU   *                                                 Y02669
PARMLEN  EQU   *                                                 Y02669
         SPACE
         DC    AL1(L'PMJOBN-1)  LENGTH OF STOPMN OPERAND         Y02669
PMJOBN   DC    C'JOBNAMES '     STOPMN OPERAND                   Y02669
         SPACE
         DC    AL1(L'PMSTAT-1)                                   Y02669
PMSTAT   DC    C'STATUS '                                        Y02669
         SPACE
         DC    AL1(L'PMDSN-1)                                    Y02669
PMDSN    DC    C'DSNAME '                                        Y02669
         SPACE
         DC    AL1(L'PMSPACE-1)                                  Y02669
PMSPACE  DC    C'SPACE '                                         Y02669
         SPACE
         DC    AL1(L'PMSESS-1)                                   Y02669
PMSESS   DC    C'SESS '                                          Y02669
         SPACE
PARMTABN EQU   *                                                 Y02669
         EJECT
***********************************************************************
*                                                                     *
*                       STOPTR  BRANCH  TABLE                    Y02669
*                                                                     *
*        THIS TABLE MUST BE IN A ONE-TO-ONE CORRESPONDENCE WITH  Y02669
*        THE 'STOPTR COMPARE TABLE'                              Y02669
*                                                                     *
***********************************************************************
         SPACE 2
TRKBRTAB DS    0H                                                Y02669
         B     TRKJOBS            BRANCH TO PROCESS PT JOBS      Y02669
         B     TRKJOBS            BRANCH TO PROCESS PT J         Y02669
         B     TRKTS              BRANCH TO PROCESS PT TS        Y02669
         B     TRKA               BRANCH TO PROCESS PT A         Y02669
         SPACE 2
***********************************************************************
*  NOTE: THE FOLLOWING TABLE AND THE STOPMN OPERAND TABLE MUST   Y02669
*        BE THE LAST ITEMS IN THIS MODULE SINCE A USING FOR THE  Y02669
*        TABLES, DEPENDING ON THE FUNCTION BEING PROCESSED, IS   Y02669
*        ISSUED ON REGISTER 7 AND THUS CAN REESTABLISH THE       Y02669
*        MODULE'S BASE REGISTER FOR ANY CODE WHICH FOLLOWS THESE Y02669
*        TABLES.                                                 Y02669
*                                                                     *
*            S T O P T R    O P E R A N D    T A B L E           Y02669
*                                                                     *
*        THIS TABLE MUST BE IN A ONE-TO-ONE CORRESPONDENCE WITH  Y02669
*        THE 'STOPTR BRANCH TABLE'                               Y02669
*                                                                     *
***********************************************************************
         SPACE 2
PARMTAB2 EQU   *                                                 Y02669
PARMLEN2 EQU   *                                                 Y02669
         SPACE
         DC    AL1(L'PTJOB-1)     LENGTH OF STOPTR OPERAND       Y02669
PTJOB    DC    C'JOBS '           STOPTR OPERAND                 Y02669
         SPACE
         DC    AL1(L'PTJ-1)                                      Y02669
PTJ      DC    C'J '                                             Y02669
         SPACE
         DC    AL1(L'PTTS-1)                                     Y02669
PTTS     DC    C'TS '                                            Y02669
         SPACE
         DC    AL1(L'PTA-1)                                      Y02669
PTA      DC    C'A '                                             Y02669
         SPACE
PARMTA2N EQU   *                                                 Y02669
         EJECT
         IEEBASEA
         EJECT
CMXSA    DSECT
         IEEXSA
         EJECT
         IEZMPL
         EJECT
         IEERDCM
         EJECT
CVT      DSECT                                                   Y02669
         CVT
         EJECT
         IEECUCM  FORMAT=NEW
         EJECT
         IHAPSA
         END
