         TITLE 'IEE7503D -- L=CCA AND LIST PROCESSOR'
***********************************************************************
*                                                                     *
* MODULE NAME      =  IEE7503D                                  Y02669*
*                                                                     *
* DESCRIPTIVE NAME =  L=CCA AND ITS DEFAULTS AND LIST           Y02669*
*                     OPERAND PROCESSOR                         Y02669*
*                                                                     *
* COPYRIGHT        =  N/A                                       Y02669*
*                                                                     *
* STATUS           =  OS/VS2 RELEASE 037  SU 17               @ZA29986*
*                                                                     *
* FUNCTION         =  THIS MODULE PROCESSES THE FOLLOWING       Y02669*
*                     OPERANDS ON THE FOLOWING COMMANDS:        Y02669*
*                     COMMAND            OPERANDS               Y02669*
*                     ______             ________               Y02669*
*                     D  C,K                                    Y02669*
*                     D  CONSOLES  (C)                          YM1916*
*                     D  U                                      Y02669*
*                     D  M                                      Y02669*
*                     D  3850                                 @Z40LPTA*
*                     D  DMN                                  @Z40BPSV*
*                     D  SLIP                                 @G17SPTJ*
*                     D  DUMP (D)                             @G33CPMR*
*                     K  COMMANDS       L=CCA AND DEFAULTS      Y02669*
*                     PT JOBS  (J)                              Y02669*
*                        TS                                     Y02669*
*                        A                                      Y02669*
*                   __________________________________________  Y02669*
*                     D  JOBS  (J)                              Y02669*
*                        TS                                     Y02669*
*                        A               L=CCA AND DEFAULTS     Y02669*
*                     TR JOBS  (J)          AND ,LIST           Y02669*
*                        TS                   OR ,L             Y02669*
*                        A                                      Y02669*
*                                                                     *
* SPECIAL NOTE    =   THE FOLLOWING TERMS ARE INTERCHANGABLE:   Y02669*
*                       NULL AND X'00'                          Y02669*
*                       GRAPHICS AND CRT                        Y02669*
*                       'Z' AND IN-LINE                         Y02669*
*                       SOURCE, ISSUING AND REQUESTING CONSOLE  Y02669*
*                       'A' AND AREA                            Y02669*
*                       'CC' AND TARGET CONSOLE ID              Y02669*
*                       'L' OPERAND AND 'L=CCA';'L=CC';'L=A'    Y02669*
*                        OUT-OF-LINE, IN AN AREA AND 'A'        Y02669*
*                     THE FOLLOWING DEFINITIONS MAY BE USEFUL:  Y02669*
*                       STATUS DISPLAY (SD)-A CRT IN OUTPUT     Y02669*
*                         ONLY MODE, DIVIDED INTO AREAS INTO    Y02669*
*                         WHICH ONLY DYNAMIC AND STATUS         Y02669*
*                         DISPLAYS CAN APPEAR                   Y02669*
*                       MESSAGE STREAM (MS)-A CRT IN OUTPUT     Y02669*
*                         ONLY MODE, WHICH CAN NOT BE DIVIDED   Y02669*
*                         INTO AREAS AND RECEIVES ONLY IN-LINE  Y02669*
*                         OPERATOR TYPE MESSAGES                Y02669*
*                       FULL CAPABILITY (FC)-A CRT IN INPUT-    Y02669*
*                         OUTPUT MODE WHICH CAN BE DIVIDED INTO Y02669*
*                         AREAS WHICH CAN RECEIVE BOTH DYNAMIC  Y02669*
*                         AND STATUS DISPLAYS BUT WHICH ALSO    Y02669*
*                         CONTAINS AN IN-LINE PORTION AT THE    Y02669*
*                         TOP OF THE SCREEN FOR OPERATOR TYPE   Y02669*
*                         MESSAGES                              Y02669*
*                         IT CAN ALSO BE USED TO ENTER COMMANDS Y02669*
*                       STATUS DISPLAY-A MULTI-LINE WTO ISSUED  Y02669*
*                         AS THE RESULT OF MOST 'DISPLAY'       Y02669*
*                         COMMANDS WHICH APPEARS IN AN AREA 'A' Y02669*
*                         (OUT-OF-LINE) ON A SD OR FC CRT IF    Y02669*
*                         'L=A' IS SPECIFIED. OR CAN APPEAR     Y02669*
*                         IN-LINE ON A MS OR FC CRT.            Y02669*
*                       DYNAMIC DISPLAY-A MULTI-LINE WTO        Y02669*
*                         RESULTING FROM A TRACK COMMAND WHICH  Y02669*
*                         IS PERIODICALLY UPDATED BASED ON A    Y02669*
*                         TIME INTERVAL.                        Y02669*
*                         THIS DISPLAY CAN ONLY APPEAR IN AN    Y02669*
*                         AREA (OUT-OF-LINE) ON A SD OR FC CRT. Y02669*
*                       IN-LINE (Z) REFERS TO THE MESSAGE       Y02669*
*                         STREAM PRODUCED ON A MS OR FC CRT.    Y02669*
*                       OUT-OF-LINE REFERS TO THE AREAS         Y02669*
*                         DEFINED ON A SD OR FC CRT.            Y02669*
*                                                                     *
* OPERATION       =   IF THE REQUESTOR IS JES2(XAN=X'22'), THE  Y02669*
*                     ENTRY POINT IS IEE7603D, REGISTERS 0 - 14 Y02669*
*                     ARE SAVED IN XASAVSTD,(REGISTER 2 POINTS  Y02669*
*                     TO XSA ON ENTRY) LOCKS ARE OBTAINED AND   Y02669*
*                     THE SPECIAL JES2 INTERFACE CODE INVOKED.  Y02669*
*                                                                     *
*                     IF THE REQUESTOR IS A JES3 ASSOCIATED   @Y02668P*
*                     CONSOLE THEN THE COMMAND WAS NOT ISSUED @Y02668P*
*                     IN COMTASK'S MEMORY. THE CURRENT TCB    @Y02668P*
*                     POINTER IN IEFPTRS IS COMPARED TO       @Y02668P*
*                     COMTASK TCB IN THE UCM(TCBPTR=UCMPAX)   @Y02668P*
*                     IF THE DON'T MATCH XASSDS3 BIT IS SET   @Y02668P*
*                     ON. ONLY ROUTABLE (L=CCA) COMMANDS      @Y02668P*
*                     WILL BE VALID. EXECPT FOR K V,USE=.     @Y02668P*
*                     ALL MESSAGES WILL BE ISSUED VIA WTO.    @Y02668P*
*                     THE FOLLOWING MODULES WILL CHECK THE    @Y02668P*
*                     BIT 1. IEE5603D                         @Y02668P*
*                         2. IEE6703D                         @Y02668P*
*                         3. IEE6903D                         @Y02668P*
*                     IN THE FOLLOWING 'A' WILL REPRESENT THE   Y02669*
*                     A FIELD IN THE L=CCA OPERAND.             Y02669*
*                                                                     *
*                     THE ROUTING FIELD IN THE XSA (XAS) IS     Y02669*
*                     INITIALIZED TO ZEROES.                    Y02669*
*                                                                     *
*                     ALL OF THE ABOVE COMMANDS ARE FIRST       Y02669*
*                     INTERROGATED FOR THE 'L=CCA' OPERAND.     Y02669*
*                     IF STOPTR (PT) DOES NOT HAVE ANY         @YM2774P
*                     OPERAND (D AND TR HAVE ALREADY BEEN      @YM2774P
*                     'CHECKED), 'IEE311I' IS ISSUED.          @YM2774P
*                     IF FOUND, IT IS TRANSLATED AND PLACED     Y02669*
*                     IN THE XSA (XAS), THE 'L' OPERAND IS      Y02669*
*                     REMOVED FROM THE OPERAND BUFFER, AND      Y02669*
*                     THE VALUES PLACED IN THE XSA ARE THEN     Y02669*
*                     VALIDITY CHECKED.                         Y02669*
*                        XASSDS = XASSDS2 (X'40') 'L' WAS       Y02669*
*                                 SPECIFIED ON THE COMMAND.     Y02669*
*                        XASCID = THE SPECIFIED 'CC' VALUE      Y02669*
*                        XASDID = THE SPECIFIED 'A' VALUE       Y02669*
*                                                                     *
*                     IF THE 'L' OPERAND IS FOUND ON A COMMAND  Y02669*
*                     ISSUED FROM A TSO TERMINAL (XAA ^= X'00'),Y02669*
*                     'IEE305I COMMAND INVALID' IS ISSUED.      Y02669*
*                                                                     *
*                     IF 'CC' SPECIFIES AN INVALID CONSOLE ID,  Y02669*
*                     I.E. 'CC' IS HIGHER THAN THE CONSOLE ID   Y02669*
*                     DEFINED BY THE LAST UCME, OR '00'         Y02669*
*                     'IEE926I 'CC' NOT A CONSOLE' IS ISSUED.   Y02669*
*                                                                     *
*                     IF 'A' IS MORE THAN 1 CHARACTER,          Y02669*
*                     'IEE156I INVALID OPERAND' IS ISSUED.      Y02669*
*                                                                     *
*      MSGRT          IF THE 'L' OPERAND IS NOT FOUND ON THE    Y02669*
*      DEFAULTS       COMMAND, THE RCT (ROUTING CONTROL TABLE), Y02669*
*                     POINTED TO BY THE REQUESTING UCME AND     Y02669*
*                     BUILT AS THE RESULT OF A MSGRT COMMAND    Y02669*
*                     FOR THE REQUESTING CONSOLE, IS SCANNED    Y02669*
*                     TO SEE IF A DEFAULT ROUTING VALUE WAS     Y02669*
*                     SPECIFIED FOR THIS COMMAND.               Y02669*
*                       K= DEFAULTS FOR ALL CONTROL (K)         Y02669*
*                          COMMANDS                             Y02669*
*                       T= DEFAULTS FOR ALL TRACK AND STOPTR    Y02669*
*                          COMMANDS                             Y20669*
*                       D=O REPRESENTS DISPLAY CONSOLES (C)     Y02669*
*                     IF SO, THE ROUTING DEFAULTS ARE PLACED    Y02669*
*                     IN THE XSA AND VALIDITY CHECKED.          Y02669*
*                        XASCID = THE MSGRT DEFAULT VALUE       Y02669*
*                        XASDID = THE MSGRT DEFAULT OR X'00'    Y02669*
*                        XASSDS = X'00' 'L' NOT SPECIFED        Y02669*
*                                       ON THE COMMAND          Y02669*
*                                                                     *
*      INTERNALLY     IF THE ISSUING CONSOLE ID IS ZERO         YM1557*
*      ISSUED         (XAU=00) THE COMMAND WAS ISSUED           YM1557*
*      COMMAND        INTERNALLY. IF THE COMMAND WAS CONTROL    YM1557*
*                     (K), CONTROL IS RETURNED WITHOUT ANY      YM1557*
*                     ERROR MESSAGE. IF THE COMMAND WAS TRACK   YM1557*
*                     OR STOPTR, 'IEE927I  SPECIFY L OPERAND'   YM1557*
*                     IS ISSUED AND CONTROL RETURNED. ALL       YM1557*
*                     DISPLAY COMMANDS WILL BE ROUTED TO THE    YM1557*
*                     MASTER CONSOLE.                           YM1557*
*
*      SYSTEM         IF THERE IS NO CORRESPONDING RCT ENTRY    Y02669*
*      DEFAULTS       FOR THIS COMMAND, THE SYSTEM DEFAULTS     Y02669*
*                     TO THE REQUESTING CONSOLE. THE ROUTING    Y02669*
*                     INFORMATION IS PLACED IN THE XSA AND      Y02669*
*                     VALIDITY CHECKED.                         Y02669*
*                        XASCID = XAU (REQUESTING CONSOLE ID)   Y02669*
*                        XASDID = NULL                          Y02669*
*                        XASSDS = X'00' 'L' NOT SPECIFED        Y02669*
*                                       ON THE COMMAND          Y02669*
*                                                                     *
*      VALIDITY       THE VALUES IN XAS ARE THEN CHECKED FOR    Y02669*
*      CHECKING       ROUTING AUTHORITY, AND TO ENSURE THAT THE Y02669*
*                     TARGET CONSOLE IS ACTIVE WITH AREA 'A'    Y02669*
*                     DEFINED.                                  Y02669*
*                                                                     *
*                     IF THE TARGET AND REQUESTING CONSOLES ARE Y02669*
*                     NOT THE SAME (XASCID ^= XAU), THE SOURCE  Y02669*
*                     CONSOLE IS CHECKED FOR ROUTING AUTHORITY  Y02669*
*                     (UCMAUTH = X'20'). IF IT DOES NOT         Y02669*
*                     HAVE AUTHORITY TO ROUTE THE COMMAND,      Y02669*
*                     'IEE345I AUTHORITY INVALID' IS ISSUED.    Y02669*
*                                                                     *
*                     IF THE TARGET CONSOLE IS A SUBSYSTEM     @Y02668P
*                     (JES3) CONSOLE UCMDISPK WILL BE          @Y02668P
*                     ON. THE COMMAND WILL BE                  @Y02668P
*                     REJECTED. ERROR MESSAGE IEE926I          @Y02668P
*                     L=CCA OPERAND INVALID-CC SUBSYSTEM       @Y02668P
*                     CONSOLE IS ISSUED.                       @Y02668P
*                                                                     *
*                     IF THE TARGET CONSOLE IS NOT ACTIVE,      Y02669*
*                     (UCMATR ^= X'10') 'IEE926I 'CC' NOT A     Y02669*
*                     CONSOLE' IS ISSUED.                       Y02669*
*                                                                     *
*                     IF THE COMMAND IS OTHER THAN CONTROL VARY Y02669*
*                     (K V) OR THE TARGET CONSOLE IS NOT        Y02669*
*                     MESSAGE STREAM (MS) - AN OUTPUT CONSOLE   Y02669*
*                     WHICH DISPLAYS ALL REGULAR MESSAGES SENT  Y02669*
*                     TO THE OPERATOR, OR STATUS DISPLAY (SD)   Y02669*
*                     - AN OUTPUT CONSOLE WHICH DISPLAYS ONLY   Y02669*
*                     THOSE COMMANDS WHICH POSSESS AN 'L=CCA'   Y02669*
*                     OPERAND (UCMDISP ^= X'06'),               Y02669*
*                     'IEE926I FULL CAPABILITY' IS ISSUED.      Y02669*
*                     (ONLY K V COMMANDS CAN BE ROUTED TO A     Y02669*
*                     FULL CAPABILITY CONSOLE)                  Y02669*
*                                                                     *
*                     IF 'A' WAS SPECIFIED AND THE TARGET IS    Y02669*
*                     EITHER A NON-CRT (UCMDISP ^= X'20') OR    Y02669*
*                     MESSAGE STREAM ONLY (UCMDISP = X'04'),    Y02669*
*                     THE AREA FIELD IN THE XAS IS SET TO 'Z'   Y02669*
*                     (X'E9') TO INDICATE AN IN-LINE DISPLAY    Y02669*
*                     AREA ( XASDID = X'E9' ).                  Y02669*
*                     HOWEVER, IF THE TARGET IS A CRT DEVICE    Y02669*
*                     AND NOT MS ONLY, THE COMMAND IS ROUTED.   Y02669*
*                     (MS AND NON-CRT HAVE NO AREAS DEFINED.)   Y02669*
*                                                                     *
*                     IF 'A' WAS SPECIFIED AND THE TARGET       Y02669*
*                     DEVICE IS CRT (UCMDISP = X'20'),          Y02669*
*                     WITH THE AREA DEFINED AS IN-LINE          Y02669*
*                     (XASDID = 'Z'), AND THE TARGET IS STATUS  Y02669*
*                     DISPLAY ONLY (UCMDISP = X'02'),           Y02669*
*                     'IEE926I 'A' NOT AN AREA' IS ISSUED.      Y02669*
*                     HOWEVER, IF THE TARGET IS NOT STATUS      Y02669*
*                     DISPLAY ONLY, THE COMMAND IS ROUTED.      Y02669*
*                     (SD HAVE ONLY AREAS DEFINED, NO IN-LINE.) Y02669*
*                                                                     *
*                     IF THE TARGET IS CRT, WITH 'A' DEFINED    Y02669*
*                     ON THE COMMAND BUT FOR AN AREA THAT IS    Y02669*
*                     NOT DEFINED TO THE DEVICE; I.E. NO SACB   Y02669*
*                     (SCREEN AREA CONTROL BLOCK) WAS BUILT AS  Y02669*
*                     THE RESULT OF A CONTROL (K) A COMMAND,    Y02669*
*                     'IEE926I 'A' NOT AN AREA' IS ISSUED, BUT  Y02669*
*                     IF AN SACB EXISTS, THE COMMAND IS ROUTED. Y02669*
*                     (THE AREA MUST BE DEFINED PRIOR TO THE    Y02669*
*                     COMMAND BEING ISSUED.)                    Y02669*
*                                                                     *
*                     IF THE TARGET IS NON-CRT, WITH THE AREA   Y02669*
*                     DEFINED ON THE COMMAND 'IEE926I 'A' NOT   Y02669*
*                     AN AREA' IS ISSUED.                       Y02669*
*                     HOWEVER, IF THE AREA IS IN-LINE           Y02669*
*                     (I.E. 'Z') THE COMMAND IS ROUTED.         Y02669*
*      K COMMANDS     ONLY ROUTABLE K COMMANDS,EXCEPT FOR K V, @ZM30446
*                     ARE VALID FROM A SUBSYSTEM ( DUMMY )     @ZM30446
*                     CONSOLE. CONTROL IS PASSED               @ZM30446
*                     TO IEE6703D FOR ADDITIONAL VALIDITY      @Y02668P
*                     CHECKING AND ROUTING                     @Y02668P
*                                                                     *
*      ALL DISPLAY    ADDITIONAL VALIDITY CHECKING IS PERFORMED Y02669*
*      COMMANDS       IN THIS MODULE FOR D M; U; C,K; AND       Y02669*
*      INCLUDING      CONSOLES (C):, D 3850                    @Z40LPTA
*      DISPLAY        IF THE AREA 'A' HAD ALREADY BEEN DEFINED  Y02669*
*      ACTIVE         AS IN-LINE (XASDID = 'Z'), CONTROL IS     Y02669*
*      COMMANDS       GIVEN TO IEE0803D TO BUILD A CSCB.        Y02669*
*                                                                     *
*                     IF THE AREA 'A' HAS BEEN DEFINED AS       Y02669*
*                     OUT-OF-LINE (A WAS SPECIFIED ON THE       Y02669*
*                     COMMAND OR DEFAULTED AND XASDID CONTAINS  Y02669*
*                     THAT VALUE) AND THE SACB INDICATES THAT A Y02669*
*                     A DYNAMIC DISPLAY, (I.E. TR), ALREADY     Y02669*
*                     EXISTS IN THAT AREA, 'IEE926I DYNAMIC     Y02669*
*                     DISPLAY IN AREA' IS ISSSUED.              Y02669*
*                     HOWEVER, IF THE AREA DOES NOT CONTAIN     Y02669*
*                     A DYNAMIC DISPLAY, CONTROL IS GIVEN TO    Y02669*
*                     IEE0803D. A DISPLAY MAY OVERLAY ANOTHER   Y02669*
*                     EXISTING STATUS DISPLAY BUT NOT A         Y02669*
*                     DYNAMIC DISPLAY.                          Y02669*
*                                                                     *
*                     IF THE AREA HAS NOT BEEN GIVEN A DEFAULT  Y02669*
*                     VALUE (XASDID = NULL) YET, ONE IS NOW     Y02669*
*                     OBTAINED FROM THE SACB'S AS FOLLOWS:      Y02669*
*                      1-THE BOTTOM-MOST EMPTY AREA             Y02669*
*                      2-IF NONE, THE AREA CONTAINING THE       Y02669*
*                        OLDEST STATIC DISPLAY                  Y02669*
*                      3-IF NO AREA IS AVAILABLE, THE DISPLAY   Y02669*
*                        IS PUT IN-LINE (XASDID = 'Z')          Y02669*
*                      4-IF THE CONSOLE HAS NO MESSAGE STREAM   Y02669*
*                        ALSO REFERRED TO AS IN-LINE,           Y02669*
*                        THE COMMAND IS CANCELLED. EITHER       Y02669*
*                        'IEE921I DISPLAY AREA BUSY OR          Y02669*
*                        NEEDS DISPLAY AREA' IS ISSUED.         Y02669*
*                     FOR ALL VALID CONDITIONS, CONTROL GOES    Y02669*
*                     TO IEE0803D TO BUILD A CSCB.              Y02669*
*                                                                     *
*      DISPLAY        FOR D JOBS (J); A; AND TS FURTHER         Y02669*
*      ACTIVE         VALIDITY CHECKING IS PERFORMED.           Y02669*
*      COMMANDS       THE OPERAND BUFFER IS SCANNED FOR         Y02669*
*                     ',LIST' (,L).                             Y02669*
*                     IF FOUND THE XAXMASK FIELD IS SHIFTED     Y02669*
*                     RIGHT ONE BIT TO TURN ON THE 'LIST'       Y02669*
*                     INDICATOR FOR THE APPROPIATE COMMAND.     Y02669*
*                     CONTROL IS THEN GIVEN TO IEE0803D.        Y02669*
*                                                                     *
*       STOPTR        ADDITIONAL VALIDITY CHECKING FOR STOPTR   Y02669*
*       COMMANDS      IS ALSO PERFORMED IN THIS MODULE.         Y02669*
*                                                               Y02669*
*                     STOPTR COMMANDS ARA INVALID ON A         @Y02668P
*                     SUBSYSTEM CONSOLE. THE UCMDISPK BIT      @Y02668P
*                     IS CHECKED IN THE CONSOLE UCME           @Y02668P
*                     IF THE TARGET IS NON-CRT, 'IEE921I        Y02669*
*                     NEEDS DISPLAY AREA' IS ISSUED.            Y02669*
*                                                                     *
*                     IF AREA 'A' IS DEFINED AS 'Z', 'IEE921I   Y02669*
*                     NEEDS DISPLAY AREA' IS ISSUED.  IT IS     Y02669*
*                     IMPOSSIBLE TO HAVE A DYNAMIC DISPLAY      Y02669*
*                     IN-LINE.                                  Y02669*
*                                                                     *
*                     IF THE AREA 'A' IS NULL, THE SACB'S ARE   Y02669*
*                     SEARCHED FOR AN AREA CONTAINING A TRACK   Y02669*
*                     DISPLAY. IF ONE IS FOUND THE XASDID FIELD Y02669*
*                     IS OVERLAID WITH THE AREA ID FROM THE     Y02669*
*                     SACB (DCMAID), AND THE ADDRESS OF THIS    Y02669*
*                     SACB IS PLACED IN XAXSACB AND CONTROL     Y02669*
*                     IS GIVEN TO IEE5503D. IF NO AREA IS       Y02669*
*                     FOUND IN THE SACB'S, 'IEE921I NO TRACK    Y02669*
*                     TO STOP' IS ISSUED.                       Y02669*
*                                                                     *
*                     IF 'A' WAS SPECIFIED ON THE COMMAND AND   Y02669*
*                     IT DOES NOT MATCH ANY AREA DEFINED BY AN  Y02669*
*                     SACB'S, 'IEE921I NO TRACK TO STOP' IS     Y02669*
*                     ISSUED.                                   Y02669*
*                                                                     *
*                     IF IT DOES MATCH, THE ADDRESS OF THE      Y02669*
*                     SACB IS PLACED IN XAXSACB AND CONTROL     Y02669*
*                     GIVEN TO IEE5503D.                        Y02669*
*                                                                     *
*       TRACK         TRACK COMMANDS REQUIRE SLIGHTLY           Y02669*
*       COMMANDS      DIFFERENT VALIDITY CHECKING.              Y02669*
*                                                                     *
*                     IF THE TARGET CONSOLE IS NON-CRT,         Y02669*
*                     'IEE921I NEEDS DISPLAY AREA' IS           Y02669*
*                     ISSUED. TRACK IS ONLY VALID TO CRT.       Y02669*
*                                                                     *
*                     IF AREA 'A' IS DEFINED AS IN-LINE 'Z',    Y02669*
*                     'IEE921I NEEDS DISPLAY AREA' IS ISSUED.   Y02669*
*                     ( TRACK COMMAND CAN NOT APPEAR IN-LINE.)  Y02669*
*                                                                     *
*                     IF THERE IS NO EMPTY AREA  ON THE SCREEN  Y02669*
*                     AND XASDID IS STILL NULL, A DEFAULT IS    Y02669*
*                     SUPPLIED (SAME AS FOR DISPLAY EXCEPT      Y02669*
*                     THAT IF NO DISPLAY AREA IS AVAILABLE      Y02669*
*                     A TRACK COMMAND WILL BE REJECTED AND      Y02669*
*                     EITHER 'IEE921I DISPLAY AREA BUSY'        Y02669*
*                     OR 'NEEDS DISPLAY AREA' IS ISSUED.        Y02669*
*                                                                     *
*                     IF THE RESULTING AREA OR IF THE AREA      Y02669*
*                     SPECIFIED ON THE COMMAND CONTAINS A       Y02669*
*                     DYNAMIC DISPLAY, 'IEE926I DISPLAY         Y02669*
*                     ALREADY EXISTS' IS ISSUED.                Y02669*
*                                                                     *
*                     IN ALL VALID CASES, THE SACB CHAIN IS     Y02669*
*                     SEARCHED FOR AN AREA CONTAINING A         Y02669*
*                     TRACK DISPLAY.                            Y02669*
*                                                                     *
*                     IF ONE IS NOT FOUND, THIS IS THE FIRST    Y02669*
*                     TR COMMAND FOR THIS CONSOLE AND CONTROL   Y02669*
*                     IS GIVEN TO IEE0803D TO CREATE A CSCB     Y02669*
*                     SO THAT IEEVWAIT CAN ATTACH THE DISPLAY/  Y02669*
*                     TRACK PROCESSOR FOR THIS CONSOLE.         Y02669*
*                                                                     *
*                     IF AN ENTRY IS FOUND, THIS TRACK COMMAND  Y02669*
*                     WAS ISSUED TO UPDATE THE TRACK ENTRY.     Y02669*
*                     HOWEVER,  IF THE DISPLAY IS CURRENTLY IN  Y02669*
*                     THE HOLD MODE (K D,H WAS ISSUED) AND THE  Y02669*
*                     OPTIONS ON THIS TRACK COMMAND ARE         Y02669*
*                     IDENTICAL TO THE ONES CURRENT BEING       Y02669*
*                     DISPLAYED, IT IS ASSUMED THAT THE         Y02669*
*                     OPERATOR IS ATTEMPTING TO BRING IT OUT    Y02669*
*                     OF ITS HOLD MODE AND 'IEE921I DISPLAY     Y02669*
*                     ALREADY EXISTS' IS ISSUED AND CONTROL     Y02669*
*                     IS RETURNED. ( THE OPERATOR SHOULD HAVE   Y02669*
*                     ISSED K D,U WHICH BRINGS IT OUT OF THE    Y02669*
*                     HOLD MODE.) IF THEY DIFFER THE OPTION     Y02669*
*                     FLAGS IN THE CORRESPONDING SACB ARE       Y02669*
*                     UPDATED AND CONTROL IS RETURNED.          Y02669*
*                                                                     *
*      JES2 SCREEN    FIRST THE CONSOLE ID(XASCID) IS           Y02669*
*      AREA VALIDITY  INTERROGATED TO INSURE THAT IT IS DEFINED Y02669*
*      CHECKING       TO THE SYSTEM, ACTIVE, AND CAPABLE OF     Y02669*
*                     RECEIVING STATUS DISPLAYS. IF NOT, RETURN Y02669*
*                     CODE =12. THEN THE AREA ID(XASDID) IS     Y02669*
*                     CHECKED. IF IT IS NOT DEFINED TO THE      Y02669*
*                     CONSOLE RC = 16. IF IT CONTAINS A DYNAMIC Y02669*
*                     TRACK DISPLAY RC = 8. IF IT CONTAINS      Y02669*
*                     A STATUS DISPLAY RC = 4. IF IT IS         Y02669*
*                     AVAILABLE OR IF THE CONSOLE WAS NON-CRT,  Y02669*
*                     RC = 0.                                   Y02669*
*                                                                     *
*      INVALID        IF ANY COMMAND OTHER THAN THE ONES        Y02669*
*      COMMANDS       SPECIFIED IN THE FUNCTION SECTION ARE     Y02669*
*                     ENCOUNTERED, 'IEE305I COMMAND INVALID'    Y02669*
*                     IS ISSUED.                                Y02669*
*                                                                     *
*      JES3 VALIDITY  1. A JES3 CONSOLE CANNOT ISSUE A TRACK   @Y02668P
*      CHECKING          STOPTRACK OR K COMMAND (EXCEPT K C,D) @Y02668P
*                        TO ITSELF OR OTHER SUBSYSTEM CONSOLES @Y02668P
*                     2. SYSTEM COMMANDS CANNOT BE ROUTED TO   @Y02668P
*                        A SUBSYSTEM CONSOLE BY USING L=CCA    @Y02668P
*                                                                     *
*                                                                     *
* NOTES                                                         Y02669*
*                                                                     *
*    CHARACTER-CODE                                             Y02669*
*    DEPENDENCIES  =  ANY CHARACTER CODE OTHER THEN EBCDIC      Y02669*
*                     WILL REQUIRE REASSEMBLY OF THIS MODULE,   Y02669*
*                     PROVIDED THAT DECIMAL DIGITS CONTINUE     Y02669*
*                     TO BE REPRESENTED IN THE FORM X'FZ',      Y02669*
*                     WHERE 'Z' IS ANY HEXADECIMAL DIGIT.       Y02669*
*                     OTHERWISE, THE CODE WHICH VERIFIES THAT   Y02669*
*                     THE CONSOLE ID ON THE ROUTING OPERAND (L) Y02669*
*                     IS NUMERIC WILL HAVE TO BE CHANGED, TOO.  Y02669*
*                                                                     *
*    DEPENDENCIES  =  MSGRT COMMAND WILL PROVIDE DEFAULT        Y02669*
*                     ROUTING FOR D JOBS (J); A; TS AND         Y02669*
*                     TR JOBS (J); TS AND A. TR=A WILL BE USED  Y02669*
*                     FOR DEFAULT ROUTING ALL TRACK AND STOPTR  Y02669*
*                     COMMANDS.                                 Y02669*
*                                                                     *
*                     SVC 100 WILL REJECT AS INVALID FROM ANY   Y02669*
*                     TIME SHARING TERMINAL D A AND THE TRACK   Y02669*
*                     COMMANDS.                                 Y02669*
*                                                                     *
*                     DIDOCS WILL NOT PROCESS K D,H AND K D,U   Y02669*
*                     THE SAME FOR TRACKING AS FOR MONITORING.  Y02669*
*                     IN ADDITION, 2 WORDS WILL BE ADDED TO THE Y02669*
*                     SACB TO CONTAIN THE INDICATORS OF THE     Y02669*
*                     DESIRED TRACK FUNCTIONS FOR THAT CONSOLE  Y02669*
*                     AND THE TRACK ECB.                        Y02669*
*                     ALSO A BIT INDICATOR IS BEING ADDED TO    Y02669*
*                     THE DCM (DCMADFLG) TO INDICATE TO SVC 34  Y02669*
*                     THAT THIS IS A DIDOCS ISSUED TRACK IN     Y02669*
*                     RESPONSE TO A K D,U COMMAND.              Y02669*
*                     (TO BRING A TRACK DISPLAY OUT OF ITS HOLD Y02669*
*                     MODE, THE OPERATOR ISSUES A K D,U COMMAND Y02669*
*                     CAUSING DIDOCS TO ISSUE AN INTERNAL       Y02669*
*                     TRACK COMMAND.)                           Y02669*
*                                                                     *
*                     THE UCM, DCM, AND ASSOCIATED CONTROL      Y02669*
*                     BLOCKS WILL BE ALTERED TO REFLECT THE     Y02669*
*                     REPLACEMENT OF THE TRACK COMMAND FOR THE  Y02669*
*                     MONITOR ACTIVE COMMAND.                   Y02669*
*                                                                     *
*                     THE READER/INTERPRETER WILL REJECT THE    Y02669*
*                     TRACK COMMAND FROM THE INPUT STREAM.      Y02669*
*                                                                     *
*                     SVC 100 WILL REJECT AS INVALID ANY        Y02669*
*                     COMMAND WHICH IS NOT VALID FROM A TSO     Y02669*
*                     TERMINAL.                                 Y02669*
*                                                                     *
*                     JES2 WILL USE ENTRY POINT IEE7603D AND    Y02669*
*                     WILL SUPPLY THE ADDRESS OF A 134 BYTE     Y02669*
*                     XSA IN REGISTER 2. THE XSA WILL CONTAIN   Y02669*
*                     BINARY ZEROES EXCEPT FOR:                 Y02669*
*                           XAN    - X'22' VERBCODE             Y02669*
*                           XASCID - 1 BYTE BINARY CONSOLE ID   Y02669*
*                           XASDID - 1 BYTE CHARACTER AREA ID   Y02669*
*                     JES2 WILL BE IN KEY 0 AND THE XSA IN      Y02669*
*                     PAGABLE STORAGE.                          Y02669*
*                                                                     *
*                     UPON RETURN, REGISTER 15 WILL CONTAIN A   Y02669*
*                     RETURN CODE                               Y02669*
*                       0 - AREA IS AVAILABLE; CONSOLE IS       Y02669*
*                           NON-CRT                             Y02669*
*                       4 - AREA ALREADY CONTAINS A DISPLAY     Y02669*
*                       8 - AREA PRESENTLY CONTAINS A TRACK     Y02669*
*                           DISPLAY                             Y02669*
*                      12 - INVALID CONSOLE ID                  Y02669*
*                             CONSOLE NOT DEFINED               Y02669*
*                             CONSOLE NOT ACTIVE                Y02669*
*                             CONSOLE INCAPABLE OF RECEIVING    Y02669*
*                             STATUS DISPLAYS                   Y02669*
*                      16 - INVALID AREA ID                     Y02669*
*                     A WTO SHOULD NEVER BE ISSUED TO THE       Y02669*
*                     REQUESTED CONSOLE AND AREA IF THE         Y02669*
*                     RETURN CODES ARE 8, 12, 16. THE           Y02669*
*                     SUBSYSTEM SHOULD DECIDE IF IT IS          Y02669*
*                     DESIGNED TO OVERLAY THE EXISTING          Y02669*
*                     DISPLAY IN THE CASE OF RETURN CODE OF 4.  Y02669*
*                                                                     *
*    RESTRICTIONS  =  TRACK COMMANDS ARE INVALID FROM ANY       Y02669*
*                     TIME SHARING TERMINAL.                    Y02669*
*                                                                     *
*                     THE L=CCA OPERAND IS NOT VALID ON ANY     Y02669*
*                     COMMAND ISSUED FROM A TIME SHARING        Y02669*
*                     TERMINAL.                                 Y02669*
*                                                                     *
*                     THE TRACK COMMAND IS ONLY VALID FOR A     Y02669*
*                     TARGET CRT DEVICE.                        Y02669*
*                                                                     *
*                     CONTROL AND TRACK COMMAND ARE NOT VALID   Y02669*
*                     FROM THE INPUT STREAM.                    Y02669*
*                                                                     *
*                     ONLY K V CAN BE ROUTED TO A FULL          Y02669*
*                     CAPABILITY CONSOLE.                       Y02669*
*                                                                     *
*                     ANY TR, OR PT COMMAND ISSUED              YM1557*
*                     INTERNALLY MUST SPECIFY THE 'L' OPERAND.  YM1557*
*                     ANY DISPLAY COMMAND ISSUED INTERNALLY     YM1557*
*                     WILL BE ROUTED TO THE MASTER CONSOLE.     YM1557*
*                     ANY CONTROL COMMAND ISSUED INTERNALLY     YM1557*
*                     WILL RESULT IN A RETURN TO THE SYSTEM     YM1557*
*                     WITH NO MESSAGE ISSUED.                   YM1557*
*                                                                     *
*    REGISTER/                                                  Y02669*
*    CONVENTIONS   =  REGISTERS 0 - 14 SAVED IN XASAVSTD AND    Y02669*
*                     RESTORED FOR ALL JES2 REQUESTS.           Y02669*
*                                                                     *
*    PATCH-LABEL   =  PATCH                                     Y02669*
*                                                                     *
* MODULE TYPE      =  CSECT                                     Y02669*
*                                                                     *
*    PROCESSOR     =  N/A                                       Y02669*
*                                                                     *
*    MODULE SIZE   =  X'989'                                  @ZA29986*
*                                                                     *
*    ATTRIBUTES    =  REENTRANT, REFRESHABLE, KEY 0,            Y02669*
*                     SUPERVISOR MODE, PAGED LPA                Y02669*
*                                                                     *
* ENTRY POINT      =  IEE7503D FROM IEE0403D                    Y02669*
*    PURPOSE       =  TO PROCESS THE K AND STOPTR OPERANDS      Y02669*
*    LINKAGE       =  BRANCH                                    Y02669*
*    INPUT DATA    =  REGISTER 2 - POINTER TO THE XSA           Y02669*
*                        XAL - POINTS TO 1ST OPERAND IN BUFFER  Y02669*
*                        XAN - CONTAINS VERB CODE X'C8' STOPTR  Y02669*
*                                                 X'90' CONTROL Y02669*
*                        XAA - CONTAINS X'00' -INVALID FROM TSO Y02669*
*                        XAV - CONTAINS COMMAND VERB            Y02669*
*                        XAR - POINTS TO COMMAND BUFFER LENGTH  Y02669*
*                        XAU - CONTAINS REQUESTING CONSOLE ID   Y02669*
*                              (00-INTERNAL OR INPUT STREAM-K)  Y02669*
*                              (00-INTERNAL - STOPTR)           Y02669*
*                        XAP - POINTS TO END OF BUFFER +1       Y02669*
*                              ( FOR IEEBUFSC )                 Y02669*
*                                                                     *
* ENTRY POINT      =  IEE7503D FROM IEE3503D                    Y02669*
*    PURPOSE       =  TO PROCESS THE D AND TR OPERANDS          Y02669*
*    LINKAGE       =  BRANCH                                    Y02669*
*    INPUT DATA    =  REGISTER 2 - POINTER TO THE XSA           Y02669*
*                        XAL - POINTS TO 1ST OPERAND IN BUFFER  Y02669*
*                        XAN - CONTAINS VERB CODE               Y02669*
*                                X'2C' - DISPLAY C,K            Y02669*
*                                X'32' - DISPLAY M              Y02669*
*                                X'3E' - DISPLAY CONSOLES (C)   YM1916*
*                                X'42' - DISPLAY R              Y02669*
*                                X'52' - DISPLAY U              Y02669*
*                                X'F3' - DISPLAY 3850         @Z40LPTA*
*                                X'88' - DISPLAY DMN          @Z40BPSV*
*                                X'A8' - DISPLAY SLIP         @G17SPTJ*
*                                X'B0' - DISPLAY DUMP (D)     @G33CPMR*
*                                X'68' - D TS; JOBS (J); A      Y02669*
*                                X'C4' - TRACK                  Y02669*
*                        XAA - CONTAINS ASID (X'00'FOR TRACK    Y02669*
*                              AND D A AND U)                   Y02669*
*                        XAV - CONTAINS COMMAND VERB            Y02669*
*                        XAR - POINTS TO COMMAND BUFFER LENGTH  Y02669*
*                        XAU - CONTAINS REQUESTING CONSOLE ID   Y02669*
*                              (00-INTERNAL OR INPUT STREAM-D)  Y02669*
*                              (00-INTERNAL - TRACK)            Y02669*
*                        XAXMASK - CONTAINS AN INDICATOR        Y02669*
*                                  SPECIFYING THE DESIRED       Y02669*
*                                  DISPLAY OR TRACK FUNCTION:   Y02669*
*                                    X'80' FOR JOBS  (J)        Y02669*
*                                    X'08' FOR TS               Y02669*
*                                    X'88' FOR A                Y02669*
*                                    X'00' FOR ALL OTHER FNS    Y02669*
*                        XAP - POINTS TO END OF BUFFER +1       Y02669*
*                              ( FOR IEEBUFSC )                 Y02669*
*                                                                     *
* ENTRY POINT      =  IEE7603D FROM JES2                        Y02669*
*    PURPOSE       =  TO VERIFY THE ROUTING OPERAND 'L=CCA'     Y02669*
*                     FOR ALL JES2 ORIENTED COMMANDS            Y02669*
*    LINKAGE       =  BRANCH                                    Y02669*
*    INPUT DATA    =  REGISTER 2 - POINTER TO XSA               Y02669*
*                        XAN - CONTAINS VERB CODE X'22'         Y02669*
*                        XASCID - DESIRED TARGET CONSOLE ID     Y02669*
*                        XASDID - DESIRED AREA ID ON TARGET     Y02669*
*                        REST OF XSA ZEROES                     Y02669*
*                                                                     *
* REGISTERS USAGE                                               Y02669*
*    SAVED         =  REGISTER 14 SAVED BEFORE ISSUING IEEBUFSC Y02669*
*                                 RESTORED AT NOL               Y02669*
*                                             OPFOUND           Y02669*
*                                             ERRONL            Y02669*
*                     REGISTERS 11 - 14 SAVED BEFORE ISSUING    Y02651*
*                               SETLOCK AT LOCKSET AND LOCKSOFF Y02651*
*                  =  REGISTERS 0 - 14 SAVED IN XASAVSTD AT     Y02669*
*                               ENTRY POINT IEE7603D.           Y02669*
*                                                                     *
*    CONTAINED     =  R0  -  USED IN ODD EVEN PAIR              Y02669*
*                     R1  -  PARM REG FOR INTERNAL SUBROUTINES  Y02669*
*                     R2  -  INPUT - BASE REG FOR XSA           Y02669*
*                     R3  -  BASE REG FOR UCM                   Y02669*
*                     R4  -  USED IN ODD EVEN PAIR              Y02669*
*                     R5  -  USED IN ODD EVEN PAIR              Y02669*
*                     R6  -  TARGET AREA ID                     Y02669*
*                     R7  -  TARGET CONSOLE ID                  Y02669*
*                     R8  -  WORK REG - OPERAND POINTER         Y02669*
*                     R9  -  WORK REG - USED FOR RCT SCAN       Y02669*
*                     R10 -  BASE REG FOR RESIDENT DCM          Y02669*
*                     R11 -  BASE REG FOR RCT & SACB            Y02669*
*                     R12 -  BASE REG FOR UCME                  Y02669*
*                     R13 -  MODULE BASE REG                    Y02669*
*                     R14 -  RETURN ADDRESS                     Y02669*
*                     R15 -  USED FOR BAL,RETURN CODE
*                                                                     *
*    RESTORED      =  REGISTERS 11 - 14 RESTORED AFTER ISSUING  Y02651*
*                               SETLOCK AT LOCKSET AND LOCKSOFF Y02651*
*                  =  REGISTER  0 - 14 RESTORED AT JES2EXIT     Y02669*
*                               BEFORE RETURNING TO JES2        Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
* NOTE: FOR ALL EXITS, IF THE 'L' OPERAND WAS SPECIFIED ON THE  Y02669*
*       COMMAND, IT WILL BE REMOVED FROM THE COMMAND BUFFER     Y02669*
*       BEFORE EXITING.                                         Y02669*
*       THE EXCEPTION, FOR N,PFK THE L= OPERAND IS PASSED TO  @ZA12152*
*       DIDOCS FOR PROCESSING.                                @ZA12152*
*                                                                     *
* EXIT - NORMAL    =  BRANCH TO IEE0803D FROM D0803D            Y02669*
*    CONDITIONS    =  TO BUILD CSCB'S FOR D COMMANDS            Y02669*
*    OUTPUT DATA   =  R2 POINTS TO XSA WHICH CONTAINS           Y02669*
*                        SAME AS INPUT PLUS                     Y02669*
*                        XASCID = 'CC' VALUE OR XAU             Y02669*
*                        XASDID = 'A' VALUE OR 'Z'              Y02669*
*                        XASSDS = XASSDS2 (X'40') IF 'L'        Y02669*
*                                 OPERAND SPECIFIED ON COMMAND  Y02669*
*                        XAN - CONTAINS VERB CODE               Y02669*
*                                 X'2C' - DISPLAY C,K           Y02669*
*                                 X'32' - DISPLAY M             Y02669*
*                                 X'3E' - DISPLAY CONSOLES (C)  YM1916*
*                                 X'52' - DISPLAY U             Y02669*
*                                 X'68' - D JOBS (J); TS; A     Y02669*
*                                 X'88' - DISPLAY DMN         @Z40BPSV*
*                                 X'A8' - DISPLAY SLIP        @G17SPTJ*
*                                 X'B0' - DISPLAY DUMP (D)    @G33CPMR*
*                     IF 'LIST' ('L') WAS SPECIFIED:            Y02669*
*                        XAXMASK  X'40' JOBS,LIST   (J,L)       Y02669*
*                                 X'04' TS,LIST     (TS,L)      Y02669*
*                                 X'44' A,LIST      (A,L)       Y02669*
*    RETURN CODES  =  NONE                                      Y02669*
* 2  CONDITIONS    =  TO BUILD CSCB'S FOR TR INITIAL COMMANDS   Y02669*
*    OUTPUT DATA   =  R2 POINTS TO XSA WHICH CONTAINS           Y02669*
*                        SAME AS INPUT PLUS                     Y02669*
*                        XASCID = 'CC' VALUE OR XAU             Y02669*
*                        XASDID = 'A' VALUE OR DEFAULT          Y02669*
*                        XASSDS = XASSDS2 (X'40') IF 'L'        Y02669*
*                                 OPERAND SPECIFIED ON COMMAND  Y02669*
*                        XAXSACB - CONTAINS ADDRESS OF DCMATECB Y02669*
*                        DCMAUTME - WILL BE SET TO THE DEFAULT  Y02669*
*                                   TIME INTERVAL - 30 SECONDS  Y02669*
*                     IF 'LIST' ('L') WAS SPECIFIED:            Y02669*
*                        DCMTMASK X'40' JOBS,LIST   (J,L)       Y02669*
*                                 X'04' TS,LIST     (TS,L)      Y02669*
*                                 X'44' A,LIST      (A,L)       Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - NORMAL    =  RETURN VIA BR R14 FROM RETURN             Y02669*
*    CONDITIONS    =  ADDITIONAL TRACK COMMAND ISSUED FOR       Y02669*
*                     ALREADY EXISTING TRACK TASK               Y02669*
*    OUTPUT DATA   =  SACB FIELDS UPDATE                        Y02669*
*    RETURN CODES  =  NONE                                      Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - NORMAL    =  BRANCH TO IEE5503D FROM D5503D            Y02669*
*    CONDITIONS    =  TO PROCESS THE PT COMMAND                 Y02669*
*    OUTPUT DATA   =  R2 POINTS TO XSA WHICH CONTAINS           Y02669*
*                      SAME AS INPUT PLUS                       Y02669*
*                      XAXSACB CONTAINS ADDRESS OF SACB         Y02669*
*                      XAS CONTAINS TARGET CONSOLE & AREA ID    Y02669*
*                        XASCID = 'CC' VALUE OR XAU             Y02669*
*                        XASDID = 'A' VALUE OR DEFAULT          Y02669*
*                        XASSDS = XASSDS2 (X'40') IF 'L'        Y02669*
*                                 OPERAND SPECIFIED ON COMMAND  Y02669*
*    RETURN CODES  =  NONE                                      Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - NORMAL    =  BRANCH TO IEE6703D FROM D6703D            Y02669*
*    CONDITIONS    =  FOR ALL K COMMANDS                        Y02669*
*    OUTPUT DATA   =  REG 2 POINTS TO XSA WHICH CONTAINS        Y02669*
*                      SAME AS INPUT PLUS                       Y02669*
*                      XAS CONTAINS TARGET CONSOLE & AREA ID    Y02669*
*                        XASCID = 'CC' VALUE OR XAU             Y02669*
*                        XASDID = 'A' VALUE OR 'Z'              Y02669*
*                        XASSDS = XASSDS2 (X'40') IF 'L'        Y02669*
*                                 OPERAND SPECIFIED ON COMMAND  Y02669*
*                        XASSDS = XASSDS3 IF A TASK OTHER     @Y02668P*
*                                 THAN COMTASK ISSUED THE     @Y02668P*
*                                 COMMAND (JES3)              @Y02668P*
*    RETURN CODES  =  NONE                                      Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - NORMAL    =  BRANCH TO IEE2903D FROM D2903D            Y02669*
*    CONDITIONS    =  FOR DISPLAY REQUESTS                      Y02669*
*    OUTPUT DATA   =  REG 2 POINTS TO XSA WHICH CONTAINS        Y02669*
*                      SAME AS INPUT PLUS                       Y02669*
*                      XAS CONTAINS TARGET CONSOLE & AREA ID    Y02669*
*                        XASCID = 'CC' VALUE OR XAU             Y02669*
*                        XASDID = 'A' VALUE OR 'Z'              Y02669*
*                        XASSDS = XASSDS2 (X'40') IF 'L'        Y02669*
*                                 OPERAND SPECIFIED ON COMMAND  Y02669*
*    RETURN CODES  =  NONE                                      Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - NORMAL   =  RETURN VIA BR R14 TO JES2 FROM JES2EXIT   Y02669*
*     CONDITIONS   =  DESIRED ROUTING REQUEST VALIDITY CHECKED  Y02669*
*     OUTPUT DATA  =  REGISTER 2 STILL POINTS TO XSA            Y02669*
*     RETURN CODES =  REGISTER 15 CONTAINS                      Y02669*
*                        0 IF CONSOLE AND AREA IDS WERE VALID   Y02669*
*                        4 IF THE AREA PRESENTLY CONTAINS       Y02669*
*                          ANOTHER DISPLAY                      Y02669*
*                        8 IF THE AREA PRESENTLY CONTAINS A     Y02669*
*                          TRACK DISPLAY                        Y02669*
*                       12 IF THE CONSOLE ID IS INVALID         Y02669*
*                                 CONSOLE ID TOO HIGH           Y02669*
*                                 CONSOLE NOT ACTIVE            Y02669*
*                                 CONSOLE FULL CAPABILITY       Y02669*
*                       16 IF THE AREA ID IS INVALID            Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - NORMAL     =  BRANCH TO IEE9403D FROM D9403D          @Z40LPTA
*    CONDITIONS     =  FOR DISPLAY 3850 REQUEST                @Z40LPTA
*    OUTPUT DATA    =  R2 PTR TO XSA WHICH CONTAINS            @Z40LPTA
*                         SAME AS INPUT PLUS                   @Z40LPTA
*                         XASCID = 'CC' VALUE OR XAU           @Z40LPTA
*                         XASDID = 'A' VALUE OR 'Z'            @Z40LPTA
*                         XAN    = (X'68') DISPLAY VERB CODE   @Z40LPTA
*    RETURN CODES    =  NONE.                                  @Z40LPTA
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
** EXIT - ERROR  = BRANCH TO IEE0503D FROM D0503D               Y02669*
*   CONDITIONS   = TO ISSUE MESSAGE 'IEE305I COMMAND INVALID',  Y02669*
*                  IF AN INVALID COMMAND IS ENCOUNTERED, OR     Y02669*
*                  IF TSO ISSUES ANY COMMAND OTHER THAN DISPLAY Y02669*
*                   OR INCLUDES THE 'L' OPERAND ON ANY COMMAND  Y02669*
*                  IF THE STOP TRACK COMMAND DOES NOT          @YM2774P
*                   HAVE ANY OPERANDS THEN ISSUE MESSAGE       @YM2774P
*                   'IEE311I PARAMETER MISSING'                @YM2774P
*                   TRACK, STOP TRACK, AND K COMMAND           @Y02668P
*                   (EXCEPT K C, D) ARE INVALID                @Y02668P
*                   FROM A SUBSYSTEM CONSOLE.                  @Y02668P
*   OUTPUT DATA  = ERROR CODES FOR MESSAGE ISSUED IN IEE0503D   Y02669*
*                  IN XSA POINTED TO BY REG 2                   Y02669*
*                  XAE WILL CONTAIN  X'05' FOR IEE305I          Y02669*
*                                    X'0B' FOR IEE311I          YM1917*
*                                    X'15' FPR IEE777I         @Y02668P
*                  XAV STILL CONTAINS COMMAND VERB FOR MESSAGE  Y02669*
*   RETURN CODES = NONE                                         Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXIT - ERROR     =  BRANCH TO IEE5603D FROM D5603D            Y02669*
*    CONDITIONS    =  ANY ERROR CONDITIONS REQUIRING A MESSAGE  Y02669*
*    OUTPUT DATA   =  REG 2 POINTS TO XSA CONTAINING ERROR      Y02669*
*                     CODES FOR IEE5603D AND IEE5903D.          Y02669*
*                     XAE WILL CONTAIN AN INDEX FOR THE         Y02669*
*                     MAJOR MESSAGE FOR IEE5603D:               Y02669*
*                        X'04' - IEE156I INVALID OPERAND        Y02669*
*                        X'08' - IEE345I CM AUTHORITY INVALID   Y02669*
*                        X'09' - IEE921I CM OP REJECTED         Y02669*
*                        X'0B' - IEE925I 'CMD' OPERAND          Y02669*
*                        X'0C' - IEE926I L=CCA OPERAND INVALID  Y02669*
*                        X'0D' - IEE927I REQUEST AMBIGIOUS      YM1557*
*                     XASTSAVE WILL CONTAIN AN INDEX FOR THE    Y02669*
*                     MINOR MESSAGE FOR IEE5903D:               Y02669*
*                      IEE921I CM OP REJECTED:                  Y02669*
*                        X'0C' - DISPLAY ALREADY EXISTS         Y02669*
*                        X'0D' - DISPLAY AREA BUSY              Y02669*
*                        X'0E' - NO TRACK TO STOP               Y02669*
*                        X'19' - NEEDS DISPLAY AREA             Y02669*
*                        X'1F' - SUBSYSTEM CONSOLE             @Y02668P
*                      IEE925I 'XAE' OPERAND:                   Y02669*
*                        X'1A' - FOR CRT ONLY                   Y02669*
*                      IEE926I L=CCA OPERAND INVALID:           Y02669*
*                        X'12' - NOT AN AREA                    Y02669*
*                        X'13' - NOT A CONSOLE                  Y02669*
*                        X'14' - FULL CAPABILITY                Y02669*
*                        X'15' - DYNAMIC DISPLAY IN AREA        Y02669*
*                        X'1F' - SUBSYSTEM CONSOLE             @Y02668P
*                      IEE927I REQUEST AMBIGIOUS                YM1557*
*                        X'17' - SPECIFY L OPERAND              YM1557*
*                     XAV STILL CONTAINS COMMAND VERB FOR THE   Y02669*
*                     MESSAGE                                   Y02669*
*                     XAU CONTAINS CONSOLE ID FOR MESSAGE       Y02669*
*                     ROUTING (MAY BE X'00' IF INTERNAL)        YM1557*
*    RETURN CODES  =  NONE                                      Y02669*
*                                                                     *
*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *
*                                                                     *
* EXTERNAL                                                      Y02669*
*   REFERENCES                                                  Y02669*
*                                                                     *
*    ROUTINES      =  BRANCHES TO  IEE6703D   IEE0803D          Y02669*
*                                  IEE5503D   IEE5603D/IEE5903D Y02669*
*                                  IEE0503D   IEE2903D          Y02669*
*                                  IEE9403D                    @Z40LPTA
*                                                                     *
*   DATA AREAS   = PSEUDO-SACB TO BE USED THROUGHOUT SVC 34     Y02669*
*                    ORG   XAX                                  Y02669*
*                  XAXMASK  DS    C       REQUEST FLAGS         Y02669*
*                  XAXJOBS  EQU   X'80'   JOBS        (J)       Y02669*
*                  XAXJOBSL EQU   X'40'   JOBS,LIST   (J,L)     Y02669*
*                  XAXTS    EQU   X'08'   TS                    Y02669*
*                  XAXTSL   EQU   X'04'   TS,LIST     (TS,L)    Y02669*
*                  XAXA     EQU   X'88'   A                     Y02669*
*                  XAXAL    EQU   X'44'   A,LIST      (A,L)     Y02669*
*                  XAXFREE  DS    C       RESERVED              Y02669*
*                  XAXUTME  DS    H       TIME INTERVAL VALUE   Y02669*
*                  XAXSACB  DS    F       ADDRESS OF SACB       Y02669*
*                                                                     *
*   DATA AREAS  =  SACB (SCREEN AREA CONTORL BLOCK)             Y02669*
*                  DCMADFLG  DS   C       STATUS INDICATORS     Y02669*
*                  DCMADD    EQU  X'80'   TRACK ACTIVE IN AREA  Y02669*
*                  DCMAHOLD  EQU  X'40'   TRACK DISPLAY HELD    Y02669*
*                  DCMATRCK  DS   C       REQUEST INDICATORS    Y02669*
*                  DCMATJOB  EQU  X'80'   JOBS        (J)       Y02669*
*                  DCMATJBL  EQU  X'40'   JOBS,LIST   (J,L)     Y02669*
*                  DCMATTS   EQU  X'08'   TS                    Y02669*
*                  DCMATTSL  EQU  X'04'   TS,LIST     (TS,L)    Y02669*
*                  DCMATA    EQU  X'88'   A                     Y02669*
*                  DCMATAL   EQU  X'44'   A,LIST      (A,L)     Y02669*
*                  DCMTUTME  DS   H       TIME INTERVAL VALUE   Y02669*
*                  DCMATECB  DS   F       TRACK ECB             Y02669*
*                                                                     *
*    CONTROL BLOCKS=  CVT           R - ADDRESS UCM & BASEA     Y02669*
*                     IEEXSA        R - INPUT INFORMATION       Y02669*
*                                   W - OUTPUT INFORMATION      Y02669*
*                     IEECUCM       R - ADDRESS DCM, RCT        Y02669*
*                                       DEVICE STATUS           Y02669*
*                     IEECVMAP (RCT) R - ROUTING DEFAULTS       Y02669*
*                     IEERDCM       R - ADDRESS SACB            Y02669*
*                     SACB          R - PRESENT TRACK STATUS    Y02669*
*                                   W - UPDATE OR INIT STATUS   Y02669*
*                                       FOR TRACK ONLY          Y02669*
*                     IHAPSA          - NECESSARY FOR SETLOCK   Y02651*
*                                                                     *
*    TABLES        =  NONE                                      Y02669*
*                                                                     *
*    MACROS        =  MODID                                     Y02669*
*                     IEEBUFSC                                  Y02669*
*                                                                     *
*    ENQUEUE                                                    Y02669*
*     RESOURCES    =  NONE                                      Y02669*
*                                                                     *
*    LOCKS HELD    =  LOCKS ARE OBTAINED AT LOCKSET AND         Y02651*
*                     RELEASED:                                 Y02651*
*                      1) AT RETURN BEFORE RETURNING            Y02651*
*                      2) AT D0503D BEFORE EXITING TO IEE0503D  Y02651*
*                      3) AT D0803D BEFORE EXITING TO IEE0803D  Y02651*
*                      4) AT JES2EXIT BEFORE RETURNING TO JES2  Y02669*
*                      5) FOR STOPTR IN IEE5503D OR IEE6703D    Y02651*
*                      6) FOR K IN IEE6703D, IEE7703D, IEE7803D Y02651*
*                        IEE6803D, IEE6903D, IEE5603D, IEE5903D Y02651*
*                                                                     *
*    CHANGE LEVEL  =  Y02669                                    Y02669*
*                     Y02651                                    Y02651*
*                     YM1557                                    YM1557*
*                     YM1916                                    YM1916*
*                     YM1917                                    YM1917*
*                     YM1920                                    YM1920*
*                     Y02668                                   @Y02668P
*                     ZM30445                                  @ZM30445
*                     ZM30446                                  @ZM30446
*                     ZA02576                                  @ZA02576
*                     ZA03611                                  @ZA03611
*                     Z40LPTA                                  @Z40LPTA
*                     Z40BPSV                                 @Z40BPSV*
*                     G17SPTJ                                 @G17SPTJ*
*                     ZA10516                                 @ZA10516*
*                     ZA12152                                 @ZA12152*
*                     G33CPMR                                 @G33CPMR*
*                     ZA16595                                 @ZA16595*
*                     ZA29986                                 @ZA29986*
*                                                                     *
* MESSAGES         =  THE FOLLOWING MESSAGE IS CONTAINED IN     Y02669*
*                     AND ISSUED FROM IEE0503D:                 Y02669*
*                                                                     *
*                      MSGID  CODE     MESSAGE TEXT             Y02669*
*                      _____  ____     ____________             Y02669*
*                     IEE305I X'05'    COMMAND INVALID          Y02669*
*                     IEE311I X'0B'    PARAMETER MISSING        YM1917*
*                                                                     *
*                     THE FOLLOWING MESSAGES ARE CONTAINED IN   Y02669*
*                     AND ISSUED FROM IEE5603D AND IEE5903D:    Y02669*
*                                                                     *
*                      MSGID  CODE  IN MESSAGE TEXT             Y02669*
*                      _____  ____  __  ____________            Y02669*
*                     IEE156I X'04' 56 INVALID OPERAND          Y02669*
*                     IEE345I X'08' 56 AUTHORITY INVALID        Y02669*
*                     IEE921I X'09' 56 REJECTED                 Y02669*
*                             X'0C' 59 DISPLAY ALREADY EXISTS   Y02669*
*                             X'0D' 59 DISPLAY AREA BUSY        Y02669*
*                             X'0E' 59 NO TRACK TO STOP         Y02669*
*                             X'19' 59 NEEDS DISPLAY AREA       Y02669*
*                             X'1F' - SUBSYSTEM CONSOLE        @Y02668P
*                     IEE925I X'0B' 56 OPERAND                  Y02669*
*                             X'1A' 59 FOR CRT ONLY             Y02669*
*                     IEE926I X'0C' 56 L OPERAND INVALID        Y02669*
*                             X'12' 59 NOT AN AREA              Y02669*
*                             X'13' 59 NOT A CONSOLE            Y02669*
*                             X'14' 59 FULL CAPABILITY          Y02669*
*                             X'15' 59 DYNAMIC DISPLAY IN AREA  Y02669*
*                             X'1F' 59 SUBSYSTEM CONSOLE       @Y02668P
*                                                                     *
* SYSGEN           =  INCLUDED WITH SVC 34 IN LOAD MODULE       Y02669*
*                     IGC0003D, DLIB AOSB3, MACRO SGIEF441,     Y02669*
*                     TARGLIB LPALIB                            Y02669*
*                                                                     *
* SYSTEM LIBRARIES =  NONE                                      Y02669*
*                                                                     *
***********************************************************************
         EJECT
IEE7503D CSECT                                                   Y02669
         ENTRY IEE7603D            ENTRY POINT FOR JES2 ONLY     Y02669
IEE7603D EQU   *                                                 Y02669
* A-190710,190726,190741 C- 190740,190744                      @ZM30445
* C- 461000 D - 539043-539122                                  @ZM30446
* A-208600                                                     @ZA02576
* THIS APAR WRITTEN TO SUPPORT MSS 'D 3850' S/C Z40LPTA        @ZA07152
* A - CODE ADDED TO SUPPORT 'DISPLAY DUMP'                     @G33CPMR
* A (COMTASK ) APPROX. 111724                                  @ZA12152
* A (KNOL    ) APPROX. 187600                                  @ZA12152
* A (NCCHAR  ) APPROX. 602920                                  @ZA12152
* C (ERRONL  ) ADD LABEL LFIRST                                @ZA16595
* C (NEXTAREA) TEST FOR POSSIBLE GENED AREAS NOT IN USE        @ZA29986
*
* *********************************************************************
*                                                                     *
*                                REGISTER USAGE                       *
*                                                                     *
***********************************************************************
REG0     EQU   0                   USED WITH REG1 AS EVEN/ODD PAIR
REG1     EQU   1                   PARM REG FOR INTERNAL SUBROUTINES
*                                  USED WITH REG0 AS EVEN/ODD PAIR
XSABASE  EQU   2                   INPUT-BASE REGISTER FOR XSA
UCMBASE  EQU   3                   BASE REGISTER FOR UCM
WORK4    EQU   4                   WORK REG, PART OF EVEN/ODD PAIR
WORK5    EQU   5                   WORK REG, PART OF EVEN/ODD PAIR
AREA     EQU   6                   TARGET AREA ID
CONSID   EQU   7                   TARGET CONSOLE ID
OPPTR    EQU   8                   POINTS WITHIN COMMAND OPERAND FIELD
RCTSCAN  EQU   9                   WORK REG FOR RCT SCAN
RDCMBASE EQU   10                  BASE REGISTER FOR RESIDENT DCM
RCTBASE  EQU   11                  BASE ROUTING CONTROL TABLE
SACBBASE EQU   11                  BASE SCREEN AREA CONTROL BLOCK
UCMEBASE EQU   12                  BASE REG - UCME (SOURCE THEN TARGET)
MYBASE   EQU   13                  BASE REGISTER FOR THIS ROUTINE
RETURN14 EQU   14                  RETURN REGISTER               Y02669
R15      EQU   15
         EJECT
***********************************************************************
*                                                                     *
*                           CONSTANTS                                 *
*                                                                     *
***********************************************************************
E0       EQU   0                   CONSTANT
E1       EQU   1                   CONSTANT
E2       EQU   2                   CONSTANT
E3       EQU   3                   CONSTANT
E4       EQU   4                   CONSTANT
E5       EQU   5                   CONSTANT                      Y02669
E6       EQU   6                   CONSTANT                      Y02669
E7       EQU   7                   CONSTANT - MASK FOR ICM INSTRUCTION
E8       EQU   8                   CONSTANT - MASK FOR ICM INSTRUCTION
***********************************************************************
*                                                                     *
*                         CHARACTER EQUATES                           *
*                                                                     *
***********************************************************************
BLANK    EQU   C' '                BLANK
COMMA    EQU   C','                COMMA
ACHAR    EQU   C'A'                CHARACTER A FOR COMPARE       Y02669
CCHAR    EQU   C'C'                CHARACTER C FOR COMPARE       Y02669
DCHAR    EQU   C'D'                CHARACTER D FOR COMPARE
KCHAR    EQU   C'K'                CHARACTER K FOR COMPARE
OCHAR    EQU   C'O'                CHARACTER O FOR COMPARE
PCHAR    EQU   C'P'                CHARACTER P FOR COMPARE     @G33CPMR
VCHAR    EQU   C'V'                CHARACTER V                   Y02669
TCHAR    EQU   C'T'                CHARACTER T                   Y02669
Z        EQU   C'Z'                ID OF MESSAGE STREAM AREA (INLINE)
***********************************************************************
*                     BRANCH CONDITION CODES                          *
***********************************************************************
NOMORE   EQU   8                   BRANCH ON ZERO
EQ       EQU   8                   BRANCH ON EQUAL
NONEON   EQU   8                   BRANCH ON ZERO
***********************************************************************
*                                                                     *
*                         VERB CODES                                  *
*                                                                     *
***********************************************************************
K        EQU   X'90'               VERB CODE FOR K (CONTROL)
D        EQU   X'68'               VERB CODE FOR D (DISPLAY)
DR       EQU   X'42'               VERB CODE FOR D R             Y02669
DU       EQU   X'52'               VERB CODE FOR D U             Y02669
D38      EQU   X'F3'               VERB CODE FOR D 3850        @Z40LPTA
TR       EQU   X'C4'               VERB CODE FOR TR (TRACK)      Y02669
PT       EQU   X'C8'               VERB CODE FOR PT (STOPTR)     Y02669
JES2     EQU   34                  VERB CODE FOR JES2            Y02669
NULL     EQU   X'00'               CODE INDICATES NO AREA SPECIFIED
*                                  OR DEFAULT PROVIDED BY MSGRT
F0       EQU   X'F0'               MASK FOR DETERMINING NUMERICS
C0       EQU   X'C0'               MASK FOR DETERMINING LETTERS  Y02669
DBLWORD  EQU   8                   LENGTH OF A DOUBLE WORD
PTRCVT   EQU   16                  LOCATION OF POINTER TO CVT
ASTRICK  EQU   X'5C'               SPECIAL AREA FILLER         @Z40LPTA
XFF      EQU   X'FF'               MASK FOR NI                   Y02651
         EJECT
***********************************************************************
*                                                                     *
*                     ERROR MESSAGE CODES                             *
*                                                                     *
***********************************************************************
MSG156   EQU   04                  IEE156I CM INVALID OPERAND -  Y02669
MSG305   EQU   05                  IEE305I CM COMMAND INVALID    Y02669
MSG311   EQU   11                  IEE311I PARAMETER MISSING     YM1917
MSG345   EQU   08                  IEE345I CM AUTHORITY INVALID  Y02669
MSG921   EQU   09                  IEE921I CM OP REJECTED        Y02669
MSG921A  EQU   12                       DISPLAY ALREADY EXISTS   Y02669
MSG921B  EQU   13                       DISPLAY AREA BUSY        Y02669
MSG921C  EQU   14                       NO TRACK TO STOP         Y02669
MSG921D  EQU   25                       NEEDS DISPLAY AREA       Y02669
MSG921E  EQU   31                       SUBSYSTEM CONSOLE      @Y02668P
MSG925   EQU   11                  IEE925I 'COMMAND' OPERAND     Y02669
MSG925A  EQU   26                       FOR CRT ONLY             Y02669
MSG926   EQU   12                  IEE926I L=CCA OPERAND INVALID Y02669
MSG926A  EQU   18                       CCA NOT AN AREA          Y02669
MSG926B  EQU   19                       CC NOT A CONSOLE         Y02669
MSG926C  EQU   20                       CC FULL CAPABILITY       Y02669
MSG926D  EQU   21                       DYNAMIC DISPLAY IN AREA  Y02669
MSG926E  EQU   31                       SUBSYSTEM CONSOLE      @Y02668P
MSG927   EQU   13                  IEE927I REQUEST AMBIGIOUS     YM1557
MSG927A  EQU   23                       SPECIFY L OPERAND        YM1557
***********************************************************************
*                                                                     *
*                 JES2 INTERFACE RETURN CODES                    Y02669
*                                                                     *
***********************************************************************
OKCCA    EQU   0                   0 - VALID ROUTING REQUEST     Y02669
AREANUSE EQU   4                   4 - ANOTHER DISPLAY IN AREA   Y02669
TRINAREA EQU   8                   8 - TRACK DISPLAY IN AREA     Y02669
BADCONSL EQU   12                  12 - INVALID CONSOLE ID       Y02669
*                                       CONSOLE ID TOO HIGH      Y02669
*                                       CONSOLE NOT ACTIVE       Y02669
*                                       CONSOLE FULL CAPABILITY  Y02669
BADAREA  EQU   16                  16 - INVALID AREA REQUEST     Y02669
         EJECT
***********************************************************************
*                                                                     *
*                ESTABLISH ADDRESSABILITY                             *
*                REGISTER 2 POINTS TO XSA                             *
*                INITIALIZE ROUTING FIELD (XAS) TO ZEROES             *
*                OBTAIN THE LOCAL AND CMS LOCKS                       *
* NOTE: IF REQUEST FROM JES2 BRANCH TO SPECIAL CODE AT JES2CODE       *
***********************************************************************
         USING DXSA,XSABASE        BASE XSA                      Y02669
         STM   REG0,RETURN14,XASAVSTD   SAVE REGISTERS FOR JES2  Y02669
         SPACE
         BALR  MYBASE,0
         USING *,MYBASE            BASE THIS MODULE
         MODID BRANCH=YES                                        Y01886
         L     WORK4,PTRCVT        GET CVT POINTER               Y02669
         USING DCVT,WORK4          BASE CVT                      Y02669
         L     WORK5,CVTTCBP       SET TCB POINTER             @Y02668P
         L     UCMBASE,CVTCUCB     GET UCM POINTER
         DROP  WORK4                                             Y02669
         USING TCBASCB,WORK5       BASE IEFPTRS                @Y02668P
         L     WORK4,TCBPTR        CURRENT TCB ADDRESS,SAVE    @Y02668P
*                                  WORK4 UNTIL AFTER THE TCB   @Y02668P
*                                  IS IDENTIFIED               @Y02668P
         DROP  WORK5
         USING UCM,UCMBASE         BASE UCM                      Y02669
         USING UCMLIST,UCMEBASE    BASE UCM ENTRY
         USING DCMTSRT,RDCMBASE    BASE RESIDENT DCM             Y02669
         USING DRCT,RCTBASE        BASE ROUTING CONTROL TABLE
         SPACE 2
LOCKSET  EQU   *                                                 Y02651
         STM   RCTBASE,RETURN14,XASAVLOC                         Y02651
         SETLOCK OBTAIN,TYPE=LOCAL,MODE=UNCOND,RELATED=(K,TR,D,PT,     *
               LOCKSOFF,IEE5503D,5603D,5903D,6703D,6803D,6903D,7703D,  *
               7803D)
         SETLOCK OBTAIN,TYPE=CMS,MODE=UNCOND,RELATED=(K,TR,D,PT,       *
               LOCKSOFF,IEE5503D,5603D,5903D,6703D,6803D,6903D,7703D,  *
               7803D)
         LM    RCTBASE,RETURN14,XASAVLOC                         Y02651
         CLI   XAN,JES2                 REQUEST FROM JES2?     @YM4796P
         BE    JES2CODE                 YES,BR TO  JES2 CODE   @YM4796P
         XC    XAS(DBLWORD),XAS         ZERO OUT XAS           @YM2773P
         OI    XASSDS,XASSDS5           INDICATE LOCKS HELD    @YM2773P
         C     WORK4,UCMPXA        CURRENT TCB = COMTASK?      @Y02668P
         BE    COMTASK             YES,BRANCH                  @Y02668P
         OI    XASSDS,XASSDS3      NO,ANOTHER TASK ISSUE CMD   @Y02668P
COMTASK  EQU   *
         EJECT
***********************************************************************
*                                                                     *
*     INVOKE IEEBUFSC TO SCAN THE OPERAND BUFFER POINTED TO BY XAL    *
*     FOR THE L= OPERAND AND RETURN THE ADDRESS OF THE 'CCA' VALUE    *
*     AND ITS LENGTH.                                                 *
*                                                                     *
*     OPERATION  1. SET UP PTR TO BUFFER -1 AND PTR TO KEYWORD LENGTH *
*                2. IF BUFFER PTR = SPECIFIED KEYWORD GOTO (3).       *
*                   A. IF NOT, INCREMENT BUFFER PTR BY ONE.           *
*                   B. IF PTR IS AT END OF BUFFER SET UP NOT FOUND    *
*                      RETURN CODE (8) IN R15 AND EXIT.               *
*                   C. IF PTR IS AT A COMMA, GO BACK TO OPERATION 2.  *
*                   D. IF PTR IS AT A BLANK SET UP NOT FOUND RETURN   *
*                      CODE (8) IN R15 AND EXIT.                      *
*                   E. OTHERWISE GO TO OPERATION 2.A.                 *
*                3. AT FOUND ROUTINE SET UP R14 TO POINT TO KEYWORD   *
*                   VALUE                                             *
*                4. SCAN KEYWORD VALUE TO THE END OF THE BUFFER, A    *
*                   COMMA OR A BLANK                                  *
*                5. CALCULATE LENGTH OF KEYWORD VALUE.                *
*                6. IF LENGTH IS < OR = ZERO, SET UP AN INVALID KEYWORD
*                   VALUE LENGTH RETURN CODE (4) IN R15 AND EXIT.     *
*                7. OTHERWISE, SET GOOD RETURN CODE (0) IN R15 AND EXIT
*                                                                     *
*    INPUT       R0 - PTR TO LAST BYTE OF BUFFER + 1                  *
*                R1 - PTR TO BEGINNING OF BUFFER                      *
*                R15- PTR TO KEYWORD WITH THE LENGTH OF THE KEYWORD   *
*                     IN THE HIGH ORDER BYTE.                         *
*                     EG. KEYWORD FORMAT 'DSNAME=' R15 WOULD POINT    *
*                     TO THE 'D' WITH A 6 IN THE HIGH ORDER BYTE.     *
*                                                                     *
*     OUTPUT     R1   LENGTH OF KEYWORD VALUE                         *
*                R14  PTR TO FIRST BYTE OF KEYWORD VALUE IN BUFFER    *
*                     OR WORKAREA                                     *
*                R15  RETURN CODE OF ZERO.                            *
*                                                                     *
*     ERROR      R15  CONTAINS 4 IF KEYWORD HAS AN INVALID LENGTH     *
*     OUTPUT                   8 IF KEYWORD WAS NOT FOUND IN BUFFER   *
*                                                                     *
*     EXIT       TO NEXT SEQUENTIAL INSTRUCTION AFTER MACRO CALL      *
*                                                                     *
*     NOTES      LISTS OF OPERANDS IN PARENTHESES ARE NOT PROCESSED.  *
*                THIS MACRO IS INTENDED TO PROCESS ONLY A SINGLE      *
*                VALUE FOR A GIVEN KEYWORD.                           *
*                                                                     *
***********************************************************************
         EJECT
         ICM   REG0,E7,XAP+1       R0 PTR TO BUFFER END + 1      Y02669
         ICM   REG1,E7,XAL+1       R1 PTR TO BUFFER BEGINNING    Y02669
         BZ    KNOL                BRANCH IF NO POINTER        @ZA12152
         CLC   E0(E2,REG1),NCCHAR  IS COMMAND K N,PFK          @ZA12152
         BE    KNOL                YES, BYPASS SCAN FOR L=     @ZA12152
         LA    R15,LEQUALS         R15 PTR TO KEYWORD EQUATE     Y02669
         ICM   R15,E8,C1               HI-ORDER BYTE IS LENGTH   Y02669
         LR    WORK5,RETURN14      SAVE RETURN ADDRESS           Y02669
         IEEBUFSC                                                Y02669
***********************************************************************
*                                                                     *
*        REG1 - LENGTH OF VALUE FOLLOWING THE KEYWORD                 *
*               STORED IN XASAVSTD                                    *
*        RETURN14 - PTR TO FIRST BYTE OF KEYWORD VALUE                *
*                   LOADED INTO OPPTR                                 *
*        REG15 - RETURN CODE 0 - ALL WENT WELL                        *
*                            4 - PTR TO END OF BUFFER + 1 WAS WRONG   *
*                            8 - KEYWORD WAS NOT FOUND                *
*        XASAVSTD - LENGTH OF 'CCA' INFORMATION                       *
*        XASTSAVE - PTR TO 'C' OF L=CCA                               *
*        XASAVSTD+4 - PTR TO , PRECEDING OPERAND                      *
*        XASAVSTD+8 - PTR TO 'L' OF L=CCA                             *
*                                                                     *
*  ERR - IEE156I INVALID OPERAND - L= IS FIRST OPERAND ON COMMAND     *
***********************************************************************
         SPACE 2
         CH    R15,C4               DETERMINE RETURN CODE        Y02669
         BH    NOL                  8 - NO 'L' IN OPERAND BUFFER Y02669
         BL    OPFOUND              0 - L OPERAND WAS FOUND      Y02669
ERRONL   EQU   *                                                 Y02669
         SH    RETURN14,C2              POINT TO 'L'           @YM5459P
LFIRST   LR    OPPTR,RETURN14           FOR IEE5603D           @ZA16595
         LR    RETURN14,WORK5       4 - ERROR CONDITION          Y02669
         B     ERR156                   IEE156I INVALID OPERAND  Y02669
OPFOUND  EQU   *
         ST    REG1,XASAVSTD        SAVE LENGTH OF KEYWORD VALUE Y02669
         LR    OPPTR,RETURN14           SAVE ADDR OF 'CCA' VALUE Y02669
         ST    OPPTR,XASTSAVE           SAVE CURSOR POINTER      Y02669
         BCTR  RETURN14,0               POINT TO EQUAL SIGN      Y02669
         BCTR  RETURN14,0               POINT TO 'L'             Y02669
         ST    RETURN14,XASAVSTD+E8     SAVE ADDR OF 'L'         Y02669
         BCTR  RETURN14,0               POINT TO PRECEDING ,     Y02669
         CLI   E0(RETURN14),COMMA       'L=' NOT ALLOWED AS    @ZA16595
*                                       FIRST OPERAND: TEST IT @ZA16595
         BE    LCOM                     NOT FIRST: OK          @ZA16595
         LA    RETURN14,E1(RETURN14)    FIRST: ERROR, POSITION @ZA16959
*                                       CURSOR TO 'L' AND      @ZA16959
         B     LFIRST                   GO ISSUE ERROR MESSAGE @ZA16959
LCOM     ST    RETURN14,XASAVSTD+E4     SAVE ADDR OF ,           Y02669
         LR    RETURN14,WORK5           RESTORE REGISTER 14      Y02669
         EJECT
***********************************************************************
*                                                                     *
*                'L' OPERAND FOUND ON COMMAND                         *
*                                                                     *
*  ERR - IEE156I INVALID OPERAND - 'L=CCA' INVALID FROM TSO TERMINAL  *
*                                   'CC' = 0000                       *
***********************************************************************
         SPACE 2
LFOUND   EQU   *
         CLC   XAA(E2),ZERO        IS COMMAND FROM TSO TERMINAL? Y02669
         BNE   ERR305              YES, NO L OPERAND ALLOWED     YM1917
         CLC   E0(E2,OPPTR),F0F0   DOES L=00? - INVALID CONSOLE  Y02669
         BE    ERR926B             ID - ERROR MSG                Y02669
         OI    XASSDS,XASSDS2      INDICATE L OPERAND SPECIFIED ON CMD
         TM    E0(OPPTR),F0        1ST CHAR A NUMBER (FIRST 4 BITS F)?
         BNO   CCNTSPEC            BRANCH IF NOT A NUMBER (NO CC)
         SPACE 2
***********************************************************************
*                                                                     *
*                'CC' OR 'C' SPECIFIED ON COMMAND                     *
*                                                                     *
*  ERR - IEE926I 'CC' NOT A CONSOLE - TARGET CONSOLE ID WAS INVALID   *
***********************************************************************
         SPACE  2
         TM    E1(OPPTR),F0        IS SECOND CHARACTER A NUMBER? Y02669
         BO    TWODIGIT            YES, TWO-DIGIT NUMBER
         PACK  XAP(DBLWORD),E0(E1,OPPTR)  PACK THE CONSOLE ID
         LA    OPPTR,E1(OPPTR)     OPPTR=1ST CHAR AFTER 'CC'     Y02669
         B     CHECKCC             VALIDITY CHECK
TWODIGIT EQU   *
         PACK  XAP(DBLWORD),E0(E2,OPPTR)  PACK THE CONSOLE ID
         LA    OPPTR,E2(OPPTR)     OPPTR=1ST CHAR AFTER 'CC'     Y02669
CHECKCC  EQU   *
         CVB   CONSID,XAP          CONVERT CONSOLE ID TO BINARY
         STC   CONSID,XASCID       STORE CONSOLE ID IN XSA
         L     UCMEBASE,UCMVEL     BASE LAST UCMENTRY
         CLC   XASCID(E1),UCMID    ID HIGHER THAN HIGHEST VALID  Y02669
         BH    ERR926B             CONSOLES ID? YES, ERROR       Y02669
         EJECT
***********************************************************************
*                                                                     *
*    CHECK IF AREA ID IS SPECIFIED. AT THIS POINT OPPTR POINTS TO     *
*    THE FIRST CHAR AFTER THE CONSOLE ID                              *
*    IF NO 'A' IS SPECIFIED XASDID WILL REMAIN X'00' - NULL           *
*                                                                     *
***********************************************************************
         SPACE  2
         CLI   E0(OPPTR),BLANK     IS POINTER AT END OF COMMAND? Y02669
         BE    REMOVEL             YES, NO 'A' IS SPECIFIED
         CLI   E0(OPPTR),COMMA     IS POINTER AT END OF OPERAND? Y02669
         BE    REMOVEL             NO , NO 'A' IS SPECIFIED
         SPACE
***********************************************************************
*                                                                     *
*                'A' OPERAND FOUND ON COMMAND                         *
*                                                                     *
*    IT IS ASSUMED THAT THE FIRST CHAR AFTER THE CONSOLE ID           *
*    (OR AFTER THE = IF CC IS NOT SPECIFIED) IS THE AREA ID.          *
*    IF THE ID IS NOT VALID IT WILL BE DISCOVERED WHEN LOOKING FOR    *
*    AN SACB TO MATCH IT.                                             *
*                                                                     *
*  ERR - IEE156I INVALID OPERAND - 'A' WAS MORE THAN 1 CHARACTER      *
***********************************************************************
         SPACE  2
STOREA   EQU   *
         TM    E0(OPPTR),C0        1ST CHAR A LETTER (FIRST 3 BITS F)?
         BNO   ERR926A             IF NOT LETTER-INVALID AREA    Y02669
         MVC   XASDID(E1),E0(OPPTR)  MOVE AREA ID INTO XSA       Y02669
         LA    OPPTR,E1(OPPTR)     INCREMENT POINTER TO NEXT CHARY02669
         CLI   E0(OPPTR),BLANK     IS POINTER AT END OF COMMAND? Y02669
         BE    REMOVEL             YES, BRANCH TO REMOVE L
         CLI   E0(OPPTR),COMMA     IS POINTER AT END OF OPERAND  Y02669
         BE    REMOVEL             YES, BRANCH TO REMOVE L
         B     ERR156              BRANCH TO ERROR ROUTINE       Y02669
         EJECT
***********************************************************************
*                                                                     *
*                'L' OPERAND WAS NOT SPECIFIED ON THE COMMAND         *
*         CHECK RCT FOR ROUTING DEFAULTS SET BY MSGRT COMMAND         *
*                                                                     *
***********************************************************************
         SPACE  2
NOL      EQU   *                   NO L OPERAND WAS FOUND
         LR    RETURN14,WORK5      RESTORE R14 AFTER IEEBUFSC    Y02669
KNOL     EQU   *                  COMMAND WAS N,PFK BUFSC SKIP @ZA12152
         CLC   XAA(E2),ZERO        IS COMMAND FROM TSO TERMINAL? Y02669
         BNE   TESTOPER            YES, CHECK DISPLAY OPERANDS   Y02669
         SPACE  2
***********************************************************************
*                                                                     *
*                'CC' NOR 'C' WAS SPECIFIED ON THE COMMAND            *
*                                                                     *
***********************************************************************
         SPACE  2
CCNTSPEC EQU   *
         SR    REG1,REG1          CLEAR PARAMETER REG
         ICM   REG1,E1,XAU        GET SOURCE CONSOLE ID          Y02669
         BNZ   NOTINTEN           IF ZERO NO DEFAULTING          YM1557
INTERNAL EQU   *                                                 YM1557
         CLI   XAV,DCHAR          ONLY A DISPLAY COMMAND CAN BE  YM1557
*                                 ISSUED INTERNALLY WITHOUT    @YM7698P
*                                 THE L OPERAND. THE K, TR AND @YM7698P
*                                 STOPTR COMMANDS CAN NOT BE   @YM7698P
*                                 ISSUED INTERNALLY WITHOUT    @YM7698P
         BNE   RETURN             THE L OPERAND                @YM7698P
         LR    WORK4,UCMBASE      GET ADDRESS OF UCM             YM1557
         LA    WORK5,E4           PUT 4 IN REG5 FOR SUBSTRACT    YM1557
         SR    WORK4,WORK5        UCM MINUS 4 AND                YM1557
         L     WORK4,E0(WORK4)    GET REAL ADDRESS OF UCM PREF @ZA03611
         USING UCMPRFX,WORK4      ESTABLISH UCM PREFIX BASE      YM1557
         L     UCMEBASE,UCMMCENT  GET UCME ADDR FOR MASTER       YM1557
         DROP  WORK4                                             YM1557
         MVC   XASCID(E1),UCMID   USE MASTER CONSOLE ID AS       YM1557
         MVC   XAU(E1),UCMID      BOTH SOURCE AND TARGET       @ZM30445
         MVI   XASDID,Z           DEFAULT WITH INSTREAM          YM1557
         TM    XASSDS,XASSDS2     L OPERAND SPECIFIED ?        @ZM30445
         BNO   TSTCMND            NO,CHECK COMND BEING ISSUED  @ZM30445
         LA    OPPTR,E1(OPPTR)    BUMP OPERAND PTR             @ZM30445
         B     REMOVEL            YES, GO REMOVE IT            @ZM30445
***********************************************************************
*                  GET ISSUING CONSOLE'S UCM ENTRY                    *
***********************************************************************
NOTINTEN EQU   *                                                 YM1557
         BAL   R15,FINDUCME        BASE SOURCE UCM ENTRY         Y02669
         EJECT
***********************************************************************
*                                                                     *
*                 GET ISSUING CONSOLE'S ROUTING CONTROL TABLE         *
*                                                                     *
***********************************************************************
         SPACE  2
         ICM   RCTBASE,E7,UCMRCTA  SEE IF RCT IS PRESENT         Y02669
         BZ    NOCOMP              NO, ROUTE TO ISSUING CONSOLE
         ST    RCTBASE,XAD         SAVE BASE RCT ADDRESS         Y02669
ACTIVE   EQU   *                                                 Y02669
         TM    RCTCODE,RCTACT      IS ENTRY ACTIVE?              Y02669
         BO    COMPCMD             YES, GO COMPARE               Y02669
LASTRCT  EQU   *                                                 Y02669
         TM    RCTCODE,RCTCON      NO, IS IT LAST ENTRY?         Y02669
         BO    NEXTRCT             NO, LOAD NEXT POINTER         Y02669
         BM    NOCOMP              YES, COMPARE NOT FOUND              X
                                        ROUTE TO ISSUING
         LA    RCTBASE,RCTENT2     POINT TO NEXT ENTRY
         B     ACTIVE              GO TEST IT
NEXTRCT  EQU   *                                                 Y02669
         ICM   RCTBASE,E7,RCTCODE+1 LOAD NEW POINTER             Y02669
         BZ    NOCOMP              IF NONE, ROUTE TO ISSUING     Y02669
         B     ACTIVE              GO TEST IT                    Y02669
         SPACE
***********************************************************************
*         CHECK RCT FOR ENTRY FOR COMMAND FOR A DEFAULT 'CC' VALUE    *
***********************************************************************
COMPCMD  EQU   *                                                 Y02669
         CLC   XAV(E1),RCTVER      DO COMMAND AND TABLE MATCH?
         BE    OPTEST              YES, GO TEST OPERAND
         CLI   XAV,CCHAR          NO, IS COMMAND VERB 'CONTROL'? Y02669
         BNE   TRCMD               NO, IS CMD VERB TR?           Y02669
         CLI   RCTVER,KCHAR        YES, IS TABLE ENTRY 'K'?
         BE    TESTENT             YES, BYPASS OPERAND CHECK     Y02669
         B     LASTRCT             NO, RETURN TO TABLE SCAN      Y02669
TRCMD    EQU   *                                                 Y02669
         CLI   RCTVER,TCHAR        IS TABLE ENTRY 'T'?           Y02669
         BNE   LASTRCT             NO, RETURN TO TABLE SCAN      Y02669
         CLI   XAN,PT              IS COMMAND VERB 'STOPTR'?     Y02669
         BE    TESTENT             YES, BYPASS OPERAND CHECK     Y02669
         B     LASTRCT             NO, RETURN TO TABLE SCAN      Y02669
         SPACE  2
OPTEST   EQU   *
         CLI   RCTVER,KCHAR        IS TABLE ENTRY 'K'?           Y02669
         BE    TESTENT             YES, BYPASS OPERAND TEST      Y02669
         CLI   RCTVER,TCHAR        IS TABLE ENTRY 'T'FOR TR?     Y02669
         BE    TESTENT             YES, BYPASS OPERAND TEST      Y02669
         L     RCTSCAN,XAL         POINT TO OPERAND
         CLC   E0(E2,RCTSCAN),DCBLANK   IS OPERAND D C         @YM4791P
         BE    DCOPER                                          @YM4791P
         CLC   E0(L'CONSOLES,RCTSCAN),CONSOLES   IS OPERAND 'CONSOLES'?
         BE    DCOPER              YES, COMPARE WITH RCT         YM1916
         CLI   E0(RCTSCAN),DCHAR   IS 1ST CHAR A 'D'?          @G33CPMR
         BNE   PROCEED             NO, AVOID 'D' CHAR. TESTS   @G33CPMR
         CLC   E0(L'DUMP,RCTSCAN),DUMP IS OPERAND 'DUMP'?      @G33CPMR
         BE    DDOPER              YES, COMPARE WITH RCT       @G33CPMR
         CLC   E0(L'DMN,RCTSCAN),DMN IS OPERAND 'DMN'?         @G33CPMR
         BNE   DDOPER              NO, COMPARE WITH RCT        @G33CPMR
PROCEED  EQU   *                                               @G33CPMR
         CLC   RCTOPD(E1),E0(RCTSCAN)  DOES OPERAND MATCH TABLE? YM1916
         BE    TESTENT             YES, GO TO INSERT VALUES      Y02669
         BNE   DACTCMD             NO                            Y02669
DCOPER   EQU   *                                                 Y02669
         CLI   RCTOPD,OCHAR        YES, IS TABLE ENTRY 'O'?
         BNE   LASTRCT             NO, RETURN TO TABLE SCAN      Y02669
         B     TESTENT             YES, GO INSERT VALUES         Y02669
DDOPER   EQU   *                                               @G33CPMR
         CLI   RCTOPD,PCHAR        IS TABLE ENTRY 'P'?         @G33CPMR
         BNE   LASTRCT             NO, RETURN TO TABLE SCAN    @G33CPMR
         B     TESTENT             YES, GO TO VALUE INSERTION  @G33CPMR
         SPACE 2
DACTCMD  EQU   *                                                 Y02669
         CLI   RCTOPD,ACHAR        IS TABLE ENTRY 'A'?           Y02669
         BNE   LASTRCT             NO, RETURN TO TABLE SCAN      Y02669
         CLI   XAN,D               IS COMMAND VERB D JOBS; TS; A?Y02669
         BNE   LASTRCT             NO, RETURN TO TABLE SCAN      Y02669
         EJECT
***********************************************************************
*                                                                     *
*  IF NO DEFAULT WAS FOUND IN RCT THE DISPLAY IS ROUTED TO THE        *
*  ISSUING CONSOLE                                                    *
*  'CC' EQUALS XAU VALUE AND 'A' EQUALS X'00'                         *
*  OTHERWISE, THE DEFAULTS FOUND IN THE RCT ARE STORED IN THE XSA.    *
*                                                                     *
*  IF 'A' WAS SPECIFIED ON THE COMMAND ITS VALUE OVERRIDES RCT VALUE  *
*                                                                     *
***********************************************************************
         SPACE  2
TESTENT  EQU   *
         OI    XASSDS,XASSDS4      SET 'L' OPERAND FROM RCT    @ZA02576
         MVC   XASDID(E1),RCTAID   STORE AREA ID (00-NONE)       Y02669
         CLI   XASDID,ASTRICK       THIS SPECIAL RCTAID FILLER @Z40LPTA
         BNE   NOTAST              NO, PROCEED                 @Z40LPTA
         MVI   XASDID,NULL         RE-SET TO NULL (00)         MZ40LPTA
NOTAST   EQU   *                                               @Z40LPTA
         CLI   RCTCID,NULL         IS CONSOLE ID PROVIDED IN RCT?Y02669
         BE    NOCOMP              NO, ROUTE TO ISSUING          Y02669
         MVC   XASCID(E1),RCTCID   YES,STORE CONSOLE ID FROM RCT Y02669
         B     LTEST               SEE IF L OPERAND WAS PRESENT
NOCOMP   EQU   *                                                 Y02669
         MVC   XASCID(E1),XAU      PUT SOURCE ID IN XSA
LTEST    EQU   *
         TM    XASSDS,XASSDS2      WAS L OPERAND PRESENT ON COMMAND?
         BO    STOREA              YES, GET AREA OPERAND FROM IT
         B     VALIDITY            NO, VALIDITY CHECK VALUES     Y02669
         EJECT
***********************************************************************
*                                                                     *
*    'L' OPERAND IS REMOVED FROM COMMAND SO THAT OTHER COMMAND        *
*    HANDLERS WILL NOT SEE IT                                         *
*    AT ENTRY HERE OPPTR POINTS TO THE BLANK OR COMMA WHICH DELIMITS  *
*    THE 'L' OPERAND.                                                 *
*    A POINTER TO THE 'L' IS SAVED IN XASAVSTD + 8                    *
*                     ',' PRECEDING THE OPERAND IN XASAVSTD+4         *
*   THE LENGTH OF THE OPERAND IS SAVED IN XASAVSTD                    *
*                                                                     *
***********************************************************************
         SPACE
MOVEOP   MVC   E0(E0,WORK5),E0(OPPTR)  PUSH REST OF OPERANDS OVER
BLANKOUT MVC   E1(E0,WORK5),E0(WORK5)  BLANK OUT FIELD POINTED TO
*                                       BY WORK5
         SPACE
REMOVEL  EQU   *
         L     WORK4,XASAVSTD      GET LENGTH OF L OPERAND       Y02669
         LA    WORK4,E1(WORK4)     LESS 2 FOR PROPAGATE & EX     Y02669
         CLI   E0(OPPTR),BLANK     IS POINTER AT END OF COMMAND? Y02669
         BNE   REMOVELC            NO, MOVE REST OF COMMAND OVER
         SPACE
***********************************************************************
*                                                                     *
*          'L' OPERAND IS AT END OF COMMAND SO JUST BLANK IT OUT      *
*                                                                     *
***********************************************************************
         SPACE
         L     WORK5,XASAVSTD+E4   GET ADDRESS OF COMMA BEFORE L Y02669
         MVI   E0(WORK5),BLANK     BLANK FIRST BYTE, THEN        Y02669
         EX    WORK4,BLANKOUT      BLANK OUT REST OF OPERAND     Y02669
         B     VALIDITY            CHECK CONSOLE AND AREA VALIDITY
         SPACE 2
***********************************************************************
*                                                                     *
*        'L' OPERAND IS NOT LAST OPERAND. REMOVE IS, MOVE THE REST OF *
*       THE COMMAND OVER, AND BLANK WHERE COMMAND WAS MOVED FROM      *
*                                                                     *
***********************************************************************
         SPACE
REMOVELC EQU   *
         L     WORK5,XAR           GET PARM LIST POINTER       @ZA10516
         LA    WORK5,E0(WORK5)     COMPUTE END OF PARM LIST BY:@ZA10516
         LR    R15,WORK5           GET BEGINNING ADDR OF LIST  @ZA10516
         AH    R15,0(WORK5)        ADD LENGTH OF PARM LIST     @ZA10516
         BCTR  R15,E0              POINT TO LAST CHARACTER       Y02669
         SR    R15,OPPTR           GET LENGTH OF OPERAND FIELD   Y02669
*                                  AFTER L OPERAND (-1 FOR EX)   Y02669
         L     WORK5,XASAVSTD+E4   GET ADDRESS OF COMMA BEFORE L Y02669
         EX    R15,MOVEOP          PUSH REST OF OPERANDS OVER    Y02669
         LA    WORK5,E1(WORK5)     CALCULATE ADDRESS OF FIELD TO Y02669
         AR    WORK5,R15             BE BLANKED                  Y02669
*                                  (WORK4 STILL HAS ADJUSTED     Y02669
*                                       LENGTH OF L OPERAND)     Y02669
         MVI   E0(WORK5),BLANK     BLANK FIRST BYTE THEN         Y02669
         EX    WORK4,BLANKOUT      PROPAGATE THRU REST OF FIELD  Y02669
         EJECT
***********************************************************************
*                                                                     *
*  AT ENTRY TO THIS SECTION OF CODE, THE TARGET CONSOLE ID AND AREA   *
* ID HAVE BEEN PUT IN THE XSA, EITHER FROM THE 'L' PARAMETER, THE     *
* DEFAULTS SPECIFIED BY THE MSGRT COMMAND, OR SYSTEM DEFINED          *
* DEFAULTS (ISSUING CONSOLE)                                          *
*                                                                     *
*        THIS CODE CHECKS FOR ROUTING AUTHORITY, AND ENSURES THAT     *
* 'CC' SPECIFIES AN ACTIVE CONSOLE WITH AREA 'A' DEFINED.             *
*                                                                     *
*   ERR - IEE345I AUTHORITY INVALID - ISSUING CONSOLE WASN'T ALLOWED  *
*                                     TO ISSUE COMMAND                *
*         IEE926I 'CC' NOT A CONSOLE     - TARGET CONSOLE NOT ACTIVE  *
*         IEE926I 'CC' FULL CAPABILITY   - COMMAND VALID ONLY TO MS   *
*                                          OR SD CONSOLE              *
*                                                                     *
***********************************************************************
         SPACE 2
VALIDITY EQU   *
         USING DCMACB,SACBBASE     BASE SCREEN AREA CONTROL BLOCK
         CLC   XASCID(E1),XAU      IS TARGET AND SOURCE SAME?    Y02669
         BE    SRCEQTRG            IF YES, BRANCH
         SPACE  2
***********************************************************************
*                                                                     *
*                 TARGET AND SOURCE ARE NOT THE SAME                  *
*   GET THE UCM ENTRY ADDRESS OF THE SOURCE AND BASE THE UCME         *
*                                                                     *
***********************************************************************
         SPACE  2
         SR    REG1,REG1                                         Y02669
         IC    REG1,XAU            GET SOURCE CONSOLE ID         Y02669
         LTR   REG1,REG1           IF ZERO, ISSUED INTERNALLY    YM1557
         BZ    AUTHOK              ASSUME AUTHORITY OK           YM1557
         BAL   R15,FINDUCME        FIND ITS UCME                 Y02669
         TM    UCMAUTH,UCMAUTH3    DOES SOURCE CONSOLE HAVE CONS
*                                       AUTHORITY TO ROUTE?
         BZ    ERR345              NO,  BRANCH TO ERROR ROUTINE  Y02669
         EJECT
***********************************************************************
*                                                                     *
*   GET THE UCM ENTRY ADDRESS OF THE TARGET CONSOLE AND BASE THE UCME *
*                                                                     *
***********************************************************************
         SPACE  2
AUTHOK   EQU   *                                                 YM1557
         SR    REG1,REG1                                         Y02669
         IC    REG1,XASCID         GET TARGET CONSOLE ID         Y02669
         BAL   R15,FINDUCME        FIND ITS UCME                 Y02669
         TM    UCMDISP2,UCMDISPK   IF ON SUBSYSTEM CONSOLE     @Y02668P
         BO    ERR926E             BRANCH TO ERROR ROUTINE     @Y02668P
         TM    UCMATR,UCMUF        IS CONSOLE ACTIVE?
         BZ    ERR926B             NO,  BRANCH TO ERROR ROUTINE  Y02669
         CLI   XAN,K               IS THIS CONTROL COMMAND?
         BNE   CHKFULC             NO, GO CHECK FOR FULL CAPABILITY
         L     WORK4,XAL           GET POINTER TO 1ST OPERAND    Y02669
         CLI   E0(WORK4),VCHAR     IS COMMAND K V?               Y02669
         BE    CHKAREA             YES, ALLOW ROUTING EVEN IF          X
                                        FULL CAPABILITY
         SPACE  2
***********************************************************************
*                                                                     *
*     TARGET CONSOLE MUST BE MESSAGE STREAM OR STATUS DISPLAY         *
*     OR THE SAME AS ISSUING CONSOLE FOR ALL COMMANDS EXCEPT K V      *
*                                                                     *
*   ERR - IEE926I 'CC' FULL CAPABILITY                                *
*                                                                     *
***********************************************************************
         SPACE  2
CHKFULC  EQU   *
         TM    UCMDISP,UCMDISPF+UCMDISPG IS CONSOLE MS OR SD?
         BZ    ERR926C             NO,  BRANCH TO ERROR ROUTINE  Y02669
         B     CHKAREA             BRANCH TO CHECK AREA
         EJECT
***********************************************************************
*                                                                     *
*                TARGET IS THE SAME AS THE SOURCE                     *
*   GET THE UCM ENTY ADDRESS OF THE TARGET CONSOLE AND BASE THE UCME  *
*                                                                     *
***********************************************************************
         SPACE  2
SRCEQTRG EQU   *
         SR    REG1,REG1                                         Y02669
         IC    REG1,XASCID         FIND TARGET CONSOLE UCME      Y02669
         BAL   R15,FINDUCME                                      Y02669
         SPACE  2
***********************************************************************
*                                                                     *
*   FROM THIS POINT ON IN THIS MODULE, UCMEBASE CONTAINS THE          *
*   ADDRESS OF THE TARGET CONSOLE'S UCM ENTRY.                        *
*                                                                     *
***********************************************************************
         SPACE  2
CHKAREA  EQU   *
         CLI   XASDID,NULL        IS AREA NULL,NO 'A' SPECIFIED? Y02669
         BNZ   AREAEXST            NO, DOES AREA EXIST ON TARGET
         TM    UCMDISP,UCMDISPC    YES, IS TARGET CONSOLE A CRT?
         BZ    SETZ                NO, BRANCH TO SET AREA=Z
         TM    UCMDISP,UCMDISPF    YES, IS TARGET MESSAGE STREAM?
         BZ    CHKCMND             NO,BRANCH TO CHECK COMMAND
         SPACE  2
***********************************************************************
*                                                                     *
*   AREA REMAINS NULL AT THIS POINT IF TARGET IS A STATUS DISPLAY     *
*   OR FULL CAPABILITY CRT CONSOLE                                    *
*  IF TARGET IS NON-CRT OR MESSAGE STREAM ONLY CRT, INDICATE AREA=Z   *
*                                                                     *
***********************************************************************
         SPACE  2
SETZ     EQU   *
         MVI   XASDID,Z            IF TARGET IS NON-CRT OR MSG STREAM
         B     CHKCMND             BRANCH TO CHECK COMMAND
         EJECT
***********************************************************************
*                                                                     *
*       FIND OUT IF THE AREA SPECIFIED EXISTS ON THE TARGET           *
*                                                                     *
*  ERR - IEE926I 'A' NOT AN AREA - TARGET IS STATUS DISPLAY ONLY      *
*                                  AND AREA 'Z' WAS SPECIFIED         *
*        IEE926I 'A' NOT AN AREA - TARGET IS NOT A CRT AND AREA IS    *
*                                  NOT IN-LINE                        *
***********************************************************************
         SPACE  2
AREAEXST EQU   *
         TM    UCMDISP,UCMDISPC    IS TARGET A CRT?
         BO    CRT                 IF YES, BRANCH
         CLI   XASDID,Z            IS AREA = Z?
         BNE   ERR926A             NO,  BRANCH TO ERROR ROUTINE  Y02669
         B     CHKCMND             YES, BRANCH TO CHECK COMMAND
CRT      EQU   *                   TARGET IS A CRT
         CLI   XASDID,Z            IS AREA = Z?
         BNE   OUTOLINE            NO, BRANCH
         TM    UCMDISP,UCMDISPG    IS TARGET STATUS DISPLAY ONLY?
         BO    ERR926A             YES, BRANCH TO ERROR ROUTINE  Y02669
         B     CHKCMND             NO, BRANCH TO CHECK COMMAND
         SPACE  2
***********************************************************************
*                                                                     *
*   FIND THE SCREEN CONTROL BLOCK FOR THE SPECIFIED AREA.             *
*   ADDRESS IS PUT IN SACBBASE, ZERO IF NONE FOUND.                   *
*                                                                     *
*  ERR - IEE926I NOT AN AREA - SPECIFIED AREA DOES NOT EXIST (NO SACB)*
***********************************************************************
         SPACE  2
OUTOLINE EQU   *
         BAL   R15,FINDSACB                                      Y02669
         LTR   SACBBASE,SACBBASE   IS SPECIFIED AREA DEFINED?
         BZ    ERR926A             NO,  BRANCH TO ERROR ROUTINE  Y02669
         EJECT
***********************************************************************
*                                                                     *
*    IF THE COMMAND IS CONTROL, 1-IF K COMMAND ISSUED INTERNALLY      *
*                                 RETURN TO CALLER                    *
*                               2-IF K COMMAND ISSUED GO TO IEE6703D  *
*    OTHERWISE CONTINUE                                               *
*                                                                     *
***********************************************************************
         SPACE 2
CHKCMND  EQU   *
         CLI   XAN,K               IS COMMAND CONTROL?
         BE    D6703D              YES, BRANCH TO PROCESS CMND @ZM30446
         SPACE 3
***********************************************************************
*                                                                     *
*     THE ONLY COMMANDS HANDLED BEYOND THIS POINT ARE:                *
*         DISPLAY   C,K; CONSOLES (C); MATRIX; UNITS;                 *
*                   JOBS (J); TS; A                                   *
*         TRACK     JOBS (J); TS; A                                   *
*         STOPTR    JOBS (J); TS; A                                   *
*                                                                     *
*  ERR - IEE305I COMMAND INVALID - OTHER THAN D, TR, PT               *
***********************************************************************
         SPACE  2
TSTCMND  EQU   *
         CLI   XAN,TR              IS COMMAND TRACK?             Y02669
         BE    DACTIVE             YES, BRANCH                   Y02669
         CLI   XAN,PT              IS COMMAND STOPTR?            Y02669
         BE    STOPTR              YES, BRANCH                   Y02669
TESTOPER EQU   *                                                 Y02669
         CLI   XAV,DCHAR           IS COMMAND DISPLAY?           Y02669
         BNE   ERR305              ANY OTHER COMMAND IS ERROR    YM1917
         SPACE  2
         CLI   XAN,D               IS COMMAND D JOBS (J); TS; A? Y02669
         BNE   DISPLAY             NO, BRANCH                    Y02669
         EJECT
***********************************************************************
*                                                                     *
*     DISPLAY INIT; JOBS (J); TS; A ARE FIRST SEACHED FOR ,LIST       *
*         (,L) THEN TREATED LIKE THE OTHER DISPLAY COMMANDS           *
*                                                                     *
***********************************************************************
         SPACE  2
DACTIVE  EQU   *                                                 Y02669
         L     REG1,XAL            GET ADDRESS OF OPERANDS       Y02669
         CLI   XAXMASK,XAXJOBS     IS THIS JOBS?                 Y02669
         BH    ACTIVEOP            NO, ACTIVE                    Y02669
         BL    TS                  NO, TS                        Y02669
         CLC   E0(E2,REG1),JOCHAR  ARE FIRST 2 CHARS 'JO'?       Y02669
         BNE   ACTIVEOP            NO, MIGHT BE JUST 'J'         Y02669
         LA    REG1,E4(REG1)       FOR JOBS BUMP PAST 'S'        Y02669
         B     COMPARE             END OF OPERANDS?              Y02669
ACTIVEOP EQU   *                                                 Y02669
         LA    REG1,E1(REG1)       FOR 'A' AND 'J' BUMP PAST     Y02669
         B     COMPARE             END OF OPERANDS?              Y02669
TS       EQU   *                                                 Y02669
         LA    REG1,E2(REG1)       FOR TS BUMP PAST 'S'          Y02669
COMPARE  EQU   *                                                 Y02669
         CLI   E0(REG1),BLANK      IS OPERAND FOLLOWED BY BLANK? Y02669
         BE    DORTR               YES, CONTINUE                 Y02669
         CLC   E0(E6,REG1),LIST    IS OPERAND AFTER , 'LIST '?   Y02669
         BE    LISTOPT             YES, CHANGE BITS              Y02669
         CLC   E0(E3,REG1),JUSTL   NO, IS IT JUST 'L '?          Y02669
         BNE   ERR156A             NO, INVALID OPERAND           Y02669
LISTOPT  EQU   *
         SR    WORK4,WORK4                                       Y02669
         IC    WORK4,XAXMASK       GET REQUEST FLAGS FROM XSA    Y02669
         SRL   WORK4,E1            SHIFT RIGHT 1 FOR 'LIST'      Y02669
         STC   WORK4,XAXMASK       RESTORE REQUEST FLAGS         Y02669
DORTR    EQU   *                                                 Y02669
         CLI   XAN,TR              TRACK COMMAND?                Y02669
         BE    TRACK               YES                           Y02669
         EJECT
***********************************************************************
*                                                                     *
*     DISPLAY CONSOLES (C); C,K; M; AND U ARE CONSIDERED DISPLAYS     *
*                                                                     *
*  ERR - IEE926I DYNAMIC DISPLAY IN AREA - DISPLAY ALREADY IN AREA    *
***********************************************************************
         SPACE  2
DISPLAY  EQU   *                   DISPLAY COMMAND
         CLC   XAA(E2),ZERO        IS THIS FROM A TSO USER?      Y02669
         BNE   D0803D              YES, EXIT                     Y02669
         CLI   XASDID,Z            IS AREA = Z?
         BE    D0803D              YES, EXIT                     Y02669
         CLI   XASDID,NULL         AREA SPECIFIED IN CMD OR BY DEFAULT
         BE    DEFAREA             NO, PROVIDE DEFAULT AREA
         TM    DCMADFLG,DCMADD     TRACK ALREADY IN AREA?
         BO    ERR926D             YES, BRANCH TO ERROR ROUTINE  Y02669
         B     D0803D              GO BUILD CSCB - IEE0803D      Y02669
         EJECT
***********************************************************************
*                                                                     *
*        IF NO AREA IS SPECIFIED IN THE COMMAND AND NO DEFAULT IS     *
* FOUND IN THE ROUTING CONTROL TABLE, A DEFAULT AREA IS ESTABLISHED   *
* AS FOLLOWS:                                                         *
*        (1) THE BOTTOM-MOST EMPTY AREA                               *
*        (2) IF NONE EMPTY, THE AREA CONTAINING THE OLDEST STATIC     *
*            DISPLAY                                                  *
*        (3) IF NO DISPLAY AREAS AVAILABLE, A TRACK COMMAND IS        *
*            REJECTED; A STATIC DISPLAY IS PUT IN LINE                *
*        (4) IF THE CONSOLE HAS NO MESSAGE STREAM, A DISPLAY COMMAND  *
*            IS REJECTED                                              *
*                                                                     *
*   ERR-IEE921I NEEDS DISPLAY AREA - 1 TRACK WITH NO SACB DEFINED     *
*                                    2 DISPLAY WITH NO SACB DEFINED   *
*                                    AND TARGET IS STATUS DISPLAY     *
*               DISPLAY AREA BUSY  - NO STATUS DISPLAY TO OVERLAY     *
***********************************************************************
         SPACE
DEFAREA  EQU   *
         L     RDCMBASE,UCMXB      GET ADDRESS OF RESIDENT DCM   Y02669
         CLI   DCMTOPAR,E0         ANY AREAS DEFINED FOR TARGET?
         BE    MSGSTRM             NO, PUT IN LINE               Y02669
         SPACE
***********************************************************************
*     LOOK FOR BOTTOM-MOST EMPTY AREA                                 *
***********************************************************************
         SPACE
SRCHAREA EQU   *                                                 Y02669
         SR    WORK4,WORK4                                       Y02669
         L     SACBBASE,DCMADSDS   GET POINTER TO FIRST SACB     Y02669
CONTSRCH EQU   *
         TM    DCMASACB,DCMAUSE    DOES SACB DEFINE AN AREA?
         BNO   STATIC              NO, ANY STATIC DISPLAY TO OVERLAY
         TM    DCMAFLG1,DCMADISP   AREA CONTAIN DISPLAY?
         BO    FIRSTARE            YES, LOOK FOR ANOTHER AREA    Y02669
         TM    DCMADFLG,DCMADD     AREA RESERVED FOR TRACK?      Y02669
         BO    UPDATE              YES,LOOK AT NEXT              Y02669
         B     AFOUND              IF ONLY 1 AREA THIS WILL BE 0 Y02669
FIRSTARE EQU   *
         TM    DCMADFLG,DCMADD     AREA RESERVED FOR TRACK?      Y02669
         BO    UPDATE              YES,LOOK AT NEXT              Y02669
         LTR   WORK4,WORK4         NO, FIRST DEFAULT AREA?       Y02669
         BZ    FIRST               YES                           Y02669
CHKNEXTA EQU   *                                                 Y02669
         CLC   DCMATIME(L'DCMATIME),DCMATIME-DCMACB(WORK4) NO    Y02669
         BH    UPDATE              IS THIS DISPLAY OLDER?
FIRST    EQU   *
         LR    WORK4,SACBBASE      YES
UPDATE   EQU   *
         ICM   SACBBASE,E7,DCMACBNX+1 POINT TO NEXT SACB         Y02669
         BNZ   CONTSRCH            CONTINUE SEARCH               Y02669
         SPACE
***********************************************************************
*     LOOK FOR OLDEST STATIC DISPLAY TO OVERLAY                  Y02669
***********************************************************************
         SPACE
STATIC   EQU   *
         LTR   WORK4,WORK4         WAS ANY STATIC DISPLAY FOUND?
         BZ    MSGSTRM             NO, TRY TO PUT IN MESS STREAM Y02669
PUTINOLD EQU   *
         LR    SACBBASE,WORK4      YES, BASE SACB FOR AREA WITH
*                                       OLDEST DISPLAY
AFOUND   EQU   *                                                 Y02669
         MVC   XASDID(E1),DCMAID   PUT ID OF AREA FOUND IN XSA   Y02669
         B     D0803D              GO BUILD CSCB FOR DISPLAY     Y02669
MSGSTRM  EQU   *
         TM    UCMDISP,UCMDISPE+UCMDISPF MSG ST OR FULL CAPABILITY?
         BZ    ERR921D             NO, BRANCH TO ERROR ROUTINE   Y02669
         MVI   XASDID,Z            INDICATE MSG STREAM IN XSA
         B     D0803D              EXIT
         EJECT
***********************************************************************
*                                                                     *
*                  STOPTR HANDLING                                    *
*   ERR - IEE311I PARAMETER MISSING - PT REQUIRES AN OPERAND   @YM2774P
*   ERR - IEE925I OPERAND FOR CRT ONLY - TARGET IS NON-CRT            *
***********************************************************************
         SPACE  2
STOPTR   EQU   *                   STOP TRACK COMMAND            Y02669
         TM    UCMDISP2,UCMDISPK   IS THIS SUBSYSTEM CONSOLE?  @Y02668P
         BO    ERR921E             YES, BRANCH FOR ERROR       @Y02668P
         ICM   REG1,E7,XAL+1       R1 PTR BUFFER BEGINNING     @YM2774P
         BZ    ERR311              PT MUST HAVE AN OPERAND     @YM2774P
         TM    UCMDISP,UCMDISPC    IS TARGET A CRT?              Y02669
         BNO   ERR925A             ERR-COMMAND VALID ONLY TO CRT Y02669
         CLI   XASDID,Z            AREA = Z (INLINE NOT IN AREA)?
         BE    ERR921D             YES - TRACK CAN'T BE INLINE   Y02669
         SPACE  2
         L     RDCMBASE,UCMXB      GET ADDRESS OF RESIDENT DCM   Y02669
         CLI   XASDID,NULL         AREA NULL?
         BE    FINDTRAR            YES, FIND AREA WITH TRACK     Y02669
         SPACE  2
***********************************************************************
*                                                                     *
*     SEE IF AREA SPECIFIED HAS SPECIFIED TRACK IN IT                 *
*  ERR - IEE921I NO TRACK TO STOP - AREA NOT DEFINED BY SACB          *
***********************************************************************
         SPACE  2
         TM    DCMADFLG,DCMADD     IS THERE A TRACK IN AREA?     Y02669
         BNO   ERR921C             NO,  BRANCH TO ERROR ROUTINE  Y02669
         B     PROCESPT            YES                           Y02669
         EJECT
***********************************************************************
*                                                                     *
*    LOOK FOR AREA CONTAINING SPECIFIED DYNAMIC DISPLAY               *
*                                                                     *
*  ERR - IEE921I NO TRACK TO STOP - NO AREA CONTAINS TRACK DISPLAY    *
***********************************************************************
         SPACE  2
FINDTRAR EQU   *
         L     SACBBASE,DCMADSDS   GET FIRST SACB ADDRESS
CONTINU1 EQU   *
         TM    DCMASACB,DCMAUSE    DOES SACB DEFINE AN ACTIVE AREA?
         BNO   ERR921C             NO,  BRANCH TO ERROR ROUTINE  Y02669
         TM    DCMADFLG,DCMADD     IS THERE A TRACK IN AREA?     Y02669
         BO    TRFOUND             YES, BRANCH TO STORE AREA ID  Y02669
         ICM   SACBBASE,E7,DCMACBNX+1 NO, IS THERE ANOTHER SACB?
         BZ    ERR921C             NO, NO AREA WITH TRACK        Y02669
         B     CONTINU1            YES, CONTINUE SEARCH
TRFOUND  EQU   *                                                 Y02669
         MVC   XASDID(E1),DCMAID   PUT AREA ID IN XAS
PROCESPT EQU   *
         ST    SACBBASE,XAXSACB    PUT ADDRESS OF SACB IN XSA    Y02669
         B     D5503D              GO TO IEE5503D                Y02669
         EJECT
***********************************************************************
*                                                                     *
*        IN THE CASE OF A TRACK COMMAND, IF AN OUT-OF-LINE DISPLAY    *
*  IS REQUESTED, A SPECIFIED TARGET AREA (EITHER BY THE 'L' OPERAND   *
*  OR THE ROUTING CONTROL TABLE) IS CHECKED FOR AVAILABILITY.         *
*  IF NO AREA IS SPECIFIED, A DEFAULT AREA IS PROVIDED.               *
*  CERTAIN CONDITIONS ARE THEN SET UP AND APPROPIATE EXIT TAKEN.      *
*                                                                     *
*   ERR-IEE925I OPERAND FOR CRT ONLY - TRACK TO NON-CRT               *
*       IEE921I NEEDS DISPLAY AREA - TRACK TO AREA = Z (IN-LINE)      *
*                                    TARGET HAS NO DEFINED AREAS      *
***********************************************************************
         SPACE 2
TRACK    EQU   *                   TRACK COMMAND
         TM    UCMDISP2,UCMDISPK   IS THIS SUBSYSTEM CONSOLE?  @Y02668P
         BO    ERR921E             YES, BRANCH CMD INVALID     @Y02668P
         TM    UCMDISP,UCMDISPC    IS TARGET CONSOLE A CRT?
         BZ    ERR925A             NO, BRANCH TO ERROR ROUTINE   Y02669
         CLI   XASDID,Z            IS AREA = Z?
         BE    ERR921D             YES, ERR TR CAN'T GO INLINE   Y02669
         L     RDCMBASE,UCMXB      INSURE ADDRESSABILITY TO DCM  Y02669
         CLI   DCMTOPAR,E0         ANY AREAS DEFINED FOR TARGET? Y02669
         BE    ERR921D             NO, TR NEEDS DISPLAY AREA     Y02669
         SPACE
         SR    WORK4,WORK4         INIT FOR BOTTOM-MOST          Y02669
         ST    SACBBASE,XAXSACB    SAVE SACB BASE OF 'A'         Y02669
         L     SACBBASE,DCMADSDS   GET POINTER TO FIRST SACB
CHKFORTR EQU   *                                                 Y02669
         TM    DCMADFLG,DCMADD     DOES THIS AREA HAVE TRACK?    Y02669
         BO    TRHERE              YES, BRANCH TO CHECK FURTHER  Y02669
         TM    DCMAFLG1,DCMADISP   IS THERE DISPLAY IN AREA    @YM4789P
         BO    NEXTAREA            YES,CHECK NEXT AREA         @YM4789P
         LTR   WORK4,WORK4         IS THIS AN EMPTY AREA       @YM4793P
         BNZ   NEXTAREA            NO CONTINUE SEARCH          @YM4793P
         LR    WORK4,SACBBASE      YES,EMPTY AREA FOR TRACK    @YM4789P
NEXTAREA EQU   *
         ICM   SACBBASE,E7,DCMACBNX+1 BASE NEXT SACB             Y02669
         BZ    RESTBASE            NO MORE SACBS               @ZA29986
         TM    DCMASACB,DCMAUSE    SACB DEFINE ACTIVE AREA?    @ZA29986
         BO    CHKFORTR            YES, CONTINUE SEARCH        @ZA29986
         SPACE  2
RESTBASE EQU   *                                                 Y02669
         CLI   XASDID,NULL        WAS AREA SPECIFIED IN COMMAND? Y02669
         BE    USEMPTY             NO, DEFAULT AVAILABLE?        Y02669
         L     SACBBASE,XAXSACB    USE 'A' FROM CMND OR DEFAULT  Y02669
         B     TRSETUP                                           Y02669
USEMPTY  EQU   *
         LTR   WORK4,WORK4         IS THIS AREA EMPTY          @YM4789P
         BZ    INUSE               NO,DISPLAY IN AREA          @YM4789P
         LR    SACBBASE,WORK4      YES,AREA IS EMPTY           @YM4789P
         MVC   XASDID(E1),DCMAID   AREA ID IN THE SACB         @YM4789P
         B     TRSETUP                                         @YM4789P
INUSE    EQU   *                   DISPLAY IN AREA             @YM4789P
         L     SACBBASE,DCMADSDS                               @YM4789P
         MVC   XASDID(E1),DCMAID                               @YM4789P
         B     TRSETUP                                           Y02669
         EJECT
***********************************************************************
* SPECIFIED TR ALREADY ON TARGET CONSOLE.  THIS IS AN ERROR           *
*  ERR - IEE921I DISPLAY ALREADY EXISTS - HOLD FLAG ON                *
***********************************************************************
         SPACE  2
TRHERE   EQU   *
         CLI   XASDID,NULL         WAS AN AREA SPECIFIED       @YM4793P
         BE    TRHERE1             NO, AREA OK                 @YM4793P
         C     SACBBASE,XAXSACB    IS TR IN AREA SPECIFIED     @YM4793P
         BNE   ERR921A             NO, ERROR                   @YM4793P
TRHERE1  EQU   *                                               @YM4793P
         TM    DCMADFLG,DCMAHOLD   YES, HOLD MODE FLAG ON?       Y02669
         BNO   TRSETUP             NO, SET UP FOR TRACK COMMAND  Y02669
         CLC   DCMATRCK,XAXMASK    YES, CHANGING OPTIONS?        Y02669
         BE    ERR921A             NO, ERROR - CAN'T GET OUT OF        X
                                   HOLD MODE BY OPERATOR ISSUING TR
         SPACE 2
***********************************************************************
*  RETURN IF ONLY UPDATING ALREADY EXISTING TRACK DISPLAY             *
*  GOTO IEE0803D IF THIS IS THE INITIAL TRACK REQUEST                 *
***********************************************************************
         SPACE 2
TRSETUP  EQU   *                                                 Y02669
         OC    DCMATRCK(E1),XAXMASK  GET NEW OPTIONS FROM XSA    Y02669
         TM    DCMADFLG,DCMADD    TRACK ALREADY DEFINED IN AREA? Y02669
         BO    RETURN              YES, ALTER BITS AND RETURN    Y02669
         OI    DCMADFLG,DCMADD     RESERVE AREA FOR TRACK        Y02669
         MVC   DCMAUTME(E2),UDEFAULT SET DEFAULT TIME INTERVAL   Y02669
         LA    WORK4,DCMATECB      GET ADDRESS OF TRACK ECB      Y02669
         ST    WORK4,XAXSACB       PASS ADDRESS TO IEE0803D      Y02669
         B     D0803D              EXIT TO CREATE CSCB           Y02669
         EJECT
***********************************************************************
*                                                                     *
* FIND AND BASE UCM ENTRY SUBROUTINE                                  *
*    INPUT - REG1   - CONSOLE ID, RIGHT JUSTIFIED                     *
*                     (HIGH ORDER BYTES MUST BE ZERO)                 *
*    OUTPUT         - POINTER TO UCM ENTRY SPECIFIED BY CONSOLE       *
*                     ID IN REGISTER UCMEBASE                         *
*    REGISTER USAGE - DESTROYS CONTENTS OF FOLLOWING REGS:            *
*                     REG0, REG1, UCMEBASE                            *
*                     R15 IS RETURN REGISTER                          *
*                                                                     *
***********************************************************************
         SPACE 2
FINDUCME EQU   *
         L     UCMEBASE,UCMVEA     POINT TO FIRST ENTRY IN UCM
         BCTR  REG1,E0             SUBTRACT ONE FROM CONSOLE ID
         M     REG0,UCMVEZ         MULTIPLY ENTRY SIZE BY REDUCED ID
         AR    UCMEBASE,REG1       POINT TO DESIRED UCM ENTRY
         BR    R15                 RETURN TO MAIN LINE
         EJECT
***********************************************************************
*                                                                     *
* FIND AND BASE SACB SUBROUTINE                                       *
*    GIVEN A PARTICULAR CRT CONSOLE AND A ONE CHARACTER AREA ID       *
* THIS SUBROUTINE FINDS AND BASES THE SCREEN AREA CONTROL BLOCK       *
* (SACB) WHICH MATCHES THAT ID.  IF NONE IS FOUND, A ZERO IS          *
* RETURNED AS THE BASE.                                               *
*    INPUT -  XASDID   - ONE CHARACTER AREA ID (POINTED TO BY REG2)   *
*             UCMEBASE - ADDRESS OF UCM ENTRY OF A CRT CONSOLE        *
*    OUTPUT - SACBBASE - ADDRESS OF SACB WITH MATCHING ID             *
*                        (ZERO IF NO MATCH)                           *
*    REGISTER USAGE    - DESTROYS CONTENTS OF FOLLOWING REGS:         *
*                         SACBBASE, RDCMBASE                          *
*                         R15 IS RETURN REGISTER                      *
*                                                                     *
***********************************************************************
         SPACE 2
FINDSACB EQU   *
         L     RDCMBASE,UCMXB      GET RESIDENT DCM ADDRESS
         ICM   SACBBASE,E7,DCMADSDS+1   GET FIRST SACB ADDRESS   Y02669
         BZ    SACBEXIT                                          Y02669
CONTINUE EQU   *
         TM    DCMASACB,DCMAUSE    DOES SACB DEFINE AN AREA?
         BNO   ENDOSACB            NO, BRANCH TO RETURN
         CLC   XASDID,DCMAID       COMPARE PASSED ID AND SACB ID Y02669
         BCR   EQ,R15              EQUAL? RETURN TO MAIN LINE
         ICM   SACBBASE,E7,DCMACBNX+1 NOT EQUAL, GET NEXT SACB   Y02669
         BZ    SACBEXIT            IF NO MORE SACB'S, RETURN     Y02669
         B     CONTINUE            CONTINUE SEARCH
ENDOSACB EQU   *
         SR    SACBBASE,SACBBASE   INDICATE NO SACB MATCH FOUND
SACBEXIT EQU   *                                                 Y02669
         BR    R15                 RETURN TO MAIN LINE
         EJECT
***********************************************************************
*                                                                     *
*                   RELEASE CMS AND LOCAL LOCKS SUBROUTINE            *
*                                                                     *
***********************************************************************
LOCKSOFF EQU   *
         STM   RCTBASE,RETURN14,XASAVLOC  SAVE REGS FOR SETLOCK  Y02651
         SETLOCK RELEASE,TYPE=CMS,RELATED=(LOCKSET)
         SETLOCK RELEASE,TYPE=LOCAL,RELATED=(LOCKSET)
         LM    RCTBASE,RETURN14,XASAVLOC  RESTORE REGS AFTER     Y02651
         NI    XASSDS,XFF-XASSDS5       INDICATE LOCKS NOT HELD  Y02651
         BR    R15                        RETURN TO MAIN LINE    Y02651
         EJECT
***********************************************************************
*                                                                     *
*                      SPECIAL JES2 INTERFACE                         *
*                                                                     *
*              SVC 34 WILL RETURN TO THE ADDRESS PROVIDED      @YM7238P
*              IN REGISTER 14. AT THAT TIME, REGISTER 15       @YM7238P
*              WILL CONTAIN ONE OF THE FOLLOWING RETURN        @YM7238P
*              CODES                                           @YM7238P
*                 0   REQUESTED AREA NOT IN USE                @YM7238P
*                 4   AREA IN USE BY STATIC DISPLAY            @YM7238P
*                 8   AREA IN USE BY TRACK;DISPLAY MAY NOT     @YM7238P
*                     BE OVERLAID                              @YM7238P
*                 12  SPECIFIED CONSOLE NOT SYSGENED, NOT      @YM7238P
*                     ACTIVE OR NOT A TARGET FOR STATUS        @YM7238P
*                     DISPLAYS                                 @YM7238P
*                 16  REQUESTED AREA NOT DEFINED ON SPECIFIED  @YM7238P
*                     CONSOLE.                                 @YM7238P
*                                                                     *
***********************************************************************
JES2CODE EQU   *                                                 Y02669
         OI    XASSDS,XASSDS5          INDICATE LOCKS HELD     @YM4796P
         L     UCMEBASE,UCMVEL         BASE LAST UCM ENTRY       Y02669
         CLC   XASCID(E1),UCMID        ID HIGHER THAN HIGHEST    Y02669
*                                      VALID CONSOLE ID?         Y02669
         BNH   VALIDCON                NO, CONSOLE ID IS VALID   Y02669
         LA    WORK4,BADCONSL          YES, CONSOLE ID TOO HIGH  Y02669
         B     JES2EXIT                RETURN WITH 12 ERROR CODE Y02669
VALIDCON EQU   *                                                 Y02669
         SR    REG1,REG1                                         Y02669
         IC    REG1,XASCID             GET TARGET CONSOLE ID     Y02669
         BAL   R15,FINDUCME            FIND ITS UCME             Y02669
         TM    UCMATR,UCMUF            IS CONSOLE ACTIVE?        Y02669
         BO    ACTIVEID                YES, CONSOLE IS ACTIVE    Y02669
         LA    WORK4,BADCONSL          NO, CONSOLE NOT ACTIVE    Y02669
         B     JES2EXIT                RETURN WITH 12 ERROR CODE Y02669
ACTIVEID EQU   *                                                 Y02669
         TM    UCMDISP,UCMDISPC        IS TARGET CONSOLE A CRT?  Y02669
         BO    CRTCONSL                YES                       Y02669
         B     OKAREA                  NO, RETURN WITH 0 RETURN  Y02669
*                                      CODE                      Y02669
CRTCONSL EQU   *                                                 Y02669
         CLI   XASDID,Z               IS THE AREA IN LINE Z    @YM7238P
         BNE   NOTZ                   YES,CHECK TYPE OF CONSL  @YM7238P
         TM    UCMDISP,UCMDISPF       IS THIS AN MS CONSOLE    @YM7238P
         BO    OKAREA                 YES,BRANCH Z IS VALID    @YM7238P
         TM    UCMDISP,UCMDISPE       IS THIS AN FC CONSOLE    @YM7238P
         BO    FCTEST1                YES,BRANCH CK IN-LINE    @YM7238P
         B     RTCODE16               BRANCH SD CONSL INVALID  @YM7238P
NOTZ     EQU   *                                               @YM7238P
         TM    UCMDISP,UCMDISPG       IS THIS SD CONSOLE       @YM7238P
         BO    OKCONSL                YES,BR CK FOR VALID AREA @YM7238P
         TM    UCMDISP,UCMDISPE       IS THIS FC CONSOLE       @YM7238P
         BO    FCTEST2                YES,BR CK CMD SOURCE     @YM7238P
         B     RTCODE16               BRANCH MS CONSL INVALID  @YM7238P
FCTEST1  CLC   XASCID(E1),XAU         DOES SOURCE=TARGET CONSL @YM7238P
         BE    OKAREA                 YES,BRANCH IN-LINE VALID @YM7238P
         B     RTCODE12               NO,CAN NOT ROUTE TO FC   @YM7238P
FCTEST2  CLC   XASCID(E1),XAU         DOES SOURCE=TARGET CONSL @YM7238P
         BE    OKCONSL                YES,BR OUT-OF-LINE VALID @YM7238P
RTCODE12 EQU   *                                               @YM7238P
         LA    WORK4,BADCONSL         RETURN CODE 12           @YM7238P
         B     JES2EXIT                                        @YM7238P
OKCONSL  EQU   *                                                 Y02669
         BAL   R15,FINDSACB            FIND AREA'S SACB          Y02669
         LTR   SACBBASE,SACBBASE       IS SPECIFIED AREA DEFINED?Y02669
         BNZ   AREAIS                  YES, AREA EXISTS ON       Y02669
*                                      CONSOLE                   Y02669
RTCODE16 EQU   *                                               @YM7238P
         LA    WORK4,BADAREA           NO, AREA NOT DEFINED TO   Y02669
*                                      CRT                       Y02669
         B     JES2EXIT                RETURN WITH 16 ERROR CODE Y02669
AREAIS   EQU   *                                                 Y02669
         TM    DCMADFLG,DCMADD         TRACK ALREADY IN AREA?    Y02669
         BZ    NOTRACK                 NO                        Y02669
         LA    WORK4,TRINAREA          YES, AREA CONTAINS TRACK  Y02669
         B     JES2EXIT                RETURN WITH 8 ERROR CODE  Y02669
NOTRACK  EQU   *                                                 Y02669
         TM    DCMAFLG1,DCMADISP       AREA CONTAIN DISPLAY?     Y02669
         BZ    OKAREA                  NO                        Y02669
         LA    WORK4,AREANUSE          YES, AREA CONTAINS        Y02669
*                                      ANOTHER DISPLAY           Y02669
         B     JES2EXIT                RETURN WITH 4 ERROR CODE  Y02669
OKAREA   EQU   *                                                 Y02669
         LA    WORK4,OKCCA             CONSOLE & AREA VALID      Y02669
JES2EXIT EQU   *                                                 Y02669
         BAL   R15,LOCKSOFF            HAVE LOCKS RELEASED       Y02669
         LR    R15,WORK4               LOAD RETURN CODE          Y02669
         LM    REG0,RETURN14,XASAVSTD  RESTORE JES2 REGISTERS    Y02669
         BR    RETURN14                RETURN TO JES2            Y02669
         EJECT
***********************************************************************
*                                                                     *
*                             EXITS                                   *
*                                                                     *
***********************************************************************
RETURN   EQU   *                                                 Y02669
         BAL   R15,LOCKSOFF       RELEASE LOCKS HELD             Y02651
         BR    RETURN14           RETURN TO SYSTEM               Y02669
         SPACE 2
D5503D   EQU   *                                                 Y02669
         L     R15,VCON5503       EXIT TO 2ND LOAD OF STOPTR     Y02669
         BR    R15                                               Y02669
         SPACE 2
D2903D   EQU   *                                                 Y02669
         L     R15,VCON2903       EXIT TO DISPLAY REQUEST MOD    Y02669
         BR    R15                                               Y02669
         SPACE 2
D6703D   EQU   *                                                 Y02669
         L     R15,VCON6703       EXIT TO HANDLE CONTROL COMMAND Y02669
         BR    R15                FOR GRAPHICS CONSOLES          Y02669
         SPACE 2
D9403D   EQU   *                                               @Z40LPTA
         MVI   XAN,D              RE-SET TO DISPLAY VERB       @Z40LPTA
         L     R15,VCON9403       EXIT TO MSS CMND PROCESSING  @Z40LPTA
         BR    R15                                             @Z40LPTA
         SPACE 2
D0803D   EQU   *
         BAL   R15,LOCKSOFF       RELEASE LOCKS HELD             Y02651
         OI    XASSDS,XASSDS1     INDICATE STATUS DISPLAY-MEANS  Y02669
*                                 SOMETHING IN XASCID, XASDID    Y02669
         CLI   XAN,DR             IF THIS IS A DISPLAY REQUEST   Y02669
         BE    D2903D             GO TO IEE2903D, ELSE           Y02669
         CLI   XAN,D38            THIS A D 3850 COMMAND?       @Z40LPTA
         BE    D9403D             GO TO IEE9403D               @Z40LPTA
         L     R15,VCON0803       GO BUILD CSCB                  Y02669
         BR    R15                                               Y02669
         EJECT
***********************************************************************
*                                                                     *
*                          ERROR CODES                                *
*   XAE = MAJOR ERROR CODE FOR IEE5603D                               *
*   XASTSAVE = MINOR CODE FOR IEE5903D                                *
*   XASOPTR (XASTSAVE+1) = OPERAND POINTER FOR IEE5603D & IEE5903D    *
*                                                                     *
***********************************************************************
         SPACE 2
***********************************************************************
*              IEE156I INVALID OPERAND                                *
***********************************************************************
ERR156A  EQU   *
         CLC   E0(E6,REG1),LISTC   ADDITIONAL BAD OPERAND?       YM1920
         BNE   BADAOP              NO, LIST NOT SPECIFIED        YM1920
         LA    REG1,E5(REG1)       YES, POINT TO INFO AFTER LIST YM1920
BADAOP   EQU   *                                                 YM1920
         LR    OPPTR,REG1          SET UP POINTER                Y02669
ERR156   EQU   *                                                 Y02669
         MVI   XAE,MSG156          MSG CODE-IEE156I INVALID OP   Y02669
         ST    OPPTR,XASTSAVE      SAVE POINTER TO BAD OPERAND   Y02669
         B     D5603D              BRANCH TO ERROR EXIT          Y02669
         SPACE
***********************************************************************
*              IEE345I AUTHORITY INVALID                              *
***********************************************************************
ERR345   EQU   *                                                 Y02669
         MVI   XAE,MSG345          MSG CODE-IEE345 AUTH INVALID  Y02669
         B     STOREOP             BRANCH TO ERROR EXIT          Y02669
         SPACE
***********************************************************************
*              IEE921I CM OP REJECTED                                 *
*                      DISPLAY ALREADY EXISTS                         *
*                      DISPLAY AREA BUSY                              *
*                      NO TRACK TO STOP                               *
*                      NEEDS DISPLAY AREA                             *
*                      SUBSYSTEM CONSOLE                       @Y02668P
***********************************************************************
ERR921A  EQU   *
         MVI   XASTSAVE,MSG921A    2ND CODE-DISP ALREADY EXISTS  Y02669
         B     ERR921              BRANCH TO GET MAJOR CODE      Y02669
         SPACE
ERR921B  EQU   *                                                 Y02669
         MVI   XASTSAVE,MSG921B    2ND CODE-DISPLAY AREA BUSY    Y02669
         B     ERR921              BRANCH TO GET MAJOR CODE      Y02669
         SPACE
ERR921C  EQU   *                                                 Y02669
         MVI   XASTSAVE,MSG921C    2ND CODE-NO TRACK TO STOP     Y02669
         B     ERR921              BRANCH TO GET MAJOR CODE      Y02669
ERR921E  EQU   *                                               @Y02668P
         MVI   XASTSAVE,MSG921E    2ND CODE-CC SUBSYSTEM       @Y02668P
         B     ERR921              CONSOLE                     @Y02668P
         SPACE
ERR921D  EQU   *                                                 Y02669
         MVI   XASTSAVE,MSG921D    2ND CODE-NEEDS DISPLAY AREA   Y02669
ERR921   EQU   *                                                 Y02669
         MVC   XASOPTR(E3),XAL+E1  MOVE OPERAND POINTER IN       Y02669
         MVI   XAE,MSG921          MSG CODE-IEE921I CM OP REJ    Y02669
         B     D5603D              BRANCH TO ERROR EXIT          Y02669
         SPACE
***********************************************************************
*              IEE925I OPERAND                                        *
*                      FOR CRT ONLY                                   *
***********************************************************************
ERR925A  EQU   *                                                 Y02669
         MVC   XASOPTR(E3),XAL+E1  MOVE OPERAND POINTER IN       Y02669
         MVI   XASTSAVE,MSG925A    2ND CODE-FOR CRT ONLY         Y02669
         MVI   XAE,MSG925          MSG CODE-IEE925I OPERAND      Y02669
         B     D5603D                                            Y02669
         SPACE
***********************************************************************
*              IEE926I L OPERAND INVALID                              *
*                      DYNAMIC DISPLAY IN AREA                        *
***********************************************************************
ERR926D  EQU   *                                                 Y02669
         MVI   XASTSAVE,MSG926D    2ND CODE-DYNAMIC DISP IN AREA Y02669
         MVI   XAE,MSG926          MSG CODE-IEE926I L OP INVALID Y02669
         B     STOREOP                                           Y02669
         SPACE
***********************************************************************
*              IEE926I L OPERAND INVALID                              *
*                      CCA NOT AN AREA                                *
*                      CC NOT A CONSOLE                               *
*                      CC FULL CAPABILITY                             *
*                      CC SUBSYSTEM CONSOLE                    @Y02668P
***********************************************************************
ERR926A  EQU   *                                                 Y02669
         MVI   XASTSAVE,MSG926A    2ND CODE-CCA NOT AN AREA      Y02669
         B     ERR926              BRANCH TO GET MAJOR CODE      Y02669
         SPACE
ERR926B  EQU   *                                                 Y02669
         MVI   XASTSAVE,MSG926B    2ND CODE-CC NOT A CONSOLE     Y02669
         BCTR  OPPTR,E0            SET CURSOR POINTER TO         Y02669
         BCTR  OPPTR,E0            BEGINNING OF OPERAND          Y02669
         B     ERR926              BRANCH TO GET MAJOR CODE      Y02669
ERR926E  EQU   *                                               @Y02668P
         L     OPPTR,XASTSAVE      CURSOR PTR AFTER L=         @Y02668P
         MVI   XASTSAVE,MSG926E    2ND-CODE-CC OR CCA          @Y02668PX
                                   SUBSYSTEM CONSOLE           @Y02668P
         B     ERR926                                          @Y02668P
         SPACE
ERR926C  EQU   *                                                 Y02669
         MVI   XASTSAVE,MSG926C    2ND CODE-CC FULL CAPABILITY   Y02669
ERR926   EQU   *                                                 Y02669
         MVI   XAE,MSG926          MSG CODE-IEE926I L OP INVALID Y02669
         B     STOREOP                                           Y02669
         SPACE
***********************************************************************
*              IEE927I REQUEST AMBIGIOUS                         YM1557
*                      SPECIFY L OPERAND                         YM1557
***********************************************************************
ERR927   EQU   *                                                 YM1557
         MVI   XASTSAVE,MSG927A    2ND CODE- SPECIFY L OPERAND   YM1557
         MVI   XAE,MSG927          MSG CODE-IEE927I REQUEST      YM1557
         BCTR  OPPTR,E0            POINT INVALID OP              Y02669
         BCTR  OPPTR,E0            POINTER TO L                  Y02669
STOREOP  EQU   *                                                 Y02669
         STCM  OPPTR,E7,XASOPTR    SAVE CURSOR POINTER           Y02669
         SPACE 3
***********************************************************************
*                                                                     *
*                         ERROR EXITS                                 *
*                                                                     *
***********************************************************************
D5603D   EQU   *                                                 Y02669
         L     R15,VCON5603        GET ADDR OF MSG MOD IEE5603D  Y02669
         BR    R15                                               Y02669
         SPACE  2
ERR305   EQU   *                                                 YM1917
         MVI   XAE,MSG305          MSG CODE-IEE305I INVALID CMD  YM1917
         B     D0503D                                            YM1917
         SPACE
ERR311   EQU   *                                                 YM1917
         MVI   XAE,MSG311          MSG CODE-IEE311I PARM MISSING YM1917
D0503D   EQU   *                                                 YM1917
         BAL   R15,LOCKSOFF        RELEASE LOCKS HELD            YM1917
         L     R15,VCON0503        BRANCH TO IEE0503D TO ISSUE   YM1917
         BR    R15                                               YM1917
         EJECT
***********************************************************************
*                                                                     *
*                             CONSTANTS                               *
*                                                                     *
***********************************************************************
         SPACE 2
ZERO     DC    H'0'                CONSTANT                      Y02669
ZEROES   DC    F'0'                CONSTANT                      Y02669
C1       DC    X'01'                                             Y02669
C2       DC    H'2'                CONSTANT                    @YM5459P
C3       DC    H'3'                CONSTANT
C4       DC    H'4'
F0F0     DC    C'00'               INVALID CONSOLE ID COMPARE    Y02669
UDEFAULT DC    H'0030'             UTME DEFAULT OF 30 SECONDS    Y02669
CONSOLES DC    C'CONSOLES'         CONSOLES OPERAND FOR COMPARE
DUMP     DC    C'DUMP'             DUMP OPERAND FOR COMPARE    @G33CPMR
DMN      DC    C'DMN'              DMN OPERAND FOR COMPARE     @G33CPMR
CCOMMAK  DC    C'C,K'              C,K OPERAND FOR COMPARE       YM1916
LEQUALS  DC    C'L='               INPUT FOR IEEBUFSC            Y02669
LIST     DC    C',LIST '           COMPARE FOR LIST OPERAND      Y02669
LISTC    DC    C',LIST,'           COMPARE FOR ADDITIONAL OPER   YM1920
JUSTL    DC    C',L '              COMPARE FOR SHORT L FOR LIST  Y02669
JOCHAR   DC    C'JO'               COMPARE FOR BEGINNING OF JOBS Y02669
DCBLANK  DC    C'C '               C OPERAND FOR D CONSOLES     @YM4791
CDCHAR   DC    C'C,D'              COMPARE FOR D C,D           @Y02668P
NCCHAR   DC    C'N,'               COMPARE FOR K N,PFK         @ZA12152
         SPACE
***********************************************************************
*                                                                     *
*                     MODULE NAMES FOR BRANCHES                       *
*                                                                     *
***********************************************************************
         SPACE  2
VCON2903 DC    V(IEE2903D)         DISPLAY REQUESTS              Y02669
VCON6703 DC    V(IEE6703D)         CONTROL COMMAND - GRAPHICS    Y02669
VCON5603 DC    V(IEE5603D)         ROUTING MESSAGE MODULE        Y02669
VCON0803 DC    V(IEE0803D)         CSCB CREATION ROUTINE         Y02669
VCON5503 DC    V(IEE5503D)         STOPTR HANDLER, SECOND LOAD   Y02669
VCON0503 DC    V(IEE0503D)         SVC 34 MESSAGE MODULE         Y02669
VCON9403 DC    V(IEE9403D)         MSS INTERFACE MODULE        @Z40LPTA
         SPACE  2
***********************************************************************
*                                                                     *
*                        MAINTAINANCE AREA                            *
*                                                                     *
***********************************************************************
         SPACE
PATCH    DC    5CL25'** IEE7503D PATCH AREA **'                @ZA29986
         EJECT
         IEFPTRS
         EJECT
DXSA     DSECT
         IEEXSA
         EJECT
         IEERDCM
         EJECT
DRCT     DSECT
         IEECVMAP RCT
         EJECT
         IEECUCM FORMAT=NEW
         EJECT
DCVT     DSECT
         CVT
         EJECT
         IHAPSA
         END
