 /*********************************************************  @G24LB26*/
 /*                                                          @G24LB26*/
 /*  NAME:  IEZBFTBL (VOLUME BEST FIT TABLE)                 @G24LB26*/
 /*                                                          @G24LB26*/
 /*  FUNCTION - MAPPING MACRO (ALLOCATE TO MOUNTED VOLUMES)  @G24LB26*/
 /*             TABLE HOLDING ALL VOLUMES IN A SPECIFIC      @G24LB26*/
 /*             GROUP ELIGIBLE TO BE SELECTED FOR A          @G24LB26*/
 /*             PARTICULAR REQUEST.  CERTAIN FIELDS WILL BE  @G24LB26*/
 /*             SORTED WHEN THE LIST IS COMPLETE IN ORDER    @G24LB26*/
 /*             OF THEIR FITNESS FOR THE REQUEST. ALSO IS    @G24LB26*/
 /*             PASSED TO THE TABLE PROCESSOR ROUTINE TO     @G24LB26*/
 /*             OBTAIN ADDITIONAL INFORMATION ON THE         @G24LB26*/
 /*             VOLUMES.                                     @G24LB26*/
 /*                                                          @G24LB26*/
 /*  STATUS:  MVS/VS1                                        @G24LB26*/
 /*                                                          @G24LB26*/
 /*  CHANGE ACTIVITY -                                       @ZA28479*/
 /*         CHANGE 1 10/21/77 SPACE ACCOUNTING. ADD FIELD    @ZA28479*/
 /*                           BFMTDOHT.                      @ZA28479*/
 /*                                                          @ZA28479*/
 /*********************************************************  @G24LB26*/
 DCL 1 BFTBL BASED   BDY(WORD),                            /*@G24LB26*/
       2 BFHEADER,                  /*HEADER SECTION         @G24LB26*/
         3 BFLEN       FIXED(15),   /*LENGTH OF TABLE        @G24LB26*/
         3 BFGROUP     CHAR(8),     /*GROUP NAME             @G24LB26*/
         3 BFTOTIDX    FIXED(15),   /*# ELEMENTS IN ARRAY    @G24LB26*/
         3 BFDEVTP     FIXED(31),   /*INTERNAL DEVICE TYPE   @G24LB26*/
                                    /*OR UCB ADDRESS         @G24LB26*/
         3 BFJOBID     FIXED(31),   /*TCB ADDRESS (VS1) OR   @G24LB26*/
                                    /*ASID (VS2)             @G24LB26*/
         3 BFREQFLS    BIT(8),      /*REQUEST FLAGS          @G24LB26*/
           4 BFNSHRFL  BIT(1),      /*REQUEST IS NON-SHARE   @G24LB26*/
           4 *         BIT(7),      /*UNUSED                 @G24LB26*/
       2 BFENTRY(*),                /*START OF ARRAY         @G24LB26*/
         3 BFSLOT,                  /*SLOT SECTION           @G24LB26*/
           4 BFVOLID   CHAR(6),     /*SERIAL NUM OF VOL      @G24LB26*/
           4 BFGRPIDX  FIXED(8),    /*EXTENSION INDEX NUM    @G24LB26*/
           4 BFGRPSLT  FIXED(8),    /*EXTENSION SLOT NUM     @G24LB26*/
           4 BFHPFLGS  BIT(8),      /*HIGH PRIORITY SORTING  @G24LB26*/
                                    /*FLAGS                  @G24LB26*/
             5 BFHRSKFL BIT(1),     /*VOL IS HIGH RISK       @G24LB26*/
             5 BFWAITFL BIT(1),     /*WAIT OR ENQUEUED       @G24LB26*/
                                    /*VOLUME                 @G24LB26*/
             5 *        BIT(6),     /*THESE 6 BITS MUST      @G24LB26*/
                                    /*REMAIN UNUSED          @G24LB26*/
           4 BFOVERUS  FIXED(8),    /*CON USERS IF OVER MAX  @G24LB26*/
           4 BFVOLEXP  CHAR(4),     /*VOL EXPIRATION DATE    @G24LB26*/
           4 BFLPFLGS  BIT(8),      /*LOW PRIORITY SORTING   @G24LB26*/
                                    /*FLAGS                  @G24LB26*/
             5 BFSHCNFL BIT(1),     /*SHAREABILITY           @G24LB26*/
                                    /*CONSIDERATION          @G24LB26*/
             5 BFNMTDFL BIT(1),     /*VOL NOT MOUNTED OR     @G24LB26*/
                                    /*SCHEDULED TO BE        @G24LB26*/
                                    /*MOUNTED                @G24LB26*/
             5 BFMTDOHF BIT(1),     /*VOL MOUNTED OTHER HOST @G24LB26*/
                                    /*AND NOT SCHEDULED TO   @G24LB26*/
                                    /*BE MOUNTED THIS HOST   @G24LB26*/
             5 *        BIT(5),     /*UNUSED                 @G24LB26*/
           4 BFCONUSE   FIXED(8),   /*NUM CONCURRENT USERS   @G24LB26*/
           4 BFMISCFL   BIT(16),    /*MISCELLANEOUS FLAGS    @G24LB26*/
             5 BFBADPRI BIT(1),     /*NOT FOR PRIMARY SEL    @G24LB26*/
             5 BFBADSEC BIT(1),     /*NOT FOR SECONDARY SEL  @G24LB26*/
             5 BFSELPRI BIT(1),     /*VOL SELECTED FOR       @G24LB26*/
                                    /*PRIMARY                @G24LB26*/
             5 BFSELSEC BIT(1),     /*VOL SELECTED FOR       @G24LB26*/
                                    /*SECONDARY              @G24LB26*/
             5 BFJESOK  BIT(1),     /*JES3 VERIFIED VOL,     @G24LB26*/
                                    /*VS/2 ONLY              @G24LB26*/
             5 BFVOLREJ BIT(1),     /*JES3 REJECTED VOL,     @G24LB26*/
                                    /*OR TEST ENQ FAILED OR  @G24LB26*/
                                    /*VOL WAIT AND DYN ALLOC @G24LB26*/
             5 BFNOENQ  BIT(1),     /*VOL NOT ENQUEUED,      @G24LB26*/
                                    /*VS/2 ONLY              @G24LB26*/
             5 BFENQUED BIT(1),     /*VOL ENQUEUED,          @G24LB26*/
                                    /*VS/2 ONLY              @G24LB26*/
             5 BFNOTMTD BIT(1),     /*VOL NOT MOUNTED        @G24LB26*/
             5 BFMTDOTH BIT(1),     /*VOL MOUNTED OTHER HOST @G24LB26*/
             5 BFMTDOHT BIT(1),     /*VOL MTD OTHER HOST,    @ZA28479*/
                                    /*MAY OR MAY NOT BE      @ZA28479*/
                                    /*EXCLUSIVE OTHER HOST   @ZA28479*/
             5 *        BIT(5);     /*UNUSED                 @ZA28479*/
