         TITLE 'IFG0194A                      TAPE/DIRECT ACCESS VOLUMEX
                AND UNIT SELECTION ROUTINE'                      Y02134
IFG0194A CSECT
         ENTRY IFG0554A                 ENTRY FOR FEOV/EOV       Y02134
         ENTRY IFG0204A                 ENTRY FROM CLOSE         Y02134
IFG0554A EQU   IFG0194A                 ALIAS ENTRY POINT        Y02134
IFG0204A EQU   IFG0194A                 ALIAS ENTRY POINT        Y02134
***********************************************************************
*                                                                     *
*        VS 2 RELEASE 03 DELETIONS/CHANGES                            *
*0000092637,092638,092664,092684                               @ZA03196
*0000                                                          @ZA02218
*0000785500-789550,795504,800000-802000                        @ZA14612
*                                                                     *
*        VS2 RELEASE O37 CHANGES/DELETIONS                            *
*C357997,A353580-353860,A795511,A882000-883800                 @ZA15095
*A882210-882220,A973210-973290,A882010-882020,A883810-883830   @ZA18540
*C520000                                                       @ZA19129
*                                                                     *
*        VS 2 RELEASE 02 DELETIONS/CHANGES                            *
*0000                                                           ZA00700
*0000139700-140300                                              ZA00142
*0000                                                          @Y30LSFY
*0000                                                          @ZM32752
*0000140864                                                    @ZA00002
*0000697027-697029                                             @ZA05469
*                                                                     *
* MODULE NAME = IFG0194A (OS/VS2)                                     *
*                                                                     *
* DESCRIPTIVE NAME = TAPE/DIRECT ACCESS VOLUME AND UNIT SELECTION     *
*                                                                     *
* COPYRIGHT = NONE                                                    *
*                                                                     *
* STATUS = RELEASE 2, LEVEL 0                                         *
*                                                                     *
* FUNCTION = THIS ROUTINE PERFORMS ALL VOLUME MOUNTING AND            *
*            DISMOUNTING FUNCTIONS FOR OPEN/CLOSE/EOV.                *
*                                                                     *
*            NEW VOLUME MOUNT FUNCTION - THE TYPE OF VOLUME           *
*            MOUNTING IS DERIVED FROM THE DATA SET ORGANIZATION.      *
*                                                                     *
*              A. PHYSICAL SEQUENTIAL & EXCP - VOLUME INDICATED BY    *
*                 DXVOLSEQ IS MOUNTED AND THE VOLUME INDICATED BY     *
*                 DXVOLSEQ + 1 IS LOOK-AHEAD MOUNTED.                 *
*                                                                     *
*              B. FOR NON-PHYSICAL SEQUENTIAL DATA SETS ALL VOLUMES   *
*                 ARE PARALLEL MOUNTED.                               *
*                                                                     *
*              C. ISAM OR BPAM - ALL VOLUMES ON ALL CONCATENATED      *
*                 DATA SETS ARE PARALLEL MOUNTED.                     *
*                                                                     *
*            THE VOLUME TO BE MOUNTED IS SELECTED AS INDICATED BY     *
*            DXVOLSEQ AND THE VOLUME SERIAL NUMBER IS SAVED AT        *
*            DXVOLSR2. THE VOLUME SERIAL NUMBER INDICATED BY          *
*            DXVOLSEQ + 1 IS SAVED AT DXVOLSR3. DXVOLSR3 WILL BE 0    *
*            IF DXVOLSEQ + 1 IS GREATER THAN THE CURRENT NO.          *
*            OF VOLUMES OR THE VOLUME COUNT SPECIFIED ON THE VOL      *
*            PARAMETER OF THE JCL.                                    *
*                                                                     *
*            IF THE VOLUME AT DXVOLSR2 IS OF THE FORM $TTRXX WHERE    *
*            $ = X'FF' AND TTR IS THE ADDRESS OF A JFCB EXTENSION,    *
*            THIS INDICATES THAT EITHER A NON-SPECIFIC REQUEST FOR    *
*            A TAPE VOLUME WAS MADE (TTR = 0 OR IS THE ADDRESS OF     *
*            THE JFCB WE ARE INTERRROGATING) OR A TAPE VOLUME WAS     *
*            REQUESTED USING VOL=REF ON THE JCL (IN WHICH CASE TTR    *
*            POINTS TO THE REFERENCED JFCB). IF THE REFERENCED        *
*            VOLUME IS ALSO NON-SPECIFIC, THE REFERENCED JFCB IS      *
*            REPOINTED TO THE CURRENT JFCB. THIS WILL CAUSE DD1 TO    *
*            LOOK LIKE IT IS REFERENCING DD2 IF DD2 ORIGINALLY        *
*            REFERENCED DD1 AND DD2 WAS OPENED FIRST. NOTE- VOL=REF   *
*            FOR TAPE ALWAYS USES THE LAST VOLUME OF THE REFERENCED   *
*            JFCB. IF ONLY ONE VOLUME EXISTS THIS WILL BE THE VOLUME  *
*            WHICH WILL BE USED.                                      *
*                                                                     *
*            THE VOLUME SPECIFIED BY DXVOLSR1 IS DISMOUNTED IF        *
*            THE NUMBER VOLUMES IS GREATER THAN THE NUMBER OF UNITS.  *
*            FOR EOV INPUT, THE DATA MANAGEMENT COUNT IN THE UCB      *
*            IS DECREMENTED. IF DXVOLSR2 COULD NOT BE OBTAINED        *
*            THE COUNT WOULD REMAIN INCREMENTED AND EOF PROCESSING    *
*            IS REQUESTED.                                            *
*                                                                     *
*            A UNIT IS SELECTED FOR DXVOLSR2 BASED ON THE ATTRIBUTES  *
*            OF THE VOLUME MOUNTED VERSUS THE ATTRIBUTES OF THE       *
*            VOLUME REQUESTED. THE DATA MANAGEMENT COUNT IS           *
*            INCREMENTED WHEN THE UNIT HAS BEEN SELECTED TO INFORM    *
*            THE SYSTEM THAT PROCESSING ON THIS UNIT IS IN PROGRESS.  *
*                                                                     *
*            WHEN A VOLUME SERIAL NUMBER (OR SCRTCH) AND A UNIT       *
*            HAVE BEEN SELECTED, THIS ROUTINE PASSES CONTROL TO A     *
*            TAPE OR DIRECT ACCESS MOUNT/VERIFY ROUTINE TO INSURE     *
*            THE CORRECT VOLUME IS MOUNTED ON THE SPECIFIED UNIT.     *
*            THE MOUNT/VERIFY ROUTINE RETURNS CONTROL TO THIS         *
*            ROUTINE TO SELECT ANOTHER VOLUME FOR PARALLEL MOUNTING   *
*            OR TO LOOK AHEAD MOUNT DXVOLSR3.                         *
*                                                                     *
*            BEFORE LOOK-AHEAD MOUNTING DXVOLSR3, IF DXVOLSR2 IS      *
*            SCRTCH, THE JFCB IS UPDATED TO REFLECT THE NEW VOLUME    *
*            SERIAL NUMBER FROM THE UCB.                              *
*                                                                     *
*            ONE OTHER ENTRY INTO THIS ROUTINE, PROVIDES              *
*            FOR PROCESSING DISP=MOD OUTPUT, WHERE THE LAST           *
*            VOLUME OF THE DATA SET IS NOT THE LAST VOLUME            *
*            SPECIFIED ON THE JCL.                                    *
*                                                                     *
*            ANOTHER ENTRY INTO THIS ROUTINE, PROVIDES A              *
*            VOLUME REJECT FUNCTION, WHEREBY THE VOLUME SPECIFIED     *
*            BY DXVOLSR2 COULD NOT BE USED.                           *
*                                                                     *
* NOTES = THIS MODULE MAY BE ENTERED FROM OPEN, CLOSE, OR EOV.        *
*                                                                     *
*    DEPENDENCIES = AS FOLLOWS -                                      *
*                                                                     *
*            1. CLASS ONE CHARACTER CODE.  THE EBCDIC CHARACTER CODE  *
*            WAS USED FOR ASSEMBLY.  THE MODULE MUST BE REASSEMBLED   *
*            IF A DIFFERENT CHARACTER SET IS USED FOR EXECUTION.      *
*                                                                     *
*            2. ALL VOLUMES MOUNTED AND RECOGNIZED (LABEL READ AND    *
*               VOLUME SERIAL NUMBER IN THE UCBVOLI FIELD OF THE UCB) *
*               HAVE BEEN PREVIOUSLY ENQD IF THE VOLUMES ARE NOT      *
*               SHAREABLE.                                            *
*                                                                     *
*            3. A PRIVATE REQUEST FOR A DIRECT ACCESS VOLUME WILL     *
*               RESULT IN EXCLUSIVE USE OF THE VOLUME UNLESS IT       *
*               HAS BEEN SPECIFICALLY REQUESTED.                      *
*                                                                     *
*            4. A PUBLIC REQUEST FOR A DIRECT ACCESS VOLUME MAY       *
*               BE SHARED WITH ANY OTHER PUBLIC DA REQUEST.           *
*                                                                     *
*            5. A PRIVATE OR PUBLIC REQUEST FOR TAPE WILL RESULT IN   *
*               EXCLUSIVE USE OF THE VOLUME.                          *
*                                                                     *
*            6. IF A REQUEST (DEFFERED OR NOT) IS MADE FOR A          *
*               PRIVATE TAPE VOLUME AND THE ALLOCATED UNIT            *
*               CONTAINS A VOLUME OTHER THAN THE ONE REQUESTED,       *
*               THE MOUNTED VOLUME WILL BE DISMOUNTED BY THE          *
*               SCHEDULER (ALSO FOR NON-DEFERED REQUESTS, A           *
*               MOUNT MESSAGE FOR THE RIGHT VOLUME IS ISSUED.         *
*                                                                     *
*            7. ALSO, FOR A NON-SPECIFIC REQUEST FOR A PRIVATE        *
*               TAPE VOLUME, SCHEDULER WILL UNLOAD ANY VOLUME         *
*               ON AN ALLOCATED UNIT. (A VOLUME READY AND VERIFIED    *
*               IS LEFT OVER FROM A PREVIOUS OPEN/CLOSE).             *
*                                                                     *
*    RESTRICTIONS =                                                   *
*            1. IF A PROCESS REQUIRES TWO DATA SETS TO SHARE THE      *
*               SAME NON-SPECIFIC TAPE VOLUME AND UNIT WITHOUT        *
*               AN INTERVENING MOUNT AND DISMOUNT, THE VOL=REF        *
*               FACILITY OF THE JCL MUST BE USED. IF NOT USED,        *
*               THE MOUNTED VOLUME WILL BE DISMOUNTED AND A           *
*               REQUEST WILL BE MADE FOR A SCRTCH OR PRIVAT VOLUME.   *
*                                                                     *
*    REGISTER CONVENTIONS =                                           *
*            R2 POINTS TO DCB                                         *
*            R3 IS THE BASE REGISTER                                  *
*            R4 POINTS TO OPEN WORK AREA                              *
*            R5 POINTS TO THE RESIDENT ROUTINE                        *
*            R6 POINTS TO THE WTG TABLE                               *
*            R7 POINTS TO THE CURRENT PARAMETER LIST ENTRY            *
*            R8 POINTS TO THE CURRENT WTG TABLE ENTRY                 *
*            R9 POINTS TO THE DD ENTRY IN THE TIOT                    *
*            R10 POINTS TO THE UCB                                    *
*                                                                     *
* PATCH LABEL = DEFINED IN THE XCTLTABL MACRO AT THE END OF THE       *
*               LISTING FOR THIS MODULE.                              *
*                                                                     *
* MODULE TYPE = CONTROL                                               *
*                                                                     *
*     PROCESSOR = ASSEMBLER XF                                        *
*                                                                     *
*     MODULE SIZE = SEE EXTERNAL SYMBOL DICTIONARY OR LOC FIELD ON    *
*                   ORG STATEMENT AT END OF LISTING                   *
*                                                                     *
*     ATTRIBUTES = REENTRANT, REFRESHABLE,READ-ONLY, ENABLED,         *
*                  PRIVILEGED, SUPERVISOR STATE, DATA MANAGEMENT KEY, *
*                  LINK PACK AREA RESIDENT/PAGEABLE                   *
*                                                                     *
* ENTRY POINT =                                                       *
*        IFG0194A FROM OPEN                                           *
*        IFG0204A FROM CLOSE                                          *
*        IFG0554A FROM EOV                                            *
*                                                                     *
*                  BRANCH OFFSET 0 = INITIAL ENTRY TO PERFORM         *
*                                    VOLUME AND UNIT SELECTION        *
*                                    FUNCTIONS LISTED ABOVE.          *
*                                                                     *
*                  BRANCH OFFSET 4 = ENTRY TO PERFORM DISPOSITION     *
*                                    ONLY FOR VOLUME SPECIFIED BY     *
*                                    DXVOLSR1 IN THE WORK AREA.       *
*                                                                     *
*                  BRANCH OFFSET 8 = RETURN FROM DISMOUNT ROUTINE.    *
*                                                                     *
*                  BRANCH OFFSET 12 = RETURN FROM VOLUME              *
*                                     MOUNT/VERIFY ROUTINE. ALSO      *
*                                     PARALLEL MOUNT RENTRY.          *
*                                                                     *
*                  BRANCH OFFSET 16 = ENTRY TO REJECT VOLUME          *
*                                     SPECIFIED BY DXVOLSR2.          *
*                                                                     *
*                  BRANCH OFFSET 20 = ENTRY TO REJECT VOLUME          *
*                                     SPECIFIED BY DXVOLSR2 FOR       *
*                                     DISP=MOD ON DIRECT ACCESS.      *
*                                                                     *
*                  BRANCH OFFSET 24 = ENTRY FROM PROBLEM              *
*                                     DETERMINATION TO RETRY          *
*                                     MOUNT VERIFY WITH NEW VOLUMES.  *
*                                                                     *
*                  BRANCH OFFSET 24 = RETURN FROM LOOK/AHEAD MOUNT    *
*                                     MESSAGE ROUTINE.                *
*                                                                     *
*                  BRANCH OFFSET 28 = RETURN FROM REJECT DXVOLSR2     *
*                                     MESSAGE ROUTINE.                *
*                                                                     *
*        LINKAGE = THIS MODULE IS ENTERED AS A SUBROUTINE.            *
*                                                                     *
*                  MVC   DXVOLSEQ,VOLSEQ    INDICATE WHICH VOLUME     *
*                  MVC   DXRETMOD(5),RETID  SAVE RETURN ID/VCON       *
*                  MVI   DXRETCOD,RETOFF    SAVE RETURN BRANCH OFFSET *
*                  IECRES LOAD,MODID=ID4A,BRCODE=OFFSET,BRANCH=QUEUED *
*                                                                     *
* INPUT = AS FOLLOWS -                                                *
*                                                                     *
*                REGISTERS 2-10 INITIALIZED AS TO OPEN/CLOSE/EOV      *
*                REGISTER CONVENTIONS.                                *
*                                                                     *
*                DXVOLSEQ - SEQUENCE NUMBER OF VOLUME TO BE MOUNTED.  *
*                DXVOLSR1 - VOLUME TO BE DISMOUNTED.                  *
*                DXUCBADR - ADDRESS OF UCB FOR DXVOLSR1 OR            *
*                           ADDRESS TO START SEARCH FOR VOLUME        *
*                           TO BE MOUNTED.                            *
*                                                                     *
*                OTHER AREAS AS TO OPEN, CLOSE, AND EOV CONVENTION.   *
*                                                                     *
* OUTPUT =                                                            *
*                                                                     *
*                 DXVOLSR1 - SET TO ZERO.                             *
*                 DXVOLSR2 - VOLUME MOUNTED AND VERIFIED OR SCRTCH.   *
*                 DXVOLSR3 - VOLUME LOOK-AHEAD MOUNTED.               *
*                 DXUCBADR - ADDRESS OF UCB FOR DXVOLSR2.             *
*                 DXUNITOF - OFFSET OF UCB IN TIOT ENTRY.             *
*                                                                     *
*                 VOLUME MOUNTED AND VERIFIED ON UCB SPECIFIED BY     *
*                 DXUCBADR WITH UCBDMCT INCREMENTED.                  *
*                                                                     *
*                 JFCB UPDATED AND WRITTEN TO JOBQUE.                 *
*                                                                     *
* EXIT-NORMAL = NORMAL EXIT IS TO THE CALLER VIA                      *
*                                                                     *
*               IC    RET,DXSAVOFF     LOAD RETURN OFFSET             *
*               IECRES LOAD,MODID=DXSAVMOD,BRCODE=(RET),BRANCH=QUEUED *
*                                                                     *
*               SUBROUTINE EXITS-                                     *
*                                                                     *
*               IFG0194C VERIFY VOLUME ON DA                          *
*               IFG0194F VERIFY VOLUME ON TAPE                        *
*               IFG0194J ISSUE MOUNT/DISMOUNT MESSAGES                *
*                                                                     *
* EXIT-ERROR = IFG0190P ALIAS = IFG0200P,IFG0550P                     *
*                                                                     *
* EXTERNAL REFERENCES = AS FOLLOWS -                                  *
*                                                                     *
*        ROUTINES =                                                   *
*                                                                     *
*         IFG019RA - OPEN/CLOSE/EOV RESIDENT ROUTINE                  *
*                    FOR IECRES FUNCTIONS.                            *
*                                                                     *
*         IFG0194C - DIRECT ACCESS MOUNT/VERIFY FUNCTION.             *
*                                                                     *
*         IFG0194F - TAPE MOUNT/VERIFY FUNCTION.                      *
*                                                                     *
*         IFG0194J - MESSAGE FUNCTION.                                *
*                                                                     *
*         DEQ - DEQ VOLUME RESOURCE.                                  *
*                                                                     *
*        DATA-AREAS = SEE DESCRIPTION OF IECDSECT FOR DESCRIPTION     *
*                     OF OPEN, CLOSE, AND EOV WORKAREAS.              *
*                                                                     *
*        CONTROL-BLOCKS = SEE LIST SPECIFIED BY IECDSECS              *
*                         MACRO CALL AT END OF LISTING.               *
*                                                                     *
* TABLES = AS FOLLOWS -                                               *
*          OPEN/EOV INTERNAL ABEND CODE TABLE. ONE ERROR GENERATES A  *
*          DIFFERENT ABEND CODE FOR OPEN AND EOV.                     *
*                                                                     *
* MACROS = THE FOLLOWING MACROS ARE USED-                             *
*                                                                     *
*              IECDSECS                                               *
*              IECEQU                                                 *
*              IECRES                                                 *
*              DMABCOND                                               *
*              XCTLTABL                                               *
*              SAVE                                                   *
*              RETURN                                                 *
*              SETLOCK                                                *
*              MODESET                                                *
*              DEQ                                                    *
*                                                                     *
* CHANGE ACTIVITY = FIRST RELEASE                                     *
*                                                                     *
***********************************************************************
         EJECT
         IECEQU AOS=YES
*
         USING FORCORE,RCORE
         USING IHADCB,RDCB
         USING TIOENTRY,RTIOT
         USING WTG,RWTG
         USING WTGENTRY,RWTGC           RWTGC POINTS TO WTGENTRY Y02134
         USING UCBOB,RUCB               RUCB POINTS TO UCB       Y02134
         USING USERPRML,RPARC           POINTS TO CURRENT ENTRY  Y02134
EABD185  EQU   185                      B37-4 ABEND CODE       @ZA02218
         BALR  RBASE,R0                 ESTABLISH ADDRESSABILITY
         USING *,RBASE
*                                                                Y02134
VUS00000 EQU   *                        GO TO BRANCH TABLE       Y02134
*                                                                Y02134
         B     VUS00100(RET)            BRANCH ON INDEX          Y02134
*                                                                Y02134
VUS00100 EQU   *                        BRANCH TABLE             Y02134
*                                                                Y02134
         B     VUS03000                 BRANCH TO INITIAL ENTRY  Y02134
         B     VUS02100                 BRANCH VOLUME DISPOSITN  Y02134
         B     VUS05010                 BRANCH RETURN FROM DISMT Y02134
         B     VUS01000                 BRANCH RETURN FROM MOUNT Y02134
         B     VUS02000                 BRANCH DXVOLSR2 REJECT   Y02134
         B     VUS02500                 BRANCH DISP=MOD REJECT   Y02134
         B     VUS11000                 BRANCH PD RETRY ENTRY    Y02080
         B     VUS09100                 LOOK AHEAD MOUNT RETURN  YM1350
         B     VUS02050                 REJECT RETURN            YM1108
         B     VUS02010                 I/O ERROR RETURN       @ZA03196
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        RETURN HERE FROM MOUNT VERIFY.                          Y02134
*        PHYSICAL SEQUENTIAL DATA SETS GO TO LOOK AHEAD MOUNT.   Y02134
*        NON-PHYSICAL SEQUENTIAL DATA SETS GO TO PARALLEL MOUNT. Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS01000 EQU   *                                                 Y02134
*                                                                Y02134
*        CHECK FOR PO BEFORE EXCP, DSORG FIELD EXISTS IN COPY    YM3925
*        EVEN IF NOT IN USERS DCB.                               YM3925
         TM    DCBDSORG,DCBDSGPO        TEST FOR BPAM DATA SET   YM3925
         BO    VUS03220                 BRANCH TO PARALLEL MOUNT YM3925
*                                                                YM3925
         TM    DCBMACRF,DCBMEXCP        IS ACCESS METHOD EXCP    Y02134
         BO    VUS06000                 BR IF EXCP, ASSUME PS    Y02134
*                                                                Y02134
VUS01010 EQU   *                        NOT EXCP                 Y02134
*                                                                Y02134
         NC    DCBDSORG,DCBDSORG        CHECK IF DSORG SPECIFIED Y02134
         BZ    VUS06000                 BRANCH 0, ASSUME PS      Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGPS        PHYSICAL SEQUENTIAL DCB  Y02134
         BO    VUS06000                 BR IF YES                Y02134
         B     VUS03220                 BRANCH TO PARALLEL MOUNT Y02134
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME SPECIFIED BY DXVOLSR2 IS TO BE REJECTED.         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS02000 EQU   *                                                 Y02134
*                                                                Y02134
         LR    RB,R0                    SAVE PD CODE IN RB       YM1108
         MVC   DXSAVMOD,DXRETMOD        SAVE RETURN LOAD         Y02134
         MVC   DXSAVOFF,DXRETCOD        SAVE RETURN OFFSET       Y02134
*                                                                Y02134
         MVC   DXWORK(L'UCBVOLI),UCBVOLI SAVE VOLUME TO DISMOUNT YM2850
*                                                                Y02080
         MVI   DXEXTSW,K0               SET DISP=MOD SW TO 0     YM1108
*                                                                YM1108
         L     RET,DXDSAB               FETCH DSAB ADDRESS       YM5957
         MODESET EXTKEY=SCHED           ASSUME DSAB KEY          YM5957
         NI    DSABFLG4-DSAB(RET),X'FF'-DSABCKVL VOLUME REJECTED YM5957
*                                       RESET ANY RESIDUAL CHKPT YM5957
*                                       SECURE FLAG IN DSAB      YM5957
         MODESET EXTKEY=DATAMGT         RESUME D/M KEY           YM5957
         CLI   UCBDMCT,X01              TEST FOR ONE USER        YM1472
         BNE   VUS02050                 BRANCH IF MORE THAN 1    YM1472
         TM    UCBSTAT,UCBRESV+UCBSYSR+UCBPRES RSRVD,SYSRES,PERM YM1108
         BNZ   VUS02050                 BRANCH IF ANY OR ALL     YM1108
         TM    UCBTBYT3,UCB3TAPE        TEST IF 2400 UNIT      @ZA03196
         BO    VUS02002                 BRANCH IF TAPE         @ZA03196
         TM    UCBSTAB,UCBBSVL          TEST FOR SHARED VOLUME   YM1108
         BNO   VUS02050                 BRANCH IF SHARED UNIT    YM1108
         EJECT
*
***********************************************************************
*
*        IF AN I/O ERROR OCCURRED ON TAPE SETUP FOR NEW TAPE.
*
***********************************************************************
*
VUS02002 EQU   *                        TEST FOR I/O ERROR     @ZA03196
*
         TM    DXECB,ECBNOERR           TEST FOR I/O ERROR     @ZA03196
         BO    VUS02020                 BRANCH IF NO ERROR     @ZA03196
*
         LTR   RB,RB                    CHECK IF WRONG VOL     @ZA03196
         BZ    VUS02020                 BRANCH TO GET NEW VOL  @ZA03196
         CLC   DXVOLSR2,SCRTCH          CHECK IF SCRTCH VOLUME @ZA03196
         BE    VUS02020                 BRANCH IF SCRATCH      @ZA03196
*
VUS02005 EQU   *                        ISSUE IEC513D MSG      @ZA03196
*
         XC    DXWORK2,DXWORK2          CLEAR REPLY AREA       @ZA03196
         LA    R1,DXWORK2               LOAD POINTER TO REPLY  @ZA03196
         LA    R0,K1                    LOAD LENGTH OF REPLY   @ZA03196
         SLL   R0,K24                   SHIFT TO HI BYTE       @ZA03196
         OR    R1,R0                    MOVE LENGTH TO REG 1   @ZA03196
*                                                              @ZA03196
         MVC   DXRETMOD,ID4A4A          RETURN MOD NAME        @ZA03196
         MVI   DXRETCOD,K36             RETURN OFFSET          @ZA03196
         IECRES LOAD,MODID=ID4A4J,BRCODE=K68,BRANCH=QUEUED     @ZA03196
         EJECT
*
***********************************************************************
*
*        RETURN WITH OPERATOR REPLY TO I/O ERROR MSG.
*
***********************************************************************
*
VUS02010 EQU   *                        CHECK REPLY            @ZA03196
*                                                              @ZA03196
         CLI   DXWORK2,CHARU            CHECK IF ABEND REQSTD  @ZA03196
         BE    VUS02020                 BRANCH TO ABEND        @ZA03196
*                                                              @ZA03196
         CLI   DXWORK2,CHARM            CHECK FOR NEW VOL      @ZA03196
         BNE   VUS02005                 BRANCH TO ASK AGAIN    @ZA03196
*                                                              @ZA03196
         XR    RB,RB                    CLEAR TO BYPASS ABEND  @ZA03196
*                                                                YM1108
VUS02020 EQU   *                        EXIT TO DISMOUNT ROUTINE YM2850
*                                                                YM2850
         MVC   DXRETMOD,ID4A4A          MOVE IN RETURN MOD NAME  YM1108
         MVI   DXRETCOD,K32             SET BRANCH RETURN CODE   YM1108
*                                                                YM1108
         IECRES LOAD,MODID=ID4A4J,BRCODE=K12,BRANCH=QUEUED       YM1157
         EJECT
*
***********************************************************************
*
*        RETURN FROM REWIND UNLOAD OF BAD VOLUME.
*
***********************************************************************
*                                                                YM1108
VUS02050 EQU   *                        RETURN FROM DISMOUNT     YM1108
*                                                                YM1108
         MVC   DXVOLSR1,DXWORK          SET UP DISMOUNTED VOLUME YM2850
*                                                                Y02134
*        NOTE - ERROR CODE PASSED IN REG 0 SAVED IN REG 11       YM1108
*                                                                Y02134
         LTR   R0,RB                    CHECK IF WRONG VOL       YM7396
         BZ    VUS02060                 BRANCH TO GET RIGHT VOL  YM7396
         CLC   DXVOLSR1,SCRTCH          CHECK FOR VOLSER SCRTCH  YM3008
         BE    VUS02070                 BRANCH SPECIFIC REQUEST  YM3111
         CLC   DXVOLSR2,SCRTCH          CHECK FOR SCRTCH VOLUME  Y02134
         BNE   VUS02070                 BRANCH SPECIFIC REQUEST  YM3111
*                                                                Y02134
         CLC   DXVOLSR1,UCBVOLI         CHECK IF STILL MOUNTED   YM1108
         BE    VUS02060                 BYPASS DEQ IF STILL UP   YM1108
*                                                                YM1108
         MVC   DXWORK1(K16),DEQLIST     INIT DEQ LIST            Y02134
         L     RF,DXJSCBAD              LOAD JSCB ADDRESS        Y02134
         L     R0,JSCBTCBP-IEZJSCB(,RF) INITIATOR TCB POINTER    Y02134
VUS02450 DEQ   (,DXVOLSR1),TCB=(0),MF=(E,DXWORK2),               Y02134X
               RELATED=(SYSZVOLS,IFG0194C(DAMV0202))             Y02134
*                                                                Y02134
VUS02060 EQU   *                        DECREMENT COUNT          YM1108
*                                                                Y02134
         BAL   RET,VUS07100             DECREMENT DM COUNT       Y02134
*                                                                Y02134
         STC   RB,DXRETCOD              INDICATE REJECT          YM1157
         XC    DXUCBSAV,DXUCBSAV        CLEAR TO RESAVE FIRST    Y02134
         B     VUS05020                 BRANCH TO GET NEW UNIT   YM1157
*                                                                YM3111
VUS02070 EQU   *                        PROB DET                 YM3111
*                                                                YM3111
         CLC   DXCALLID,ZEROS           CHECK IF ALREADY SET UP  YM4647
         BNZ   VUS02080                 BRANCH IF SET UP         YM4647
*                                                                YM4647
         MVC   DXCALLID,DXSAVMOD        SET UP ERROR MOD         YM3111
*                                                                YM4647
VUS02080 EQU   *                        PROB DET                 YM3111
*                                                                YM3111
         MVC   DXRETMOD,ID4A4A          SET UP RETRY MOD         YM3111
         IECRES LOAD,MODID=ID4A0P,BRCODE=K24,BRANCH=QUEUED       YM3111
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME SPECIFIED BY UCBVOLI IS TO BE DISMOUNTED.        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS02100 EQU   *                                                 Y02134
*                                                                Y02134
         MVC   DXSAVMOD,DXRETMOD        SAVE RETURN LOAD         Y02134
         MVC   DXSAVOFF,DXRETCOD        SAVE RETURN OFFSET       Y02134
*                                                                Y02134
         MVC   DXVOLSR1,UCBVOLI         MOVE BAD VOL TO DISMOUNT Y02134
         MVC   DXUCBSAV,DXUCBADR        SAVE UCB ADDRESS         YM1192
         MVC   DXUCBSAV(K1),DXUNITOF+K1 SAVE TIOT OFFSET         YM1192
         MVC   DXDSAB,DXDSABAD          SAVE DSAB ADDRESS        YM1192
         B     VUS05000                 BRANCH TO DISMOUNT       Y02134
*                                                                Y02134
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        FOR DIRECT ACCESS DISP=MOD, THE LAST VOLUME IS          Y02134
*        MOUNTED FIRST. IF THE DATA SET DOES NOT EXIST           Y02134
*        ON THE LAST VOLUME, A SEARCH IS MADE FOR THE            Y02134
*        END OF THE DATA SET, STARTING WITH THE FIRST VOLUME.    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS02500 EQU   *                                                 Y02134
*                                                                Y02134
         LH    RC,DXVOLSEQ              LOAD VOL SEQ NO.         Y02134
         CLC   JFCBNVOL,DXVOLSEQ+K1     CHECK TO SEE IF LAST VOL Y02134
         BE    VUS02510                 BRANCH IF LAST VOLUME    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        IF NOT THE LAST VOLUME AND THE VOLUME REJECT            Y02134
*        INDICATOR IS ON THE VOLUME SEQUENCE NUMBER IS           Y02134
*        TO BE DECREMENTED BY ONE, BECAUSE THE LAST VOLUME       Y02134
*        INDICATOR IN THE DSCB WAS NOT ON AND WAS NOT FOUND      Y02134
*        WHEN VOLUME N-1 WAS VERIFIED. THIS IS BECAUSE           Y02134
*        THE DATA SET WAS PROBABLY CREATED BEFORE RELEASE 17.    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         TM    DXEXTSW,ALLBITS          TEST FOR NO DSCB FOUND   Y02134
         BNO   VUS02520                 INCREMENT TO NEXT        Y02134
*                                                                Y02134
         BCTR  RC,R0                    DECREMENT COUNT          Y02134
         B     VUS02530                 BRANCH TO STORE COUNT    Y02134
*                                                                Y02134
VUS02510 EQU   *                        LAST VOLUME              Y02134
*                                                                Y02134
         MVI   DXEXTSW,K0               SET REJECT OFF           Y02134
         LH    RC,ONE                   SET COUNT TO ONE         Y02134
         B     VUS02530                 BRANCH TO STORE COUNT    Y02134
*                                                                Y02134
VUS02520 EQU   *                        INCREMENT TO NEXT DSCB   Y02134
*                                                                Y02134
         LA    RC,K1(RC)                INCREMENT BY ONE         Y02134
*                                                                Y02134
VUS02530 EQU   *                        STORE COUNT              Y02134
*                                                                Y02134
         STH   RC,DXVOLSEQ              UPDATE VOL SEQ NO.       Y02134
*                                                                Y02134
         CLC   JFCBNVOL,DXVOLSEQ+K1     CHECK TO SEE IF LAST VOL Y02134
         BE    VUS10060                 BR IF DATA SET NOT FOUND Y02134
*                                                                Y02134
         BAL   RET,VUS07100             DECREMENT DM COUNT       Y02134
*                                                                Y02134
         XC    DXUCBSAV,DXUCBSAV        CLEAR TO SAVE FIRST UNIT Y02134
*                                                                Y02134
         MVC   DXVOLSR1,DXVOLSR2        DISMOUNT CURRENT VOLUME  Y02134
         B     VUS03300                 BRANCH TO FIND VOLUMES   Y02134
         EJECT
*                                                                Y02134
VUS03000 EQU   *                        INITIAL ENTRY            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        THE TYPE OF VOLUME MOUNTING REQUIRED IS DERIVED         Y02134
*        FROM THE DATA SET ORGANIZATION.                         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         XC    DXUCBSAV,DXUCBSAV        CLEAR TO SAVE FIRST UNIT Y02134
*                                                                Y02134
         LA    RCORE,DXDSCB             CLEAR HI ORDER BYTE      Y02134
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE DEVICE     Y02134
         BO    VUS03200                 BRANCH IF TAPE           Y02134
         ST    RCORE,CORECH(,RCORE)     INITIALIZE FOR DA VERIFY Y02134
*                                                                Y02134
VUS03200 EQU   *                        CHECK DSORG              YM3925
*                                                                Y02134
         MVC   DXDSAB,DXDSABAD          INITIALIZE DSAB POINTER  Y02134
*                                                                Y02134
         MVC   DXSAVMOD,DXRETMOD        SAVE RETURN LOAD         Y02134
         MVC   DXSAVOFF,DXRETCOD        SAVE RETURN OFFSET       Y02134
*                                                                Y02134
*        CHECK FOR PO BEFORE EXCP, DSORG FIELD EXISTS IN COPY    YM3925
*        EVEN IF NOT IN USERS DCB.                               YM3925
         TM    DCBDSORG,DCBDSGPO        TEST FOR BPAM DATA SET   YM3925
         BO    VUS03210                 BRANCH TO PARALLEL MOUNT YM3925
*                                                                YM3925
         TM    DCBMACRF,DCBMEXCP        IS ACCESS METHOD EXCP    Y02134
         BO    VUS03300                 BRANCH IF EXCP           YM1228
*                                                                Y02134
         NC    DCBDSORG,DCBDSORG        CHECK IF DSORG SPECIFIED Y02134
         BZ    VUS03300                 BRANCH 0, ASSUME PS      Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGPS        PHYSICAL SEQUENTIAL DCB  Y02134
         BO    VUS03300                 BR IF YES                Y02134
*                                                                Y02134
VUS03210 EQU   *                        DSORG FOR DA DATA SETS   YM3925
*                                                                YM3925
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE DEVICE     Y02134
         LA    R0,K2                    OPEN=413=04 EOV=NA       YM1238
         BO    VUS10090                 BRANCH IF TAPE           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        FOR NON-PHYSICAL SEQUENTIAL DATA SETS ALL               Y02134
*        VOLUMES ARE TO BE MOUNTED. THE FIRST VOLUME             Y02134
*        IS SET UP AND THE REST ARE PARALLEL MOUNTED.            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         XC    DXVOLSEQ,DXVOLSEQ        INITIALIZE TO ZERO       Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGDA        TEST IF BDAM             Y02134
         BZ    VUS03218                 BRANCH IF NOT BDAM       Y02134
*                                                                Y02134
         MVI   DCBDRDX,K0               SET BDAM EXTENTS TO 0    Y02134
*                                                                Y02134
VUS03218 EQU   *                        PARALLEL MOUNT           Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGIS        TEST IF ISAM             Y02134
         BNO   VUS03220                 BRANCH IF NOT ISAM       Y02134
*                                                                Y02134
         MVC   DXDEBSYS(K1),DXDCBLST    SAVE ISAM OPTIONS        YM7051
ISAMOUT  EQU   X'7C'                    ISAM OUTPUT OPTIONS      YM7535
         TM    DCBMACRF+K1,ISAMOUT      TEST FOR ISAM OUTPUT     YM7535
         BZ    VUS03220                 BRANCH IF NOT OUTPUT     YM7535
*                                                                Y02134
         OI    DXDCBLST,PLISTOUT        SET OUTPUT IN PARM LIST  Y02134
*                                                                Y02134
VUS03220 EQU   *                        PARALLEL MOUNT           Y02134
*                                                                Y02134
         L     RB,DXDSAB                POINTER TO CURRENT DSAB  Y02134
         L     RTIOT,DSABTIOT-DSAB(RB)  LOAD TIOT POINTER        Y02134
         LH    RC,DXVOLSEQ              LOAD VOL SEQ INDEX       Y02134
         LA    RC,K1(RC)                INCREMENT BY ONE         Y02134
         STH   RC,DXVOLSEQ              UPDATE VOL SEQ COUNT     Y02134
*                                                                Y02134
         LA    RET,DXJBF                WORK AREA JFCB           Y02134
         C     RB,DXDSABAD              CHECK FOR CONCATENATION  Y02134
         BE    VUS03221                 BRANCH IF NO CONCATENATN Y02134
         LA    RET,TIOEJFCB             LOAD POINTER TO JFCB PTR Y02134
*                                                                Y02134
         IECRES LOCJFCB,(RET)                                    Y02134
*                                                                Y02134
VUS03221 EQU   *                        NO CONCATENATION         Y02134
*                                                                Y02134
         CLC   JFCBNVOL-INFMJFCB(K1,RET),ONE+K1 VALIDITY CHECK   Y02134
         LA    R0,K3                    OPEN=413-18 EOV=NA       Y02080
         BL    VUS10090                 BRANCH TO PROB DET       Y02134
         BE    VUS03222                 BRANCH PASSED BPAM CHECK Y02134
         CLC   DXVOLSEQ,ONE             CHECK FOR FIRST VOLUME   Y02134
         BE    VUS03222                 BRANCH PASSED BPAM CHECK Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGPO        CHECK FOR BPAM           Y02134
         BO    VUS03223                 BRANCH IF BPAM           Y02134
*                                                                Y02134
VUS03222 EQU   *                        NOT BPAM                 Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGDA        TEST IF BDAM           @ZA14612
         BZ    VUS03224                 BRANCH IF NOT BDAM     @ZA14612
*                                                              @ZA14612
         CLI   K0(RCORE),CHAR1          FMT 1 IN WORK AREA     @ZA14612
         BNE   VUS03224                 BRANCH IF NOT FMT 1    @ZA14612
*                                                              @ZA14612
         L     RF,LASTVOL               GO TEST FOR LAST VOLUME@ZA14612
         BR    RF                       INDICATOR IN DSCB      @ZA14612
*                                                              @ZA14612
VUS03224 EQU   *                                               @ZA14612
*                                                              @ZA14612
         CLC   JFCBNVOL-INFMJFCB(K1,RET),DXVOLSEQ+K1 MORE VOLS   Y02134
         BNL   VUS03230                 BRANCH TO FIND VOLUME    Y02134
*                                                                Y02134
VUS03223 EQU   *                        BPAM                     Y02134
*                                                                Y02134
         XC    DXVOLSEQ,DXVOLSEQ        INITIALIZE TO ZERO       Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGIS        ISAM DCB                 Y02134
         BO    VUS03225                 BR IF YES                Y02134
*                                                                Y02134
         TM    DCBDSORG,DCBDSGPO        BPAM DCB                 Y02134
         BO    VUS03225                 BR IF YES                Y02134
*                                                                Y02134
         B     VUS09000                 BRANCH, ALL FINISHED     Y02134
*                                                                Y02134
VUS03225 EQU   *                        ISAM DCB                 Y02134
*                                                                Y02134
         L     RB,DSABFCHN-DSAB(RB)     LOAD NEXT DSAB PTR       Y02134
         LTR   RB,RB                    TEST FOR END OF CHAIN    Y02134
         BZ    VUS09000                 BRANCH, END OF CHAIN     Y02134
*                                                                Y02134
         L     RTIOT,DSABTIOT-DSAB(RB)  LOAD TIOT POINTER        Y02134
         CLC   TIOEDDNM,BLANKS          CHECK FOR CONCATENATION  Y02134
         BNE   VUS09000                 BR IF NOT, ALL FINISHED  Y02134
*                                                                Y02134
         ST    RB,DXDSAB                SAVE CURRENT DSAB PTR   SA66776
*                                                               SA66776
         L     RUCB,TIOEFSRT-K1         GET UCB ADDRESS         SA66776
         LA    RUCB,UCBOB               CLEAR HI BYTE           SA66776
         LTR   RUCB,RUCB                CHECK FOR UNIT ADDRESS  SA66776
         LA    R0,K13                   OPEN=013-64 EOV=NA       Y02080
         BZ    VUS10090                 BRANCH IF DD DUMMY      ZA00142
*                                                                Y02134
         TM    UCBTBYT3,UCB3DACC        TEST FOR DA DEVICE      SA66776
         LA    R0,K15                   OPEN=C13-18 EOV=NA      SA66776
         BNO   VUS10090                 BRANCH IF NOT DA        SA66776
*                                                               SA66776
         TM    DCBDSORG,DCBDSGPO        BPAM DCB                 Y02134
         BO    VUS03227                 BR IF YES                Y02134
*                                                                Y02134
         OI    JFCBMASK+K4,JFCMISAM     ISAM PARALLEL MOUNT SW   Y02134
         B     VUS03220                 BRANCH, CHECK NEXT JFCB  Y02134
*                                                                Y02134
VUS03227 EQU   *                        BPAM DCB                 Y02134
*                                                                Y02134
         TM    DXDCBLST,PLISTOP1-PLISTIN  TEST FOR INPUT       @ZA00002
         LA    R0,K14                   OPEN=C13-10 EOV=NA       YM2899
         BNZ   VUS10090                 BRANCH IF OUTPUT         YM2899
*                                                                Y02134
         OI    JFCBMASK+K4,JFCMBPAM     BPAM CONCATENATION SW ON Y02134
         B     VUS03220                 BRANCH, CHECK NEXT JFCB  Y02134
*                                                                Y02134
VUS03230 EQU   *                        FIND VOLUME              Y02134
*                                                                Y02134
         CLC   DXVOLSEQ,ONE             CHECK FOR FIRST VOLUME   Y02134
         BNH   VUS03235                 BRANCH IF FIRST VOLUME   Y02134
         OI    JFCBMASK+K4,JFCMISAM     SET PARALLEL MOUNT SW    Y02134
*                                                                Y02134
VUS03235 EQU   *                        FIRST VOLUME             Y02134
*                                                                Y02134
         CLC   JFCBNVOL-INFMJFCB(K1,RET),JFCBVLSQ-INFMJFCB(RET)  Y02080
         LA    R0,K4                    OPEN=413-1C EOV=NA       Y02080
         BL    VUS10090                 BRANCH TO PROB DET       Y02080
*                                                                Y02134
         LA    RD,JFCBVOLS-INFMJFCB(RET) LOAD POINTER TO VOLUMES Y02134
         LA    RET,JFCBEXAD-INFMJFCB(RET) SET UP POINTER FOR EXT Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN MAIN JFCB                           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         CH    RC,VOL5                  VOL SEQ LESS THAN 5      Y02134
         BNH   VUS03250                 BRANCH, VOL SER IN MAIN  Y02134
         SH    RC,VOL5                  DECREMENT FOR INDEXING   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN JFCB EXTENSION.                     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03240 EQU   *                                                 Y02134
*                                                                Y02134
         CLC   K0(L'JFCBEXAD,RET),ZEROS IS THERE AN EXTENSION    Y02134
         LA    R0,K5                    OPEN=113-18 EOV=NA       Y02080
         BZ    VUS10090                 BRANCH IF NO EXTENSION   Y02134
         IECRES LOCJFCB,(RET)           LOCATE JFCB EXTENSION    Y02134
         LA    RD,K4(RET)               LOAD POINTER TO VOLUMES  Y02134
         CH    RC,VOL15                 VOL SEQ LESS THAN 15     Y02134
         BNH   VUS03250                 BRANCH, VOL SER IN EXT   Y02134
         SH    RC,VOL15                 DECREMENT FOR INDEXING   Y02134
         B     VUS03240                 BRANCH TO CHECK NEXT EXT Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME TO BE PARALLEL MOUNTED HAS BEEN FOUND.           Y02134
*        RC CONTAINS THE POSITION OF THE VOLUMES IN THE LIST.    Y02134
*        RD POINTS TO THE LIST OF VOLUMES.                       Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03250 EQU   *                        SET UP NEXT VOLUME       YM3822
*                                                                Y02134
         BCTR  RC,R0                    DECREMENT FOR INDEXING   Y02134
         MH    RC,VOL6                  CONVERT SEQ NO TO OFFSET Y02134
         AR    RD,RC                    ADD OFFSET TO BASE       Y02134
         MVC   DXVOLSR2,K0(RD)          MOVE VOL N TO WA         Y02134
*                                                                YM3822
         TM    UCBJBNR,UCBVRDEV         TEST FOR VIO UCB         YM3822
         BO    VUS04200                 BRANCH IF VIO DATA SET   YM3822
*                                                                YM3822
         LA    R0,K6                    OPEN=413-18 EOV= NA      Y02080
         CLC   DXVOLSR2,ZEROS           CHECK FOR ZEROS          Y02134
         BZ    VUS10090                 BRANCH IF NO VOLUME      Y02134
         CLC   DXVOLSR2,BLANKS          CHECK FOR BLANKS         Y02134
         BZ    VUS10090                 BRANCH IF NO VOLUME      YM3822
         XC    DXVOLSR3,DXVOLSR3        INVALIDATE NEXT VOL SLOT Y02134
         B     VUS05000                 BRANCH TO SELECT UNIT    Y02134
         EJECT
*                                                                Y02134
VUS03300 EQU   *                        FIND VOLUMES             Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        SEARCH FOR THE CURRENT AND NEXT VOLUME TO BE MOUNTED.   Y02134
*        (N & N+1) N IS THE NUMBER SPECIFIED BY DXVOLSEQ.        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         LH    RC,DXVOLSEQ              LOAD VOLUME SEQUENCE NO. Y02134
         LA    RD,JFCBVOLS              LOAD POINTER TO VOLUMES  Y02134
         LA    RET,JFCBEXAD             SET UP POINTER FOR EXT   Y02134
*                                                                Y02134
         TM    DXDCBLST,PLISTM03        CHECK FOR RDBACK         Y02134
         BNM   VUS03320                 BRANCH IF NOT RDBACK     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        SEARCH FOR THE N+1 VOLUME FIRST.                        Y02134
*        FOR RDBACK N+1 IS REALLY VOLUME N-1.                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         CH    RC,ONE                   CHECK FOR ONLY 1 VOLUME  Y02134
         BE    VUS03360                 BRANCH TO GET CURRENT    Y02134
         BCTR  RC,R0                    IT'S NEVER NEGATIVE      Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN MAIN JFCB                           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03320 EQU   *                                                 Y02134
*                                                                Y02134
         CH    RC,VOL5                  VOL SEQ LESS THAN 5      Y02134
         BL    VUS03340                 BRANCH, IF BOTH IN MAIN  Y02134
         BE    VUS03350                 BRANCH IF 1ST IN MAIN    Y02134
         SH    RC,VOL5                  DECREMENT FOR INDEXING   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN JFCB EXTENSION.                     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03330 EQU   *                                                 Y02134
*                                                                Y02134
         CLC   K0(L'JFCBEXAD,RET),ZEROS IS THERE AN EXTENSION    Y02134
         LA    R0,K9                    OPEN=113-08 EOV=837-08   Y02080
         BZ    VUS10085                 BRANCH IF NO EXTENSION   YM6514
         IECRES LOCJFCB,(RET)           LOCATE JFCB EXTENSION    Y02134
         LA    RD,K4(RET)               LOAD POINTER TO VOLUMES  Y02134
         CH    RC,VOL15                 VOL SEQ LESS THAN 15     Y02134
         BL    VUS03340                 BRANCH, BOTH VOLS IN EXT Y02134
         BE    VUS03350                 BRANCH IF 1ST IN MAIN    Y02134
         SH    RC,VOL15                 DECREMENT FOR INDEXING   Y02134
         B     VUS03330                 BRANCH TO CHECK NEXT EXT Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME N & N+1 HAVE BEEN LOCATED IN THE SAME EXTENSION. Y02134
*        RC CONTAINS THE POSITION OF THE VOLUMES IN THE LIST.    Y02134
*        RD POINTS TO THE LIST OF VOLUMES.                       Y02134
*        RE POINTS TO THE ADDRESS OF THE NEXT EXTENSION.         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03340 EQU   *                                                 Y02134
*                                                                Y02134
         BCTR  RC,R0                    DECREMENT FOR INDEXING   Y02134
         MH    RC,VOL6                  CONVERT SEQ NO TO OFFSET Y02134
         AR    RD,RC                    ADD OFFSET TO BASE       Y02134
         MVC   DXVOLSR2,K0(RD)          MOVE VOL N TO WA         Y02134
         AH    RD,VOL6                  ADD VOLUME SERIAL LENGTH Y02134
         B     VUS03360                 BRANCH TO GET VOLUME N+1 Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME N & N+1 HAVE BEEN LOCATED. VOLUME N IS IN        Y02134
*        THE LIST POINTED TO BY RD. VOLUME N+1 IS THE FIRST      Y02134
*        VOLUME IN THE EXTENSION POINTED BY RE.                  Y02134
*        RC CONTAINS THE POSITION OF VOLUME N IN THE LIST.       Y02134
*        RD POINTS TO THE LIST OF VOLUMES.                       Y02134
*        RE POINTS TO THE ADDRESS OF THE NEXT EXTENSION.         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03350 EQU   *                                                 Y02134
*                                                                Y02134
         BCTR  RC,R0                    DECREMENT FOR INDEXING   Y02134
         MH    RC,VOL6                  CONVERT SEQ NO TO OFFSET Y02134
         AR    RD,RC                    ADD OFFSET TO BASE       Y02134
         MVC   DXVOLSR2,K0(RD)          MOVE VOL N TO WA         Y02134
         CLC   K0(L'JFCBEXAD,RET),ZEROS IS THERE AN EXTENSION    Y02134
         BZ    VUS03370                 BRANCH IF NO EXTENSION   Y02134
         IECRES LOCJFCB,(RET)           LOCATE JFCB EXTENSION    Y02134
         LA    RD,K4(RET)               LOAD POINTER TO VOLUMES  Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME N+1 IS POINTED TO BY RD.                         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03360 EQU   *                                                 Y02134
*                                                                Y02134
         MVC   DXVOLSR3,K0(RD)          MOVE VOL N+1 TO WA       Y02134
*                                                                Y02134
VUS03370 EQU   *                        NO EXTENSIONS            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        SWAP VOLUME N WITH N+1 IF RDBACK                        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         TM    DXDCBLST,PLISTM03        CHECK FOR RDBACK         Y02134
         BNM   VUS03380                 BRANCH IF NOT RDBACK     Y02134
*                                                                Y02134
         XC    DXVOLSR2,DXVOLSR3        SWAP VOLUME SERIAL NO.S  Y02134
         XC    DXVOLSR3,DXVOLSR2        IF RDBACK, N=N+1 & N+1=N Y02134
         XC    DXVOLSR2,DXVOLSR3        VOLUMES ARE NOW SWAPPED  Y02134
*                                                                Y02134
VUS03380 EQU   *                        NOT RDBACK               Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VALIDITY CHECK VOLUME N+1                               Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         CLC   JFCBNVOL,DXVOLSEQ+K1     VOL N+1 AVAILABLE        Y02134
         BH    VUS03390                 BRANCH IF N+1 IS REAL    Y02134
*                                                                Y02134
         MVC   DXVOLSR3,ZEROS           INVALIDATE VOL N+1       Y02134
*                                                                Y02134
         CLC   JFCBVLCT,DXVOLSEQ+K1     VOL N+1 REQUESTED        Y02134
         BNH   VUS03390                 BRANCH, BYPASS N+1 MOUNT Y02134
*                                                                Y02134
         TM    DXDCBLST,PLISTOIN        OUTPUT OR OUTIN          Y02134
         BNO   VUS03390                 BRANCH IF NEITHER        Y02134
*                                                                Y02134
         MVC   DXVOLSR3,SCRTCH          MAKE VOL N+1 IS SCRATCH  Y02134
*                                                                Y02134
VUS03390 EQU   *                        NEITHER OUTPUT NOR OUTIN Y02134
*                                                                Y02134
         CLC   DXVOLSR3,BLANKS          CHECK FOR BLANKS         Y02134
         BNE   VUS03400                 BRANCH IF NOT BLANKS     Y02134
*                                                                Y02134
         MVC   DXVOLSR3,ZEROS           INVALIDATE VOL N+1       Y02134
*                                                                Y02134
VUS03400 EQU   *                        NOT BLANK                Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VALIDITY CHECK VOLUME N                                 Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         TM    UCBJBNR,UCBVRDEV         TEST FOR VIO UCB         YM3822
         BO    VUS04200                 BRANCH IF VIO DATA SET   YM3822
*                                                                YM3822
         CLC   DXVOLSR2,ZEROS           CHECK FOR ZEROS          Y02134
         BZ    VUS04100                 ASSUME SCRTCH            Y02134
*                                                                Y02134
         CLC   DXVOLSR2,BLANKS          CHECK FOR BLANKS         Y02134
         BE    VUS04100                 ASSUME SCRTCH            Y02134
*                                                                Y02134
         TM    UCBTBYT3,UCB3DACC        TEST FOR DIRECT ACCESS   Y02134
         BO    VUS05000                 ASSUME SPECIFIC REQUEST  Y02134
*                                                                Y02134
         CLI   DXVOLSR2,VIRTUAL         CHECK FOR VIRTUAL VOLUME Y02134
         BNE   VUS05000                 ASSUME SPECIFIC REQUSET  Y02134
*                                                                Y02134
         CLI   JFCBVOLS,VIRTUAL         CHECK FOR FIRST VOLUME   Y02134
         BNE   VUS04100                 JFCB IS OUT OF ORDER     Y02134
*                                                                Y02134
         CLC   DXVOLSR2+K1(K3),BLANKS   CHECK FOR VIRTUAL SCRTCH Y02134
         BE    VUS04000                 BRANCH, SCRTCH REQUEST  SA54607
*                                                                Y02134
         CLC   DXVOLSR2+K1(K3),ZEROS    CHECK FOR VIRTUAL SCRTCH Y02134
         BE    VUS04000                 BRANCH, SCRTCH REQUEST  SA54607
*                                                                Y02134
         CLC   DXVOLSR2+K1(K3),TIOEJFCB CHECK FOR VIRTUAL SCRTCH Y02134
         BE    VUS04000                 BRANCH IF SCRTCH REQUEST Y02134
         B     VUS03430                 BRANCH, PROCESS VOL=REF  Y02134
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOL=REF=DDNAME WAS SPECIFIED.                           Y02134
*        GET LAST VOLUME IN REFERENCED JFCB.                     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03430 EQU   *                                                 Y02134
*                                                                Y02134
         LA    RET,INFMJFCB             LOAD POINTER TO JFCB     Y02134
*                                                                Y02134
VUS03440 EQU   *                        BOTH VOLS IN EXTENSION   Y02134
*                                                                Y02134
         LR    RD,RET                   SAVE LAST JFCB POINTER   Y02134
         LA    RET,JFCBVOLS-INFMJFCB+K1(RET) POINT TO REF JFCB   Y02134
*                                                                Y02134
         IECRES LOCJFCB,(RET)           LOCATE JFCB              Y02134
*                                                                Y02134
         CLI   JFCBVOLS-INFMJFCB(RET),VIRTUAL VOL=REF SER NO.    Y02134
         BE    VUS03445                 BRANCH, CHECK IF VOL=REF Y02134
*                                                                Y02134
         CLC   JFCBVOLS-INFMJFCB(K6,RET),ZEROS CHECK FOR SCRTCH  Y02080
         BE    VUS03450                 BRANCH TO ASSUME SCRTCH  Y02134
*                                                                Y02134
         CLC   JFCBVOLS-INFMJFCB(K6,RET),BLANKS CHECK FOR SCRTCH Y02080
         BE    VUS03450                 BRANCH TO ASSUME SCRTCH  Y02134
*                                                                Y02134
         CLI   JFCBNVOL-INFMJFCB(RET),K0 CHECK IF SPECIFIED      Y02080
         BNE   VUS03460                 BRANCH TO FIND LAST VOL  Y02080
         B     VUS03450                 BRANCH TO ASSUME SCRTCH  Y02134
*                                                                Y02134
VUS03445 EQU   *                        CHECK IF VOL = REF       Y02134
*                                                                Y02134
         CLC   JFCBVOLS-INFMJFCB+K1(K3,RET),BLANKS GDG SCRTCH    Y02134
         BE    VUS03450                 BRANCH IF SCRTCH REQUEST Y02134
*                                                                Y02134
         CLC   JFCBVOLS-INFMJFCB+K1(L'JFCBEXAD,RET),ZEROS        Y02134
         BZ    VUS03450                 BRANCH IF NO JFCB        Y02134
*                                                                Y02134
         CLC   JFCBVOLS-INFMJFCB(K4,RET),JFCBVOLS-INFMJFCB(RD)   Y02134
         BE    VUS03450                 BRANCH IF SCRTCH REQUEST Y02134
         B     VUS03440                 BRANCH, GET NEXT JFCB    Y02134
*                                                                Y02134
VUS03450 EQU   *                        ASSUME SCRTCH            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOL=REF WAS SPECIFIED TO A DATA SET WHICH ALSO HAS      Y02134
*        NON-SPECIFIC VOLUMES. THIS MEANS THAT THE DATA SETS     Y02134
*        ARE BEING OPENED IN AN ORDER DIFFERENT THAN WAS         Y02134
*        SPECIFIED ON THE JCL. THE ENTRIES WILL BE RECHAINED     Y02134
*        IN THE ORDER OPENED.                                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         TM    DXDCBLST,PLISTOIN        OUTPUT OR OUTIN          Y02134
         LA    R0,K7                    OPEN=413-18 EOV=837-08   Y02134
         BNO   VUS10085                 BRANCH IF NEITHER        YM6514
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        READ IN THE REFERENCED JFCB AND UPDATE THE SERIAL NO.   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         IECRES GET,LV=JFCBLGTH,ID=JFCB,                         Y02134X
               PREFIX=YES,STM=(2,14,WTGPREFX)                    Y02134
*                                                                Y02134
         MVC   K0(JFCBLGTH,R1),K0(RET)  MOVE IN REFERENCED JFCB  Y02134
         LR    RET,R1                   LOAD JFCB PTR            Y02134
         MVC   DXVOLSR2,JFCBVOLS-INFMJFCB(RET) NEW VOL SER TO WA Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHANGE REFERENCED JFCB TO POINT TO THE CURRENT JFCB.    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         MVC   JFCBVOLS-INFMJFCB+K1(K3,RET),TIOEJFCB             Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        WRITE THE JFCB BACK.                                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         LA    R1,JFCBVOLS-INFMJFCB+K1(RD) POINT TO REF JFCB PTR Y02134
*                                                                Y02134
         IECRES WRJFCB,(R1),(RET)       WRITE JFCB BACK          Y02134
*                                                                Y02134
         IECRES FREE,A=(RET),PREFIX=YES,STM=(0,14,WTGPREFX)      Y02134
*                                                                Y02134
         B     VUS04000                 BRANCH TO ASSUME SCRTCH  Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        FIND LAST VOLUME IN REFERENCED JFCB                     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03460 EQU   *                                                 Y02134
*                                                                Y02134
         XR    RC,RC                    CLEAR FOR IC INST        Y02134
         IC    RC,JFCBNVOL-INFMJFCB(RET) GET NUMBER OF VOLUMES   Y02134
         LA    RD,JFCBVOLS-INFMJFCB(RET) LOAD POINTER TO VOLUMES Y02134
         LA    RET,JFCBEXAD-INFMJFCB(RET) SET UP POINTER FOR EXT Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN MAIN JFCB                           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         CH    RC,VOL5                  VOL SEQ LESS THAN 5      Y02134
         BNH   VUS03480                 BRANCH, VOL SER IN MAIN  Y02134
         SH    RC,VOL5                  DECREMENT FOR INDEXING   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN JFCB EXTENSION.                     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03470 EQU   *                                                 Y02134
*                                                                Y02134
         CLC   K0(L'JFCBEXAD,RET),ZEROS IS THERE AN EXTENSION    Y02134
         LA    R0,K9                    OPEN=113-08 EOV=837-08   Y02080
         BZ    VUS10090                 BRANCH IF NO EXTENSION   Y02134
         IECRES LOCJFCB,(RET)           LOCATE JFCB EXTENSION    Y02134
         LA    RD,K4(RET)               LOAD POINTER TO VOLUMES  Y02134
         CH    RC,VOL15                 VOL SEQ LESS THAN 15     Y02134
         BNH   VUS03480                 BRANCH, VOL SER IN EXT   Y02134
         SH    RC,VOL15                 DECREMENT FOR INDEXING   Y02134
         B     VUS03470                 BRANCH TO CHECK NEXT EXT Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        REFERENCED VOLUME HAS BEEN FOUND.                       Y02134
*        RC CONTAINS THE POSITION OF THE VOLUMES IN THE LIST.    Y02134
*        RD POINTS TO THE LIST OF VOLUMES.                       Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS03480 EQU   *                                                 Y02134
*                                                                Y02134
         BCTR  RC,R0                    DECREMENT FOR INDEXING   Y02134
         MH    RC,VOL6                  CONVERT SEQ NO TO OFFSET Y02134
         AR    RD,RC                    ADD OFFSET TO BASE       Y02134
         MVC   DXVOLSR2,K0(RD)          MOVE VOL N TO WA         Y02134
         MVC   JFCBVOLS(L'DXVOLSR2),DXVOLSR2 MOVE VOLUME TO JFCB Y02134
*                                                                YM5928
         OI    JFCBTSDM,JFCVSL          SET VOL SER LIST MODIF-  A45617
*                                       IED SO SCHEDULER WILL    A45617
*                                       RECATALOG WITH NEW VOL   A45617
*                                                                YM5928
         OI    JFCBMASK+K4,X80          INDICATE JFCB MODIFIED   Y02134
*                                                              @Y30LSFY
         XC    JFCVINDX(K2),JFCVINDX    CLEAR VVIC INDICATORS  @Y30LSFY
*                                                              @Y30LSFY
*                                                                YM5928
         LA    R1,TIOEJFCB              POINT TO MAIN JFCB       YM5928
         LA    RET,DXJBF                POINT TO WORK AREA JFCB  YM5928
*                                                                YM5928
         IECRES WRJFCB,(R1),(RET)       WRITE JFCB BACK          YM5928
*                                                                YM5928
         B     VUS05000                 BRANCH TO SELECT UNIT    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME WAS NON-SPECIFIC. INDICATE SCRTCH.               Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS04000 EQU   *                                                 Y02134
*                                                                Y02134
         MVC   JFCBVOLS(K6),BLANKS      MOVE BLANKS TO JFCB      Y02134
*                                                                Y02134
VUS04100 EQU   *                        SCRATCH REQUESTED        YM3822
*                                                                Y02134
         TM    DXDCBLST,PLISTOIN        OUTPUT OR OUTIN          Y02134
         LA    R0,K7                    OPEN=413-18 EOV=837-08   Y02080
         BNO   VUS10085                 BRANCH IF NEITHER        YM6514
*                                                                YM5928
         MVC   DXVOLSR2,SCRTCH          INDICATE SCRTCH REQUEST  Y02134
*                                                               SA54607
         CLC   DXVOLSEQ,ONE             CHECK FOR FIRST VOLUME  SA54607
         BNE   VUS05000                 BRANCH, BYPASS NEW-MOD  SA54607
         TM    JFCBIND2,JFCOLD          IS DISP EQUAL 'MOD'-    SA50304
         BO    VUS05000                 BR IF NOT- CONTINUE     SA54607
*                                                               SA54607
         OI    JFCBIND2,JFCNEW          CHANGE 'MOD' TO 'NEW'   SA50304
         OI    JFCBFLG2,JFCMODNW        INDICATE THAT 'MOD' WAS SA50304
*                                       CHANGED TO 'NEW'        SA50304
         B     VUS05000                 BRANCH TO GET UNIT       YM3822
*                                                                YM3822
VUS04200 EQU   *                        VIO DATA SET             YM3822
*                                                                YM3822
         CLC   DXVOLSEQ,ONE             CHECK FOR VOLUME 1       YM3855
         LA    R0,K10                   OPEN=413-04 EOV=B37-04   YM3855
         BNE   VUS10070                 BRANCH IF NOT ONE VOLUME YM3855
*                                                                YM3855
         MVC   DXVOLSR2,VUSVIO          SET UP VIO VOLSER        YM3822
         XC    DXVOLSR3,DXVOLSR3        CLEAR LOOK-AHEAD         YM3822
         EJECT
*                                                                Y02134
VUS05000 EQU   *                        VOLUME DISPOSITION       Y02134
         TM    JFCBLTYP,JFCBLP          TEST FOR BLP           @ZA15095
         L     RF,SETMODE               EPA OF SETMODE RTN     @ZA15095
         BOR   RF                       BRANCH TO CLEAR        @ZA15095
*                                       VOLUME SERIAL          @ZA15095
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        UNIT SELECTION                                          Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS05005 XC    DXUCBADR(K2),DXUCBADR    CLEAR HI TWO BYTES     @ZA15095
         L     RUCB,DXUCBADR            LOAD CURRENT UCB ADDRESS Y02134
*                                                                YM7516
         TM    UCBTBYT3,UCB3DACC        TEST FOR DA UNIT         YM7516
         BO    VUS05010                 BRANCH IF DIRECT ACCESS  YM7516
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        FINISH POSITIONING VOL N-1                              YM7516
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         NC    DXVOLSR1,DXVOLSR1        TEST FOR VOL SPECIFIED   Y02134
         BZ    VUS05010                 BRANCH IF NO VOLUME      Y02134
         CLC   UCBVOLI,DXVOLSR1         VOLUME ON DEVICE         Y02134
         BNE   VUS05010                 BYPASS IF VOL NOT ON DEV YM7516
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        GO TO FINISH POSITIONING                                YM7516
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         MVC   DXRETMOD,ID4A4A          RETURN ID TO SAVE AREA   Y02134
         MVI   DXRETCOD,K8              SET RETURN INDICATOR     Y02134
*                                                                Y02134
         IECRES LOAD,MODID=ID4A4J,BRCODE=K0,BRANCH=QUEUED        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        RETURN HERE FROM POSITIONING                            YM7516
*        OR                                                      Y02134
*        MOUNT AND VERIFY VOLUME SPECIFIED BY DXVOLSR2           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS05010 EQU   *                                                 Y02134
*                                                                Y02134
         MVI   DXRETCOD,K0              SET REJECT OFF           YM1157
*                                                                YM1157
VUS05020 EQU   *                        ENTRY FROM REJECT        YM1157
*                                                                YM1157
         NC    DXVOLSR2,DXVOLSR2        CHECK IF VOL N SPECIFIED Y02134
         BZ    VUS09100                 BRANCH IF NONE SPECIFIED Y02134
*                                                                Y02134
         L     RF,WTGPREFX              LOAD POINTER TO PREFIX   Y02134
         STM   R0,RF,IECREGSV-IECPREFX(RF) SAVE REGISTERS        Y02134
*                                                                Y02134
         MODESET EXTKEY=ZERO            SETLOCK KEY              Y02082
*                                                                Y02134
VUS05110 SETLOCK OBTAIN,TYPE=LOCAL,MODE=UNCOND,                  Y02134X
               RELATED=(LOCAL,IFG0194A(VUS05430))                Y02134
*                                                                Y02134
         MODESET EXTKEY=DATAMGT         RETURN TO DATA MGMNT KEY Y02082
*                                                                Y02134
         LTR   RD,RD                    TEST FOR SETLOCK ERROR   Y02134
         LM    R0,RF,IECREGSV-IECPREFX(RF) RESTORE REGISTERS     Y02134
         BNZ   VUS10000                 BRANCH IF ERROR          Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        FIND NUMBER OF UNITS                                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         XR    RB,RB                    CLEAR REG FOR IC INST    Y02134
         IC    RB,TIOELNGH              GET TIOT ENTRY LENGTH    Y02134
         LA    RF,TIOEFSRT-K1-TIOENTRY  GET LENGTH TO UCBS       Y02134
         SR    RB,RF                    GET LENGTH OF UCB LIST   Y02134
         SRL   RB,K2                    DIVIDE BY FOUR           Y02134
         STH   RB,DXNOUNIT              SAVE NUMBER OF UNITS     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        FIND UNIT TO PROCESS DXVOLSR2                           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         XR    RC,RC                    SET OFFSET TO ZERO       Y02134
*                                                                Y02134
VUS05120 EQU   *                                                 Y02134
*                                                                Y02134
         BAL   RET,VUS07000             BRANCH TO GET NEXT UNIT  Y02134
         CLM   RUCB,K7,DXUCBADR+K1      VERIFY UCB ADDRESS     @ZA19129
         BE    VUS05130                 BRANCH IF OFFSET FOUND   Y02134
         BCT   RB,VUS05120              LOOP IF MORE UCBS        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        IF THE UNIT IS NOT WITHIN THE TIOT ENTRY, IT MUST BE    Y02134
*        IN THE PREVIOUS CONCATENATED ENTRY. GET FIRST UNIT.     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         XR    RC,RC                    SET TO ZERO TO GET FIRST Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        UNIT HAS BEEN LOCATED. IF PREVIOUS DISMOUNT             Y02134
*        OCCURRED, GET NEXT UNIT.                                Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS05130 EQU   *                                                 Y02134
*                                                                Y02134
         NC    DXVOLSR1,DXVOLSR1        TEST FOR PREVIOUS VOLUME Y02134
         BZ    VUS05140                 BRANCH IF NONE           Y02134
         BAL   RET,VUS07000             GET NEXT UNIT            Y02134
*                                                                Y02134
VUS05140 EQU   *                        NO PREVIOUS VOLUME       Y02134
*                                                                Y02134
         LTR   RC,RC                    PREVIOUS CONCATENATION   Y02134
         BNZ   VUS05150                 BRANCH IF UNIT FOUND     Y02134
         BAL   RET,VUS07000             LINK TO GET FIRST UNIT   Y02134
*                                                                Y02134
VUS05150 EQU   *                        UNIT FOUND               Y02134
*                                                                Y02134
         NC    DXVOLSR1,DXVOLSR1        TEST FOR PREVIOUS VOLUME Y02134
         BZ    VUS05160                 BRANCH IF NONE           Y02134
         CLC   UCBVOLI,DXVOLSR1         TEST FOR PREVIOUS VOLUME Y02134
         BNE   VUS05160                 BRANCH NOT PREVIOUS      Y02134
         BAL   RET,VUS07000             GET NEXT UNIT            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        LOCATE COMPATIBLE UNIT FOR VOLUME SPECIFIED.            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS05160 EQU   *                                                 Y02134
*                                                                Y02134
         STH   RC,DXUNITOF              SAVE CURRENT OFFSET      Y02134
         ST    RUCB,DXUCBADR            SAVE CURRENT UCB ADDRESS Y02134
         LH    RB,DXNOUNIT              GET NUMBER OF UNITS      Y02134
         LA    R1,DXVOLSR2              POINTER TO VOLUME N      Y02134
         L     RD,WTGPREFX              POINTER TO PREFIX        Y02134
         LA    RD,IECREGSV-IECPREFX(,RD) POINTER TO SAVE AREA    Y02134
         BAL   RET,VUS08000             LINK TO GET UNIT         Y02134
*                                                                YM1472
         LA    R0,K12(R0,R0)            NO UNITS AVAIL. CODE     YM7503
         MVI   DXRETCOD,K10             OPEN=413-04 EOV=B37-04   YM1472
         CR    RF,R0                    CHECK FOR NO UNITS AVAIL.YM1472
         BE    VUS05425                 BRANCH IF NO UNITS       YM1472
*                                                                YM1472
         ST    RUCB,DXUCBADR            SAVE CURRENT UCB ADDRESS Y02134
         STH   RC,DXUNITOF              SAVE TIOT ENTRY OFFSET   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        VOLUME AND UNIT HAVE BEEN FOUND.                        Y02134
*        INCREMENT DATA MANAGEMENT COUNT TO TIE UP UNIT          Y02134
*        WHILE MOUNT/VERIFY IN PROGRESS.                         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS05400 EQU   *                                                 Y02134
*                                                                Y02134
         LA    RF,K1                    LOAD INCREMENT VALUE     Y02134
*        SHIFT INCREMENT TO USE WITH CS INSTRUCTION              Y02134
         SLL   RF,K24-(K8*((UCBDMCT-UCBOB)-(((UCBDMCT-UCBOB)/K4)*K4)))
         L     R0,UCBOB+(((UCBDMCT-UCBOB)/K4)*K4) ALLIGN & LOAD  Y02134
*                                                                Y02134
VUS05410 EQU   *                        LOOP TO DECREMENT        Y02134
*                                                                Y02134
         LR    RET,R0                   SAVE CONTENTS OF CS OPER Y02134
         SLL   RET,K25                  LEAVE ONLY UCBDMCT COUNT Y02134
         SRL   RET,K25                  SHIFT COUNT FOR TEST     Y02134
         CLM   RET,B'0001',MAXIMUM      CHECK FOR 127 USERS      Y02134
         BL    VUS05415                 BRANCH IF NOT 127 USERS  Y02134
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE DEVICE     YM1472
         BO    VUS05416                 BRANCH IF TAPE UNIT      YM1472
         TM    UCBSTAB,UCBBSTR          CHECK FOR STORAGE VOLUME YM1472
         BNZ   VUS05423                 BRANCH IF STORAGE        YM1472
         TM    UCBSTAT,UCBRESV+UCBSYSR+UCBPRES RSRVD,SYSRES,PERM YM1472
         BNZ   VUS05423                 BRANCH IF ANY OR ALL     YM1472
         TM    UCBSTAB,UCBBSVL          TEST FOR SHARED VOLUME   YM1472
         BNO   VUS05423                 OK TO USE SHARED UNIT    YM1472
         MVI   DXRETCOD,K11             OPEN=213-28 EOV=B37-0C   YM1238
         B     VUS05425                 BRANCH OUT               Y02134
*                                                                Y02134
VUS05415 EQU   *                        LESS THAN 127 USERS      Y02134
*                                                                Y02134
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE UNIT       Y02134
         BNO   VUS05420                 BRANCH IF NOT TAPE       Y02134
*                                                                Y02134
         LTR   RET,RET                  TEST FOR COUNT OF ZERO   Y02134
         BZ    VUS05420                 BRANCH, TAPE NOT IN USE  Y02134
*                                                                YM1472
VUS05416 EQU   *                        TOO MANY USERS ON TAPE   YM1472
*                                                                YM1472
         MVI   DXRETCOD,K12             OPEN=513-04 EOV=837-0C   YM1238
         B     VUS05425                 BRANCH OUT               Y02134
*                                                                Y02134
VUS05420 EQU   *                        NOT TAPE                 Y02134
*                                                                Y02134
         LR    R1,R0                    LOAD OLD VALUE TO MODIFY Y02134
         AR    R1,RF                    ADD INCREMENT TO COUNT   Y02134
*                                                                Y02134
         MODESET EXTKEY=SUPR            UCB KEY                  Y02082
*                                                                Y02134
         CS    R0,R1,UCBOB+(((UCBDMCT-UCBOB)/K4)*K4) UPDAT COUNT Y02134
*                                                                Y02134
         MODESET EXTKEY=DATAMGT         RETURN TO DATA MGMNT KEY Y02082
*                                                                Y02134
         BNE   VUS05410                 BRANCH IF NOT THE SAME   Y02134
*                                                                YM1472
         OI    DXATGENS,DXATDMCT        DM COUNT INCREMENTED     YM1472
*                                                                Y02134
VUS05423 EQU   *                        UNIT OK TO USE           YM1472
*                                                                YM1472
         MVI   DXRETCOD,K0              SET RETURN CODE TO ZERO  Y02134
*                                                                Y02134
VUS05425 EQU   *                        CONTINUE                 Y02134
*                                                                Y02134
         L     RF,WTGPREFX              LOAD PREFIX ADDRESS      Y02134
         STM   R0,RF,IECREGSV-IECPREFX(RF) SAVE REGISTERS        Y02134
*                                                                Y02134
         MODESET EXTKEY=ZERO            SETLOCK KEY              Y02082
VUS05430 SETLOCK RELEASE,TYPE=LOCAL,    RELEASE LOCK             Y02134X
               RELATED=(LOCAL,IFG0194A(VUS05110))                Y02134
*                                                                Y02134
         MODESET EXTKEY=DATAMGT         RETURN TO DATA MGMNT KEY Y02082
*                                                                Y02134
         LTR   RD,RD                    TEST FOR SETLOCK ERROR   Y02134
         LM    R0,RF,IECREGSV-IECPREFX(RF) RESTORE REGISTERS     Y02134
         BNZ   VUS10000                 BRANCH IF ERROR          Y02134
         XR    R0,R0                    PREPARE FOR IC INST      Y02134
         IC    R0,DXRETCOD              PICK UP SAVED ERROR CODE Y02134
*                                                                Y02134
         CLI   DXRETCOD,K10             TEST FOR NO UNITS        Y02134
         BE    VUS10070                 BRANCH TO ERROR ROUTINE  Y02134
*                                                                Y02134
         LTR   R0,R0                    TEST FOR UNIT ERROR      Y02134
         BNZ   VUS10090                 BRANCH IF UNIT ERROR     Y02134
*                                                                Y02134
         NC    DXUCBSAV,DXUCBSAV        TEST FOR FIRST UNIT      Y02134
         BNZ   VUS05510                 BRANCH IF FIRST SPECIFD  Y02134
*                                                                Y02134
         MVC   DXUCBSAV,DXUCBADR        SAVE FIRST UNIT          Y02134
         MVC   DXUCBSAV(K1),DXUNITOF+K1 SAVE TIOT OFFSET         Y02134
*                                                                Y02134
VUS05510 EQU   *                        FIRST SPECIFIED          Y02134
*                                                                Y02134
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE DEVICE     Y02134
         BNO   VUS05600                 BRANCH NOT TAPE DEVICE   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        GO VERIFY VOLUME ON TAPE UNIT.                          Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         IECRES LOAD,MODID=ID4A4F,BRCODE=K0,BRANCH=QUEUED        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        GO VERIFY VOLUME ON DIRECT ACCESS UNIT.                 Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS05600 EQU   *                                                 Y02134
*                                                                Y02134
         IECRES LOAD,MODID=ID4A4C,BRCODE=K0,BRANCH=QUEUED        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        UPDATE JFCB WITH NEW VOLUME FROM DXVOLSR2.              Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS06000 EQU   *                        PHYSICAL SEQUENTIAL      Y02134
*                                                                Y02134
         CLC   DXVOLSR2,SCRTCH          CHECK FOR SCRTCH VOLUME  Y02134
         BNE   VUS06500                 BRANCH, LOOK AHEAD MOUNT Y02134
*                                                                YM3840
         LH    RC,DXVOLSEQ              LOAD VOL SEQ NO.         YM3195
         LA    RD,JFCBVOLS              LOAD POINTER TO VOLUMES  Y02134
         LA    RET,JFCBEXAD             SET UP POINTER FOR EXT   Y02134
         MVC   DXWORK1,TIOEJFCB         SAVE JFCB PTR            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN MAIN JFCB                           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         CH    RC,VOL5                  VOL SEQ LESS THAN 5      Y02134
         BNH   VUS06200                 BRANCH, VOL SER IN MAIN  Y02134
         SH    RC,VOL5                  DECREMENT FOR INDEXING   Y02134
*                                                                Y02134
         IECRES GET,LV=JFCBLGTH,ID=JFCB,                         Y02134X
               PREFIX=YES,STM=(2,14,WTGPREFX)                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME IN JFCB EXTENSION.                     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS06100 EQU   *                                                 Y02134
*                                                                Y02134
         CLC   K0(L'JFCBEXAD,RET),ZEROS IS THERE AN EXTENSION    Y02134
         LA    R0,K9                    OPEN=113-08 EOV=837-08   Y02080
         BZ    VUS10090                 BRANCH IF NO EXTENSION   Y02134
         MVC   DXWORK1,JFCBEXAD-JFCBEXAD(RET) SAVE JFCB PTR      Y02134
         IECRES LOCJFCB,(RET)           LOCATE JFCB EXTENSION    Y02134
         MVC   K0(JFCBLGTH,R1),K0(RET)  MOVE JFCB TO WORK AREA   Y02134
         LR    RET,R1                   LOAD POINTER TO JFCB     Y02134
         LA    RD,K4(RET)               LOAD POINTER TO VOLUMES  Y02134
         CH    RC,VOL15                 VOL SEQ LESS THAN 15     Y02134
         BNH   VUS06200                 BRANCH, VOL SER IN EXT   Y02134
         SH    RC,VOL15                 DECREMENT FOR INDEXING   Y02134
         B     VUS06100                 BRANCH TO CHECK NEXT EXT Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        REFERENCED VOLUME HAS BEEN FOUND.                       Y02134
*        RC CONTAINS THE POSITION OF THE VOLUMES IN THE LIST.    Y02134
*        RD POINTS TO THE LIST OF VOLUMES.                       Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS06200 EQU   *                                                 Y02134
*                                                                Y02134
         BCTR  RC,R0                    DECREMENT FOR INDEXING   Y02134
         MH    RC,VOL6                  CONVERT SEQ NO TO OFFSET Y02134
         AR    RD,RC                    ADD OFFSET TO BASE       Y02134
         MVC   K0(L'UCBVOLI,RD),UCBVOLI MOVE VOLUME N TO JFCB    Y02134
         LH    RC,DXVOLSEQ              GET VOL SEQ OF VOL N     Y02134
         STC   RC,JFCBNVOL              UPDATE NO. OF VOLS       Y02134
*                                                                YM5928
         OI    JFCBTSDM,JFCVSL          SET VOL SER LIST MODIF-  A45617
*                                       IED SO SCHEDULER WILL    A45617
*                                       RECATALOG WITH NEW VOL   A45617
*                                                                YM5928
         OI    JFCBMASK+K4,X80          INDICATE JFCB MODIFIED   Y02134
*                                                                Y02134
         XC    JFCVINDX(K2),JFCVINDX    CLEAR VVIC INDICATORS  @Y30LSFY
*                                                              @Y30LSFY
         TM    JFCBTSDM,JFCNWRIT        JFCB UPDATE INHIBITED    Y02134
         BO    VUS06500                 BRANCH TO INHIBIT UPDATE Y02134
*                                                                YM3840
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE UNIT       YM3840
         BNO   VUS06250                 BRANCH IF NOT TAPE UNIT  YM3840
         TM    JFCBLTYP,JFCNL+JFCNSL+JFCBLP TEST FOR NL TAPE     YM3840
         BZ    VUS06250                 BRANCH IF NOT NL SCRTCH  YM3840
*                                                                YM3840
         OI    JFCBMASK+K5,X01          SET L NO. GENERATED      YM3840
*                                                                Y02134
VUS06250 EQU   *                        PREPARE TO UPDATE VOLSER YM3840
*                                                                Y02134
         CLC   DXVOLSEQ,VOL5            CHECK IF IN MAIN JFCB    Y02134
         BNH   VUS06300                 BRANCH IF VOLUME IN MAIN Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        WRITE THE JFCB BACK.                                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         LA    R1,DXWORK1               POINT TO SAVED JFCB PTR  Y02134
*                                                                Y02134
         IECRES WRJFCB,(R1),(RET)       WRITE JFCB BACK          Y02134
*                                                                Y02134
         IECRES FREE,A=(RET),PREFIX=YES,STM=(0,14,WTGPREFX)      Y02134
*                                                                Y02134
VUS06300 EQU   *                        VOLUME IN MAIN           Y02134
*                                                                Y02134
         LA    R1,TIOEJFCB              POINT TO MAIN JFCB       Y02134
         LA    RET,DXJBF                POINT TO WORK AREA JFCB  Y02134
*                                                                Y02134
         IECRES WRJFCB,(R1),(RET)       WRITE JFCB BACK          Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        LOOK AHEAD MOUNT VOL N+1                                Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS06500 EQU   *                                                 Y02134
*                                                                Y02134
         TM    UCBTBYT2,UCBRVDEV        TEST FOR VIRTUAL VOLUME@ZA05469
         BO    VUS09100                 YES, SKIP LOOKAHEAD    @ZA05469
*                                       NO,                    @ZA05469
         CLC   DXVOLSR3,ZEROS           SEE IF VOL N+1 SPECIFIED Y02134
         BZ    VUS09100                 BRANCH IF NONE SPECIFIED Y02134
         CLC   DXNOUNIT,ONE             CHECK FOR ONE UNIT       Y02134
         BNH   VUS09100                 BRANCH IF ONLY ONE UNIT  Y02134
*                                                                Y02134
         L     RF,WTGPREFX              LOAD POINTER TO PREFIX   Y02134
         STM   R0,RF,IECREGSV-IECPREFX(RF) SAVE REGISTERS        Y02134
*                                                                Y02134
         MODESET EXTKEY=ZERO            SETLOCK KEY              Y02134
*                                                                Y02134
VUS06550 SETLOCK OBTAIN,TYPE=LOCAL,MODE=UNCOND,                  Y02134X
               RELATED=(LOCAL,IFG0194A(VUS06560))                Y02134
*                                                                Y02134
         MODESET EXTKEY=DATAMGT         RETURN TO DATA MGMNT KEY Y02082
*                                                                Y02134
         LTR   RD,RD                    TEST FOR SETLOCK ERROR   Y02134
         LM    R0,RF,IECREGSV-IECPREFX(RF) RESTORE REGISTERS     Y02134
         BNZ   VUS10000                 BRANCH IF ERROR          Y02134
*                                                                Y02134
         LH    RC,DXUNITOF              GET CURRENT OFFSET       Y02134
         L     RUCB,DXUCBADR            GET CURRENT UCB ADDRESS  Y02134
         LH    RB,DXNOUNIT              GET NUMBER OF UNITS      Y02134
         BAL   RET,VUS07000             LINK TO GET NEXT UNIT    Y02134
*                                                                YM1472
         MVI   DXRETCOD,K0              SET REJECT OFF           YM1472
         MVC   DXVOLSR1,DXVOLSR2        SET UP LAST VOLUME       YM1472
         LA    R1,DXVOLSR3              POINTER TO VOLUME N+1    Y02134
         L     RD,WTGPREFX              POINTER TO PREFIX        Y02134
         LA    RD,IECREGSV-IECPREFX(,RD) POINTER TO SAVE AREA    Y02134
         BAL   RET,VUS08000             LINK TO GET UNIT         Y02134
*                                                                YM1472
         L     R1,WTGPREFX              LOAD POINTER TO PREFIX   Y02134
         STM   R0,RF,IECREGSV-IECPREFX(R1) SAVE REGISTERS        Y02134
*                                                                Y02134
         MODESET EXTKEY=ZERO            SETLOCK KEY              Y02134
*                                                                Y02134
VUS06560 SETLOCK RELEASE,TYPE=LOCAL,    RELEASE LOCK             Y02134X
               RELATED=(LOCAL,IFG0194A(VUS06550))                Y02134
*                                                                Y02134
         MODESET EXTKEY=DATAMGT         RETURN TO DATA MGMNT KEY Y02082
*                                                                Y02134
         LTR   RD,RD                    TEST FOR SETLOCK ERROR   Y02134
         LM    R0,RF,IECREGSV-IECPREFX(R1) RESTORE REGISTERS     Y02134
         BNZ   VUS10000                 BRANCH IF ERROR          Y02134
*                                                                Y02134
         B     VUS06600(RF)             BRANCH TO BRANCH TABLE   Y02134
*                                                                Y02134
VUS06600 EQU   *                        BRANCH TABLE             Y02134
*                                                                Y02134
         B     VUS09100                 BRANCH TO UNIT LOCATED   YM1472
         B     VUS06800                 BRANCH TO EMPTY UNIT     Y02134
         B     VUS06800                 BRANCH TO UNIT DISMOUNT  Y02134
         B     VUS09100                 BRANCH TO SKIP LOOKAHEAD Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        LOOK AHEAD DEVICE LOCATED.                              Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS06800 EQU   *                                                 Y02134
*                                                                Y02134
         MVC   DXRETMOD,ID4A4A          RETURN ID TO SAVE AREA   YM1350
         MVI   DXRETCOD,K28             SET RETURN INDICATOR     YM3081
*                                                                Y02134
         IECRES LOAD,MODID=ID4A4J,BRCODE=K16,BRANCH=QUEUED       Y02134
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        THIS ROUTINE LOCATES A UNIT TO PROCESS A SPECIFIED      Y02134
*        VOLUME SERIAL NUMBER.                                   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        DXVOLSR1 CONTAINS THE VOLUME SERIAL OF THE LAST VOLUME  YM1472
*        PROCESSED.                                              YM1472
*                                                                YM1472
*        REG   ENTRY CONTENTS           EXIT CONTENTS            Y02134
*        R0    WORK REGISTER            SAME AS ENTRY            Y02134
*        R1    POINTER TO VOLUME SERIAL SAME AS ENTRY            Y02134
*        R2    DCB ADDRESS              SAME AS ENTRY            Y02134
*        R3    BASE REGISTER            SAME AS ENTRY            Y02134
*        R4    O/C/E WORK AREA POINTER  SAME AS ENTRY            Y02134
*        R5    NOT USED                 SAME AS ENTRY            Y02134
*        R6    O/C/E WTG TABLE POINTER  SAME AS ENTRY            Y02134
*        R7    NOT USED                 SAME AS ENTRY            Y02134
*        R8    NOT USED                 SAME AS ENTRY            Y02134
*        R9    POINTER TO TIOT ENTRY    SAME AS ENTRY            Y02134
*        RA    POINTER TO UCB           POINTER TO NEW UCB       Y02134
*        RB    NUMBER OF UNITS IN TIOT  SAME AS ENTRY            Y02134
*        RC    TIOT ENTRY UCB OFFSET    NEW TIOT ENTRY OFFSET    Y02134
*        RD    POINTER TO SAVE AREA     SAME AS ENTRY            Y02134
*        RET   RETURN POINT             SAME AS ENTRY            Y02134
*        RF    WORK REGISTER            RETURN CODE              Y02134
*                                       0 = VOLUME MOUNTED       Y02134
*                                       4 = UNIT AVAILABLE       Y02134
*                                       8 = WRONG VOLUME MOUNTED Y02134
*                                      12 = MOUNT ISSUED, BAD VOLY02134
*                                      16 = NO UNIT AVAILABLE    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08000 EQU   *                                                 Y02134
*                                                                Y02134
         SAVE  (14,12)                  SAVE REGISTERS           Y02134
         LTR   RB,RB                    TEST FOR 1 OR MORE UNITS Y02134
         BNP   VUS08860                 BRANCH IF NO UNITS       Y02134
         TM    UCBJBNR,UCBVRDEV         TEST FOR VIO UCB         YM3192
         BO    VUS08810                 BR IF VIO                YM3192
         XC    DXWORK1,DXWORK1          CLEAR WORK AREA          Y02134
         XC    DXWORK2,DXWORK2          CLEAR WORK AREA          Y02134
         XC    DXWORK3,DXWORK3          CLEAR WORK AREA          Y02134
         XC    DXWORK4,DXWORK4          CLEAR WORK AREA          Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR UNLOADED UNIT WITH NO ACTIVITY                Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08100 EQU   *                                                 Y02134
*                                                                Y02134
         NC    DXVOLSR1,DXVOLSR1        TEST FOR PREVIOUS VOLUME YM1472
         BZ    VUS08105                 BRANCH IF NONE           YM1472
         CLC   DXVOLSR1,UCBVOLI         TEST FOR PREVIOUS VOLUME YM1472
         BNE   VUS08105                 BRANCH IF DIFFENENT VOL  YM4647
         CLC   DXVOLSR1,0(R1)           SEE IF LAST SAME AS NEXT YM4647
         BNE   VUS08800                 BRANCH NO TO DISMOUNT    YM4647
*                                                                YM1472
VUS08105 EQU   *                        CHECK OUT UNIT           YM1472
*                                                                YM1472
         TM    UCBDMCT,ALLBITS          TEST FOR ACTIVITY        Y02134
         BNZ   VUS08200                 BRANCH IF ANY ACTIVITY   Y02134
         CLC   UCBVOLI,ZEROS            CHECK IF NOTHING MOUNTED Y02134
         BZ    VUS08820                 BRANCH IF UNIT EMPTY     Y02134
         CLC   UCBVOLI,BLANKS           CHECK IF NOTHING MOUNTED Y02134
         BZ    VUS08820                 BRANCH IF UNIT EMPTY     Y02134
         CLI   UCBVOLI,VIRTUAL          UNVERIFIED SCRATCH TEST  Y02134
         BE    VUS08820                 BRANCH IF UNIT EMPTY     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR VOLUME ALREADY MOUNTED                        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08200 EQU   *                                                 Y02134
*                                                                Y02134
         CLC   UCBVOLI,K0(R1)           UNIT ASSIGNED TO VOLUME  Y02134
         BE    VUS08810                 BRANCH IF UNIT LOCATED   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR SCRATCH VOLUME MOUNTING                       Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         CLI   DXRETCOD,K0              TEST FOR REJECT          YM1157
         BNE   VUS08800                 BRANCH IF REJECT         YM1157
         CLC   SCRTCH,K0(R1)            CHECK FOR SCRTCH REQ     Y02134
         BNE   VUS08800                 BRANCH NOT SCRTCH        Y02134
         CLI   UCBDMCT,UCBMOUNT         SEE IF MOUNT IN PROGRESS Y02134
         BNE   VUS08300                 BRANCH IF NOT MOUNTING   Y02134
         CLI   UCBVOLI,VIRTUAL          TEST FOR SCRTCH MOUNT    Y02134
         BE    VUS08810                 BRANCH IF SCRTCH MOUNT   Y02134
         CLC   UCBVOLI,ZEROS            TEST FOR NON-SPECIFIC    Y02134
         BE    VUS08810                 BRANCH, OK FOR SCRTCH    Y02134
         CLC   UCBVOLI,BLANKS           CHECK FOR MOUNT BLANKS   Y02134
         BE    VUS08810                 BRANCH, OK FOR SCRTCH    Y02134
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE DEVICE     Y02134
         BO    VUS08800                 BRANCH, PURGE SPEC MOUNT YM7503
         L     RF,UCBEXTPT              LOAD UCB EXTENSION PTR   Y02146
         TM    UCBFLP1-UCBCMEXT(RF),UCBNSRCH TEST NON-SCRTCH VOL Y02146
         BO    VUS08800                 BRANCH IF NONSCRTCH VOL  YM7503
         B     VUS08810                 BR TO USE COMPATIBLE VOL Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CHECK FOR SCRATCH VOLUME ALREADY MOUNTED                Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08300 EQU   *                                                 Y02134
*                                                                Y02134
         TM    UCBTBYT3,UCB3DACC        TEST FOR DA DEVICE       Y02134
         BO    VUS08400                 BRANCH IF DA DEVICE      Y02134
         TM    UCBDMCT,ALLBITS          TEST FOR ACTIVITY        Y02134
         BNZ   VUS08850                 BRANCH IF DEVICE IN USE  Y02134
         TM    JFCBLTYP,JFCNSL          TEST FOR NSL REQUSET     Y02134
         BO    VUS08800                 BRANCH IF NSL SCRTCH     Y02134
         TM    JFCBLTYP,JFCBLP          TEST FOR BLP REQUEST     Y02134
         BO    VUS08800                 BRANCH TO GET UNIT       Y02134
*                                                                YM2899
         L     RF,UCBEXTPT              LOAD UCB EXTENSION PTR   YM2899
         TM    UCBFLP1-UCBCMEXT(RF),UCBNSRCH  TEST IF NON-SCRTCH YM2899
         BO    VUS08800                 BRANCH IF NONSCRTCH VOL  YM2899
*                                                                Y02134
VUS08400 EQU   *                        DA DEVICE                Y02134
*                                                                Y02134
         TM    TIOESTTA,TIOSDSP1        TEST FOR PRIVATE REQUEST YM1108
         BNO   VUS08450                 BRANCH IF NOT PRIVATE    YM1108
         TM    UCBSTAB,UCBBPRV          TEST FOR PRIVATE SCRTCH  YM1108
         BNO   VUS08800                 BRANCH IF NO PRIVATE VOL YM1108
         L     RF,UCBEXTPT              LOAD UCB EXTENSION PTR   Y02146
         TM    UCBFLP1-UCBCMEXT(RF),UCBNSRCH TEST IF NON-SCRTCH  Y02146
         BO    VUS08800                 BRANCH IF NONSCRTCH VOL  Y02134
         B     VUS08500                 BRANCH, COMPATIBLE VOL   YM1108
*                                                                Y02134
VUS08450 EQU   *                        NOT PRIVATE VOL          Y02134
*                                                                Y02134
         TM    UCBTBYT3,UCB3TAPE        TEST FOR TAPE            Y02134
         BO    VUS08500                 BRANCH IF TAPE           Y02134
         TM    UCBSTAB,UCBBPRV          TEST FOR PRIVATE SCRTCH  Y02134
         BO    VUS08800                 BRANCH IF PRIVATE VOL    Y02134
*                                                                Y02134
VUS08500 EQU   *                        PRIVATE VOLUME           Y02134
*                                                                Y02134
         TM    UCBSTAT,UCBDADI          TEST FOR SL VOL          Y02134
         BNO   VUS08600                 BRANCH IF NOT SL         Y02134
         TM    JFCBLTYP,JFCSL           TEST FOR SL REQUEST      Y02134
         BO    VUS08815                 BRANCH, USE THIS UNIT    Y02134
         B     VUS08800                 BRANCH, INCOMPATIBLE VOL Y02134
*                                                                Y02134
VUS08600 EQU   *                        NOT SL                   Y02134
*                                                                Y02134
         TM    UCBTBYT3,UCB3DACC        TEST FOR DIRECT ACCESS   Y02134
         BO    VUS08815                 BRANCH TO USE THIS UNIT  Y02134
         TM    UCBSTAB,UCBBSTR          TEST FOR ANSI VOLUME     Y02134
         BNO   VUS08700                 BRANCH IF NOT ANSI VOL   Y02134
         TM    JFCBLTYP,JFCBAL          TEST FOR USASI REQUEST   Y02134
         BO    VUS08810                 BRANCH TO USE THIS UNIT  Y02134
         B     VUS08800                 BRANCH, INCOMPATIBLE VOL Y02134
*                                                                Y02134
VUS08700 EQU   *                        NOT ANSI VOL             Y02134
*                                                                Y02134
         TM    UCBTFL1,UCBNSLTP         TEST FOR NSL VOLUME      Y02134
         BO    VUS08800                 BRANCH IF NSL MOUNTED    Y02134
         TM    UCBTFL1,UCBNLTP          TEST FOR NL VOLUME       Y02134
         BNO   VUS08810                 BRANCH, VOL NOT IDENTIFD Y02134
         TM    JFCBLTYP,JFCNL           TEST FOR NL REQUEST      Y02134
         BO    VUS08810                 BRANCH TO USE THIS UNIT  Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        THE VOLUME IS NOT MOUNTED ON ONE OF THE                 Y02134
*        ALLOCATED UNITS. FIND THE FIRST AVAILABLE UNIT.         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08800 EQU   *                                                 Y02134
*                                                                Y02134
         TM    UCBTBYT3,UCB3TAPE        TEST FOR 2400 UNIT       YM2850
         BO    VUS08801                 BRANCH IF NON SHR UNIT   YM2850
         TM    UCBSTAB,UCBBSVL          TEST FOR SHARED VOLUME   YM2850
         BNO   VUS08850                 DO NOT USE SHARED UNIT   YM2850
*                                                                YM2850
VUS08801 EQU   *                        NONSHARED UNIT           YM2850
*                                                                YM2850
         TM    UCBSTAT,UCBRESV+UCBSYSR+UCBPRES RSRVD,SYSRES,PERM YM4647
         BNZ   VUS08850                 BRANCH IF ANY OR ALL     YM4647
         TM    UCBDMCT,UCBDMC           CHECK FOR ANY USERS      Y02134
         BNZ   VUS08850                 BRANCH YES, SKIP         Y02134
         TM    UCBDMCT,UCBMOUNT         WRONG MOUNT PENDING      Y02134
         BO    VUS08830                 BRANCH, WRONG MOUNT PEND YM7503
         CLC   UCBVOLI,ZEROS            TEST FOR NO VOL SER      Y02134
         BZ    VUS08820                 BRANCH, NOTHING MOUNTED  Y02134
         CLC   UCBVOLI,BLANKS           TEST FOR BLANK VOL SER   Y02134
         BE    VUS08820                 BRANCH, NOTHING MOUNTED  Y02134
         CLI   UCBVOLI,VIRTUAL          TEST FOR SCRTCH UNIT     Y02134
         BE    VUS08820                 BRANCH, USE EMPTY UNIT   Y02134
         B     VUS08830                 DISMOUNTABLE VOLUME      YM4647
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        UNIT HAS BEEN FOUND WITH VOLUME MOUNTED OR              Y02134
*        IN THE PROCESS OF BEING MOUNTED.                        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08810 EQU   *                                                 Y02134
         XR    RF,RF                    ZERO OUT RETURN CODE     Y02134
         XC    DXVOLSR1,DXVOLSR1        CLEAR REJECT VOLUME      YM3925
         B     VUS08950                 BRANCH TO RETURN         YM3925
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        UNIT HAS BEEN FOUND WITH VOLUME MOUNTED OR              Y02134
*        IN THE PROCESS OF BEING MOUNTED.                        Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08815 EQU   *                                                 Y02134
*                                                                Y02134
         NC    DXWORK1,DXWORK1          PREVIOUS EMPTY UNIT TEST Y02134
         BNZ   VUS08850                 BRANCH IF PREVIOUS UNIT  Y02134
         ST    RUCB,DXWORK1             SAVE EMPTY UNIT ADDRESS  Y02134
         STC   RC,DXWORK1               SAVE EMPTY UNIT OFFSET   Y02134
         B     VUS08850                 BRANCH TO GET MORE UNITS Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        UNIT AVAILABLE WITH NOTHING MOUNTED.                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08820 EQU   *                                                 Y02134
*
         NC    DXWORK2,DXWORK2          PREVIOUS EMPTY UNIT TEST Y02134
         BNZ   VUS08850                 BRANCH IF PREVIOUS UNIT  Y02134
         ST    RUCB,DXWORK2             SAVE EMPTY UNIT ADDRESS  Y02134
         STC   RC,DXWORK2               SAVE EMPTY UNIT OFFSET   Y02134
         B     VUS08850                 BRANCH TO GET MORE UNITS Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        DISMOUNTABLE UNIT WITH WRONG VOLUME MOUNTED OR MOUNTING YM7503
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08830 EQU   *
*
         NC    DXWORK3,DXWORK3          PREVIOUS UNIT TEST       Y02134
         BNZ   VUS08850                 BRANCH IF PREVIOUS UNIT  Y02134
         ST    RUCB,DXWORK3             SAVE UNIT ADDRESS        Y02134
         STC   RC,DXWORK3               SAVE UNIT OFFSET         Y02134
         B     VUS08850                 BRANCH TO GET MORE UNITS Y02134
*                                                                Y02134
VUS08850 EQU   *                        GET MORE UNITS           Y02134
*                                                                Y02134
         BAL   RET,VUS07000             LINK TO GET NEXT UNIT    Y02134
         BCT   RB,VUS08100              LOOP IF MORE UCBS        Y02134
*                                                                YM4611
         XR    RC,RC                    CLEAR REG FOR IC INST    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        TEST FOR UNIT AVAILABLE WITH NOTHING MOUNTED.           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         LA    RF,K4                    EMPTY UNIT RETURN CODE   Y02134
         IC    RC,DXWORK2               INSERT UNIT OFFSET       Y02134
         L     RUCB,DXWORK2             LOAD UCB ADDRESS         Y02134
         LA    RUCB,UCBOB               CLEAR HI ORDER BYTE      Y02134
         NC    DXWORK2,DXWORK2          TEST FOR EMPTY UNIT      Y02134
         BNZ   VUS08900                 BRANCH TO RETURN         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        TEST FOR SCRTCH VOLUME ALREADY MOUNTED.                 Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         XR    RF,RF                    VOL MOUNTED RETURN CODE  Y02134
         IC    RC,DXWORK1               INSERT UNIT OFFSET       Y02134
         L     RUCB,DXWORK1             LOAD UCB ADDRESS         Y02134
         LA    RUCB,UCBOB               CLEAR HI ORDER BYTE      Y02134
         NC    DXWORK1,DXWORK1          TEST FOR EMPTY UNIT      Y02134
         BNZ   VUS08900                 BRANCH TO RETURN         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        TEST FOR DISMOUNTABLE UNIT WITH WRONG VOLUME MOUNTED.   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         LA    RF,K8                    DISMOUNT REQUIRED        Y02134
         IC    RC,DXWORK3               INSERT UNIT OFFSET       Y02134
         L     RUCB,DXWORK3             LOAD UCB ADDRESS         Y02134
         LA    RUCB,UCBOB               CLEAR HI ORDER BYTE      Y02134
         NC    DXWORK3,DXWORK3          TEST FOR DISMNTABLE UNIT YM1472
         BNZ   VUS08910                 BRANCH TO RETURN         YM1472
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        NO AVAILABLE UNIT EXISTS.                               Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS08860 EQU   *                                                 Y02134
*                                                                Y02134
         LA    RF,K12                   LOAD ERROR RETURN CODE   YM7503
         B     VUS08990                 BRANCH TO RETURN         Y02134
*                                                                YM1472
*****************************************************************YM1472
*                                                                YM1472
*        VOLUME AND UNIT HAVE BEEN FOUND.                        YM1472
*                                                                YM1472
*****************************************************************YM1472
*                                                                YM1472
VUS08900 EQU   *                                                 Y02134
*                                                                Y02134
         XC    DXVOLSR1,DXVOLSR1        CLEAR REJECT             YM1472
         B     VUS08920                 BYPASS REJECT INDICATOR  YM1472
*                                                                YM1472
VUS08910 EQU   *                        SET UP FOR REJECT        YM1472
*                                                                YM1472
         MVI   DXVOLSR1,ALLBITS         SET REJECT INDICATOR     YM1472
*                                                                YM1472
VUS08920 EQU   *                        CHECK FOR UNIT NOT READY YM1472
*                                                                YM1472
         TM    UCBJBNR,UCBVRDEV         TEST FOR VIO             YM3822
         BO    VUS08950                 BRANCH IF VIO            YM3822
         TM    UCBDMCT,UCBMOUNT         TEST FOR MOUNT PENDING   YM1472
         BO    VUS08925                 BRANCH IF MOUNT PENDING  YM1472
         TM    UCBFL1,UCBNOTRD          TEST IF UNIT READY       YM1472
         BO    VUS08930                 BRANCH IF NOT READY      YM1472
*                                                                YM1472
VUS08925 EQU   *                        CHECK FOR EMPTY UNIT     YM1472
*                                                                YM1472
         CLC   UCBVOLI,ZEROS            TEST FOR NO VOL SER      YM1472
         BZ    VUS08940                 BRANCH, NOTHING MOUNTED  YM1472
         CLC   UCBVOLI,BLANKS           TEST FOR BLANK VOL SER   YM1472
         BE    VUS08930                 BRANCH, RESOLVE BLANKS   YM1472
         CLI   UCBVOLI,VIRTUAL          TEST FOR SCRTCH UNIT     YM1472
         BNE   VUS08940                 BRANCH, VOLSER OK        YM1472
*                                                                YM1472
VUS08930 EQU   *                        INDICATE EMPTY UNIT      YM1472
*                                                                YM1472
         MODESET EXTKEY=SUPR            UCB KEY                  YM1472
*                                                                YM1472
         XC    UCBVOLI,UCBVOLI          CLEAR VOLUME SERIAL      YM1472
*                                                                YM1472
         MODESET EXTKEY=DATAMGT         RESET KEY                YM1472
*                                                                YM1472
VUS08940 EQU   *                        CHECK REJECT SWITCH      YM1472
*                                                                YM1472
         CLI   DXVOLSR1,ALLBITS         CHECK FOR REJECT         YM1472
         BNE   VUS08950                 BRANCH TO RETURN         YM1472
*                                                                YM1472
         MVC   DXVOLSR1,UCBVOLI         VOL SER TO REJECT        YM1472
*                                                                YM1472
         MODESET EXTKEY=SUPR            UCB KEY                  YM1472
*                                                                YM1472
         XC    UCBVOLI,UCBVOLI          CLEAR VOLUME SERIAL      YM1472
*                                                                YM1472
         MODESET EXTKEY=DATAMGT         RESET KEY                YM1472
*                                                                YM1472
VUS08950 EQU   *                        SET UP TO RETURN         YM1472
*                                                                YM1472
         ST    RUCB,K60(RD)             SAVE NEW UCB ADDRESS     Y02134
         ST    RC,K68(RD)               SAVE NEW UCB OFFSET      Y02134
*                                                                Y02134
VUS08990 EQU   *                        RETURN                   Y02134
*                                                                Y02134
         RETURN (14,12),RC=(15)         RETURN                   Y02134
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        FIND NEXT UNIT IN TIOT DD ENTRY                         Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS07000 EQU   *                                                 Y02134
*                                                                Y02134
         LTR   RC,RC                    TEST FOR INVALID OFFSET  Y02134
         BZ    VUS07004                 BRANCH, NO OFFSET        Y02134
         LA    RC,K4(RC)                INCREMENT TO NEXT ENTRY  Y02134
         EX    RC,VUS07008              CLI   TIOELNGH,K0        Y02134
         BH    VUS07006                 BRANCH IF NOT, GET UCB   Y02134
*                                                                Y02134
VUS07004 EQU   *                        NO OFFSET                Y02134
*                                                                Y02134
         LA    RC,TIOEFSRT-K1-TIOENTRY  GET OFFSET TO FIRST UCB  Y02134
*                                                                Y02134
VUS07006 EQU   *                        GET UCB                  Y02134
*                                                                Y02134
         L     RUCB,TIOENTRY(RC)        LOAD NEXT UCB ADDRESS    Y02134
         LA    RUCB,UCBOB               ZERO HI ORDER BYTE       Y02134
         BR    RET                      RETURN                   Y02134
*                                                                Y02134
VUS07008 CLI   TIOELNGH,K0              CHECK FOR END OF ENTRY   Y02134
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        DECREMENT DATA MANAGEMENT COUNT                         Y02134
*                                                                Y02134
*        REGS - RUCB POINTS TO UCB, RET POINTS TO RETURN ADDRESS Y02134
*        REGS R0, R1, RF WILL BE USED AS WORK REGS               Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS07100 EQU   *                                                 Y02134
*                                                                Y02134
         L     R0,UCBOB+(((UCBDMCT-UCBOB)/K4)*K4) ALLIGN & LOAD  Y02134
*                                                                Y02134
         MODESET EXTKEY=SUPR            UCB KEY                  Y02082
*                                                                Y02134
VUS07110 EQU   *                        LOOP                     Y02134
*                                                                Y02134
         LR    RF,R0                    SAVE CS OPERAND          Y02134
         SLL   RF,K25                   LEAVE ONLY UCBDMCT COUNT Y02134
         LTR   RF,RF                    TEST FOR COUNT OF 0      Y02134
         BZ    VUS07120                 BRANCH IF ALREADY 0      Y02134
*                                                                Y02134
         LR    R1,R0                    LOAD OLD VALUE TO MODIFY Y02134
         SH    R1,ONE                   ADD DECREMENT TO COUNT   Y02134
         CS    R0,R1,UCBOB+(((UCBDMCT-UCBOB)/K4)*K4) UPDAT COUNT Y02134
         BNE   VUS07110                 BRANCH IF NOT THE SAME   Y02134
*                                                                Y02134
VUS07120 EQU   *                        ALREADY 0                Y02134
*                                                                Y02134
         MODESET EXTKEY=DATAMGT         RETURN TO DATA MGMNT KEY Y02082
*                                                                Y02134
         RETURN ,                       RETURN TO CALLER         Y02134
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        BRANCH TO CONTINUE PROCESSING                           Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS09000 EQU   *                                                 Y02134
*                                                                Y02134
         MVC   DXVOLSEQ,ONE             RESET VOLSEQ TO ONE      Y02134
         MVC   DXVOLSR2,JFCBVOLS        RESTORE FIRST VOL SER    Y02134
*                                                                Y02134
         TM    DCBMACRF,DCBMEXCP        IS ACCESS METHOD EXCP    YM3111
         BO    VUS09100                 BR IF EXCP, ASSUME PS    YM3111
*                                                                YM3111
         TM    DCBDSORG,DCBDSGDA        TEST IF BDAM             YM1289
         BZ    VUS09010                 BR IF NO                 YM7051
*                                                                YM1289
         SR    R0,R0                    GET NO. OF BDAM EXTENTS  YM1289
         IC    R0,DCBDRDX               SO FAR FROM DCB          YM1289
         SR    R1,R1                    GET BDAM EXTENTS ON 1ST  YM1289
         IC    R1,DS1NOEPV-DS1FMTID(,RCORE) VOL FROM F1 DSCB     YM1289
         SR    R0,R1                    SUBTRACT FROM CURRENT    YM1289
         STC   R0,DCBDRDX               UPDATE COUNT IN DCB      YM1289
         B     VUS09100                 RETURN TO CALLER         YM7051
*                                                                YM7051
VUS09010 EQU   *                        CHECK FOR ISAM           YM7051
*                                                                YM7051
         TM    DCBDSORG,DCBDSGIS        TEST IF ISAM             Y02134
         BNO   VUS09100                 BRANCH IF NOT ISAM       Y02134
*                                                                YM7535
ISAMLOAD EQU   X'18'                    QISAM LOAD MODE          YM7535
         TM    DCBMACRF+K1,ISAMLOAD     TEST FOR QISAM LOAD MODE YM7535
         BZ    VUS09100                 BRANCH IF NOT            YM7535
*                                                                Y02134
*        SETUP FOR ISAM 03B ABEND                                YM7535
         MVC   DXDCBLST(K1),DXDEBSYS    RESTORE ISAM OPTIONS     YM7051
*                                                                YM1289
VUS09100 EQU   *                        RESTORE POINTERS & EXIT  Y02134
*                                                                Y02134
         MVC   DXDSAB,DXDSABAD          RESTORE DSAB PTRS        Y02134
         L     RTIOT,DXDSABAD           GET DSAB ADDRESS         Y02134
         L     RTIOT,DSABTIOT-DSAB(,RTIOT) RESTORE TIOT ENTRY    Y02134
         MVC   DXUCBADR+K1(K3),DXUCBSAV+K1  RESTORE 1ST UCB ADDR YM6530
         MVC   DXUNITOF+K1(K1),DXUCBSAV RESTORE ENTRY OFFSET     Y02134
         XC    DXUCBSAV,DXUCBSAV        CLEAR TO SAVE FIRST UNIT Y02134
         XC    DXVOLSR1,DXVOLSR1        CLEAR DISMOUNTED VOL     YM3081
         L     RUCB,DXUCBADR            LOAD FIRST UCB           YM1350
*                                                                Y02134
         MVC   WTGIDTTR,DXSAVMOD        RETURN ROUTINE ADDR      YM1350
         XR    RET,RET                  CLEAR REG FOR IC INST    Y02134
         IC    RET,DXSAVOFF             PICK UP RETURN OFFSET    Y02134
*                                                                Y02134
         IECRES LOAD,BRANCH=QUEUED                               Y02134
         EJECT
VUS10000 EQU   *                                               @ZA14612
*                                                              @ZA14612
         L     RF,SETLOCK               EPA OF SETLOCK ROUTINE @ZA14612
         BR    RF                       BRANCH TO ABEND 0C1    @ZA14612
*                                                              @ZA14612
VUS10060 EQU   *                                               @ZA14612
*                                                              @ZA14612
         L     RF,EXIT213               EPA OF 213 EXIT ROUTINE@ZA14612
         BR    RF                       BRANCH TO EXIT         @ZA14612
*                                                              @ZA14612
VUS10070 EQU   *                                               @ZA14612
*                                                              @ZA14612
         L     RF,EXITB37               EPA OF B37 EXIT ROUTINE@ZA14612
         BR    RF                       BRANCH TO EXIT         @ZA14612
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        DETERMINE PROPER RETURN CODE FOR NON-SPECIFIC VOLUME    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS10085 EQU   *                                                 Y02134
*                                                                Y02134
         CLC   JFCBNVOL,JFCBVLSQ+K1     INVALID VOL SEQ          Y02134
         BNL   VUS10090                 BRANCH IF VOLSEQ INVALID YM3916
*                                                                Y02134
         LA    R0,K8                    OPEN=413-1C EOV=837-08   Y02134
*                                                                Y02134
VUS10090 EQU   *                        GET INTERNAL CODE        Y02080
*                                                                Y02080
*****************************************************************Y02134
*                                                                Y02134
*        DETERMINE PROPER RETURN CODE FOR OPEN OR EOV            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         LA    RF,VUSERRFF              GET LENGTH OF CODE LIST  Y02080
         CR    RF,R0                    CHECK IF CODE IS OK      Y02134
         BNL   VUS10092                 BRANCH IF OK             Y02080
         LA    R0,K0                    SET BAD CODE IN REG 0    Y02080
*                                                                Y02134
VUS10092 EQU   *                        OKAY                     Y02080
*                                                                Y02080
         LA    RF,VUSERR19              POINT TO OPEN CODES      Y02080
         CLC   VUSEOV,WTGMODNM+K3       CHECK FOR EOV CALL       Y02080
         BNE   VUS10095                 BRANCH IF OPEN           Y02080
         LA    RF,VUSERR55              POINT TO EOV  CODES      Y02080
*                                                                Y02134
VUS10095 EQU   *                        OPEN                     Y02080
*                                                                Y02080
         AR    RF,R0                    ADD ERROR OFFSET         Y02080
         IC    R0,K0(RF)                INSERT PROPER PD CODE    Y02080
*                                                                Y02134
VUS10100 EQU   *                        SPECIFIC REQUEST         Y02080
*                                                                Y02080
         MVC   DXRETMOD,ID4A4A          SET UP RETURN MOD        Y02080
         LA    RET,K24                  SET UP RETURN OFFSET     Y02080
*                                                                Y02134
*        NOTE - 4A RETURN MODULE SAVED AT DXSAVMOD               Y02080
*                                                                Y02080
         DMABCOND (0),ID4A0P                                     Y02080
         EJECT
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        THIS IS THE ENTRY WHEN THE USER, IN HIS ABEND EXIT      Y02134
*        ROUTINE, HAS SUPPLIED A NEW VOLUME SERIAL NUMBER AND    Y02134
*        RETURNED A CODE, TO PROBLEM DETERMINATION, SPECIFYING   Y02134
*        THAT AN ATTEMPT TO RECOVER BE MADE.                     Y02134
*                                                                Y02080
*        CALLER RETURN IS ALREADY SET UP AT DXSAVMOD.            Y02080
*                                                                Y02080
*        NECESSARY CLEANUP IS PERFORMED TO ALLOW THE DCB TO BE   Y02134
*        REPROCESSED.                                            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
VUS11000 EQU   *                                                 Y02080
*                                                                Y02080
         MVC   DXRETMOD,DXSAVMOD        SET UP RETURN LOAD       YM5728
         MVC   DXRETCOD,DXSAVOFF        SET UP RETURN OFFSET     YM5728
         L     RUCB,DXUCBADR            LOAD CURRENT UCB ADDRESS Y02080
         L     RTIOT,DXTIOTAD           LOAD TIOT ADDRESS        Y02080
         XC    DXVOLSR1,DXVOLSR1        ZERO OUT OLD VOLSER      Y02080
         TM    DXATGENS,DXATDMCT        TEST IF COUNT INCR       YM5728
         BNO   VUS03000                 RETRY WITH NEW VOLS      YM5728
         BAL   RET,VUS07100             DECREMENT DM COUNT       Y02134
         B     VUS03000                 RETRY WITH NEW VOLS      YM5728
         EJECT
*
***********************************************************************
*
*        CONSTANTS
*
***********************************************************************
*
SETLOCK  DC    A(VUS20000)              SETLOCK ERROR ROUTINE  @ZA14612
EXIT213  DC    A(VUS20060)              213 EXIT ROUTINE       @ZA14612
EXITB37  DC    A(VUS20070)              B37 EXIT ROUTINE       @ZA14612
SETMODE  DC    A(VUS20075)              SETMODE ROUTINE        @ZA15095
LASTVOL  DC    A(VUS20090)              LAST VOLUME ROUTINE    @ZA14612
DEQLIST  DS    0F                       DEQ LIST                 YM5728
         DEQ   (SYSZVOLS,,6,SYSTEM),TCB=0,MF=L                   YM5728
SYSZVOLS DC    CL8'SYSZVOLS'            VOL SER ENQ MAJOR NAME   Y02134
SCRTCH   DC    CL6'SCRTCH'              SCRTCH VOLUME SERIAL     Y02134
BLANKS   DC    CL8' '                   BLANK VOLUME SERIAL      Y02134
ZEROS    DC    XL6'00'                  ZERO VOLUME SERIAL       Y02134
VUSVIO   DC    CL6'VIO   '              DUMMY VOL SER FOR VIO    YM3822
VOL5     DC    H'5'                     THE CONSTANT FIVE        Y02134
VOL6     DC    H'6'                     THE CONSTANT SIX         Y02134
VOL15    DC    H'15'                    THE CONSTANT FIFTEEN     Y02134
ONE      DC    H'1'                     CONSTANT ONE             Y02134
MAXIMUM  DC    X'7F'                    MAXIMUM DA USERS = 127   Y02134
VUSB37   DC    X'00000B37'              B37 ABEND CODE           Y02134
VUSEOV   DC    CL3'055'                 EOV MODULE ID            Y02134
*                                                                Y02080
         EJECT
VUSERR19 DC    AL1(0)                   OPEN ERROR CODES         Y02080
         DC    AL1(0)                   CODE 1 = CANNOT HAPPEN   Y02080
         DC    AL1(1)                   CODE 2 = 413-04          YM1238
         DC    AL1(8)                   CODE 3 = 413-18          Y02080
         DC    AL1(37)                  CODE 4 = 413-1C          Y02080
         DC    AL1(50)                  CODE 5 = 113-18          Y02080
         DC    AL1(8)                   CODE 6 = 413-18          Y02080
         DC    AL1(8)                   CODE 7 = 413-18          Y02080
         DC    AL1(37)                  CODE 8 = 413-1C          Y02080
         DC    AL1(32)                  CODE 9 = 113-08          Y02080
         DC    AL1(1)                   CODE 10 = 413-04         Y02080
         DC    AL1(246)                 CODE 11 = 213-28         YM1238
         DC    AL1(6)                   CODE 12 = 513-04         Y02080
         DC    AL1(43)                  CODE 13 = 013-64         Y02080
         DC    AL1(42)                  CODE 14 = C13-10         Y02080
         DC    AL1(220)                 CODE 15 = C13-18        SA66776
*                                                                Y02134
VUSERR55 DC    AL1(0)                   EOV  ERROR CODES         Y02080
         DC    AL1(0)                   CODE 1 = CANNOT OCCUR    Y02080
         DC    AL1(0)                   CODE 2 = CANNOT OCCUR    Y02080
         DC    AL1(0)                   CODE 3 = CANNOT OCCUR    Y02080
         DC    AL1(0)                   CODE 4 = CANNOT OCCUR    Y02080
         DC    AL1(0)                   CODE 5 = CANNOT OCCUR    Y02080
         DC    AL1(0)                   CODE 6 = CANNOT OCCUR    Y02080
         DC    AL1(197)                 CODE 7 = 837-08          Y02080
         DC    AL1(197)                 CODE 8 = 837-08          Y02080
         DC    AL1(197)                 CODE 9 = 837-08          Y02080
         DC    AL1(168)                 CODE 10 = B37-04         Y02080
         DC    AL1(247)                 CODE 11 = B37-0C         YM1238
         DC    AL1(248)                 CODE 12 = 837-0C         YM1238
         DC    AL1(0)                   CODE 13 = CANNOT OCCUR   Y02080
         DC    AL1(0)                   CODE 14 = CANNOT OCCUR   Y02080
         DC    AL1(0)                   CODE 15 = CANNOT OCCUR  SA66776
VUSERRFF EQU   VUSERR55-VUSERR19        LENGTH OF CODE LIST      Y02080
*                                                                Y02134
         EJECT
         XCTLTABL ID=(ID4A4A,4A,ID4A4C,4C,ID4A4F,4F,ID4A4J,4J,   Y02134X
               ID4A0P,0P,ID4A5G,5G,,IFG0554T),                   YM6514X
               SVC=019,BRT=SHORT,LENGTH=4K                     @ZA14612
         EJECT                                                 @ZA14612
*                                                              @ZA14612
***************************************************************@ZA14612
*                                                              @ZA14612
*        AN ERROR HAS BEEN DETECTED.                           @ZA14612
*                                                              @ZA14612
***************************************************************@ZA14612
*                                                              @ZA14612
VUS20000 DMABCOND PCK,SETLOCK           SETLOCK INTERFACE ERROR@ZA14612
*                                                              @ZA14612
***************************************************************@ZA14612
*                                                              @ZA14612
*        GO TO 213 EXIT ROUTINE                                @ZA14612
*                                                              @ZA14612
***************************************************************@ZA14612
*                                                              @ZA14612
VUS20060 EQU   *                                               @ZA14612
*                                                              @ZA14612
         LA    R0,K30                   LOAD 213-04 CODE       @ZA14612
         IECRES LOAD,MODID=ID4A5G,BRCODE=K0,BRANCH=QUEUED      @ZA14612
*                                                              @ZA14612
***************************************************************@ZA14612
*                                                              @ZA14612
*        GO TO B37 EXIT ROUTINE                                @ZA14612
*                                                              @ZA14612
***************************************************************@ZA14612
*                                                              @ZA14612
VUS20070 EQU   *                                               @ZA14612
*                                                              @ZA14612
         TM    DXDCBLST,PLISTOIN        OUTPUT OR OUTIN        @ZA14612
         BNO   VUS10090                 BRANCH IF NEITHER      @ZA14612
*                                                              @ZA14612
         CLC   VUSEOV,WTGMODNM+K3       CHECK FOR EOV          @ZA14612
         BNE   VUS10090                 BRANCH IF NOT EOV      @ZA14612
*                                                              @ZA14612
         ICM   RD,B'1111',VUSB37        LOAD B37 ABEND CODE    @ZA14612
         LA    RC,EABD185               LOAD B37 ABEND CODE    @ZA14612
         IECRES LOAD,MODNM=IFG0554T,BRCODE=K0,BRANCH=QUEUED    @ZA14612
         EJECT                                                 @ZA14612
*                                                              @ZA14612
         USING  *,RF                   ESTABLISH ADDRESSABILITY@ZA18540
*                                                              @ZA18540
VUS20075 EQU   *                                               @ZA15095
         CLC   VUSOPEN,WTGMODNM+K3      OPEN IN CONTROL?       @ZA18540
         BNE   VUS05005                 CONTINUE               @ZA18540
         MODESET EXTKEY=SUPR                                   @ZA15095
*                                                              @ZA15095
         NI    UCBSTAT,ALLBITS-UCBDADI  SET SL NOT MOUNTED     @ZA15095
         XC    UCBVOLI,UCBVOLI          CLEAR VOL SERIAL       @ZA15095
*                                                              @ZA15095
         MODESET EXTKEY=DATAMGT                                @ZA15095
*                                                              @ZA15095
         B     VUS05005                 CONTINUE BLP PROCESS   @ZA15095
*                                                              @ZA18540
         DROP  RF                                              @ZA18540
*                                                              @ZA18540
***************************************************************@ZA14612
*                                                              @ZA14612
*   TEST LAST FMT 1 AREA FOR THE LAST VOLUME INDICATOR         @ZA14612
*                                                              @ZA14612
***************************************************************@ZA14612
*                                                              @ZA14612
         USING  *,RF                   ESTABLISH ADDRESSABILITY@ZA14612
*                                                              @ZA14612
VUS20090 EQU   *                                               @ZA14612
*                                                              @ZA14612
         STM   RTIOT,RET,DXREG9         SAVE REGS 9-14 IN W.A. @ZA14612
         LR    RB,RCORE                 GET FIRST AREA POINTER @ZA14612
*                                                              @ZA14612
VUS20100 EQU   *                                               @ZA14612
*                                                              @ZA14612
         NC    CORECH(R4,RB),CORECH(RB) IS THERE A NEXT AREA   @ZA14612
         BZ    VUS20400                 NO, GO TEST FOR THE    @ZA14612
*                                       LAST VOLUME INDICATOR  @ZA14612
VUS20200 EQU   *                                               @ZA14612
*                                                              @ZA14612
         L     RA,CORECH(RB)            POINT TO NEXT AREA     @ZA14612
         CLI   K0(RA),CHAR1             IS IT A FORMAT 1 AREA  @ZA14612
         BNE   VUS20300                 BRANCH IF NOT          @ZA14612
         LR    RB,RA                    USE IT AS CURRENT      @ZA14612
         B     VUS20100                 DO IT AGAIN            @ZA14612
*                                                              @ZA14612
VUS20300 EQU   *                        NEXT AREA IS NOT       @ZA14612
*                                       A FORMAT 1 AREA        @ZA14612
         LA    RA,L'JFCBDSNM(,RA)       INCREMENT PASSED KEY   @ZA14612
         NC    CORECH(K4,RA),CORECH(RA) IS THERE A NEXT AREA   @ZA14612
         BZ    VUS20400                 NO USE PREVIOUS AREA   @ZA14612
         LR    RB,RA                    UPDATE REG B           @ZA14612
         B     VUS20200                 TRY AGAIN              @ZA14612
*                                                              @ZA14612
VUS20400 EQU   *                        TEST FOR LAST VOLUME   @ZA14612
*                                       REG B POINTS TO DSCB   @ZA14612
*                                                              @ZA14612
         TM    K49(RB),K128             IS THIS THE LAST VOLUME@ZA14612
         BO    VUS20500                 BRANCH IF IT IS        @ZA14612
         LM    RTIOT,RET,DXREG9         RESTORE REGS 9-14      @ZA14612
         B     VUS03224                 TEST FOR MORE VOLUMES  @ZA14612
*                                                              @ZA14612
VUS20500 EQU   *                                               @ZA14612
*                                                              @ZA14612
         LM    RTIOT,RET,DXREG9         RESTORE REGS 9-14      @ZA14612
         B     VUS03223                 DON'T USE ANY          @ZA14612
*                                       MORE VOLUMES           @ZA14612
         DROP  RF                                              @ZA14612
*                                                              @ZA14612
*
***********************************************************************
*
*        CONSTANTS
*
***********************************************************************
*
VUSOPEN  DC    CL3'019'                 OPEN MODULE ID         @ZA18540
*
         EJECT                                                 @ZA14612
         XCTLTABL BRT=YES,LENGTH=                              @ZA14612
         IECDSECS CVT,TCB,JSCB,TIOT,PSA,DSAB,DCB,DSCB1,UCB,      Y02146X
               MAIN,WTG,PREFX,EXPAND=YES                         Y02146
         END
