         TITLE 'IFG0553C                      MICROFICHE NAME FOR EMODVO
               OL1 - LABEL EDITOR ROUTINE'                       Y02134
IFG0553C CSECT
         ENTRY EMODVOL1                                          Y02080
EMODVOL1 EQU   IFG0553C                 ALLIAS ENTRY POINT       Y02080
*                                                                Y02134
***********************************************************************
*            VS2 RELEASE 3 ADDITIONS/DELETIONS                        *
*0000                                                         @ZA11116*
*            VS2 SU32 DELETIONS                                       *
*                                                              @G32DSMI
***********************************************************************
* MODULE NAME = IFG0553C (OS/VS2)                                     *
*                                                                     *
* DESCRIPTIVE NAME = EMODVOL1 - STANDARD LABEL EDITING ROUTINE        *
*                                                                     *
* COPYRIGHT = NONE                                                    *
*                                                                     *
* STATUS = RELEASE 2, LEVEL 0                                         *
*                                                                     *
* FUNCTION =                                                          *
*                                                                     *
*        REGISTERS ARE INITIALIZED FROM REG 5 & 6 TO PROCESS EACH DCB *
*        THAT REQUIRES LABEL PROCESSING AS INDICATED BY JFCBMASK+5    *
*        NOTE - FOR EOV THERE IS ONLY ONE DCB                         *
*                                                                     *
*        1. DETERMINES PROCESSING TO BE PERFORMED:                    *
*           A. FOR SPECIFIC REQUESTS - SETS REJECT VOLUME INDICATOR   *
*           ON (UCBDMCT BIT 0) AND GOES TO PROCESS NEXT DCB. IF       *
*           THE LABEL IS CHANGED ON A TAPE WHICH IS SPECIFICALLY      *
*           REQUESTED, THIS MAY CAUSE PROBLEMS IN SUBSEQUENT          *
*           STEPS WHICH REFER TO THE TAPE BY ITS ORIGINAL LABEL.      *
*                                                                     *
*           B. FOR NON-SPECIFIC REQUESTS - CHECKS FOR LABEL OR        *
*           DENSITY CONFLICTS;                                        *
*                                                                     *
*           C. IF A LABEL CONFLICT EXISTS AND A SL IS TO BE           *
*           DESTROYED, WRITES A TAPE MARK, RESETS THE LABEL           *
*           DESTROY BIT JFCSLDES IN THE JFCBMASK+5 FIELD,             *
*           AND GOES TO PROCESS THE NEXT DCB.                         *
*                                                                     *
*           D. IF A LABEL CONFLICT EXISTS AND A SL IS TO BE           *
*           CREATED, CREATES A VOL1 LABEL, RESETS THE LABEL           *
*           CREATE BIT JFCSLCRE IN THE JFCBMASK+5 FIELD,              *
*           AND GOES TO PROCESS THE NEXT DCB.                         *
*                                                                     *
*           E. FOR DENSITY CONFLICTS, RESOLVES THE DENSITY            *
*           CONFLICT, RESETS THE DENSITY CONFLICT BIT                 *
*           JFCDUAL IN THE JFCBMASK+5 FIELD,                          *
*           AND GOES TO PROCESS THE NEXT DCB.                         *
*                                                                     *
*           F. IF A SITAUTION IS ENCOUNTERED WHERE IT BECOMES         *
*           DESIREABLE TO HAVE A NEW TAPE MOUNTED, SET THE            *
*           UCBMOUNT BIT IN THE UCBDMCT FIELD OF THE UCB.             *
*                                                                     *
* NOTES = IF THE JFCSLDES BIT IS NOT RESET IN THE JFCBMASK+5          *
*         THE LABEL ON THE TAPE WILL BE DESTROYED                     *
*         WHEN YOU RETURN TO IGG0550P.                                *
*       * IF THE JFCSLCRE BIT IS NOT RESET IN THE JFCBMASK+5          *
*         A LABEL WILL BE CREATED ON THE TAPE                         *
*         WHEN YOU RETURN TO IGG0550P.                                *
*       * IF THE JFCDUAL BIT IS NOT RESET IN THE JFCBMASK+5           *
*         THIS ROUTINE WILL NOT BE REENTERED IF A SUBSEQUENT          *
*         DENSITY CONFLICT OCCURS IN LABEL VERIFICATION.              *
*       * THE LABELS ARE ALWAYS REWRITTEN AT THE DENSITY              *
*         INDICATED IN THE DXDEBMOD FIELD OF THE WORK AREA.           *
*       * BOTH JFCSLDES AND JFCSLCRE MAY BE ON IF SWITCHING           *
*         FROM SL TO AL OR AL TO SL.                                  *
*       * THIS MODULE IS A DUPLICATE FOR OMODVOL1.                    *
*       * **NOTE** IGG0550P WILL CHECK FOR PASSWORD PROTECTION @G32DSMI
*         OR RACF PROTECTION ON THE TAPE PRIOR TO PERFORMING   @G32DSMI
*         THE LABEL EDITING ACTION REQUESTED. A SECURITY       @G32DSMI
*         FAILURE WILL CAUSE NO EDITING ACTION TO OCCUR.       @G32DSMI
*                                                                     *
*    DEPENDENCIES =                                                   *
*            CLASS ONE CHARACTER CODE.  THE EBCDIC CHARACTER CODE     *
*            WAS USED FOR ASSEMBLY.  THE MODULE MUST BE REASSEMBLED   *
*            IF A DIFFERENT CHARACTER SET IS USED FOR EXECUTION.      *
*            THIS ROUTINE USES REGISTER CONVENTIONS AND WORK AREAS    *
*            DEFINED FOR OPEN/CLOSE/EOV ROUTINES.                     *
*            OPEN/CLOSE/EOV DEPEND ON THIS ROUTINE FOR UPHOLDING      *
*            THE SECURITY AND THE INTEGRITY OF THE MACHINE.           *
*                                                                     *
*    RESTRICTIONS = NONE                                              *
*                                                                     *
*    REGISTER CONVENTIONS =                                           *
*            R2 POINTS TO DCB                                         *
*            R4 POINTS TO OPEN WORK AREA                              *
*            R5 POINTS TO THE RESIDENT ROUTINE                        *
*            R6 POINTS TO THE WTG TABLE                               *
*            R7 POINTS TO THE CURRENT PARAMETER LIST ENTRY            *
*            R8 POINTS TO THE CURRENT WTG TABLE ENTRY                 *
*            R9 POINTS TO THE DD ENTRY IN THE TIOT                    *
*            R10 POINTS TO THE UCB                                    *
*                                                                     *
*    PATCH LABEL = SEE NEXT TO LAST LABEL BEFORE ORG STATEMENT AT END *
*                  OF LISTING.                                        *
*                                                                     *
* MODULE TYPE = CONTROL (OPEN, CLOSE, EOV DATA MANAGEMENT)            *
*                                                                     *
*    PROCESSOR = ASSEMBLER XF                                         *
*                                                                     *
*    MODULE SIZE = SEE EXTERNAL SYMBOL DICTIONARY OR LOC FIELD ON     *
*                  ORG STATEMENT AT END OF LISTING                    *
*                                                                     *
*    ATTRIBUTES = REENTRANT, REFRESHABLE,READ-ONLY, ENABLED,          *
*                 PRIVILEGED, SUPERVISOR STATE, KEY 0,                *
*                 LINK PACK AREA RESIDENT/PAGEABLE                    *
*                                                                     *
* ENTRY POINT = IFG0553C                                              *
*               EMODVOL1                                              *
*                                                                     *
*    PURPOSE = TO RESOLVE LABEL AND DENSITY CONFLICTS IN A MANNER     *
*              APPROPRIATE WITH THE OPERATING CHARACTERISTICS         *
*              PARTICULAR TO THE INSTALLATION.                        *
*                                                                     *
*    LINKAGE = AS FOLLOWS -                                           *
*        LA    RPAR,PARMLIST            GET PARM LIST POINTER         *
*        LA    RWTG,WTG                 GET WTG TABLE POINTER         *
*        MODESET EXTKEY=ZERO            KEY 0                         *
*        IECRES LOAD,MODNM=EMODVOL1,BRANCH=DIRECT,PREFIX=WTGPREFX     *
*                                                                     *
* INPUT = RWTG (REG 6) POINTS TO WTG TABLE,                           *
*            R2 POINTS TO DCB                                         *
*            R4 POINTS TO OPEN WORK AREA                              *
*            R5 POINTS TO THE RESIDENT ROUTINE                        *
*            R6 POINTS TO THE WTG TABLE                               *
*            R7 POINTS TO THE CURRENT PARAMETER LIST ENTRY            *
*            R8 POINTS TO THE CURRENT WTG TABLE ENTRY                 *
*            R9 POINTS TO THE DD ENTRY IN THE TIOT                    *
*            R10 POINTS TO THE UCB                                    *
*                                                                     *
* OUTPUT = THE JFCBMASK+5 BITS MUST BE SET AS STATED ABOVE.           *
*          UCBDMCT - UCBMOUNT SET ON IF VOLUME DISMOUNT DESIRED.      *
*                                                                     *
* EXIT-NORMAL = IGG0550P (IFG0553D) LABEL EDITOR AND DESTROY          *
*               STANDARD LABEL                                        *
*                                                                     *
* EXIT-ERROR = NONE                                                   *
*                                                                     *
* EXTERNAL REFERENCES = SEE BELOW                                     *
*                                                                     *
*    ROUTINES = IGG0550P,IFG019RA.                                    *
*                                                                     *
*    DATA AREAS = WTG TABLE, OPEN WORK AREA.                          *
*                SEE PARAMETERS OF THE IECDSECS MACRO.                *
*                                                                     *
*    CONTROL BLOCK = UCB, JFCB.                                       *
*                SEE PARAMETERS OF THE IECDSECS MACRO.                *
*                                                                     *
* TABLES = SEE XCTLTABL MACRO.                                        *
*                                                                     *
* MACROS = IECDSECS,IECEQU,IECRES,XCTLTABL.                           *
*                                                                     *
* CHANGE ACTIVITY = NEW RELEASE (LEVEL 0)                             *
*                                                                     *
***********************************************************************
         EJECT
*                  SYMBOLIC REGISTER USAGE
*
*
*                  ESTABLISH ADDRESSABILITY
         BALR  RBASE,R0                 SET BASE REGISTER
         USING *,RBASE
         USING FORCORE,RCORE            WORK AREA
         USING WTG,RWTG                 WHERE-TO-GO TABLE        Y02080
         USING WTGENTRY,RWTGC           RWTGC POINTS TO CURRENT  Y02134
         USING USERPRML,RPARC           RPARC POINTS TO CURRENT  Y02134
         USING IHADCB,RDCB              DCB
         USING UCBOB,RUCB               UCB
         USING TIOENTRY,RTIOT           TIOT ENTRY
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        INITIALIZE REGISTERS                                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         LR    RPARC,RPAR               POINT TO FIRST ENTRY     Y02134
         LA    RWTGC,DXXENTRY           POINT TO FIRST ENTRY   @ZA11116
*                                                                Y02134
EMO01000 EQU   *                        PROCESS DCB IN THIS LOAD Y02134
*                                                                Y02134
         CLC   WTGIDTTR(K2),ID3C3C      CHECK IF RIGHT DCB       Y02134
         BNE   EMO09000                 BRANCH IF ENTRY NOT SAME Y02134
*                                                                Y02134
         L     RDCB,DXDCBLST            LOAD DCB POINTER         Y02134
         L     RCORE,WTGCORE-K1         LOAD WORK AREA POINTER   Y02134
         L     RTIOT,DXDSAB             LOAD POINTER TO DSAB     Y02134
         L     RTIOT,DSABTIOT-DSAB(,RTIOT) GET TIOENTRY POINTER  Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        GET UCB ADDRESS FROM WORK AREA                          Y02134
*        FIRST ENTRY IN TIOT MAY NOT BE VALID                    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         L     RUCB,DXUCBADR            GET CURRENT UCB ADDRESS  Y02134
*
*****************************************************************Y02134
*                                                                Y02134
*CHECK JFCB MASK TO DETERMINE IF PROCESSING IS REQUIRED ON THIS DCB
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         TM    JFCBMASK+K5,LBLERR       TEST FOR CONFLICT IN LABEL
         BNZ   EMO02000                 BRANCH IF LABEL CONFLICT Y02134
*                                                                Y02134
         TM    JFCBMASK+K5,JFCDUAL      TEST IF CONFLICT IN DENS Y02134
         BNO   EMO08000                 BRANCH IF NO CONFLICT    Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        DENSITY CONFLICTS TO BE RESOLVED HERE                   Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        THE FOLLOWING TWO INSTRUCTIONS, IF ACTIVATED, WILL      YM3819
*        CAUSE A SPECIFICALLY REQUESTED TAPE TO BE REJECTED      YM3819
*        WHEN A DENSITY CONFLICT OCCURS. IF THE TAPE IS          YM3819
*        REMOUNTED BY THE OPERATOR WITHOUT TAMPERING WITH        YM3819
*        THE LABEL, THE TAPE WILL BE ACCEPTED AND THIS ROUTINE   YM3819
*        WILL NOT RECEIVE CONTROL AGAIN. THE LABEL ON THE TAPE   YM3819
*        WILL BE REWRITTEN AT THE USER REQUESTED DENSITY.        YM3819
*                                                                YM3819
*        IF THE FOLLOWING TWO INSTRUCTIONS ARE NOT ACTIVATED     YM3819
*        THE LABEL ON THE TAPE WILL BE REWRITTEN AT THE USER     YM3819
*        REQUESTED DENSITY WITHOUT AN INTERVENING DISMOUNT.      YM3819
*                                                                YM3819
*****************************************************************YM3819
*                                                                YM3819
         CLC   DXVOLSR2,SCRTCH          CHECK IF SCRATCH REQUEST YM3819
*        BNE   EMO08000                 BRANCH IF NOT SCRTCH     YM3819
*        FOLLOWING INSTRUCTION MAY BE ZAPPED TO 4770             YM3819
         NOP   EMO08000                 BRANCH IF NOT SCRTCH     YM3819
*                                                                YM3819
*****************************************************************YM3819
*                                                                YM3819
* INSERT YOUR INSTRUCTIONS TO RESOLVE DENSITY CONFLICTS,         Y02134
* AND REMOVE ASTERISKS TO ACTIVATE THE NEXT INSTRUCTION          Y02134
* AND REASSEMBLE THIS MODULE.                                    Y02134
*                                                                Y02134
******** NI    JFCBMASK+K5,ALLBITS-JFCDUAL SET DENSITY RESOLVED  Y02134
*                                                                Y02134
         B     EMO09000                 BRANCH TO GET NEXT ENTRY Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        RESOLVE LABEL CONFLICTS HERE                            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
EMO02000 EQU   *                        CHECK LABEL CONFLICT
*                                                                Y02134
         TM    JFCBMASK+K5,JFCSLDES     DESTROY SL LABEL         Y02134
         BNO   EMO04000                 NO, CHECK IF CREATE      YM4647
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        DESTROY STANDARD LABEL HERE                             Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
EMO03000 EQU   *                        USER ROUTINE HERE        Y02134
*                                                                Y02134
*****************************************************************YM3819
*                                                                YM3819
*        THE FOLLOWING TWO INSTRUCTIONS, IF ACTIVATED, WILL      YM3819
*        CAUSE A SPECIFICALLY REQUESTED TAPE TO BE REJECTED      YM3819
*        WHEN A LABEL CONFLICT OCCURS (NL REQUESTED & SL         YM3819
*        MOUNTED).                                               YM3819
*                                                                YM3819
*        IF THE FOLLOWING TWO INSTRUCTIONS ARE NOT ACTIVATED     YM3819
*        THE LABEL ON THE TAPE WILL BE REWRITTEN AT THE USER     YM3819
*        REQUESTED LABEL TYPE WITHOUT AN INTERVENING DISMOUNT,   YM3819
*        REGARDLESS IF THE REQUEST IS SPECIFIC OR NOT.           YM3819
*                                                                YM3819
*****************************************************************YM3819
*                                                                YM3819
         CLC   DXVOLSR2,SCRTCH          CHECK IF SCRATCH REQUEST YM3819
*        BNE   EMO08000                 BRANCH IF NOT SCRTCH     YM3819
*        FOLLOWING INSTRUCTION MAY BE ZAPPED TO 4770             YM3819
         NOP   EMO08000                 BRANCH IF NOT SCRTCH     YM3819
*                                                                YM3819
*****************************************************************YM3819
*                                                                YM3819
* INSERT YOUR INSTRUCTIONS TO DESTROY THE VOLUME LABEL           Y02134
* ACCORDING TO PUBLICATION GC28-6680,                            Y02134
* AND REMOVE ASTERISKS TO ACTIVATE THE NEXT INSTRUCTION,         Y02134
* AND REASSEMBLE THIS MODULE.                                    Y02134
*                                                                Y02134
******** NI    JFCBMASK+K5,ALLBITS-JFCSLDES SET DESTROY SL OFF   Y02134
*                                                                Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CREATE STANDARD LABEL HERE                              Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
EMO04000 EQU   *                        USER ROUTINE HERE        Y02134
*                                                                Y02134
         TM    JFCBMASK+K5,JFCSLCRE     CREATE SL LABEL          YM4647
         BNO   EMO09000                 NO, RETURN TO REVERIFY   YM4647
*                                                                YM4647
*****************************************************************YM3819
*                                                                YM3819
*        THE FOLLOWING TWO INSTRUCTIONS WILL                     YM3819
*        CAUSE A SPECIFICALLY REQUESTED TAPE TO BE REJECTED      YM3819
*        WHEN A LABEL CONFLICT OCCURS (SL REQUESTED & NL         YM3819
*        MOUNTED).                                               YM3819
*                                                                YM3819
*        IF THE FOLLOWING TWO INSTRUCTIONS ARE NOT ACTIVATED     YM4647
*        THE LABEL ON THE TAPE WILL BE REWRITTEN AT THE USER     YM4647
*        REQUESTED LABEL TYPE WITHOUT AN INTERVENING DISMOUNT,   YM4647
*        REGARDLESS IF THE REQUEST IS SPECIFIC OR NOT.           YM4647
*                                                                YM3819
*****************************************************************YM3819
*                                                                YM3819
         CLC   DXVOLSR2,SCRTCH          CHECK IF SCRATCH REQUEST YM3819
*        BNE   EMO08000                 BRANCH IF NOT SCRTCH     YM4647
*        FOLLOWING INSTRUCTION MAY BE ZAPPED TO 4770             YM4647
         NOP   EMO08000                 BRANCH IF NOT SCRTCH     YM4647
*                                                                YM3819
*****************************************************************YM3819
*                                                                YM3819
* INSERT YOUR INSTRUCTIONS TO CREATE A STANDARD LABEL            Y02134
* ACCORDING TO PUBLICATION GC28-6680,                            Y02134
* AND REMOVE ASTERISKS TO ACTIVATE THE NEXT INSTRUCTION,         Y02134
* AND REASSEMBLE THIS MODULE.                                    Y02134
*                                                                Y02134
******** NI    JFCBMASK+K5,ALLBITS-JFCSLCRE SET CREATE SL OFF    Y02134
*                                                                Y02134
         B     EMO09000                 BRANCH TO GET NEXT ENTRY Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        REJECT MOUNTED VOLUME HERE                              Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
EMO08000 EQU   *                        REJECT VOLUME HERE       Y02134
*                                                                Y02134
         OI    UCBDMCT,UCBMOUNT         SET REJECT ON            Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        GET NEXT ENTRY IN PARM LIST                             Y02134
*        ONLY ONE ENTRY FOR EOV                                  Y02134
*****************************************************************Y02134
*                                                                Y02134
EMO09000 EQU   *                        GET NEXT DCB             Y02134
*                                                                Y02134
******** TM    DXDCBLST,LASTNTRY        TEST FOR LAST ENTRY      Y02134
******** LA    RPARC,DXDCBLST+L'DXDCBLST GET NEXT ENTRY          Y02134
******** LA    RWTGC,WTGENTRY+L'WTGENTRY GET NEXT ENTRY          Y02134
******** BNO   EMO01000                 BR TO PROCESS NEXT ENTRY Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        RETURN TO IGG0550P                                      Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
         IECRES LOAD,MODNM=IDRETURN,PREFIX=WTGPREFX,             Y02134X
               BRANCH=DIRECT                                     Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
*        CONSTANTS                                               Y02134
*                                                                Y02134
*****************************************************************Y02134
*                                                                Y02134
ID3C3C   DC    CL2'3C'                  ID OF OMODVOL1/EMODVOL1  Y02134
SCRTCH   DC    CL6'SCRTCH'              NON SPECIFIC REQUEST     Y02134
*                                                                Y02134
         XCTLTABL ID=(IDRETURN,IGG0550P),BRT=YES,LENGTH=         Y02134
*                                                                Y02134
         IECDSECS CVT,TCB,DSAB,TIOT,DCB,UCB,MAIN,WTG,PREFX,      Y02134X
               EXPAND=YES                                        Y02134
         IECEQU
         END
