         TITLE '      IFG0558X   CHECKPOINT AT EOV ROUTINE '
IFG0558X CSECT
*                                                                     *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                                                     *
* MODULE NAME = IFG0558X                                              *
*                                                                     *
* DESCRIPTIVE NAME = CHECKPOINT AT EOV ROUTINE                        *
*                                                                     *
* COPYRIGHT = NONE                                                    *
*                                                                     *
* STATUS = RELEASE 2, LEVEL 0                                         *
*                                                                     *
* FUNCTION = THIS ROUTINE IS INVOKED BY EOV FOR BSAM/QSAM DATA SETS   *
*            AT WHOSE EOV OCCURRANCES A CHECKPOINT IS TO BE TAKEN.    *
*                                                                     *
*            THE 'CHKPT' MACRO WILL INVOKE OS/VS CHECKPOINT.          *
*                                                                     *
*            A SEQUENTIAL CHECKPOINT DCB WILL DEFINE THE CHECKPOINT   *
*            DATA SET WITH THE DDNAME OF 'SYSCKEOV'.                  *
*                                                                     *
*            SUCCESSFUL COMPLETION OF CHECKPOINT, AS DETERMINED BY    *
*            RETURN CODES 0 AND 4 FROM 'CHKPT', WILL CAUSE IMMEDIATE  *
*            EXIT TO IFG0551L.                                        *
*                                                                     *
*            UNSUCCESSFUL COMPLETION, AS DETERMINED BY ALL OTHER      *
*            RETURN CODES, WILL RESULT IN A WARNING MESSAGE BEING     *
*            GENERATED PRIOR TO EXIT EXCEPT WHEN:                     *
*                                                                     *
*                RETURN CODE X'08' FOR DIRECT ACCESS OR X'14' FOR     *
*                TAPE OCCURS -- THE 'CHKPT' MACRO WILL BE RE-ISSUED   *
*                ONE TIME TO ATTEMPT TO WRITE GOOD CHECKPOINT RECORDS *
*                ON THE NEW EXTENT OR VOLUME IF AVAILABLE.            *
*                                                                     *
*                SECOND TRY FAILURE WILL RESULT IN THE WARNING        *
*                MESSAGE AS ABOVE.                                    *
*                                                                     *
*            THE FOLLOWING IDENTIFIES THE NEW WARNING MESSAGE:        *
*                                                                     *
*                IEC067I  CHKPT=EOV FACILITY EXECUTED UNSUCCESSFULLY  *
         EJECT
* NOTES = THIS ROUTINE IS WRITTEN AS A DCB USER EXIT.                 *
*                                                                     *
*         IT DOES NOT CONFORM TO REGISTER SAVING AND CHAINING         *
*         CONVENTIONS.                                                *
*                                                                     *
*         REFERENCE OS/VS DATA MANAGEMENT SERVICES SRL FOR MORE       *
*         DETAIL.                                                     *
*                                                                     *
*    DEPENDENCIES = CLASS ONE CHARACTER CODE.  EBCDIC CHARACTER CODE  *
*                   USED FOR ASSEMBLY.                                *
*                                                                     *
*    RESTRICTIONS = NONE                                              *
*                                                                     *
*    REGISTER CONVENTIONS = NONE                                      *
*                                                                     *
*    PATCH LABEL = CKVPTCH                                            *
*                                                                     *
* MODULE TYPE = CONTROL SUBROUTINE (EOV USER EXIT)                    *
*                                                                     *
*    PROCESSOR = ASSEMBLER XF                                         *
*                                                                     *
*    MODULE SIZE = SEE LOC FIELD ON CKVCEND STATEMENT PRIOR TO DSECT  *
*                                                                     *
*    ATTRIBUTES = REUSABLE, USER KEY                                  *
*                                                                     *
* ENTRY POINT = IFG0558X                                              *
*                                                                     *
*    PURPOSE = SEE FUNCTION                                           *
*                                                                     *
*    LINKAGE = ENTERED VIA LOAD/SYNCH FROM MODULE IFG0551L            *
*                                                                     *
* INPUT = REGISTER 1 POINTS TO DCB OF DATA SET AT EOV (NOT USED).     *
*         REGISTER 14 POINTS TO RETURN ADDRESS IN MODULE IFG0551L.    *
*                                                                     *
* OUTPUT = CHECKPOINT RECORDS ON EOV CHECKPOINT DATA SET (SYSCKEOV).  *
*          REGISTER 15 = 0.                                           *
*                                                                     *
* EXIT-NORMAL = RETURN TO IFG0551L VIA BRANCH.                        *
*                                                                     *
* EXIT-ERROR = NONE                                                   *
         EJECT
* EXTERNAL REFERENCES = SEE BELOW                                     *
*                                                                     *
*    ROUTINES = SVC 63 VIA CHKPT MACRO                                *
*                                                                     *
*    DATA AREAS = WORK SPACE FOR SAVE AREA AND MISCELLANEOUS FIELDS   *
*                                                                     *
*    CONTROL BLOCKS = DCB                                             *
*                                                                     *
* TABLES = NONE                                                       *
*                                                                     *
* MACROS = CHKPT                                                      *
*          WTO                                                        *
*                                                                     *
* CHANGE-ACTIVITY = NONE                                              *
*                                                                     *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
         EJECT
         IECDSECS DCB,PREFX                                     Z30JSFC
         IECEQU                                                 Z30JSFC
         SPACE 1
MAXCODE  EQU   X'18'                    MAXIMUM CKPT/RST RETCODEZ30JSFC
         SPACE 3
*                                                                     *
* * * *      ESTABLISH ADDRESSABILITY AND FORWARD SAVE AREA     * * * *
*                                                                     *
         BALR  RBASE,0                  ESTABLISH ADDRESSABILITYZ30JSFC
         USING *,RBASE                  ADDRESSABILITY          Z30JSFC
         USING CKVDSECT,RCORE           IFG0558X WORKAREA       Z30JSFC
         GETMAIN R,LV=CKVLENTH          GET SUBPOOL 0 CORE      Z30JSFC
         LR    RCORE,R1                 GET WORKAREA ADDRESS    Z30JSFC
         XC    CKVDSECT(CKVLENTH),CKVDSECT ZERO WORK AREA       Z30JSFC
         ST    RET,CKVSVRE              STORE RETURN ADDR.      Z30JSFC
         LA    RD,CKVSVAR               PROVIDE SAVEAREA FORWARDZ30JSFC
         SPACE 3
*                                                                     *
* * *                ISSUE 'CHKPT' MACRO PROVIDING                * * *
* * * *             THE DCB TO BE OPENED AND CLOSED             * * * *
* * *                     BY OS/VS CHECKPOINT                     * * *
*                                                                     *
CKVCKPT  EQU *                          ISSUE CHECKPOINT        Z30JSFC
         MVC   CKVDCBD,CKVDCB           MOVE CHECKPOINT DCB     Z30JSFC
*                                       TO WORK AREA            Z30JSFC
         LA    R1,CKVDCBD               GET DCB ADDRESS         Z30JSFC
         ST    R1,CKVLIST               STORE IN PARM LIST      Z30JSFC
         OI    CKVLIST,LASTNTRY         SET LAST ENTRY          Z30JSFC
         LA    R1,CKVLIST               POINT TO PARM LIST      Z30JSFC
         CHKPT MF=(E,(1))               INVOKE OS/VS CHECKPOINT Z30JSFC
         LA    R0,MAXCODE               GET MAX EXPECTED CODE   Z30JSFC
         CR    RF,R0                    CHECK IF > EXPECTED     Z30JSFC
         BH    CKVEROR                  BRANCH IF UNEXPECTED    Z30JSFC
         B     CKVBRTB(RF)              BRANCH BASED ON CODE    Z30JSFC
         EJECT
*                                                                     *
* * * *       BRANCH TABLE TO SERVICE RETURN CODE OPTIONS       * * * *
*                                                                     *
CKVBRTB  EQU  *                         BRANCH ENTRY            Z30JSFC
         B     CKVEXIT                  RC=00, EXIT (CHKPT)     Z30JSFC
         B     CKVEXIT                  RC=04, EXIT (RSTRT)     Z30JSFC
         B     CKVRTRY                  RC=08, RETRY (D/A)      Z30JSFC
         B     CKVEROR                  RC=0C, ISSUE MESSAGE    Z30JSFC
         B     CKVEROR                  RC=10, ISSUE MESSAGE    Z30JSFC
         B     CKVRTRY                  RC=14, RETRY (TAPE)     Z30JSFC
         B     CKVEROR                  RC=18, ISSUE MESSAGE    Z30JSFC
         SPACE 3
*                                                                     *
* * *   ISSUE UNSUCCESSFUL CHECKPOINT MESSAGE AND RETURN TO EOV   * * *
*                                                                     *
CKVEROR  EQU   *                        ERROR ENTRY             Z30JSFC
         WTO   'IEC067I  CHKPT=EOV FACILITY EXECUTED UNSUCCESSFULLY',  X
               ROUTCDE=(2,11),          SEND TO OPER/PGMR              X
               DESC=6                   JOB STATUS MSG.         Z30JSFC
         SPACE 1
         B     CKVEXIT                  EXIT TO RESUME EOV      Z30JSFC
         SPACE 3
*                                                                     *
* * * *             TEST RETRY SWITCH TO ALLOW ONLY             * * * *
* * * *              ONE RETRY FOR GOOD CHECKPOINT              * * * *
*                                                                     *
CKVRTRY  EQU   *                        RETRY ENTRY             Z30JSFC
         TM    CKVRTSW,ALLBITS          IS THIS FIRST RETRY?    Z30JSFC
         BO    CKVEROR                  NO - ISSUE MESSAGE      Z30JSFC
         OI    CKVRTSW,ALLBITS          YES - SET RETRY SWITCH  Z30JSFC
         B     CKVCKPT                  GO TO RE-ISSUE CHKPT    Z30JSFC
         SPACE 3
*                                                                     *
* * * *              RETURN TO EOV MODULE IFG0551L              * * * *
*                                                                     *
CKVEXIT  EQU   *                        EXIT POINT              Z30JSFC
         L     RET,CKVSVRE              RESTORE EOV RETURN ADDR Z30JSFC
         FREEMAIN R,LV=CKVLENTH,A=(RCORE) FREE WORK AREA        Z30JSFC
         RETURN RC=0                    RETURN                  Z30JSFC
         EJECT
*                                                                     *
* * * *         DCB FOR SEQUENTIAL CHECKPOINT DATA SET          * * * *
*                                                                     *
CKVDCB   DCB   DSORG=PS,MACRF=W,RECFM=U,         . SEQUENTIAL          X
               DEVD=DA,OPTCD=W,                  . CHECKPOINT          X
               DDNAME=SYSCKEOV                   . DCB          Z30JSFC
         EJECT
         XCTLTABL SVC=055,BRT=YES,LENGTH=                       Z30JSFC
         IECDSECS EXPAND=YES                                    Z30JSFC
*                                                                     *
* * * *                 MISCELLANEOUS WORK AREA                 * * * *
*                                                                     *
CKVDSECT DSECT                                                  Z30JSFC
CKVSVRE  DS    F                        RETURN REG(14) SAVE AREAZ30JSFC
CKVSVAR  DS    XL72                     SAVE AREA FOR IFG0558X  Z30JSFC
CKVRTSW  DS    XL1                      RETRY INDICATOR/SWITCH  Z30JSFC
CKVLIST  DS    A                        CHECKPOINT PARMLIST     Z30JSFC
         DS    0D                       ALIGN BOUNDRY           Z30JSFC
CKVDCBD  DS    XL96                     CHECKPOINT/RESTART DCB  Z30JSFC
CKVEND   DS    0D                       END OF WORKAREA         Z30JSFC
CKVLENTH EQU   *-CKVDSECT               LENGTH FOR GETMAIN      Z30JSFC
         END
