HALT     TITLE 'IGC0003C - HALTIO SVC ROUTINE - SVC 33'
*****************************************************************
*                                                               *
* MODULE NAME = IGC0003C ( OS/VS2  )                            *
*                                                               *
* DESCRIPTIVE NAME = HALT I/O SVC ROUTINE                       *
*                                                               *
* COPYRIGHT = NONE                                              *
*                                                               *
* STATUS = RELEASE 3, LEVEL=0                                   *
*                                                               *
* FUNCTION = THIS MODULE ENABLES USERS TO HALT I/O ACTIVITY     *
*            ON A TP OR CTC DEVICE.                             *   CTC
*       TP - FOR A TP DEVICE THE METHOD USED TO STOP THE I/O    *   CTC
*            ACTIVITY IS EITHER TO LINK TO THE IOS HALT I/O     *
*            SUBROUTINE TO ISSUE A HDV INSTRUCTION OR THE EXCP  *
*            CCW NOP MODIFY ROUTINE TO MODIFY A CCW TO A NOP.   *
*            THE METHOD SELECTED IS DETERMINED BY THE INPUT     *
*            PARAMETERS PASSED TO THIS MODULE.  THE CCW MODIFY  *
*            OPTION, HOWEVER, IS ONLY VALID FOR I/O ORIGIN-     *
*            ALLY PASSED TO IOS BY THE EXCP DRIVER.             *
*            VALIDITY CHECKING IS DONE TO ASSURE THAT;          *
*              - A VALID UCB ADDRESS IS PASSED.                 *
*              - THE INPUT UCB IS ASSOCIATED WITH A TP DEVICE.  *
*              - THE INPUT UCB IS ASSOCIATED WITH AN ACTIVE     *
*                DEVICE.                                        *
*              - THE INPUT UCB IS ASSOCIATED WITH A REQUEST     *
*                WHICH IS SCHEDULED FOR PROCESSING IN THE       *
*                CALLER OF SVC 33'S MEMORY.                     *
*      CTC - FOR A CTC DEVICE A VALID HALTIO REQUEST MAY OCCUR  *   CTC
*            WITH THE UCB NOT ACTIVE.  THE CHANNEL MAY EVEN BE  *   CTC
*            BUSY WITH ANOTHER DEVICE.  IF A CTC IS BEING HALTED*   CTC
*            AND THE UCB IS NOT ACTIVE AND IT IS ON A SELECTOR  *   CTC
*            CHANNEL THEN SPECIAL CTC PROCESSING WILL OCCUR.    *   CTC
*            SPECIAL CTC PROCESSING WILL SCHEDULE A SUBROUTINE  *   CTC
*            WITHIN SVC-33 USING A SRB WITH CPU AFFINITY.  THE  *   CTC
*            SCHEDULED SUBROUTINE WILL ASSURE THE SELECTOR -    *   CTC
*            CHANNEL IS CLEARED BEFORE ENTERING THE IOS HALTIO  *   CTC
*            SUBROUTINE (IECIHIO).                              *   CTC
*            F0R A CTC DEVICE WITH AN ACTIVE CHANNEL PROGRAM,   *   CTC
*            OR ON A CHANNEL OTHER THAN A SELECTOR CHANNEL      *   CTC
*            THE SPECIAL CTC PROCESSING WILL NOT BE USED AND    *   CTC
*            PROCESSING WILL BE THE SAME AS FOR TP DEVICES.     *   CTC
*            VALIDITY CHECKING IS DONE TO ASSURE THAT;          *   CTC
*              - A VALID UCB ADDRESS IS PASSED.                 *   CTC
*              - THE INPUT UCB IS ASSOCIATED WITH A CTC DEVICE. *   CTC
*              - IF THERE IS AN ACTIVE REQUEST THAT IT WAS      *   CTC
*                SCHEDULED IN THE SAME MEMORY AS THE CALLER OF  *   CTC
*                SVC-33.                                        *   CTC
*                                                               *
* NOTES = SEE BELOW                                             *
*                                                               *
*    DEPENDENCIES = NONE                                        *
*                                                               *
*    RESTRICTIONS = NONE                                        *
*                                                               *
*    REGISTER CONVENTIONS = SEE GENERAL REGISTER ASSIGNMENT     *
*                           COMMENT SECTION WITHIN THIS         *
*                           MODULE                              *
*                                                               *
*    PATCH LABEL = HALTPATC - A 5 PER CENT DC AREA              *
*                                                               *
*    LOCKS AQUIRED = IOSUCB - (ALSO RELEASED )                  *
*                    LOCAL - IF CTC SUBROUTINE IS EXECUTED      *   CTC
*                                                               *
* MODULE TYPE = CONTROL                                         *
*                                                               *
*    PROCESSOR = ASSEMBLER XF                                   *
*                                                               *
*    MODULE SIZE = 600 DECIMAL BYTES                            *
*                                                               *
*    ATTRIBUTES = RE-ENTRANT                                    *
*                 REFRESHABLE                                   *
*                 SUPERVISOR STATE                              *
*                 PROTECT KEY 0                                 *
*                 TYPE 3 SVC                                    *
*                                                               *
* ENTRY POINT = IGC0003C FROM SVC SLIH                          *
*                                                               *
*    PURPOSE = SEE FUNCTION                                     *
*                                                               *
*    LINKAGE = REGISTER     CONTENTS                            *
*                                                               *
*                 0      OFFSET FROM IOB TO VIRTUAL CCW         *
*                        CORRESPONDING TO REAL CCW TO BE        *
*                        MODIFIED TO A NOP  (ONLY USED WHEN     *
*                        BYTE 1 OF REGISTER 1 = X'80' -         *
*                        OTHERWISE CONTENTS ARE IGNORED)        *
*                                                               *
*                 1      BYTE 0 = IGNORED                      @YM01069
*                        BYTE 1 = INPUT OPTION                  *
*                                 X'00' - USE IOS HALT I/O      *
*                                         SUBROUTINE            *
*                                 X'80' - USE EXCP CCW MODIFY   *
*                                         SUBROUTINE            *
*                        BYTES 2-3 = ADDRESS OF UCB ASSOCIATED  *
*                                    WITH HALT REQUEST          *
*                                                               *
*                 3      ADDRESS OF THE CVT                     *
*                                                               *
*                 5      ADDRESS OF SVRB                        *
*                                                               *
*                 7      ADDRESS OF ASCB                        *
*                                                               *
*                 14     RETURN ADDRESS                         *
*                                                               *
*    INPUT = CONSISTS OF THE FOLLOWING                          *
*                                                               *
*       - POINTER TO UCB ASSOCIATED WITH HALT REQUEST           *
*       - SPECIFICATION OF HALT OR CCW MODIFY OPTION            *
*       - OFFSET FROM IOB TO VIRTUAL CCW CORRESPONDING TO REAL  *
*          CCW TO BE SET TO NOP IF CCW MODIFY OPTION SPECIFIED  *
*       - POINTER TO CVT                                        *
*       - POINTER TO SVRB                                       *
*       - POINTER TO ASCB                                       *
*       - RETURN ADDRESS                                        *
*                                                               *
*    OUTPUT = ACTION TAKEN INDICATED BY RETURN CODE IN REGISTER *
*             15                                                *
*                                                               *
*    EXIT - NORMAL = RETURN CODE IN REGISTER 15 SET TO X'00'.   *
*                                                               *
*               - I/O HAS BEEN HALTED BY IOS SUBROUTINE OR      *
*                 CORRESPONDING REAL CCW MODIFIED TO A NOP      *
*                 AS REQUESTED ON INPUT                         *
*                                                               *
*               - IOSCOD FIELD OF IOSB SET TO X'48' TO SHOW     *
*                 PURGED REQUEST IF HALT OPTION REQUESTED       *
*                                                               *
*    EXIT - ERROR = REGISTER 15 RC.     REASON                  *
*                                                               *
*                     X'04'         DEVICE NOT ACTIVE           *
*                                                               *
*                     X'08'         NON-TP DEVICE               *
*                                                               *
*                     X'0C'         INVALID UCB OR I/O ERROR    *
*                                   ATTEMPTING TO HALT THE      *
*                                   DEVICE                      *
*                                                               *
*                     X'10'         MODIFY OPTION SELECTED      *
*                                   BUT ORIGINAL EXCP REQUEST   *
*                                   WAS NOT VIRTUAL             *
*                                                               *
*                     X'14'         MODIFY OPTION SELECTED BUT  *
*                                   THE TRANSLATION IS NOT YET  *
*                                   COMPLETE                    *
*                                                               *
*                     X'18'         MODIFY OPTION SELECTED BUT  *
*                                   INPUT VIRTUAL CCW ADDRESS   *
*                                   IS NOT WITHIN ORIGINAL CCW  *
*                                   CHAIN GIVEN TO EXCP         *
*                                                               *
*                     X'1C'         MODIFY OPTION SELECTED BUT  *
*                                   ORIGINAL I/O REQUEST WAS    *
*                                   NOT MADE BY THE EXCP DRIVER *
*                                                               *
* EXTERNAL REFERENCES = SEE BELOW                               *
*                                                               *
*    ROUTINES = THE FOLLOWING                                   *
*             - IOS HALT I/O SUBROUTINE - IECIHIO               *
*             - EXCP CCW NOP MODIFY SUBROUTINE - IECVEXPR(+4)   *
*                                                               *
*    DATA-AREAS = NONE                                          *
*                                                               *
*    CONTROL BLOCKS = CONTROL BLOCK       MAPPING MACRO         *
*                                                               *
*                      PSA                 IHAPSA               *
*                      FRRS                IHAFRRS              *
*                      IOSB                IECDIOSB             *
*                      UCB                 IEFUCBOB             *
*                      IOQ                 IECDIOQ              *
*                      CVT                 CVT                  *
*                      VOID                IECDVOID             *
*                      IOCM                IECDIOCM             *
*                      SVRB                IHARB                *
*                      SDWA                IHASDWA              *
*                      ASCB                IHAASCB              *
*                                                               *
* TABLES/WORKAREAS =                                            *
*                                                               *
*          SVRB EXTENSION                                       *
*          FRR PARAMETER AREA                                   *
*          GETMAINED REGISTER SAVE AREA - SUBPOOL 253           *
*          GETMAINED SRB - SUBPOOL 245                          *
*                                                               *
* MACROS = THE FOLLOWING EXECUTABLE MACROS                      *
*                                                               *
*          GETMAIN                                              *
*          SETLOCK                                              *
*          SETFRR                                               *
*          FREEMAIN                                             *
*          PGFIX                                                *
*          PGFREE                                               *
*          SETRP                                                *
*          WAIT                                                 *
*          ESTAE                                                *
*                                                               *
* CHANGE ACTIVITY = AS FOLLOWS                                  *
*                   NONE                                        *
*****************************************************************
         EJECT
         IHAPSA
         EJECT
         IHAFRRS
         EJECT
         IECDIOSB
         EJECT
         IEFUCBOB LIST=YES,PREFIX=YES
         EJECT
         IECDIOQ
         EJECT
         CVT   DSECT=YES
         EJECT
         IECDVOID
         EJECT
         IECDIOCM
         EJECT
         IHARB
         SPACE 1
         ORG   RBEXSAVE
ECB      DS    F                   ECB USED FOR PGFIX
R0R1SAVE DS    2F                  REGISTER 0 AND 1 SAVE AREA
R14SAVE  DS    F                   REGISTER 14 SAVE AREA
RCODSAVE DS    F                   SA FOR RETURN CODE          @ZA06062
R12TO14  DS    3F             SA FOR REGS 12,13,14 FOR SETLOCK @ZA06062
         ORG   R12TO14                                          @ZA9235
*  FOLLOWING MUST NOT EXCEED X'20' BYTES                        @ZA9235
ESTACOR  DS    XL(28)         AREA FOR ESTAE PARMLIST           @ZA9235
         EJECT
         IHASDWA
         EJECT
*****************************************************************
*                                                               *
*        THE FOLLOWING DSECTS MAP SVC 33'S USAGE OF THE         *
*        SDWA VARIABLE RECORDING AREA                           *
*                                                               *
*****************************************************************
         SPACE 1
VRECORD  DSECT
VRECREG0 DS    F                   SVC 33 INPUT REGISTER 0
VRECREG1 DS    F                   SVC 33 INPUT REGISTER 1
VRECIOQ  DS    XL(IOQL)            IOQ
VRECUCBP DS    XL(UCBCURPX)        UCB - PREFIX PORTION
VRECUCBB DS    XL(UCBDEV-UCBOB)    UCB - BASIC PORTION
VRECLEN  EQU   *-VRECORD           LENGTH OF VARIABLE DATA AREA
ESTAREC  DSECT                     RECORDING AREA FOR ESTAE    @ZA09235
EFREEA   DS    F                   FREEMAIN ADDR               @ZA09235
EFREELEN DS    F                   FREEMAIN LEN AND SP         @ZA09235
ERECUCBP DS    XL(UCBCURPX)        UCB PREFIX PORTION          @ZA09235
ERECUCBB DS    XL(UCBDEV-UCBOB)    UCB BASIC PORTION           @ZA09235
ERECLEN  EQU   *-ESTAREC           LENGTH OF VARIABLE AREA     @ZA09235
*                                  *USED FOR RECORDING
         EJECT
*****************************************************************
*                                                               *
*        THE FOLLOWING DSECT MAPS SVC 33'S USAGE OF THE         *
*        FRR AND ESTAE PARM AREAS                               *
*                                                               *
*****************************************************************
         SPACE 1
FRRPARM  DSECT
FRRUCBA  DS    A                   POINTER TO UCB (PREFIX)
FRRIOQA  DS    A                   IOQ ADDRESS
FRRREG0  DS    F                   SVC 33 INPUT PARAMETER
*                                  *REGISTER 0
FRRREG1  DS    F                   SVC 33 INPUT PARAMETER
*                                  *REGISTER 1
FRRUCBF  DS    H                   SAVE AREA FOR UCBSFLS
FRRFLAG  DS    XL1                 FLAGS FOR FRR               @ZA06062
FRRUCBLK EQU   X'01'               INDICATES UCBLOCK HELD      @ZA06062
FRRLOCLK EQU   X'02'               INDICATES LOCAL LOCAL HELD  @ZA06062
*                                  *ACROSS IECIHIO INTERFACE
ESTAPARM DSECT                     DSECT FOR ESTAE             @ZA09235
ESTLIST  ESTAE MF=(L)                                          @ZA09235
ESTALEN  EQU   *-ESTLIST           LENGTH FOR PARMLIST         @ZA09235
ESTADATA DS    0F                  LABEL FOR USING IN ESTAE    @ZA09235
FREEADDR DS    F                   ADDR FOR FREEMAIN           @ZA09235
FREELEN  DS    F                   LEN + SP FOR FREEMAIN       @ZA09235
EUCBADR  DS    F                   UCB ADDRESS                 @ZA09235
GMLEN    EQU   *-ESTAPARM          LENGTH FOR GETMAIN          @ZA09235
         EJECT
         IHAASCB
         EJECT                                                      CTC
*****************************************************************   CTC
*                                                               *   CTC
*        DSECT OF IOS CHANNEL TABLE ENTRY (IECRCTBL)            *   CTC
*                                                               *   CTC
*****************************************************************   CTC
CHT      DSECT                                                      CTC
         DS    C                   RESERVED                         CTC
CHTTYPE  DS    C                   CHANNEL TYPE                     CTC
*                                                                   CTC
* CODE DEFINITIONS FOR CHTTYPE                                      CTC
*                                                                   CTC
CHTMPX   EQU   1                   BYTE MULTIPLEXOR CHANNEL         CTC
CHTHSMPX EQU   2                   HI-SPEED MULTIPLEXOR CHANNEL     CTC
CHTSEL   EQU   3                   SELECTOR CHANNEL                 CTC
CHTBMPX  EQU   4                   BLOCK MULTIPLEXOR CHANNEL        CTC
*                                                                   CTC
CHTMASK  DS    H                   CHANNEL MASK FOR (IRTCHMSK)      CTC
CHTCSM   DS    A                   ADDRESS OF CHANNEL SEARCH MODULE CTC
CHTEL    EQU   8                   SIZE OF ENTRY IN BYTES           CTC
CHTELP2  EQU   3                   SIZE OF ENTRY IN POWERS OF 2     CTC
         EJECT                                                      CTC
         IHASRB                                                     CTC
         EJECT
*                                                                   CTC
HALTCSAV DS    14F                 R0 - R13 SAVE AREA               CTC
HALTCS14 DS    F                   R14 SAVE AREA                    CTC
HALTCS15 DS    F                   R15 SAVE AREA                    CTC
HALTCECB DS    F                   ECB FOR CTC HALT SUBROUTINE      CTC
         IECDCRCA                                              @Y30CQLG
         IECDLCH
         IHACSD
         IHAPCCA
         IECDCAT
         EJECT                                                      CTC
IGC0003C CSECT
         SPACE 1
*****************************************************************
*                                                               *
*        GENERAL REGISTER ASSIGNMENT                            *
*                                                               *
*****************************************************************
         SPACE 1
R0       EQU   0                   INPUT REGISTER 0
R1       EQU   1                   INPUT REGISTER 1
R2       EQU   2                   REG FOR ESTAE               @ZA09235
IOSBREG  EQU   2                   IOSB ADDRESSABILITY REGISTER
CVTREG   EQU   3                   CVT ADDRESSABILITY REGISTER
WORKD    EQU   4                   WORK REGISTER
SVRBREG  EQU   5                   SVRB ADDRESSABILITY REGISTER
BASEREG  EQU   6                   BASE REGISTER
UCBREG   EQU   7                   UCB ADDRESSABILITY REGISTER
WORKA    EQU   8                   WORK REGISTER
WORKB    EQU   9                   WORK REGISTER
WORKC    EQU   10                  WORK REGISTER
R11      EQU   11                  WORK REGISTER
FRRREG   EQU   12                  FRR PARAMETER AREA REGISTER
SAVREG   EQU   13                  SAVE AREA REGISTER
RETREG   EQU   14                  RETURN REGISTER
R15      EQU   15                  BRANCH/RETURN CODE REGISTER
         SPACE 1
*****************************************************************
*                                                               *
*        GENERAL EQUATES                                        *
*                                                               *
*****************************************************************
         SPACE 1
ONEBYTE  EQU   8                   SHIFT VALUE OF ONE BYTE
MODPURGE EQU   4                   OFFSET FROM ENTRY POINT OF
*                                  *EXCP DRIVER VOID PURGE
*                                  *ROUTINE TO CCW MODIFY
*                                  *ROUTINE
RETCOD4  EQU   X'04'               DEVICE NOT ACTIVE RETURN CODE
RETCOD8  EQU   X'08'               DEVICE NOT ELIGABLE FOR HALT
*                                  *BY SVC 33 RETURN CODE
RETCODC  EQU   X'0C'               INVALID UCB OR I/O ERROR
*                                  *TRYING TO HALT RETURN CODE
RETCOD1C EQU   X'1C'               MODIFY OPTION INVALID BECAUSE
*                                  *ORIGINAL I/O REQUEST ISSUED
*                                  *BY OTHER THAN EXCP DRIVER
CPU0     EQU   X'80'               CPU 0 ID FOR SRB                 CTC
CPU1     EQU   X'40'               CPU 1 ID FOR SRB                 CTC
L0       EQU   0                   CONSTANT OF ZERO
L4       EQU   4                   CONSTANT OF FOUR
L8       EQU   8                   CONSTANT OF EIGHT
L12      EQU   12                  CONSTANT OF TWELVE
L20      EQU   20                  CONSTANT OF 20              @ZA06062
L229     EQU   229                 SUBPOOL 229                 @ZA09235
L24      EQU   24                  THREE BYTE SHIFT VALUE
L245     EQU   245                 SUBPOOL 253                 @ZA09235
CC7F     EQU   X'7F'               ECB COMPLETION CODE - OK
CC41     EQU   X'41'               ECB COMPLETION CODE - BAD
ALL      EQU   X'FF'               MASK FOR AND                @ZA06062
TWO      EQU   2                   CONSTANT 2                  @Y30CQLG
         EJECT
*****************************************************************
*                                                               *
*        INITIALIZATION PROCESSING                              *
*                                                               *
*****************************************************************
         SPACE 1
         BALR  BASEREG,0           ESTABLISH
         USING *,BASEREG           *BASE ADDRESSABILITY
         USING PSA,0
         SPACE 1
         LR    WORKD,UCBREG        ESTABLISH ASCB
         USING ASCB,WORKD          *ADDRESSABILITY
         SPACE 1
         USING RBBASIC,SVRBREG     INDICATE SVRB ADDRESSABILITY
         ST    RETREG,R14SAVE      SAVE SUPERVISOR RETURN
*                                  *ADDRESS
         STM   R0,R1,R0R1SAVE      SAVE REGISTER 0 AND 1
         SPACE 1
***                                                           ***
*        VALIDITY CHECK UCB                                     *
***                                                           ***
         SPACE 1
         LR    UCBREG,R1           INITIALIZE UCB              @YM01069
         N     UCBREG,HIGH2OFF     *REGISTER TO POINT
         LA    WORKA,UCBPRFX       *TO UCB
         SR    UCBREG,WORKA        *PREFIX SECTION
         BM    HALT0705            BRANCH IF NEGATIVE ADDRESS
         SPACE 1
         USING UCB,UCBREG          INDICATE UCB PREFIX
*                                  *ADDRESSABILITY
         SPACE 1
         CLI   UCBID,UCBSTND       TEST FOR VALID UCB
         BNE   HALT0705            BRANCH IF NOT VALID
         SPACE 1
***                                                           ***
*        CHECK FOR ACCEPTABLE DEVICE FOR HALT PROCESSING        *
***                                                           ***
         SPACE 1
         CLI   UCBTBYT3,UCB3COMM   TEST FOR TP DEVICE TYPE
         BE    HALT0020            YES/BR-                          CTC
         CLI   UCBTBYT3,UCB3CTC    Q/CTC DEVICE TYPE                CTC
         BE    HALT0020            YES, BR                      SUP3270
         SPACE
*** SUPPORT FOR 3270 - 3277 DISPLAY DEVICE                      SUP3270
         LA    R1,UCB3DISP         CLASS FLAG FOR 3270,TBYT3    SUP3270
         SLL   R1,8                SHIFT OVER 1 BYTE            SUP3270
         LA    R1,UCB3211(R1)      ADD TYPE BYTE, TBYT4         SUP3270
         LH    WORKA,UCBTBYT3      CHECK FOR 3270-3277          SUP3270
         CLR   R1,WORKA                                         SUP3270
         SPACE
         BNE   HALT0720            BRANCH IF IT IS NOT
         SPACE 1
***                                                           ***
*        CHECK FOR ACTIVE CHANNEL PROGRAM                       *
***                                                           ***
         SPACE 1
HALT0020 DS    0H                                                   CTC
         SLR   WORKA,WORKA         INDICATE REQUEST WAS ACTIVE      CTC
         TM    UCBFLA,UCBACTV      TEST FOR CHANNEL PROGAM
*                                  *ACTIVE
         BO    HALT0050            YES/BR-                          CTC
         CLI   UCBTBYT3,UCB3CTC    Q/CTC DEVICE TYPE                CTC
         BNE   HALT0735            NO/BR-INACTIVE AND NOT CTC       CTC
***                                                           ***   CTC
*        INACTIVE CTC - TEST FOR SELECTOR CHANNEL               *   CTC
***                                                           ***   CTC
         SPACE 1
         USING CVT,CVTREG                                           CTC
         IC    WORKA,UCBCHA        CHANNEL ADDRESS                  CTC
         LA    WORKB,CHTEL         CHANNEL TABLE ENTRY LENGTH       CTC
         MR    WORKA,WORKA         ACTUALY MULTS WORKA*WORKB & @YM07225
*                                  PUTS RESULT IN WORKB INDEX  @YM07225
         L     WORKA,CVTIXAVL      IOCOM ADDRESS                    CTC
         L     WORKA,IOCCTBL-IOCOM(,WORKA)   GET ADDR CHAN TBL @OZ02362
         AR    WORKB,WORKA         ADD FOR CHT ENTRY                CTC
         USING CHT,WORKB                                            CTC
         CLI   CHTTYPE,CHTSEL      TEST FOR SELECTOR CHANNEL        CTC
         BE    HALT2000            YES/BR-CTC SCHEDULE SUBROUTINE   CTC
         DROP  CVTREG,WORKB                                         CTC
HALT0050 DS    0H                                                   CTC
***                                                           ***
*        OBTAIN A REGISTER SAVE AREA                            *
***                                                           ***
         SPACE 1
         L     R0,SAVELEN          LOAD AMOUNT OF CORE TO
*                                  *GETMAIN AND ITS SUBPOOL
         SPACE 1
         GETMAIN R,LV=(0)          GETMAIN SAVE AREA
         SPACE 1
         LR    WORKB,R1            SAVE GETMAINED SAVE AREA    @OZ02116
*                                  *ADDRESS
         SPACE 1
***                                                           ***
*        FIX ALL PAGES OF THIS MODULE                           *
***                                                           ***
         SPACE 1
         SR    R11,R11             CLEAR WORK REGISTER
         ST    R11,ECB             SET ECB FOR PGFIX TO 0
         LA    R11,ECB             LOAD POINTER TO ECB
         SPACE 1
         PGFIX R,                  FIX IGC0003C PAGES                  *
               A=IGC0003C,         *MODULE START ADDRESS               *
               EA=HALT9999,        *MODULE END ADDRESS + 1             *
               ECB=(11)            *ECB ADDRESS
         SPACE 1
         LTR   R15,R15             CHECK FOR 0 RC              @ZA07162
         BZ    HALT0138            IF RC=0, NO WAIT            @ZA07162
         WAIT  ECB=ECB             WAIT FOR PAGEFIX COMPLETION
         SPACE 1
***                                                           ***
*        OBTAIN UCB LOCK  AND SET AN FRR                        *
***                                                           ***
         SPACE 1
HALT0138 LA    R11,UCBLOCK         POINT AT UCB LOCK           @OZ00762
         SPACE 1
HALT0140 EQU   *
         SETLOCK OBTAIN,           OBTAIN THE UCB LOCK                 *
               TYPE=IOSUCB,        *UCB LOCK                           *
               ADDR=(11),          *POINTER TO LOCK                    *
               MODE=UNCOND,        *SPIN TILL AQUIRED                  *
               RELATED=(UCB,IGC0003C(HALT0884))
         SPACE 1
         LR    SAVREG,WORKB        PLACE GETMAINED AREA ADDR.  @OZ02116
*                                  *IN SAVE AREA REGISTER
         SPACE 1
         SETFRR A,                 SET UP FRR FOR RECOVERY             *
               FRRAD=FRRADCON,     *ADDRESS OF FRR ROUTINE             *
               PARMAD=(FRRREG),    *REGISTER FOR FRR AREA              *
               WRKREGS=(WORKB,WORKC) *WORK REGISTERS
         SPACE 1
***                                                           ***
*        INITIALIZE FRR PARAMETER AREA                          *
***                                                           ***
         SPACE 1
         USING FRRPARM,FRRREG      INDICATE FRR PARAMETER
*                                  *AREA ADDRESSABILITY
         SPACE 1
         ST    UCBREG,FRRUCBA      STORE UCB ADDRESS IN FRR
*                                  *PARAMETER AREA
         MVC   FRRREG0(L8),R0R1SAVE SAVE INPUT PARAMETER            CTC
*                                  *REGISTERS 0 AND 1 IN
*                                  *FRR PARAMETER AREA
         OI    FRRFLAG,FRRUCBLK    INDICATE UCBLOCK HELD       @ZA06062
         SPACE 1
***                                                           ***
*        INSURE DEVICE IS STILL ACTIVE OR CTC DEVICE            *
***                                                           ***
         SPACE 1
         TM    UCBFLA,UCBACTV      TEST FOR CHANNEL PROGAM
*                                  *ACTIVE
         BO    HALT0150            YES/BR-IOSB ASSOCIATION          CTC
         CLI   UCBTBYT3,UCB3CTC    Q/IS IT A CTC DEVICE             CTC
         BNE   HALT0730            BR-INACTIVE AND NOT CTC          CTC
         LTR   WORKA,WORKA         Q/WAS UCBACTV AT FIRST TEST      CTC
         BZ    HALT0730            YES/BR-REQUEST COMPLETED         CTC
         SLR   IOSBREG,IOSBREG     INDICATE NO IOSB            @OZ02363
         B     HALT0300            BR-TO HALT INACTIVE CTC          CTC
         SPACE 1
***                                                           ***
*        DETERMINE IF AN IOSB IS ASSOCIATED WITH THE UCB        *
***                                                           ***
         SPACE 1
HALT0150 DS    0H                                                   CTC
         TM    UCBFLA,UCBPST       TEST IF A NORMAL IOQ/IOSB
*                                  *IS ASSOCIATED WITH THIS UCB
         BO    HALT0180            BRANCH IF THERE IS
         SPACE 1
         TM    UCBFLB,UCBSPST      TEST IF A SENSE IOQ IS
*                                  *ASSOCIATED WITH THIS UCB
         BNO   HALT0730            BRANCH IF THERE IS NOT
         SPACE 1
         TM    UCBFLC,UCBWAA       TEST IF AN IOSB IS POINTED
*                                  *TO BY THE IOQ
         BO    HALT0730            BRANCH IF THERE IS NOT
         SPACE 1
***                                                           ***
*        LOCATE IOSB                                            *
***                                                           ***
         SPACE 1
HALT0180 EQU   *
         L     WORKC,UCBIOQ        ESTABLISH IOQ
         USING IOQ,WORKC           *ADDRESSABILITY
         SPACE 1
         ST    WORKC,FRRIOQA       STORE IOQ ADDRESS IN FRR
*                                  *PARAMETER AREA
         SPACE 1
         L     IOSBREG,IOQIOSB     ESTABLISH IOSB
         USING IOSB,IOSBREG        *ADDRESSABILITY
         SPACE 1
***                                                           ***
*        DETERMINE WHETHER INVOKER OF SVC33 IS AUTHORIZED       *
*        TO HALT THIS DEVICE                                    *
***                                                           ***
         SPACE 1
         CLC   ASCBASID,IOSASID    TEST IF THE INVOKER OF SVC33
*                                  *IS IN THE SAME MEMORY AS THE
*                                  *MEMORY TO BE USED FOR I/O
*                                  *INTERRUPT PROCESSING
         BNE   HALT0702            BRANCH IF IT IS NOT         @ZA06062
         SPACE 1
***                                                           ***
*        DETERMINE WHICH OPTION HAS BEEN SELECTED               *
***                                                           ***
         SPACE 1
         LM    R0,R1,R0R1SAVE      RESTORE ENTRY PARAMETERS         CTC
         SLL   R1,ONEBYTE          TEST IF MODIFY
         LTR   R1,R1               *OPTION
         BNM   HALT0300            BRANCH IF IT IS NOT
         EJECT
*****************************************************************
*                                                               *
*        MODIFY OPTION PROCESSING                               *
*                                                               *
*****************************************************************
         SPACE 1
***                                                           ***
*        VALIDITY CHECK IOS DRIVER                              *
***                                                           ***
         SPACE 1
         SR    WORKA,WORKA         CLEAR WORK REGISTER
         IC    WORKA,IOSDVRID      INSERT DRIVER ID
         SPACE 1
         CLI   IOSDVRID,IOSXCPID   TEST IF EXCP DROVE IOS
         BNE   HALT0740            BRANCH IF IT DID NOT
         SPACE 1
***                                                           ***
*        SET UP TO LINK TO DRIVER TO COMPLETE PROCESSING        *
***                                                           ***
         SPACE 1
         LA    WORKB,VOIEL         CALCULATE OFFSET TO DRIVER
         MR    WORKA,WORKA         *VOID ENTRY
         SPACE 1
         USING CVT,CVTREG          INDICATE CVT ADDRESSABILITY
         SPACE 1
         L     R15,CVTIXAVL        ESTABLISH IOCOM
         USING IOCOM,R15           *ADDRESSABILITY
         SPACE 1
         L     R15,IOCVOID         ESTABLISH ADDRESSABILITY
         LA    R15,0(WORKB,R15)    *TO DRIVER
         USING VOID,R15            *VOID ENTRY
         SPACE 1
         L     R15,VOIPRG          LOAD EXCP PURGE ROUTINE
*                                  *ADDRESS
         SPACE 1
***                                                           ***
*        LINK TO DRIVER MODIFY ROUTINE                          *
***                                                           ***
         SPACE 1
         BAL   RETREG,MODPURGE(R15) BRANCH AND LINK TO EXCP CCW
*                                  *MODIFY ROUTINE
         SPACE 1
         ST    R15,RCODSAVE        SAVE RETURN CODE            @ZA06062
         B     HALT0690            BRANCH TO EXIT LINKAGE
         EJECT
*****************************************************************
*                                                               *
*        HALT OPTION PROCESSING                                 *
*                                                               *
*****************************************************************
         SPACE 1
***                                                           ***
*        SET UP AND LINK TO IOS HIO ROUTINE                     *
***                                                           ***
         SPACE 1
HALT0300 EQU   *
         MVC   FRRUCBF,UCBSFLS     SAVE UCB STATUS FLAGS ACROSS
*                                  *HIO INTERFACE
         SPACE 1
         L     R15,CVTIXAVL        ESTABLISH IOCOM
         USING IOCOM,R15           *ADDRESSABILITY
         SPACE 1
         L     R15,IOCHIO          LOAD IOS HIO ROUTINE ADDRESS
         SPACE 1
         BALR  RETREG,R15          BRANCH AND LINK TO IOS HIO
*                                  *ROUTINE
         SPACE 1
         MVC   UCBSFLS,FRRUCBF     RESTORE UCB STATUS FLAGS
         SPACE 1
         SR    WORKB,WORKB         RESET UCBSFLS SAVE AREA IN
         STH   WORKB,FRRUCBF       *FRR PARAMETER AREA TO 0
         ST    R15,RCODSAVE        SAVE RETCODE                @ZA06062
         SPACE 1
         LTR   R15,R15             SAVE RETURN CODE IN WORK    @ZA06062
*                                  *REGISTER AND TEST IT FOR
*                                  *0
         BNZ   HALT0700            BRANCH IF IT IS NOT
         LTR   IOSBREG,IOSBREG     IS THERE AN IOSB ASSOCIATED @OZ02363
         BZ    HALT0690
         SPACE 1
         MVI   IOSCOD,IOSPRGC      SET PURGED REQUEST CODE IN
*                                  *IOSB
         NI    IOSFLA,X'FF'-IOSERR RESET ERROR ROUTINE IN
*                                  *CONTROL INDICATOR
         EJECT
*****************************************************************
*                                                               *
*        EXIT LINKAGE                                           *
*                                                               *
*****************************************************************
         SPACE 1
HALT0690 EQU   *                   FREE RESOURCES
         BAL   WORKC,HALT0880      LINK TO RELEASE FRR,
*                                  *UCB LOCK,
         BAL   WORKC,HALT0885      *UNFIX THIS MODULE + FREE SAVE   CTC
*  DELETED FOR PTM 3477                                        @YM3477P
         SPACE 1
         L     R15,RCODSAVE        SET RETURN CODE             @ZA06062
         SPACE 1
HALT0695 EQU   *                   RESTORE REGISTERS AND RETURN
         LM    R0,R1,R0R1SAVE      RESTORE REGISTERS 0 AND 1
         L     RETREG,R14SAVE      RESTORE SUPERVISOR RETURN
*                                  *ADDRESS
         BR    RETREG              RETURN TO SUPERVISOR
         SPACE 1
HALT0700 EQU   *                   FREE RESOURCES
         CH    R15,HLTRTRY         SHOULD HALT BE RETRIED      @OZ00762
         BNE   HALT0702            NO, CONTINUE                @OZ00762
         XR    R15,R15
         LA    WORKC,HALT0138      SET TO REGET UCBLOCK AFTER  @OZ00762
         B     HALT0880            ENABLE TO CLEAR CU BZY,RTRY @OZ00762
HALT0702 BAL   WORKC,HALT0880      LINK TO RELEASE FRR,        @OZ00762
*                                  *UCB LOCK,
         BAL   WORKC,HALT0885      *UNFIX THIS MODULE               CTC
*   DELETED FOR PTM3477                                        @YM3477P
         SPACE 1
HALT0705 EQU   *                   SET RETURN CODE
         LA    R15,RETCODC         SET INVALID UCB OR I/O ERROR
*                                  *RETURN CODE
         SPACE 1
         B     HALT0695            BRANCH TO EXIT LINKAGE
         SPACE 1
HALT0720 EQU   *                   SET RETURN CODE
         LA    R15,RETCOD8         SET INVALID DEVICE TYPE
*                                  *RETURN CODE
         SPACE 1
         B     HALT0695            BRANCH TO EXIT LINKAGE
         SPACE 1
HALT0730 EQU   *                   FREE RESOURCES
         BAL   WORKC,HALT0880      LINK TO RELEASE FRR,
*                                  *UCB LOCK,
         BAL   WORKC,HALT0885      *UNFIX THIS MODULE               CTC
*  DELETED FOR PTM 3477                                        @YM3477P
         SPACE 1
HALT0735 EQU   *                   SET RETURN CODE
         LA    R15,RETCOD4         SET NO CHANNEL PROGRAM ACTIVE
*                                  *RETURN CODE
         SPACE 1
         B     HALT0695            BRANCH TO EXIT LINKAGE
         SPACE 1
HALT0740 EQU   *                   FREE RESOURCES
         BAL   WORKC,HALT0880      LINK TO RELEASE FRR,
*                                  *UCB LOCK,
         BAL   WORKC,HALT0885      *UNFIX THIS MODULE + FREE SAVE   CTC
*  DELETED FOR PTM 3477                                        @YM3477P
         SPACE 1
         LA    R15,RETCOD1C        SET INVALID DRIVER ID
*                                  *RETURN CODE
         SPACE 1
         B     HALT0695            BRANCH TO EXIT LINKAGE
         EJECT
*****************************************************************
*                                                               *
*        TERMINATION SUBROUTINE                                 *
*                                                               *
*****************************************************************
         SPACE 1
***                                                           ***
*        RELEASE FRR                                            *
***                                                           ***
         SPACE 1
HALT0880 EQU   *
         SPACE 1
         SETFRR D,                 DELETE FRR                          *
               WRKREGS=(R11,WORKB) *WORK REGISTERS             @ZA06062
         SPACE 1
***                                                           ***
*        RELEASE UCB LOCK                                       *
***                                                           ***
         SPACE 1
         LR    WORKB,SAVREG        SAVE CONTENTS OF REGISTER
*                                  *13 ACROSS SETLOCK AND PGFREE
*                                  *INTERFACE
HALT0884 EQU   *
         LA    R11,UCBLOCK         LOAD ADDRESS OF UCB LOCK
         SPACE 1
         STM   FRRREG,RETREG,R12TO14  SAVE REGS                @ZA06062
         SETLOCK RELEASE,          RELEASE THE UCB LOCK                *
               TYPE=IOSUCB,        *UCB LOCK                           *
               ADDR=(11),          *POINTER TO LOCK                    *
               RELATED=(UCB,IGC0003C(HALT0140))
         LM    FRRREG,RETREG,R12TO14   RESTORE REGS            @ZA06062
         BR    WORKC               RETURN TO CALLER                 CTC
         SPACE 1
***                                                           ***
*        FREE (UNFIX) ALL PAGES OF THIS MODULE                  *
***                                                           ***
         SPACE 1
HALT0885 DS    0H                                                   CTC
         PGFREE R,                 FREE IGC0003C PAGES                 *
               A=IGC0003C,         *MODULE START ADDRESS               *
               EA=HALT9999         *MODULE END ADDRESS + 1
         SPACE 1
***                                                           ***
*        FREEMAIN SAVE AREA                                     *
***                                                           ***
         SPACE 1
HALT0890 EQU   *
         L     R0,SAVELEN          LOAD AMOUNT OF CORE TO FREE
*                                  *AND ITS SUBPOOL
         LR    R1,WORKB            LOAD ADDRESS OF CORE TO BE
*                                  *FREED
         SPACE 1
         FREEMAIN R,LV=(0),A=(1)   FREEMAIN SAVE AREA
         SPACE 1
         BR    WORKC               RETURN TO CALLER
         SPACE 1
         EJECT                                                      CTC
*****************************************************************   CTC
*                                                               *   CTC
*        SPECIAL CTC HALT ROUTINE - USED TO HALT CTC DEVICES ON *   CTC
*              A SELECTOR CHANNEL IF THE UCB IS NOT ACTIVE.     *   CTC
*           OPERATION;                                          *   CTC
*              FIX ALL PAGES OF SVC-33.                         *   CTC
*              GETMAIN AN SRB/SAVE/ECB AREA.                    *   CTC
*              INITIALIZE THE SRB AND SAVE THE REGISTERS.       *   CTC
*              SCHEDULE THE SUBROUTINE WITH CPU AFFINITY.       *   CTC
*              WAIT ON AN ECB FOR THE SUBROUTINE TO COMPLETE.   *   CTC
*              GET THE IECIHIO RETURN CODE FROM THE ECB;        *   CTC
*                 X'7F' = RETURN CODE ZERO                      *   CTC
*                 X'41' = RETURN CODE TWELVE                    *   CTC
*              FREE ALL PAGES OF THIS MODULE                    *   CTC
*              RETURN TO CALLER OF SVC-33.                      *   CTC
*        SPECIAL CTC SUBROUTINE;                                *   CTC
*           OPERATION;                                          *   CTC
*              TEST CHANNEL TO CLEAR THE SELECTOR CHANNEL.      *   CTC
*              OBTAIN THE UCBLOCK.                              *   CTC
*              SET AN FRR.                                      *   CTC
*              TEST CHANNEL TO ASSURE IT IS STILL AVAILABLE.    *   CTC
*              BALR TO THE IOS HALTIO SUBROUTINE(IECIHIO)       *   CTC
*              RELEASE THE FRR,AND THE UCBLOCK.                 *   CTC
*              GET THE LOCAL LOCK.                              *   CTC
*              POST THE ECB FOR SVC-33.                         *   CTC
*              RELEASE THE LOCAL LOCK.                          *   CTC
*              RETURN TO SVC-33.                                *   CTC
*                                                               *   CTC
*****************************************************************   CTC
HALT2000 DS    0H                                                   CTC
         XC    ECB,ECB                                              CTC
         LA    R0,ECB                                               CTC
***                                                           ***   CTC
*        FIX ALL PAGES OF THIS MODULE                           *   CTC
***                                                           ***   CTC
         PGFIX R,A=IGC0003C,EA=HALT9999,ECB=(R0)                    CTC
         WAIT  ECB=ECB                                              CTC
* SET UP FOR ESTAE TO PROTECT GETMAIN FOR SRB                  @ZA09235
         USING ESTAPARM,WORKD                                  @ZA09235
         LA    WORKD,ESTACOR         GET ADDR OF SVRBAREA FOR  @ZA09235
*                                  *ESTAE PARMLIST             @ZA09235
         MVC   ESTLIST(ESTALEN),ESTAPLST  PUT LIST FORM IN     @ZA09235
*                                  SVRB                        @ZA09235
         LA    R11,ESTADATA        SET UP FOR ESTAE            @ZA09235
         USING ESTADATA,R11                                    @ZA09235
         SR    WORKB,WORKB         CLEAR REG                   @ZA09235
         ST    WORKB,FREEADDR      INIT FREEADDR TO 0 FOR ESTAE@ZA09235
         LA    WORKA,HALTESTA      GET ADDR OF ESTAE ROUTINE   @ZA09235
         ESTAE (WORKA),PURGE=NONE,TERM=YES,PARAM=(R11),MF=(E,(WORKD))
         ST    UCBREG,EUCBADR      PUT UCB ADDR IN ESTAE AREA  @ZA09235
***                                                           ***   CTC
*        GETMAIN SPACE FOR SRB/SAVE/ECB                         *   CTC
***                                                           ***   CTC
         LA    R0,HALTCECB+L4-SRBSECT GETMAIN SIZE
         LA    R1,L245             SUBPOOL ID                  @ZA09235
         SLL   R1,L24                                               CTC
         OR    R0,R1                                                CTC
         ST    R0,FREELEN          PUT LEN IN ESTAE LIST       @ZA09235
         GETMAIN R,LV=(0)                                           CTC
         ST    R1,FREEADDR         PUT ADDR IN ESTAE LIST      @ZA09235
         SPACE 1                                                    CTC
* R1 HAS SRB/SAVE/ECB ADDRESS                                   *   CTC
         SPACE 1                                                    CTC
***                                                           ***   CTC
*        SRB INITIALIZATION                                     *   CTC
***                                                           ***   CTC
         USING SRBSECT,WORKA                                        CTC
         LR    WORKA,R1                                             CTC
         XC    L0(HALTCECB+L4-SRBSECT,WORKA),L0(WORKA) CLEAR CORE   CTC
         LA    R1,HALT3000         ADDRESS OF SCHEDULED SUBROUTINE  CTC
         ST    R1,SRBEP                                             CTC
         MVI   SRBCPAFF,CPU0       SET CPU ID = 0                   CTC
         CLI   UCBCPU,0            Q/CPU 0                          CTC
         BE    HALT2200            YES/BR-                          CTC
         MVI   SRBCPAFF,CPU1       SET CPU ID = 1                   CTC
HALT2200 DS    0H                                                   CTC
         MVI   SRBPRIOR,SRBPSYS    SET NO ENQUEUE PRIORITY IN SRB   CTC
         L     WORKD,PSAAOLD       GET ASCB ADDR               @ZA06062
         ST    WORKD,SRBASCB       ASCB ADDRESS                     CTC
         USING ASCB,WORKD                                      @ZA09235
         MVC   SRBPASID,ASCBASID   PURGEDQ ASID                     CTC
         MVC   SRBPTCB,PSATOLD-PSA PURGEDQ TCB                      CTC
         MVC   SRBID,SRBEBD        MOVE SRB EBIDIC INTO SRB    @OZ05656
         STM   R0,R15,HALTCSAV     SAVE ALL REGS FOR SCHEDULED RTN  CTC
***                                                           ***   CTC
*        SCHEDULE THE CTC HALT SUBROUTINE                       *   CTC
***                                                           ***   CTC
         LR    R1,WORKA            RESTORE SRB @ IN GR1        @OZ05966
         SCHEDULE SRB=(1),SCOPE=GLOBAL  GPR'S 0,1,14,15 LOST   @ZA06062
***                                                           ***   CTC
*        WAIT FOR THE CTC SUBROUTINE TO COMPLETE                *   CTC
***                                                           ***   CTC
         WAIT  ECB=HALTCECB                                     *   CTC
***                                                           ***   CTC
*        GET COMPLETION CODE FROM SUBROUTINE                    *   CTC
***                                                           ***   CTC
         SR    WORKD,WORKD                                          CTC
         CLI   HALTCECB,CC7F       TEST FOR GOOD RETURN             CTC
         BE    HALT2400            BRANCH YES                       CTC
         LA    WORKD,L12           SET RETURN CODE             @ZA06062
***                                                           ***   CTC
*        FREE THE SRB/SAVE/ECB AREA                             *   CTC
***                                                           ***   CTC
HALT2400 DS    0H                                                   CTC
         LA    R0,HALTCECB+L4-SRBSECT LENGTH TO FREE                CTC
         LA    R1,L245             SUBPOOL ID                  @ZA09235
         SLL   R1,L24                                               CTC
         OR    R0,R1                                                CTC
         FREEMAIN R,LV=(0),A=(WORKA)                                CTC
         SR    R1,R1               CLEAR REG                   @ZA09235
         ST    R1,FREEADDR         ZERO FREE ADDR              @ZA09235
         ESTAE 0                   DELETE ESTAE                @ZA09235
***                                                           ***   CTC
*        FREE ALL PAGES OF THIS MODULE                          *   CTC
***                                                           ***   CTC
         PGFREE R,                                                  CTCX
               A=IGC0003C,         MODULE START ADDRESS             CTCX
               EA=HALT9999         MODULE ENDING ADDRESS+1          CTC
         LR    R15,WORKD           SET RETURN CODE                  CTC
         B     HALT0695            BR-RETURN                        CTC
         EJECT                                                      CTC
*****************************************************************   CTC
*                                                               *   CTC
*        SCHEDULED CTC HALT SUBROUTINE                          *   CTC
*                                                               *   CTC
*****************************************************************   CTC
         SPACE 2                                                    CTC
HALT3000 DS    0H                                                   CTC
         LR    WORKA,R0                                             CTC
         LM    R1,SAVREG,HALTCSAV+L4  SET SVC-33 REGISTERS          CTC
         DROP  WORKA                                                CTC
         USING SRBSECT,WORKD                                        CTC
         LR    WORKD,R0                                             CTC
         LA    SAVREG,HALTCSAV     SETUP SAVE AREA PTR              CTC
         ST    RETREG,SRBSAVE      SAVE RETURN ADDRESS              CTC
         SPACE 2                                                    CTC
***                                                           ***   CTC
*        ESTABLISH AN FRR                                       *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
         SETFRR A,                                                  CTCX
               FRRAD=FRRADCON,                                      CTCX
               PARMAD=(FRRREG),                                     CTCX
               WRKREGS=(WORKB,WORKC)  VOLATILE WORK REGS            CTC
         SPACE 2                                                    CTC
***                                                           ***   CTC
*        INITIALIZE FRR PARAMETER AREA                          *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
         USING FRRPARM,FRRREG                                       CTC
         ST    UCBREG,FRRUCBA                                       CTC
         MVC   FRRREG0(L8),R0R1SAVE                                 CTC
***                                                           ***   CTC
*        CLEAR THE SELECTOR CHANNEL                             *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
HALT3100 DS    0H                                                   CTC
         L     SVRBREG,HALTCSAV+L20  RELOAD SVRB PTR           @ZA06062
         LH    WORKB,UCBCHAN       GET CHANNEL ADDRESS              CTC
         SPACE
         BAL   RETREG,CRHHOOK1     CHECK IF CRH ACTIVE, FOR TCH@Y30CQLG
*                                  IF CRH ACTIVE RETURNS+4
         SPACE
         TCH   0(WORKB)                                             CTC
         BC    4+2,HALT3100        BUSY/INT PEND-LOOP UNTIL AVAIL   CTC
         SPACE 2                                                    CTC
         BC    1,HALT3300          NOT OPER-BR TO ERROR             CTC
         SPACE 2                                                    CTC
***                                                           ***   CTC
*        GET UCBLOCK                                            *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
         LA    R11,UCBLOCK                                          CTC
         SETLOCK OBTAIN,                                            CTCX
               TYPE=IOSUCB,        UCB LOCK                         CTCX
               ADDR=(11),          PTR TO LOCK                      CTCX
               MODE=UNCOND,        SPIN UNTIL GOTTEN                CTCX
               REGS=USE,                                       @ZA06062X
               RELATED=(UCB,IGC0003C(HALT0884))                     CTC
         SPACE 2                                                    CTC
         OI    FRRFLAG,FRRUCBLK    INDICATE UCBLOCK HELD       @ZA06062
         SPACE 2                                                    CTC
***                                                           ***   CTC
* TEST TO SEE IF THE CHANNEL IS STILL CLEAR                     *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
         LH    WORKB,UCBCHAN       CHANNEL ADDRESS                  CTC
         SPACE
         BAL   RETREG,CRHHOOK2                                 @Y30CQLG
         SPACE
         TCH   0(WORKB)                                             CTC
         BC    8,HALT3200          BR-CHANNEL IS AVAIL              CTC
         LA    WORKB,L12           SET ERROR RETURN CODE       @ZA11353
         BC    1,HALT3250          BR-NOT OPERATIONAL               CTC
         BAL   WORKC,HALT0884      DELETE FRR & FREE UCB LOCK       CTC
         NI    FRRFLAG,ALL-FRRUCBLK CLEAR LOCK HELD INDICATOR  @ZA06062
         B     HALT3100            BR-TRY AGAIN                     CTC
         SPACE 2                                                    CTC
***                                                           ***   CTC
*        LINK TO IOS HALT ROUTINE (IECIHIO)                     *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
HALT3200 DS    0H                                                   CTC
         SR    IOSBREG,IOSBREG     SETUP R2 FOR HALT           @ZA06062
         MVC   FRRUCBF,UCBSFLS     SAVE UCB STATUS FLAGS            CTC
         L     R15,CVTIXAVL        IOCOM ADDRESS                    CTC
         L     R15,IOCHIO          PTR TO IECIHIO                   CTC
         BALR  RETREG,R15          BALR TO HALTIO SUBROUTINE        CTC
         LA    WORKB,L0(R15)       SAVE THE RETURN CODE        @ZA11353
         MVC   UCBSFLS,FRRUCBF     RESTORE UCB FLAGS                CTC
         XC    FRRUCBF,FRRUCBF     CLEAR FLAG SAVE AREA             CTC
         SPACE 2                                                    CTC
***                                                           ***   CTC
*        RELEASE UCBLOCK                                        *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
HALT3250 BAL   WORKC,HALT0884        FREE UCB LOCK                  CTC
         NI    FRRFLAG,ALL-FRRUCBLK  CLEAR LOCK HELD INDICATOR @ZA06062
         SPACE 2                                                    CTC
***                                                           ***   CTC
*        POST INTERFACE                                         *   CTC
*              GET LOCAL LOCK                                   *   CTC
*              BRANCH ENTER POST                                *   CTC
*              FREE THE LOCAL LOCK                              *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
HALT3300 SETLOCK OBTAIN,                                            CTCX
               TYPE=LOCAL,                                          CTCX
               MODE=UNCOND,                                         CTCX
               REGS=USE,                                            CTCX
               RELATED=(POST,IGC0003C(HALT3500))                    CTC
         OI    FRRFLAG,FRRLOCLK    INDICATE LOCAL LOCK HELD    @ZA06062
         SPACE
         BAL   RETREG,CRHHOOK3     IF DIAG ACTIVE, DEDIAGNOSE  @Y30CQLG
         SPACE
         LA    WORKC,CC7F          POST INTFC-RC=0                  CTC
         SLL   WORKC,L24           GET COMP.CODE IN HIGH ORD @OZ05697
         LTR   WORKB,WORKB         POST INTFC-RC=0             @ZA11353
         BZ    HALT3400            POST INTFC-                      CTC
         LA    WORKC,CC41          POST INTFC-                      CTC
         SLL   WORKC,L24          GET COMP.CODE IN HIGH ORD BYT@OZ05697
HALT3400 DS    0H                  POST INTFC-                      CTC
         LA    R11,HALTCECB        POST INTFC-ECB ADDRESS           CTC
         L     R15,CVT0PT02        POST INTFC-ENTRY POINT           CTC
         BALR  RETREG,R15          POST INTFC-ENTER POST            CTC
HALT3500 DS    0H                                                   CTC
         SETLOCK RELEASE,                                           CTCX
               TYPE=LOCAL,                                          CTCX
               REGS=USE,                                            CTCX
               RELATED=(POST,IGC0003C(HALT3300))                    CTC
         NI    FRRFLAG,ALL-FRRLOCLK  CLR LOCK HELD INDICATOR   @ZA06062
         SPACE 2                                                    CTC
***                                                           ***   CTC
* RETURN TO SVC-33                                              *   CTC
***                                                           ***   CTC
         SPACE 2                                                    CTC
         SETFRR D,WRKREGS=(R11,WORKB)  DELETE FRR              @ZA06062
         L     RETREG,SRBSAVE      GET RETURN ADDRESS               CTC
         BR    RETREG              RETURN TO CALLER                 CTC
         DROP  WORKD                                                CTC
         EJECT
         TITLE 'IGC0003C - CRHHOOK1 ROUTINE'
************************************************************   @Y30CQLG
*                                                              @Y30CQLG
*        CHANNEL RECONFIGURATION HARDWARE HOOK 1 ROUTINE       @Y30CQLG
*                 IF CRH ACTIVE, ADJUST FOR CRH CHANNEL        @Y30CQLG
*                                                              @Y30CQLG
************************************************************   @Y30CQLG
         SPACE
         USING CRCA,WORKC                                      @Y30CQLG
         SPACE
CRHHOOK1 L     WORKC,CVTCRCA       CHECK FOR ADDR TO CRCA      @Y30CQLG
         LTR   WORKC,WORKC         IS CRH ACTIVE ?             @Y30CQLG
         BZR   RETREG              NOT ACTIVE, RETURN          @Y30CQLG
         SPACE
         TM    CRCAFLG1,CRCADIAG   IS DIAG ON                  @Y30CQLG
         BOR   RETREG              DONT DO IT AGAIN            @Y30CQLG
         SPACE
*        IF THERE IS NOT A PATH FROM THIS CPU, NEED TO DIAG ETC@Y30CQLG
         IOSGEN  MAP,TABLE=PTH,UCB=(UCBREG),LINKR=(R11),VAR=1  @Y30CQLG
         LH    R15,PSACPUSA        GET CPU ADDR                @Y30CQLG
         LTR   R15,R15             AM I ON CPU 00 ?            @Y30CQLG
         BNZ   CRHLBL01            NO, THEN ON CPU1            @Y30CQLG
         SPACE
         IC    R15,CPU0PRIM        ON CPU0, GET PRIM PATH MSK  @Y30CQLG
         SLL   R15,8               MAKE ROOM FOR SECONDARY     @Y30CQLG
         IC    R15,CPU0SEC         GET SECONDARY PATH MSK      @Y30CQLG
         LTR   R15,R15             IS THER A PATH FOR THIS CPU @Y30CQLG
         BNZR  RETREG              THERE IS A PATH, RETURN     @Y30CQLG
         B     CRHLBL10            NO PATH, GO DIAG ETC FOR CRH@Y30CQLG
CRHLBL01 XR    R15,R15             CLEAR R15, CHECK CPU 1      @Y30CQLG
         IC    R15,CPU1PRIM        GET PRIM PATH MASK, CPU 1   @Y30CQLG
         SLL   R15,8               SHIFT 1 BYTE                @Y30CQLG
         IC    R15,CPU1SEC         GET SECONDARY PATH MSK      @Y30CQLG
         LTR   R15,R15             IS THERE A PATH, THIS CPU   @Y30CQLG
         BNZR  RETREG              THRE IS A PATH, RETURN      @Y30CQLG
         SPACE
CRHLBL10 SETFRR A,FRRAD=FRRADCON,PARMAD=(FRRREG),              @Y30CQLGX
               WRKREGS=(WORKA,R11)  SET THE FRR FOR UCBLCK     @Y30CQLG
         SPACE
         USING FRRPARM,FRRREG
         ST    UCBREG,FRRUCBA      STORE UCB ADDR IN FRR PARM  @Y30CQLG
         MVC   FRRREG0(L8),R0R1SAVE     SAVE INPUT REGS 0 & 1  @Y30CQLG
         SPACE
         LA    R11,UCBLOCK         GET UCBLOCK SO CAN DIAG ETC @Y30CQLG
         SETLOCK OBTAIN,TYPE=IOSUCB,ADDR=(11),MODE=UNCOND,             X
               RELATED=(UCB,IGC0003C(HALT0884))                @Y30CQLG
         OI    FRRFLAG,FRRUCBLK    INDICATE UCBLOCK HELD       @ZA06062
         STCM  WORKB,TWO,CRCAMCWF  IND CHAN ADDR TO MCW        @Y30CQLG
         OI    UCBFLB,UCBIORST     IND PATH FROM INOP CPU      @Y30CQLG
         OI    CRCAMCWF,CRCAMCWI+CRCAMCWM  TURN ON BITS 26 & 27@Y30CQLG
         STH   WORKB,CRCACHAN      STORE CHANNEL & UNIT FOR CRH@Y30CQLG
         ICM   WORKB,TWO,CHAN6     INDICATE CHAN 6 FOR CRH     @Y30CQLG
         OI    CRCAFLG1,CRCADIAG   INDICATE OUTSTANDING DIAG   @Y30CQLG
         SPACE
         DC    X'8300'             DIAGNOSE INSTR              @Y30CQLG
         DC    S(CRCAMCW)          ADDR OF MCW                 @Y30CQLG
         SPACE
         SPACE
         SETLOCK RELEASE,TYPE=IOSUCB,ADDR=(11),   RELEASE      @Y30CQLGX
               RELATED=(UCB,IGC0003C(HALT0884))                @Y30CQLG
         NI     FRRFLAG,ALL-FRRUCBLK  CLR LOCK HELD INDICATOR  @ZA06062
         SPACE
         SETFRR D,WRKREGS=(WORKA,R11)                          @Y30CQLG
         SPACE
         BR    RETREG              RETURN TO MAINLINE          @Y30CQLG
         SPACE
         DROP  WORKC
         EJECT
         TITLE 'IGC0003C - CRHHOOK2 ROUTINE'
*************************************************************  @Y30CQLG
*                                                              @Y30CQLG
*        CRH HOOK ROUTINE2                                     @Y30CQLG
*              IF CRH IS ACTIVE, DO TCH, DEDIAG & RETURN       @Y30CQLG
*              WITH PROPER C.C.                                @Y30CQLG
*                                                              @Y30CQLG
*************************************************************  @Y30CQLG
         SPACE
         USING CRCA,WORKC
         SPACE
CRHHOOK2 L     WORKC,CVTCRCA       CHECK FOR ADDR TO CRCA      @Y30CQLG
         LTR   WORKC,WORKC         IS CRH ACTIVE ?             @Y30CQLG
         BZR   RETREG              NO, RETURN                  @Y30CQLG
         SPACE
         TM    UCBFLB,UCBIORST     IS THE DEVICE ON THIS CPU ? @Y30CQLG
         BZR   RETREG              YES, RETURN                 @Y30CQLG
         SPACE
         TM    CRCAFLG1,CRCADIAG   MAKE SURE DIAG OUTSTANDING  @Y30CQLG
         BO    HOOK201             YES DON'T DIAG AGAIN        @Y30CQLG
         SPACE
         STH   WORKB,CRCACHAN      CHAN & UNIT ADDR FOR CRH    @Y30CQLG
         STCM  WORKB,TWO,CRCAMCWF  INDICATE CHAN ADDR TO MCW   @Y30CQLG
         OI    CRCAMCWF,CRCAMCWI+CRCAMCWM    TURN ON 26 & 27   @Y30CQLG
         OI    CRCAFLG1,CRCADIAG   INDICATE OUTSTANDING DIAG   @Y30CQLG
         DC    X'8300'             DIAG OP CODE                @Y30CQLG
         DC    S(CRCAMCW)          ADDR OF MCW                 @Y30CQLG
         SPACE
HOOK201  ICM   WORKB,TWO,CHAN6     INDICATE CHAN 6, CRH        @Y30CQLG
         TCH   0(WORKB)            DO REAL TCH TO CHAN6, CRH   @Y30CQLG
         BALR  R15,0               SAVE RTN CODE               @Y30CQLG
         XI    CRCAMCWF,CRCAMCWM   TURN OFF BIT 27 TO DEDIAG   @Y30CQLG
         SPACE
         DC    X'8300'             DEDIAGNOSE TURN OFF CRH     @Y30CQLG
         DC    S(CRCAMCW)          ADDR OF MCW                 @Y30CQLG
         SPACE
         XI    CRCAFLG1,CRCADIAG   INDICATE DIAG NOT OUTSTANDNG@Y30CQLG
         LH    WORKB,UCBCHAN       RELOAD WORKB WITH ORIG CHAN @Y30CQLG
         SPM   R15                 RESET C. C.                 @Y30CQLG
         B     4(RETREG)           RETURN NEXT INSTR AFTER TCH @Y30CQLG
         SPACE
         DROP  WORKC
         DROP  BASEREG
         EJECT
         TITLE 'CRH HOOK ROUTINE 3'
*************************************************************  @Y30CQLG
*                                                              @Y30CQLG
*        CRH HOOK ROUTINE 3                                    @Y30CQLG
*              IF CRH ACTIVE, DEDIAG                           @Y30CQLG
*                                                              @Y30CQLG
*************************************************************  @Y30CQLG
         SPACE
         USING CRCA,WORKC                                      @Y30CQLG
         SPACE
CRHHOOK3 L     WORKC,CVTCRCA       CHECK FOR ADDR TO CRCA      @Y30CQLG
         LTR   WORKC,WORKC         IS CRH ACTIVE               @Y30CQLG
         BZR   RETREG              NO, RETURN                  @Y30CQLG
         SPACE
         TM    CRCAFLG1,CRCADIAG   IS DIAG ON                  @Y30CQLG
         BZR   RETREG              NO, RETURN                  @Y30CQLG
         SPACE
         XI    CRCAMCWF,CRCAMCWM   TURN OFF BIT 27             @Y30CQLG
         SPACE
         DC    X'8300'             DEDIAGNOSE TURN OFF CRH     @Y30CQLG
         DC    S(CRCAMCW)          ADDR OF MCW                 @Y30CQLG
         SPACE
         XI    CRCAFLG1,CRCADIAG   INDICATE NO DIAG            @Y30CQLG
         SPACE
         BR    RETREG              RETURN                      @Y30CQLG
         EJECT
         TITLE 'HALT FRR'
*****************************************************************
*                                                               *
*        FRR ROUTINE                                            *
*                                                               *
*****************************************************************
         SPACE 1
HALT0900 EQU   *
         BALR  BASEREG,0           RE-ESTABLISH BASE
         USING *,BASEREG           *ADDRESSABILITY
         USING CVT,CVTREG                                      @ZM30451
         L     CVTREG,FLCCVT       LOAD CVT PTR                @ZM30451
         SPACE 1
         LR    WORKA,RETREG        SAVE RTM RETURN ADDRESS
         SPACE
         BAL   RETREG,CRHFRR       CHECK FOR CRH ACTIVE        @Y30CQLG
         SPACE
         SPACE 1
         USING SDWA,R1             INDICATE SDWA ADDRESSABILITY
         SPACE 1
         L     FRRREG,SDWAPARM     ESTABLISH FRR PARAMETER AREA
*                                  *ADDRESSABILITY
         SPACE 1
***                                                           ***
*        RESTORE UCBFLS FIELD IF SAVED                          *
***                                                           ***
         SPACE 1
         L     UCBREG,FRRUCBA      ESTABLISH UCB ADDRESSABILITY
         SPACE 1
         LH    WORKB,FRRUCBF       LOAD CONTENTS OF UCBSFLS SAVE
*                                  *AREA IN FRR PARAMETER AREA
         SPACE 1
         LTR   WORKB,WORKB         TEST FOR ANYTHING TO RESTORE YM01120
         BZ    HALT0920            BRANCH IF NOTHING
         SPACE 1
         STH   WORKB,UCBSFLS       RESTORE UCBSFLS FIELD
         SPACE 1
***                                                           ***
*        DETERMINE IF THIS FRR HAS BEEN PERCOLATED TO           *
***                                                           ***
         SPACE 1
HALT0920 EQU   *
         TM    SDWAERRC,SDWAPERC   TEST IF PERCOLATION IS FLAGGED
         BO    HALT0980            BRANCH IF IT IS INDICATED
         SPACE 1
***                                                           ***
*        FILL IN VARIABLE RECORDING DATA AREA OF SDWA           *
***                                                           ***
         SPACE 1
         LA    WORKB,SDWAVRA       ESTABLISH ADDRESSABILITY TO
         USING VRECORD,WORKB       *SDWA VARIABLE RECORDING DATA
*                                  *AREA
         SPACE 1
         MVC   VRECREG0,FRRREG0    MOVE SVC 33 INPUT REGISTERS 0
         MVC   VRECREG1,FRRREG1    *AND 1 TO RECORDING AREA
         SPACE 1
         L     WORKC,FRRIOQA       ESTABLISH IOQ ADDRESSABILITY
         USING IOQ,WORKC
         LTR   WORKC,WORKC         Q/ANY IOQ                        CTC
         BZ    HALT0940            NO/BR-SKIP RECORDING IOQ         CTC
         SPACE 1
         MVC   VRECIOQ(IOQL),IOQ   MOVE IOQ TO RECORDING AREA
         SPACE 1
HALT0940 DS    0H                                                   CTC
         MVC   VRECUCBP(UCBCURPX),UCBPXST   MOVE UCB PREFIX AND
         MVC   VRECUCBB(UCBDEV-UCBOB),UCBOB *BASE TO RECORDING
*                                            *AREA
         SPACE 1
         MVI   SDWAURAL,VRECLEN    INDICATE LENGTH OF VARIABLE
*                                  *RECORDING AREA USED
         SPACE 1
***                                                           ***
*        ISSUE SETRP TO INDICATE PERCOLATION                    *
*                                RECORDING                      *
*                                DUMPING                        *
***                                                           ***
         SPACE 1                                               @ZA06062
         SETRP RC=0,               INDICATE PERCOLATION                *
               RECORD=YES,         *RECORDING ON                       *
               RECPARM=RECPARM,    *LOGREC                             *
               DUMP=YES,           *TAILORED                           *
               DUMPOPT=DUMPOPT     *DUMP
         SPACE 1
         SPACE 1
***                                                           ***
*        ISSUE SETRP TO INDICATE PERCOLATION                    *
*                                FREEING OF UCB LOCK            *
***                                                           ***
         SPACE 1
HALT0980 EQU   *
         TM    FRRFLAG,FRRUCBLK  IS UCBLOCK HELD?              @ZA06062
         BZ    HALT0945          NO DON'T FREE                 @ZA06062
         SETRP RC=0,                                           @ZA06062*
               FRELOCK=IOSUCB(UCBLOCK)                         @ZA06062
HALT0945 TM    FRRFLAG,FRRLOCLK  IS LOCAL LOCK HELD?           @ZA06062
         BZ    HALT0990          NO DON'T FREE                 @ZA06062
         SETRP FRELOCK=LOCAL                                   @ZA06062
         SPACE 1
***                                                           ***
*        RETURN TO RTM                                          *
***                                                           ***
         SPACE 1
HALT0990 EQU   *
         LR    RETREG,WORKA        RESTORE RETURN ADDRESS
         BR    RETREG              RETURN TO RTM
         EJECT
         TITLE 'CRH HOOK FOR FRR'
***************************************************************@Y30CQLG
*
*        CRH HOOK FOR FRR
*              IF CRH ACTIVE, DEDIAGNOSE
*
***************************************************************@Y30CQLG
         SPACE
         USING CRCA,WORKC                                      @Y30CQLG
         SPACE
CRHFRR   L     WORKC,CVTCRCA       CHECK FOR ADDR TO CRCA      @Y30CQLG
         LTR   WORKC,WORKC         IS CRH ACTIVE ?             @Y30CQLG
         BZR   RETREG              NO, RETURN                  @Y30CQLG
         SPACE
         TM    CRCAFLG1,CRCADIAG   IS THERE AN OUTSTANDING DIAG@Y30CQLG
         BZR   RETREG              NO DIAG, RETURN             @Y30CQLG
         SPACE
         XI    CRCAMCWF,CRCAMCWM   TURN OFF BIT 27 TO DEDIAG   @Y30CQLG
         SPACE
         DC    X'8300'             DEDIAGNOSE TURN OFF CRH     @Y30CQLG
         DC    S(CRCAMCW)          ADDR OF MCW                 @Y30CQLG
         SPACE
         XI    CRCAFLG1,CRCADIAG   INDICATE NO DIAG            @Y30CQLG
         SPACE
         BR    RETREG              RETURN TO FRR               @Y30CQLG
         EJECT
*****************************************************************
*                                                               *
*        ESTAE ROUTINE                                          *
*                                                               *
***************************************************************@ZA09235
HALTESTA BALR  FRRREG,0            ESTABLISH ADDRESSABILITY    @ZA09235
         USING *,FRRREG                                        @ZA09235
         DROP  R11                                             @ZA09235
         LR    SAVREG,RETREG       SAVE RET ADDR               @ZA09235
         LR    WORKA,R2            GET PARM LIST ADDR          @ZA09235
         SR    WORKC,WORKC         CLEAR REG                   @ZA09235
         LA    R11,L12                                         @ZA09235
         CR    R11,R0              IS THERE AN SDWA            @ZA09235
         BE    HALTES01            NO-GO ON                    @ZA09235
         L     WORKA,L0(R1)        GET PARM LIST ADDR          @ZA09235
         LR    WORKC,R1            GET SDWA ADDRESSABILITY     @ZA09235
         USING ESTADATA,WORKA                                  @ZA09235
HALTES01 L     WORKB,FREEADDR      GET FREEMAIN ADDR           @ZA09235
         LTR   WORKB,WORKB         IS IT ZERO?                 @ZA09235
         BZ    HALTES02            YES DON'T DO FREEMAIN       @ZA09235
         L     R0,FREELEN          GET LEN AND SP FOR FREEMAIN @ZA09235
         FREEMAIN R,LV=(0),A=(WORKB)                           @ZA09235
         SPACE 1                                               @ZA09235
HALTES02 LTR   WORKC,WORKC         IS THERE AN SDWA            @ZA09235
         BZ    HALTES04            NO - EXIT                   @ZA09235
         USING SDWA,WORKC                                      @ZA09235
         TM    SDWAERRC,SDWAPERC   IS IT PERCOLATION           @ZA09235
         BO    HALTES03            YES - EXIT                  @ZA09235
         L     WORKB,SDWAVRA       GET ADDR OF VAR REC AREA    @ZA09235
         USING ESTAREC,WORKB
         MVC   EFREEA(L8),FREEADDR PUT DATA IN RECORDING AREA  @ZA09235
         L     UCBREG,EUCBADR      GET UCB ADDRESS             @ZA09235
         MVC   ERECUCBP(UCBCURPX),UCBPXST  MOVE UCB PREFIX AND @ZA09235
         MVC   ERECUCBB(UCBDEV-UCBOB),UCBOB BASE TO REC AREA   @ZA09235
         MVI   SDWAURAL,ERECLEN    FILL IN LEN OF VAR REC AREA @ZA09235
         SETRP DUMP=YES,DUMPOPT=DUMPOPT,RECORD=YES,RECPARM=RECPARME,WKA*
               REA=(WORKC)
HALTES03 SETRP RC=0,WKAREA=(WORKC) SET TO CONTINUE TERMINATION @ZA09235
HALTES04 EQU   *                                               @ZA09235
         SR    R15,R15             SET 0 RETURN CODE           @ZA09235
         LR    RETREG,SAVREG       RESTORE RETURN ADDR         @ZA09235
         BR    RETREG              RETURN                      @ZA09235
         EJECT
*****************************************************************
*                                                               *
*        CONSTANTS                                              *
*                                                               *
*****************************************************************
HLTRTRY  DC    X'00FF'                                         @OZ00762
         SPACE 1
         DS    0F                  FULL WORD BOUNDRY
SRBEBD   DC    C'SRB '             SRB EBIDIC                  @OZ05656
SAVELEN  DC    AL1(253),AL3(72)    SUBPOOL AND LENGTH OF SAVE
*                                  *AREA FOR CALLED SUBROUTINES
CHAN6    DC    X'06'               FOR CRH CHAN 6              @Y30CQLG
PTH      DS    0D                  PATH TABLE FOR IOSGEN       @Y30CQLG
DADDR1   DC    X'0000'             DEVICE ADDR PRIM PATH       @Y30CQLG
CPU0PRIM DC    X'00'               PRIM PATH CPU 0 STATUS      @Y30CQLG
CPU1PRIM DC    X'00'               PRIM PATH CPU 1 STATUS      @Y30CQLG
DADDR2   DC    X'0000'             DEVICE ADDR SEC PATH        @Y30CQLG
CPU0SEC  DC    X'00'               SEC PATH CPU 0              @Y30CQLG
CPU1SEC  DC    X'00'               SEC PATH CPU 1              @Y30CQLG
         SPACE 1
*                                  *THE FOLLOWING MACRO IS THE
*                                  *LIST FORM OF SNAP TO INDICATE
DUMPOPT  SNAP  SDATA=(TRT,CB),     *DUMP TRACE DATA, TASK RELATED      *
               STORAGE=(IGC0003C,HALT9999), *CONTROL BLOCKS AND        *
               MF=L                *MODULE STORAGE
         SPACE 1
RECPARM  DS    0F                  FIXED RECORDING AREA PARAMETER
*                                  *LIST
         DC    C'IGC0003C'         MODULE MICROFICHE NAME
         DC    C'IGC0003C'         CSECT NAME
         DC    C'HALT0900'         RECOVERY ROUTINE ID
RECPARME DS    0F              FIXED RECORDING AREA PARAMETER  @ZA09235
         DC    C'IGC0003C'         FICHE NAME                  @ZA09235
         DC    C'IGC0003C'         CSECT NAME                  @ZA09235
         DC    C'HALTESTA'         RECOVERY RTN                @ZA09235
         SPACE 1
ESTAPLST ESTAE MF=(L),PURGE=NONE,TERM=YES                      @ZA09235
FRRADCON DC    A(HALT0900)         ADDRESS OF FRR ROUTINE
HIGH2OFF DC    X'0000FFFF'         MASK TO SET HIGH ORDER TWO
*                                  *BYTES OF A REGISTER TO 0
         SPACE 1
HALTPLEN EQU   (*-IGC0003C)/20     5 PER CENT PATCH AREA LENGTH
HALTPATC DC    XL(HALTPLEN)'00'    PATCH AREA
         SPACE 1
HALT9999 EQU   *                   END OF MODULE + 1
*                                  *THIS MUST REMAIN THE LAST
*                                  *LABEL OF IGC0003C
         END
