         TITLE 'IGG019M3 - BTAM 2260 DEVICE I/O MODULE'
***********************************************************************
*                                                                     *
* MODULE NAME: IGG019M3  (OS/VS)                                      *
*                                                                     *
* DESCRIPTIVE NAME: BTAM 2260 DEVICE I/O MODULE                       *
*                                                                     *
* COPYRIGHT: NONE                                                     *
*                                                                     *
* STATUS: RELEASE 2                                                   *
*                                                                     *
* FUNCTION:    THE DEVICE INPUT/OUTPUT MODULE PROVIDES THE DEVICE     *
*              DEPENDENT VARIABLE INFORMATION NEEDED BY THE BTAM      *
*              READ/WRITE ROUTINE TO PREPARE THE TELE-PROCESSING      *
*              CHANNEL PROGRAMS.                                      *
*                                                                     *
*              EACH OF THE FIRST 32 BYTES CONTAINS AN OFFSET TO THE   *
*              CHANNEL COMMAND WORD OFFSETS FOR EACH TYPE OF I/O      *
*              OPERATION.                                             *
*                                                                     *
*              FOLLOWING THE CCW OFFSETS ARE ALL OF THE DIFFERENT     *
*              CCW'S THAT ARE USED BY THIS DEVICE TYPE. FOLLOWING     *
*              THE CCW'S ARE THE NECESSARY SPECIAL CONTROL            *
*              CHARACTERS.                                            *
*                                                                     *
*              EXAMPLE OF CCW OFFSETS (I.E. CONDENSED CHANNEL         *
*              PROGRAM)                                               *
*                                                                     *
*                   DC   X'0403050906820008'                          *
*                                                                     *
*                        04 = COUNT OF CCW'S                          *
*               03,05,09,06 = CCW'S 3,5,9 & 6 COMPRISE THE CHANNEL    *
*                             PROGRAM                                 *
*                        82 = COUNT OF RESET FUNCTION CCW'S (0 BIT    *
*                             = 1 INDICATES RESET)                    *
*                     00,08 = CCW'S 0 & 8 COMPRISE RESET CHANNEL      *
*                             PROGRAM.                                *
*                                                                     *
*        THE DEVICE I/O MODULE CONTAINS NO EXECUTABLE CODE. ITS       *
*        ADDRESS IS LOADED INTO THE BTAM READ/WRITE ROUTINE BY OPEN   *
*        AT OPEN TIME.                                                *
*                                                                     *
* CHANGE ACTIVITY AS FOLLOWS:                                         *
*            APARS                                                    *
*       YA03258  (1/31/74)                                            *
*       ZA26706  (10/03/77)                                           *
*                                                                     *
***********************************************************************
         EJECT
IGG019M3 CSECT
A        DC    X'FF'          00
         DC    AL1(RI-A)      01        READ INITIAL
         DC    AL1(WI-A)      02        WRITE INITIAL
         DC    AL1(RT-A)      03        READ CONTINUE
         DC    AL1(WT-A)      04        WRITE CONTINUE
         DC    X'FF'          05
         DC    X'FF'          06
         DC    AL1(RP-A)      07        READ REPEAT
         DC    AL1(WA-A)      08        WRITE ACKNOWLEDGE
         DC    AL1(RS-A)      09        READ SKIP
         DC    AL1(WN-A)      0A        WRITE NEGATIVE ACKNOWLEDGE
         DC    AL1(RB-A)      0B        READ BUFFER
         DC    AL1(WL-A)      0C        WRITE AT LINE ADDRESS
         DC    X'FF'          0D
         DC    AL1(WS-A)      0E        WRITE ERASE
         DC    9X'FF'                                              000B
         DC    AL1(C-A)                 OFFSET TO CCW'S
         DC    7X'FF'                                              000B
         SPACE 2
*                   CHANNEL PROGRAMS
RI       DC    X'05000103080E8112'
WI       DC    X'0700020509130D108112'                           A35729
RT       DC    X'020A0F8112'
WT       DC    X'03130D108112'                                   A35729
RP       DC    X'020B0F8112'
WA       DC    X'0112'
WN       DC    X'0111'
RB       DC    X'05000104080E8112'
WL       DC    X'0700020609130C108112'                           A35729
WS       DC    X'0700020709130D108112'                           A35729
RS       DC    X'0114'
         SPACE 2
*                   C C W 'S               I/O      F L A G S    LENGTH
*                                       OPERATION CD   CC   SLI     =
C        DC    X'010C00AF80020010'     WR STX,EOTSX            @ZA26706
         DC    X'0114040080030002'      WR POLLING X              02
         DC    X'0114000080030002'      WR ADDRESS X               2
         DC    X'010C00A860040001'      WR CODE RD      X    X    1
         DC    X'010C00A960040001'      WR CODE RB      X    X    1
         DC    X'010C00AA60040001'      WR CODE WR      X    X    1
         DC    X'010C00AB60040001'      WR CODE WL      X    X    1
         DC    X'010C00AC60040001'      WR CODE WE      X    X    1
         DC    X'0204000080050002'      RD RESP PL X               2
         DC    X'0208000040060001'      RD RESP AD      X          1
         DC    X'010C00AD60020001'      WR ACK          X    X    1
         DC    X'010C00AE60020001'      WR NAC          X    X    1
         DC    X'0104000060100400'      WR AT LINE      X    X   LENGTH
         DC    X'0104000060110400'      WR TEXT         X    X   LENGTH
         DC    X'0204000220110402'      RD TEXT              X LENGTH-2
         DC    X'0204000020110400'      RD TEXT              X   LENGTH
         DC    X'0208000100200001'      RD RESP                    1
         DC    X'010C00B020210003'      WR EOT'S             X    3
         DC    X'010C00AF20210004'      WR STX,EOTS          X    4
         DC    X'010C00AF80080001'      WR STX     X              1
         DC    X'0200000030220400'      RD SKIP   SKIP FLAG  X   LENGTH
         SPACE 2
         DC    X'40'                    CODE FOR READ           00
         DC    X'50'                    CODE FOR READ BUFF      01
         DC    X'A0'                    CODE FOR WRITE          02
         DC    X'B0'                    CODE FOR WRITE LINE     03
         DC    X'E0'                    CODE FOR WRITE ERASE    04
         DC    X'06'                    ACK                     05
         DC    X'15'                    NAC                     06
         DC    X'02'                    STX                     07
         DC    15X'04'                 EOT SEQUENCE           08 A26881
         CNOP  0,8
         END
