         TITLE 'IGG09405 - STCOM ROUTINE'
IGG09405 CSECT
***********************************************************************
*                                                                M5113
*                                                                     *
*                                                                 M5113
*    RELEASE 23 DELETIONS/CHANGES                                     *
*    RELEASE 22 DELETIONS/CHANGES                                     *
*    RELEASE 21 DELETIONS/CHANGES                                     *
*                                                                     *
* STATUS -                                                            *
*    CHANGE LEVEL 01                                                  *
*                                                                     *
* FUNCTION -                                                          *
*    THIS ROUTINE IS TO SERVE THE MACRO STCOM', WHICH IS USED         *
*    BY PROFILE COMMAND AND OTHER PROBLEM PROGRAMS TO SPECIFY WHETHER *
*    OR NOT THE USER WISHES TO RECEIVE INTER-TERMINAL COMMUNICATIONS. *
*    ONCE THE USER STOPS INTER-TERMINAL COMMUNICATIONS VIA ISSUANCE   *
*    OF THIS MACRO, ANY SUBSEQUENT TPUTS WITH TJID TO THIS TERMINAL   *
*    WILL BE IGNORED AND A RETURN CODE OF '0C' WILL BE PASSED BACK TO *
*    THE CALLER.                                                      *
*                                                                     *
*    INTERNALLY, THIS ROUTINE WILL SET ON BITS IN TSB (TSBJTOFF,      *
*    TSBNOMSG) IF THE USER SPECIFIES NO INTER-TERMINAL COMMUNICATIONS.*
*    OTHERWISE, THE ABOVE MENTIONED BITS IN TSB WILL BE ON.  THE      *
*    TSBJTOFF BIT WILL BE CHECKED BY TPUT IF THE TJID OPTION IS       *
*    SPECIFIED.                                                       *
*                                                                     *
* ENTRY POINTS -                                                      *
*         IGG09405 - CONTROL IS RECEIVED FROM MODULE IGC0009D         *
*                                                                     *
* INPUT -                                                             *
*    REGISTER 0 CONTAINS PARAMETER                                    *
*    REGISTER 1 CONTAINS PARAMETER                                    *
*    REGISTER 3 CONTAINS CVT ADDRESS                                  *
*    REGISTER 4 CONTAINS TCB ADDRESS                                  *
*    REGISTER 5 CONTAINS TSB ADDRESS                                  *
*                                                                     *
* OUTPUT -                                                            *
*    REGISTER 15 HAS RETURN CODE:                                     *
*           0 - SUCCESSFUL                                            *
*           4 - INVALID PARAMETER                                     *
*                                                                     *
* EXTERNAL REFERENCES -                                               *
*         NONE                                                        *
*                                                                     *
* EXITS, NORMAL -                                                     *
*         BR 14 WITH 0 IN REGISTER 15                                 *
*                                                                     *
* EXITS, ERROR -                                                      *
*         BR 14 WITH 4 IN REGISTER 15                                 *
*                                                                     *
* TABLES/WORK AREAS -                                                 *
*    CONTROL BLOCKS ARE DESCRIBED IN THE DSECTS AT END OF LISTING     *
*                                                                     *
* ATTRIBUTES -                                                        *
*    REENTRANT, REFRESHABLE, SUPERVISOR MODE                    YS02019
*                                                                     *
* CHARACTER CODE DEPENDENCY -                                         *
*    THIS MODULE IS NOT CHARACTER DEPENDENT                           *
* NOTES -                                                             *
*    YS02019 - VS/2 RELEASE 2 CHANGES                                 *
*                                                                     *
***********************************************************************
         SPACE 3
********
******** REGISTER EQUATES
********
RWORK0   EQU   0                        WORK REGISTER
RPARM0   EQU   0                        PARAMETER REGISTER
RWORK1   EQU   1                        WORK REGISTER
RPARM1   EQU   1                        PARAMETER REGISTER
RWORK2   EQU   2                        WORK REGISTER
RWORK3   EQU   3                        WORK REGISTER
RCVT     EQU   3                        POINTS TO CVT
RTCB     EQU   4                        POINTS TO TCB
RTSB     EQU   5                        POINTS TO TSB
RWORK6   EQU   6                        WORK REG 6              YS02019
RWORK7   EQU   7                        WORK REGISTER           YS02019
RWORK8   EQU   8                        WORK REGISTER           YS02019
RWORK9   EQU   9                        WORK REGISTER           YS02019
RWORK10  EQU   10                       WORK REGISTER           YS02019
RWORK11  EQU   11                       WORK REGISTER           YS02019
RBASE    EQU   12                       BASE REGISTER
RCONST   EQU   13                       POINTS TO PROGRAM CONSTANTS
R13      EQU   13                       REGISTER 13
R14      EQU   14                       REGISTER 14
RLINK    EQU   14                       LINK REGISTER
RTRNSTBL EQU   14                       POINTS TO TRANSLATE TABLES
R15      EQU   15                       REGISTER 15
RBRANCH  EQU   15                       BRANCH REGISTER
RCODE    EQU   15                       RETURN CODE REGISTER
         SPACE 2
********
******** DISPLACEMENT,LENGTH AND MASK EQUATES
********
BYTE1    EQU   8                        MASK FOR BYTE 1 IN REG.
BYTE2    EQU   4                        MASK FOR BYTE 2 IN REG.
BYTE3    EQU   2                        MASK FOR BYTE 3 IN REG.
BYTE4    EQU   1                        MASK FOR BYTE 4 IN REG.
BYTE34   EQU   3                        MASK FOR BYTES 3&4 IN REG.
BYTE234  EQU   7                        MASK FOR BYTES 2&3&4 IN REG.
D0       EQU   0                        DISPLACEMENT OF 0
D1       EQU   1                        DISPLACEMENT OF 1
D2       EQU   2                        DISPLACEMENT OF 2
D3       EQU   3                        DISPLACEMENT OF 3
D4       EQU   4                        DISPLACEMENT OF 4
D5       EQU   5                        DISPLACEMENT OF 5
D6       EQU   6                        DISPLACEMENT OF 6
D14      EQU   14                       DISPLACEMENT OF 14
L1       EQU   1                        LENGTH OF 1
L2       EQU   2                        LENGTH OF 2
L3       EQU   3                        LENGTH OF 3
L4       EQU   4                        LENGTH OF 4
L8       EQU   8                        LENGTH OF 8
L16      EQU   16                       LENGTH OF 16
L20      EQU   20                       LENGTH OF 20
L24      EQU   24                       LENGTH OF 24
L32      EQU   32                       LENGTH OF 32
RETCODE4 EQU   4                        RETURN CODE OF 4
RETCODE8 EQU   8                        RETURN CODE OF 8
RETCODEC EQU   12                       RETURN CODE OF 12
HIPRI    EQU   X'E4'                    TCAM POSTING PRIORITY
COMPLETE EQU   X'40'                    TO TEST POST COMPLETE BIT
CZERO    EQU   0                        TO TEST FOR ZERO
CONE     EQU   1                        CONSTANT OF 1
CTWO     EQU   2                        CONSTANT OF 2
CTHREE   EQU   3                        CONSTANT OF 3
CFIFTEEN EQU   15                       CONSTANT OF 15
CSIXTEEN EQU   16                       CONSTANT OF 16
CFOX     EQU   X'FF'                    TO TEST FOR 'FF'
BLANK    EQU   X'40'                    TO TEST FOR BLANK
OFF      EQU   X'FF'                    TO TURN BITS OFF
         SPACE 3
********
******** ESTABLISH BASE REGISTER AND ADDRESSABILITY
********
         USING *,RBASE                  CSECT BASE - SET BY ENTRY POINT
         USING TSB,RTSB                 TSB ADDRESSABILITY
         B     PASTID                   SKIP MODULE ID.         YS02019
         DC    C'IGG09405'              MODULE ID.              YS02019
         DC    X'3124'                  DATE - 05/04/73         YS02019
PASTID   EQU   *                                                YS02019
********
******** CHECK PARAMETERS
********
         SLDL  RPARM0,L1                .CHECK
         LTR   RPARM1,RPARM1            .FOR
         BNZ   PARMBAD                  .VALID
         SRDL  RPARM0,L1                .PARAMETER
         LR    RWORK10,RLINK            SAVE RETURN ADDR        YS02019
         SPACE 2
********
******** MAIN FUNCTION OF ROUTINE
********
         MODESET EXTKEY=TCAM            GET TCAMS KEY           YS02019
         LTR   RPARM1,RPARM1            BIT BE SET OR RESET ?
         BZ    NOCOM                    SET FOR NO INTERCOM
COM      EQU   *
         NI    TSBSTAT,OFF-TSBITOFF     ALLOW INTERCOMMUNICATION
         B     RETURN                   RETURN TO CALLER
NOCOM    EQU   *
         OI    TSBSTAT,TSBITOFF         DISALOW INTERCOMMUNICATION
RETURN   EQU   *
         MODESET EXTKEY=SUPR            RETURN TO SUPR KEY      YS02019
         LR    RLINK,RWORK10            RESTORE RETURN ADDR     YS02019
         SR    RCODE,RCODE              SET GOOD RETURN CODE
         BR   R14                       RETURN TO CALLER        YS02019
PARMBAD  EQU   *
         LA    RCODE,RETCODE4           SET INVALID PARM CODE
         BR    R14                      GO BACK TO CALLER       YS02019
         DS    5F                       MAINTENANCE PATCH AREA   Y01018
         EJECT
         IHAPSA
         EJECT
         IKJTSB  LIST=YES
         END
