         TITLE 'IGG0940B - GTSIZE ROUTINE'
IGG0940B CSECT
         COPY  LCGASMS2                 LIBRARY CONTROL         YS02019
*0000765800-769000                                               M0590
*                                                                 M5113
***********************************************************************
*                                                                     *
*    RELEASE 23 DELETIONS/CHANGES                                     *
*    RELEASE 22 DELETIONS/CHANGES                                     *
*    RELEASE 21 DELETIONS/CHANGES                                     *
*                                                                     *
* STATUS -                                                            *
*    CHANGE LEVEL 001                                                 *
*                                                                     *
* FUNCTION -                                                          *
*    THIS ROUTINE IS TO SERVE THE MACRO 'GTSIZE', WHICH               *
*    FURNISHES THE USER PHYSICAL LINE SIZE OF HIS TERMINAL (KEPT IN   *
*    TSBLNZ) IN REGISTER 1. FOR DISPLAY STATION, THIS ROUTINE WILL    *
*    ALSO FURNISH THE SCREEN LENGTH (KEPT IN TSBLNNO) IN REGISTER 0.  *
*                                                                     *
* ENTRY POINTS -                                                      *
*         IGG0940B - CONTROL IS RECEIVED FROM MODULE IGC0009D         *
*                    VIA BRANCH ACCORDING ENTRY CODE 11, WITH THE     *
*                    LOCAL AND THE CMS LOCKS HELD AND IN              *
*                    SUPERVISOR KEY                                   *
*                                                                     *
* INPUT -                                                             *
*    REGISTER 0 CONTAINS PARAMETER                                    *
*    REGISTER 1 CONTAINS PARAMETER                                    *
*    REGISTER 3 CONTAINS CVT ADDRESS                                  *
*    REGISTER 4 CONTAINS TCB ADDRESS                                  *
*    REGISTER 5 CONTAINS TSB ADDRESS                                  *
*    REGISTER 10 CONTAINS THE ADDRESS OF AN ARRAY OF CONSTANTS
*    REGISTER 12 CONTAINS THE BASE ADDRESS
*    REGISTER 14 CONTAINS THE RETURN ADDRESS
*                                                                     *
* OUTPUT -                                                            *
*    REGISTER 0 HAS SCREEN LENGTH (OR 0 FOR NON-DISPLAY STATION)      *
*    REGISTER 1 HAS LINE SIZE                                         *
*    REGISTER 15 HAS RETURN CODE:                                     *
*           0 - SUCCESSFUL                                            *
*           4 - INVALID PARAMETER                                     *
* EXTERNAL REFERENCES -                                               *
*         NONE                                                        *
*                                                                     *
* EXITS, NORMAL -                                                     *
*         BR 14 WITH 0 IN REGISTER 15                                 *
*                                                                     *
* EXITS, ERROR -                                                      *
*         BR 14 WITH 4 IN REGISTER 15                                 *
*                                                                     *
* TABLES/WORK AREAS -                                                 *
*    CONTROL BLOCKS ARE DESCRIBED IN THE DSECTS AT END OF LISTING     *
*                                                                     *
* ATTRIBUTES -                                                        *
*    REENTRANT, REFRESHABLE, SUPERVISOR MODE, ENABLED                 *
*                                                                     *
* CHARACTER CODE DEPENDENCY -                                         *
*    THIS MODULE IS NOT CHARACTER DEPENDENT                           *
* NOTES -                                                             *
*    NONE                                                             *
*                                                                     *
***********************************************************************
         SPACE 3
********
******** EQUATES FOR REGISTERS AND MASKS
********
RPARM0   EQU   0                        PARAMETER REG 0
RWORK0   EQU   0                        WORK REGISTER
RPARM1   EQU   1                        PARAMETER REG 1
RWORK1   EQU   1                        WORK REGISTER
RWORK2   EQU   2                        WORK REGISTER
RWORK3   EQU   3                        WORK REGISTER
RCVT     EQU   3                        POINTS TO CVT
RTCB     EQU   4                        POINTS TO TCB
RTSB     EQU   5                        POINTS TO TSB           YS02019
RBASE    EQU   12                       BASE REGISTER
RCONST   EQU   13                       POINTS TO PROGRAM CONSTANTS
R13      EQU   13                       REGISTER 13
R14      EQU   14                       REGISTER 14
RLINK    EQU   14                       LINK REGISTER
R15      EQU   15                       REGISTER 15
RBRANCH  EQU   15                       BRANCH REGISTER
RCODE    EQU   15                       RETURN CODE REGISTER
         SPACE 2
********
******** DISPLACEMENT,LENGTH AND MASK EQUATES
********
BYTE1    EQU   8                        MASK FOR BYTE 1 IN REG.
BYTE2    EQU   4                        MASK FOR BYTE 2 IN REG.
BYTE3    EQU   2                        MASK FOR BYTE 3 IN REG.
BYTE4    EQU   1                        MASK FOR BYTE 4 IN REG.
BYTE34   EQU   3                        MASK FOR BYTES 3&4 IN REG.
BYTE234  EQU   7                        MASK FOR BYTES 2&3&4 IN REG.
D0       EQU   0                        DISPLACEMENT OF 0
D1       EQU   1                        DISPLACEMENT OF 1
D2       EQU   2                        DISPLACEMENT OF 2
D3       EQU   3                        DISPLACEMENT OF 3
D4       EQU   4                        DISPLACEMENT OF 4
D5       EQU   5                        DISPLACEMENT OF 5
D6       EQU   6                        DISPLACEMENT OF 6
D14      EQU   14                       DISPLACEMENT OF 14
L1       EQU   1                        LENGTH OF 1
L2       EQU   2                        LENGTH OF 2
L3       EQU   3                        LENGTH OF 3
L4       EQU   4                        LENGTH OF 4
L8       EQU   8                        LENGTH OF 8
L16      EQU   16                       LENGTH OF 16
L24      EQU   24                       LENGTH OF 24
L32      EQU   32                       LENGTH OF 32
RETCODE4 EQU   4                        RETURN CODE OF 4
RETCODE8 EQU   8                        RETURN CODE OF 8
RETCODEC EQU   12                       RETURN CODE OF 12
HIPRI    EQU   X'E4'                    TCAM POSTING PRIORITY
COMPLETE EQU   X'40'                    TO TEST POST COMPLETE BIT
CZERO    EQU   0                        TO TEST FOR ZERO
CONE     EQU   1                        CONSTANT OF 1
CTWO     EQU   2                        CONSTANT OF 2
CTHREE   EQU   3                        CONSTANT OF 3
CFIFTEEN EQU   15                       CONSTANT OF 15
CSIXTEEN EQU   16                       CONSTANT OF 16
CFOX     EQU   X'FF'                    TO TEST FOR 'FF'
BLANK    EQU   X'40'                    TO TEST FOR BLANK
OFF      EQU   X'FF'                    TO TURN BITS OFF
EXIT     EQU   3                        TO INVOKE EXIT,SVC 3
IDLE     EQU   X'17'                    TERM. CTL. CHAR. - IDLE
NL       EQU   X'15'                    TERM. CTL. CHAR. - NEW LINE
CR       EQU   X'0D'                    TERM. CTL. CHAR. - CARRIAGE RET
LF       EQU   X'25'                    TERM. CTL. CHAR. - LINE FEED
HEX18    EQU   X'18'                    TERM. CTL. CHAR. - TWX CHAR.
HEX20    EQU   X'20'                    TO TEST BIT 2 OF BYTE
LNSZ40   EQU   40                       LINE SIZE 40
LNSZ64   EQU   64                       LINE SIZE 64
LNSZ80   EQU   80                       LINE SIZE 80
LNNO6    EQU   6                        LINES PER SCREEN - 6
LNNO12   EQU   12                       LINES PER SCREEN - 12
LNNO15   EQU   15                       LINES PER SCREEN - 15
QINEC    EQU   21                       QTIP ENTRY CODE/TCLEARQ/INPUT
QOUTEC   EQU   22                       QTIP ENTRY CODE/TCLEARQ/OUTPUT
         SPACE 3
********
******** ESTABLISH BASE REGISTER AND ADDRESSABILITY
********
         USING *,RBASE                  BASE REGISTER ADDRESSABILITY
         USING TSB,RTSB                 TSB ADDRESSABILITY
         B     VALIDITY                 BR AROUND ID AND DATE   YS02019
         DC    C'IGG0940B'              MODULE IDENTIFIER       YS02019
         DC    X'3123'                  DATE - 05/03/73         YS02019
********
******** CHECK FOR VALID PARAMETERS
********
VALIDITY EQU   *                                                YS02019
         LTR   RPARM1,RPARM1            ANY PARM SPECIFIED
         BNZ   PARMBAD                  YES,INVALID
********
******** PERFORM MAIN FUNCTION OF ROUTINE
********
         SR    RPARM0,RPARM0            CLEAR REGISTER
         SR    RPARM1,RPARM1            CLEAR REGISTER           Y01018
         CLI   TSBLNSZ,CZERO            TSB INDICATE LINE SIZE
         BE    PARMBAD                  NO,ERROR CONDITION
         IC    RPARM1,TSBLNSZ           YES,GET LINE SIZE FOR CALLER
         TM    TSBSTAT,TSBDSPLY         IS THIS A DISPLAY TERMINAL
         BZ    RETURN                   NO,RETURN TO CALLER
         CLI   TSBLNNO,CZERO            YES,IS THERE A SCREEN SIZE
         BE    PARMBAD                  NO,ERROR CONDITION
         IC    RPARM0,TSBLNNO           YES,GET SCREEN SIZE FOR CALLER
RETURN   EQU   *
         SR    RCODE,RCODE              SET GOOD RETURN CODE
         BR    RLINK                    RETURN TO SUPERVISOR    YS02019
PARMBAD  EQU   *
         LA    RCODE,RETCODE4           SET PARM INVALID RETURN CODE
         BR    RLINK                    RETURN TO SUPERVISOR    YS02019
         DC    9F'0'                    MAINTENANCE PATCH AREA  YS02019
*                                       Y0101                   YS02019
         EJECT
         IKJTSB
         END
