 /********************************************************************/
 /*                                                                  */
 /*   MACRO NAME = IGGCCA                                            */
 /*                                                                  */
 /*   DESCRIPTIVE NAME = CATALOG COMMUNICATIONS AREA                 */
 /*                                                                  */
 /*   FUNCTION = THE CCA IS USED BY CATALOG MANAGEMENT TO PASS       */
 /*              INFORMATION BETWEEN ITS PROCEDURES.  THE CCA IS     */
 /*              BUILT EACH TIME AN OS/VS COMPONENT ISSUES THE CATLG */
 /*              MACRO INSTRUCTION (SVC 26) TO PROCESS A CATALOG     */
 /*              RECORD.  THE CCA CONTAINS INFORMATION ABOUT THE     */
 /*              CATALOG BEING PROCESSED, AND ABOUT THE CATALOG      */
 /*              RECORD AND ITS EXTENSIONS CONTAINED IN EACH OF      */
 /*              THE SIX BUFFERS AVAILABLE TO PROCESS THE USER'S     */
 /*              REQUEST.                                            */
 /*                                                                  */
 /*   CONTROL BLOCK STRUCTURE = THE CCA IS POINTED TO BY REGISTER 11 */
 /*                                                                  */
 /*   INCLUDED MACROS = NONE                                         */
 /*                                                                  */
 /*   METHOD OF ACCESS = PL/S - NO DECLARES NECESSARY                */
 /*                                                                  */
 /*   CREATED BY: CSECT IGG0CLC9 IN LOAD MODULE IGG0CLC9             */
 /*                                                                  */
 /*   STATUS = VS/2 RELEASE 3   (CHANGE FLAG @Y30SSXX)               */
 /*                                                                  */
 /*   DATE OF LAST CHANGE = 15 MAY 78                        @Y30SSPJ*/
 /*   C 572000                                               @ZA24944*/
 /*   I 692260-692268, 692929                                @ZA27024*/
 /*   I 691305-330                                           @ZA30975*/
 /*   C 514000,690826,693355                                 @ZA29164*/
 /*   I 514300-515600,690827-690828,693256                   @ZA29164*/
 /*   I                                                      @ZA29164*/
 /*                                                                  */
 /********************************************************************/
    DECLARE
*     1 IGGCCA BASED(CCAPTR),       /* SET BASE TO CCA               */
*       2 CCAID FIXED(15),          /* CCA ID = 'ACCA'X      @Y30SSPJ*/
*       2 CCASZ FIXED(15),          /* SIZE OF CCA           @Y30SSPJ*/
*       2 CCAPROB FIXED(31),        /* PROBLEM DETERMINATION WORD
*                                                            @Y30SSPJ*/
*         3 CCAMODID CHAR(2),       /* ERROR MODULE ID       @Y30SSPJ*/
*         3 CCAERRCD FIXED(15)      /* ERROR CODES           @Y30SSPJ*/
                   BDY(BYTE),
*           4 CCAREASN CHAR(1),     /* SET REASON CODE       @Y30SSPJ*/
*             5 CCACDR CHAR(1),     /* REFER REASON CODE     @Y30SSPJ*/
*           4 CCARETRN CHAR(1),     /* SET RETURN CODE       @Y30SSPJ*/
*             5 CCACD1 CHAR(1),     /* REFER RETURN CODE     @Y30SSPJ*/
*       2 * CHAR(5),                /* RESERVED              @Y30SSPJ*/
*       2 CCACD2 PTR(8),            /* RETURN CODE 2                 */
*       2 CCAFLG1 PTR(8),           /* FLAGS                         */
*         3 CCAF1LPS BIT(1),        /* STOP LOOP IF ON               */
*         3 CCAF1CNS BIT(1),        /* CATALOG NAME SEARCH     Y02020*/
*         3 CCAF1LRD BIT(1),        /* CCR READ INTO CORE            */
*         3 CCAF1KEY BIT(1),        /* GET BY TRUE NAME              */
*         3 CCAF1KGE BIT(1),        /* GET NEXT                      */
*         3 CCAF1CR BIT(1),         /* CHKPT CCR REQ                 */
*         3 CCAF1UP BIT(1),         /* GET FOR UPDATE                */
*         3 CCAF1DK BIT(1),         /* DELETE, KEEP CI               */
*       2 CCAFLG2 PTR(8),           /* FLAGS                         */
*         3 CCAF2SYS BIT(1),        /* SYSTEM CALLER                 */
*         3 CCAF2NVC BIT(1),        /* NO VALIDITY CHECK             */
*         3 CCAF2CCT BIT(1),        /* 0 = CONCAT CTLG SEARCH,       */
*                                   /* 1 = SINGLE CTLG SEARCH        */
          3 CCAF2XEQ BIT(1),        /* 0 = SHARED ENQ,               */
                                    /* 1 = EXCLUSIVE ENQ             */
          3 CCAF2RHS BIT(1),        /* RECURSIVE CALL TO CATLG       */
          3 CCAF2COB BIT(2),        /* COMB. OF CATLG OPEN/BUILD     */
            4 CCAF2CO BIT(1),       /* CATLG BEING OPENED            */
            4 CCAF2CB BIT(1),       /* CATLG OPEN DURING BUILD       */
*         3 CCAF2SMO BIT(1),        /* SEARCH MASTER CATLG ONLY      */
*       2 CCAFLG3 PTR(8),           /* FLAGS                         */
*         3 CCAEXGR1 BIT(1),        /* EXIT INDICATOR                */
*         3 CCAGC4   BIT(1),        /* GROUP CODE 4 IN SCNC          */
*         3 CCAGDSP  BIT(1),        /* GENDSP                        */
*         3 CCAEXGR2 BIT(1),        /* EXIT INDICATOR                */
*         3 CCANF    BIT(1),        /* NOT FOUND CONDITION           */
*         3 CCAELC2  BIT(1),        /* EXIT INDICATOR                */
*         3 CCALFT   BIT(1),        /* FIRST TIME                    */
*         3 CCAEGREC BIT(1),        /* EXIT INDICATOR                */
*       2 CCAFLG4 PTR(8),           /* FLAGS                         */
*         3 CCAF4DRQ BIT(1),        /* DEQ REQD                      */
*         3 CCAF4BYS BIT(1),        /* BYPASS SECURITY               */
*         3 CCAGVNC  BIT(1),        /* NOT COMPLETE                  */
*         3 CCAGVNF  BIT(1),        /* RELREPNO NOT FOUND            */
*         3 CCAGVNBS BIT(1),        /* NO BUFFER SPACE               */
*         3 CCAGVEX  BIT(1),        /* EXIT BIT                      */
*         3 CCAGVNE BIT(1),         /* NON-EXISTENT FIELD            */
*         3 CCATCOMP BIT(1),        /* TEST COMPLETE                 */
*       2 CCAFLG5 PTR(8),           /* FLAGS                         */
*         3 CCAMEX2  BIT(1),        /* EXIT INDICATOR                */
*         3 CCAMEX   BIT(1),        /* EXIT BIT                      */
*         3 CCAMEX1  BIT(1),        /* EXIT BIT                      */
*         3 CCAMODPA BIT(1),        /* PUT-ADD                       */
*         3 CCATHIT BIT(1),         /* SUCCESSFUL TEST               */
*         3 CCATEX BIT(1),          /* EXIT INDICATOR                */
*         3 CCATEX1 BIT(1),         /* EXIT INDICATOR                */
*         3 CCATEX2 BIT(1),         /* EXIT INDICATOR                */
*       2 CCAFLG6  PTR(8),          /* FLAGS                         */
*         3 CCAMCODR BIT(1),        /* DEQ REQ SW                    */
*         3 CCADELP BIT(1),         /* DELETED GOP                   */
*         3 CCAMNOSP BIT(1),        /* NO SPACE IN MOVE OCCUR        */
*         3 CCAINIT  BIT(1),        /* INSERT INIT SW FOR VAR FIELD  */
*         3 CCASUPFD BIT(1),        /* SUPPRESS FIELD INFO           */
*         3 CCAREUSE BIT(1),        /* RE-USE RECORD AREAS           */
*         3 CCAEXT   BIT(1),        /* EXTRACT IN PROCESS            */
*         3 CCAMOD   BIT(1),        /* MODIFY IN PROCESS             */
*       2 CCATCB PTR(31),           /* PTR TO TCB                    */
*         3 CCALBCYL PTR(31),       /* LABEL CYLINDER DATA PTR       */
*       2 CCARB  PTR(31),           /* PTR TO RB                     */
*         3 CCADPL PTR(31),         /* DADSM PARMETER LIST PTR       */
*       2 CCACPL PTR(31),           /* PTR TO CPL                    */
*       2 CCAACB PTR(31),           /* PTR TO CATALOG ACB            */
*       2 CCANPCCB PTR(31),         /* PTR TO NEXT PCCB IN CHAIN     */
*       2 CCAURAB  PTR(31),         /* PTR TO RAB TO USE             */
*       2 CCASRCH  CHAR(44),        /* SEARCH ARGUMENT               */
*         3 CCASRID CHAR(3),        /* CI NUMBER                     */
*           4 CCASRCIN PTR(24) BDY(BYTE), /* CI NUMBER               */
*         3 *       CHAR(41),       /* RESERVED                      */
*       2 CCARAB0,                  /* RECORD AREA BLOCK 0           */
*         3 CCAR0FLG PTR(8),        /* CCARAB0 FLAGS                 */
*           4 CCAR0UR BIT(1),       /* RAB IN-USE EXT, MOD           */
*           4 CCAR0U1 BIT(1),       /* RAB IN-USE TESTS              */
*           4 CCAR0U2 BIT(1),       /* RAB IN-USE TESTS              */
*           4 CCAR0WR BIT(1),       /* WRITE IT BEFORE READ          */
*           4 CCAR0PA BIT(1),       /* PUT-ADD WRITE                 */
*           4 *       BIT(2),       /* RESERVED                      */
*           4 CCAR0UPD BIT(1),      /* UPDATE BUFFER NOT REUSED      */
*         3 CCAR0RPL PTR(8),        /* LAST ASSIGN, RPL INDEX        */
*         3 * CHAR(2),              /* RESERVED                      */
*         3 CCAR0REC PTR(31),       /* PTR TO RECORD AREA            */
*         3 CCAR0SEG,               /* SEGMENT PTRS                  */
*           4 CCACPE20 PTR(31),     /* PTR REPEATING CONTROL INFO    */
*           4 CCACPE30 PTR(31),     /* PTR TO FIRST OCCURRENCE       */
*           4 CCACPE40 PTR(31),     /* PTR TO END OF RECORD          */
*       2 CCARAB1,                  /* RECORD AREA BLOCK 1           */
*         3 CCAR1FLG PTR(8),        /* FLAGS                         */
*           4 CCAR1UR BIT(1),       /* SAME AS CCAR0UR               */
*           4 CCAR1U1 BIT(1),       /* SAME AS CCAR0U1               */
*           4 CCAR1U2 BIT(1),       /* SAME AS CCAR0U2               */
*           4 CCAR1WR BIT(1),       /* SAME AS CCAR0WR               */
*           4 CCAR1PA BIT(1),       /* SAME AS CCAROPA               */
*           4 *       BIT(2),       /* RESERVED                      */
*           4 CCAR1UPD BIT(1),      /* SAME AS CCAR0UPD              */
*         3 CCAR1RPL PTR(8),        /* SAME AS CCAR0RPL              */
*         3 * CHAR(2),              /* RESERVED                      */
*         3 CCAR1REC PTR(31),       /* SAME AS CCAR0REC              */
*         3 CCAR1SEG,               /* SAME AS CCAR0SEG              */
*           4 CCACPE21 PTR(31),     /* SAME AS CCACPE20              */
*           4 CCACPE31 PTR(31),     /* SAME AS CCACPE30              */
*           4 CCACPE41 PTR(31),     /* SAME AS CCACPE40              */
*       2 CCARAB2,                  /* RECORD AREA BLOCK 2           */
*         3 CCAR2FLG PTR(8),        /* FLAGS                         */
*           4 CCAR2UR BIT(1),       /* SAME AS CCAR0UR               */
*           4 CCAR2U1 BIT(1),       /* SAME AS CCAR0U1               */
*           4 CCAR2U2 BIT(1),       /* SAME AS CCAR0U2               */
*           4 CCAR2WR BIT(1),       /* SAME AS CCAR0WR               */
*           4 CCAR2PA  BIT(1),      /* SAME AS CCAROPA               */
*           4 *        BIT(2),      /* RESERVED                      */
*           4 CCAR2UPD BIT(1),      /* SAME AS CCAR0UPD              */
*         3 CCAR2RPL PTR(8),        /* SAME AS CCAR0RPL              */
*         3 * CHAR(2),              /* RESERVED                      */
*         3 CCAR2REC PTR(31),       /* SAME AS CCAR0REC              */
*         3 CCAR2SEG,               /* SAME AS CCAR0SEG              */
*           4 CCACPE22 PTR(31),     /* SAME AS CCACPE20              */
*           4 CCACPE32 PTR(31),     /* SAME AS CCACPE30              */
*           4 CCACPE42 PTR(31),     /* SAME AS CCACPE40              */
*       2 CCARAB3,                  /* RECORD AREA BLOCK 3           */
*         3 CCAR3FLG PTR(8),        /* FLAGS                         */
*           4 CCAR3UR BIT(1),       /* SAME AS CCAR0UR               */
*           4 CCAR3U1 BIT(1),       /* SAME AS CCAR0U1               */
*           4 CCAR3U2 BIT(1),       /* SAME AS CCAR0U2               */
*           4 CCAR3WR BIT(1),       /* SAME AS CCAR0WR               */
*           4 CCAR3PA BIT(1),       /* SAME AS CCAROPA               */
*           4 *       BIT(2),       /* RESERVED                      */
*           4 CCAR3UPD BIT(1),      /* SAME AS CCAR0UPD              */
*         3 CCAR3RPL PTR(8),        /* SAME AS CCAR0RPL              */
*         3 * CHAR(2),              /* RESERVED                      */
*         3 CCAR3REC PTR(31),       /* SAME AS CCAR0REC              */
*         3 CCAR3SEG,               /* SAME AS CCAR0SEG              */
*           4 CCACPE23 PTR(31),     /* SAME AS CCACPE20              */
*           4 CCACPE33 PTR(31),     /* SAME AS CCACPE30              */
*           4 CCACPE43 PTR(31),     /* SAME AS CCACPE40              */
        2 CCARAB4,                  /* RECORD AREA BLOCK 4           */
          3 CCAR4FLG PTR(8),        /* FLAGS                         */
            4 CCAR4UR BIT(1),       /* SAME AS CCAR0UR               */
            4 CCAR4U1 BIT(1),       /* SAME AS CCAR0U1               */
            4 CCAR4U2 BIT(1),       /* SAME AS CCAR0U2               */
            4 CCAR4WR BIT(1),       /* SAME AS CCAR0WR               */
*           4 CCAR4PA  BIT(1),      /* SAME AS CCAROPA               */
*           4 *        BIT(2),      /* RESERVED                      */
            4 CCAR4UPD BIT(1),      /* SAME AS CCAR0UPD              */
          3 CCAR4RPL   PTR(8),      /* SAME AS CCAR0RPL              */
          3 *          CHAR(2),     /* RESERVED                      */
          3 CCAR4REC PTR(31),       /* SAME AS CCAR0REC              */
          3 CCAR4SEG,               /* SAME AS CCAR0SEG              */
            4 CCACPE24 PTR(31),     /* SAME AS CCACPE20              */
            4 CCACPE34 PTR(31),     /* SAME AS CCACPE30              */
            4 CCACPE44 PTR(31),     /* SAME AS CCACPE40              */
        2 CCARAB5,                  /* RECORD AREA BLOCK 5           */
          3 CCAR5FLG PTR(8),        /* FLAGS                         */
            4 CCAR5UR BIT(1),       /* SAME AS CCAR0UR               */
            4 CCAR5U1 BIT(1),       /* SAME AS CCAR0U1               */
            4 CCAR5U2 BIT(1),       /* SAME AS CCAR0U2               */
            4 CCAR5WR BIT(1),       /* SAME AS CCAR0WR               */
*           4 CCAR5PA BIT(1),       /* SAME AS CCAROPA               */
*           4 *       BIT(2),       /* RESERVED                      */
            4 CCAR5UPD BIT(1),      /* SAME AS CCAR0UPD              */
          3 CCAR5RPL PTR(8),        /* SAME AS CCAR0RPL              */
          3 *        CHAR(2),       /* RESERVED                      */
          3 CCAR5REC PTR(31),       /* SAME AS CCAR0REC              */
          3 CCAR5SEG,               /* SAME AS CCAR0SEG              */
            4 CCACPE25 PTR(31),     /* SAME AS CCACPE20              */
            4 CCACPE35 PTR(31),     /* SAME AS CCACPE30              */
            4 CCACPE45 PTR(31),     /* SAME AS CCACPE40              */
        2 CCARPLK PTR(8),           /* ASSIGNED RPL COUNT            */
        2 CCARPLF PTR(8),           /* INDEX TO FOUND RPL            */
        2 CCARPLX PTR(8),           /* WORK BYTE FOR SUBFUNCTION     */
                                    /* IGGPRPLM                      */
        2 CCARPLT PTR(8),           /* WORK BYTE FOR SUBFUNCTION     */
                                    /* IGGPRPLM                      */
        2 CCATIQRN CHAR(6),         /* TIOT ENQ MINOR NAME   @Y30SSPJ*/
          3 CCATASID CHAR(2),       /* ASID                  @Y30SSPJ*/
          3 CCATQDB PTR(31)BDY(BYTE),/* QDB ADDRESS          @Y30SSPJ*/
        2 CCASC PTR(8),             /* SEARCH CODE             Y02020*/
        2 CCAQLEN PTR(8),           /* QUALIFIER LENGTH | 0    Y02020*/
        2 CCARPL1 PTR(31),          /* PTR TO RPL IN USE             */
*       2 CCADESA CHAR(44),         /* DADSM EXTENTS SAVEAREA        */
*         3 CCANDEXT PTR(8),        /* COUNT OF EXTENTS              */
*         3 CCAIXEXT PTR(8),        /* EXTENT INDEX VALUE            */
*         3 CCASSVOL FIXED(15) BDY(BYTE), /* DATA SET DIR SEQ. NO.   */
*         3 CCAEXTDE(5) CHAR(8),    /* EXTENT DESCRIPTORS            */
*           4 CCAEXTSS CHAR(2),     /* DESCRIPTOR SPACE SEQ. NO.     */
*           4 CCAEXTAD CHAR(4),     /* EXTENT STARTING CCHH          */
*             5 CCAEXTCC FIXED(15) BDY(BYTE), /* STARTING CYLINDER   */
              5 CCAEXTHH FIXED(15) BDY(BYTE), /* STARTING TRACK      */
*           4 CCAEXTTH FIXED(15) BDY(BYTE), /* NO. TRACKS IN EXTENT  */
*       2 CCAASCIK PTR(8),          /* COUNT OF CI'S REQD            */
*       2 CCACRRP PTR(8),           /* RPL USED TO READ CCR          */
*       2 CCAASCIX PTR(8),          /* ASSIGNED CI ARRAY INDEX       */
*       2 CCAASCI(3) PTR(24) BDY(BYTE), /* ASSIGNED CI NUMBERS       */
*       2 CCAEQDQ CHAR(16) BDY(WORD), /* ENQ/DEQ PARAMETER LIST      */
*         3 CCAEDXFF PTR(8),        /* END OF PL IND BYTE - X'FF'    */
*         3 CCAEDRLN PTR(8),        /* LENGTH OF MINOR NAME          */
*         3 CCAEDOPT PTR(8),        /* ENQ-DEQ OPTIONS               */
*           4 CCAEDSHR BIT(1),      /* 1=SHARED, 0=EXCLUSIVE         */
*           4 *        BIT(7),      /* OTHER OPT. (SET BY MACRO)     */
*         3 CCAEDRCD PTR(8),        /* ENQ/DEQ RETURN CODE           */
*         3 CCAEDQNM PTR(31),       /* MAJOR NAME PTR                */
*         3 CCAEDRNM PTR(31),       /* MINOR NAME PTR                */
*         3 CCAEDUCB PTR(31),       /* UCB PTR                       */
*       2 CCAMLRET PTR(31),         /* MAIN LINE RTN SAVE AREA PTR   */
        2 CCAMSSPL CHAR(12) BDY(WORD), /* STORAGE MGMT WORKAREA      */
          3 CCAMNLLP PTR(31),       /* LENGTH OF LIST POINTER  Y02020*/
*         3 CCAMNPTR PTR(31),       /* ADDRESS OF RETURN ADDRESS     */
          3 *        FIXED(8),      /* STORAGE MANAGEMENT BYTE       */
          3 CCAMNSPL FIXED(8),      /* REQUIRED SUBPOOL              */
          3 *        FIXED(15),     /* STORAGE MANAGEMENT AREA       */
*       2 CCARPRM CHAR(4),          /* RETURN PARMS                  */
*       2 CCACMS CHAR(8) BDY(WORD), /* CMS AREA                      */
*         3 CCACMSWA PTR(31),       /* PTR TO CMS WORK AREA          */
*         3 CCAEXCMS PTR(31),       /* SECONDARY CMS WORK AREA ADDR  */
*       2 CCALUME CHAR(0),          /* START OF FIELD MGMT WORK AREA */
*                                   /* AND INTERFACE VARIABLES FOR   */
*                                   /* LOCATE, UPDATE, MODIFY, AND   */
*                                   /* EXTRACT ROUTINES              */
*       2 CCACPE5 PTR(31),          /* ADDR OF OCCURRENCE PTR        */
*       2 CCACPE51 PTR(31),         /* ALTERNATE PTR TO GOP          */
*       2 CCACPE52 PTR(31),         /* ALT GOP PTR                   */
*       2 CCACPE53 PTR(31),         /* ALT GOP PTR                   */
*       2 CCACPE6 PTR(31),          /* ADDR OF OCCURRENCE            */
*       2 CCACPE61 PTR(31),         /* ALT PTR TO OCCURRENCE         */
*         3 CCARABSE PTR(31),       /* SAVE EXTRACT CALLER URAB      */
*       2 CCACPE7 PTR(31),          /* ADDR OF FIELD VALUE           */
*         3 CCAIDPT PTR(31),        /* INSERT DATA ADDR              */
*       2 CCACPE71 PTR(31),         /* ALT PTR TO FIELD VALUE        */
*       2 CCAGOPLN FIXED(15),       /* GROUP OCCURRENCE PTR LENGTH   */
*       2 CCASL    FIXED(15),       /* SEQUENCE NUMBER LENGTH        */
*       2 CCAILNG PTR(31),          /* INSERT LENGTH                 */
*       2 CCAFLPT  PTR(31),         /* FIELD LIST POINTER            */
*         3 CCATFLPT PTR(31),       /* FIELD LIST PTR FOR TESTS      */
*       2 CCARABPT PTR(31),         /* RAB PTR                       */
*       2 CCADICT  CHAR(4),         /* DICTIONARY INFO               */
*       2 CCAXCPL  PTR(31),         /* EXTRACT CPL PTR               */
*         3 CCAMCPL PTR(31),        /* MODIFY CPL PTR                */
*       2 CCARABB PTR(31),          /* BASE RAB PTR                  */
*       2 CCARABF PTR(31),          /* FIRST RAB PTR                 */
*       2 CCARABL PTR(31),          /* LAST RAB PTR                  */
*       2 CCACBASE CHAR(3),         /* BASE CI NUMBER                */
*       2 CCAGC    PTR(8),          /* GROUP CODE                    */
*       2 CCALREL  FIXED(15),       /* LOGICAL RELREPNO              */
*         3 CCALREL1 CHAR(2),       /* LOGICAL RELREPNO              */
*       2 CCASN   FIXED(15),        /* SEQUENCE NO.                  */
*         3 CCASN1  CHAR(2),        /* SEQUENCE NO.                  */
*       2 CCAUSKEY BIT(8),          /* USER KEY              @ZA29164*/
*       2 CCAMSFLG BIT(8),          /* MISC. FLAGS           @ZA29164*/
*         3 CCACICIN BIT(1),        /* INTEGRITY CHECK INCOMPLETE
*                                                            @ZA29164*/
*         3 CCACPLOK BIT(1),        /* CPL IS VALID          @ZA29164*/
*         3 CCASKYSS BIT(1),        /* CALLER HAS SYSTEM KEY OR IS IN
*                                      SUPERVISOR STATE      @ZA29164*/
*         3 CCAGDGUY BIT(1),        /* AUTHORIZED CALLER     @ZA29164*/
*         3 *        BIT(4),        /* RESERVED              @ZA29164*/
*       2 CCAIXFPL FIXED(15),       /* FIELD ARRAY INDEX             */
*       2 CCAIXREL FIXED(15),       /* INDEX RELREPNO                */
*       2 CCATNREL FIXED(15),       /* NEXT RELREPNO                 */
*       2 CCATNUM FIXED(15),        /* NO. OF SUCCESSFUL RELREPNOS   */
*       2 CCATREL(16) FIXED(15),    /* SUCCESSFUL REL REPNOS         */
*       2 CCATNO FIXED(15),         /* TOTAL NO. SUCCESSFUL RELREPNOS*/
*       2 CCATEST PTR(31),          /* TEST FIELD PTR                */
*       2 CCARBA CHAR(20),          /* EXTENT HOLDER FIELD           */
*         3 CCACRAVL BDY(BYTE),     /* CRA VOLSER            @Y30SSRS*/
*           4 CCASS FIXED(15) BDY(BYTE), /* SPACE DESCRIPTOR @Y30SSRS*/
*                                   /* SEQUENCE NO.          @Y30SSRS*/
*           4 CCACCHH1 CHAR(4),     /* CCHH-LOW              @Y30SSRS*/
*         3 CCACRADT BDY(BYTE),     /* CRA DEVICE TYPE       @Y30SSRS*/
*           4 CCACCHH2 CHAR(4),     /* CCHH-HIGH             @Y30SSRS*/
*         3 CCATT FIXED(15) BDY(BYTE), /* TT-NO. TRACKS              */
*         3 CCARBA1 CHAR(4),        /* RBA-LOW                       */
*         3 CCARBA2 CHAR(4),        /* RBA-HIGH                      */
*       2 CCATLNG FIXED(15),        /* TESTED EXTENT LENGTH          */
*         3 CCATLEN  FIXED(15),     /* MODIFIED VAR LENGTH           */
*       2 CCARBAL FIXED(15),        /* RBA EXTENT BALANCE            */
*       2 CCACNIX FIXED(15),        /* COMBINATION NAME INDEX        */
*       2 CCASMFIX FIXED(15),       /* DEFINE CATALOG SMF INDEX      */
*       2 CCAIDPT2 PTR(31),         /* AVAILABLE SPACE IN WORK AREA  */
*       2 CCAIDPT3 PTR(31),         /* INCOMPLETE VAR FIELDS         */
*         3 CCARABSM PTR(31),       /* SAVE MODIFY CALLER URAB       */
*       2 CCAGVCT FIXED(15),        /* COUNTER                       */
*       2 CCANEVV CHAR(2),          /* NON-EXISTENT VARIABLE         */
*                                   /* VALUE LENGTH                  */
*       2 CCAGVEXT CHAR(3),         /* EXTENSION RCD POINTER         */
*       2 CCANEFV CHAR(1),          /* NON-EXISTENT FIXED VALUE      */
*       2 CCASAVID CHAR(1),         /* SAVE RECORD ID        @ZA24944*/
*       2 CCAGRGC  BIT(8),          /* GROUP CODE                    */
*         3 CCARCDID CHAR(1),       /* RECORD ID             @Y30SSRS*/
*       2 CCAGRHI  FIXED(15),       /* HIGH RELREPNO         @Y30SSRS*/
*         3 CCASSEQ  FIXED(15),     /* SAVE SEQ. NO.         @Y30SSRS*/
*           4 CCAGRHI1 CHAR(2),     /* HIGH RELREPNO         @Y30SSRS*/
*       2 CCAIXTPL FIXED(15),       /* INDEX TO TEST FPL             */
*       2 CCADLEN  FIXED(15),       /* MODIFY DELETE LENGTH          */
*       2 CCADIFF FIXED(15),        /* SPACE DIFFERENCE              */
*       2 CCAREPCT FIXED(15),       /* RELREPNO COUNT                */
*       2 CCADISP FIXED(15),        /* DISPLACEMENT INTO VAR FIELD   */
*       2 CCASVCI  CHAR(3),         /* SAVE CI OF BASE               */
*       2 CCASVCI1 CHAR(3),         /* SAVE SPACE CI                 */
*       2 CCADTA   PTR(31),         /* DICTIONARY PTR                */
*       2 CCACDTA  PTR(31),         /* INDEX COMBO TABLE PTR         */
*       2 CCADTCT  FIXED(15),       /* COUNT DICT ENTRIES            */
*       2 CCACDTCT FIXED(15),       /* COUNT INDEX COMB TABLE        */
*       2 CCACWAP  PTR(31),         /* CONTROLLER WORK AREA          */
*       2 CCASDWAP PTR(31),         /* POINTER TO SDWA         Y02020*/
*       2 CCAILNG3 FIXED(31),       /* MODIFY LENGTH                 */
*       2 CCAILNG2 FIXED(31),       /* MODIFY LENGTH                 */
*       2 CCAALPTR PTR(31),         /* SPACE MNGMT SUB-FUNCTION WA   */
        2 CCASMFPT PTR(31),         /* SMF DATA PTR                  */
        2 CCALCPL PTR(31),          /* LSPACE CPL PTR - INTERNAL CALL*/
        2 CCAFLG7 PTR(8),           /* FLAGS                         */
          3 CCALSP   BIT(1),        /* LSPACE INTERNAL CALL          */
          3 CCASMFEX BIT(1),        /* SMF EXIT INDICATOR            */
          3 CCASMFA  BIT(1),        /* DO SMF IN MODIFY              */
          3 CCASMFBR BIT(1),        /* DO GET FOR BASE RECORD        */
          3 CCAONCE  BIT(1),        /* MOVE ONLY ONE OCCURRENCE      */
          3 CCAROREQ BIT(1),        /* READ-ONLY REQUEST             */
          3 CCAFEOV  BIT(1),        /* FORCE EOV                     */
          3 CCACRABU BIT(1),        /* CRA BEING BUILT       @Y30SSPJ*/
        2 CCAFLG8 BIT(8),           /* FLAGS                 @Y30SSPJ*/
          3 CCADSRCL BIT(1),        /* DEFINE SPACE          @Y30SSPJ*/
                                    /*   RECURSIVE CALL      @Y30SSPJ*/
          3 CCAVBUFI BIT(1),        /* VOL RECORDS BUFFERED  @Y30SSPJ*/
          3 CCASCRA  BIT(1),        /* SUPPRESS CRA UPDATES  @Y30SSPJ*/
          3 CCASCICK BIT(1),        /* SUPPRESS CRA CI CHECK @Y30SSSB*/
          3 CCALPIND BIT(1),        /* LOOP CONTROL IN BUFFER        */
                                    /*   SCAN FOR GETS       @Y30SSSB*/
          3 CCAVRIND BIT(1),        /* VOLUME RECORD BUFFER CHAIN    */
                                    /*   TO BE CHECKED       @Y30SSSB*/
          3 CCALEOD  BIT(1),        /* EOF ON LOW KEYS       @Y30SSSB*/
          3 CCAAUCAT BIT(1),        /* VOLUME HAS UCAT       @Y30LSPS*/
        2 CCAFLG9 BIT(8),           /* FLAGS                 @Y30SSSB*/
          3 CCARABYC BIT(1),        /* BYPASS CAT I/O        @Y30SSSB*/
          3 CCARAEOV BIT(1),        /* CRA EOV               @Y30SSSB*/
          3 CCARALRD BIT(1),        /* CRA CCR HAS BEEN READ @Y30SSSB*/
          3 CCARACR  BIT(1),        /* CRA CCR CHK PT REQD   @Y30SSSB*/
          3 CCAUCRA  BIT(1),        /* USE UCRA TRANS TAB    @Y30SSSB*/
          3 CCARAACT BIT(1),        /* CRA ACTIVE            @Y30SSSB*/
          3 CCARAICI BIT(1),        /* INHIBIT CAT I/O       @Y30SSSB*/
          3 CCARESUM BIT(1),        /* ON = REPLACE SUM,ELSE @Y30SSSB*/
                                    /* OFF = INCREMENT SUM   @Y30SSSB*/
        2 CCANORBA PTR(8),          /* NO. OF RBA'S THAT ARE         */
                                    /* NEEDED IN CB STRT     @Y30SSSB*/
        2 CCASMFRD PTR(31),         /* SMF RECORD POINTER            */
        2 CCASMFCT FIXED(15),       /* COUNT OF SMF RECORDS          */
        2 CCASMFLG BIT(16),         /* SMF RECORD FLAGS              */
          3 CCASMFG1 BIT(8),        /* SMF FLAGS                     */
            4 CCASMFUC BIT(1),      /* UNCATALOG - SMF RCD 67        */
                                    /* (ALWAYS SET ON)               */
              5 CCASMFDF BIT(1),    /* DEFINE - SMF RCD 63           */
            4 CCASMFSR BIT(1),      /* SCRATCH - SMF RCD 67. SET ON  */
                                    /* ONLY IF RECORD ID IS D, I OR A*/
              5 CCASMFAL  BIT(1),   /* ALTER - SMF RCD 63            */
            4 *        BIT(6),      /* RESERVED                      */
          3 CCASMFG2 BIT(8),        /* SMF FLAGS                     */
        2 CCASMFLN FIXED(15),       /* SMF RECORD LENGTH             */
        2 CCACHAIN CHAR(3),         /* CHAIN CI NO. - START OF VOLUME*/
                                    /* ENTRY TRANSLATION WORKAREA    */
        2 CCACI1   CHAR(3),         /* SAVE CI AREA                  */
        2 CCACI2   CHAR(3),         /* SAVE CI AREA                  */
        2 CCACI3   CHAR(3),         /* SAVE CI AREA                  */
        2 CCAVARLN FIXED(15),       /* INSERT LENGTH                 */
        2 CCARRAB PTR(31),          /* RELATIVE BASE RAB ADDR        */
        2 CCARBASE PTR(31),         /* RELATIVE BASE RAB ADDR        */
        2 CCAVARPT PTR(31),         /* POINTER TO INSERT INFO        */
        2 CCADELN FIXED(15),        /* DELETE LENGTH                 */
        2 CCAVAR   CHAR(20),        /* INSERT INFO SAVE AREA         */
        2 CCAVAR1  CHAR(20),        /* INSERT INFO SAVE AREA         */
        2 CCADEL1  CHAR(3),         /* START DELETE CI               */
        2 CCADEL2  CHAR(3),         /* END DELETE CI                 */
        2 CCAXLATE CHAR(40),        /* TRANSLATION WORK AREA         */
        2 CCAR14S  PTR(31),         /* CLC9 REG 14 SAVE AREA         */
        2 CCABMINP,                 /* INPUT PARAMETERS              */
          3 CCABMTRK FIXED(15),     /* STARTING TRACK                */
          3 CCABMLIM FIXED(15),     /* CHECK LIMIT, NN FOR SET       */
          3 CCABMMIN FIXED(15),     /* COND CHECK MINIMUM            */
          3 CCABMFLG BIT(8),        /* STATE AND FUNCTION CODE       */
            4 CCABMST  BIT(1),      /* STATE TO SET/COND. CHECK      */
            4 CCABMCHK BIT(1),      /* ON = PERFORM CHECK            */
            4 CCABMSET BIT(1),      /* ON = PERFORM SET              */
            4 CCABMCCK BIT(1),      /* ON = PERFORM COND. CHECK      */
            4 CCABMLST BIT(1),      /* ON = LAST SET REQ (WRITE)     */
          3 *        PTR(8),        /* RESERVED                      */
        2 CCABMOUT,                 /* OUTPUT PARAMETERS             */
          3 CCABMONN FIXED(15),     /* TRK NUMBER (CK/COND.CK)       */
          3 CCABMOTR FIXED(15),     /* STARTING TRK(COND.CHK.)       */
          3 CCABMOFG BIT(8),        /* OUTPUT FLAGS                  */
            4 CCABMOST BIT(1),      /* STATE OF BITS (CHECK)         */
        2 *      CHAR(2),           /* RESERVED                      */
        2 CCABMPAD CHAR(1),         /* PADDING CHARACTER             */
        2 CCABMGOP PTR(31),         /* CURRENT BIT MASK GOP          */
        2 CCABMPTR PTR(31),         /* CURRENT BIT MASK BYTE         */
        2 CCABMEND PTR(31),         /* END OF CURRENT BIT MASK       */
        2 CCABMBT1 FIXED(15),       /* BIT COUNT - FIRST BYTE        */
        2 CCABMBTL FIXED(15),       /* BIT COUNT - LAST BYTE         */
        2 CCABMBYT FIXED(15),       /* NUMBER OF FULL BYTES          */
        2 CCABMSTR FIXED(15),       /* CURRENT BIT MASK START TRACK  */
        2 CCABMWK1 PTR(31),         /* WORK FIELD                    */
        2 CCABMWK2 PTR(31),         /* WORK FIELD                    */
        2 CCABMWK3 PTR(31),         /* WORK FIELD                    */
        2 CCABMWK4 PTR(31),         /* WORK FIELD                    */
        2 CCABMRB1 PTR(31),         /* FIRST BIT MAP RAB PTR         */
        2 CCABMRB2 PTR(31),         /* SECOND RAB POINTER            */
        2 CCATEMPS CHAR(40),        /* PL/S TEMP AREA                */
        2 CCAMNCAT CHAR(256) BDY(WORD),/* CONTIGUOUS AREA FOR
                                           TRACKING            Y02020*/
          3 CCAMNAT CHAR(248) BDY(WORD),/* TRACKING BUFFER     Y02020*/
            4 MNATTOP BIT(8),       /* TOP ENTRY - 1ST BYTE    Y02020*/
              5 MNATFULL BIT(1),    /* BUFFER FULL             Y02020*/
              5 *        BIT(7),    /* RESERVED                Y02020*/
            4 * CHAR(239),          /* BODY OF BUFFER          Y02020*/
            4 MNATFLGS BIT(8),      /* MOST RECENT ENTRY-1ST BYTE
                                                               Y02020*/
              5 MNATVAL BIT(1),     /* VALID ENTRY BIT         Y02020*/
              5 *       BIT(2),     /* RESERVED              @ZA29164*/
              5 MNATFCLS BIT(1),    /* CLASS 'F' CORE IF OFF @ZA29164*/
              5 *       BIT(3),     /* RESERVED              @ZA29164*/
              5 MNATSCLS BIT(1),    /* CLASS 'S' CORE          Y02020*/
            4 MNATARG1 PTR(24) BDY(BYTE),/* REMAINDER OF 1ST WD
                                                               Y02020*/
            4 MNATARG2 PTR(32),     /* MOST RECENT ENTRY-2ND WD
                                                               Y02020*/
          3 CCAMNLL PTR(8),         /* G/F MAIN LEN LIST-EOL BYTE
                                                               Y02020*/
          3 CCAMNLEN PTR(24) BDY(BYTE),/* G/F MAIN LENGTH      Y02020*/
          3 CCAMNADR PTR(31),       /* G/F MAIN ADDRESS        Y02020*/
        2 CCAARFWA CHAR(16) BDY(WORD),/* SPILL RTN WORK AREA   Y02020*/
          3 ARFGMLEN PTR(31),       /* LEN LIST FOR GETMAIN    Y02020*/
            4 ARFGMLP PTR(8),       /* END-OF-LIST BYTE        Y02020*/
            4 ARFLEN PTR(24) BDY(BYTE),/* LENGTH               Y02020*/
          3 ARFGMADR PTR(31),       /* ADDRESS FOR GETMAIN     Y02020*/
          3 ARFSBSCH PTR(31),       /* PTR TO 1ST SPILL BLOCK  Y02020*/
          3 ARFSBECH PTR(31),       /* PTR TO LAST SPILL BLOCK Y02020*/
        2 CCARVFG1 BIT(8),          /* RECOVERY FLAGS          Y02020*/
          3 RVCCAV BIT(1),          /* CCA VALID               Y02020*/
          3 RVARFI BIT(1),          /* TRACKING DATA INCOMPLETE
                                                               Y02020*/
          3 RVCMSFG BIT(1),         /* CMS FUNCTION GATE       Y02020*/
          3 RVESBO BIT(1),          /* ESTAE BACKOUT IN CONTROL
                                                               Y02020*/
          3 RVESBOR BIT(1),         /* ESTAE BACKOUT REGISTER  Y02020*/
          3 * BIT(3),               /* RESERVED                Y02020*/
        2 * CHAR(2),                /* RESERVED                Y02020*/
        2 CCAFLG11 BIT(8),          /*                       @ZA30975*/
          3 * BIT(4),               /* RESERVED              @ZA30975*/
          3 CCAF11B5 BIT(1),        /* B5 AND B6 ON - NO CATLG PASSWORD
                                                             @ZA30975*/
          3 CCAF11B6 BIT(1),        /* B5 AND B6 EQUAL TO 01, NO UPDATE
                 PASSWORD, EQUAL TO 10, THERE IS UPDATE PW   @ZA30975*/
          3 * BIT(2),               /* RESERVED              @ZA30975*/
        2 CCAREGS CHAR(348),        /* REGISTER SAVE AREA      Y02020*/
          3 *        CHAR(4),       /* USER SAVE AREA ADDR     Y02020*/
          3 CCAMODNM CHAR(8),       /* LOAD MODULE NAME      @Y30SSSB*/
        2 CCABZSAV   PTR(31),       /* SAVE AREA FOR CLBZ    @Y30SSSB*/
          3 CCADSPWA PTR(31),       /* DEF SPACE W.A. PTR    @Y30SSDW*/
        2 CCACUMPL PTR(31),         /* CATLG UPGRADE MGMNT   @Y30SSPJ*/
                                    /* PARM LIST PTR         @Y30SSPJ*/
        2 CCATNQCT PTR(8),          /* TIOT ENQ COUNT        @Y30SSPJ*/
        2 CCASBASE PTR(24),         /* SAVE BASE CI          @Y30SSPJ*/
                                    /* FOR UPGD PROCESS      @Y30SSPJ*/
        2 CCACRACI PTR(31),         /* CRA REC PTR ARRAY AD  @Y30SSSB*/
        2 CCARAACB PTR(31),         /* CRA ACB ADDRESS       @Y30SSSB*/
        2 CCARARPL PTR(31),         /* CRA RPL ADDRESS       @Y30SSSB*/
        2 CCARARBA PTR(31),         /* CRA RBA               @Y30SSSB*/
        2 CCARAREC PTR(31),         /* RECORD POINTER        @Y30SSSB*/
        2 CCARALSA PTR(31),         /* CRA LOCAL SAVE AREA   @Y30SSSB*/
          3 CCACRABT FIXED(15),     /* BLOCKS/TRACK FOR CRA RECORD
                                       CONSTRUCTION (CLB4)   @Y30SSDW*/
          3 *        CHAR(2),       /* RESERVED              @Y30SSDW*/
        2 CCAFLG10 BIT(8),          /* FLAG BYTE             @Y30SSSB*/
          3 CCAINCPL BIT(1),        /* INVALID CPL, VAL CK   @Y30SSSB*/
          3 CCAPDMW  BIT(1),        /* PROB DET MSG PUT      @Y30SSSB*/
          3 CCACATAC BIT(1),        /* CAT ACTIVE, CRA       @Y30SSSB*/
          3 CCARAFEV BIT(1),        /* CRA FORCED EOV CH     @Y30SSSB*/
          3 CCARARTC BIT(1),        /* RECOVERY EXIT, RETURN TO      */
                                    /*   CALLER              @Y30SSSB*/
          3 CCAPRANX BIT(1),        /* PRIME CRA GONE        @Y30SSRG*/
          3 CCADEFUN BIT(1),        /* DEFINE UNIQUE SAVE    @ZA27024*/
          3 *        BIT(1),        /* RESERVED              @ZA27024*/
        2 CCASUMTT PTR(24) BDY(BYTE), /* CRA SUM TT VALUE    @Y30SSSB*/
        2 CCADICTS CHAR(4),         /* DATA/INDEX ID TS      @Y30SSSB*/
        2 CCARANCA (2) PTR(31),     /* NORMAL RECORD BUFFER CHAIN    */
                                    /*  START, END ADDRS     @Y30SSSB*/
        2 CCARAVCA (2) PTR(31),     /* VOLUME RECORD BUFFER CHAIN    */
                                    /*  START, END ADDRS     @Y30SSSB*/
        2 CCAVTS   CHAR(8),         /* VOLUME TIMESTAMP      @Y30SSSB*/
        2 CCAREWKA PTR(31),         /* REUSE WORKAREA ADDR   @Y30SSSB*/
*       2 CCASMFP,                  /* SMF SAVE AREA         @Y30SSSB*/
                                    /* FOR PROB DETER        @Y30SSSB*/
          3 CCASMFMD CHAR(2),       /* MODULE ID             @Y30SSSB*/
          3 CCASMFRC CHAR(1),       /* REASON CODE           @Y30SSSB*/
          3 CCASMFCD CHAR(1),       /* RETURN CODE           @Y30SSSB*/
*       2 CCAPROBX FIXED(31),       /* AUXILLARY SAVE AREA   @Y30SSSB*/
                                    /* FOR CCAPROB           @Y30SSSB*/
*         3 CCAMODDX CHAR(2),       /* ERROR MODULE ID       @Y30SSSB*/
          3 CCAERCDX FIXED(15) BDY(BYTE), /* ERROR CODES     @Y30SSSB*/
            4 CCARESNX CHAR(1),     /* REASON CODE           @Y30SSSB*/
            4 CCARETRX CHAR(1),     /* RETURN CODE           @Y30SSSB*/
*       2 CCADGDGA PTR(31),         /* DEL FULL GDG WA       @Y30SSSB*/
          3 CCASAVMN PTR(31),       /* SAVE GOTTEN CORE ADDR @ZA27024*/
*       2 CCAOCPL PTR(31),          /* ORIGINAL CPL ADDRESS  @ZA29164*/
*       2 * CHAR(40),               /* RESERVED              @ZA29164*/
        2 CCAEND  CHAR(0);          /* END CCA - MUST END ON         */
                                    /*   FULLWORD BOUNDARY   @Y30SSSB*/
                                    /*   FULLWORD BOUNDARY   @Y30SSSB*/
