 /********************************************************************/
 /*                                                                  */
 /*   MACRO NAME = IGGMCDCL                                          */
 /*                                                                  */
 /*   DESCRIPTIVE NAME = VSAM CATALOG COMMON DECLARATIONS            */
 /*                                                                  */
 /*   FUNCTION = CONTAINS THE COMMONLY USED CONTROL BLOCK FORMATS,   */
 /*              CONSTANTS, FUNDAMENTAL MAPPINGS, AND INTERFACE      */
 /*              MACROS FOR VSAM CATALOG MANAGEMENT PROCESSING.      */
 /*                                                                  */
 /*   INCLUDED MACROS = IGGMCTRC      IKJTCB       IEZCTGWA          */
 /*                     IGGCCA        IFGACB       IEZCTGVL          */
 /*                     IEZCTGPL      AMCBS        IEZCTGCV          */
 /*                     IEZCTGFL      IGGCAXWA                       */
 /*                     CVT           IEZCTGFV                       */
 /*                                                                  */
 /*   METHOD OF ACCESS = PL/S - RESPECIFY A BASE FOR SOME STRUCTURES */
 /*                                                                  */
* /* MACSTAT Y-2 74218/ZA00964                              @ZA00964*/
 /*   STATUS = VS/2 RELEASE 3   (CHANGE FLAG @Y30SSXX)               */
 /*                                                                  */
 /*   DATE OF LAST CHANGE = 17 JAN 75                        @Y30SSPJ*/
 /*                                                                  */
 /********************************************************************/
 % EXECPARM:
           MACRO;                   /* SETS MACRO VARIABLES IN       */
                                    /* ACCORDANCE WITH EXEC STMT     */
                                    /* MACPARM VALUES                */
           DCL SYSTEM CHAR EXT ;    /* TYPE OF SYSTEM INDICATOR      */
           IF MACPARM = 'DOS'       /* COMPILE FOR DOS SYSTEM?       */
             THEN SYSTEM = 'DOS';   /* YES - INDICATE DOS            */
             ELSE SYSTEM = 'AOS';   /* NO  - DEFAULT TO AOS          */
 % END;
           %DCL SYSTEM CHAR EXT;    /* TYPE OF SYSTEM INDICATOR      */
           ?EXECPARM;
 /********************************************************************/
 /*                    DECLARATION  OF  CONSTANTS                    */
 /********************************************************************/
    DECLARE
      ON       BIT(1)    CONSTANT('1'B), /* ON - A '1' BIT           */
      OFF      BIT(1)    CONSTANT('0'B), /* OFF - A '0' BIT          */
      TRUEGC   BIT(8)    CONSTANT('3F'X), /* TO GET TRUE GROUP CODE  */
      LOWKEYTY BIT(8)    CONSTANT('00'X), /* LOW KEY TYPE            */
      CATEXTID CHAR(1)   CONSTANT('E'), /* CATALOG EXTENSION RECRD ID*/
      VOLEXTID CHAR(1)   CONSTANT('W'), /* VOLUME EXTENSION RECRD ID */
      VOLGOPLN FIXED(31) CONSTANT(6), /* VOLUME GOP LENGTH           */
      CATGOPLN FIXED(31) CONSTANT(5), /* CATALOG GOP LENGTH          */
      VOLSNLN  FIXED(31) CONSTANT(2), /* VOLUME SEQUENCE NO. LENGTH  */
      CATSNLN  FIXED(31) CONSTANT(1), /* CATALOG SEQUENCE NO. LENGTH */
      PSWDGC   FIXED(31) CONSTANT(4), /* SECURITY GROUP CODE         */
      K256     FIXED(31) CONSTANT(256),/* CONSTANT OF 256    @Y30SSPJ*/
      IGNOREIT FIXED(31) CONSTANT(0),  /* CLEAR ERROR CODE   @Y30SSPJ*/
      NOGETMSE FIXED(31) CONSTANT(0),  /* NO GETMAIN ERROR CODE
                                                             @Y30SSPJ*/
      VSRELID1 PTR(8) CONSTANT(0),  /* VSAM REL 1 ID         @Y30SSPJ*/
      VSRELID2 PTR(8) CONSTANT(1),  /* VSAM REL 2 ID         @Y30SSPJ*/
      CURELID  PTR(8) CONSTANT(1),  /* CURRENT VSAM REL ID   @Y30SSPJ*/
      SSUBPOOL FIXED(8) CONSTANT(241), /* SYSTEM SUBPOOL NO. @Y30SSDW*/
      FSUBPOOL FIXED(8) CONSTANT(229), /* FETCH PROTECTED SP @ZA29164*/
      USUBPOOL FIXED(8) CONSTANT(252),/* USER SUBPOOL NUMBER @Y30SSDW*/
      SASIZ    FIXED(31) CONSTANT(12); /* SAVE AREA SIZE             */
 /********************************************************************/
 /*                   DECLARATION  OF  REGISTERS                     */
 /********************************************************************/
    DECLARE                         /* GENERAL REGISTERS             */
      (RP0 REG(0),                  /* PARAMETER REG 0               */
       RP1 REG(1),                  /* PARAMETER REG 1               */
       WKR2 REG(2),                 /* WORK REGISTER                 */
       WKR3 REG(3),                 /* WORK REGISTER                 */
       WKR4 REG(4),                 /* WORK REGISTER                 */
       RSAVE REG(13),               /* SAVE AREA REGISTER 13         */
       RET   REG(14),               /* RETURN REGISTER               */
       RGO   REG(15)) PTR(31);      /* ENTRY POINT REGISTER          */
 /********************************************************************/
 /*                   DECLARATION  OF  REGISTERS                     */
 /********************************************************************/
    DECLARE                         /* GENERAL REGISTERS             */
      (RZERO REG(0),                /* GR 0 FOR GETMAIN USAGE        */
       RWA  REG(2),                 /* GR 2 POINTS TO WORK AREA      */
       RNDX REG(3),                 /* GR 3 FOR INDEX                */
       RFLD REG(4),                 /* GR 4 POINTS TO FIELD          */
       RWK1 REG(6),                 /* GR 6 IS WORK REGISTER         */
       RWK2 REG(7),                 /* GR 7 IS WORK REGISTER         */
       RWK3 REG(8),                 /* GR 8 IS WORK REGISTER         */
       RWK4 REG(9)) PTR(31);        /* GR 9 IS WORK REGISTER         */
 /********************************************************************/
 /*                   DECLARATION  OF  REGISTERS                     */
 /********************************************************************/
    DECLARE                         /* GENERAL REGISTERS             */
      R0  REG(0)  PTR(31),          /* STANDARD REGISTER             */
      R1  REG(1)  PTR(31),          /* STANDARD REGISTER             */
      R2  REG(2)  PTR(31) RSTD,     /* LOCAL WORK REGISTER           */
      R3  REG(3)  PTR(31) RSTD,     /* LOCAL WORK REGISTER           */
      R4  REG(4)  PTR(31) RSTD,     /* LOCAL WORK REGISTER           */
      R5  REG(5)  PTR(31),          /* WORK REGISTER                 */
      R6  REG(6)  PTR(31),          /* WORK REGISTER                 */
      R7  REG(7)  PTR(31),          /* WORK REGISTER                 */
      R8  REG(8)  PTR(31),          /* WORK REGISTER                 */
      R9  REG(9)  PTR(31),          /* WORK REGISTER                 */
      R10 REG(10) PTR(31),          /* WORK REGISTER                 */
      R11 REG(11) PTR(31) RSTD,     /* CCA BASE REGISTER             */
      R12 REG(12) PTR(31) RSTD,     /* ASSEMBLER CODE BASE REGISTER  */
      R13 REG(13) PTR(31),          /* SAVE AREA REGISTER            */
      R14 REG(14) PTR(31),          /* RETURN REGISTER               */
      R15 REG(15) PTR(31);          /* ENTRY POINT REGISTER          */
 @EJECT;
 %INCLUDE SYSLIB(IGGMCTRC);         /* CATALOG RETURN CODES          */
 @EJECT;
 %CCAPTR   = 'R11';                 /* NEW CCA BASE VARIABLE         */
 %INCLUDE SYSLIB(IGGCCA);           /* CATALOG COMMUNICATIONS AREA   */
 %DCL IGGMODID CHAR EXT;            /* GLOBAL MODULE ID      @Y30SSPJ*/
 @EJECT;
 %CTGPLPTR = 'CCACPL';              /* NEW IEZCTGPL BASE             */
 %INCLUDE SYSLIB(IEZCTGPL);         /* CATALOG PARAMETER LIST        */
 @EJECT;
 %CTGFLPTR = 'CCAFLPT';             /* TEMPORARY IEZCTGFL BASE       */
 %INCLUDE SYSLIB(IEZCTGFL);         /* FIELD PARAMETER LIST          */
 @EJECT;
 /********************************************************************/
 /*                  USER  WORKAREA  DECLARATION                     */
 /********************************************************************/
    DECLARE
      1 USERWKA  BASED(CTGWKA),     /* USER WORK AREA                */
        2 WKALEN1  FIXED(15),       /* LENGTH OF USER WORK AREA      */
        2 WKALEN2  FIXED(15),       /* REQUIRED LENGTH FOR USER AREA */
        2 WKA      CHAR(0),         /* SPACE FOR RETRIEVED INFO      */
          3 ENTNAMES(*),            /* ENTRYNAME SET ARRAY           */
            4 ENTNAMET CHAR(1),     /* ENTRYNAME TYPE CODE           */
            4 ENTNAMEV CHAR(44);    /* ENTRYNAME VALUE               */
    DCL CTGVLDSP FIXED(15) BASED(ADDR(CTGFLDGC)); /* PARTIAL EXTRACT */
 /********************************************************************/
 /*                DECLARATION  OF  FIELD  NAME                      */
 /********************************************************************/
    DECLARE FIELDNM CHAR(8) BASED(CTGFLDNM); /* FIELD NAME           */
 /********************************************************************/
 /*      CATALOG  FIELD  DICTIONARY  INFORMATION  DECLARATION        */
 /********************************************************************/
    DECLARE
      1 DICTENT BASED(ADDR(CTGFLDWA)), /* DICTIONARY ENTRY           */
        2 DICTFLGS BIT(8),          /* FIELD FLAGS                   */
          3 DICTVAR  BIT(1),        /* VARIABLE OR FIXED             */
          3 DICTREP  BIT(1),        /* REPEATING OR NON-REPEATING    */
          3 DICTCOMB BIT(1),        /* COMBINATION/SPECIAL OR NOT    */
          3 DICTFLG  BIT(1),        /* FLAG FIELD OR NOT             */
          3 DICTMV   BIT(1),        /* MULTI-VALUED OR NOT           */
          3 DICTSCRA BIT(1),        /* SUPPRESS CRA UPDATES  @Y30SSPJ*/
          3 DICTUPGD BIT(1),        /* UPGRADE ENTRY NAME    @Y30SSPJ*/
          3 *        BIT(1),        /* RESERVED              @Y30SSPJ*/
        2 DICTST   PTR(8),          /* STARTING POINT                */
        2 DICTLC   PTR(8),          /* LENGTH OR COUNT               */
        2 DICTGC   PTR(8);          /* GROUP CODE                    */
    DECLARE
      DTCMBMK BIT(8) CONSTANT('F0'X), /* COMBINATION MASK    @Y30SSPJ*/
                                      /* A FIELD IS A COMBINATION    */
                                      /* FIELD IF DICTFLGS&DTCMBMK = */
                                      /* DTCMB                       */
      /* THE FOLLOWING DICTIONARY ENTRY FLAGS ARE USED TO IDENTIFY   */
      /*   SPECIAL FIELD NAMES                                       */
      DTCMB   BIT(8) CONSTANT('20'X), /* COMBINATION FIELD NAME      */
      DTSRR   BIT(8) CONSTANT('60'X), /* RELREPNO, REPNO FIELD NAME  */
      DTSTR   BIT(8) CONSTANT('E8'X), /* TRBAEXT FIELD NAME          */
      DTSCC   BIT(8) CONSTANT('60'X); /* CATACB, CNTREPNO, GENDSP    */
                                      /* FIELD NAMES                 */
    /* THE FOLLOWING DICTIONARY ENTRY VALUES ARE USED TO IDENTIFY    */
    /*    SPECIFIC SPECIAL FIELD NAMES                               */
    DECLARE
      DTREL   BIT(8) CONSTANT('03'X), /* RELREPNO - CTGFLDWA(2)      */
      DTREP   BIT(8) CONSTANT('04'X), /* REPNO    - CTGFLDWA(2)      */
      DTCNT   BIT(8) CONSTANT('02'X), /* CNTREPNO - CTGFLDWA(2)      */
      DTCAT   BIT(8) CONSTANT('01'X), /* CATACB   - CTGFLDWA(2)      */
      DTGDP   BIT(8) CONSTANT('05'X); /* GENDSP   - CTGFLDWA(2)      */
 /********************************************************************/
 /*      CATALOG  FIELD  DICTIONARY  INFORMATION  DECLARATION        */
 /********************************************************************/
    DECLARE
      1 DIC2ENT BASED(ADDR(CCADICT)), /* DICTIONARY ENTRY            */
        2 DIC2FLGS BIT(8),          /* FIELD FLAGS                   */
          3 DIC2VAR  BIT(1),        /* VARIABLE OR FIXED             */
          3 DIC2REP  BIT(1),        /* REPEATING OR NON-REPEATING    */
          3 DIC2COMB BIT(1),        /* COMB/SPECIAL OR NOT           */
          3 DIC2FLG  BIT(1),        /* FLAG FIELD OR NOT             */
          3 DIC2MV   BIT(1),        /* MULTI-VALUED OR NOT           */
          3 DIC2SCRA BIT(1),        /* SUPPRESS CRA UPDATES  @Y30SSPJ*/
          3 DIC2UPGD BIT(1),        /* UPGRADE ENTRY NAME    @Y30SSPJ*/
          3 *        BIT(1),        /* RESERVED              @Y30SSPJ*/
        2 DIC2ST   PTR(8),          /* STARTING POINT                */
        2 DIC2LC   PTR(8),          /* LENGTH OR COUNT               */
        2 DIC2GC   PTR(8);          /* GROUP CODE                    */
 /********************************************************************/
 /*              CATALOG  ENTRY  POINTERS  DECLARATION               */
 /********************************************************************/
    DECLARE
      1 CCARAB BASED(CCAURAB),      /* CATALOG RAB POINTERS          */
        2 CCARABFL PTR(8),          /* RAB FLAGS                     */
          3 CCARBUR BIT(1),         /* RAB IN-USE MODIFY, EXTRACT    */
          3 CCARBU1 BIT(1),         /* RAB IN-USE TESTS              */
          3 CCARBU2 BIT(1),         /* RAB IN-USE TESTS              */
          3 CCARBWR BIT(1),         /* WRITE IT BEFORE READ          */
          3 CCARBPA BIT(1),         /* PUT-ADD WRITE                 */
          3 *       BIT(2),         /* RESERVED                      */
          3 CCARBUPD BIT(1),        /* UPDATE BUFFER NOT REUSED      */
        2 CCARABRI PTR(8),          /* LAST ASSIGNED RPL INDEX       */
        2 * CHAR(2),                /* RESERVED                      */
        2 CCACPE1 PTR(31),          /* START ENTRY RECORD PTR        */
        2 CCACPE2 PTR(31),          /* PTR TO REPEATING FIELD        */
                                    /* CONTROL INFORMATION           */
        2 CCACPE3 PTR(31),          /* PTR TO FIRST GROUP OCCURRENCE */
        2 CCACPE4 PTR(31);          /* PTR TO END OF RECORD ENTRY    */
 /********************************************************************/
 /*             GROUP  OCCURRENCE  POINTER  DECLARATION              */
 /********************************************************************/
    DECLARE
      1 CATGOPT BASED(CCACPE5),     /* GROUP OCCURRENCE POINTER      */
        2 CATGOCI CHAR(3),          /* CI NUMBER OR                  */
          3 CATGOFLG BIT(8),        /* GROUP OCCURRENCE FLAG         */
            4 *      BIT(8),        /* RESERVED                      */
          3 CATGODSP FIXED(15)      /* DISPLACEMENT TO GROUP         */
                     BDY(BYTE),     /*   OCCURRENCE                  */
        2 CATGOGC1 PTR(8),          /* GROUP CODE INFORMATION        */
          3 CATGOEXT BIT(1),        /* GROUP OCCURRENCE OUTSIDE RECD */
          3 CATGODEL BIT(1),        /* DELETED GROUP OCCURRENCE PTR  */
          3 CATGOGC  BIT(6),        /* TRUE GROUP CODE               */
        2 CATGOSEQ PTR(8);          /* (HIGH) SEQUENCE NUMBER        */
 /********************************************************************/
 /*       DECLARATION  OF  VARIABLE  LENGTH  SEQUENCE  NUMBER        */
 /********************************************************************/
    DECLARE CATSN CHAR(2) BASED(ADDR(CATGOSEQ)); /* SEQUENCE NUMBER  */
 /********************************************************************/
 /*        DECLARATION  OF  CATALOG  RECORD  KEY  INFORMATION        */
 /********************************************************************/
    DECLARE
      1 CATRCD   BASED(CCACPE1),    /* CATALOG RECORD                */
        2 RCDKEYTY CHAR(1),         /* KEY TYPE                      */
        2 RCDCINO  PTR(24),         /* RECORD CI NUMBER              */
        2 RCDRELID PTR(8),          /* RELEASE INDICATOR     @Y30SSPJ*/
        2 RCDCRAVL CHAR(6),         /* CRA VOLUME SERIAL     @Y30SSPJ*/
        2 RCDCRACI PTR(24) BDY(BYTE), /* CRA CI NUMBER       @Y30SSPJ*/
        2 RCDCRADT CHAR(4),         /* CRA DEVICE TYPE       @Y30SSPJ*/
        2 RCDRACTS CHAR(4),         /* CRA CREATION TIME STAMP
                                                             @Y30SSSB*/
        2 RCDDICTS CHAR(4),         /* D/I IDENTIFIER TIME STAMP
                                                             @Y30SSSB*/
        2 *        CHAR(18),        /* RESERVED              @Y30SSSB*/
        2 RCDID    CHAR(1),         /* RECORD ID                     */
        2 RCDSIZ   FIXED(15) BDY(BYTE), /* RECORD SIZE               */
        2 NREPCV   PTR(8),          /* COUNT OF VARIABLE FIELDS      */
        2 NREPLF   PTR(8);          /* LENGTH OF FIXED FIELDS        */
 /********************************************************************/
 /*     DECLARATION OF CONTROL INFORMATION FOR REPEATING FIELDS      */
 /********************************************************************/
    DECLARE
      1 REPCNTRL BASED(CCACPE2),    /* CONTROL INFORMATION           */
        2 EXTRCDPT CHAR(5),         /* EXTENSION RECORD POINTER      */
          3 EXTGOCI  CHAR(3),       /* CI NUMBER OF EXTENSION RECORD */
          3 *        CHAR(2),       /* RESERVED                      */
        2 REPCNT   PTR(8),          /* COUNT OF GROUP OCCURRENCE PTRS*/
        2 REPGOPT(*) CHAR(5);       /* VARIABLE NUMBER OF GROUP      */
                                    /*   OCCURRENCE POINTERS         */
 /********************************************************************/
 /*             DECLARATION  OF  VARIABLE  LENGTH  FIELD             */
 /********************************************************************/
    DECLARE
      1 VARFIELD BASED(CCACPE7),    /* VARIABLE FIELD                */
        2 VARLEN  FIXED(15) BDY(BYTE), /* LENGTH OF FIELD            */
        2 VARINFO CHAR(512);        /* FIELD INFORMATION             */
 /********************************************************************/
 /*              DECLARATION  OF  CONTROL  INFORMATION               */
 /********************************************************************/
    DECLARE
      1 CNTRLINF BASED(CCACPE6),    /* CONTROL INFORMATION           */
        2 CNTVAR  PTR(8),           /* COUNT OF VARIABLE FIELDS      */
        2 LENFIX  PTR(8);           /* LENGTH OF FIXED FIELDS        */
 @EJECT;
 /********************************************************************/
 /*               DECLARATION  OF  CVT  BASE  POINTER                */
 /********************************************************************/
    DECLARE CVTPTR PTR(31) LOCATION(16); /* BASE FOR CVT             */
 %INCLUDE SYSLIB(CVT);              /* COMMUNICATIONS VECTOR TABLE   */
 %DEACT OFF;
 @EJECT;
 /********************************************************************/
 /*             POINTERS  TO  NEXT  AND  CURRENT  TCB'S              */
 /********************************************************************/
    DECLARE
      1 TCBWORDS BASED(CVTTCBP),    /* TCB LIST                      */
        2 NEXTTCB PTR(31),          /* POINTER TO NEXT TCB           */
        2 TCBPTR  PTR(31);          /* POINTER TO CURRENT TCB        */
 %INCLUDE SYSLIB(IKJTCB);           /* TASK CONTROL BLOCK            */
 @EJECT;
 %INCLUDE SYSLIB(IHAPSA);
 @EJECT;
 %INCLUDE SYSLIB(IFGACB);
 RFY IFGACB BASED(CCAACB);          /* ACB BASE IN CCA               */
 @EJECT;
 /********************************************************************/
 /*                    ACCESS  METHOD  FLAGS                         */
 /********************************************************************/
    DECLARE
      1 MCATDSTR PTR(31) BDY(BYTE) DEF(CVTAMFF), /* FLAGS ARE IN CVT */
        2 CVTAMF1 BIT(8),           /* VSAM CVT FLAGS        @Y30SSPJ*/
          3 AMF1OPEN BIT(1),        /* MASTER CATALOG OPEN           */
          3 AMF1NONX BIT(1),        /* MASTER CATALOG NON-EXISTENT   */
          3 * BIT(6),               /* RESERVED                      */
        2 * PTR(24) BDY(BYTE);      /* RESERVED                      */
 %DECLARE MCATOPEN CHAR,            /* MASTER CATALOG OPEN           */
          MCATNONX CHAR,            /* MASTER CATALOG NON-EXISTENT   */
          CBSPTR   CHAR;            /* ADDRESS OF THE AMCBS          */
 %MCATOPEN = 'AMF1OPEN';            /* MASTER CATALOG OPEN           */
 %MCATNONX = 'AMF1NONX';            /* MASTER CATALOG NON-EXISTENT   */
 %CBSPTR = 'CVTCBSP';               /* ADDRESS OF THE AMCBS          */
 @EJECT;
 %INCLUDE SYSLIB(AMCBS);            /* ACCESS METHOD CNTRL BLK STRUC */
 @EJECT;
 /********************************************************************/
 /*              D U M M Y     D E C L A R A T I O N S               */
 /********************************************************************/
    DECLARE CI CHAR(3) BASED;       /* DUMMY CI NUMBER               */
    DECLARE TNAM CHAR(44) BASED;    /* DUMMY TRUE NAME               */
 @EJECT;
 %CAXWAPTR = 'ACBUAPTR';
 %INCLUDE SYSLIB(IGGCAXWA);         /* CATALOG AUXILIARY WORKAREA    */
 @EJECT;
 %INCLUDE SYSLIB(IEZCTGFV);         /* FIELD VECTOR TABLE            */
 @EJECT;
 %INCLUDE SYSLIB(IEZCTGWA);         /* SCHEDULER WORK AREA           */
 @EJECT;
 %INCLUDE SYSLIB(IEZCTGVL);         /* CATALOG VOLUME LIST           */
 @EJECT;
 %INCLUDE SYSLIB(IEZCTGCV);         /* CONTROL VOLUME LIST           */
 @EJECT;
 /********************************************************************/
 /*                WORK  AREA  FOR  OS/VS  LOCATE                    */
 /********************************************************************/
    DECLARE
      1 MWA BASED(RWA),             /* WORK AREA FOR OS/VS LOCATE    */
        3 VOLENTCT FIXED(15),       /* VOLUME ENTRY COUNT            */
        3 VLST(20) BDY(HWORD),      /* VOLUME LIST                   */
          5 DEVTYP CHAR(4),         /* DEVICE TYPE                   */
          5 VOLSER CHAR(6),         /* VOLUME SERIAL NUMBER          */
          5 FSN PTR(16) BDY(BYTE),  /* FILE SEQUENCE NUMBER          */
        3 * CHAR(10),               /* BINARY ZEROS                  */
        3 CVOLIND BDY(BYTE),        /* CAN BE DEVICE TYPE            */
          5 TTR1 CHAR(3),           /* TTR - NEXT BLOCK(OVER 20 VOL) */
          5 BIN0 BIT(8),            /* BINARY ZEROS                  */
        3 TTR2 CHAR(3),             /* TTR FOR NEXT BLOCK READ       */
        3 CVLSER CHAR(6);           /* VOLUME SERIAL OF REQ CVOL     */
 % CALLSF: MACRO ;                      /*GENERATES LABELED CALLS   */
     DCL IGGMODID EXT;              /* GLOBAL MODULE ID      @Y30SSPJ*/
     DCL IGGBASE2 EXT;              /* SECONDARY BASE REG    @ZA29164*/
     DCL (XMODTBL,XMODTBLB,WARG) CHAR;  /* WORK VARIABLES    @Y30SSPJ*/
     DCL (HIT1,HIT2) FIXED;         /* WORK VARIABLES        @Y30SSPJ*/
     XMODTBL='-SLEN-SLIV-AA;-ACDV-AB;-MCO-AC;-MCO2-TINQ'|| /*@Y30SSPJ*/
  '-TIDQ-RAOP-AD;-DCME-MEBM-AE;-DELC-EMSG-EMIO-AF;-IORA'|| /*@Y30SSPJ*/
       '-ISCI-PUPC-PDE-PDEC-PAD-PADC-AOCI-AXCI-RCCR'    || /*@Y30SSPJ*/
       '-CCCR-XIO-AG;-SCAT-QNQC-AH;-FDSP-DFM1-AI;-DBDI' || /*@Y30SSPJ*/
       '-AJ;-DCMB-AK;-DTIM-DEF-AL;-DBVC-SLOC-SLEI-SLEL' || /*@Y30SSPJ*/
  '-AM;-DRDA-DSCB-DCCE-PSEM-AN;-DEFB-DEFX-AO;-DCDA-AP;' || /*@Y30SSPJ*/
  '-DEFS-AQ;-SALL-AR;-DCRC-DEFC-AS;-CDVR-ARFS-AT;-SALS' || /*@Y30SSPJ*/
       '-AU;-MOD-UPD-DEL2-SGOP-AV;-ADGO-GNEX-GREL-IGOP' || /*@Y30SSPJ*/
       '-PREC-AW;-ALT2-EXPD-SHNK-MGO-DGO-DGOP-AX;-SCNC' || /*@Y30SSPJ*/
       '-AY;-EXT-LOC-AZ;-GLOC-GDGL-A2;-SC2-GRE-RPLM'    || /*@Y30SSPJ*/
       '-RPLF-A3;-DGDS-ERUG-A4;-DELX-DALL-DELB'         || /*@Y30SSPJ*/
     '-A5;-DFS2-BJFB-CBPT-CRTC-A6;-VMSC-DEMV-DVMV-DUSC' || /*@Y30SSDW*/
       '-MCRA-DF4T-A7;-DFRS-A8;-CMRR-CCBC-A9;' ;           /*@Y30SSPJ*/
     XMODTBLB='-TSTS-GVAL-GREC-BA;-UPDE-BB;-INIT-SVOL'  || /*@Y30SSPJ*/
       '-BC;-ALT-BD;-ALVL-ALEC-ANVV-BE;-SSCR-BF;-DEL'   || /*@Y30SSPJ*/
       '-DOPN-DEXA-DLXT-BG;-DEFA-DAVO-BH;-GET-GETC-TNXO'|| /*@Y30SSPJ*/
       '-UCRS-TXO-BI;-GDSP-BJ;-LSP-LDCS-BK;-DELS-DUCB'  || /*@Y30SSPJ*/
       '-DLVM-DLET-BL;-CKAU-BM;-ALVR-VRD-ALMD-ALSV-BN;' || /*@Y30SSPJ*/
       '-RAPA-RAPU-RAG-RAPD-RARC-XRIO-RASC-BO;-SPAC-RET1'||/*@Y30SSPJ*/
       '-BP;-LSTC-BQ;-BMR-BR;-XVAL-XEXT-BS;-XMOD-XLT2'  || /*@Y30SSPJ*/
       '-XEL2-XDGO-BT;-F4RD-F4WR-F4DQ-BU;-SMFA-SMFF-SMFS'||/*@Y30SSPJ*/
       '-SMFL-SMFR-SMFZ-BV;-DEIN-BW;-DSPC-BX;-DRSP-BY;' || /*@Y30SSPJ*/
       '-CONV-GALO-VALI-BZ;'                            || /*@Y30SSSG*/
       '-DCNV-ITER-CMKY-DPBI-DCCO-FPDA-B0;'             || /*@Y30SSSG*/
       '-UADD-UDEL-B1;-AUPG-B2;'                        || /*@Y30SSDW*/
       '-SMF-SMFG-B3;-DCRA-B4;-DEAX-DIAX-DEPT-DIPT'     || /*@Y30SSDW*/
       '-DCLS-DUPG-B5;-WTSO-SPSC-INMD-B6;-RUS-B7;-DFBO' || /*@Y30SSPJ*/
       '-DUND-CNBO-B8;-PAIX-PATH-B9;'                   || /*@Y30SSDW*/
       '-RAPV-MODI-WCRA-WCAT-DA;' ;                        /*@Y30SSDW*/
     WARG='-'||MACLIST(1)||'-';     /* SEARCH AGUMENT        @Y30SSPJ*/
     HIT1=INDEX(XMODTBL,WARG);      /* SEARCH TABLE A        @Y30SSPJ*/
     IF HIT1=0 THEN                 /* NAME FOUND TABLE A?   @Y30SSPJ*/
       DO;                          /* NO-NOT FOUND IN TBL A @Y30SSPJ*/
         XMODTBL=XMODTBLB;          /* SET-UP TABLE A WITH B @Y30SSPJ*/
         HIT1=INDEX(XMODTBL,WARG);  /* SEARCH TABLE B        @Y30SSPJ*/
       END;                                                /*@Y30SSPJ*/
     ELSE;                                                 /*@Y30SSPJ*/
     IF HIT1^=0 THEN                /* NAME FOUND TABLES?    @Y30SSPJ*/
       DO;                          /* YES-NAME FOUND        @Y30SSPJ*/
         HIT2=INDEX(XMODTBL(HIT1:LENGTH(XMODTBL)),';')+HIT1-1;
                                    /* SEARCH FOR MOD NAME   @Y30SSPJ*/
         IF IGGMODID(2:3) ^= XMODTBL(HIT2-2:HIT2-1) THEN
           DO;                                             /*@Y30SSPJ*/
             ANS ('/') SKIP COL(MACCOL);                   /*@Y30SSPJ*/
             ANS ('*') COL(MACCOL+1);                      /*@Y30SSPJ*/
             ANS(' CALL TO MODULE IGG0CL' ||               /*@Y30SSPJ*/
                       XMODTBL(HIT2-2:HIT2-1) || ' ')
                         COL(MACCOL+2);                    /*@Y30SSPJ*/
             ANS('*') COL(70);      /*GENERATE COMMENT       @Y30SSPJ*/
             ANS('/') COL(71);      /*GENERATE COMMENT       @Y30SSPJ*/
           END;                                            /*@Y30SSPJ*/
         ELSE;                                             /*@Y30SSPJ*/
       END;                             /*                   @Y30SSPJ*/
     ANS ('Z' || MACINDEX(2:4) || MACLIST(1) || ':') SKIP COL(2);
     IF IGGBASE2 ^= '' THEN         /* BASE REG 2 SPECIFIED? @ZA29164*/
       DO;                          /* YES                   @ZA29164*/
         ANS('DO;') COL(MACCOL);    /*                       @ZA29164*/
         ANS('CALL IGGP'||MACLIST(1)||';') COL(MACCOL+2);/*  @ZA29164*/
         ANS('GEN(RLDSBREG IGGP'||MACLIST(1)||');') COL(MACCOL+2);
         ANS('/* RELOAD SECONDARY BASE REG */') COL(37);/*   @ZA29164*/
         ANS('END;') COL(MACCOL);   /*                       @ZA29164*/
       END;                         /*                       @ZA29164*/
     ELSE                           /* NO BASE REG 2         @ZA29164*/
       ANS ('CALL IGGP' || MACLIST(1) || ';') COL(MACCOL); /*@Y30SSPJ*/
     ANS (' ') SKIP COL(MACCOL+LENGTH(MACLIST)+7);         /*@Y30SSPJ*/
 % END ;
 % CATPROB:                         /* PROBLEM DETERMINATION @Y30SSPJ*/
     MACRO KEYS(MODID,ECD,RCD);     /* MACRO                 @Y30SSPJ*/
     DCL IGGMODID CHAR EXT;         /* EBCDIC MODULE ID      @Y30SSPJ*/
     DCL (ERROR,REASON,ERRCOM) CHAR;/* WORK VARIABLES        @Y30SSPJ*/
     DCL (RSNTBL,RNUM,CARG) CHAR;   /* WORK VARIABLES        @Y30SSPJ*/
     DCL (HIT1,HIT2,RCODE) FIXED;   /* WORK VARIABLES        @Y30SSPJ*/
     RCODE = 0;                     /* INTERNAL RETURN CODE  @Y30SSPJ*/
     RNUM = '0,1,2,3,4,5,6,7,8,9,'; /* NUMERIC SEARCH TABLE  @Y30SSPJ*/
     RSNTBL = '0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,'||
              '30,32,34,36,38,40,42,44,46,48,50,52,54,' ||
              '56,58,60,62,64,66,68,70,72,74,76,78,80,' ||
              '82,84,86,88,90,92,94,96,98,100,102,104,' ||
              '106,108,110,112,114,116,118,120,122,124,'||
              '126,' ;              /* REASON SEARCH TABLE   @Y30SSPJ*/
     IF MODID(1) = '*' THEN         /* MODULE ID ONLY?       @Y30SSPJ*/
       DO;                          /* YES-MODULE ID ONLY    @Y30SSPJ*/
         ANS('/') SKIP COL(MACCOL);                        /*@Y30SSPJ*/
         ANS('*') COL(MACCOL+1);                           /*@Y30SSPJ*/
         ANS(' SET CCAMODID=' || IGGMODID ||
          ', CCAREASN=RSN0, CCARETRN=RCS ' ) COL(MACCOL+2);/*@Y30SSPJ*/
         ANS('*') COL(70);                                 /*@Y30SSPJ*/
         ANS('/') COL(71);                                 /*@Y30SSPJ*/
         ANS('CCAPROB=IGGSEGID;') SKIP COL(MACCOL);        /*@Y30SSPJ*/
         ANS(' ') COL(MACCOL+LENGTH(MACKEYS)+8);           /*@Y30SSPJ*/
         RETURN CODE(RCODE);        /* RETURN                @Y30SSPJ*/
       END;                                                /*@Y30SSPJ*/
     ELSE;                                                 /*@Y30SSPJ*/
     ERROR = ECD(1);                /* PICK-UP ERROR CODE    @Y30SSPJ*/
     IF RCD(1) = '' THEN            /*REASON CODE SPECIFIED? @Y30SSPJ*/
       IF MODID(1) ^= 'NO' THEN     /*NO-MODID = NO?         @Y30SSPJ*/
         REASON = '0';              /*DEFAULT REASON         @Y30SSPJ*/
       ELSE;                                               /*@Y30SSPJ*/
     ELSE                           /*YES-REASON SPECIFIED   @Y30SSPJ*/
       REASON = RCD(1);             /*USER SPECIFIED REASON  @Y30SSPJ*/
     IF REASON ^= '' THEN           /*REASON SPECIFIED?      @Y30SSPJ*/
       IF REASON(1:1) ^= '(' THEN   /*YES-IS IT IN A FIELD?  @Y30SSPJ*/
         DO;                        /*NO-NOT IN A FIELD      @Y30SSPJ*/
           CARG = REASON(1:1)||','; /*FORM SEARCH ARGUMENT   @Y30SSPJ*/
           HIT1 = INDEX(RNUM,CARG); /*SEARCH FOR NUMERIC     @Y30SSPJ*/
           IF HIT1 = 0 THEN         /*1ST CHAR NUMERIC?      @Y30SSPJ*/
             DO;                    /*NO-INVALID SPECIFIED   @Y30SSPJ*/
               RCODE = 12;          /*SET RETURN CODE        @Y30SSPJ*/
               ERRCOM = '   ***ERROR***   RCD(' || REASON ||
                 ') IS NOT AN VALID NUMERIC REASON CODE.'; /*@Y30SSPJ*/
               ANS(COMMENT(ERRCOM)) SKIP COL(2);           /*@Y30SSPJ*/
               REASON = '(' || REASON || ')';              /*@Y30SSPJ*/
               GO TO BV;            /*BYPASS FURTHER TESTS   @Y30SSPJ*/
             END;                                          /*@Y30SSPJ*/
           ELSE;                                           /*@Y30SSPJ*/
 BU:       HIT2=HIT2+1;             /*REASON CHAR POINTER    @Y30SSPJ*/
           IF REASON(HIT2:HIT2)='0' THEN /*REASON CHAR = 0?  @Y30SSPJ*/
             IF HIT2<LENGTH(REASON)THEN /*FINISHED ?         @Y30SSPJ*/
               GO TO BU;            /*NO-LOOP TIL DONE       @Y30SSPJ*/
             ELSE;                                         /*@Y30SSPJ*/
           ELSE;                                           /*@Y30SSPJ*/
           CARG = REASON(HIT2:LENGTH(REASON)) || ',';      /*@Y30SSPJ*/
           HIT1 = INDEX(RSNTBL,CARG); /*SEARCH FOR REASON    @Y30SSPJ*/
           IF HIT1 = 0 THEN         /*VALID REASON CODE?     @Y30SSPJ*/
             DO;                    /*NO-NONE FOUND          @Y30SSPJ*/
               RCODE = 12;          /*SET RETURN CODE        @Y30SSPJ*/
               ERRCOM = '   ***ERROR***   RCD(' || REASON ||
                 ') IS NOT A VALID REASON CODE.';          /*@Y30SSPJ*/
               ANS(COMMENT(ERRCOM)) SKIP COL(2);           /*@Y30SSPJ*/
               REASON = '(' || REASON || ')';              /*@Y30SSPJ*/
               GO TO BV;            /*SKIP FURTHER TESTS     @Y30SSPJ*/
             END;                                          /*@Y30SSPJ*/
           ELSE;                                           /*@Y30SSPJ*/
           REASON = 'RSN'||REASON(HIT2:LENGTH(REASON));    /*@Y30SSPJ*/
         END;                                              /*@Y30SSPJ*/
       ELSE;                                               /*@Y30SSPJ*/
     ELSE                           /*NO-NO REASON CODE      @Y30SSPJ*/
       IF ERROR = '' THEN           /*ERROR CODE SPECIFIED?  @Y30SSPJ*/
         IF MODID(1) = 'NO' THEN    /*NO-MODID = NO?         @Y30SSPJ*/
           DO;                      /*YES-DEFAULT TO 0       @Y30SSPJ*/
             ERROR = 'RCS';         /*SET ERROR TO 0         @Y30SSPJ*/
             REASON = 'RSN0';       /*SET REASON TO 0        @Y30SSPJ*/
           END;                                            /*@Y30SSPJ*/
         ELSE;                                             /*@Y30SSPJ*/
       ELSE;                                               /*@Y30SSPJ*/
 BV: IF ERROR ^= 'RCS' THEN         /*REASON NOT RCS?        @Y30SSPJ*/
       IF REASON ^= '' THEN         /*YES-REASON NOT NULL?   @Y30SSPJ*/
         IF REASON(1:1) ^= '(' THEN /*YES-REASON NOT FIELD?  @Y30SSPJ*/
           IF MODID(1) ^= 'NO' THEN /*YES-MODID NOT NO?      @Y30SSPJ*/
             DO;                    /*YES-ALL TRUE           @Y30SSPJ*/
               ANS('/') SKIP COL(MACCOL);                  /*@Y30SSPJ*/
               ANS('*') COL(MACCOL+1);                     /*@Y30SSPJ*/
               ANS(' SET CCAMODID=' || IGGMODID ||
                 ', CCAREASN=' || REASON || ', CCARETRN='||
                 ERROR ) COL(MACCOL+2);                    /*@Y30SSPJ*/
               ANS('*') COL(70);                           /*@Y30SSPJ*/
               ANS('/') COL(71);                           /*@Y30SSPJ*/
               ANS('CCAPROB=IGGSEGID+('||REASON||'*K256)+'||ERROR||';')
                  COL(MACCOL);                             /*@Y30SSPJ*/
               ANS(' ') COL(MACCOL+LENGTH(MACKEYS)+8);     /*@Y30SSPJ*/
               RETURN CODE(RCODE);  /* RETURN                @Y30SSPJ*/
             END;                                          /*@Y30SSPJ*/
           ELSE;                                           /*@Y30SSPJ*/
         ELSE;                                             /*@Y30SSPJ*/
       ELSE;                                               /*@Y30SSPJ*/
     ELSE;                                                 /*@Y30SSPJ*/
     IF MODID(1) ^= 'NO' THEN       /*MODID NOT NO?          @Y30SSPJ*/
       IF ERROR = 'RCS' THEN        /*YES-ERROR CODE RCS?    @Y30SSPJ*/
         IF REASON(1:1) ^= '(' THEN /*YES-REASON NOT FIELD?  @Y30SSPJ*/
           DO;                      /*YES-ALL TRUE           @Y30SSPJ*/
             ANS(COMMENT(' CLEAR CCAMODID, CCAREASN, AND CCARETRN'))
               COL(MACCOL+2);                              /*@Y30SSPJ*/
             ANS('CCAPROB=IGNOREIT;') SKIP COL(MACCOL);    /*@Y30SSPJ*/
             ANS(' ') COL(MACCOL+LENGTH(MACKEYS)+8);       /*@Y30SSPJ*/
             RETURN CODE(RCODE);    /* RETURN                @Y30SSPJ*/
           END;                                            /*@Y30SSPJ*/
         ELSE;                                             /*@Y30SSPJ*/
       ELSE;                                               /*@Y30SSPJ*/
     ELSE;                                                 /*@Y30SSPJ*/
     ANS('DO;') COL(MACCOL);                               /*@Y30SSPJ*/
     IF MODID(1) ^= 'NO' THEN       /*MODID NOT NO?          @Y30SSPJ*/
       DO;                          /*YES-GENERATE MODULE ID @Y30SSPJ*/
         ANS('CCAMODID='||IGGMODID||';')SKIP COL(MACCOL+2);/*@Y30SSPJ*/
         ANS(COMMENT('SET MODULE ID')) COL(MACCOL+17);     /*@Y30SSPJ*/
       END;                                                /*@Y30SSPJ*/
     ELSE;                                                 /*@Y30SSPJ*/
     IF REASON ^= '' THEN           /*REASON NOT NULL?       @Y30SSPJ*/
       DO;                          /*YES-GENTERATE REASON   @Y30SSPJ*/
         IF REASON(1:1) = '(' THEN  /*REASON IN A FIELD/REG? @Y30SSPJ*/
           REASON=REASON(2:LENGTH(REASON)-1);/*YES-DEL PAREN @Y30SSPJ*/
         ELSE;                                             /*@Y30SSPJ*/
         ANS('CCAREASN='||REASON||';') SKIP COL(MACCOL+2); /*@Y30SSPJ*/
         ANS(COMMENT('SET REASON CODE')) COL(MACCOL+LENGTH(REASON)+13);
       END;                                                /*@Y30SSPJ*/
     ELSE;                                                 /*@Y30SSPJ*/
     IF ERROR ^= '' THEN            /*ERROR NOT NULL?        @Y30SSPJ*/
       DO;                          /*YES-GENERATE ERROR     @Y30SSPJ*/
         ANS('CCARETRN='||ERROR||';') COL(MACCOL+2);       /*@Y30SSPJ*/
         ANS(COMMENT('SET RETURN CODE')) COL(MACCOL+LENGTH(ERROR)+13);
       END;                                                /*@Y30SSPJ*/
     ELSE;                                                 /*@Y30SSPJ*/
     ANS('END;') SKIP COL(MACCOL);                         /*@Y30SSPJ*/
     ANS(' ') COL(MACCOL+LENGTH(MACKEYS)+8);               /*@Y30SSPJ*/
     RETURN CODE(RCODE);            /* RETURN                @Y30SSPJ*/
 % END;                                                    /*@Y30SSPJ*/
 % SPAT:   MACRO;                       /*SPILL,PUSH & TRACK   Y02020*/
           ANS ('DO;') SKIP COL(MACCOL);                    /* Y02020*/
           ANS ('IF MNATFULL=ON THEN') SKIP COL(MACCOL+2);  /* Y02020*/
           ANS ('/*TRACKING TABLE FULL       */') COL(41);  /* Y02020*/
           ANS ('?CALLSF (ARFS);') SKIP RESCAN COL(MACCOL+2);
                                                            /* Y02020*/
           ANS ('/*YES,CALL SPILL ROUTINE    */') COL(41);  /* Y02020*/
           ANS ('GEN REFS(CCAMNAT)') SKIP COL(MACCOL+2);    /* Y02020*/
              ANS ('(MVC CCAMNAT(248,11),CCAMNAT+8(11));')
                 SKIP COL(MACCOL+5);                        /* Y02020*/
           ANS ('/*PUSH AND TRACK            */') COL(41);
                                                            /* Y02020*/
           ANS ('END;') SKIP COL(MACCOL);                   /* Y02020*/
 % END ;                                                    /* Y02020*/
 % GETREC: ADDREC: UPDREC: DELREC:
           MACRO KEYS(BY,FOR,RAB);
           DCL(IOI1,IOI2,IOI3,IOI4) CHAR;
           DCL OP(1) CHAR;
           OP(1) = MACNAME(1:1);
           ANS ('DO;') SKIP COL(MACCOL);
           IF    OP(1) ^= 'G'
             THEN GOTO IOIJ ;
           IF BY(1) = '*'
            THEN GOTO IOIB ;
           IF BY(1) = 'KEY' THEN
             DO;
     ANS ('CCAF1KEY = ON;     /* TRUE NAME GET*/')        /* @Y30SSDW*/
         SKIP COL(MACCOL);                                /* @Y30SSDW*/
     ANS ('CCAF1KGE = OFF;    /*NOT GET NEXT */')         /* @Y30SSDW*/
         SKIP COL(MACCOL);                                /* @Y30SSDW*/
               GOTO IOIB ;
             END;
           IF BY(1) = 'KGE' THEN
             DO;
     ANS ('CCAF1KEY = ON;     /*TRUE NAME GET*/')         /* @Y30SSDW*/
         SKIP COL(MACCOL);                                /* @Y30SSDW*/
     ANS ('CCAF1KGE = ON;     /* GET NEXT    */')         /* @Y30SSDW*/
         SKIP COL(MACCOL);                                /* @Y30SSDW*/
               GOTO IOIB ;
             END;
     ANS ('CCAF1KEY = OFF;    /*GET BY CI NO. */')        /* @Y30SSDW*/
         SKIP COL(MACCOL);                                /* @Y30SSDW*/
     ANS ('CCAF1KGE = OFF;    /*NOT GET NEXT */')         /* @Y30SSDW*/
         SKIP COL(MACCOL);                                /* @Y30SSDW*/
 IOIB:     IF FOR(1) = '*'
             THEN GOTO IOIJ ;
             IF FOR(1) = 'NUP'
             THEN
     ANS ('CCAF1UP = OFF;     /*NOT GET UPD  */') SKIP COL(MACCOL);
             ELSE
     ANS ('CCAF1UP = ON;      /*GET FOR UPD  */') SKIP COL(MACCOL);
 IOIJ:     IF RAB(1) = ''
             THEN GOTO IOIM ;
           IOI1 = RAB(1) ;
           IF LENGTH(IOI1) = 1
             THEN IOI1 = 'CCARAB' || RAB(1) ;
     ANS ('CCAURAB = ADDR( IOI1 ) ; /*SET RAB PTR*/')
             SKIP RESCAN COL(MACCOL) ;
 IOIM:
           IF OP(1) = 'G' THEN          /* GETREC ?                 */
             DO;                        /* YES                      */
               ANS ('?CALLSF (GET);') SKIP RESCAN COL(MACCOL+2);
               ANS ('/* GET RECORD             */') COL(41);
               GOTO IOIP;
             END;
           IF OP(1) = 'A' THEN          /* ADDREC ?                 */
             DO;                        /* YES                      */
               ANS ('?CALLSF (PAD);') SKIP RESCAN COL(MACCOL+2);
               ANS ('/* PUT-ADD RECORD         */') COL(41);
               GOTO IOIP;
             END;
           IF OP(1) = 'U' THEN          /* UPDREC ?                 */
             DO;                        /* YES                      */
               ANS ('?CALLSF (PUPC);') SKIP RESCAN COL(MACCOL+2);
               ANS ('/* PUT-UPDATE RECORD      */') COL(41);
               GOTO IOIP;
             END;
                                        /* DELREC                   */
           ANS ('?CALLSF (PDE);') SKIP RESCAN COL(MACCOL+2);
           ANS ('/* PUT-DELETE RECORD        */') COL(41);
 IOIP: ANS ('END;') SKIP COL(MACCOL);
 % END ;
 % GETMS: FREEMS:
           MACRO KEYS(CLASS,RFR);                           /* Y02020*/
           DCL (SPL, POOLTYPE) CHAR;    /*SUBPOOL VARIABLES  @Y30SSDW*/
           IF CLASS(1) = 'S' THEN                         /* @Y30SSDW*/
             DO;                                          /* @Y30SSDW*/
               SPL = 'SSUBPOOL';                          /* @Y30SSDW*/
               POOLTYPE = 'SYSTEM SUBPOOL 241';           /* @Y30SSDW*/
             END;                                         /* @Y30SSDW*/
           ELSE;                                          /* @Y30SSDW*/
           IF CLASS(1) = 'U' THEN                         /* @Y30SSDW*/
             DO;                                          /* @Y30SSDW*/
               SPL  = 'USUBPOOL';                         /* @Y30SSDW*/
               POOLTYPE = 'USER SUBPOOL 252';             /* @Y30SSDW*/
             END;                                         /* @Y30SSDW*/
           ELSE;                                          /* @Y30SSDW*/
           IF CLASS(1) = 'F' THEN                         /* @ZA29164*/
             DO;                                          /* @ZA29164*/
               SPL = 'FSUBPOOL';                          /* @ZA29164*/
               POOLTYPE = 'SYSTEM SUBPOOL 229';           /* @ZA29164*/
             END;                                         /* @ZA29164*/
           ELSE;                                          /* @ZA29164*/
           ANS ('R1 = ADDR(CCAMSSPL);') SKIP COL(MACCOL); /* @Y30SSDW*/
           ANS ('/* POINT TO STRG MGMT P.L.  */') COL(41);/* @Y30SSDW*/
           IF SPL ^= '' THEN
             DO;
               ANS ('CCAMNSPL = SPL;') SKIP COL(MACCOL);  /* @Y30SSDW*/
               ANS ('/* STORE SUBPOOL VALUE      */') COL(41);
             END;
           IF MACNAME = 'FREEMS' THEN
             DO;
               IF RFR(1)^='NO' THEN     /*RFR ALLOWED          Y02020*/
                 ANS ('?SPAT;') SKIP RESCAN COL(MACCOL);
                                        /*GEN SPILL,PUSH,TRK   Y02020*/
               ANS ('/') SKIP COL(MACCOL);                /* @Y30SSDW*/
               ANS ('*') COL(MACCOL+1);                   /* @Y30SSDW*/
               ANS (' RELEASE MAIN STORAGE FROM ' || POOLTYPE)
                   COL(MACCOL+3);                         /* @Y30SSDW*/
               ANS ('*') COL(70);                         /* @Y30SSDW*/
               ANS ('/') COL(71);                         /* @Y30SSDW*/
               ANS ('GEN(FREEMAIN L,MF=(E,(1))) ;') SKIP COL(MACCOL);
                                                            /* Y02020*/
             END;
           ELSE
             DO;
               ANS ('/') SKIP COL(MACCOL);                /* @Y30SSDW*/
               ANS ('*') COL(MACCOL+1);                   /* @Y30SSDW*/
               ANS (' ALLOCATE MAIN STORAGE FROM ' || POOLTYPE)
                   COL(MACCOL+3);                         /* @Y30SSDW*/
               ANS ('*') COL(70);                         /* @Y30SSDW*/
               ANS ('/') COL(71);                         /* @Y30SSDW*/
               ANS ('GEN(GETMAIN LC,MF=(E,(1)));') SKIP COL(MACCOL);
                                                            /* Y02020*/
               ANS ('/* GET MAIN STORAGE SPACE   */') COL(41);
               ANS ('GEN(LR R1,R15);')  SKIP COL(MACCOL);
               ANS ('/* PICK UP RETURN CODE      */') COL(41);
               ANS ('IF R1 ^= NOGETMSE THEN') SKIP COL(MACCOL);
               ANS ('/* CHECK FOR ERROR RETURN   */') COL(41);
               ANS ('?CATPROB ECD(RCINSSWA) RCD(2);')      /*@Y30SSPJ*/
                    SKIP RESCAN COL(MACCOL);               /*@Y30SSPJ*/
               ANS ('/* INSUFFICIENT CORE ERROR  */')      /*@Y30SSPJ*/
                   COL(41);                                /*@Y30SSPJ*/
               ANS(' ') COL(MACCOL+LENGTH(MACKEYS)+6);     /*@Y30SSPJ*/
               IF RFR(1)^='NO' THEN     /*RFR ALLOWED          Y02020*/
                 DO;                                        /* Y02020*/
                   ANS ('ELSE') SKIP COL(MACCOL);           /* Y02020*/
                   ANS ('/* GETMAIN SUCCESSFUL            */')
                       COL(37);                           /* @Y30SSDW*/
                   ANS ('DO;') SKIP COL(MACCOL+2);          /* Y02020*/
                   ANS ('?SPAT;') SKIP RESCAN COL(MACCOL+4);/* Y02020*/
                   IF CLASS(1)='S' THEN /*CLASS S CORE         Y02020*/
                     DO;                                    /* Y02020*/
                       ANS ('MNATSCLS=ON;') SKIP COL(MACCOL+4);
                                                            /* Y02020*/
                       ANS ('/*INDICATE CLASS S CORE     */')
                            COL(41);                        /* Y02020*/
                     END;                                   /* Y02020*/
                   ANS ('END;') SKIP COL(MACCOL+2);         /* Y02020*/
                 END;                                       /* Y02020*/
             END;
 % END;
 % CKADD:
         MACRO KEYS(START,LENG);
         DCL SYSTEM CHAR EXTERNAL;
         DCL (STADD,EDADD) CHAR;
         STADD=START(1);
         EDADD=LENG(1);
 CKN1:   VALC2 = 'V1' || MACINDEX;
         ANS('RFY (R7,R8,R9,R10,R13,R14,R15) RESTRICTED;')  /* Y02020*/
               SKIP COL(MACCOL);                            /* Y02020*/
         ANS('R7=R4;') SKIP COL(MACCOL);                    /* Y02020*/
         ANS('/* SAVE REGISTERS */') COL(41);               /* Y02020*/
         ANS('R8=R11;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('R9=R12;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('R10=R13;') SKIP COL(MACCOL);                  /* Y02020*/
         ANS('GEN REFS(PSALITA,FLC)                         /* Y02020*/
               SETS(R11,R12,R13,R14,R15)                    /* Y02020*/
               SEQFLOW;') SKIP COL(MACCOL);                 /* Y02020*/
         ANS('SETLOCK OBTAIN,TYPE=LOCAL,MODE=UNCOND,')      /* Y02020*/
               SKIP COL(10);                                /* Y02020*/
         ANS('X') COL(72);                                  /* Y02020*/
         ANS('RELATED=(''AFTER VALIDITY CHECK ROUTINE'')')  /* Y02020*/
               COL(16);                                     /* Y02020*/
         ANS('@ENDGEN;') SKIP COL(MACCOL);                  /* Y02020*/
         ANS('R11=R8;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('/* RESTORE CCA BASE */') COL(41);             /* Y02020*/
         ANS('R12=R9;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('/* RESTORE MODULE BASE */') COL(41);          /* Y02020*/
         ANS('RFY (R14,R15) UNRESTRICTED;')                 /* Y02020*/
               SKIP COL(MACCOL);                            /* Y02020*/
         ANS('RFY (R1) RESTRICTED;') SKIP COL(MACCOL);      /* Y02020*/
         IF STADD^='R1' THEN ANS('R1=STADD;')               /* Y02020*/
               SKIP COL(MACCOL);                            /* Y02020*/
         ANS('R13=R2;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('R2=STADD+EDADD-1;') SKIP COL(MACCOL);         /* Y02020*/
         ANS('/* ENDING ADDRESS TO CHECK */') COL(41);      /* Y02020*/
         ANS('R4=CCATCB;') SKIP COL(MACCOL);                /* Y02020*/
         ANS('CALL CVT0VL00;') SKIP COL(MACCOL);            /* Y02020*/
         ANS('/* INVOKE VALIDITY CHK RTN */') COL(41);      /* Y02020*/
         ANS('R2=R13;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('RFY (R1) UNRESTRICTED;') SKIP COL(MACCOL);    /* Y02020*/
         ANS('BC (8,VL'||MACINDEX||');') SKIP COL(MACCOL);  /* Y02020*/
         ANS('/* SKIP IF AREA VALID */') COL(41);           /* Y02020*/
         ANS('?CATPROB ECD(RCBLKVCK) RCD(2);') SKIP COL(MACCOL);
                                        /*USE PROB DET MACRO @Y30SSPJ*/
         ANS('/* SET RETURN CODE */') COL(41);              /* Y02020*/
         ANS('VL'||MACINDEX||':') SKIP COL(MACCOL);         /* Y02020*/
         ANS('RFY (R14,R15) RESTRICTED;') SKIP COL(MACCOL); /* Y02020*/
         ANS('GEN REFS(PSALITA,FLC)                         /* Y02020*/
               SETS(R11,R12,R13,R14,R15)                    /* Y02020*/
               SEQFLOW;') SKIP COL(MACCOL);                 /* Y02020*/
         ANS('SETLOCK RELEASE,TYPE=LOCAL,') SKIP COL(10);   /* Y02020*/
         ANS('X') COL(72);                                  /* Y02020*/
         ANS('RELATED=(''BEFORE VALIDITY CHECK ROUTINE'')') /* Y02020*/
               COL(16);                                     /* Y02020*/
         ANS('@ENDGEN;') SKIP COL(MACCOL);                  /* Y02020*/
         ANS('R4=R7;') SKIP COL(MACCOL);                    /* Y02020*/
         ANS('/* RESTORE REGISTERS */') COL(41);            /* Y02020*/
         ANS('R11=R8;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('R12=R9;') SKIP COL(MACCOL);                   /* Y02020*/
         ANS('R13=R10;') SKIP COL(MACCOL);                  /* Y02020*/
         ANS('RFY (R13,R14,R15) UNRESTRICTED;')             /* Y02020*/
               SKIP COL(MACCOL);                            /* Y02020*/
         ANS('RFY (R7,R8,R9,R10) UNRSTD;') SKIP COL(MACCOL);
 %END;
 % RECID   ='CCASRID' ;
 % RECKY    = 'CCASRCH' ;
 % VOLKY ='CCASRCH = CCASRCH  && CCASRCH; /*CLEAR SRCH ARG FLD*/
           CCASRCH(1:6)' ;
 % IDCOUNT = 'CCAASCIK' ;
 % INVOKE = 'RFY (R5,R6,R7,R8,R9,R10) RSTD;
             RFY (R5,R6,R7,R8,R9,R10) UNRSTD;
             CALL';

 % CATERR: MACRO KEYS(TYPE,REASON,CCHN,R); /*CATALOG ERROR MACRO    */
           IF R(1)^='' THEN
             DO;
               ANS ('/****************************/') SKIP COL(41);
               ANS ('/' || '*' || R(1) ) SKIP COL(41);
               ANS ('*' || '/') COL(69);
               ANS ('/****************************/') SKIP COL(41);
             END;
           ANS ('DO;') SKIP COL(MACCOL);
           IF TYPE(1)= '' THEN
             GOTO CB;
           IF TYPE(1) ^= '*' THEN
             DO;
               ANS ('CCACD1='|| TYPE(1) ||';') SKIP COL(MACCOL+2);
               ANS ('/*STORE ERROR TYPE CODE     */') COL(41);
             END;
 CB:       IF REASON(1) = '' THEN
             GOTO CC;
           IF REASON(1) ^= '*' THEN
             DO;
               ANS ('CCACD2='|| REASON(1) ||';') SKIP COL(MACCOL+2);
               ANS ('/* STORE REASON CODE        */') COL(41);
             END;
 CC:       IF CCHN(1) = 'YES' THEN
             DO;
               ANS ('CCACRRP=128;') SKIP COL(MACCOL+2);
               ANS ('/*INCLUDE CALLER CHN IN MSG */') COL(41);
             END;
           ANS ('?CALLSF (EMSG);') SKIP RESCAN COL(MACCOL+2);
           ANS ('/*ISSUE WTP ERROR MSG       */') COL(41);
           ANS ('END;') SKIP COL(MACCOL);
 % END;

 @LIST ON;
