 /********************************************************************/
 /*                                                                  */
 /*   MACRO NAME = IGGMCMWA                                          */
 /*                                                                  */
 /*   DESCRIPTIVE NAME = VSAM CATALOG MANAGEMENT SERVICES WORK AREAS */
 /*                                                                  */
 /*   FUNCTION = CONTAINS MAPPING STRUCTURES FOR THE CMS EXTENSION   */
 /*              WORK AREA, THE CMS DELETE WORK AREA, THE CMS        */
 /*              DELETE-ERASE WORK AREA, THE ENQUEUE PARAMETER       */
 /*              LIST, AND THE CATALOG UPGRADE MANAGEMENT PARAMETER  */
 /*              LIST AND ASSOCIATED WORK AREA.                      */
 /*                                                                  */
 /*   CONTROL BLOCK STRUCTURE = THE CMSWA AND THE DELWKA ARE POINTED */
 /*                             TO BY THE CCA (CCACMSWA).  THE       */
 /*                             CUMPLWA IS ALSO POINTED TO BY THE    */
 /*                             CCA (CCACUMPL).                      */
 /*                                                                  */
 /*   INCLUDED MACROS = NONE                                         */
 /*                                                                  */
 /*   METHOD OF ACCESS = PL/S - NO DECLARES NECESSARY                */
 /*                                                                  */
 /*   CREATED BY: CORE OBTAINED AND FREED BY IGG0CLAT (IGGPCDVR)     */
 /*                                                                  */
 /*   STATUS = VS/2 RELEASE 4   (CHANGE FLAG @Z40RSXX)       @Z40RSRC*/
 /*                                                                  */
 /*   DATE OF LAST CHANGE = 23 FEB 78                        @ZA30382*/
 /*                                                                  */
 /*   A 871322,871324                                        @ZA30382*/
 /*                                                                  */
 /********************************************************************/
 /********************************************************************/
 /* DECLARATION OF BASE REGISTER FOR DEFINE SIZE PARAMETER TABLE MAP */
 /********************************************************************/
    DECLARE DCWABASE REG(4) PTR(31); /* ADDR OF DEVICE CHARACTERISTICS
                                        AND WORKAREA ARRAY ELEMENT   */
 /********************************************************************/
 /*          DECLARATION OF DEFINE SIZE PARAMETER TABLE MAP          */
 /********************************************************************/
    DECLARE
      1 DSPTBL BASED(DCWABASE),     /* DEFINE SIZE PARAMETER TABLE   */
        2 DSDEVCHA CHAR(20),        /* DEVICE CHARACTERISTICS AREA   */
        2 DSMPHYBS FIXED(15),       /* MAXIMUM ALLOWABLE PHYSICAL BLOCK
                                       SIZE FOR THIS DEVICE TYPE     */
        2 DSPHYBPT (4) FIXED(15),   /* ARRAY OF PHYSICAL BLOCKS PER
                                       TRACK FOR 512, 1024, 2048, AND
                                       4096 BYTE BLOCKS RESPECTIVELY */
        2 DSNPHYBS FIXED(15);       /* NUMBER OF ALLOWABLE PHYSICAL
                                       BLOCK SIZES FOR THIS DEVICE   */
 /********************************************************************/
 /*        DECLARATION  FOR  THE  CMS  EXTENSION  WORK  AREA         */
 /********************************************************************/
    DECLARE
      1 CMSWA BASED(CCACMSWA),      /* CMS EXTENSION WORKAREA        */
        2 CMWFLAGS BIT(8),          /* OPTION & PROCESS INDICATE     */
          3 CMFGDLSC BIT(1),        /* TEMP SECURITY DELETE          */
          3 CMPROCM  BIT(1),        /* PROCESSING CONTROL - DSPC     */
          3 CMPROC   BIT(1),        /* PROCESSING CONTROL - DSPC     */
          3 CMSRACDL BIT(1),        /* RAC-INDICATED DELETE  @Z40RSTC*/
          3 CMSUALOC BIT(1),        /* UNALLOC REQUIRED      @Z40RSRC*/
        2 CMCISREQ PTR(8),          /* CI'S ORIGINALLY OBTAINED      */
                                    /*   FOR DEFINE ONLY             */
        2 CMCISUSE PTR(8),          /* NO. OF CI'S PROCESSED         */
          3 CMWRKATR BIT(8),        /* AMDSB DATA ATTRIBUTES         */
            4 CMAMDDST BIT(1),      /* KEY, CHRONOLOGIC              */
            4 CMAMDWCK BIT(1),      /* WRITE CHECK                   */
            4 CMAMDSDT BIT(1),      /* SEQUENCE SET WITH DATA        */
            4 CMAMDRPL BIT(1),      /* REPLICATION                   */
            4 CMAMDORD BIT(1),      /* USE VOLUMES IN LIST ORDER     */
            4 CMAMDRNG BIT(1),      /* KEY RANGE DATA SET            */
        2 CMWACI (3) PTR(24) BDY(BYTE), /* CI'S FOR DEFINE USAGE ONLY*/
 /*                                                          @Z40RSTC*/
 /*   END OF NON-OVERLAID FIELDS IN CMS WORK AREA            @Z40RSRC*/
 /*                                                          @Z40RSRC*/
        2 CMWFVTPT PTR(31),         /* SECONDARY FVT BASE            */
        2 CMASCFPL PTR(31),         /* ADDRESS OF ASSOCIATION FPL    */
        2 CMCPLFVT PTR(31),         /* ADDRESS OF CALLER FVT-SAVED   */
          3 CMEXTFLS PTR(31),       /* EXTRACT CORE REFERENCE        */
        2 CMNRMEXT PTR(31),         /* ADDRESS OF NORMAL EXTENT LIST */
          3 CMLDTFPL PTR(31),       /* LOW DATA FPL ADDRESS - DEFC   */
            4 CMRNMLST PTR(31),     /* RENAME PARAMETER LIST REFER   */
        2 CMSSEXTS PTR(31),         /* ADDRESS OF SS W/DATA EXT LIST */
          3 CMHDTFPL PTR(31),       /* HIGH DATA FPL ADDRESS - DEFC  */
            4 CMTSTFPL PTR(31),     /* TEST FPL ADDRESS              */
        2 CMCANVOL PTR(31),         /* ADDRESS OF CANDIDATE VOL LIST */
          3 CMHIXFPL PTR(31),       /* HIGH INDEX FPL ADDRESS - DEFC */
            4 CMDTAFPL PTR(31),     /* DATA FPL ADDRESS              */
        2 PROCESS  CHAR(2),         /* LOOP CONTROL CHARACTERS       */
        2 SPACINDC CHAR(2),         /* SPACE OBTAINED INDICATOR      */
        2 CMRNGCTR FIXED(15),       /* RELATIVE SEQUENCE NUMBER FOR  */
                                    /*   RANGE BEING PROCESSED       */
          3 CMADDVOL FIXED(15),     /* COUNT OF VOLS TO ADD - ALTER  */
        2 CMVOLCTR FIXED(15),       /* RELATIVE SEQUENCE NO. FOR VOL */
                                    /*   ENTRY BEING PROCESSED       */
          3 CMRMVVOL FIXED(15),     /* REMOVE VOLUME COUNT - ALTER   */
        2 CMSPCFVT PTR(31),         /* REFERENCE TO A SPACE FVT      */
        2 CMSDFSPC PTR(32),         /* DEFINE - DEFINE SPACE FEEDBACK*/
          3 CMSWAFLG BIT(8),        /* INTERFACE FLAGS - PASSED      */
          3 CMSDEFCI PTR(24) BDY(BYTE), /* CI OF UNIQUE ENTRY - PASS */
        2 CMSCRAVI CHAR(6),         /* CRA VOLID             @Y30SSSG*/
        2 CMSCRADT CHAR(4),         /* CRA DEVTYPE           @Y30SSSG*/
        2 * CHAR(2),                /* RESERVED              @Y30SSSG*/
        2 CMVLLIST,                 /* SINGLE ENTRY VOLUME LIST      */
          3 CMVLLTLN FIXED(15),     /* LENGTH OF VOLUME LIST         */
          3 CMVLRTCD FIXED(15),     /* RETURN CODE FIELDS            */
            4 CMVLRT1 PTR(8),       /* NORMAL RETURN CODE            */
            4 CMVLRT2 PTR(8),       /* RESERVED                      */
          3 CMVLVOL CHAR(6),        /* VOLUME SERIAL NUMBER          */
        2 CMTOTVOL FIXED(15) BDY(BYTE), /* TOTAL NO. OF VOL SERS     */
        2 CMSPCATR (2) CHAR(16) BDY(WORD), /* SPACE ATTRIBUTES       */
          3 CMPHYBLK PTR(32),       /* PHYSICAL BLOCK SIZE           */
          3 CMSWADTY PTR(32),       /* DEVICE TYPE                   */
          3 CMBLKTRK FIXED(15),     /* BLOCKS PER TRACK              */
          3 CMTRKPCA FIXED(15),     /* TRACKS PER CONTROL AREA       */
          3 CMTRKCYL FIXED(15),     /* TRACKS PER CYLINDER           */
        2 CMCNVLEN FIXED(15),       /* SIZE OF CANDIDATE VOLUME LIST */
        2 CMCNVNO  FIXED(15),       /* NO. OF CANDIDATE VOLUMES      */
        2 CMFPLIDX PTR(31),         /* ADDRESS OF FPL INDEX          */
        2 *        CHAR(2),         /* UNUSED - PRESERVE ALIGNMENT   */
        2 WORKPROC CHAR(2),         /* LOOP CONTROL - IGGPDEXD       */
        2 CMEXTLST PTR(31),         /* EXTENT LIST REFERENCE IN FPL  */
        2 CMSRCCOR PTR(31),         /* CATALOG DEFINE RECORD CORE PTR*/
        2 CMDEFSWA PTR(31),         /* PTR TO DEFINE SPACE WORKAREA  */
        2 CMWASTOR CHAR(8) BDY(WORD), /* TIME WORKAREA               */
          3 CMPOT    FIXED(31),     /* WORKAREA - POWER-2 - DSPC     */
            4 CMDCONV    FIXED(31), /* VALUE TO CONVERT   - DSPC     */
              5 CMIXFANO FIXED(31), /* INDEX CI FANOUT    - DSPC     */
                6 CMWT FIXED(31),   /* TIME WORKAREA                 */
          3 CMPOTX   FIXED(31),     /* WORKAREA - POWER-2 - DSPC     */
            4 CMPBI  FIXED(31),     /* BLOCK SIZE INDEX   - DSPC     */
              5 CMIXFANS FIXED(31), /* SQUARE ROOT OF CMIXFANO       */
                6 CMNCITL FIXED(31), /* NO. CI THIS INDEX LEVEL-DSPC */
        2 CMWABYTB CHAR(4) BDY(WORD), /* DATE WORKAREA               */
          3 CMPBIX   FIXED(31),     /* BLOCK SIZE CONTROL - DSPC     */
            4 CMNEST   FIXED(31),   /* NO. OF INDEX SECTIONS - DSPC  */
              5 CMFANT   FIXED(31), /* WORKAREA FOR CMIXFANS         */
                6 CMSPCQTY FIXED(31),  /* WORKAREA FOR SPCQTY - DSPC */
                  7 CMNCILL FIXED(31), /* NO. CI LAST IX LEVEL- DSPC */
        2 CMWGENAM CHAR(44),        /* GENERATED ENTRYNAME           */
          3 CMWFIXID CHAR(12),      /* FIXED IDENTIFIER              */
          3 CMWDATE,                /* DATE                          */
            4 CMWYEAR CHAR(2),      /* YEAR                          */
            4 CMWDAY CHAR(3),       /* DAY                           */
          3 CMWTIME,                /* TIME                          */
            4 CMWDELM1 CHAR(2),     /* DELIMITER .T                  */
            4 CMWTIME1 CHAR(7),     /* FIRST 7 DIGITS                */
            4 CMWDELM2 CHAR(2),     /* DELIMITER                     */
            4 CMWTIME2 CHAR(7),     /* SECOND 7 DIGITS               */
          3 CMWPAD CHAR(9),         /* PAD OF BLANKS                 */
 /********************************************************************/
 /*         DECLARATION OF CMS DEFINE WORKAREA USED FOR FPL'S        */
 /*         AND FIELD VALUES                                         */
 /********************************************************************/
        2 DCWIX    PTR(15) BDY(WORD), /* WORKAREA INDEX              */
        2 *        PTR(15),         /* RESERVED                      */
        2 DCWAREPT PTR(31),         /* POINTER TO ARRAY ELEMENT TO   */
                                    /*   BE PROCESSED                */
        2 DCWFPLPT PTR(31),         /* PTR TO FPL TO BE PROCESSED    */
                                    /*   IN ARRAY ELEMENT            */
        2 DCWPLSPS,                 /* SET OF SPACE PARAMETERS       */
                                    /*   FPL AND FIELD VALUE         */
          3 DCWPLSPL,               /* SPACE PARAMETER FPL           */
            4 DCWPLSPH CHAR(LENGTH(CTGFL)), /* FPL HEADER            */
            4 DCWPLSPA CHAR(LENGTH(CTGFLDAT)), /* FPL FIELD PAIR     */
          3 DCWPLSPV CHAR(7),       /* SPACE PARAMETER FIELD VALUE   */
          3 *        CHAR(1),       /* RESERVED                      */
        2 DCWPLBFS,                 /* SET OF BUFFER SIZE FPL        */
                                    /*   AND FIELD VALUE             */
          3 DCWPLBFL,               /* BUFFER SIZE FPL               */
            4 DCWPLBFH CHAR(LENGTH(CTGFL)), /* FPL HEADER            */
            4 DCWPLBFA CHAR(LENGTH(CTGFLDAT)), /* FPL FIELD PAIR     */
          3 DCWPLBFV CHAR(4),       /* BUFFER SIZE FIELD VALUE       */
        2 DCWPLEDS,                 /* EXPIRATION DATE SET OF        */
                                    /*   FPL AND FIELD               */
          3 DCWPLEDL,               /* EXPIRATION DATE FPL           */
            4 DCWPLEDH CHAR(LENGTH(CTGFL)),   /* FPL HEADER          */
            4 DCWPLEDA CHAR(LENGTH(CTGFLDAT)), /* FPL FIELD PAIR     */
          3 DCWPLEDV CHAR(3),       /* EXPIRATION DATE FIELD         */
          3 *        CHAR(1),       /* RESERVED                      */
        2 DCWPLARS,                 /* SET OF AVG LOGICAL RECRD SIZE */
                                    /*   FPL AND FIELD VALUE         */
          3 DCWPLARL,               /* AVG LOGICAL RECORD SIZE FPL   */
            4 DCWPLARH CHAR(LENGTH(CTGFL)), /* FPL HEADER            */
            4  DCWPLARA CHAR(LENGTH(CTGFLDAT)), /* FPL FIELD PAIR    */
          3 DCWPLARV PTR(31),       /* AVG LOGICAL RECORD SIZE VALUE */
        2 DCWPLCDS,                 /* CREATION DATE SET OF FPL      */
                                    /*   AND FIELD                   */
          3 DCWPLCDL,               /* CREATION DATE FPL             */
            4 DCWPLCDH CHAR(LENGTH(CTGFL)), /* FPL HEADER            */
            4 DCWPLCDA CHAR(LENGTH(CTGFLDAT)), /* FPL FIELD PAIR     */
          3 DCWPLCDV CHAR(3),       /* CREATION DATE FIELD           */
          3 *        CHAR(1),       /* RESERVED                      */
        2 CMSDSPTB (2) CHAR(LENGTH(DSPTBL)), /* ARRAY FOR DATA AND
                                       INDEX DEVICE CHARACTERISTICS
                                       AND WORK VALUES               */
        2 CMBLPTKS FIXED(15),       /* BLOCKS PER TRACK FOR          */
                                    /*   SEQUENCE SET ON DATA DEVICE */
        2 CMUCATCI CHAR(3),         /* UCAT ENTRY CI NUMBER          */
          3 CMDEFACI CHAR(3),       /* ALIEN ENTRY CI NUMBER         */
        2 *       CHAR(3),          /* RESERVED                      */
        2 CMSCTREC(20) PTR(31),     /* CATALOG RECORD ADDRESS ARRAY
                                                               Y02020*/
        2 CMBYTTRK FIXED(31),       /* BYTES PER TRACK         Y02020*/
        2 CMBYTAU FIXED(31),        /* BYTES PER ALLOC UNIT  @Z40RSTC*/
 /*                                                          @Z40RSRC*/
 /*   RAC-RELATED FIELDS                                     @Z40RSRC*/
 /*                                                          @Z40RSRC*/
        2 CMSRACBF(3) PTR(31),      /* BUFFER TO PREVENT     @Z40RSTC
                                  OVERLAY BY DELETE WRK AREA @Z40RSTC*/
        2 CMSRAC,                   /* RAC FIELDS            @Z40RSRC*/
          3 CMSRETCD PTR(8),        /* RAC RETURN CODE       @Z40RSRC*/
          3 CMSREACD PTR(8),        /* RAC REASON CODE       @Z40RSRC*/
          3 * CHAR(2),              /* RESERVED              @Z40RSRC*/
          3 CMSRDSNA PTR(31),       /* DSNAME ADDR FOR RAC   @Z40RSRC*/
          3 CMSRDSVL PTR(31),       /* VOLSER ADDR FOR RAC   @Z40RSTC*/
          3 CMSWARFV PTR(31),       /* CURRENT FVT FOR RAC   @Z40RSRC
                                       DEFINITION PURPOSES   @Z40RSRC*/
          3 CMSWRMAC PTR(31),       /* ADDR OF RDDFLIST      @ZA30382*/
          3 * CHAR(12),             /* RESERVED              @ZA30382*/
          3 CMSROBJA,               /* RAC ARRAY NAME        @Z40RSTC*/
            4 CMSROBJN(3) CHAR(44), /* RAC OBJECT NAME ARRAY @Z40RSTC*/
                5 CMSROBJI CHAR(1), /* INITIAL CHAR STATE ID @Z40RSTC*/
                5 * CHAR(43),       /* OBJECT NAME (LOW      @Z40RSTC
                                       ORDER 43 CHARACTERS)  @Z40RSTC*/
          3 CMSDYNAL PTR(31);       /* ADDR OF DYNAMIC ALLOCATION
                                       WORK AREA             @Z40RSRC*/
 /********************************************************************/
 /*     CONSTANTS USED TO SET AND/OR TEST FIELDS DECLARED ABOVE      */
 /********************************************************************/
    DECLARE                         /* CHARACTER CONSTANTS           */
      CMWENTPD CHAR(10) CONSTANT('          '), /* ENTRYNAME PAD OF
                                                   BLANKS            */
      CMWENTDL CHAR(2)  CONSTANT('.T'), /* DELIMITER OF TIME FIELD   */
      CMWENTCD CHAR(12) CONSTANT('VSAMDSET.DFD'), /* FIXED PORTION OF
                                                   ENTRYNAME         */
      EDNULVAL BIT(24) CONSTANT('00000F'X); /* EXPIRATION DATE
                                               NULLVALUE             */
    %IF DELWKA99 = ''               /*IF DELWKA NOT REQUIRED @Y30SSPJ*/
      %THEN %GOTO DELWKA01;         /* THEN DONT GENERATE IT @Y30SSPJ*/
    %STOPFMT1:;                     /* SHOW NEXT COMMENT     @Y30SSPJ*/
 /********************************************************************/
 /*          DECLARATION OF THE CMS DELETE WORK AREA                 */
 /********************************************************************/
    DECLARE
      1 DELWKA CHAR(516) BDY(WORD) BASED(CCACMSWA), /* CMS   @Z40RSTC*/
                                    /* DELETE AREA           @Z40RSTC*/
                                    /* OVERLAPS CMSWA        @Y30SSPJ*/
       2 * CHAR(12),                /* DO NOT USE THIS FIELD @Z40RSRC*/
       2 DELNMOBJ PTR(8),           /* NUM OF ASSOCIATED OBJ @Y30SSPJ*/
       2 DELR3SAV PTR(8),           /* R3 SAVE AREA          @Y30SSPJ*/
       2 DELRTYPE CHAR(1),          /* CTGTYPE SAVE AREA     @Y30SSPJ*/
       2 DELERRIX PTR(8),           /* INDEX TO ERR ARRAY    @Y30SSPJ*/
       2 * CHAR(1),                 /* DELETE FLAGS          @Y30SSPJ*/
        3 DELCODE BIT(1),           /* ERROR CODE FLAG1      @Y30SSPJ*/
        3 DELCODE2 BIT(1),          /* ERROR CODE FLAG2      @Y30SSPJ*/
        3 DELKYRNG BIT(1),          /* KEY RANGE DATA SET    @Y30SSPJ*/
        3 DELCAT BIT(1),            /* DELETE COPIED CATALOG @Y30SSPJ*/
        3 DELGNMR BIT(1),           /* GET NUM OF PATHS      @Y30SSPJ*/
        3 DELCODE3 BIT(1),          /* ERROR CODE FLAG3      @Y30SSPJ*/
        3 DELCODE4 BIT(1),          /* ERROR CODE FLAG4      @Y30SSPJ*/
        3 DELCODE5 BIT(1),          /* ERROR CODE FLAG5      @Y30SSPJ*/
          4 DELSPSCR BIT(1),        /* VTOC MODIFIED         @Y30SSPJ*/
       2 DELSGRRN FIXED(15)BDY(BYTE),/* SPACE GRP RELREPNO   @Y30SSPJ*/
        3 DELI PTR(8),              /* DO LOOP COUNTER I     @Y30SSPJ*/
         4 DELSPFLG BIT(8),         /* DELETE SPACE FLAG     @Y30SSPJ*/
          5 DELNONVS BIT(1),        /* NON-VSAM FOUND IN CRA @Y30SSPJ*/
          5 * BIT(7),               /* RESERVED              @Y30SSPJ*/
        3 DELE PTR(8),              /* DO LOOP COUNTER E     @Y30SSPJ*/
       2 DELC PTR(8),               /* DO LOOP COUNTER C     @Y30SSPJ*/
       2 DELVRRN CHAR(2) BDY(WORD), /* RELREPNO VALUE        @Y30SSPJ*/
        3 TSTRELSP CHAR(2),         /* SPACE HEADER GROUP    @Y30SSPJ*/
                                    /* RELREPNO VALUE        @Y30SSPJ*/
       2 DELVCRN CHAR(2),           /* CNTREPNO VALUE        @Y30SSPJ*/
        3 TSTCNTRN CHAR(2),         /* TEST VAL FOR CNTREPNO @Y30SSPJ*/
       2 DELXTNWA BDY(WORD),        /* IF INSUFF. WA FOR VGO @Y30SSPJ*/
        3 DELXLOC PTR(31),          /* LOCATION OF GVO WA    @Y30SSPJ*/
         4 DELTSFPL PTR(31),        /* TEST FPL REFERENCE    @Y30SSPJ*/
        3 DELXLEN PTR(31),          /* LENGTH OF EXT'N WKA   @Y30SSPJ*/
         4 TSTSPDUU FIXED(15),      /* UNITS USED IN DATA SPACE
                                                             @Y30SSPJ*/
         4 * CHAR(1)BDY(BYTE),      /* RESERVED              @Z40RSRC*/
         4 DELRACF  BIT(8),         /* RAC FLAG BYTE         @Z40RSRC*/
          5 DELINDC BIT(1),         /* RAC-INDICATED CLUSTER @Z40RSRC
                                       OR AIX                @Z40RSRC*/
          5 DELINDD BIT(1),         /* RAC-INDICATED DATA    @Z40RSRC*/
          5 DELINDI BIT(1),         /* RAC-INDICATED INDEX   @Z40RSRC*/
          5 *       BIT(5),         /* RESERVED              @Z40RSRC*/
       2 DELERRSX BDY(BYTE),        /* ERROR RET CODE ARRAY  @Y30SSPJ*/
        3 DELERRS(5)CHAR(4),        /* ERR RET CODE ARRAY    @Y30SSPJ*/
         4 DELMODID CHAR(2),        /* ERROR MODULE ID       @Y30SSPJ*/
         4 DELREASN CHAR(1),        /* ERROR REASON CODE     @Y30SSPJ*/
         4 DELRETCD CHAR(1),        /* ERROR RETURN CODE     @Y30SSPJ*/
 /********************************************************************/
 /*   WORK AREA INTERFACE FOR EXTRACT/MODIFY SUBFUNCTIONS    @Y30SSPJ*/
 /********************************************************************/
       2 DELEXTWA,                  /*WORKAREA FOR INTERFACE @Y30SSPJ*/
        3 DELCPL CHAR(LENGTH(CTGPL)),/* CPL FOR EXTRACT AND  @Y30SSPJ*/
         4 DELCPLST CHAR(LENGTH(CTGPL)),/* INTERNAL CPL      @Y30SSPJ*/
        3 * CHAR(LENGTH(CTGFIELD)), /*        MODIFY         @Y30SSPJ*/
         4 DELCPLFL CHAR(LENGTH(CTGFIELD)),/* CPL FPL PTR    @Y30SSPJ*/
        3 DELFPLR CHAR(LENGTH(CTGFL)),/* FPL FOR EXTRACT AND @Y30SSPJ*/
         4 DELFPLDT CHAR(LENGTH(CTGFL)),/* DATA RETRIEVE FPL @Y30SSPJ*/
        3 * CHAR(LENGTH(CTGFLDAT)), /*        MODIFY         @Y30SSPJ*/
         4 DELFL1DT CHAR(LENGTH(CTGFLDAT)),/* FPL DATA LENGTH/ADDR
                                                             @Y30SSPJ*/
        3 DELNMR CHAR(8),           /* FIELD NAME            @Y30SSPJ*/
         4 DELFL1NM CHAR(8),        /* FIELD NAME VALUE      @Y30SSPJ*/
        3 DELFPLT CHAR(LENGTH(CTGFL)),/* 1ST TEST FPL FOR    @Y30SSPJ*/
         4 DELFPLT1 CHAR(LENGTH(CTGFL)),/* TEST FPL ONE      @Y30SSPJ*/
        3 * CHAR(LENGTH(CTGFLDAT)), /* EXTRACT AND MODIFY    @Y30SSPJ*/
         4 DELT1DT CHAR(LENGTH(CTGFLDAT)),/* FPL DATA LENGTH/ADDR
                                                             @Y30SSPJ*/
        3 DELNMT CHAR(8),           /* FIELD NAME            @Y30SSPJ*/
         4 DELT1NM CHAR(8),         /* FIELD NAME VALUE      @Y30SSPJ*/
        3 DELFPLT2 CHAR(LENGTH(CTGFL)),/*2ND TEST FPL AND    @Y30SSPJ*/
        3 * CHAR(LENGTH(CTGFLDAT)), /* EXTRACT AND MODIFY    @Y30SSPJ*/
         4 DELT2DT CHAR(LENGTH(CTGFLDAT)),/* FPL DATA LENGTH/ADDR
                                                             @Y30SSPJ*/
        3 DELNMT2 CHAR(8),          /* FIELD NAME            @Y30SSPJ*/
         4 DELT2NM CHAR(8),         /* FIELD NAME VALUE      @Y30SSPJ*/
       2 DELNCOBJ PTR(8) BDY(BYTE), /*NUM OF CLUSTER OBJECTS @Y30SSPJ*/
       2 DELNGOBJ PTR(8),           /* NUMBER OF AIX'S       @Y30SSPJ*/
       2 DELNROBJ PTR(8),           /* NUMBER OF PATH'S      @Y30SSPJ*/
       2 DELUSINX PTR(8),           /* CURR ENTRY IN USERWKA @Y30SSPJ*/
       2 DELR13SA FIXED(31) BDY(WORD), /* REG 13 SAVEAREA    @Y30SSPJ*/
       2 DELDIV BDY(WORD),          /* WA FOR ASSOCIATIONS   @Y30SSPJ*/
 /********************************************************************/
 /*       EXTRACT ASSOCIATION WORK AREA FOR GENERAL USE      @Y30SSPJ*/
 /********************************************************************/
        3 DELASCWA BDY(WORD),       /* ASSOCIATION FIELD WKA @Y30SSPJ*/
         4 DELALNG FIXED(15),       /* WORK AREA LENGTH      @Y30SSPJ*/
         4 DELARLNG FIXED(15),      /* REQUIRED WICA LENGTH  @Y30SSPJ*/
         4 DELASSO(3)CHAR(4)BDY(BYTE),/* ARRAY ASSOCIATIONS  @Y30SSPJ*/
          5 DELATYPE CHAR(1),       /* ASSOC OBJECT TYPE     @Y30SSPJ*/
          5 DELACI PTR(24)BDY(BYTE),/* ASSOC OBJECT CI NUM   @Y30SSPJ*/
 /********************************************************************/
 /*        EXTRACT ASSOCIATION SAVE AREA FOR CLUSTER USE     @Y30SSPJ*/
 /********************************************************************/
        3 DELCASWA BDY(WORD),       /* CLUSTER ASSOC WKA     @Y30SSPJ*/
         4 DELCALNG FIXED(15),      /* WORK AREA LENGTH      @Y30SSPJ*/
         4 DELCRLNG FIXED(15),      /* REQUIRED WICA LENGTH  @Y30SSPJ*/
         4 DELCASSO(3)CHAR(4)BDY(BYTE),/* ASSOCIATIONS ARRAY @Y30SSPJ*/
          5 DELCTYPE CHAR(1),       /* ASSOC OBJECT TYPE     @Y30SSPJ*/
          5 DELCCI PTR(24)BDY(BYTE),/* ASSOC OBJECT CI NUM   @Y30SSPJ*/
 /********************************************************************/
 /*        EXTRACT ASSOCIATION SAVE AREA FOR AIX USE         @Y30SSPJ*/
 /********************************************************************/
        3 DELGASWA BDY(WORD),       /* AIX ASSOC WKA         @Y30SSPJ*/
         4 DELGALNG FIXED(15),      /* WORK AREA LENGTH      @Y30SSPJ*/
         4 DELGRLNG FIXED(15),      /* REQUIRED WICA LENGTH  @Y30SSPJ*/
         4 DELGASSO(3)CHAR(4)BDY(BYTE),/* ASSOCIATIONS ARRAY @Y30SSPJ*/
          5 DELGTYPE CHAR(1),       /* ASSOC OBJECT TYPE     @Y30SSPJ*/
          5 DELGCI PTR(24)BDY(BYTE),/* ASSOC OBJECT CI NUM   @Y30SSPJ*/
 /********************************************************************/
 /*        EXTRACT ASSOCIATION SAVE AREA FOR PATH USE        @Y30SSPJ*/
 /********************************************************************/
        3 DELRASWA BDY(WORD),       /* PATH ASSOC WKA        @Y30SSPJ*/
         4 DELRALNG FIXED(15),      /* WORK AREA LENGTH      @Y30SSPJ*/
         4 DELRRLNG FIXED(15),      /* REQUIRED WICA LENGTH  @Y30SSPJ*/
         4 DELRASSO(5)CHAR(4)BDY(BYTE),/* ASSOCIATIONS ARRAY @Y30SSPJ*/
          5 DELRTYP CHAR(1),        /* ASSOC OBJECT TYPE     @Y30SSPJ*/
          5 DELRCI PTR(24)BDY(BYTE),/* ASSOC OBJECT CI NUM   @Y30SSPJ*/
 /********************************************************************/
 /*   WORK AREA FOR EXTRACTING A SPECIFIC TYPE ASSOCIATION   @Y30SSPJ*/
 /********************************************************************/
        3 DELXASWA BDY(WORD),       /* SPECIFIC TYPE ASC WA  @Y30SSPJ*/
         4 DELXALNG FIXED(15),      /* WORK AREA LENGTH      @Y30SSPJ*/
         4 DELXRLNG FIXED(15),      /* REQUIRED WICA LENGTH  @Y30SSPJ*/
         4 DELXASSO(1)CHAR(4)BDY(BYTE),/* ASSOCIATIONS ARRAY @Y30SSPJ*/
          5 DELXTYPE CHAR(1),       /* ASSOC OBJECT TYPE     @Y30SSPJ*/
          5 DELXCI PTR(24)BDY(BYTE),/* ASSOC OBJECT CI NUM   @Y30SSPJ*/
 /********************************************************************/
 /*     WORK AREA FOR EXTRACTING VOLUME GROUP OCCURENCES     @Y30SSPJ*/
 /********************************************************************/
        3 DELVGOWA BDY(BYTE),       /* WORK AREA FOR VGO     @Y30SSPJ*/
         4 DELVLNG CHAR(2),         /* LENGTH OF WORK AREA   @Y30SSPJ*/
         4 DELVRLNG CHAR(2),        /* WA REQUIRED LENGTH    @Y30SSPJ*/
         4 DELTVRRN CHAR(2),        /* RELREPNO RETURN       @Y30SSPJ*/
         4 DELVGO CHAR(LENGTH(GVO)),/*VOLUME GROUP OCCURANCE @Y30SSPJ*/
 /********************************************************************/
 /*        WORK AREA FOR EXTRACTING DATA SPACE GROUP         @Y30SSPJ*/
 /********************************************************************/
        3 DELSHWA BDY(BYTE),        /* DATA SPACE GROUP WA   @Y30SSPJ*/
         4 DELSLNG CHAR(2),         /* LENGTH OF WORK AREA   @Y30SSPJ*/
         4 DELSRLNG CHAR(2),        /* WA REQUIRED LENGTH    @Y30SSPJ*/
         4 DELDSGRP CHAR(LENGTH(CTVDSPAC)),/* DATA SPACE GRP @Y30SSPJ*/
 /********************************************************************/
 /*        WORK AREA FOR EXTRACTING DATA SET DIRECTORIES     @Y30SSPJ*/
 /********************************************************************/
        3 DELDSDWA BDY(BYTE),       /* WORK AREA FOR DS DIR  @Y30SSPJ*/
         4 DELDLNG CHAR(2),         /* LENGTH OF WORK AREA   @Y30SSPJ*/
         4 DELDRLNG CHAR(2),        /* WA REQUIRED LENGTH    @Y30SSPJ*/
         4 DELDIRCT CHAR(LENGTH(CTVDIR)),/* DS DIRECTORY     @Y30SSPJ*/
          5 DELDRRN CHAR(2),        /* DS DIRECTORY RELREPNO @Y30SSPJ*/
         4 * CHAR(1),               /* NOT USED              @Y30SSPJ*/
 /********************************************************************/
 /*            WORK AREA FOR SCRATCH FUNCTION                @Y30SSPJ*/
 /********************************************************************/
       2 DELSCRWA BDY(WORD),        /* SCRATCH WORK AREA     @Y30SSPJ*/
 /********************************************************************/
 /*                    SCRATCH PARAMETER LIST                @Y30SSPJ*/
 /********************************************************************/
        3 DELSCRTH CHAR(16)BDY(WORD),/* SCRATCH PARAM LIST   @Y30SSPJ*/
         4 DELSCRFL FIXED(31)BDY(WORD), /* SCRATCH OPT BYTES @Y30SSPJ*/
         4 DELSDSNM PTR(31),        /*DATA SET NAME TO SCRTH @Y30SSPJ*/
         4 DELSNUSE PTR(31),        /*UNUSED                 @Y30SSPJ*/
         4 DELSVLST PTR(31),        /*VOLUME LIST REFERENCE  @Y30SSPJ*/
 /********************************************************************/
 /*                     SCRATCH VOLUME LIST                  @Y30SSPJ*/
 /********************************************************************/
        3 DELSVOL,                  /*SCRATCH VOLUME LIST    @Y30SSPJ*/
         4 DELSNVLS FIXED(15),      /*NUM VOL LIST ENTRIES   @Y30SSPJ*/
         4 DELSDEVT CHAR(4),        /*VOL DEV TYPE CODE      @Y30SSPJ*/
         4 DELSVLID CHAR(6),        /*VOL SERIAL NUMBER      @Y30SSPJ*/
         4 DELSRCD1 PTR(8),         /*UNUSED RTN CODE        @Y30SSPJ*/
         4 DELSRCD2 PTR(8),         /*LIST ENTRY RTN CODE    @Y30SSPJ*/
       2 DELVOLL CHAR(LENGTH(IDAVVOLL)), /*VOLUME LIST       @Y30SSPJ*/
        3 DELSVMTL CHAR(LENGTH(IDAVVOLL)),/*VOLUME LIST      @Y30SSPJ*/
       2 DELSVTET CHAR(LENGTH(VVOLLENT)),/*VOLUME MNT ENTRY  @Y30SSPJ*/
       2 DELPROC CHAR(2),           /* PROCESS LOOP CONTROL  @Y30SSPJ*/
       2 DELTIOPT PTR(31),          /* TIOT PTR FOR VOLUME MOUNT
                                                             @Y30SSPJ*/
         3 * CHAR(1),               /* NOT USED              @Y30SSPJ*/
         3 DELHECIN PTR(24),        /* HORIZONTAL EXTENSION CI NUMBER
                                                             @Y30SSPJ*/
       2 DELOPNPL PTR(31)BDY(WORD), /* PARAM LIST FOR OPEN   @Y30SSPJ*/
        3 DELOPOPT CHAR(1)BDY(BYTE),/* OPEN OPTIONS          @Y30SSPJ*/
         4 DELOPEND BIT(1),         /* MARK END OF PARAM LST @Y30SSPJ*/
         4 * BIT(7),                /* USED BY OPEN          @Y30SSPJ*/
        3 DELACBPT PTR(24)BDY(BYTE),/* PTR TO ACB TO OPEN    @Y30SSPJ*/
       2 * CHAR(2),                 /* RESERVED              @Y30SSPJ*/
       2 F FIXED(15) BDY(BYTE);     /* ERASE LOOP COUNTER    @Y30SSPJ*/
 /********************************************************************/
 /*      DECLARATION FOR PASSWORD BASE REFERENCE FIELD       @Y30SSPJ*/
 /********************************************************************/
    DECLARE PASSSTRT CHAR(8) BASED; /* PASSWORD BASE REF FLD @Y30SSPJ*/
 /********************************************************************/
 /*        DECLARATION FOR CMS DELETE-ERASE WORK AREA        @Y30SSPJ*/
 /********************************************************************/
    DECLARE
      1 DELERASE BDY(WORD)BASED(CCAR5REC),/* ERASE WORK AREA @Y30SSPJ*/
       2 DELACB CHAR(LENGTH(IFGACB)),/* ACB FOR ERASE        @Y30SSPJ*/
       2 DELRPL CHAR(LENGTH(IFGRPL)),/* RPL FOR ERASE        @Y30SSPJ*/
       2 DELRCPTR PTR(31),          /* RECORD AREA POINTER   @Y30SSPJ*/
       2 DELPASSW,                  /* PASSWORD STRUCTURE    @Y30SSPJ*/
        3 DELPASLN BDY(BYTE)FIXED(8),/* PASSWORD LENGTH      @Y30SSPJ*/
        3 DELPASWD CHAR(8);         /* PASSWORD VALUE        @Y30SSPJ*/
 /********************************************************************/
 /*           DECLARATION FOR ENQUEUE PARAMETER LIST         @Y30SSPJ*/
 /********************************************************************/
    DECLARE
      1 DELENQ BDY(WORD)BASED(CCAR5REC),/*ENQ PARAMETER LIST @Y30SSPJ*/
       2 ENQPROPR,                  /* ENQ MAIN SECTION LBL  @Y30SSPJ*/
        3 ENQOPT1 BIT(8),           /* OPTION BYTE 1         @Y30SSPJ*/
         4 ENQEND BIT(1),           /* ON FOR END OF LIST    @Y30SSPJ*/
         4 ENQNEW BIT(1),           /* OFF FOR NES FORM      @Y30SSPJ*/
         4 * BIT(5),                /* RESERVED              @Y30SSPJ*/
         4 ENQDIR BIT(1),           /* ON FOR DIRECTED FORM  @Y30SSPJ*/
        3 ENQRLEN FIXED(8),         /* RNAME LENGTH          @Y30SSPJ*/
        3 ENQOPT2 BIT(8),           /* OPTION BYTE 2         @Y30SSPJ*/
         4 ENQSHR BIT(1),           /* ENQ FOR SHARED CONTRL @Y30SSPJ*/
         4 * BIT(7),                /* RESERVED              @Y30SSPJ*/
          3 ENQRETC FIXED(8),       /* ENQ/DEQ RETURN CODE   @Y30SSPJ*/
          3 ENQQNADR PTR(31),       /* QNAME ADDRESS         @Y30SSPJ*/
          3 ENQRNADR PTR(31),       /* RNAME ADDRESS         @Y30SSPJ*/
       2 RNAME CHAR(8),             /* RNAME FOR ENQ         @Y30SSPJ*/
        3 RNAMECI CHAR(3),          /* CONTROL INTERVAL NO.  @Y30SSPJ*/
        3 RNAMECA CHAR(4),          /* CATALOG ACB POINTER   @Y30SSPJ*/
        3 RNAMEID CHAR(1);          /* ALPHA ID TYPE         @Y30SSPJ*/

    %IF DELWKA98 = ''               /*IF OBTAIN WORK AREA NOT REQUIRED
                                                             @Y30SSPJ*/
      %THEN %GOTO DELWKA01;         /* THEN DONT GENERATE IT @Y30SSPJ*/
    %STOPFMT2:;                     /* SHOW NEXT COMMENT     @Y30SSPJ*/
 /********************************************************************/
 /*         OBTAIN PARAMETER LIST - IN RECORD AREA 4         @Y30SSPJ*/
 /********************************************************************/
    DECLARE
      1 DELOCAM BASED(CCAR4REC),    /* OBTAIN CAM LIST       @Y30SSPJ*/
        2 DELOPT CHAR(4),           /* OBTAIN OPTIONS        @Y30SSPJ*/
        2 DELODSN PTR(31),          /* ADDRESS OF FORMAT 1 DSCB CCHHR
                                                             @Y30SSPJ*/
        2 DELOVOL PTR(31),          /* ADDR OF VOLUME SERIAL @Y30SSPJ*/
        2 DELOBUF PTR(31),          /* ADDR OF DSCB BUFFER   @Y30SSPJ*/
        2 DELDBUF CHAR(148);        /* BUFFER FOR DSCB       @Y30SSPJ*/
    DECLARE
      DELOSEK BIT(32) CONSTANT('C0800000'X); /* SEEK OPTION  @Y30SSPJ*/
 /********************************************************************/
 /*          DISK ADDRESS OF DSCB - IN RECORD AREA 4         @Y30SSPJ*/
 /********************************************************************/
    DECLARE
      1 DELDSADR DEF(DELDBUF)POS(149),/* DSCB DISK ADDRESS   @Y30SSPJ*/
        2 DELDSCC FIXED(15)BDY(BYTE),/* DSCB CYLINDER ADDR   @Y30SSPJ*/
        2 DELDSHH FIXED(15)BDY(BYTE),/* DSCB HEAD ADDRESS    @Y30SSPJ*/
        2 DELDSREC PTR(8)BDY(BYTE); /* DSCB RECORD NUMBER    @Y30SSPJ*/
 /********************************************************************/
 /*               FORMAT 4 EXTENT DESCRIPTION                @Y30SSPJ*/
 /********************************************************************/
    DECLARE
      1 DELVTOCE DEF(DS4VTOCE),     /* FORMAT 4 EXTENT DESCRIPTION
                                                             @Y30SSPJ*/
        2 * CHAR(2),                /* NOT USED              @Y30SSPJ*/
        2 DELDS4BG CHAR(4),         /* VTOC START CCHH       @Y30SSPJ*/
        2 DELDS4EN CHAR(4);         /* VTOC END CCHH         @Y30SSPJ*/
 /********************************************************************/
 /*      VOLUME INFORMATION SAVE AREA - IN RECORD AREA 4     @Y30SSPJ*/
 /********************************************************************/
    DECLARE
      1 * DEF(DELDSREC) POS(2),     /* VOL ENTRY SAVE AREA   @Y30SSPJ*/
        2 * CHAR(1) BDY(BYTE),      /* RESERVED              @Y30SSPJ*/
        2 DELVLSER CHAR(6),         /* VOLUME SERIAL         @Y30SSPJ*/
        2 DELDEVT CHAR(4),          /* VOLUME DEVICE TYPE    @Y30SSPJ*/
        2 DELUCB FIXED(31) BDY(BYTE), /* VOLUME UCB ADDRESS  @Y30SSPJ*/
        2 DELNOTRK FIXED(15) BDY(BYTE); /* TRACKS PER CYL    @Y30SSPJ*/
 %DELWKA01:;                                               /*@Y30SSPJ*/
 /********************************************************************/
 /*       THE FOLLOWING STRUCTURE DESCRIBES THE CATALOG      @Y30SSPJ*/
 /*         UPGRADE INTERFACE PARAMETER LIST AND ALSO ITS    @Y30SSPJ*/
 /*         ASSOCIATED WORKAREA.  THE CALLER MUST SUPPLY     @Y30SSPJ*/
 /*         CORE FOR BOTH AND INITIALIZE CUMPL TO DELETE OR  @Y30SSPJ*/
 /*         ADD AN UPGRADE ASSOCIATION.  CCACUMPL MUST BE    @Y30SSPJ*/
 /*         INITIALIZED TO POINT TO CUMPLWA.                 @Y30SSPJ*/
 /********************************************************************/
    DECLARE
      1 CUMPLWA BASED(CCACUMPL),    /* UPGRADE MGMNT PL/WA   @Y30SSPJ*/
        2 CUMPL,                    /* INTERFACE PARM LIST   @Y30SSPJ*/
          3 CUMIN BIT(8),           /* INPUT FLAGS           @Y30SSPJ*/
            4 CUMFDEL BIT(1),       /* FORCE DELETE Y ENTRY  @Y30SSPJ*/
            4 CUMNRG  BIT(1),       /* NO RE-GET BASE RCD    @Y30SSPJ*/
            4 CUMFUNC BIT(1),       /* SET INTERNALLY:       @Y30SSPJ*/
                                    /*   0 - DELETE, 1 - ADD @Y30SSPJ*/
            4 *       BIT(5),       /* RESERVED              @Y30SSPJ*/
          3 CUMOUT BIT(8),          /* OUTPUT FLAGS          @Y30SSPJ*/
            4 CUMDUP  BIT(1),       /* DUPLICATE UPGRD ASSOC @Y30SSPJ*/
            4 CUMNE   BIT(1),       /* NON-EXISTENT Y ENTRY  @Y30SSPJ*/
                                    /*   OR UPGRADE ASSOC    @Y30SSPJ*/
            4 *       BIT(6),       /* RESERVED              @Y30SSPJ*/
          3 CUMDCI PTR(24) BDY(BYTE), /* AIX DATA CI         @Y30SSPJ*/
          3 CUMICI PTR(24) BDY(BYTE), /* AIX INDEX CI        @Y30SSPJ*/
        2 CUMWA CHAR(152) BDY(WORD), /* UPGRD MGMNT WRK AREA @Y30SSPJ*/
          3 CUMFLAG  BIT(8),        /* UPGRADE MGMNT FLAGS   @Y30SSPJ*/
            4 CUMEXIT BIT(1),       /* FORCE DELETE EXIT IND @Y30SSPJ*/
            4 *       BIT(7),       /* RESERVED              @Y30SSPJ*/
          3 CUMBCI  PTR(24),        /* CALLER'S BASE RECORD CI
                                         NUMBER              @Y30SSPJ*/
          3 CUMURAB PTR(31),        /* SAVE CALLER'S CCAURAB @Y30SSPJ*/
          3 CUMRABB PTR(31),        /* SAVE CALLER'S CCARABB @Y30SSPJ*/
          3 CUMRAB1 PTR(31),        /* FIRST FREE RAB ADDR   @Y30SSPJ*/
          3 CUMRAB2 PTR(31),        /* SECOND FREE RAB ADDR  @Y30SSPJ*/
          3 CUMRABD PTR(31),        /* DATA RAB ADDRESS      @Y30SSPJ*/
          3 *       CHAR(1),        /* RESERVED              @Y30SSPJ*/
          3 CUMSRID PTR(24),        /* SRCH ARG OF NEXT RCD  @Y30SSPJ*/
          3 *       PTR(8),         /* RESERVED              @Y30SSPJ*/
          3 CUMDY   PTR(24),        /* DATA RECORD CI NO.    @Y30SSPJ*/
          3 CUMFN1  CHAR(8),        /* FIELD NAME            @Y30SSPJ*/
          3 CUMFN2  CHAR(8),        /* TEST FIELD NAME       @Y30SSPJ*/
          3 CUMWASS2,               /* ASSOCIATION WORK AREA @Y30SSPJ*/
            4 CUMWREL FIXED(15),    /* RELREPNO              @Y30SSPJ*/
            4 CUMWASS,              /* SINGLE ASSOCIATION    @Y30SSPJ*/
              5 CUMWTYP CHAR(1),    /* TYPE D ENTRY          @Y30SSPJ*/
                6 CUMTYPE CHAR(1),  /* NEXT RECORD TYPE GET  @Y30SSPJ*/
              5 CUMWCI  PTR(24)     /* D ENTRY CI NO.        @Y30SSPJ*/
                      BDY(BYTE),    /*                       @Y30SSPJ*/
            4 CUMWTYP2  CHAR(1),    /* TYPE I ENTRY          @Y30SSPJ*/
            4 CUMWCI2   PTR(24)     /* I ENTRY CI NO.        @Y30SSPJ*/
                      BDY(BYTE),    /*                       @Y30SSPJ*/
          3 CUMFMWA CHAR(88),       /* FIELD MGMNT WORK AREA @Y30SSPJ*/
          3 *       CHAR(2),        /* RESERVED              @Y30SSPJ*/
          3 CUMRC   CHAR(4);        /* SAVE RETURN CODE      @Y30SSPJ*/
