 /********************************************************************/
 /*                                                                  */
 /*   MACRO NAME = IGGMUPDE                                          */
 /*                                                                  */
 /*   DESCRIPTIVE NAME = UPDATE-EXTEND COMMON DECLARATIONS           */
 /*                                                                  */
 /*   FUNCTION = COMMON DECLARATIONS FOR VSAM UPDATE-EXTEND          */
 /*                MODULES IGG0CLBB, IGG0CLBC, AND IGG0CLBP,         */
 /*                AND FOR DEFINE CRA MODULE IGG0CLB4.               */
 /*              CONTAINS COMMON WORK AREA MAPPINGS AND INTERFACE    */
 /*                PARAMETER LISTS FOR EXTERNALLY CALLED SUB-        */
 /*                FUNCTIONS USED WITHIN THE VSAM UPDATE-EXTEND      */
 /*                AND DEFINE CRA FUNCTIONS.                         */
 /*                                                                  */
 /*   INCLUDED MACROS = IGGMVEDC                                     */
 /*                     IGGMCDCL                                     */
 /*                     IGGMCMDM                                     */
 /*                     IDAAMDSB                                     */
 /*                     IGGMSAWA                                     */
 /*                                                                  */
 /*   METHOD OF ACCESS = PL/S - RESPECIFY A BASE FOR SOME STRUCTURES */
 /*                                                                  */
 /*   STATUS = VS/2 RELEASE 3   (CHANGE FLAG @Y30SSXX)               */
 /*                                                                  */
 /*   DATE OF LAST CHANGE = 02 JUL 74                                */
 /*                         27 APR 76                        @ZA07574*/
 /*                                                                  */
 /********************************************************************/
 /********************************************************************/
 /*           C O N S T A N T     D E C L A R A T I O N S            */
 /********************************************************************/
    DECLARE
      KEYLENSP FIXED(31) CONSTANT(128), /* MAX KEY LENGTH INFO       */
      MXEXT  FIXED(31)  CONSTANT(123), /* MAX EXTENTS        @XA02392*/
      UPDVOL   CHAR(8) CONSTANT('UPDVOL  '), /* UPDATE VOLUME        */
                                        /* OCCURRENCE                */
      NOAVFPL  FIXED(31)  CONSTANT(2), /* NO. OF MODIFY FPL'S        */
      CATDSBGC FIXED(31) CONSTANT(1), /* AMDSB GROUP CODE            */
      NOVOLENT FIXED(31) CONSTANT(1), /* NO. OF VOLUME ENTRIES       */
      MXALEX   FIXED(31) CONSTANT(5), /* MAX EXTENTS SUBALLOCATE     */
      PRENTNO  FIXED(31) CONSTANT(1), /* NO. OF PRLIST ENTRIES       */
      MXM      FIXED(31) CONSTANT(256), /* MAX CLEAR                 */
      MXEXTC   FIXED(31) CONSTANT(16), /* CATALOG MAX EXTENTS        */
      UPWALEN FIXED(31) CONSTANT(512), /* UPDATE WORK AREA LENGTH    */
      NOEXTFPL FIXED(31) CONSTANT(1), /* NO. OF MODIFY FPL'S         */
      EXTENT   CHAR(8) CONSTANT('EXTENT  '), /* EXTENT NAME          */
      CATVOLGC  FIXED(31) CONSTANT(3), /* VOLUME GROUP CODE          */
      ITYPEXT CHAR(8) CONSTANT('ITYPEXT '), /* INDICATOR     @Y30SSRS*/
                                        /* TYPE OF EXTENT    @Y30SSRS*/
      CNTREPNO CHAR(8) CONSTANT('CNTREPNO'), /* MAXIMUM      @Y30SSRS*/
                                        /* COUNT OF OCCURR   @Y30SSRS*/
      RELREPNO CHAR(8) CONSTANT('RELREPNO'), /* RELATIVE     @Y30SSRS*/
                                        /* REPETITION NUMBER @Y30SSRS*/
      ENTVOL   CHAR(8) CONSTANT('ENTVOL  '), /* ENTIRE       @Y30SSRS*/
                                        /* VOLUME OCCURRENCE @Y30SSRS*/
      AMDSBCAT CHAR(8) CONSTANT('AMDSBCAT'), /* AMDSB        @Y30SSRS*/
      NODAT    FIXED(31) CONSTANT(1), /* NO. FPL DATA/RNG PAIR       */
      NOFPLS   FIXED(31) CONSTANT(2), /* NO. OF FPL'S                */
      NAME     CHAR(8) CONSTANT('NAME    '), /* CI NO.       @Y30SSRS*/
      TYPE     CHAR(8) CONSTANT('TYPE    '), /* ENTRY TYPE           */
      CATCLSID CHAR(1) CONSTANT('C'),  /* CLUSTER ID                 */
      CATDATID CHAR(1) CONSTANT('D'), /* DATA ID                     */
      CATIXID  CHAR(1) CONSTANT('I'), /* INDEX ID                    */
      NOKEYFPL  FIXED(31) CONSTANT(3), /* KEY RANGE EXTEND NO.       */
      NONKYFPL  FIXED(31) CONSTANT(1), /* NON-KEY RANGE EXTEND NO.   */
      UPEVOL    CHAR(6)   BASED(CTGFLPT(1)); /* MAP VOLSER INPUT     */
 /********************************************************************/
 /*             WORK AREA FOR SEARCH VOLUME SUB-FUNCTION             */
 /********************************************************************/
    DECLARE
      1 UPSVOL BASED(ADDR(IDAAMDSB)+LENGTH(IDAAMDSB)), /* PARAMETERS */
        2 UPLKEYLN  FIXED(15),      /* LOW KEY LENGTH                */
        2 UPHKEYLN  FIXED(15),      /* HIGH KEY LENGTH               */
        2 UPLKEYPT  PTR(31),        /* LOW KEY POINTER               */
        2 UPHKEYPT  PTR(31),        /* HIGH KEY POINTER              */
        2 UPVOLSER  CHAR(6),        /* VOLUME SERIAL NUMBER          */
        2 UPSREL    FIXED(15),      /* RELATIVE REPETITION   @Y30SSRS*/
                                    /* NO. (RELREPNO)        @Y30SSRS*/
        2 UPSCNT    FIXED(15),      /* MAX NO. OF REPETITION @Y30SSRS*/
                                    /* NOS. FOR PROCESSING   @Y30SSRS*/
        2 UPSEND    CHAR(0);        /* WORKAREA END                  */
    DECLARE KEYINFO CHAR(1) BASED(R2+UPLKEYPT); /* MAP LOW KEY AREA  */
    DECLARE ENTVOL1 CHAR(256) BASED(ADDR(UPREND)); /* MAP VOL OCCURR */
    DECLARE ENTVOL2 CHAR(256) BASED(CTGFLPT(1)); /* MAP VOL OCCURR   */
 /********************************************************************/
 /*              MAPPING  FOR  'RELREPNO'  INFORMATION               */
 /********************************************************************/
    DECLARE
      1 RELINFO BASED(R3),          /* RELREPNO INFORMATION          */
        2 RELREP FIXED(15),         /* RELREPNO                      */
        2 *      CHAR(2);           /* ALIGNMENT                     */
 /********************************************************************/
 /*                MAPPING  FOR  EXTENT  INFORMATION                 */
 /********************************************************************/
    DECLARE
      1 UPEXT1 BASED(R4),           /* EXTENT INFORMATION            */
        2 UPEXTLN1 FIXED(15),       /* EXTENT LENGTH                 */
        2 UPEXTD2  CHAR(256);       /* EXTENT DATA                   */
 /********************************************************************/
 /*                     MAPPING  FOR  KEY  AREA                      */
 /********************************************************************/
    DECLARE
      1 UPKEY1  BASED(UPLKEYPT),    /* MAP KEY AREA                  */
        2 UPKEYD1  CHAR(64);        /* KEY                           */
 /********************************************************************/
 /*                     MAPPING  FOR  KEY  AREA                      */
 /********************************************************************/
    DECLARE
      1 UPKEY BASED(R4),            /* MAP KEY AREA                  */
        2 UPKEYLN  FIXED(15),       /* KEY LENGTH                    */
        2 UPKEYD   CHAR(64);        /* KEY DATA                      */
    DECLARE CLSCINO CHAR(3) BASED(CTGFLPT(1)); /* CLUSTER CI NO.     */
 /********************************************************************/
 /*             MAPPING  FOR  USER  RETURN  INFORMATION              */
 /********************************************************************/
    DECLARE
      1 WKAINFO BASED(CTGWKA),      /* MAP USER RETURN INFORMATION   */
        2 *       CHAR(4),          /* WORK AREA LENGTHS             */
        2 WKARETN,                  /* RETURN INFORMATION            */
          3 WKAREL   FIXED(15),     /* OBJECT RELREPNO               */
          3 WKARELSS FIXED(15),     /* SEQUENCE SET RELREPNO         */
          3 WKAEXT   FIXED(15),     /* NO. OF OLD EXTENTS            */
          3 WKAEXTSS FIXED(15);     /* NO. OF OLD SEQ. SET EXTENTS   */
 /********************************************************************/
 /*                   MAPPING  FOR  EXTEND  INPUT                    */
 /********************************************************************/
    DECLARE
      1 WKAINPUT DEFINED(WKARETN),  /* MAP EXTEND INPUT      @XA02392*/
        2 WKAIEXT FIXED(15),        /* INDEX NUMBER OF EXTENTS
                                                             @XA02392*/
        2 WKAICI  CHAR(3);          /* INDEX CI NO.          @XA02392*/
 @EJECT;
 %INCLUDE SYSLIB(IGGMVEDC);         /* VOLUME ENTRY DECLARES         */
 @EJECT;
 /********************************************************************/
 /*         DECLARATION  OF  UPDATE  PARAMETER  INFORMATION          */
 /********************************************************************/
    DECLARE
      1 UPREPARM  BASED(CCAR2REC),  /* UPDATE EXTEND/RELEASE         */
        2 UPNTCYL  FIXED(15),       /* NO. TRACKS/CYLINDER           */
          3 UPNIEXT FIXED(15),      /* INDEX NO. OF EXTENTS          */
        2 UPICINO  CHAR(3),         /* INDEX CI NO.          @Y30SSRS*/
        2 UPVCINO  CHAR(3),         /* VOLUME CI NO.                 */
        2 UPFLGS   BIT(8),          /* FLAGS                         */
          3 UPKEYDS  BIT(1),        /* KEYED DATA SET                */
          3 *        BIT(1),        /* RESERVED                      */
          3 UPSSWD   BIT(1),        /* SEQUENCE SET WITH DATA        */
          3 UPREPL   BIT(1),        /* REPLICATION                   */
          3 *        BIT(1),        /* RESERVED                      */
          3 UPKRAN   BIT(1),        /* KEY RANGE                     */
          3 *        BIT(1),        /* RESERVED                      */
          3 *        BIT(1),        /* RESERVED                      */
        2 UPFLG2S   BIT(16),        /* UPDATE FLAGS                  */
          3 UPUSSWD BIT(1),         /* EXTEND REQUEST                */
          3 UPKREQ  BIT(1),         /* KEYS EQUAL                    */
          3 UPSRCH  BIT(1),         /* SEARCH FOR VOLUME             */
          3 UPKSET  BIT(1),         /* KEY RANGE SET                 */
          3 UPEXIT  BIT(1),         /* SEARCH EXIT INDICATOR         */
          3 UPSMFL  BIT(1),         /* WRITE LSPACE SMF RECORD 69    */
        2 *       CHAR(1),          /* RESERVED              @Y30SSRS*/
        2 UPCHIA  PTR(32),          /* SAVE HI-ALLOCATED RBA         */
        2 UPEXTLN FIXED(15),        /* EXTENT LENGTH (EXISTING)      */
        2 UPREND  CHAR(0);          /* END UPREPARM                  */
 /********************************************************************/
 /*                 MAPPING  FOR  VOLUME  OCCURRENCE                 */
 /********************************************************************/
    DECLARE
      1 UPENTVOL BASED(ADDR(UPREND)), /* VOLUME OCCURRENCE           */
        2 UPVOLFIX,                 /* FIXED INFO OF VOLUME          */
          3 UPRLRP   FIXED(15),     /* RELREPNO                      */
          3 UPDEVT   CHAR(4),       /* DEVICE TYPE                   */
          3 UPVOL    CHAR(6),       /* VOLUME SERIAL                 */
          3 UPFLSQ   FIXED(15),     /* FILE SEQUENCE NUMBER          */
          3 UPVLFLG  BIT(8),        /* VOLUME FLAGS                  */
            4 UPVLPR   BIT(1),      /* PRIME VOLUME                  */
            4 UPVLCN   BIT(1),      /* CANDIDATE VOLUME              */
            4 UPVLOV   BIT(1),      /* OVERFLOW VOLUME               */
            4 *        BIT(5),      /* RESERVED                      */
          3 UPNOEXT  PTR(8),        /* NO. OF EXTENTS                */
          3 UPHKBA   PTR(32) BDY(BYTE), /* HI KEY CI VALUE           */
          3 UPHUBA   PTR(32) BDY(BYTE), /* HI USED RBA               */
          3 UPHABA   PTR(32) BDY(BYTE), /* HI ALLOCATED RBA          */
          3 UPPHYB   PTR(32) BDY(BYTE), /* PHYSICAL BLOCK SIZE       */
          3 UPNBTK   FIXED(15),     /* BLOCKS PER TRACK              */
          3 UPNTKA   FIXED(15),     /* TRACKS ALLOCATION UNIT        */
          3 UPITEX   BIT(8),        /* TYPE OF EXTENT                */
            4 UPSSDT   BIT(1),      /* SEQUENCE SET WITH DATA        */
            4 UPEXTNP  BIT(1),      /* EXTENTS NOT PRE-FORMATTED     */
            4 *        BIT(6),      /* RESERVED                      */
          3 UPSSDS   FIXED(15) BDY(BYTE), /* SS NO. OF DIRECTORY     */
        2 UPVOLVAR,                 /* VARIABLE VOLUME INFO          */
          3 UPLKLN   FIXED(15) BDY(BYTE), /* LOW KEY LENGTH          */
          3 *        CHAR(64),      /* KEY SPACE                     */
          3 UPHKLN   FIXED(15) BDY(BYTE), /* HI KEY LENGTH           */
          3 *        CHAR(64),      /* KEY SPACE                     */
          3 UPEXLN   FIXED(15) BDY(BYTE), /* EXTENT LENGTH           */
          3 *        CHAR(3),       /* ALIGNMENT                     */
        2 UPEND    CHAR(0);         /* END OF VOLUME OCCURRENCE      */
    DECLARE CLEAR CHAR(256) BASED(CCAALPTR); /* MAP SUBALL WORK AREA */
 /********************************************************************/
 /*                MAPPING  FOR  EXTENT  INFORMATION                 */
 /********************************************************************/
    DECLARE
      1 EXTINF1 BASED,              /* EXTENT INFORMATION            */
        2 EXTILEN  FIXED(15),       /* EXTENT INSERT LENGTH          */
        2 EXTINFO(*) BDY(HWORD),    /* EXTENT DESCRIPTORS            */
          3 EXTSS    FIXED(15),     /* SPACE SEQUENCE NUMBER         */
          3 EXTST,                  /* START CCHH                    */
            4 EXTCC1   FIXED(15),   /* START CYLINDER                */
            4 EXTHH1   FIXED(15),   /* START HEAD                    */
          3 EXTEND,                 /* END CCHH                      */
            4 EXTCC2   FIXED(15),   /* END CYLINDER                  */
            4 EXTHH2   FIXED(15),   /* END HEAD                      */
          3 EXTTT    FIXED(15),     /* TOTAL TRACKS                  */
          3 EXTDDDD1 PTR(32) BDY(HWORD), /* LOW DDDD                 */
          3 EXTDDDD2 PTR(32) BDY(HWORD); /* HIGH DDDD                */
 /********************************************************************/
 /*          MAPPING  FOR  UPDATE-EXTEND  SPACE  WORK  AREA          */
 /********************************************************************/
    DECLARE
      1 UPSPARM BASED(CCAR3REC-LENGTH(UPSPARM)), /* SPACE WORKAREA   */
        2 UPSOPT   CHAR(1),         /* SPACE OPTIONS                 */
        2 UPSPQTY  CHAR(3),         /* SPACE QUANTITY                */
        2 UPSFLG   BIT(8),          /* SPACE FLAGS                   */
          3 UPSUNIQ  BIT(1),        /* UNIQUE DATA SET               */
          3 UPSEXIT  BIT(1),        /* SPACE EXIT INDICATOR          */
          3 *        BIT(6),        /* RESERVED                      */
        2 UPDSDIR  CHAR(3),         /* DATA SET DIRECTORY NUMBER     */
        2 UPUDSNPT PTR(31),         /* UNIQUE DATA SET NAME PTR      */
        2 UPMXBLK  FIXED(15),       /* MAXIMUM BLOCK SIZE            */
          3 UPUCB   CHAR(2),        /* UCB ADDR OF VOLUME    @Y30SSRS*/
        2 UPCNT    FIXED(15),       /* MAX NO. OF REPETITION NOS.    */
                                    /* TO BE PROCESSED (CNTREPNO)    */
        2 UPREL    FIXED(15),       /* RELATIVE REPETITION NUMBER    */
                                    /* (RELREPNO)                    */
        2 UPNODSP  FIXED(15),       /* DATA SPACE COUNT              */
        2 UPSPACE  CHAR(3),         /* SPACE QUANTITY                */
        2 UPSOPT1  CHAR(1),         /* SPACE OPTIONS                 */
        2 UPCRDT1,                  /* CREATION DATE                 */
          3 UPCRDT,                 /* YEAR AND DAY                  */
            4 UPCRDTY   PTR(8),     /* YEAR                          */
            4 UPCRDTDD FIXED(15) BDY(BYTE), /* DAY                   */
          3 UPCRDTF  PTR(8),        /* SIGN CHARACTER                */
        2 UPSPHDR,                  /* SPACE HEADER AREA             */
          3 UPSHREL  FIXED(15) BDY(BYTE), /* RELREPNO                */
          3 *        CHAR(21),      /* RESERVED                      */
        2 UPDIRSP  CHAR(12),        /* DATA SET DIRECTORY AREA       */
          3 UPDDREL  FIXED(15) BDY(BYTE), /* RELREPNO                */
          3 *        CHAR(10),      /* RESERVED                      */
        2 UPSPWA   CHAR(135),       /* EXTRACT/MODIFY CPL/FPL AREA   */
          3 UPSTIME CHAR(8),        /* S/370 TOD FOR         @Y30SSRS*/
                                    /* FORMAT-4 UPDATE       @Y30SSRS*/
        2 UPSPD,                    /* SPACE DESCRIPTOR AREA         */
          3 UPSDREL  FIXED(15) BDY(BYTE), /* RELREPNO                */
          3 *        CHAR(16),      /* RESERVED              @ZA07574*/
        2 *        CHAR(1),         /* FREE BYTE             @ZA07574*/
        2 UPSECQTY CHAR(3);         /* UNIQ SEC ALLOC        @ZA07574*/
 @EJECT;
 %INCLUDE SYSLIB(IGGMCDCL);         /* CATALOG COMMON DECLARES       */
 @EJECT;
 %INCLUDE SYSLIB(IGGMCMDM);         /* CATALOG RECORD DECLARES       */
 @EJECT;
 %INCLUDE SYSLIB(IDAAMDSB);         /* AMDSB                         */
 @EJECT;
 %INCLUDE SYSLIB(IGGMSAWA);         /* SUBALLOCATE WORK AREA         */
     DCL IDAAMDS1 CHAR(256) BASED(CTGFLPT(1)); /* MOVE AMDSB         */
     RFY IDAAMDSB BASED(ADDR(UPREND)+LENGTH(UPENTVOL)); /* MAP AMDSB */
