 /********************************************************************/
 /*                                                                  */
 /*   MACRO NAME = IGGMVEDC                                          */
 /*                                                                  */
 /*   DESCRIPTIVE NAME = CATALOG VOLUME ENTRY RECORD                 */
 /*                                                                  */
 /*   FUNCTION = THESE ARE THE MAPPING STRUCTURE DECLARATIONS FOR    */
 /*              THE DERIVED VOLUME ENTRY RECORD AND THE NEW         */
 /*              VOLUME ENTRY RECORD.  THIS INCLUDES STRUCTURES      */
 /*              FOR THE FIXED HEADER INFORMATION, THE DERIVED       */
 /*              SPACE HEADER GROUP, THE DERIVED SPACE DESCRIPTOR    */
 /*              GROUP, AND THE DERIVED DATA SET DIRECTORY GROUP.    */
 /*              ALSO CONTAINED ARE THE NEW DATA SPACE GROUP, THE    */
 /*              NEW DATA SET DIRECTORY GROUP, THE NEW DATA SET      */
 /*              DIRECTORY EXTENSION GROUP, AND THE NEW SPACE MAP    */
 /*              GROUP.                                              */
 /*                                                                  */
 /*   INCLUDED MACROS = NONE                                         */
 /*                                                                  */
 /*   METHOD OF ACCESS = PL/S - RESPECIFY A BASE FOR SOME STRUCTURES */
 /*                                                                  */
 /*   STATUS = VS/2 RELEASE 3   (CHANGE FLAG @Y30SSXX)               */
 /*                                                                  */
 /*   DATE OF LAST CHANGE = 21 JUN 74                                */
 /*                                                                  */
 /********************************************************************/
 /********************************************************************/
 /*     DECLARATION OF THE FIXED INFORMATION IN THE VOLUME RECORD    */
 /********************************************************************/
    DECLARE
      1 CTVOLRCD BASED(ADDR(NREPLF)) POSITION(2), /* VOLUME RECORD   */
        2 CTVOLNME CHAR(44),        /* ENTRY NAME - 6 BYTES FOR      */
                                    /*   VOLUME SERIAL NUMBER AND    */
                                    /*   38 BYTES OF BINARY ZERO     */
                                    /*   PADDING ON RIGHT            */
        2 CTVOLTIM CHAR(8),         /* VOLUME TIME STAMP             */
        2 CTVDVCHR CHAR(20),        /* DEVICE CHARACTERISTICS        */
          3 CTVDEVTY CHAR(4),       /* VOLUME DEVICE TYPE            */
          3 CTVMXBLK PTR(31)BDY(BYTE), /* MAXIMUM DEVICE BLOCK SIZE  */
          3 CTVNOCYL FIXED(15) BDY(BYTE), /* CYLINDERS THIS DEVICE   */
          3 CTVNOTRK FIXED(15) BDY(BYTE), /* TRACKS PER CYLINDER     */
          3 CTVTRKSZ FIXED(15) BDY(BYTE),/* BYTES PER TRACK          */
          3 CTVBKNLB PTR(8),        /* BLOCK OVERHEAD, KEYED NOT LAST*/
          3 CTVBKLB  PTR(8),        /* BLOCK OVERHEAD, KEYED LAST    */
          3 CTVBONK  PTR(8),        /* BLOCK OVERHEAD, NOT KEYED     */
          3 CTVDVFLG BIT(8),        /* FLAGS                         */
            4 *        BIT(7),      /* RESERVED                      */
            4 CTVTLFLG BIT(1),      /* IF ON, USE TOLERANCE FACTOR   */
                                    /* ON ALL BLOCKS EXCEPT LAST     */
          3 CTVTLFAC FIXED(15) BDY(BYTE), /* TOLERANCE FACTOR        */
        2 CTVOLFLG BIT(8),          /* VOLUME RECORD FLAGS           */
        2 CTVSYEDS PTR(8),          /* SYSTEM ALLOWED DS EXTENTS     */
                                    /*   PER SUBALLOCATION           */
        2 *        CHAR(4);         /* RESERVED                      */
 /********************************************************************/
 /*     DECLARATION OF THE SPACE HEADER GROUP STRUCTURE THAT CAN     */
 /*       BE DERIVED FOR EACH DATA SPACE ON THE VOLUME.  NOTE:       */
 /*       IF ALL DATA SET DIRECTORIES ON THE VOLUME ARE NOT          */
 /*       RETRIEVED IN THE SAME EXTRACT CALL, THE BITS CTVSPUCT      */
 /*       AND CTVSPMCT, AND THE FIELD CTVSPNDS MAY BE INCORRECT.     */
 /********************************************************************/
    DECLARE
      1 CTVSPHDR BASED,             /* SPACE HEADER GROUP            */
        2 CTVSPHCV PTR(8),          /* COUNT OF VARIABLE FIELDS      */
        2 CTVSPHLF PTR(8),          /* LENGTH OF FIXED INFORMATION   */
        2 CTVF1TIM CHAR (8),        /* FORMAT 1 DSCB TIME STAMP      */
        2 CTVF1ADR CHAR(5),         /* CCHHR OF THE FORMAT 1 DSCB    */
        2 CTVSPFLG BIT(8),          /* DATA SPACE FLAGS              */
          3 CTVSPUNQ BIT(1),        /* UNIQUE DATA SPACE             */
          3 CTVSPAUT BIT(1),        /* AUTOMATICALLY BUILT DATA SPACE*/
          3 CTVSPUCT BIT(1),        /* USER CATALOG IN DATA SPACE    */
          3 CTVSPMCT BIT(1),        /* MASTER CATALOG IN DATA SPACE  */
          3 CTVSPCDF BIT(1),        /* DATA SPACE BUILT BY CATLG DEF */
          3 *        BIT(3),        /* RESERVED                      */
        2 CTVSPNDS FIXED(15) BDY(BYTE), /* NO. DATA SETS IN DATA SPC */
        2 CTVSPNEX PTR(8),          /* NO. DATA SPACE EXTENTS        */
        2 CTVSPSPO PTR(8),          /* DATA SPACE CREATION SPACE OPTS*/
        2 CTVSPSQT PTR(24) BDY(BYTE); /* SECONDARY SPACE QUANTITY    */
 /********************************************************************/
 /*     DECLARATION OF THE SPACE DESCRIPTOR GROUP STRUCTURE THAT     */
 /*       CAN BE DERIVED FOR EACH EXTENT OF EVERY DATA SPACE.        */
 /********************************************************************/
    DECLARE
      1 CTVDESGP BASED,             /* SPACE DESCRIPTOR GROUP        */
        2 CTVDESCV PTR(8),          /* COUNT OF VARIABLE FIELDS      */
        2 CTVDESLF PTR(8),          /* LENGTH OF FIXED INFORMATION   */
        2 CTVDESUU FIXED(15),       /* ALLOCATION UNITS USED         */
        2 CTVEXTST CHAR(4),         /* EXTENT STARTING CCHH ADDRESS  */
          3 CTVSTDES PTR(31) BDY(BYTE), /* CALCULATING CCHH FORM     */
            4 CTVSTCC FIXED(15) BDY(BYTE), /* CYLINDER START         */
            4 CTVSTHH FIXED(15) BDY(BYTE), /* TRACK START            */
        2 CTVDESTK FIXED(15) BDY(BYTE), /* NO. OF TRACKS IN EXTENT   */
        2 CTVDESHD FIXED(15) BDY(BYTE), /* SEQUENCE NUMBER OF THE    */
                                    /* ASSOCIATED SPACE HEADER GROUP */
        2 CTVMAPLN PTR(15) BDY(BYTE), /* SPACE MAP LENGTH            */
        2 CTVMAP (*) PTR(8);        /* SPACE MAP-RUN LENGTH CODE     */
 /********************************************************************/
 /*     DECLARATION OF THE DATA SET DIRECTORY GROUP THAT CAN BE      */
 /*       DERIVED FOR EACH DATA SET ON THE VOLUME.                   */
 /********************************************************************/
    DECLARE
      1 CTVDSGRP BASED,             /* DATA SET DIRECTORY GROUP      */
        2 CTVDSGCV PTR(8),          /* COUNT OF VARIABLE FIELDS      */
        2 CTVDSGLF PTR(8),          /* LENGTH OF FIXED INFORMATION   */
        2 CTVDSNEX PTR(8),          /* NO. OF DATA SET EXTENTS       */
        2 CTVDSCID PTR(24) BDY(BYTE), /* DATA SET CI ID VALUE        */
        2 CTVDSDFL BIT(8),          /* DATA SET FLAGS                */
          3 CTVDSDCN BIT(1),        /* CANDIDATE ON VOL - NO SPACE   */
          3 *        BIT(7),        /* RESERVED                      */
        2 CTVDSXCT PTR(15) BDY(BYTE), /* SPACE HDR DIRECTORY LENGTH  */
        2 CTVDSDIR (*) CHAR(3) BDY(BYTE), /* SPACE HDR SSN ENTRIES   */
          3 CTVSPSSN FIXED(15) BDY(BYTE), /* SPACE HDR GROUP SEQ NO. */
          3 CTVSPSXC PTR(8);        /* NO. OF DS EXT'S IN THE SPACE  */
 /********************************************************************/
 /*    DECLARATION OF THE VOLUME RECORD GROUP OCCURRENCE POINTERS    */
 /********************************************************************/
    DECLARE
      1 CTVGOPT BASED,              /* GROUP OCCURRENCE POINTERS     */
        2 CTVGOCI CHAR(3),          /* CI NUMBER OR                  */
          3 CTVGOFLG BIT(8),        /*   GROUP OCCURRENCE FLAG       */
          3 CTVGODSP FIXED(15)      /* DISPLACEMENT TO GROUP         */
                     BDY(BYTE),     /*   OCCURRENCE                  */
        2 CTVGOGCI PTR(8),          /* GROUP CODE INFORMATION        */
          3 CTVGOEXT BIT(1),        /* GROUP OCCURRENCE OUTSIDE RECRD*/
          3 CTVGODEL BIT(1),        /* DELETED GROUP OCCURRENCE      */
          3 CTVGOGC  BIT(6),        /* TRUE GROUP CODE               */
        2 CTVGOSEQ FIXED(15) BDY(BYTE); /* (HIGH) SEQUENCE NUMBER    */
 /********************************************************************/
 /*         DECLARATION OF THE TRUE GROUP CODES IN THE GROUP         */
 /*         OCCURRENCE POINTERS (CTVGOGCI)                           */
 /********************************************************************/
    DECLARE                         /* TRUE GROUP CODES IN G.O.P.'S  */
      CTVDSPHC BIT(8) CONSTANT('06'X), /* DATA SPACE HEADER CODE     */
      CTVDSDGC BIT(8) CONSTANT('08'X), /* DATA SPACE DIRECTORY CODE  */
      CTVDSPDC BIT(8) CONSTANT('07'X); /* DATA SPACE DESCRIPTOR CODE */
 /********************************************************************/
 /*     DECLARATION OF CONTROL INFORMATION FOR REPEATING FIELDS      */
 /********************************************************************/
    DECLARE
      1 CTVRCTRL BASED(CCACPE2),    /* CONTROL INFORMATION           */
        2 CTVEXRPT CHAR(5),         /* REPEATING CONTROL INFORMATION */
          3 CTVEXGCI CHAR(3),       /* CI NUMBER OF EXTENSION RECORD */
          3 *        CHAR(2),       /* RESERVED                      */
        2 CTVEXRCT PTR(8),          /* COUNT OF GROUP OCCUR PTRS     */
        2 CTVEGOPT (*) CHAR(6);     /* VARIABLE NO. OF GOP'S         */
 /********************************************************************/
 /*          DECLARATION OF VOLUME ENTRY GROUP CODES (NEW)           */
 /********************************************************************/
    DECLARE
      SMGC    FIXED(8) CONSTANT(5), /* SPACE MAP GROUP CODE          */
      DSPGC   FIXED(8) CONSTANT(6), /* DATA SPACE GROUP CODE         */
      DIRGC   FIXED(8) CONSTANT(8); /* DATA SET DIRECTORY GROUP CODE */
 /********************************************************************/
 /*     DECLARATION OF THE SPACE MAP GROUP USED TO SUBALLOCATE       */
 /*       VSAM DATA SPACE FOR THE ENTIRE VOLUME. THERE EXIST         */
 /*       AS MANY MAXIMUM LENGTH OCCURRENCES AS ARE NECESSARY        */
 /*       TO CONTAIN THE ENTIRE MAP.                                 */
 /********************************************************************/
    DECLARE
      1 CTVSPMAP BASED,             /* SPACE MAP GROUP               */
        2 CTVSMSN  FIXED(15),       /* SPACE MAP GROUP SEQUENCE NO.  */
          3 CTVSMCV  PTR(8),        /* COUNT OF VARIABLE FIELDS      */
          3 CTVSMLF  PTR(8),        /* LENGTH OF FIXED INFORMATION   */
        2 CTVSMLEN FIXED(15),       /* BIT MAP LENGTH                */
        2 CTVSMBIT CHAR(*);         /* VARIABLE LENGTH SPACE BIT MAP */
 /********************************************************************/
 /*     DECLARATION OF DATA SPACE GROUP STRUCTURE THAT EXISTS        */
 /*     FOR EACH VSAM DATA SPACE ON THE VOLUME                       */
 /********************************************************************/
    DECLARE
      1 CTVDSPAC BASED,             /* DATA SPACE GROUP MAPPING      */
        2 CTVDSPSN  FIXED(15),      /* DATA SPACE GROUP SEQUENCE NO. */
          3 CTVDSPCV PTR(8),        /* COUNT OF VARIABLE FIELDS      */
          3 CTVDSPLF PTR(8),        /* LENGTH FIXED INFORMATION      */
        2 CTVF1TME CHAR(8),         /* FORMAT 1 DSCB TIME STAMP      */
        2 CTVF1PTR CHAR(5),         /* CCHHR OF THE FORMAT 1 DSCB    */
        2 CTVDSPFG BIT(8),          /* DATA SPACE FLAGS              */
          3 CTVUNIQ  BIT(1),        /* UNIQUE DATA SPACE             */
          3 CTVAUTO  BIT(1),        /* AUTOMATICALLY BUILT SPACE     */
          3 *        BIT(2),        /* RESERVED                      */
          3 CTVCATDF BIT(1),        /* DATA SPACE BUILT BY CATLG DEF */
          3 *        BIT(3),        /* RESERVED                      */
        2 CTVNOEXT PTR(8),          /* NO. OF DATA SPACE EXTENTS     */
        2 CTVSPOPT BIT(8),          /* DATA SPACE CREATION SPACE OPTS*/
        2 CTVSQT PTR(24) BDY(BYTE), /* SECONDARY SPACE QUANTITY      */
        2 CTVSPEXT CHAR(64),        /* DATA SPACE EXTENT DESCRIPTOR  */
                                    /*   AREA                        */
          3 CTVEXTNT(16) CHAR(4)    /* EXTENT DESCRIPTORS            */
                         BDY(BYTE),
            4 CTVEXTT FIXED(15)     /* RELATIVE TRACK NO. OF THE     */
                      BDY(BYTE),    /*   EXTENT                      */
            4 CTVEXTNN FIXED(15)    /* NO. OF TRACKS IN THE EXTENT   */
                       BDY(BYTE);
 /********************************************************************/
 /*     DECLARATION OF DATA SET DIRECTORY GROUP THAT EXISTS FOR      */
 /*       EACH DATA SET ON THE VOLUME                                */
 /********************************************************************/
    DECLARE
      1 CTVDIR BASED,               /* DATA SET DIRECTORY            */
        2 CTVDSDSN FIXED(15),       /* DATA SET DIRECTORY SEQ. NO.   */
          3 CTVDSDCV PTR(8),        /* COUNT OF VARIABLE FIELDS      */
          3 CTVDSDLF PTR(8),        /* LENGTH OF FIXED FIELDS        */
        2 CTVDSDCI PTR(24) BDY(BYTE); /* ASSOCIATED DATA SET CI NO.  */
 /********************************************************************/
 /*     DECLARATION OF DATA SET DIRECTORY GROUP EXTENSION THAT       */
 /*     EXISTS FOR VOLUMES OWNED BY RECOVERABLE CATALOGS.            */
 /********************************************************************/
    DECLARE
      1 CTVDEXTN BASED(ADDR(CTVDIR) + LENGTH(CTVDIR)),/* DATA  SET   */
                                    /* DIRECTORY EXTENSION   @Y30SSJG*/
        2 CTVDIRTS CHAR(4),         /* TIME STAMP            @Y30SSJG*/
        2 CTVDRSTT PTR(24) BDY(BYTE); /* DATA SET SUM TT     @Y30SSJG*/
