P40  TITLE 'IKJEFP40 -- SYNTAX CHECKING ROUTINE FOR COMMAND SCAN       *
               VERSION ZERO DATE 3/16/73'                        Y02666
**********************************************************************
*
* TITLE -- IKJEFP40-  SYNTAX CHECKER ROUTINE FOR COMMAND SCAN
*
* STATUS -- VERSION ZERO
*
*
* FUNCTION/OPERATION -- GENERALIZED SCAN SYNTAX CHECKS PARAMETERS FOR
*     COMMAND SCAN ACCORDING TO CONTROL INFORMATION
*     SET UP BY THE CALLER FOR THE SCAN. THIS ROUTINE ALSO TRANSLATES
*     A PARAMETER TO UPPER CASE.
*
* ENTRY POINTS -- GENSCAN -- ENTRY POINT AT WHICH SYNTAX CHECKING OF  A
*     PARAMETER IS DONE ACCORDING TO CONTROL INFORMATION PASSED.
*    TRANSQ -- ENTRY POINT AT WHICH A PARAMETER IS TRANSLATED TO UPPER-
*     CASE.
*    TRANSX -- ENTRY POINT TO TRANSLATE TO UPPERCASE IF PARAMETER
*     IS KNOWN NOT TO BE DEFAULTED.
*    TRTAB -- ENTRY POINT TO ACCESS TABLE ASSIGNING A CHARACTER CLASS
*     TO AN EBCIDIC CHARACTER.
*    UPPERTAB -- ENTRY POINT TO ACCESS TRANSLATE TABLE ONLY.
*
* INPUT -- PDWORD CONTAINS A POINTER TO A WORD CONTAINING
*     CONTROL INFORMATION
*      **********************************************************
*      / SCAN OPTIONS / FIRST CHARACTER /OTHER CHARACTER/MAXIMUM/
*      /              /    TYPE         /    TYPE       /LENGTH /
*      **********************************************************
*      0              1                 2               3
*     REGISTER FOUR CONTAINS POINTER TO PARAMETER TO BE SCANNED.
*     --TRANSLATION FUNCTION - PPOINTER CONTAINS ADDRESS
*     OF PARAMETER TO BE TRANSLATED
*     PLENGTH CONTAINS LENGTH OF PARAMETER.
*
* OUTPUT -- THE TRANSLATED PARAMETER IN ITS ORIGINAL BUFFER.
*
* EXTERNAL REFERENCES -- NONE
*
* EXITS- NORMAL-- FROM SCAN ROUTINE EXITS TO THE LINK REGISTER
*     ADDRESS
*      +0 - IF PARAMETER IS MISSING
*      +4 - IF PARAMETER IS INVALID
*      +8 - IF END OF BUFFER IS ALSO PARAMETER END
*      +12- IF PARAMETER NOT AT END OF BUFFER
*
*      --FROM TRANSLATION ROUTINE EXITS TO LINK REGISTER ADDRESS
*
* EXITS- ERROR-- NONE
*
* TABLES/WORKAREAS -- TRTAB-TABLE WHICH ASSIGNS CHARACTER CLASS TO
*    EACH EBCIDIC CONFIGURATION
*    CHARTYPE-TABLE WHICH SELECTS MASK FOR TYPE OF CHARACTER REQUIRED
*    BY ROUTINE WHICH TEST CHARACTER TYPE.
*    IKJEFPWA-MACRO MAPPING WORK AREA.
* ATTRIBUTES -- REENTRANT
*
* CHARACTER CODE DEPENDENCY -- CLASS C. THE OPERATION OF THIS MODULE
*     IS DEPENDENT UPON AN INTERNAL REPRESENTATION OF THE EXTERNAL CHAR
*     ACTER SET WHICH IS EQUIVALENT TO THE ONE USED AT ASSEMBLY TIME.
*     THE CODING HAS BEEN ARRANGED SO THAT REDEFINITION OF 'CHARACTER'
*     CONSTANTS,BY REASSEMBLY, WILL RESULT IN A CORRECT MODULE FOR THE
*     NEW DEFINITION.
*
* RELEASE 20 SUPPORT CODE -- 20035
*
* RELEASE 2.0  (VS/2)   --- Y02666
*
* NOTES: FLAGGING CODES ARE R=DCRR,A=APAR,M=PTM,F=PTF,S=SUPPORT CODE
*
**********************************************************************
         EJECT
IKJEFP40 CSECT
*A 0-999999                                                      Y02666
         ENTRY GENSCAN,TRANSQ,TRANSX,TRTAB,UPPERTAB
         SPACE
*
* REGISTER EQUATES.
*
R0       EQU   0                       SCRATCH/PARAMETER REGISTER --
*                                         MUST BE 0
R1       EQU   1                       SCRATCH/PARAMETER REGISTER --
*                                         MUST BE 1
R2       EQU   2                       GENERAL SCRATCH REGISTER
R3       EQU   3                       GENERAL SCRATCH REGISTER
XINPUT   EQU   4                       NEXT CHARACTER TO SCAN
XINPUTB  EQU   5                       LAST CHARACTER SCANNED USED TO
*                                         COMPUTE LENGTH OF SCANNED
*                                         DATA
XPCE     EQU   6                       IF CALLED BY PARSE, POINTS TO
*                                         THE CURRENT PCE. IF CALLED
*                                         BY COMMAND SCAN, NOT USED.
R7       EQU   7                       NOT USED
LINK2    EQU   8                       SECOND LEVEL RETURN REGISTER
LINK1    EQU   9                       FIRST LEVEL RETURN REGISTER
R10      EQU   10                      NOT USED
R11      EQU   11                      NOT USED
R12      EQU   12                      NOT USED
WORKBASE EQU   13                      BASE REGISTER FOR COMMON
*                                         WORKSPACE -- MUST BE 13
R14      EQU   14                      SCRATCH REGISTER
BASE     EQU   15                      LINKAGE REGISTER USED AS
*                                         BASE REGISTER
*
* BIT SETTINGS FOR CHARACTER TYPES USED BY TYPETEST
*
HEX      EQU   X'80'                   HEX CHARACTER
OLETTER  EQU   X'40'                   LETTER NOT A HEX CHARACTER
NATL     EQU   X'20'                   NATIONAL CHARACTER
NUMBER   EQU   X'10'                   NUMBER
SEPAR    EQU   X'08'                   SEPARATOR
NSEPDLIM EQU   X'04'                   DELIMITER THAT IS NOT A
*                                         A SEPARATOR
NDLIMSPC EQU   X'02'                   SPECIAL CHARACTER THAT IS NOT
*                                         ALSO A DELIMITER OR A
*                                         SEPARATOR
CMDDLIM  EQU   X'01'                   COMMAND NAME DELIMITER
INVALID  EQU   X'00'                   INVALID CHARACTER
         SPACE
*
* BIT PATTERNS USED TO TEST THE OPTIONS SELECTED BY THE PARSE USER AND
* REFLECTED IN VARIOUS BYTES IN THE PCE.
*
PCEFPRPT EQU   B'00010000'             BIT 3  - PROMPT IS SPECIFIED
PCEFDFLT EQU   B'00001000'             BIT 4  - DEFAULT IS SPECIFIED
PCEFSUBF EQU   B'00000100'             BIT 5  - SUBFIELD IS SPECIFIED
PCEFHELP EQU   B'00000010'             BIT 6  - HELP IS SPECIFIED
PCEFVCHK EQU   B'00000001'             BIT 7  - VALIDITY CHECK EXIT IS
*                                         SPECIFIED
PCEFLIST EQU   B'10000000'             BIT 8  - LIST IS SPECIFIED
PCEFASIS EQU   B'01000000'             BIT 9  - NO TRANSLATION REQUIRED
PCEFRNGE EQU   B'00100000'             BIT 10 - RANGE IS SPECIFIED
PCEFINST EQU   B'00010000'             BIT 11 - INSERT IS SPECIFIED
PCEFASTK EQU   B'10000000'             BIT 0 - ASTERISK IS ALLOWED
PCEFMAXL EQU   B'01000000'             BIT 1  - MAXLNTH IS SPECIFIED
         SPACE
*
* OFFSETS FOR REQUIRED FIELDS IN PARSE PCE'S.
*
PCEPCLLN EQU   0                       PCL LENGTH FIELD IN IKJPARM
PCEPDLLN EQU   2                       PDL LENGTH FIELD IN IKJPARM
PCEKYEND EQU   4                       IKJKEYWD OR END-OF-FIELD OFFSET
*                                         IN IKJPARM
PCEFLGB1 EQU   0                       FLAG BYTE 1
PCEFLGB2 EQU   1                       FLAG BYTE 2
PCELEN   EQU   2 - 3                   PCE LENGTH
PCEPDEO  EQU   4 - 5                   PDE OFFSET INTO PDL
PCEPOST  EQU   6                       TYPE OF POSITIONAL PARAMETER
PCENAML  EQU   4                       LENGTH OF NAME FOR IKJNAME
PCENAMN  EQU   5 - N                   NAME SPECIFIED
PCEOPT   EQU   6                       IKJIDENT OPTION BYTE
PCEFIRST EQU   7                       IKJIDENT FIRST CHARACTER FLAGS
PCEOTHER EQU   8                       IKJIDENT OTHER CHARACTER FLAGS
PCEPARMT EQU   9                       IKJIDENT PARAMETER TYPE SEGMENT
         EJECT
*
* MISCELLANEOUS EQUATES
*
ZERO     EQU   0                       USED AS A 0
ONE      EQU   1                       USED AS A 1
TWO      EQU   2                       USED AS A 2
FOUR     EQU   4                       USED AS A 4
EIGHT    EQU   8                       USED AS A 8
TWELVE   EQU   12                      USED AS A 12
CC1      EQU   1                       CONDITION CODE 1
CC5      EQU   5                       CONDITION CODE 5
CC8      EQU   8                       CONDITION CODE 8
CC13     EQU   13                      CONDITION CODE 13
IPDLMAXE EQU   10                      MAXIMUM INPUT STACKING DEPTH FOR
*                                         A INPUT PUSHDOWN STACK
ASTERISK EQU   C'*'                    ASTERISK
LEFTPRN  EQU   C'('                    LEFT PARENTHESIS
RIGHTPRN EQU   C')'                    RIGHT PARENTHESIS
HE0      EQU   X'E0'                   USED TO OBTAIN TYPE INDICATOR
*                                         FROM PCE FLAG BYTE
         SPACE
*
* PCE TYPE USED TO DETERMINE THE TYPE OF PCE BEING PROCESSED
*
KEYWDPCE EQU   X'40'                   KEYWORD PCE
RSVDPCE  EQU   X'A0'                   RESERVED WORD PCE FOR     F41448
*                                      COBOL                     F41448
     EJECT
GENSCAN  DS    0H                      GENERALIZED SCAN ROUTINE
         SPACE
*
* ESTABLISH ADDRESSABILITY USING THE CALL REGISTER (R15) AS A
* BASE REGISTER.
*
         USING *,BASE                  ESTABLISH ADDRESSABILITY
*                                         FOR GENSCAN
         MODID BRANCH=YES              FLAG                      Y01886
         USING PWORK,WORKBASE          ESTABLISH ADDRESSABILITY TO
*                                         COMMON WORKAREA
         L     R3,PDWORD               GET PTR TO GENSCAN CONTROL INFO
         USING GCONTROL,R3             ESTABLISH ADDRESSABILITY TO
*                                         GENSCAN CONTROL INFORMATION
         LA    XINPUT,ONE(XINPUT)      GET NEXT CHARACTER
         CLC   GFIRST(L'GFIRST+L'GOTHER),H0 IS THIS AN ANY/ANY
*                                         COMBINATION
         BZ    GS02                    IF YES BRANCH --- IF ASTERISK
*                                         WAS SUPPLIED IT IS REDUNDANT
*                                         WITH ANY/ANY OPTIONS
         SPACE
         TM    GOPTIONS,ASTKALWD       IS AN ASTERISK ALLOWED
         BZ    GS02                    NO, CONTINUE
         SPACE
         CLI   ZERO(XINPUT),ASTERISK   IS THERE AN ASTERISK IN THE
*                                         BUFFER
         BNE   GS02                    NO, CONTINUE
         SPACE
GS015    DS    0H                      ONE CHARACTER RETURN
         LA    XINPUT,ONE(XINPUT)      GET NEXT CHARACTER
         LR    XINPUTB,XINPUT          SET LAST CHAR SCANNED
         C     XINPUTB,ENDINPUT        IS SCAN AT END OF CURRENT INPUT
         BNL   EIGHT(LINK2)            YES, RETURN +8 INDICATING THE
*                                         PARAMETER IS GOOD AND ENDED
*                                         AT THE END OF THE BUFFER
         SPACE
         B     TWELVE(LINK2)           NO, RETURN +12 INDICATING THE
*                                         PARAMETER IS GOOD BUT THE
*                                         DELIMITER MUST BE CHECKED
         SPACE
GS02     DS    0H                      CHECK FIRST CHARACTER
         XR    R2,R2                   CLEAR DEPTH METER
         XR    R1,R1                   CLEAR WORK REG
         IC    R1,GFIRST               GET FIRST CHARACTER TYPE
*                                         INDICATOR AND USE AS INDEX
*                                         INTO TYPE DESIRED TABLE
         IC    R1,CHARTYPE(R1)         SET TYPE DESIRED FOR TYPETEST
         BAL   LINK1,TYPETEST          TEST FIRST CHARACTER SYNTAX
         SPACE
         B     ZERO(LINK2)             +0 RETURN - THE FIRST CHARACTER
*                                         IS INVALID.  RETURN IS TO
*                                         LINK2+0 INDICATING THE
*                                         PARAMETER IS MISSING
*
*                                      +4 RETURN - THE FIRST CHARACTER
*                                         IS VALID, CONTINUE
         SPACE
         CLI   ZERO(XINPUT),LEFTPRN    IS THE CHARACTER A LEFT PAREN
         BNE   GS03                    IF NO SKIP SETTING OF DEPTH
*                                         METER
         SPACE
         LA    R2,ONE                  SET DEPTH METER TO ONE
         SPACE
GS03     DS    0H                      * * * *
         ST    XINPUT,PPOINTR          SAVE PTR TO PARM
         TM    GOPTIONS,MAXLSPEC       IS A MAX LENGTH SPECIFIED
         BZ    GS04                    NO, CONTINUE
         SPACE
         CLI   GMAXLNTH,ONE            IS THE MAX LENGTH 1
         BE    GS015                   YES, RETURN AS WITH ASTERISK
         SPACE
         XR    R0,R0                   ZERO LOOP CONTROL REG
         IC    R0,GMAXLNTH             USE MAXLENGTH FOR CONTROL OF
*                                         SCAN LOOP
         SPACE
GS04     DS    0H                      * * * *
         XR    R1,R1                   CLEAR WORK REG
         IC    R1,GOTHER               GET OTHER CHARACTER TYPE
*                                         INDICATOR AND USE AS INDEX
*                                         INTO TYPE DESIRED TABLE
         IC    R1,CHARTYPE(R1)         SET TYPE DESIRED FOR TYPETEST
         SPACE
GSLOOP   DS    0H                      OTHER LOOP
         LA    XINPUT,ONE(XINPUT)      GET NEXT CHARACTER
         LR    XINPUTB,XINPUT          SET LAST CHARACTER SCANNED
         C     XINPUT,ENDINPUT         IS SCAN AT END OF CURRENT INPUT
         BNL   EIGHT(LINK2)            YES, RETURN +8 INDICATING THE
*                                         PARAMETER IS GOOD AND ENDED
*                                         AT THE END OF THE BUFFER
         SPACE
         BAL   LINK1,TYPETEST          TEST CHARACTER SYNTAX
         SPACE
         B     GS06                   +0 RETURN - THE CHARACTER IS
*                                         AN INVALID OTHER CHARACTER.
*                                         SEE IF THE DEPTH METER
*                                         SHOULD BE CHECKED
         SPACE
*                                      +4 RETURN - THE CHARACTER IS
*                                         A VALID OTHER CHARACTER,
*                                         CONTINUE
         SPACE
         CLI   GOTHER,ZERO             IS THE OTHER CHARACTER AN ANY
         BNE   GS05                    IF NO SKIP OVER FURTHER CHECKS
         SPACE
         CLI   ZERO(XINPUT),LEFTPRN    IS THE NEXT CHARACTER A LEFT
*                                         PARENTHESIS
         BNE   GS05                    IF NO SKIP SETTING DEPTH METER
         SPACE
         LA    R2,ONE(R2)              INCREMENT DEPTH METER BY ONE
         SPACE
GS05     DS    0H                      * * * *
         TM    GOPTIONS,MAXLSPEC       IS A MAX LENGTH SPECIFIED
         BZ    GSLOOP                  NO, SCAN NEXT CHAR
         SPACE
         BCT   R0,GSLOOP               YES, DECREMENT LENGTH AND TEST
*                                         FOR 0, IF GREATER THAN 0 SCAN
*                                         NEXT CHARACTER
         SPACE
         B     FOUR(LINK2)             IF R0 REACHES 0, THE
*                                      MAXIMUM LENGTH HAS BEEN
*                                      EXCEEDED.  RETURN IS TO LINK2+4
*                                      INDICATING THE PARAMETER IS
*                                      INVALID
         SPACE
GS06     DS    0H                      * * * *
         CLI   GOTHER,ZERO             IS THE OTHER CHARACTER AN ANY
         BNE   TWELVE(LINK2)           RETURN TO LINK2+12 INDICATING
*                                         THE PARAMETER IS GOOD BUT THE
*                                         DELIMITER MUST BE CHECKED
         SPACE
         CLI   ZERO(XINPUT),RIGHTPRN   IS CHARACTER A RIGHT PAREN
         BNE   TWELVE(LINK2)           IF NO EXIT
         SPACE
         BCTR  R2,ZERO                 DECREMENT DEPTH METER
         LTR   R2,R2                   IS THIS THE END OF THE PARAMETER
         BM    TWELVE(LINK2)           IF YES EXIT
         SPACE
         B     GS05                    NOT PARAMETER END CONTINUE SCAN
         EJECT
***********************************************************************
*                                                                     *
*                 CHARACTER TYPE TEST ROUTINE                         *
*                                                                     *
* THIS IS AN INTERNAL SUBROUTINE WHICH CHECKS THE CURRENT CHARACTER   *
* FOR BELONGING TO A SPECIFIED CHARACTER CLASS.                       *
*                                                                     *
* INPUT - R1=MASK OF DESIRED CHARACTER CLASS.                         *
*         XINPUT=POINTER TO CHARACTER TO BE CHECKED.                  *
*                                                                     *
* OUTPUT - THE RESULT OF THE TEST IS INDICATED BY THE                 *
*          LOCATION TO WHICH CONTROL IS RETURNED.                     *
*                                                                     *
* EXISTS - LINK1+0=THE CHARACTER DOES NOT BELONG TO THE SPECIFIED     *
*                  CHARACTER CLASS.                                   *
*                                                                     *
*        - LINK1+4=THE CHARACTER BELONGS TO THE CHARACTER CLASS       *
*                  SPECIFIED.                                         *
*                                                                     *
***********************************************************************
         SPACE
TYPETEST DS    0H                      * * * *
         XR    R14,R14                 CLEAR WORK REG
         IC    R14,ZERO(XINPUT)        USE CURRENT CHARACTER AS
*                                         OFFSET INTO TESTING TABLE
         LA    R14,TRTAB(R14)          GET ADDRESS IN TABLE
*                                         CORRESPONDING TO CHARACTER IN
*                                         QUESTION
         EX    R1,TYPETM               EXECUTE TEST UNDER MASK OF
*                                         TYPEBYTE FOR CHARACTER
*                                         SPECIFICATION GIVEN IN R1
*
         BC    CC5,FOUR(LINK1)         TYPE MATCHES, RETURN +4
*
         BR    LINK1                   TYPE DOESN'T MATCH, RETURN +0
         EJECT
***********************************************************************
*                                                                     *
*                       TRANSLATE INPUT DATA SUBROUTINE               *
*                                                                     *
*   THIS ROUTINE TRANSLATES THE INPUT DATA TO UPPERCASE UNLESS THE    *
* ASIS OPTION WAS SELECTED BY THE USER. THE INPUT DATA ADDRESS IS     *
* TAKEN FROM PPOINTR AND THE LENGTH FROM PLENGTH. IF THE DATA LIES    *
* WITHIN THE PCE (DEFAULTED) NO TRANSLATION TAKES PLACE.              *
*                                                                     *
***********************************************************************
         SPACE
TRANSQ   DS    0H                      TRANSLATION SUBROUTINE
         USING *,BASE                  ESTABLISH ADDRESSABILITY
*                                         FOR TRANSLATE ROUTINE
         SPACE
         MVC   PDWORD(ONE),PCEFLGB1(XPCE) MOVE PCE FLAG
*                                         BYTE TO WORKAREA
         NI    PDWORD,HE0              ISOLATE PCE TYPE INDICATORS
         CLI   PDWORD,KEYWDPCE         IS IT A KEYWORD PCE
         BE    TRANS                   YES, TRANSLATE
         SPACE
         CLI   PDWORD,RSVDPCE          IS IT A RESERVED WORD     F41448
*                                      PCE?                      F41448
         BO    TRANS                   IF YES, TRANSLATE         F41448
*                                      REGARDLESS                F41448
         SPACE
         TM    PCEFLGB2(XPCE),PCEFASIS SHOULD THE DATA BE TRANSLATED TO
*                                         UPPERCASE
         BCR   CC1,LINK1               NO RETURN TO CALLER
         SPACE
TRANS    DS    0H                      BRANCHED TO FOR TRANSLATION OF A
*                                         KEYWORD PARAMETER
         NC    PPOINTR+ONE(L'PPOINTR-ONE),PPOINTR+ONE IS THERE ANY DATA
*                                         TO CONVERT
         BCR   CC8,LINK1               IF NO RETURN TO CALLER
         SPACE
         CLC   PPOINTR+ONE(L'PPOINTR-ONE),PTABLEAD+ONE DOES THE DATA
*                                         LIE WITHIN THE PCL
         BL    TRANS1                  NO BRANCH - CANNOT BE DEFAULT
         SPACE
         CLC   PPOINTR+ONE(L'PPOINTR-ONE),PTABLEND+ONE DOES THE DATA
*                                         LIE WITHIN THE PCL
         BCR   CC13,LINK1              YES WITHIN PCL MUST BE DEFAULT
         B     TRANS1                  BRANCH AROUND TRANSX CODE
         SPACE
***********************************************************************
*                                                                     *
* IF TRANSLATE ROUTINE IS ENTERED AT ENTRY POINT TRANSX, ADJUST       *
* THE VALUE OF THE BASE REGISTER (WHICH IS ALSO THE CALL REGISTER)    *
* SO THAT LOCATIONS ABOVE THIS ENTRY POINT WILL BE ADDRESSABLE        *
* BY INSTRUCTIONS BELOW THIS ENTRY POINT.                             *
***********************************************************************
         SPACE
TRANSX   DS    0H                      * * * *
         LA    R14,TRANSX-TRANSQ       LOAD SCRATCH REG WITH
*                                         DIFFERENCE IN ENTRY POINTS
         SR    BASE,R14                ADJUST VALUE OF BASE REGISTER
*                                         TO THAT AT ENTRY POINT TRANSQ
         SPACE
*
* PROCEED TO TRANSLATE DATA TO UPPERCASE.
*
TRANS1   DS    0H                      * * * *
         LH    R14,PLENGTH             LOAD LENGTH OF DATA
         LTR   R14,R14                 ANY DATA TO TRANSLATE
         BCR   CC8,LINK1               NO ZERO RETURN TO CALLER
         SPACE
         BCTR  R14,ZERO                REDUCE FOR 'EX' INSTRUCTION
         L     R1,PPOINTR              LOAD ADDRESS OF DATA
         EX    R14,TRANSTR             TRANSLATE TO UPPERCASE
         BR    LINK1                   RETURN TO CALLER
         SPACE
H0       DC    H'0'                    HALFWORD OF ZEROES FOR CLC
         SPACE
DATE     DC    C'IKJEFP40-4/7/73'      MODULE LEVEL IDENTITY CONSTANT
         SPACE
*
* TRANSLATION TABLE USED BY TRANSLATE TO UPPERCASE ROUTINE.
*
UPPERTAB DC    256AL1(*-UPPERTAB)      GENERATE BASIC TABLE
         ORG   UPPERTAB+C'A'-X'40'     POSITION INST. CTR. TO SMALL A-Z
         DC    (C'Z'-C'A'+1)AL1(*-UPPERTAB+X'40') RAISE LOWER TO UPPER
         ORG   UPPERTAB+256            RESET INSTRUCTION COUNTER
         EJECT
*
* THIS 256 BYTE TABLE IS USED TO TEST IF A GIVEN INPUT CHARACTER
* BELONGS TO A CHARACTER CLASS. EACH EBCIDIC CONFIGURATION IS
* ASSIGNED A CHARACTER CLASS
*
TRTAB    DC    256AL1(INVALID)         INITIALIZE TYPETEST TABLE
         ORG   TRTAB+X'05'
         DC    AL1(SEPAR+CMDDLIM)      HORIZONTAL TAB = SEPARATOR AND
*                                         COMMAND DELIMITER
         ORG   TRTAB+C'
'                                               22700002
         DC    AL1(NSEPDLIM+CMDDLIM)   NEW LINE CHAR = DELIMITER AND
*                                         COMMAND DELIMITER
         ORG   TRTAB+C' '
         DC    AL1(SEPAR+CMDDLIM)      BLANK = SEPARATOR AND COMMAND
*                                         DELIMITER
         ORG   TRTAB+X'4A'
         DC    AL1(NDLIMSPC)           CENT SIGN = SPECIAL
         ORG   TRTAB+C'.'
         DC    AL1(NDLIMSPC+CMDDLIM)   PERIOD = SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'<'
         DC    AL1(NDLIMSPC)           LESS THAN = SPECIAL
         ORG   TRTAB+C'('
         DC    AL1(NDLIMSPC+CMDDLIM)   LEFT PAREN = SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'+'
         DC    AL1(NDLIMSPC)           PLUS = SPECIAL
         ORG   TRTAB+C'|'
         DC    AL1(NDLIMSPC)           OR = SPECIAL
         ORG   TRTAB+C'&&'
         DC    AL1(NDLIMSPC+CMDDLIM)   AMPERSAND = SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'!'
         DC    AL1(NDLIMSPC)           EXCLAMATION = SPECIAL
         ORG   TRTAB+C'$'
         DC    AL1(NATL)               DOLLAR SIGN = NATIONAL
         ORG   TRTAB+C'*'
         DC    AL1(NDLIMSPC)           ASTERISK= SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C')'
         DC    AL1(NSEPDLIM+CMDDLIM)   RIGHT PAREN = SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C';'
         DC    AL1(NSEPDLIM+CMDDLIM)   SEMICOLON = DELIMITER
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'^'
         DC    AL1(NDLIMSPC)           NOT SIGN = SPECIAL
         ORG   TRTAB+C'-'
         DC    AL1(NDLIMSPC+CMDDLIM)   MINUS = SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'/'
         DC    AL1(NDLIMSPC+CMDDLIM)   SLASH = DELIMITER
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C','
         DC    AL1(SEPAR+CMDDLIM)      COMMA = SEPARATOR
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'%'
         DC    AL1(NDLIMSPC)           PERCENT = SPECIAL
         ORG   TRTAB+C'_'
         DC    AL1(NDLIMSPC)           DASH = SPECIAL
         ORG   TRTAB+C'>'
         DC    AL1(NDLIMSPC)           GREATER-THAN = SPECIAL
         ORG   TRTAB+C'?'
         DC    AL1(NDLIMSPC)           QUESTION MARK = SPECIAL
         ORG   TRTAB+C':'
         DC    AL1(NDLIMSPC)           COLON = SPECIAL
         ORG   TRTAB+C'#'
         DC    AL1(NATL)               POUND SIGN = NATIONAL
         ORG   TRTAB+C'@'
         DC    AL1(NATL)               AT SIGN = NATIONAL
         ORG   TRTAB+C''''             QUOTE
         DC    AL1(NDLIMSPC+CMDDLIM)   QUOTE = SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'='
         DC    AL1(NDLIMSPC+CMDDLIM)   EQUAL = SPECIAL
*                                         AND COMMAND DELIMITER
         ORG   TRTAB+C'"'
         DC    AL1(NDLIMSPC)           DOUBLE QUOTE = SPECIAL
         ORG   TRTAB+X'81'
         DC    (C'F'-C'A'+1)AL1(HEX)   SMALL A THRU F = HEX
         DC    (C'I'-C'G'+1)AL1(OLETTER) SMALL G THRU I = OTHER LETTER
         ORG   TRTAB+C'J'
         ORG   TRTAB+X'91'
         DC    (C'R'-C'J'+1)AL1(OLETTER) SMALL J THRU R = OTHER LETTER
         ORG   TRTAB+C'S'
         ORG   TRTAB+X'A2'
         DC    (C'Z'-C'S'+1)AL1(OLETTER) SMALL S THRU Z = OTHER LETTER
         ORG   TRTAB+C'A'
         DC    (C'F'-C'A'+1)AL1(HEX)   A THRU F = HEX
         DC    (C'I'-C'G'+1)AL1(OLETTER) G THRU I = OTHER LETTER
         ORG   TRTAB+C'J'
         DC    (C'R'-C'J'+1)AL1(OLETTER) J THRU R = OTHER LETTER
         ORG   TRTAB+C'S'
         DC    (C'Z'-C'S'+1)AL1(OLETTER) S THRU Z = OTHER LETTER
         ORG   TRTAB+C'0'
         DC    (C'9'-C'0'+1)AL1(NUMBER) 0 THRU 9 = NUMBER
         ORG   TRTAB+256
         SPACE
TRANSTR  TR    ZERO(*-*,R1),UPPERTAB   TO CONVERT TO UPPERCASE
         SPACE
TYPETM   TM    ZERO(R14),*-*           USED BY TYPETEST TO TEST
*                                         FOR A CHARACTER TYPE
         SPACE
*
* TABLE USED BY GENSCAN TO SELECT THE CORRECT MASK TO BE PASSED
* TO TYPETEST GIVEN THE TYPE OF CHARACTER REQUIRED IN THE FORM
* OF A HEX NUMBER.
*
CHARTYPE DC    0H'0'                   * * * *
ANY      DC    AL1(HEX+OLETTER+NATL+NUMBER+NDLIMSPC) ALL ALPHABETIC
*                                         NUMERIC,NATIONAL AND
*                                         ENTERABLE SPECIAL CHARACTERS
*                                         THAT ARE NOT DELIMITERS
ALPHA    DC    AL1(HEX+OLETTER+NATL)   ALL ALPHABETIC CHARACTERS
NUMERIC  DC    AL1(NUMBER)             ALL NUMERIC CHARACTERS
ALPHANUM DC    AL1(HEX+OLETTER+NATL+NUMBER) ALL ALPHABETIC AND
*                                         NUMERIC CHARACTERS
NONATABC DC    AL1(HEX+OLETTER)        ALL ALPHABETICS EXCLUDING  R0301
*                                      THE NATIONAL CHARACTERS    R0301
NONATNUM DC    AL1(HEX+OLETTER+NUMBER) ALL ALPHABETICS AND        R0301
*                                      NUMERICS EXCLUDING THE     R0301
*                                      NATIONAL CHARACTERS        R0301
         SPACE
*
* MAPPING OF THE CONTROL INFORMATION PASSED TO GENSCAN BY THE CALLER
* THE ADDRESS OF THE CONTROL INFORMATION IS CONTAINED IN PDWORD
* IN THE COMMON WORKAREA.
*
GCONTROL DSECT
GOPTIONS DS    X                       FLAG BYTE WHICH INDICATES THE
*                                         SCAN OPTIONS
GFIRST   DS    X                       FIRST CHARACTER TYPE - A HEX
*                                         NUMBER
GOTHER   DS    X                       OTHER CHARACTERS TYPE - A HEX
*                                         NUMBER
GMAXLNTH DS    X                       MAXIMUM LENGTH (OPTIONAL)
         SPACE
*
* GENSCAN OPTIONS FLAGS.
*
ASTKALWD EQU   X'80'                   AN ASTERISK MAY BE SUBSTITUTED
*                                         FOR THE PARM
MAXLSPEC EQU   X'40'                   MAXIMUM LENGTH IS SPECIFIED
         EJECT
PWORK    IKJEFPWA
         END
