PRPU     TITLE 'HASP PRINT/PUNCH SERVICE PROLOG'
***********************************************************************
*                                                                     *
* MODULE NAME = HASJES20 ( HASPPRPU CSECT )                           *
*                                                                     *
* DESCRIPTIVE NAME = JES2 PRINT/PUNCH PROCESSOR                       *
*                                                                     *
* COPYRIGHT = NONE                                                    *
*                                                                     *
* STATUS = OS/VS2 MVS   --  SEE &VERSION (BELOW) FOR JES2 LEVEL       *
*                                                                     *
* FUNCTION = THE HASPPRPU CSECT PROVIDES SELECTIVE PROCESSING OF      *
*            JOB SYSOUT OUTPUT WITH INTERFACES TO THE CONSOLE         *
*            OPERATOR FOR DIRECTION.                                  *
*                                                                     *
* NOTES = SEE BELOW                                                   *
*                                                                     *
*    DEPENDENCES = IF A 3211 PRINTER IS TO BE USED AS AN OUTPUT       *
*                  DEVICE, THE PRINT POSITION INDEXING FEATURE        *
*                  MUST BE INSTALLED.                                 *
*                                                                     *
*                  THE SYSTEM LIBRARY SYS1.IMAGELIB MUST CONTAIN      *
*                  ANY UCS OR FCB IMAGES SPECIFIED BY USER JCL.       *
*                  IF NOT, THE OPERATOR WILL GET A DIAGNOSTIC         *
*                  MESSAGE AND BE REQUIRED TO SPECIFY AN IMAGE        *
*                  NAME WHICH IS IN SYS1.IMAGELIB.                    *
*                                                                     *
*                  ALL FCB IMAGES OR CARRIAGE TAPES USED WITH JES2    *
*                  MUST CONTAIN A CHANNEL 1 PUNCH IN LINE POSITION    *
*                  1 FOR PROPER FORMS ALLIGNMENT.                     *
*                                                                     *
*                  3525 PRINT CCW'S DIRECTED TO A 3525 WITHOUT THE    *
*                  PRINT FEATURE WILL BE CONVERTED TO PUNCH CCW'S.    *
*                                                                     *
*                  A REQUEST TO INTERPRET CARDS ON A 3525 PUNCH       *
*                  WILL BE IGNORED IF THE PRINT FEATURE IS NOT        *
*                  INSTALLED.                                         *
*                                                                     *
*    RESTRICTIONS = IF THE FUNC=I INTERPRET FEATURE IS TO BE USED     *
*                   WITH A 3525 PUNCH, THE JES2 INITIALIZATION        *
*                   PARAMETER &NOPUCCW MUST BE AT LEAST 2.            *
*                                                                     *
*                   ALL DATA AREAS PRINTED/PUNCHED-FROM MUST BE       *
*                   PAGE-FIXED FOR EXCPVR PROCESSING.                 *
*                                                                     *
*    REGISTER CONVENTIONS = R0  =       = PARAMETER REGISTER          *
*                           R1  =       = PARAMETER REGISTER          *
*                           R2  = PW    = WORK REGISTER               *
*                           R3  = PBUF  = BUFFER ADDRESSABILITY       *
*                           R4  = PC1   = CCW WORK REGISTER 1         *
*                           R5  = PC2   = CCW WORK REGISTER 2         *
*                           R6  = BASE4 = PROCESSOR ADDRESSABILITY    *
*                           R7  = PL    = SECONDARY LINK REGISTER     *
*                           R8  = BASE3 = PROCESSOR ADDRESSABILITY    *
*                           R9  =       = RESERVED                    *
*                           R10 = JCT   = JCT ADDRESSABILITY          *
*                           R11 = BASE1 = HCT ADDRESSABILITY          *
*                           R12 = BASE2 = PROCESSOR ADDRESSABILITY    *
*                           R13 = SAVE  = PCE ADDRESSABILITY          *
*                           R14 = LINK  = PRIMARY LINK REGISTER       *
*                           R15 =       = RETURN REGISTER             *
*                           SEE ENTRY POINT DESCRIPTIONS FOR          *
*                           EXCEPTIONS.                               *
*                                                                     *
*    PATCH LABEL = NONE                                               *
*                                                                     *
* MODULE TYPE = PROCEDURE, TABLE ( CSECT TYPE )                       *
*                                                                     *
*    PROCESSOR = OS/VS ASSEMBLER                                      *
*                                                                     *
*    MODULE SIZE = SEE $DLENGTH MACRO EXPANSION(S) AT END OF ASSEMBLY *
*                                                                     *
*    ATTRIBUTES = READ ONLY, EXCEPT WHEN DEBUG OPTION SELECTED, AND   *
*                 HASP REENTRANT                                      *
*                                                                     *
* ENTRY POINT = HASPIMAG                                              *
*                                                                     *
*    PURPOSE = THIS ENTRY IS IDENTIFIED AND ATTACHED BY MODULE        *
*              HASPINIT.  ITS FUNCTION IS TO LOAD FCB AND UCS IMAGES  *
*              FROM SYS1.IMAGELIB WITHOUT CAUSING THE MAIN JES2 TASK  *
*              TO WAIT.                                               *
*                                                                     *
*    LINKAGE = THIS SUBTASK IS ACTIVATED WHEN NEEDED BY A JES2 PRINT  *
*              PUNCH PROCESSOR BY A OS/VS POST WITH A BUFFER ADDRESS  *
*              AS THE POST CODE.  THIS BUFFER CONTAINS A BLDL LIST    *
*              FOR THE REQUIRED IMAGE.  WHEN THE IMAGE IS LOADED,     *
*              THE SUBTASK MOVES IT TO THE BUFFER AND USES $$POST     *
*              TO ALERT THE WAITING PRINT/PUNCH PROCESSOR.  THE       *
*              LOADED IMAGE IS THEN DELETED AND THE SUBTASK WAITS.    *
*                                                                     *
* ENTRY POINT = HASPHOPE                                              *
*                                                                     *
*    PURPOSE = THIS IS THE ENTRY POINT TO THE JES2 OUTPUT PROCESSOR.  *
*              THE OUTPUT PROCESSOR CONVERTS THE OUTPUT REQUIREMENTS  *
*              FOR A JOB DESCRIBED BY THE PDDB'S IN THE JOB'S OUTPUT  *
*              IOT(S) TO JOE(S).  AS EACH JOE IS ADDED TO THE JOT, IT *
*              IMMEDIATELY BECOMES AVAILABLE FOR SELECTION BY A       *
*              PRINT/PUNCH PROCESSOR.  IF ANY SPIN DATA SET IOT'S     *
*              ARE ON THE $UNSPUNQ, THEIR REQUIREMENTS ARE ADDED TO   *
*              THE JOT IF SPACE IS AVAILABLE.                         *
*                                                                     *
*    LINKAGE = VIA $WAIT AND $POST OBEYING JES2 CONVENTIONS.          *
*                                                                     *
* ENTRY POINT = HASPPPI1                                              *
*                                                                     *
*    PURPOSE = THIS IS THE ENTRY POINT TO THE JES2 PRINT/PUNCH        *
*              PROCESSOR.  THIS PROCESSOR AQUIRES OUTPUT WORK FROM    *
*              THE JOT USING ITS DCT AS A PARAMETER LIST.  THE DATA   *
*              SETS REPRESENTED ARE READ FORM THE SPOOL VOLUME,       *
*              DEBLOCKED, AND PRINTED/PUNCHED TO THE USER             *
*              SPECIFICATIONS.  WHEN ALL OUTPUT WORK FOR A JOB HAS    *
*              BEEN COMPLETED, THE JOB IS MOVED TO THE PURGE QUEUE.   *
*                                                                     *
*    LINKAGE = VIA $WAIT AND $POST OBEYING JES2 CONVENTIONS.          *
*                                                                     *
* INPUT = SEE ENTRY POINT DESCRIPTION                                 *
*                                                                     *
* OUTPUT = SEE ENTRY POINT DESCRIPTION                                *
*                                                                     *
* EXIT NORMAL = SEE ENTRY POINT DESCRIPTION                           *
*                                                                     *
* EXIT ERROR = USES JES2 MACRO $DISTERR TO INFORM THE OPERATOR        *
*                                                                     *
* EXTERNAL REFERENCES = SEE BELOW                                     *
*                                                                     *
*    ROUTINES = SEE MACROS FOR SERVICES USED                          *
*                                                                     *
*    DATA AREAS = SEE $HASPCB MACRO EXPANSION                         *
*                                                                     *
*    CONTROL BLOCKS - SEE $OUTWORK MACRO EXPANSION FOR PCE WORK       *
*                     AREA EXTENSION FOR ENTRY POINT HASPHOPE         *
*                                                                     *
*                     SEE $PPPWORK MACRO EXPANSION FOR PCE WORK       *
*                     AREA EXTENSION FOR ENTRY POINT HASPPPI1         *
*                                                                     *
* TABLES - NONE                                                       *
*                                                                     *
* MACROS = JES2   - $$POST, $#ADD, $#BLD, $#CKPT, $#GET, $#JCT,       *
*                   $#PUT, $#REM, $DISTERR, $DOM, $EXCP, $EXTP,       *
*                   $FREEBUF, $FREUNIT, $GETBUF, $GETSMFB, $GETUNIT,  *
*                   $IOERROR, $PGSRVC, $POST, $PURGE, $QGET, $QLOC,   *
*                   $QPUT, $QUESMFB, $TIME, $WAIT, $WTO               *
*                                                                     *
* MACROS = SYSTEM - BLDL, DELETE, ESTAE, IMGLIB, LOAD, POST, RETURN,  *
*                   SAVE, SETPRT, SETRP, TIME, WAIT                   *
*                                                                     *
* CHANGE ACTIVITY                                                     *
*                                                                     *
*     RELEASE 4.0 = OZ02418,OZ02421,OZ02422,OZ02442,OZ02443,OZ02444,  *
*                   OZ03343,OZ04338,OZ04340,OZ04967,OZ04969,OZ05777,  *
*                   OZ05780,OZ06730,OZ06740,OZ06748,OZ07433,OZ07447,  *
*                   OZ08187,OZ08230,OZ09020,OZ09042,OZ09064,OZ09079,  *
*                   OZ09082,OZ10324                                   *
*                                                                     *
*     RELEASE 4.1 = OZ09073,OZ10376,OZ10377,OZ11743,OZ11760,OZ11775,  *
*                   OZ11790,OZ12298,OZ12301,OZ12302,OZ13254,OZ14412,  *
*                   OZ14424,OZ14447,OZ14914,OZ15250,OZ15275,OZ16691   *
*                                                                     *
***********************************************************************
         TITLE 'HASP CONTROL BLOCK GENERATION MACRO'
         SPACE 5
*
*       $HASPCB                    GENERATE HASP CONTROL BLOCK
*
         SPACE 1
         MACRO
         $HASPCB &DOC=NO,&LIST=NO
         GBLC  &PRINT,&GEN,&DATA
         PUSH  PRINT
         PRINT &PRINT
         $CVT  LIST=&LIST          GENERATE OS CVT DSECT             R4
         $DCB  LIST=&LIST          GENERATE OS DCB DSECT
         $DEB  LIST=&LIST          GENERATE OS DEB DSECT
         $UCB  LIST=&LIST          GENERATE OS UCB DSECT
         $SDWA LIST=&LIST          GENERATE OS SDWA DSECT            R4
         $SETPRT LIST=&LIST        GENERATE OS SPPARM DSECT          R4
         $TED  DOC=&DOC            GENERATE HASP TED DSECT
         $TAB  DOC=&DOC            GENERATE HASP TAB DSECT           R4
         $PCIE DOC=&DOC            GENERATE HASP PCIE DSECT          R4
         $SVT  DOC=&DOC            GENERATE HASP SSVT DSECT
         $SJB  DOC=&DOC            GENERATE HASP SJB DSECT
         $HCT  DOC=&DOC            GENERATE HASP HCT DSECT
         $PCE  DOC=&DOC            GENERATE HASP PCE DSECT
         $LRC  DOC=&DOC            GENERATE HASP LRC DSECT
         $BUFFER DOC=&DOC          GENERATE HASP BUFFER DSECT
         $SMF  DOC=&DOC            GENERATE HASP SMF DSECT
         $JQE  DOC=&DOC            GENERATE HASP JQE DSECT
         $JOE  DOC=&DOC            GENERATE HASP JOE DSECT
         $JOT  DOC=&DOC            GENERATE HASP JOT DSECT
         $QSE  DOC=&DOC            GENERATE HASP QSE DSECT
         $JCT  DOC=&DOC            GENERATE HASP JCT DSECT
         $CAT  DOC=&DOC            GENERATE HASP CAT DSECT
         $PDDB DOC=&DOC            GENERATE HASP PDDB DSECT
         $IOT  DOC=&DOC            GENERATE HASP IOT DSECT
         $RAT  DOC=&DOC            GENERATE HASP RAT DSECT
         $DCT  DOC=&DOC            GENERATE HASP DCT DSECT
         $FMH  DOC=&DOC            GENERATE SNA FM HEADER DSECT      R4
         $OUTWORK DOC=&DOC         GENERATE HASP OUTWORK DSECT
         $PPPWORK DOC=&DOC         GENERATE HASP PPPWORK DSECT
         SPACE 1
         POP   PRINT
         PRINT &GEN,&DATA          SET ASSEMBLY PRINT OPTIONS
         MEND
         TITLE 'HASP OUTPUT PROCESSOR EXECUTIVE'
         SPACE 5
HASPPRPU START 0                   HASP PRINT/PUNCH SERVICE
         SPACE 5
         COPY  $HASPGEN            COPY HASPGEN PARAMETERS
         TITLE 'HASP CONTROL BLOCKS'
         SPACE 5
HASPPRPU $ENTRY BASE=,CSECT=YES    PROVIDE PROCESSOR IDENTIFICATION
         SPACE 5
*
*                             DOCUMENTATION OPTIONS FOR THIS ASSEMBLY
*
         SPACE 3
        $SYSPARM (OFF,GEN,NODATA,NO,NO)
         SPACE 5
*
*                             GENERATE HASP CONTROL BLOCKS
*
         SPACE 3
        $HASPCB DOC=&DOC,LIST=&LIST  GENERATE HASP CONTROL BLOCKS
         TITLE 'HASP OUTPUT PROCESSOR -- MAIN ENTRY'                 R4
***********************************************************************
*                                                                     *
*              HOPE REGISTER DEFINITIONS                              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         SPACE 3
JOE      EQU   6                   JOB OUTPUT ELEMENT BASE
RNP      EQU   7                   NON-PROCESS RETURN REGISTER
JOT      EQU   8                   JOB OUTPUT TABLE BASE
         SPACE 15                                                    R4
***********************************************************************
*                                                                     *
*        HASP OUTPUT PROCESSOR -- MAIN ENTRY POINT                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
HASPHOPE $ENTRY BASE=BASE2         HOPE MAIN ENTRY                   R4
         TITLE 'HASP OUTPUT PROCESSOR -- SPIN DATA SET PROCESSING'   R4
***********************************************************************
*                                                                     *
*        ADD ANY STACKED SPIN DATA SETS TO THE JOT IF POSSIBLE        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         L     JOT,$AQSE           LOAD QSE BASE ADDRESS
         USING QSEDSECT,JOT        ACTIVATE QSE ADDRESSABILITY
         MVC   PCEJQE,$ZEROS       CLEAR JQE ADDRESS           @OZ32566
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA
         OC    $UNSPUNQ,$UNSPUNQ   ANY QUEUED SPIN IOTS
         BZ    OPSPIN3             BRANCH IF NO
         L     R1,$JOTABLE         ADDRESS JOB OUTPUT TABLE
         USING JOTDSECT,R1         ACTIVATE JOT ADDRESSABILITY
         LH    R2,JOTFREC          NUMBER OF AVAILABLE JOES
         BCTR  R2,R0               JOE FOR A CHAR-JOE
         BCTR  R2,R0               JOE FOR A WORK-JOE
         CH    R2,$MINJOES         BELOW MINIMUM FREE LIMIT...       R4
         BL    OPSPIN3             BRANCH IF YES
         DROP  R1                  SUSPEND JOT ADDRESSABILITY
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        GET A BUFFER FOR THE SPIN IOT IF ONE NOT OWNED               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         ICM   R1,15,OPIOTBUF      IOT BUFFER ALREADY GOTTEN
         BNZ   OPSPIN1             BRANCH IF YES
        $GETBUF ,                  GET A BUFFER FOR THE IOT
         ST    R1,OPIOTBUF         SAVE BUFFER ADDRESS
         BNZ   OPSPIN1             BRANCH IF BUFFER AVAILABLE
        $WAIT  BUF                 WAIT FOR A FREE BUFFER
         B     HASPHOPE            GO TRY AGAIN
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        READ FIRST SPIN IOT FROM THE UNSPUN QUEUE                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPSPIN1  DS    0H
         L     R0,$UNSPUNQ         GET IOT TRACK ADDRESS
         MVI   PCEDEVTP,PCEDARD    SET DCT FOR READ
         BAL   RNP,OPIOCK          READ AND CHECK IOT
         BZ    OPSPIN2             BRANCH IF READ GOOD
OPSPNIOT $DISTERR                  INDICATE CONTROL BLOCK ERROR
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA
         XC    $UNSPUNQ,$UNSPUNQ   CLEAR SPIN IOT QUEUE
         B     OPSPIN3             EXIT FROM SPIN LOOP
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF UNSPUN QUEUE HAS CHANGED - START OVER                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPSPIN2  DS    0H
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA
         L     R1,OPIOTBUF         ADDRESS IOT BUFFER
         USING IOTDSECT,R1         ACTIVATE IOT ADDRESSABILITY
         CLC   $UNSPUNQ,PCESEEK    HAS SPIN QUEUE CHANGED            R4
         BNE   HASPHOPE            BRANCH IF YES
         CLC   PCESEEK,IOTTRACK    VALIDATE IOT                      R4
         BNE   OPSPNIOT            BRANCH IF IOT BAD                 R4
         L     R2,$IOTPDDB         POINT TO                          R4
         ALR   R2,R1                1ST PDDB IN IOT                  R4
         USING PDBDSECT,R2         ACTIVATE PDDB ADDRESSABILITY
         MVC   OPCLASS,PDBCLASS    SAVE SYSOUT CLASS OF PDDB
         L     R3,IOTJQOFF         HOLD JQE OFFSET
         LR    JCT,R3              COPY JQE OFFSET
         AL    JCT,$JOBQPTR        ADD JOB QUEUE BASE
         USING JQEDSECT,JCT        ACTIVATE JQE ADDRESSABILITY
         TM    JQEFLAGS,QUEPURGE   HAS $CJOB BEEN ISSUED
         BO    OPSPNCJ             BRANCH IF YES
         DROP  R1,R2               SUSPEND IOT,PDDB ADDRESSABILITY
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        BUILD A PROTOTYPE JOE PAIR FROM THE PDDB IN THE SPIN IOT     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
        $#BLD  JOES=OPWORK,PDDB=(R2),JQE=(R3) CONVERT PDDB TO JOES   R4
         LA    JOE,OPWORK          ADDRESS WORK-JOE
         USING JOEDSECT,JOE        ACTIVATE JOE ADDRESSABILITY
         L     R1,OPIOTBUF         ADDRESS SPIN IOT
         USING IOTDSECT,R1         ACTIVATE IOT ADDRESSABILITY
         MVC   JOEIOTTR,IOTTRACK   SET SPIN IOT TRACK ADDRESS
         MVI   JOEFLAG,$JOESPIN    SET SPIN FLAG IN WORK-JOE
         DROP  R1,JOE              SUSPEND IOT,JOE ADDRESSABILITY
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ATTEMPT TO ADD THE PROTOTYPE JOE PAIR TO THE JOT             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LA    R0,OPWORK           ADDRESS WORK-JOE PROTOTYPE
         LA    R1,OPCHAR           ADDRESS CHAR-JOE PROTOTYPE
        $#ADD  WORK=(R0),CHAR=(R1) ADD TO THE JOT                    R4
         BNZ   OPSPIN3             BRANCH IF JOT FULL
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        DE-QUEUE IOT FROM $UNSPUNQ -- RE-QUEUE JOB FOR $PURGE        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPSPNCJ  DS    0H
         L     R1,OPIOTBUF         ADDRESS IOT
         USING IOTDSECT,R1         ACTIVATE IOT ADDRESSABILITY
         MVC   $UNSPUNQ,IOTSPIOT   DEQUEUE SPIN IOT FROM CHAIN
         L     R2,$IOTPDDB         POINT TO                          R4
         ALR   R2,R1                1ST PDDB IN IOT                  R4
         USING PDBDSECT,R2         ACTIVATE PDDB ADDRESSABILITY
         NI    PDBFLAG1,255-PDB1HOLD TURN OFF HOLD FLAG
         OI    PDBFLAG1,PDB1PSO    TURN ON PSO FLAG
         DROP  R1,R2               SUSPEND IOT,PDDB ADDRESSABILITY
         MVI   PCEDEVTP,PCEDAWR    SET DCT FOR WRITE
         L     R0,PCESEEK          GET IOT MTTR
         BAL   RNP,OPIOCK          WRITE AND CHECK IOT
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA
         LH    R1,JQEJOECT         GET CURRENT JOE COUNT
         BCTR  R1,R0               DECREMENT BY ONE
         STH   R1,JQEJOECT         STORE BACK INTO JQE
          $QCKPT (JCT)             CHECKPOINT THE JQE          @OZ20010
         TM    JQEFLAGS,QUEPURGE   HAS $CJOB BEEN ISSUED
         BNO   HASPHOPE            BRANCH IF NO
         LH    R1,JQEJOECT         RESTORE JOE COUNT           @OZ34285
         AH    R1,JQEHLDCT         ADD HELD DATA SET COUNT
         BNZ   HASPHOPE            BRANCH IF OUTSTANDING WORK
         CLI   JQETYPE,$HARDCPY    IS JOB IN THE $HARDCPY QUEUE
         BNE   HASPHOPE            BRANCH IF NO
        $QPUT  (JCT),$PURGE        MOVE THE JOB TO $PURGE
         B     HASPHOPE            GO TO NEXT SPIN DATA SET
         DROP  JCT                 SUSPEND JQE ADDRESSABILITY
         TITLE 'HASP OUTPUT PROCESSOR -- MAIN PROCESSOR'             R4
***********************************************************************
*                                                                     *
*        *** MAIN PROCESSOR ***        LOCATE A JOB                   *
*                                                                     *
*        FREE THE SPIN IOT BUFFER AND ATTEMPT TO GET A JOB            *
*                                                                     *
***********************************************************************
OPSPIN3  DS    0H
         ICM   R1,15,OPIOTBUF      IS THERE AN IOT BUFFER TO FREE
         BZ    OPQLOC              BRANCH IF NO
        $FREEBUF (R1)              FREE IOT BUFFER
         XC    OPIOTBUF,OPIOTBUF   ZERO IOT BUFFER POINTER
OPQLOC   DS    0H
         OC    QSEOPCKP,QSEOPCKP   IS THIS A RESTART
         BZ    OPQGET              BRANCH IF NO
        $QLOC  QSEOPJNO            LOCATE JOB QUEUE ELEMENT
         BZ    OPQGET              BRANCH IF JOB NOT FOUND
         USING JQEDSECT,R1         ACTIVATE JQE ADDRESSABILITY
         OC    JQEFLAGS,$SIDBUSY   SET JOB QUEUE ELEMENT BUSY
         DROP  R1                  SUSPEND JQE ADDRESSABILITY
          $QCKPT (R1)              CHECKPOINT THE JQE          @OZ20010
         B     OPJOB               GO PROCESS PARTIAL JOB
OPQGET   DS    0H
         USING JCTDSECT,JCT        ACTIVATE JCT ADDRESSABILITY
        $QGET  $OUTPUT             ATTEMPT TO GET A JOB
         BNZ   OPSETUP             BRANCH IF NEW JOB FOUND
*
*        $QGET NON-PROCESS EXIT ROUTINE
*
        $WAIT  JOB,INHIBIT=NO      $WAIT FOR JOB TO BE QUEUED
         B     HASPHOPE            TRY AGAIN
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SETUP TO PROCESS NEW JOB                                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPSETUP  DS    0H
         SLR   R0,R0               CLEAR REGISTER
         STH   R0,QSEOPCKP         SET PARTIAL JOE COUNT ZERO
OPJOB    DS    0H
        $ACTIVE R=R14              INDICATE PROCESSOR ACTIVE
         ST    R1,PCEJQE           STORE JQE ADDRESS           @OZ32566
         USING JQEDSECT,R1         ACTIVATE JQE ADDRESSABILITY
         MVC   QSEOPJNO,JQEJOBNO   SET JOB NUMBER FOR WARM START
         DROP  R1                  SUSPEND JQE ADDRESSABILITY
         LR    R2,R1               HOLD A(JOB QUEUE ELEMENT)
        $TIME                      GET SIGN-ON TIME/DATE
         STM   R0,R1,OPTIMEON      SAVE IN PCE FOR JCT UPDATE
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        GET ADDRESS OF JCT IN BUFFER FROM JCT MANAGER                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         SLR   JOE,JOE             CLEAR JOE REG IN CASE OF JCT ERR  R4
         LR    JCT,R2              SET JCT = A(JOB QUEUE ELEMENT)
        $#JCT  READ,REFRESH=YES    GET ADDR OF UPDATED JCT BUFF     R41
         ST    JCT,OPJCTBUF        SAVE JCT BUFFER ADDRESS
         BZ    OPNOJCT             BRANCH IF READ WAS IN ERROR
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        JCT VALID - DETERMINE IF THIS JOB HAD THE NOTIFY OPTION      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
JCTOK    DS    0H
         OC    QSEOPCKP,QSEOPCKP   IS THIS A RESTART
         BNZ   OPNOTX              BRANCH IF YES - SKIP NOTIFY
         TM    JCTTSUID,255-C' '   WAS NOTIFY REQUESTED...           R4
         BZ    OPNOTX              BR IF NO TO SKIP NOTIFY           R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SCAN JOB QUEUE FOR TS USER TO NOTIFY                         *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA
         L     R1,=V($QINDEX)      LOCATE
         LA    R2,CATTSUCL-(255-QUECLASS) HEAD
         IC    R2,0(R1,R2)         OF THE TSU
         LA    R2,$JQHEADS-2-QUECHAIN(R2) EXECUTION QUEUE
         EJECT                                                       R4
         SPACE 1                                                     R4
OPJQELP  LH    R2,QUECHAIN(,R2)    GET NEXT JQE OFFSET               R4
         N     R2,=F'65535'        CLEAR LEFT HALFWORD
         BZ    OPENDQ              BRANCH IF END OF JQES
         SLL   R2,2                EXPAND TO FULL ADDRESS OFFSET
         AL    R2,$JOBQPTR         ADD JOB QUEUE BASE
         USING JQEDSECT,R2         ACTIVATE JQE ADDRESSABILITY
         TM    JQEFLAGS,QUEBUSY    IS THIS USER ACTIVE
         BZ    OPJQELP             BRANCH IF NO - SKIP
         CLC   JQEJNAME(7),JCTTSUID IS THIS THE RIGHT USER
         BNE   OPJQELP             BRANCH IF NO - SKIP
         MVC   PCEWA(1),JQEFLAGS   COPY JQE BUSY FLAGS
         NI    PCEWA,QUEBUSY       ISOLATE BUSY FLAGS
         CLC   PCEWA(1),$SIDBUSY   IS THE JOB BUSY ON THIS SYSTEM
         BE    OPTSWTO             BRANCH IF YES
         SPACE 1                                                     R4
         DROP  R2                  SUSPEND JQE ADDRESSABILITY
         SLR   R2,R2               CLEAR REGISTER 2
         IC    R2,PCEWA            COPY BUSY FLAGS
         BCTR  R2,0                DECREMENT BY 1
         LA    R3,1                SETUP FOR BUSY TO AFFINITY SHIFT
         SLL   R3,0(R2)            CREATE AFFINITY BIT
OPTSAFF  L     R1,PCEJQE           ADDR OF CURRENT JOB'S JQE   @OZ32566
         USING JQEDSECT,R1         ACTIVATE JQE ADDRESSABILITY
         NI    JQEFLAG2,255-QUESYSAF RESET CURRENT AFFINITY
         STC   R3,PCEWA            STORE NEW AFFINITY
         OC    JQEFLAG2,PCEWA      MOVE IN NEW AFFINITY
        $QPUT  (R1),$OUTPUT        REQUEUE JQE TO $OUTPUT
        $#JCT  FREE                RELEASE JCT BUFFER
        $DORMANT                   INDICATE PROCESSOR INACTIVE
         B     HASPHOPE            TRY TO SELECT OTHER WORK
         DROP  R1                  SUSPEND JQE ADDRESSABILITY
         SPACE 1                                                     R4
OPENDQ   DS    0H
         CLI   JCTTSUAF,0          SEE IF TSO SOURCE AFFINITY       R41
         BE    OPTSWTO               PRESENT, BRANCH IF NOT         R41
         CLC   JCTTSUAF,$SIDAFF    SOURCE AFFINITY FOR THIS SYSTEM
         BE    OPTSWTO             BRANCH IF YES
         IC    R3,JCTTSUAF         SELECT SOURCE AFFINITY
         B     OPTSAFF             ALTER AFFINITY AND $QPUT
         EJECT                                                       R4
*
*        TS USER BUSY ON THIS SYSTEM - USE $WTO TO NOTIFY
*
OPTSWTO  DS    0H
         MVC   M165LOC,MSG165      SET BASIC MESSAGE FORMAT          R4
         MVC   M165JNAM,JCTJNAME   INSERT JOBNAME                    R4
         MVC   M165JBID,JCTJOBID     AND JOB NUMBER                  R4
         LA    R1,M165JNAM-1       SET PTR TO JOBNAME - 1            R4
OPSCNJ   DS    0H
         LA    R1,1(,R1)           INCREMENT POINTER                 R4
         CLI   0(R1),C' '          IS THIS CHARACTER BLANK
         BNE   OPSCNJ              IF NOT TRY AGAIN
         TM    JCTJTFLG,JCTJTJF+JCTJTCF  ENDED BY CC SET
         BNO   OPENDMSG            IF NOT BRANCH TO END MESG
         CLI   JCTPSN1,C' '        IS STEPNAME BLANK
         BE    OPSCNJS2            IF YES SKIP STEPNAME              R4
         MVC   1(8,R1),JCTPSN1     INSERT STEPNAME
OPSCNJS1 DS    0H
         LA    R1,1(,R1)           INCREMENT POINTER                 R4
         CLI   0(R1),C' '          IS THIS CHARACTER BLANK
         BNE   OPSCNJS1            IF NOT TRY AGAIN
OPSCNJS2 DS    0H                                                    R4
         CLI   JCTPSN2,C' '        IS STEP NAME BLANK...             R4
         BE    OPSCNJS4            IF YES SKIP STEPNAME              R4
         MVC   1(8,R1),JCTPSN2     INSERT STEPNAME
OPSCNJS3 DS    0H                                                    R4
         LA    R1,1(,R1)           INCREMENT POINTER                 R4
         CLI   0(R1),C' '          IS THIS CHARACTER BLANK
         BNE   OPSCNJS3            IF NOT TRY AGAIN                  R4
OPSCNJS4 DS    0H                                                    R4
         MVC   1(9,R1),=C'ENDED CC='  SET REASON                     R4
         LA    R1,10(,R1)          SET PTR TO CONDITION CODE         R4
         LH    R2,JCTJTCC          GET CONDITION CODE
         CVD   R2,0(,R1)           CONVERT TO DECIMAL                R4
         UNPK  0(4,R1),0(8,R1)     UNPACK FOUR LOW DIGITS
         OI    3(R1),X'F0'         SET ZONE TO X'F'
         LA    R1,4(,R1)           SET POINTER TO NEXT FIELD         R4
         B     OPLOUSM2            SET LOGON USERID MESSAGE          R4
         EJECT                                                       R4
OPENDMSG DS    0H
         MVC   1(5,R1),=C'ENDED'   SET ENDED MESSAGE
         LA    R1,6(,R1)           SET PTR TO NEXT FIELD             R4
         CLI   JCTJTFLG,JCTJTJF    IS IT JCL ERROR
         BE    *+14                IF YES SET JCL MESSAGE
         OC    JCTCNVRC,JCTCNVRC   IS IT JCL ERROR
         BZ    *+14                IF NOT TRY OTHER
         MVC   0(11,R1),=C'- JCL ERROR'  SET REASON
         B     OPLOUSM             SET LOGON USERID MSG
         L     R2,PCEJQE           ADDRESS CURRENT JOB         @OZ32566
         USING JQEDSECT,R2        ESTABLISH JQE ADDRESSABILITY
         TM    JQEFLAGS,QUEOPCAN   WAS JOB CANCELLED
         BNO   *+14                BRANCH IF NO
         MVC   0(11,R1),=C'- CANCELLED' SET REASON
         B     OPLOUSM             SET LOGON USERID
         TM    JCTJTFLG,JCTJTABD   WAS JOB ABENDED
         BZ    OPLOUSM2            NO, SET LOGON USERID MESSAGE      R4
         MVC   0(11,R1),=C'- ABENDED  '   SET REASON
OPLOUSM  DS    0H                                                    R4
         LA    R1,11(,R1)          INCREMENT PTR TO NEXT FIELD       R4
OPLOUSM2 DS   0H                                                     R4
         MVC   0(14,R1),=C''',LOGON,USER=('  SET USER ID             R4
         LA    R1,14(,R1)          INCREMENT POINTER                 R4
         MVC   0(7,R1),JCTTSUID    SET USERID
OPSCNUID DS    0H
         LA    R1,1(,R1)           INCREMENT POINTER                 R4
         CLI   0(R1),C' '          WAS THIS CHARACTER A BLANK
         BNE   OPSCNUID            IF NOT TRY NEXT
         MVI   0(R1),C')'          TERMINATE USERID WITH ')'
        $WTO   M165LOC,L'MSG165,JOB=NO,TYPE=SVC34 ISSUE 'SEND' CMD   R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        INITIALIZE PRIMARY PDDB TO JOE CONVERSION LOOP               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPNOTX   DS    0H
         MVC   OPJOBCPY,JCTCPYCT   JOB LEVEL COPY COUNT
         CLI   OPJOBCPY,X'00'      IS THE JOB COPY COUNT ZERO
         BNE   *+8                 BRANCH IF NO
         MVI   OPJOBCPY,X'01'      FORCE COUNT TO ONE
         CLC   OPJOBCPY,$JCOPYLM   TOO MANY JOB COPIES REQUESTED     R4
         BNH   SKIP70              BR IF NO                          R4
         MVC   OPJOBCPY,$JCOPYLM   USE MAXIMUM                       R4
SKIP70   MVC   OPMSGCLS,JCTMCLAS   JOB MESSAGE CLASS
         MVC   OPJBKEY,JCTJBKEY    HOLD JOB KEY FOR VALIDITY CHECK
*
*        READ CHAIN OF IOT'S
*
         SLR   R3,R3               CLEAR PREVIOUS BUFFER POINTER
        $GETBUF WAIT=YES           GET BUFFER FOR IOT                R4
         ST    R1,OPIOTBUF         SAVE IOT BUFFER ADDRESS           R4
         L     R0,JCTIOT           GET ADDRESS OF 1ST REGULAR IOT
OPIOTRD  LR    JCT,R1              LOAD BASE FOR IOT                 R4
         USING IOTDSECT,JCT        ACTIVATE IOT ADDRESSABILITY
         MVI   PCEDEVTP,PCEDARD    INDICATE READ OPERATION
         BAL   RNP,OPIOCK          CALL I/O AND CHECK ROUTINE
         BZ    *+10                BRANCH IF GOOD READ
         XC    IOTJBKEY,IOTJBKEY   ZERO IOT JOB KEY
         CLC   IOTJBKEY,OPJBKEY    IS THIS IOT VALID
         BE    IOTOK               BRANCH IF YES
*
*        IOT JOB KEY DOES NOT MATCH JCT JOB KEY
*
OPIOTER  $DISTERR                  INDICATE CONTROL BLOCK ERROR
        $FREEBUF (JCT)             FREE BUFFER OF INVALID IOT
         LTR   JCT,R3              IS AT LEAST 1 IOT VALID
         BNZ   PDBSCAN             PROCESS WHAT IS LEFT
         ST    R3,OPIOTBUF         SET BUFFER ADDRESS TO ZERO
         B     OPQPUT              TERMINATE
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IOT JOB KEY MATCHES JCT JOB KEY                              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IOTOK    DS    0H
         ICM   R3,15,IOTIOTTR      LOAD AND TEST NEXT IOT TRACK      R4
         BZ    PDBSCAN             BRANCH IF NO MORE IOT'S
        $GETBUF WAIT=YES           GET BUFFER FOR IOT                R4
         ST    R1,IOTIOT           CHAIN TO PREVIOUS IOT
         LR    R0,R3               TTR FOR OPIOCK                    R4
         LR    R3,JCT              HOLD PREVIOUS IOT BASE
         B     OPIOTRD             CONTINUE READING IOT'S
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        ALL VALID IOTS HAVE BEEN READ - START PDDB SCAN              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PDBSCAN  DS    0H
         XC    IOTIOT,IOTIOT       ZERO CHAIN POINTER OF LAST IOT
         XC    OPCKPT,OPCKPT       ZERO CHECKPOINT JOE INDEX
         L     JCT,OPIOTBUF        ADDRESS IOT BUFFER
PDBIOT   DS    0H
         L     R2,$IOTPDDB         POINT TO                          R4
         ALR   R2,JCT               1ST PDDB IN IOT                  R4
         USING PDBDSECT,R2         ACTIVATE PDDB ADDRESSABILITY
         L     R3,IOTPDDBP         OFFSET AFTER LAST PDDB
         AR    R3,JCT              ADD CURRENT BUFFER ORIGIN
         ST    R3,OPDBEND          SAVE IN PCE
PDBNEXT  DS    0H
         XC    OPDDB,OPDDB         CLEAR RESTART POINTER
         C     R2,OPDBEND          END OF PDDBS ID THIS IOT
         BNE   PDBPDB              BRANCH IF NO
         ICM   JCT,15,IOTIOT       LOAD AND TEST IOT CHAIN
         BZ    OPQPUT              BRANCH IF NO MORE IOTS
         B     PDBIOT              SETUP NEW IOT ADDRESSES
PDBPDB   DS    0H
         TM    PDBFLAG1,PDB1NULL+PDB1NSOT IS PDDB NULL/NOT SYSOUT
         BZ    PDBJOE              BRANCH IF NO - BUILD JOES
         LA    R2,PDBLENG(,R2)     STEP TO THE NEXT PDDB
         B     PDBNEXT             CONTINUE SCAN
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        BUILD CHAR-JOE AND WORK-JOE FROM PDDB                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PDBJOE   MVC   OPCLASS,PDBCLASS    GET SYSOUT CLASS FROM PDDB        R4
         USING JOEDSECT,JOE        ACTIVATE JOE ADDRESSABILITY
         L     R3,PCEJQE           ADDRESS OF JQE              @OZ32566
         SL    R3,$JOBQPTR         MINUS JOB QUEUE ORIGIN
         LA    JOE,OPCHAR          ADDRESS CHAR-JOE
        $#BLD  JOES=OPWORK,PDDB=(R2),JQE=(R3) CONVERT PDDB TO JOES   R4
         SPACE 1                                                     R4
         OI    PDBFLAG1,PDB1NULL   SHOW 1ST PDDB PROCESSED          R41
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SCAN REMAINING PDDB'S FOR A CHARACTERISTICS MATCH            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DDBNEXT  DS    0H
         LA    R2,PDBLENG(,R2)     STEP TO THE NEXT PDDB
DDBIOT   DS    0H
         CL    R2,OPDBEND          END OF PDDB'S IN THIS IOT
         BNE   DDBPDB              BRANCH IF NO
         ICM   JCT,15,IOTIOT       LOAD AND TEST IOT CHAIN
         BZ    DDBEND              BRANCH IF NO MORE IOTS
         L     R2,$IOTPDDB         POINT TO                          R4
         ALR   R2,JCT               1ST PDDB IN IOT                  R4
         L     R3,IOTPDDBP         OFFSET AFTER LAST PDDB
         ALR   R3,JCT              ADD BUFFER ORIGIN
         ST    R3,OPDBEND          SAVE IN PCE
         B     DDBIOT              PROCESS PDDBS IN THIS IOT
DDBPDB   DS    0H
         TM    PDBFLAG1,PDB1NULL+PDB1NSOT IS PDDB NULL/NOT SYSOUT
         BNZ   DDBNEXT             BRANCH IF YES
*
*        CHECK SYSOUT CLASS
*
         CLC   PDBCLASS,OPCLASS    DOES SYSOUT CLASS MATCH
         BNE   DDBSTEP             BRANCH IF NO
         EJECT                                                      R41
*
*        CHECK REMAINING CHARACTERISTICS
*
         LA    JOE,OPWORK          ADDRESS WORK JOE                  R4
         TM    JOEFLAG2,$JOEDMND   TEST FOR DEMAND-SETUP OPTION      R4
         LA    JOE,OPCHAR          ADDRESS CHAR-JOE
         BZ    SKIP80              SKIP CLASS TEST IF NOT CHOSEN     R4
         CLC   PDBCLASS,OPMSGCLS   DOES SYSOUT CLASS = MSGCLASS
         BE    OPDEMAND            BRANCH IF YES - DEMAND SETUP
SKIP80   CLC   JOEFORM(12),PDBFORMS CHECK FORMS, FCB, UCS
         BNE   DDBSTEP             BRANCH IF NOT MATCHING
         CLC   JOEFLASH,PDBFLASH   CHECK FLASH FRAME ID              R4
         BNE   DDBSTEP             BRANCH IF NOT MATCHING            R4
         TM    PDBFLAG2,PDB2BRST   VERIFY                            R4
         BZ    SKIP90               THAT                             R4
         TM    JOECFLAG,$JOEBRST     PDDB                            R4
         BZ    DDBSTEP                BURSTER                        R4
         B     OPDEMAND                SPECIFICATION                 R4
SKIP90   TM    JOECFLAG,$JOEBRST        MATCHES                      R4
         BO    DDBSTEP                   CHAR-JOE                    R4
         SPACE 1                                                    R41
OPDEMAND CLC   JOEWTRID,PDBWTRID   CHECK EXTERNAL WRITER NAME       R41
         BNE   DDBSTEP             BRANCH IF NOT MATCHING
         LA    JOE,OPWORK          ADDRESS WORK-JOE
         CLC   JOEDEST,PDBDEST     CHECK DESTINATION
         BNE   DDBSTEP             BRANCH IF NOT MATCHING
         SPACE 1                                                    R41
OPMATCH  SLR   R0,R0               CLEAR WORK REGISTER              R41
         SLR   R1,R1               CLEAR COPY-GROUPS SUM            R41
         LA    R5,8                SET MAX COPY-GROUPS              R41
         SPACE 1                                                    R41
OPCOPLUP IC    R0,PDBCOPYG-1(R5)   COMPUTE TOTAL                    R41
         ALR   R1,R0                COPY COUNT OF                   R41
         BCT   R5,OPCOPLUP           3800 COPY-GROUPS               R41
         CLM   R1,1,PDBCOPYS       USE NORMAL                       R41
         BNL   OPRECCT              COPY COUNT                      R41
         IC    R1,PDBCOPYS           IF LARGER                      R41
         SPACE 1                                                    R41
OPRECCT  L     R0,PDBRECCT         ADD LINE/CARD                    R41
         MR    R0,R0                COUNT OF PDDB                   R41
         AL    R1,JOERECCT           TO WORK-JOE                    R41
         ST    R1,JOERECCT            TOTAL                         R41
         OI    PDBFLAG1,PDB1NULL   SET PDDB PROCESSED BIT
         B     DDBNEXT             GO PROCESS NEXT PDDB
         SPACE 1                                                    R41
***********************************************************************
*                                                                     *
*        CHARACTERISTICS DO NOT MATCH -- CHECK NEXT PDDB              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DDBSTEP  OC    OPDDB,OPDDB         RESTART POINT SELECTED...        R41
         BNZ   DDBNEXT             BRANCH IF YES
         ST    R2,OPDDB            SET PDDB AS RESTART POINT
         ST    JCT,OPIOT           AND SAVE IOT BUFFER ADDRESS
         B     DDBNEXT             CONTINUE SCAN OF PDDB'S
         EJECT                                                      R41
***********************************************************************
*                                                                     *
*        ALL MATCHING PDDB'S HAVE BEEN PROCESSED                      *
*                                                                     *
*        ADD A WORK-JOE FOR EACH JOB COPY                             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DDBEND   SLR   RNP,RNP             GET JOB LEVEL                    R41
         IC    RNP,OPJOBCPY         COPY COUNT                      R41
         SPACE 1                                                    R41
OPJCOPY  LH    R5,OPCKPT           GET INDEX OF LAST JOE ADDED      R41
         LA    R5,1(,R5)           INCR FOR THIS JOE                R41
         CH    R5,QSEOPCKP         WAS THIS JOE ALREADY ADDED...    R41
         BNH   OPSKIP              BR IF YES                        R41
         SPACE 1                                                    R41
         USING JQEDSECT,R1         PROVIDE JQE ADDRESSABILITY       R41
         SPACE 1                                                    R41
OPJOTADD L     R1,PCEJQE           ADDRESS OF JQE              @OZ32566
         TM    JQEFLAGS,QUEPURGE   HAS $CJOB BEEN ISSUED...         R41
         BO    OPQPUT              BR IF YES                        R41
         SPACE 1                                                    R41
         DROP  R1                  DROP JQE ADDRESSABILITY          R41
         SPACE 1                                                    R41
         LA    R0,OPWORK           PROTOTYPE WORK-JOE               R41
         LA    R1,OPCHAR           PROTOTYPE CHAR-JOE               R41
        $#ADD  WORK=(R0),CHAR=(R1) ADD WORK TO JOT                  R41
         BZ    OPADOK              BR IF ADD SUCCESSFUL             R41
        $DORMANT                   ALLOW $PJES2 WHILE $WAITING      R41
        $WAIT  JOT                 WAIT FOR JOT SERVICE             R41
        $ACTIVE                    SHOW HASPHOPE ACTIVE AGAIN       R41
         B     OPJOTADD            TRY AGAIN                        R41
         SPACE 1                                                    R41
OPADOK   STH   R5,QSEOPCKP         SAVE COUNT FOR WARM START        R41
         SPACE 1                                                    R41
OPSKIP   STH   R5,OPCKPT           SAVE COUNT IN PCE                R41
         BCT   RNP,OPJCOPY         ADD A JOE FOR ALL JOB COPIES     R41
         EJECT                                                      R41
***********************************************************************
*                                                                     *
*        SELECT PDDB WITH NEW CHARACTERISTICS                         *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
         ICM   R2,15,OPDDB         IS RESTART ADDRESS 0
         BZ    OPQPUT              BRANCH IF YES
         L     JCT,OPIOT           RESTART IOT BASE
         L     R3,IOTPDDBP         OFFSET AFTER LAST PDDB
         ALR   R3,JCT              ADD BUFFER ORIGIN
         ST    R3,OPDBEND          SAVE IN PCE
         B     PDBNEXT             CONTINUE PDDB SCAN
         SPACE 3                                                    R41
***********************************************************************
*                                                                     *
*        ALL PDDBS HAVE BEEN PROCESSED - MAKE JOB AVAILABLE           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPQPUT   DS    0H
         L     JCT,OPJCTBUF        ADDRESS JCT IN BUFFER
         USING JCTDSECT,JCT        ACTIVATE JCT ADDRESSABILITY
         MVC   JCTOUTON(8),OPTIMEON HOPE SIGN-ON TIME/DATE
        $TIME                      SIGN-OFF TIME/DATE
         STM   R0,R1,JCTOUTOF      HOPE SIGN-OFF TIME/DATE
         MVC   JCTOTSID,$SID       SET SID FOR TYPE 26 SMF RECORD
        $#JCT  WRITE               COPY JCT TO SPOOL
        $#JCT  FREE                RELEASE JCT BUFFER
         EJECT                                                      R41
***********************************************************************
*                                                                     *
*        IF NO JOES OR HQRS MOVE JOB TO $PURGE, ELSE $HARDCPY         *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPNOJCT  DS    0H
         L     R1,PCEJQE           ADDRESS OF JQE              @OZ32566
         LA    R0,$PURGE           SET NEXT QUEUE AS $PURGE
         USING JQEDSECT,R1         ACTIVATE JQE ADDRESSABILITY
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA
         LH    R3,JQEJOECT         IS SUM OF JOE COUNT
         AH    R3,JQEHLDCT          AND HOLD COUNT ZERO
         DROP  R1                  SUSPEND JQE ADDRESSABILITY
         BZ    *+8                 BRANCH IF YES
         LA    R0,$HARDCPY         SET NEXT QUEUE AS $HARDCPY
        $QPUT  (R1),(R0)           MOVE JOB TO NEXT QUEUE
         MVC   PCEJQE,$ZEROS       CLEAR JQE ADDRESS           @OZ32566
         EJECT                                                       R4
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        FREE CHAIN OF IOT BUFFERS USED DURING JOE CREATION           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPTERM   DS    0H
         USING IOTDSECT,JCT        ACTIVATE IOT ADDRESSABILITY
         ICM   JCT,15,OPIOTBUF     LOAD AND TEST BUFFER POINTER
         BZ    OPPURGE             BRANCH IF ZERO
         MVC   OPIOTBUF,IOTIOT     COPY NEXT BUFFER POINTER
        $FREEBUF (JCT)             FREE IOT BUFFER CHAIN
         B     OPTERM              CYCLE THROUGH ALL IOT BUFFERS
OPPURGE  DS    0H
         SLR   R0,R0               CLEAR REGISTER
         STH   R0,QSEOPJNO         CLEAR JOB NUMBER
         STH   R0,QSEOPCKP         CLEAR PARTIAL JOE COUNT
        $DORMANT                   INDICATE PROCESSOR DORMANT
         B     HASPHOPE            TRY TO GET ANOTHER JOB
         DROP  JOE                 SUSPEND JOE ADDRESSABILITY
         TITLE 'HASP OUTPUT PROCESSOR -- BUFFER READ/WRITE AND CHECK SUC
               BROUTINE'                                             R4
***********************************************************************
*                                                                     *
*        BUFFER READ/WRITE AND CHECK SUBROUTINE                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
OPIOCK   DS    0H
         USING BUFDSECT,R1         ACTIVATE BUFFER ADDRESSABILITY
         LA    R15,IOBCCW1         RESET IOBSTART
         ST    R15,IOBSTART         TO FIRST CCW IN CHAIN
         DROP  R1                  SUSPEND BUFFER ADDRESSABILITY
         ST    R1,PCEBUFAD         STORE BUFFER ADDRESS IN DCT
         ST    R0,PCESEEK          STORE TRACK ADDRESS IN DCT
         LA    R1,PCEDADCT         SETUP DCT ADDRESS FOR $EXCP
        $EXCP  (R1),WAIT=YES       INITIATE I/O AND WAIT             R4
         BMR   RNP                 RETURN IF ERROR WITH NON-ZERO CC  R4
         SR    R15,R15             SET GOOD RETURN CODE
         BR    RNP                 RETURN TO CALLER
         DROP  BASE2               SUSPEND LOCAL ADDRESSABILITY
         TITLE 'HASP OUTPUT PROCESSOR -- LITERAL POOL AND CONSTANTS' R4
         LTORG
         SPACE 2                                                     R4
        $MID   165                 GENERATE MESSAGE ID
MSG165   DC    CL86'IDSE ''&MID.JOBNNNNN  JOBNAMES'
         ORG   MSG165
         DC    X'165F'             MESSAGE ID
         ORG   ,
         SPACE 1                                                     R4
M165LOC  EQU   JCTWORK,L'MSG165    LOCATION FOR MESSAGE              R4
M165JNAM EQU   M165LOC+25,8        LOCATION FOR JOB NAME             R4
M165JBID EQU   M165LOC+15,8        LOCATION FOR JOBID                R4
         TITLE 'HASP PRINT/PUNCH SERVICE PROCESSOR'
***********************************************************************
*                                                                     *
*              PRINT/PUNCH REGISTER DEFINITIONS                       *
*                                                                     *
***********************************************************************
         SPACE 3
PW       EQU   WA                  WORK REGISTER
PBUF     EQU   WB                  BUFFER POINTER
PC1      EQU   WC                  CCW REGISTER 1                    R4
PC2      EQU   WD                  CCW REGISTER 2
BASE4    EQU   WE                  PRPU THIRD LOCAL BASE REGISTER    R4
PL       EQU   WF                  INTERNAL LINKAGE REGISTER
         SPACE 5
***********************************************************************
*                                                                     *
*              PPFLAG SWITCH DEFINITIONS                              *
*                                                                     *
***********************************************************************
         SPACE 3
PPWSW    EQU   X'80'               PRINT/PUNCH WRITE SWITCH
PPDELSW  EQU   X'40'               PRINT/PUNCH SUSPEND SWITCH
PPDALOC  EQU   X'20'               PRINT/PUNCH ALLOCATION IOT
PRDELSW  EQU   X'10'               PRINT/PUNCH TERMINATION SWITCH
PPFUNCI  EQU   X'08'               PUNCH INTERPRET REQUESTED
PPRDERR  EQU   X'04'               PRINT/PUNCH DATA READ ERROR
PPJCTIOT EQU   X'02'               PRINT/PUNCH JCT/IOT READ ERROR
PPNEWS   EQU   X'01'               PRINT JES2-NEWS PROCESS SWITCH   R41
         SPACE 2                                                     R4
***********************************************************************
*                                                                     *
*              PPFLAG2 SWITCH DEFINITIONS                             *
*                                                                     *
***********************************************************************
         SPACE 2                                                     R4
PPTCEL   EQU   X'80'               TRACK-CELL DE-SPOOLING SWITCH     R4
PPRSW    EQU   X'40'               PRINT/PUNCH READ SWITCH           R4
PPCKPT   EQU   X'20'               PRINT/PUNCH CKPT-NEEDED SWITCH    R4
PPCKPTA  EQU   X'10'               PRINT/PUNCH CKPT-ALLOWED SWITCH   R4
PPCIWAIT EQU   X'08'               PRINT/PUNCH PCI WAIT SWITCH       R4
PPOPTJ   EQU   X'04'               PRINTER OPTCD=J SWITCH            R4
PPFDS    EQU   X'02'               FIRST SYSOUT DATA SET SWITCH      R4
PSMFDSER EQU   X'01'               DATA BUFFER ERROR FLAG FOR SMF    R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- PROCESSOR INITIALIZATION'
***********************************************************************
*                                                                     *
*        HASP PRINT/PUNCH PROCESSOR -- MAIN ENTRY POINT               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
HASPPPI1 $ENTRY BASE=(BASE2,BASE3,BASE4)  PRINT/PUNCH MAIN ENTRY     R4
         SPACE 3                                                     R4
         USING BUFDSECT,PBUF       ACTIVATE BUFFER ADDRESSABILITY
         USING JCTDSECT,JCT        ACTIVATE JCT ADDRESSABILITY
         USING DCTDSECT,R1         ACTIVATE DCT ADDRESSABILITY
         SPACE 3                                                     R4
         LA    BASE3,2048(,BASE2)  SETUP SECOND LOCAL
         LA    BASE3,2048(,BASE3)   BASE REGISTER
         LA    BASE4,2048(,BASE3)  SETUP THIRD LOCAL                 R4
         LA    BASE4,2048(,BASE4)   BASE REGISTER                    R4
         SPACE 1                                                     R4
         XC    PBUFADDR,PBUFADDR   CLEAR PRIMARY BUFFER ADDRESS     R41
         XC    PDDBSKIP,PDDBSKIP   CLEAR REPOSITIONING COUNTERS FOR  R4
         XC    PBUFSKIP,PBUFSKIP    CARDS/LINES AND BUFFER           R4
         XC    PCKPT,PCKPT         CLEAR CHECKPOINT DATA AREA        R4
         XC    PPCKPTR,PPCKPTR     CLEAR CHECKPOINT DATA POINTER     R4
         SPACE 1                                                     R4
         MVC   PCEJQE,$ZEROS       CLEAR JQE ADDRESS           @OZ32566
         L     R1,$HASPECB         ADDRESS GETMAINED AREA      @OZ32235
         LA    R1,$SVBLANK(,R1)    POINT TO BLANK DATA         @OZ32235
         STCM  R1,7,PUCCWBL+1      INITIALIZE BLANK CARD CCW   @OZ32235
         STCM  R1,7,PUSPACCW+1     INITIALIZE 3525 SPACER CCW  @OZ32235
         MVI   PSMFDCI,0           CLEAR SMF FLAGS             @OZ32566
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        TRY TO GET AN IDLE PRINTER OR PUNCH                          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PGETUNIT $GETUNIT PCEDCT           ATTEMPT ACQUIRE PRT/PUN DCT @OZ32566
         BNZ   PGOTUNIT            BR IF DEVICE AVAILABLE            R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        DEVICE NOT AVAILABLE -- ISSUE $WAIT                          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PCEID,PCERJEID      TEST FOR REMOTE TERMINAL          R4
         BO    PRJEDEV             BR IF SO                          R4
        $WAIT  UNIT,INHIBIT=NO      ELSE $WAIT FOR U/R DEVICE        R4
         B     PGETUNIT            THEN BR TO TRY AGAIN              R4
         SPACE 1                                                     R4
PRJEDEV $WAIT  WORK                $WAIT FOR RJE DEVICE              R4
         B     PGETUNIT            THEN BR TO TRY AGAIN              R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        GOT A PRINTER OR PUNCH -- SHOW PROCESSOR ACTIVE              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PGOTUNIT MVI   PPFLAG,0            CLEAR PRINT/PUNCH FLAGS          R41
         MVI   PPFLAG2,0           CLEAR PRINT/PUNCH FLAGS           R4
         CLI   DCTDEVTP,DCTPUN     IF DEVICE                         R4
         BNE   SKIP100              IS LOCAL PUNCH,                  R4
         XC    PULMTTR(7),PULMTTR  CLEAR PUNCH RESTART POINTER       R4
         SPACE 1                                                     R4
SKIP100  MVI   DCTFLAGS,0          RESET OPERATOR COMMANDS
        $ACTIVE R=PW               INDICATE PROCESSOR ACTIVE
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        FOR REMOTE PRINTERS -- PRINT ANY SPOOLED MESSAGES            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PCEID,PCELCLID+PCEPUSID  TEST PROCESSOR TYPE
         BNZ   PTSTINIT            BR IF NOT REMOTE PRINTER    @OZ30048
         TM    DCTPPSW,DCTPPSWS    SUPPRESS SEP PAGE
         BO    PGETJOB             BRANCH IF YES
         CLI   $SPOLMSG,0          IF NO MSG SPOOL BUFFERS,          R4
         BE    PGETJOB              BR TO GET A JOB                  R4
         CLI   DCTDEVTP,DCTRCON    IF NOT A CONSOLE DCT        @OZ26040
         BNE   PSNACHK               GO CHECK FOR SNA REMOTE   @OZ26040
PGETRAT  DS    0H                                              @OZ26040
         L     R1,DCTDCB           POINT                       @OZ26040
         L     R1,MDCTRAT            TO RAT                    @OZ26040
         B     PMSGS               GO CHECK FOR MESSAGES       @OZ26040
PSNACHK  DS    0H                                              @OZ26040
         TM    MDCTTYPE,DCTPLU1    IF NOT AN SNA REMOTE             R41
         BNO   PGETRAT             GET RAT ADDRESSABLILITY     @OZ26040
         L     R1,DCTDCB           POINT                            R41
         L     R1,MDCTRAT           TO RAT                          R41
         USING RATDSECT,R1         EST. RAT ADDRESSABILITY     @OZ26040
         TM    RATCONF,RATCONFC    IF RMT HAS A CONSOLE,       @OZ33415
         BO    PGETJOB               GO GET A JOB              @OZ26040
PMSGS    DS    0H                                                   R41
         $QSUSE                    REQUEST ACCESS TO SHARED Q  @OZ26040
         SLR   R14,R14               GET NUMBER OF REMOTE      @OZ26040
         IC    R14,RATCONRT+1          WITH CONSOLE            @OZ26040
         BCTR  R14,0                     LESS 1 FOR OFFSET     @OZ26040
         LR    R15,R14             SAVE REMOTE NUMBER (-1)     @OZ26040
         MH    R15,=Y(RATTLE)      CALCULATE RAT DISPLACEMENT  @OZ26040
         A     R15,$RATABLE        GET ADDRESS OF RAT ENTRY    @OZ26040
         CLR   R15,R1              IF CONSOLE AND PRINTER ARE  @OZ26040
         BE    PMSGDSP               SAME RMT, DE-SPOOL MSGS   @OZ26040
         L     R15,RATLDCT-RATDSECT(,R15) IF CONSOLE'S REMOTE  @OZ26040
         LTR   R15,R15             SIGNED ON THIS SYSTEM,      @OZ26040
         BNZ   PGETJOB               DON'T PRINT MESSAGES      @OZ26040
         USING DCTDSECT,R1         RESTORE DCT ADDRESSABILITY  @OZ26040
         LR    R15,R14             THREE TIMES                 @OZ26040
         ALR   R15,R14               REMOTE NUMBER             @OZ26040
         ALR   R15,R14                 MINUS ONE               @OZ26040
         SLR   R14,R14             COMPUTE BIT AND             @OZ26040
         D     R14,=F'8'             BYTE OFFSET               @OZ26040
         AL    R15,$RMTSON             TO REMOTE SIGNON BITS   @OZ26040
         ICM   R15,12,0(R15)       PICK UP BITS                @OZ26040
         SLL   R15,0(R14)           ISOLATE BITS FOR           @OZ26040
         SRL   R15,29                  THIS REMOTE             @OZ26040
         LTR   R15,R15             IF RMT SIGNED ON ANY SYSTM  @OZ26040
         BNZ   PGETJOB              THEN DON'T PRINT MESSAGES  @OZ26040
PMSGDSP  DS    0H                                              @OZ26040
         SLR   PBUF,PBUF           SET NO BUFFER INDICATION    @OZ26040
         SLR   JCT,JCT             SET NO JOB INDICATION       @OZ26040
         MVC   PPKEY,=C'MSPOOL'    SET SPECIAL JOB/DATA KEY    @OZ26040
         SLR   PW,PW               INITIALIZE                  @OZ26040
         BCTR  PW,0                 LINE COUNT
         ST    PW,PRLINECT           LIMIT
PRSMTEST DS    0H
*              THIS LINE DELETED BY APAR NUMBER                @OZ26040
         L     R1,PCEDCT           ADDRESS PRINTER DCT         @OZ32566
         SLR   R15,R15             MULTIPLY                          R4
         IC    R15,DCTROUTE         REMOTE                           R4
         LA    PW,0(R15,R15)         NUMBER                          R4
         ALR   PW,R15                 BY THREE                       R4
         AL    PW,$MSPOOLQ         ADD MESSAGE QUEUE BASE ADDRESS    R4
         CLC   0(1,PW),1(PW)       PRINTED VERSUS WRITTEN QUEUES
         BE    PRMDONE             BRANCH IF NO SPOOL MESSAGES
        $GETBUF WAIT=YES           GET A DATA BUFFER                 R4
         ST    R1,PBUFSAVE         SAVE BUFFER ADDRESS
         ST    R1,PBUFADDR         SAVE BUFFER ADDRESS               R4
         LR    PBUF,R1             ESTABLISH BUFFER ADDRESSABILITY
         ST    PBUF,PINIOB         INITIALIZE INPUT IOB ADDRESS      R4
         MVI   PBUFOPT,1           FORCE SINGLE BUFFERING            R4
         EJECT                                                       R4
         L     R1,PCEDCT           RELOAD PRINTER DCT ADDRESS  @OZ32566
         TM    MDCTTYPE,DCTPSNA    TEST FOR SNA REMOTES              R4
         BNO   *+14                NO, SKIP---                       R4
         MVC   PRMTSSEL,MDCTSEL    SAVE RMT SELECT BYTE IN WORK AREA R4
         MVI   MDCTSEL,FMHCNSLE+DCTPOUTB USE OUTBOUND CONSOLE SELECT R4
        $EXTP  OPEN,(R1)           OPEN REMOTE FOR SPOOL MESSAGES    R4
         BZ    PRMFINI             OPEN FAILED, BR -- TERMINATE      R4
         SPACE 2                                                     R4
PRFNDMSG DS    0H                                                    R4
        $QSUSE                     REQUEST ACCESS TO SHARED QUEUES   R4
         L     R1,PCEDCT           LOAD DCT ADDR IN CASE DESTR @OZ32566
         SLR   R15,R15             MULTIPLY                          R4
         IC    R15,DCTROUTE         REMOTE                           R4
         LA    PW,0(R15,R15)         NUMBER                          R4
         ALR   PW,R15                 BY THREE                       R4
         AL    PW,$MSPOOLQ         ADD MESSAGE QUEUE BASE            R4
         CLC   0(1,PW),1(PW)       CHECK PRINTED VS. WRITTEN QUEUES  R4
         BE    PRMDONE             BRANCH IF NO MESSAGES TO PRINT    R4
         MVC   PJCTBUF(2),0(PW)    SAVE POINTERS TO PRINTED/WRITTEN  R4
         MVC   0(1,PW),1(PW)       SHOW ALL PRINTED                  R4
        $POST  $HASPECF,CKPW       POST CHECKPOINT PROCESSOR         R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        INITIALIZE PROCESSOR TO PRINT A SINGLE SPOOL MESSAGE BUFFER  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRMTMSG  DS    0H                  *
         MVI   PBFAVAIL,1          SHOW ONLY 1 DATA BUFFER AVAILABLE R4
         SLR   PW,PW               CLEAR WORK REGISTER
         IC    PW,PJCTBUF          GET PRINTED RECORD NUMBER
         LA    PW,1(,PW)           ADD ONE FOR NEXT VALID BUFFER
         STC   PW,PJCTBUF          STORE FOR LATER
         SLR   R15,R15             IS RECORD                         R4
         IC    R15,$SPOLMSG         NUMBER                           R4
         LA    PW,1(,PW)             GREATER THAN                    R4
         SR    PW,R15                 &SPOLMSG - 1                   R4
         BNP   *+8                 BRANCH IF NO
         MVI   PJCTBUF,0           SET RECORD NUMBER TO ZERO
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        COMPUTE MTTR FROM RECORD NUMBER                              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         L     R1,PCEDCT           ADDRESS PRINTER DCT         @OZ32566
         SLR   PW,PW               MULTIPLY                          R4
         IC    PW,DCTROUTE           &SPOLMSG                        R4
         SLR   R14,R14               BY REMOTE                       R4
         MR    R14,PW                NUMBER                          R4
         IC    R14,PJCTBUF         GET RELATIVE RECORD NUMBER
         ALR   R15,R14             ADD TO REMOTE SPACE BASE
         L     PW,$TEDADDR         ADDRESS FIRST TRACK EXTENT DATA
         LH    PW,TNRT-TEDDSECT(,PW) GET NUMBER OF TRACKS/RECORD
         SLR   R14,R14             CLEAR WORK REGISTER
         DR    R14,PW              DIVIDE RECORD # BY RECS/TRACK
         SLL   R15,8               MOVE TO TT IN MTTR
         LA    R14,1(,R14)         STEP PAST RECORD ZERO
         ALR   R15,R14             ADD REMAINING RECORD NUMBER
         L     R14,$DACKPT         GET BASE MTTR                     R4
         LH    R14,2(,R14)          FOR SPOOL MESSAGES               R4
         SLL   R14,8               MOVE TO TT IN MTTR
         ALR   R15,R14             ADD TO CURRENT RECORD OFFSET
         BAL   PL,PRDBUF           INITIATE READ OF NEXT DATA BLOCK
         BAL   PL,PRDCHK           CHECK READ
         CLI   BUFECBCC,X'7F'      TEST COMPLETION CODE
         BNE   PRSMBEOB            BRANCH IF IN ERROR
         XC    HDBNXTRK,HDBNXTRK   CLEAR CHAIN TRACK
         B     PNXTBLK             PRINT DATA BLOCK                  R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        END OF, OR ERROR IN SPOOLED MESSAGE BUFFER                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRSMBEOB DS    0H
         MVI   PPFLAG,0            RESET ANY ERROR INDICATIONS
         CLC   PJCTBUF(1),PJCTBUF+1 END OF CURRENT SERIES
         BNE   PRMTMSG             BRANCH IF NO
         B     PRFNDMSG            CHECK FOR ADDITIONAL MESSAGES     R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        END OF ALL SPOOLED MESSAGE BUFFERS FOR THIS REMOTE           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRMDONE  DS    0H
         LTR   PBUF,PBUF           WERE ANY SPOOLED MESSAGES PRINTED
         BZ    PGETJOB             BRANCH IF NO
         LM    PC1,PC2,PRCCWSP     LOAD 3 SPACE CCW
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         L     R1,PCEDCT           GET ADDRESS OF REMOTE DCT   @OZ32566
        $EXTP  CLOSE,(R1)          CLOSE REMOTE DCT                  R4
PRMFINI  TM    MDCTTYPE,DCTPSNA    TEST FOR SNA REMOTES              R4
         BNO   *+10                NO, SKIP NEXT                     R4
         MVC   MDCTSEL,PRMTSSEL    RESTORE ORIG. SNA DEVICE SELECT   R4
         $FREEBUF (PBUF)           FREE HASP BUFFER                  R4
         TM    $PRTOPTS,$RPRBOPT   TEST REMOTE PRT BUFFERING OPTION  R4
         BZ    PNOJOB              BR IF SINGLE BUFFERING            R4
         MVI   PBUFOPT,2            ELSE FORCE DOUBLE BUFFERING      R4
         B     PNOJOB              THEN BR TO FREE THE UNIT          R4
         SPACE 1                                                     R4
PTSTINIT DS    0H                                              @OZ30048
         TM    PCEID,PCEPUSID      IS DEVICE A PUNCH...        @OZ30048
         BO    PGETJOB             BR IF YES, GET A JOB        @OZ30048
         CLI   PDEVTYPE+3,UCB3800  NON-IMPACT PRINTER...       @OZ30048
         BE    PGETJOB             BR IF YES, GET A JOB        @OZ30048
         TM    DCTPPSW2,DCTINIT    FIRST USE OF UNIT...        @OZ30048
         BO    PGETJOB             BR IF NO, GET A JOB         @OZ30048
         OI    DCTPPSW2,DCTINIT    TURN ON FIRST-USE FLAG      @OZ30048
         L     R15,=A(PINITSU)     CALL IMAGE                  @OZ30048
         BALR  PL,R15                READ ROUTINE              @OZ30048
         B     PGETJOB             FINSHED, GET A JOB          @OZ30048
         DROP  R1                  SUSPEND DCT ADDRESSABILITY
         EJECT
***********************************************************************
*                                                                     *
*        ATTEMPT TO GET WORK FROM THE JOB OUTPUT TABLE (JOT)          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PGETJOB  DS    0H
         L     R1,PCEDCT           GET PRINTER/PUNCH           @OZ32566
         LA    R1,0(,R1)             DCT ADDRESS               @OZ26040
         CLI   DCTDEVTP-DCTDSECT(R1),DCTRCON IF A CONSOLE DCT       R41
         BE    PNOJOB                         DONT LOOK FOR JOB     R41
        $#GET  DCT=(R1)            IS THERE WORK
         BNZ   PGOTJOB             BRANCH IF WORK FOUND
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        NO WORK IS CURRENTLY AVAILABLE - FREE RESOURCES              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNOJOB   L     PW,PCEDCT           GET ADDR OF PRINT/PUNCH DCT @OZ32566
         SPACE 1                                                     R4
         USING DCTDSECT,PW         ESTABLISH DCT ADDRESSABILITY
         SPACE 1                                                     R4
         OI    DCTSTAT,DCTHOLD     SET DEVICE UNAVAILABLE            R4
        $FREUNIT (PW)              FREE DEVICE
        $DORMANT                   INDICATE PROCESSOR INACTIVE
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE
         BO    HASPPPI1            BRANCH IF REMOTE
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        ISSUE DEVICE INACTIVE MESSAGE TO THE OPERATOR                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    DCTPPSW,DCTPPSWI    INACTIVE MESSAGE ISSUED
         BO    PJOTWAIT            BRANCH IF YES
         OI    DCTPPSW,DCTPPSWI    SET INACTIVE MESSAGE ISSUED
        $MID   160                                                   R4
         MVC   PMESSAGE(2),=X'160F' INACTIVE MESSAGE ID
         MVC   PMESSAGE+2(8),DCTDEVN GET DEVICE NAME
         MVC   PMESSAGE+10(18),=CL18' INACTIVE - CLASS='
         SLR   R15,R15             SET                               R4
         IC    R15,$NUMCLAS         CLASS                            R4
         BCTR  R15,0                 FROM                            R4
         EX    R15,PMOVMSG            Q=                             R4
         LA    R0,29(,R15)         GET TOTAL MESSAGE SIZE            R4
        $WTO   PMESSAGE,(R0),JOB=NO,  ISSUE MESSAGE                  R4C
               ROUTE=$LOG+$UR,CLASS=$TRIVIA,PRI=$ST
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        WAIT FOR WORK TO BE ADDED TO THE JOB OUTPUT TABLE            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PJOTWAIT DS    0H
        $WAIT  JOT                 WAIT FOR JOT ACTIVITY
         NI    DCTSTAT,255-DCTHOLD RELEASE DCT
         B     HASPPPI1            GO BACK AND TRY AGAIN
         SPACE 1                                                     R4
PMOVMSG  MVC   PMESSAGE+28(*-*),DCTCLASS  *** EXECUTE ONLY ***       R4
         SPACE 1                                                     R4
         DROP  PW                  DROP DCT ADDRESSABILITY
         EJECT
***********************************************************************
*                                                                     *
*        A JOB HAS BEEN GOTTEN FROM THE JOB OUTPUT TABLE              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PGOTJOB  DS    0H
         ST    R0,PCHJOE           SAVE CHAR-JOE ADDRESS
         ST    R1,PCEJQE           STORE JQE ADDRESS           @OZ32566
         ST    R15,PWKJOE          SAVE WORK-JOE ADDRESS
         LR    JCT,R1              SET JCT = A(JOB QUEUE ELEMENT)
        $TIME  ,                   GET TIME OF DAY
         STM   R0,R1,PTIMEON       PRPU SIGN-ON TIME/DATE
         MVI   PDCTFLAG,0          CLEAR PRPU COPY OF DCTFLAGS       R4
         OI    PPFLAG2,PPFDS       SHOW 1ST DATA SET BEING PROCESSED R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF NOT LOCAL -- INITIALIZE REMOTE PRINTER OR PUNCH           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PCEID,PCELCLID      TEST PROCESSOR TYPE
         BO    PLOCAL              BRANCH IF LOCAL
        $EXTP  OPEN,PCEDCT         OPEN REMOTE FOR PRT OR PNCH @OZ32566
         BNZ   PGOTJB1             OPEN SUCCEEDED              @OZ25817
         USING JOEDSECT,R1         PROVIDE JOE ADDRESSABILITY  @OZ25817
         L     R1,PWKJOE           MAKE WORKJOE ADDRESSABLE    @OZ25817
         TM    JOEFLAG,$JOECKV     IS CHKPT-JOE DATA VALID?    @OZ25817
         BZ    PJOEPUT             NO, BR TO REQUEUE WK-JOE    @OZ25817
         LH    R15,JOECKPT         HALF-WD OFFSET OF CKPT-JOE  @OZ25817
         N     R15,=F'65535'       CLEAR LEFT HALFWORD         @OZ25817
         SLL   R15,2               EXPANDTO BYTE OFFSET        @OZ25817
         AL    R15,$JOTABLE        ADD JOB OUTPUT TABLE ORG.   @OZ25817
         ST    R15,PCKJOE          SAVE CKPT-JOE @ IN PCE      @OZ25817
         MVC   PCKJOE(1),JOEFLAG     SAVE WK-JOE FLAGS IN PCE  @OZ25817
         B     PJPUTI              RETURN TO JOT WITH CKPT     @OZ25817
         DROP  R1                                              @OZ25817
         SPACE 1                                               @OZ25817
*************************************************************  @OZ25817
*                                                              @OZ25817
*        OPEN SUCCEEDED                                        @OZ25817
*                                                              @OZ25817
*************************************************************  @OZ25817
PGOTJB1  DS    0H                                              @OZ25817
        $GETBUF WAIT=YES           GET A DATA BUFFER                 R4
         ST    R1,PBUFSAVE         SAVE BUFFER ADDRESS               R4
         CLI   PBUFOPT,2           TEST FOR DOUBLE BUFFERING         R4
         BNE   PRMTNOSB            BR IF NOT                         R4
        $GETBUF WAIT=YES           GET SECOND DATA BUFFER            R4
         SPACE 1                                                     R4
PRMTNOSB LR    PBUF,R1             ACTIVATE BUFFER ADDRESSABILITY    R4
         ST    PBUF,PBUFADDR       SAVE BUFFER ADDRESS               R4
         ST    PBUF,PINIOB         INITIALIZE INPUT IOB POINTER      R4
         B     PSIGNON             BRANCH AROUND LOCAL DEVICE INIT  R41
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        LOCAL DEVICE INITIALIZATION                                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PLOCAL   DS    0H
         L     PW,PCEDCT           GET ADDRESS OF PRT/PNCH DCT @OZ32566
         SPACE 1                                                     R4
         USING DCTDSECT,PW         ESTABLISH DCT ADDRESSABILITY
         SPACE 1                                                     R4
         NI    DCTPPSW,255-DCTPPSWI RESET INACTIVE MESSAGE SWITCH
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        DETERMINE DE-SPOOLING METHOD    (SINGLE OR TRACK-CELL)       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LA    PC2,1               INITIALIZE TRACK-CELL SIZE TO 1   R4
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE               R4
         BZ    PGETBUFS            FORCE SINGLE IF NOT LOCAL PRINTER R4
         TM    DCTPPFL,DCTTCEL     TEST FOR DESPOOL=TRKCEL SPECIFIED R4
         BZ    PGETBUFS            BRANCH IF NO                      R4
         CLI   $TCELSIZ,1          IF TRACK-CELL SIZE IS NOT GT 1    R4
         BNH   PGETBUFS             THEN FORCE DESPOOL=SINGLE        R4
         L     R15,PWKJOE          PICK UP WORK-JOE ADDRESS          R4
         SPACE 1                                                     R4
         USING JOEDSECT,R15        ACTIVATE JOE ADDRESSABILITY       R4
         SPACE 1                                                     R4
         TM    JOEFLAG2,$JOETCEL   WAS DATA SET TRACK-CELL'ED ...    R4
         BZ    PGETBUFS            FORCE DESPOOL=SINGLE IF NOT       R4
         OI    PPFLAG2,PPTCEL      INDICATE TRACK-CELL DESPOOLING    R4
         IC    PC2,$TCELSIZ        PICK UP &TCELSIZ VALUE            R4
         SPACE 1                                                     R4
         DROP  R15,PW              SUSPEND JOE/DCT ADDRESSABILITY    R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        GET DATA BUFFERS FROM THE MAIN HASP BUFFER POOL              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PGETBUFS DS    0H                                                    R4
        $GETBUF WAIT=YES,NUM=(PC2),FIX=YES  GET FIRST BUFFER(S)      R4
         ST    R1,PBUFSAVE         SAVE ADDR OF 1ST BUFFER (CHAIN)   R4
         CLI   PBUFOPT,2           TEST BUFFERING OPTION             R4
         BNE   PNOSECBF            BRANCH IF NOT DOUBLE BUFFERING    R4
        $GETBUF WAIT=YES,NUM=(PC2),FIX=YES  GET SECOND BUFFER(S)     R4
         SPACE 1                                                     R4
PNOSECBF DS    0H                  ACTIVATE DATA                     R4
         LR    PBUF,R1              BUFFER ADDRESSABILITY            R4
         ST    PBUF,PBUFADDR       SAVE PRIMARY DATA BUFFER ADDRESS  R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        GET BUFFER FOR OUTPUT CCW'S                                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
*                                                                    R4
*        FOR 3800 PRINTERS, GET A PAGE BUFFER                        R4
*                                                                    R4
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BNE   PNOT3800            BR IF NOT                         R4
        $GETBUF WAIT=YES,TYPE=PAGE,FIX=YES  GET OUTPUT CCW BUFFER    R4
         LA    PW,(4096-(BUFSTART-BUFDSECT)-2*((JOESIZE+7)/8*8)-2*PCIESC
               IZE)/16*16/2        CALCULATE SIZE OF OUTPUT CCW AREA R4
         B     PGBFINIT            GO TO INITIALIZE BUFFER VARIABLES R4
*                                                                    R4
*        FOR NON - 3800 DEVICES, GET A PP BUFFER                     R4
*                                                                    R4
PNOT3800 DS    0H                                                    R4
        $GETBUF WAIT=YES,TYPE=PP,FIX=YES  GET OUTPUT CCW BUFFER      R4
         SLR   PW,PW               CLEAR WORK REGISTER FOR IC        R4
         IC    PW,$NOPRCCW         PICK UP PRINT CCW COUNT           R4
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE               R4
         BO    SKIP110             BRANCH IF PRINTER                 R4
         IC    PW,$NOPUCCW          ELSE PICK UP PUNCH CCW COUNT     R4
SKIP110  SLL   PW,3                MAXIMUM OUTPUT CCW CHAIN LENGTH   R4
PGBFINIT DS    0H                                                    R4
         ST    R1,POUTIOB          SAVE OUTPUT IOB ADDRESS           R4
         L     PL,PCEDCT                 AND SAVE              @OZ32566
         ST    R1,DCTBUFAD-DCTDSECT(,PL)  IN DCT                     R4
         LA    R1,BUFSTART-BUFDSECT(,R1)            ADDRESS OF FIRST R4
         ST    R1,POUTCCWA                              CCW AREA     R4
         LA    R14,PCIESIZE+(JOESIZE+7)/8*8(PW,R1)  ADDRESS OF SECONDR4
         ST    R14,POUTCCWN                             CCW AREA     R4
         STH   PW,PCCWLAST         OFFSET TO PCIE (CCW AREA LENGTH)  R4
         L     R15,POUTIOB         GET IOB ADDRESS             @OZ29106
         LR    PL,R14              COPY 2ND CCW ADDRESS        @OZ29106
         SLR   PL,R15              COMPUTE OFFSET INTO IOB     @OZ29106
         SRL   PL,3                DIVIDE BY 8                 @OZ29106
         STH   PL,PPBDISPL-BUFDSECT(R15) SAVE OFFSET/8         @OZ29106
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        INITIALIZE PCIE AT END OF EACH CCW AREA                      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING PCIDSECT,PL         ACTIVATE PCIE ADDRESSABILITY      R4
         SPACE 1                                                     R4
         LA    PL,0(PW,R1)         ADDRESS OF 1ST PCIE               R4
         MVC   PCI1CCW(PCIESIZE),PCNOPTIC   MOVE IN NOP/TIC SKELETON R4
         LRA   R0,0(,R14)          PLACE REAL ADDRESS OF SECOND      R4
         STCM  R0,7,PCI2DADR        CCW AREA INTO 1ST PCIE'S TIC     R4
         LA    PL,0(PW,R14)        ADDRESS OF 2ND PCIE               R4
         MVC   PCI1CCW(PCIESIZE),PCNOPTIC   MOVE IN NOP/TIC SKELETON R4
         LRA   R0,0(,R1)           PLACE REAL ADDRESS OF FIRST       R4
         STCM  R0,7,PCI2DADR        CCW AREA INTO 2ND PCIE'S TIC     R4
         DROP  PL                  SUSPEND PCIE ADDRESSABILITY       R4
         SPACE 1                                                     R4
         SH    R1,=H'8'            INITIALIZE CURRENT                R4
         ST    R1,PCCWPT            CCW POINTER                      R4
         TM    PCEID,PCERJEID+PCEPRSID  TEST PROCESSOR TYPE    @OZ32288
         BNZ   PRTNOPUN            BRANCH IF NOT LOCAL PUNCH   @OZ32288
         ST    R1,PUERRPT          SET PUNCH ERROR CUTOFF            R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        INITIALIZE BUFFER INFORMATION FOR INPUT CCW'S                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRTNOPUN TM    PPFLAG2,PPTCEL      TEST DESPOOLING METHOD      @OZ32288
         BO    PGETPPIN            BR IF TRACK-CELL                  R4
         ST    PBUF,PINIOB          ELSE USE DATA BUFFER FOR IOB     R4
         B     PFIXDEB             GO FIX DEB                        R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ACQUIRE A PP BUFFER FOR TRACK-CELL INPUT CCW'S               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PGETPPIN DS    0H                                                    R4
        $GETBUF WAIT=YES,TYPE=PP   GET PP BUFFER                     R4
         ST    R1,PINIOB           SAVE PP BUFFER'S IOB ADDRESS      R4
         SLR   PW,PW               CLEAR PW FOR PINMTTRT CALCULATION R4
         IC    PW,$TCELSIZ         ADDRESS OF MTTR/BUFFER ADDR TABLE R4
         LA    R0,1(PW,PW)          = &TCELSIZ*3    (SRCH/TIC/READ)  R4
         ALR   PW,R0                     +1           (SET SECTOR)   R4
         SLL   PW,3                      *8                          R4
         LA    PW,IOBCCW1-BUFDSECT(PW,R1) PLUS IOBCCW1               R4
         ST    PW,PINMTTRT         SAVE TABLE ADDRESS                R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PAGE FIX PRINT/PUNCH DEB FOR THE DURATION OF THIS OUTPUT     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PFIXDEB  L     R1,PCEDCT           ADDRESS LOCAL PRT/PNCH DCT  @OZ32566
         USING DCTDSECT,R1         PICK-UP DCB ADDRESS               R4
         L     R1,DCTDCB            FROM DCT                         R4
         USING DCBDSECT,R1         PICK-UP DEB ADDRESS               R4
         L     R1,DCBDEBAD          FROM DCB                         R4
         LA    R0,DEBBASND-DEBDSECT  SIZE OF JES2 DEB                R4
         LA    R2,PPLSAVE          ADDRESS OF A PSEUDO-ECB           R4
        $PGSRVC FIX,(R1),(R0),(R2)   FIX PRINTER/PUNCH DEB           R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ISSUE PRINT/PUNCH JOB SIGN ON MESSAGE TO THE OPERATOR        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
        $MID   150                                                   R4
PSIGNON  MVC   PMESSAGE(2),=X'150F' ON DEVICE MESSAGE ID            R41
         MVC   PMESSAGE+2(4),=CL4'ON' MESSAGE TEXT
         L     R1,PCEDCT           GET ADDRESS OF PRT/PNCH DCT @OZ32566
         USING DCTDSECT,R1         ESTABLISH DCT ADDRESSABILITY
         MVC   PMESSAGE+5(8),DCTDEVN DEVICE NAME
         L     R1,PWKJOE           ADDRESS WORK-JOE                 R41
         USING JOEDSECT,R1         PROVIDE JOE ADDRESSABILITY       R41
         L     R1,JOERECCT         GET SYSOUT RECORD COUNT          R41
         LA    R0,13               SHORT MESSAGE LENGTH             R41
         LTR   R1,R1               TEST RECORD COUNT                R41
         BZ    PONWTO              BR IF NONE                       R41
         CVD   R1,PCCWORK          MAKE RECORD COUNT DECIMAL        R41
         MVC   PMESSAGE+13(10),PPRTSTAT   MAKE RECORD               R41
         ED    PMESSAGE+13(10),PCCWORK+4   COUNT READABLE           R41
         MVC   PMESSAGE+23(6),=C' LINES'  ASSUME PRINTER            R41
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE              R41
         BO    PONPRTR             BR IF PRINTER                    R41
         MVC   PMESSAGE+23(6),=C' CARDS'  ELSE, SETUP FOR PUNCH     R41
         SPACE 1                                                    R41
PONPRTR  LA    R0,29               ASSUME MORE THAN 1 RECORD        R41
         BCT   R1,PONWTO           BR IF PLURAL                     R41
         LA    R0,28                ELSE DROP THE 'S'               R41
         SPACE 1                                                    R41
PONWTO  $WTO   PMESSAGE,(R0),      'JOB ON PRINTER/PUNCH' MESSAGE   R41C
               ROUTE=$LOG+$UR,JOB=YES,  (NOTE: R10 = ADDR OF JQE)   R41C
               CLASS=$TRIVIA,PRI=$ST                                R41
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        LOCATE CHECKPOINT-JOE                                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
         USING JOEDSECT,R1         PROVIDE JOE ADDRESSABILITY       R41
         SPACE 1                                                    R41
         L     R1,PWKJOE           ADDRESS WORK-JOE                 R41
         LH    R14,JOECKPT         HALFWORD OFFSET OF CKPT-JOE
         N     R14,=F'65535'       CLEAR LEFT HALFWORD
         SLL   R14,2               EXPAND TO BYTE OFFSET             R4
         AL    R14,$JOTABLE        ADD JOB OUTPUT TABLE ORIGIN
         ST    R14,PCKJOE          SAVE A(CKPT-JOE) IN PCE
         MVC   PCKJOE(1),JOEFLAG   COPY WORK-JOE FLAGS
         SPACE 3                                                    R41
***********************************************************************
*                                                                     *
*        GET ADDRESS OF JCT IN BUFFER FROM JCT MANAGER                *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
        $#JCT  READ                GET ADDR OF JCT BUFFER           R41
         ST    JCT,PJCTBUF         SAVE JCT BUFFER ADDRESS          R41
         BZ    PRPUEXIT            GO TO EXIT - BAD JCT             R41
         EJECT                                                 @OZ26939
***************************************************************@OZ26939
*                                                              @OZ26939
*        ATTACH HASPIMAG SUBTASK FOR PRINTERS                  @OZ26939
*                                                              @OZ26939
***************************************************************@OZ26939
         SPACE 1                                               @OZ26939
         TM    PCEID,PCEPUSID      TEST PROCESSOR TYPE         @OZ26939
         BO    PCLDSTRT            BR IF PUNCH                 @OZ26939
         L     R1,PRIMGDTE          ELSE GET HASPIMAG DTE ADDR @OZ26939
         OC    0(4,R1),0(R1)       SUBTASK ALREADY ACTIVE...   @OZ26939
         BNZ   PCLDSTRT            BR IF YES                   @OZ26939
         L     R15,=A(PIMAGLST)     ELSE MOVE ATTACH PARMS     @OZ26939
         MVC   $CSAVREG(PIMAGLL),0(R15)  TO WORK AREA          @OZ26939
         LR    PW,R1               SAVE DTE ADDRESS            @OZ26939
         ATTACH ECB=4(,R1),MF=(E,(1)),SF=(E,$CSAVREG)          @OZ26939
         ST    R1,0(,PW)           STORE TCB ADDRESS INTO DTE  @OZ26939
         SPACE 1                                               @OZ26939
PATWAIT $WAIT  IMAG                WAIT FOR SUBTASK TO INIT    @OZ26939
         TM    0(PW),X'80'         HASPIMAG INITIALIZED...     @OZ26939
         BNO   PATWAIT             BR IF NOT YET               @OZ26939
         SPACE 1                                               @OZ26939
         EJECT                                                       R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SETUP PRINT/PUNCH DEVICE BY CHAR-JOE                         *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCLDSTRT DS    0H
         L     R1,PCHJOE                ADDRESS CHAR-JOE             R4
         L     PL,PCEDCT           ADDRESS PRINTER DCT         @OZ32566
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BNE   PSETUPST            BR IF NOT - SETUP FOR IMPACT PRT  R4
         MVC   SPFORMS(2*4),JOEFORM USE JOE FORMS AND FCB ID         R4
         MVC   SPFLASH,JOEFLASH    SET FLASH (SEP WON'T FLASH)      R41
         MVC   SPMODF,=C'****'     RESET COPY MODIFICATION           R4
         MVI   SPCOPYN,1           FORCE ONLY 1 COPY OF HEADER       R4
         MVI   SPCOPYS,1           INDICATE STARTING COPY NUMBER     R4
         MVI   SPFLAG,SPSEP        INIT FLAGS FOR SEP PAGE          R41
         TM    JOECFLAG,$JOEBRST   SET FOR                           R4
         BZ    PSETUP               NOBURST OR                      R41
         OI    SPFLAG,SPBURST        BURST                           R4
         B     PSETUP              ENTER COMMON SETUP                R4
         SPACE 1                                                    R41
PSETUPST DS    0H                  SETUP STANDARD PRINTER/PUNCH      R4
         LA    R1,JOEFORM          ADDRESS SETUP PORTION             R4
         MVI   DCTACPTN-DCTDSECT(PL),X'00' DISABLE COMPACT FOR SEP  R41
         MVI   PRINDEX,X'81'       SET 3211 INDEX TO 1
         SPACE 1                                                     R4
PSETUP   DS    0H                                                    R4
         L     R15,=A(PRPUDSV)     CALL DEVICE                       R4
         BALR  PL,R15               SETUP VERIFICATION               R4
         TM    PPFLAG,PRDELSW      IS JOB ABORTED
         BO    PPABORT             BRANCH IF YES
         EJECT                                                      R41
***********************************************************************
*                                                                     *
*        OFFSET-STACK 3800 BURSTER JOBS                               *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
         USING DCTDSECT,R1         PROVIDE DCT ADDRESSABILITY       R41
         SPACE 1                                                    R41
         L     R1,PCEDCT           ADDR OF DCT                 @OZ32566
         CLI   PDEVTYPE+3,UCB3800  TEST DEVICE TYPE                 R41
         BNE   PTESTSEP            BR IF NOT 3800 PRINTER           R41
         LM    PC1,PC2,PCCWOFST     ELSE ISSUE AN OFFSET-           R41
         BAL   PL,PPPUT              STACK COMMAND                  R41
         SPACE 2                                                    R41
***********************************************************************
*                                                                     *
*        SEE IF OUTPUT SEPARATORS ARE WANTED                          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PTESTSEP L     R1,PCEDCT           ADDR OF DCT                 @OZ32566
         TM    DCTPPSW,DCTPPSWS    SUPPRESS SEPARATORS...      @OZ18408
         BO    PCKPTNIT            BR IF YES                        R41
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE
         BO    PRINTSEP            BRANCH IF PRINT
         SPACE 1                                                    R41
         DROP  R1                  SUSPEND DCT ADDRESSABILITY       R41
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PRODUCE PUNCH SEPARATOR LACE CARD                            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PUNCHSEP DS    0H
         SPACE 1                                                    R41
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE              R41
         BZ    PUNCHRM             BR IF NOT REMOTE                 R41
         TM    MDCTFEAT-DCTDSECT(R1),DCTPSHDR IF NO SETUP HDR FOR   R41
         BNO   PUNCHRM                         PUNCH, SKIP PDIR     R41
         MVI   PPDIRID,X'01'       INDICATE SEPARATOR PDIR          R41
         LA    PL,2                USE A RECORD COUNT OF 2          R41
         ST    PL,PPDSRCT           FOR PUNCH                       R41
         L     R15,=A(PPDIR)       POINT TO SEPARATOR ROUTINE       R41
         BALR  PL,R15              PUT SEPARATOR PDIR               R41
         SPACE 1                                                    R41
PUNCHRM  MVC   PCCWORK(4),JCTROOMN SET UP ROOM NUMBER               R41
         MVC   PCCWORK+4(4),JCTJOBID+4 AND JOB NUMBER
         LA    R1,BUFSTART         ADDRESS BUFFER AS A WORK AREA
         SLR   PW,PW               PREPARE FOR SCAN
PUCHGEN  MVI   0(R1),X'6A'         START EACH FIELD WITH 12-11 PUNCH
         IC    R0,PCCWORK(PW)      MOVE NEXT CHARACTER
         STC   R0,1(,R1)            TO IMAGE
         OI    1(R1),X'30'         CONVERT
         CLI   1(R1),X'F0'          EACH
         BH    PUCHARD               CHARACTER
         BE    PUCHAR0                FROM
         MVI   1(R1),X'EA'             EBCDIC
PUCHAR0  XI    1(R1),X'E0'              TO
PUCHARD  XI    1(R1),X'60'               12-11-X PUNCH
         MVC   2(7,R1),1(R1)       PROPAGATE CHARACTER
         MVI   9(R1),X'6A'         TERMINATE FIELD WITH 12-11 PUNCH
         LA    R1,10(,R1)          STEP TO NEXT FIELD
         LA    PW,1(,PW)           STEP TO NEXT CHARACTER
         CL    PW,=F'8'            TEST
         BL    PUCHGEN             BRANCH IF NOT LAST CHARACTER
         LA    PC1,BUFSTART        SETUP PUNCH CCW
         AL    PC1,PUCCW           ADD LEFT HALF OF CCW
         L     PC2,PUCCW+4         AND RIGHT HALF OF CCW
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         LM    PC1,PC2,PUCCWBL     LOAD BLANK CARD CCW
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BAL   PL,PPWRITE           INITIATE WRITE
         BAL   PL,PPCHECK            AND CHECK
         B     PCKPTNIT            CONTINUE                         R41
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PRODUCE PRINT SEPARATOR PAGE                                 *
*                                                                     *
*        USE MAIN PRINT LOOP TO PRINT JES2-NEWS (IF AVAILABLE)        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRINTSEP L     R1,$NEWSTTR         GET JES2-NEWS MTTR               R41
         LTR   R1,R1               NEWS AVAILABLE...                R41
         BZ    PHEADER1            BR IF NOT                        R41
         OI    PPFLAG,PPNEWS        ELSE SET NEWS FLAG              R41
         ST    R1,PCEJMTTR         SET NEWS MTTR                    R41
         MVC   PCEEJRCB,PCCW+2     SET INITIAL RCB OFFSET           R41
         MVC   PPKEY,=C'$$NEWS'    SET SPECIAL JOB/DS KEY           R41
         MVC   PRLINECT,=F'-1'     SET LARGE PAGE SIZE              R41
         SPACE 1                                                    R41
PHEADER1 LA    R1,=C'START'        ASSUME COLD START                R41
         TM    PCKJOE,$JOECKV      'START' OR 'CONT'...             R41
         BZ    PHEADER2            BR IF COLD START                 R41
         LA    R1,=C'CONT '         ELSE SET FOR CONTINUATION       R41
         SPACE 1                                                    R41
PHEADER2 BAL   PL,PRINTID          PRODUCE PRINT SEPARATOR PAGE     R41
         TM    PPFLAG,PPNEWS       NEWS AVAILABLE...                R41
         BZ    PCKPTNIT            BR IF NO                         R41
         B     PNEWSGO              ELSE USE MAIN PRINT LOOP        R41
         SPACE 1                                                    R41
PENDNEWS LM    PC1,PC2,PRCCWEJ     * RETURN HERE AFTER JES2-NEWS *  R41
         BAL   PL,PPPUT            SKIP                             R41
         BAL   PL,PPWRITE           TO TOP                          R41
         BAL   PL,PPCHECK            OF PAGE                        R41
         NI    PPFLAG,255-PPNEWS   RESET JES2-NEWS INDICATOR        R41
         EJECT                                                      R41
***********************************************************************
*                                                                     *
*        INITIALIZE PRPU CHECKPOINT DATA AREA                         *
*                                                                     *
*        COPY CKPT-JOE TO PRPU CHECKPOINT DATA AREA ON WARM START     *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
         USING JOEDSECT,R1         PROVIDE JOE ADDRESSABILITY       R41
         SPACE 1                                                    R41
PCKPTNIT MVC   PPJOBKEY,JCTJBKEY   SET JOB KEY                      R41
         TM    PCKJOE,$JOECKV      WARM START...                    R41
         BO    PWARMNIT            BR IF YES                        R41
         XC    PCKPT,PCKPT         CLEAR PRPU CKPT DATA AREA        R41
         L     PW,$IOTPDDB         SET INITIAL                      R41
         STH   PW,PDDBDISP          PDDB DISPLACEMENT               R41
         MVC   PCEIOTTR,JCTIOT     SET IOT MTTR                     R41
         SPACE 1                                                    R41
         L     R1,PWKJOE           ADDRESS WORK-JOE                 R41
         TM    JOEFLAG,$JOESPIN    SPIN JOE...                      R41
         BZ    PPAGSIZE            BR IF NO                         R41
         MVC   PCEIOTTR,JOEIOTTR    ELSE SET SPIN-IOT MTTR          R41
         OI    PSMFDCI,SMFSPIN     SET SPIN SMF FLAG                R41
         B     PPAGSIZE            CONTINUE                         R41
         SPACE 1                                                    R41
PWARMNIT L     R1,PCKJOE           ADDRESS CKPT-JOE                 R41
         MVC   PCKPT,0(R1)         REFRESH PRPU CKPT DATA AREA      R41
         MVC   PBUFSKIP,PCEJBOFF   SET BUFFER OFFSET FOR RESTART    R41
         OI    PSMFDCI,SMFINTRC    SET WARM-START SMF FLAG          R41
         SPACE 1                                                    R41
         DROP  R1                  SUSPEND JOE ADDRESSABILITY       R41
         SPACE 1                                                    R41
***********************************************************************
*                                                                     *
*        SETUP LOGICAL PAGE SIZE                                      *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
PPAGSIZE SLR   PW,PW               GET JOB'S                        R41
         IC    PW,JCTLINCT          PAGE SIZE (LINE COUNT)          R41
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE              R41
         BO    PSETSIZE            BR IF PRINT                      R41
         IC    PW,=AL1(100)         ELSE USE 100 CARDS FOR PUNCH    R41
         SPACE 1                                                    R41
PSETSIZE BCTR  PW,0                SET LOGICAL                      R41
         ST    PW,PRLINECT          PAGE SIZE                       R41
         SPACE 1                                                    R41
         TM    PPFLAG,PPDELSW      DELETION DURING PRPU INIT...     R41
         BO    PPDSEND             BR IF YES                        R41
         TITLE 'HASP PRINT/PUNCH SERVICE -- DATA SET SELECTION'
***********************************************************************
*                                                                     *
*        PROCESSOR INITIALIZATION COMPLETE - READ/CHECK/VERIFY IOT    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PENDINIT DS    0H
         L     R15,PCEIOTTR        GET IOT MTTR
         BAL   PL,PRDBUF           INITIATE READ OF IOT
         BAL   PL,PRDCHK           CHECK READ
         LR    JCT,PBUF            ADDRESS IOT IN BUFFER
         USING IOTDSECT,JCT        ACTIVATE IOT ADDRESSABILITY
         TM    PPFLAG,PPRDERR      TEST FOR I/O ERROR ON READ
         BO    PIOTPRE             BR IF YES                         R4
         CLC   IOTJBKEY,PPJOBKEY   IS THIS IOT VALID
         BE    PPIOTOK             BRANCH IF YES
PIOTPRE  $DISTERR                  INDICATE CONTROL BLOCK ERROR
         OI    PPFLAG,PPJCTIOT+PRDELSW REASON FOR TERMINATION
         B     PPDONE              ABORT JOB
PPIOTOK  DS    0H
         TM    IOTFLAG1,IOT1SPIN   SPIN DATA SET...                  R4
         BNO   *+8                 BRANCH IF NO - NO $PURGE AT EOJ
         OI    PPFLAG,PPDALOC      SET $PURGE REQUIRED FLAG
         LH    PC1,PDDBDISP        GET NEXT PDDB DISPLACEMENT
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        SELECT NEW PDDB -- GO READ NEXT IOT IF NONE LEFT             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING JCTDSECT,R15        PROVIDE JCT ADDRESSABILITY        R4
         USING PDBDSECT,PC2        PROVIDE PDDB ADDRESSABILITY       R4
         SPACE 1                                                     R4
PDDBNEXT DS    0H
         L     R15,PJCTBUF         ADDRESS JCT IN BUFFER
         LA    PC2,0(PC1,JCT)      GENERATE ABSOLUTE ADDRESS
         CL    PC1,IOTPDDBP        END OF PDDB'S IN THIS IOT
         BNE   PPPDB               BRANCH IF NO
         TM    IOTFLAG1,IOT1SPIN   IS THIS A SPIN/HOLD IOT
         BO    PPDONE              BRANCH IF YES
         ICM   R15,15,IOTIOTTR     LOAD AND TEST NEXT TRACK
         BZ    PPDONE              BRANCH IF NO MORE IOTS
         ST    R15,PCEIOTTR        SAVE NEW TRACK IN PCE
         L     R14,$IOTPDDB        SET INITIAL PDDB OFFSET           R4
         STH   R14,PDDBDISP        IN CKPT AREA OF PCE
         B     PENDINIT            READ NEXT IOT
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        VERIFY THAT PDDB MATCHES WORK/CHAR-JOE -- ELSE SKIP IT       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPPDB    DS    0H
         TM    PDBFLAG1,PDB1NULL+PDB1NSOT IS PDDB NULL/NOT SYSOUT
         BNZ   PDDBSRCH            BRANCH IF YES
*
*        CHECK SYSOUT CLASS
*
         L     PW,PWKJOE           ADDRESS WORK JOE                  R4
         USING JOEDSECT,PW         ACTIVATE JOE ADDRESSABILITY       R4
         CLC   PDBCLASS,JOEPDBCL   DOES CLASS MATCH JOE              R4
         BNE   PDDBSRCH            BRANCH IF NO
*
*        CHECK REMAINING CHARACTERISTICS
*
         TM    JOEFLAG2,$JOEDMND   TEST FOR DEMAND-SETUP OPTION      R4
         L     PW,PCHJOE           ADDRESS CHAR-JOE
         BZ    SKIP130             SKIP CLASS TEST IF NOT CHOSEN     R4
         CLC   PDBCLASS,JCTMCLAS   DOES SYSOUT CLASS = MSGCLASS
         BE    PPDEMAND            BRANCH IF YES - DEMAND SETUP
SKIP130  CLC   JOEFORM(12),PDBFORMS CHECK FORMS, FCB, UCS
         BNE   PDDBSRCH            BRANCH IF NOT MATCHING
         CLC   JOEFLASH,PDBFLASH   CHECK OVERLAY-FRAME               R4
         BNE   PDDBSRCH            IF NOT MATCHING, BRANCH           R4
         TM    PDBFLAG2,PDB2BRST   VERIFY                            R4
         BZ    SKIP140              THAT                             R4
         TM    JOECFLAG,$JOEBRST     PDDB                            R4
         BZ    PDDBSRCH               BURSTER                        R4
         B     PPDEMAND                SPECIFICATION                 R4
SKIP140  TM    JOECFLAG,$JOEBRST        MATCHES                      R4
         BO    PDDBSRCH                  CHAR-JOE                    R4
PPDEMAND CLC   JOEWTRID,PDBWTRID   CHECK EXTERNAL WRITER NAME        R4
         BNE   PDDBSRCH            BRANCH IF NOT MATCHING
         L     PW,PWKJOE           ADDRESS WORK-JOE
         CLC   JOEDEST,PDBDEST     CHECK DESTINATION
         BE    PGOTPDDB            BR IF SAME -- DATA SET SELECTED   R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        PDDB DOES NOT MATCH WORK/CHAR-JOE -- STEP TO NEXT            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PDDBSRCH DS    0H                                                    R4
         LH    PC1,PDDBDISP        GET CURRENT PDDB OFFSET           R4
         LA    PC1,PDBLENG(,PC1)   STEP TO NEXT PDDB                 R4
         STH   PC1,PDDBDISP        SAVE NEW PDDB OFFSET IN CKPT AREA R4
         B     PDDBNEXT            GO TEST NEW PDDB                  R4
         SPACE 1                                                     R4
         DROP  PW,R15              SUSPEND JOE/JCT ADDRESSABILITY    R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PDDB MATCHES -- SETUP PRPU WORK AREA FOR DATA SET            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PGOTPDDB DS    0H                                                    R4
         MVC   PMESSAGE(12),PDBFORMS SAVE SETUP
         MVC   PRINDEX,PDBINDEX    SAVE 3211 INDEX VALUE
         MVC   PPDSCPY,PDBCOPYS    SAVE DATA SET COPY REQUEST
         MVC   PPDSKEY,PDBDSKEY    SAVE DATA SET KEY
         L     R15,PCEDCT          ADDRESS PRINTER DCT         @OZ32566
         MVC   DCTACPTN-DCTDSECT(1,R15),DCTDCPTN-DCTDSECT(R15)      R41
         CLI   PDBCPTN,X'FF'       IF PDDB CPT NUMBER NOT SPECIFIED R41
         BE    PSYSOUT              USE DEFAULT                     R41
         MVC   DCTACPTN-DCTDSECT(1,R15),PDBCPTN USE CPT IN PDDB     R41
PSYSOUT  MVI   PPDIRID,X'00'       INDICATE SYSOUT PDIR             R41
         MVC   PPDSRCT(4),PDBRECCT SAVE PDDB DATASET RECORD COUNT   R41
         TM    PCKJOE,$JOECKV      SAVE 1ST MTTR                     R4
         BO    SKIP150              OF DATA SET IF                   R4
         MVC   PCEJMTTR,PDBMTTR      COLD START - ELSE USE CKPT JOE  R4
         SPACE 1                                                     R4
SKIP150  TM    PDBFLAG2,PDB2TCEL   DOES PDDB SPECIFY TRACK-CELL      R4
         BO    PTSTNIPR            BR IF YES                         R4
         TM    PPFLAG2,PPTCEL      TEST FOR PREVIOUS TRACK-CELL'ING  R4
         BZ    PTSTNIPR            BR IF NO                          R4
*                                                                    R4
*        NON-TRACK-CELL'ED PDDB DETECTED IN TRACK-CELL'ED JOE        R4
*                                                                    R4
        $FREEBUF PINIOB            FREE-UP INPUT CCW BUFFER          R4
         NI    PPFLAG2,255-PPTCEL  RESET DESPOOLING METHOD TO SINGLE R4
         ST    PBUF,PINIOB         USE IOB IN DATA BUFFER            R4
         SPACE 1                                                     R4
         CLI   PBUFOPT,2           TEST BUFFER OPTION                R4
         BL    PNOTCEL1            BR IF NOT DOUBLE                  R4
         L     R1,PBUFSAVE         ADDRESS SECONDARY BUFFER CHAIN    R4
         L     R1,BUFCHAIN-BUFDSECT(,R1)  FREE ALL BUT 1ST           R4
        $FREEBUF (R1),MULTIPLE             BUFFER IN CHAIN           R4
         SPACE 1                                                     R4
PNOTCEL1 L     R1,PBUFADDR         ADDRESS PRIMARY BUFFER CHAIN      R4
         L     R1,BUFCHAIN-BUFDSECT(,R1)  FREE ALL BUT 1ST           R4
        $FREEBUF (R1),MULTIPLE             BUFFER IN CHAIN           R4
         SPACE 1                                                     R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SETUP FOR 3800 PRINTERS                                      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PTSTNIPR DS    0H                                                    R4
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BNE   PSMFTST             BR IF NOT                         R4
         MVC   PFLASHC,PDBFLSHC    MOVE FLASH COUNT TO WORK AREA     R4
         XC    PCOPYGRP,PCOPYGRP   ZERO COPY GROUPS IN WORK AREA     R4
         MVI   SPFLAG,0            CLEAR SETUP FLAGS                 R4
         TM    PDBFLAG2,PDB2BRST   TEST FOR BURSTING                 R4
         BZ    PNOBURST            BR IF NO                          R4
         OI    SPFLAG,SPBURST      SET BURST SETUP FLAG              R4
PNOBURST DS    0H                                                    R4
         MVC   SPFORMS(2*4),PDBFORMS  FORMS/FCB                      R4
         MVC   SPCHAR1(6*4+2*1),PDBCHAR1 CHAR1-4/FLSH/MODF/FLSHC/TRC R4
         CLI   PDBCOPYG,0          TST FOR NULL COPY GROUPS          R4
         BE    PNIPSET1            BR IF YES                         R4
         SLR   R0,R0               CLEAR                             R4
         SLR   R1,R1                WORK                             R4
         SLR   PW,PW                 REGISTERS                       R4
         LA    PL,8                MAX COPY GROUPS                   R4
         SPACE 1                                                     R4
PCGROUPS IC    R0,PDBCOPYG(R1)     INITIALIZE                        R4
         ALR   PW,R0                COPY-GROUPS                      R4
         CH    PW,=H'255'            ENSURING THAT                   R4
         BH    PCGT255                TOTAL                          R4
         STC   R0,PCOPYGRP(R1)         DOES NOT                      R4
         LA    R1,1(,R1)                EXCEED                       R4
         BCT   PL,PCGROUPS               255                         R4
         STC   PW,PPDSCPY                 DATA SET                   R4
         B     PNIPSET1                    COPIES                    R4
         SPACE 1                                                     R4
PCGT255  SH    PW,=H'255'          IF EXCESSION,                     R4
         SLR   R0,PW                AJUST FOR                        R4
         STC   R0,PCOPYGRP(R1)       ONLY 255                        R4
         MVI   PPDSCPY,255            COPIES                         R4
         SPACE 1                                                     R4
         EJECT                                                       R4
PNIPSET1 DS    0H                                                    R4
         SLR   R1,R1               DETERMINE                         R4
         LA    R0,3                 MAXIMUM                          R4
         LA    PL,PDBCHAR2           VALUE                           R4
PCNTRC   CLC   =C'****',0(PL)         ALLOWED                        R4
         BE    SKIP160                 FOR            ( OPTCD=J )    R4
         LA    R1,1(,R1)                IMBEDDED                     R4
         LA    PL,4(,PL)                 TABLE-                      R4
         BCT   R0,PCNTRC                  REFERENCE-                 R4
SKIP160  STC   R1,PRMAXTRC                 CHARACTERS (TRC)          R4
         SPACE 1                                                     R4
         TM    PCKJOE,$JOECKV      TEST FOR WARM START               R4
         BO    PNIPWARM            BR IF YES                         R4
         MVI   SPCOPYS,1           ELSE, INIT STARTING COPY NUMBER   R4
         MVI   SPCOPYN,1           ASSUME 1 COPY                     R4
         CLI   PCOPYGRP,0          TEST FOR COPY GROUPS SPECIFIED    R4
         BE    SKIP170             BR IF NOT--ASSUMPTION OK          R4
         MVC   SPCOPYN,PCOPYGRP    ELSE, USE 1ST TRANSMISSION COUNT  R4
SKIP170  MVI   PDDBCPYG,0          RESET COPY GROUP OFFSET           R4
         B     PSMFTST             CONTINUE                          R4
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        SETUP FOR 3800 DATA SET WARM START                           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNIPWARM DS    0H                                                    R4
         LA    R1,1                ASSUME 1 COPY THIS TRANSMISSION   R4
         CLI   PCOPYGRP,0          TEST FOR COPY GROUPS SPECIFIED    R4
         BE    SKIP180             BR IF NOT - ASSUMPTION OK         R4
         IC    R1,PDDBCPYG         ELSE, PICK UP OFFSET TO           R4
         IC    R1,PCOPYGRP(R1)      COPY GROUP AND GET COUNT         R4
SKIP180  STC   R1,SPCOPYN          MOVE NO. COPIES TO SETUP LIST     R4
         MVI   SPFLASHC,0          ASSUME NO FLASHING                R4
         IC    R1,PPRCPYCT         GET TOTAL COPIES ALREADY PRINTED  R4
         SLR   R0,R0               COMPUTE NUMBER                    R4
         IC    R0,PFLASHC           OF COPIES LEFT TO                R4
         SR    R0,R1                 BE FLASHED                      R4
         BNP   SKIP190             BR IF ALL DONE -- ASSUMPTION OK   R4
         STC   R0,SPFLASHC         ELSE MOVE COUNT REMAINING TO LIST R4
SKIP190  LA    R1,1(,R1)           INDICATE STARTING                 R4
         STC   R1,SPCOPYS           COPY NUMBER                      R4
         SPACE 1                                                     R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        WRITE A NEW TYPE 6 SMF RECORD IF PERTINENT DATA HAS CHANGED  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PSMFTST  CLI   $NUMSMFB,2          TEST SMF BUFFER COUNT             R4
         BL    PPDBFLGS            BR IF SMF NOT SUPPORTED           R4
         SPACE 1                                                     R4
         L     PW,PCEDCT           PROVIDE DCT                 @OZ32566
         USING DCTDSECT,PW          ADDRESSABILITY                   R4
         SPACE 1                                                     R4
         TM    PCKJOE,$JOECKV      TEST FOR WARM START               R4
         BO    PPDBFLGS            BR IF YES                         R4
         TM    PPFLAG2,PPFDS       TEST FOR 1ST DATA SET             R4
         BO    PPDBFLGS            BR IF YES                         R4
         SPACE 1                                                     R4
         TM    PSMFDCI,SMFINTRC+SMFOPCC WARM-START/CARR. OVERRIDE... R4
         BNZ   PNEW6               SMF6 IF EITHER                    R4
         TM    PPFLAG2,PSMFDSER    DATA BUFFER ERROR...              R4
         BO    PNEW6               SMF6 IF YES                       R4
         SPACE 1                                                     R4
         CLC   DCTFORMS,PDBFORMS   FORMS MATCH...                    R4
         BNE   PNEW6               SMF6 IF NOT                       R4
         SPACE 1                                                     R4
         CLC   DCTFCB,PDBFCB       FCB MATCH...                      R4
         BE    PSMFTPUN            BR IF YES                        R41
         CLC   PDBFCB,=CL4'****'   TEST FOR DEFAULT                  R4
         BNE   PNEW6               SMF6 IF NOT                       R4
         TM    DCTPPSW,DCTPPSWB    IS DEVICE FCB STANDARD...         R4
         BNZ   PNEW6               SMF6 IF NOT                       R4
         SPACE 1                                                     R4
PSMFTPUN CLI   PDEVTYPE+3,UCB3525  TEST FOR 3525 PUNCH               R4
         BNE   PSMFTUCS            BR IF NOT                        R41
         TM    PDEVTYPE+1,X'03'    ARE PRINT FEATURE(S) AVAILABLE... R4
         BZ    PSMFTUCS            BR IF NOT                        R41
         LA    R1,PBFUNCI          COMPARE FUNC=I                    R4
         BAL   PL,PBITSAME          SPECIFICATIONS                   R4
         SPACE 1                                                    R41
PSMFTUCS CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER            R41
         BE    PSMFNIP             BR IF YES                        R41
         CLC   DCTUCS,PDBUCS       UCS MATCH...                     R41
         BE    PPDBFLGS            BR IF YES                        R41
         CLC   PDBUCS,=C'****'     TEST FOR DEFAULT                 R41
         BNE   PNEW6               SMF6 IF NOT                      R41
         TM    DCTPPSW,DCTPPSWU    IS DEVICE UCS STANDARD...   @OZ28771
         BNZ   PNEW6               SMF6 IF NOT                      R41
         B     PPDBFLGS            NEW SMF TYPE 6 NOT NEEDED        R41
         EJECT                                                      R41
PSMFNIP  CLC   =C'****',PDBCHAR1   IF CHAR1 NOT SPECIFIED,     @OZ31424
         BNE   *+10                 THEN UPDATE IN-STORAGE     @OZ31424
         MVC   PDBCHAR1,DCTUCS       PDDB WITH PRINTER DEFAULT @OZ31424
         LA    R0,4                CHECK 4 PDDB CHAR FIELDS    @OZ31424
         LA    R15,4                WITH 4 DCT  CHAR FIELDS    @OZ31424
         LA    R1,PDBCHAR1         R1 = POINTER TO PDDB CHARS  @OZ31424
         SPACE 1                                               @OZ31424
PSCHLUP1 LA    PL,DCTCHAR1         PL = POINTER TO DCT CHARS   @OZ31424
         SPACE 1                                               @OZ31424
PSCHLUP2 CLC   0(4,PL),0(R1)       DOES THIS                   @OZ31424
         BE    PSCHNEXT             PDDB CHAR                  @OZ31424
         LA    PL,4(,PL)             MATCH ANY                 @OZ31424
         BCT   R0,PSCHLUP2            DCT CHAR FIELD...        @OZ31424
         B     PNEW6               NEW SMF6 IF NOT             @OZ31424
         SPACE 1                                               @OZ31424
PSCHNEXT LA    R1,4(,R1)           LOOP, CHECKING              @OZ31424
         CLC   =C'****',0(R1)       ALL SPECIFIED              @OZ31424
         BE    *+8                   PDDB CHAR FIELDS.         @OZ31424
         BCT   R15,PSCHLUP1        FALL THROUGH IF ALL MATCH   @OZ31424
         CLC   DCTFLASH(2*4),PDBFLASH  FLASH, MODIFY MATCH...  @OZ31424
         BNE   PNEW6               SMF6 IF NOT                 @OZ31424
         CLC   PCOPYGRP,PDBCOPYG   COPY GROUPS MATCH...              R4
         BNE   PNEW6               SMF6 IF NOT                       R4
         CLC   PFLASHC,PDBFLSHC    FLASH COUNTS MATCH...             R4
         BNE   PNEW6               SMF6 IF NOT                       R4
         SPACE 1                                                     R4
         LA    R1,PBOPTJ           COMPARE OPTCD=J                   R4
         BAL   PL,PBITSAME          SPECIFICATIONS                   R4
         LA    R1,PBBURST          COMPARE BURSTER                   R4
         BAL   PL,PBITSAME          SPECIFICATIONS                   R4
         SPACE 1                                                     R4
         B     PPDBFLGS            BR IF NEW TYPE 6 NOT NEEDED       R4
         SPACE 5
PBFUNCI  TM    PDBFUNC,X'80'       FUNC=I        *** EXECUTE ***     R4
         TM    PPFLAG,PPFUNCI                    ***   ONLY  ***     R4
         SPACE 1                                                     R4
PBOPTJ   TM    PDBFLAG2,PDB2OPTJ   DCB=OPTCD=J   *** EXECUTE ***     R4
         TM    PPFLAG2,PPOPTJ                    ***   ONLY  ***     R4
         SPACE 1                                                     R4
PBBURST  TM    PDBFLAG2,PDB2BRST   BURST=Y/N     *** EXECUTE ***     R4
         TM    DCTPPSW2,DCTNIBRS                 ***   ONLY  ***     R4
         SPACE 2                                                     R4
*                                                                    R4
*        PBITSAME -- RETURNS TO CALLER IF FLAGS MATCH                R4
*                                                                    R4
         SPACE 1                                                     R4
PBITSAME EX    0,0(,R1)            TEST                              R4
         BZ    SKIP200              IF                               R4
         EX    0,4(,R1)              BOTH                            R4
         BZ    PNEW6                  FLAG                           R4
         BR    PL                      BITS                          R4
SKIP200  EX    0,4(,R1)                 MATCH                        R4
         BZR   PL                  RETURN -- BITS MATCH. ELSE,       R4
         EJECT
***********************************************************************
*                                                                     *
*        NEW TYPE 6 NEEDED -- CALL PPSMF6 AND RESET DATA              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNEW6    L     R15,=A(PPSMF6)      GENERATE NEW                @OZ32776
         BALR  PL,R15               TYPE-6 SMF RECORD          @OZ32776
         SPACE 1                                                     R4
        $TIME  ,                   GET CURRENT TIME/DATE             R4
         STM   R0,R1,PTIMEON       RESET PRPU START TIME/DATE        R4
         XC    PPLNCDCT,PPLNCDCT         CURRENT RECORD COUNT        R4
         XC    PRPAGECT,PRPAGECT         CURRENT PAGE COUNT          R4
         XC    PPJNDS,PPJNDS             CURRENT DATA SET COUNT      R4
         XC    PSMFDCI,PSMFDCI           DATA SET CONTROL INDICATORS R4
         NI    PPFLAG2,255-PSMFDSER      DATA BUFFER ERROR FLAG      R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SET FUNC=I AND OPTCD=J FLAGS                                 *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPDBFLGS NI    PPFLAG,255-PPFUNCI  SET 3525                          R4
         TM    PDBFUNC,X'80'        PUNCH-INTERPRET                  R4
         BZ    SKIP210               FLAG IF                         R4
         OI    PPFLAG,PPFUNCI         REQUESTED                      R4
SKIP210  NI    PPFLAG2,255-PPOPTJ  SET 3800                          R4
         TM    PDBFLAG2,PDB2OPTJ    DCB=OPTCD=J                      R4
         BZ    PDDBFCHK              FLAG IF                         R4
         OI    PPFLAG2,PPOPTJ         REQUESTED                      R4
         SPACE 1                                                     R4
         DROP  PC2                 SUSPEND PDDB ADDRESSABILITY       R4
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        SETUP DEVICE BY PDDB SPECIFICATIONS                          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PDDBFCHK DS    0H                                                    R4
         LA    R1,PMESSAGE         ADDRESS NEW SETUP SPECIFICATION
         L     R15,=A(PRPUDSV)     CALL DEVICE                       R4
         BALR  PL,R15               SETUP VERIFICATION               R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        OFFSET-STACK 3800 BURSTER DATA SETS                          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PPFLAG2,PPFDS       TEST FOR FIRST DATA SET           R4
         BO    PWARMTST            BR IF YES                         R4
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BNE   PWARMTST            BR IF NOT                         R4
         CLC   PPDSKEY,=AL2(IOTPDBOD/PDBLENG)  SYSTEM PDDB...       R41
         BNH   PWARMTST            SKIP OFFSET-STACK IF YES         R41
         LM    PC1,PC2,PCCWOFST    SELECT OFFSET-STACK CCW           R4
         BAL   PL,PPPUT            ADD TO CHAIN                      R4
         SPACE 1                                                     R4
PWARMTST DS    0H                                                    R4
         TM    PCKJOE,$JOECKV      IS THIS A DATASET WARM START
         BO    PFASTRT             BRANCH IF YES
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        RESTART POINT FOR DATA SET COPIES                            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNXTCPY  DS    0H
         IC    PL,PPJNDS           GET DATA SET COUNTER
         LA    PL,1(,PL)           INCREMENT BY ONE
         STC   PL,PPJNDS           STORE NEW COUNTER VALUE
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF FIRST DATA SET -- INITIALIZE CHECKPOINT-JOE               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCKINIT  DS    0H
         MVC   PCEEJRCB,PCCW+2     1ST RCB DISPLACEMENT
         XC    PDDBPGCT,PDDBPGCT   CLEAR DATASET PAGE COUNT
         TM    PPFLAG2,PPFDS       TEST FOR 1ST DATA SET             R4
         BZ    PFASTRT             BR IF NOT                         R4
         SPACE 1                                                     R4
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        INITIALIZE AND CHECKPOINT THE CKPT-JOE                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING JOEDSECT,PL         PROVIDE JOE ADDRESSABILITY        R4
         SPACE 1                                                     R4
         L     PL,PCKJOE           ADDRESS CKPT-JOE                  R4
         MVC   JOEJRCB(L'PCKPT),PCKPT INITIALIZE CHECKPOINT-JOE      R4
         SPACE 1                                                     R4
        $#CKPT JOE=0(,PL),TYPE=A   CKPT THE CKPT-JOE                 R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SHOW CKPT-JOE IS VALID -- CHECKPOINT THE WORK-JOE            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         L     PL,PWKJOE           ADDRESS WORK-JOE
         OI    JOEFLAG,$JOECKV     SET CKPT-JOE VALID FLAG
        $#CKPT JOE=0(,PL),TYPE=A   CKPT THE WORK-JOE                 R4
         SPACE 1                                                     R4
         DROP  PL                  SUSPEND JOE ADDRESSABILITY
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        RESET WARM START INDICATION IN WORK AREA                     *
*                                                                     *
*        MOVE A COPY OF THE CKPT-JOE TO THE PP BUFFER                 *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PFASTRT  DS    0H                                                    R4
         NI    PCKJOE,255-$JOECKV  RESET WARM START BIT              R4
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BO    PRMTPDIR            BR IF REMOTE                     R41
         LM    PC1,PC2,POUTCCWA         INITIALIZE                   R4
         AH    PC1,PCCWLAST              PPBUF                       R4
         AH    PC2,PCCWLAST               COPIES                     R4
         MVC   PCIESIZE(L'PCKPT,PC1),PCKPT OF                        R4
         MVC   PCIESIZE(L'PCKPT,PC2),PCKPT  CKPT-JOE                 R4
         B     PBSPINIT            BR TO CONTINUE                   R41
         EJECT                                                      R41
***********************************************************************
*                                                                     *
*        CALL PPDIR SUBROUTINE IF REMOTE SETUP HEADER IS NEEDED       *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
         USING DCTDSECT,PW         PROVIDE DCT ADDRESSABILITY       R41
         SPACE 1                                                    R41
PRMTPDIR L     PW,PCEDCT           ADDRESS OF REMOTE DCT       @OZ32566
         TM    MDCTFEAT,DCTPSHDR   NEED SETUP HEADER...             R41
         BZ    PBSPINIT            BR IF NO                         R41
         L     R15,=A(PPDIR)        ELSE CALL PDIR                  R41
         BALR  PL,R15                SUBROUTINE                     R41
         SPACE 1                                                    R41
         DROP  PW                  SUSPEND DCT ADDRESSABILITY       R41
         SPACE 2                                                     R4
***********************************************************************
*                                                                     *
*        INITIALIZE BACKSPACE TABLE                                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PBSPINIT DS    0H                                                    R4
         LH    R15,$BSPSIZ         IF NO BSP                         R4
         LTR   R15,R15              TABLE ENTRIES,                   R4
         BZ    PBSFSGO               BR TO PRIME THE BUFFERS         R4
         L     PW,PDDBPGCT         SET FIRST
         LA    PW,1(,PW)            BACKSPACE
         ST    PW,PBSPGCT            FRAME PAGE
         SH    R15,=H'7'           RESET                             R4
         LA    R15,PBSPTBL(R15)     LAST                             R4
         XC    0(7,R15),0(R15)       ENTRY                           R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- MAIN PROCESSOR'          R4
***********************************************************************
*                                                                     *
*        MAIN PRINT/PUNCH LOOP INITIALIZATION                         *
*                                                                     *
*        RESTART POINT FOR $F/$B PROCESSING                           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PBSFSGO  DS    0H                  $F/$B RESTART POINT
         LA    R1,PCKPT            INITIALIZE CHECKPOINT             R4
         ST    R1,PPCKPTR           DATA POINTER                     R4
         STCK  PCEFORM             SET START POINT FOR CKPT INTERVAL
         OI    PPFLAG2,PPCKPTA      AND ALLOW CHECKPOINTS            R4
         MVI   PUNLINE,X'FD'       SET FOR HIGHEST PRINT LINE CCW
         MVC   PRTRCCW,PXTABCCW    SHOW CHAR1 SELECTED         @OZ24675
         SPACE 1                                                    R41
PNEWSGO  XC    PPLC,PPLC           CLEAR PAGE LINE COUNTER          R41
         EJECT ,                                               @OZ27703
************************************************************** @OZ27703
* INITIALIZE PBUFSKIP TO PROPER VALUE IF THIS IS TRACKCELLED * @OZ27703
* PROCESSOR.  THIS IS DONE BY CALCULATING ALL RECORDS IN A   * @OZ27703
* TRACK CELL IN TURN AND CHECKING FOR A MATCH ON THE INPUT   * @OZ27703
* MTTR VALUE.  WHEN THIS MATCH OCCURS, THE PBUFSKIP VALUE    * @OZ27703
* WILL BE SET TO THE CORRECT VALUE.                          * @OZ27703
*                                                            * @OZ27703
* REGISTER USAGE - - - - - - - -                             * @OZ27703
* R15 - - WORK AND COUNTER FOR RECORDS IN THE CELL           * @OZ27703
* R14 - - SUB PERMUTATION NUMBER                             * @OZ27703
* PL  - - WORKING R VALUE                                    * @OZ27703
* R0  - - RECORDS PER TRACK THIS SPOOL DEVICE                * @OZ27703
* R1  - - RECORD INCREMENT VALUE                             * @OZ27703
************************************************************** @OZ27703
         SPACE 2                                               @OZ27703
         TM    PPFLAG2,PPTCEL      IF NOT TRACKCELL PROCESSOR  @OZ27703
         BZ    PGO100               DON'T CALCULATE PBUFSKIP   @OZ27703
         SPACE 1                                               @OZ27703
*              LOOP INITIALIZATION                             @OZ27703
         XR    R15,R15             CLEAR WORK REG TO ZERO      @OZ27703
         LR    R1,R15              CLEAR RECINCR REG TO ZERO   @OZ27703
         LA    PL,1                SET INITIAL R TO 1          @OZ27703
         LR    R14,PL              SET INITIAL SPN TO 1        @OZ27703
         IC    R15,PCEJMTTR        FROM M VALUE                @OZ27703
         MH    R15,=AL2(TEDSIZ)     POINT TO                   @OZ27703
         AL    R15,$TEDADDR          PROPER TED                @OZ27703
         LH    R0,TNRT-TEDDSECT(,R15) GET RECORDS PER TRACK    @OZ27703
         LR    R15,R1              CLEAR COUNT TO ZERO         @OZ27703
         IC    R1,$RECINCR         GET RECORD INCREMENT VALUE  @OZ27703
         SPACE 1                                               @OZ27703
PGO001   CLM   PL,1,PCEJMTTR+3     DOES R MATCH                @OZ27703
         BE    PGO099              BR IF YES, EXIT             @OZ27703
         SPACE 1                                               @OZ27703
         LA    R15,1(,R15)         BUMP COUNT BY 1             @OZ27703
         CLM   R15,1,$TCELSIZ      IS FULL CELL ACCOUNTED FOR  @OZ27703
         BL    PGO002              BR IF NO, KEEP ON COUNTING  @OZ27703
         XR    R15,R15             RESET COUNT TO ZERO         @OZ27703
PGO002   AR    PL,R1               BUMP R BY RECINCR           @OZ27703
         CR    PL,R0               CHECK IF R FITS ON TRACK    @OZ27703
         BNH   PGO001              BR IF YES, LOOP BACK        @OZ27703
         SPACE 1                                               @OZ27703
         LA    R14,1(,R14)         BUMP SPN                    @OZ27703
         LR    PL,R14              AND USE AS NEW R            @OZ27703
         B     PGO001              GO CHECK R                  @OZ27703
         SPACE 1                                               @OZ27703
PGO099   STC   R15,PBUFSKIP        SET SKIP VALUE              @OZ27703
PGO100   DS    0H                  RESUME                      @OZ27703
         EJECT ,                                               @OZ27703
***********************************************************************
*                                                                     *
*        DE-SPOOL INITIAL BUFFER OR TRACK-CELL                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         L     R15,PCEJMTTR        1ST TRACK OF DATA SET             R4
         IC    R1,PBUFOPT          INITIALIZE AVAILABLE INPUT        R4
         STC   R1,PBFAVAIL          BUFFER COUNT TO BUFFERING OPTION R4
         BAL   PL,PRDTCEL          READ FIRST DATA BLOCK(S)          R4
         SPACE 1                                                     R4
         TM    PPFLAG,PPDELSW      TEST FOR DELETION                 R4
         BO    PPDSEND             BR IF YES                         R4
         BAL   PL,PRDTCHK          ELSE, WAIT FOR I/O TO COMPLETE    R4
         B     P1STBLK              AND GO PROCESS 1ST DATA BUFFER   R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        WAIT FOR SPOOL INPUT TO COMPLETE                             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCHKNBLK DS    0H                                                    R4
         TM    PPFLAG,PPDELSW      TEST FOR SUSPENSION/TERMINATION   R4
         BO    PPDSEND             BR IF YES                         R4
         BAL   PL,PRDTCHK           ELSE, CHECK INPUT I/O            R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PROCESS NEW DATA SET BUFFER                                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNXTBLK  DS    0H
         MVC   PCEEJRCB,PCCW+2     SET 1ST RCB DISPLACEMENT
         SPACE 1                                                     R4
P1STBLK  DS    0H                                                    R4
         TM    PPFLAG2,PPTCEL      TEST DE-SPOOLING METHOD           R4
         BZ    SKIP220             BRANCH IF SINGLE                  R4
         OC    PPFLAG,BUFECBCC     COPY STATUS OF NEW BUFFER         R4
         B     PVALCHK              AND GO CHECK IT                  R4
SKIP220  CLC   HDBKEY,PPKEY        IS THIS DATA BUFFER VALID         R4
         BE    PVALCHK             BR IF YES                         R4
         OI    PPFLAG,PPRDERR      SET DATA BUFFER VALIDITY ERROR    R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        EXIT IF BUFFER READ/VALIDITY ERROR OR SUSPENSION             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PVALCHK  DS    0H                                                    R4
         TM    PPFLAG,PPRDERR+PPDELSW SUSPEND OR READ ERROR
         BNZ   PPDSEND             BRANCH IF YES
         LM    PC1,PC2,PCCW        SETUP CCW IN PC1 AND PC2
         ICM   PC1,3,PCEEJRCB      GET RCB DISPLACEMENT
         ALR   PC1,PBUF            ADJUST FOR THIS BUFFER
         LH    R1,PCEEJRCB         GET CURRENT RCB DISPL
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        PROCESS NEW LOGICAL RECORD -- TEST FOR END-OF-BLOCK          *
*                                                                     *
*        IF DATA BUFFER IS AVAILABLE -- BEGIN TO DE-SPOOL INTO IT     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNXTCCW  DS    0H
         STH   R1,PCEEJRCB         SAVE EJECT RCB DISPL
         CLI   PBFAVAIL,0          TEST FOR AVAILABLE INPUT BUFFER   R4
         BE    PALLACTV            SKIP IF ALL ARE ACTIVE            R4
        $COUNT R=PW                ********************************* R4
         BAL   PL,PRDTCNXT         BEGIN INPUT OF NEXT DATA BLOCK(S) R4
PALLACTV DS    0H                                                    R4
         TM    0(PC1),X'FF'        TEST RECORD CONTROL BYTE
         BO    PCPEND              BRANCH IF END OF BLOCK
         EJECT                                                       R4
         ICM   PC2,2,=X'00'        ENSURE LENGTH RESET               R4
         IC    PC2,0(,PC1)         SET TEXT LENGTH                   R4
         LR    PW,PC1              COPY LRC ADDRESS                  R4
         SPACE 1                                                     R4
         USING LRCDSECT,PW         PROVIDE LRC ADDRESSABILITY        R4
         SPACE 1                                                     R4
         TM    LRCFLAG1,LRC1SPAN   TEST FOR SPANNED RECORD           R4
         BZ    PNOSPAN             BR IF NO                          R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SPANNED RECORD -- PRINT/PUNCH ONLY FIRST SEGMENT             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         ICM   PC2,3,LRCSEGL       GET SEGMENT LENGTH                R4
         LA    PC1,LRCSTEXT        ASSUME NOT 1ST SEGMENT            R4
         TM    LRCFLAG1,LRC1SBGN   TEST ASSUMPTION                   R4
         BZ    PNXTRCB             BR IF VALID TO SKIP SEGMENT       R4
         LA    PC1,LRCSFTXT        POINT TO START OF 1ST-SEG TEXT    R4
         STM   PC1,PC2,PCCWORK     SAVE CCW FOR ANALYSIS             R4
         B     PFSPCK               AND BR TO ANALYZE                R4
         SPACE 1                                                     R4
         DROP  PW                  KILL LRC ADDRESSABILITY           R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        PICK UP USER CARRIAGE CONTROL (IF ANY) -- TEST FOR ASA       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNOSPAN  DS    0H                  *                                 R4
         LA    PC1,3(,PC1)         TEXT ADDRESS (NO CC)
         TM    1(PW),LRC1CCTL      IS USER CC SPECIFIED
         BNO   *+8                 <*** BRANCH IF NO
         LA    PC1,1(,PC1)            * STEP OVER CC
         STM   PC1,PC2,PCCWORK        * STORE CCW FOR ANALYSIS
         BNO   *+10                <*** BRANCH IF NO CC
         MVC   PCCWORK(1),3(PW)    MOVE IN USER CC
         TM    1(PW),LRC1ONUL      IS NOT SYSOUT BIT SET
         BO    PNXTRCB             BRANCH IF YES
         TM    1(PW),LRC1CCTL+LRC1TASA IS CC ASA
         BNO   PFSPCK              BRANCH IF NO - START ANALYSIS
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        CONVERT ASA CARRIAGE CONTROL TO PROPER IMMEDIATE CCW         *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PASA001  LA    PW,PASANUM          NUMBER OF ASA TABLE ENTRIES @OZ32776
         CLI   PDEVTYPE+3,X'0C'    IS DEVICE A 3525 PUNCH...   @OZ28353
         BE    PASA005             BR. YES...USE 3525 TABLE    @OZ28353
         L     PC2,=A(PASAMCH)     ASA TO MCH CONVERSION TABLE @OZ32776
         SPACE 1                                               @OZ32776
PASA002  DS    0H                  *
         CLC   PCCWORK(1),0(PC2)   DOES USER CC MATCH ASA ENTRY
         BE    PASA004             BRANCH IF YES                     R4
         LA    PC2,2(,PC2)         STEP TO NEXT TABLE ENTRY
         BCT   PW,PASA002          CYCLE THROUGH TABLE
         L     PC2,PCCWORK+4       RESTORE 2ND HALF OF CCW           R4
         OC    PCCWORK+6(2),PCCWORK+6 CC NOT FOUND -- CHECK DATA LEN R4
         BZ    PNXTRCB             IGNORE REC IF BAD CC & NO DATA    R4
         TM    PCEID,PCEPRSID      CHECK PROCESSOR TYPE              R4
         BO    PASA003             BRANCH IF PRINT                   R4
         CLI   PDEVTYPE+3,X'0C'    IS DEVICE A 3525 PUNCH...   @OZ33253
         BE    PASA006             BRANCH IF YES               @OZ33253
         MVI   PCCWORK,X'41'       DEFAULT PUNCH CC = STACKER 1      R4
         B     PNIMDCMD            BRANCH TO COUNT RECORDS           R4
PASA005  DS    0H                  *                           @OZ28353
         L     PC2,=A(PASAMCH2)    3525 ASA TO MCH TABLE       @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
         B     PASA002             CONTINUE CONVERSION         @OZ28353
PASA006  DS    0H                                              @OZ33253
         MVI   PCCWORK,X'01'       DEFAULT 3525 CC = STACK 1   @OZ33253
         B     PNIMDCMD            BRANCH TO COUNT RECORDS     @OZ33253
PASA003  DS    0H                  *                                 R4
         L     PC2,=A(PASAMCH)     DEFAULT TO SINGLE SPACE     @OZ32776
PASA004  DS    0H                  *                                 R4
         MVC   PCCWORK(1),1(PC2)   SELECT MCH CC VALUE
         L     PC2,PCCWORK+4       RESTORE SECOND PART OF CCW
         TM    PCEID,PCEPUSID      CHECK PROCESSOR TYPE              R4
         BO    PNIMDCMD            NO ASA IMMEDIATE CMDS FOR PUNCH   R4
PFSPCK   DS    0H                  *
         TM    PCCWORK,X'02'       TEST COMMAND TYPE
         BZ    PNIMDCMD            BRANCH IF NOT IMMEDIATE
         L     PC2,PCCW+4          SET COUNT TO ZERO
         B     PNOCOUNT            CONTINUE ANALYSIS
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        UPDATE TOTAL LINE OR CARD COUNT FOR THIS JOE                 *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNIMDCMD DS    0H                  CMND IS NOT IMMEDIATE
         CLC   PDDBSKIP,PDDBPGCT   SKIP COUNT VS PAGE COUNT
         BH    PNOCOUNT            DON'T COUNT IF HIGH
         L     PW,PPLNCDCT         UPDATE
         LA    PW,1(,PW)            LINE OR CARD
         ST    PW,PPLNCDCT           COUNT
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF PUNCH -- UPDATE DATA SET CARD COUNT                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNOCOUNT DS    0H                  *
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE
         BO    PRINT               BRANCH IF PRINT
         CLI   PDEVTYPE+3,X'0C'    IS DEVICE A 3525...         @OZ28353
         BNE   PUSTKR1             BR. NO...DONT TEST          @OZ28353
         CLI   PCCWORK,X'01'       IS CC STACKER ONE...        @OZ28353
         BE    PUSTKR2             BR. YES...CONTINUE TESTS    @OZ28353
PUSTKR1  DS    0H                  *                           @OZ28353
         CLI   PCCWORK,X'41'       IS CC PUNCH STACKER 2
         BE    *+12                BRANCH IF YES
         CLI   PCCWORK,X'81'       IS CC PUNCH STACKER 3
         BNE   PUNPRT              BRANCH IF NO
PUSTKR2  DS    0H                  *                           @OZ28353
         MVI   PUNLINE,X'05'       RESET PUNCH-PRINT LINE NO.
         TM    PPFLAG,PPDELSW      CHECK FOR SUSPENSION        @OZ29138
         BO    PPDSEND             BRANCH IF YES               @OZ29138
         L     PW,PDDBPGCT         UPDATE
         LA    PW,1(,PW)            PUNCH
         ST    PW,PDDBPGCT           CARD COUNT
         L     PW,PPLC             UPDATE                            R4
         LA    PW,1(,PW)            LOGICAL                          R4
         ST    PW,PPLC               PAGE COUNT                      R4
         B     PUPDTBSP            GO UPDATE BSP TABLE               R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        CC IS NOT A PUNCH COMMAND - CHECK FOR 3525 PRINT             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PUNPRT   DS    0H
         TM    PCCWORK,X'05'       IS CC A 3525 PRINT-LINE
         BNO   PUBADCC             BRANCH IF NO
         CLI   PDEVTYPE+3,X'0C'    IS DEVICE A 3525
         BNE   PUBADCC             BRANCH IF NO
         TM    PPFLAG,PPFUNCI      WAS INTERPRET REQUESTED
         BO    PRNOPRNT            BRANCH IF YES - SKIP PRINT CCW
         TM    PDEVTYPE+1,X'30'    IS EITHER PRINT FEATURE IN
         BZ    PUBADCC             BRANCH IF NO
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        CHECK FOR VALID 3525 PRINT-LINE COMMAND                      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         CLC   PCCWORK(1),PUNLINE  IS LINE NUMBER IN SEQUENCE
         BNH   PUBADCC             BRANCH IF NO
         SLR   PW,PW               CLEAR REG PW
         IC    PW,PCCWORK          GET PRINT COMMAND
         SRL   PW,3                CONVERT TO PRINT LINE NUMBER
         CLM   PW,1,=X'03'         DOES LINE REQUIRE ML PRINT
         BNH   *+12                BRANCH IF NO
         TM    PDEVTYPE+1,X'10'    IS ML PRINT FEATURE INSTALLED
         BZ    PUBADCC             BRANCH IF NO
         MVC   PUNLINE,PCCWORK     SET NEW LINE NUMBER
         B     PUPDTBSP            GO UPDATE BSP TABLE               R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        FOR NO USER CC, BAD USER CC, BAD PRINT LINE - USE CC=41      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PUBADCC  DS    0H                  *
         MVI   PCCWORK,X'41'       SET CC PUNCH STACKER 2
         B     PNOCOUNT            UPDATE CARD COUNTERS
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        CHECK VALIDITY OF PRINTER CCW COMMAND                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRINT    DS    0H
         TM    PDCTFLAG,DCTSPACE   TEST FOR FORCE SINGLE SPACE
         BZ    PRNSPACE            BRANCH IF NOT SINGLE SPACING
         TM    PSMFDCI,SMFOPCC     TEST CARRIAGE OVERRIDE      @OZ20006
         BO    PRLOOP              BR. YES...NOT FIRST TIME    @OZ20006
         LM    PC1,PC2,PCCWORK     LOAD CURRENT COMMAND CCW    @OZ20006
         STM   PC1,PC2,PMESSAGE    SAVE CURRENT COMMAND CCW    @OZ20006
         LM    PC1,PC2,PRCCWSP1    LOAD IMMEDIATE SPACE CCW    @OZ20006
         BAL   PL,PPPUT            ADD CCW TO CHAIN            @OZ20006
         LM    PC1,PC2,PMESSAGE    RELOAD CURRENT COMMAND CCW  @OZ20006
         STM   PC1,PC2,PCCWORK     RESTORE CURRENT COMMAND CCW @OZ20006
PRLOOP   DS    0H                                              @OZ20006
         OI    PSMFDCI,SMFOPCC     SET OPERATOR CARRIAGE OVERRIDE
         TM    PCCWORK,X'02'       TEST FOR IMMEDIATE COMMAND
         BO    PRNOPRNT            BRANCH IF YES
         IC    PL,PDCTFLAG         GET SPACE CONTROL BITS
         SLL   PL,3                POSITION FOR WRITE/SPACE
         STC   PL,PCCWORK          STORE INTO CCW
         OI    PCCWORK,X'01'       SET WRITE BIT
         NI    PCCWORK,X'19'       TURN OFF ALL EXTRAS
         B     PRGOODCC            CONTINUE PROCESSING
         SPACE 1                                                     R4
PRNSPACE TM    PCCWORK,X'01'       TEST LOW ORDER BIT OF COMMAND
         BZ    PRBADCC             BIT MUST BE ONE OR COMMAND IS BAD
         TM    PCCWORK,X'84'       TEST BITS 0 AND 5
         BM    PRSKIP              BRANCH IF EJECT OR INVALID
         TM    PCCWORK,X'64'       NO, TEST BITS 1, 2, AND 5
         BZ    PRGOODCC            BITS MUST BE ZERO OR COMMAND IS BAD
         CLI   PCCWORK,X'73'       TEST FOR BLOCK DATA CHECK COMMAND
         BE    PRNOPRNT            IGNORE IF BLOCK DATA CHECK
         SPACE 1                                                     R4
PRBADCC  NI    PCCWORK,X'02'       BAD COMMAND
         OI    PCCWORK,X'09'       CONVERT TO SINGLE SPACE
         LH    PW,PCEEJRCB         GET LRC OFFSET IN BUFFER
         ALR   PW,PBUF             ADD BUFFER ORIGIN
         NI    1(PW),255-LRC1TASA  RESET ASA BIT
         SPACE 1                                                     R4
PRGOODCC SLR   PL,PL               CLEAR REGISTER
         IC    PL,PCCWORK          PICK UP COMMAND
         SRL   PL,3                SHIFT OUT LOW-ORDER BITS
         AL    PL,PPLC             INCREMENT LINE                    R4
         ST    PL,PPLC              COUNT BY SPACE VALUE             R4
         CL    PL,PRLINECT         COMPARE LINE COUNT WITH MAXIMUM   R4
         BNH   PRNOVFL             BRANCH IF LESS
         NI    PCCWORK,X'02'       CONVERT COMMAND
         OI    PCCWORK,X'89'       TO EJECT
         SPACE 1                                                     R4
PRSKIP   DS    0H
         CLI   PCCWORK,X'89'       TEST COMMAND VALIDITY
         BL    PRBADCC             BRANCH IF INVALID (LOW)
         BH    PRN8940             BRANCH IF NOT EJECT
         LH    PW,PCEEJRCB         GET RCB OFFSET
         ALR   PW,PBUF             ADD BUFFER ORIGIN
         CLC   0(2,PW),=X'01A0'    CHECK FOR PRINT
         BNE   PRN8940             BRANCH IF NO
         CLC   3(2,PW),=X'8940'     AND SKIP TO CHNL 1
         BNE   PRN8940             BRANCH IF NO
         OI    PCCWORK,X'02'       CONVERT TO 8B
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        UPDATE DATA SET PAGE COUNT                                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRN8940  DS    0H
         CLI   PCCWORK,X'E3'       TEST COMMAND VALIDITY
         BH    PRBADCC             BRANCH IF INVALID (HIGH)
         XC    PPLC,PPLC           CLEAR PAGE LINE COUNTER           R4
         TM    PPFLAG,PPDELSW      CHECK FOR SUSPENSION        @OZ29138
         BO    PPDSEND             BRANCH IF YES               @OZ29138
         L     PW,PDDBPGCT         UPDATE
         LA    PW,1(,PW)            DATASET
         ST    PW,PDDBPGCT           PAGE COUNT
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        PUSH DOWN BACKSPACE TABLE -- ADD NEW ENTRY                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PUPDTBSP DS    0H                                                    R4
         LH    R15,$BSPSIZ         IF NO BSP                         R4
         LTR   R15,R15              TABLE ENTRIES,                   R4
         BZ    PSKPBSP               BR TO CHECK SKIP COUNT          R4
         CLC   PDDBPGCT,PBSPGCT    IS THIS A FRAME PAGE
         BNE   PSKPBSP             BRANCH IF NO
         SLR   PW,PW               STEP TO                           R4
         IC    PW,$BSPGCT           NEXT                             R4
         AL    PW,PBSPGCT            PAGE FRAME                      R4
         ST    PW,PBSPGCT          SAVE FOR LATER
         LA    PW,PBSPTBL          POINT TO BACKSPACE TABLE          R4
         SH    R15,=H'8'           TABLE LENGTH - 1, LESS 1 ENTRY    R4
         BM    SKIP230             BR IF 1-ENTRY TABLE               R4
         EX    R15,PPBSPUSH         ELSE PUSH DOWN TABLE ENTRIES     R4
SKIP230  ALR   PW,R15              POINT TO LAST ENTRY - 1           R4
         MVC   1(4,PW),PCEJMTTR    SAVE BUFFER MTTR                  R4
         MVC   5(2,PW),PCEEJRCB    SAVE LINE RCB                     R4
         MVC   7(1,PW),PCEJBOFF    SAVE BUFFER OFFSET                R4
         B     PSKPBSP             THEN BR TO CHECK SKIP COUNT       R4
         SPACE 1                                                     R4
PPBSPUSH MVC   0(*-*,PW),7(PW)     *** EXECUTE ONLY ***              R4
         SPACE 1                                                     R4
PSKPBSP  CLC   PDDBSKIP,PDDBPGCT   COMPARE SKIP COUNT WITH PG COUNT  R4
         BH    PRNOPRNT            DON'T UPDATE IF HIGH
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE
         BO    PPCHKPT             BRANCH IF PRINT
         L     R1,PCEDCT           GET ADDR OF RMT PRT/PCH DCT @OZ32566
         NI    DCTPPFL-DCTDSECT(R1),255-DCTEJECT CLR PCH LOGCL EJECT R4
         L     PW,PPLC             TEST FOR END OF                   R4
         CL    PW,PRLINECT          PUNCH LOGICAL PAGE               R4
         BNH   PRZCL               BRANCH IF NO
         XC    PPLC,PPLC           CLEAR PUNCH LOGICAL PAGE COUNT    R4
         OI    DCTPPFL-DCTDSECT(R1),DCTEJECT SET PUNCH LOGICAL EJECT R4
         B     PPCHKP1             BRANCH AROUND PRT TESTS     @OZ33145
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        NEW PAGE -- UPDATE TOTAL PAGE COUNT FOR THIS JOE             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPCHKPT  CLI   PCCWORK,X'8B'       TEST FOR SKIP IMMED CMD     @OZ33145
         BNE   PPCHKP1             IF NO-VALID EJECT SO COUNT  @OZ33145
         L     R1,PCEDCT           PICK UP DCT ADDRESS         @OZ33145
         TM    DCTPPFL-DCTDSECT(R1),DCTEJECT  CHECK IF CHAN 1  @OZ33145
         BO    PRMTCHKT            IF CHAN 1, DON'T UPDATE     @OZ33145
PPCHKP1  L     PW,PRPAGECT         UPDATE                      @OZ33145
         LA    PW,1(,PW)            TOTAL                            R4
         ST    PW,PRPAGECT           PAGE COUNT                      R4
*                                  THIS LINE DELETED BY APAR   @OZ29138
*                                  THIS LINE DELETED BY APAR   @OZ29138
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        FOR REMOTES -- UPDATE CKPT-JOE AT EACH NEW PAGE              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRMTCHKT TM    PCEID,PCERJEID      TEST PROCESSOR TYPE         @OZ33145
         BNO   PLCLCKPT            BRANCH IF NOT REMOTE              R4
         LA    R1,PCKPT            POINT TO CHECKPOINT DATA          R4
         ST    R1,PPCKPTR          SAVE FOR PPPCKPT                  R4
         BAL   PL,PPPCKPT1         SCHEDULE CKPT-JOE FOR CHECKPOINT  R4
         B     PRNOVFL             SKIP LOCAL CHECKPOINT             R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        FOR LOCALS -- UPDATE COPY OF CKPT-JOE AT EACH NEW PAGE       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PLCLCKPT DS    0H                                                    R4
         L     R1,POUTCCWA         GET POINTER                       R4
         AH    R1,PCCWLAST          TO CKPT-JOE MINUS PCIESIZE       R4
         OI    PCISGNAL-PCIDSECT(R1),PCICKPT  SHOW NEW CKPT PRESENT  R4
         MVC   PCIESIZE(L'PCKPT,R1),PCKPT UPDATE COPY OF CKPT-JOE    R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IF &PRTRANS=YES -- TRANSLATE ALL UNPRINTABLES TO BLANKS      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRNOVFL  CLC   PDDBSKIP,PDDBPGCT   COMPARE SKIP COUNT WITH PAGE COUNT
         BH    PRNOPRNT            DON'T PRINT IF HIGH
         TM    $PRTOPTS,$PRTRANS   TEST FOR PRINT TRANSLATE          R4
         BZ    PRZCL               BR IF NO                          R4
         TM    PCEID,PCEPUSID      TEST PROCESSOR TYPE
         BO    PRZCL               BRANCH IF PUNCH
         CLC   PDEVTYPE+2(2),=AL1(UCB3UREC,UCB3211)  LOCAL 3211...   R4
         BE    PRZCL               BRANCH IF YES
         CLC   PDEVTYPE+2(2),=AL1(UCB3UREC,UCB3800)  3800 PRINTER... R4
         BE    PRZCL               BRANCH IF YES                     R4
         CLM   PC2,1,=X'00'        IS THE COUNT ZERO
         BE    PRZCL               BRANCH IF YES - SKIP TRANSLATE
         LA    R1,0(PC1,PC2)       R1 = ADDRESS OF NEXT RCB
         SLR   R1,PBUF             GENERATE OFFSET OF RCB
         CH    R1,$BUFLENG         TEST FOR END OF BUFFER            R4
         BNL   PRZCL               BRANCH IF OUT OF BUFFER
         LA    R1,X'FF'(PC2)       R1 = COMMAND BYTE COUNT - 1
         L     R15,=A(PTRTBL)      TRANSLATE UNPRINTABLES      @OZ32776
         EX    R1,PRXTR             TO BLANKS                  @OZ32776
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IF OPTCD=J -- SELECT APPROPRIATE 3800 TRANSLATE TABLE        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRZCL    L     PC1,PCCWORK         PICK UP MODIFIED COMMAND          R4
         TM    PPFLAG2,PPOPTJ      TEST FOR OPTCD=J SPECIFIED        R4
         BZ    PRCHAIN             BRANCH IF NO                      R4
         CL    PC2,PCCW+4          BYPASS IF FIRST PASS FOR         R41
         BE    PRCHAIN              ASA CC (OR INVALID)             R41
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER            R41
         BNE   PRZCL2              BR IF OPTCD=J NOT SUPPORTED      R41
         NI    0(PC1),X'0F'        USE BINARY PART OF TRC            R4
         SLR   R1,R1               PICK-UP                          R41
         CLC   PRMAXTRC,0(PC1)      TABLE REF CHAR                  R41
         BL    PRCHKTRC              OR USE ZERO                    R41
         IC    R1,0(PC1)              IF INVALID                    R41
         SPACE 1                                                    R41
PRCHKTRC LA    R1,PXTABCCW(R1)     ADDR OF SELECT CCW OP CODE       R41
         CLC   PRTRCCW,0(R1)       TABLE ALREADY SELECTED...        R41
         BE    PRZCL2              SKIP UNNECESSARY CCW IF YES      R41
         STM   PC1,PC2,PBLKWORK    SAVE 'DATA' CCW                  R41
         LM    PC1,PC2,PRCCWEJ     CONSTRUCT SELECT-                R41
         ICM   PC1,8,0(R1)          TRANSLATE-TABLE CCW             R41
         BAL   PL,PPPUT            ADD CCW TO CHAIN                 R41
         STCM  PC1,8,PRTRCCW       SAVE NEW SELECT CCW OP CODE      R41
         LM    PC1,PC2,PBLKWORK    RESTORE 'DATA' CCW               R41
         SPACE 1                                                    R41
PRZCL2   AL    PC1,=F'1'           ACCOUNT FOR TABLE ID              R4
         SL    PC2,=F'1'           ACCOUNT FOR TABLE ID              R4
         SPACE 1                                                    R41
*              THIS LINE DELETED BY APAR NUMBER                @OZ25297
*              THIS LINE DELETED BY APAR NUMBER                @OZ25297
*              THIS LINE DELETED BY APAR NUMBER                @OZ25297
*              THIS LINE DELETED BY APAR NUMBER                @OZ25297
*              THIS LINE DELETED BY APAR NUMBER                @OZ25297
*              THIS LINE DELETED BY APAR NUMBER                @OZ25297
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        ADD NEW PRINT/PUNCH COMMAND TO CCW CHAIN                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRCHAIN  DS    0H                                                    R4
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IF FUNC=I -- PRODUCE APPROPRIATE 3525 PRINT-LINE CCW'S       *
*                                                                     *
*        LINE 1 -- CARD COLS.  1-64 -- STARTING IN PRINT POSITION 1   *
*        LINE 3 -- CARD COLS. 65-80 -- STARTING IN PRINT POSITION 49  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PPFLAG,PPFUNCI      INTERPRET REQUESTED
         BNO   PRNOPRNT            BRANCH IF NO
         CLI   PDEVTYPE+3,X'0C'    IS DEVICE A 3525
         BNE   PRNOPRNT            BRANCH IF NO
         TM    PDEVTYPE+1,X'30'    IS EITHER PRINT FEATURE IN
         BZ    PRNOPRNT            BRANCH IF NO
         SPACE 1                                                     R4
         OI    PSMFDCI,SMFPUPRT    SET 3525 INTERPRET SMF FLAG       R4
         SPACE 1                                                     R4
         STM   PC1,PC2,PCCWORK     SAVE ORIGINAL PUNCH CCW
         ICM   PC1,8,=X'0D'        CONVERT CCW TO PRINT LINE 1
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         CLC   PCCWORK+6(2),=H'64' IS TEXT OVER 64 CHARACTERS...     R4
         BNH   PRNOPRNT            BR IF NOT                         R4
         L     PL,PCCWPT           PICK UP CURRENT CCW POINTER
         LA    PL,2*8(,PL)         VALUE AFTER 2 CCW'S ADDED         R4
         SL    PL,POUTCCWA         CONVERT TO CCW CHAIN OFFSET       R4
         CH    PL,PCCWLAST         IS THERE ROOM IN CHAIN            R4
         BL    *+8                 BRANCH IF YES
         BAL   PL,PPWRITE          FORCE CCW CHAIN TO BE CLEARED
         LM    PC1,PC2,PUSPACCW    LOAD DATA CHAIN PRINT LINE 3
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         LM    PC1,PC2,PCCWORK     RESTORE ORIGINAL PUNCH CCW
         LA    PC1,0(,PC1)         CLEAR CCW COMMAND
         SH    PC2,=H'64'          COMPUTE RESIDUAL COUNT OVER 64
         AH    PC1,=H'64'          COMPUTE NEW STARTING ADDRESS
         ICM   PC1,8,=X'1D'        CONVERT CCW TO PRINT LINE 3
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         LM    PC1,PC2,PCCWORK     RESTORE ORIGINAL PUNCH CCW
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        RE-CYCLE RECORD IF CARRIAGE CONTROL WAS ASA                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRNOPRNT DS    0H                                                    R4
         TM    PCEID,PCEPUSID      IS THIS A PUNCH PROCESSOR
         BO    PNXTRCB             BRANCH IF YES
         LH    PW,PCEEJRCB         GET LRC DISPLACEMENT
         ALR   PW,PBUF             ADD BUFFER ORIGIN
         MVI   PCCWORK,X'01'       SET CC TO WRITE NO-SPACE
         LM    PC1,PC2,PCCWORK     RESTORE CCW REGISTERS
         TM    1(PW),LRC1CCTL+LRC1TASA ASA CARRIAGE CONTROL
         BNO   PNXTRCB             BRANCH IF NO
         NI    1(PW),255-LRC1TASA  RESET ASA BIT
         B     PFSPCK              START CCW ANALYSIS
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        STEP TO NEXT LOGICAL RECORD CONTROL BLOCK                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNXTRCB  LA    PC1,0(PC1,PC2)      TEXT ADDRESS + TEXT LENGTH        R4
         LR    R1,PC1              COPY NEXT RCB ADDRESS
         SLR   R1,PBUF             GENERATE OFFSET OF RCB
         CH    R1,$BUFLENG         TEST FOR END OF BUFFER            R4
         BL    PNXTCCW             BRANCH IF RCB IS IN BUFFER
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        END OF DATA BUFFER                                           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCPEND   ICM   R15,15,HDBNXTRK     LOAD AND TEST CHAIN TRACK         R4
         BZ    PPDSEND             BRANCH IF END OF DATA SET         R4
         ST    R15,PCEJMTTR        SAVE CHAIN TRACK FOR CKPT         R4
         TM    PPFLAG2,PPTCEL      TEST DE-SPOOLING METHOD           R4
         BZ    PENDTCEL             BRANCH IF SINGLE                 R4
         SLR   PL,PL               INCREMENT                         R4
         IC    PL,PCEJBOFF          BUFFER OFFSET                    R4
         LA    PL,1(,PL)             WITHIN THIS                     R4
         STC   PL,PCEJBOFF            TRACK-CELL                     R4
         L     R1,PBUFADDR         ADDR OF FIRST BUFFER IN CHAIN     R4
         CH    PL,BUFCHNCT-BUFDSECT(,R1) TEST FOR END OF TRACK-CELL  R4
         BNL   PENDTCEL            BRANCH IF YES                     R4
         L     PBUF,BUFCHAIN       ESTABLISH ADDRESSABILITY ON       R4
         B     PNXTBLK              NEXT BUFFER AND GO PROCESS IT    R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PROCESSING COMPLETE FOR CURRENT BUFFER OR TRACK-CELL         *
*                                                                     *
*        SETUP TO DE-SPOOL NEXT DATA BUFFER(S)                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PENDTCEL DS    0H                                                    R4
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BO    SKIP250             BR IF REMOTE                      R4
         CLC   PDDBSKIP,PDDBPGCT   COMPARE SKIP AND PAGE COUNTERS    R4
         BNH   PNOTSKIP            BR IF SKIP COUNT NOT HIGH         R4
SKIP250  IC    R1,PBFAVAIL         INCREMENT NUMBER                  R4
         LA    R1,1(,R1)            OF AVAILABLE                     R4
         STC   R1,PBFAVAIL           INPUT BUFFERS                   R4
         B     PCHREAD             GO FOR NEXT INPUT BUFFER          R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        INDICATE IN PCIE THAT THE FINAL BUFFER HAS BEEN PROCESSED    *
*                                                                     *
* NOTE - WHEN THE OUTPUT FOR THIS BUFFER IS COMPLETE, PPCHECK WILL    *
*        INCREMENT THE AVAILABLE BUFFER COUNT  (PBFAVAIL)             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNOTSKIP DS    0H                                                    R4
         L     R1,POUTCCWA         IF CCW AREA IS EMPTY,             R4
         CL    R1,PCCWPT            INSERT A NOP                     R4
         BNH   PSETFINL              TO ASSURE                       R4
         LM    PC1,PC2,PCCWNOP        INDICATION IS                  R4
         ICM   PC2,B'1000',=X'60'      SEEN BY                       R4
         BAL   PL,PPPUT                 PPCHECK                      R4
         L     R1,POUTCCWA         RELOAD CCW AREA ADDRESS           R4
         SPACE 1                                                     R4
         USING PCIDSECT,R1         INDICATE THAT EXECUTION           R4
PSETFINL AH    R1,PCCWLAST          OF THIS CCW AREA                 R4
         OI    PCISGNAL,PCIFNLBF     WILL CAUSE A DATA               R4
         DROP  R1                     BUFFER TO BECOME AVAILABLE     R4
         SPACE 1                                                     R4
         BAL   PL,PPWRITE          FORCE CCW AREA SWAP               R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF READ HAS ALREADY BEEN STAGED -- GO WAIT FOR COMPLETION    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCHREAD  TM    PPFLAG2,PPRSW       TEST FOR READ IN PROGRESS         R4
         BO    PCHKNBLK            BRANCH IF YES                     R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IF ANY BUFFERS ARE AVAILABLE  -- STAGE NEXT READ             *
*        ELSE -- WAIT FOR PPCHECK TO MAKE A BUFFER AVAILABLE          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCHAVAIL CLI   PBFAVAIL,0          TEST FOR AVAILABLE INPUT BUFFER   R4
         BH    PSTGREAD            STAGE NEXT READ IF YES            R4
         TM    PPFLAG,PPWSW        TEST FOR WRITE IN PROGRESS        R4
         BZ    PLOSTINT            BRANCH IF NO                      R4
         BAL   PL,PCIWAIT          WAIT FOR OUTPUT I/O               R4
         B     PCHAVAIL             AND TRY AGAIN                    R4
         SPACE 1                                                     R4
PLOSTINT IC    R1,PBUFOPT          RESET AVAILABLE INPUT BUFFER      R4
         STC   R1,PBFAVAIL          COUNT TO BUFFERING OPTION        R4
        $COUNT                     * RECORD USAGE *                  R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        BEGIN THE NEXT DE-SPOOLING OPERATION -- GO WAIT COMPLETION   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PSTGREAD BAL   PL,PRDTCNXT         STAGE READ FOR NEXT BUFFER(S)     R4
        $COUNT                     ********************************  R4
         B     PCHKNBLK            GO CHECK AND PROCESS IT           R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- DATA SET TERMINATION'
***********************************************************************
*                                                                     *
*        ***  DATA SET TERMINATION  ***                               *
*                                                                     *
*        SPECIAL TERMINATION FOR RMT SPOOLED MESSAGES AND JES2-NEWS   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPDSEND  DS    0H
         BAL   PL,PPWRITE          FORCE EXECUTION OF LAST CCW AREA  R4
         LTR   JCT,JCT             IS THIS A SPOOL MESSAGE PRINTER
         BZ    PRSMBEOB            BRANCH IF YES
         TM    PPFLAG,PPNEWS       WAS THIS JES2-NEWS DATA SET...   R41
         BO    PENDNEWS            RETURN TO MAIN LINE IF YES       R41
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        WRITE PROGRAMMER MESSAGE IF DATA SET I/O OR VALIDITY ERROR   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         NI    PPFLAG2,255-PPCKPTA SUSPEND CHECKPOINTS               R4
         TM    PPFLAG,PPRDERR      DATA BUFFER READ ERROR
         BNO   PPIOTCK             BRANCH IF NO
        $MID   185                                                   R4
         LA    R1,=C'$HASP185 '    MESSAGE ID                        R4
         LA    R15,=C' TERMINATED '  MESSAGE TEXT                    R4
         BAL   PL,PRMSG            ADD MSG TO OUTPUT, ISSUE WTO      R4
         SPACE 1                                                     R4
         OI    PPFLAG2,PSMFDSER    SET DATA SET ERROR FLAG FOR SMF   R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        RE-READ IOT                                                  *
*                                                                     *
*        SEE IF IOT IS ALREADY BEING DE-SPOOLED                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPIOTCK  DS    0H                                                    R4
         NI    PPFLAG,255-PPRDERR  CLEAR ERROR CONDITION, IF ANY     R4
         TM    PPFLAG2,PPRSW       TEST FOR READ IN PROGRESS         R4
         BZ    PPIOTCK1            BR IF NOT                         R4
         CLC   PCESEEK(4),PCEIOTTR TEST FOR IOT READ IN PROGRESS     R4
         BE    PPIOTCK3            BR IF YES                         R4
         BAL   PL,PRDTCHK           ELSE, CLEAR INPUT I/O            R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SEE IF ANY BUFFERS ARE AVAILABLE FOR DE-SPOOLING THE IOT     *
*                                                                     *
*        NOTE - IF NONE AVAILABLE -- WAIT FOR PPCHECK TO FREE ONE     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPIOTCK1 CLI   PBFAVAIL,0          TEST FOR AVAILABLE INPUT BUFFER   R4
         BH    PPIOTCK2            BRANCH IF YES                     R4
         TM    PPFLAG,PPWSW        TEST FOR WRITE OUTSTANDING        R4
         BZ    PPIOTCK2            BRANCH IF NO                      R4
         BAL   PL,PCIWAIT          WAIT FOR AVAILABLE INPUT BUFFER   R4
         B     PPIOTCK1             AND TRY AGAIN                    R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        BUFFER AVAILABLE -- READ AND VERIFY THE IOT                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPIOTCK2 L     R15,PCEIOTTR        GET IOT TRACK ADDRESS             R4
         BAL   PL,PRDBUF           INITIATE READ OF IOT              R4
PPIOTCK3 BAL   PL,PRDCHK           CHECK READ                        R4
         TM    PPFLAG,PRDELSW      TEST FOR TERMINATION
         BO    PPDONE              BRANCH IF YES
         LR    JCT,PBUF            ADDRESS IOT IN BUFFER
         TM    PPFLAG,PPRDERR      TEST FOR I/O ERROR ON READ
         BO    PIOTPOST            BR IF YES                         R4
         CLC   IOTJBKEY,PPJOBKEY   IS THIS IOT VALID
         BE    PRSUSTST            BRANCH IF YES
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF INPUT OR VALIDITY ERROR -- TELL OPERATOR AND KILL OUTPUT  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PIOTPOST $DISTERR                  INDICATE CONTROL BLOCK ERROR
         OI    PPFLAG,PPJCTIOT+PRDELSW REASON FOR TERMINATION
         B     PPDONE              ABORT JOB
         EJECT
***********************************************************************
*                                                                     *
*        IF $F OR $B COMMAND -- INFORM OPERATOR AND PROGRAMMER        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRSUSTST BAL   PL,PPCHECK          FINAL I/O. CHK OPER CMDS    @OZ29138
         TM    PPFLAG,PPDELSW      CHECK FOR SUSPENSION        @OZ29138
         BZ    PRLOGCK             BRANCH IF NO
         NI    PPFLAG,255-PPDELSW  RESET SUSPEND FLAG
         ICM   R1,15,PDDBSKIP      CHECK FOR $B,D OR $F,D      @OZ29138
         BNP   PRSPDS              BR IF YES, PDDBSKIP IS OK   @OZ29138
         L     R1,PDDBPGCT         GET CURRENT PAGE COUNT      @OZ29138
         A     R1,PFSBSCT          CALCULATE PDDBSKIP          @OZ29138
         BNP   PRSPNEG             SETUP FOR $B,D              @OZ29138
         CL    R1,=F'16777215'     IS SKIP TARGET TOO LARGE... @OZ29138
         BNH   PRSPPOS             BR IF NO, ELSE SET MAXIMUM  @OZ29138
         L     R1,=F'16777215'     GET MAX $F PAGE COUNT       @OZ29138
         B     PRSPPOS             UPDATE PDDBSKIP             @OZ29138
PRSPNEG  SLR   R1,R1               SETUP FOR $B,D              @OZ29138
PRSPPOS  ST    R1,PDDBSKIP         SAVE NEW SKIP COUNT         @OZ29138
PRSPDS   DS    0H                                              @OZ29138
        $MID   170                                                   R4
PRSPMSG  LA    R1,=C'$HASP170 '    POINT TO MESSAGE ID AND          R41
         LA    R15,=C' FWD-SPACED '  MESSAGE TEXT                    R4
         CLC   PDDBSKIP,PDDBPGCT   IS THIS A FWD-SPACE
         BH    *+8                 BRANCH IF YES
         LA    R15,=C' BACKSPACED '  MESSAGE TEXT                    R4
         BAL   PL,PRMSG            ADD MSG TO OUTPUT, ISSUE WTO      R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        GO TERMINATE DATA SET IF $F DEVICE,DSET WAS REQUESTED        *
*                                                                     *
*        CAUSED BY OPERATOR COMMAND OR PRINTER 'CANCEL' BUTTON        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         CLC   PDDBSKIP,=F'-1'     TEST SKIP COUNT
         BE    PRLOGCK             $F DEVICE,DS
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SKIP TO NEW PAGE, OR ADD A BLANK CARD -- THEN RE-POSITION    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LM    PC1,PC2,PRCCWEJ     SELECT PAGE-EJECT CCW             R4
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE
         BO    *+8                 BRANCH IF PRINT
         LM    PC1,PC2,PUCCWBL     SELECT BLANK CARD CCW
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BAL   PL,PPWRITE           INITIATE WRITE
         BAL   PL,PPCHECK            AND CHECK
         XC    PULMTTR(7),PULMTTR  CLEAR PUNCH RESTART POINTER       R4
         CLC   PDDBSKIP,PDDBPGCT   FWD/BSP PAGE COUNT
         BH    PBSFSGO             $F DEVICE,N
         LH    R15,$BSPSIZ         IF NO BSP                         R4
         LTR   R15,R15              TABLE ENTRIES,                   R4
         BZ    PBSPBEG               BR TO RESTART DS               R41
         EJECT
***********************************************************************
*                                                                     *
*        $B COMMAND - SEARCH BACKSPACE TABLE FOR RESTART POINT        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         SH    R15,=H'8'           TABLE LENGTH - 1, LESS 1 ENTRY    R4
         LA    R1,PBSPTBL          POINT TO 1ST TABLE ENTRY          R4
         SPACE 1                                                     R4
PBSPSRCH ALR   R1,R15              POINT TO LAST TABLE ENTRY - 1     R4
         SLR   PL,PL               GET                               R4
         IC    PL,$BSPGCT           PAGE FRAME                       R4
         LNR   PL,PL                 FOR LAST                        R4
         A     PL,PBSPGCT             ENTRY                          R4
         BNP   PBSPBEG             BR IF START OF DATA SET           R4
         OC    5(2,R1),5(R1)       TEST FOR VALID ENTRY              R4
         BZ    PBSPBEG             BR IF NO -- RESTART DS            R4
         ST    PL,PBSPGCT          SAVE FOR LATER
         MVC   PCEJMTTR,1(R1)      HOLD BUFFER MTTR                  R4
         MVC   PCEEJRCB,5(R1)      HOLD LINE RCB                     R4
         MVC   PBUFSKIP,7(R1)      HOLD BUFFER OFFSET                R4
         LA    R1,PBSPTBL          POINT TO 1ST TABLE ENTRY          R4
         LTR   R15,R15             IF SINGLE-ENTRY TABLE,            R4
         BM    PBSPINVL             DON'T POP UP TABLE ENTRIES      R41
         EX    R15,PPOP1           POP UP BSP TABLE                 R41
         EX    R15,PPOP2            TO PREVIOUS ENTRY               R41
         SPACE 1                                                    R41
PBSPINVL DS    0H                                              @OZ29138
         CL    PL,PDDBSKIP         IS PAGE BEFORE BSP POINT
         BNH   PBSPFND             BR IF CORRECT PAGE FRAME    @OZ29138
         XC    0(7,R1),0(R1)       INVALIDATE UNUSED ENTRY     @OZ29138
         B     PBSPSRCH            GO CHECK NEXT FRAME         @OZ29138
PBSPFND  DS    0H                                              @OZ29138
         ST    PL,PDDBPGCT         SET AS NEW PAGE COUNTER
         SLR   PL,PL               GET                         @OZ29138
         IC    PL,$BSPGCT           PAGE FRAME AND BUMP        @OZ29138
         A     PL,PDDBPGCT            BY FRAME SIZE            @OZ29138
         ST    PL,PBSPGCT          SAVE FOR NEXT TABLE UPDATE  @OZ29138
         B     PBSFSGO             GO TO RESTART PROCESS
         SPACE 1                                                     R4
PPOP1    MVC   BUFSTART(*-*),0(R1) *** EXECUTE ONLY ***             R41
PPOP2    MVC   7(*-*,R1),BUFSTART  *** EXECUTE ONLY ***             R41
         SPACE 2                                                     R4
PBSPBEG  LH    PC1,PDDBDISP        GET PDDB OFFSET                   R4
         LA    PC2,0(PC1,JCT)      GET PDDB ADDRESS                  R4
         MVC   PCEJMTTR,PDBMTTR-PDBDSECT(PC2)  RESET 1ST MTTR        R4
         MVI   PBUFSKIP,0          RESET TRACK-CELL BUFFER OFFSET    R4
         B     PCKINIT             BACKSPACE TO START OF DATA SET    R4
         EJECT
***********************************************************************
*                                                                     *
*        PRINT HASP JOB STATISTICS AFTER LOG DATA SET                 *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRLOGCK  SLR   R0,R0               RESET PAGE COUNT            @OZ28599
         ST    R0,PDDBPGCT          AND SAVE IT                @OZ28599
         TM    PCEID,PCEPUSID      IS THIS A PUNCH PROCESSOR
         BO    PNEXTDDB            BRANCH IF YES
         LH    PC1,PDDBDISP        GET PDDB OFFSET INTO THE IOT
         LA    PC2,0(PC1,JCT)      ADD IOT BASE
         TM    PDBFLAG1-PDBDSECT(PC2),PDB1LOG IS THIS THE LOG PDDB
         BNO   PNEXTDDB            BRANCH IF NO
         L     JCT,PJCTBUF         ADDRESS JCT BUFFER
         USING JCTDSECT,JCT        ACTIVATE JCT ADDRESSABILITY
         MVC   PMESSAGE,PJOBSTAT   SETUP STATISTICS TITLE
         BAL   PL,PRCOMENT         WRITE STATISTICS HEADER
         ICM   R1,15,JCTXDTON      OBTAIN EXECUTION DATE            R41
         BZ    PRNODATE            BR IF NONE                       R41
         BAL   PL,PPDATE           CONVERT TO PRINTABLE             R41
         MVC   PMESSAGE+10(L'PXEQDATE),PXEQDATE  FORMAT MESSAGE     R41
         BAL   PL,PRCOMENT          AND ADD TO STATISTICS           R41
PRNODATE DS    0H                                                   R41
         MVC   PMESSAGE,PRDRSTAT   SETUP CARDS READ
         L     R1,JCTCARDS         GET INPUT CARD COUNT
         CVD   R1,PCCWORK          CONVERT TO DECIMAL
         ED    PMESSAGE(10),PCCWORK+4 EDIT INPUT CARD COUNT          R4
         BAL   PL,PRCOMENT         WRITE CARDS READ
         MVC   PMESSAGE,PPRTSTAT   SETUP SYSOUT PRINT
         L     R1,JCTLINES         GET SYSOUT LINE COUNT
         CVD   R1,PCCWORK          CONVERT TO DECIMAL
         ED    PMESSAGE(10),PCCWORK+4 EDIT SYSOUT RECORD COUNT       R4
         BAL   PL,PRCOMENT         WRITE SYSOUT RECORD COUNT
         MVC   PMESSAGE,PPUNSTAT   SETUP SYSOUT PUNCH
         L     R1,JCTPUNCH         GET SYSOUT PUNCH COUNT
         CVD   R1,PCCWORK          CONVERT TO DECIMAL
         ED    PMESSAGE(10),PCCWORK+4 EDIT SYSOUT RECORD COUNT       R4
         BAL   PL,PRCOMENT         WRITE SYSOUT RECORD COUNT
         MVC   PMESSAGE,PXEQSTAT   SETUP EXECUTION TIME
         SLR   R0,R0               ZERO TIME ACCUMULATOR             R4
         SLR   R1,R1               ZERO TIME ACCUMULATOR             R4
         CLC   JCTXDTON,JCTXDTOF   COMPARE START AND STOP DATES      R4
         BE    PRDATEQ             BRANCH IF SAME DAY                R4
         BL    PRDATELO            BRANCH IF SPANNED AT LEAST 1 DAY  R4
PRDATEHI DS    0H                                                    R4
         MVC   PMESSAGE+1(9),=C'(UNKNOWN)' JOB STOPPED BEFORE START  R4
         B     PRXTIME             BRANCH TO PRINT MESSAGE           R4
         EJECT                                                       R4
PRDATELO DS    0H                                                    R4
         TM    JCTXDTON+3,X'0F'    VALIDATE START DATE (00YYDDDF)    R4
         BNO   PRDATEHI            ERR MSG IF DATE NOT STORED        R4
         TM    JCTXDTOF+3,X'0F'    VALIDATE STOP DATE                R4
         BNO   PRDATEHI            ERR MSG IF DATE NOT STORED        R4
         CLC   JCTXDTON(2),JCTXDTOF COMPARE YEAR PORTIONS ONLY       R4
         BE    PRYEAREQ            BRANCH IF SAME YEAR               R4
         BH    PRDATEHI            ERR MSG IF YEARS BACKWARD         R4
         ZAP   PCCWORK,JCTXDTON    ISOLATE START YEAR                R4
         SRP   PCCWORK,64-3,0        AS PACKED DECIMAL NUMBER        R4
         CVB   PL,PCCWORK          CONVERT START YEAR TO BINARY,     R4
         O     PL,=F'3'              TEST FOR LEAP YEAR              R4
         BNZ   SKIP270             SKIP IF NOT LEAP YEAR             R4
         LA    R1,1                ALLOW FOR 366-DAY YEAR            R4
SKIP270  LA    R1,365(,R1)         ADD 365 DAYS FOR YEAR CHANGE      R4
PRYEAREQ DS    0H                                                    R4
         ZAP   PCCWORK,JCTXDTOF+2(2) COMPUTE DIFFERENCE              R4
         SP    PCCWORK,JCTXDTON+2(2)   BETWEEN JOB START             R4
         CVB   PL,PCCWORK              DAY AND JOB STOP DAY          R4
         AR    R1,PL               ADD TO ADJUSTMENT FOR YEAR CHANGE R4
         M     R0,=A(24*60*60*100) CONVERT DAYS TO HUNDREDTHS OF SEC R4
PRDATEQ  DS    0H                                                    R4
         AL    R1,JCTXEQOF         ADD STOP TIME                     R4
         BC    12,SKIP280          SKIP IF NO CARRY                  R4
         AL    R0,=F'1'            ADJUST HIGH END                   R4
SKIP280  SL    R1,JCTXEQON         SUBTRACT START TIME               R4
         BC    3,SKIP290           SKIP IF CARRY                     R4
         BCTR  R0,0                ADJUST HIGH END                   R4
SKIP290  LTR   R0,R0               CHECK HIGH END                    R4
         BM    PRDATEHI            ERR MSG IF TOTAL TIME NEGATIVE    R4
         BP    PRBIGMIN            BRANCH IF TOO LARGE FOR EDIT      R4
         CL    R1,=A(9999999*60)   CHECK LOW END                     R4
         BH    PRBIGMIN            BRANCH IF TOO LARGE FOR EDIT      R4
         D     R0,=F'60'           CONVERT TO HUNDREDTHS OF MINUTES  R4
         B     PRTIMCVD            BRANCH TO CVD AND EDIT            R4
PRBIGMIN DS    0H                                                    R4
         D     R0,=A(60*100)       CONVERT TIME TO WHOLE MINUTES     R4
         MVC   PMESSAGE(10),PRDRSTAT REPLACE EDIT PATTERN            R4
PRTIMCVD DS    0H                                                    R4
         CVD   R1,PCCWORK          CONVERT TO DECIMAL                R4
         ED    PMESSAGE(10),PCCWORK+4 EDIT EXECUTION TIME            R4
PRXTIME  DS    0H                                                    R4
         BAL   PL,PRCOMENT         WRITE EXECUTION TIME
         DROP  JCT                 SUSPEND JCT ADDRESSABILITY
         LR    JCT,PBUF            RESTORE IOT BASE ADDRESS
         EJECT
***********************************************************************
*                                                                     *
*        PREPARE FOR SELECTION OF THE NEXT DATA SET                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNEXTDDB DS    0H
         XC    PDDBSKIP,PDDBSKIP   CLEAR SKIP COUNT
         L     R1,PCEDCT           ADDRESS PRINT/PUNCH DCT     @OZ32566
         USING DCTDSECT,R1         ACTIVATE DCT ADDRESSABILITY
         NI    DCTFLAGS,255-DCTSPACE RESET SPACE CONTROL SWITCHES
         DROP  R1                  SUSPEND DCT ADDRESSABILITY
         NI    PDCTFLAG,255-DCTSPACE RESET SPACE CONTROL SWITCHES
         LM    PC1,PC2,PRCCWEJ     SELECT SKIP TO CH 1 CCW
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE
         BO    *+8                 BRANCH IF PRINT
         LM    PC1,PC2,PUCCWBL     SELECT BLANK CARD CCW
         BAL   PL,PPPUT            PUT CCW ONTO CHAIN
         BAL   PL,PPWRITE           SEND CHAIN TO DEVICE
         BAL   PL,PPCHECK            CHECK WRITE
         L     PW,PRPAGECT         UPDATE                           R41
         LA    PW,1(,PW)            TOTAL                           R41
         ST    PW,PRPAGECT           PAGE COUNT                     R41
         XC    PULMTTR(7),PULMTTR  CLEAR PUNCH RESTART POINTER       R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        RE-CYCLE SAME DATA SET UNTIL DS COPIES IS SATISFIED          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING PDBDSECT,PC1        PROVIDE PDDB ADDRESSABILITY       R4
         SPACE 1                                                     R4
PRCPYTST DS    0H
         SPACE 1                                                     R4
         NI    PPFLAG2,255-PPFDS   RESET 1ST DATA SET SWITCH         R4
         SPACE 1                                                     R4
         LH    PC1,PDDBDISP        PDDB OFFSET INTO THE IOT          R4
         LA    PC1,0(PC1,PBUF)     ADD IOT BASE                      R4
         MVC   PCEJMTTR,PDBMTTR    RESET 1ST MTTR OF DATA SET        R4
         SPACE 1                                                     R4
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BE    PRCPYGRP            HANDLE COPY GROUPS IF YES         R4
         SPACE 1                                                     R4
         IC    PW,PPRCPYCT         INCREMENT
         LA    PW,1(,PW)           COPY
         STC   PW,PPRCPYCT         COUNT
         CLC   PPDSCPY,PPRCPYCT    ENOUGH COPIES OF DATA SET
         BH    PNXTCPY             BR IF ANOTHER COPY NEEDED         R4
         SPACE 1                                                     R4
PRCPYEND DS    0H                                                    R4
         MVI   PPRCPYCT,X'00'      SET COPY COUNT TO ZERO
         B     PDDBSRCH            FIND NEXT MATCHING PDDB
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        HANDLE COPIES (COPY GROUPS) AND FLASH COUNTS FOR 3800        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRCPYGRP DS    0H                                                    R4
         SLR   PW,PW               CLEAR WORK REGISTER               R4
         LA    R1,1                ASSUME 1 COPY JUST PRINTED        R4
         CLI   PCOPYGRP,0          TEST FOR COPY GROUPS              R4
         BE    SKIP300             BR IF NOT - ASSUMPTION CORRECT    R4
         IC    R1,PDDBCPYG         ELSE, USE COUNT                   R4
         IC    R1,PCOPYGRP(R1)      OF COPIES JUST PRINTED           R4
SKIP300  IC    PW,PPRCPYCT         UPDATE TOTAL                      R4
         LA    PW,0(R1,PW)          NUMBER OF                        R4
         STC   PW,PPRCPYCT           COPIES PRINTED                  R4
         CLM   PW,1,PPDSCPY        TEST FOR ALL COPIES PRINTED       R4
         BNL   PRCPYEND            BR IF YES                         R4
         LA    R1,1(,PW)           PLACE NEW STARTING                R4
         STC   R1,SPCOPYS           COPY NO. INTO SETUP LIST         R4
         MVI   SPFLASHC,0          ASSUME NO MORE FLASHING           R4
         CLI   PDBFLASH,C'*'       FLASHING...                 @OZ18407
         BE    PRCPYN              BR IF NO                    @OZ18407
         IC    R1,PDBFLSHC         GET NO. COPIES NEEDING FLASH      R4
         SR    R1,PW               SUBTRACT NUMBER ALREADY FLASHED   R4
         BNP   PRCPYN              BR IF DONE FLASHING         @OZ18407
         STC   R1,SPFLASHC          ELSE SET REMAINING COUNT   @OZ18407
         SPACE 1                                               @OZ18407
PRCPYN   MVI   SPCOPYN,1           ASSUME 1 COPY THIS XMISSION @OZ18407
         CLI   PCOPYGRP,0          TEST FOR COPY GROUPS              R4
         BE    PTSTRXMT            BR IF NO                         R41
         SLR   PW,PW               ELSE, UPDATE                      R4
         IC    PW,PDDBCPYG          COPY GROUP OFFSET                R4
         LA    PW,1(,PW)             IN CKPT-AREA AND                R4
         STC   PW,PDDBCPYG            PLACE NUMBER                   R4
         IC    PW,PCOPYGRP(PW)         OF COPIES INTO                R4
         STC   PW,SPCOPYN               SETUP LIST                   R4
         B     PSETRXMT            BR TO SETUP FOR RE-XMIT          R41
         SPACE 1                                                    R41
PTSTRXMT CLI   PDBMODF,C'*'        USING COPY-MOD...                R41
         BNE   PSETRXMT            BR IF YES                        R41
         CLI   PDBFLASH,C'*'       FLASHING...                      R41
         BE    PRCPYOFS            BR IF NO                         R41
         CLC   PDBFLSHC,PPRCPYCT   TURN-OFF FLASHING...        @OZ18407
         BNE   PRCPYOFS            BR IF NO                    @OZ18407
         SPACE 1                                                    R41
PSETRXMT MVI   SPFLAG,SPREXMIT     SET RETRANSMISSION INDICATION    R41
         L     R15,=A(P3800DSV)    CALL 3800                         R4
         BALR  PL,R15               DEVICE SETUP VERIFICATION        R4
         SPACE 1                                                     R4
PRCPYOFS LM    PC1,PC2,PCCWOFST    ADD OFFSET-STACKER               R41
         BAL   PL,PPPUT             CCW TO CHAIN                    R41
         ICM   PC1,8,PXTABCCW      SELECT                      @OZ24675
         BAL   PL,PPPUT             CHAR1                      @OZ24675
         B     PNXTCPY             BR FOR NEXT COPY (COPIES)         R4
         SPACE 1                                                     R4
         DROP  PC1                 SUSPEND PDDB ADDRESSABILITY       R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- PPPUT SUBROUTINE'        R4
***********************************************************************
*                                                                     *
*        PPPUT -- SUBROUTINE TO ADD A NEW PRINT OR PUNCH COMMAND TO   *
*                 THE CURRENT CCW AREA                                *
*                                                                     *
*        PC1,PC2 - 8 BYTE CHANNEL COMMAND WORD                        *
*        PL      - RETURN ADDRESS                                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ELIMINATE EXTRANEOUS PAGE EJECTS                             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING DCTDSECT,R1         PROVIDE DCT ADDRESSABILITY        R4
         SPACE 1                                                     R4
PPPUT    TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE               R4
         BNO   PPPUT2              BRANCH IF NOT PRINTER             R4
         L     R1,PCEDCT           LOAD ADDRESS OF PRINTER DCT @OZ32566
         CLM   PC1,8,=X'89'        TEST CHANNEL COMMAND CODE
         BL    PPPUT1              BR IF NOT POSSIBLE EJECT
         BE    PCHAN1              BR IF PRINT, SKIP TO CHANNEL 1    R4
         CLM   PC1,8,=X'8B'        TEST CHANNEL COMMAND CODE
         BNE   PPPUT1              BRANCH IF NOT IMED SKP CHNL 1
         TM    DCTPPFL,DCTEJECT    CHECK IF ALREADY AT CHANNEL 1     R4
         BOR   PL                  RETURN IF YES                     R4
         SPACE 1                                                     R4
PCHAN1   OI    DCTPPFL,DCTEJECT    SHOW PRINTER EJECTED
         B     PPPUT2                AND CONTINUE
PPPUT1   NI    DCTPPFL,255-DCTEJECT RESET CHANNEL 1 SWITCH
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ALTERNATE PPPUT ENTRY POINT                                  *
*                                                                     *
*        IF REMOTE -- SEND CCW TO LINE MANAGER                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPPUT2   TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BZ    PTESTCHN            BR IF NOT REMOTE TERMINAL         R4
         STM   PC1,PC2,POUTCCWA    PASS CCW TO                       R4
         L     R1,PCEDCT           GET PRINT/PUNCH DCT ADDRESS @OZ32566
         LA    R0,POUTCCWA         LOAD ADDRESS OF CCW               R4
         TM    DCTPPFL,DCTEJECT    END OF PAGE...                   R41
         BZ    *+8                 NO, SKIP--JUST CALL RTAM          R4
         ICM   R0,8,*              SET END PAGE INDICATOR FOR RTAM   R4
        $EXTP  PUT,(R1),(R0)       PASS RESUEST TO RTAM              R4
         B     PPCHECK             CHECK OPERATOR COMMANDS           R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SEE IF OK TO ADD NEW CCW TO THIS CCW AREA                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING PCIDSECT,PW         PROVIDE PCIE ADDRESSABILITY       R4
         SPACE 1                                                     R4
PTESTCHN LH    PW,PCCWLAST         GET OFFSET TO PCIE                R4
         AL    PW,POUTCCWA         ADD BASE TO OBTAIN TRUE ADDRESS   R4
         SPACE 1                                                     R4
         TM    PCISGNAL,PCIACTIV   USE IS OK IF THIS                 R4
         BZ    PPCHAIN              CCW AREA IS NOT ACTIVE,          R4
         TM    PPFLAG,PPWSW          OR IF A WRITE                   R4
         BZ    PPCHAIN                IS NOT IN PROGRESS             R4
         SPACE 1                                                     R4
         ST    PL,PLSAVE           ELSE, WAIT                        R4
         BAL   PL,PCIWAIT           FOR A PCI                        R4
         L     PL,PLSAVE             AND                             R4
         B     PTESTCHN               TRY AGAIN                      R4
         SPACE 1                                                     R4
         DROP  R1,PW               SUSPEND DCT/PCIE ADDRESSABILITY  R41
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        TRANSLATE VIRTUAL DATA ADDRESS TO REAL AND ADD CCW TO CHAIN  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPCHAIN  DS    0H
         L     PW,PCCWPT           PICK UP CURRENT CCW POINTER       R4
         LA    PW,8(,PW)           BUMP CCW POINTER
         STM   PC1,PC2,0(PW)       ADD CCW TO CHAIN
         LRA   R1,0(,PC1)          CONVERT VIRT DATA ADDRESS TO REAL R4
         STCM  R1,7,1(PW)           AND UPDATE IN CHAIN              R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        MERGE IMMEDIATE CCW WITH PREVIOUS PRINT, NO SPACE CCW        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         OC    6(2,PW),6(PW)       TEST BYTE COUNT
         BNZ   PPENDCHK            BRANCH IF NOT ZERO
         TM    0(PW),X'02'         IS THIS AN IMMED CMD...     @OZ25297
         BO    PPCHAIN1            YES, CHK IF COMBINE         @OZ25297
         MVI   5(PW),C' '          SET BLANK SOURCE            @OZ25297
         LRA   R1,5(,PW)           GET REAL ADDRESS OF BLANK   @OZ25297
         STCM  R1,7,1(PW)           IN CCW                     @OZ25297
         B     PPNOMERG              AND SET LENGTH TO 1       @OZ25297
PPCHAIN1 DS    0H                                              @OZ25297
         L     R1,POUTCCWA         ADDRESS 1ST CCW IN CHAIN          R4
         CLR   PW,R1               IS CURRENT CCW AFTER 1ST
         BNH   PPNOMERG            BR IF NOT                         R4
         L     R1,PCCWPT           R1 = ADDRESS OF LAST CCW
         CLI   0(R1),X'01'         TEST LAST COMMAND
         BNE   PPNOMERG            BR IF NOT PRINT, NO SPACE         R4
         NI    0(PW),X'FD'         REMOVE IMMEDIATE BIT              R4
         MVC   0(1,R1),0(PW)       MOVE COMMAND TO PREVIOUS CCW      R4
         BR    PL                  RETURN -- NO CCW ADDED            R4
         SPACE 1                                                     R4
PPNOMERG MVI   7(PW),X'01'         FORCE BYTE COUNT NON-ZERO
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        UPDATE CCW POINTER AND PUNCH RESTART POINTERS                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPENDCHK ST    PW,PCCWPT           SAVE CCW POINTER
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE
         BO    PNOTPUN             BRANCH IF PRINT
         TM    0(PW),X'05'         IS CCW A 3525 PRINT-LINE
         BO    PNOTPUN             BRANCH IF YES - DON'T CKPT
         L     R1,POUTIOB          ADDRESS OF OUTPUT IOB             R4
         MVC   PPBNMTTR-BUFDSECT(4,R1),PCEJMTTR  UPDATE PUNCH        R4
         MVC   PPBNRCB-BUFDSECT(2,R1),PCEEJRCB    RESTART            R4
         MVC   PPBNBOFF-BUFDSECT(1,R1),PCEJBOFF    POINTERS          R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        USE PPWRITE IF CCW AREA IS FULL                              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PNOTPUN  DS    0H
         LA    PW,8(,PW)           COMPUTE OFFSET OF NEXT            R4
         SL    PW,POUTCCWA          SLOT IN CCW CHAIN                R4
         CH    PW,PCCWLAST         TEST FOR END OF CCW CHAIN         R4
         BLR   PL                  RETURN IF NOT                     R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- PPWRITE SUBROUTINE'      R4
***********************************************************************
*                                                                     *
*        PPWRITE -- SUBROUTINE TO SCHEDULE A NEW CCW AREA FOR OUTPUT  *
*                                                                     *
*        PL    - RETURN ADDRESS                                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         DROP  PBUF                ACTIVATE LOCAL ADDRESSABILITY ON  R4
         USING BUFDSECT,R15         OUTPUT IOB (BUFFER PREFIX)       R4
         SPACE 1                                                     R4
         CNOP  0,8                                                   R4
PPWRITE  ST    PL,PLSAVE           SAVE RETURN ADDRESS               R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        RETURN IF REMOTE OR IF CCW AREA IS EMPTY                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BOR   PL                  RETURN IF REMOTE                  R4
         L     PW,PCCWPT           PICK UP CCW POINTER               R4
         L     R1,POUTCCWA         ADDR OF CCW AREA                  R4
         CLR   PW,R1               RETURN IF                         R4
         BLR   PL                   NO CCW'S                         R4
         L     R15,POUTCCWN        GET NEW CCW CHAIN ADDRESS   @OZ29138
         AH    R15,PCCWLAST        POINT TO THE PCIE,          @OZ29138
         LA    R15,PCIESIZE(,R15)   AND THE CCW CHKPT AREA.    @OZ29138
         MVC   JOEPPCT-JOEDSECT(,R15),PDDBPGCT UPDATE PAGE CNT @OZ29138
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SET PCI AND INITIALIZE PCIE AT END OF NEW CCW AREA           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING PCIDSECT,R1         PROVIDE PCIE ADDRESSABILITY       R4
         SPACE 1                                                     R4
         L     R15,POUTIOB         OUTPUT IOB ADDRESSABILITY         R4
         OI    4(R1),X'08'         SET PCI AT TOP OF NEW AREA        R4
         AH    R1,PCCWLAST         COMPUTE ADDR OF PCIE              R4
         OI    PCISGNAL,PCIACTIV+PCIBUSY  SHOW PCIE ACTIVE AND BUSY  R4
         NI    PCI1FLGS,255-X'40'  TURN OFF COMMAND CHAINING         R4
         ST    PW,PPBLVCCN         SAVE POINTER TO LAST VALID CCW    R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF CCW AREA IS NOT FULL -- ADD A TIC TO THE PCIE             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LA    PW,8(,PW)           ADDR OF NEXT SLOT IN CHAIN        R4
         CLR   PW,R1               COMPARE WITH PCIE ADDRESS         R4
         BE    PPWSWAP             BRANCH IF EQUAL (FLUSH WITH PCIE) R4
         LRA   R1,0(,R1)           CONVERT VIRT PCIE ADDRESS TO REAL R4
         MVC   0(8,PW),PCNOPTIC+8  ADD A TIC ON THE CHAIN            R4
         STCM  R1,7,1(PW)           AND POINT IT TO THE PCIE         R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        SWAP CCW AREA POINTERS                                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPWSWAP  DS    0H                                                    R4
        $COUNT                     ******* RECORD USAGE *******      R4
         LM    R0,R1,POUTCCWA      SWAP                              R4
         ST    R0,POUTCCWN          CCW AREA                         R4
         ST    R1,POUTCCWA           POINTERS                        R4
         ST    R0,PPBCCWNX         NEW CHAIN ADDR INTO BUFFER PREFIX R4
         LR    PW,R1               RESET POINTER                     R4
         SH    PW,=H'8'             TO FIRST CCW                     R4
         ST    PW,PCCWPT             IN NEXT AREA                    R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF WRITE IN PROGRESS -- PREPARE TO CHAIN NEW CCW AREA ON     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PPFLAG,PPWSW        TEST FOR WRITE IN PROGRESS        R4
         BZ    PWEXCPVR            IF NOT - GO START ONE             R4
         AH    R1,PCCWLAST         ADDRESS OF PCIE ON ACTIVE CHAIN   R4
         L     R0,PCI1CCW+4        RIGHT HALF OF PCIE NOP CCW        R4
         O     R0,=A(PCIBUSY)      SET BUSY FLAG FOR COMPARE         R4
         LR    PW,R0               COPY AND BUILD A                  R4
         ICM   PW,8,=X'60'          CHAINED NOP  (CC+SLI)            R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        ATTEMPT TO SET THE COMMAND-CHAINING BIT IN THE PCIE AT       *
*                THE END OF THE CURRENTLY ACTIVE CHAIN                *
*                                                                     *
*       IF SUCCESSFUL, THE TIC WILL PASS CONTROL TO THE NEW CHAIN     *
*              IF NOT, WAIT FOR THE CHE AND RE-ISSUE I/O              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         CS    R0,PW,PCI1CCW+4     APPEND NEW CHAIN TO ACTIVE CHAIN  R4
         BE    PPWRETRN            BR IF SUCCESSFUL                  R4
         BAL   PL,PPCHECK          WAIT FOR CHANNEL-END (CHE)        R4
         SPACE 1                                                     R4
         DROP  R1                  SUSPEND PCIE ADDRESSABILITY       R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SETUP FOR OUTPUT I/O AND CALL $EXCP (VR)                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PWEXCPVR DS    0H                                                    R4
         L     R15,POUTIOB         RE-ESTABLISH IOB ADDRESSABILITY   R4
         L     R1,POUTCCWN         ADDRESS OF NEW CHANNEL PROGRAM    R4
         ST    R1,IOBSTART         PLACE INTO IOB FOR IOS            R4
         NI    4(R1),255-X'08'     RESET PCI FLAG                    R4
         TM    PCEID,PCERJEID+PCEPRSID  TEST PROCESSOR TYPE    @OZ32288
         BNZ   PRTNOTPU            BRANCH IF NOT LOCAL PUNCH   @OZ32288
         LR    R14,R1              PRESET                      @OZ32288
         SH    R14,=H'8'            ERROR                      @OZ32288
         ST    R14,PUERRPT           POINTER                   @OZ32288
PRTNOTPU AH    R1,PCCWLAST         OBTAIN PCIE ADDRESS         @OZ32288
         ST    R1,PPBPCIE          PLACE INTO IOB FOR APPENDAGE      R4
         MVC   PPBLVCCC,PPBLVCCN   LAST VALID CCW ADDR INTO IOB      R4
         MVC   PPBCMTTR(7),PPBNMTTR UPDATE PUNCH RESTART POINTERS    R4
         MVI   PPBFLAG1,X'00'      CLEAR CSW SAVE              @OZ29106
         L     R1,PCEDCT           PICK UP DCT ADDRESS         @OZ32566
        $EXCP  (R1),TYPE=VR        INITIATE WRITE (EXCPVR)           R4
        $COUNT                     ******* RECORD USAGE *******      R4
         SPACE 1                                                     R4
         OI    PPFLAG,PPWSW        INDICATE WRITE HAS BEEN STAGED    R4
         SPACE 2                                                     R4
PPWRETRN DS    0H                                                    R4
         L     PL,PLSAVE           RESTORE LINK REGISTER             R4
         BR    PL                   AND RETURN                       R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- 3800 ERROR RECOVERY'     R4
***********************************************************************
*                                                                     *
*        ***  3800 PAPER JAM RECOVERY  ***                            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING DCTDSECT,PW         ESTABLISH DCT ADDRESSABILITY      R4
         SPACE 1                                                     R4
P3800ERR DS    0H                                                    R4
         OI    DCTFLAGS,DCTSTOP    HALT DEVICE AND                   R4
         NI    DCTPPSW2,255-DCTNIJAM  RESET JAM INDICATION           R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        OBTAIN APPROXIMATE BACKUP PAGE COUNT FOR OPERATOR            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LH    R14,DCTJAMCT        OBTAIN 3800 PAGES-LOST COUNT      R4
         L     R1,IOBCSW-1         POINT TO START OF                 R4
         LA    R1,0(,R1)            REMAINING CHAIN                  R4
         CLM   R1,7,IOBSTART+1     CSW VALID...                     R41
         BNL   P38FRSTL            BR IF YES                        R41
         ICM   R1,7,IOBSTART+1      ELSE USE IOBSTART               R41
         SPACE 1                                                    R41
P38FRSTL L     R0,PPBLVCCC         POINT TO END OF CHAIN            R41
         SPACE 1                                                     R4
P38PGLUP CLI   0(R1),X'89'         ADD                               R4
         BL    SKIP320              NUMBER                           R4
         LA    R14,1(,R14)           OF                              R4
SKIP320  LA    R1,8(,R1)              SKIP TO                        R4
         CLR   R1,R0                   CHANNEL N                     R4
         BNH   P38PGLUP                 CCWS                         R4
         CL    R0,PPBLVCCC         CHECK FOR FIRST LOOP              R4
         BNE   P38JMSG             BR IF NOT                         R4
         L     R1,PPBPCIE                  GO INFORM                 R4
         TM    PCI1FLGS-PCIDSECT(R1),X'40'  OPERATOR IF END          R4
         BZ    P38JMSG                       OF CCW AREAS            R4
         L     R1,PPBCCWNX         RESET POINTERS TO                 R4
         L     R0,PPBLVCCN          NEXT CCW AREA AND                R4
         B     P38PGLUP              GO COUNT ITS PAGES              R4
         SPACE 1                                                     R4
P38JMSG  DS    0H                                                    R4
         CVD   R14,$CSAVREG              EDIT THE                    R4
         UNPK  $CSAVREG(4),$CSAVREG+5(3)  COUNT OF                   R4
         OI    $CSAVREG+3,X'F0'            LOST PAGES                R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        INFORM OPERATOR OF JAM -- GIVE APPROXIMATE PAGE COUNT        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LR    R0,JCT              SAVE JCT REG ACROSS $WTO    @OZ25856
         L     JCT,PJCTBUF         GET JCT ADDRESS FOR $WTO INFO     R4
         SPACE 1                                                     R4
        $MID   195                 SET MESSAGE IDENTIFIER            R4
         MVC   PMESSAGE(2),=X'195E'    MOVE MESSAGE NUMBER           R4
         MVC   PMESSAGE+2(8),DCTDEVN   MOVE DEVICE NAME              R4
         MVC   PMESSAGE+10(29),=CL29' PAPER JAM--APPROX PAGES=XXXX'  R4
         MVC   PMESSAGE+35(4),$CSAVREG MOVE PAGES-LOST COUNT         R4
        $WTO   PMESSAGE,39,JOB=YES,    INFORM OPERATOR OF JAM        R4C
               ROUTE=$LOG+$UR,CLASS=$DOMACT,PRI=$ST                  R4
         CLC   DCTJAMCT(2),PDDBPGCT+2 HAS DS BOUNDARY PASSED   @OZ31193
         BNH   PREJECT             IF NOT, SKIP MSG            @OZ31193
         $MID  196                                             @OZ31193
         MVC   PMESSAGE(2),=X'196E'                            @OZ31193
         MVC   PMESSAGE+2(8),DCTDEVN MOVE DEVICE               @OZ31193
         MVC   PMESSAGE+10(26),PEFMSG MOVE MESSAGE TEXT        @OZ31193
         $WTO  PMESSAGE,36,JOB=YES,   ISSUE $E NECESSARY MSG   @OZ31193*
               PRI=$ST,CLASS=$ACTION,ROUTE=$LOG+$UR            @OZ31193
PREJECT  LR    JCT,R0              RESTORE JCT CONTENTS        @OZ31193
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        WAIT FOR OPERATOR RESPONSE -- RE-CHECK I/O                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PWAITOP  DS    0H                  WAIT HERE FOR OPERATOR ACTION     R4
        $WAIT  IO                  WAIT FOR $POST                    R4
         TM    DCTFLAGS,DCTSTOP    TEST FOR $S PRT COMMAND           R4
         BO    PWAITOP             BR IF NOT                         R4
         SPACE 1                                                     R4
        $DOM   CMB=(R1)            DELETE OPERATOR MESSAGE           R4
         B     PPCHECK             GO CHECK FOR OPERATOR ACTION      R4
PEFMSG   DC    CL26' $E NECESSARY FOR RECOVERY'                @OZ31193
         SPACE 1                                                     R4
         DROP  PW                  SUSPEND DCT ADDRESSABILITY        R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- PUNCH ERROR RECOVERY'    R4
***********************************************************************
*                                                                     *
*        ***  PUNCH ERROR RECOVERY AND RESTART  ***                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PUNCHERR DS    0H
         CLC   IOBCSW(3),IOBSTART+1 VERIFY                           R4
         BNH   PUNCHER1             THAT                       @OZ32288
         SH    PW,=H'8'            ADJ CSW ADDR FOR COMPARE    @OZ31058
         CL    PW,PPBLVCCC            ERROR                          R4
         LA    PW,8(,PW)           RE-ADJUST CSW ADDR          @OZ31058
         BNH   PUNCHER1                WAS                           R4
         L     R14,PPBPCIE              IN                           R4
         LA    R1,PCIESIZE(,R14)         ACTIVE                      R4
         CLR   PW,R1                      CCW AREA                   R4
         BL    PCISIMUL             ELSE, GO SIMULATE PCI            R4
         SPACE 1                                                     R4
PUNCHER1 DS    0H                                                    R4
         L     R1,PPBPCIE          ADDRESS 1ST POSSIBLE              R4
         USING PCIDSECT,R14        GAIN ADDRESSABILITY         @OZ31211
         NI    PCISGNAL,255-PCIFNLBF  TURN OFF FINAL BUFFER    @OZ31211
         DROP  R14                 DROP PCIE ADDRESSABILITY    @OZ31211
         SH    R1,PCCWLAST          CCW IN CHAIN                     R4
         SL    PW,=F'16'           DECREMENT RESTART ADDRESS
         CLI   PDEVTYPE+3,X'02'    TEST PUNCH DEVICE TYPE
         BE    PURANGE             BRANCH IF 2540P
         CLI   PDEVTYPE+3,X'0C'    TEST PUNCH DEVICE TYPE
         BE    PU3525              BRANCH IF 3525
         LA    PW,8(,PW)           INCREMENT RESTART ADDRESS BY 8
         B     PURANGE             BRANCH TO CHECK RESTART ADDRESS
PU3525   DS    0H
         CR    PW,R1               IS RESTART ADDRESS BEFORE CCW'S   R4
         BL    PUBACKUP            BRANCH IF YES
         TM    0(PW),X'05'         IS CCW A PUNCH CCW
         BM    PURANGE             BRANCH IF YES
         SL    PW,=F'8'            DECREMENT RESTART ADDRESS
         B     PU3525              CONTINUE BACKWARD SCAN
PUBACKUP DS    0H
         OC    PULMTTR(6),PULMTTR  IS RESTART POINTER ZERO
         BNZ   PURSTRT             BRANCH IF RESTART POINTER NOT ZERO
         LR    PW,R1               SET RESTART ADDRESS TO 1ST CCW
         B     PURANGE             GO RANGE CHECK ADDRESS
PURSTRT  DS    0H
         MVC   PCEJMTTR,PULMTTR    SET RESTART BUFFER MTTR
         MVC   PCEEJRCB,PULRCB     SET RESTART RCB OFFSET
         MVC   PBUFSKIP,PULBOFF    SET RESTART BUFFER OFFSET         R4
         LA    PL,PBSFSGO          SETUP RESTART LOCATION
         B     PPWINIT             EXIT TO COMPLETE RESTART          R4
         SPACE 1                                                     R4
PURANGE  DS    0H
         CR    PW,R1               IS RESTART ADDRESS BEFORE CCW'S   R4
         BL    PUBACKUP            BRANCH IF YES
         CL    PW,PUERRPT          IS RESTART ADDRESS BEFORE LAST
         BNH   PURETRY             BRANCH IF YES
         ST    PW,PUERRPT          NEW ERROR -- SAVE CCW FOR TEST
         TITLE 'HASP PRINT/PUNCH SERVICE -- CHANNEL PROGRAM RESTART' R4
***********************************************************************
*                                                                     *
*        RESTART PRINT/PUNCH CHANNEL PROGRAM AFTER ERROR              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRESTRT  TM    PCEID,PCEPRSID      IS THIS A PRINTER PCE...    @OZ33052
         BO    PPRTHALT            YES..ISSUE MSG AND STOP.    @OZ33052
         TM    IOBCSW+4,X'3F'      TEST FOR CHANNEL FAILURES   @OZ33052
         BZ    PUREST              CONTINUE IF NONE.           @OZ33052
PPRTHALT L     R1,PCEDCT           GET DCT ADDR.               @OZ33052
         LR    R0,JCT              SAVE JCT REG ACROSS $WTO    @OZ20792
         OI    DCTFLAGS-DCTDSECT(R1),DCTSTOP STOP THE DEVICE   @OZ20792
         L     JCT,PJCTBUF         LOAD JCT FOR $WTO           @OZ20792
         SPACE 1                                               @OZ20792
         $MID  191                                             @OZ20792
         MVC   PMESSAGE(2),=X'191F' MOVE MSG ID                @OZ20792
         MVC   PMESSAGE+2(8),DCTDEVN-DCTDSECT(R1) MOVE DEVICE  @OZ20792
         MVC   PMESSAGE+10(29),PURMSG MOVE MSG TEXT            @OZ20792
         $WTO  PMESSAGE,39,JOB=YES, ISSUE PRINTER HALTED MSG   @OZ20792*
               ROUTE=$LOG+$UR,CLASS=$ACTION,PRI=$ST            @OZ20792
         LR    JCT,R0              RESTORE JCT REG CONTENTS    @OZ20792
         SPACE 1                                               @OZ20792
PUREIFC  L     R1,PCEDCT           LOAD DCT ADDRESS            @OZ32566
         USING DCTDSECT,R1         ESTABLISH DCT ADDRESS       @OZ20792
         L     R15,POUTIOB         PICK UP OUTPUT IOB ADDR     @OZ20792
         TM    DCTFLAGS,DCTSTOP    TEST FOR $S DEVICE          @OZ20792
         BZ    PUREST              BRANCH IF TRUE START        @OZ20792
         NI    DCTFLAGS,255-DCTDELET-DCTRSTRT-DCTBKSP RESET    @OZ20792
         DROP  R1                                              @OZ20792
         $WAIT IO                  WAIT FOR POST FROM COMM     @OZ20792
         B     PUREIFC             TEST FOR $S DEVICE          @OZ20792
         SPACE 1                                               @OZ20792
PUREST   ST    PW,IOBSTART         RESTART CCW LIST            @OZ20792
         EJECT                                                 @OZ20792
***********************************************************************
*                                                                     *
*        RESTART PRINT/PUNCH CHANNEL PROGRAM AT IOBSTART              *
*                                                                     *
*        FOR 3525 PUNCH -- MAKE DEVICE 'NOT-READY'                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PURETRY  DS    0H
         CLI   PDEVTYPE+3,X'0C'    TEST DEVICE TYPE
         BNE   PUREXCP             BRANCH IF NOT A 3525 PUNCH
         L     R1,PRPUUCB          ADDRESS 3525 UCB                 R41
        MODESET EXTKEY=ZERO        GET ZERO KEY TO CHANGE THE UCB
         OI    UCBFL1-UCBDSECT(R1),UCBNOTRD  SET NOT-READY          R41
        MODESET EXTKEY=HASP        RETURN TO NORMAL HASP KEY
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        RESET PCI FLAG (IF ANY) IN RESTART CCW                       *
*                                                                     *
*        RE-ISSUE $EXCP (VR) AND RE-CHECK COMPLETION                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PUREXCP  L     R1,IOBSTART         RESET PCI FLAG                    R4
         MVI   PPBFLAG1,X'00'      CLEAR CSW SAVE              @OZ29106
         NI    4(R1),255-X'08'      IN RESTART CCW                   R4
         SPACE 1                                                     R4
         L     R1,PCEDCT           ADDRESS PRINTER/PUNCH DCT   @OZ32566
        $EXCP  (R1),TYPE=VR        RESTART I/O                       R4
         B     PPPWAIT             GO WAIT FOR I/O COMPLETION        R4
PURMSG   DC    CL29' HALTED/POSSIBLE DATA LOST   '             @OZ20792
         TITLE 'HASP PRINT/PUNCH SERVICE -- ERROR DETECTION AND CORRECTC
               ION'                                                  R4
***********************************************************************
*                                                                     *
*        PPCHECK SUBROUTINE -- DETECT PRINT/PUNCH ERRORS AND RECOVER  *
*                                                                     *
*        PL    - RETURN ADDRESS                                       *
*                                                                     *
***********************************************************************
         SPACE 2                                                     R4
***********************************************************************
*                                                                     *
*        ENTRY POINT FOR PCI WAITS                                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCIWAIT  DS    0H                  ENTRY FOR PCI WAITS               R4
         L     R15,POUTIOB         PICK UP OUTPUT IOB ADDRESSABILITY R4
         TM    PPFLAG,PPWSW        TEST FOR WRITE IN PROGRESS        R4
         BZ    PPCHECK             BR IF NOT                         R4
         TM    BUFECBCC,X'7F'      HAS I/O COMPLETED...              R4
         BNZ   PPCHECK             BR IF YES                         R4
         OI    PPFLAG2,PPCIWAIT    INDICATE WAIT FOR PCI             R4
         L     R14,PPBPCIE         PICK UP ACTIVE PCIE ADDRESS       R4
         USING PCIDSECT,R14        ESTABLISH PCIE ADDRESSABILITY     R4
         TM    PCISGNAL,PCIACTIV   IS PCIE ON ACTIVE CHAIN...        R4
         BZ    PPPWAIT             WAIT IF NOT                       R4
         TM    PCISGNAL,PCIBUSY    HAS PCIE EXECUTED...              R4
         BZ    PPCHECK             BR IF YES                         R4
         DROP  R14                 SUSPEND PCIE ADDRESSABILITY       R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        $WAIT PRINT/PUNCH PROCESSOR FOR AN I/O POST                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPPWAIT  DS    0H
        $WAIT  IO                  WAIT FOR IO POST
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        MAIN PPCHECK ENTRY POINT                                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPCHECK  DS    0H
         LTR   JCT,JCT             IS THIS A SPOOL MESSAGE PRINTER
         BZR   PL                  BRANCH IF YES
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE
         BO    PCOMTEST            BRANCH IF REMOTE
         TM    PPFLAG,PPWSW        TEST FOR WRITE INITIATED
         BZ    PPWINIT             BYPASS CHECK IF NO                R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        FIRST -- CHECK FOR OPERATOR CONSOLE ACTION                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PCOMTEST DS    0H
         L     PW,PCEDCT           GET ADDRESS OF PRT/PNCH DCT @OZ32566
         USING DCTDSECT,PW         ESTABLISH DCT ADDRESSABILITY
         NI    PDCTFLAG,255-DCTSPACE RESET OP CARRIAGE CONTROL
         OC    PDCTFLAG,DCTFLAGS   STACK OPERATOR FLAGS
*
*        $C/$E/$I DEVICE                                             R4
*
         TM    DCTFLAGS,DCTDELET+DCTRSTRT NEW $C, $E, OR $I
         BZ    POPCANJ             BRANCH IF NO
         TM    PPFLAG,PRDELSW      PREVIOUS $C, $E, OR $I
         BO    PPABORT             BRANCH IF YES
         B     PRDEL               SET TERMINATION FLAG
*
*        $CJOB,PURGE  (WHEN ISSUED ON ANOTHER SYSTEM)                R4
*
POPCANJ  DS    0H
         L     R1,PCEJQE           ADDRESS OF JQE              @OZ32566
         USING JQEDSECT,R1         ACTIVATE JQE ADDRESSABILITY
         TM    JQEFLAGS,QUEOPCAN+QUEPURGE $CJOB,PURGE ISSUED
         BNO   POPSPACE            BRANCH IF NO
         DROP  R1                  SUSPEND JQE ADDRESSABILITY
         OI    PDCTFLAG,DCTDELET   SET TERMINATION REASON
         B     PRDEL               GO COMPLETE TERMINATION
*
*        $B/$F DEVICE,PAGES OR ,DSET                                 R4
*
POPSPACE DS    0H
         TM    DCTFLAGS,DCTBKSP    BSP/FSP
         BZ    POPRPT              BRANCH IF NO
         NI    PDCTFLAG,255-DCTBKSP RESET FS/BS FLAG
         TM    PPFLAG,PPNEWS       IGNORE $B/$F IF                  R41
         BO    POPRPT               PRINTING JES2-NEWS              R41
         NI    DCTPPFL,255-DCTEJECT  RESET TOP-OF-PAGE FLAG          R4
         TM    PCEID,PCERJEID+PCEPUSID  RJE DEVICE OR PUNCH... @OZ29138
         BNZ   PPDEL                    BR IF YES              @OZ29138
         CLC   DCTCSW,=F'0'        DID INT. REQUIRED OCCUR     @OZ29138
         BE    PPDEL               BR IF NO, SET SUSPEND FLAG  @OZ29138
         ST    PL,PMESSAGE         SAVE LINKAGE REGISTER       @OZ29138
         L     R15,=A(PPCNTPGE)    GET PAGE COUNT ROUTINE ADDR @OZ29138
         BALR  PL,R15              GO AND COUNT PAGES          @OZ29138
         L     PL,PMESSAGE         RESTORE LINKAGE REGISTER    @OZ29138
         B     PPDEL               SET SUSPEND FLAG
         EJECT
*
*        $N DEVICE
*
POPRPT   DS    0H
         TM    PDCTFLAG,DCTRPT     TEST FOR $N
         BNO   PPIOTEST            BRANCH IF NO
         TM    PPFLAG,PRDELSW      PREVIOUS $C, $E, $I         @OZ29256
         BO    PPIOTST             BRANCH IF YES               @OZ29256
         L     R1,PWKJOE           ADDRESS WORK-JOE
         USING JOEDSECT,R1         ACTIVATE JOE ADDRESSABILITY
         TM    JOEFLAG,$JOESPIN    IS THIS A SPIN JOE
         BNO   PREPADD             BRANCH IF NO
         DROP  R1                  SUSPEND JOE ADDRESSABILITY
         IC    R1,PPDSCPY          GET DATASET COPY COUNT
         LA    R1,1(,R1)           INCREMENT
         STC   R1,PPDSCPY          STORE OVER OLD VALUE
         B     PREPWTO             GO TELL THE OPERATOR
*
*        ADD A COPY OF THE WORK-JOE TO THE JOB OUTPUT TABLE
*
PREPADD  DS    0H
        $#ADD  WORK=PWKJOE,CHAR=PCHJOE COPY JOE BACK INTO QUEUE      R4
         BNZ   PPIOTEST            BRANCH IF $#ADD FAILED
*
*        WRITE OPERATOR MESSAGE IF ADD SUCCESSFUL
*
PREPWTO  DS    0H
         NI    PDCTFLAG,255-DCTRPT RESET $N FLAG
        $MID   170                                                   R4
         MVC   PMESSAGE(2),=X'170F' MESSAGE NUMBER
         MVC   PMESSAGE+2(8),DCTDEVN DEVICE NAME
         MVC   PMESSAGE+10(12),=CL12' REPEATED' OPERATOR MESSAGE
        $WTO   PMESSAGE,22,        INFORM THE OPERATOR                 C
               ROUTE=$LOG+$UR,CLASS=$NORMAL,PRI=$ST,JOB=NO
         B     PPIOTEST            CHECK THE IO STATUS
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        INDICATE PROCESSOR SUSPENSION AND/OR TERMINATION             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRDEL    OI    PPFLAG,PRDELSW      TERMINATION - $C, $E, $I
PPDEL    OI    PPFLAG,PPDELSW      SUSPENSION  - $B, $F
         EJECT
***********************************************************************
*                                                                     *
*        CHECK FOR ACTIVE PCIE EXECUTION                              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING PCIDSECT,R14        PROVIDE PCIE ADDRESSABILITY       R4
PPIOTEST DS    0H                                                    R4
         NI    DCTFLAGS,255-DCTDELET-DCTRSTRT-DCTRPT-DCTBKSP RESET   R4
PPIOTST  DS    0H                                              @OZ29256
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BO    PPZTEST             BR IF REMOTE                      R4
         SPACE 1                                                     R4
         L     R15,POUTIOB         PICK UP OUTPUT IOB ADDRESSABILITY R4
         L     R14,PPBPCIE         PICK UP ACTIVE PCIE ADDRESS       R4
         SPACE 1                                                     R4
PPCICHK  DS    0H                                                    R4
         TM    PCISGNAL,PCIBUSY    HAS PCI ELEMENT EXECUTED...       R4
         BO    PPZTEST             BR IF STILL BUSY                  R4
         XC    DCTCSW(4),DCTCSW    CLEAR INT. REQUIRED INDIC   @OZ29138
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        PCIE HAS BEEN EXECUTED -- UPDATE BUFFER AVAILABLE COUNT      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPCIUPDT DS    0H                                                    R4
         NI    PCISGNAL,255-PCIACTIV   FLAG PCIE OFF ACTIVE CHAIN    R4
         TM    PCISGNAL,PCIFNLBF   TEST IF DATA BUFFER(S) FINISHED   R4
         BZ    PPCUPJOE            BR IF NOT                         R4
         IC    R1,PBFAVAIL         INCREMENT NUMBER OF               R4
         LA    R1,1(,R1)            BUFFERS AVAILABLE FOR            R4
         STC   R1,PBFAVAIL           DE-SPOOLING (TO FORCE READ)     R4
         NI    PCISGNAL,255-PCIFNLBF RESET FLAG                      R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SHOW CHECKPOINT NEEDED                                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPCUPJOE DS    0H                                                    R4
         TM    PPFLAG2,PPCKPTA     ARE CHECKPOINTS ALLOWED           R4
         BZ    PPCNCKPT            BR IF NO                          R4
         TM    PCISGNAL,PCICKPT    IS NEW CKPT DATA PRESENT...       R4
         BZ    PPCNCKPT            BR IF NO                          R4
         NI    PCISGNAL,255-PCICKPT  RESET NEW CKPT FLAG             R4
         LA    R1,PCIESIZE(,R14)   POINT TO CHECKPOINT DATA          R4
         ST    R1,PPCKPTR          SAVE FOR PPPCKPT                  R4
         OI    PPFLAG2,PPCKPT      INDICATE CKPT-DATA NEEDS UPDATE   R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IF PCIE NOT LAST ON CHAIN -- UPDATE CCW-AREAS INFORMATION    *
*                                                                     *
***********************************************************************
         SPACE 2                                                     R4
PPCNCKPT TM    PCEID,PCERJEID+PCEPRSID   TEST PROCESSOR TYPE   @OZ30757
         BNZ   PPCHKCC             BR IF NOT LOCAL PUNCH       @OZ30757
         L     R1,PPBCCWNX         PRESET                      @OZ30757
         SH    R1,=H'8'             PUNCH                      @OZ30757
         ST    R1,PUERRPT            ERROR POINTER             @OZ30757
         MVC   PULMTTR(7),PPBCMTTR    AND MTTR                 @OZ30757
         SPACE 1                                               @OZ30757
PPCHKCC  TM    PCI1FLGS,X'40'      CHECK NEXT CCW AREA READY   @OZ30757
         BZ    PPZTEST             BR IF COMMAND-CHAIN NOT ON  @OZ30757
         L     R1,PPBCCWNX         UPDATE IOBSTART TO REFLECT  @OZ30757
         ST    R1,IOBSTART          EXECUTION IN NEXT AREA     @OZ30757
         AH    R1,PCCWLAST         COMPUTE NEW PCIE ADDR AND   @OZ30757
         ST    R1,PPBPCIE           UPDATE PPBPCIE TO SHOW IT  @OZ30757
         L     R1,PPBLVCCN         UPDATE PTR TO LAST VALID    @OZ30757
         ST    R1,PPBLVCCC          CHAN CMND IN CURRENT AREA  @OZ30757
         MVC   PPBCMTTR(7),PPBNMTTR UPDATE RESTART POINTER     @OZ30757
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IF $Z DEVICE -- $WAIT FOR OPERATOR ACTION                    *
*                                                                     *
*        CHECK FOR LOCAL PRINT/PUNCH I/O COMPLETION                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPZTEST  TM    DCTFLAGS,DCTSTOP    TEST FOR OPERATOR $Z              R4
         BO    PPPWAIT             BRANCH IF YES
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BO    PPWINIT             BR IF REMOTE                      R4
         SPACE 1                                                     R4
         TM    BUFECBCC,X'7F'      TEST I/O COMPLETION CODE
         BZ    PCITEST             BRANCH IF NOT CHANNEL END         R4
         BO    PPCGOOD             BRANCH IF NO ERRORS               R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ERROR DETECTED -- START ANALYSIS                             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         L     R1,PRPUUCB          ADDRESS DEVICE UCB
         TM    UCBFLB-UCBDSECT(R1),UCBIORST TEST UCB DEVICE STATUS
         BZ    PDEVOK              BRANCH IF M/P HAS NOT ABORTED
         CLC   $RELEASE,=C'02'     TEST OS/VS2 RELEASE NUMBER        R4
         BE    PPIODRN             BR IF RELEASE 2                   R4
         L     R1,CVTPTR           GET ADDRESS OF CVT                R4
         CLC   CVTCRCA-CVT(,R1),=XL4'0'  TEST RECOVERY MODE          R4
         BNE   PDEVOK              BR IF DEVICE STILL ACCESSIBLE     R4
PPIODRN  DS    0H                                                    R4
         OI    DCTSTAT,DCTDRAIN    FORCE A DRAIN OF THE DEVICE
         OI    PDCTFLAG,DCTRSTRT+DCTBKSP FORCE AN INTERRUPT ($I)
         B     PPABORT             SUSPEND USE OF THE DEVICE
         SPACE 1                                                     R4
PDEVOK   DS    0H
        $COUNT                     ********************************* R4
         TM    PPFLAG,PPDELSW      TEST FOR OPERATOR ACTION
         BO    PPWINIT             BYPASS ERROR RECOVERY IF ABORT    R4
         SPACE 1                                                     R4
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BNE   SKIP330             BR IF NOT                         R4
         TM    DCTPPSW2,DCTNIJAM    ELSE, GO HANDLE IF               R4
         BO    P3800ERR              PAPER JAM                       R4
         SPACE 1                                                     R4
         DROP  PW                  SUSPEND DCT ADDRESSABILITY
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        FOR PRINTERS - IF INVALID CSW -- RETRY USING IOBSTART        *
*                       IF VALID CSW   -- RESTART AT CCW AFTER ERROR  *
*                       IF IN PCIE     -- SIMULATE PCI INTERRUPTION   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
SKIP330  L     PW,IOBCSW-1         PICK UP COMMAND ADDRESS FROM CSW
         LA    PW,0(,PW)           CLEAR HIGH-ORDER BYTE
         TM    PCEID,PCEPUSID      TEST PROCESSOR TYPE               R4
         BO    PUNCHERR            BR IF PUNCH                       R4
         SPACE 1                                                     R4
         CLC   IOBCSW(3),IOBSTART+1  COMPARE CSW WITH FIRST CCW      R4
         BNH   PURETRY             BR IF ZERO OR INVALID             R4
         CL    PW,PPBLVCCC         COMP RESTART CCW WITH END OF AREA R4
         BNH   PRESTRT             BRANCH IF VALID RESTART CCW       R4
         SPACE 1                                                     R4
         L     R14,PPBPCIE         ADDRESS THE PCIE                  R4
         LA    R1,PCIESIZE(,R14)   IF RESTART CCW IS                 R4
         CLR   PW,R1                OUT OF ACTIVE AREA,              R4
         BNL   PURETRY               RESTART AT IOBSTART             R4
         SPACE 1                                                     R4
PCISIMUL DS    0H                                                    R4
         NI    PCISGNAL,255-PCIBUSY  SIMULATE PCI INTERRUPTION       R4
         XC    IOBCSW(3),IOBCSW    CLEAR CSW TO FORCE RETRY          R4
         TM    PCI1FLGS,X'40'      SEE IF NEXT CCW AREA IS READY     R4
         BO    PPCIUPDT            BR IF YES -- RECYCLE              R4
         B     PPWINIT              ELSE GO TO CLEAN UP              R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        NORMAL I/O COMPLETION                                        *
*                                                                     *
*        IF CHE NOT ON EXPECTED PCIE -- HANDLE LOST INTERRUPTS        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPCGOOD  DS    0H                                                    R4
         CL    R14,PPBPCIE         TEST FOR PCIE SWAP                R4
         BE    PPWINIT             BR IF ALL I/O COMPLETE            R4
         L     R14,PPBPCIE         ELSE, ADDRESS NEW PCIE            R4
         TM    PCISGNAL,PCIBUSY    HAS THIS NEW PCIE EXECUTED...     R4
         BO    PUREXCP             RESTART AT IOBSTART IF NOT        R4
         B     PPCIUPDT            ELSE, PROCESS NEW PCIE            R4
         SPACE 1                                                     R4
         DROP  R14                 SUSPEND PCIE ADDRESSABILITY       R4
         SPACE 2                                                     R4
PPWINIT  DS    0H                                                    R4
         NI    PPFLAG,255-PPWSW    INDICATE WRITE COMPLETED          R4
         B     PPPCKPT             GO TO CHECKPOINT ROUTINE          R4
         SPACE 1                                                     R4
PCITEST  DS    0H                                                    R4
         TM    PPFLAG2,PPCIWAIT    TEST FOR PCI WAIT                 R4
         BZ    PPPWAIT             BR IF NO                          R4
         SPACE 1                                                     R4
         DROP  R15                 RE-ESTABLISH DATA BUFFER          R4
         USING BUFDSECT,PBUF        ADDRESSABILITY                   R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- PROCESSOR CHECKPOINT'    R4
***********************************************************************
*                                                                     *
*        CHECKPOINT PROCESSOR STATUS IF THIS CPU OWNS THE CKPT-DATA   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPPCKPT  NI    PPFLAG2,255-PPCIWAIT  RESET PCI-WAIT INDICATION       R4
         TM    PPFLAG2,PPCKPT      CKPT DATA NEED UPDATING...       R41
         BZR   PL                  RETURN IF NOT                    R41
         NI    PPFLAG2,255-PPCKPT  RESET CKPT-NEEDED FLAG            R4
         SPACE 1                                                    R41
PPPCKPT1 TM    PPFLAG2,PPCKPTA     CHECKPOINTS ALLOWED...           R41
         BZR   PL                  RETURN IF NOT                    R41
        $QSUSE TYPE=TEST           DOES THIS CPU OWN THE CKPT DATA   R4
         BZ    PPPCHUSE            BR IF YES                         R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        FORCE CKPT IF $CKPTIME INTERVAL HAS EXPIRED                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         STCK  PCER1               GET CURRENT TIME                  R4
         L     PW,PCER1            UPPER WORD ALMOST SECONDS         R4
         SL    PW,PCEFORM          IF NOT LONGER                     R4
         C     PW,$CKPTIME          THAN LIMIT,                      R4
         BLR   PL                    TRY AGAIN NEXT TIME -- RETURN   R4
        $QSUSE                     REQUEST ACCESS TO CHECKPOINT DATA R4
PPPCHUSE DS    0H                                                    R4
         STCK  PCEFORM             SAVE CKPT TIME FOR NEXT PASS      R4
         L     R1,PCKJOE           ADDRESS OF CKPT-JOE               R4
         USING JOEDSECT,R1         ACTIVATE JOE ADDRESSABILITY       R4
         L     PW,PPCKPTR          PICK-UP CKPT DATA POINTER         R4
         MVC   JOEJRCB(L'PCKPT),0(PW) UPDATE CKPT-JOE                R4
         DROP  R1                  SUSPEND JOE ADDRESSABILITY        R4
        $#CKPT JOE=0(,R1),TYPE=A   SCHEDULE CKPT-JOE FOR CHECKPOINT  R4
         BR    PL                   AND RETURN                       R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- TRACK-CELL READ ROUTINES'
***********************************************************************
*                                                                     *
*        PRDTCNXT - READ NEXT TRACK-CELL OR IOT                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRDTCNXT DS    0H                                                    R4
         TM    PPFLAG2,PPTCEL      TEST DESPOOLING METHOD            R4
         BZ    PRDNXTB             BRANCH IF SINGLE                  R4
         ICM   R15,15,PCENXTRK     TEST FOR ZERO CHAIN               R4
         BZ    SKIP340             BRANCH IF LAST BLOCK READ         R4
         TM    PPFLAG,PPDELSW+PPRDERR TEST FOR SUSPEND OR READ ERROR R4
         BZ    PRDTCEL             BRANCH IF NO                      R4
SKIP340  TM    PPFLAG,PPNEWS       RETURN IF END OF            @OZ25573
         BOR   PL                   JES2-NEWS DATA SET         @OZ25573
         L     R15,PCEIOTTR        GET IOT MTTR                @OZ25573
         B     PRDBUF              GO TO SINGLE BUFFER READ ROUTINE  R4
         SPACE 2                                                     R4
***********************************************************************
*                                                                     *
*        PRDTCEL - READ ENTIRE TRACK-CELL                             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRDTCEL  DS    0H                                                    R4
         TM    PPFLAG2,PPTCEL      TEST DESPOOLING METHOD            R4
         BZ    PRDBUF              BRANCH IF SINGLE                  R4
         ST    PL,PLSAVE           SAVE RETURN REGISTER              R4
         STM   PW,PBUF,PCEWA       SAVE WORKING REGISTERS            R4
         L     PBUF,PINIOB         ACTIVATE INPUT IOB ADDRESSABILITY R4
         ST    R15,PCESEEK         SAVE BUFFER MTTR                  R4
         STCM  R15,8,IOBXTENT      SET IOB EXTENT FIELD TO (M)       R4
         SLR   PL,PL               EXPAND $RECINCR                   R4
         IC    PL,$RECINCR          TO A FULLWORD                    R4
         SLR   R0,R0               COMPUTE                           R4
         SLR   R1,R1                SUB-                             R4
         IC    R1,PCESEEK+3          PERMUTATION                     R4
         DR    R0,PL                  NUMBER            R0 = 000P    R4
         LTR   R0,R0                   OF                            R4
         BNZ   SKIP350                  INITIAL                      R4
         IC    R0,$RECINCR               BUFFER                      R4
SKIP350  STC   R0,PCESEEK          REPLACE MTTR WITH PTTR            R4
         LR    R0,R15              R0 = MTTR            . MTTR       R4
         SRL   R15,24              COMPUTE ADDRESS      . 000M       R4
         MH    R15,=AL2(TEDSIZ)     OF TED FOR THIS     . M * TEDSIZ R4
         AL    R15,$TEDADDR          EXTENT             . TED ENTRY  R4
         USING TEDDSECT,R15        ACTIVATE TED ADDRESSABILITY       R4
         SPACE 1                                                     R4
         SLR   PW,PW               CLEAR TRACK-CELL BUFFER COUNT     R4
         IC    PW,PBUFSKIP         PICK UP RESTART POINT (IF ANY)    R4
         SPACE 1                                                     R4
         L     R14,PBUFSAVE        GET INPUT BUFFER-HEAD             R4
         L     R1,PINMTTRT         GET ADDR OF MTTR/BFR-ADDR TBL     R4
         EJECT                                                       R4
*                                                                    R4
*        BUILD TABLE OF MTTR'S AND CORRESPONDING BUFFER ADDRS        R4
*                                                                    R4
PRDINSRT ST    R0,0(,R1)           PLACE MTTR AND BUFFER-            R4
         ST    R14,4(,R1)           ADDRESS INTO THIS ENTRY          R4
         MVI   BUFECBCC-BUFDSECT(R14),0 RESET ANY ERROR CONDITIONS   R4
         LA    PW,1(,PW)           INCREMENT BUFFER COUNT            R4
         CLM   PW,1,$TCELSIZ       TEST FOR END OF TRACK-CELL        R4
         BNL   PRDTSORT            BR IF YES -- TABLE BUILT          R4
         ALR   R0,PL               R0 = MTTR + $RECINCR              R4
PRECHECK CLM   R0,1,TNRT+1         TEST RECORD NUMBER                R4
         BNH   PRDINCR             BRANCH IF STILL ON TRACK          R4
         CLC   PCESEEK(1),$RECINCR IF RECORD WAS LAST ON TRACK,      R4
         BNL   PRDTSORT             THEN HANDLE -STUNTED- TRACK-CELL R4
         IC    R0,PCESEEK          GET BEGINNING RECORD NUMBER,      R4
         AL    R0,=F'1'             UP IT BY 1, AND SAVE NEW         R4
         STC   R0,PCESEEK            BEGINNING RECORD NUMBER         R4
         B     PRECHECK            RE-CHECK NEW MTTR                 R4
PRDINCR  LA    R1,2*4(,R1)         INCR TO NEXT TABLE EnTRY, AND     R4
         L     R14,BUFCHAIN-BUFDSECT(,R14) STEP TO NEXT BUFFER       R4
         B     PRDINSRT            LOOP UNTIL TABLE BUILT            R4
         DROP  R15                 SUSPEND TED ADDRESSABILITY        R4
*                                                                    R4
*        MTTR/BUFAD TABLE COMPLETE -- SORT IT                        R4
*                                                                    R4
PRDTSORT DS    0H                                                    R4
         L     R14,PBUFSAVE        ADDRESS OF FIRST BUFFER IN CHAIN  R4
         SLR   R15,R15             OBTAIN RESTART                    R4
         IC    R15,PBUFSKIP         BUFFER OFFSET                    R4
         SLR   PW,R15              COMPUTE ACTUAL NUMBER OF BUFFERS  R4
         STC   R15,BUFCHOFF-BUFDSECT(,R14) SAVE OFFSET OF 1ST BUFFER R4
         STH   PW,BUFCHNCT-BUFDSECT(,R14) INSERT BUFFER CHAIN COUNT  R4
         MVI   PBUFSKIP,0          RESET RESTART BUFFER OFFSET       R4
         LR    R15,PW              DECREMENT BUFFER                  R4
         BCT   R15,SKIP360          COUNT BY 1 AND SKIP              R4
         B     PRDBCCWS              THE SORT IF ONLY 1              R4
         SPACE 1                                                     R4
SKIP360  L     R1,PINMTTRT         ADDRESS OF MTTR/BUFAD TABLE       R4
         LA    R14,8               SET BXLE INCREMENT VALUE          R4
         SLL   R15,3               SET BXLE LIMIT VALUE              R4
         ALR   R15,R1               = R1 + 8*(COUNT-1). (LAST ENTRY) R4
         SPACE 1                                                     R4
PRLOOP1  CLR   R1,R15              IF END OF TABLE,                  R4
         BE    PRDBCCWS             BYPASS LAST COMPARE              R4
PRLOOP1R LA    PL,8(,R1)           SET/RESET COMPARE POINTER         R4
PRLOOP2  CLC   3(1,R1),3(PL)       TEST CURRENT REC NBR WITH NEXT    R4
         BH    PRSWAP              BR IF HIGH                        R4
         BXLE  PL,R14,PRLOOP2      INCR AND LOOP FOR COMPARE         R4
         BXLE  R1,R14,PRLOOP1      INCR AND LOOP FOR ENTIRE TABLE    R4
         SPACE 1                                                     R4
PRSWAP   XC    0(8,R1),0(PL)       SWAP POSITIONS OF MTTRS           R4
         XC    0(8,PL),0(R1)        AND BUFFER ADDRESSES WHICH       R4
         XC    0(8,R1),0(PL)         WERE OUT OF ORDER               R4
         B     PRLOOP1R            RESTART CURRENT COMPARE           R4
         EJECT                                                       R4
*                                                                    R4
*        BUILD SET-SECTOR CCW AT IOBCCW1 (IF SUPPORTED)              R4
*                                                                    R4
*        BUILD INPUT CCW'S STARTING AT IOBCCW2                       R4
*                                                                    R4
PRDBCCWS DS    0H                                                    R4
         SLR   R15,R15             OBTAIN                            R4
         IC    R15,0(,R1)           POINTER                          R4
         MH    R15,=AL2(TEDSIZ)      TO                              R4
         AL    R15,$TEDADDR           TRACK-EXTENT-DATA              R4
         USING TEDDSECT,R15        ACTIVATE TED ADDRESSABILITY       R4
         MVC   PCEWC,TNTC          SAVE NUMBER OF TRACKS/CYLINDER    R4
         SPACE 1                                                     R4
PRDSECTR DS    0H                                                    R4
         L     R1,PINMTTRT         POINT TO MTTR/BUFAD TABLE         R4
         TM    $RUNOPTS,$RPS       TEST FOR RPS IN SYSTEM            R4
         BZ    PRDBUILD            BR IF NOT                         R4
         L     PL,TRPS             GET ADDRESS OF RPS TABLE          R4
         MVI   IOBCCW1,X'03'       SET FIRST CHANNEL COMMAND TO NOP  R4
         LTR   PL,PL               TEST IF EXTENT SUPPORTS RPS       R4
         BZ    PRDBUILD            BR IF NO -- SKIP RPS COMPUTATION  R4
         MVI   IOBCCW1,X'23'       SET CHANNEL COMMAND TO SET SECTOR R4
         SLR   R14,R14             CLEAR REGISTER FOR IC             R4
         IC    R14,3(,R1)          GET RECORD NUMBER                 R4
         IC    R14,0(R14,PL)       GET CORRESPONDING SECTOR NUMBER   R4
         STC   R14,IOBCCW1+5       PUT SECTOR NUMBER IN SET-SECTOR   R4
         DROP  R15                 SUSPEND TED ADDRESSABILITY        R4
         SPACE 1                                                     R4
PRDBUILD DS    0H                                                    R4
         LA    PL,IOBCCW2          POINT TO CCW-BUILD AREA           R4
         MVI   IOBCCW4,6           SET OP-CODE TO READ-DATA         R41
         OI    IOBCCW4+4,X'40'     SET COMMAND-CHAIN BIT IN SKELETON R4
         B     SKIP370             FIRST SET OF CCWS ARE BUILT       R4
         SPACE 1                                                     R4
PRDSKEL  LA    PL,3*8(,PL)         POINT TO NEXT CCW-BUILD AREA      R4
         MVC   0(3*8,PL),IOBCCW2       SRCH/TIC/READ CCW SKELETON    R4
SKIP370  L     R14,4(,R1)              ADDRESS OF DATA               R4
         LA    R14,BUFSTART-BUFDSECT(,R14) PORTION OF BUFFER         R4
         LA    R0,3(,R1)               POINT TO CCHHR                R4
         STCM   R0,7,0*8+1(PL)         SRCH TARGET   (CCHHR)         R4
         STCM   PL,7,1*8+1(PL)         TIC TARGET    (SRCH CCW)      R4
         STCM  R14,7,2*8+1(PL)         READ TARGET   (BUF ADDR)      R4
         SPACE 1                                                     R4
         L     R14,0(,R1)          R14 = MTTR                        R4
         LA    R14,0(,R14)         R14 = 0TTR                        R4
         SPACE 1                                                     R4
*                                    MBBCCHHR FROM MTTR CONVERSION   R4
         STC   R14,7(,R1)           -------R                         R4
         SRDL  R14,40               ISOLATE TT IN R14/R15            R4
         D     R14,PCEWC            COMPUTE CYLINDER AND HEAD        R4
         STCM  R14,3,5(R1)          -----HHR     HEAD                R4
         STCM  R15,3,3(R1)          ---CCHHR   CYLINDER              R4
         XC    0(3,R1),0(R1)        000CCHHR     MBB                 R4
         LA    R1,8(,R1)           POINT TO NEXT ENTRY               R4
         BCT   PW,PRDSKEL          LOOP FOR ENTIRE TABLE             R4
         EJECT                                                       R4
*                                                                    R4
*        ISSUE EXCP FOR THIS TRACK-CELL                              R4
*                                                                    R4
         NI    2*8+4(PL),255-X'40' ZERO COMMAND-CHAIN IN LAST CCW    R4
         ST    PBUF,PCEBUFAD       IOB ADDRESS TO DA DCT             R4
         LA    R0,IOBCCW1          RESET IOBSTART                    R4
         ST    R0,IOBSTART          FOR READ                         R4
         L     R1,PINMTTRT         SET IOBSEEK TO 1ST BUFFER'S       R4
         MVC   IOBSEEK(7),1(R1)     TRACK ADDRESS                    R4
         MVC   PCESEEK,$HASPDCB    SET UP DA DCT SO THAT $EXCP       R4
         MVI   PCEDEVTP,DCTINT      WILL NOT CONVERT PCESEEK         R4
         LA    R1,PCEDADCT         POINT TO IOB                      R4
        $EXCP  (R1)                ISSUE READ FOR TRACK-CELL         R4
         OI    PPFLAG2,PPRSW       INDICATE READ IN PROGRESS         R4
         IC    PL,PBFAVAIL         DECREMENT NUMBER                  R4
         BCTR  PL,0                 OF AVAILABLE                     R4
         STC   PL,PBFAVAIL           INPUT BUFFERS                   R4
         LM    PW,PBUF,PCEWA       RESTORE WORKING REGISTERS         R4
         L     PL,PLSAVE           RESTORE RETURN REGISTER           R4
         BR    PL                   AND RETURN                       R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PRDTCHK - WAIT FOR, AND CHECK COMPLETION OF TRACK-CELL READ  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRDTCHK  DS    0H                                                    R4
         TM    PPFLAG2,PPTCEL      TEST DESPOOLING METHOD            R4
         BZ    PRDCHK              BR IF SINGLE                      R4
         ST    PL,PLSAVE           SAVE RETURN REGISTER              R4
         L     PBUF,PINIOB         ACTIVATE INPUT IOB ADDRESSABILITY R4
         L     R1,PBUFSAVE         SWAP                              R4
         MVC   PBUFSAVE,PBUFADDR    TRACK-CELL                       R4
         ST    R1,PBUFADDR           BUFFER POINTERS                 R4
         NI    IOBCCW4+4,255-X'40' TURN OFF COMMAND CHAINING BIT     R4
PRDTCIO  DS    0H                                                    R4
         TM    BUFECBCC,X'7F'      TEST I/O COMPLETION               R4
         BO    PRDTSCAN            BRANCH IF NO ERRORS               R4
         BNZ   PRDTCERR            BRANCH IF ERROR                   R4
        $WAIT  IO                  WAIT FOR CHANNEL END              R4
         B     PRDTCIO              AND CHECK AGAIN                  R4
         SPACE 1                                                     R4
PRDTCERR DS    0H                                                    R4
         L     PL,IOBCSW-1         OBTAIN POINTER                    R4
         LA    PL,0(,PL)            TO FAILING                       R4
         SH    PL,=H'8'              CHANNEL COMMAND                 R4
         LA    R15,3*8             ADDRESS OF                        R4
         MH    R15,BUFCHNCT-BUFDSECT(,R1) LAST POSSIBLE              R4
         LA    R15,IOBCCW1-BUFDSECT(R15,PBUF) READ COMMAND           R4
         CLM   PL,7,IOBSTART+1     VALIDATE CSW                      R4
         BL    SKIP380              CONTENTS AND                     R4
         CLR   PL,R15                USE IOBSTART                    R4
         BL    SKIP390                IF CSW IS                      R4
SKIP380  L     PL,IOBSTART             OUT OF RANGE                  R4
SKIP390  LA    R14,8               BXLE INCREMENT VALUE              R4
PRDTLOCR CLI   0(PL),6             LOCATE READ                       R4
         BE    SKIP400              CCW FOR BUFFER                   R4
         BXLE  PL,R14,PRDTLOCR       IN ERROR                        R4
         SPACE 1                                                     R4
SKIP400  L     R1,0(,PL)           EXTRACT BUFFER ADDRESS            R4
         SL    R1,=A(BUFSTART-BUFDSECT)  ADJUST TO START OF BUFFER   R4
         OI    BUFECBCC-BUFDSECT(R1),PPRDERR SET READ ERROR FLAG     R4
         CLR   PL,R15              WAS ERROR BUFFER LAST IN CHAIN    R4
         BNE   *+6                 BR IF NOT                         R4
         SLR   PL,PL                ELSE CLEAR RESTART ADDRESS       R4
        $IOERROR (PBUF)            RECORD THE BAD NEWS               R4
         LTR   PL,PL               TEST FOR POSSIBLE RESTART         R4
         BZ    PRDTSCAN            BR IF NO -- ALL BUFFERS READ      R4
         LA    PL,8(,PL)           GET RESTART CCW ADDRESS           R4
         ST    PL,IOBSTART          AND PLACE IN IOB                 R4
         L     R1,0(,PL)           GET POINTER TO SEARCH TARGET      R4
         MVC   IOBSEEK+2(5),0(R1)  MOVE CCHHR INTO IOB               R4
         LA    R1,PCEDADCT         PICK UP DA DCT ADDRESS            R4
        $EXCP  (R1)                RESTART CHANNEL PROGRAM           R4
         L     R1,PBUFADDR         PICK UP BUFFER CHAIN HEAD ADDRESS R4
         B     PRDTCIO              AND GO CHECK ON I/O              R4
         SPACE 3                                                     R4
*                                                                    R4
*        ALL BUFFERS HAVE BEEN READ (OR SKIPPED)                     R4
*                                                                    R4
*        VALIDATE CHAIN AND OBTAIN MTTR OF NEXT TRACK-CELL           R4
*                                                                    R4
PRDTSCAN DS    0H                                                    R4
         L     PBUF,PBUFADDR       ADDRESSABILITY ON 1ST BUFFER      R4
         LH    R15,BUFCHNCT        NUMBER OF BUFFERS IN THIS CHAIN   R4
         LA    PBUF,PBUFADDR-(BUFCHAIN-BUFDSECT) INITIAL POSITIONING R4
PRDTSCN1 L     PBUF,BUFCHAIN       ADDRESSABILITY ON NEXT BUFFER     R4
         CLC   HDBKEY,PPKEY        DO DATA AND JOB KEYS MATCH        R4
         BNE   PRDTRDER            BR IF NOT                         R4
         TM    BUFECBCC,PPRDERR    DID READ FOR THIS BUFFER FAIL     R4
         BO    PRDTUPDT            BR IF YES                         R4
         OC    HDBNXTRK,HDBNXTRK   TEST FOR LAST BUFFER IN DATA SET  R4
         BZ    PRDTUPDT            BR IF YES                         R4
         BCT   R15,PRDTSCN1        CHECK ALL BUFFERS IN CHAIN        R4
         MVC   PCENXTRK,HDBNXTRK   MTTR OF NEXT TRACK-CELL           R4
         L     PBUF,PBUFADDR       ESTAB. DATA BUFFER ADDRESSABILITY R4
         LH    R1,BUFCHNCT         OBTAIN BUFFER COUNT               R4
         B     PRDTCRET             AND RETURN                       R4
         SPACE 1                                                     R4
*                                                                    R4
*        BUFFER IN CHAIN IS NOT VALID -- UPDATE CHAIN COUNT          R4
*                                                                    R4
PRDTRDER OI    BUFECBCC,PPRDERR    INDICATE BUFFER VALIDITY ERROR    R4
PRDTUPDT XC    PCENXTRK,PCENXTRK   FORCE END OF                      R4
         XC    HDBNXTRK,HDBNXTRK    BUFFER CHAIN                     R4
         L     PBUF,PBUFADDR       ESTAB. DATA BUFFER ADDRESSABILITY R4
         LH    R1,BUFCHNCT         COUNT OF BUFFERS IN CHAIN         R4
         BCTR  R15,0               COMPUTE NUMBER OF                 R4
         SLR   R1,R15               VALID BUFFERS IN CHAIN           R4
         SPACE 1                                                     R4
PRDTCRET IC    R15,BUFCHOFF        UPDATE BUFFER CHAIN COUNT         R4
         ALR   R1,R15               TO REFLECT RESTART               R4
         STH   R1,BUFCHNCT           OFFSET                          R4
         STC   R15,PCEJBOFF        INIT TRACK-CELL BUFFER OFFSET     R4
         NI    PPFLAG2,255-PPRSW   INDICATE READ COMPLETED           R4
         L     PL,PLSAVE           RESTORE RETURN REGISTER           R4
         BR    PL                   AND RETURN TO CALLER             R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- SINGLE DATA BUFFER READ ROUC
               TINES'                                                R4
***********************************************************************
*                                                                     *
*        SINGLE BUFFER READ ROUTINES -- PRDNXTB -- PRDBUF -- PRDCHK   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
*
*        READ NEXT DATA BLOCK - IOT IF CHAIN TRACK IS ZERO
*
PRDNXTB  ICM   R15,15,HDBNXTRK     IS CHAIN TRACK ZERO
         BZ    *+12                BRANCH IF LAST BLOCK
         TM    PPFLAG,PPDELSW+PPRDERR  SUSPEND OR READ ERROR
         BZ    PRDBUF              BRANCH IF NO
         TM    PPFLAG,PPNEWS       RETURN IF END OF                 R41
         BOR   PL                   JES2-NEWS DATA SET              R41
         L     R15,PCEIOTTR        GET IOT TRACK ADDRESS
         DROP  PBUF                KILL BUFFER ADDRESSABILITY
*
*        READ DATA BUFFER FROM SPOOL PACK
*
PRDBUF   L     R1,PINIOB           PICK UP INPUT IOB ADDRESS         R4
         L     LINK,PBUFSAVE       PICK UP DATA BUFFER ADDRESS       R4
         ST    R1,PCEBUFAD         IOB ADDRESS TO DA DCT             R4
         USING BUFDSECT,R1         ESTABLISH BUFFER ADDRESSABILITY
         LA    LINK,BUFSTART-BUFDSECT(,LINK) START OF DATA AREA      R4
         STCM  LINK,7,IOBCCW4+1    PLACE INTO READ CCW               R4
         LA    LINK,IOBSEEK+2      PLACE SEEK TARGET ADDRESS         R4
         STCM  LINK,7,IOBCCW2+1     INTO SEARCH CCW                  R4
         LA    R0,IOBCCW1          RESET IOBSTART
         ST    R0,IOBSTART         FOR READ
         MVI   PCEDEVTP,PCEDARD    SET DA DCT TO READ                R4
         LA    R1,PCEDADCT         SETUP DCT ADDRESS FOR $EXCP
         ST    R15,PCESEEK         STORE TRACK ADDRESS IN DCT
        $EXCP  (R1)                INITIATE READ
         OI    PPFLAG2,PPRSW       INDICATE READ IN PROGRESS         R4
         IC    R1,PBFAVAIL         DECREMENT NUMBER                  R4
         BCTR  R1,0                 OF AVAILABLE                     R4
         STC   R1,PBFAVAIL           INPUT BUFFERS                   R4
         BR    PL                  RETURN
               EJECT                                                 R4
*
*        WAIT/CHECK SPOOL READ COMPLETION                            R4
*
PRDCHK   DS    0H                  *
         L     PBUF,PBUFSAVE       SWAP PRIMARY                      R4
         MVC   PBUFSAVE,PBUFADDR    AND SECONDARY                    R4
         ST    PBUF,PBUFADDR         BUFFER POINTERS                 R4
         L     R1,PINIOB           PICK UP INPUT IOB ADDRESSABILITY  R4
PRDRECHK TM    BUFECBCC,X'7F'      TEST FOR I/O COMPLETE
         BO    PRDRETRN            BRANCH IF READ SUCCESSFUL         R4
         BNZ   PRDERROR            BRANCH IF ERROR DETECTED
        $WAIT  IO                  WAIT FOR I/O POST
         B     PRDRECHK            GO BACK AND TRY AGAIN
PRDERROR DS    0H
         OI    PPFLAG,PPRDERR      READ ERROR, SET FLAG
        $IOERROR PINIOB            LOG I/O ERROR                     R4
         SPACE 1                                                     R4
PRDRETRN DS    0H                                                    R4
         NI    PPFLAG2,255-PPRSW   INDICATE READ COMPLETED           R4
         BR    PL                  AND RETURN
         DROP  R1                  KILL LOCAL BUFFER ADDRESSABILITY  R4
         USING BUFDSECT,PBUF       ESTAB. DATA BUFFER ADDRESSABILITY R4
         TITLE 'HASP PRINT/PUNCH SERVICE -- PROCESSOR TERMINATION'
***********************************************************************
*                                                                     *
*        CHECK TERMINATION REASON - PUNCH BLANK TO FLUSH PUNCH        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPDONE   DS    0H
         NI    PPFLAG,255-PPDELSW  RESET DELETE FLAG
         TM    PPFLAG,PRDELSW      IS THIS A NORMAL TERMINATION
         BZ    PRTRAILR            BRANCH IF YES
         TM    PCEID,PCEPRSID      IS THIS A PRINT PROCESSOR
         BO    PRABEOJ             BRANCH IF YES
         LM    PC1,PC2,PUCCWBL     LOAD BLANK CARD CCW
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BAL   PL,PPWRITE           INITIATE WRITE
         BAL   PL,PPCHECK            AND CHECK
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        PROVIDE REASON FOR TERMINATION ON PROGRAMMERS LISTING        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRABEOJ  DS    0H                  *
        $MID   170                                                   R4
         LA    R1,=C'$HASP170 '    MESSAGE ID                        R4
         TM    PDCTFLAG,DCTRSTRT+DCTBKSP $I - (INTERRUPT)
         BNO   PGMRST              BRANCH IF NO
         LA    R15,=C' INTERRUPTED'  MESSAGE TEXT                    R4
         OI    PSMFDCI,SMFINTRP    SET SMF FLAG
         B     PGMRMSG             GO WRITE MESSAGE
         SPACE 1                                                     R4
PGMRST   DS    0H
         TM    PDCTFLAG,DCTRSTRT   $E - (RESTART)
         BNO   PGMDEL              BRANCH IF NO
         LA    R15,=C' RESTARTED  '  MESSAGE TEXT                    R4
         OI    PSMFDCI,SMFRESTR    SET SMF FLAG
         B     PGMRMSG             GO WRITE MESSAGE
         SPACE 1                                                     R4
PGMDEL   DS    0H
         TM    PDCTFLAG,DCTDELET   $C - (CANCEL)
         BNO   PGMTRM              BRANCH IF NO
         LA    R15,=C' DELETED    '  MESSAGE TEXT                    R4
         OI    PSMFDCI,SMFOPSTP    SET SMF FLAG
         B     PGMRMSG             GO WRITE MESSAGE
         SPACE 1                                                     R4
PGMTRM   DS    0H
        $MID   185                                                   R4
         LA    R1,=C'$HASP185 '    MESSAGE ID                        R4
         LA    R15,=C' TERMINATED '  MESSAGE TEXT                    R4
PGMRMSG  DS    0H
         BAL   PL,PRMSG            ADD MSG TO OUTPUT, ISSUE WTO      R4
         EJECT
***********************************************************************
*                                                                     *
*        SETUP DEVICE BY CHAR-JOE AND PRINT TRAILER PAGE              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING JOEDSECT,R1         PROVIDE CHAR-JOE ADDRESSABILITY   R4
         SPACE 1                                                     R4
PRTRAILR DS    0H
         L     R15,=A(PPSMF6)      GENERATE FINAL              @OZ32776
         BALR  PL,R15               TYPE-6 SMF RECORD          @OZ32776
         L     R1,PCHJOE           ADDRESS CHAR-JOE
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BNE   PTSETUP             BR IF NOT                         R4
         MVC   SPFORMS(2*4),JOEFORM USE JOE FORMS AND FCB ID         R4
         MVC   SPFLASH,JOEFLASH    SET FLASH (SEP WON'T FLASH)      R41
         MVC   SPMODF,=C'****'     RESET COPY MODIFICATION           R4
         MVI   SPCOPYN,1           FORCE ONLY 1 COPY OF TRAILER      R4
         MVI   SPCOPYS,1           INDICATE STARTING COPY NUMBER     R4
         MVI   SPFLAG,SPSEP        INIT FLAGS FOR SEP PAGE          R41
         TM    JOECFLAG,$JOEBRST   SET FOR                           R4
         BZ    PTRSETUP             NOBURST OR                      R41
         OI    SPFLAG,SPBURST        BURST                           R4
         B     PTRSETUP            ENTER COMMON SETUP                R4
         SPACE 1                                                     R4
PTSETUP  LA    R1,JOEFORM          ADDRESS IMPACT PRINTER SETUP DATA R4
         MVI   PRINDEX,X'81'       RESET 3211 INDEX TO 1
         SPACE 1                                                     R4
PTRSETUP L     R15,=A(PRPUDSV)     CALL DEVICE                       R4
         BALR  PL,R15               SETUP VERIFICATION               R4
         SPACE 1                                                     R4
         DROP  R1                  SUSPEND JOE ADDRESSABILITY        R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        PRODUCE PRINT TRAILER PAGE                                   *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING DCTDSECT,R1         PROVIDE DCT ADDRESSABILITY        R4
         SPACE 1                                                     R4
         TM    PCEID,PCEPUSID      IS THIS A PUNCH PROCESSOR
         BO    PSPINIOT            BRANCH TO CLEANUP IF YES    @OZ25079
         L     R1,PCEDCT           ADDRESS PRINT DCT           @OZ32566
         MVI   DCTACPTN,0          DISABLE COMPACTION FOR SEP       R41
         TM    DCTPPSW,DCTPPSWS    SUPPRESS SEPARATOR PAGE...        R4
         BO    PEDGMARK            BR IF YES                        R41
         LA    R1,=CL5' END'       TRAILER PAGE ID
         BAL   PL,PRINTID          PRINT TRAILER PAGE
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        ADD 3800 EDGE-MARK CCW IF REQUESTED BY DEVICE                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PEDGMARK CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER            R41
         BNE   PSPINIOT            BRANCH IF NOT               @OZ25079
         L     R1,PCEDCT           ADDRESS PRINTER DCT         @OZ32566
         TM    DCTPPSW2,DCTNIMRK   TEST FOR EDGE-MARKING             R4
         BZ    SKIP420             BR IF NO                          R4
         LM    PC1,PC2,PCCWEM      LOAD EDGE-MARK CCW                R4
         BAL   PL,PPPUT            PLACE ON CHAIN                    R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ADD 3800 CLEAR-PRINT CCW TO FLUSH OUTPUT                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
SKIP420  LM    PC1,PC2,PCCWCP      LOAD CLEAR PRINT CCW              R4
         BAL   PL,PPPUT            PLACE ON CHAIN                    R4
         BAL   PL,PPWRITE          INITIATE OUTPUT                   R4
         BAL   PL,PPCHECK          WAIT FOR OUTPUT                   R4
         B     PSPINIOT            CLEANUP                     @OZ25079
         SPACE 1                                                     R4
         DROP  R1                  SUSPEND DCT ADDRESSABILITY        R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        CALL PPSMF6 TO BUILD AND WRITE FINAL TYPE 6 SMF RECORD       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PPABORT  L     BASE2,=A(HASPPPI1)  ENSURE BASE2 RESTORED            R41
         L     R15,=A(PPSMF6)      GENERATE TYPE-6             @OZ32776
         BALR  PL,R15               SMF RECORD                 @OZ32776
         SPACE 2                                                     R4
***********************************************************************
*                                                                     *
*        PURGE SPIN DATA SETS TRACKS FROM THE SPOOL PACKS             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING DCTDSECT,R1         PROVIDE DCT ADDRESSABILITY        R4
         SPACE 1                                                     R4
PSPINIOT DS    0H
         TM    PPFLAG,PPDALOC      IS THIS AN ALLOCATION IOT
         BZ    PRPUEXIT            BRANCH IF NO
         TM    PDCTFLAG,DCTRSTRT+DCTBKSP TERMINATION FOR $I OR $E
         BNZ   PRPUEXIT            BRANCH IF YES - DO NOT $PURGE
         L     R1,PCEDCT           LOAD PRINT/PUNCH DCT BASE   @OZ32566
         CLI   DCTBUFCT,0          IS UR I/O HUNG
         BNE   PRPUEXIT            BRANCH IF YES - CAN'T PURGE
         L     R15,PCEIOTTR        GET IOT TRACK ADDRESS
         BAL   PL,PRDBUF           READ IOT INTO A BUFFER
         BAL   PL,PRDCHK           CHECK READ
         LR    JCT,PBUF            ADDRESS IOT IN BUFFER
         USING IOTDSECT,JCT        ACTIVATE IOT ADDRESSABILITY
         TM    PPFLAG,PPRDERR      TEST FOR I/O ERROR ON READ
         BO    PSPNIOTR            BR IF YES                         R4
         CLC   IOTJBKEY,PPJOBKEY   IS THE IOT VALID
         BE    PSPNIOT1            BRANCH IF YES
PSPNIOTR $DISTERR                  INDICATE CONTROL BLOCK ERROR
         OI    PPFLAG,PPJCTIOT+PRDELSW SET REASON FOR TERMINATION
         B     PRPUEXIT            SKIP REMAINDER OF IOT PURGE
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        RESET SPIN IOT ALLOCATION BIT AND PURGE DATA SET TRACKS      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PSPNIOT1 DS    0H
         L     PL,$IOTPDDB         POINT TO                          R4
         ALR   PL,JCT               1ST (AND ONLY) PDDB IN IOT       R4
         USING PDBDSECT,PL         ACTIVATE PDDB ADDRESSABILITY
         NI    PDBFLAG1,255-PDB1PSO RESET PSO BIT FOR WARM START
         DROP  PL                  SUSPEND PDDB ADDRESSABILITY
         TM    IOTFLAG1,IOT1ALOC   ALLOCATION IOT                    R4
         BZ    PSPNIOT4            NO, SKIP $PURGE                   R4
         NI    IOTFLAG1,255-IOT1ALOC RESET ALLOCATION IOT BIT        R4
         SPACE 1                                                     R4
        $PURGE IOTTGMAP            PURGE TRACKS USED FOR DATASET
PSPNIOT4 MVC   PCESEEK,IOTTRACK    SET IOT TRACK INTO DA DCT         R4
         MVI   PCEDEVTP,PCEDAWR    SET DA DCT TO WRITE
         ST    JCT,PCEBUFAD        SET ADDRESS OF IOT BUFFER
         LA    R1,PCEDADCT         ADDRESS DA DCT
         L     PL,BUFCHAIN         SAVE BUFCHAIN VALUE               R4
        $EXCP  (R1),WAIT=YES       WRITE IOT TO SPOOL                R4
         ST    PL,BUFCHAIN         REPLACE BUFCHAIN VALUE            R4
         BO    PRPUEXIT            BR IF I/O GOOD                    R4
         SPACE 1                                                     R4
PSPNIOTW $DISTERR                  INDICATE CONTROL BLOCK ERROR
         EJECT
***********************************************************************
*                                                                     *
*        RELEASE JCT AND DISPOSE OF JOB OUTPUT ELEMENT                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRPUEXIT L     JCT,PJCTBUF         ADDRESS JCT BUFFER                R4
        $#JCT  FREE                RELEASE JCT BUFFER
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        IF THERE IS A $N STACKED - REQUEUE JOE (NO CKPT DATA)        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PJOEOPN  TM    PDCTFLAG,DCTRPT     $N - (REPEAT)                     R4
         BZ    PJOEOPI             BRANCH IF NO
PJOEPUT $#PUT  WORK=PWKJOE         RETURN TO JOT - NO CHECKPOINT     R4
         B     PPNOJOE             BRANCH TO CONTINUE                R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        TERMINATION FOR $I - REQUEUE JOE (CKPT DATA)                 *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PJOEOPI  TM    PDCTFLAG,DCTRSTRT+DCTBKSP  $I - (INTERRUPT)           R4
         BNO   PJOEOPE             BRANCH IF NO
PJPUTI  $#PUT  WORK=PWKJOE,PRC=PCKJOE  RETURN TO JOT WITH CKPT       R4
         B     PPNOJOE             BRANCH TO CONTINUE                R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        TERMINATION FOR $E - REQUEUE JOE (NO CKPT DATA)              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PJOEOPE  TM    PDCTFLAG,DCTRSTRT   $E - (RESTART)                    R4
         BO    PJOEPUT             BRANCH IF YES                     R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        TERMINATION FOR $C - DELETE JOE                              *
*                                                                     *
*        TERMINATION FOR IO ERROR READING DATA OR CONTROL BLOCKS      *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PJOEIOE $#REM  WORK=PWKJOE         REMOVE WORK-JOE FROM JOT          R4
         SPACE 1                                                     R4
PPNOJOE  MVC   PCEJQE,$ZEROS       CLEAR JQE ADDRESS           @OZ32566
         TM    PCEID,PCELCLID      TEST FOR LOCAL DEVICE             R4
         BO    PAUSCHK             BRANCH IF YES - CHECK FOR PAUSE
         L     R1,PCEDCT           ADDRESS REMOTE DCT          @OZ32566
         USING DCTDSECT,R1         ACTIVATE DCT ADDRESSABILITY
         OI    DCTSTAT,DCTHOLD     SET DEVICE UNAVAILABLE           R41
         TM    PDCTFLAG,DCTDELET+DCTRSTRT CHECK FOR $C OR $E        R41
         BZ    PPCLOSE             BR IF NO, DO NORMAL CLOSE        R41
         TM    PDCTFLAG,DCTRSTRT+DCTBKSP  CHECK FOR $I              R41
         BNO   PPNCLOSE            BR IF NO, USE NEGATIVE CLOSE     R41
PPCLOSE  $EXTP CLOSE,(R1)          CLOSE REMOTE DEVICE              R41
         B     PFRESRCE            GO FREE PRPU RESOURCES            R4
         SPACE 2                                                    R41
PPNCLOSE $EXTP NCLOSE,(R1)         NEGATIVE CLOSE REMOTE DEVICE     R41
         B     PFRESRCE            GO FREE PRPU RESOURCES           R41
         SPACE 1                                                    R41
         DROP  R1                  SUSPEND DCT ADDRESSABILITY       R41
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        IF $T DEVICE,P=YES - ISSUE WTO AND PAUSE DEVICE              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PAUSCHK  L     R1,PCEDCT           ADDRESS PRINT/PUNCH DCT     @OZ32566
         USING DCTDSECT,R1         ACTIVATE DCT ADDRESSABILITY
         TM    DCTPPFL,DCTPAUSE    OPERATOR REQUESTED PAUSE
         BZ    PFRESRCE            BR IF NO -- GO FREE RESOURCES     R4
         OI    DCTSTAT,DCTPAUSE    PAUSE THE DEVICE                  R4
        $MID   175                                                   R4
         MVC   PMESSAGE(2),=X'175F' MESSAGE ID
         MVC   PMESSAGE+2(8),DCTDEVN DEVICE NAME
         DROP  R1                  SUSPEND DCT ADDRESSABILITY
         MVC   PMESSAGE+10(7),=C' PAUSED' ACTION
        $WTO   PMESSAGE,17,        INFORM OPERATOR                     C
               ROUTE=$LOG+$UR,CLASS=$NORMAL,PRI=$ST,JOB=NO
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        *** FREE PRINT/PUNCH CONTROL BLOCKS AND DATA BUFFERS  **     *
*                                                                     *
*        IF TRACK-CELL DE-SPOOLING -- FREE INPUT IOB BUFFER           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PFRESRCE DS    0H                                                    R4
         TM    PPFLAG2,PPRSW       IF A READ IS                      R4
         BZ    SKIP430              OUTSTANDING, CLEAR               R4
         BAL   PL,PRDTCHK            ALL INPUT I/O                   R4
         SPACE 1                                                     R4
SKIP430  TM    PPFLAG2,PPTCEL      FREE IOB BUFFER                   R4
         BZ    PFROUT               USED FOR DE-SPOOLING             R4
        $FREEBUF PINIOB              TRACK-CELL'S                    R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        FOR LOCAL DEVICES -- FREE OUTPUT IOB BUFFER                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING DCTDSECT,R1         PROVIDE DCT ADDRESSABILITY        R4
         SPACE 1                                                     R4
PFROUT   TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BO    PFRBUFS             BR IF REMOTE                      R4
         L     R1,PCEDCT           ADDRESS PRINT/PUNCH DCT     @OZ32566
         CLI   DCTBUFCT,0          HAS ALL OUTPUT I/O COMPLETED...   R4
         BE    PFROIOB             BR IF YES                         R4
        $WAIT  IO                  ELSE, WAIT FOR I/O TO COMPLETE    R4
         B     PFROUT              AND TRY AGAIN                     R4
         SPACE 1                                                     R4
         DROP  R1                  SUSPEND DCT ADDRESSABILITY        R4
         SPACE 1                                                     R4
PFROIOB $FREEBUF POUTIOB           FREE OUTPUT IOB BUFFER            R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        FREE ALL DATA BUFFERS                                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PFRBUFS  L     PC1,PBUFADDR        OBTAIN PRIMARY AND                R4
         L     PC2,PBUFSAVE         SECONDARY DATA BUFFER ADDRESSES  R4
         LTR   PC1,PC1             ANY BUFFERS PRESENT...            R4
         BZ    PFREUNIT            BR IF NOT                         R4
         TM    PPFLAG2,PPTCEL      TEST DE-SPOOLING METHOD           R4
         BO    PFRBUF1             BR IF TRACK-CELL                  R4
         XC    BUFCHAIN-BUFDSECT(,PC1),BUFCHAIN-BUFDSECT(PC1) CLEAR  R4
         XC    BUFCHAIN-BUFDSECT(,PC2),BUFCHAIN-BUFDSECT(PC2)  CHAIN R4
PFRBUF1  CLI   PBUFOPT,1           TEST FOR SINGLE BUFFERING         R4
         BE    PFRBUF2             BR IF YES                         R4
        $FREEBUF (PC1),MULTIPLE    FREE PRIMARY DATA BUFFER(S)       R4
         SPACE 1                                                     R4
PFRBUF2 $FREEBUF (PC2),MULTIPLE    FREE SECONDARY DATA BUFFER(S)     R4
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        FREE PRINT/PUNCH/REMOTE DCT                                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PFREUNIT $FREUNIT PCEDCT           FREE PRINT/PUNCH DCT        @OZ32566
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        PAGE-FREE LOCAL PRINT/PUNCH DATA-EXTENT-BLOCK (DEB)          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BO    PDORMANT            BR IF REMOTE                      R4
         L     R1,PCEDCT           GET LOCAL PRPU DCT ADDRESS  @OZ32566
         USING DCTDSECT,R1         PICK-UP DCB ADDRESS               R4
         L     R1,DCTDCB            FROM DCT                         R4
         USING DCBDSECT,R1         PICK-UP DEB ADDRESS               R4
         L     R1,DCBDEBAD          FROM DCB                         R4
         LA    R0,DEBBASND-DEBDSECT  SIZE OF JES2 DEB                R4
        $PGSRVC FREE,(R1),(R0)     PAGE-FREE THE DEB                 R4
         SPACE 2                                                     R4
***********************************************************************
*                                                                     *
*        INDICATE THAT THE PRINT/PUNCH PROCESSOR IS DORMANT           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PDORMANT DS    0H                                                    R4
        $DORMANT                   INDIACTE PROCESSOR INACTIVE       R4
         SPACE 1                                                     R4
         B     HASPPPI1            RESTART AT PRPU INITIALIZATION    R4
         SPACE 1                                               @OZ32776
         PRINT OFF                 THIS SECTION DELETED BY     @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
         PRINT ON                  THIS SECTION DELETED BY     @OZ32776
         TITLE 'HASP PRINT/PUNCH SERVICE -- SEPARATOR PAGE ROUTINE'
***********************************************************************
*                                                                     *
*        SEPARATOR PAGE ROUTINE                                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRINTID  DS    0H
         ST    PL,PCERETN          SAVE RETURN ADDRESS               R4
         L     JCT,PJCTBUF         PICK UP JCT ADDRESS               R4
         USING JCTDSECT,JCT        ACTIVATE JCT ADDRESSABILITY
         ST    R1,PCEFORM          SAVE ADDRESS OF ID TYPE
         NI    PPFLAG,255-PPDELSW  RESET SUSPEND SWITCH
         CLI   PDEVTYPE+3,UCB3800  TEST FOR 3800 PRINTER             R4
         BNE   BLKLCNT             BR IF NOT                         R4
         CLI   $PRIDCT,0           TEST FOR ZERO LINE COUNT          R4
         BE    PRNOID              BR IF YES                         R4
         LM    PC1,PC2,PCCWEM      SELECT                            R4
         ICM   PC1,8,PXTABCCW       DEFAULT CHARACTER SET           R41
         BAL   PL,PPPUT              FOR ID PAGES                    R4
BLKLCNT  DS    0H                                                    R4
         SLR   PL,PL               REMOTE SEPARATOR                  R4
         IC    PL,$TPIDCT           LINE COUNT                       R4
         TM    PCEID,PCELCLID      TEST FOR LOCAL PRINTER
         BZ    BLKRMT              BRANCH IF NOT LOCAL
         IC    PL,$PRIDCT          LOCAL SEPARATOR LINE COUNT        R4
BLKRMT   DS    0H
         LTR   PL,PL               IS LINE COUNT ZERO
         BZ    PRNOID              BRANCH IF YES
         SPACE 1                                                    R41
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE              R41
         BZ    PBLKTST             BR IF NOT REMOTE                 R41
         L     R1,PCEDCT           ADDRESS PRINTER DCT         @OZ32566
         TM    MDCTFEAT-DCTDSECT(R1),DCTPSHDR IF PTR DOES NOT REQ   R41
         BNO   PBLKTST                         SETUP HDR, SKIP PDIR R41
         MVI   PPDIRID,X'01'       INDICATE SEPARATOR PDIR          R41
         ST    PL,PPDSRCT          USE SEPARATOR LINE CNT FROM HCT  R41
         L     R15,=A(PPDIR)       POINT TO SEPARATOR ROUTINE       R41
         BALR  PL,R15              PUT SEPARATOR PDIR               R41
         L     PL,PPDSRCT          RESTORE SEPARATOR LINE COUNT     R41
PBLKTST  DS    0H                                                   R41
         LA    R1,BUFSTART-BUFDSECT  STARTING LINE DISPLACEMENT      R4
         CLM   PL,1,=AL1(30)       AT LEAST 30 LINES REQUESTED
         BL    BLKSKIP             BRANCH IF NO
         MVC   PCCWORK,JCTJNAME    JOB NAME FROM JOB CARD            R4
         LA    PL,BLKPRT           POINT TO BLOCK LETTER ROUTINE    R41
         BALR  PL,PL               TO PRINT JOBNAME NON-SLANTED,    R41
         NOPR  0                    CHANGE 'BALR' AND 'NOPR'        R41C
                                     TO 'BAL PL,0(,PL)'             R41
         EJECT                                                      R41
         LM    PC1,PC2,PRCCWSP     LOAD SPACE CCW
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         XC    PCCWORK,PCCWORK     CLEAR WORK AREA
         MVC   PCCWORK(1),JCTJOBID GET JOB TYPE
         MVC   PCCWORK+1(4),JCTJOBID+4 AND JOB NUMBER
SKIP490  CLI   PCCWORK+1,C' '          LEFT-                         R4
         BNE   SKIP480                  JUSTIFY                      R4
         MVC   PCCWORK+1(4),PCCWORK+2    JOB                         R4
         B     SKIP490                    NUMBER                     R4
SKIP480  L     PL,PWKJOE           ADDRESS WORK-JOE                  R4
         USING JOEDSECT,PL         ACTIVATE JOE ADDRESSABILITY       R4
         MVC   PCCWORK+6(1),JOECURCL  GET SYSOUT CLASS               R4
         MVI   PCCWORK+7,C' '      SET TRAILING BLANK                R4
         L     R1,PLNDISPL         GET NEW LINE DISPLACEMENT         R4
         LA    PL,BLKPRT           POINT TO BLOCK LETTER ROUTINE    R41
         BAL   PL,0(,PL)           TO PRINT JOBID/CLASS SLANTED,    R41C
                                    CHANGE 'BAL' TO 'BALR PL,PL'    R41C
                                     AND 'NOPR 0'                   R41
         SPACE 1                                                    R41
         LM    PC1,PC2,PRCCWSP     LOAD SPACE CCW
         ICM   PC1,8,=X'13'        SET CCW TO DOUBLE SPACE
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         L     R1,PLNDISPL         GET NEW LINE DISPLACEMENT         R4
BLKSKIP  DS    0H
         LA    PC1,0(R1,PBUF)      GET NEW LINE BUFFER ADDRESS       R4
         L     R1,PCEFORM          ID PAGE TYPE
         MVC   0(6,PC1),=C'***** ' PP001 FRAME CHARACTERS            R4
         MVC   6(121,PC1),5(PC1)   BLANK MIDDLE OF LINE              R4
         MVC   128(4,PC1),0(PC1)   PP129 FRAME CHARACTERS            R4
         L     PL,PWKJOE           ADDRESS WORK-JOE                  R4
         MVC   4(1,PC1),JOECURCL   PP005 SYSOUT CLASS                R4
         MVC   127(1,PC1),JOECURCL PP128 SYSOUT CLASS                R4
         DROP  PL                  DROP JOE ADDRESSABILITY           R4
         MVC   7(5,PC1),0(R1)      PP008 START/END/CONT              R4
         MVC   120(5,PC1),0(R1)    PP121 START/END/CONT              R4
         MVC   14(8,PC1),JCTJOBID  PP015 JOB NUMBER                  R4
         MVC   110(8,PC1),JCTJOBID PP111 JOB NUMBER                  R4
         MVC   24(8,PC1),JCTJNAME  PP025 JOB NAME                    R4
         MVC   100(3,PC1),=C'SYS'  PP101 SYS                         R4
         MVC   104(4,PC1),$SID     PP105 SYSTEM ID                   R4
         MVC   34(20,PC1),JCTPNAME PP035 PROGRAMMER NAME             R4
         MVC   56(4,PC1),=C'ROOM'  PP057 ROOM                        R4
         MVC   61(4,PC1),JCTROOMN  PP062 ROOM NUMBER                 R4
         TIME  DEC                 GET TIME AND DATE
         L     R15,=A(PTIMASK)     MOVE EDIT MASK              @OZ32776
         MVC   67(11,PC1),0(R15)    FOR TIME                   @OZ32776
         CL    R0,=X'12000000'     TEST TIME
         BL    PMORNING            BRANCH IF AM
         MVI   76(PC1),C'P'        CHANGE FROM AM TO PM              R4
         SL    R0,=X'12000000'     SUBTRACT TWELVE HOURS
         EJECT                                                      R41
PMORNING ST    R0,PCCWORK          STORE ADJUSTED TIME
         CLI   PCCWORK,X'00'       TEST FOR ZERO HOURS
         BNE   *+8                 BRANCH IF NOT
         MVI   PCCWORK,X'12'       CONVERT ZERO TO TWELVE
         TM    PCCWORK,X'08'       TEST FOR ADJUSTMENT ERROR
         BZ    *+8                 BRANCH IF NO ERROR
         NI    PCCWORK,X'09'       CORRECT FOR BINARY SUBTRACT ERROR
         ED    66(9,PC1),PCCWORK   PP068 HH.MM.SS                    R4
         BAL   PL,PPDATE           CONVERT DATE TO PRINTABLE        R41
         MVC   67+11(10,PC1),PMESSAGE  PP080 DAY MONTH YEAR         R41
         L     PL,PCEDCT           ADDRESS PRINT DCT           @OZ32566
         USING DCTDSECT,PL         ACTIVATE DCT ADDRESSABILITY
         MVC   90(8,PC1),DCTDEVN   PP091 DEVICE NAME                 R4
         DROP  PL                  SUSPEND DCT ADDRESSABILITY
PRIDOUT  DS    0H                  END OF SETUP
         TM    $PRTOPTS,$PRTRANS   IF TRANSLATION NOT REQ'D,         R4
         BZ    PRIDNOTR             DON'T DO IT                @OZ32776
         CLC   PDEVTYPE+2(2),=AL1(UCB3UREC,UCB3211)  LOCAL 3211...   R4
         BE    PRIDNOTR            BR IF YES                   @OZ32776
         CLC   PDEVTYPE+2(2),=AL1(UCB3UREC,UCB3800)  3800 PRINTER... R4
         BE    PRIDNOTR            BR IF YES                   @OZ32776
         L     R15,=A(PTRTBL)      TRANSLATE UNPRINTABLES      @OZ32776
         TR    0(132,PC1),0(R15)    TO BLANKS                  @OZ32776
PRIDNOTR DS    0H                                              @OZ32776
         SLR   JCT,JCT             INITIALIZE LINE COUNT       @OZ32776
         IC    JCT,$TPIDCT          FOR REMOTE SEPARATOR             R4
         TM    PCEID,PCELCLID      TEST FOR LOCAL PRINT
         BZ    PRIEJECT            BRANCH IF NOT LOCAL
         IC    JCT,$PRIDCT         LOCAL SEPARATOR LINE COUNT        R4
PRIEJECT DS    0H
         CLM   JCT,1,=AL1(30)      TEST FOR BLOCK LETTERS USED       R4
         BL    *+8                 BRANCH IF NO
         SH    JCT,=H'29'          ACCOUNT FOR BLOCK LETTER LINES    R4
         AL    PC1,PRCCWID         CONSTRUCT                         R4
         L     PC2,PRCCWID+4        PRINT CCW                        R4
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BCT   JCT,PPPUT           GENERATE SEPARATOR PAGE           R4
         L     JCT,PJCTBUF         RELOAD JCT ADDRESS                R4
         TM    PPFLAG,PPNEWS       JES2-NEWS AVAILABLE...           R41
         BO    PRNOID              SKIP EJECT IF YES                R41
         LM    PC1,PC2,PRCCWEJ     LOAD EJECT CCW
         BAL   PL,PPPUT            ADD CCW TO CHAIN
PRNOID   BAL   PL,PPWRITE          INITIATE WRITE
         BAL   PL,PPCHECK           AND CHECK
         L     PL,PCERETN          RESTORE RETURN ADDRESS            R4
         BR    PL                   AND EXIT
         TITLE 'HASP PRINT/PUNCH SERVICE -- BLOCK LETTER ROUTINE'
BLKPRT   ST    PL,PCEFORM+4        SAVE RETURN REGISTER              R4
         MVC   PBLKWORK,PCCWORK    SAVE TEXT                         R4
         OC    PCCWORK(8),=8X'C0'  SHIFT ALL TO 4TH QUADRANT
         TR    PCCWORK(8),BLOCKTR-192 TRANSLATE TO INDEX VALUE
         SLR   R4,R4               LINE 0 OF 12
BLKBLD   STH   R4,PCEFORM+8        SAVE LINE COUNTER                 R4
         ST    R1,PLNDISPL         SAVE NEW LINE DISPLACEMENT        R4
         ALR   R1,PBUF             GET NEW LINE BUFFER ADDRESS       R4
         MVI   0(R1),C' '          FILL LINE                         R4
         MVC   1(131,R1),0(R1)      WITH BLANKS                      R4
         LA    R7,7                SCAN FOR                          R4
BLKCENTR LA    R15,PBLKWORK(R7)      LAST                            R4
         CLI   0(R15),C' '            NON-                           R4
         BNE   SKIP510                 BLANK                         R4
         BCT   R7,BLKCENTR              CHARACTER                    R4
SKIP510  LA    R7,1(,R7)           COMPUTE                           R4
         MH    R7,=H'7'             BEGINNING                        R4
         SH    R7,=H'67'             PRINT POSITION                  R4
         LCR   R7,R7                  TO CENTER                      R4
         LA    R5,0(R7,R1)             BLOCK LETTERS                 R4
         TM    PCEFORM+4,X'80'     TEST FOR SLANTING OPTION          R4
         BO    SKIP520             BR IF NO                          R4
         LA    R5,6(,R5)           ELSE, SLANT                       R4
         SRL   R4,1                 BLOCK                            R4
         SLR   R5,R4                 LETTERS                         R4
SKIP520  SLR   R4,R4               SET FOR A LETTER INDEX OF 0       R4
BLKLUP   IC    R7,PBLKWORK(R4)     USE RELATIVE TEXT LETTER          R4
         STC   R7,$POSTSAV          TO FORM BLOCK TEXT               R4
         LA    R7,PCCWORK(R4)      CURRENT LETTER INDEX
         SLR   R15,R15             CLEAR REGISTER
         ICM   R15,1,0(R7)         GET TRANSLATED LETTER INDEX
         BZ    BLKSTOR             BRANCH IF INDEX ZERO
         BCTR  R15,0               DECREMENT BY ONE
         MH    R15,=H'24'          CONVERT TO DISPLACEMENT
         AH    R15,PCEFORM+8       SELECT FOR LINE WITHIN LETTER
         LA    R15,BLOCKA(R15)     LETTER MASK ADDRESS
         ICM   R15,12,0(R15)       LETTER MASK BITS
BLKSTOR  LA    R7,12               BLOCK WIDTH OF 12                 R4
BLKLOOP  ALR   R15,R15             SHIFT LEFT AND TEST HIGH BIT      R4
         BC    12,SKIP530          BRANCH IF OFF                     R4
         MVC   0(1,R5),$POSTSAV    OVERSTORE BLANK TO FORM BLOCK     R4
SKIP530  LA    R5,1(,R5)           INCREMENT COL NUMBER
         BCT   R7,BLKLOOP          BRANCH TO FILL 12 COL'S
         LA    R5,2(,R5)           2 BLANKS BETWEEN BLOCKS
         LA    R4,1(,R4)           STEP TO NEXT LETTER INDEX
         CL    R4,=F'8'            HAVE WE DONE 8 BLOCKS
         BL    BLKLUP              BRANCH IF NO
         LM    PC1,PC2,PRCCWID     GET PRINT CCW
         ALR   PC1,PBUF            ADD BUFFER ORIGIN
         AL    PC1,PLNDISPL         AND LINE DISPLACEMENT            R4
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         L     R1,PLNDISPL         GET NEW                           R4
         LA    R1,132(,R1)          LINE DISPLACEMENT                R4
         LA    R4,132(,R1)         CHECK FOR ROOM                    R4
         CH    R4,$BUFLENG          IN BUFFER                        R4
         BNH   SKIP540             BR IF YES                         R4
         BAL   PL,PPWRITE          FORCE WRITE
         BAL   PL,PPCHECK          CHECK WRITE
         LA    R1,BUFSTART-BUFDSECT  SET STARTING LINE DISPLACEMENT  R4
SKIP540  LH    R4,PCEFORM+8        GET LINE COUNTER
         LA    R4,2(,R4)           STEP TO NEXT LINE
         CH    R4,=H'24'           LAST LINE FINISHED
         BL    BLKBLD              BRANCH IF NO
         ST    R1,PLNDISPL         SAVE NEW LINE DISPLACEMENT        R4
         L     PL,PCEFORM+4        LOAD RETURN REGISTER
         BR    PL                  RETURN TO CALLER
         TITLE 'HASP PRINT/PUNCH SERVICE -- DATE SUBROUTINE'        R41
***********************************************************************
*                                                                     *
*        PPDATE - SUBROUTINE TO FORMAT THE DATE                       *
*                                                                     *
* INPUT  R1    - DATE IN THE FORM 00YYDDDC                            *
*        PL    - RETURN ADDRESS                                       *
*                                                                     *
* OUTPUT R0, R1, PW  - UNPREDICTABLE                                  *
*        PMESSAGE    - FORMATTED DATE IN THE FORM ' DD MMM YY'        *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
PPDATE   LTR   R1,R1               TEST FOR VALID DATE              R41
         BZR   PL                  RETURN IF NO                     R41
         ST    R1,PCCWORK+4        STORE DATE                       R41
         L     PW,=A(PYEARTAB)     COPY DATE CONVERSION TABLE  @OZ32776
         MVC   PMONTHS,0(PW)        FOR POSSIBLE UPDATE        @OZ32776
         TM    PCCWORK+5,X'01'     AJUST                            R41
         BO    PTDEDYR              TABLE                           R41
         TM    PCCWORK+5,X'12'       ON                             R41
         BM    PTDEDYR                LEAP                          R41
         MVI   PFEB,29                 YEARS                        R41
         SPACE 1                                                    R41
PTDEDYR  MVC   PMESSAGE+7(3),=X'402120'  GET PATTERN                R41
         ED    PMESSAGE+7(3),PCCWORK+5    AND EDIT THE YEAR (YY)    R41
         XC    PCCWORK(6),PCCWORK  CLEAR ALL BUT JULIAN DAY         R41
         SLR   R0,R0               CLEAR FOR IC                     R41
         CVB   R1,PCCWORK          CONVERT TO BINARY DAY            R41
         LA    PW,PMONTHS-4        ADDR OF DATE CONVERSION TABLE    R41
         SPACE 1                                                    R41
PTDATLUP SLR   R1,R0               CONVERT                          R41
         LA    PW,4(,PW)            JULIAN DAY                      R41
         IC    R0,0(,PW)             TO                             R41
         CLR   R0,R1                  STANDARD DAY                  R41
         BL    PTDATLUP                *  (R1)                      R41
         SPACE 1                                                    R41
         CVD   R1,PCCWORK          CONVERT TO DECIMAL DAY           R41
         MVI   PMESSAGE,C' '       CLEAR 1ST BYTE OF AREA           R41
         UNPK  PMESSAGE+1(2),PCCWORK+6(2)  PLACE DAY (DD)           R41
         OI    PMESSAGE+2,X'F0'             INTO PMESSAGE           R41
         MVI   PMESSAGE+3,C' '     INSERT DELIMITER                 R41
         MVC   PMESSAGE+4(3),1(PW)  MOVE EBCDIC MONTH (MMM)         R41
         BR    PL                   AND RETURN                      R41
         TITLE 'HASP PRINT/PUNCH SERVICE -- MESSAGE AND COMMENT SUBROUTC
               INES'                                                 R4
***********************************************************************
*                                                                     *
*        PRMSG/PRCOMENT -- ADD MSG TO OUTPUT AND SEND TO OPERATOR     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRMSG    ST    PL,PPLSAVE          SAVE RETURN ADDRESS               R4
         MVI   PMESSAGE,C' '                     BLANK OUT           R4
         MVC   PMESSAGE+1(L'PMESSAGE-1),PMESSAGE  MESSAGE AREA       R4
         MVC   PMESSAGE(9),0(R1)   MOVE IN MESSAGE ID                R4
         PACK  PMSGNO,5(3,R1)      SAVE ID IN PACKED FORM            R4
         L     R1,PCEDCT           GET DCT ADDRESS             @OZ32566
         MVC   PMESSAGE+9(8),DCTDEVN-DCTDSECT(R1)  FILL IN DEV NAME  R4
         MVC   PMESSAGE+17(12),0(R15)  MOVE IN MESSAGE TEXT          R4
         TM    $RUNOPTS,$MSGID     IF MESSAGE IDS REQUESTED,         R4
         BO    SKIP550              BR TO INCLUDE MSG ID             R4
         MVC   PMESSAGE+1(29),PMESSAGE+8 ELSE OVERLAY MSG ID        R41
SKIP550  BAL   PL,PRCOMENT         ADD MESSAGE TO OUTPUT             R4
         LA    R1,PMESSAGE+7       POINT TO MESSAGE TEXT - 2         R4
         TM    $RUNOPTS,$MSGID     IF MESSAGE IDS REQUESTED,         R4
         BO    SKIP560              BR TO INSERT MESSAGE NUMBER      R4
         LA    R1,PMESSAGE           ELSE FIRST UPDATE TEXT ADDRESS  R4
SKIP560  MVC   0(2,R1),PMSGNO      MOVE PACKED MSG NUMBER TO MSG     R4
        $WTO   (R1),22,ROUTE=$LOG+$UR,  INFORM OPERATOR              R4C
               CLASS=$NORMAL,PRI=$ST,JOB=NO                          R4
         L     PL,PPLSAVE          RESTORE RETURN ADDRESS            R4
         BR    PL                   AND RETURN                       R4
         EJECT                                                      R41
***********************************************************************
*                                                                     *
*        PRCOMENT -- ADD MSG TO OUTPUT ONLY                           *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
PRCOMENT TM    PCEID,PCEPUSID      TEST PROCESSOR TYPE               R4
         BOR   PL                  RETURN IF PUNCH
         ST    PL,PCERETN          SAVE RETURN ADDRESS               R4
         LM    PC1,PC2,PRCCWSP     LOAD SPACE CCW
         ICM   PC1,8,=X'13'        FORCE DOUBLE SPACING             R41
         L     PW,PPLC             INCREMENT                         R4
         LA    PW,4(,PW)            PAGE                             R4
         ST    PW,PPLC               LINE COUNT                      R4
         CL    PW,PRLINECT         COMPARE WITH MAXIMUM              R4
         BNH   PRCOMSP             BRANCH IF NOT HIGH
         LA    PW,1                RESET PAGE                        R4
         ST    PW,PPLC              LINE COUNTER                     R4
         LM    PC1,PC2,PRCCWEJ     LOAD EJECT CCW
         SPACE 1                                                    R41
PRCOMSP  BAL   PL,PPPUT            ADD CCW TO CHAIN                  R4
         LA    R1,BUFSTART         FIXED-STORAGE PRINT AREA         R41
         XC    PMESSAGE,BUFSTART             SWAP                   R41
         XC    BUFSTART(L'PMESSAGE),PMESSAGE  MESSAGE AND           R41
         XC    PMESSAGE,BUFSTART               PRINT AREAS          R41
         SPACE 1                                                    R41
         LM    PC1,PC2,PRCCWCOM    LOAD PRINT CCW
         OR    PC1,R1              UPDATE DATA-ADDRESS               R4
         BAL   PL,PPPUT            ADD CCW TO CHAIN
         BAL   PL,PPWRITE          INITIATE WRITE
         BAL   PL,PPCHECK           AND CHECK
         SPACE 1                                                    R41
         XC    PMESSAGE,BUFSTART             SWAP                   R41
         XC    BUFSTART(L'PMESSAGE),PMESSAGE  MESSAGE AND           R41
         XC    PMESSAGE,BUFSTART               PRINT AREAS          R41
         SPACE 1                                                    R41
         L     PL,PCERETN          RESTORE RETURN ADDRESS            R4
         BR    PL                  AND RETURN
         TITLE 'HASP PRINT/PUNCH SERVICE -- VARIABLE STORAGE'
*
*                             MISCELLANEOUS CONSTANTS
*
         PRINT OFF                 THIS SECTION DELETED BY     @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
         PRINT ON                  THIS SECTION DELETED BY     @OZ32776
         SPACE 1                                               @OZ28353
PJOBSTAT DC    CL39'------ JES2 JOB STATISTICS ------'
PXEQDATE DC    CL29' JOB EXECUTION DATE'                            R41
PRDRSTAT DC    X'40206B2020206B202120',CL29' CARDS READ'             R4
PPRTSTAT DC    X'40206B2020206B202120',CL29' SYSOUT PRINT RECORDS'   R4
PPUNSTAT DC    X'40206B2020206B202120',CL29' SYSOUT PUNCH RECORDS'   R4
PXEQSTAT DC    X'4020206B2021204B2020',CL29' MINUTES EXECUTION TIME' R4
*
*                             CCW SKELETONS AND STORAGE
*
PDIRCCW  CCW   X'FE',BUFSTART-BUFDSECT,X'60',68 PDIR CCW            R41
PRCCWEJ  CCW   X'8B',0,X'60',1
PRCCWID  CCW   X'09',*-*,X'60',132                                   R4
PUCCW    CCW   X'41',*-*,X'60',80
PCCW     CCW   *-*,HDBSTART-BUFDSECT,X'60',*-*
*              THIS LINE DELETED BY APAR NUMBER                @OZ32235
*              THIS LINE DELETED BY APAR NUMBER                @OZ32235
PUCCWBL  CCW   X'01',*-*,X'60',1     BLANK SPACER CCW.         @OZ32235
PRCCWSP  CCW   X'1B',0,X'60',1
PRCCWSP1 CCW   X'0B',0,X'60',1                                 @OZ20006
PRCCWCOM CCW   X'09',*-*,X'60',L'PMESSAGE                            R4
PCCWNOP  CCW   X'03',0,X'20',64    COUNT USED AS A BLANK
PUNFOLD  CCW   X'23',0,X'60',1     3211 UNFOLD CONTROL CCW
PUCSGATE CCW   X'EB',0,X'60',1     1403 UCS GATE CONTROL CCW
PUCSBDC  CCW   X'73',0,X'60',1     BLOCK DATA CHECK CCW
PUCSLOAD CCW   X'FB',BUFSTART+1-BUFDSECT,X'60',240 LOAD UCSB CCW
PFCBLOAD CCW   X'63',BUFSTART+2-BUFDSECT,X'60',0 LOAD FCB CCW
*              THIS LINE DELETED BY APAR NUMBER                @OZ32235
*              THIS LINE DELETED BY APAR NUMBER                @OZ32235
PUSPACCW CCW   X'1D',*-*,X'A0',48    BLANK SPACER CCW.         @OZ32235
PCCWEM   CCW   X'17',0,X'60',1     EDGE MARK CCW                     R4
PCCWCP   CCW   X'87',0,X'60',1     CLEAR PRINT CCW                   R4
PCCWOFST CCW   X'07',0,X'60',1     OFFSET-STACK (END-OF-XMISSION)    R4
PCNOPTIC DC    X'03',AL3(*-*),X'20',AL1(0,PCIEPRPU,0)         PCIE   R4
         DC    X'08',AL3(*-*),A(*-*)                        SKELETON R4
*              THIS LINE DELETED BY APAR NUMBER                @OZ32235
PRXTR    TR    0(*-*,PC1),0(R15)   *** EXECUTED ***  &PRTRANS  @OZ32776
         EJECT
***********************************************************************
*                                                                     *
*        PRINT/PUNCH PROCESSOR LITERAL POOL                           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         DS    0D
         LTORG
         EJECT
         PRINT OFF                 THIS SECTION DELETED BY     @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
*                                  THIS LINE DELETED BY APAR   @OZ32776
         PRINT ON                  THIS SECTION DELETED BY     @OZ32776
***********************************************************************
*                                                                     *
*        BLOCK LETTER TABLE                                           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
BLOCKTR  DC    X'0001020304050607080900000000000000'
         DC    X'0A0B0C0D0E0F1011120013000000000000'
         DC    X'1415161718191A1B000000000000'
         DC    X'1C1D1E1F202122232425002627000000'
         SPACE 2                                                     R4
BLOCKA   DC    X'7FE0FFF0C030C030C030FFF0FFF0C030C030C030C030C030'
BLOCKB   DC    X'FFE0FFF0C030C030C060FFC0FFC0C060C030C030FFF0FFE0'
BLOCKC   DC    X'7FE0FFF0C030C000C000C000C000C000C000C030FFF07FE0'
BLOCKD   DC    X'FF80FFC0C060C030C030C030C030C030C030C060FFC0FF80'
BLOCKE   DC    X'FFF0FFF0C000C000C000FF00FF00C000C000C000FFF0FFF0'
BLOCKF   DC    X'FFF0FFF0C000C000C000FF00FF00C000C000C000C000C000'
BLOCKG   DC    X'7FE0FFF0C030C000C000C000C1F0C1F0C030C030FFF07FE0'
BLOCKH   DC    X'C030C030C030C030C030FFF0FFF0C030C030C030C030C030'
BKOCKI   DC    X'7FE07FE0060006000600060006000600060006007FE07FE0'
BLOCKJ   DC    X'3FF03FF0030003000300030003000300C300C300FF007E00'
BLOCKK   DC    X'C030C060C0C0C180C300FE00FE00C300C180C0C0C060C030'
BLOCKL   DC    X'C000C000C000C000C000C000C000C000C000C000FFF0FFF0'
BLOCKM   DC    X'C030E070F0F0D9B0CF30C630C030C030C030C030C030C030'
BLOCKN   DC    X'C030E030F030D830CC30C630C330C1B0C0F0C070C030C010'
BLOCKO   DC    X'FFF0FFF0C030C030C030C030C030C030C030C030FFF0FFF0'
BLOCKP   DC    X'FFE0FFF0C030C030C030FFF0FFE0C000C000C000C000C000'
BLOCKQ   DC    X'7FE0FFF0C030C030C030C030C030C330C1B0C0F0FFE07FB0'
BLOCKR   DC    X'FFE0FFF0C030C030C030FFF0FFE0C300C180C0C0C060C030'
BLOCK$   DC    X'06007FE0FFF0C630E6007FC03FE00670C630FFF07FE00600'
BLOCKS   DC    X'7FE0FFF0C030C000E0007FC03FE000700030C030FFF07FE0'
BLOCKT   DC    X'FFF0FFF00600060006000600060006000600060006000600'
BLOCKU   DC    X'C030C030C030C030C030C030C030C030C030C030FFF07FE0'
BLOCKV   DC    X'C030C030C030C030C030C030C030606030C019800F000600'
BLOCKW   DC    X'C030C030C030C030C030C030C630CF30D9B0F0F0E070C030'
BLOCKX   DC    X'C030C030606030C019800F000F00198030C06060C030C030'
BLOCKY   DC    X'C030C030606030C019800F00060006000600060006000600'
BLOCKZ   DC    X'FFF0FFF0006000C001801FC01FC00C00180030007FF0FFF0'
BLOCK0   DC    X'3FC07FE0C0F0C1B0C330C630CC30D830F030E0307FE03FC0'
BLOCK1   DC    X'06000E001E0006000600060006000600060006007FE07FE0'
BLOCK2   DC    X'7FE0FFF0C0300030003000600180060018006000FFF0FFF0'
BLOCK3   DC    X'7FE0FFF0C0300030003001E001E000300030C030FFF07FE0'
BLOCK4   DC    X'038007800D80198031807FF0FFF001800180018001800180'
BLOCK5   DC    X'FFF0FFF0C000C000C000FF80FFC0006000300030FFF0FFE0'
BLOCK6   DC    X'7FE0FFF0C030C000C000FFE0FFF0C030C030C030FFF07FE0'
BLOCK7   DC    X'FFF0FFE0C0C0018003000600060006000600060006000600'
BLOCK8   DC    X'7FE0FFF0C030C03060603FC03FC06060C030C030FFF07FE0'
BLOCK9   DC    X'7FE0FFF0C030C030C030FFF0FFF000300030C030FFF07FE0'
BLOCK#   DC    X'30C030C0FFF0FFF030C030C030C030C0FFF0FFF030C030C0'
BLOCK@   DC    X'3FC07FE0C030003000301E303F306330C330C3307FE03FC0'
         EJECT                                                 @OZ32776
***************************************************************@OZ32776
*                                                              @OZ32776
*        &PRTRANS TRANSLATE TABLE                              @OZ32776
*                                                              @OZ32776
*        TRANSLATE LOWER-CASE TO UPPER-CASE                    @OZ32776
*        TRANSLATE INVALID PN-TRAIN CHARACTERS TO BLANKS       @OZ32776
*                                                              @OZ32776
***************************************************************@OZ32776
         SPACE 1                                               @OZ32776
PTRTBL   DC    C'                                '  TRANSLATE  @OZ32776
         DC    C'                                '  TABLE      @OZ32776
         DC    C'           .<(+|&&          $*);^' USED TO    @OZ32776
         DC    C'-/         ,%_>?          :#@''="' TRANSLATE  @OZ32776
         DC    C' ABCDEFGHI       JKLMNOPQR      '  ALL        @OZ32776
         DC    C'  STUVWXYZ      0123456789      '  ILLEGAL    @OZ32776
         DC    C' ABCDEFGHI       JKLMNOPQR      '  CHARACTERS @OZ32776
         DC    C'  STUVWXYZ      0123456789      '  TO BLANKS  @OZ32776
         SPACE 5                                               @OZ32776
***************************************************************@OZ32776
*                                                              @OZ32776
*        MISCELLANEOUS CONSTANTS NOT DIRECTLY ADDRESSABLE      @OZ32776
*                                                              @OZ32776
***************************************************************@OZ32776
         SPACE 1                                               @OZ32776
PTIMASK  DC    X'21204B20204B202040C1D4'  TIME MASK            @OZ32776
PYEARTAB DC    AL1(31),C'JAN',AL1(28),C'FEB'  JULIAN           @OZ32776
         DC    AL1(31),C'MAR',AL1(30),C'APR'   TO DAY          @OZ32776
         DC    AL1(31),C'MAY',AL1(30),C'JUN'    AND            @OZ32776
         DC    AL1(31),C'JUL',AL1(31),C'AUG'     MONTH         @OZ32776
         DC    AL1(30),C'SEP',AL1(31),C'OCT'      CONVERSION   @OZ32776
         DC    AL1(30),C'NOV',AL1(255),C'DEC'      TABLE       @OZ32776
         SPACE 1                                               @OZ32776
PMONTHS  EQU   $CSAVREG,12*4       AREA FOR COPY OF ABOVE TABLE@OZ32776
PFEB     EQU   PMONTHS+4           ENTRY FOR FEBRUARY          @OZ32776
         SPACE 1                                               @OZ32776
PASAMCH  DC    X'400BF18BF013601BF293F39BF4A3F5ABF6B3F7BBF8C3F9CBC1D3C2C
               DBC3E34E03E541E681000B' ASA TO MACHINE CNVRT    @OZ33326
PASANUM  EQU   (*-PASAMCH)/2                                   @OZ32776
         SPACE 1                                               @OZ32776
PASAMCH2 DC    X'400BF18BF013601BF293F39BF4A3F5ABF6B3F7BBF8C3F9CBC1D3C2C
               DBC3E34E03E501E641000B' 3525 ASA TO MCH CNVRT   @OZ33326
         TITLE 'HASP PRINT SERVICE  --  PAGE COUNT ROUTINE'    @OZ29138
****************************************************************OZ29138
*                                                              *OZ29138
*        CALCULATE WHERE THE INTERVENTION REQURIED (PAPER JAM, *OZ29138
*        FORMS CHECK, ETC.) OCCURED FOR DATA SET REPOSITIONING *OZ29138
*                                                              *OZ29138
****************************************************************OZ29138
         SPACE 3                                               @OZ29138
PPCNTPGE DS    0H                                              @OZ29138
         LR    R14,R15             GET SUBROUTINE ENTRY ADDR   @OZ29138
         SPACE 1                                               @OZ29138
         USING PPCNTPGE,R14        ESTAB. SUBROUTINE BASE      @OZ29138
         USING DCTDSECT,PW         ESTAB. DCT ADDRESSABILITY   @OZ29138
         SPACE 1                                               @OZ29138
PPCKIO   L     R15,POUTIOB         GET OUTPUT CCW IOB ADDR     @OZ29138
         TM    BUFECBCC-BUFDSECT(R15),X'7F' I/O COMPLETE...    @OZ29138
         BNZ   PPCKCSW             BR IF YES                   @OZ29138
        $WAIT  IO                  WAIT FOR I/O POST           @OZ29138
         B     PPCKIO              ENSURE I/O HAS STOPPED      @OZ29138
         SPACE 1                                               @OZ29138
*        FIND OUT WHICH CCW CHAIN THE CSW BELONGS TO           @OZ29138
         SPACE 1                                               @OZ29138
PPCKCSW  CLC   DCTCSW+1(3),POUTCCWA+1 IS CSW IN ACTIVE AREA... @OZ29138
         BNH   PNXTCHN             BR IF NO, CHECK OTHER       @OZ29138
         L     R1,POUTCCWA         GET ACTIVE AREA POINTER     @OZ29138
         AH    R1,PCCWLAST         POINT TO THE PCIE,          @OZ29138
         LA    R1,PCIESIZE(,R1)     AND THE CCW CHKPT AREA.    @OZ29138
         CLM   R1,7,DCTCSW+1       IS CSW IN ACTIVE AREA...    @OZ29138
         BNH   PNXTCHN             BR IF CSW ADDR IS BEYOND END@OZ29138
         L     R1,POUTCCWA         GET START OF CCW CHAIN ADDR @OZ29138
         B     PFNDEND             FIND END OF CCW AREA        @OZ29138
PNXTCHN  DS    0H                                              @OZ29138
         CLC   DCTCSW+1(3),POUTCCWN+1 IS IT IN THIS AREA...    @OZ29138
         BNH   PPGERET             BR IF NOT THIS AREA EITHER  @OZ29138
         L     R1,POUTCCWN         GET NEXT AREA POINTER       @OZ29138
         AH    R1,PCCWLAST         POINT TO THE PCIE,          @OZ29138
         LA    R1,PCIESIZE(,R1)     AND THE CCW CHKPT AREA.    @OZ29138
         CLM   R1,7,DCTCSW+1       IS CSW IN THIS AREA...      @OZ29138
         BNH   PPGERET             BR IF CSW ADDR IS BEYOND END@OZ29138
         L     R1,POUTCCWN         GET START OF CCW CHAIN ADDR @OZ29138
         SPACE 1                                               @OZ29138
*        CSW IS VALID, FIND THE END OF THE CCW CHAIN.          @OZ29138
         SPACE 1                                               @OZ29138
PFNDEND  DS    0H                                              @OZ29138
         L     R0,DCTCSW           GET CSW ADDRESS AND         @OZ29138
         SH    R0,=H'8'             DECREMENT BY 8             @OZ29138
         LR    R15,R1              GET START OF CCW CHAIN      @OZ29138
         AH    R15,PCCWLAST        POINT TO THE LAST CCW+8     @OZ29138
         LR    R2,R1               GET START OF CCW CHAIN      @OZ29138
PINCR8   LA    R2,8(,R2)           INCREMENT BY ONE CCW        @OZ29138
         CR    R15,R2              END OF CHAIN...             @OZ29138
         BE    PFULLCHN            BR IF NOT A SHORT CHAIN     @OZ29138
         CLI   0(R2),X'08'         IS CCW A TIC (SHORT CHAIN)  @OZ29138
         BNE   PINCR8              BR IF NO, CHECK NEXT CCW    @OZ29138
PFULLCHN DS    0H                                              @OZ29138
         SPACE 1                                               @OZ29138
*        COUNT PAGES FROM END OF CCW CHAIN TO THE CSW          @OZ29138
         SPACE 1                                               @OZ29138
         SLR   R15,R15             CLEAR REGISTER FOR COUNT    @OZ29138
PCNTPGE  DS    0H                                              @OZ29138
         CR    R2,R0               CCW EQUAL TO INT REQ CCW... @OZ29138
         BNH   PCHNEND             BR IF TOP CONTINUE COUNTING @OZ29138
         CLI   0(R2),X'89'         IS CCW A CARRIAGE SKIP...   @OZ29138
         BL    *+8                 BR IF NO, NOT A PAGE        @OZ29138
         LA    R15,1(,R15)         ADD PAGE TO ACCUMULATOR     @OZ29138
         SH    R2,=H'8'            DECREMENT TO THE NEXT CCW   @OZ29138
         B     PCNTPGE             GO CHECK THE NEXT CCW       @OZ29138
         SPACE 1                                               @OZ29138
*        END OF PAGE COUNTING, NOW UPDATE PAGE COUNT IN PCE.   @OZ29138
         SPACE 1                                               @OZ29138
PCHNEND  DS    0H                                              @OZ29138
         LNR   R15,R15             COMPLEMENT FOR SUBTRACT     @OZ29138
         AH    R1,PCCWLAST         POINT TO THE PCIE,          @OZ29138
         LA    R1,PCIESIZE(,R1)     AND THE CCW CHKPT AREA.    @OZ29138
         A     R15,JOEPPCT-JOEDSECT(,R1) UPDATE FROM JOE CNT   @OZ29138
         ST    R15,PDDBPGCT        UPDATE CURRENT PAGE COUNT   @OZ29138
         SPACE 1                                               @OZ29138
         SLR   R1,R1               DOES A                      @OZ29138
         LH    R1,$BSPSIZ           BACKSPACE                  @OZ29138
         LTR   R1,R1                 TABLE EXIST...            @OZ29138
         BZ    PPGERET             BR IF NO, RETURN            @OZ29138
         SH    R1,=H'7'            DECREMENT BY ONE ENTRY      @OZ29138
         LA    R1,PBSPTBL(R1)      GET ADDRESS OF LAST ENTRY   @OZ29138
         XC    0(7,R1),0(R1)       INVALIDATE BACKSPACE TABLE  @OZ29138
         SPACE 1                                               @OZ29138
PPGERET  DS    0H                                              @OZ29138
         L     PW,PCEDCT           REFRESH DCT POINTER         @OZ29138
         BR    PL                  RETURN TO CALLER            @OZ29138
         SPACE 1                                               @OZ29138
         DROP  PW,R14                                          @OZ29138
         TITLE 'HASP PRINT/PUNCH SERVICE -- TYPE 6 SMF RECORD' @OZ32776
***************************************************************@OZ32776
*                                                              @OZ32776
*        PPSMF6 -- SUBROUTINE TO GENERATE TYPE 6 SMF RECORD    @OZ32776
*                                                              @OZ32776
*        PL  - RETURN ADDRESS                                  @OZ32776
*        R15 - ENTRY ADDRESS                                   @OZ32776
*                                                              @OZ32776
***************************************************************@OZ32776
         SPACE 1                                               @OZ32776
         USING SMFDSECT,R1         PROVIDE SMF ADDRESSABILITY  @OZ32776
         USING JCTDSECT,JCT        PROVIDE JCT ADDRESSABILITY  @OZ32776
         USING PPSMF6,BASE2        PROVIDE LOCAL ADDR'BILITY   @OZ32776
         SPACE 1                                               @OZ32776
PPSMF6   DS    0H                                              @OZ32776
         ST    BASE2,PDSVSAVE      SAVE BASE2                  @OZ32776
         ST    PL,PPLSAVE          SAVE RETURN ADDRESS         @OZ32776
         LR    BASE2,R15           LOAD BASE REGISTER          @OZ32776
         CLI   $NUMSMFB,2          TEST SMF BUFFER COUNT       @OZ32776
         BL    PPSMFRET            RETURN IF SMF NOT SUPPORTED @OZ32776
         L     JCT,PJCTBUF         ADDRESS JCT BUFFER          @OZ32776
         TM    JCTSMFLG,JCTNOTY6   SHOULD TYPE 6 BE BYPASSED...@OZ32776
         BO    PPSMFRET            RETURN IF YES               @OZ32776
         SPACE 1                                               @OZ32776
        $GETSMFB WAIT=YES          GET AN SMF BUFFER           @OZ32776
         SPACE 1                                               @OZ32776
***************************************************************@OZ32776
*                                                              @OZ32776
*        BUILD BASE SECTION                                    @OZ32776
*                                                              @OZ32776
***************************************************************@OZ32776
         XC    SMFRDW+2(SMF6END1-SMFRDW-2),SMFRDW+2 CLEAR REC  @OZ32776
         MVC   SMF6JBN,JCTJMRJN    JOBNAME               -JBN  @OZ32776
         MVC   SMF6RST(8),JCTRDRON RDR START TIME/DATE   -RST/D@OZ32776
         MVC   SMF6UIF,JCTUSEID    USER ID               -UIF  @OZ32776
         MVC   SMF6JNM,JCTJOBID+4  JOB NUMBER            -JNM  @OZ32776
         SPACE 1                                               @OZ32776
         L     PW,PWKJOE           CURRENT                     @OZ32776
         USING JOEDSECT,PW          SYSOUT                     @OZ32776
         MVC   SMF6OWC,JOECURCL      CLASS               -OWC  @OZ32776
         SPACE 1                                               @OZ32776
         L     PW,PCEDCT           PROVIDE DCT                 @OZ32566
         USING DCTDSECT,PW          ADDRESSABILITY             @OZ32776
         MVC   SMF6FMN,DCTFORMS    FORMS ID              -FMN  @OZ32776
         MVC   SMF6OUT,DCTDEVN     OUTPUT DEVICE NAME    -OUT  @OZ32776
         MVC   SMF6FCB,DCTFCB      FCB ID                -FCB  @OZ32776
         MVC   SMF6UCS,DCTUCS      UCS ID                -UCS  @OZ32776
         SPACE 1                                               @OZ32776
         MVC   SMF6WST(8),PTIMEON  PRPU START TIME/DATE  -WST/D@OZ32776
         MVC   SMF6NLR,PPLNCDCT    TOTAL RECORD COUNT    -NLR  @OZ32776
         MVC   SMF6PGE,PRPAGECT    TOTAL PAGE COUNT      -PGE  @OZ32776
         MVC   SMF6NDS,PPJNDS      DATA SET COUNT        -NDS  @OZ32776
         MVC   SMF6DCI(1),PSMFDCI  CONTROL INDICATORS    -DCI  @OZ32776
         TM    PPFLAG2,PSMFDSER    IF DATA                     @OZ32776
         BZ    SKIP440              BUFFER ERROR...            @OZ32776
         OI    SMF6IOE,SMFDSER       SET SMF IOE FLAG    -DSER @OZ32776
SKIP440  TM    PPFLAG,PPJCTIOT     IF CONTROL                  @OZ32776
         BNO   SKIP450              BUFFER ERROR...            @OZ32776
         OI    SMF6IOE,SMFCBER       SET SMF IOE FLAG    -CBER @OZ32776
         EJECT                                                 @OZ32776
***************************************************************@OZ32776
*                                                              @OZ32776
*        BUILD JES2-ONLY SECTION                               @OZ32776
*                                                              @OZ32776
***************************************************************@OZ32776
         SPACE 1                                               @OZ32776
SKIP450  MVC   SMF6RTE,DCTNO       DEVICE ROUTE CODE     -RTE  @OZ32776
         SPACE 1                                               @OZ32776
         LA    PC1,SMF6END1-SMF6LN1 LENGTH OF BASE             @OZ32776
         STH   PC1,SMF6LN1           AND JES2-ONLY SECT. -LN1  @OZ32776
         LA    LINK,SMF6END1-SMFRDW TOTAL LENGTH, SO FAR       @OZ32776
         SPACE 1                                               @OZ32776
***************************************************************@OZ32776
*                                                              @OZ32776
*        BUILD NON-IMPACT (3800) PRINTER SECTION               @OZ32776
*                                                              @OZ32776
***************************************************************@OZ32776
         SPACE 1                                               @OZ32776
         CLI   PDEVTYPE+3,UCB3800  NON-IMPACT PRINTER...       @OZ32776
         BNE   PPSMF6A             BR IF NOT                   @OZ32776
         SPACE 1                                               @OZ32776
         LA    PC1,SMF6LN1(PC1)    PROVIDE NON-IMPACT PRINTER  @OZ32776
         USING SMF6NIPX,PC1         SECTION ADDRESSABILITY     @OZ32776
         SPACE 1                                               @OZ32776
         MVC   SMF6CPS,PCOPYGRP    COPY GROUPS           -CPS  @OZ32776
         MVC   SMF6CHR(4*4),DCTCHAR1  MOVE CHARS,        -CHR  @OZ32776
         MVC   SMF6MID,DCTMODF         MODIFY, AND       -MID  @OZ32776
         MVC   SMF6FLI,DCTFLASH         FLASH TO TYPE 6  -FLI  @OZ32776
         LA    R15,SMF6CHR         POINT TO SMF FIELDS         @OZ32776
         LA    R0,4+1+1            CHAR1,2,3,4, MODIFY, FLASH  @OZ32776
         SPACE 1                                               @OZ32776
PPSMFLUP CLC   0(4,R15),=C'****'   TEST FOR NULL               @OZ32776
         BNE   *+10                LEAVE VALUE IF NO           @OZ32776
         MVC   0(4,R15),=C'    '    ELSE MOVE IN BLANKS        @OZ32776
         LA    R15,4(,R15)         INCR SMF FIELD ADDR         @OZ32776
         BCT   R0,PPSMFLUP         CHECK ALL VALUES            @OZ32776
*              THIS LINE DELETED BY APAR NUMBER                @OZ32776
         MVC   SMF6FLC,PFLASHC     COPY FLASH COUNT      -FLC  @OZ32776
         TM    DCTPPSW2,DCTNIBRS   SET                         @OZ32776
         BZ    SKIP460              BURST OR                   @OZ32776
         OI    SMF6BID,SMF6BTS       NOBURST             -BTS  @OZ32776
SKIP460  TM    PPFLAG2,PPOPTJ      SET                         @OZ32776
         BZ    SKIP470             DCB=OPTCD=J                 @OZ32776
         OI    SMF6BID,SMF6OPJ       SPECIFICATION       -OPJ  @OZ32776
         SPACE 1                                               @OZ32776
SKIP470  LA    R0,SMF6END2-SMF6LN2 LENGTH OF NON-              @OZ32776
         STH   R0,SMF6LN2           IMPACT PRINTER SECT. -LN2  @OZ32776
         OI    SMF6IND,SMF6FEXT    SHOW SECTION EXISTS   -IND  @OZ32776
         LA    LINK,SMF6END1-SMFRDW+SMF6END2-SMF6NIPX  NEW LEN @OZ32776
         SPACE 1                                               @OZ32776
         DROP  PC1           SUSPEND EXTENSION ADDRESSABILITY  @OZ32776
         EJECT                                                 @OZ32776
***************************************************************@OZ32776
*                                                              @OZ32776
*        SET HEADER INFORMATION AND QUEUE SMF BUFFER FOR WRITE @OZ32776
*                                                              @OZ32776
***************************************************************@OZ32776
         SPACE 1                                               @OZ32776
PPSMF6A  DS    0H                                              @OZ32776
         STH   LINK,SMFRDW         TOTAL RECORD LENGTH   -RDW  @OZ32776
         MVI   SMF6SBS+1,2         JES2 SUBSYSTEM ID     -SBS  @OZ32776
         MVI   SMFHDRTY,SMFOUTTP   SET RECORD ID TO 6    -HDRTY@OZ32776
         SPACE 1                                               @OZ32776
        $QUESMFB                   QUEUE TYPE 6 FOR WRITE      @OZ32776
PPSMFRET L     BASE2,PDSVSAVE      RESTORE BASE2               @OZ32776
         L     PL,PPLSAVE          RESTORE RETURN REGISTER     @OZ32776
         BR    PL                   AND RETURN TO CALLER       @OZ32776
         SPACE 1                                               @OZ32776
         DROP  R1,PW,BASE2         KILL SMF/DCT ADDRESSABILITY @OZ32776
         TITLE 'HASP PRINT/PUNCH SERVICE -- PDIR SUBROUTINE'        R41
***********************************************************************
*                                                                     *
*        BUILD AND TPPUT A REMOTE SETUP HEADER (PDIR)                 *
*                                                                     *
***********************************************************************
         SPACE 1                                                    R41
PPDIR    DS    0H                  REMOTE SETUP HEADER              R41
         USING PPDIR,BASE2         ESTABLISH ADDRESSABILITY         R41
         ST    BASE2,PDSVSAVE      SAVE OLD BASE2                   R41
         ST    PL,PPLSAVE          SAVE RETURN                      R41
         LR    BASE2,R15           LOAD BASE REGISTER               R41
         USING FMHDSECT-(BUFSTART-BUFDSECT),PBUF  ADDRESSABILITY    R41
         MVI   FMHLNGTH,FMHPDLPP   LENGTH OF PDIR TO BE SENT        R41
         MVI   FMHBYTE1,FMHTYPE2   SET PDIR TYPE INDICATOR          R41
         MVI   FMHSEL,1            SET SELECTION INDICATOR          R41
         MVC   FMHPDRID,PPDIRID    SET PDIR IDENTIFIER              R41
         SPACE 1                                                    R41
PPDDATE  MVC   FMHPDATE,=C'00/00/00' SET DEFAULT DATE               R41
         LA    LINK,PTIMEON        POINT TO PRINT TIME AND DATE     R41
PPDEXDA  UNPK  FMHPDATE+6(3),5(2,LINK) PUT YEAR IN PDIR             R41
         ZAP   PCCWORK(8),4(4,LINK) CONVERT                         R41
         SRP   PCCWORK,64-3,0        YEAR                           R41
         CVB   R1,PCCWORK             TO BINARY                     R41
         L     PW,=A(PYEARTAB)     COPY DATA CONVERSION TABLE  @OZ32776
         MVC   PMONTHS,0(PW)       FOR POSSIBLE UPDATE         @OZ32776
         SLR   R0,R0               CLEAR REGISTER                   R41
         LA    R0,3                PREPARE TEST FOR LEAP YEAR       R41
         NR    R0,R1               IF NOT LEAP YEAR                 R41
         BNZ   PPDCONV              USE TABLE AS IS                 R41
         MVI   PFEB,29             SET TABLE FOR LEAP YEAR          R41
PPDCONV  ZAP   PCCWORK(8),6(2,LINK)     CONVERT MONTH AND           R41
         CVB   R1,PCCWORK          DAY TO BINARY                    R41
         SLR   R0,R0               CLEAR REGISTER                   R41
         LA    PW,PMONTHS-4        PREPARE TO SCAN CONVERSION TABLE R41
         LA    PC1,0               START WITH MONTH 0               R41
PPDDTLP  LA    PC1,1(,PC1)         INCREMENT MONTH                  R41
         SR    R1,R0               CONVERT                          R41
         LA    PW,4(,PW)            FROM JULIAN                     R41
         IC    R0,0(,PW)             DATE TO                        R41
         CLR   R0,R1                  MONTH                         R41
         BL    PPDDTLP                 AND DAY                      R41
         CVD   PC1,PCCWORK         PREPARE MONTH                    R41
         UNPK  FMHPDATE(2),PCCWORK+6(2) PUT MONTH IN PDIR           R41
         OI    FMHPDATE+1,X'F0'    MAKE LAST CHARACTER PRINTABLE    R41
         CVD   R1,PCCWORK          PREPARE DAY                      R41
         UNPK  FMHPDATE+3(2),PCCWORK+6(2) PUT DAY IN PDIR           R41
         OI    FMHPDATE+4,X'F0'    MAKE LAST CHARACTER PRINTABLE    R41
         SPACE 1                                                    R41
PPDTIME  MVC   FMHPDTIM,=C'00.00.00' SET DEFAULT TIME               R41
         L     PC2,0(,LINK)        PREPARE TIME                     R41
         SLR   PC1,PC1              FOR DIVISION                    R41
         D     PC1,=A(60*60*100)   CALCULATE HOURS                  R41
         CVD   PC2,PCCWORK         PREPARE HOURS                    R41
         UNPK  FMHPDTIM(2),PCCWORK+6(2) PUT HOURS IN PDIR           R41
         OI    FMHPDTIM+1,X'F0'    MAKE UNITS DIGIT PRINTABLE       R41
         SRDL  PC1,32              PREPARE REMAINDER FOR DIVISION   R41
         D     PC1,=A(60*100)      CALCULATE MINUTES                R41
         CVD   PC2,PCCWORK         PREPARE MINUTES                  R41
         UNPK  FMHPDTIM+3(2),PCCWORK+6(2) PUT MINUTES IN PDIR       R41
         OI    FMHPDTIM+4,X'F0'    MAKE UNITS DIGIT PRINTABLE       R41
         CVD   PC1,PCCWORK         PREPARE SECONDS                  R41
         UNPK  FMHPDTIM+6(4),PCCWORK+5(3) PUT SECONDS IN PDIR       R41
         SPACE 1                                                    R41
PPDFROMS MVI   FMHPDFRM,C' '       CLEAR REST OF PDIR               R41
         MVC   FMHPDFRM+1(FMHPDEND-FMHPDFRM-1),FMHPDFRM             R41
         L     R1,PCEDCT           ADDRESS PRINTER DCT         @OZ32566
         USING DCTDSECT,R1                                          R41
         MVC   FMHPDFRM(4),DCTFORMS PUT FORMS IN PDIR               R41
         SPACE 1                                                    R41
PPDFCB   MVC   FMHPDFCB(4),DCTFCB  PUT FCB NAME IN PDIR             R41
         SPACE 1                                                    R41
PPDTRAIN MVC   FMHPDUCS(4),DCTUCS  PUT UCS NAME IN PDIR             R41
         SPACE 1                                                    R41
         DROP  R1                                                   R41
         SPACE 1                                                    R41
         L     R1,PPDSRCT                   CONVERT RECORD COUNT    R41
         CVD   R1,PCCWORK                    TO DECIMAL             R41
         MVC   FMHPDCNT,=X'4020202020202120'  AND EDIT              R41
         ED    FMHPDCNT,PCCWORK+4              INTO PDIR            R41
         SPACE 1                                                    R41
         MVC   FMHPDJOB,JCTJNAME   SET JOBNAME IN PDIR              R41
         SPACE 1                                                    R41
         SLR   PW,PW               COMPUTE                          R41
         SLR   R1,R1                NUMBER                          R41
         CLI   PPDIRID,X'01'         OF                             R41
         BE    PPDCOPY                REMAINING                     R41
         IC    PW,PPDSCPY              COPIES                       R41
         IC    R1,PPRCPYCT              MINUS                       R41
         SLR   PW,R1                     ONE                        R41
         BCTR  PW,0                       = ADDTIONAL COPIES        R41
         SPACE 1                                                    R41
PPDCOPY  CVD   PW,PCCWORK          CONVERT COPY COUNT TO DECIMAL    R41
         MVC   FMHPDCPY+2(6),=X'402020202120'  SET EDIT PATTERN     R41
         ED    FMHPDCPY+2(6),PCCWORK+5  EDIT COUNT INTO PDIR        R41
         SPACE 1                                                    R41
PPDPUT   LM    PC1,PC2,PDIRCCW     INITIALIZE                       R41
         ALR   PC1,PBUF             OUTPUT                          R41
         STM   PC1,PC2,POUTCCWA      CCW                            R41
         LA    R0,POUTCCWA         PICK UP ADDRESS OF CCW           R41
         L     R1,PCEDCT           PICK UP DCT ADDRESS         @OZ32566
         $EXTP PUT,(R1),(R0)       PASS CCW TO RTAM                 R41
         BNM   PPDEXIT             BR IF PDIR ACCEPTED              R41
         SPACE 1                                                    R41
         LA    PW,0                ELIMINATE PDIR ADDITIONAL COPIES R41
         B     PPDCOPY              AND RESEND PDIR                 R41
         SPACE 1                                                    R41
         SPACE 1                                                    R41
PPDEXIT  SLR   R1,R1               DECREMENT                        R41
         IC    R1,PPDSCPY           REMAINING                       R41
         SLR   R1,PW                 NUMBER                         R41
         STC   R1,PPDSCPY             OF COPIES                     R41
         L     BASE2,PDSVSAVE      RESTORE BASE2                    R41
         L     PL,PPLSAVE          RESET RETURN REGISTER            R41
         BR    PL                  RETURN                           R41
         SPACE 3                                                    R41
FMHPDLPP EQU   FMHPDSTP-FMHDSECT   LENGTH OF PDIR SENT BY HASPPRPU  R41
         SPACE 3                                                    R41
         USING BUFDSECT,PBUF       RE-ESTABLISH ADDRESSABILITY      R41
         EJECT                                                      R41
         LTORG                     SETUP HEADER LITERAL POOL        R41
         TITLE 'HASP PRINT/PUNCH SERVICE -- DEVICE SETUP VERIFICATION'
***********************************************************************
*                                                                     *
*        DEVICE SETUP/VERIFICATION FOR PUNCHES AND IMPACT PRINTERS    *
*                                                                     *
*        PL  - RETURN ADDRESS                                         *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
PRPUDSV  DS    0H                                                    R4
         ST    BASE2,PDSVSAVE      SAVE BASE                         R4
         LR    BASE2,R15           SETUP LOCAL                       R4
         USING PRPUDSV,BASE2        ADDRESSABILITY                   R4
         CLI   PDEVTYPE+3,UCB3800  TEST FOR NON-IMPACT PRINTER       R4
         BNE   SKIP570             BRANCH IF NOT                     R4
         L     R15,=A(P3800DSV)    ELSE, GO TO 3800                  R4
         B     P3800DS              DEVICE SETUP VERIFICATION        R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        DETERMINE IF AN OPERATOR MESSAGE IS NECESSARY                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
SKIP570  ST    PL,PCERETN          SAVE RETURN ADDRESS               R4
         L     JCT,PJCTBUF         ADDRESS JCT BUFFER                R4
         USING JCTDSECT,JCT        ACTIVATE JCT ADDRESSABILITY       R4
         LR    PL,R1               COPY PARAMETER REGISTER           R4
         L     PC1,PCEDCT          ADDRESS PRINT/PUNCH DCT     @OZ32566
         STCM  PC1,7,BUFDCT+1      ENSURE CORRECT DCT ADDR IN BUFFER R4
         USING DCTDSECT,PC1        ACTIVATE DCT ADDRESSABILITY       R4
         NI    DCTPPSW,255-DCTPPSWO RESET OPERATOR $T ALLOWED        R4
         MVC   PCEFORM(12),DCTFORMS SAVE CURRENT DEVICE SETUP        R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        CHECK FORMS                                                  *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVFORM  DS    0H                                                    R4
         CLC   DCTFORMS,0(PL)      FORMS CHANGE                      R4
         BE    DSVFCB              BRANCH IF NO                      R4
         OI    DCTPPSW,DCTPPSWO    SET OPERATOR $T ALLOWED           R4
         MVC   DCTFORMS,0(PL)      NEW FORMS ID TO DCT               R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        CHECK FCB AND INDEX                                          *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVFCB   DS    0H                                                    R4
         TM    PCEID,PCEPUSID      IS THIS A PUNCH PROCESSOR         R4
         BO    DSVMSG              BRANCH IF YES                     R4
         TM    PRINDEX,X'C0'       WAS AN INDEX VALUE SUPPLIED       R4
         BZ    SKIP580             BRANCH IF NO - AVOID RELOAD       R4
         CLC   PRINDEX,DCTINDEX    DOES NEW INDEX = OLD              R4
SKIP580  BE    *+8                 BRANCH IF YES - AVOID RELOAD      R4
         OI    DCTPPSW,DCTPPSWC    SET FCB LOAD REQUIRED SW          R4
         CLC   DCTFCB,4(PL)        FCB CHANGE                        R4
         BE    DSVUCSB             BRANCH IF NO                      R4
         CLC   4(4,PL),=CL4'****'  IS STANDARD FCB REQUESTED         R4
         BNE   DSVFCB05            BRANCH IF NO                      R4
         TM    DCTPPSW,DCTPPSWB    IS DEVICE FCB STANDARD            R4
         BZ    DSVUCSB             BRANCH IF YES                     R4
         CLC   DCTFCB,$PRTFCB      IF INSTALLATION DEFAULT IS        R4
         BNE   DSVFCB05              CURRENTLY LOADED,               R4
         NI    DCTPPSW,X'FF'-DCTPPSWB RESET 'NON-STD' BIT            R4
         B     DSVUCSB               AND BRANCH TO TEST UCS          R4
         SPACE 1                                                     R4
DSVFCB05 DS    0H                                                    R4
         OI    DCTPPSW,DCTPPSWB+DCTPPSWC NON-STD+CARRIAGE LOAD       R4
         L     R0,4(,PL)           PICK UP REQUESTED FCB             R4
         CL    R0,=CL4'****'       CHECK FOR DEFAULT SPECIFICATION   R4
         BNE   SKIP590               SKIP IF NOT                     R4
         L     R0,$PRTFCB          REPLACE WITH DEFAULT NAME         R4
         NI    DCTPPSW,X'FF'-DCTPPSWB RESET 'NON-STANDARD' BIT       R4
SKIP590  ST    R0,DCTFCB           STORE FCB ID IN DCT               R4
         CLI   PDEVTYPE+3,X'09'    IS DEVICE 3211                    R4
         BE    DSVUCSB             BRANCH IF YES                     R4
         OI    DCTPPSW,DCTPPSWO    SET OPERATOR $T ALLOWED           R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        CHECK UCS                                                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVUCSB  DS    0H                                                    R4
         CLC   DCTUCS,8(PL)        UCSB CHANGE                       R4
         BE    DSVMSG              BRANCH IF NO                      R4
         CLC   8(4,PL),=CL4'****'  IS STANDARD UCS REQUESTED         R4
         BNE   DSVUCSB5            BRANCH IF NO                      R4
         TM    DCTPPSW,DCTPPSWU    IS DEVICE UCS STANDARD            R4
         BZ    DSVMSG              BRANCH IF YES                     R4
         CLC   DCTUCS,$PRTUCS      IF INSTALLATION DEFAULT IS        R4
         BNE   DSVUCSB5              CURRENTLY LOADED,               R4
         NI    DCTPPSW,X'FF'-DCTPPSWU RESET 'NON-STD' BIT            R4
         B     DSVMSG                AND BRANCH TO ISSUE MESSAGE     R4
DSVUCSB5 DS    0H                                                    R4
         OI    DCTPPSW,DCTPPSWO+DCTPPSWT+DCTPPSWU MSG+NONSTD+TRAIN   R4
         L     R0,8(,PL)           PICK UP REQUESTED UCS             R4
         CL    R0,=CL4'****'       CHECK FOR DEFAULT SPECIFICATION   R4
         BNE   SKIP600             SKIP IF NOT                       R4
         L     R0,$PRTUCS          REPLACE WITH DEFAULT NAME         R4
SKIP600  ST    R0,DCTUCS           STORE UCS ID IN DCT               R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        ISSUE LOCAL OPERATOR SETUP MESSAGE                           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVMSG   DS    0H                                                    R4
*              THIS LINE DELETED BY APAR NUMBER                @OZ18398
*              THIS LINE DELETED BY APAR NUMBER                @OZ18398
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE              R41
         BZ    DSVMSG1             BR IF NOT REMOTE                 R41
         TM    MDCTFEAT,DCTPSHDR   IF A SETUP HEADER WILL BE SENT   R41
         BO    DSVEXIT             EXIT IF YES                 @OZ18398
DSVMSG1  TM    DCTPPSW,DCTPPSWO    SETUP MESSAGE REQUIRED...   @OZ18398
         BZ    DSVLUCSB            BRANCH IF NO                @OZ18398
         SPACE 1                                                     R4
        $MID   190                                                   R4
         SPACE 1                                                    R41
         MVC   BUFSTART+7(2),=X'190E' MOVE MESSAGE ID          @OZ18398
         MVC   BUFSTART+9(8),JCTJOBID   MOVE JOB NUMBER              R4
         MVI   BUFSTART+17,C' '         SPACE BEFORE JOB NAME        R4
         MVC   BUFSTART+18(8),JCTJNAME  MOVE JOB NAME                R4
         MVC   BUFSTART+26(54),=CL54' SETUP -- PRINTER1 -- F = FORM -- C
               C = FCBI -- T = UCSI'    MOVE SETUP MESSAGE           R4
         MVC   BUFSTART+36(8),DCTDEVN   MOVE DEVICE NAME             R4
         MVC   BUFSTART+52(4),DCTFORMS  MOVE FORMS ID                R4
         MVC   BUFSTART+64(4),DCTFCB    MOVE FCB ID                  R4
         MVC   BUFSTART+76(4),DCTUCS    MOVE UCS ID                  R4
         LA    R0,73               LENGTH FOR PRINTER MESSAGE        R4
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE               R4
         BO    *+8                 BRANCH IF PRINT                   R4
         LA    R0,49               LENGTH FOR PUNCH MESSAGE          R4
         LA    R15,DSVWTOA         POINT TO REG. $DOMACT $WTO LIST   JN
         TM    PCEID,PCERJEID      IF THIS IS A LOCAL DEVICE,        JN
         BZ    DSVWTO                GO ISSUE $WTO                   JN
         L     R1,DCTDCB           POINT TO RAT ENTRY                JN
         L     R1,MDCTRAT-DCTDSECT(,R1)  FOR THIS REMOTE             JN
         TM    RATCONF-RATDSECT(R1),RATCONFI IF 'INFO' SETUP         JN
         BZ    DSVWTO                NOT REQUESTED, GO ISSUE $WTO    JN
         LA    R15,DSVWTOI         POINT TO 'INFO' $DOMACT $WTO LIST JN
         SPACE 1                                                     JN
DSVWTO  $WTO   BUFSTART+7,(R0),MF=(E,0(,R15)) ISSUE LOCAL SETUP      JN
         TM    PCEID,PCERJEID      IS THIS A REMOTE TERMINAL         R4
         BZ    DSVSTOP             BR IF NOT                         R4
         TM    MDCTSTAT,DCTABORT   TEST RJE STATUS                   R4
         BO    DSVSTART            BR IF ABORTING                    R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        ISSUE REMOTE TERMINAL OPERATOR SETUP MESSAGE                 *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         ST    R1,PPLC             SAVE $DOM CMB ADDRESS             R4
         LA    R0,73               LENGTH FOR PRINTER MESSAGE        R4
         TM    PCEID,PCEPRSID      TEST PROCESSOR TYPE               R4
         BO    SKIP610             BRANCH IF PRINT                   R4
         LA    R0,49               LENGTH FOR PUNCH MESSAGE          R4
SKIP610  L     R1,DCTDCB           ADDRESS LINE DCT                  R4
         L     R1,MDCTRAT-DCTDSECT(,R1) ADDRESS REMOTE CONSOLE       R4
         ICM   R0,2,RATCONRT+1-RATDSECT(R1) GET CONSOLE ROUTE CODE   R4
        $WTO   BUFSTART+7,(R0),JOB=NO,  ISSUE REMOTE SETUP           R4C
               RMT=YES,CLASS=$ACTION,PRI=$ST                         R4
         TM    PCEID,PCEPUSID      TEST PROCESSOR TYPE               R4
         BO    DSVBFLSH            BRANCH IF REMOTE PUNCH            R4
         LM    PC1,PC2,PRCCWEJ     LOAD EJECT CCW                    R4
         BAL   PL,PPPUT            SEND CCW TO REMOTE                R4
         MVC   BUFSTART(9),=C'$HASP190 '  MOVE MESSAGE ID            R4
         LA    PC1,BUFSTART        SET MESSAGE ADDRESS               R4
         LA    PC2,80               AND LENGTH                       R4
         TM    $RUNOPTS,$MSGID     IF MESSAGE IDS REQUIRED,          R4
         BO    SKIP620              BR TO SET COMMAND TYPE           R4
         MVI   BUFSTART+7,C'$'       ELSE RESET MESSAGE ID,          R4
         LA    PC1,BUFSTART+7         MESSAGE ADDRESS,               R4
         LA    PC2,73                  AND MESSAGE LENGTH            R4
SKIP620  ICM   PC1,8,=X'71'        SET COMMAND TYPE                  R4
         BAL   PL,PPPUT2           SEND CCW TO REMOTE                R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ENSURE THAT REMOTE TERMINAL BUFFER IS FLUSHED                *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVBFLSH DS    0H                                                    R4
         ICM   PC1,8,=X'FF'        BUFFER FLUSH COMMAND              R4
         BAL   PL,PPPUT2           SEND CCW TO REMOTE                R4
         L     R1,PPLC             RESTORE $DOM CMB ADDRESS          R4
         L     PC1,PCEDCT          ADDRESS PRINT/PUNCH DCT     @OZ32566
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        WAIT FOR OPERATOR ACTION                                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVSTOP  OI    DCTFLAGS,DCTSTOP    STOP THE DEVICE                   R4
         SPACE 1                                                     R4
DSVWAIT  TM    DCTFLAGS,DCTSTOP    TEST FOR $S DEVICE                R4
         BZ    DSVSTART            BRANCH IF TRUE START              R4
         NI    DCTFLAGS,255-DCTDELET-DCTRSTRT-DCTBKSP RESET
        $WAIT  IO                  WAIT FOR A POST FROM COMM         R4
         B     DSVWAIT             TEST FOR $S DEVICE                R4
DSVSTART DS    0H                                                    R4
         LTR   R1,R1               BRANCH AROUND IF                  R4
         BZ    DSVNODOM              NOTHING TO $DOM                 R4
        $DOM   CMB=(R1)            DELETE THE MESSAGE                R4
DSVNODOM DS    0H                                                    R4
         TM    DCTFLAGS,DCTDELET+DCTRSTRT $C OR $E OR $I             R4
         BZ    DSVLUCSB            BRANCH IF NO                      R4
         OI    PPFLAG,PPDELSW+PRDELSW  CAUSE DATA SET TERMINATION    R4
         OC    PDCTFLAG,DCTFLAGS   PROVIDE REASON FOR TERMINATION    R4
         NI    DCTFLAGS,255-DCTDELET-DCTRSTRT-DCTBKSP  RESET DCT     R4
         MVC   DCTFORMS(12),PCEFORM REVERT TO PREVIOUS SETUP         R4
         TM    PCEID,PCERJEID      TEST FOR REMOTE                   R4
         BZ    DSVLUCSB            BR IF NOT                         R4
         TM    MDCTSTAT,DCTABORT   TEST FOR REMOTE ABORTING          R4
         BO    DSVEXIT             EXIT IF YES                       R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        LOAD UCSB IF SUPPORTED BY DEVICE                             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVLUCSB DS    0H                                                    R4
         NI    DCTPPSW,255-DCTPPSWO $T NOT ALLOWED                   R4
         TM    PCEID,PCEPUSID      TEST PROCESSOR ID                 R4
         BO    DSVEXIT             BRANCH IF PUNCH                   R4
         LM    PC1,PC2,PRCCWEJ     GET EJECT CCW                     R4
         BAL   PL,PPPUT            ADD TO CHAIN                      R4
         BAL   PL,PPWRITE          STAGE WRITE                       R4
         BAL   PL,PPCHECK          CHECK WRITE                       R4
         L     PC1,PCEDCT          ADDRESS PRINT/PUNCH DCT     @OZ32566
         TM    PDEVTYPE+1,X'80'    TEST UCSB OPTION FIELD            R4
         BZ    DSVLFCB             BRANCH IF NOT SUPPORTED           R4
         TM    DCTPPSW,DCTPPSWT    UCSB LOAD REQUIRED                R4
         BZ    DSVLFCB             BRANCH IF NO                      R4
         NI    DCTPPSW,255-DCTPPSWT RESET TRAIN LOAD BIT             R4
         CLC   DCTUCS,=C'0   '     IS UCS = 0 REQUESTED              R4
         BE    DSVLFCB             BRANCH IF YES - SKIP UCS LOAD     R4
         MVC   BUFSTART(4),=X'0001003A' BLDL PARAMETER LIST          R4
         MVC   BUFSTART+4(4),=C'UCS1' IMAGE PREFIX FOR 1403 UCS      R4
         MVC   BUFSTART+8(4),DCTUCS USER UCS IMAGE ID                R4
         CLI   PDEVTYPE+3,X'08'    IS THIS A 1403                    R4
         BE    DSVUCS01            BRANCH IF YES                     R4
         MVI   BUFSTART+7,C'2'     IMAGE PREFIX FOR 3211 UCS         R4
DSVUCS01 DS    0H                  *                                 R4
         TM    $IMAGECB,X'40'      IS IMAGE LOADER TASK BUSY         R4
         BNO   DSVUCS02            BRANCH IF NO                      R4
        $WAIT  IMAG                WAIT FOR IMAGE TASK TO $$POST     R4
         B     DSVUCS01            TRY AGAIN                         R4
DSVUCS02 DS    0H                  *                                 R4
         MVI   BUFECBCC,X'80'      SET BUFFER ECB AS WAITING         R4
         POST  $IMAGECB,(R3)       POST WITH BUFFER ADDRESS          R4
DSVUCS03 DS    0H                  *                                 R4
        $WAIT  IMAG                WAIT FOR IMAGE TASK TO $$POST     R4
         TM    BUFECBCC,X'7F'      TEST STATUS OF LOAD REQUEST       R4
         BZ    DSVUCS03            BRANCH IF NOT COMPLETE            R4
         BM    DSVUCSNO            BRANCH IF IMAGE NOT FOUND         R4
         L     R15,PRPUUCB         ADDRESS PRINTER UCB               R4
         USING UCBDSECT,R15        ACTIVATE UCB ADDRESSABILITY       R4
         L     R15,UCBXTADR        ADDRESS UCB EXTENSION             R4
         USING UCBUCS,R15          UCB EXTENSION ADDRESSABILITY      R4
        MODESET EXTKEY=ZERO        GET ZERO KEY TO CHANGE THE UCB    R4
         OI    UCBUCSOP,UCBUCSO1+UCBUCSO2  SET DEFAULT UCS AND FOLD R41
         NI    DCTPPSW,255-DCTPPSWU RESET NON-STANDARD UCS FLAG      R4
         TM    BUFSTART,X'80'      STANDARD UCS IMAGE...            R41
         BO    DSVSETU             BR IF YES                        R41
         OI    DCTPPSW,DCTPPSWU    SET NON-STANDARD UCS FLAG         R4
         NI    UCBUCSOP,255-UCBUCSO1 RESET DEFAULT UCS IMAGE FLAG    R4
DSVSETU  MVC   UCBUCSID,DCTUCS     SET UCS IMAGE ID IN UCB          R41
         TM    BUFSTART,X'40'      FOLD THE UCS IMAGE...            R41
         BO    DSVKEY              BR IF YES                        R41
         NI    UCBUCSOP,255-UCBUCSO2  ELSE, RESET UCS FOLD BIT      R41
DSVKEY   MODESET EXTKEY=HASP       RETURN TO NORMAL HASP KEY        R41
         DROP  R15                 SUSPEND UCB EXTENSION BASE        R4
         LM    PC1,PC2,PUNFOLD     3211 UNFOLD CONTROL CCW           R4
         TM    BUFSTART,X'40'      FOLD THE UCS IMAGE...            R41
         BZ    DSVC3211            BR IF NO                         R41
         ICM   PC1,8,=X'43'         ELSE REPLACE CC WITH 3211 FOLD  R41
DSVC3211 CLI   PDEVTYPE+3,UCB3211  IS DEVICE A 3211...              R41
         BE    DSVUCS04            BR IF YES                        R41
         LM    PC1,PC2,PUCSGATE    1403 UCS GATE CCW                 R4
         SPACE 1                                                    R41
DSVUCS04 BAL   PL,PPPUT2           ADD CCW TO CHAIN                 R41
         SPACE 1                                                    R41
         LM    PC1,PC2,PUCSBDC     BLOCK DATA CHECK CCW              R4
         BAL   PL,PPPUT2           ADD CCW TO CHAIN                  R4
         LM    PC1,PC2,PUCSLOAD    LOAD UCSB CCW                     R4
         ALR   PC1,PBUF            ADJUST DATA ADDRESS               R4
         CLI   PDEVTYPE+3,UCB1403  TEST DEVICE TYPE                  R4
         BE    DSVC1403            BR IF 1403                       R41
         AH    PC2,=H'272'         ADDITIONAL UCSB LENGTH FOR 3211   R4
         B     DSVUCPUT            BR TO CONTINUE                   R41
         SPACE 1                                                    R41
DSVC1403 TM    BUFSTART,X'40'      FOLD THE UCS IMAGE...            R41
         BZ    DSVUCPUT            BR IF NO                         R41
         ICM   PC1,8,=X'F3'         ELSE REPLACE CC WITH 1403 FOLD  R41
         SPACE 1                                                    R41
DSVUCPUT BAL   PL,PPPUT2           ADD CCW TO CHAIN                 R41
         BAL   PL,PPWRITE           INITIATE WRITE                   R4
         BAL   PL,PPCHECK           AND CHECK                        R4
         L     PC1,PCEDCT          ADDRESS PRINT/PUNCH DCT     @OZ32566
         B     DSVLFCB             GO LOAD FCB IMAGE                 R4
DSVUCSNO DS    0H                  *                                 R4
        $MID   180                                                   R4
         MVC   BUFSTART(2),=X'180F' MOVE MESSAGE NUMBER              R4
         MVC   BUFSTART+2(8),DCTDEVN MOVE DEVICE NAME                R4
         MVC   BUFSTART+10(26),=CL26' FCB  IMAGE XXXX NOT FOUND'     R4
         MVC   BUFSTART+11(4),=C'UCSB' MOVE BUFFER ID                R4
         MVC   BUFSTART+22(4),DCTUCS MOVE UCS ID                     R4
        $WTO   BUFSTART,36,JOB=YES, ISSUE BUFFER LOAD FAIL MSG       R4C
               ROUTE=$LOG+$UR,CLASS=$ACTION,PRI=$ST                  R4
         OI    DCTPPSW,DCTPPSWT+DCTPPSWO LOAD UCS + MSG              R4
         B     DSVMSG              GO PRINT SETUP MESSAGE            R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        LOAD FCB IF SUPPORTED BY DEVICE                              *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
DSVLFCB  DS    0H                                                    R4
         TM    DCTPPSW,DCTPPSWC    FCB LOAD REQUIRED                 R4
         BZ    DSVEXIT             BRANCH IF NO                      R4
         NI    DCTPPSW,255-DCTPPSWC RESET CARRIAGE LOAD BIT          R4
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE              R41
         BZ    DSVFCB00            BR IF NOT REMOTE                 R41
*              THIS LINE DELETED BY APAR NUMBER                @OZ18398
*              THIS LINE DELETED BY APAR NUMBER                @OZ18398
         SPACE 1                                                    R41
DSVFCB00 CLI   PDEVTYPE+3,UCB1403  TEST DEVICE TYPE                 R41
         BE    DSVEXIT             BRANCH IF 1403                    R4
         MVC   BUFSTART(4),=X'0001003A' BLDL PARAMETER LIST          R4
         MVC   BUFSTART+4(4),=C'FCB2' IMAGE PREFIX FOR 3211 FCB      R4
         MVC   BUFSTART+8(4),DCTFCB USER FCB IMAGE ID                R4
DSVFCB01 DS    0H                  *                                 R4
         TM    $IMAGECB,X'40'      IS IMAGE LOADER TASK BUSY         R4
         BNO   DSVFCB02            BRANCH IF NO                      R4
        $WAIT  IMAG                WAIT FOR IMAGE TASK TO $$POST     R4
         B     DSVFCB01            TRY AGAIN                         R4
DSVFCB02 DS    0H                  *                                 R4
         MVI   BUFECBCC,X'80'      SET BUFFER ECB AS WAITING         R4
         POST  $IMAGECB,(R3)       POST WITH BUFFER ADDRESS          R4
DSVFCB03 DS    0H                  *                                 R4
        $WAIT  IMAG                WAIT FOR IMAGE TASK TO $$POST     R4
         TM    BUFECBCC,X'7F'      TEST STATUS OF LOAD REQUEST       R4
         BZ    DSVFCB03            BRANCH IF NOT COMPLETE            R4
         BM    DSVFCBNO            BRANCH IF IMAGE NOT FOUND         R4
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BO    DSVFCBRJ            BRANCH IF REMOTE                  R4
         L     R15,PRPUUCB         ADDRESS PRINTER UCB               R4
         USING UCBDSECT,R15        ACTIVATE UCB ADDRESSABILITY       R4
         L     R15,UCBXTADR        ADDRESS UCB EXTENSION             R4
         USING UCBUCS,R15          UCB EXTENSION ADDRESSABILITY      R4
        MODESET EXTKEY=ZERO        GET ZERO KEY TO CHANGE THE UCB    R4
         OI    UCBFCBOP,UCBFCBO1   SET DEFAULT FCB IMAGE FLAG        R4
         TM    BUFSTART,X'80'      IS THIS A STANDARD FCB IMAGE      R4
         BO    SKIP660             BRANCH IF YES                     R4
         NI    UCBFCBOP,255-UCBFCBO1 RESET DEFAULT FCB IMAGE FLAG    R4
SKIP660  MVC   UCBFCBID,DCTFCB     COPY FCB IMAGE ID TO THE UCB      R4
        MODESET EXTKEY=HASP        RETURN TO NORMAL HASP KEY         R4
         DROP  R15                 SUSPEND UCB EXTENSION BASE        R4
DSVFCBRJ DS    0H                                                    R4
         NI    DCTPPSW,255-DCTPPSWB RESET NON-STANDARD FCB FLAG      R4
         TM    BUFSTART,X'80'      IS THIS A STANDARD FCB IMAGE      R4
         BO    SKIP670             BRANCH IF YES                     R4
         OI    DCTPPSW,DCTPPSWB    SET NON-STANDARD FCB FLAG         R4
SKIP670  TM    PRINDEX,X'C0'       SPECIFIC INDEX REQUESTED          R4
         BZ    DSVFCB04            BRANCH IF NO                      R4
         CLC   PRINDEX,BUFSTART+2  IS IMAGE INDEX BEING CHANGED      R4
         BE    DSVFCB04            BRANCH IF NO                      R4
         OI    DCTPPSW,DCTPPSWC    SET LOAD FCB FLAG                 R4
         MVC   BUFSTART+2(1),PRINDEX SET 3211 INDEX VALUE            R4
DSVFCB04 DS    0H                  *                                 R4
         MVC   DCTINDEX,BUFSTART+2 SAVE NEW INDEX VALUE              R4
         LM    PC1,PC2,PFCBLOAD    LOAD FCB CCW                      R4
         ALR   PC1,PBUF            ADJUST DATA ADDRESS               R4
         IC    PC2,BUFSTART+1      GET LENGTH OF FCB IMAGE           R4
         TM    PCEID,PCERJEID      TEST PROCESSOR TYPE               R4
         BZ    SKIP680             BRANCH IF LOCAL                   R4
         ICM   PC1,8,=X'61'        SET SPECIAL RJE FCB CC            R4
SKIP680  BAL   PL,PPPUT2           ADD CCW TO CHAIN                  R4
         BAL   PL,PPWRITE           INITIATE WRITE                   R4
         BAL   PL,PPCHECK           AND CHECK                        R4
         B     DSVEXIT             RETURN TO CALLER                  R4
DSVFCBNO DS    0H                  *                                 R4
        $MID   180                                                   R4
         MVC   BUFSTART(2),=X'180F' MOVE MESSAGE NUMBER              R4
         MVC   BUFSTART+2(8),DCTDEVN MOVE DEVICE NAME                R4
         MVC   BUFSTART+10(26),=CL26' FCB  IMAGE XXXX NOT FOUND'     R4
         MVC   BUFSTART+22(4),DCTFCB MOVE FCB ID                     R4
        $WTO   BUFSTART,36,JOB=YES, ISSUE BUFFER LOAD FAIL MSG       R4C
               ROUTE=$LOG+$UR,CLASS=$ACTION,PRI=$ST                  R4
         OI    DCTPPSW,DCTPPSWC+DCTPPSWO SET LOAD AND MSG REQD       R4
         B     DSVMSG              GO PRINT SETUP MESSAGE            R4
         SPACE 2                                                     R4
DSVEXIT  DS    0H                                                    R4
         L     PL,PCERETN          RESTORE RETURN ADDRESS            R4
         L     BASE2,PDSVSAVE      RESTORE BASE2                     R4
         BR    PL                  RETURN TO CALLER                  R4
         DROP  PC1                 SUSPEND DCT ADDRESSABILITY        R4
         EJECT                                                       JN
*        PARM LISTS FOR LOCAL SETUP $WTO'S                          R41
         SPACE 4                                                    R41
         DS    0F                                                   R41
DSVWTOA $WTO   JOB=NO,             ACTION-TYPE MESSAGE,             R41C
               ROUTE=$LOG+$UR,      DISPLAYABLE VIA '$DO'           R41C
               CLASS=$DOMACT,                                       R41C
               PRI=$ST,                                             R41C
               MF=L                                                 R41
         SPACE 3                                                    R41
DSVWTOI $WTO   JOB=NO,             INFO-TYPE MESSAGE,               R41C
               ROUTE=$LOG+$UR,      DISPLAYABLE VIA '$DO'           R41C
               CLASS=$DOMACT+$NORMAL,                               R41C
               PRI=$ST,                                             R41C
               MF=L                                                 R41
         TITLE ' HASP PRINT/PUNCH SERVICE -- 3800 DEVICE SETUP'      R4
P3800DSV DS    0H                  ENTRY TO 3800 DEVICE SETUP        R4
         ST    BASE2,PDSVSAVE      SAVE BASE                         R4
         SPACE 1                                                     R4
         USING SPPARM-(BUFSTART-BUFDSECT),PBUF SPPARM ADDRESSABILITY R4
         SPACE 1                                                     R4
P3800DS  ST    PL,PCESAVEA         SAVE RETURN ADDRESS               R4
         LR    BASE2,R15           ESTABLISH LOCAL                   R4
         USING P3800DSV,BASE2       ADDRESSABILITY                   R4
         USING DCTDSECT,PC1        ESTABLISH DCT ADDRESSABILITY      R4
         L     PC1,PCEDCT          ADDRESS 3800 DCT            @OZ32566
         STCM  PC1,7,BUFDCT+1-BUFDSECT(PBUF)   PLACE DCT ADDR IN BUF R4
         USING UCB3800X,PC2        ESTABLISH 3800 UCB                R4
         L     PC2,PRPUUCB             EXTENSION                     R4
         L     PC2,UCBXTADR-UCBDSECT(,PC2) ADDRESSABILITY            R4
         L     JCT,PJCTBUF         ESTABLISH JCT                     R4
         USING JCTDSECT,JCT         ADDRESSABILITY                   R4
*                                                                    R4
*        CLEAR SETPRT PARMLIST AND WORKAREA - SAVE DCT STATUS        R4
*                                                                    R4
         XC    SPPARM(SPWSAVE1-SPPARM),SPPARM  INITIALIZE AREA       R4
         TM    DCTPPSW2,DCTNINIT   DOES 3800 NEED INITIALIZATION...  R4
         BZ    SPRXMIT             BR IF NO                          R4
         OI    SPWFLAG,SPWINIT+SPWSETP  TELL SETPRT                  R4
         NI    DCTPPSW2,255-DCTNINIT  RESET UNTIL $DRAINED           R4
         MVC   DCTCHAR1,DCTUCS     PRESET                           R41
         MVI   DCTCHAR2,C'*'        DCT FIELDS                      R41
         MVC   DCTCHAR2+1(5*4-1),DCTCHAR2  (CHARS,FLASH,MODIFY)     R41
         XC    PREVCPYS(2),PREVCPYS  AND PREV. START AND # OF COPIES R4
         SPACE 1                                                     R4
SPRXMIT  DS    0H                                                    R4
         TM    SPFLAG,SPREXMIT     TEST FOR RE-TRANSMISSION          R4
         BO    SPRRXMIT            BR IF YES                         R4
         MVC   SPWFORMS(3*4),DCTFORMS SAVE FORMS, FCB, DEFAULT CHAR1 R4
         MVC   SPWCHAR1(4*4),DCTCHAR1 SAVE CHAR1,2,3,4 VALUES        R4
         TM    DCTPPSW2,DCTNIBRS   SAVE DCT                          R4
         BZ    SKIP690              BURSTER                          R4
         OI    SPWFLAG,SPWDCTB       STATUS                          R4
         SPACE 3                                                     R4
SKIP690  CLC   DCTFORMS,SPFORMS    TEST FOR FORMS CHANGE             R4
         BE    SPRFCB              BR IF NO                          R4
         OI    SPWFLAG,SPWWTO      INDICATE SETUP MSG NEEDED         R4
         MVC   DCTFORMS,SPFORMS    MOVE IN NEW FORMS ID              R4
         SPACE 1                                                     R4
SPRFCB   DS    0H                                                    R4
         CLI   SPFCB,C'*'          TEST FOR DEFAULT REQUEST          R4
         BNE   SKIP700             BR IF NO                          R4
         MVC   SPFCB,PRDFCB        ELSE, USE INSTALLATION DEFAULT    R4
SKIP700  CLC   DCTFCB,SPFCB        TEST FOR FCB CHANGE               R4
         BE    SPRCHARS            BR IF NO                          R4
         MVC   DCTFCB,SPFCB        MOVE IN NEW FCB IMAGE ID          R4
         EJECT                                                       R4
*        SETUP CHARACTER SETS (CHARS) REQUEST                       R41
*                                                                   R41
*        THIS SECTION BUILDS THE CHAR SET REQUEST USING THE         R41
*        SETUP PARMLIST (SPCHAR1-4.) IF ROOM PERMITS, THE DE-       R41
*        FAULT CHAR (DCTUCS) IS ADDED TO THE LIST.  FOR EACH        R41
*        CHAR, DCTCHAR'S FIELDS ARE SEARCHED TO DETERMINE IF IT     R41
*        IS ALREADY LOADED. IF NOT, THE NEW REQUESTS REPLACE        R41
*        THE OLD SETUP. IF ALREADY LOADED, THE CCW OP-CODE TO       R41
*        SELECT THE CORRESPONDING TANSLATE TABLE IS RECORDED        R41
*        IN THE 'PXTABCCW' TABLE ENTRY FOR THAT CHAR.               R41
*        THE FINAL TABLE REPRESENTS A MAPPING OF THE REQUESTED      R41
*        CHARACTER SETS TO THEIR ACTUAL POSITIONS IN THE 3800,      R41
*        ELIMINATING UNNECESSARY RE-LOADS DUE TO CHARACTER          R41
*        SET POSITIONS.                                             R41
         SPACE 1                                                    R41
SPRCHARS MVC   PXTABCCW,SPRXCCWS   INITIALIZE CCW OP-CODE TABLE     R41
         TM    SPFLAG,SPSEP        SETUP FOR SEPARATORS...          R41
         BZ    SPRCNSEP            BR IF NOT                        R41
         MVI   PRMAXTRC,0          ONLY USE 1 CHAR SET              R41
         BAL   PL,SPRTDFLT         DEFAULT CHAR SET LOADED...       R41
         BNZ   SPRCNDEF            BR IF NOT                        R41
         MVC   PXTABCCW(1),0(PW)    ELSE SET CCW FOR DEFAULT        R41
         B     SPRFLASH              AND BR TO CONTINUE             R41
         SPACE 1                                                    R41
SPRCNDEF TM    DCTPPSW2,DCTSEPNL   MUST LOAD DEFAULT FOR SEP...     R41
         BZ    SPRCHDEF            BR IF YES                        R41
         B     SPRFLASH             ELSE USE 1ST CHAR SET           R41
         SPACE 1                                                    R41
SPRCNSEP CLI   SPCHAR1,C'*'        WAS CHARS SPECIFIED...           R41
         BNE   SPRCH1              BR IF YES                        R41
         SPACE 1                                                    R41
SPRCHDEF MVC   SPCHAR1,DCTUCS      USE JES2 PRINTER DEFAULT         R41
         MVC   SPCHAR2,=C'****'    CLEAR CHAR2                 @OZ28970
         MVC   SPCHAR3(2*4),SPCHAR2 CHAR3 AND CHAR4            @OZ28970
         SPACE 1                                                    R41
SPRCH1   SLR   R1,R1               ONLY BUILD                       R41
         IC    R1,PRMAXTRC          ENTRIES FOR                     R41
         LA    R1,1(,R1)             REQUESTED CHARS                R41
         LA    PL,PXTABCCW         ADDR OF CCW TABLE                R41
         LA    R14,SPCHAR1         ADDR OF CHARS REQUEST            R41
         SPACE 1                                                    R41
SPRCH2   LA    R15,DCTCHAR1        ADDR OF CURRENT CHARS LOADED     R41
         LA    R0,4                SET LOOP COUNTER                 R41
         SPACE 1                                                    R41
SPRCH3   CLC   0(4,R14),0(R15)     THIS CHAR ALREADY LOADED...      R41
         BE    SPRCH4              BR IF YES                        R41
         LA    R15,4(,R15)          ELSE CHECK WITH NEXT            R41
         BCT   R0,SPRCH3             CHAR ALREADY LOADED            R41
         SPACE 1                                                    R41
         MVC   PXTABCCW,SPRXCCWS   NOT LOADED ALREADY -- SET TO     R41
         MVC   DCTCHAR1(4*4),SPCHAR1  LOAD ALL REQUESTED CHARS      R41
         B     SPRCH5                  AND BR TO CONTINUE           R41
         EJECT                                                      R41
SPRCH4   LNR   R15,R0              LOADED ALREADY -- DETERMINE      R41
         LA    R15,SPRXCCWS+4(R15)  ACTUAL POSITION OF CHAR AND     R41
         MVC   0(1,PL),0(R15)        INSERT CORRECT CCW OP-CODE     R41
         LA    R14,4(,R14)         INCR TO NEXT REQUESTED           R41
         LA    PL,1(,PL)            CHAR AND GO DETERMINE           R41
         BCT   R1,SPRCH2             IF IT IS ALREADY LOADED        R41
         SPACE 1                                                    R41
SPRCH5   IC    R1,PRMAXTRC         GET NUMBER OF CHARACTER          R41
         LA    R1,1(,R1)            SETS REQUESTED                  R41
         CLM   R1,1,UCBCGMNO       POTENTIAL WCGM'S LEFT...         R41
         BNL   SPRFLASH            BR IF NO                         R41
         BAL   PL,SPRTDFLT         DEFAULT CHAR SET LOADED...       R41
         BZ    SPRFLASH            BR IF YES.   ELSE,               R41
         SLL   R1,2                ADD DEFAULT                      R41
         LA    R1,DCTCHAR1(R1)      TO LIST                         R41
         MVC   0(4,R1),DCTUCS        OF CHARS                       R41
         OI    SPWFLAG,SPWDEFLT    SET INDICATION                   R41
         SPACE 2                                                    R41
SPRFLASH DS    0H                                                    R4
         CLI   SPFLASH,C'*'        TEST FOR FLASH SPECIFIED          R4
         BNE   SPRF1               BR IF YES                         R4
         CLI   DCTFLASH,C'*'       TEST FOR FLASH CURRENTLY ACTIVE   R4
         BE    SPRBURST            BR IF NO                          R4
         B     SPRF2               ELSE, GO TO RESET DCT             R4
SPRF1    CLC   DCTFLASH,SPFLASH    TEST FOR CHANGE                   R4
         BE    SPRBURST            BR IF NO                          R4
         OI    SPWFLAG,SPWWTO      INDICATE SETUP MSG NEEDED         R4
SPRF2    MVC   DCTFLASH,SPFLASH    UPDATE DCT WITH NEW REQUEST       R4
         SPACE 1                                                     R4
SPRBURST DS    0H                                                    R4
         TM    SPFLAG,SPBURST      TEST FOR BURST=YES SPECIFIED      R4
         BZ    SPRB1               BR IF NOT - BURSTER NOT WANTED    R4
         TM    DCTPPSW2,DCTNIBRS   TEST FOR DEVICE ALREADY BURSTING  R4
         BO    SPRMODFY            BR IF YES                         R4
         OI    DCTPPSW2,DCTNIBRS   ELSE, MOVE IN B=Y STATUS          R4
         B     SPRB2                AND GO FLAG FOR WTO              R4
SPRB1    TM    DCTPPSW2,DCTNIBRS   TEST BURST STATUS                 R4
         BZ    SPRMODFY            BR IF NOT ACTIVE                  R4
         NI    DCTPPSW2,255-DCTNIBRS  ELSE, MOVE IN B=N STATUS       R4
SPRB2    OI    SPWFLAG,SPWWTO      INDICATE SETUP MSG NEEDED         R4
         SPACE 1                                                     R4
SPRMODFY DS    0H                                                    R4
         CLI   SPMODF,C'*'         TEST FOR COPY MOD SPECIFIED       R4
         BNE   SPRM1               BR IF YES                         R4
         CLI   DCTMODF,C'*'        TEST FOR COPY MOD ALREADY ACTIVE  R4
         BE    SPRMSG              BR IF NO                          R4
         OI    SPWFLAG,SPWCLRM     INDICATE COPY MOD MUST BE CLEARED R4
SPRM1    MVC   DCTMODF,SPMODF      MOVE IN NEW COPY MODIFICATION ID  R4
         EJECT                                                       R4
*                                                                    R4
*        ISSUE OPERATOR SETUP MSG FOR FORMS(F) - FLASH(O) - BURST(B) R4
*                                                                    R4
         SPACE 1                                                     R4
SPRMSG   DS    0H                                                    R4
         TM    SPWFLAG,SPWWTO      TEST FOR SETUP MSG NEEDED         R4
         BZ    SPRLFCB             BR IF NO                          R4
         SPACE 1                                                     R4
SPRMSG1  DS    0H                                                    R4
        $MID   190                                                   R4
         MVC   SPWMSG+7(2),=X'190E'     MOVE MESSAGE NUMBER          R4
         MVC   SPWMSG+9(8),JCTJOBID     MOVE JOB NUMBER              R4
         MVI   SPWMSG+17,C' '           MOVE SPACE BEFORE JOBNAME    R4
         MVC   SPWMSG+18(8),JCTJNAME    MOVE JOBNAME                 R4
         MVC   SPWMSG+26(52),=CL52' SETUP -- PRINTERN -- F = FORM -- O C
               = FLSH -- B = N'         MOVE SETUP MESSAGE           R4
         MVC   SPWMSG+36(8),DCTDEVN     MOVE DEVICE NAME             R4
         MVC   SPWMSG+52(4),DCTFORMS    MOVE FORMS ID                R4
         MVC   SPWMSG+64(4),DCTFLASH    MOVE FLASH-FRAME ID          R4
         TM    DCTPPSW2,DCTNIBRS        TEST FOR B=N                 R4
         BZ    *+8                      BR IF YES                    R4
         MVI   SPWMSG+76,C'Y'           MOVE IN B=Y                  R4
         DROP  JCT                 SUSPEND JCT ADDRESSABILITY        R4
         SPACE 1                                                     R4
        $WTO   SPWMSG+7,72,JOB=NO,  ISSUE PRINTER SETUP MESSAGE      R4C
               ROUTE=$LOG+$UR,CLASS=$DOMACT,PRI=$ST                  R4
         NI    SPWFLAG,255-SPWWTO  RESET WTO FLAG                    R4
         SPACE 1                                                     R4
SPRSTOP  OI    DCTFLAGS,DCTSTOP    STOP ($Z) THE PRINTER             R4
         EJECT                                                       R4
*                                                                    R4
*        WAIT FOR $S COMMAND FROM OPERATOR                           R4
*                                                                    R4
SPRWAIT  DS    0H                                                    R4
         TM    DCTFLAGS,DCTSTOP    TEST FOR $S PRINTER               R4
         BZ    SPRSTART            BR IF START COMMAND ISSUED        R4
         NI    DCTFLAGS,255-DCTDELET+DCTRSTRT+DCTBKSP  RESET FLAGS   R4
        $WAIT  IO                  WAIT FOR A POST FROM COMM         R4
         B     SPRWAIT             GO CHECK FOR $S                   R4
         SPACE 1                                                     R4
SPRSTART DS    0H                                                    R4
        $DOM   CMB=(R1)            DELETE SETUP MESSAGE              R4
         L     R1,PCEJQE           ADDRESS OF JQE              @OZ32566
         TM    JQEFLAGS-JQEDSECT(R1),QUEOPCAN+QUEPURGE $CJ,P...     R41
         BNO   SPROPTST            BR IF NOT $CJ,P                  R41
         OI    DCTFLAGS,DCTDELET    ELSE SIMULATE $C PRT            R41
         SPACE 1                                                    R41
SPROPTST TM    DCTFLAGS,DCTDELET+DCTRSTRT  $C, $E OR $I...          R41
         BZ    SPRLFCB             BR IF NO                          R4
         OI    PPFLAG,PPDELSW+PRDELSW  ELSE CAUSE TERMINATION       R41
         OC    PDCTFLAG,DCTFLAGS   PROVIDE TERMINATION REASON       R41
         NI    DCTFLAGS,255-DCTDELET-DCTRSTRT-DCTBKSP RESET    @OZ18409
*                                                                    R4
*        IF PRINTER CANCELLED -- REVERT TO PREVIOUS SETUP           R41
*                                                                    R4
         MVC   DCTFORMS(3*4),SPWFORMS  RESET FORMS-FCB-DEFAULT CHARS R4
         MVC   DCTCHAR1(4*4),SPWCHAR1  RESET CHAR1,2,3,4 VALUES      R4
         MVC   DCTFLASH,=C'****'   CLEAR FLASH AND                   R4
         MVC   DCTMODF,=C'****'     COPY MODIFICATION                R4
         NI    DCTPPSW2,255-DCTNIBRS     RESET                       R4
         TM    SPWFLAG,SPWDCTB            THE                        R4
         BZ    SPRLFCB                     BURSTER                   R4
         OI    DCTPPSW2,DCTNIBRS            STATUS                   R4
         EJECT                                                       R4
*                                                                    R4
*        DETERMINE IF A CALL TO SETPRT (SVC 81) IS NECESSARY         R4
*                                                                    R4
SPRLFCB  DS    0H                                                    R4
         CLI   DCTFCB,C'*'         TEST FOR FCB SPECIFIED            R4
         BNE   SPRLFC1             BR IF YES                         R4
         CLI   UCBFCBNM,X'40'      TEST FOR HDWR DEFAULT LOADED     R41
         BE    SPRLCHRS            BR IF YES                         R4
         OI    SPWFLAG,SPWINIT+SPWSETP  INIT-PRT SETS HDWR DEFAULT  R41
         B     SPRLCHRS            BR TO CONTINUE                   R41
         SPACE 1                                                    R41
SPRLFC1  TM    SPWFLAG,SPWINIT     INITIALIZE PRINTER...            R41
         BO    SPRLFC2             FORCE FCB UPDATE IF YES          R41
         CLC   DCTFCB,UCBFCBNM     FCB MATCH PRINTER FCB...         R41
         BE    SPRLCHRS            BR IF YES                        R41
         SPACE 1                                                    R41
SPRLFC2  MVC   SPPFCB,DCTFCB       SHOW SETPRT CALL NEEDED          R41
         OI    SPWFLAG,SPWSETP      TO LOAD FCB                     R41
         SPACE 1                                                     R4
SPRLCHRS DS    0H                                                    R4
         TM    SPWFLAG,SPWINIT     IF INIT BIT IS SET,               R4
         BO    SPRLC2               THEN FORCE UPDATE                R4
         LA    R1,DCTCHAR1         IF ANY                            R4
         LA    R15,UCBCHAR1         CHARS VALUES                     R4
         LA    R0,4                  DIFFER                          R4
SPRLC1   CLI   0(R1),C'*'             FROM                           R4
         BE    SPRLFLSH                UCB,                          R4
         CLC   0(4,R1),0(R15)          THEN                          R4
         BNE   SPRLC2                   BRANCH                       R4
         LA    R1,4(,R1)                 TO                          R4
         LA    R15,4(,R15)                LOAD                       R4
         BCT   R0,SPRLC1                   ALL SPECIFIED             R4
         B     SPRLFLSH            ELSE, DON'T CALL SETPRT           R4
         SPACE 1                                                     R4
SPRLC2   CLI   DCTCHAR1,C'*'       UNTIL                             R4
         BE    SPRLFLSH             END                              R4
         MVC   SPPXLAT1,DCTCHAR1     OF                              R4
         OI    SPWFLAG,SPWSETP        CHARS                          R4
         CLI   DCTCHAR2,C'*'           VALUES                        R4
         BE    SPRLFLSH                 IS                           R4
         MVC   SPPXLAT2,DCTCHAR2         REACHED,                    R4
         CLI   DCTCHAR3,C'*'              UPDATE                     R4
         BE    SPRLFLSH                    PARAMETER                 R4
         MVC   SPPXLAT3,DCTCHAR3            LIST                     R4
         CLI   DCTCHAR4,C'*'                 AND INDICATE            R4
         BE    SPRLFLSH                       SETPRT                 R4
         MVC   SPPXLAT4,DCTCHAR4               CALL NEEDED           R4
         EJECT                                                 @OZ18414
SPRLFLSH CLI   DCTFLASH,C'*'       FLASH SPECIFIED...          @OZ18414
         BNE   SPRLF1              BR IF YES                   @OZ18414
         CLI   UCBIMAGE,X'40'      FLASH ACTIVE...             @OZ18414
         BE    SPRLBRST            BR IF NO                    @OZ18414
         MVC   SPPIMAGE,=X'40000000'  ELSE RESET FLASH ID      @OZ18414
         B     SPRLF2              BR TO CONTINUE              @OZ18414
         SPACE 1                                               @OZ18414
SPRLF1   MVC   SPPIMAGE,DCTFLASH   SET FLASH ID                @OZ18414
         TM    SPFLAG,SPSEP        SEPARATOR PAGE...           @OZ18414
         BO    SPRLF2              SET ZERO COUNT IF YES       @OZ18414
         ICM   PW,1,SPFLASHC       GET FLASH-COPY COUNT        @OZ18414
         BNZ   *+8                 USE IT IF NOT ZERO          @OZ18414
         LA    PW,255               ELSE USE MAX COPY COUNT    @OZ18414
         STC   PW,SPPFRMNR         SET FLASH COUNT             @OZ18414
         TM    SPWFLAG,SPWINIT     INITIALIZE PRINTER...       @OZ18414
         BO    SPRLBRST            BR IF YES                   @OZ18414
         CLC   DCTFLASH,UCBIMAGE   THIS FLASH LOADED...        @OZ18414
         BNE   SPRLF2              BR IF NO                    @OZ18414
         CLM   PW,1,PREVFLCT       FLASH COUNT LOADED...       @OZ18414
         BE    SPRLBRST            SKIP SETPRT IF YES          @OZ18414
         SPACE 1                                               @OZ18414
SPRLF2   OI    SPWFLAG,SPWSETP     INDICATE SETPRT NEEDED      @OZ18414
         SPACE 1                                                     R4
SPRLBRST DS    0H                                                    R4
         TM    UCBOPTNS,UCBBRSTR   TEST FOR 3800 BURSTER FEATURE     R4
         BZ    SPRLMODF            BR IF NO                          R4
         TM    DCTPPSW2,DCTNIBRS   TEST FOR BURSTER WANTED           R4
         BZ    SPRLB1              BR IF NO                          R4
         TM    UCBACTIV,UCBBRSTA   TEST FOR BURSTER ACTIVE           R4
         BO    SPRLMODF            BR IF YES                         R4
         OI    SPPFLAG1,SPPBURST   INDICATE BURST=YES                R4
         B     SKIP740             GO TO INDICATE SETPRT NEEDED      R4
SPRLB1   TM    UCBACTIV,UCBBRSTA   TEST FOR BURSTER ACTIVE           R4
         BZ    SPRLMODF            BR IF NO                          R4
SKIP740  OI    SPWFLAG,SPWSETP     INDICATE SETPRT NEEDED            R4
         SPACE 1                                                     R4
SPRLMODF DS    0H                                                    R4
         CLI   DCTMODF,C'*'        TEST FOR MODIFY SPECIFIED         R4
         BNE   SPRLM1              BR IF YES                         R4
         TM    SPWFLAG,SPWCLRM     TEST FOR COPY MOD RESET NEEDED    R4
         BZ    SPRPCOPY            BR IF NO                          R4
         LA    R1,SPRNULLM         ADDR OF NULL COPY MOD             R4
         ST    R1,SPPMDPTA          INTO SETPRT LIST                 R4
         OI    SPPFLAG2,SPPMODI    INDICATE IN-CORE COPY MOD         R4
         B     SPRLM2              GO INDICATE SETPRT NEEDED         R4
SPRLM1   MVC   SPPMODPT,DCTMODF    MOVE COPY MOD ID INTO LIST        R4
         SLR   PW,PW               ASSUME MODIFY-TRC=0         @OZ18410
         CLC   SPMODFT,PRMAXTRC    IS MODIFY-TRC VALID...      @OZ18410
         BH    *+8                 BR IF NOT TO USE TRC=0      @OZ18410
         IC    PW,SPMODFT           ELSE USE SUPPLIED MOD-TRC  @OZ18410
         IC    PW,PXTABCCW(PW)     DETERMINE                   @OZ18410
         SLL   PW,26                ACTUAL                     @OZ18410
         SRL   PW,30                 CHARACTER SET             @OZ18410
         STC   PW,SPPTRC              POSITION                 @OZ18410
SPRLM2   OI    SPWFLAG,SPWSETP     INDICATE SETPRT NEEDED            R4
         SPACE 1                                                     R4
SPRPCOPY DS    0H                                                    R4
         CLC   PREVCPYS(2),SPCOPYS TEST IF COUNTS MATCH PREVIOUS     R4
         BE    SPRSETP             BR IF YES                         R4
         OI    SPWFLAG,SPWSETP     ELSE, FORCE A CALL TO SETPRT      R4
         EJECT                                                       R4
*                                                                    R4
*        CALL SETPRT (SVC 81) VIA JES2 IMAGE LOADER SUB-TASK         R4
*                                                                    R4
SPRSETP  DS    0H                                                    R4
         TM    SPWFLAG,SPWSETP     TEST FOR SETPRT NEEDED            R4
         BZ    SPREXIT             BR IF NO                          R4
         SPACE 1                                                     R4
         TM    SPWFLAG,SPWINIT     TEST FOR INIT NEEDED              R4
         BZ    SKIP750             BR IF NO                          R4
         OI    SPPFLAG1,SPPINIT    ELSE, TELL SETPRT                 R4
SKIP750  B     SPRSETP1            GO CALL SETPRT                    R4
         SPACE 1                                                     R4
SPRRXMIT DS    0H                                                    R4
         MVC   SPPFRMNR,SPFLASHC   MOVE IN FLASH COUNT               R4
         OI    SPPFLAG1,SPPREX     INDICATE RE-XMISSION TO SETPRT    R4
         NI    SPFLAG,255-SPREXMIT RESET PRPU FLAG INDICATION        R4
         SPACE 1                                                     R4
SPRSETP1 DS    0H                                                    R4
         L     R1,DCTDCB           ADDRESS 3800 DCB                  R4
         NI    DCBIFLGS-DCBDSECT(R1),255-DCBIFEC  RESET ERROR COND.  R4
         ST    R1,SPPDCBA          SET DCB ADDR IN PARMLIST          R4
         MVI   SPPFDUNF,SPPFBLK+SPPEXTL  BLK DATA CK + EXTENDED LIST R4
         OI    SPPFLAG1,SPPBFREQ+SPPBTREQ BYPASS ALL POSSIBLE WTOR'S R4
         MVC   SPPCPYNR(1),SPCOPYN MOVE IN NUMBER OF COPIES          R4
         MVC   SPPSTCNR(1),SPCOPYS MOVE IN STARTING COPY NUMBER      R4
         MVC   PREVCPYS(2),SPCOPYS SAVE COPY COUNTS                  R4
         MVC   PREVFLCT,SPPFRMNR   SAVE FLASH COUNT            @OZ18414
         SPACE 1                                                     R4
*        THIS CARD REPLACED BY OZ26939                         @OZ26939
*        THIS CARD REPLACED BY OZ26939                         @OZ26939
*        THIS CARD REPLACED BY OZ26939                         @OZ26939
*        THIS CARD REPLACED BY OZ26939                         @OZ26939
         SPACE 1                                               @OZ26939
         L     R1,PRIMGDTE         ADDR OF HASPIMAG DTE        @OZ26939
         LA    R1,8(,R1)           ADDR OF HASPIMAG WORK-ECB   @OZ26939
         MVI   BUFECBCC-BUFDSECT(PBUF),X'80'  SET ECB AS WAITING     R4
         POST  (1),(PBUF)          POST (PASSING BUFFER ADDR)  @OZ26939
         SPACE 1                                                     R4
SPRWIM2  DS    0H                                                    R4
        $WAIT  IMAG                WAIT FOR POST FROM SUB-TASK       R4
         TM    BUFECBCC-BUFDSECT(PBUF),X'7F' TEST STATUS OF SETPRT   R4
         BZ    SPRWIM2             BR IF NOT COMPLETE                R4
         BO    SPREXIT             BR IF SUCCESSFUL                  R4
         EJECT                                                       R4
*                                                                    R4
*        SETPRT ERROR DETECTED - ATTEMPT RECOVERY                    R4
*                                                                    R4
         SPACE 1                                                     R4
SPRERR   DS    0H                                                    R4
         CLI   SPWRTCDE,0          TEST RETURN CODE                  R4
         BE    SPREXIT             EXIT IF ALL IS OK                 R4
         CLI   SPWRTCDE,X'30'      TEST FOR MAX WGCM'S EXCEEDED      R4
         BE    SPRRWCGM            BR IF YES -- INFORM OPERATOR      R4
        $MID   180                                                   R4
         MVC   SPWMSG(2),=X'180E'     MOVE MESSAGE NUMBER            R4
         MVC   SPWMSG+2(8),DCTDEVN    MOVE DEVICE NAME               R4
         MVC   SPWMSG+10(28),=CL28' FCB   IMAGE XXXX NOT FOUND '     R4
         CLI   SPWRTCDE,4          TEST FOR NOT-FOUND CONDITION      R4
         BE    SPREASON            BR IF YES                         R4
         MVC   SPWMSG+28(9),=CL9'I/O ERROR'  ASSUME I/O ERROR        R4
         CLI   SPWRTCDE,8          TEST FOR I/O ERROR CONDITION      R4
         BE    SPREASON            BR IF YES                         R4
         MVC   SPWMSG+28(10),=CL10'LOAD ERROR'  ASSUME LOAD ERROR    R4
         SPACE 1                                                     R4
SPREASON DS    0H                                                    R4
         SR    R1,R1               OBTAIN REASON-                    R4
         IC    R1,SPWRSCDE          CODE                             R4
         SLL   R1,1                MULT BY 2 FOR OFFSET              R4
         LA    PL,SPRESONS(R1)     ADDR OF REASON TABLE ENTRY        R4
         MVC   SPWMSG+11(5),0(PL)  MOVE IMAGE TYPE INTO MESSAGE      R4
         L     PL,4(,PL)           LOAD ERROR RTN ADDRESS            R4
         BR    PL                  GO TO ERROR RTN                   R4
         SPACE 3                                                     R4
SPRRWCGM TM    SPWFLAG,SPWDEFLT    ATTEMPT TO LOAD DEFAULT...       R41
         BZ    SPRRWCG2            BR IF NO                         R41
         SLR   R1,R1                ELSE REMOVE                     R41
         IC    R1,PRMAXTRC           DEFAULT                        R41
         SLL   R1,2                   CHAR (DCTUCS)                 R41
         LA    PW,DCTCHAR1+4(R1)       FROM                         R41
         MVC   0(4,PW),=C'****'         REQUEST                     R41
         NI    SPWFLAG,255-SPWDEFLT RESET DEFAULT-ATTEMPT FLAG      R41
         LA    PW,SPPXLAT1+4(R1)   REMOVE REQUEST FROM              R41
         XC    0(4,PW),0(PW)        SETPRT PARMLIST                 R41
         B     SPRSETP1            BR TO RETRY SETPRT REQUEST       R41
         SPACE 1                                                    R41
        $MID   155                 MESSAGE IDENTIFIER                R4
         SPACE 1                                                    R41
SPRRWCG2 MVC   SPWMSG(2),=X'155E'  MOVE MESSAGE ID                  R41
         MVC   SPWMSG+2(8),DCTDEVN MOVE DEVICE NAME                  R4
         MVC   SPWMSG+10(28),=CL28' MAX. OF N WCGM''S EXCEEDED'      R4
         MVC   SPWMSG+19(1),UCBCGMNO MOVE WCGM COUNT                 R4
         OI    SPWMSG+19,X'F0'     MAKE PRINTABLE                    R4
         B     SPRRMSG             GO TELL OPERATOR                  R4
         SPACE 1                                                     R4
         EJECT                                                      R41
SPRRUNKN OI    SPWFLAG,SPWWTO      SHOW SETUP MSG NEEDED            R41
         B     SPRRMSG             GO TELL OPERATOR                  R4
         SPACE 1                                                    R41
SPRRMODF OI    SPWFLAG,SPWCLRM     INDICATE CLEAR IF OPER RESETS    R41
         MVC   SPWMSG+23(4),DCTMODF MOVE MODF IMAGE TO MESSAGE       R4
         B     SPRRMSG             GO TELL OPERATOR                  R4
         SPACE 1                                                     R4
SPRRFCB  MVC   SPWMSG+23(4),DCTFCB MOVE FCB IMAGE ID TO MSG         R41
         B     SPRRMSG             GO TELL OPERATOR                  R4
         SPACE 1                                                    R41
SPRRCHAR CLI   SPWRTCDE,X'1C'      TEST FOR LOAD ERROR              R41
         BNE   SPRRC2              BR IF NOT - CHAR UNKNOWN          R4
         LA    R1,SPPXLAT1         ADDR OF SETPRT CHAR LIST          R4
         LA    R15,UCBCHAR1        ADDR OF UCB CHAR LIST             R4
         LA    R0,4                DETERMINE                         R4
SPRRC1   CLI   0(R15),X'40'         ID                               R4
         BE    SPRRC3                OF                              R4
         LA    R15,4(,R15)            CHAR SET                       R4
         LA    R1,4(,R1)               IN                            R4
         BCT   R0,SPRRC1                ERROR                        R4
SPRRC2   LA    R1,SPRESONS+1       ADDR OF 'UNKN' REASON             R4
SPRRC3   MVC   SPWMSG+23(4),0(R1)  MOVE CHAR SET ID TO MESSAGE       R4
         B     SPRRMSG             GO TELL OPERATOR                  R4
         SPACE 1                                                     R4
SPRRFLSH OI    SPWFLAG,SPWWTO      SHOW SETUP MSG NEEDED            R41
         SPACE 1                                                    R41
SPRRMSG  DS    0H                  ** ISSUE ERROR MSG TO OPERATOR ** R4
        $WTO   SPWMSG,38,JOB=YES,  TELL OPERATOR                     R4C
               ROUTE=$LOG+$UR,CLASS=$DOMACT,PRI=$ST                 R41
         OI    SPWFLAG,SPWSETP     FORCE CALL TO SETPRT              R4
         XC    SPPARM(SPWFORMS-SPPARM),SPPARM  RESET PARMLIST        R4
         TM    SPWFLAG,SPWWTO      TEST FOR SETUP MSG NEEDED         R4
         BZ    SPRSTOP             BR IF NOT -- WAIT FOR OPERATOR    R4
        $DOM   CMB=(R1)             ELSE DELETE 1ST MESSAGE         R41
         B     SPRMSG1             GO ISSUE SETUP MESSAGE           R41
         SPACE 1                                                    R41
SPREXIT  TM    UCBOPTNS,UCBBRSTR   DON'T VALIDATE STACKER IF        R41
         BZ    SPREXIT1             BTSS NOT INSTALLED               R4
         TM    DCTPPSW2,DCTNIBRS   TEST WANTED-STACKER               R4
         BZ    SPRXCFS             BR IF CFS                         R4
         TM    UCBACTIV,UCBBRSTA   TEST ACTUAL-STACKER               R4
         BZ    SPRMSG1             ERROR IF NOT BTSS                 R4
         B     SPREXIT1            OK IF CFS                         R4
SPRXCFS  TM    UCBACTIV,UCBBRSTA   TEST ACTUAL-STACKER               R4
         BO    SPRMSG1             ERROR IF BTSS                     R4
         EJECT                                                      R41
SPREXIT1 L     BASE2,PDSVSAVE      RESTORE BASE                     R41
         LM    PC1,PC2,PRCCWEJ     SKIP TO                          R41
         BAL   PL,PPPUT             TOP                             R41
         ICM   PC1,8,PXTABCCW               SELECT             @OZ24675
         BAL   PL,PPPUT                      CHAR1             @OZ24675
         BAL   PL,PPWRITE            OF                             R41
         BAL   PL,PPCHECK            PAGE                           R41
         L     PL,PCESAVEA         RESTORE RETURN REGISTER           R4
         BR    PL                  RETURN                            R4
         SPACE 4                                                    R41
*                                                                   R41
*        SPRTDFLT -- SUBR TO TEST IF DEFAULT CHAR IS LOADED         R41
*                                                                   R41
* OUTPUT CC    = ZERO     - DEFAULT ALREADY LOADED                  R41
*        CC    = NON-ZERO - DEFAULT NOT LOADED                      R41
*        PW    = ADDR OF CCW TO SELECT DEFAULT (IF CC=ZERO)         R41
*                                                                   R41
         SPACE 1                                                    R41
SPRTDFLT LA    R0,4                COUNT OF CHARS                   R41
         LA    R15,DCTCHAR1        ADDR OF CHARS                    R41
         LA    PW,SPRXCCWS         ADDR OF SELECT CCW TABLE         R41
         SPACE 1                                                    R41
SPRTDLUP CLC   DCTUCS,0(R15)       IS THIS THE DEFAULT...           R41
         BER   PL                  RETURN IF YES  (CC=ZERO)         R41
         LA    R15,4(,R15)         ADDR OF NEXT CHAR VALUE          R41
         LA    PW,1(,PW)            AND ITS SELECT CCW              R41
         BCT   R0,SPRTDLUP         LOOP FOR ALL CHARS               R41
         BR    PL                  RETURN NOT FOUND (CC=N-ZERO)     R41
         EJECT                                                      R41
*                                                                    R4
*        TABLE OF REASON KEYWORDS AND ASSOCIATED PROCESSORS          R4
*                                                                    R4
         SPACE 2                                                     R4
SPRESONS DS    0F                                                    R4
         SPACE 1                                                     R4
         DC    C' UNKN',AL3(SPRRUNKN)    00                          R4
         DC    C' CHAR',AL3(SPRRCHAR)    04                          R4
         DC    C'MODFY',AL3(SPRRMODF)    08                          R4
         DC    C' UNKN',AL3(SPRRUNKN)    0C                          R4
         DC    C'GRAPH',AL3(SPRRCHAR)    10                          R4
         DC    C'FLASH',AL3(SPRRFLSH)    14                          R4
         DC    C' UNKN',AL3(SPRRUNKN)    18                          R4
         DC    C' WCGM',AL3(SPRRCHAR)    1C                          R4
         DC    C'  FCB',AL3(SPRRFCB)     20                          R4
         SPACE 3                                                     R4
*                                                                    R4
*        NULL IN-CORE COPY MODIFICATION RECORD TO RESET 3800 WITHOUT R4
*                     ISSUING AN INITIALIZE-PRT                      R4
*                                                                    R4
         SPACE 2                                                     R4
SPRNULLM DS    0F                                                    R4
         SPACE 1                                                     R4
         DC    CL4'****'                COPY MOD IDENTIFICATION      R4
         DC    XL2'0'                   RESERVED                     R4
         DC    XL2'7'                   LENGTH OF COPY MOD RECORD    R4
         DC    XL1'0'                   STARTING COPY NUMBER         R4
         DC    XL1'1'                   NUMBER OF COPIES             R4
         DC    XL1'1'                   STARTING LINE NUMBER         R4
         DC    XL1'1'                   NUMBER OF LINES              R4
         DC    XL1'1'                   STARTING POSITION            R4
         DC    XL1'1'                   NUMBER OF CHARACTERS         R4
         DC    CL1' '                   MODIFICATION TEXT            R4
         SPACE 4                                                     R4
SPRXCCWS DS    0XL4                3800 TABLE-SELECT CCW OPS        R41
         DC    X'47'               SELECT TRANSLATE TABLE 0         R41
         DC    X'57'               SELECT TRANSLATE TABLE 1         R41
         DC    X'67'               SELECT TRANSLATE TABLE 2         R41
         DC    X'77'               SELECT TRANSLATE TABLE 3         R41
         EJECT                                                      R41
         LTORG                                                       R4
         SPACE 2                                                     R4
         TITLE 'HASP PRINT/PUNCH SERVICE-- FCB/UCS IMAGE READ' @OZ30048
************************************************************** @OZ30048
*                                                            * @OZ30048
* ENTER FROM HASPPRPU INITIALIZATION.  READ UCS/FCB IMAGE.   * @OZ30048
* ENTERED FOR THE FIRST USE OF UNIT ONLY.  MUST READ IMAGE   * @OZ30048
* TO DETERMINE IF UCS OR FCB IS A NON-STANDARD IMAGE.        * @OZ30048
*                                                            * @OZ30048
************************************************************** @OZ30048
         SPACE 1                                               @OZ30048
PINITSU  DS    0H                                              @OZ30048
         ST    BASE2,PDSVSAVE      SAVE BASE REGISTER          @OZ30048
         LR    BASE2,R15           SET UP                      @OZ30048
         USING PINITSU,BASE2         LOCAL ADDRESSABILITY      @OZ30048
         ST    PL,PCERETN          SAVE RETURN REGISTER        @OZ30048
         L     R1,PRIMGDTE         GET HASPIMAGE DTE ADDRESS   @OZ30048
         OC    0(4,R1),0(R1)       SUBTASK ALREADY ACTIVE...   @OZ30048
         BNZ   PGOTIMAG            BR IF YES                   @OZ30048
         SPACE 1                                               @OZ30048
         L     R15,=A(PIMAGLST)    ELSE MOVE ATTACH PARMS      @OZ30048
         MVC   $CSAVREG(PIMAGLL),0(R15)  TO WORK AREA          @OZ30048
         LR    PW,R1               SAVE DTE ADDRESS            @OZ30048
         ATTACH ECB=4(,R1),MF=(E,(1)),SF=(E,$CSAVREG)          @OZ30048
         ST    R1,0(,PW)           STORE TCB ADDRESS IN DTE    @OZ30048
         SPACE 1                                               @OZ30048
PIMGATCH DS    0H                                              @OZ30048
        $WAIT  IMAG                WAIT FOR SUBTASK TO INIT    @OZ30048
         TM    0(PW),X'80'         HASPIMAGE INITIALIZED...    @OZ30048
         BZ    PIMGATCH            BR IF NOT, TRY AGAIN        @OZ30048
         SPACE 1                                               @OZ30048
PGOTIMAG DS    0H                                              @OZ30048
         L     PC1,PCEDCT          ESTABLISH DCT               @OZ30048
         USING DCTDSECT,PC1          ADDRESSABILITY            @OZ30048
         NI    DCTPPSW,255-DCTPPSWU RESET NON-STD SWITCH       @OZ30048
         TM    PDEVTYPE+1,X'80'    UCS ALLOWED...              @OZ30048
         BZ    PFCBLD              BR IF NOT SUPPORTED         @OZ30048
         CLC   DCTUCS,=C'0   '     IS UCS = 0 REQUESTED...     @OZ30048
         BE    PFCBLD              BR IF YES, CHECK FCB        @OZ30048
         SPACE 1                                               @OZ30048
         $GETBUF PNOBUF            GET BUFFER FOR UCS READ     @OZ30048
         LR    WB,R1               LOAD BUFFER INTO R3         @OZ30048
         USING BUFDSECT,WB         SET BUFFER ADDRESSABILITY   @OZ30048
         MVC   BUFSTART(4),=X'0001003A' BLDL PARAMETER LIST    @OZ30048
         MVC   BUFSTART+4(4),=C'UCS1' IMAGE PREFIX-1403 USC    @OZ30048
         MVC   BUFSTART+8(4),DCTUCS USER UCS IMAGE ID          @OZ30048
         STCM  PC1,7,BUFDCT+1      ENSURE CORRECT DCT ADDR     @OZ30048
         CLI   PDEVTYPE+3,X'08'    IS DEVICE A 1403...         @OZ30048
         BE    PUCS01              BR IF YES, SKIP NEXT        @OZ30048
         MVI   BUFSTART+7,C'2'     IMAGE PREFIX FOR 3211 UCS   @OZ30048
         SPACE 1                                               @OZ30048
PUCS01   DS    0H                                              @OZ30048
         TM    $IMAGECB,X'40'      IS IMAGE LOADER TASK BUSY...@OZ30048
         BZ    PUCS02              BR IF NO, CONTINUE          @OZ30048
         $WAIT IMAG             WAIT FOR IMAGE TASK TO $$POST  @OZ30048
         B     PUCS01              TRY AGAIN                   @OZ30048
         SPACE 1                                               @OZ30048
PUCS02   DS    0H                                              @OZ30048
         MVI   BUFECBCC,X'80'      SET BUFFER ECB AS WAITING   @OZ30048
         POST  $IMAGECB,(R3)       POST WITH BUFFER ADDRESS    @OZ30048
         SPACE 1                                               @OZ30048
PUCS03   DS    0H                                              @OZ30048
         $WAIT IMAG             WAIT FOR IMAGE TASK TO $$POST  @OZ30048
         TM    BUFECBCC,X'7F'      IS LOAD REQUEST COMPLETE... @OZ30048
         BZ    PUCS03              BR IF NOT, TRY AGAIN        @OZ30048
         BM    PUCSMS              BR TO 'NOT FOUND' MSG       @OZ30048
         TM    BUFSTART,X'80'      IS IT A STD UCS IMAGE...    @OZ30048
         BO    PUCSEX              BR IF YES                   @OZ30048
         OI    DCTPPSW,DCTPPSWU    TURN ON NON-STD SWITCH      @OZ30048
         DROP  WB                  DROP BUFFER ADDRESSIBILITY  @OZ30048
         SPACE 1                                               @OZ30048
PUCSEX   $FREEBUF  (WB)            FREE BUFFER                 @OZ30048
         SPACE 1                                               @OZ30048
PFCBLD   DS    0H                                              @OZ30048
         NI    DCTPPSW,255-DCTPPSWB TURN OFF NON-STAND SWITCH  @OZ30048
         CLI   PDEVTYPE+3,X'08'    IS DEVICE A 1403...         @OZ30048
         BNE   PGTBUF              BR IF NOT A 1403            @OZ30048
         CLC   $PRTFCB,DCTFCB      IS IT STD FCB...            @OZ30048
         BE    PSUPEX              BR IF YES, RETURN TO INIT   @OZ30048
         OI    DCTPPSW,DCTPPSWB    TURN ON NON-STD SWITCH      @OZ30048
         B     PSUPEX              FINSHED, RETURN TO INIT     @OZ30048
         SPACE 1                                               @OZ30048
PGTBUF   $GETBUF PNOBUF            GET BUFFER FOR FCB READ     @OZ30048
         LR    WB,R1               LOAD R3 WITH BUFF ADDRESS   @OZ30048
         USING BUFDSECT,WB         SET BUFFER ADDRESSABILITY   @OZ30048
         MVC   BUFSTART(4),=X'0001003A' BLDL PARAMETER LIST    @OZ30048
         MVC   BUFSTART+4(4),=C'FCB2' IMAGE PREFIX-3211 FCB    @OZ30048
         MVC   BUFSTART+8(4),DCTFCB   USER FCB IMAGE ID        @OZ30048
         STCM  PC1,7,BUFDCT+1      ENSURE CORRECT DCT ADDR     @OZ30048
         SPACE 1                                               @OZ30048
PFCB01   DS    0H                                              @OZ30048
         TM    $IMAGECB,X'40'      IS IMAGE TASK BUSY...       @OZ30048
         BZ    PFCB02              BR IF NO, CONTINUE          @OZ30048
         $WAIT IMAG             WAIT FOR IMAGE TASK TO $$POST  @OZ30048
         B     PFCB01              TRY AGAIN                   @OZ30048
         SPACE 1                                               @OZ30048
PFCB02   DS    0H                                              @OZ30048
         MVI   BUFECBCC,X'80'      SET BUFFER ECB AS WAITING   @OZ30048
         POST  $IMAGECB,(R3)       POST WITH BUFFER ADDRESS    @OZ30048
         SPACE 1                                               @OZ30048
PFCB03   DS    0H                                              @OZ30048
         $WAIT IMAG             WAIT FOR IMAGE TASK TO $$POST  @OZ30048
         TM    BUFECBCC,X'7F'      LOAD REQUEST FINISHED...    @OZ30048
         BZ    PFCB03              BR IF NOT, TRY AGAIN        @OZ30048
         BM    PFCBMS              BR TO 'NOT FOUND' MSG       @OZ30048
         TM    BUFSTART,X'80'      TEST FOR STD IMAGE...       @OZ30048
         BO    PFCBEX              BR IF YES, SKIP NEXT        @OZ30048
         OI    DCTPPSW,DCTPPSWB    TURN ON NON-STD SWITCH      @OZ30048
         DROP  WB                  DROP BUFFER ADDRESSABILITY  @OZ30048
         SPACE 1                                               @OZ30048
PFCBEX   $FREEBUF (WB)             FREE BUFFER                 @OZ30048
         B     PSUPEX              FINSHED, GET A JOB          @OZ30048
         SPACE 1                                               @OZ30048
PUCSMS   DS    0H                                              @OZ30048
         USING BUFDSECT,WB         GET BUFFER ADDRESSABILITY   @OZ30048
         MVC   BUFSTART(2),=X'000F' MOVE MESSAGE NUMBER        @OZ30048
         MVC   BUFSTART+2(8),DCTDEVN MOVE DEVICE NAME          @OZ30048
         MVC   BUFSTART+10(26),=CL26' UCS  IMAGE XXXX NOT FOUND' Z30048
         MVC   BUFSTART+22(4),DCTUCS MOVE UCS ID               @OZ30048
        $WTO   BUFSTART,36,JOB=NO, ISSUE BUFFER LOAD FAIL MSG  @OZ30048*
               ROUTE=$LOG+$UR,CLASS=$ACTION,PRI=$ST            @OZ30048
         B     PUCSEX              FINISHED UCS, FREE BUFFER   @OZ30048
         SPACE 1                                               @OZ30048
PFCBMS   DS    0H                                              @OZ30048
         USING BUFDSECT,WB         GET BUFFER ADDRESSABILITY   @OZ30048
         MVC   BUFSTART(2),=X'000F' MOVE MSG ID                @OZ30048
         MVC   BUFSTART+2(8),DCTDEVN MOVE DEVICE NAME          @OZ30048
         MVC   BUFSTART+10(26),=CL26' FCB  IMAGE XXXX NOT FOUND' Z30048
         MVC   BUFSTART+22(4),DCTFCB MOVE FCB ID               @OZ30048
        $WTO   BUFSTART,36,JOB=NO, ISSUE BUFFER LOAD FAIL MSG  @OZ30048*
               ROUTE=$LOG+$UR,CLASS=$ACTION,PRI=$ST            @OZ30048
         B     PFCBEX              FINISHED FCB, FREE BUFFER   @OZ30048
         SPACE 1                                               @OZ30048
PNOBUF  $WTO   PNBUFF,PMSGL,JOB=NO,     BUFFER SHORTAGE MSG    @OZ30048*
               ROUTE=$LOG+$UR,CLASS=$NORMAL,PRI=$ST            @OZ30048
         B     PSUPEX              NO BUFFERS, GET A JOB       @OZ30048
         SPACE 1                                               @OZ30048
PNBUFF   $MSG  000,'NO BUFFERS TO DETERMINE IF STD UCS/FCB'    @OZ30048
PMSGL    EQU   *-PNBUFF                                        @OZ30048
         SPACE 1                                               @OZ30048
PSUPEX   DS    0H                                              @OZ30048
         L     R1,PCEDCT           GET DCT ADDRESS FOR RETURN  @OZ30048
         L     PL,PCERETN          RESTORE RETURN REGISTER     @OZ30048
         L     BASE2,PDSVSAVE      RESTORE BASE REGISTER       @OZ30048
         BR    PL                  RETURN TO INITIALIZATION    @OZ30048
         SPACE 1                                               @OZ30048
         LTORG                                                 @OZ30048
         DROP  ,                   SUSPEND ALL ADDRESSABILITY  @OZ30048
         TITLE 'HASP PRINT PUNCH SERVICE -- IMAGE LOADER TASK' @OZ30048
***********************************************************************
*                                                                     *
*        HASP IMAGE LOADER SUBTASK                                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
HASPIMAG $ENTRY BASE=R15           IMAGE LOADER MAIN ENTRY
         DROP  R15                 SUSPEND LOCAL ADDRESSABILITY
         SAVE  (14,12)             SAVE CALLER'S REGISTERS
         LR    BASE2,R15           ESTABLISH BASE REGISTER           R4
         USING HASPIMAG,BASE2      PROVIDE LOCAL ADDRESSABILITY      R4
         L     BASE1,=V(HASP)      SET HCT ADDRESS             @OZ26939
         USING HCTDSECT,BASE1      PROVIDE HCT ADDRESSABILITY        R4
         LR    R3,R1               SAVE DTE ADDR PASSED        @OZ26939
         GETMAIN R,LV=IMGSWLEN     GETMAIN SAVE/WORK AREA            R4
         ST    R13,4(,R1)          STORE BACKWARD POINTER            R4
         LR    R4,R13              HOLD CALLER'S SAVE ADDRESS
         LR    R13,R1              ADDRESS MY SAVE AREA              R4
         USING IMGDSECT,R13        PROVIDE SAVE AREA ADDRESSABILITY  R4
         ST    R13,8(,R4)          STORE FWD POINTER
         ST    R3,IMGDTE           STORE DTE ADDRESS           @OZ26939
         XC    IMGDCB,IMGDCB       CLEAR SYS1.IMAGLIB DCB ADDR @OZ26939
         SPACE 3                                                     R4
***********************************************************************
*                                                                     *
*        ESTABLISH ESTAE EXIT FOR ABNORMAL TERMINATIONS               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LA    R1,12(,SAVE)        ESTAE PARAMETER LIST AREA         R4
         MVC   0(IMGABNDL,R1),IMGABND  MOVE-IN ESTAE PARMLIST        R4
         ESTAE PARAM=(13),MF=(E,(1))  ESTABLISH IMGESTAE EXIT        R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        OPEN SYS1.IMAGELIB FOR IMPACT PRINTERS                @OZ26939
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LA    R0,$IMAGTCB         LOAD PRINTER DTE . . .      @OZ26939
         LA    R3,0(R3)            CLEAR HI-ORDER BYTE         @OZ26939
         CLR   R0,R3               IMPACT PRINTER DTE. . .     @OZ26939
         BNE   IMGACTIV            BR IF NO                    @OZ26939
         IMGLIB OPEN               OPEN IMAGE LIBRARY
         ST    R1,IMGDCB           SAVE IMAGELIB DCB ADDRESS
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SIGNAL ATTACHOR THAT IMAGE LOADER TASK IS ACTIVE             *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGACTIV OI    0(R3),X'80'         SHOW HASPIMAG INITIALIZED   @OZ26939
         LA    R0,$IMAGTCB         LOAD PRINTER DTE            @OZ26939
         CLR   R0,R3               IS IT IMPACT PRINTER...     @OZ26939
         BNE   IMG$$P              IF 3800, $$POST             @OZ26939
         POST  $PIMGECB            POST HASPINIT               @OZ26939
         B     IMGWAIT             BR TO WAIT                  @OZ26939
IMG$$P $$POST  TYPE=IMAG,R11=HCT   POST ATTACHOR               @OZ26939
         EJECT                                                 @OZ26939
***********************************************************************
*                                                                     *
*        WAIT FOR WORK POST OR $PHASP POST                            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGWAIT  L     R3,IMGDTE           ADDR OF HASPIMAG DTE        @OZ26939
         LA    R1,8(,R3)           ADDR OF HASPIMAG WORK-ECB   @OZ26939
         WAIT  ECB=(1)             WAIT FOR WORK OR $PJES2     @OZ26939
         MVI   8(R3),X'00'         RESET WORK-ECB              @OZ26939
         ICM   R3,7,8+1(R3)        IS THERE NEW WORK...        @OZ26939
         BNZ   IMGBLDL             BR IF YES. ELSE SHUTDOWN    @OZ26939
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        SHUTDOWN ASSUMED - CLOSE SYS1.IMAGELIB                       *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         L     R1,IMGDCB           GET IMAGELIB DCB ADDR       @OZ26939
         LTR   R1,R1               WAS IMAGELIB OPENED...      @OZ26939
         BZ    IMGEXIT             BR IF NO                    @OZ26939
         IMGLIB CLOSE,(1)          CLOSE IMAGELIB              @OZ26939
         SPACE 1                                               @OZ26939
IMGEXIT  L     R13,IMGSAV+4        RESTORE CALLER'S R13        @OZ26939
         RETURN (14,12),RC=0       RETURN WITH ZERO COMP CODE
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        BLDL AGAINST SYS1.IMAGELIB TO PROVIDE PARM LIST FOR LOAD     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGBLDL  DS    0H                  *
         USING BUFDSECT,R3         ACTIVATE BUFFER ADDRESSABILITY
         SPACE 1                                                     R4
         ST    R3,IMGBUFAD         SAVE BUFFER ADDRESS FOR ESTAE     R4
         SPACE 1                                                     R4
         L     R1,BUFDCT           OBTAIN ADDRESS OF PRINT/PUNCH DCT R4
         TM    DCTPPSW2-DCTDSECT(R1),DCTNIPRT  TEST FOR 3800 PRINTER R4
         BO    IMGSTPRT            BRANCH IF YES                     R4
         L     R1,IMGDCB           ADDRESS IMAGELIB DCB FOR BLDL
         BLDL  (R1),BUFSTART       CONSTRUCT LIST FOR LOAD
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        POST BUFFER WITH ERROR IF BLDL FAILED                        *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         LTR   R15,R15             WAS BLDL SUCCESSFUL
         BZ    IMGLOAD             BRANCH IF YES
         MVI   BUFECBCC,X'41'      POST BUFFER WITH PERM ERROR
         B     IMGRETN             GO TO POST PRPU                   R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        LOAD IMAGE FROM IMAGELIB USING BLDL LIST                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGLOAD  DS    0H                  *
         LM    R6,R7,BUFSTART+4    SAVE NAME OF LOADED IMAGE
         STM   R6,R7,IMGNAME        FOR DELETE LATER
         LA    R0,BUFSTART+4       ADDRESS DESCRIPTOR LIST
         L     R1,IMGDCB           ADDRESS DCB FOR IMAGELIB
         LOAD  DE=(0),DCB=(1)      LOAD REQUESTED IMAGE
         LR    R2,R0               COPY IMAGE BASE
         CLI   IMGNAME,C'F'        IS THIS AN FCB IMAGE
         BNE   IMGUCS              BRANCH IF NO
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        COPY FCB IMAGE FROM LOAD MODULE TO CALLER'S BUFFER           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGFCB   DS    0H                  *
         MVC   BUFSTART(2),0(R2)   MOVE FLAG BYTE AND LENGTH
         LA    R4,BUFSTART+2       START OF IMAGE IN BUFFER
         SLR   R5,R5               CLEAR REGISTER
         IC    R5,1(,R2)           GET IMAGE LENGTH
         LA    R6,2(,R2)           START OF IMAGE IN LOAD MODULE
         LR    R7,R5               COPY IMAGE LENGTH
         TM    2(R2),X'C0'         DOES FIRST BYTE DEFINE AN INDEX
         BNZ   IMGFCBI             BRANCH IF YES
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        NO INDEX BYTE SUPPLIED - DEFAULT TO X'81'                    *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         MVI   BUFSTART+2,X'81'    SET INDEX VALUE OF X'81'
         LA    R4,1(,R4)           INCREMENT BUFFER START ADDRESS
         LA    R5,1(,R5)           GET NEW IMAGE LENGTH
         STC   R5,BUFSTART+1       STORE INTO BUFFER
IMGFCBI  DS    0H                  *
         MVCL  R4,R6               MOVE IMAGE TO BUFFER
         B     IMGDELET            GO DELETE LOADED MODULE
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        COPY UCS IMAGE FROM LOAD MODULE TO CALLER'S BUFFER           *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGUCS   DS    0H                  *
         MVC   BUFSTART(1),0(R2)   MOVE FLAG BYTE
         LA    R4,BUFSTART+1       START OF IMAGE IN BUFFER
         LA    R5,512              ASSUME 3211 IMAGE LENGTH
         CLI   IMGNAME+3,C'2'      IS THIS A 3211 IMAGE
         BE    *+8                 BRANCH IF YES
         LA    R5,240              SET 1403 IMAGE LENGTH
         SLR   R6,R6               CLEAR REGISTER
         IC    R6,1(,R2)           NUMBER OF VERIFY LINES
         LA    R6,2(R6,R2)         START OF IMAGE IN LOAD MODULE
         LR    R7,R5               COPY MODULE LENGTH
         MVCL  R4,R6               COPY IMAGE TO BUFFER
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        DELETE THE LOADED IMAGE MODULE                               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGDELET DS    0H                  *
         DELETE EPLOC=IMGNAME      REMOVE MODULE FORM LOAD LIST
         MVI   BUFECBCC,X'7F'      POST BUFFER WITH GOOD CC
IMGRETN  DS    0H                                                    R4
       $$POST  TYPE=IMAG,R11=HCT   POST HASP PRINT PROCESSOR
         B     IMGWAIT             WAIT FOR NEXT WORK/$PHASP POST
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        INTERFACE TO SETPRT (SVC 81) FOR 3800 PRINTERS               *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
IMGSTPRT DS    0H                                                    R4
         USING SPPARM-(BUFSTART-BUFDSECT),PBUF SPPARM ADDRESSABILITY R4
         SETPRT MF=(E,SPPARM)      INVOKE SETPRT SVC                 R4
         SLR   R6,R6               CLEAR REG FOR INSERT              R4
         MVI   SPWRSCDE,X'00'      CLEAR REASON CODE AREA            R4
         ICM   R6,8,=X'7F'         ASSUME NO ERROR                   R4
         LTR   R15,R15             NORMAL RETURN                     R4
         BZ    IMGRETNJ            BRANCH IF YES                     R4
         ICM   R6,8,=X'41'         SET ERROR POST CODE               R4
*        FIND LEFT-MOST NON-ZERO BYTE AS THE RETURN CODE             R4
IMGSPTRC STCM  R15,1,SPWRTCDE      SAVE RIGHT-MOST BYTE              R4
         SRL   R15,8               SHIFT OUT SAVED BYTE              R4
         LTR   R15,R15             ALL ZEROES LEFT                   R4
         BNZ   IMGSPTRC            IF NOT, REPEAT PROCESS            R4
*        SAVE REASON CODE IF RETURN CODE = 04, 08, OR 12             R4
         CLI   SPWRTCDE,X'0C'      IS R/C LTE 12                     R4
         BH    IMGRETNJ            BRANCH IF YES                     R4
         STC   R0,SPWRSCDE         SAVE REASON CODE                  R4
IMGRETNJ DS    0H                                                    R4
         STCM  R6,8,BUFECBCC-BUFDSECT(R3)   SET POST CODE            R4
         B     IMGRETN                      GO POST PRINT PROC       R4
         EJECT                                                       R4
***********************************************************************
*                                                                     *
*        ESTAE EXIT ROUTINE -- REINSTATE SUB-TASK                     *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING IMGESTAE,R15        PROVIDE LOCAL ADDRESSABILITY      R4
         USING SDWA,R1             PROVIDE SDWA ADDRESSABILITY       R4
         SPACE 1                                                     R4
IMGESTAE LR    R3,R1               ASSUME NO SDWA -- GET COMP. CODE  R4
         LA    R4,12               TEST FOR                          R4
         CLR   R0,R4                SDWA PRESENT                     R4
         BE    SKIP760             BR IF NONE -- INFO IS IN REGS     R4
         L     R3,SDWAABCC         OBTAIN COMPLETION CODE            R4
         L     R2,SDWAPARM         OBTAIN HASPIMAG SAVE AREA ADDRESS R4
SKIP760  ST    R3,IMGABCC-IMGDSECT(,R2)  SAVE CC FOR RETRY ROUTINE   R4
         CLR   R0,R4               WAS AN SDWA PROVIDED...           R4
         BE    IMGESTA1            BR IF NO                          R4
         SETRP RC=4,RETADDR=IMGRETRY,FRESDWA=YES  SETUP FOR RETRY    R4
         BR    R14                 SCHEDULE RETRY ROUTINE            R4
         SPACE 1                                                     R4
IMGESTA1 LA    R15,4               INDICATE RETRY ROUTINE PROCESSING R4
         BALR  R0,R14              RETURN WITH R0=RETRY ROUTINE ADDR R4
         SPACE 1                                                     R4
***********************************************************************
*                                                                     *
*        ESTAE RETRY ROUTINE -- INFORM OPERATOR OF ABEND              *
*        RE-ESTABLISH ENVIRONMENT (SAVE, BASE1, BASE2, R3)            *
*                                                                     *
***********************************************************************
         SPACE 1                                                     R4
         USING IMGRETRY,R15        PROVIDE LOCAL ADDRESSABILITY      R4
         USING BUFDSECT,R3         PROVIDE BUFFER ADDRESSABILITY     R4
         SPACE 1                                                     R4
IMGRETRY LR    SAVE,R1             RESTORE HASPIMAG SAVE AREA ADDR   R4
         L     BASE1,=V(HASP)      RESTORE BASE1                     R4
         L     BASE2,=A(HASPIMAG)  RESTORE BASE2                     R4
         DROP  R15                 PICK-UP ADDRESSABILITY ON BASE2   R4
         L     R3,IMGBUFAD         RESTORE BUFFER ADDRESS            R4
         LA    R1,BUFSTART+(SPWMSG-SPPARM)  MESSAGE AREA             R4
         MVC   0(IMGAMSGL,R1),IMGAMSG  MOVE IN MESSAGE               R4
         L     R2,BUFDCT           DCT ADDRESS FROM BUFFER           R4
         MVC   IMGAMDN(8,R1),DCTDEVN-DCTDSECT(R2) INSERT DEVICE NAME R4
         UNPK  IMGAMCC(3,R1),IMGABCC+1(2)         INSERT ABEND       R4
         OI    IMGAMCC+2(R1),X'F0'                 COMPLETION        R4
         TR    IMGAMCC(3,R1),IMGTRTAB-X'F0'         CODE             R4
       $$WTO   (R1)                INFORM OPERATOR OF REINSTATEMENT  R4
         MVI   BUFECBCC,X'40'      SET ERROR CODE                    R4
         B     IMGRETN             GO TO POST PRPU                   R4
         EJECT                                                 @OZ26939
         SPACE 5                                               @OZ26939
PIMAGLST ATTACH EP=HASPIMAG,SM=SUPV,SF=L  HASPIMAG ATTACH LIST @OZ26939
PIMAGLL  EQU   *-PIMAGLST          LENGTH OF ATTACH LIST       @OZ26939
         EJECT                                                       R4
IMGTRTAB DC    C'0123456789ABCDEF' HEX TO EBCDIC TRANSLATION TABLE   R4
         SPACE 2                                                     R4
        $MID   179                 MESSAGE IDENTIFIER                R4
IMGAMSG  WTO   '&MID.******** IMAGE LOADER SUB-TASK REINSTATED AFTER ABC
               END (***)',ROUTCDE=(7),DESC=(4),MF=L                  R4
IMGAMSGL EQU   *-IMGAMSG           LENGTH OF ABOVE MESSAGE           R4
IMGAMDN  EQU   IMGAMSGL-4-59       DEVICE NAME INSERT                R4
IMGAMCC  EQU   IMGAMSGL-4-4        ABEND COMPLETION CODE INSERT      R4
         SPACE 3                                                     R4
IMGABND  ESTAE IMGESTAE,RECORD=YES,MF=L  ESTAE PARAMETER LIST        R4
IMGABNDL EQU   *-IMGABND           LENGTH OF ABOVE                   R4
         SPACE 2                                                     R4
IMGDSECT DSECT                     SAVE AREA DSECT                   R4
IMGSAV   DS    18F                 IMAGE LOADER SAVE AREA
IMGNAME  DS    2F                  NAME OF LOADED MODULE
IMGDCB   DS    F                   ADDRESS OF IMAGELIB DCB
IMGBUFAD DS    F                   BUFFER ADDRESS FOR ESTAE          R4
IMGABCC  DS    F                   ABEND COMPLETION CODE FOR RETRY   R4
IMGDTE   DS    F                   ADDRESS OF OUR DTE          @OZ26939
IMGSWLEN EQU   *-IMGDSECT          LENGTH OF SAVE/WORK AREA          R4
         SPACE 1                                                     R4
HASPPRPU CSECT                     END OF SAVE AREA DSECT            R4
         LTORG                     IMAGE LOADER LITERAL POOL
         SPACE 5
$DLENGTH $DLENGTH                  COMPUTE CONTROL SECTION LENGTH
APARNUM  DC    CL5'34285'          APAR NUMBER
         END
