         TITLE 'IGCT105C - EXCLUSIVE CONTROL TASK RECOVERY CLEANUP RTN *
                        '
IGCT105C CSECT ,                                                   0001
@MAINENT DS    0H                                                  0001
         USING *,@15                                               0001
         B     @PROLOG                                             0001
         DC    AL2(@EP00001-@MAINENT)
         DC    AL1(24)                                             0001
         DC    C'IGCT105C Z25827 01/19/78'                         0001
BACKUP5C DS    0H                                                  0001
         USING *,@15                                               0001
         B     @PROLOG                                             0001
         DC    AL2(@EP00665-BACKUP5C)
         ENTRY BACKUP5C
         DROP  @15
@PROLOG  STM   @14,@12,12(@13)                                     0001
         BALR  @09,0                                               0001
@PSTART  DS    0H                                                  0001
         USING @PSTART,@09                                         0001
         AH    @15,4(,@15)                                         0001
         BR    @15                                                 0001
@EP00001 DS    0H                                                  0002
RBPRFXLN EQU 64
RBPRFLNA EQU 32
PRBLEN   EQU 136
SIRBLEN  EQU 200
TIRBLEN  EQU 136
IRBLEN   EQU 128
SVRBLEN  EQU 200
SPDFID   EQU   X'FE'
*/********************************************************************/
*                                                                  0158
*                                                                  0158
*/********************************************************************/
*/*  THE FOLLOWING ROUTINE PERFORMS THE CLEANUP FUNCTION FOR THE     */
*/*  ESTAE.  THIS ROUTINE IS ENTERED FOR ALL ERROR CONDITIONS.  INPUT*/
*/*  TO THE ROUTINE IS THE AUDIT TRAIL BIT SET BY EXCLUSIVE CONTROL  */
*/*  MODULE TO INDICATE HOW MUCH PROCESSING WAS COMPLETED BEFORE THE */
*/*  ABEND.THIS BIT IS FOUND IN THE SECOND BYTE OF THE SVRB EXTENDED */
*/*  SAVEAREA. AS THIS ROUTINE PERFORMS CLEANUP IT LIKEWISE WILL SET */
*/*  CLEANUP RC BITS INDICATING WHAT WAS DONE.  A SECOND LEVEL       */
*/*  ESTAE WILL BE ESTABLISHED FOR THIS ROUTINE, SHOULD A PROGRAM    */
*/*  CHECK OCCUR DURING PROCESSING. THE SECOND LEVEL ESTAE WILL      */
*/*  RETURN CONTROL TO THE CALLING ROUTINE.                          */
*/*              THE CLEANUP RETURN CODES ARE AS FOLLOWS:            */
*/*                  01     CLEANUP NOT ATTEMPTED. THE REQUEST FOR   */
*/*                         WHICH THE SVC WAS ENTERED IS UNFULFILLED.*/
*/*                         ALL CONTROL BLOCKS REMAIN AS THEY WERE   */
*/*                         BEFORE THE SVC WAS ISSUED.               */
*/*                  02     AN ATTEMPT WAS MADE TO COMPLETE ENQ/DEQ  */
*/*                         PROCESSING FOR THE CURRENT REQUEST. IF   */
*/*                         USER ERROR THE CURRENT REQUEST WAS RE-   */
*/*                         MOVED FROM ALL QUEUES. IF NOT A USER     */
*/*                         ERROR PROCESSING OF THE CURRENT REQUEST  */
*/*                         WAS COMPLETED. IN ALL CASES IF THERE WAS */
*/*                         A WAITING IOB SUFFICIENTLY PROCESSED, THE*/
*/*                         I/0 WAS STARTED. THE DCB WITH EXCLUSIVE  */
*/*                         CONTROL SHOULD BE REUSABLE.              */
*/*                  03     ENQ - AN ATTEMPT WAS MADE TO RESTORE ALL */
*/*                         CONTROL BLOCKS TO THEIR STATE BEFORE THE */
*/*                         SVC WAS ISSUED.THE BLOCK WAS NOT ENQUEUED*/
*/*                         THE DCB WITH EXCLUSIVE CONTROL SHOULD BE */
*/*                         REUSABLE.                                */
*/*                  04     ENQ/DEQ - NO CLEANUP WAS DONE. IT WAS    */
*/*                         DETERMINED THAT NONE WAS REQUIRED AS ALL */
*/*                         SVC PROCESSING HAD COMPLETED.THE DCB WITH*/
*/*                         EXCLUSIVE CONTROL SHOULD BE REUSABLE.    */
*/*                         PROCEED CAUTIOUSLY,SINCE ERROR MAY BE    */
*/*                         ASSOCIATED WITH COMPLETED REQUEST.       */
*/*                  05     ENQ/DEQ - NO CLEANUP WAS DONE. ABENDING  */
*/*                         TASK IS ENQUEUED ON A BLOCK WHICH IS INAC*/
*/*                         CESSIBLE TO OTHER USERS. USE OF EXCLUSIVE*/
*/*                         CONTROL IS UNPREDICTABLE.                */
*/*                  06     DEQ - NO CLEANUP WAS ATTEMPTED. THE BLOCK*/
*/*                         FOR WHICH THE ABENDING TASK HAD CONTROL  */
*/*                         WAS DEQUEUED FROM THE SYSTEM QUEUE BUT   */
*/*                         REMAINS ON THE EXCLUSIVE CONTROL LIST.   */
*/*                         THIS BLOCK IS INACCESSIBLE TO OTHER USERS*/
*/*                  07     CLEANUP NOT ATTEMPTED BECAUSE DEBCHK     */
*/*                         FOUND INVALID DEB, AUDIT TRAIL BIT WAS   */
*/*                         DETERMINED TO BE INVALID, OR CONDITION   */
*/*                         IS UNKNOWN. RESULTS ARE UNPREDICTABLE.   */
*/*                  08     CLEANUP ROUTINE ABENDED. CONDITION IS    */
*/*                         UNKNOWN. RESULTS ARE UNPREDICTABLE.      */
*/*                                                                  */
*/*  NOTE1:REGISTERS 0-15 UPON ENTRY TO THIS MODULE ARE SAVED(R13)   */
*/*        AND RESTORED UPON EXIT. UPON RETURN FROM THIS MODULE R15  */
*/*        WILL HAVE A RC OF EITHER 0=CLEANUP COMPLETED, OR 4=CLEANUP*/
*/*        ABENDED BEFORE RETURN.                                    */
*/*                                                                  */
*/*  NOTE2:BLKCTR(R7) VALUES IDENTIFY THE RESULTS OF VALIDATING THE  */
*/*        CONTROL BLOCKS. THEY SHOULD BE INTERPRETED AS FOLLOWS:    */
*/*            1 = IOB ERROR (NOT RELEX)                             */
*/*            2 = DCB ERROR                                         */
*/*            3 = BLKREF ERROR (RELEX)                              */
*/*            4 = DECB ERROR (NOT RELEX)                            */
*/*            5 = DEB ERROR                                         */
*/*            6 = NEXT IOB ERROR                                    */
*/*            7 = SYSTEM ERROR                                      */
*/*           10 = ERROR IN PREVIOUS RECOVERY RTN OR ANOTHER RB LEVEL*/
*/*                (BRANCHED TO DUMPRTN OR CLEANRTN)                 */
*/********************************************************************/
*                                                                  0158
*CLEANUP:                                                          0158
*   ;                               /* CLEAN UP RESOURCES            */
CLEANUP  DS    0H                                                  0159
*   RFY                                                            0159
*    (GPR14P) RSTD;                 /* RESTRICT RETURN ADDR          */
*   WKASAV14=GPR14P;                /* SAVE RET ADDR FOR BACKUP      */
         ST    GPR14P,WKASAV14(,RBSECPTR)                          0160
*   RFY                                                            0161
*     GPR14P UNRSTD;                                               0161
*   WKALST=ELIST;                   /* MV LIST FORM INTO WKAREA      */
         MVC   WKALST(20,RBSECPTR),ELIST                           0162
*   WKACLENX=0;                     /* INITIALIZE CLEANUP RET COD    */
         MVI   WKACLENX(RBSECPTR),X'00'                            0163
*   WKALOCK='0'B;                   /* MAKE SURE DEBCHK GETS LOCK    */
         NI    WKALOCK(RBSECPTR),B'10111111'                       0164
*/* ISSUE ESTAE MACRO TO SET UP BACKUP ROUTINE SHOULD CLEANUP ABEND  */
*                                                                  0165
*   GEN REFS(WKALST,SDWAPTR);                                      0165
*                                                                  0165
*                                      /* ISSUE SECOND LEVEL ESTAE  */
        ESTAE PARAM=(SDWAPTR),MF=(E,WKALST(RBSECPTR))  SET UP YM5757*/
*/* NO CONTROL BLOCKS CHANGED, NO CLEANUP NECESSARY - ALL WILL REMAIN*/
*/* AS IT WAS BEFORE THE SVC WAS ISSUED.                             */
*                                                                  0166
*   IF WKATRALX^>WKAVALRC THEN      /* IF ONLY ARG PUT IN SVRB OR  0166
*                                      NOTHING DONE AT ALL           */
         CLI   WKATRALX(RBSECPTR),1                                0166
         BH    @RF00166                                            0166
*     DO;                           /* NO CHANGES WERE MADE          */
*       WKACLENX=CLEANRC1(1);       /* SET CLEANUP CODE FOR NONE     */
         IC    @12,CLEANRC1                                        0168
         STC   @12,WKACLENX(,RBSECPTR)                             0168
*       GOTO EXITCLN;               /* GO RETURN TO MAIN RTN         */
         B     EXITCLN                                             0169
*     END;                                                         0170
*                                                                  0170
*/* BEFORE FURTHER PROCESSING MUST ESTABLISH THE ADDRESS OF THE ENTRY*/
*/* IN THE READX LIST WHICH MATCHES THE ARGUMENT IN THE SVRB, IF ONE */
*/* EXISTS. THE ENTRY MAY HAVE TO BE CLEARED, OR THE IOBS CHANGED.   */
*                                                                  0171
*CLNLOCK:                                                          0171
*   ;                               /* LOCAL LOCK BEFORE CHG BLKS    */
@RF00166 DS    0H                                                  0171
CLNLOCK  DS    0H                                                  0172
*   RFY                                                            0172
*     GPR00P RSTD;                  /* RESTRICT SAVE REG       YM7519*/
*   GPR00P=SAVEPTR;                 /* SAVE RTM SAVE ADDR      YM7519*/
         LR    GPR00P,SAVEPTR                                      0173
*   DO;                             /* SETLOCK (OBTAIN) MODE(UNCOND)
*                                      TYPE(LOCAL) RELATED(RDXLST,IGC
*                                      T105C,(POSTREQ,ENQ,EXCPNXT,EXI
*                                      TCLN))                        */
*     RFY                                                          0175
*      (GPR11P,                                                    0175
*       GPR12P,                                                    0175
*       GPR13P,                                                    0175
*       GPR14P) RSTD;                                              0175
*     GPR13P=PSALITA;               /* ADDRESS OF LOCK INTERFACE   0176
*                                      TABLE.                        */
         L     GPR13P,PSALITA                                      0176
*     GEN(LM GPR11P,GPR13P,576(GPR13P));/* LOAD LOCKWORD, CLHT AND 0177
*                                      ENTRY ADRS.                   */
         LM GPR11P,GPR13P,576(GPR13P)
*     GEN(BALR GPR14P,GPR13P);      /* BALR TO SETLOCK RTN.          */
         BALR GPR14P,GPR13P
*     RFY                                                          0179
*      (GPR11P,                                                    0179
*       GPR12P,                                                    0179
*       GPR13P,                                                    0179
*       GPR14P) UNRSTD;                                            0179
*   END;                            /* SETLOCK (OBTAIN) MODE(UNCOND)
*                                      TYPE(LOCAL) RELATED(RDXLST,IGC
*                                      T105C,(POSTREQ,ENQ,EXCPNXT,EXI
*                                      TCLN))                        */
*   SAVEPTR=GPR00P;                 /* RESTOR RTM SAVE AD      YM7519*/
         LR    SAVEPTR,GPR00P                                      0181
*   RFY                                                            0182
*     GPR00P UNRSTD;                /*                         YM7519*/
*   WKALOCK='1'B;                   /* SET LOCK BIT ON FOR DEBCHK    */
         OI    WKALOCK(RBSECPTR),B'01000000'                       0183
*   CALL DEBCHECK;                  /* MUST FIRST VALIDATE DEB       */
         BAL   @14,DEBCHECK                                        0184
*   IF WKADEBXC^=0 THEN             /* IF DEB IS BAD,NO CLEANUP      */
         CLI   WKADEBXC(RBSECPTR),0                                0185
         BE    @RF00185                                            0185
*     DO;                           /* SET UP TO RETURN              */
*       WKACLENX=CLEANRC1(7);       /* CLEANUP RC = BAD DEB          */
         IC    @12,CLEANRC1+6                                      0187
         STC   @12,WKACLENX(,RBSECPTR)                             0187
*       GOTO EXITCLN;               /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0188
*     END;                                                         0189
*   RFY                                                            0190
*    (RDXPTR,                                                      0190
*     RDXENTRG) RSTD;               /* RESTRICT BASE REGS            */
@RF00185 DS    0H                                                  0191
*   RFY                                                            0191
*     DEBBASIC BASED(WKASVDEB);     /* ESTABLISH DEB BASE            */
*   RFY                                                            0192
*     DEBXTN BASED(DEBXTNP);        /* DEB EXTENSION BASE            */
*   RDXPTR=DEBXXARG;                /* READ EXCLUSIVE LIST PTR       */
         L     @12,WKASVDEB(,RBSECPTR)                             0193
         L     @12,DEBAPPAD(,@12)                                  0193
         L     @12,DEBXTNP(,@12)                                   0193
         L     RDXPTR,DEBXXARG(,@12)                               0193
*   RFY                                                            0194
*     GPR00P RSTD;                  /* RESTRC REG TO SAVE SEG PTR    */
*   RFY                                                            0195
*     RDXLIST BASED(RDXPTR);        /* ESTABLISH BASE FOR RDXLIST    */
*   GPR00P=RDXNEXT;                 /* SAVE ADDR TO NEXT SEGMENT     */
         L     GPR00P,RDXNEXT(,RDXPTR)                             0196
*RDXLOOP:                                                          0197
*   DO I=1 TO RDXMAXCT;             /* SEARCH 9 ENTRIES/SEGMENT      */
RDXLOOP  LA    I,1                                                 0197
@DL00197 DS    0H                                                  0198
*     IF RDXENT(1:8)=WKANQARG THEN  /* IF ENTRY MATCHES,SAVE         */
         CLC   RDXENT(8,RDXPTR),WKANQARG(RBSECPTR)                 0198
         BNE   @RF00198                                            0198
*       DO;                         /* THE ADDRESS IN THE SVRB       */
*         RDXENTRG=ADDR(RDXENT);    /* PUT ENTRY INTO BASE REG       */
         LA    RDXENTRG,RDXENT(,RDXPTR)                            0200
*         RFY                                                      0201
*           RDXENTRY BASED(RDXENTRG);/* RDXENTRY GETS A BASE         */
*         WKAXADDR=RDXENTRG;        /* SAVE ENTRY ADDR IN SVRB       */
         ST    RDXENTRG,WKAXADDR(,RBSECPTR)                        0202
*         GOTO CHKENQ;              /* CONTINUE CLEANUP              */
         B     CHKENQ                                              0203
*       END;                                                       0204
*     ELSE                          /* IF THAT ENTRY NOT THE ONE     */
*       RDXPTR=RDXPTR+LENGTH(RDXENTRY);/* GO TO NEXT ENTRY           */
@RF00198 AL    RDXPTR,@CF00073                                     0205
*   END RDXLOOP;                    /* RETURN TO CHECK NEXT ENTRY    */
         AL    I,@CF00099                                          0206
         C     I,@CF01554                                          0206
         BNH   @DL00197                                            0206
*   IF GPR00P^=0 THEN               /* IS THERE ANOTHER SEGMENT      */
         LTR   GPR00P,GPR00P                                       0207
         BZ    @RF00207                                            0207
*     DO;                           /* IF YES, GO CHECK NEXT SEG     */
*       RDXPTR=GPR00P;              /* YES, MAKE IT BASE FOR LIST    */
         LR    RDXPTR,GPR00P                                       0209
*       GPR00P=RDXNEXT;             /* SAVE SEGMENT AFTER THIS       */
         L     GPR00P,RDXNEXT(,RDXPTR)                             0210
*       GOTO RDXLOOP;               /* REPEAT THE SEARCH             */
         B     RDXLOOP                                             0211
*     END;                                                         0212
*   ELSE                            /* NO MATCH FOR ARG IS FOUND     */
*     WKAXADDR=0;                   /* CANT CLEAN UP                 */
@RF00207 SLR   @12,@12                                             0213
         ST    @12,WKAXADDR(,RBSECPTR)                             0213
*   RFY                                                            0214
*    (RDXPTR,                                                      0214
*     GPR00P) UNRSTD;                                              0214
*                                                                  0214
*/* ENQ REQUESTS: IF CONTROL BLOCKS WERE PARTIALLY CHANGED AND NOT   */
*/* USER ERROR,CONTINUE STEPS NOT COMPLETED BY SVC BEFORE ABEND. USER*/
*/* ERROR WAS DETERMINED, ATTEMPT TO UNDO WHAT THE SVC HAD DONE, BY  */
*/* EITHER DEQUEUING THE REQUEST FROM THE SYSTEM QUEUE AND EXCLUSIVE */
*/* LIST OR BY REMOVING THE CURRENT REQUESTOR IOB FROM THE WAIT QUEUE*/
*/* IF ALL CONTROL BLOCKS WERE CHANGED,IE. PROCESSING WAS ESSENTIALLY*/
*/* COMPLETE, NO CLEANUP IS DONE, REGARDLESS OF CAUSE OF ERROR.      */
*                                                                  0215
*CHKENQ:                                                           0215
*   IF WKAENQRQ='1'B THEN           /* CHECK FOR ENQ REQUEST         */
CHKENQ   TM    WKAENQRQ(RBSECPTR),B'00000100'                      0215
         BNO   @RF00215                                            0215
*     DO;                           /* WAS IOB PUT ON UNPOST QUE     */
*       IF(WKATRALX=WKAIOBST)&(BLKCTR>NXTIOBER) THEN/* IF IOB TO GET
*                                      CONTROL BEFORE CUR IOB SAVED
*                                      IN WORKAREA BUT CURRENT IOB 0217
*                                      NOT YET ADDED TO UNPOSTED QUE
*                                      & NOT USERERR, PUT ON         */
         CLI   WKATRALX(RBSECPTR),80                               0217
         BNE   @RF00217                                            0217
         C     BLKCTR,@CF01050                                     0217
         BNH   @RF00217                                            0217
*         DO;                       /* UNPOSTED QUEUE                */
*           IF WKAXADDR=0 THEN      /* IF NO RDXENTRY ADDR           */
         L     @12,WKAXADDR(,RBSECPTR)                             0219
         LTR   @12,@12                                             0219
         BNZ   @RF00219                                            0219
*             DO;                   /* CANNOT CLEAN UP               */
*               WKACLENX=CLEANRC1(7);/* CLEANUP CODE = UNCERTAIN     */
         IC    @12,CLEANRC1+6                                      0221
         STC   @12,WKACLENX(,RBSECPTR)                             0221
*               GOTO EXITCLN;       /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0222
*             END;                                                 0223
*           RFY                                                    0224
*             IOBSTDRD BASED(RDXUQND);/* CONTINUE ADDING NEW IOB TO
*                                      UNPOST QUE-LAST IOB ADDED IS
*                                      LAST IOB IN RDXLIST ENTRY- IT
*                                      SHOULD BE SAME AS WKAIOBQX    */
@RF00219 DS    0H                                                  0225
*           DO;                     /* MODESET KEYADDR(WKASVKEY)     */
*             SPKA(WKASVKEY);       /* SET PSW KEY                   */
         SLR   @12,@12                                             0226
         IC    @12,WKASVKEY(,RBSECPTR)                             0226
         SPKA  0(@12)                                              0226
*           END;                    /* MODESET KEYADDR(WKASVKEY)   0227
*                                      CHANGE TO USER KEY            */
*           IOBDQPTR=ABS(RBGRS1);   /* ST CURRENT IOB IN PREVIOUS- 0228
*                                      FINISH CHAINING IOB(IOBDQPTR) */
         L     @12,RDXUQND(,RDXENTRG)                              0228
         L     @11,RBGRS1(,RBSECPTR)                               0228
         LPR   @11,@11                                             0228
         ST    @11,IOBDQPTR(,@12)                                  0228
*           DO;                     /* MODESET EXTKEY(DATAMGT)       */
*             SPKA(80);             /* SET PSW KEY                   */
         LA    @12,80                                              0230
         SPKA  0(@12)                                              0230
*           END;                    /* MODESET EXTKEY(DATAMGT) CHANGE
*                                      TO DATAMGT KEY                */
*           RDXUQND=ABS(RBGRS1);    /* PUT CURRENT IOB LAST ON Q     */
         ST    @11,RDXUQND(,RDXENTRG)                              0232
*           IF RDXIOBUQ=0 THEN      /* IF NO OTHER IOB IS WAITING    */
         L     @12,RDXIOBUQ(,RDXENTRG)                             0233
         LTR   @12,@12                                             0233
         BNZ   @RF00233                                            0233
*             RDXIOBUQ=ABS(RBGRS1); /* MAKE CUR IOB FIRST ON QUE     */
         ST    @11,RDXIOBUQ(,RDXENTRG)                             0234
*           DO;                     /* MODESET EXTKEY(SUPR)          */
@RF00233 DS    0H                                                  0236
*             SPKA(0);              /* SET PSW KEY                   */
         SLR   @12,@12                                             0236
         SPKA  0(@12)                                              0236
*           END;                    /* MODESET EXTKEY(SUPR) RETURN TO
*                                      SUPER KEY                     */
*           WKACLENX=CLEANRC1(2);   /* ESTABLISH CLEANUP RC - OK     */
         IC    @12,CLEANRC1+1                                      0238
         STC   @12,WKACLENX(,RBSECPTR)                             0238
*           GOTO EXITCLN;           /* GO RETURN TO MAIN RTN         */
         B     EXITCLN                                             0239
*         END;                      /* CUR IOB PUT ON UNPOST QUE     */
*       ELSE                        /* IF IOB IS TO BE QUEUED AND    */
*         IF(WKATRALX=WKAIOBST)&(BLKCTR^>NXTIOBER) THEN/* IF A USER
*                                      ERROR WAS FOUND               */
@RF00217 CLI   WKATRALX(RBSECPTR),80                               0241
         BNE   @RF00241                                            0241
         C     BLKCTR,@CF01050                                     0241
         BH    @RF00241                                            0241
*           DO;                     /* UNDO ANY CHANGES MADE         */
*             IF WKAXADDR=0 THEN    /* IF NO RDXENTRY ADDR           */
         L     @12,WKAXADDR(,RBSECPTR)                             0243
         LTR   @12,@12                                             0243
         BNZ   @RF00243                                            0243
*               DO;                 /* CANNOT CLEAN UP               */
*                 WKACLENX=CLEANRC1(7);/* CLEANUP CODE = UNCERTAIN   */
         IC    @12,CLEANRC1+6                                      0245
         STC   @12,WKACLENX(,RBSECPTR)                             0245
*                 GOTO EXITCLN;     /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0246
*               END;                                               0247
*             ELSE                  /* TRY TO CLEAN UP IN CASE       */
*               DO;                 /* IOB WAS STORED IN PREVIOUS    */
@RF00243 DS    0H                                                  0249
*                 RFY                                              0249
*                   IOBSTDRD BASED(WKAIOBQX);/* PREV IOB IS BASE     */
*                 IF IOBDQPTR=ABS(RBGRS1) THEN/* IF PREVIOUS IOB     */
         L     @12,WKAIOBQX(,RBSECPTR)                             0250
         L     @11,RBGRS1(,RBSECPTR)                               0250
         LPR   @11,@11                                             0250
         C     @11,IOBDQPTR(,@12)                                  0250
         BNE   @RF00250                                            0250
*                   DO;             /* PTS TO CUR IOB,DEQUEUE CUR    */
*                     DO;           /* MODESET KEYADDR(WKASVKEY)     */
*                       SPKA(WKASVKEY);/* SET PSW KEY                */
         SLR   @11,@11                                             0253
         IC    @11,WKASVKEY(,RBSECPTR)                             0253
         SPKA  0(@11)                                              0253
*                     END;          /* MODESET KEYADDR(WKASVKEY)   0254
*                                      CHANGE TO USER KEY            */
*                     IOBDQPTR=0;   /* ZERO FORWARD CHAIN OF PREV    */
         SLR   @11,@11                                             0255
         ST    @11,IOBDQPTR(,@12)                                  0255
*                     DO;           /* MODESET EXTKEY(DATAMGT)       */
*                       SPKA(80);   /* SET PSW KEY                   */
         LA    @04,80                                              0257
         SPKA  0(@04)                                              0257
*                     END;          /* MODESET EXTKEY(DATAMGT) CHANGE
*                                      TO DATAMGT KEY                */
*                     RDXUQND=ADDR(IOBSTDRD);/* MAKE PREV IOB LAST   */
         ST    @12,RDXUQND(,RDXENTRG)                              0259
*                     DO;           /* MODESET EXTKEY(SUPR)          */
*                       SPKA(0);    /* SET PSW KEY                   */
         SPKA  0(@11)                                              0261
*                     END;          /* MODESET EXTKEY(SUPR) RETURN TO
*                                      SUPER KEY                     */
*                     WKACLENX=CLEANRC1(3);/* CLEANUP RC=CLEANUP OK  */
         IC    @12,CLEANRC1+2                                      0263
         STC   @12,WKACLENX(,RBSECPTR)                             0263
*                   END;                                           0264
*                 ELSE              /* IF ANOTHER IOB HAS BEEN       */
*                   WKACLENX=CLEANRC1(7);/* QUEUED,DONT CLEANUP MAY
*                                      BE OK, MAY NOT-UNCERT         */
         B     @RC00250                                            0265
@RF00250 IC    @12,CLEANRC1+6                                      0265
         STC   @12,WKACLENX(,RBSECPTR)                             0265
*               END;                                               0266
*             GOTO EXITCLN;         /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0267
*           END;                                                   0268
*       IF WKATRALX=WKAIOBQD THEN   /* USER/SYSTEM ERROR - ALL       */
@RF00241 CLI   WKATRALX(RBSECPTR),96                               0269
         BNE   @RF00269                                            0269
*         DO;                       /* CHANGES MADE SO LEAVE THEM    */
*           WKACLENX=CLEANRC1(4);   /* CLEANUP RC =OK BUT CAUTION    */
         IC    @12,CLEANRC1+3                                      0271
         STC   @12,WKACLENX(,RBSECPTR)                             0271
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0272
*         END;                                                     0273
*       IF WKATRALX=WKARDXAD THEN   /* ENTRY PUT ON RDXLIST BUT      */
@RF00269 CLI   WKATRALX(RBSECPTR),32                               0274
         BNE   @RF00274                                            0274
*         DO;                       /* NOT ON SYSTEM QUEUE           */
*           IF BLKCTR>NXTIOBER THEN /* SYSERR-PUT ON SYSTEM QUEUE    */
         C     BLKCTR,@CF01050                                     0276
         BNH   @RF00276                                            0276
*             DO;                   /* ABEND TASK MUST RELEASE IT    */
*               WKAIOBQX=ABS(RBGRS1);/* PASS IOB TO ENQ IN SVRB      */
         L     @12,RBGRS1(,RBSECPTR)                               0278
         LPR   @12,@12                                             0278
         ST    @12,WKAIOBQX(,RBSECPTR)                             0278
*               GOTO ENQ;           /* GO TO ENQ ROUTINE             */
         B     ENQ                                                 0279
*             END;                                                 0280
*           ELSE                    /* ELSE CLEAR ENTRY IN RDXLST    */
*             DO;                   /* 1ST CHECK IF ENTRY FOUND AND
*                                      NO OTHER IOBS ARE WAITING     */
@RF00276 DS    0H                                                  0282
*               IF WKAXADDR=0 THEN  /* IF NO RDXENTRY ADDR           */
         L     @12,WKAXADDR(,RBSECPTR)                             0282
         LTR   @12,@12                                             0282
         BNZ   @RF00282                                            0282
*                 DO;               /* CANNOT CLEAN UP               */
*                   WKACLENX=CLEANRC1(7);/* CLEAN CODE = UNCERTAIN   */
         IC    @12,CLEANRC1+6                                      0284
         STC   @12,WKACLENX(,RBSECPTR)                             0284
*                   GOTO EXITCLN;   /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0285
*                 END;                                             0286
*               IF RDXIOBUQ^=0 THEN /* ANOTHER IOB ALREADY WAITNG    */
@RF00282 SLR   @12,@12                                             0287
         L     @11,RDXIOBUQ(,RDXENTRG)                             0287
         CR    @11,@12                                             0287
         BE    @RF00287                                            0287
*                 DO;               /* TRY TO START NEXT REQUEST     */
*                   WKAIOBQX=RDXIOBUQ;/* PASS IOB TO EXCP IN SVRB    */
         ST    @11,WKAIOBQX(,RBSECPTR)                             0289
*                   RFY                                            0290
*                     IOBSTDRD BASED(RDXIOBUQ);/* IOB BASE WAITS     */
*                   DO;             /* MODESET EXTKEY(DATAMGT)       */
*                     SPKA(80);     /* SET PSW KEY                   */
         LA    @04,80                                              0292
         SPKA  0(@04)                                              0292
*                   END;            /* MODESET EXTKEY(DATAMGT) CHANGE
*                                      TO KEY 5                      */
*                   IF IOBDQPTR=0 THEN/* IF WAITING IOB ONLY ONE     */
         C     @12,IOBDQPTR(,@11)                                  0294
         BNE   @RF00294                                            0294
*                     DO;           /* GIVE CONTROL TO THAT IOB      */
*                       RDXIOBUQ=0; /* AND CLEAR UNPOSTED QUEUE,     */
         ST    @12,RDXIOBUQ(,RDXENTRG)                             0296
*                       RDXUQND=0;  /* FIRST AND LAST IOB PTRS       */
         ST    @12,RDXUQND(,RDXENTRG)                              0297
*                     END;                                         0298
*                   ELSE            /* IF MORE THAN ONE IOB WAITS    */
*                     RDXIOBUQ=IOBDQPTR;/* GIVE CONTROL TO 1ST AND 0299
*                                      MAKE NEXT IOB 1ST WAITING     */
         B     @RC00294                                            0299
@RF00294 L     @12,RDXIOBUQ(,RDXENTRG)                             0299
         L     @12,IOBDQPTR(,@12)                                  0299
         ST    @12,RDXIOBUQ(,RDXENTRG)                             0299
*                   DO;             /* MODESET EXTKEY(SUPR)          */
@RC00294 DS    0H                                                  0301
*                     SPKA(0);      /* SET PSW KEY                   */
         SLR   @12,@12                                             0301
         SPKA  0(@12)                                              0301
*                   END;            /* MODESET EXTKEY(SUPR) CHANGE TO
*                                      KEY ZERO                      */
*                   GOTO ENQ;       /* GO TO ENQ FOR NEXT IOB        */
         B     ENQ                                                 0303
*                 END;                                             0304
*               ELSE                /* CLEAR RDXENTRY ENTRY          */
*                 DO;               /* REMOVE CUR REQ FROM XLIST     */
@RF00287 DS    0H                                                  0306
*                   DO;             /* MODESET EXTKEY(DATAMGT)       */
*                     SPKA(80);     /* SET PSW KEY                   */
         LA    @12,80                                              0307
         SPKA  0(@12)                                              0307
*                   END;            /* MODESET EXTKEY(DATAMGT) CHANGE
*                                      TO KEY 5                      */
*                   RDXENTRY=RDXENTRY&&RDXENTRY;/* CLEAR READX ENTRY */
         XC    RDXENTRY(16,RDXENTRG),RDXENTRY(RDXENTRG)            0309
*                   DO;             /* MODESET EXTKEY(SUPR)          */
*                     SPKA(0);      /* SET PSW KEY                   */
         SLR   @12,@12                                             0311
         SPKA  0(@12)                                              0311
*                   END;            /* MODESET EXTKEY(SUPR) CHANGE TO
*                                      KEY ZERO                      */
*                   WKACLENX=CLEANRC1(3);/* CLEANUP RC-WORK UNDONE   */
         IC    @12,CLEANRC1+2                                      0313
         STC   @12,WKACLENX(,RBSECPTR)                             0313
*                   GOTO EXITCLN;   /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0314
*                 END;                                             0315
*             END;                                                 0316
*         END;                                                     0317
*       IF(WKATRALX=WKAENQ)&(BLKCTR>NXTIOBER) THEN/* IF ENQD, AND NO
*                                      USER ERROR                    */
@RF00274 CLI   WKATRALX(RBSECPTR),48                               0318
         BNE   @RF00318                                            0318
         C     BLKCTR,@CF01050                                     0318
         BNH   @RF00318                                            0318
*         DO;                       /* FINISH REQUEST BY EXCPING     */
*           WKAIOBQX=ABS(RBGRS1);   /* PASS IOB TO EXCP IN SVRB      */
         L     @12,RBGRS1(,RBSECPTR)                               0320
         LPR   @12,@12                                             0320
         ST    @12,WKAIOBQX(,RBSECPTR)                             0320
*           GOTO EXCPNXT;           /* GO TO EXCP ROUTINE            */
         B     EXCPNXT                                             0321
*         END;                                                     0322
*       IF(WKATRALX=WKAENQ)&(BLKCTR^>NXTIOBER) THEN/* USER ERR       */
@RF00318 CLI   WKATRALX(RBSECPTR),48                               0323
         BNE   @RF00323                                            0323
         C     BLKCTR,@CF01050                                     0323
         BH    @RF00323                                            0323
*         DO;                       /* DONT TRY CLEANUP,IOBS MAY     */
*           WKACLENX=CLEANRC1(5);   /* BE WAITING -CNTRL BLKS BAD    */
         IC    @12,CLEANRC1+4                                      0325
         STC   @12,WKACLENX(,RBSECPTR)                             0325
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0326
*         END;                                                     0327
*       IF WKATRALX=WKAEXCPE THEN   /* IF EXCP ALREADY ISSUED        */
@RF00323 CLI   WKATRALX(RBSECPTR),64                               0328
         BNE   @RF00328                                            0328
*         DO;                       /* THERE IS NOTHING TO DO        */
*           WKACLENX=CLEANRC1(4);   /* CLEANUP RC=PROCESSING DONE    */
         IC    @12,CLEANRC1+3                                      0330
         STC   @12,WKACLENX(,RBSECPTR)                             0330
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0331
*         END;                                                     0332
*     END;                          /* END OF ENQ REQUESTS           */
@RF00328 DS    0H                                                  0334
*/* DEQ REQUESTS: IF CONTROL BLOCKS WERE PARTIALLY CHANGED AND USER  */
*/* ERROR WAS DETERMINED, CONTINUE DEQUEUING REQUEST IF NO OTHER RE- */
*/* QUESTS ARE WAITING. DONT CLEANUP - MAY INVALIDATE CONTROL BLOCKS */
*/* FURTHER. IF NOT A USER ERROR CONTINUE DEQUEUING THIS REQUEST.    */
*/* GIVE THE BLOCK TO NEXT WAITING IOB AND ISSUE A NEW EXCP (SO THE  */
*/* ASI CAN POST THE REQUEST) OR ISSUE DEQ AND CLEAR RDXLIST ENTRY.  */
*                                                                  0334
*   IF WKADEQRQ='1'B THEN           /* IS IT A DEQ REQUEST           */
@RF00215 TM    WKADEQRQ(RBSECPTR),B'00000010'                      0334
         BNO   @RF00334                                            0334
*     DO;                           /* COMPLETE OR UNDO CHANGES      */
*       IF(WKATRALX=WKAIOBSV)&(BLKCTR>NXTIOBER) THEN/* IOB WAITING,
*                                      AND NOT A                     */
         CLI   WKATRALX(RBSECPTR),4                                0336
         BNE   @RF00336                                            0336
         C     BLKCTR,@CF01050                                     0336
         BNH   @RF00336                                            0336
*         DO;                       /* USER ERROR,CONTINUE DEQING    */
*           IF WKAXADDR=0 THEN      /* IF NO RDXENTRY ADDR           */
         L     @12,WKAXADDR(,RBSECPTR)                             0338
         LTR   @12,@12                                             0338
         BNZ   @RF00338                                            0338
*             DO;                   /* CANNOT CLEAN UP               */
*               WKACLENX=CLEANRC1(7);/* CLEANUP CODE = UNCERTAIN     */
         IC    @12,CLEANRC1+6                                      0340
         STC   @12,WKACLENX(,RBSECPTR)                             0340
*               GOTO EXITCLN;       /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0341
*             END;                                                 0342
*           RFY                                                    0343
*             IOBSTDRD BASED(WKAIOBQX);/* MAKE NEXT IOB BASE- IT WAS
*                                      SAVED IN SVRB BY SVC          */
@RF00338 DS    0H                                                  0344
*           DO;                     /* MODESET EXTKEY(DATAMGT)       */
*             SPKA(80);             /* SET PSW KEY                   */
         LA    @12,80                                              0345
         SPKA  0(@12)                                              0345
*           END;                    /* MODESET EXTKEY(DATAMGT) CHANGE
*                                      TO KEY 5                      */
*           RDXIOBUQ=IOBDQPTR;      /* TAKE NEXT IOB OFF QUE-PUT 1ST
*                                      IOB AFTER,1ST ON UNPOSTED     */
         L     @12,WKAIOBQX(,RBSECPTR)                             0347
         L     @12,IOBDQPTR(,@12)                                  0347
         ST    @12,RDXIOBUQ(,RDXENTRG)                             0347
*           IF IOBDQPTR=0 THEN      /* IF NEX IOB IS ONLY WAITING    */
         SLR   @11,@11                                             0348
         CR    @12,@11                                             0348
         BNE   @RF00348                                            0348
*             RDXUQND=0;            /* ZERO LAST IOB IN UNPOSTED     */
         ST    @11,RDXUQND(,RDXENTRG)                              0349
*           DO;                     /* MODESET EXTKEY(SUPR)          */
@RF00348 DS    0H                                                  0351
*             SPKA(0);              /* SET PSW KEY                   */
         SLR   @12,@12                                             0351
         SPKA  0(@12)                                              0351
*           END;                    /* MODESET EXTKEY(SUPR) CHANGE TO
*                                      SUPER KEY                     */
*           GOTO EXCPNXT;           /* RESTART NEXT IOB REQUEST      */
         B     EXCPNXT                                             0353
*         END;                                                     0354
*       IF(WKATRALX=WKAIOBSV)&(BLKCTR^>NXTIOBER) THEN/* USER         */
@RF00336 CLI   WKATRALX(RBSECPTR),4                                0355
         BNE   @RF00355                                            0355
         C     BLKCTR,@CF01050                                     0355
         BH    @RF00355                                            0355
*         DO;                       /* ERR IOB WAITS,DONT TAMPER     */
*           WKACLENX=CLEANRC1(5);   /* CLEANUP RC = CAUTION          */
         IC    @12,CLEANRC1+4                                      0357
         STC   @12,WKACLENX(,RBSECPTR)                             0357
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0358
*         END;                                                     0359
*       IF WKATRALX=WKAIOBDQ&BLKCTR>NXTIOBER THEN/* IOB DEQD,NOT USER
*                                      ERROR,START NEXT REQUEST      */
@RF00355 CLI   WKATRALX(RBSECPTR),5                                0360
         BNE   @RF00360                                            0360
         C     BLKCTR,@CF01050                                     0360
         BH    @RT00360                                            0360
*         GOTO EXCPNXT;             /* GO TO EXCP,ASI WILL FINISH  0361
*                                      PROCESSING NEXT IOB REQUEST   */
*       IF(WKATRALX=WKAIOBDQ)&(BLKCTR^>NXTIOBER) THEN/* IF IOB WAS 0362
*                                      DEQUEUED                      */
@RF00360 CLI   WKATRALX(RBSECPTR),5                                0362
         BNE   @RF00362                                            0362
         C     BLKCTR,@CF01050                                     0362
         BH    @RF00362                                            0362
*         DO;                       /* AND USER ERROR, DONT CLEAN    */
*           WKACLENX=CLEANRC1(5);   /* CLEANUP RC = CAUTION          */
         IC    @12,CLEANRC1+4                                      0364
         STC   @12,WKACLENX(,RBSECPTR)                             0364
*           GOTO EXITCLN;           /* RETURN TO MAIN RTN            */
         B     EXITCLN                                             0365
*         END;                                                     0366
*       IF(WKATRALX=WKADEQ)&(BLKCTR>NXTIOBER) THEN/* REQ DEQD        */
@RF00362 CLI   WKATRALX(RBSECPTR),2                                0367
         BNE   @RF00367                                            0367
         C     BLKCTR,@CF01050                                     0367
         BNH   @RF00367                                            0367
*         DO;                       /* BUT RDXENTRY NOT CLEARED      */
*           IF WKAXADDR=0 THEN      /* IF NO RDXENTRY ADDR           */
         L     @12,WKAXADDR(,RBSECPTR)                             0369
         LTR   @12,@12                                             0369
         BNZ   @RF00369                                            0369
*             DO;                   /* CANNOT CLEAN UP               */
*               WKACLENX=CLEANRC1(7);/* CLEAN CODE = UNCERTAIN       */
         IC    @12,CLEANRC1+6                                      0371
         STC   @12,WKACLENX(,RBSECPTR)                             0371
*               GOTO EXITCLN;       /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0372
*             END;                                                 0373
*           IF RDXIOBUQ^=0 THEN     /* IF NOT USER ERROR,MUST        */
@RF00369 SLR   @12,@12                                             0374
         L     @11,RDXIOBUQ(,RDXENTRG)                             0374
         CR    @11,@12                                             0374
         BE    @RF00374                                            0374
*             DO;                   /* CLEAR RDXENTRY OR GIVE BLK TO
*                                      NEXT IOB-1ST ON UNPOSTED Q    */
*               RFY                                                0376
*                 IOBSTDRD BASED(RDXIOBUQ);/* NEX IOB IS IOB BASE    */
*               WKAIOBQX=RDXIOBUQ;  /* PUT IOB IN SVRB TO EXCP       */
         ST    @11,WKAIOBQX(,RBSECPTR)                             0377
*               DO;                 /* MODESET EXTKEY(DATAMGT)       */
*                 SPKA(80);         /* SET PSW KEY                   */
         LA    @04,80                                              0379
         SPKA  0(@04)                                              0379
*               END;                /* MODESET EXTKEY(DATAMGT) CHANGE
*                                      TO KEY 5                      */
*               RDXIOBUQ=IOBDQPTR;  /* MAKE IOB CHAINED TO IT 1ST ON
*                                      UNPOSTED QUEUE                */
         L     @11,IOBDQPTR(,@11)                                  0381
         ST    @11,RDXIOBUQ(,RDXENTRG)                             0381
*               IF IOBDQPTR=0 THEN  /* IF NEXT IOB IS ONLY IOB,      */
         C     @12,IOBDQPTR(,@11)                                  0382
         BNE   @RF00382                                            0382
*                 RDXUQND=0;        /* ZERO LAST IOB-NONE WAITING    */
         ST    @12,RDXUQND(,RDXENTRG)                              0383
*               DO;                 /* MODESET EXTKEY(SUPR)          */
@RF00382 DS    0H                                                  0385
*                 SPKA(0);          /* SET PSW KEY                   */
         SLR   @12,@12                                             0385
         SPKA  0(@12)                                              0385
*               END;                /* MODESET EXTKEY(SUPR) RETURN TO
*                                      KEY 0                         */
*               GOTO ENQ;           /* REISSUE ENQ AND EXCP FOR NEXT
*                                      IOB ADDED AFTER DEQ           */
         B     ENQ                                                 0387
*             END;                                                 0388
*           ELSE                    /* IF NO IOB WAITING FOR BLK     */
*             DO;                   /* MUST CLEAR ENTRY IN RDXLST    */
@RF00374 DS    0H                                                  0390
*               DO;                 /* MODESET EXTKEY(DATAMGT)       */
*                 SPKA(80);         /* SET PSW KEY                   */
         LA    @12,80                                              0391
         SPKA  0(@12)                                              0391
*               END;                /* MODESET EXTKEY(DATAMGT) CHANGE
*                                      TO KEY 5                      */
*               RDXENTRY=RDXENTRY&&RDXENTRY;/* CLEAR READX ENTRY     */
         XC    RDXENTRY(16,RDXENTRG),RDXENTRY(RDXENTRG)            0393
*               DO;                 /* MODESET EXTKEY(SUPR)          */
*                 SPKA(0);          /* SET PSW KEY                   */
         SLR   @12,@12                                             0395
         SPKA  0(@12)                                              0395
*               END;                /* MODESET EXTKEY(SUPR) CHANGE TO
*                                      KEY ZERO                      */
*               WKACLENX=CLEANRC1(2);/* CLEANRC = TRIED & LOOKS OK   */
         IC    @12,CLEANRC1+1                                      0397
         STC   @12,WKACLENX(,RBSECPTR)                             0397
*               GOTO EXITCLN;       /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0398
*             END;                                                 0399
*         END;                      /* END OF DEQD,NOT USER ERROR    */
*       IF(WKATRALX=WKADEQ)&(BLKCTR^>NXTIOBER) THEN/* REQ DEQD       */
@RF00367 CLI   WKATRALX(RBSECPTR),2                                0401
         BNE   @RF00401                                            0401
         C     BLKCTR,@CF01050                                     0401
         BH    @RF00401                                            0401
*         DO;                       /* USER ERROR - NO CLEANUP       */
*           WKACLENX=CLEANRC1(6);   /* CLEANUP RC = CAUTION          */
         IC    @12,CLEANRC1+5                                      0403
         STC   @12,WKACLENX(,RBSECPTR)                             0403
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0404
*         END;                                                     0405
*       IF WKATRALX=WKARDXZR THEN   /* RDXENTRY IS CLEARED-ALL OK    */
@RF00401 CLI   WKATRALX(RBSECPTR),3                                0406
         BNE   @RF00406                                            0406
*         DO;                       /* PROCESSING COMPLETE FOR       */
*           WKACLENX=CLEANRC1(4);   /* THIS IOB. CLEANUP RC = OK     */
         IC    @12,CLEANRC1+3                                      0408
         STC   @12,WKACLENX(,RBSECPTR)                             0408
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0409
*         END;                                                     0410
*       IF WKATRALX=WKAENQRD THEN   /* IOB PUT ON WAIT QUE DURING DEQ
*                                      - MUST ENQ AND EXCP ON NEXT 0411
*                                      IOB SAVED IN SVRB BY SVC 53   */
@RF00406 CLI   WKATRALX(RBSECPTR),10                               0411
         BE    @RT00411                                            0411
*         GOTO ENQ;                 /* GO TO ROUTINE, ENQ & EXCP     */
*       IF WKATRALX=WKAENQX THEN    /* IF ENQD ON WAITING REQUEST  0413
*                                      ISSUE EXCP TO COMPLETE IT     */
         CLI   WKATRALX(RBSECPTR),8                                0413
         BE    @RT00413                                            0413
*         GOTO EXCPNXT;             /* GO EXCP-NEXT IOB IN SVRB      */
*       IF WKATRALX=WKAEXCPX THEN   /* IF EXCP DONE, ALL OK          */
         CLI   WKATRALX(RBSECPTR),9                                0415
         BNE   @RF00415                                            0415
*         DO;                       /* THE PROCESSING IS COMPLETE    */
*           WKACLENX=CLEANRC1(4);   /* CLEANUP RC = OK               */
         IC    @12,CLEANRC1+3                                      0417
         STC   @12,WKACLENX(,RBSECPTR)                             0417
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0418
*         END;                                                     0419
*       IF WKATRALX=WKAFEED THEN    /* FEEDBK DONE, BUT NO POST      */
@RF00415 CLI   WKATRALX(RBSECPTR),6                                0420
         BNE   @RF00420                                            0420
*         DO;                       /* POST REQUEST REGARDLESS       */
*POSTREQ:                                                          0422
*           ;                       /* FREE LOCK BEFORE POST SVC     */
POSTREQ  DS    0H                                                  0423
*           RFY                                                    0423
*             GPR00P RSTD;          /* RESTRICT SAVE REG       YM7519*/
*           GPR00P=SAVEPTR;         /* SAVE RTM SAVE ADDR      YM7519*/
         LR    GPR00P,SAVEPTR                                      0424
*           DO;                     /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK))                          */
*             RFY                                                  0426
*              (GPR11P,                                            0426
*               GPR12P,                                            0426
*               GPR13P,                                            0426
*               GPR14P) RSTD;                                      0426
*             GPR13P=PSALITA;       /* ADDRESS OF LOCK INTERFACE   0427
*                                      TABLE.                        */
         L     GPR13P,PSALITA                                      0427
*             GEN(LM GPR11P,GPR13P,588(GPR13P));/* LOAD LOCKWORD,  0428
*                                      CLHT AND ENTRY ADRS.          */
         LM GPR11P,GPR13P,588(GPR13P)
*             GEN(BALR GPR14P,GPR13P);/* BALR TO SETLOCK RTN.        */
         BALR GPR14P,GPR13P
*             RFY                                                  0430
*              (GPR11P,                                            0430
*               GPR12P,                                            0430
*               GPR13P,                                            0430
*               GPR14P) UNRSTD;                                    0430
*           END;                    /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK)) FREE LOCAL LOCK          */
*           SAVEPTR=GPR00P;         /* RESTOR RTM SAV ADDR     YM7519*/
         LR    SAVEPTR,GPR00P                                      0432
*           RFY                                                    0433
*            (GPR01P,                                              0433
*             WRKREG) RSTD;         /* OF CAUSE OF ERROR             */
*           RFY                                                    0434
*             IOBSTDRD BASED(ABS(RBGRS1));/* BASE IOB ON CURRENT     */
*           WRKREG=WKAIOBQX;        /* SET UP WORKREG WITH NEXT      */
         L     WRKREG,WKAIOBQX(,RBSECPTR)                          0435
*           IF RBGRS1<0 THEN        /* IF NOT RELEX REQ              */
         L     @12,RBGRS1(,RBSECPTR)                               0436
         LTR   @12,@12                                             0436
         BNM   @RF00436                                            0436
*             DO;                   /* CHECK IF WA,WRITTEN OUT       */
*               IF IOBADDTY='1'B&IOBADDVU='1'B THEN/* IF WRITE-ADD,  */
         LPR   @12,@12                                             0438
         TM    IOBADDTY(@12),B'00000010'                           0438
         BNO   @RF00438                                            0438
         TM    IOBADDVU(@12),B'00000010'                           0438
         BNO   @RF00438                                            0438
*                 WRKREG=ABS(RBGRS1);/* REC WRITTEN OUT, THEN FINISH
*                                      PROCESSING CURRENT IOB -SELF
*                                      FORMAT MOD WILL HANDLE NEXT   */
         LR    WRKREG,@12                                          0439
*             END;                                                 0440
@RF00438 DS    0H                                                  0441
*           RFY                                                    0441
*             IOBSTDRD BASED(WRKREG);/* IOB BASE CURRENT OR NEX      */
@RF00436 DS    0H                                                  0442
*           GPR00P=IOBSTAT2;        /* PUT CONDITION CODE IN REG     */
         SLR   GPR00P,GPR00P                                       0442
         IC    GPR00P,IOBSTAT2(,WRKREG)                            0442
*           GEN(SLL  GPR00P,16);    /* MOVE RC TO THIRD BYTE         */
         SLL  GPR00P,16
*           GPR01P=IOBECBPT;        /* PUT ECB ADDR IN R 1           */
         L     GPR01P,IOBECBPT(,WRKREG)                            0444
*           GEN(POST (1),(0));      /* POST THE REQUEST COMPLETE     */
         POST (1),(0)
*           IF DCBMRCK^='1'B THEN   /* IF CHECK ISNT SPECIFIED       */
         TM    DCBMRCK+40(DCBPTR),B'00000001'                      0446
         BO    @RF00446                                            0446
*             DO;                   /* MUST RELEASE IOB HERE         */
*               DO;                 /* MODESET KEYADDR(WKASVKEY)     */
*                 SPKA(WKASVKEY);   /* SET PSW KEY                   */
         SLR   @12,@12                                             0449
         IC    @12,WKASVKEY(,RBSECPTR)                             0449
         SPKA  0(@12)                                              0449
*               END;                /* MODESET KEYADDR(WKASVKEY)   0450
*                                      CHANGE TO USER KEY            */
*               IOBDAYLI=0;         /* MAKE THIS IOB AVAILABLE       */
         MVI   IOBDAYLI(WRKREG),X'00'                              0451
*               DO;                 /* MODESET EXTKEY(SUPR)          */
*                 SPKA(0);          /* SET PSW KEY                   */
         SLR   @12,@12                                             0453
         SPKA  0(@12)                                              0453
*               END;                /* MODESET EXTKEY(SUPR) RETURN TO
*                                      SUPER KEY                     */
*             END;                                                 0455
*           WKACLENX=CLEANRC1(2);   /* CLEANUP RC = OK,HOPEFULLY     */
@RF00446 IC    @12,CLEANRC1+1                                      0456
         STC   @12,WKACLENX(,RBSECPTR)                             0456
*           IF WRKREG^=ABS(RBGRS1) THEN/* IF JUST POSTED NEXT IOB, 0457
*                                      THE CURRENT IOB WILL NOT GET
*                                      POSTED,BUT READX Q CLEANED UP */
         L     @12,RBGRS1(,RBSECPTR)                               0457
         LPR   @12,@12                                             0457
         CR    WRKREG,@12                                          0457
         BNE   @RT00457                                            0457
*             GOTO EXITCLN;         /* SO RETURN TO MAIN ROUTINE     */
*           RFY                                                    0459
*            (WRKREG) UNRSTD;       /* RELEASE TEMPORARY IOB BASE    */
*         END;                      /* END IF FEEDBK DONE            */
*       RFY                                                        0461
*         IOBSTDRD BASED(ABS(RBGRS1));/* CURRENT IOB BASE            */
@RF00420 DS    0H                                                  0462
*       IF IOBADDTY='1'B&IOBADDVU='1'B&WKAIOBQX^=0 THEN/* WA REQ     */
         L     @12,RBGRS1(,RBSECPTR)                               0462
         LPR   @12,@12                                             0462
         TM    IOBADDTY(@12),B'00000010'                           0462
         BNO   @RF00462                                            0462
         TM    IOBADDVU(@12),B'00000010'                           0462
         BNO   @RF00462                                            0462
         L     @12,WKAIOBQX(,RBSECPTR)                             0462
         LTR   @12,@12                                             0462
         BNZ   @RT00462                                            0462
*         GOTO EXCPNXT;             /* REC WRITTEN,START NEXT REQ    */
*       IF WKATRALX=WKAPOST THEN    /* IF IOB POSTED, ALL OK         */
@RF00462 CLI   WKATRALX(RBSECPTR),7                                0464
         BNE   @RF00464                                            0464
*         DO;                       /* PROCESSING COMPLETED          */
*           WKACLENX=CLEANRC1(4);   /* CLEANRC = OK                  */
         IC    @12,CLEANRC1+3                                      0466
         STC   @12,WKACLENX(,RBSECPTR)                             0466
*           GOTO EXITCLN;           /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0467
*         END;                                                     0468
*     END;                          /* END OF DEQ PROCESSING         */
*/* IF NEITHER ENQ NOR DEQ AUDIT TRAIL BIT ON, UNKNOWN CONDITION     */
*                                                                  0470
*   ELSE                            /* ALL BITS CHECK AND STILL      */
*     DO;                           /* NO HIT,EITHER EARLY ABEND,    */
         B     @RC00334                                            0470
@RF00334 DS    0H                                                  0471
*       WKACLENX=CLEANRC1(7);       /* AUDIT TRAIL BIT INVALID       */
         IC    @12,CLEANRC1+6                                      0471
         STC   @12,WKACLENX(,RBSECPTR)                             0471
*       GOTO EXITCLN;               /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0472
*     END;                                                         0473
*                                                                  0473
*/* THE FOLLOWING ENQ/EXCP ROUTINE WILL ENQ ON THE BLOCK IN THE SVRB */
*/* UNDER THE TCB WHICH OPENED THE DCB IF THE RECORD IS IMMEDIATELY  */
*/* AVAILABLE.  IF IT IS NOT AVAILABLE, ENQ UNDER THE CURRENT TCB,   */
*/* WAIT FOR CONTROL, THEN QUICKLY DEQ IT AND ENQ AGAIN UNDER THE    */
*/* TCB WHICH OPENED THE DCB.  ONCE CONTROL IS OBTAINED IN THIS      */
*/* CLEVER, EFFICIENT, ECONOMICAL, DIRECT, UNCOMPLICATED FASHION,    */
*/* EXCP ON THE REQUEST SO YOU CAN GET OUT OF HERE AND GET HELP.     */
*                                                                  0474
*ENQ:                                                              0474
*   ;                               /* ENQ ON REQUEST                */
@RC00334 DS    0H                                                  0474
ENQ      DS    0H                                                  0475
*   RFY                                                            0475
*     GPR00P RSTD;                  /* RESTRICT SAVE REG       YM7519*/
*   GPR00P=SAVEPTR;                 /* SAVE RTM SAVE ADDR      YM7519
*                                      FREE LOCK BEFORE ENQ SVC      */
         LR    GPR00P,SAVEPTR                                      0476
*   DO;                             /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK))                          */
*     RFY                                                          0478
*      (GPR11P,                                                    0478
*       GPR12P,                                                    0478
*       GPR13P,                                                    0478
*       GPR14P) RSTD;                                              0478
*     GPR13P=PSALITA;               /* ADDRESS OF LOCK INTERFACE   0479
*                                      TABLE.                        */
         L     GPR13P,PSALITA                                      0479
*     GEN(LM GPR11P,GPR13P,588(GPR13P));/* LOAD LOCKWORD, CLHT AND 0480
*                                      ENTRY ADRS.                   */
         LM GPR11P,GPR13P,588(GPR13P)
*     GEN(BALR GPR14P,GPR13P);      /* BALR TO SETLOCK RTN.          */
         BALR GPR14P,GPR13P
*     RFY                                                          0482
*      (GPR11P,                                                    0482
*       GPR12P,                                                    0482
*       GPR13P,                                                    0482
*       GPR14P) UNRSTD;                                            0482
*   END;                            /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK)) FREE LOCAL LOCK          */
*   SAVEPTR=GPR00P;                 /* RESTOR RTM SAVE AD      YM7519*/
         LR    SAVEPTR,GPR00P                                      0484
*   RFY                                                            0485
*     GPR00P UNRSTD;                /*                         YM7519*/
*   IF BLKCTR=DEBERR THEN           /* DEB ERROR, CANT CLEANUP       */
         C     BLKCTR,@CF01507                                     0486
         BNE   @RF00486                                            0486
*     DO;                           /* NEED VALID DEB TO ENQ BLK     */
*       WKACLENX=CLEANRC1(7);       /* CLEANUP RC= CAUTION           */
         IC    @12,CLEANRC1+6                                      0488
         STC   @12,WKACLENX(,RBSECPTR)                             0488
*       GOTO EXITCLN;               /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0489
*     END;                                                         0490
*   CALL DEBCHECK;                  /* TO COMPARE TCBS MUST HAVE   0491
*                                      VALIDATED DEB                 */
@RF00486 BAL   @14,DEBCHECK                                        0491
*   IF WKADEBXC^=0 THEN             /* IF DEB BAD SET RC AND RET     */
         CLI   WKADEBXC(RBSECPTR),0                                0492
         BE    @RF00492                                            0492
*     DO;                           /* SET RET CODE FOR UNKNOWN      */
*       WKACLENX=CLEANRC1(7);       /* CLEANUP RC = CAUTION          */
         IC    @12,CLEANRC1+6                                      0494
         STC   @12,WKACLENX(,RBSECPTR)                             0494
*       GOTO EXITCLN;               /* RETURN TO MAIN ROUTINE        */
         B     EXITCLN                                             0495
*     END;                                                         0496
*   RFY                                                            0497
*     DEBBASIC BASED(WKASVDEB);     /* ESTABLISH DEB,PTR IN SVRB     */
@RF00492 DS    0H                                                  0498
*   WKANQLST=ENQLIST;               /* MV ENQ LIST INTO WORKAREA     */
         MVC   WKANQLST(12,RBSECPTR),ENQLIST                       0498
*   RFY                                                            0499
*     WRKREG RSTD;                  /* RESTRICT WORK REG             */
*/*            THE FOLLOWING MACRO WILL ENQ ON THE RECORD UNDER      */
*/*            THE TCB WHICH OPENED THE DCB IF THE RECORD IS         */
*/*            IMMEDIATELY AVAILABLE.                                */
*                                                                  0500
*ENQUSE:                                                           0500
*   ;                               /* ENQ ON TCB IN DEB FIRST       */
ENQUSE   DS    0H                                                  0501
*   IF PSATOLD^=DEBTCBB THEN        /* TCBS NOT SAME,MAY NEED 2      */
         L     @12,WKASVDEB(,RBSECPTR)                             0501
         L     @12,DEBTCBB-1(,@12)                                 0501
         LA    @12,0(,@12)                                         0501
         C     @12,PSATOLD                                         0501
         BE    @RF00501                                            0501
*     DO;                           /* ENQ,CANT ENQ ONLY ON TCB IN 0502
*                                      DEB CUZ DIRECT ENQ WONT WAIT  */
*       WRKREG=DEBTCBB;             /* PUT DEB TCB IN REG            */
         LR    WRKREG,@12                                          0503
*       RFY                                                        0504
*         GPR01P RSTD;              /* RESTRICT PARM REG       YM7597*/
*       GPR01P=ADDR(WKANQLST);      /* ENQ PARM LST IN R1      YM7597*/
         LA    GPR01P,WKANQLST(,RBSECPTR)                          0505
*       GEN REFS(WKANQARG,WKANQLST);                               0506
           ENQ  (,WKANQARG(RBSECPTR),E,,SYSTEM),TCB=(11),RET=USE,     XX
               MF=(E,(1))                                       YM7597
*       RFY                                                        0507
*         GPR01P UNRSTD;            /* RELEASE PARM REG        YM7597*/
*       RFY                                                        0508
*         GPR15P RSTD;              /* RESTRICT RETURN REG           */
*       IF GPR15P=0 THEN            /* IF GOT CONTROL OF RECORD      */
         LTR   GPR15P,GPR15P                                       0509
         BZ    @RT00509                                            0509
*         GOTO EXCPNXT;             /* GO EXCP AGAIN TO READ IT      */
*       RFY                                                        0511
*        (WRKREG,                                                  0511
*         GPR15P) UNRSTD;                                          0511
*     END;                                                         0512
*                                                                  0512
*/*            THE FOLLOWING MACRO WILL REQUEST THE RECORD           */
*/*            UNDER THIS TCB, AND WAIT UNTIL THE RECORD IS          */
*/*            AVAILABLE.                                            */
*                                                                  0513
*ENQHAVE:                                                          0513
*   ;                               /* ENQ UNDER CURRENT TCB         */
@RF00501 DS    0H                                                  0513
ENQHAVE  DS    0H                                                  0514
*   GEN REFS(WKANQARG,WKANQLST);                                   0514
*                                                                  0514
        ENQ   (,WKANQARG(RBSECPTR),E,,SYSTEM),RET=HAVE,               XX
               MF=(E,WKANQLST(RBSECPTR))                        YM7597
*/*            THE FOLLOWING MACRO WILL DEQ THE RECORD IN ORDER      */
*/*            TO SUBSEQUENTLY ENQ IT UNDER THE RIGHT TCB.           */
*                                                                  0515
*   IF PSATOLD^=DEBTCBB THEN        /* IF WAS ENQ UNDER WRONG TCB    */
         L     @12,WKASVDEB(,RBSECPTR)                             0515
         L     @11,PSATOLD                                         0515
         L     @12,DEBTCBB-1(,@12)                                 0515
         LA    @12,0(,@12)                                         0515
         CR    @11,@12                                             0515
         BE    @RF00515                                            0515
*     DO;                           /* DEQ AND REENQ AGAIN FAST      */
*       GEN REFS(WKANQLST);                                        0517
           DEQ   (,,,SYSTEM),MF=(E,WKANQLST(RBSECPTR)),RET=NONE YM7597
*       GOTO ENQUSE;                /* GO RE-ENQ ON RIGHT TCB        */
         B     ENQUSE                                              0518
*     END;                                                         0519
*                                                                  0519
*/* EXCP ON REQUEST NOW THAT EXCLUSIVE CONTROL HAS BEEN OBTAINED     */
*                                                                  0520
*EXCPNXT:                                                          0520
*   ;                               /* START NEXT REQUEST            */
@RF00515 DS    0H                                                  0520
EXCPNXT  DS    0H                                                  0521
*   RFY                                                            0521
*     GPR00P RSTD;                  /* RESTRICT SAVE REG       YM7519*/
*   GPR00P=SAVEPTR;                 /* SAVE RTM SAVE ADDR      YM7519
*                                      FREE LOCK BEFORE ENQ SVC      */
         LR    GPR00P,SAVEPTR                                      0522
*   DO;                             /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK))                          */
*     RFY                                                          0524
*      (GPR11P,                                                    0524
*       GPR12P,                                                    0524
*       GPR13P,                                                    0524
*       GPR14P) RSTD;                                              0524
*     GPR13P=PSALITA;               /* ADDRESS OF LOCK INTERFACE   0525
*                                      TABLE.                        */
         L     GPR13P,PSALITA                                      0525
*     GEN(LM GPR11P,GPR13P,588(GPR13P));/* LOAD LOCKWORD, CLHT AND 0526
*                                      ENTRY ADRS.                   */
         LM GPR11P,GPR13P,588(GPR13P)
*     GEN(BALR GPR14P,GPR13P);      /* BALR TO SETLOCK RTN.          */
         BALR GPR14P,GPR13P
*     RFY                                                          0528
*      (GPR11P,                                                    0528
*       GPR12P,                                                    0528
*       GPR13P,                                                    0528
*       GPR14P) UNRSTD;                                            0528
*   END;                            /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK)) FREE LOCAL LOCK          */
*   SAVEPTR=GPR00P;                 /* RESTOR RTM SAVE AD      YM7519*/
         LR    SAVEPTR,GPR00P                                      0530
*   RFY                                                            0531
*     GPR00P UNRSTD;                /*                         YM7519*/
*   RFY                                                            0532
*     IOBSTDRD BASED(WKAIOBQX);     /* ESTABLISH IOB BASE-NEX IOB    */
*   DO;                             /* MODESET KEYADDR(WKASVKEY)     */
*     SPKA(WKASVKEY);               /* SET PSW KEY                   */
         SLR   @12,@12                                             0534
         IC    @12,WKASVKEY(,RBSECPTR)                             0534
         SPKA  0(@12)                                              0534
*   END;                            /* MODESET KEYADDR(WKASVKEY)   0535
*                                      CHANGE TO USER KEY            */
*   IOBENQUE='1'B;                  /* TURN ON ENQ BIT-THIS IOB HAS
*                                      EXCLUSIVE CONTROL             */
         L     @12,WKAIOBQX(,RBSECPTR)                             0536
         OI    IOBENQUE(@12),B'00001000'                           0536
*   IOBDQPTR=IOBECBPT;              /* SAVE USER ECB ADDR IN CHAIN 0537
*                                      FIELD,RESTORED BY CEA         */
         L     @11,IOBECBPT(,@12)                                  0537
         ST    @11,IOBDQPTR(,@12)                                  0537
*   IOBECBPT=ADDR(IOBSTBYT);        /* GIVE IOS DUMMY ECB      YM7570*/
         LA    @11,IOBSTBYT(,@12)                                  0538
         ST    @11,IOBECBPT(,@12)                                  0538
*   RFY                                                            0539
*     GPR01P RSTD;                  /* RESTRICT EXCP PARM REG        */
*   GPR01P=ADDR(IOBSTDRD);          /* PUT ADDR OF IOB IN PARMREG    */
         LR    GPR01P,@12                                          0540
*   GENERATE;                                                      0541
        EXCP  (1)                      INITIATE I/O '1'B THIS REQ
*   DO;                             /* MODESET EXTKEY(SUPR)          */
*     SPKA(0);                      /* SET PSW KEY                   */
         SLR   @12,@12                                             0543
         SPKA  0(@12)                                              0543
*   END;                            /* MODESET EXTKEY(SUPR) RETURN TO
*                                      SUPER KEY(0)                  */
*   WKACLENX=CLEANRC1(2);           /* CLEANUP CODE-WORK COMPLETE    */
         IC    @12,CLEANRC1+1                                      0545
         STC   @12,WKACLENX(,RBSECPTR)                             0545
*   RFY                                                            0546
*     GPR01P UNRSTD;                                               0546
*                                                                  0546
*/*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *   */
*/*THE FOLLOWING ROUTINE SETS UP FOR RETURN FROM THE CLEANUP SUBRTN. */
*/*IF THIS ROUTINE IS ENTERED THE SECONDARY ESTAE WAS NOT ENTERED.   */
*/*THE SECONDARY ESTAE IS THEREFORE DELETED AND REGISTER 15 IS ZEROED*/
*/*TO INDICATE TO THE MAIN ROUTINE THAT THE SECONDARY ESTAE HAS BEEN */
*/*RELEASED.                                                         */
*/*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *   */
*                                                                  0547
*EXITCLN:                                                          0547
*   ;                               /* RETURN TO MAIN ROUTINE        */
EXITCLN  DS    0H                                                  0548
*   RFY                                                            0548
*     GPR00P RSTD;                  /* RESTRICT SAVE REG       YM7519*/
*   GPR00P=SAVEPTR;                 /* SAVE RTM SAVE ADDR      YM7519*/
         LR    GPR00P,SAVEPTR                                      0549
*   DO;                             /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK))                          */
*     RFY                                                          0551
*      (GPR11P,                                                    0551
*       GPR12P,                                                    0551
*       GPR13P,                                                    0551
*       GPR14P) RSTD;                                              0551
*     GPR13P=PSALITA;               /* ADDRESS OF LOCK INTERFACE   0552
*                                      TABLE.                        */
         L     GPR13P,PSALITA                                      0552
*     GEN(LM GPR11P,GPR13P,588(GPR13P));/* LOAD LOCKWORD, CLHT AND 0553
*                                      ENTRY ADRS.                   */
         LM GPR11P,GPR13P,588(GPR13P)
*     GEN(BALR GPR14P,GPR13P);      /* BALR TO SETLOCK RTN.          */
         BALR GPR14P,GPR13P
*     RFY                                                          0555
*      (GPR11P,                                                    0555
*       GPR12P,                                                    0555
*       GPR13P,                                                    0555
*       GPR14P) UNRSTD;                                            0555
*   END;                            /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(RDXLST,IGCT105C,(CLNLO
*                                      CK)) FREE LOCAL LOCK          */
*   SAVEPTR=GPR00P;                 /* RESTOR RTM SAVE AD      YM7519*/
         LR    SAVEPTR,GPR00P                                      0557
*   RFY                                                            0558
*     GPR00P UNRSTD;                /*                         YM7519*/
*   IF WKARLCOR^=0 THEN             /* CORE FOR RELEX STILL THERE    */
         L     @12,WKARLCOR(,RBSECPTR)                             0559
         LTR   @12,@12                                             0559
         BZ    @RF00559                                            0559
*     DO;                           /* FREE THE RELEX WORKAREA       */
*       DO;                         /* FREEMAIN(RU)                0561
*                                      LV(RELEXWKA)A(RELAREA) SP(250)*/
*         RESPECIFY                                                0562
*          (GPR01F,                                                0562
*           GPR15F,                                                0562
*           GPR00F) RESTRICTED;                                    0562
*         GPR01F=0;                 /* REG 1 MUST BE ZERO            */
         SLR   GPR01F,GPR01F                                       0563
*         GPR15F=3;                 /* SET TYPE                      */
         LA    GPR15F,3                                            0564
*         GPR00F=RELEXWKA;          /* LENGTH REQUESTED              */
         LA    GPR00F,132                                          0565
*         GPR01F=ADDR(RELAREA);     /* SET REGISTER ONE              */
         LR    GPR01F,@12                                          0566
*         GPR15F=GPR15F+((250)*256);/* SUBPOOL VALUE                 */
         AL    GPR15F,@CF03559                                     0567
*         SVC(120);                 /* RC/RU-FORM OF GETMAIN         */
         SVC   120                                                 0568
*         RESPECIFY                                                0569
*          (GPR01F,                                                0569
*           GPR15F,                                                0569
*           GPR00F) UNRESTRICTED;                                  0569
*       END;                        /* FREEMAIN(RU)                0570
*                                      LV(RELEXWKA)A(RELAREA) SP(250)
*                                      YM7519                        */
*     END;                                                         0571
*   GEN(ESTAE 0);                   /* REMOVE ESTAE FROM QUE         */
@RF00559 DS    0H                                                  0572
         ESTAE 0
*   RETURN CODE(0);                 /* RETURN TO CALLING ROUTINE,  0573
*                                      PASS BACK RC=0 IN R15         */
         SLR   @15,@15                                             0573
         L     @14,12(,@13)                                        0573
         LM    @00,@12,20(@13)                                     0573
         BR    @14                                                 0573
*                                                                  0574
*/********************************************************************/
*/*                         SUBROUTINES                              */
*/********************************************************************/
*/********************************************************************/
*/*  THE FOLLOWING SUBROUTINE VALIDITY CHECKS THE DEB. A BRANCH ENTRY*/
*/*  IS TAKEN WHICH REQUIRES THAT THE LOCAL LOCK BE HELD. INPUT TO   */
*/*  DEBCHK IS AS FOLLOWS:                                           */
*/*                           REGISTER 1 - DEB ADDRESS               */
*/*                           REGISTER 10 - TCB ADDRESS              */
*/*                           REGISTER 14 - RETURN ADDRESS           */
*/*                           REGISTER 15 - DEBCHK ROUTINE ADDRESS   */
*/*  DEBCHK RETURNS TO 4 BYTES PAST REGISTER 14 IF AN ERROR OCCURS.  */
*/*  REGISTER 15 WILL HAVE 4 IF THERE IS AN ERROR. REGISTERS 1-14    */
*/*  UPON ENTRY TO THIS SUBROUTINE ARE SAVED, AND RESTORED UPON EXIT.*/
*/*  INPUT TO THIS ROUTINE ARE: THE DCB ADDRESS IN DCBPTR(R6), SDWA  */
*/*  ADDRESS IN SDWAPTR(R5) AND THE SVRB ADDRESS IN RBSECPTR(R8).    */
*/*  THE LOCK BIT IS ON WHEN ENTRY IS FROM THE CLEANUP ROUTINE.      */
*/*  WHEN THIS BIT IS ON THIS RTN ASSUMES THE LOCK IS BEING HELD.    */
*/*  THE DEB ADDRESS AND DEBCHK RETURN CODE ARE PASSED TO THE CALLING*/
*/*  ROUTINE IN THE SVRB.                                            */
*/********************************************************************/
*                                                                  0574
*DEBCHECK:                                                         0574
*   PROC OPTIONS(NOSAVE);           /* DEBCHECK SUBROUTINE           */
DEBCHECK DS    0H                                                  0575
*   RFY                                                            0575
*    (GPR01P,                                                      0575
*     GPR14P) RSTD;                                                0575
*   STM(GPR01P,GPR14P,SAVESDWA);    /* SAVE REGS 1-14 IN SDWA        */
         STM   GPR01P,GPR14P,SAVESDWA(SDWAPTR)                     0576
*   RFY                                                            0577
*    (GPR01P,                                                      0577
*     GPR14P) UNRSTD;                                              0577
*   IF WKALOCK='0'B THEN            /* IF LOCK NOT ON, GET LOCK      */
         TM    WKALOCK(RBSECPTR),B'01000000'                       0578
         BNZ   @RF00578                                            0578
*     DO;                           /* SOME CALLERS DONT HAVE LCK    */
*/*           GET THE LOCAL LOCK                                     */
*                                                                  0580
*DEBLOCK:                                                          0580
*       ;                           /* GET LOCAL LOCK FOR DEBCHK     */
DEBLOCK  DS    0H                                                  0581
*       DO;                         /* SETLOCK (OBTAIN) MODE(UNCOND)
*                                      TYPE(LOCAL) RELATED(DEBDCB,IGC
*                                      T105C,(DEBFREE))              */
*         RFY                                                      0582
*          (GPR11P,                                                0582
*           GPR12P,                                                0582
*           GPR13P,                                                0582
*           GPR14P) RSTD;                                          0582
*         GPR13P=PSALITA;           /* ADDRESS OF LOCK INTERFACE   0583
*                                      TABLE.                        */
         L     GPR13P,PSALITA                                      0583
*         GEN(LM GPR11P,GPR13P,576(GPR13P));/* LOAD LOCKWORD, CLHT 0584
*                                      AND ENTRY ADRS.               */
         LM GPR11P,GPR13P,576(GPR13P)
*         GEN(BALR GPR14P,GPR13P);  /* BALR TO SETLOCK RTN.          */
         BALR GPR14P,GPR13P
*         RFY                                                      0586
*          (GPR11P,                                                0586
*           GPR12P,                                                0586
*           GPR13P,                                                0586
*           GPR14P) UNRSTD;                                        0586
*       END;                        /* SETLOCK (OBTAIN) MODE(UNCOND)
*                                      TYPE(LOCAL) RELATED(DEBDCB,IGC
*                                      T105C,(DEBFREE)) GET LOCAL  0587
*                                      LOCK                          */
*     END;                                                         0588
*                                                                  0588
*/*   GO TO DEBCHK ROUTINE  - HOWEVER, SINCE THERE IS ONE FIELD IN   */
*/*   THE DEB WHICH DEBCHK ASSUMES IS IN KEY 5 CORE (DEBTBLOF) MUST  */
*/*   FIRST PASS THIS FIELD TO THE SYSTEM VALIDITY CHECK ROUTINE TO  */
*/*   ENSURE ITS CORE LOCATION.                                      */
*                                                                  0589
*   RFY                                                            0589
*    (GPR01P,                                                      0589
*     GPR02P,                                                      0589
*     GPR04P,                                                      0589
*     GPR10P,                                                      0589
*     GPR14P,                                                      0589
*     GPR15P) RSTD;                                                0589
@RF00578 DS    0H                                                  0590
*   GPR01P=DCBDEBA;                 /* PUT DEB ADDR IN R1            */
         L     GPR01P,DCBDEBA+39(,DCBPTR)                          0590
         LA    GPR01P,0(,GPR01P)                                   0590
*   GPR01P=GPR01P-LENGTH(DEBTBLOF); /* GET ADDR OF FIELD       YM5738*/
         BCTR  GPR01P,0                                            0591
         BCTR  GPR01P,0                                            0591
*   GPR02P=GPR01P+1;                /* END OF FIELD            YM5738*/
         LA    GPR02P,1                                            0592
         ALR   GPR02P,GPR01P                                       0592
*   GPR04P=ADDR(DEBKEY)-KEYOFFS;    /* ADDR OF DEB KEY=5       YM5738*/
         LA    GPR04P,DEBKEY                                       0593
         SL    GPR04P,@CF02573                                     0593
*   CALL CVT0VL00;                  /* VALIDITY CHECK RTN      YM5738*/
         L     @12,CVTPTR                                          0594
         L     @15,CVT0VL00(,@12)                                  0594
         BALR  @14,@15                                             0594
*   BC(8,GOCHKDEB);                 /* DEBTBLOP IS GOOD        YM5738*/
         BC    8,GOCHKDEB                                          0595
*BADDEB:                                                           0596
*   ;                               /* DEBTBLOP IS BAD         YM5738*/
BADDEB   DS    0H                                                  0597
*   WKASVDEB=GPR01P;                /* PUT INVALID FIELD ADDR WHERE
*                                      DEB ADDR SHOULD GO      YM5738*/
         ST    GPR01P,WKASVDEB(,RBSECPTR)                          0597
*   WKADEBXC=DEBBAD;                /* RC=4 -INVALID DEB       YM5738*/
         MVI   WKADEBXC(RBSECPTR),X'04'                            0598
*   GOTO CHKLOCK;                   /* GO CHECK LOCK           YM5738*/
         B     CHKLOCK                                             0599
*   RFY                                                            0600
*    (GPR02P,                                                      0600
*     GPR04P) UNRSTD;               /* RELEASE REGS 2,4        YM5738*/
*/*       READY TO GO TO DEBCHK ROUTINE                              */
*                                                                  0601
*GOCHKDEB:                                                         0601
*   ;                               /* GO TO DEBCHK ROUT       YM5738*/
GOCHKDEB DS    0H                                                  0602
*   GPR01P=GPR01P+LENGTH(DEBTBLOF); /* GET DEB ADDR IN R1      YM5738*/
         AL    GPR01P,@CF00111                                     0602
*   GPR10P=PSATOLD;                 /* PUT TCB ADDR IN R10           */
         L     GPR10P,PSATOLD                                      0603
*   CALL CVTDEBVR;                  /* BRANCH ENTRY TO DEBCHK        */
         L     @12,CVTPTR                                          0604
         L     @12,CVTEXT2(,@12)                                   0604
         L     @15,CVTDEBVR(,@12)                                  0604
         BALR  @14,@15                                             0604
*   GEN(NOP 0);                     /* USE UP 4 BYTES ON RETURN      */
         NOP 0
*   WKASVDEB=GPR01P;                /* SAVE VALIDATED DEB ADDR IN  0606
*                                      SVRB WORKAREA                 */
         ST    GPR01P,WKASVDEB(,RBSECPTR)                          0606
*   WKADEBXC=RETURNCD;              /* SAVE DEBCHK RC IN TOP BYTE    */
         STC   RETURNCD,WKADEBXC(,RBSECPTR)                        0607
*   RFY                                                            0608
*     DEBBASIC BASED(WKASVDEB);     /* EST ADDRESSABILITY TO DEB     */
*   IF(RETURNCD=0)&(DEBDCBB^=DCBPTR) THEN/* IF DEB VALID ENSURE IT 0609
*                                      POINTS TO DCB                 */
         LTR   RETURNCD,RETURNCD                                   0609
         BNZ   @RF00609                                            0609
         L     @12,WKASVDEB(,RBSECPTR)                             0609
         L     @12,DEBDCBB-1(,@12)                                 0609
         LA    @12,0(,@12)                                         0609
         CR    DCBPTR,@12                                          0609
         BE    @RF00609                                            0609
*     WKADEBXC=DEBBAD;              /* NO,PUT IN RC=4-INVALID DEB    */
         MVI   WKADEBXC(RBSECPTR),X'04'                            0610
*CHKLOCK:                                                          0611
*   ;                               /* SHOULD LOCK BE FREE     YM5738*/
@RF00609 DS    0H                                                  0611
CHKLOCK  DS    0H                                                  0612
*   IF WKALOCK='0'B THEN            /* IF LOCK SW NOT ON, THEN       */
         TM    WKALOCK(RBSECPTR),B'01000000'                       0612
         BNZ   @RF00612                                            0612
*     DO;                           /* MUST FREE LOCK GOTTEN HERE    */
*/*                   RELEASE THE LOCAL LOCK                         */
*                                                                  0614
*DEBFREE:                                                          0614
*       ;                           /* RELEASE LOCAL LOCK -DEBCHK    */
DEBFREE  DS    0H                                                  0615
*       DO;                         /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(DEBDCB,IGCT105C,(DEBLO
*                                      CK))                          */
*         RFY                                                      0616
*          (GPR11P,                                                0616
*           GPR12P,                                                0616
*           GPR13P,                                                0616
*           GPR14P) RSTD;                                          0616
*         GPR13P=PSALITA;           /* ADDRESS OF LOCK INTERFACE   0617
*                                      TABLE.                        */
         L     GPR13P,PSALITA                                      0617
*         GEN(LM GPR11P,GPR13P,588(GPR13P));/* LOAD LOCKWORD, CLHT 0618
*                                      AND ENTRY ADRS.               */
         LM GPR11P,GPR13P,588(GPR13P)
*         GEN(BALR GPR14P,GPR13P);  /* BALR TO SETLOCK RTN.          */
         BALR GPR14P,GPR13P
*         RFY                                                      0620
*          (GPR11P,                                                0620
*           GPR12P,                                                0620
*           GPR13P,                                                0620
*           GPR14P) UNRSTD;                                        0620
*       END;                        /* SETLOCK (RELEASE) TYPE(LOCAL)
*                                      RELATED(DEBDCB,IGCT105C,(DEBLO
*                                      CK)) FREE LOCAL LOCK          */
*     END;                                                         0622
*   RFY                                                            0623
*     GPR14P RSTD;                                                 0623
@RF00612 DS    0H                                                  0624
*   LM(GPR01P,GPR14P,SAVESDWA);     /* RESTORE REGS1-14 FROM SDWA    */
         LM    GPR01P,GPR14P,SAVESDWA(SDWAPTR)                     0624
*   WKALOCK='0'B;                   /* ZERO LOCK SW IN ALL CASES     */
         NI    WKALOCK(RBSECPTR),B'10111111'                       0625
*   END DEBCHECK;                                                  0626
@EL00002 DS    0H                                                  0626
@EF00002 DS    0H                                                  0626
@ER00002 BR    @14                                                 0626
*   RFY                                                            0627
*    (GPR01P,                                                      0627
*     GPR10P,                                                      0627
*     GPR14P,                                                      0627
*     GPR15P) UNRSTD;                                              0627
*                                                                  0628
*/********************************************************************/
*/* THE FOLLOWING SUBROUTINE RECEIVES CONTROL FROM THE 2ND LEVEL     */
*/* ESTAE, THE DUMPRTN1 ROUTINE, TO TAKE AN SVC DUMP. A HEADER       */
*/* IS BUILT IN THE SDWA TO IDENTIFY THE DUMP. THE ONLY REGS CHANGED */
*/* BY THIS SUBROUTINE ARE 0,1 AND 15. UPON ENTRY THE LOCAL LOCK     */
*/* IS NOT HELD.  THE SDUMP SVC WILL RETURN A RETURN CODE IN R15,    */
*/* THE VALUES FOR WHICH ARE AS FOLLOWS:                             */
*/*                                00 - SDUMP SUCCESSFUL             */
*/*                                04 - PARTIAL SDUMP TAKEN          */
*/*                                08 - SDUMP UNSUCCESSFUL           */
*/* THE SDUMP RETURN CODE PRESENTED TO THE USER IS THIS VALUE +4.    */
*/*                                04 - SDUMP SUCCESSFUL             */
*/*                                08 - PARTIAL SDUMP TAKEN          */
*/*                                0C - SDUMP UNSUCCESSFUL           */
*/* A BIT IS SET IN THE SVRB WORKAREA TO IDENTIFY THE RETURN CODE.   */
*/********************************************************************/
*                                                                  0628
*SDUMP:                                                            0628
*   PROC OPTIONS(NOSAVE);           /* TAKE AN SVC DUMP              */
SDUMP    DS    0H                                                  0629
*   RFY                                                            0629
*    (GPR00P,                                                      0629
*     GPR01P,                                                      0629
*     GPR14P,                                                      0629
*     GPR15P) RSTD;                                                0629
*   HEADER=HEADING;                 /* SET UP HEADER                 */
         MVC   HEADER(27,SDWAPTR),HEADING                          0630
*   JOBNAME=TIOCNJOB;               /* JOB NAME                      */
         L     @12,PSATOLD                                         0631
         L     @12,TCBTIO(,@12)                                    0631
         MVC   JOBNAME(8,SDWAPTR),TIOCNJOB(@12)                    0631
*   STEPNAME=TIOCSTEP;              /* STEP NAME                     */
         MVC   STEPNAME(8,SDWAPTR),TIOCSTEP(@12)                   0632
*   SDUMPLST=DUMPLIST;              /* MOVE LIST FORM TO VAR.AREA    */
         MVC   SDUMPLST(24,SDWAPTR),DUMPLIST                       0633
*   GPR01P=ADDR(SDUMPLST);          /* PUT DUMP LIST ADDR IN REG     */
         LA    GPR01P,SDUMPLST(,SDWAPTR)                           0634
*   GEN REFS(HEADER) SETS(SDUMPLST,GPR00P,GPR01P);                 0635
        SDUMP HDRAD=HEADER(SDWAPTR),MF=(E,(1))  TAKE SVC DUMP
*   SDWAEAS='1'B;                   /* SET DUMP TAKEN FLAG           */
         OI    SDWAEAS(SDWAPTR),B'00001000'                        0636
*   IF RETURNCD=0 THEN              /* IF SDUMP SUCCESSFUL           */
         LTR   RETURNCD,RETURNCD                                   0637
         BNZ   @RF00637                                            0637
*     WKADMP04='1'B;                /* SAVE RETURN CODE = 4          */
         OI    WKADMP04(RBSECPTR),B'00001000'                      0638
*   IF RETURNCD=DUMPRC4 THEN        /* IF PARTIAL SDUMP TAKEN        */
@RF00637 C     RETURNCD,@CF00092                                   0639
         BNE   @RF00639                                            0639
*     WKADMP08='1'B;                /* SAVE RETURN CODE = 8          */
         OI    WKADMP08(RBSECPTR),B'00010000'                      0640
*   IF RETURNCD=DUMPRC8 THEN        /* IF SDUMP UNSUCCESSFUL         */
@RF00639 C     RETURNCD,@CF00047                                   0641
         BNE   @RF00641                                            0641
*     WKADMP0C='1'B;                /* SAVE RETURN CODE = C          */
         OI    WKADMP0C(RBSECPTR),B'00100000'                      0642
*   END SDUMP;                      /* RETURN TO MAIN ROUTINE        */
@EL00003 DS    0H                                                  0643
@EF00003 DS    0H                                                  0643
@ER00003 BR    @14                                                 0643
*   RFY                                                            0644
*    (GPR00P,                                                      0644
*     GPR01P,                                                      0644
*     GPR14P,                                                      0644
*     GPR15P) UNRSTD;                                              0644
*                                                                  0645
*/********************************************************************/
*/*  THE FOLLOWING SUBROUTINE IS CALLED BY THE DUMPRTN1 IN THE SECOND*/
*/*  LEVEL ESTAE TO SET UP THE SDWA WHEN A SYSTEM ERROR HAS BEEN     */
*/*  DETECTED. THE FOLLOWING RECORDING INFORMATION WILL BE PUT IN    */
*/*  THE VARIABLE PORTION OF THE SDWA (FROM WHERE RTM WILL GET IT TO */
*/*  WRT ON SYS1.LOGREC):  SVC AUDIT TRAIL BYTE                      */
*/*                        RETURN CODE FROM CLEANRTN                 */
*/*                        RETURN CODE FROM SDUMP                    */
*/*                        REGISTERS 0 AND 1 AT TIME SVC ISSUED      */
*/*                        RB OLD PSW CONTAINING THE ADDRESS+2 FROM  */
*/*                           WHERE THE SVC WAS ISSUED               */
*/*                        OLD ABEND CODE IF PERCOLATED ERROR        */
*/*  THE ONLY REGISTER CHANGED BY THIS SUBROUTINE IS REG 1.          */
*/********************************************************************/
*                                                                  0645
*RECORD:                                                           0645
*   PROC OPTIONS(NOSAVE);           /* PUT SYSTEM INFO IN SDWA       */
RECORD   DS    0H                                                  0646
*   RFY                                                            0646
*     GPR14P RSTD;                  /* RESTRICT RETURN ADDR          */
*   RECAUDIT=WKATRALX;              /* MV AUDIT TRAIL BIT IN SDWA    */
         IC    @15,WKATRALX(,RBSECPTR)                             0647
         STC   @15,RECAUDIT(,SDWAPTR)                              0647
*   IF WKADMP04='1'B THEN           /* IF SDUMP SUCCESSFUL THEN      */
         TM    WKADMP04(RBSECPTR),B'00001000'                      0648
         BNO   @RF00648                                            0648
*     RECRCDP=DUMPRC1(1);           /* MV INTO REC AREA RET COD=4    */
         IC    @15,DUMPRC1                                         0649
         STC   @15,RECRCDP(,SDWAPTR)                               0649
*   IF WKADMP08='1'B THEN           /* IF PARTIAL SDUMP TAKEN        */
@RF00648 TM    WKADMP08(RBSECPTR),B'00010000'                      0650
         BNO   @RF00650                                            0650
*     RECRCDP=DUMPRC1(2);           /* MV INTO REC AREA RET COD=8    */
         IC    @15,DUMPRC1+1                                       0651
         STC   @15,RECRCDP(,SDWAPTR)                               0651
*   IF WKADMP0C='1'B THEN           /* IF SDUMP WAS UNSUCCESSFUL     */
@RF00650 TM    WKADMP0C(RBSECPTR),B'00100000'                      0652
         BNO   @RF00652                                            0652
*     RECRCDP=DUMPRC1(3);           /* MV INTO REC AREA RET COD=C    */
         IC    @15,DUMPRC1+2                                       0653
         STC   @15,RECRCDP(,SDWAPTR)                               0653
*   RECRCLN=WKACLENX;               /* MV CLEANUP RTN RET CODE IN    */
@RF00652 IC    @15,WKACLENX(,RBSECPTR)                             0654
         STC   @15,RECRCLN(,SDWAPTR)                               0654
*   RECREG0=RBGRS0;                 /* REG0 WHEN SVC WAS ISSUED      */
         L     @15,RBGRS0(,RBSECPTR)                               0655
         ST    @15,RECREG0(,SDWAPTR)                               0655
*   RECREG1=RBGRS1;                 /* REG1 WHEN SVC WAS ISSUED      */
         L     @15,RBGRS1(,RBSECPTR)                               0656
         ST    @15,RECREG1(,SDWAPTR)                               0656
*   RFY                                                            0657
*     GPR01P RSTD;                  /* RESTRICT WORK REG             */
*   GPR01P=RBLINK;                  /* GET PREVIOUS RB ADDR          */
         L     GPR01P,RBLINK(,RBSECPTR)                            0658
*   RFY                                                            0659
*     RBBASIC BASED(GPR01P);        /* BASE FOR PREVIOUS RB          */
*   RECOPSW=RBOPSW;                 /* GET PSW FROM PREVIOUS RB      */
         MVC   RECOPSW(8,SDWAPTR),RBOPSW(GPR01P)                   0660
*   RECPERC=SDWACMPC;               /* PUT IN PREVIOUS ABEND CODE    */
         MVC   RECPERC(3,SDWAPTR),SDWACMPC(SDWAPTR)                0661
*   RFY                                                            0662
*     RBBASIC BASED(RBSECPTR);      /* RETURN TO RB BASE FOR SVC     */
*   END RECORD;                     /* RETURN TO MAIN ROUTINE        */
@EL00004 DS    0H                                                  0663
@EF00004 DS    0H                                                  0663
@ER00004 BR    @14                                                 0663
*   RFY                                                            0664
*    (GPR01P,                                                      0664
*     GPR14P) UNRSTD;                                              0664
*                                                                  0665
*/********************************************************************/
*/*  THE FOLLOWING ROUTINE IS THE SECOND LEVEL ESTAE FOR IGCT105C.   */
*/*  IT IS ENTERED ONLY IF AN ERROR OCCURS DURING THE PROCESSING OF  */
*/*  THE CLEANUP ROUTINE.  IT WILL EXECUTE MANY OF THE FUNCTIONS OF  */
*/*  THE FIRST LEVEL ESTAE, BUT WILL DO NO VALIDATION OR CLEANUP.ITS */
*/*  PRIMARY PURPOSE IS TO RETURN CONTROL TO THE MAIN ESTAE AFTER    */
*/*  RECORDING THAT A SECONDARY ESTAE WAS ENTERED.                   */
*/*                          INPUT: SDWA(NEW FOR THIS ESTAE) ADDRESS */
*/*                                 SDWA(FOR PRIMARY ESTAE) WHICH    */
*/*                                   CONTAINS SVRB ADDR FOR SVC 53  */
*/********************************************************************/
*                                                                  0665
*BACKUP5C:                                                         0665
*   ENTRY;                          /* SECOND LEVEL ESTAE FUNCT      */
@EP00665 DS    0H                                                  0666
*   IF GPR00F=12 THEN               /* IF SAVE AREA AND      @ZA25827*/
         C     GPR00F,@CF03306                                     0666
         BNE   @RF00666                                            0666
*     DO;                           /* SDWA NOT PRESENT,     @ZA25827*/
*       GEN(LM 14,12,12(13));       /* RESTORE REGISTERS     @ZA25827*/
         LM 14,12,12(13)
*       GEN(BR (14));               /* AND RETURN TO         @ZA25827*/
         BR (14)
*     END;                          /* RTM                   @ZA25827*/
*   ELSE                            /*                       @ZA25827*/
*     DO;                           /*                       @ZA25827*/
         B     @RC00666                                            0671
@RF00666 DS    0H                                                  0672
*       GEN(STM @14,@12,12(@13));   /* ST REGS IN 2ND RTN     SAVAREA*/
         STM @14,@12,12(@13)
*     END;                          /*                       @ZA25827*/
*   RFY                                                            0674
*    (GPR01P) RSTD;                 /* RESTRICT INPUT AND RETRY      */
@RC00666 DS    0H                                                  0675
*   SDWAPTR=GPR01P;                 /* GET ADDRESS OF NEW SDWA       */
         LR    SDWAPTR,GPR01P                                      0675
*   SDWASR15=CLEANERR;              /* RC=4(TESTED BY 1ST LEVEL ON 0676
*                                      RET FROM CLEANUP INDICATES  0676
*                                      ENTRY INTO 2ND LEVEL ESTAE    */
         MVC   SDWASR15(4,SDWAPTR),@CF00092                        0676
*   SDWACSCT='IGCT105C';            /* PUT CSECT ID IN NEW SDWA      */
         MVC   SDWACSCT(8,SDWAPTR),@CC03380                        0677
*   SDWAREXN='BACKUP5C';            /* PUT ESTAE ID IN NEW SDWA      */
         MVC   SDWAREXN(8,SDWAPTR),@CC03549                        0678
*   GPR01P=SDWAPARM;                /* RESTORE OLD SDWA PASSED AS  0679
*                                      PARM IN NEW SDWA- SAVE IN REG1
*                                      (WILL USE OLD SDWA FOR      0679
*                                      USERRTN1 PROCESSING)          */
         L     GPR01P,SDWAPARM(,SDWAPTR)                           0679
*   RBSECPTR=GPR01P->SDWAPARM;      /* RESTORE SVRB FROM OLD SDWA    */
         L     RBSECPTR,SDWAPARM(,GPR01P)                          0680
*/********************************************************************/
*/*  ISSUE MACRO TO ANALYZE ABEND AND GIVE CONTROL TO APPROPRIATE RTN*/
*/********************************************************************/
*                                                                  0681
*   DO;                             /* ?IGGSTART USER(USERRTN1)SDUMP(
*                                      DUMPRTN1)CLEAN(DUMPRTN1)      */
*/*  MACCOMP Y-2 SC1D0/SJ0082                                        */
*/*  MACSTAT Y-2 75030/ZA02517                                       */
*/********************************************************************
*                                                                    */
*/* THIS ROUTINE ANALYSES FLAGS IN THE SDWA TO DETERMINE             */
*/* WHAT TYPE OF FUNCTION TO PERFORM                                 */
*/* THE FLAG COMBINATIONS TESTED AND THE ACTIONS TAKEN ARE SUMMARIZED*/
*/* IN THE FOLLOWING TABLE:                                          */
*/*                                                                  */
*/*   PREVIOUS  || CAUSE OF ERROR ENTRY                              */
*/*  PROCESSING || MCHK/PGIO/TEXC  OTHER                             */
*/*  ___________||________________________                           */
*/*       *EAS  ||        C          C                               */
*/*       *CLUP ||        C          C                               */
*/*       *STAF ||        S          S                               */
*/*       *PERC ||        C          U                               */
*/*       *NRBE ||        C          U                               */
*/*        NONE ||        C          U                               */
*/*                                                                  */
*/*       * THESE BITS ARE NOT ALL MUTUALLY EXCLUSIVE. THEY ARE      */
*/*         TESTED IN THE ORDER LISTED.                              */
*/*                                                                  */
*/*       * USER ABENDS ALWAYS GO TO THE CLEANRTN.                   */
*/*                                                                  */
*/*       FUNCTIONS TO BE PERFORMED:                                 */
*/*                                                                  */
*/* C (DUMPRTN1)--CLEAN UP RESOURCES AND PERCOLATE ERROR             */
*/*                                                                  */
*/* S (DUMPRTN1)--PREVIOUS RECOVERY ROUTINE FAILED.                  */
*/* TAKE A DUMP FOR IT, THEN TREAT AS C                              */
*/*                                                                  */
*/* U (USERRTN1)--VALIDATE USER INPUT. IF IT IS IN ERROR,            */
*/* TAKE PROBLEM DETERMINATION ACTIONS AND THEN CLEAN UP.            */
*/* IF IT IS VALID, ASSUME A SYSTEM ERROR IN COVERED FUNCTION.       */
*/* TAKE A DUMP, THEN TREAT AS C                                     */
*/********************************************************************
*                                                                    */
*                                                                  0682
*     IF SDWAEAS='1'B               /* LOWER ESTAE ALREADY DUMPED    */
*         |SDWACLUP='1'B THEN       /* OR RTM SAYS CLEANUP           */
         TM    SDWAEAS(SDWAPTR),B'00001000'                        0682
         BO    @RT00682                                            0682
         TM    SDWACLUP(SDWAPTR),B'10000000'                       0682
         BO    @RT00682                                            0682
*       GOTO DUMPRTN1;              /* GO CLEAN UP & GET OUT         */
*     RFY                                                          0684
*       GPR15F RSTD;                /* RESTRICT WORKREG 15 ZA02517   */
*     GPR15F='FFF'X;                /* SET USER ABEND MASK ZA02517   */
         LA    GPR15F,4095                                         0685
*     GEN REFS(SDWACMPC,SDWAPTR)(N GPR15F,SDWACMPC-1(SDWAPTR));    0686
         N GPR15F,SDWACMPC-1(SDWAPTR)
*     IF GPR15F^=0 THEN             /* IF USER ABEND CODE ZA02517    */
         LTR   GPR15F,GPR15F                                       0687
         BNZ   @RT00687                                            0687
*       GOTO DUMPRTN1;              /* GO CLEAN UP & EXIT ZA02517    */
*     RFY                                                          0689
*       GPR15F UNRSTD;              /* RELEASE WORKREG 15 ZA02517    */
*     IF SDWASTAF='1'B THEN         /* A LOWER LEVEL ESTAE/FRR       */
         TM    SDWASTAF(SDWAPTR),B'10000000'                       0690
         BO    @RT00690                                            0690
*       GOTO DUMPRTN1;              /* WAS EXECUTING AND FAILED TAKE
*                                      A DUMP FOR IT                 */
*     IF SDWATEXC='1'B              /* SOME NON-PROGRAM TYPE         */
*         |SDWAPGIO='1'B            /* ERROR HAS OCCURRED            */
*         |SDWAMCHK='1'B THEN       /* GO CLEAN UP & GET OUT         */
         TM    SDWATEXC(SDWAPTR),B'10000011'                       0692
         BNZ   @RT00692                                            0692
*       GOTO DUMPRTN1;                                             0693
*     GOTO USERRTN1;                /* CHECK USER INPUT              */
         B     USERRTN1                                            0694
*/********************************************************************
*                                                                    */
*                                                                  0695
*   END;                                                           0695
*                                                                  0695
*/********************************************************************/
*/*  THE USERRTN1 RECEIVES CONTROL IF IT IS DETERMINED THAT A PROGRAM*/
*/*  CHECK OCCURRED DURING CLEANUP, OR A LOWER RB OR ESTAE CALLED    */
*/*  RTM OR ISSUED AN ABEND. THIS ROUTINE WILL ISSUE A SETRP         */
*/*  TO RETRY BY RETURNING TO THE FIRST LEVEL ESTAE AT THE POINT     */
*/*  WHERE THE CLEANUP ROUTINE WAS CALLED.  THIS ADDRESS WAS STORED  */
*/*  IN THE SVRB WORKAREA AT WKASAV14 BY THE CLEANUP SUBROUTINE.     */
*/********************************************************************/
*                                                                  0696
*USERRTN1:                                                         0696
*   ;                               /* PGM CHK OCCURRED IN CLEAN     */
USERRTN1 DS    0H                                                  0697
*   SDWASRSV(5:4+LENGTH(SAVESDWA))=GPR01P->SAVESDWA;/* PUT REGS 1-14,
*                                      WHICH WERE LAST SAVED IN OLD
*                                      SDWA, BY EITHER DEBCHECK OR 0697
*                                      CLEANUP ROUTINE, INTO NEW SDWA
*                                      AT LABEL SDWASRSV - THIS    0697
*                                      ENSURES ALL RESTRICTED REGS 0697
*                                      WILL BE SAME ON RETURN TO   0697
*                                      PRIMARY ESTAE. NO OTHER REGS
*                                      ARE NECESSARY EXCEPT FOR    0697
*                                      MAINTENANCE PURPOSES.         */
         MVC   SDWASRSV+4(56,SDWAPTR),SAVESDWA(GPR01P)             0697
*   DO;                             /* SETRP RETADDR(WKASAV14->EST2RE
*                                      T)DUMP(NO)RECORD(NO)RC(4)FRESD
*                                      WA(YES)RETREGS(YES)WKAREA(SDWA
*                                      PTR)REGS(14)                  */
*     RESPECIFY                                                    0699
*      (GPR00P,                                                    0699
*       GPR01P,                                                    0699
*       GPR14P,                                                    0699
*       GPR15P) RSTD;                                              0699
*     GPR01P=SDWAPTR;                                              0700
         LR    GPR01P,SDWAPTR                                      0700
*     GPR01P->SDWARCDE=4;           /* STORE RC INTO SDWA            */
         MVI   SDWARCDE(GPR01P),X'04'                              0701
*     GPR01P->SDWARTYA=ADDR(WKASAV14->EST2RET);/* SAVE RETRY ADDRESS */
         L     @12,WKASAV14(,RBSECPTR)                             0702
         ST    @12,SDWARTYA(,GPR01P)                               0702
*     GPR01P->SDWAFREE='1'B;        /* SET FLAG TO FREE SDWA         */
*     GPR01P->SDWARCRD='0'B;        /* TURN OFF RECORD INDICATOR     */
         OI    SDWAFREE(GPR01P),B'00000100'                        0704
         NI    SDWARCRD(GPR01P),B'01111111'                        0704
*     GPR01P->SDWAREQ='0'B;         /* TURN OFF DUMP INDICATOR       */
         NI    SDWAREQ(GPR01P),B'01111111'                         0705
*     GPR01P->SDWAUPRG='1'B;        /* TURN ON RETREGS INDICATOR     */
         OI    SDWAUPRG(GPR01P),B'00001000'                        0706
*     GEN(RETURN (14));                                            0707
         RETURN (14)
*     RESPECIFY                                                    0708
*      (GPR00P,                                                    0708
*       GPR01P,                                                    0708
*       GPR14P,                                                    0708
*       GPR15P) UNRSTD;                                            0708
*   END;                                                           0709
*   RFY                                                            0710
*    (GPR01P,                                                      0710
*     GPR14P) UNRSTD;               /* RELEASE RETURN ADDRESS AND OLD
*                                      SDWA PTR-THATS ALL FOLKS      */
*/********************************************************************/
*/* THE DUMPRTN1 RECEIVES CONTROL IF IT IS DETERMINED THAT OTHER     */
*/* THAN A PROGRAM CHECK OCCURRED IN THE CLEANUP ROUTINE. THIS ESTAE */
*/* WILL ATTEMPT NO FURTHER ERROR ANALYSIS. IT WILL TAKE A SYSTEM    */
*/* DUMP, ASSUMING A SYSTEM ERROR, RECORD THE ERROR AND RETURN TO RTM*/
*/* FOR PERCOLATION. IT WILL RETURN TO RTM, HOWEVER, VIA IGCT005C, SO*/
*/* THAT THE SAVEAREA OBTAINED UPON ENTRY TO IGCT005C MAY BE FREED.  */
*/* THE RETRY EXIT WILL THEREFORE BE USED IN THIS CASE TO RETURN TO  */
*/* AN ENTRY POINT IN THE MAIN ROUTINE(BACRETRY) FROM WHICH A SETRP  */
*/* BACK TO RTM WILL BE ISSUED.                                      */
*/********************************************************************/
*                                                                  0711
*DUMPRTN1:                                                         0711
*   ;                               /* MUST BE A SYSTEM ERROR        */
DUMPRTN1 DS    0H                                                  0712
*   RFY                                                            0712
*    (WRKREG,                                                      0712
*     GPR01P) RSTD;                 /* RESTRICT SAVE REG       YM7597*/
*   WRKREG=GPR01P;                  /* SAVE OLD SDWA PTR       YM7519*/
         LR    WRKREG,GPR01P                                       0713
*   RFY                                                            0714
*     GPR01P UNRSTD;                /* RELEASE REG             YM7597*/
*   CALL SDUMP;                     /* GO TAKE SVC DUMP              */
         BAL   @14,SDUMP                                           0715
*   MESSAGE=SYSER;                  /* SET UP WTP MESSAGE            */
         MVC   MESSAGE(52,SDWAPTR),SYSER                           0716
*   RETCOD2=CLEANRC2(8);            /* CLEANUP RC -BAD CLEANUP       */
         MVC   RETCOD2(2,SDWAPTR),CLEANRC2+14                      0717
*   IF WKADMP04='1'B THEN           /* IF DUMP TAKEN SUCCESSFULLY    */
         TM    WKADMP04(RBSECPTR),B'00001000'                      0718
         BNO   @RF00718                                            0718
*     RETCOD=DUMPRC(1);             /* MV INTO MESSAGE RET CODE=4    */
         MVC   RETCOD(2,SDWAPTR),DUMPRC                            0719
*   IF WKADMP08='1'B THEN           /* IF PARTIAL DUMP TAKEN         */
@RF00718 TM    WKADMP08(RBSECPTR),B'00010000'                      0720
         BNO   @RF00720                                            0720
*     RETCOD=DUMPRC(2);             /* MV INTO MESSAGE RET CODE=8    */
         MVC   RETCOD(2,SDWAPTR),DUMPRC+2                          0721
*   IF WKADMP0C='1'B THEN           /* IF DUMP UNSUCCESSFUL          */
@RF00720 TM    WKADMP0C(RBSECPTR),B'00100000'                      0722
         BNO   @RF00722                                            0722
*     RETCOD=DUMPRC(3);             /* MV INTO MESSAGE RET CODE=C    */
         MVC   RETCOD(2,SDWAPTR),DUMPRC+4                          0723
*   MSGJOB=TIOCNJOB;                /* MV IN JOB NAME                */
@RF00722 L     @12,PSATOLD                                         0724
         L     @12,TCBTIO(,@12)                                    0724
         MVC   MSGJOB(8,SDWAPTR),TIOCNJOB(@12)                     0724
*   MSGSTEP=TIOCSTEP;               /* MV IN STEP NAME               */
         MVC   MSGSTEP(8,SDWAPTR),TIOCSTEP(@12)                    0725
*/* MUST VALIDATE THE TIOT ENTRY BEFORE GETTING DDNAME FOR MESSAGE.  */
*/* THIS IS DONE BY GETTING THE PROTECTED ADDRESS OF THE TIOT ENTRY  */
*/* FROM THE DSAB, POINTED TO BY THE DEB.  THE DEB MUST BE VALIDITY  */
*/* CHECKED FIRST.                                                   */
*                                                                  0726
*   CALL DEBCHECK;                  /* GO VALIDATE THE DEB           */
         BAL   @14,DEBCHECK                                        0726
*   RFY                                                            0727
*     DEBBASIC BASED(WKASVDEB);     /* VALID DEB SAVED IN SVRB       */
*   IF WKADEBXC=0 THEN              /* IF DEB RET CODE=0, DEB OK     */
         CLI   WKADEBXC(RBSECPTR),0                                0728
         BNE   @RF00728                                            0728
*     MSGDDN=TIOEDDNM;              /* MV DDNAME INTO GTF BUFFER     */
         L     @12,WKASVDEB(,RBSECPTR)                             0729
         L     @12,DEBAPPAD(,@12)                                  0729
         L     @12,DEBXTNP(,@12)                                   0729
         L     @12,DEBXDSAB(,@12)                                  0729
         L     @12,DSABTIOT(,@12)                                  0729
         MVC   MSGDDN(8,SDWAPTR),TIOEDDNM(@12)                     0729
*   ELSE                            /* IF DEB BAD,MUST DUMMY DDN     */
*     MSGDDN=NODDNAME;              /* ASSIGN DDN AS ASTERISKS       */
         B     @RC00728                                            0730
@RF00728 MVC   MSGDDN(8,SDWAPTR),NODDNAME                          0730
*   DO;                             /* WTOP MF(E,MESSAGE)            */
@RC00728 DS    0H                                                  0732
*     RFY                                                          0732
*       GPR01P RSTD;                /* RESTRICT REG 1                */
*     GPR01P=ADDR(MESSAGE);         /* ADDRESS CONTROL PROGRAM LIST  */
         LA    GPR01P,MESSAGE(,SDWAPTR)                            0733
*     SVC(35);                      /* ISSUE WTO SVC                 */
         SVC   35                                                  0734
*     RFY                                                          0735
*       GPR01P UNRSTD;              /* RELEASE REG 1                 */
*   END;                            /* WTOP MF(E,MESSAGE) ISSUE WTP  */
*RECORD1A:                                                         0737
*   CALL RECORD;                    /* GO PUT RECORD INFO IN SDWA    */
RECORD1A BAL   @14,RECORD                                          0737
*   SDWASRSV(5:4+LENGTH(SAVESDWA))=WRKREG->SAVESDWA;/* PUT YM7519  0738
*                                      1-14, WHICH WERE LAST SAVED IN
*                                      OLD SDWA, BY EITHER DEBCHECK
*                                      OR CLEANUP ROUTINE, INTO NEW
*                                      SDWA AT LABEL SDWASRSV - THIS
*                                      ENSURES ALL RESTRICTED REGS 0738
*                                      WILL BE SAME ON RETURN TO   0738
*                                      PRIMARY ESTAE. NO OTHER REGS
*                                      ARE NECESSARY EXCEPT FOR    0738
*                                      MAINTENANCE PURPOSES.         */
         MVC   SDWASRSV+4(56,SDWAPTR),SAVESDWA(WRKREG)             0738
*   DO;                             /* SETRP RETADDR(BACRETRY)DUMP(NO
*                                      )RECORD(YES)REGS(14)WKAREA(SDW
*                                      APTR)RC(4)FRESDWA(YES)RETREGS(
*                                      YES)                          */
*     RESPECIFY                                                    0740
*      (GPR00P,                                                    0740
*       GPR01P,                                                    0740
*       GPR14P,                                                    0740
*       GPR15P) RSTD;                                              0740
*     GPR01P=SDWAPTR;                                              0741
         LR    GPR01P,SDWAPTR                                      0741
*     GPR01P->SDWARCDE=4;           /* STORE RC INTO SDWA            */
         MVI   SDWARCDE(GPR01P),X'04'                              0742
*     GPR01P->SDWARTYA=ADDR(BACRETRY);/* SAVE RETRY ADDRESS          */
         L     @12,@CV03471                                        0743
         ST    @12,SDWARTYA(,GPR01P)                               0743
*     GPR01P->SDWAFREE='1'B;        /* SET FLAG TO FREE SDWA         */
*     GPR01P->SDWARCRD='1'B;        /* TURN ON RECORD INDICATOR      */
         OI    SDWAFREE(GPR01P),B'10000100'                        0745
*     GPR01P->SDWAREQ='0'B;         /* TURN OFF DUMP INDICATOR       */
         NI    SDWAREQ(GPR01P),B'01111111'                         0746
*     GPR01P->SDWAUPRG='1'B;        /* TURN ON RETREGS INDICATOR     */
         OI    SDWAUPRG(GPR01P),B'00001000'                        0747
*     GEN(RETURN (14));                                            0748
         RETURN (14)
*     RESPECIFY                                                    0749
*      (GPR00P,                                                    0749
*       GPR01P,                                                    0749
*       GPR14P,                                                    0749
*       GPR15P) UNRSTD;                                            0749
*   END;                            /* YM7519 DIAGNOSTIC INFO.TO BE
*                                      PUT OUT ON SYSTEM DATA SET    */
*   RFY                                                            0751
*     WRKREG UNRSTD;                /* RELEASE WORKREG         YM7519*/
*/********************************************************************/
*                                                                  0752
*   END IGCT105C                    /* END OF CLEANUP SUB- ESTAE     */
*/********************************************************************/
*                                                                  0752
*                                                                  0752
*/* THE FOLLOWING INCLUDE STATEMENTS WERE FOUND IN THIS PROGRAM.     */
*/*%INCLUDE SYSLIB  (CVT     )                                       */
*/*%INCLUDE SYSLIB  (IHADCBDF)                                       */
*/*%INCLUDE SYSLIB  (IHADCB  )                                       */
*/*%INCLUDE SYSLIB  (IEZDEB  )                                       */
*/*%INCLUDE SYSLIB  (IHADECB )                                       */
*/*%INCLUDE SYSLIB  (IHADSAB )                                       */
*/*%INCLUDE SYSLIB  (IEZIOB  )                                       */
*/*%INCLUDE SYSLIB  (IHAPSA  )                                       */
*/*%INCLUDE SYSLIB  (IHARB   )                                       */
*/*%INCLUDE SYSLIB  (IKJRB   )                                       */
*/*%INCLUDE SYSLIB  (IGGRDX  )                                       */
*/*%INCLUDE SYSLIB  (IHASDWA )                                       */
*/*%INCLUDE SYSLIB  (IGGSPD  )                                       */
*/*%INCLUDE SYSLIB  (IKJTCB  )                                       */
*/*%INCLUDE SYSLIB  (IEFTIOT1)                                       */
*                                                                  0752
*       ;                                                          0752
@EL00001 DS    0H                                                  0752
@EF00001 DS    0H                                                  0752
@ER00001 LM    @14,@12,12(@13)                                     0752
         BR    @14                                                 0752
@DATA    DS    0H
@DATD    DSECT
         DS    0F
IGCT105C CSECT
         DS    0F
@CF00099 DC    F'1'
@CF00111 DC    F'2'
@CF00092 DC    F'4'
@CF01507 DC    F'5'
@CF01050 DC    F'6'
@CF00047 DC    F'8'
@CF01554 DC    F'9'
@CF03306 DC    F'12'
@CF00073 DC    F'16'
@CF02573 DC    F'28'
@CF03559 DC    F'64000'
@DATD    DSECT
         DS    0D
IGCT105C CSECT
         DS    0F
@SIZDATD DC    AL1(0)
         DC    AL3(@ENDDATD-@DATD)
@CV03471 DC    V(BACRETRY)
         DS    0D
ERRUSER  DC    F'565'
DEBKEY   DC    XL1'50'
SCHEDKEY DC    AL1(16)
@CC03380 DC    C'IGCT105C'
@CC03549 DC    C'BACKUP5C'
HEADING  DS    CL27
         ORG   HEADING
@NM00166 DC    AL1(26)
HEADTEXT DC    CL26'IGCT105C,        ,        '
         ORG   HEADING+27
         DS    CL1
PATCH    DS    CL150
         ORG   PATCH
PATCHID  DC    CL8'IGCT105C'
PATCHIT  DC    142CL1'*'
         ORG   PATCH+150
ERRSYS   DC    X'135000'
GTON     DC    CL1'8'
MAJNAME  DC    CL8'SYSZGGLG'
NODDNAME DC    CL8'********'
USERER   DS    CL52
         ORG   USERER
USERER01 DC    H'48'
USERER02 DC    B'1000000000000000'
USERER03 DC    CL44'IEC902I 235,00,00,        ,        ,        '
USERER04 DC    B'0000010000000000'
USERER05 DC    B'0000000000100000'
         ORG   USERER+52
SYSER    DS    CL52
         ORG   SYSER
SYSER01  DC    H'48'
SYSER02  DC    B'1000000000000000'
SYSER03  DC    CL44'IEC903I 135,00,00,        ,        ,        '
SYSER04  DC    B'0000010000000000'
SYSER05  DC    B'0000000000100000'
         ORG   SYSER+52
DUMPRC1  DC    AL1(4)
         DC    AL1(8)
         DC    AL1(12)
DUMPRC   DC    CL2'04'
         DC    CL2'08'
         DC    CL2'0C'
CLEANRC1 DC    AL1(1)
         DC    AL1(2)
         DC    AL1(3)
         DC    AL1(4)
         DC    AL1(5)
         DC    AL1(6)
         DC    AL1(7)
         DC    AL1(8)
CLEANRC2 DC    CL2'01'
         DC    CL2'02'
         DC    CL2'03'
         DC    CL2'04'
         DC    CL2'05'
         DC    CL2'06'
         DC    CL2'07'
         DC    CL2'08'
BLKERR1  DC    AL1(1)
         DC    AL1(2)
         DC    AL1(3)
         DC    AL1(4)
         DC    AL1(5)
         DC    AL1(6)
         DC    AL1(0)
BLKERR2  DC    CL1'1'
         DC    CL1'2'
         DC    CL1'3'
         DC    CL1'4'
         DC    CL1'5'
         DC    CL1'6'
         DC    CL1'0'
BLKERR3  DC    AL1(1)
         DC    AL1(2)
         DC    AL1(3)
         DC    AL1(132)
         DC    AL1(133)
         DC    AL1(134)
         DC    AL1(0)
IGCT105C CSECT
DUMPLIST SDUMP SDATA=(PSA,NUC,SQA,RGN),MF=L
ELIST    ESTAE BACKUP5C,RECORD=YES,MF=L
ENQLIST  ENQ   (MAJNAME,,S,8),MF=L
@DATD    DSECT
         ORG   *+1-(*-@DATD)/(*-@DATD) INSURE DSECT DATA
@ENDDATD EQU   *
IGCT105C CSECT
@00      EQU   00                      EQUATES FOR REGISTERS 0-15
@01      EQU   01
@02      EQU   02
@03      EQU   03
@04      EQU   04
@05      EQU   05
@06      EQU   06
@07      EQU   07
@08      EQU   08
@09      EQU   09
@10      EQU   10
@11      EQU   11
@12      EQU   12
@13      EQU   13
@14      EQU   14
@15      EQU   15
BASEREG  EQU   @09
GPR00F   EQU   @00
GPR01F   EQU   @01
GPR15F   EQU   @15
GPR00P   EQU   @00
GPR01P   EQU   @01
GPR14P   EQU   @14
GPR15P   EQU   @15
GPR02P   EQU   @02
GPR04P   EQU   @04
GPR10P   EQU   @10
GPR11P   EQU   @11
GPR12P   EQU   @12
GPR13P   EQU   @13
I        EQU   @00
RDXPTR   EQU   @02
SDWAPTR  EQU   @05
DCBPTR   EQU   @06
BLKCTR   EQU   @07
RBSECPTR EQU   @08
RDXENTRG EQU   @10
WRKREG   EQU   @11
SAVEPTR  EQU   @13
RETURNCD EQU   @15
GPR14F   EQU   @14
MVBLKPTR EQU   @04
TRCPTR   EQU   @02
CVTPTR   EQU   16
PSA      EQU   0
FLCRNPSW EQU   PSA
FLCROPSW EQU   PSA+8
FLCICCW2 EQU   PSA+16
PSAEEPSW EQU   PSA+132
PSAESPSW EQU   PSA+136
FLCSVILC EQU   PSAESPSW+1
PSAEPPSW EQU   PSA+140
FLCPIILC EQU   PSAEPPSW+1
FLCPICOD EQU   PSAEPPSW+2
PSAPICOD EQU   FLCPICOD+1
FLCTEA   EQU   PSAEPPSW+4
FLCPER   EQU   PSA+152
FLCMCLA  EQU   PSA+168
FLCIOEL  EQU   FLCMCLA+4
FLCIOA   EQU   FLCMCLA+16
FLCFSA   EQU   FLCMCLA+80
PSATOLD  EQU   PSA+540
PSASUPER EQU   PSA+552
PSASUP1  EQU   PSASUPER
PSASUP2  EQU   PSASUPER+1
PSASUP3  EQU   PSASUPER+2
PSASUP4  EQU   PSASUPER+3
PSACLHT  EQU   PSA+640
PSALKSA  EQU   PSA+696
PSAHLHI  EQU   PSA+760
PSALITA  EQU   PSA+764
PSADSSFL EQU   PSA+836
PSADSSF1 EQU   PSADSSFL
PSADSSF3 EQU   PSADSSFL+2
PSADSSF4 EQU   PSADSSFL+3
PSARSVT  EQU   PSA+896
NOBASE   EQU   0
CVTMAP   EQU   0
CVT0VL00 EQU   CVTMAP+24
CVTDAR   EQU   CVTMAP+72
CVTFLGS1 EQU   CVTDAR
CVTDCB   EQU   CVTMAP+116
CVTIOQET EQU   CVTMAP+120
CVTIERLC EQU   CVTMAP+144
CVTHEAD  EQU   CVTMAP+160
CVTSV76C EQU   CVTHEAD
CVTOPTA  EQU   CVTMAP+182
CVTOPTB  EQU   CVTMAP+183
CVTGTF   EQU   CVTMAP+236
CVTGTFST EQU   CVTGTF
CVTGTFS  EQU   CVTGTFST
CVTSTATE EQU   CVTGTFST
CVTTMODE EQU   CVTGTFST
CVTFORM  EQU   CVTGTFST
CVTAQAVT EQU   CVTMAP+240
CVTTCMFG EQU   CVTAQAVT
CVTVOLM2 EQU   CVTMAP+244
CVTTATA  EQU   CVTVOLM2
CVTTSKS  EQU   CVTTATA
CVTVOLF2 EQU   CVTTSKS
CVTTAT   EQU   CVTTATA+1
CVTATER  EQU   CVTMAP+248
CVTEXT1  EQU   CVTMAP+252
CVTPURG  EQU   CVTMAP+260
CVTQMSG  EQU   CVTMAP+268
CVTDMSR  EQU   CVTMAP+272
CVTRSV37 EQU   CVTDMSR
CVTDMSRF EQU   CVTRSV37
CVTERPV  EQU   CVTMAP+316
CVTINTLA EQU   CVTMAP+320
CVTAPF   EQU   CVTMAP+324
CVTEXT2  EQU   CVTMAP+328
CVTHJES  EQU   CVTMAP+332
CVTPGSIA EQU   CVTMAP+348
CVTA1F1  EQU   CVTMAP+356
CVTSYSK  EQU   CVTMAP+357
CVTVOLM1 EQU   CVTMAP+380
CVTVOLF1 EQU   CVTVOLM1
CVTATMCT EQU   CVTMAP+388
CVTXTNT1 EQU   0
CVTXTNT2 EQU   0
CVTDSSV  EQU   CVTXTNT2
CVTFLGBT EQU   CVTXTNT2+5
CVTDEBVR EQU   CVTXTNT2+8
CVTQID   EQU   CVTXTNT2+24
CVTRV400 EQU   CVTXTNT2+52
CVTRV409 EQU   CVTXTNT2+53
CVTATCVT EQU   CVTXTNT2+64
CVTRV429 EQU   CVTXTNT2+84
CVTRV438 EQU   CVTXTNT2+85
CVTRV457 EQU   CVTXTNT2+112
CVTRV466 EQU   CVTXTNT2+113
CVTFIX   EQU   0
CVTRELNO EQU   CVTFIX+252
IHADCB   EQU   0
IHADCS00 EQU   0
DCBFDAD  EQU   IHADCS00+5
IHADCS01 EQU   0
DCBDVTBL EQU   IHADCS01
IHADCS11 EQU   0
DCBRELB  EQU   IHADCS11
DCBREL   EQU   DCBRELB+1
DCBBUFCB EQU   IHADCS11+4
DCBDSORG EQU   IHADCS11+10
DCBDSRG1 EQU   DCBDSORG
DCBDSRG2 EQU   DCBDSORG+1
DCBIOBAD EQU   IHADCS11+12
DCBODEB  EQU   DCBIOBAD
DCBLNP   EQU   DCBODEB
DCBQSLM  EQU   DCBLNP
DCBIOBAA EQU   DCBODEB+1
IHADCS50 EQU   0
DCBSVCXL EQU   IHADCS50
DCBEODAD EQU   IHADCS50+4
DCBBFALN EQU   DCBEODAD
DCBHIARC EQU   DCBBFALN
DCBBFTEK EQU   DCBHIARC
DCBBFT   EQU   DCBBFTEK
DCBEXLST EQU   IHADCS50+8
DCBRECFM EQU   DCBEXLST
DCBRECLA EQU   DCBRECFM
IHADCS24 EQU   0
DCBOFLGS EQU   IHADCS24+8
DCBOFLWR EQU   DCBOFLGS
DCBIFLG  EQU   IHADCS24+9
DCBIBPCT EQU   DCBIFLG
DCBMACR  EQU   IHADCS24+10
DCBMACR1 EQU   DCBMACR
DCBMRFE  EQU   DCBMACR1
DCBMRGET EQU   DCBMRFE
DCBMRAPG EQU   DCBMACR1
DCBMRRD  EQU   DCBMRAPG
DCBMRCI  EQU   DCBMACR1
DCBMRMVG EQU   DCBMRCI
DCBMRLCG EQU   DCBMACR1
DCBMRABC EQU   DCBMACR1
DCBMRPT1 EQU   DCBMRABC
DCBMRSBG EQU   DCBMRPT1
DCBMRCRL EQU   DCBMACR1
DCBMRCHK EQU   DCBMRCRL
DCBMRRDX EQU   DCBMRCHK
DCBMRDMG EQU   DCBMACR1
DCBMRCK  EQU   DCBMRDMG
DCBMACR2 EQU   DCBMACR+1
DCBMRPUT EQU   DCBMACR2
DCBMRWRT EQU   DCBMACR2
DCBMRMVP EQU   DCBMACR2
DCBMR5WD EQU   DCBMACR2
DCBMRLDM EQU   DCBMR5WD
DCBMRLCP EQU   DCBMRLDM
DCBMR4WD EQU   DCBMACR2
DCBMRPT2 EQU   DCBMR4WD
DCBMRTMD EQU   DCBMRPT2
DCBMR3WD EQU   DCBMACR2
DCBMRCTL EQU   DCBMR3WD
DCBMRSTK EQU   DCBMRCTL
DCBMR1WD EQU   DCBMACR2
DCBMRSWA EQU   DCBMR1WD
DCBMRDMD EQU   DCBMRSWA
IHADCS25 EQU   0
DCBMACRF EQU   IHADCS25+2
DCBMACF1 EQU   DCBMACRF
DCBMFFE  EQU   DCBMACF1
DCBMFGET EQU   DCBMFFE
DCBMFAPG EQU   DCBMACF1
DCBMFRD  EQU   DCBMFAPG
DCBMFCI  EQU   DCBMACF1
DCBMFMVG EQU   DCBMFCI
DCBMFLCG EQU   DCBMACF1
DCBMFABC EQU   DCBMACF1
DCBMFPT1 EQU   DCBMFABC
DCBMFSBG EQU   DCBMFPT1
DCBMFCRL EQU   DCBMACF1
DCBMFCHK EQU   DCBMFCRL
DCBMFDMG EQU   DCBMACF1
DCBMACF2 EQU   DCBMACRF+1
DCBMFPUT EQU   DCBMACF2
DCBMFWRT EQU   DCBMACF2
DCBMFMVP EQU   DCBMACF2
DCBMF5WD EQU   DCBMACF2
DCBMFLDM EQU   DCBMF5WD
DCBMFLCP EQU   DCBMFLDM
DCBMF4WD EQU   DCBMACF2
DCBMFPT2 EQU   DCBMF4WD
DCBMFTMD EQU   DCBMFPT2
DCBMF3WD EQU   DCBMACF2
DCBMFCTL EQU   DCBMF3WD
DCBMFSTK EQU   DCBMFCTL
DCBMF1WD EQU   DCBMACF2
DCBMFSWA EQU   DCBMF1WD
DCBMFDMD EQU   DCBMFSWA
DCBDEBAD EQU   IHADCS25+4
DCBIFLGS EQU   DCBDEBAD
DCBIFPCT EQU   DCBIFLGS
DCBDEBA  EQU   DCBDEBAD+1
IHADCS26 EQU   0
DCBWRITE EQU   IHADCS26
DCBREAD  EQU   DCBWRITE
DCBWRITA EQU   DCBREAD+1
IHADCS30 EQU   0
DCBCHECK EQU   IHADCS30
DCBOPTCD EQU   DCBCHECK
DCBXARG  EQU   IHADCS30+32
DCBDRDX  EQU   IHADCS30+36
DEBBASIC EQU   0
DEBTCBAD EQU   DEBBASIC
DEBTCBB  EQU   DEBTCBAD+1
DEBDEBAD EQU   DEBBASIC+4
DEBIRBAD EQU   DEBBASIC+8
DEBOFLGS EQU   DEBIRBAD
DEBOPATB EQU   DEBBASIC+12
DEBFLGS1 EQU   DEBBASIC+14
DEBUSRPG EQU   DEBBASIC+16
DEBRRQ   EQU   DEBBASIC+20
DEBECBAD EQU   DEBRRQ
DEBDCBAD EQU   DEBBASIC+24
DEBPROTG EQU   DEBDCBAD
DEBDCBB  EQU   DEBDCBAD+1
DEBAPPAD EQU   DEBBASIC+28
DEBBASND EQU   DEBBASIC+32
DEBDASD  EQU   0
DEBUCBAD EQU   DEBDASD
DEBACSMD EQU   0
DEBVOLSQ EQU   DEBACSMD
DEBVOLBT EQU   DEBVOLSQ
DEBDSNM  EQU   DEBACSMD+4
DEBUTSAA EQU   DEBDSNM
DEBACS01 EQU   0
DEBDSNAM EQU   DEBACS01
DEBACS02 EQU   0
DEBDBLK  EQU   DEBACS02
DEBACS03 EQU   0
DEBACS04 EQU   0
DEBTBFRA EQU   DEBACS04
DEBTCCWA EQU   DEBACS04+4
DEBACS05 EQU   0
DEBFUCBA EQU   DEBACS05
DEBLUCBA EQU   DEBACS05+4
DEBACS06 EQU   0
DEBACS07 EQU   0
DEBACS08 EQU   0
DEBSUBNM EQU   0
DEB      EQU   0
DEBAVT   EQU   DEB
DEBEOEA  EQU   DEBAVT
DEBEOEAB EQU   DEBEOEA
DEBSIOA  EQU   DEBAVT+4
DEBSIOAB EQU   DEBSIOA
DEBPCIA  EQU   DEBAVT+8
DEBPCIAB EQU   DEBPCIA
DEBCEA   EQU   DEBAVT+12
DEBCEAB  EQU   DEBCEA
DEBXCEA  EQU   DEBAVT+16
DEBXCEAB EQU   DEBXCEA
DEBPREFX EQU   DEB+20
DEBXTNP  EQU   DEBPREFX+8
DEBTBLOF EQU   DEBPREFX+14
DEBXTN   EQU   0
DEBXFLG1 EQU   DEBXTN+2
DEBXDSAB EQU   DEBXTN+4
DEBXXARG EQU   DEBXTN+20
DECB     EQU   0
DECBPTR  EQU   DECB+4
DECTYPE  EQU   DECBPTR
DECTYPE1 EQU   DECTYPE
DECTYPE2 EQU   DECTYPE+1
DECNEXT  EQU   DECB+20
DECBS01  EQU   0
DECBECB  EQU   DECBS01
DECBRB   EQU   DECBECB
DECBECBF EQU   DECBRB
DECBTYP1 EQU   DECBS01+4
DECBTYP2 EQU   DECBS01+5
DECBEXC1 EQU   DECBS01+24
DECBEXC2 EQU   DECBS01+25
DECBS02  EQU   0
@NM00022 EQU   DECBS02
DECCC2   EQU   @NM00022+1
DECCC3   EQU   @NM00022+2
@NM00024 EQU   DECBS02+4
@NM00025 EQU   @NM00024
@NM00026 EQU   @NM00024+1
DECTYPRU EQU   @NM00026
DECNA    EQU   DECBS02+28
DECBS03  EQU   0
DECCPOL  EQU   DECBS03+20
@NM00034 EQU   DECBS03+36
DECBS04  EQU   0
@NM00037 EQU   DECBS04+4
@NM00038 EQU   @NM00037
@NM00041 EQU   DECBS04+8
DECBUFCT EQU   @NM00041
DECONLTT EQU   DECBUFCT
DECENTRY EQU   DECBS04+20
DECFLAGS EQU   DECBS04+24
DECWACK  EQU   DECFLAGS
DECREVRS EQU   DECFLAGS
DECERRST EQU   DECBS04+29
DECPOLPT EQU   DECBS04+36
DSAB     EQU   0
DSABTIOT EQU   DSAB+16
DSABGIDP EQU   DSAB+24
DSABANMP EQU   DSAB+28
DSABORG  EQU   DSAB+32
DSABORG1 EQU   DSABORG
DSABORG2 EQU   DSABORG+1
DSABFLG1 EQU   DSAB+34
DSABFLG2 EQU   DSAB+35
DSABFLG3 EQU   DSAB+36
DSABFLG4 EQU   DSAB+37
DSABANMI EQU   0
DSABGIDL EQU   0
IOBPREFX EQU   0
IOBCFLG1 EQU   IOBPREFX
IOBSTDRD EQU   0
IOBFLAG1 EQU   IOBSTDRD
IOBCYCCK EQU   IOBFLAG1
IOBRSTRT EQU   IOBFLAG1
IOBFLAG2 EQU   IOBSTDRD+1
IOBPURGE EQU   IOBFLAG2
IOBRDHA0 EQU   IOBFLAG2
IOBSENS0 EQU   IOBSTDRD+2
IOBS0B7  EQU   IOBSENS0
IOBSENS1 EQU   IOBSTDRD+3
IOBECBPT EQU   IOBSTDRD+4
IOBFLAG3 EQU   IOBSTDRD+8
IOBFL3   EQU   IOBFLAG3
IOBCNC   EQU   IOBFL3
IOBICL   EQU   IOBFL3
IOBCSW   EQU   IOBSTDRD+9
IOBIOCSW EQU   IOBCSW
IOBSTBYT EQU   IOBIOCSW+3
IOBUSTAT EQU   IOBSTBYT
IOBCSTAT EQU   IOBSTBYT+1
IOBSTART EQU   IOBSTDRD+16
IOBDCBPT EQU   IOBSTDRD+20
IOBFLAG4 EQU   IOBDCBPT
IOBRESTR EQU   IOBSTDRD+24
IOBINCAM EQU   IOBSTDRD+28
IOBBTAMF EQU   IOBINCAM
RBPRFX   EQU   0
RBBASIC  EQU   0
RBEXRTNM EQU   RBBASIC
RBTMFLD  EQU   RBEXRTNM
RBSTAB   EQU   RBBASIC+10
XSTAB    EQU   RBSTAB
RBSTAB1  EQU   XSTAB
XSTAB1   EQU   RBSTAB1
RBFTCKPT EQU   XSTAB1
RBSTAB2  EQU   XSTAB+1
XSTAB2   EQU   RBSTAB2
RBTCBNXT EQU   XSTAB2
RBFACTV  EQU   XSTAB2
RBFDYN   EQU   XSTAB2
RBECBWT  EQU   XSTAB2
RBOPSW   EQU   RBBASIC+16
XRBPSW   EQU   RBOPSW
RBOPSWB2 EQU   XRBPSW+1
RBLINK   EQU   RBBASIC+28
XRBLNK   EQU   RBLINK
RBWCF    EQU   XRBLNK
RBLINKB  EQU   XRBLNK+1
RBGRSAVE EQU   RBBASIC+32
XRBREG   EQU   RBGRSAVE
RBGRS0   EQU   XRBREG
RBGRS1   EQU   XRBREG+4
RBGRS2   EQU   XRBREG+8
RBGRS3   EQU   XRBREG+12
RBGRS4   EQU   XRBREG+16
RBGRS5   EQU   XRBREG+20
RBGRS6   EQU   XRBREG+24
RBGRS7   EQU   XRBREG+28
RBGRS8   EQU   XRBREG+32
RBGRS9   EQU   XRBREG+36
RBGRS10  EQU   XRBREG+40
RBGRS11  EQU   XRBREG+44
RBGRS12  EQU   XRBREG+48
RBGRS13  EQU   XRBREG+52
RBGRS14  EQU   XRBREG+56
RBGRS15  EQU   XRBREG+60
XRBESA   EQU   RBBASIC+96
RBEXSAVE EQU   XRBESA
RBPREFIX EQU   0
RBRSV019 EQU   RBPREFIX+14
RBRSV028 EQU   RBPREFIX+15
RBRSV041 EQU   RBPREFIX+27
RBRSV054 EQU   RBPREFIX+39
RBRTOPSW EQU   RBPREFIX+40
RBRTPSW2 EQU   RBRTOPSW+8
RBRTICIL EQU   RBRTPSW2
RBFLAGS1 EQU   RBPREFIX+56
RBINTCDA EQU   RBPREFIX+61
RBSECT   EQU   0
RBPPSAV  EQU   RBSECT
@NM00083 EQU   RBSECT+10
@NM00084 EQU   @NM00083
RBTRSVRB EQU   @NM00084
@NM00086 EQU   @NM00083+1
RBETXR   EQU   @NM00086
RBEP     EQU   RBSECT+12
RBPGMQ   EQU   RBSECT+24
@NM00093 EQU   RBSECT+28
IRBEND   EQU   RBSECT+96
@NM00097 EQU   RBSECT+96
RBRSV138 EQU   @NM00097+7
RBRSV150 EQU   RBSECT+167
RDXLIST  EQU   0
RDXNEXT  EQU   RDXLIST+4
RDXENT   EQU   RDXLIST+8
RDXENTRY EQU   0
RDXIOBUQ EQU   RDXENTRY+8
RDXUQND  EQU   RDXENTRY+12
SDWA     EQU   0
SDWAPARM EQU   SDWA
SDWAFIOB EQU   SDWA+4
SDWAABCC EQU   SDWAFIOB
SDWACMPF EQU   SDWAABCC
SDWAREQ  EQU   SDWACMPF
SDWACMPC EQU   SDWAABCC+1
SDWACTL1 EQU   SDWA+8
SDWACMKA EQU   SDWACTL1
SDWAMWPA EQU   SDWACTL1+1
SDWAPMKA EQU   SDWACTL1+4
SDWACTL2 EQU   SDWA+16
SDWACMKP EQU   SDWACTL2
SDWAMWPP EQU   SDWACTL2+1
SDWAPMKP EQU   SDWACTL2+4
SDWAGRSV EQU   SDWA+24
SDWANAME EQU   SDWA+88
SDWAEC1  EQU   SDWA+104
SDWAEMK1 EQU   SDWAEC1
SDWAMWP1 EQU   SDWAEC1+1
SDWAINT1 EQU   SDWAEC1+2
SDWANXT1 EQU   SDWAEC1+4
SDWAAEC1 EQU   SDWA+112
SDWAILC1 EQU   SDWAAEC1+1
SDWAINC1 EQU   SDWAAEC1+2
SDWAICD1 EQU   SDWAINC1+1
SDWAEC2  EQU   SDWA+120
SDWAEMK2 EQU   SDWAEC2
SDWAMWP2 EQU   SDWAEC2+1
SDWAINT2 EQU   SDWAEC2+2
SDWANXT2 EQU   SDWAEC2+4
SDWAAEC2 EQU   SDWA+128
SDWAILC2 EQU   SDWAAEC2+1
SDWAINC2 EQU   SDWAAEC2+2
SDWAICD2 EQU   SDWAINC2+1
SDWASRSV EQU   SDWA+136
SDWASR15 EQU   SDWASRSV+60
SDWAIDNT EQU   SDWA+200
SDWAMCH  EQU   SDWA+204
SDWASTCK EQU   SDWAMCH
SDWAMCHI EQU   SDWAMCH+8
SDWAMCHS EQU   SDWAMCHI
SDWAMCHD EQU   SDWAMCHI+1
SDWARSR1 EQU   SDWAMCH+12
SDWARSR2 EQU   SDWAMCH+13
SDWAFLGS EQU   SDWA+232
SDWAERRA EQU   SDWAFLGS
SDWAMCHK EQU   SDWAERRA
SDWATEXC EQU   SDWAERRA
SDWAPGIO EQU   SDWAERRA
SDWAERRB EQU   SDWAFLGS+1
SDWAERRC EQU   SDWAFLGS+2
SDWASTAF EQU   SDWAERRC
SDWAEAS  EQU   SDWAERRC
SDWAERRD EQU   SDWAFLGS+3
SDWACLUP EQU   SDWAERRD
SDWAIOFS EQU   SDWA+238
SDWARTYA EQU   SDWA+240
SDWACPUA EQU   SDWA+248
SDWAPARQ EQU   SDWA+252
SDWARCDE EQU   SDWAPARQ
SDWAACF2 EQU   SDWAPARQ+1
SDWARCRD EQU   SDWAACF2
SDWAUPRG EQU   SDWAACF2
SDWAFREE EQU   SDWAACF2
SDWAACF3 EQU   SDWAPARQ+2
SDWAACF4 EQU   SDWAPARQ+3
SDWALKWA EQU   SDWA+256
SDWALKWS EQU   SDWALKWA
SDWARECP EQU   SDWA+292
SDWACSCT EQU   SDWARECP+8
SDWAREXN EQU   SDWARECP+16
SDWASNPA EQU   SDWA+320
SDWADUMP EQU   SDWASNPA
SDWADPFS EQU   SDWADUMP+1
SDWADDAT EQU   SDWASNPA+4
SDWASDAT EQU   SDWADDAT
SDWAPDAT EQU   SDWADDAT+2
SDWADPSA EQU   SDWA+328
SDWADPSL EQU   SDWADPSA
SDWARA   EQU   SDWA+400
SDWADPVA EQU   SDWARA+2
SDWAVRA  EQU   SDWARA+4
SPDBFR   EQU   0
SPDHDR   EQU   SPDBFR
SPDTRRCD EQU   0
SPDRCDHD EQU   SPDTRRCD
SPDBLKID EQU   SPDRCDHD+1
SPDDATA1 EQU   SPDTRRCD+2
TCBFIX   EQU   0
TCBFRS   EQU   TCBFIX
TCB      EQU   0
TCBPIE   EQU   TCB+4
TCBPMASK EQU   TCBPIE
TCBTIO   EQU   TCB+12
TCBCMP   EQU   TCB+16
TCBCMPF  EQU   TCBCMP
TCBCPP   EQU   TCBCMPF
TCBSTCC  EQU   TCBCMPF
TCBCDBL  EQU   TCBCMPF
TCBCWTO  EQU   TCBCMPF
TCBCIND  EQU   TCBCMPF
TCBCMSG  EQU   TCBCMPF
TCBTRN   EQU   TCB+20
TCBABF   EQU   TCBTRN
TCBMSS   EQU   TCB+24
TCBPKF   EQU   TCB+28
TCBFLGS  EQU   TCB+29
TCBFLGS1 EQU   TCBFLGS
TCBFLGS2 EQU   TCBFLGS+1
TCBFLGS3 EQU   TCBFLGS+2
TCBFLGS4 EQU   TCBFLGS+3
TCBFLGS5 EQU   TCBFLGS+4
TCBABWF  EQU   TCBFLGS5
TCBJPQ   EQU   TCB+44
TCBPURGE EQU   TCBJPQ
TCBGRS   EQU   TCB+48
TCBFSA   EQU   TCB+112
TCBTME   EQU   TCB+120
TCBJSTCB EQU   TCB+124
TCBTSFLG EQU   TCB+148
TCBTSTSK EQU   TCBTSFLG
TCBSTAB  EQU   TCB+160
TCBNSTAE EQU   TCBSTAB
TCBTCT   EQU   TCB+164
TCBTCTGF EQU   TCBTCT
TCBSCNDY EQU   TCB+172
TCBNDSP  EQU   TCBSCNDY
TCBNDSP1 EQU   TCBNDSP+1
TCBNDSP2 EQU   TCBNDSP+2
TCBNDSP3 EQU   TCBNDSP+3
TCBJSCB  EQU   TCB+180
TCBRECDE EQU   TCBJSCB
TCBEXT1  EQU   TCB+196
TCBBITS  EQU   TCB+200
TCBNDSP4 EQU   TCBBITS
TCBNDSP5 EQU   TCBBITS+1
TCBFLGS6 EQU   TCBBITS+2
TCBFLGS7 EQU   TCBBITS+3
TCBDAR   EQU   TCB+204
TCBDARC  EQU   TCBDAR
TCBEXT2  EQU   TCB+208
TCBFOE   EQU   TCB+244
TCBESTAE EQU   TCB+264
TCBESTRM EQU   TCBESTAE+1
TCBFBYT1 EQU   TCB+276
TCBFBYT2 EQU   TCB+277
TCBRV124 EQU   TCB+278
TCBRV133 EQU   TCB+279
TCBXTNT2 EQU   0
TCBGTF   EQU   TCBXTNT2
TCBTFLG  EQU   TCBGTF
TCBRV146 EQU   TCBXTNT2+28
TCBRV155 EQU   TCBXTNT2+29
TCBRV164 EQU   TCBXTNT2+30
TCBRV173 EQU   TCBXTNT2+31
TIOT1    EQU   0
TIOCNJOB EQU   TIOT1
TIOCSTEP EQU   TIOT1+8
TIOENTRY EQU   0
TIOESTTA EQU   TIOENTRY+1
TIOERLOC EQU   TIOENTRY+2
TIOELINK EQU   TIOERLOC+1
TIOESYOT EQU   TIOELINK
TIOEDDNM EQU   TIOENTRY+4
TIOESTTC EQU   TIOENTRY+15
TIOESTTB EQU   TIOENTRY+16
POOLSTAR EQU   TIOENTRY+20
EST2RET  EQU   0
RELAREA  EQU   0
ASUBRTN  EQU   0
IKEBC    EQU   0
IKEBF15  EQU   0
IKEBF31  EQU   0
IKEBP15  EQU   0
IKEBP31  EQU   0
IKEBP8   EQU   0
SPDRDATA EQU   0
CVTS01   EQU   CVTPGSIA
CVTLPDIA EQU   CVTS01+12
CVTDIRST EQU   CVTLPDIA
CVTSLIDA EQU   CVTS01+24
CVTCTLFG EQU   CVTS01+50
CVTRV210 EQU   CVTS01+424
CVTRV219 EQU   CVTS01+425
CVTRV228 EQU   CVTS01+426
CVTRV237 EQU   CVTS01+427
CVTMFRTR EQU   CVTS01+452
CVTRV262 EQU   CVTS01+468
CVTRV271 EQU   CVTS01+469
CVTRV280 EQU   CVTS01+470
CVTRV289 EQU   CVTS01+471
CVTGSDA  EQU   CVTS01+600
DEBBAS02 EQU   DEBBASND
DEBDDS1  EQU   DEBBAS02
DEBSUCBA EQU   DEBDDS1
DEBDEVED EQU   DEBDDS1+4
DEBRDCB  EQU   DEBDDS1+4
DEBPDCB  EQU   DEBDDS1+8
DEBWDCB  EQU   DEBDDS1+12
DEBACS09 EQU   DEBDEVED
DEBASC09 EQU   DEBACS09
DEBEAMFG EQU   DEBASC09+3
DEBDSID  EQU   DEBASC09+4
DEBEOD   EQU   DEBASC09+12
DEBBOE   EQU   DEBEOD
DEBEODRV EQU   DEBBOE
DEBEODTT EQU   DEBBOE+1
DEBEOD0  EQU   DEBBOE+2
DEBEODSS EQU   DEBBOE+3
DEBBAS01 EQU   DEBBASND
DEBISAM  EQU   DEBBAS01
DEBFIEAD EQU   DEBISAM
DEBFPEAD EQU   DEBISAM+4
DEBFOEAD EQU   DEBISAM+8
DEBEXPT  EQU   DEBISAM+12
DEBRPSID EQU   DEBEXPT
DECBS05  EQU   DECNEXT
IOBORG01 EQU   IOBPREFX+8
IOBNIOBA EQU   IOBORG01
IOBNFLG1 EQU   IOBNIOBA
IOBPRTOV EQU   IOBNFLG1
IOBORG02 EQU   IOBPREFX+8
IOBDQADA EQU   IOBORG02
IOBDEQIN EQU   IOBDQADA
IOBORG03 EQU   IOBPREFX+12
IOBORG04 EQU   IOBINCAM
IOBORG12 EQU   IOBINCAM
IOBFL4   EQU   IOBORG12
IOBEXTEN EQU   IOBSTDRD+32
IOBSEEK  EQU   IOBEXTEN
IOBBB    EQU   IOBSEEK+1
IOBCC    EQU   IOBSEEK+3
IOBHH    EQU   IOBSEEK+5
IOBORG05 EQU   IOBEXTEN
IOBORG06 EQU   IOBEXTEN
IOBNXTPT EQU   IOBORG06+4
IOBSTATA EQU   IOBNXTPT
IOBORG07 EQU   IOBEXTEN+8
W1IEXTEN EQU   IOBORG07
IOBORG08 EQU   IOBEXTEN+8
IOBINDCT EQU   IOBORG08+4
IOBUNSQR EQU   IOBORG08+5
IOBFCHAD EQU   IOBORG08+8
IOBORG09 EQU   IOBEXTEN+8
IOBDPLAD EQU   IOBORG09+4
IOBDAYLI EQU   IOBDPLAD
IOBDTYPE EQU   IOBORG09+8
IOBDTYP2 EQU   IOBORG09+9
IOBADDTY EQU   IOBDTYP2
IOBDSTAT EQU   IOBORG09+10
IOBSTAT1 EQU   IOBDSTAT
IOBENQUE EQU   IOBSTAT1
IOBADDVU EQU   IOBSTAT1
IOBSTAT2 EQU   IOBDSTAT+1
IOBDQPTR EQU   IOBORG09+20
IOBORG10 EQU   IOBEXTEN+8
IOBSEEK2 EQU   IOBORG10
IOBORG11 EQU   IOBEXTEN
IOBERCT  EQU   IOBORG11
IOBUCBXV EQU   IOBERCT
IOBNAME  EQU   IOBORG11+4
IOBORG13 EQU   IOBEXTEN
IOBSKADR EQU   IOBORG13
PSARSVTE EQU   PSARSVT
RBSECS03 EQU   RBEP
RBCDE    EQU   RBSECS03
RBCDFLGS EQU   RBCDE
RBSECS05 EQU   RBPGMQ
RBSQE    EQU   RBSECS05
RBSECS06 EQU   RBPGMQ
RBIQE    EQU   RBSECS06
RBSECS07 EQU   RBPGMQ
RBIQE2   EQU   RBSECS07
RBSECS08 EQU   IRBEND
RBSECS09 EQU   IRBEND
TIOEDEVE EQU   TIOESTTB
TIOSTTB  EQU   TIOEDEVE
WKAREAX  EQU   RBEXSAVE
WKASW1   EQU   WKAREAX
WKALOCK  EQU   WKASW1
WKADMP0C EQU   WKASW1
WKADMP08 EQU   WKASW1
WKADMP04 EQU   WKASW1
WKAENQRQ EQU   WKASW1
WKADEQRQ EQU   WKASW1
WKATRALX EQU   WKAREAX+1
WKASVKEY EQU   WKAREAX+2
WKACLENX EQU   WKAREAX+3
WKASAV14 EQU   WKAREAX+4
WKAIOBQX EQU   WKAREAX+8
WKANQARG EQU   WKAREAX+12
WKARLCOR EQU   WKAREAX+20
WKAXADDR EQU   WKAREAX+24
WKALST   EQU   WKAREAX+28
WKASVDEB EQU   WKALST
WKADEBXC EQU   WKASVDEB
WKANQLST EQU   WKALST+8
WKATRACX EQU   WKANQLST
SDWAWORK EQU   SDWAVRA
MESSAGE  EQU   SDWAWORK
RETCOD   EQU   MESSAGE+16
RETCOD2  EQU   MESSAGE+19
MSGJOB   EQU   MESSAGE+22
MSGSTEP  EQU   MESSAGE+31
MSGDDN   EQU   MESSAGE+40
SAVESDWA EQU   SDWAWORK+52
HEAD     EQU   MESSAGE
HEADER   EQU   HEAD
JOBNAME  EQU   HEADER+10
STEPNAME EQU   HEADER+19
SDUMPLST EQU   HEAD+27
RECORDWK EQU   MESSAGE
RECAUDIT EQU   RECORDWK
RECRCDP  EQU   RECORDWK+1
RECRCLN  EQU   RECORDWK+2
RECREG0  EQU   RECORDWK+4
RECREG1  EQU   RECORDWK+8
RECOPSW  EQU   RECORDWK+12
RECPERC  EQU   RECORDWK+20
FLC      EQU   PSA
TRCLIST  EQU   WKATRACX
         AGO   .@UNREFD                START UNREFERENCED COMPONENTS
@NM00167 EQU   RECORDWK+3
@NM00165 EQU   HEADER+18
@NM00164 EQU   HEADER+9
MODNAME  EQU   HEADER+1
HEADLEN  EQU   HEADER
@NM00163 EQU   MESSAGE+48
@NM00162 EQU   MESSAGE+39
@NM00161 EQU   MESSAGE+30
@NM00160 EQU   MESSAGE+21
@NM00159 EQU   MESSAGE+18
RCODE    EQU   RETCOD+1
GTFLAG   EQU   RETCOD
@NM00158 EQU   MESSAGE+15
ABND     EQU   MESSAGE+12
@NM00157 EQU   MESSAGE+11
MSGNO    EQU   MESSAGE+4
@NM00156 EQU   MESSAGE
@NM00155 EQU   WKANQLST+8
WKAQPREF EQU   WKALST+4
@NM00154 EQU   WKASVDEB+1
WKATRXLG EQU   WKACLENX
@NM00153 EQU   WKASW1
WKAD2305 EQU   WKASW1
TIOUCBP  EQU   TIOEDEVE+1
TIOVERF  EQU   TIOSTTB
TIOUNLD  EQU   TIOSTTB
TIOMNTD  EQU   TIOSTTB
TIOSETU  EQU   TIOSTTB
TIOVLSR  EQU   TIOSTTB
TIOPVIO  EQU   TIOSTTB
TIOREQD  EQU   TIOSTTB
TIOUSED  EQU   TIOSTTB
RBRSV161 EQU   RBSECS09+64
RBSIRBWA EQU   RBSECS09
RBIQEWRK EQU   RBSECS08+4
RBNEXAV  EQU   RBSECS08
RBIQEA   EQU   RBIQE2
RBIQE1   EQU   RBIQE+1
RBUSE    EQU   RBIQE
RBSQEA   EQU   RBSQE+1
@NM00098 EQU   RBSQE
RBCDE1   EQU   RBCDE+1
RBCDLOAD EQU   RBCDFLGS
RBCDXCTL EQU   RBCDFLGS
RBCDSYNC EQU   RBCDFLGS
RBCDNODE EQU   RBCDFLGS
RBCDSAVE EQU   RBCDFLGS
RBCDATCH EQU   RBCDFLGS
RBRSV009 EQU   RBCDFLGS
RBNOCELL EQU   RBCDFLGS
PSARSAV  EQU   PSARSVTE+60
PSARSTK  EQU   PSARSVTE+56
PSAESAV3 EQU   PSARSVTE+52
PSAESTK3 EQU   PSARSVTE+48
PSAESAV2 EQU   PSARSVTE+44
PSAESTK2 EQU   PSARSVTE+40
PSAESAV1 EQU   PSARSVTE+36
PSAESTK1 EQU   PSARSVTE+32
PSAPSAV  EQU   PSARSVTE+28
PSAPSTK  EQU   PSARSVTE+24
PSAMSAV  EQU   PSARSVTE+20
PSAMSTK  EQU   PSARSVTE+16
PSASSAV  EQU   PSARSVTE+12
PSASSTK  EQU   PSARSVTE+8
PSANSTK  EQU   PSARSVTE+4
PSACSTK  EQU   PSARSVTE
IOBSKSS  EQU   IOBSKADR+3
IOBSK0   EQU   IOBSKADR+2
IOBSKTT  EQU   IOBSKADR+1
IOBSKRV  EQU   IOBSKADR
IOBCSWSV EQU   IOBORG11+17
IOBSENSV EQU   IOBORG11+16
IOBRCD   EQU   IOBORG11+12
IOBMDREC EQU   IOBORG11+8
IOBNAMEA EQU   IOBNAME+1
IOBNAMSZ EQU   IOBNAME
IOBERCTA EQU   IOBERCT+1
IOBRTYPE EQU   IOBUCBXV
IOBRDCHP EQU   IOBORG10+20
IOBNEXTA EQU   IOBORG10+16
IOBREADA EQU   IOBORG10+12
IOBBUFC  EQU   IOBORG10+8
IOBSK2R  EQU   IOBSEEK2+7
IOBSK2HH EQU   IOBSEEK2+5
IOBSK2CC EQU   IOBSEEK2+3
IOBSK2BB EQU   IOBSEEK2+1
IOBSK2M  EQU   IOBSEEK2
IOBCHNPR EQU   IOBORG09+40
IOBDNCRF EQU   IOBORG09+32
IOBRSV35 EQU   IOBORG09+24
IOBRSV34 EQU   IOBORG09+18
IOBDBYTN EQU   IOBORG09+16
IOBDCPND EQU   IOBORG09+12
IOBSIORT EQU   IOBSTAT1
IOBBUFF  EQU   IOBSTAT1
IOBPASS2 EQU   IOBSTAT1
IOBSYNCH EQU   IOBSTAT1
IOBNEWVL EQU   IOBSTAT1
IOBABNRM EQU   IOBSTAT1
IOBRELEX EQU   IOBDTYP2
IOBTYPE  EQU   IOBDTYP2
IOBRQUST EQU   IOBDTYP2
IOBSUFFX EQU   IOBDTYP2
IOBSBLKL EQU   IOBDTYP2
IOBSKEY  EQU   IOBDTYP2
IOBRELBL EQU   IOBDTYPE
IOBRDEXC EQU   IOBDTYPE
IOBDYNBF EQU   IOBDTYPE
IOBACTAD EQU   IOBDTYPE
IOBFDBCK EQU   IOBDTYPE
IOBEXTSC EQU   IOBDTYPE
IOBOVFLO EQU   IOBDTYPE
IOBVERFY EQU   IOBDTYPE
IOBDPLB  EQU   IOBDPLAD+1
IOBDIOBS EQU   IOBORG09+2
IOBDBYTR EQU   IOBORG09
IOBBCHAD EQU   IOBORG08+12
IOBFCHNB EQU   IOBFCHAD+1
IOBCOUNT EQU   IOBFCHAD
IOBASYN  EQU   IOBORG08+7
IOBAPP   EQU   IOBORG08+6
IOBRSV32 EQU   IOBUNSQR
IOBRSV31 EQU   IOBUNSQR
IOBRSV30 EQU   IOBUNSQR
IOBKNRWR EQU   IOBUNSQR
IOBKNWR  EQU   IOBUNSQR
IOBNTAV2 EQU   IOBUNSQR
IOBNTAV1 EQU   IOBUNSQR
IOBCPBSY EQU   IOBUNSQR
IOBCHNNL EQU   IOBINDCT
IOBRSV29 EQU   IOBINDCT
IOBRSV28 EQU   IOBINDCT
IOBRSV27 EQU   IOBINDCT
IOBKEYAD EQU   IOBINDCT
IOBOVPTR EQU   IOBINDCT
IOBUNSCH EQU   IOBINDCT
IOBDEQCP EQU   IOBINDCT
IOBCCWAD EQU   IOBORG08
W1OEXTEN EQU   W1IEXTEN
IOBCCW   EQU   IOBORG06+8
IOBNXTPB EQU   IOBNXTPT+1
IOBRSV26 EQU   IOBSTATA
IOBRSV25 EQU   IOBSTATA
IOBRSV24 EQU   IOBSTATA
IOBRSV23 EQU   IOBSTATA
IOBRSV22 EQU   IOBSTATA
IOBRSV21 EQU   IOBSTATA
IOBRSV20 EQU   IOBSTATA
IOBAVLFL EQU   IOBSTATA
IOBRSV37 EQU   IOBORG06+1
IOBUCBXG EQU   IOBORG06
IOBCPA   EQU   IOBORG05+32
IOBERINF EQU   IOBORG05+16
IOBERCCW EQU   IOBORG05+8
IOBSNDPT EQU   IOBORG05+7
IOBRCVPT EQU   IOBORG05+6
IOBWORK  EQU   IOBORG05+1
IOBUCBX  EQU   IOBORG05
IOBR     EQU   IOBSEEK+7
IOBHH2   EQU   IOBHH+1
IOBHH1   EQU   IOBHH
IOBCC2   EQU   IOBCC+1
IOBCC1   EQU   IOBCC
IOBBB2   EQU   IOBBB+1
IOBBB1   EQU   IOBBB
IOBM     EQU   IOBSEEK
IOBRSV46 EQU   IOBFL4
IOBRSV47 EQU   IOBFL4
IOBENT   EQU   IOBFL4
IOBEQP   EQU   IOBFL4
IOBBUS   EQU   IOBFL4
IOBDCK   EQU   IOBFL4
IOBREJ   EQU   IOBFL4
IOBOVR   EQU   IOBFL4
IOBCRILC EQU   IOBORG04+1
IOBCRDCC EQU   IOBORG04
IOBGQECB EQU   IOBORG03
IOBSWAP  EQU   IOBORG02+4
IOBDQADB EQU   IOBDQADA+1
IOBRSV13 EQU   IOBDEQIN
IOBRSV12 EQU   IOBDEQIN
IOBRSV11 EQU   IOBDEQIN
IOBRSV10 EQU   IOBDEQIN
IOBRSV09 EQU   IOBDEQIN
IOBRSV08 EQU   IOBDEQIN
IOBRSV07 EQU   IOBDEQIN
IOBDEQ   EQU   IOBDEQIN
IOBNECB  EQU   IOBORG01+4
IOBNIOBB EQU   IOBNIOBA+1
IOBFIRST EQU   IOBNFLG1
IOBUPERR EQU   IOBNFLG1
IOBSPAN  EQU   IOBNFLG1
IOBBKSPC EQU   IOBNFLG1
IOBUPDAT EQU   IOBNFLG1
IOBREAD  EQU   IOBNFLG1
IOBWRITE EQU   IOBNFLG1
IOBSEGMT EQU   IOBPRTOV
DECBLFMT EQU   DECBS05+8
DECBLNNM EQU   DECBS05+4
DECBHDRA EQU   DECBS05
DEBEXPTA EQU   DEBEXPT+1
DEBRSV12 EQU   DEBRPSID
DEBRSV11 EQU   DEBRPSID
DEBRSV10 EQU   DEBRPSID
DEBRSV09 EQU   DEBRPSID
DEBRPSAP EQU   DEBRPSID
DEBRPSO  EQU   DEBRPSID
DEBRPSI  EQU   DEBRPSID
DEBRPSP  EQU   DEBRPSID
DEBFOEB  EQU   DEBFOEAD+1
DEBNOEE  EQU   DEBFOEAD
DEBFPEB  EQU   DEBFPEAD+1
DEBNPEE  EQU   DEBFPEAD
DEBFIEB  EQU   DEBFIEAD+1
DEBNIEE  EQU   DEBFIEAD
DEBBOESS EQU   DEBEODSS
DEBBOE0  EQU   DEBEOD0
DEBBOETT EQU   DEBEODTT
DEBBOERV EQU   DEBEODRV
DEBRV008 EQU   DEBDSID+7
DEBWTPTI EQU   DEBDSID+6
DEBEXDTE EQU   DEBDSID
DEBRV007 EQU   DEBEAMFG
DEBRV006 EQU   DEBEAMFG
DEBRV005 EQU   DEBEAMFG
DEBRV004 EQU   DEBEAMFG
DEBSECVL EQU   DEBEAMFG
DEBVAMSG EQU   DEBEAMFG
DEBDSOPN EQU   DEBEAMFG
DEBMULTI EQU   DEBEAMFG
DEBVSEQU EQU   DEBASC09+2
DEBDSSQL EQU   DEBASC09+1
DEBVOLAC EQU   DEBASC09
DEBWDCBA EQU   DEBWDCB+1
DEBRSV08 EQU   DEBWDCB
DEBPDCBA EQU   DEBPDCB+1
DEBRSV07 EQU   DEBPDCB
DEBRDCBA EQU   DEBRDCB+1
DEBRSV06 EQU   DEBRDCB
DEBSUCBB EQU   DEBSUCBA+1
DEBSDVM  EQU   DEBSUCBA
CVTRV628 EQU   CVTS01+728
CVTRV627 EQU   CVTS01+724
CVTRV626 EQU   CVTS01+720
CVTRV625 EQU   CVTS01+716
CVTRV624 EQU   CVTS01+712
CVTRV623 EQU   CVTS01+708
CVTRV622 EQU   CVTS01+704
CVTRV621 EQU   CVTS01+700
CVTIHASU EQU   CVTS01+696
CVTRV619 EQU   CVTS01+692
CVTRV618 EQU   CVTS01+688
CVTRV617 EQU   CVTS01+684
CVTRV616 EQU   CVTS01+680
CVTRV615 EQU   CVTS01+676
CVTRV614 EQU   CVTS01+672
CVTRV613 EQU   CVTS01+668
CVTTCASP EQU   CVTS01+664
CVT0PT03 EQU   CVTS01+660
CVT0PT0E EQU   CVTS01+656
CVTRV609 EQU   CVTS01+652
CVTCGK   EQU   CVTS01+648
CVTRAC   EQU   CVTS01+644
CVTHSM   EQU   CVTS01+640
CVTRV605 EQU   CVTS01+636
CVTRV604 EQU   CVTS01+632
CVTEFF02 EQU   CVTS01+628
CVTCBBR  EQU   CVTS01+624
CVTSSCR  EQU   CVTS01+620
CVTEVENT EQU   CVTS01+616
CVTCRCA  EQU   CVTS01+612
CVTTPIO  EQU   CVTS01+608
CVTADV   EQU   CVTS01+604
CVTGSDAB EQU   CVTGSDA
CVTQV3   EQU   CVTS01+596
CVTQV2   EQU   CVTS01+592
CVTQV1   EQU   CVTS01+588
CVTRPT   EQU   CVTS01+584
CVTSSRB  EQU   CVTS01+580
CVTCSDRL EQU   CVTS01+576
CVTEXP1  EQU   CVTS01+572
CVTRMPMT EQU   CVTS01+568
CVTRMPTT EQU   CVTS01+564
CVTVPSA  EQU   CVTS01+560
CVTVSTOP EQU   CVTS01+556
CVTGTFR8 EQU   CVTS01+552
CVTQUIT  EQU   CVTS01+548
CVTVACR  EQU   CVTS01+544
CVTWTCB  EQU   CVTS01+540
CVTSTPRS EQU   CVTS01+536
CVT0PT02 EQU   CVTS01+532
CVTDARCM EQU   CVTS01+528
CVTIRECM EQU   CVTS01+524
CVTJRECM EQU   CVTS01+520
CVTVEMS0 EQU   CVTS01+516
CVTSPFRR EQU   CVTS01+512
CVTRLSTG EQU   CVTS01+508
CVT0TC0A EQU   CVTS01+504
CVTGMBR  EQU   CVTS01+500
CVTLFRM  EQU   CVTS01+496
CVTRMBR  EQU   CVTS01+492
CVTVIOP  EQU   CVTS01+488
CVTRV307 EQU   CVTS01+486
CVTRV306 EQU   CVTS01+484
CVTRV305 EQU   CVTS01+482
CVTRV304 EQU   CVTS01+480
CVTRV303 EQU   CVTS01+478
CVTRV302 EQU   CVTS01+476
CVTTRCA  EQU   CVTS01+472
CVTRV297 EQU   CVTRV289
CVTRV296 EQU   CVTRV289
CVTRV295 EQU   CVTRV289
CVTRV294 EQU   CVTRV289
CVTRV293 EQU   CVTRV289
CVTRV292 EQU   CVTRV289
CVTRV291 EQU   CVTRV289
CVTRV290 EQU   CVTRV289
CVTRV288 EQU   CVTRV280
CVTRV287 EQU   CVTRV280
CVTRV286 EQU   CVTRV280
CVTRV285 EQU   CVTRV280
CVTRV284 EQU   CVTRV280
CVTRV283 EQU   CVTRV280
CVTRV282 EQU   CVTRV280
CVTRV281 EQU   CVTRV280
CVTRV279 EQU   CVTRV271
CVTRV278 EQU   CVTRV271
CVTRV277 EQU   CVTRV271
CVTRV276 EQU   CVTRV271
CVTRV275 EQU   CVTRV271
CVTRV274 EQU   CVTRV271
CVTRV273 EQU   CVTRV271
CVTRV272 EQU   CVTRV271
CVTRV270 EQU   CVTRV262
CVTRV269 EQU   CVTRV262
CVTRV268 EQU   CVTRV262
CVTRV267 EQU   CVTRV262
CVTRV266 EQU   CVTRV262
CVTRV265 EQU   CVTRV262
CVTRV264 EQU   CVTRV262
CVTRV263 EQU   CVTRV262
CVTVFP   EQU   CVTS01+464
CVTVSI   EQU   CVTS01+460
CVTVPSIB EQU   CVTS01+456
CVTMFACT EQU   CVTMFRTR
CVTMFCTL EQU   CVTS01+448
CVTPVBP  EQU   CVTS01+444
CVTPWI   EQU   CVTS01+440
CVTRV254 EQU   CVTS01+438
CVTRV253 EQU   CVTS01+436
CVTRV252 EQU   CVTS01+434
CVTRV251 EQU   CVTS01+433
CVTRV250 EQU   CVTS01+432
CVTRV249 EQU   CVTS01+431
CVTRV248 EQU   CVTS01+430
CVTRV247 EQU   CVTS01+429
CVTRV246 EQU   CVTS01+428
CVTRV245 EQU   CVTRV237
CVTRV244 EQU   CVTRV237
CVTRV243 EQU   CVTRV237
CVTRV242 EQU   CVTRV237
CVTRV241 EQU   CVTRV237
CVTRV240 EQU   CVTRV237
CVTRV239 EQU   CVTRV237
CVTRV238 EQU   CVTRV237
CVTRV236 EQU   CVTRV228
CVTRV235 EQU   CVTRV228
CVTRV234 EQU   CVTRV228
CVTRV233 EQU   CVTRV228
CVTRV232 EQU   CVTRV228
CVTRV231 EQU   CVTRV228
CVTRV230 EQU   CVTRV228
CVTRV229 EQU   CVTRV228
CVTRV227 EQU   CVTRV219
CVTRV226 EQU   CVTRV219
CVTRV225 EQU   CVTRV219
CVTRV224 EQU   CVTRV219
CVTRV223 EQU   CVTRV219
CVTRV222 EQU   CVTRV219
CVTRV221 EQU   CVTRV219
CVTRV220 EQU   CVTRV219
CVTRV218 EQU   CVTRV210
CVTRV217 EQU   CVTRV210
CVTRV216 EQU   CVTRV210
CVTRV215 EQU   CVTRV210
CVTRV214 EQU   CVTRV210
CVTRV213 EQU   CVTRV210
CVTRV212 EQU   CVTRV210
CVTRV211 EQU   CVTRV210
CVTLCCAT EQU   CVTS01+420
CVTPCCAT EQU   CVTS01+416
CVTIPCRP EQU   CVTS01+412
CVTIPCRI EQU   CVTS01+408
CVTIPCDS EQU   CVTS01+404
CVTAIDVT EQU   CVTS01+400
CVTSSAP  EQU   CVTS01+396
CVTEHCIR EQU   CVTS01+392
CVTEHDEF EQU   CVTS01+388
CVTDAIR  EQU   CVTS01+384
CVTPERFM EQU   CVTS01+380
CVT044R2 EQU   CVTS01+376
CVTFETCH EQU   CVTS01+372
CVTRSTWD EQU   CVTS01+368
CVTSPOST EQU   CVTS01+364
CVTIOBP  EQU   CVTS01+360
CVTASMVT EQU   CVTS01+356
CVTRECRQ EQU   CVTS01+352
CVTWSAC  EQU   CVTS01+348
CVTRV149 EQU   CVTS01+344
CVTWSAL  EQU   CVTS01+340
CVTSPSA  EQU   CVTS01+336
CVTGLMN  EQU   CVTS01+332
CVTVEAC0 EQU   CVTS01+328
CVT062R1 EQU   CVTS01+324
CVTRPOST EQU   CVTS01+320
CVTDQIQE EQU   CVTS01+316
CVTCSD   EQU   CVTS01+312
CVTLKRMA EQU   CVTS01+308
CVTRSPIE EQU   CVTS01+304
CVTRENQ  EQU   CVTS01+300
CVTLQCB  EQU   CVTS01+296
CVTFQCB  EQU   CVTS01+292
CVTQCS01 EQU   CVTS01+288
CVTAPFT  EQU   CVTS01+284
CVTPARRL EQU   CVTS01+280
CVTVWAIT EQU   CVTS01+276
CVTGSPL  EQU   CVTS01+272
CVTLSMQ  EQU   CVTS01+268
CVTGSMQ  EQU   CVTS01+264
CVTEXPRO EQU   CVTS01+260
CVTOPCTP EQU   CVTS01+256
CVTSIC   EQU   CVTS01+252
CVTTPIOS EQU   CVTS01+248
CVTRTMS  EQU   CVTS01+244
CVTSDBF  EQU   CVTS01+240
CVTSCBP  EQU   CVTS01+236
CVTSDMP  EQU   CVTS01+232
CVTSV60  EQU   CVTS01+228
CVTRTMCT EQU   CVTS01+224
CVTASCBL EQU   CVTS01+220
CVTASCBH EQU   CVTS01+216
CVTGDA   EQU   CVTS01+212
CVTASVT  EQU   CVTS01+208
CVTVVMDI EQU   CVTS01+204
CVTAQTOP EQU   CVTS01+200
CVTIOSCS EQU   CVTS01+196
CVTSDRM  EQU   CVTS01+192
CVTOPTE  EQU   CVTS01+188
CVTSTXU  EQU   CVTS01+184
CVTQUIS  EQU   CVTS01+180
CVTPARS  EQU   CVTS01+176
CVTS1EE  EQU   CVTS01+172
CVTFRAS  EQU   CVTS01+168
CVTQSAS  EQU   CVTS01+164
CVTCRAS  EQU   CVTS01+160
CVTCRMN  EQU   CVTS01+156
CVTDELCP EQU   CVTS01+152
CVTFRECL EQU   CVTS01+148
CVTGETCL EQU   CVTS01+144
CVTBLDCP EQU   CVTS01+140
CVTAUTHL EQU   CVTS01+136
CVTSCAN  EQU   CVTS01+132
CVTRV144 EQU   CVTS01+130
CVTMAXMP EQU   CVTS01+128
CVTSTCK  EQU   CVTS01+124
CVTRV139 EQU   CVTS01+123
CVTDSSAC EQU   CVTS01+122
CVTRV513 EQU   CVTS01+121
CVTIOSPL EQU   CVTS01+120
CVTPTGT  EQU   CVTS01+116
CVTCSPIE EQU   CVTS01+112
CVTSMFEX EQU   CVTS01+108
CVTOLT0A EQU   CVTS01+104
CVTSRBRT EQU   CVTS01+100
CVTPUTL  EQU   CVTS01+96
CVTASCRL EQU   CVTS01+92
CVTASCRF EQU   CVTS01+88
CVTRV326 EQU   CVTS01+84
CVTRV325 EQU   CVTS01+80
CVTRV324 EQU   CVTS01+76
CVT0VL01 EQU   CVTS01+72
CVTSHRVM EQU   CVTS01+68
CVTRV332 EQU   CVTS01+64
CVTTAS   EQU   CVTS01+60
CVTRSCN  EQU   CVTS01+56
CVTTRAC2 EQU   CVTS01+54
CVTTRACE EQU   CVTS01+52
CVTAPG   EQU   CVTS01+51
CVTSDTRC EQU   CVTCTLFG
CVTGTRCE EQU   CVTCTLFG
CVTNOMP  EQU   CVTCTLFG
CVTRSV79 EQU   CVTCTLFG
CVTDSTAT EQU   CVTCTLFG
CVTRSV78 EQU   CVTCTLFG
CVTRV333 EQU   CVTCTLFG
CVTRV323 EQU   CVTCTLFG
CVTSPVLK EQU   CVTS01+49
CVTRSV77 EQU   CVTS01+48
CVTRV331 EQU   CVTS01+44
CVTRV330 EQU   CVTS01+40
CVTRV329 EQU   CVTS01+36
CVTRV328 EQU   CVTS01+32
CVTRV322 EQU   CVTS01+28
CVTSLID  EQU   CVTSLIDA+1
CVTSYLK  EQU   CVTSLIDA
CVTRV321 EQU   CVTS01+20
CVTRV320 EQU   CVTS01+16
CVTLPDIR EQU   CVTLPDIA+1
CVTRSV69 EQU   CVTDIRST
CVTRSV68 EQU   CVTDIRST
CVTRSV67 EQU   CVTDIRST
CVTRSV66 EQU   CVTDIRST
CVTRSV65 EQU   CVTDIRST
CVTRSV64 EQU   CVTDIRST
CVTRSV63 EQU   CVTDIRST
CVTDICOM EQU   CVTDIRST
CVTPVTP  EQU   CVTS01+8
CVTLPDSR EQU   CVTS01+4
CVTGETL  EQU   CVTS01
TIOTFEND EQU   POOLSTAR+16
TIOPSLOT EQU   POOLSTAR+13
TIOPSTTB EQU   POOLSTAR+12
TIOPPOOL EQU   POOLSTAR+4
TIOPNSRT EQU   POOLSTAR+3
@NM00152 EQU   POOLSTAR+2
TIOPNSLT EQU   POOLSTAR+1
@NM00151 EQU   POOLSTAR
TIOEFSRT EQU   TIOENTRY+17
TIOSVERF EQU   TIOESTTB
TIOSUNLD EQU   TIOESTTB
TIOSMNTD EQU   TIOESTTB
TIOSSETU EQU   TIOESTTB
TIOSVLSR EQU   TIOESTTB
TIOSPVIO EQU   TIOESTTB
TIOSREQD EQU   TIOESTTB
TIOSUSED EQU   TIOESTTB
TIOSBALS EQU   TIOESTTC
TIOSBALP EQU   TIOESTTC
TIOSVOLS EQU   TIOESTTC
TIOSVOLP EQU   TIOESTTC
TIOSAFFS EQU   TIOESTTC
TIOSAFFP EQU   TIOESTTC
TIOSDEFR EQU   TIOESTTC
TIOSDKCR EQU   TIOESTTC
TIOEJFCB EQU   TIOENTRY+12
TIOTREM  EQU   TIOELINK
TIOESSDS EQU   TIOESYOT
TIOESYIN EQU   TIOELINK
TIOEQNAM EQU   TIOELINK
TIOEDYNM EQU   TIOELINK
TIOTTERM EQU   TIOELINK
TIOTRV01 EQU   TIOELINK
TIOSYOUT EQU   TIOELINK
TIOEWTCT EQU   TIOERLOC
TIOSDSP2 EQU   TIOESTTA
TIOSDSP1 EQU   TIOESTTA
TIOSLABL EQU   TIOESTTA
TIOSDADS EQU   TIOESTTA
TIOSJBLB EQU   TIOESTTA
TIOSPLTS EQU   TIOESTTA
TIOSPLTP EQU   TIOESTTA
TIOSLTYP EQU   TIOESTTA
TIOELNGH EQU   TIOENTRY
@NM00150 EQU   TCBXTNT2+32
TCBRV181 EQU   TCBRV173
TCBRV180 EQU   TCBRV173
TCBRV179 EQU   TCBRV173
TCBRV178 EQU   TCBRV173
TCBRV177 EQU   TCBRV173
TCBRV176 EQU   TCBRV173
TCBRV175 EQU   TCBRV173
TCBRV174 EQU   TCBRV173
TCBRV172 EQU   TCBRV164
TCBRV171 EQU   TCBRV164
TCBRV170 EQU   TCBRV164
TCBRV169 EQU   TCBRV164
TCBRV168 EQU   TCBRV164
TCBRV167 EQU   TCBRV164
TCBRV166 EQU   TCBRV164
TCBRV165 EQU   TCBRV164
TCBRV163 EQU   TCBRV155
TCBRV162 EQU   TCBRV155
TCBRV161 EQU   TCBRV155
TCBRV160 EQU   TCBRV155
TCBRV159 EQU   TCBRV155
TCBRV158 EQU   TCBRV155
TCBRV157 EQU   TCBRV155
TCBRV156 EQU   TCBRV155
TCBRV154 EQU   TCBRV146
TCBRV153 EQU   TCBRV146
TCBRV152 EQU   TCBRV146
TCBRV151 EQU   TCBRV146
TCBRV150 EQU   TCBRV146
TCBRV149 EQU   TCBRV146
TCBRV148 EQU   TCBRV146
TCBRV147 EQU   TCBRV146
TCBRV145 EQU   TCBXTNT2+26
TCBRV144 EQU   TCBXTNT2+24
TCBRV143 EQU   TCBXTNT2+20
TCBRV142 EQU   TCBXTNT2+16
TCBRSV49 EQU   TCBXTNT2+12
TCBEVENT EQU   TCBXTNT2+8
TCBRCMP  EQU   TCBXTNT2+5
TCBRSVAB EQU   TCBXTNT2+4
TCBGTFA  EQU   TCBGTF+1
TCBRSV47 EQU   TCBTFLG
TCBRSV46 EQU   TCBTFLG
TCBRSV45 EQU   TCBTFLG
TCBRSV44 EQU   TCBTFLG
TCBRSV43 EQU   TCBTFLG
TCBDSPIT EQU   TCBTFLG
TCBERRTN EQU   TCBTFLG
TCBASYNC EQU   TCBTFLG
@NM00149 EQU   TCB+296
TCBSVCA2 EQU   TCB+292
TCBSWASA EQU   TCB+288
TCBDBTB  EQU   TCB+284
TCBRPT   EQU   TCB+280
TCBRV141 EQU   TCBRV133
TCBRV140 EQU   TCBRV133
TCBRV139 EQU   TCBRV133
TCBRV138 EQU   TCBRV133
TCBRV137 EQU   TCBRV133
TCBRV136 EQU   TCBRV133
TCBRV135 EQU   TCBRV133
TCBRV134 EQU   TCBRV133
TCBRV132 EQU   TCBRV124
TCBRV131 EQU   TCBRV124
TCBRV130 EQU   TCBRV124
TCBRV129 EQU   TCBRV124
TCBRV128 EQU   TCBRV124
TCBRV127 EQU   TCBRV124
TCBRV126 EQU   TCBRV124
TCBRV125 EQU   TCBRV124
TCBRV123 EQU   TCBFBYT2
TCBRV122 EQU   TCBFBYT2
TCBECBNV EQU   TCBFBYT2
TCBSSSYN EQU   TCBFBYT2
TCBFPRAP EQU   TCBFBYT2
TCBFDW   EQU   TCBFBYT2
TCBFMW   EQU   TCBFBYT2
TCBCNCB  EQU   TCBFBYT2
TCBLLH   EQU   TCBFBYT1
TCBRV113 EQU   TCBFBYT1
TCBEOT   EQU   TCBFBYT1
TCBRTM2  EQU   TCBFBYT1
TCBPGNLY EQU   TCBFBYT1
TCBNDIOS EQU   TCBFBYT1
TCBACTIV EQU   TCBFBYT1
TCBEOTFM EQU   TCBFBYT1
TCBAFFN  EQU   TCB+274
TCBCCPVI EQU   TCB+272
TCBUKYSP EQU   TCB+268
TCBMODE  EQU   TCBESTAE+3
TCBERTYP EQU   TCBESTAE+2
TCBRV314 EQU   TCBESTRM
TCBRV313 EQU   TCBESTRM
TCBRV312 EQU   TCBESTRM
TCBRV311 EQU   TCBESTRM
TCBRV310 EQU   TCBESTRM
TCBRV309 EQU   TCBESTRM
TCBRV308 EQU   TCBESTRM
TCBETERM EQU   TCBESTRM
TCBSCBKY EQU   TCBESTAE
TCBRTM12 EQU   TCB+260
TCBTCBID EQU   TCB+256
TCBRSV96 EQU   TCB+252
TCBSWA   EQU   TCB+248
TCBFOEA  EQU   TCBFOE+1
TCBRSV42 EQU   TCBFOE
TCBRV306 EQU   TCB+240
TCBRSV41 EQU   TCB+239
TCBTID   EQU   TCB+238
TCBRSVAA EQU   TCB+237
TCBABCUR EQU   TCB+236
TCBTMSAV EQU   TCB+232
TCBIOTIM EQU   TCB+228
TCBRTWA  EQU   TCB+224
TCBBACK  EQU   TCB+220
TCBTIRB  EQU   TCB+216
TCBAECB  EQU   TCB+212
TCBEXT2A EQU   TCBEXT2+1
TCBRSV39 EQU   TCBEXT2
TCBSTMCT EQU   TCB+207
TCBSYSCT EQU   TCB+206
TCBRSV37 EQU   TCB+205
TCBEXSVC EQU   TCBDAR
TCBDARMS EQU   TCBDAR
TCBDARWT EQU   TCBDAR
TCBDARO  EQU   TCBDAR
TCBDARMC EQU   TCBDARC
TCBDARD  EQU   TCBDAR
TCBDARS  EQU   TCBDAR
TCBDARP  EQU   TCBDAR
TCBGTOFM EQU   TCBFLGS7
TCBADMP  EQU   TCBFLGS7
TCBRSTSK EQU   TCBFLGS7
TCBSVCS  EQU   TCBFLGS7
TCBSTACK EQU   TCBFLGS7
TCBRSV34 EQU   TCBFLGS7
TCBRSV33 EQU   TCBFLGS7
TCBGPECB EQU   TCBFLGS7
TCBNTJS  EQU   TCBFLGS6
TCBAPG   EQU   TCBFLGS6
TCBMIGR  EQU   TCBFLGS6
TCBRV303 EQU   TCBFLGS6
TCBSPVLK EQU   TCBFLGS6
TCBCPU   EQU   TCBFLGS6
TCBPIE17 EQU   TCBFLGS6
TCBRV    EQU   TCBFLGS6
TCBRSV79 EQU   TCBNDSP5
TCBRSV78 EQU   TCBNDSP5
TCBRSV77 EQU   TCBNDSP5
TCBRSV76 EQU   TCBNDSP5
TCBRSV75 EQU   TCBNDSP5
TCBRSV74 EQU   TCBNDSP5
TCBRSV95 EQU   TCBNDSP5
TCBRSV94 EQU   TCBNDSP5
TCBRSV93 EQU   TCBNDSP4
TCBRSV92 EQU   TCBNDSP4
TCBRSV91 EQU   TCBNDSP4
TCBRSV90 EQU   TCBNDSP4
TCBRSV89 EQU   TCBNDSP4
TCBRSV88 EQU   TCBNDSP4
TCBRSV87 EQU   TCBNDSP4
TCBRSV86 EQU   TCBNDSP4
TCBEXT1A EQU   TCBEXT1+1
TCBRSV32 EQU   TCBEXT1
TCBEXCPD EQU   TCB+192
TCBIOBRC EQU   TCB+188
TCBDDWTC EQU   TCB+186
TCBDDEXC EQU   TCB+184
TCBJSCBB EQU   TCBJSCB+1
@NM00148 EQU   TCBRECDE
TCBREC   EQU   TCBRECDE
TCBMDIDS EQU   TCB+176
TCBNDINT EQU   TCBNDSP3
TCBRSV27 EQU   TCBNDSP3
TCBRSV26 EQU   TCBNDSP3
TCBRSV25 EQU   TCBNDSP3
TCBRSV24 EQU   TCBNDSP3
TCBSRBND EQU   TCBNDSP3
TCBRV302 EQU   TCBNDSP3
TCBLJSND EQU   TCBNDSP3
TCBABE   EQU   TCBNDSP2
TCBDSS   EQU   TCBNDSP2
TCBOWAIT EQU   TCBNDSP2
TCBIWAIT EQU   TCBNDSP2
TCBNDTS  EQU   TCBNDSP2
TCBNDSVC EQU   TCBNDSP2
TCBSTPP  EQU   TCBNDSP2
TCBABD   EQU   TCBNDSP2
TCBRSV22 EQU   TCBNDSP1
TCBPIEND EQU   TCBNDSP1
TCBTPSP  EQU   TCBNDSP1
TCBDDRND EQU   TCBNDSP1
TCBRSPND EQU   TCBNDSP1
TCBRSTND EQU   TCBNDSP1
TCBDARPN EQU   TCBNDSP1
TCBDARTN EQU   TCBNDSP1
TCBNDSP0 EQU   TCBNDSP
TCBUSER  EQU   TCB+168
TCBTCTB  EQU   TCBTCT+1
TCBRSV9C EQU   TCBTCTGF
TCBRSV9B EQU   TCBTCTGF
TCBRSV9A EQU   TCBTCTGF
TCBRSV99 EQU   TCBTCTGF
TCBRSV98 EQU   TCBTCTGF
TCBRSV97 EQU   TCBTCTGF
TCBRSV20 EQU   TCBTCTGF
TCBSMFGF EQU   TCBTCTGF
TCBSTABB EQU   TCBSTAB+1
TCBSTCUR EQU   TCBNSTAE
TCBRV301 EQU   TCBNSTAE
TCBSYNCH EQU   TCBNSTAE
TCBHALT  EQU   TCBNSTAE
TCBPPSUP EQU   TCBNSTAE
TCB33E   EQU   TCBNSTAE
TCBQUIES EQU   TCBNSTAE
TCBSTABE EQU   TCBNSTAE
TCBAQE   EQU   TCB+156
TCBPQE   EQU   TCB+152
TCBTSDP  EQU   TCB+151
TCBTSLP  EQU   TCB+150
TCBSTPCT EQU   TCB+149
TCBCPUBN EQU   TCBTSFLG
TCBDYDSP EQU   TCBTSFLG
TCBRSV18 EQU   TCBTSFLG
TCBRSV17 EQU   TCBTSFLG
TCBTIOTG EQU   TCBTSFLG
TCBATT   EQU   TCBTSFLG
TCBSTPPR EQU   TCBTSFLG
TCBRV300 EQU   TCBTSTSK
TCBECB   EQU   TCB+144
TCBIQE   EQU   TCB+140
TCBLTC   EQU   TCB+136
TCBOTC   EQU   TCB+132
TCBNTC   EQU   TCB+128
TCBJSTCA EQU   TCBJSTCB+1
TCBRSV16 EQU   TCBJSTCB
TCBTQET  EQU   TCBTME
TCBTCB   EQU   TCB+116
TCBFSAB  EQU   TCBFSA+1
TCBQEL   EQU   TCBFSA
TCBGRS15 EQU   TCBGRS+60
TCBGRS14 EQU   TCBGRS+56
TCBGRS13 EQU   TCBGRS+52
TCBGRS12 EQU   TCBGRS+48
TCBGRS11 EQU   TCBGRS+44
TCBGRS10 EQU   TCBGRS+40
TCBGRS9  EQU   TCBGRS+36
TCBGRS8  EQU   TCBGRS+32
TCBGRS7  EQU   TCBGRS+28
TCBGRS6  EQU   TCBGRS+24
TCBGRS5  EQU   TCBGRS+20
TCBGRS4  EQU   TCBGRS+16
TCBGRS3  EQU   TCBGRS+12
TCBGRS2  EQU   TCBGRS+8
TCBGRS1  EQU   TCBGRS+4
TCBGRS0  EQU   TCBGRS
TCBJPQB  EQU   TCBJPQ+1
TCBRSV15 EQU   TCBPURGE
TCBRSV14 EQU   TCBPURGE
TCBRSV13 EQU   TCBPURGE
TCBRSV12 EQU   TCBPURGE
TCBRSV11 EQU   TCBPURGE
TCBRSV10 EQU   TCBPURGE
TCBRSV09 EQU   TCBPURGE
TCBJPQF  EQU   TCBPURGE
TCBJLB   EQU   TCB+40
TCBLLS   EQU   TCB+36
TCBDSP   EQU   TCB+35
TCBLMP   EQU   TCB+34
TCBPNDSP EQU   TCBFLGS5
TCBFCD1  EQU   TCBFLGS5
TCBSTP   EQU   TCBFLGS5
TCBSYS   EQU   TCBFLGS5
TCBANDSP EQU   TCBFLGS5
TCBPAGE  EQU   TCBFLGS5
TCBUXNDF EQU   TCBABWF
TCBFC    EQU   TCBFLGS5
TCBONDSP EQU   TCBFLGS4
TCBMPCND EQU   TCBFLGS4
TCBMPCVQ EQU   TCBFLGS4
TCBUXNDV EQU   TCBFLGS4
TCBHNDSP EQU   TCBFLGS4
TCBRQENA EQU   TCBFLGS4
TCBSER   EQU   TCBFLGS4
TCBNDUMP EQU   TCBFLGS4
TCBDWSTA EQU   TCBFLGS3
TCBRSV08 EQU   TCBFLGS3
TCBRSV07 EQU   TCBFLGS3
TCBRSV06 EQU   TCBFLGS3
TCBABGM  EQU   TCBFLGS3
TCBABTRM EQU   TCBFLGS3
TCBADINP EQU   TCBFLGS3
TCBFSM   EQU   TCBFLGS3
TCBFTS   EQU   TCBFLGS2
TCBFETXR EQU   TCBFLGS2
TCBFDSOP EQU   TCBFLGS2
TCBFJMC  EQU   TCBFLGS2
TCBFSMC  EQU   TCBFLGS2
TCBFABOP EQU   TCBFLGS2
TCBFSTI  EQU   TCBFLGS2
TCBFOINP EQU   TCBFLGS2
TCBFX    EQU   TCBFLGS1
TCBFS    EQU   TCBFLGS1
TCBFT    EQU   TCBFLGS1
TCBPDUMP EQU   TCBFLGS1
TCBNONPR EQU   TCBFLGS1
TCBFERA  EQU   TCBFLGS1
TCBFE    EQU   TCBFLGS1
TCBFA    EQU   TCBFLGS1
TCBZERO  EQU   TCBPKF
TCBFLAG  EQU   TCBPKF
TCBMSSB  EQU   TCBMSS+1
TCBRSV03 EQU   TCBMSS
TCBTRNB  EQU   TCBTRN+1
TCBRSV02 EQU   TCBABF
TCBOLTEP EQU   TCBABF
TCBTCP   EQU   TCBABF
TCBTCPP  EQU   TCBABF
TCBRSV01 EQU   TCBABF
TCBGRPH  EQU   TCBABF
TCBNOCHK EQU   TCBABF
TCBMOD91 EQU   TCBABF
TCBCMPC  EQU   TCBCMP+1
TCBRV318 EQU   TCBCMSG
TCBRV317 EQU   TCBCIND
TCBRV316 EQU   TCBCWTO
TCBCASID EQU   TCBCDBL
TCBNOCC  EQU   TCBSTCC
TCBDMPO  EQU   TCBCPP
TCBCSTEP EQU   TCBCMPF
TCBCREQ  EQU   TCBCMPF
TCBDEB   EQU   TCB+8
TCBPIEA  EQU   TCBPIE+1
TCBPM    EQU   TCBPMASK
@NM00147 EQU   TCBPMASK
TCBRBP   EQU   TCB
TCBFRS6  EQU   TCBFRS+24
TCBFRS4  EQU   TCBFRS+16
TCBFRS2  EQU   TCBFRS+8
TCBFRS0  EQU   TCBFRS
SPDDATA2 EQU   SPDDATA1+4
SPDBLKAD EQU   SPDDATA1
SPDPTRSW EQU   SPDBLKID
SPDRCDLN EQU   SPDRCDHD
SPDTRACE EQU   SPDBFR+9
SPDABCCD EQU   SPDHDR+8
SPDDDNAM EQU   SPDHDR
@NM00146 EQU   SDWA+512
SDWAURAL EQU   SDWARA+3
@NM00145 EQU   SDWADPVA
SDWAEBC  EQU   SDWADPVA
SDWAHEX  EQU   SDWADPVA
SDWAVRAL EQU   SDWARA
@NM00144 EQU   SDWA+396
SDWACOMP EQU   SDWA+392
SDWARCPL EQU   SDWA+364
@NM00143 EQU   SDWADPSA+32
SDWATO4  EQU   SDWADPSL+28
SDWAFRM4 EQU   SDWADPSL+24
SDWATO3  EQU   SDWADPSL+20
SDWAFRM3 EQU   SDWADPSL+16
SDWATO2  EQU   SDWADPSL+12
SDWAFRM2 EQU   SDWADPSL+8
SDWATO1  EQU   SDWADPSL+4
SDWAFRM1 EQU   SDWADPSL
@NM00142 EQU   SDWADDAT+3
@NM00141 EQU   SDWAPDAT
SDWAUSPL EQU   SDWAPDAT
SDWADPSW EQU   SDWAPDAT
SDWATJPA EQU   SDWAPDAT
SDWATLPA EQU   SDWAPDAT
SDWADREG EQU   SDWAPDAT
SDWADSAH EQU   SDWAPDAT
SDWADSAS EQU   SDWAPDAT
@NM00140 EQU   SDWADDAT+1
@NM00139 EQU   SDWASDAT
SDWAQQS  EQU   SDWASDAT
SDWACBS  EQU   SDWASDAT
SDWAGTF  EQU   SDWASDAT
SDWASWA  EQU   SDWASDAT
SDWALSQA EQU   SDWASDAT
SDWASQA  EQU   SDWASDAT
SDWANUC  EQU   SDWASDAT
@NM00138 EQU   SDWADUMP+2
@NM00137 EQU   SDWADPFS
SDWASLST EQU   SDWADPFS
@NM00136 EQU   SDWADPFS
SDWADLST EQU   SDWADPFS
SDWADPT  EQU   SDWADPFS
SDWADPID EQU   SDWADUMP
SDWADPLA EQU   SDWA+316
SDWAMODN EQU   SDWARECP
@NM00135 EQU   SDWALKWA+32
SDWATALW EQU   SDWALKWS+28
SDWATDLW EQU   SDWALKWS+24
SDWATNLW EQU   SDWALKWS+20
SDWAAPLW EQU   SDWALKWS+16
SDWAIPLW EQU   SDWALKWS+12
SDWAILLW EQU   SDWALKWS+8
SDWAIULW EQU   SDWALKWS+4
SDWAICLW EQU   SDWALKWS
SDWAFLLK EQU   SDWAACF4
SDWACMS  EQU   SDWAACF4
SDWAOPTM EQU   SDWAACF4
SDWATADB EQU   SDWAACF4
SDWATDNB EQU   SDWAACF4
SDWATNCB EQU   SDWAACF4
SDWAILCH EQU   SDWAACF4
SDWAIUCB EQU   SDWAACF4
SDWAICAT EQU   SDWAACF3
SDWAIPRG EQU   SDWAACF3
SDWASALL EQU   SDWAACF3
SDWAASMP EQU   SDWAACF3
SDWADISP EQU   SDWAACF3
@NM00134 EQU   SDWAACF3
@NM00133 EQU   SDWAACF2
@NM00132 EQU   SDWAACF2
SDWASPIN EQU   SDWAACF2
@NM00131 EQU   SDWAACF2
SDWALCPU EQU   SDWACPUA+2
@NM00130 EQU   SDWACPUA
SDWARECA EQU   SDWA+244
@NM00129 EQU   SDWA+239
@NM00128 EQU   SDWAIOFS
SDWANIOP EQU   SDWAIOFS
SDWANOIO EQU   SDWAIOFS
SDWAIOHT EQU   SDWAIOFS
SDWAIOQR EQU   SDWAIOFS
SDWAFMID EQU   SDWA+236
@NM00127 EQU   SDWAERRD
SDWAMCIV EQU   SDWAERRD
SDWARPIV EQU   SDWAERRD
SDWAMABD EQU   SDWAERRD
SDWACTS  EQU   SDWAERRD
SDWASTAE EQU   SDWAERRD
SDWANRBE EQU   SDWAERRD
@NM00126 EQU   SDWAERRC
SDWAPERC EQU   SDWAERRC
SDWAIRB  EQU   SDWAERRC
SDWASTAI EQU   SDWAERRC
SDWASRBM EQU   SDWAERRB
SDWALDIS EQU   SDWAERRB
SDWAENRB EQU   SDWAERRB
SDWATYP1 EQU   SDWAERRB
@NM00125 EQU   SDWAERRB
SDWASVCE EQU   SDWAERRA
SDWAABTM EQU   SDWAERRA
SDWASVCD EQU   SDWAERRA
SDWARKEY EQU   SDWAERRA
SDWAPCHK EQU   SDWAERRA
SDWATIME EQU   SDWAMCH+20
SDWARFSA EQU   SDWAMCH+16
@NM00124 EQU   SDWAMCH+14
SDWAVEQR EQU   SDWARSR2
SDWAPGFX EQU   SDWARSR2
SDWAFLSQ EQU   SDWARSR2
SDWAFSQA EQU   SDWARSR2
SDWANUCL EQU   SDWARSR2
SDWASPER EQU   SDWARSR2
SDWAINTC EQU   SDWARSR2
SDWAOFLN EQU   SDWARSR2
SDWACHNG EQU   SDWARSR1
SDWAMSER EQU   SDWARSR1
@NM00123 EQU   SDWARSR1
SDWACPID EQU   SDWAMCH+10
SDWATERR EQU   SDWAMCHD
SDWAFPRX EQU   SDWAMCHD
SDWAINSF EQU   SDWAMCHD
SDWAACR  EQU   SDWAMCHD
SDWASCK  EQU   SDWAMCHD
SDWAPSWU EQU   SDWAMCHD
SDWAREGU EQU   SDWAMCHD
SDWASKYF EQU   SDWAMCHD
@NM00122 EQU   SDWAMCHS
SDWARSRF EQU   SDWAMCHS
SDWARSRC EQU   SDWAMCHS
SDWAINVP EQU   SDWAMCHS
SDWATSVL EQU   SDWAMCHS
SDWARCDF EQU   SDWAMCHS
SDWASRVL EQU   SDWAMCHS
SDWASCKE EQU   SDWASTCK+4
SDWASCKB EQU   SDWASTCK
SDWALNTH EQU   SDWAIDNT+1
SDWASPID EQU   SDWAIDNT
SDWASR14 EQU   SDWASRSV+56
SDWASR13 EQU   SDWASRSV+52
SDWASR12 EQU   SDWASRSV+48
SDWASR11 EQU   SDWASRSV+44
SDWASR10 EQU   SDWASRSV+40
SDWASR09 EQU   SDWASRSV+36
SDWASR08 EQU   SDWASRSV+32
SDWASR07 EQU   SDWASRSV+28
SDWASR06 EQU   SDWASRSV+24
SDWASR05 EQU   SDWASRSV+20
SDWASR04 EQU   SDWASRSV+16
SDWASR03 EQU   SDWASRSV+12
SDWASR02 EQU   SDWASRSV+8
SDWASR01 EQU   SDWASRSV+4
SDWASR00 EQU   SDWASRSV
SDWATRN2 EQU   SDWAAEC2+4
SDWAIPC2 EQU   SDWAICD2
SDWAIMC2 EQU   SDWAICD2
SDWAIPR2 EQU   SDWAICD2
@NM00121 EQU   SDWAINC2
@NM00120 EQU   SDWAILC2
SDWAIL2  EQU   SDWAILC2
@NM00119 EQU   SDWAILC2
@NM00118 EQU   SDWAAEC2
SDWAADD2 EQU   SDWANXT2+1
@NM00117 EQU   SDWANXT2
@NM00116 EQU   SDWAEC2+3
SDWASGN2 EQU   SDWAINT2
SDWAEXP2 EQU   SDWAINT2
SDWADEC2 EQU   SDWAINT2
SDWAFPO2 EQU   SDWAINT2
SDWACC2  EQU   SDWAINT2
@NM00115 EQU   SDWAINT2
SDWAPGM2 EQU   SDWAMWP2
SDWAWAT2 EQU   SDWAMWP2
SDWAMCK2 EQU   SDWAMWP2
SDWAECT2 EQU   SDWAMWP2
SDWAKEY2 EQU   SDWAMWP2
SDWAEXT2 EQU   SDWAEMK2
SDWAIO2  EQU   SDWAEMK2
SDWATRM2 EQU   SDWAEMK2
@NM00114 EQU   SDWAEMK2
SDWAPER2 EQU   SDWAEMK2
@NM00113 EQU   SDWAEMK2
SDWATRAN EQU   SDWAAEC1+4
SDWAIPC1 EQU   SDWAICD1
SDWAIMC1 EQU   SDWAICD1
SDWAIPR1 EQU   SDWAICD1
@NM00112 EQU   SDWAINC1
@NM00111 EQU   SDWAILC1
SDWAIL1  EQU   SDWAILC1
@NM00110 EQU   SDWAILC1
@NM00109 EQU   SDWAAEC1
SDWAADD1 EQU   SDWANXT1+1
@NM00108 EQU   SDWANXT1
@NM00107 EQU   SDWAEC1+3
SDWASGN1 EQU   SDWAINT1
SDWAEXP1 EQU   SDWAINT1
SDWADEC1 EQU   SDWAINT1
SDWAFPO1 EQU   SDWAINT1
SDWACC1  EQU   SDWAINT1
@NM00106 EQU   SDWAINT1
SDWAPGM1 EQU   SDWAMWP1
SDWAWAT1 EQU   SDWAMWP1
SDWAMCK1 EQU   SDWAMWP1
SDWAECT1 EQU   SDWAMWP1
SDWAKEY1 EQU   SDWAMWP1
SDWAEXT1 EQU   SDWAEMK1
SDWAIO1  EQU   SDWAEMK1
SDWATRM1 EQU   SDWAEMK1
@NM00105 EQU   SDWAEMK1
SDWAPER1 EQU   SDWAEMK1
@NM00104 EQU   SDWAEMK1
SDWAIOBR EQU   SDWA+100
SDWAEPA  EQU   SDWA+96
@NM00103 EQU   SDWANAME+4
SDWARBAD EQU   SDWANAME
SDWAGR15 EQU   SDWAGRSV+60
SDWAGR14 EQU   SDWAGRSV+56
SDWAGR13 EQU   SDWAGRSV+52
SDWAGR12 EQU   SDWAGRSV+48
SDWAGR11 EQU   SDWAGRSV+44
SDWAGR10 EQU   SDWAGRSV+40
SDWAGR09 EQU   SDWAGRSV+36
SDWAGR08 EQU   SDWAGRSV+32
SDWAGR07 EQU   SDWAGRSV+28
SDWAGR06 EQU   SDWAGRSV+24
SDWAGR05 EQU   SDWAGRSV+20
SDWAGR04 EQU   SDWAGRSV+16
SDWAGR03 EQU   SDWAGRSV+12
SDWAGR02 EQU   SDWAGRSV+8
SDWAGR01 EQU   SDWAGRSV+4
SDWAGR00 EQU   SDWAGRSV
SDWANXTP EQU   SDWACTL2+5
SDWASGP  EQU   SDWAPMKP
SDWAEUP  EQU   SDWAPMKP
SDWADOP  EQU   SDWAPMKP
SDWAFPP  EQU   SDWAPMKP
SDWACCP  EQU   SDWAPMKP
SDWAILP  EQU   SDWAPMKP
SDWAINTP EQU   SDWACTL2+2
SDWASPVP EQU   SDWAMWPP
SDWAWATP EQU   SDWAMWPP
SDWAMCKP EQU   SDWAMWPP
@NM00102 EQU   SDWAMWPP
SDWAKEYP EQU   SDWAMWPP
SDWAEXTP EQU   SDWACMKP
SDWAIOP  EQU   SDWACMKP
SDWANXTA EQU   SDWACTL1+5
SDWASGA  EQU   SDWAPMKA
SDWAEUA  EQU   SDWAPMKA
SDWADOA  EQU   SDWAPMKA
SDWAFPA  EQU   SDWAPMKA
SDWACCA  EQU   SDWAPMKA
SDWAILA  EQU   SDWAPMKA
SDWAINTA EQU   SDWACTL1+2
SDWASPVA EQU   SDWAMWPA
SDWAWATA EQU   SDWAMWPA
SDWAMCKA EQU   SDWAMWPA
@NM00101 EQU   SDWAMWPA
SDWAKEYA EQU   SDWAMWPA
SDWAEXTA EQU   SDWACMKA
SDWAIOA  EQU   SDWACMKA
@NM00100 EQU   SDWACMPF
SDWASTCC EQU   SDWACMPF
@NM00099 EQU   SDWACMPF
SDWASTEP EQU   SDWACMPF
RDXNQARG EQU   RDXENTRY
RDXRSV01 EQU   RDXLIST
SIRBEND  EQU   RBSECT+168
SVRBEND  EQU   RBSECT+168
RBRSV158 EQU   RBRSV150
RBRSV157 EQU   RBRSV150
RBRSV156 EQU   RBRSV150
RBRSV155 EQU   RBRSV150
RBRSV154 EQU   RBRSV150
RBRSV153 EQU   RBRSV150
RBRSV152 EQU   RBRSV150
RBRSV151 EQU   RBRSV150
RBRSV149 EQU   RBSECT+166
RBRSV148 EQU   RBSECT+164
RBSCBB   EQU   RBSECT+144
TIRBEND  EQU   @NM00097+8
PRBEND   EQU   @NM00097+8
RBRSV146 EQU   RBRSV138
RBRSV145 EQU   RBRSV138
RBRSV144 EQU   RBRSV138
RBRSV143 EQU   RBRSV138
RBRSV142 EQU   RBRSV138
RBRSV141 EQU   RBRSV138
RBRSV140 EQU   RBRSV138
RBRSV139 EQU   RBRSV138
RBRSV137 EQU   @NM00097+6
RBRSV136 EQU   @NM00097+4
RBRSV135 EQU   @NM00097
@NM00096 EQU   RBSECT+32
@NM00095 EQU   @NM00093+1
@NM00094 EQU   @NM00093
RBPGMQ1  EQU   RBPGMQ+1
@NM00092 EQU   RBPGMQ
@NM00091 EQU   RBSECT+16
@NM00090 EQU   @NM00086
@NM00089 EQU   @NM00086
RBIQETP  EQU   @NM00086
RBUSIQE  EQU   RBETXR
RBATTN   EQU   @NM00086
@NM00088 EQU   @NM00086
@NM00087 EQU   @NM00086
RBPMSVRB EQU   @NM00084
RBATNXIT EQU   @NM00084
@NM00085 EQU   @NM00084
RBWAITP  EQU   @NM00084
RBFNSVRB EQU   RBTRSVRB
RBFTP    EQU   @NM00084
RBSIZE   EQU   RBSECT+8
RBABOPSW EQU   RBSECT+4
RBPPSAV1 EQU   RBPPSAV+1
@NM00082 EQU   RBPPSAV
RBPRFXND EQU   RBPREFIX+64
RBINTCOD EQU   RBINTCDA+1
RBINLNTH EQU   RBINTCDA
RBWCSA   EQU   RBPREFIX+60
RBRSV004 EQU   RBPREFIX+57
RBSSSYN  EQU   RBFLAGS1
RBSCB    EQU   RBFLAGS1
RBLONGWT EQU   RBFLAGS1
RBASIR   EQU   RBFLAGS1
RBRSV159 EQU   RBFLAGS1
RBABEND  EQU   RBFLAGS1
RBXWAIT  EQU   RBFLAGS1
RBSLOCK  EQU   RBFLAGS1
RBRTRAN  EQU   RBRTPSW2+4
RBRTINCD EQU   RBRTICIL+2
RBRTILC  EQU   RBRTICIL+1
RBRSV160 EQU   RBRTICIL
RBRTPSW1 EQU   RBRTOPSW
RBRSV062 EQU   RBRSV054
RBRSV061 EQU   RBRSV054
RBRSV060 EQU   RBRSV054
RBRSV059 EQU   RBRSV054
RBRSV058 EQU   RBRSV054
RBRSV057 EQU   RBRSV054
RBRSV056 EQU   RBRSV054
RBRSV055 EQU   RBRSV054
RBRSV053 EQU   RBPREFIX+38
RBRSV052 EQU   RBPREFIX+36
RBRSV051 EQU   RBPREFIX+32
RBPRFXST EQU   RBPREFIX+32
RBRSV050 EQU   RBPREFIX+28
RBRSV049 EQU   RBRSV041
RBRSV048 EQU   RBRSV041
RBRSV047 EQU   RBRSV041
RBRSV046 EQU   RBRSV041
RBRSV045 EQU   RBRSV041
RBRSV044 EQU   RBRSV041
RBRSV043 EQU   RBRSV041
RBRSV042 EQU   RBRSV041
RBRSV040 EQU   RBPREFIX+26
RBRSV039 EQU   RBPREFIX+24
RBRSV038 EQU   RBPREFIX+20
RBRSV037 EQU   RBPREFIX+16
RBRSV036 EQU   RBRSV028
RBRSV035 EQU   RBRSV028
RBRSV034 EQU   RBRSV028
RBRSV033 EQU   RBRSV028
RBRSV032 EQU   RBRSV028
RBRSV031 EQU   RBRSV028
RBRSV030 EQU   RBRSV028
RBRSV029 EQU   RBRSV028
RBRSV027 EQU   RBRSV019
RBRSV026 EQU   RBRSV019
RBRSV025 EQU   RBRSV019
RBRSV024 EQU   RBRSV019
RBRSV023 EQU   RBRSV019
RBRSV022 EQU   RBRSV019
RBRSV021 EQU   RBRSV019
RBRSV020 EQU   RBRSV019
RBRSV018 EQU   RBPREFIX+13
RBRSV017 EQU   RBPREFIX+12
RBSRV016 EQU   RBPREFIX+11
RBRSV015 EQU   RBPREFIX+10
RBRSV014 EQU   RBPREFIX+8
RBRSV013 EQU   RBPREFIX+4
RBRSV012 EQU   RBPREFIX
XRBREG15 EQU   RBGRS15
XRBREG14 EQU   RBGRS14
XRBREG13 EQU   RBGRS13
XRBREG12 EQU   RBGRS12
XRBREG11 EQU   RBGRS11
XRBREG10 EQU   RBGRS10
XRBREG9  EQU   RBGRS9
XRBREG8  EQU   RBGRS8
XRBREG7  EQU   RBGRS7
XRBREG6  EQU   RBGRS6
XRBREG5  EQU   RBGRS5
XRBREG4  EQU   RBGRS4
XRBREG3  EQU   RBGRS3
XRBREG2  EQU   RBGRS2
XRBREG1  EQU   RBGRS1
XRBREG0  EQU   RBGRS0
XRBLNKA  EQU   RBLINKB
XRBWT    EQU   RBWCF
@NM00081 EQU   RBBASIC+24
RBOPSWPS EQU   RBOPSWB2
@NM00080 EQU   RBOPSWB2
@NM00079 EQU   XRBPSW
@NM00078 EQU   RBBASIC+12
XRBWAIT  EQU   RBECBWT
XRBFRRB  EQU   RBFDYN
@NM00077 EQU   XSTAB2
XRBACTV  EQU   RBFACTV
XRBTCBP  EQU   RBTCBNXT
@NM00076 EQU   XSTAB1
XRBCKPT  EQU   RBFTCKPT
@NM00075 EQU   XSTAB1
@NM00074 EQU   RBBASIC+8
@NM00073 EQU   RBEXRTNM+1
RBTMIND3 EQU   RBTMFLD
RBTMIND2 EQU   RBTMFLD
RBTMCMP  EQU   RBTMFLD
RBWLIM   EQU   RBTMFLD
RBRSV005 EQU   RBTMFLD
RBTMTOD  EQU   RBTMFLD
RBTMQUE  EQU   RBTMFLD
@NM00072 EQU   RBPRFX
IOBERRCT EQU   IOBSTDRD+30
IOBRSV19 EQU   IOBINCAM+1
IOBOLTST EQU   IOBBTAMF
IOBRFTMG EQU   IOBBTAMF
IOBRSV17 EQU   IOBBTAMF
IOBRSV16 EQU   IOBBTAMF
IOBRSV15 EQU   IOBBTAMF
IOBRSV14 EQU   IOBBTAMF
IOBINUSE EQU   IOBBTAMF
IOBPRMER EQU   IOBBTAMF
IOBRSTRB EQU   IOBRESTR+1
IOBREPOS EQU   IOBRESTR
IOBDCBPB EQU   IOBDCBPT+1
IOBRSV44 EQU   IOBFLAG4
IOBJES3I EQU   IOBFLAG4
IOBRSV42 EQU   IOBFLAG4
IOBRSV41 EQU   IOBFLAG4
IOBRSV40 EQU   IOBFLAG4
IOBPMERR EQU   IOBFLAG4
IOBCC3WE EQU   IOBFLAG4
IOBGDPOL EQU   IOBFLAG4
IOBSTRTB EQU   IOBSTART+1
IOBSIOCC EQU   IOBSTART
IOBCSB7  EQU   IOBCSTAT
IOBCSB6  EQU   IOBCSTAT
IOBCSB5  EQU   IOBCSTAT
IOBCSB4  EQU   IOBCSTAT
IOBCSB3  EQU   IOBCSTAT
IOBCSB2  EQU   IOBCSTAT
IOBCSB1  EQU   IOBCSTAT
IOBCSB0  EQU   IOBCSTAT
IOBUSB7  EQU   IOBUSTAT
IOBUSB6  EQU   IOBUSTAT
IOBUSB5  EQU   IOBUSTAT
IOBUSB4  EQU   IOBUSTAT
IOBUSB3  EQU   IOBUSTAT
IOBUSB2  EQU   IOBUSTAT
IOBUSB1  EQU   IOBUSTAT
IOBUSB0  EQU   IOBUSTAT
IOBCMDA  EQU   IOBIOCSW
IOBLOG   EQU   IOBFL3
IOBJAM   EQU   IOBICL
IOBMSG   EQU   IOBFL3
IOBSDR   EQU   IOBCNC
IOBACU   EQU   IOBFL3
IOBCDC   EQU   IOBFL3
IOBICC   EQU   IOBFL3
IOBCCC   EQU   IOBFL3
IOBECBPB EQU   IOBECBPT+1
IOBECBCC EQU   IOBECBPT
IOBS1B7  EQU   IOBSENS1
IOBS1B6  EQU   IOBSENS1
IOBS1B5  EQU   IOBSENS1
IOBS1B4  EQU   IOBSENS1
IOBS1B3  EQU   IOBSENS1
IOBS1B2  EQU   IOBSENS1
IOBS1B1  EQU   IOBSENS1
IOBS1B0  EQU   IOBSENS1
IOBSNSC9 EQU   IOBS0B7
IOBS0B6  EQU   IOBSENS0
IOBS0B5  EQU   IOBSENS0
IOBS0B4  EQU   IOBSENS0
IOBS0B3  EQU   IOBSENS0
IOBS0B2  EQU   IOBSENS0
IOBS0B1  EQU   IOBSENS0
IOBS0B0  EQU   IOBSENS0
IOBPNCH  EQU   IOBFLAG2
IOBSTATO EQU   IOBFLAG2
IOBSKUPD EQU   IOBFLAG2
IOBALTTR EQU   IOBFLAG2
IOBRRT2  EQU   IOBRDHA0
IOBRRT3  EQU   IOBPURGE
IOBSENSE EQU   IOBFLAG2
IOBHALT  EQU   IOBFLAG2
IOBSPSVC EQU   IOBRSTRT
IOBUNREL EQU   IOBFLAG1
IOBIOERR EQU   IOBFLAG1
IOBFCREX EQU   IOBCYCCK
IOBRPSTN EQU   IOBFLAG1
IOBERRTN EQU   IOBFLAG1
IOBCMDCH EQU   IOBFLAG1
IOBDATCH EQU   IOBFLAG1
IOBCNOPA EQU   IOBPREFX+12
IOBCICB  EQU   IOBPREFX+8
IOBCECB  EQU   IOBPREFX+4
IOBCONOP EQU   IOBPREFX+3
IOBCINOP EQU   IOBPREFX+2
IOBRSV05 EQU   IOBPREFX+1
IOBPCI   EQU   IOBCFLG1
IOBRSTCH EQU   IOBCFLG1
IOBABAPP EQU   IOBCFLG1
IOBPTST  EQU   IOBCFLG1
IOBRSV04 EQU   IOBCFLG1
IOBRSV03 EQU   IOBCFLG1
IOBRSV02 EQU   IOBCFLG1
IOBRSV01 EQU   IOBCFLG1
DSABGRID EQU   DSABGIDL+4
DSABLLEN EQU   DSABGIDL
DSABANAM EQU   DSABANMI+1
DSABANML EQU   DSABANMI
DSABRS04 EQU   DSAB+68
DSABSIOT EQU   DSAB+64
DSABRS03 EQU   DSAB+62
DSABDCBM EQU   DSAB+56
DSABSSCM EQU   DSAB+52
DSABSSNM EQU   DSAB+48
DSABPTTR EQU   DSAB+44
DSABTCBP EQU   DSAB+40
DSABRS02 EQU   DSAB+38
@NM00047 EQU   DSABFLG4
DSABCKSI EQU   DSABFLG4
DSABCKVL EQU   DSABFLG4
DSABCKDS EQU   DSABFLG4
@NM00046 EQU   DSABFLG3
DSABJSCT EQU   DSABFLG3
DSABCATL EQU   DSABFLG3
DSABVMSC EQU   DSABFLG3
DSABVAM  EQU   DSABFLG3
DSABPASS EQU   DSABFLG3
DSABDEFR EQU   DSABFLG3
@NM00045 EQU   DSABFLG2
DSABUNAL EQU   DSABFLG2
DSABIRM  EQU   DSABFLG2
DSABOPEN EQU   DSABFLG2
DSABNUSE EQU   DSABFLG1
DSABCATM EQU   DSABFLG1
DSABPCAT EQU   DSABFLG1
DSABDCAT EQU   DSABFLG1
DSABCONV EQU   DSABFLG1
DSABDCNV EQU   DSABFLG1
DSABPALC EQU   DSABFLG1
DSABDALC EQU   DSABFLG1
@NM00044 EQU   DSABORG2
DSABTR   EQU   DSABORG2
DSABAM   EQU   DSABORG2
@NM00043 EQU   DSABORG2
DSABTQ   EQU   DSABORG2
DSABTX   EQU   DSABORG2
DSABGS   EQU   DSABORG2
DSABU    EQU   DSABORG1
DSABPO   EQU   DSABORG1
DSABMQ   EQU   DSABORG1
DSABCQ   EQU   DSABORG1
DSABCX   EQU   DSABORG1
DSABDA   EQU   DSABORG1
DSABPS   EQU   DSABORG1
DSABIS   EQU   DSABORG1
DSABSSVA EQU   DSAB+21
DSABRS01 EQU   DSAB+20
DSABOPCT EQU   DSAB+14
DSABLNTH EQU   DSAB+12
DSABBCHN EQU   DSAB+8
DSABFCHN EQU   DSAB+4
DSABID   EQU   DSAB
DECWAREA EQU   DECBS04+44
DECWLNG  EQU   DECBS04+42
DECRSV56 EQU   DECBS04+40
DECPOLPA EQU   DECPOLPT+1
DECNDXPL EQU   DECPOLPT
DECADRPT EQU   DECBS04+32
DECCSWST EQU   DECBS04+30
DECRSV55 EQU   DECERRST
DECRSV54 EQU   DECERRST
DECRSV53 EQU   DECERRST
DECDSABL EQU   DECERRST
DECDIAGN EQU   DECERRST
DECERPER EQU   DECERRST
DECUNDEF EQU   DECERRST
DECSIO3  EQU   DECERRST
DECTPCOD EQU   DECBS04+28
DECRESPN EQU   DECBS04+26
DECRLN   EQU   DECBS04+25
DECSTXNQ EQU   DECFLAGS
DECSSMSG EQU   DECREVRS
DECNEGRP EQU   DECFLAGS
DECNOBUF EQU   DECFLAGS
DECBADID EQU   DECFLAGS
DECALTAC EQU   DECFLAGS
DECDIFAC EQU   DECWACK
DECERRMS EQU   DECWACK
DECENTRA EQU   DECENTRY+1
DECCMCOD EQU   DECENTRY
DECCOUNT EQU   DECBS04+18
DECSENS1 EQU   DECBS04+17
DECSENS0 EQU   DECBS04+16
@NM00042 EQU   DECBS04+12
DECDCBAA EQU   @NM00041+1
DECRSV52 EQU   DECONLTT
DECRSV51 EQU   DECONLTT
DECRSV50 EQU   DECONLTT
DECRSV49 EQU   DECONLTT
DECRSV48 EQU   DECONLTT
DECRSV47 EQU   DECONLTT
DECRCVMS EQU   DECONLTT
DECONLTS EQU   DECONLTT
@NM00040 EQU   DECBS04+6
@NM00039 EQU   @NM00037+1
DECSLNTH EQU   @NM00038
DECSAREA EQU   @NM00038
DECSTRME EQU   @NM00038
DECRSV46 EQU   @NM00038
DECRSV45 EQU   @NM00038
DECRSV44 EQU   @NM00038
DECRSV43 EQU   @NM00038
DECRDAPL EQU   @NM00038
@NM00036 EQU   DECBS04
@NM00035 EQU   @NM00034+1
DECRSV42 EQU   @NM00034
DECADRCH EQU   DECBS03+32
DECRSV41 EQU   DECBS03+28
DECRSV40 EQU   DECBS03+26
DECUCBDX EQU   DECBS03+25
DECRSV39 EQU   DECBS03+24
DECCPOLA EQU   DECCPOL+1
DECNMMSG EQU   DECCPOL
DECRSV38 EQU   DECBS03+16
@NM00033 EQU   DECBS03+12
@NM00032 EQU   DECBS03+8
@NM00031 EQU   DECBS03+6
DECOPCOD EQU   DECBS03+5
DECRSV37 EQU   DECBS03+4
LINEDECB EQU   DECBS03
DECNAA   EQU   DECNA+1
DECRSV36 EQU   DECNA
DECRECPT EQU   DECBS02+24
DECKYADR EQU   DECBS02+20
@NM00030 EQU   DECBS02+16
@NM00029 EQU   DECBS02+12
@NM00028 EQU   DECBS02+8
@NM00027 EQU   DECBS02+6
DECRSV35 EQU   @NM00026
DECWRADD EQU   @NM00026
DECSRKEY EQU   @NM00026
DECOPRD  EQU   @NM00026
DECTYPR  EQU   DECTYPRU
DECTYPU  EQU   DECTYPRU
DECBKLNS EQU   @NM00026
DECKEYS  EQU   @NM00026
DECRELBA EQU   @NM00025
DECRDEX  EQU   @NM00025
DECDYNBF EQU   @NM00025
DECACTAD EQU   @NM00025
DECFDBCK EQU   @NM00025
DECEXSRC EQU   @NM00025
DECOFLOW EQU   @NM00025
DECVERFY EQU   @NM00025
DECRSV34 EQU   @NM00022+3
DECCCKFF EQU   DECCC3
DECCCOPT EQU   DECCC3
DECCCSDK EQU   DECCC3
DECCCWDI EQU   DECCC3
DECCCNBK EQU   DECCC3
DECCCEXS EQU   DECCC3
DECCCWRI EQU   DECCC3
DECRSV33 EQU   DECCC3
DECCCREX EQU   DECCC2
DECCCUER EQU   DECCC2
DECCCEOD EQU   DECCC2
DECCCERR EQU   DECCC2
DECCCINV EQU   DECCC2
DECCCNSP EQU   DECCC2
DECCCRLC EQU   DECCC2
DECCCRNF EQU   DECCC2
@NM00023 EQU   @NM00022
DECEXRKU EQU   DECBEXC2
DECEXASR EQU   DECBEXC2
DECRSV32 EQU   DECBEXC2
DECRSV31 EQU   DECBEXC2
DECRSV30 EQU   DECBEXC2
DECRSV29 EQU   DECBEXC2
DECRSV28 EQU   DECBEXC2
DECRSV27 EQU   DECBEXC2
DECEXDUP EQU   DECBEXC1
DECEXOFL EQU   DECBEXC1
DECEXUBK EQU   DECBEXC1
DECEXERR EQU   DECBEXC1
DECEXINV EQU   DECBEXC1
DECEXNSP EQU   DECBEXC1
DECEXRLC EQU   DECBEXC1
DECEXRNF EQU   DECBEXC1
DECBKEY  EQU   DECBS01+20
DECBLOGR EQU   DECBS01+16
DECBAREA EQU   DECBS01+12
DECBDCBA EQU   DECBS01+8
DECBLGTH EQU   DECBS01+6
DECRSV26 EQU   DECBTYP2
DECRSV25 EQU   DECBTYP2
DECBWRKN EQU   DECBTYP2
DECBWRK  EQU   DECBTYP2
DECRSV24 EQU   DECBTYP2
DECBRDKU EQU   DECBTYP2
DECRSV23 EQU   DECBTYP2
DECBRDK  EQU   DECBTYP2
DECBARS  EQU   DECBTYP1
DECBLNS  EQU   DECBTYP1
DECRSV22 EQU   DECBTYP1
DECRSV21 EQU   DECBTYP1
DECRSV20 EQU   DECBTYP1
DECRSV19 EQU   DECBTYP1
DECRSV18 EQU   DECBTYP1
DECRSV17 EQU   DECBTYP1
DECBRBA  EQU   DECBRB+1
DECRSV16 EQU   DECBECBF
DECRSV15 EQU   DECBECBF
DECRSV14 EQU   DECBECBF
DECRSV13 EQU   DECBECBF
DECRSV12 EQU   DECBECBF
DECRSV11 EQU   DECBECBF
DECBPOST EQU   DECBECBF
DECBWAIT EQU   DECBECBF
DECNEXTA EQU   DECNEXT+1
DECRSV10 EQU   DECNEXT
DECIOBPT EQU   DECB+16
DECAREA  EQU   DECB+12
DECDCBAD EQU   DECB+8
DECLNGTH EQU   DECBPTR+2
DECWRSFR EQU   DECTYPE2
DECRSV09 EQU   DECTYPE2
DECWRSZ  EQU   DECTYPE2
DECRSV08 EQU   DECTYPE2
DECWRSD  EQU   DECTYPE2
DECWRSF  EQU   DECTYPE2
DECRDSB  EQU   DECTYPE2
DECRDSF  EQU   DECTYPE2
DECRSV07 EQU   DECTYPE1
DECRSV06 EQU   DECTYPE1
DECRSV05 EQU   DECTYPE1
DECRSV04 EQU   DECTYPE1
DECRSV03 EQU   DECTYPE1
DECRSV02 EQU   DECTYPE1
DECRSV01 EQU   DECTYPE1
DECLNS   EQU   DECTYPE1
DECSDECB EQU   DECB
DEBXSAMB EQU   DEBXTN+28
DEBXOPNJ EQU   DEBXTN+24
DEBXMCF2 EQU   DEBXTN+19
DEBXMCF1 EQU   DEBXTN+18
DEBXDSO2 EQU   DEBXTN+17
DEBXDSO1 EQU   DEBXTN+16
DEBXDBPR EQU   DEBXTN+12
DEBXDCBM EQU   DEBXTN+8
DEBXRV48 EQU   DEBXTN+3
DEBXRV47 EQU   DEBXFLG1
DEBXRV46 EQU   DEBXFLG1
DEBXRV45 EQU   DEBXFLG1
DEBXRV44 EQU   DEBXFLG1
DEBXRV43 EQU   DEBXFLG1
DEBXDSSI EQU   DEBXFLG1
DEBXTSKC EQU   DEBXFLG1
DEBXCDCB EQU   DEBXFLG1
DEBXLNGH EQU   DEBXTN
DEBAMTYP EQU   DEBPREFX+13
DEBLNGTH EQU   DEBPREFX+12
DEBDCBMK EQU   DEBXTNP
DEBDSCBA EQU   DEBPREFX+1
DEBWKARA EQU   DEBPREFX
DEBXCEAD EQU   DEBXCEA+1
DEBXCENP EQU   DEBXCEAB
DEBRSV35 EQU   DEBXCEAB
DEBRSV34 EQU   DEBXCEAB
DEBRSV33 EQU   DEBXCEAB
DEBRSV32 EQU   DEBXCEAB
DEBCEAD  EQU   DEBCEA+1
DEBCENP  EQU   DEBCEAB
DEBRSV31 EQU   DEBCEAB
DEBRSV30 EQU   DEBCEAB
DEBRSV29 EQU   DEBCEAB
DEBRSV28 EQU   DEBCEAB
DEBPCIAD EQU   DEBPCIA+1
DEBPCINP EQU   DEBPCIAB
DEBRSV27 EQU   DEBPCIAB
DEBRSV26 EQU   DEBPCIAB
DEBRSV25 EQU   DEBPCIAB
DEBRSV24 EQU   DEBPCIAB
DEBSIOAD EQU   DEBSIOA+1
DEBSIONP EQU   DEBSIOAB
DEBFIX   EQU   DEBSIOAB
DEBIOVR  EQU   DEBSIOAB
DEBSIOX  EQU   DEBSIOAB
DEBPGFX  EQU   DEBSIOAB
DEBEOEAD EQU   DEBEOEA+1
DEBEOENP EQU   DEBEOEAB
DEBRSV22 EQU   DEBEOEAB
DEBRSV21 EQU   DEBEOEAB
DEBRSV20 EQU   DEBEOEAB
DEBESMVR EQU   DEBEOEAB
DEBSUBID EQU   DEBSUBNM
DEBSIOA2 EQU   DEBACS08+24
DEBRPSIO EQU   DEBACS08+20
DEBFREED EQU   DEBACS08+16
@NM00021 EQU   DEBACS08+12
@NM00020 EQU   DEBACS08+8
DEBDISAD EQU   DEBACS08+4
@NM00019 EQU   DEBACS08
DEBRPSST EQU   DEBACS07+64
DEBARECK EQU   DEBACS07+60
DEBAREWT EQU   DEBACS07+56
DEBACHK  EQU   DEBACS07+52
DEBAWRIT EQU   DEBACS07+48
DEBASETL EQU   DEBACS07+44
DEBAREAD EQU   DEBACS07+40
DEBCRECK EQU   DEBACS07+36
DEBCREWT EQU   DEBACS07+32
DEBCCHK  EQU   DEBACS07+28
DEBCWRIT EQU   DEBACS07+24
DEBCSETL EQU   DEBACS07+20
DEBCREAD EQU   DEBACS07+16
DEBWKPT5 EQU   DEBACS07+12
DEBWKPT4 EQU   DEBACS07+8
DEBGET   EQU   DEBACS07+4
@NM00018 EQU   DEBACS07
DEBPUT   EQU   DEBACS06+4
DEBDCBFA EQU   DEBACS06
DEBLUCBB EQU   DEBLUCBA+1
DEBRSV18 EQU   DEBLUCBA
DEBFUCBB EQU   DEBFUCBA+1
DEBRSV17 EQU   DEBFUCBA
DEBTCCWB EQU   DEBTCCWA+1
DEBRSV16 EQU   DEBTCCWA
DEBTBFRB EQU   DEBTBFRA+1
DEBRSV15 EQU   DEBTBFRA
DEBDBPEF EQU   DEBACS03+8
DEBDBPP  EQU   DEBACS03+4
DEBDTPP  EQU   DEBACS03
DEBDBPE  EQU   DEBDBLK+1
DEBDBPT  EQU   DEBDBLK
DEBEXTNM EQU   DEBDSNAM
DEBLRECL EQU   DEBACSMD+14
DEBBLKSI EQU   DEBACSMD+12
DEBRSV14 EQU   DEBDSNM+4
DEBUTSAB EQU   DEBUTSAA+1
DEBRSV13 EQU   DEBUTSAA
DEBVOLNM EQU   DEBACSMD+2
DEBVLSEQ EQU   DEBVOLSQ+1
DEBRSV42 EQU   DEBVOLBT
DEBRSV41 EQU   DEBVOLBT
DEBRSV40 EQU   DEBVOLBT
DEBRSV39 EQU   DEBVOLBT
DEBRSV38 EQU   DEBVOLBT
DEBRSV37 EQU   DEBVOLBT
DEBRSV36 EQU   DEBVOLBT
DEBEXFUL EQU   DEBVOLBT
DEBNMTRK EQU   DEBDASD+14
DEBENDHH EQU   DEBDASD+12
DEBENDCC EQU   DEBDASD+10
DEBSTRHH EQU   DEBDASD+8
DEBSTRCC EQU   DEBDASD+6
DEBBINUM EQU   DEBDASD+4
DEBUCBA  EQU   DEBUCBAD+1
DEBDVMOD EQU   DEBUCBAD
DEBAPPB  EQU   DEBAPPAD+1
DEBEXSCL EQU   DEBAPPAD
DEBDEBID EQU   DEBPROTG
DEBECBB  EQU   DEBECBAD+1
DEBPRIOR EQU   DEBECBAD
DEBUSRPB EQU   DEBUSRPG+1
DEBNMEXT EQU   DEBUSRPG
DEBRSV05 EQU   DEBBASIC+15
DEBXTNIN EQU   DEBFLGS1
DEBAPFIN EQU   DEBFLGS1
DEBF1CEV EQU   DEBFLGS1
DEBCINDI EQU   DEBFLGS1
DEBRSV02 EQU   DEBFLGS1
DEBRSIOA EQU   DEBFLGS1
DEBEOFDF EQU   DEBFLGS1
DEBPWCKD EQU   DEBFLGS1
DEBQSCNT EQU   DEBBASIC+13
DEBACCS  EQU   DEBOPATB
DEBPOSIT EQU   DEBOPATB
DEBZERO  EQU   DEBOPATB
DEBABEND EQU   DEBOPATB
DEBIRBB  EQU   DEBIRBAD+1
DEBRERR  EQU   DEBOFLGS
DEBLABEL EQU   DEBOFLGS
DEBSPLIT EQU   DEBOFLGS
DEBDCB   EQU   DEBOFLGS
DEBRLSE  EQU   DEBOFLGS
DEBEOF   EQU   DEBOFLGS
DEBDISP  EQU   DEBOFLGS
DEBDEBB  EQU   DEBDEBAD+1
DEBAMLNG EQU   DEBDEBAD
DEBNMSUB EQU   DEBTCBAD
DCBDYNB  EQU   IHADCS30+48
DCBDFBK  EQU   IHADCS30+44
DCBDFOR  EQU   IHADCS30+40
DCBDRDXA EQU   DCBDRDX+1
DCBMVXNO EQU   DCBDRDX
DCBXARGA EQU   DCBXARG+1
DCBXCNT  EQU   DCBXARG
DCBLIMCT EQU   IHADCS30+29
@NM00017 EQU   IHADCS30+28
DCBUQND  EQU   IHADCS30+24
DCBIOBUQ EQU   IHADCS30+20
DCBSQND  EQU   IHADCS30+16
DCBIOBSQ EQU   IHADCS30+12
DCBBLKSI EQU   IHADCS30+10
@NM00016 EQU   IHADCS30+8
DCBSYNAD EQU   IHADCS30+4
DCBCHCKA EQU   DCBCHECK+1
DCBOPTRB EQU   DCBOPTCD
DCBOPTRE EQU   DCBOPTCD
DCBOPTDB EQU   DCBOPTCD
DCBOPTA  EQU   DCBOPTCD
DCBOPTF  EQU   DCBOPTCD
DCBOPTE  EQU   DCBOPTCD
DCBOPTTO EQU   DCBOPTCD
DCBOPTW  EQU   DCBOPTCD
DCBREADA EQU   DCBWRITA
@NM00015 EQU   DCBREAD
@NM00014 EQU   DCBIFLGS
DCBIFIOE EQU   DCBIFLGS
DCBCH12  EQU   DCBIFPCT
DCBCH9   EQU   DCBIFPCT
DCBIFEC  EQU   DCBIFLGS
DCBMFSTI EQU   DCBMFDMD
DCBMFAWR EQU   DCBMFSTK
DCBMFUIP EQU   DCBMFTMD
DCBMFIDW EQU   DCBMFLCP
DCBMFWRK EQU   DCBMFMVP
DCBMFRDQ EQU   DCBMFWRT
DCBMFGTQ EQU   DCBMFPUT
DCBMFSTL EQU   DCBMACF2
DCBMFCK  EQU   DCBMFDMG
DCBMFRDX EQU   DCBMFCHK
DCBMFDBF EQU   DCBMFSBG
DCBMFRDI EQU   DCBMFLCG
DCBMFRDK EQU   DCBMFMVG
DCBMFWRQ EQU   DCBMFRD
DCBMFPTQ EQU   DCBMFGET
DCBMFECP EQU   DCBMACF1
DCBTIOT  EQU   IHADCS25
DCBMRSTI EQU   DCBMRDMD
DCBMRAWR EQU   DCBMRSTK
DCBMRUIP EQU   DCBMRTMD
DCBMRIDW EQU   DCBMRLCP
DCBMRWRK EQU   DCBMRMVP
DCBMRRDQ EQU   DCBMRWRT
DCBMRGTQ EQU   DCBMRPUT
DCBMRSTL EQU   DCBMACR2
DCBPGFXA EQU   DCBMRRDX
DCBMRDBF EQU   DCBMRSBG
DCBMRRDI EQU   DCBMRLCG
DCBMRRDK EQU   DCBMRMVG
DCBMRWRQ EQU   DCBMRRD
DCBMRPTQ EQU   DCBMRGET
DCBMRECP EQU   DCBMACR1
@NM00013 EQU   DCBIFLG
DCBIBIOE EQU   DCBIFLG
DCBICH12 EQU   DCBIBPCT
DCBICH9  EQU   DCBIBPCT
DCBIBEC  EQU   DCBIFLG
DCBOFIOF EQU   DCBOFLGS
DCBOFUEX EQU   DCBOFLGS
DCBOFTM  EQU   DCBOFLGS
DCBOFPPC EQU   DCBOFLGS
DCBOFOPN EQU   DCBOFLGS
DCBOFEOV EQU   DCBOFLGS
DCBOFLRB EQU   DCBOFLGS
DCBOFIOD EQU   DCBOFLWR
DCBDDNAM EQU   IHADCS24
DCBEXLSA EQU   DCBEXLST+1
DCBRECKL EQU   DCBRECFM
DCBRECCC EQU   DCBRECFM
DCBRECSB EQU   DCBRECFM
DCBRECBR EQU   DCBRECFM
DCBRECTO EQU   DCBRECLA
DCBRECL  EQU   DCBRECLA
DCBEODA  EQU   DCBEODAD+1
DCBBFA   EQU   DCBBFTEK
DCBH0    EQU   DCBBFTEK
DCBBFTKD EQU   DCBBFTEK
DCBBFTE  EQU   DCBBFT
DCBBFTKR EQU   DCBBFT
DCBBFTS  EQU   DCBBFT
DCBH1    EQU   DCBBFTEK
DCBSVCXA EQU   DCBSVCXL+1
@NM00012 EQU   DCBSVCXL
DCBODEBA EQU   DCBIOBAA
@NM00011 EQU   DCBQSLM
DCBUPDBT EQU   DCBQSLM
DCBUPDCM EQU   DCBQSLM
DCB1DVDS EQU   DCBQSLM
@NM00010 EQU   DCBDSRG2
DCBDSGTR EQU   DCBDSRG2
DCBACBM  EQU   DCBDSRG2
@NM00009 EQU   DCBDSRG2
DCBDSGTQ EQU   DCBDSRG2
DCBDSGTX EQU   DCBDSRG2
DCBDSGGS EQU   DCBDSRG2
DCBDSGU  EQU   DCBDSRG1
DCBDSGPO EQU   DCBDSRG1
@NM00008 EQU   DCBDSRG1
@NM00007 EQU   DCBDSRG1
DCBDSGCX EQU   DCBDSRG1
DCBDSGDA EQU   DCBDSRG1
DCBDSGPS EQU   DCBDSRG1
DCBDSGIS EQU   DCBDSRG1
DCBBUFL  EQU   IHADCS11+8
DCBBUFCA EQU   DCBBUFCB+1
DCBBUFNO EQU   DCBBUFCB
DCBDEVT  EQU   DCBREL
DCBKEYLE EQU   DCBRELB
DCBTRBAL EQU   IHADCS01+6
@NM00006 EQU   IHADCS01+5
@NM00005 EQU   IHADCS01+4
DCBDVTBA EQU   DCBDVTBL+1
@NM00004 EQU   DCBDVTBL
DCBKEYCN EQU   IHADCS00+4
DCBRELAD EQU   IHADCS00
@NM00003 EQU   IHADCB
CVTLEVL  EQU   CVTRELNO+2
CVTNUMB  EQU   CVTRELNO
CVTMDL   EQU   CVTFIX+250
@NM00002 EQU   CVTFIX+248
@NM00001 EQU   CVTFIX
CVTRV482 EQU   CVTXTNT2+128
CVTRV481 EQU   CVTXTNT2+124
CVTRV480 EQU   CVTXTNT2+120
CVTRV479 EQU   CVTXTNT2+118
CVTRV478 EQU   CVTXTNT2+117
CVTRV477 EQU   CVTXTNT2+116
CVTRV476 EQU   CVTXTNT2+115
CVTRV475 EQU   CVTXTNT2+114
CVTRV474 EQU   CVTRV466
CVTRV473 EQU   CVTRV466
CVTRV472 EQU   CVTRV466
CVTRV471 EQU   CVTRV466
CVTRV470 EQU   CVTRV466
CVTRV469 EQU   CVTRV466
CVTRV468 EQU   CVTRV466
CVTRV467 EQU   CVTRV466
CVTRV465 EQU   CVTRV457
CVTRV464 EQU   CVTRV457
CVTRV463 EQU   CVTRV457
CVTRV462 EQU   CVTRV457
CVTRV461 EQU   CVTRV457
CVTRV460 EQU   CVTRV457
CVTRV459 EQU   CVTRV457
CVTRV458 EQU   CVTRV457
CVTRV456 EQU   CVTXTNT2+108
CVTRV455 EQU   CVTXTNT2+104
CVTRV454 EQU   CVTXTNT2+100
CVTRV453 EQU   CVTXTNT2+96
CVTRV452 EQU   CVTXTNT2+94
CVTRV451 EQU   CVTXTNT2+92
CVTRV450 EQU   CVTXTNT2+90
CVTRV449 EQU   CVTXTNT2+88
CVTRV448 EQU   CVTXTNT2+87
CVTRV447 EQU   CVTXTNT2+86
CVTRV446 EQU   CVTRV438
CVTRV445 EQU   CVTRV438
CVTRV444 EQU   CVTRV438
CVTRV443 EQU   CVTRV438
CVTRV442 EQU   CVTRV438
CVTRV441 EQU   CVTRV438
CVTRV440 EQU   CVTRV438
CVTRV439 EQU   CVTRV438
CVTRV437 EQU   CVTRV429
CVTRV436 EQU   CVTRV429
CVTRV435 EQU   CVTRV429
CVTRV434 EQU   CVTRV429
CVTRV433 EQU   CVTRV429
CVTRV432 EQU   CVTRV429
CVTRV431 EQU   CVTRV429
CVTRV430 EQU   CVTRV429
CVTRV428 EQU   CVTXTNT2+80
CVTRV427 EQU   CVTXTNT2+76
CVTRV426 EQU   CVTXTNT2+72
CVTRV425 EQU   CVTXTNT2+68
CVTATACT EQU   CVTATCVT
CVTRV423 EQU   CVTXTNT2+62
CVTRV422 EQU   CVTXTNT2+60
CVTRV421 EQU   CVTXTNT2+58
CVTRV420 EQU   CVTXTNT2+56
CVTRV419 EQU   CVTXTNT2+55
CVTRV418 EQU   CVTXTNT2+54
CVTRV417 EQU   CVTRV409
CVTRV416 EQU   CVTRV409
CVTRV415 EQU   CVTRV409
CVTRV414 EQU   CVTRV409
CVTRV413 EQU   CVTRV409
CVTRV412 EQU   CVTRV409
CVTRV411 EQU   CVTRV409
CVTRV410 EQU   CVTRV409
CVTRV408 EQU   CVTRV400
CVTRV407 EQU   CVTRV400
CVTRV406 EQU   CVTRV400
CVTRV405 EQU   CVTRV400
CVTRV404 EQU   CVTRV400
CVTRV403 EQU   CVTRV400
CVTRV402 EQU   CVTRV400
CVTRV401 EQU   CVTRV400
CVTICB   EQU   CVTXTNT2+48
CVTSKTA  EQU   CVTXTNT2+44
CVTCCVT  EQU   CVTXTNT2+40
CVTRSV98 EQU   CVTXTNT2+36
CVTRSV97 EQU   CVTXTNT2+34
CVTRSV96 EQU   CVTXTNT2+32
CVTOLTEP EQU   CVTXTNT2+28
CVTQIDA  EQU   CVTQID+1
CVTRSV95 EQU   CVTQID
CVTRSV94 EQU   CVTXTNT2+20
CVTRSV93 EQU   CVTXTNT2+16
CVTRSV92 EQU   CVTXTNT2+12
CVTRSV91 EQU   CVTXTNT2+6
CVTRSV9H EQU   CVTFLGBT
CVTRSV9G EQU   CVTFLGBT
CVTRSV9F EQU   CVTFLGBT
CVTRSV9E EQU   CVTFLGBT
CVTRSV9D EQU   CVTFLGBT
CVTRSV9C EQU   CVTFLGBT
CVTVME   EQU   CVTFLGBT
CVTNPE   EQU   CVTFLGBT
CVTNUCLS EQU   CVTXTNT2+4
CVTDSSVA EQU   CVTDSSV+1
CVTRSV89 EQU   CVTDSSV
CVTRSV88 EQU   CVTXTNT1+8
CVTRSV87 EQU   CVTXTNT1+4
CVTFACHN EQU   CVTXTNT1
CVTRV488 EQU   CVTMAP+412
CVTRV487 EQU   CVTMAP+408
CVTRV486 EQU   CVTMAP+404
CVTRV485 EQU   CVTMAP+400
CVTACTAP EQU   CVTMAP+396
CVTAUTH  EQU   CVTMAP+392
CVTATMCA EQU   CVTATMCT+1
CVTATMST EQU   CVTATMCT
CVTRSV61 EQU   CVTMAP+384
CVTVOLT1 EQU   CVTVOLM1+1
CVTVOLI1 EQU   CVTVOLF1
CVTSTOA  EQU   CVTMAP+376
CVTRSV58 EQU   CVTMAP+374
CVTRSV57 EQU   CVTMAP+372
CVTDDCE  EQU   CVTMAP+368
CVTPNWFR EQU   CVTMAP+364
CVTSMF   EQU   CVTMAP+360
CVTSULK  EQU   CVTMAP+358
CVTSLKO  EQU   CVTSYSK
CVTSLKP  EQU   CVTSYSK
CVTSLKQ  EQU   CVTSYSK
CVTSLKR  EQU   CVTSYSK
CVTRSV56 EQU   CVTSYSK
CVTRSV55 EQU   CVTSYSK
CVTRSV54 EQU   CVTSYSK
CVTRSV53 EQU   CVTSYSK
CVTRSV52 EQU   CVTA1F1
CVTRSV51 EQU   CVTA1F1
CVTRSV50 EQU   CVTA1F1
CVTRSV49 EQU   CVTA1F1
CVTRSV48 EQU   CVTA1F1
CVTRSV47 EQU   CVTA1F1
CVTSRSW  EQU   CVTA1F1
CVTPFSW  EQU   CVTA1F1
CVTPCVT  EQU   CVTMAP+352
CVTRSV46 EQU   CVTMAP+344
CVTRSV45 EQU   CVTMAP+340
CVTRSV44 EQU   CVTMAP+338
CVTRSV43 EQU   CVTMAP+336
CVTHJESA EQU   CVTHJES+1
CVTRSV42 EQU   CVTHJES
CVTEXT2A EQU   CVTEXT2+1
CVTRSV41 EQU   CVTEXT2
CVTAPFA  EQU   CVTAPF+1
CVTRSV40 EQU   CVTAPF
CVTRV518 EQU   CVTINTLA
CVTRV517 EQU   CVTERPV
CVTEORM  EQU   CVTMAP+312
CVTMCHPR EQU   CVTMAP+308
CVTTZ    EQU   CVTMAP+304
CVTJEPS  EQU   CVTMAP+300
CVTJESCT EQU   CVTMAP+296
CVTMODE  EQU   CVTMAP+292
CVTPTRV  EQU   CVTMAP+288
CVTREAL  EQU   CVTMAP+284
CVTRSV39 EQU   CVTMAP+280
CVTRSV38 EQU   CVTMAP+276
CVTDMSRA EQU   CVTDMSR+1
CVTRV634 EQU   CVTDMSRF
CVTRV633 EQU   CVTDMSRF
CVTRV632 EQU   CVTDMSRF
CVTRV631 EQU   CVTDMSRF
CVTRV630 EQU   CVTDMSRF
CVTRV629 EQU   CVTDMSRF
CVTUDUMP EQU   CVTDMSRF
CVTSDUMP EQU   CVTDMSRF
CVTQMSGA EQU   CVTQMSG+1
CVTRSV36 EQU   CVTQMSG
CVTAMFF  EQU   CVTMAP+264
CVTPURGA EQU   CVTPURG+1
CVTRSV35 EQU   CVTPURG
CVTCBSP  EQU   CVTMAP+256
CVTATERA EQU   CVTATER+1
CVTSYST  EQU   CVTATER
CVTVOLT2 EQU   CVTTAT
CVTVOLI2 EQU   CVTVOLF2
CVTAQAVB EQU   CVTAQAVT+1
CVTRSV34 EQU   CVTTCMFG
CVTRSV33 EQU   CVTTCMFG
CVTRSV32 EQU   CVTTCMFG
CVTRSV31 EQU   CVTTCMFG
CVTRSV30 EQU   CVTTCMFG
CVTRSV29 EQU   CVTTCMFG
CVTRSV28 EQU   CVTTCMFG
CVTTCRDY EQU   CVTTCMFG
CVTGTFA  EQU   CVTGTF+1
CVTRSV27 EQU   CVTGTFST
CVTRNIO  EQU   CVTGTFST
CVTUSR   EQU   CVTGTFST
CVTRV318 EQU   CVTFORM
CVTRV317 EQU   CVTTMODE
CVTRV316 EQU   CVTSTATE
CVTRV315 EQU   CVTGTFS
CVTGTFAV EQU   CVTGTFS
CVT0SCR1 EQU   CVTMAP+232
CVTRV515 EQU   CVTMAP+228
CVTRMS   EQU   CVTMAP+224
CVTPATCH EQU   CVTMAP+220
CVTTSCE  EQU   CVTMAP+216
CVTLNKSC EQU   CVTMAP+214
CVTQABST EQU   CVTMAP+212
CVTMDLDS EQU   CVTMAP+208
CVTUSER  EQU   CVTMAP+204
CVTABEND EQU   CVTMAP+200
CVTSMCA  EQU   CVTMAP+196
CVTRSV18 EQU   CVTMAP+192
CVTQLPAQ EQU   CVTMAP+188
CVTQCDSR EQU   CVTMAP+184
CVTRSV17 EQU   CVTOPTB
CVTRSV16 EQU   CVTOPTB
CVTFP    EQU   CVTOPTB
CVTAPTHR EQU   CVTOPTB
CVTNLOG  EQU   CVTOPTB
CVTTOD   EQU   CVTOPTB
CVTCTIMS EQU   CVTOPTB
CVTPROT  EQU   CVTOPTB
CVTXPFP  EQU   CVTOPTA
CVTASCII EQU   CVTOPTA
CVTRSV13 EQU   CVTOPTA
CVTRSV12 EQU   CVTOPTA
CVTNIP   EQU   CVTOPTA
CVTDDR   EQU   CVTOPTA
CVTAPR   EQU   CVTOPTA
CVTCCH   EQU   CVTOPTA
CVTSNCTR EQU   CVTMAP+180
CVTQMWR  EQU   CVTMAP+176
CVTQOCR  EQU   CVTMAP+172
CVT1EF00 EQU   CVTMAP+168
CVTMZ00  EQU   CVTMAP+164
CVTSV76Q EQU   CVTSV76C
CVTRSV11 EQU   CVTMAP+156
CVT0PT01 EQU   CVTMAP+152
CVTMSER  EQU   CVTMAP+148
CVTRV516 EQU   CVTIERLC
CVTILCH  EQU   CVTMAP+140
CVT0DS   EQU   CVTMAP+136
CVTFBOSV EQU   CVTMAP+132
CVTNUCB  EQU   CVTMAP+128
CVTIXAVL EQU   CVTMAP+124
CVTSV76M EQU   CVTIOQET
CVTDCBA  EQU   CVTMAP+117
CVTMVS2  EQU   CVTDCB
CVT6DAT  EQU   CVTDCB
CVT4MPS  EQU   CVTDCB
CVTRSV09 EQU   CVTDCB
CVT4MS1  EQU   CVTDCB
CVT2SPS  EQU   CVTDCB
CVT1SSS  EQU   CVTDCB
CVTRSV08 EQU   CVTDCB
CVTSTB   EQU   CVTMAP+112
CVTQTD00 EQU   CVTMAP+108
CVTQTE00 EQU   CVTMAP+104
CVTCUCB  EQU   CVTMAP+100
CVTSJQ   EQU   CVTMAP+96
CVTPBLDL EQU   CVTMAP+92
CVTTPC   EQU   CVTMAP+88
CVTSVDCB EQU   CVTMAP+84
CVTBRET  EQU   CVTMAP+82
CVTEXIT  EQU   CVTMAP+80
CVT0FN00 EQU   CVTMAP+76
CVTDARA  EQU   CVTDAR+1
CVTRSV07 EQU   CVTFLGS1
CVTRSV06 EQU   CVTFLGS1
CVTRSV05 EQU   CVTFLGS1
CVTRSV04 EQU   CVTFLGS1
CVTRSV03 EQU   CVTFLGS1
CVTRSV02 EQU   CVTFLGS1
CVTRSV01 EQU   CVTFLGS1
CVTDMPLK EQU   CVTFLGS1
CVTXITP  EQU   CVTMAP+68
CVTZDTAB EQU   CVTMAP+64
CVTMSLT  EQU   CVTMAP+60
CVTDATE  EQU   CVTMAP+56
CVTBTERM EQU   CVTMAP+52
CVTSYSAD EQU   CVTMAP+48
CVTXTLER EQU   CVTMAP+44
CVTILK2  EQU   CVTMAP+40
CVTILK1  EQU   CVTMAP+36
CVTPRLTV EQU   CVTMAP+32
CVTPCNVT EQU   CVTMAP+28
CVTXAPG  EQU   CVTMAP+20
CVTBUF   EQU   CVTMAP+16
CVTJOB   EQU   CVTMAP+12
CVTLINK  EQU   CVTMAP+8
CVT0EF00 EQU   CVTMAP+4
CVTTCBP  EQU   CVTMAP
CVT      EQU   CVTMAP
@NM00071 EQU   PSA+3668
PSASTAK  EQU   PSA+3072
@NM00070 EQU   PSA+1048
PSAUSEND EQU   PSA+1048
PSARV062 EQU   PSA+1044
PSACDAL  EQU   PSA+1040
PSAWTCOD EQU   PSA+1036
PSAATCVT EQU   PSA+1032
PSAPCPSW EQU   PSA+1024
PSAPIR2  EQU   PSA+1020
PSARV059 EQU   PSA+1018
PSASVC13 EQU   PSA+1016
PSALSFCC EQU   PSA+1012
PSASFACC EQU   PSA+1008
PSASTOP  EQU   PSA+992
PSASTART EQU   PSA+976
PSARSPSW EQU   PSA+968
PSASRPSW EQU   PSA+960
PSARV045 EQU   PSA+892
PSARV044 EQU   PSA+888
PSARV043 EQU   PSA+884
PSARV042 EQU   PSA+880
PSARV041 EQU   PSA+876
PSARV040 EQU   PSA+872
PSARV025 EQU   PSA+868
PSADSSED EQU   PSA+868
PSADSSPR EQU   PSA+864
PSADSSFW EQU   PSA+860
PSADSS14 EQU   PSA+856
PSADSSPP EQU   PSA+848
PSADSSRP EQU   PSA+840
PSADSS05 EQU   PSADSSF4
PSADSS10 EQU   PSADSSF4
PSADSSVE EQU   PSADSSF4
PSADSSDE EQU   PSADSSF4
PSADSSC0 EQU   PSADSSF4
PSADSSIE EQU   PSADSSF4
PSADSS12 EQU   PSADSSF4
PSADSSRC EQU   PSADSSF4
PSARV057 EQU   PSADSSF3
PSARV056 EQU   PSADSSF3
PSARV055 EQU   PSADSSF3
PSADSSMC EQU   PSADSSF3
PSADSSRW EQU   PSADSSF3
PSADSSNM EQU   PSADSSF3
PSADSSES EQU   PSADSSF3
PSADSSGP EQU   PSADSSF3
PSADSSF2 EQU   PSADSSFL+1
PSADSSPI EQU   PSADSSF1
PSADSSOI EQU   PSADSSF1
PSADSSSP EQU   PSADSSF1
PSADSSTP EQU   PSADSSF1
PSADSSDW EQU   PSADSSF1
PSADSSDD EQU   PSADSSF1
PSADSSDM EQU   PSADSSF1
PSADSSMV EQU   PSADSSF1
PSADSSTS EQU   PSA+816
PSADSSWK EQU   PSA+812
PSADSSR3 EQU   PSA+808
PSADSSR2 EQU   PSA+804
PSADSSRS EQU   PSA+800
PSASTOR  EQU   PSA+796
PSACPUSA EQU   PSA+794
PSAVSTAP EQU   PSA+792
PSAWKVAP EQU   PSA+788
PSAWKRAP EQU   PSA+784
PSAMCHIC EQU   PSA+783
PSARV061 EQU   PSA+782
PSASYMSK EQU   PSA+781
PSAMCHFL EQU   PSA+780
PSACR0   EQU   PSA+776
PSAPSWSV EQU   PSA+768
PSACLHS  EQU   PSAHLHI
PSALKR15 EQU   PSALKSA+60
PSALKR14 EQU   PSALKSA+56
PSALKR13 EQU   PSALKSA+52
PSALKR12 EQU   PSALKSA+48
PSALKR11 EQU   PSALKSA+44
PSALKR10 EQU   PSALKSA+40
PSALKR9  EQU   PSALKSA+36
PSALKR8  EQU   PSALKSA+32
PSALKR7  EQU   PSALKSA+28
PSALKR6  EQU   PSALKSA+24
PSALKR5  EQU   PSALKSA+20
PSALKR4  EQU   PSALKSA+16
PSALKR3  EQU   PSALKSA+12
PSALKR2  EQU   PSALKSA+8
PSALKR1  EQU   PSALKSA+4
PSALKR0  EQU   PSALKSA
PSARV023 EQU   PSACLHT+52
PSALOCAL EQU   PSACLHT+48
PSACMSL  EQU   PSACLHT+44
PSAOPTL  EQU   PSACLHT+40
PSATPACL EQU   PSACLHT+36
PSATPDNL EQU   PSACLHT+32
PSATPNCL EQU   PSACLHT+28
PSAIOSLL EQU   PSACLHT+24
PSAIOSUL EQU   PSACLHT+20
PSAIOSCL EQU   PSACLHT+16
PSAIOSSL EQU   PSACLHT+12
PSASALCL EQU   PSACLHT+8
PSAASML  EQU   PSACLHT+4
PSADISPL EQU   PSACLHT
PSASRSA  EQU   PSA+636
PSARV050 EQU   PSA+634
PSADSSGO EQU   PSA+633
PSARECUR EQU   PSA+632
PSAHLHIS EQU   PSA+628
PSAIPCSA EQU   PSA+624
@NM00069 EQU   PSA+621
PSAIPCDM EQU   PSA+620
PSAIPCD  EQU   PSA+616
@NM00068 EQU   PSA+613
PSAIPCRM EQU   PSA+612
PSAIPCR  EQU   PSA+608
PSAMCHEX EQU   PSA+600
PSAMPSW  EQU   PSA+592
PSAEXPS2 EQU   PSA+584
PSAEXPS1 EQU   PSA+576
PSAPIREG EQU   PSA+572
PSARSREG EQU   PSA+568
PSAGPREG EQU   PSA+556
PSARV022 EQU   PSASUP4
PSARV021 EQU   PSASUP4
PSARV020 EQU   PSASUP4
PSARV019 EQU   PSASUP4
PSARV018 EQU   PSASUP4
PSARV017 EQU   PSASUP4
PSARV016 EQU   PSASUP4
PSARV015 EQU   PSASUP4
PSARV014 EQU   PSASUP3
PSARV013 EQU   PSASUP3
PSARV012 EQU   PSASUP3
PSAESTA  EQU   PSASUP3
PSASPR   EQU   PSASUP3
PSAPSREG EQU   PSASUP3
PSAPI2   EQU   PSASUP3
PSAIOSUP EQU   PSASUP3
PSALCR   EQU   PSASUP2
PSARTM   EQU   PSASUP2
PSAACR   EQU   PSASUP2
PSAIPCE2 EQU   PSASUP2
PSAIPCES EQU   PSASUP2
PSAIPCEC EQU   PSASUP2
PSAGTF   EQU   PSASUP2
PSAIPCRI EQU   PSASUP2
PSATYPE6 EQU   PSASUP1
PSATCTL  EQU   PSASUP1
PSADISP  EQU   PSASUP1
PSALOCK  EQU   PSASUP1
PSAPI    EQU   PSASUP1
PSAEXT   EQU   PSASUP1
PSASVC   EQU   PSASUP1
PSAIO    EQU   PSASUP1
PSAAOLD  EQU   PSA+548
PSAANEW  EQU   PSA+544
PSATNEW  EQU   PSA+536
PSALCCAR EQU   PSA+532
PSALCCAV EQU   PSA+528
PSAPCCAR EQU   PSA+524
PSAPCCAV EQU   PSA+520
PSACPULA EQU   PSA+518
PSACPUPA EQU   PSA+516
PSAPSA   EQU   PSA+512
FLCHDEND EQU   PSA+512
FLCCRSAV EQU   FLCMCLA+280
FLCGRSAV EQU   FLCMCLA+216
FLCFPSAV EQU   FLCMCLA+184
FLCFLA   EQU   FLCMCLA+88
FLCRGNCD EQU   FLCMCLA+84
FLCFSAA  EQU   FLCFSA+1
@NM00067 EQU   FLCFSA
@NM00066 EQU   FLCMCLA+72
FLCMCIC  EQU   FLCMCLA+64
@NM00065 EQU   FLCMCLA+20
FLCIOAA  EQU   FLCIOA+1
@NM00064 EQU   FLCIOA
@NM00063 EQU   FLCMCLA+15
@NM00062 EQU   FLCMCLA+14
@NM00061 EQU   FLCMCLA+12
FLCLCL   EQU   FLCMCLA+8
FLCIOELA EQU   FLCIOEL+1
@NM00060 EQU   FLCIOEL
FLCCHNID EQU   FLCMCLA
@NM00059 EQU   PSA+160
FLCMTRCD EQU   PSA+157
@NM00058 EQU   PSA+156
FLCPERA  EQU   FLCPER+1
@NM00057 EQU   FLCPER
@NM00056 EQU   PSA+151
FLCPERCD EQU   PSA+150
FLCMCNUM EQU   PSA+149
@NM00055 EQU   PSA+148
FLCTEAA  EQU   FLCTEA+1
@NM00054 EQU   FLCTEA
PSAPIPC  EQU   PSAPICOD
PSAPIMC  EQU   PSAPICOD
PSAPIPER EQU   PSAPICOD
PSARV049 EQU   FLCPICOD
FLCPILCB EQU   FLCPIILC
@NM00053 EQU   FLCPIILC
@NM00052 EQU   PSAEPPSW
FLCSVCN  EQU   PSAESPSW+2
FLCSILCB EQU   FLCSVILC
@NM00051 EQU   FLCSVILC
@NM00050 EQU   PSAESPSW
FLCEICOD EQU   PSAEEPSW+2
PSASPAD  EQU   PSAEEPSW
@NM00049 EQU   PSA+128
FLCINPSW EQU   PSA+120
FLCMNPSW EQU   PSA+112
FLCPNPSW EQU   PSA+104
FLCSNPSW EQU   PSA+96
FLCENPSW EQU   PSA+88
FLCTRACE EQU   PSA+84
FLCTIMER EQU   PSA+80
FLCCVT2  EQU   PSA+76
FLCCAW   EQU   PSA+72
FLCCSW   EQU   PSA+64
FLCIOPSW EQU   PSA+56
FLCMOPSW EQU   PSA+48
FLCPOPSW EQU   PSA+40
FLCSOPSW EQU   PSA+32
FLCEOPSW EQU   PSA+24
@NM00048 EQU   FLCICCW2+4
FLCCVT   EQU   FLCICCW2
FLCICCW1 EQU   FLCROPSW
FLCIPPSW EQU   FLCRNPSW
.@UNREFD ANOP                          END UNREFERENCED COMPONENTS
@RC00250 EQU   EXITCLN
@RT00360 EQU   EXCPNXT
@RT00411 EQU   ENQ
@RT00413 EQU   EXCPNXT
@RT00457 EQU   EXITCLN
@RT00462 EQU   EXCPNXT
@RF00464 EQU   @RC00334
@RT00509 EQU   EXCPNXT
@RF00641 EQU   @EL00003
@RT00682 EQU   DUMPRTN1
@RT00687 EQU   DUMPRTN1
@RT00690 EQU   DUMPRTN1
@RT00692 EQU   DUMPRTN1
@ENDDATA EQU   *
         END   IGCT105C,(C'PLS0956',0701,78025)
