         TITLE 'IGG0CLB1, CATALOG UPGRADE MANAGEMENT                   *
                        '
IGG0CLB1 CSECT ,                                                   0001
@MAINENT DS    0H                                                  0001
         USING *,@15                                               0001
         B     @PROLOG                                             0001
         DC    AL1(16)                                             0001
         DC    C'IGG0CLB1  75.084'                                 0001
         DROP  @15
@PROLOG  DS    0H                                                  0002
         USING @MAINENT,@15
         USING @DATD-(CCATEMPS-IGGCCA),R11
         DS 0F
         DC CL8'IGGPUADD'
         EJECT
         B     @PB00002                                            0094
IGGPUADD DS    0H                                                  0095
         ENTRY IGGPUADD
         DROP @15
Z003     DS    0H                                                  0097
Z003UADD DS    0H                                                  0098
         STM 12,14,12(13)
         LA 13,12(13)
         BALR 12,0
IGGSEGST DS 0H
*                                      PROCEDURE                     */
         USING *,12
*/********************************************************************
**                                                                   *
**   MODULE NAME = IGGPUADD (IGG0CLB1)                               *
**                                                                   *
**   DESCRIPTIVE NAME = ADD UPGRADE ASSOCIATION                      *
**                                                                   *
**   FUNCTION =                                                      *
**      THIS SUB-FUNCTION IS EXTERNALLY CALLED BY SUCH CMS ROUTINES  *
**      AS DEFINE AND ALTER. IT ADDS UPGRADE ASSOCIATIONS TO THE     *
**      UPGRADE ENTRY. IT WILL ALSO CREATE THE UPGRADE ENTRY, IF     *
**      IT DOES NOT EXIST AND IT WILL INSURE THAT THE UPGRADE        *
**      ASSOCIATION TO BE ADDED IS NOT A DUPLICATE. THE PROCESSING   *
**      IS AS FOLLOWS:                                               *
**       - CALL IGGPUCOM TO VALIDITY CHECK THE CALLER'S INPUT        *
**         PARAMETERS, TO PERFORM INITIALIZATION, AND TO READ THE    *
**         UPGRADE ENTRY INTO CORE. IF THE UPGRADE ENTRY DOES NOT    *
**         EXIST, ONLY THE BASE DATA RECORD IS READ.                 *
**       - IF THE UPGRADE ENTRY IS NOT EXISTENT, CALL IGGPAXCI TO    *
**         OBTAIN A CI. BUILD THE BASE UPGRADE RECORD IN-CORE,       *
**         AND THEN CALL IGGPPAD TO ADD THE BASE UPGRADE RECORD TO   *
**         THE VSAM CATALOG. NEXT, BUILD THE Y-ASSOCIATION WHICH     *
**         POINTS TO THE UPGRADE ENTRY AND CALL MODIFY (IGGPMOD)     *
**         TO ADD THIS Y-ASSOCIATION TO THE BASE DATA ENTRY.         *
**         SKIP THE NEXT STEP.                                       *
**       - IF THE UPGRADE ENTRY IS EXISTENT, CALL EXTRACT (IGGPEXT)  *
**         TO INSURE THAT THE CALLER SPECIFIED UPGRADE ASSOCIATION   *
**         TO BE ADDED WILL NOT CREATE A DUPLICATE.                  *
**       - CALL MODIFY TO ADD THE UPGRADE ASSOCIATION TO THE         *
**         UPGRADE ENTRY.                                            *
**                                                                   *
**   INPUT =                                                         *
**      THE FOLLOWING FIELDS ARE IN THE CUMPLWA DECLARATION:         *
**        CUMDCI - 3-BYTE AIX DATA CI NUMBER                         *
**        CUMICI - 3-BYTE AIX INDEX CI NUMBER                        *
**      COMMON (TO IGGPUDEL) INPUT:                                  *
**        CUMNRG - IF THIS FLAG IS SET ON, NO RE-GET OF THE ORIGINAL *
**            BASE ENTRY RECORD IS DONE.                             *
**        THE CALLER MUST PROVIDE CONTIGUOUS CORE FOR THE CATALOG    *
**        UPGRADE MGMT PARAMETER LIST AND FOR THE CATALOG            *
**        UPGRADE MGMT WORK AREA (MAPPED BY CUMPLWA). CCACUMPL       *
**        MUST CONTAIN THE ADDRESS OF THIS CORE.                     *
**        THE USER MUST SPECIFY AT LEAST 3 FREE RECORD AREAS WHICH   *
**        WILL BE USED BY CATALOG UPGRADE MGMT TO PERFORM ITS        *
**        I/O OPERATIONS. IN PARTICULAR,                             *
**        CCARABF - ADDRESS OF THE FIRST RECORD AREA BLOCK WHICH     *
**          CATALOG UPGRADE MGMT MAY POSSIBLY USE.                   *
**        CCARABL - ADDRESS OF THE LAST RECORD AREA BLOCK WHICH      *
**          CATALOG UPGRADE MGMT MAY POSSIBLY USE                    *
**        CCARABB - ADDRESS OF THE RECORD AREA BLOCK WHICH POINTS TO *
**          THE BASE RECORD OF THE ENTRY. THIS BASE RECORD MUST      *
**          HAVE AN ENTRY TYPE OF G, C, OR D. ALSO, CCARABB MUST     *
**          LIE BETWEEN CCARABF AND CCARABL.                         *
**        CCARNUR (WHERE N=1,2,3,4, OR 5) - IF THIS BIT IS SET ON IN *
**          A RECORD AREA BLOCK, IT INDICATES THE ASSOCIATED RECORD  *
**          AREA IS NOT FREE FOR CATALOG UPGRADE MGMT'S USE.         *
**                                                                   *
**   OUTPUT =                                                        *
**      THE FOLLOWING FIELDS ARE IN THE CUMPLWA DECLARATION:         *
**        CUMDUP - THIS BIT IS SET ON, IF THE REQUESTED ADD OPERATION*
**          WOULD CREATE DUPLICATE UPGRADE ASSOCIATIONS. THE ADD IS  *
**          NOT PERFORMED.                                           *
**      COMMON (TO IGGPUDEL) OUTPUT:                                 *
**        FLAG CCAREUSE IS SET OFF.                                  *
**        THE CONTENTS OF THE USER'S CCAURAB IS SAVED AND RESTORED.  *
**        THE ORIGINAL BASE RECORD REFERENCED BY CCARABB IS          *
**          RESTORED.                                                *
**        IN GENERAL, CATALOG UPGRADE MGMT WILL NOT CHANGE INPUT     *
**          FIELDS SO THAT ITS INTERFACE IS COMPLETELY REUSABLE      *
**          AFTER A PARTICULAR CALL.                                 *
**        ERROR CODES - THE MAIN ERROR CODE IS RETURNED IN CCACD1.   *
**          THE POSSIBLE RETURN CODES ARE DESCRIBED BELOW.  A        *
**          SECONDARY ERROR CODE IS RETURNED IN CCACD2. THIS FIELD   *
**          WILL CONTAIN THE RETURN CODE RECEIVED BACK FROM GET      *
**          WHEN THE CALLER'S BASE RECORD IS RESTORED. ANY NON-ZERO  *
**          CODE IN CCACD2 IS PROPAGATED TO CCACD1, PROVIDED CCACD1  *
**          IS NOT ALREADY NON-ZERO.                                 *
**          ANY ERROR CODE RETURNED BY EXTRACT OR MODIFY             *
**          ANY ERROR CODE RETURNED BY THE I/O ROUTINES              *
**            (GET, AXCI, PAD)                                       *
**          RCINENT - IF THE BASE RECORD ENTRY TYPE IS NOT G, C, OR  *
**            D.                                                     *
**          RCINCNPM - IF THE RAB REQUIREMENTS ARE NOT MET.          *
**          RCCATBAL - IF CATALOG UPGRADE MGMT ENCOUNTERS A          *
**          STRUCTURAL ERROR.                                        *
**                                                                   *
******************************************************************** */
*                                                                  0104
*   CUMFUNC=ON;                     /* INDICATE TO IGGPUCOM THAT THIS
*                                      IS AN ADD                     */
*                                                                  0104
         L     @10,CCACUMPL(,R11)                                  0104
         OI    CUMFUNC(@10),B'00100000'                            0104
*   /*****************************************************************/
*   /*                                                               */
*   /* IGGPUCOM MUST BE CALLED TO VALIDITY CHECK THE CALLER'S INPUT  */
*   /* PARAMETERS, PERFORM INITIALIZATION, AND TO READ THE UPGRADE   */
*   /* ENTRY INTO CORE, IF IT EXISTS.                                */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0105
*Z004UCOM:                                                         0105
*   CALL IGGPUCOM;                  /* COMMON INITIAL PROCESSING     */
Z004UCOM BAL   @14,IGGPUCOM                                        0105
*   IF CCACD1^=RCS THEN             /* IF AN ERROR OCCURRED          */
         CLI   CCACD1(R11),0                                       0106
         BE    @RF00106                                            0106
*Z005UEND:                                                         0107
*     CALL IGGPUEND;                /* DO TERMINATION PROCESSING AND
*                                      EXIT                          */
Z005UEND BAL   @14,IGGPUEND                                        0107
*   ELSE                                                           0108
*     ;                             /* NO ERROR                      */
@RF00106 DS    0H                                                  0109
*   IF CUMNE=ON THEN                /* IF THE UPGRADE ENTRY IS     0109
*                                      NON-EXISTENT                  */
         L     @10,CCACUMPL(,R11)                                  0109
         TM    CUMNE(@10),B'01000000'                              0109
         BNO   @RF00109                                            0109
*     DO;                           /* CREATE THE UPGRADE ENTRY      */
*                                                                  0110
*       /*************************************************************/
*       /*                                                           */
*       /* CALL TO MODULE IGG0CLAG                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0111
*Z006AXCI:                                                         0111
*       CALL IGGPAXCI;              /* GET A CI ASSIGNED FOR THE   0111
*                                      UPGRADE ENTRY                 */
Z006AXCI L     @15,@CV00041                                        0111
         BALR  @14,@15                                             0111
*       IF CCACD1^=RCS THEN         /* IF AN ERROR OCCURRED          */
         CLI   CCACD1(R11),0                                       0112
         BE    @RF00112                                            0112
*Z007UEND:                                                         0113
*         CALL IGGPUEND;            /* DO TERMINATION PROCESSING AND
*                                      EXIT                          */
Z007UEND BAL   @14,IGGPUEND                                        0113
*       ELSE                                                       0114
*         ;                         /* NO ERROR                      */
@RF00112 DS    0H                                                  0115
*       RFY                                                        0115
*         CCARAB BASED(CUMRAB2);    /* MAP RECORD AREA TO BE USED FOR
*                                      BUILDING UPGRADE ENTRY        */
*       ZIT(1)=ZIT(1)&&ZIT(1);      /* ZERO RECORD AREA-FIRST HALF   */
         L     @10,CCACUMPL(,R11)                                  0116
         L     @05,CUMRAB2(,@10)                                   0116
         L     @15,CCACPE1(,@05)                                   0116
         XC    ZIT(256,@15),ZIT(@15)                               0116
*       ZIT(2)=ZIT(2)&&ZIT(2);      /* ZERO RECORD AREA-SECOND HALF  */
         XC    ZIT+256(256,@15),ZIT+256(@15)                       0117
*       RCDCINO=CCAASCI(1);         /* SET CI NUMBER ASSIGNED        */
         MVC   RCDCINO(3,@15),CCAASCI(R11)                         0118
*       CUMWCI=CCAASCI(1);          /* UPGRADE CI NO.                */
         MVC   CUMWCI(3,@10),CCAASCI(R11)                          0119
*       RCDID=UPGDID;               /* SET UPGRADE ID                */
         MVI   RCDID(@15),C'Y'                                     0120
*       RCDSIZ=LENGTH(CATRCD)+      /* SET RECORD SIZE               */
*           LENGTH(REPCNTRL);                                      0121
         MVC   RCDSIZ(2,@15),@CH03001                              0121
*       NREPLF=LENGTH(CATRCD);      /* SET LENGTH OF ALL FIXED LENGTH
*                                      FIELDS                        */
         LA    @10,49                                              0122
         STC   @10,NREPLF(,@15)                                    0122
*       CCACPE2=CCACPE1+NREPLF;     /* SET CPE2                      */
         ALR   @10,@15                                             0123
         ST    @10,CCACPE2(,@05)                                   0123
*       CCACPE3=CCACPE2+LENGTH(REPCNTRL);/* SET CPE3                 */
         AH    @10,@CH00097                                        0124
         ST    @10,CCACPE3(,@05)                                   0124
*       CCACPE4=CCACPE3;            /* SET CPE4                      */
         ST    @10,CCACPE4(,@05)                                   0125
*       RCDCRAVL=CCARABB->CCACPE1->RCDCRAVL;/* SET CRA VOL           */
         L     @10,CCARABB(,R11)                                   0126
         L     @10,CCACPE1(,@10)                                   0126
         MVC   RCDCRAVL(6,@15),RCDCRAVL(@10)                       0126
*       RCDCRADT=CCARABB->CCACPE1->RCDCRADT;/* SET CRA CI            */
         MVC   RCDCRADT(4,@15),RCDCRADT(@10)                       0127
*       CCAURAB=CUMRAB2;            /* SET UPGRADE RAB               */
         ST    @05,CCAURAB(,R11)                                   0128
*       DO;                                                        0129
*                                                                  0129
*         /***********************************************************/
*         /*                                                         */
*         /* CALL TO MODULE IGG0CLAG                                 */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0130
*Z009PAD:                                                          0130
*         CALL IGGPPAD;             /* PUT-ADD RECORD                */
Z009PAD  L     @15,@CV00040                                        0130
         BALR  @14,@15                                             0130
*       END;                        /* ADD UPGRADE ENTRY             */
*       IF CCACD1^=RCS THEN         /* IF AN ERROR OCCURRED          */
         CLI   CCACD1(R11),0                                       0132
         BE    @RF00132                                            0132
*Z010UEND:                                                         0133
*         CALL IGGPUEND;            /* TERMINATE AND EXIT            */
Z010UEND BAL   @14,IGGPUEND                                        0133
*       ELSE                                                       0134
*         ;                         /* NO ERROR                      */
*                                                                  0134
@RF00132 DS    0H                                                  0135
*       /*************************************************************/
*       /*                                                           */
*       /* THE UPGRADE ENTRY HAS NOW BEEN WRITTEN TO THE CATALOG.    */
*       /* NEXT, THE POINTER TO IT MUST BE INSERTED INTO THE BASE    */
*       /* DATA ENTRY. INITIALIZE THE FIRST FPL                      */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0135
*       RFY                                                        0135
*         CTGPL BASED(CCAMCPL);     /* MAP CPL                       */
*       RFY                                                        0136
*         CTGFL BASED(CTGFIELD(1)); /* MAP FIRST FPL                 */
*       CTGFLCHN=0;                 /* NO TEST FPL'S                 */
         L     @10,CCAMCPL(,R11)                                   0137
         L     @10,CTGFIELD(,@10)                                  0137
         SLR   @05,@05                                             0137
         ST    @05,CTGFLCHN(,@10)                                  0137
*       CTGFLPT(1)=ADDR(CUMWASS);   /* POINT TO ASSOCIATION DATA AREA
*                                      FOR ADD                       */
         L     @05,CCACUMPL(,R11)                                  0138
         LA    @15,CUMWASS(,@05)                                   0138
         ST    @15,CTGFLPT(,@10)                                   0138
*       CTGFLNG(1)=LENGTH(CUMWASS); /* LENGTH OF ASSOCIATION         */
         MVC   CTGFLNG(4,@10),@CF00103                             0139
*       CUMWTYP=UPGDID;             /* UPGRADE ID                    */
*                                                                  0140
         MVI   CUMWTYP(@05),C'Y'                                   0140
*       /*************************************************************/
*       /*                                                           */
*       /* CALL TO MODULE IGG0CLAV                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0141
*Z011MOD:                                                          0141
*       CALL IGGPMOD;               /* CALL MODIFY TO ADD UPGRADE  0141
*                                      ASSOCIATION PTR TO BASE DATA
*                                      ENTRY                         */
Z011MOD  L     @15,@CV00037                                        0141
         BALR  @14,@15                                             0141
*       IF CCACD1^=RCS THEN         /* IF AN ERROR OCCURRED          */
         CLI   CCACD1(R11),0                                       0142
         BE    @RF00142                                            0142
*         DO;                       /* ERROR EXIT                    */
*           CCASRID=CUMWCI;         /* CI FOR DELETE                 */
         L     @10,CCACUMPL(,R11)                                  0144
         MVC   CCASRID(3,R11),CUMWCI(@10)                          0144
*           DO;                                                    0145
*             CCAF1KEY=OFF;         /* GET BY CI NO.                 */
*             CCAF1KGE=OFF;         /* NOT GET NEXT                  */
*             CCAF1UP=ON;           /* GET FOR UPD                   */
*                                                                  0148
         OI    CCAF1UP(R11),B'00000010'                            0148
         NI    CCAF1KEY(R11),B'11100111'                           0148
*             /*******************************************************/
*             /*                                                     */
*             /* CALL TO MODULE IGG0CLBI                             */
*             /*                                                     */
*             /*******************************************************/
*                                                                  0149
*Z013GET:                                                          0149
*             CALL IGGPGET;         /* GET RECORD                    */
Z013GET  L     @15,@CV00036                                        0149
         BALR  @14,@15                                             0149
*           END;                    /* GET Y RECORD                  */
*           DO;                                                    0151
*                                                                  0151
*             /*******************************************************/
*             /*                                                     */
*             /* CALL TO MODULE IGG0CLAG                             */
*             /*                                                     */
*             /*******************************************************/
*                                                                  0152
*Z015PDE:                                                          0152
*             CALL IGGPPDE;         /* PUT-DELETE RECORD             */
Z015PDE  L     @15,@CV00039                                        0152
         BALR  @14,@15                                             0152
*           END;                    /* DELETE Y RECORD               */
*Z016UEND:                                                         0154
*           CALL IGGPUEND;          /* TERMINATE AND EXIT            */
Z016UEND BAL   @14,IGGPUEND                                        0154
*         END;                      /* END ERROR EXIT                */
*       ELSE                                                       0156
*         ;                         /* NO ERROR                      */
@RF00142 DS    0H                                                  0157
*       CCARABB=CUMRAB2;            /* SET BASE RAB TO UPGRADE RAB AS
*                                      IT WOULD BE IF IGGPUCOM HAD 0157
*                                      FOUND THE UPGRADE ENTRY     0157
*                                      ALREADY EXISTENT              */
         L     @10,CCACUMPL(,R11)                                  0157
         L     @05,CUMRAB2(,@10)                                   0157
         ST    @05,CCARABB(,R11)                                   0157
*       RFY                                                        0158
*         CCARAB BASED(CCARABB);    /* MAP UPGRADE RAB               */
*       IF RCDCINO^=CUMWCI THEN     /* IF UPGRADE RCD NOT IN-CORE    */
         L     @05,CCACPE1(,@05)                                   0159
         CLC   RCDCINO(3,@05),CUMWCI(@10)                          0159
         BE    @RF00159                                            0159
*         DO;                       /* RE-GET IT                     */
*           CCASRID=CUMWCI;         /* CI FOR GET                    */
         MVC   CCASRID(3,R11),CUMWCI(@10)                          0161
*           DO;                                                    0162
*             CCAF1KEY=OFF;         /* GET BY CI NO.                 */
*             CCAF1KGE=OFF;         /* NOT GET NEXT                  */
*             CCAF1UP=ON;           /* GET FOR UPD                   */
*                                                                  0165
         OI    CCAF1UP(R11),B'00000010'                            0165
         NI    CCAF1KEY(R11),B'11100111'                           0165
*             /*******************************************************/
*             /*                                                     */
*             /* CALL TO MODULE IGG0CLBI                             */
*             /*                                                     */
*             /*******************************************************/
*                                                                  0166
*Z018GET:                                                          0166
*             CALL IGGPGET;         /* GET RECORD                    */
Z018GET  L     @15,@CV00036                                        0166
         BALR  @14,@15                                             0166
*           END;                    /* GET Y RCD                     */
*           IF CCACD1^=RCS THEN     /* IF ERROR                      */
         CLI   CCACD1(R11),0                                       0168
         BE    @RF00168                                            0168
*Z019UEND:                                                         0169
*             CALL IGGPUEND;        /* TERMINATE AND EXIT            */
Z019UEND BAL   @14,IGGPUEND                                        0169
*           ELSE                                                   0170
*             ;                     /* NO ERROR                      */
@RF00168 DS    0H                                                  0171
*         END;                      /* END RE-GET IT                 */
*       ELSE                                                       0172
*         ;                         /* UPGRADE RCD IS IN-CORE        */
@RF00159 DS    0H                                                  0173
*     END;                          /* END CREATE                    */
*   ELSE                            /* THE UPGRADE ENTRY IS EXISTENT */
*     DO;                           /* CHECK THAT THE UPGRADE      0174
*                                      ASSOCIATION TO BE ADDED IS  0174
*                                      UNIQUE                        */
*                                                                  0174
         B     @RC00109                                            0174
@RF00109 DS    0H                                                  0175
*       /*************************************************************/
*       /*                                                           */
*       /* BUILD THE EXTRACT INTERFACE AND ATTEMPT TO RETRIEVE THE   */
*       /* UPGRADE ASSOCIATION WHICH IS TO BE ADDED TO INSURE THAT IT*/
*       /* WILL NOT CREATE A DUPLICATE. INITIALIZE THE FIRST FPL     */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0175
*       RFY                                                        0175
*         CTGFL BASED(CTGFIELD(1)); /* MAP FIRST FPL                 */
*       CTGFLDGC=ASSOCGC;           /* SET ASSOCIATION GROUP CODE    */
         L     @10,CCAMCPL(,R11)                                   0176
         L     @05,CTGFIELD(,@10)                                  0176
         MVI   CTGFLDGC(@05),X'02'                                 0176
*       CUMFN1=RELREPNO;            /* RETRIEVE THE SEQUENCE NO. OF
*                                      ANY POSSIBLE DUPLICATE ATTEMPT
*                                      TO GET UPGRADE ASSOCIATION  0177
*                                      WHICH TESTS EQUAL TO THE ONE
*                                      WE ARE GOING TO ADD           */
         L     @15,CCACUMPL(,R11)                                  0177
         MVC   CUMFN1(8,@15),@CC00058                              0177
*       CUMFN2=NAME;                /* TEST AGAINST THE CI FIELD IN
*                                      THE ASSOCIATION GROUP         */
         MVC   CUMFN2(8,@15),@CC00064                              0178
*       CTGFLCHN->CTGFLNG(1)=LENGTH(CUMWCI);/* LENGTH OF TEST FIELD  */
         L     @05,CTGFLCHN(,@05)                                  0179
         MVC   CTGFLNG(4,@05),@CF00074                             0179
*       CTGFLCHN->CTGFLPT(1)=ADDR(CUMWCI);/* ADDR OF TEST FIELD      */
         LA    @14,CUMWCI(,@15)                                    0180
         ST    @14,CTGFLPT(,@05)                                   0180
*       CUMWCI=CUMDCI;              /* TEST CI NUMBER WHICH WAS    0181
*                                      SPECIFIED BY CALLER           */
         MVC   CUMWCI(3,@15),CUMDCI(@15)                           0181
*       WKALEN1=LENGTH(WKALEN1)+    /* SET MIN POSSIBLE WORK         */
*           LENGTH(WKALEN2);        /* AREA SIZE                     */
*                                                                  0182
         L     @10,CTGWKA(,@10)                                    0182
         MVC   WKALEN1(2,@10),@CH00103                             0182
*       /*************************************************************/
*       /*                                                           */
*       /* CALL TO MODULE IGG0CLAZ                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0183
*Z020EXT:                                                          0183
*       CALL IGGPEXT;               /* ATTEMPT TO RETRIEVE UPGRADE 0183
*                                      ASSOCIATION WHICH TO BE ADDED */
Z020EXT  L     @15,@CV00038                                        0183
         BALR  @14,@15                                             0183
*       CTGFLDGC=0;                 /* RE-INITLZ GROUP CODE          */
         L     @10,CCAMCPL(,R11)                                   0184
         L     @10,CTGFIELD(,@10)                                  0184
         MVI   CTGFLDGC(@10),X'00'                                 0184
*       IF CCACD1=RCVLSZ THEN       /* IF INSUFFICIENT WORK AREA   0185
*                                      ERROR CODE, IT MEANS THE    0185
*                                      UPGRADE ASSOCIATION TO BE   0185
*                                      ADDED IS A DUPLICATE          */
         CLI   CCACD1(R11),40                                      0185
         BNE   @RF00185                                            0185
*         DO;                       /* FINISHED                      */
*           CUMDUP=ON;              /* INDICATE THE UPGRADE        0187
*                                      ASSOCIATION IS A DUP          */
*                                                                  0187
         L     @10,CCACUMPL(,R11)                                  0187
         OI    CUMDUP(@10),B'10000000'                             0187
*           /*********************************************************/
*           /*                                                       */
*           /* CLEAR CCAMODID, CCAREASN, AND CCARETRN                */
*           /*                                                       */
*           /*********************************************************/
*                                                                  0188
*           CCAPROB=IGNOREIT;       /* ZERO EXPECTED RETURN CODE     */
         SLR   @10,@10                                             0188
         ST    @10,CCAPROB(,R11)                                   0188
*Z022UEND:                                                         0189
*           CALL IGGPUEND;          /* TERMINATE AND EXIT            */
Z022UEND BAL   @14,IGGPUEND                                        0189
*         END;                      /* END FINISHED                  */
*       ELSE                                                       0191
*         ;                         /* INOT THIS ERROR CODE          */
@RF00185 DS    0H                                                  0192
*       IF CCACD1^=RCS THEN         /* IF SOME OTHER ERROR           */
         CLI   CCACD1(R11),0                                       0192
         BE    @RF00192                                            0192
*Z023UEND:                                                         0193
*         CALL IGGPUEND;            /* TERMINATE AND EXIT            */
Z023UEND BAL   @14,IGGPUEND                                        0193
*       ELSE                                                       0194
*         ;                         /* NO ERROR, THE UPGRADE       0194
*                                      ASSOCIATION IS UNIQUE         */
@RF00192 DS    0H                                                  0195
*     END;                          /* END CHECK                     */
*                                                                  0195
*   /*****************************************************************/
*   /*                                                               */
*   /* BUILD THE MODIFY INTERFACE TO ADD THE CALLER SPECIFIED UPGRADE*/
*   /* ASSOCIATION TO THE UPGRADE ENTRY INITIALIZE THE FIRST FPL     */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0196
*   CTGFLDNO=1;                     /* ONE FPL LENGTH/ADDR PAIR      */
@RC00109 L     @10,CCAMCPL(,R11)                                   0196
         L     @10,CTGFIELD(,@10)                                  0196
         MVI   CTGFLDNO(@10),X'01'                                 0196
*   CTGFLCHN=0;                     /* NO TEST FPL'S                 */
         SLR   @05,@05                                             0197
         ST    @05,CTGFLCHN(,@10)                                  0197
*   CTGFLPT(1)=ADDR(CUMWASS2);      /* POINT TO UPGRADE ASSOCIATION
*                                      AREA                          */
         L     @05,CCACUMPL(,R11)                                  0198
         LA    @15,CUMWASS2(,@05)                                  0198
         ST    @15,CTGFLPT(,@10)                                   0198
*   CTGFLNG(1)=LENGTH(CUMWASS2);    /* LENGTH OF DATA                */
         MVC   CTGFLNG(4,@10),@CF00292                             0199
*   CUMFN1=ENTUPGD;                 /* UPGRADE COMBINATION NAME      */
         MVC   CUMFN1(8,@05),@CC00068                              0200
*   CUMWTYP=DATAID;                 /* DATA ENTRY TYPE               */
         MVI   CUMWTYP(@05),C'D'                                   0201
*   CUMWCI=CUMDCI;                  /* DATA CI NO.                   */
         MVC   CUMWCI(3,@05),CUMDCI(@05)                           0202
*   CUMWTYP2=INDEXID;               /* INDEX ENTRY TYPE              */
         MVI   CUMWTYP2(@05),C'I'                                  0203
*   CUMWCI2=CUMICI;                 /* INDEX CI NO.                  */
*                                                                  0204
         MVC   CUMWCI2(3,@05),CUMICI(@05)                          0204
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLAV                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0205
*Z024MOD:                                                          0205
*   CALL IGGPMOD;                   /* CALL MODIFY TO ADD THE UPGRADE
*                                      ASSOCIATION                   */
Z024MOD  L     @15,@CV00037                                        0205
         BALR  @14,@15                                             0205
*   IF CUMWREL>MAXUPGD&CCACD1=RCS THEN/* IF MAX UPGRADE ASSOCIATIONS
*                                      REACHED                       */
         L     @10,CCACUMPL(,R11)                                  0206
         LH    @05,CUMWREL(,@10)                                   0206
         CH    @05,@CH00070                                        0206
         BNH   @RF00206                                            0206
         CLI   CCACD1(R11),0                                       0206
         BNE   @RF00206                                            0206
*     DO;                           /* BACKOUT ADDED ASSOC           */
*       CTGFLDAT(1)=CTGFLDAT(1)&&CTGFLDAT(1);/* INDICATE DELETE      */
         L     @05,CCAMCPL(,R11)                                   0208
         L     @05,CTGFIELD(,@05)                                  0208
         XC    CTGFLDAT(8,@05),CTGFLDAT(@05)                       0208
*       CTGFLCHN=ADDR(CTGFL)+LENGTH(CTGFL)+LENGTH(CTGFLDAT);/* CHAIN
*                                      TEST FPL                      */
         LA    @15,24                                              0209
         ALR   @15,@05                                             0209
         ST    @15,CTGFLCHN(,@05)                                  0209
*       CTGFLCHN->CTGFLDNM=ADDR(CUMFN2);/* SET ADDR OF NAME          */
         LA    @05,CUMFN2(,@10)                                    0210
         ST    @05,CTGFLDNM(,@15)                                  0210
*       CTGFLCHN->CTGFLDNO=1;       /* NO. FPL DATA PTRS             */
         MVI   CTGFLDNO(@15),X'01'                                 0211
*       CTGFLCHN->CTGFLDCD=CTGFLDEQ;/* SET FPL TEST CONDITION        */
         MVI   CTGFLDCD(@15),X'80'                                 0212
*       CTGFLCHN->CTGFLDGC=ASSOCGC; /* SET ASSOCIATION GRP CODE      */
         MVI   CTGFLDGC(@15),X'02'                                 0213
*       CUMFN2=RELREPNO;            /* SET FIELD NAME                */
         MVC   CUMFN2(8,@10),@CC00058                              0214
*       CTGFLCHN->CTGFLNG(1)=LENGTH(CUMWREL);/* SET LENGTH RELREPNO  */
         MVC   CTGFLNG(4,@15),@CF00072                             0215
*       CTGFLCHN->CTGFLPT(1)=ADDR(CUMWREL);/* SET RELREPNO ADDR      */
*                                                                  0216
         LA    @10,CUMWREL(,@10)                                   0216
         ST    @10,CTGFLPT(,@15)                                   0216
*       /*************************************************************/
*       /*                                                           */
*       /* CALL TO MODULE IGG0CLAV                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0217
*Z025MOD:                                                          0217
*       CALL IGGPMOD;               /* DELETE ADDED ASSOCIATION      */
Z025MOD  L     @15,@CV00037                                        0217
         BALR  @14,@15                                             0217
*       IF CCACD1=RCS THEN          /* IF NO BACKOUT ERR             */
*                                                                  0218
         CLI   CCACD1(R11),0                                       0218
         BNE   @RF00218                                            0218
*         /***********************************************************/
*         /*                                                         */
*         /* SET CCAMODID='B1', CCAREASN=RSN2, CCARETRN=RCMXGRP      */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0219
*         CCAPROB=IGGSEGID+(RSN2*K256)+RCMXGRP;/* SET ERROR CODE     */
         MVC   CCAPROB(4,R11),@CF03003                             0219
*       ELSE                                                       0220
*         ;                         /* BACKOUT ERROR                 */
@RF00218 DS    0H                                                  0221
*     END;                          /* END BACKOUT                   */
*   ELSE                                                           0222
*     ;                             /* MAX NOT REACHED               */
@RF00206 DS    0H                                                  0223
*Z027UEND:                                                         0223
*   CALL IGGPUEND;                  /* TERMINATE AND EXIT            */
Z027UEND BAL   @14,IGGPUEND                                        0223
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0224
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPUDEL');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPUDEL'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00002 DS    0H                                                  0227
@EF00002 DS    0H                                                  0227
@ER00002 BR    @14                                                 0227
@PB00002 DS    0H                                                  0228
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0229
*/********************************************************************/
*/*                                                                  */
*/*        IGGPUDEL  -  PROCEDURE  EXTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0229
*IGGPUDEL:                                                         0229
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00003                                            0229
IGGPUDEL DS    0H                                                  0230
*   GEN(ENTRY IGGPUDEL );           /* IDENTIFY ENTRY-POINT SYMBOL   */
         ENTRY IGGPUDEL
*Z028:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z028UDEL:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0231
*    (R5,                                                          0231
*     R6,                                                          0231
*     R7,                                                          0231
*     R8,                                                          0231
*     R9,                                                          0231
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z028     DS    0H                                                  0231
Z028UDEL DS    0H                                                  0232
*   RFY                                                            0232
*    (R2,                                                          0232
*     R3,                                                          0232
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*   GEN(BALR 12,0);                 /* ESTABLISH ADDRESSABILITY      */
         BALR 12,0
*   GEN(LA 15,*-IGGSEGST(0,0));     /* COMPUTE MODULE BASE           */
         LA 15,*-IGGSEGST(0,0)
*   GEN(SR 12,15);                  /* SET BASE TO MODULE BASE     0237
*                                      PROCEDURE                     */
         SR 12,15
*/********************************************************************
**                                                                   *
**   MODULE NAME = IGGPUDEL (IGG0CLB1)                               *
**                                                                   *
**   DESCRIPTIVE NAME = DELETE UPGRADE ASSOCIATION                   *
**                                                                   *
**   FUNCTION =                                                      *
**      THIS SUB-FUNCTION IS EXTERNALLY CALLED BY SUCH CMS ROUTINES  *
**      AS DELETE AND ALTER. IT DELETES UPGRADE ASSOCIATIONS FROM    *
**      THE UPGRADE ENTRY AND ALSO IMPLICITLY DELETES THE UPGRADE    *
**      ENTRY IF IT BECOMES EMPTY. THE CALLER MAY ALSO EXPLICITLY    *
**      REQUEST THAT THE UPGRADE ENTRY BE DELETED EVEN IF IT IS NOT  *
**      EMPTY (FORCE DELETE). THE PROCESSING IS AS FOLLOWS:          *
**       - CALL IGGPUCOM TO VALIDITY CHECK THE CALLER'S INPUT        *
**         PARAMETERS, PERFORM INITIALIZATION, AND READ THE          *
**         UPGRADE ENTRY INTO CORE,  PROVIDED THIS IS NOT A FORCE    *
**         DELETE OF THE UPGRADE ENTRY.                              *
**       - IF THIS IS NOT A FORCE DELETE OF THE UPGRADE ENTRY, CALL  *
**         MODIFY (IGGPMOD) TO DELETE THE CALLER SPECIFIED UPGRADE   *
**         ASSOCIATION. IF THIS DELETE FAILS, INDICATE SO (CUMNE).   *
**         CALL EXTRACT (IGGPEXT) TO DETERMINE WHETHER THE UPGRADE   *
**         ENTRY IS EMPTY. IF NOT EMPTY, RETURN TO THE CALLER.       *
**         OTHERWISE, DO THE NEXT STEP.                              *
**       - IF THIS IS A FORCE DELETE OR IF THE UPGRADE ENTRY IS      *
**         EMPTY, DO THE FOLLOWING COMMON PROCESSING:   IF THE       *
**         BASE DATA RECORD IS NOT ALREADY IN-CORE, READ IT INTO     *
**         CORE. CALL MODIFY TO DELETE THE Y-TYPE ASSOCIATION IN     *
**         THE BASE DATA RECORD WHICH POINTS TO THE UPGRADE ENTRY.   *
**         READ THE BASE UPGRADE RECORD INTO CORE, IF NECESSARY.     *
**         DELETE THE BASE UPGRADE RECORD AND ALL HORIZONTAL         *
**         EXTENSIONS.                                               *
**                                                                   *
**   INPUT =                                                         *
**      THE FOLLOWING FIELDS ARE IN THE CUMPLWA DECLARATION:         *
**        CUMDCI - 3-BYTE AIX DATA CI NUMBER.                        *
**        CUMICI - 3-BYTE AIX INDEX CI NUMBER.                       *
**        CUMFDEL - IF THIS BIT IS SET ON, THE UPGRADE ENTRY IS      *
**          DELETED EVEN IF IT IS NON-EMPTY.                         *
**      COMMON (TO IGGPUADD) INPUT:                                  *
**        CUMNRG - IF THIS FLAG IS SET ON, NO RE-GET OF THE ORIGINAL *
**          BASE ENTRY RECORD IS DONE.                               *
**        THE CALLER MUST PROVIDE CONTIGUOUS CORE FOR THE CATALOG    *
**        UPGRADE MANAGEMENT PARAMETER LIST AND FOR THE CATALOG      *
**        UPGRADE MANAGEMENT WORK AREA (MAPPED BY CUMPLWA). CCACUMPL *
**        MUST CONTAIN THE ADDRESS OF THIS CORE.                     *
**        THE USER MUST SPECIFY AT LEAST 3 FREE RECORD AREAS WHICH   *
**        WILL BE USED BY CATALOG UPGRADE MANAGEMENT TO PERFORM      *
**        ITS I/O OPERATIONS. IN PARTICULAR,                         *
**        CCARABF - ADDRESS OF THE FIRST RECORD AREA BLOCK WHICH     *
**          CATALOG UPGRADE MANAGEMENT MAY POSSIBLY USE.             *
**        CCARABL - ADDRESS OF THE LAST RECORD AREA BLOCK WHICH      *
**          CATALOG UPGRADE MANAGEMENT MAY POSSIBLY USE.             *
**        CCARABB - ADDRESS OF THE RECORD AREA BLOCK WHICH POINTS TO *
**          THE BASE RECORD OF THE ENTRY. THIS BASE RECORD MUST      *
**          HAVE AN ENTRY TYPE OF G, C, OR D. ALSO, CCARABB MUST     *
**          LIE BETWEEN CCARABF AND CCARABL.                         *
**        CCARNUR (WHERE N=1,2,3,4, OR 5) - IF THIS BIT IS SET ON IN *
**          A RECORD AREA BLOCK, IT INDICATES THE ASSOCIATED RECORD  *
**          AREA IS NOT FREE FOR CATALOG UPGRADE MGMT'S USE.         *
**                                                                   *
**   OUTPUT =                                                        *
**      THE FOLLOWING FIELDS ARE IN THE CUMPLWA DECLARATION:         *
**        CUMNE - THIS BIT IS SET ON, IF THE UPGRADE ASSOCIATION TO  *
**          BE DELETED IS NON-EXISTENT OR THE UPGRADE ENTRY ITSELF   *
**          IS NON-EXISTENT.                                         *
**      COMMON (TO IGGPUADD) OUTPUT:                                 *
**        FLAG CCAREUSE IS SET OFF.                                  *
**        THE CONTENTS OF THE USER'S CCAURAB IS SAVED AND RESTORED.  *
**        THE ORIGINAL BASE RECORD REFERENCED BY CCARABB IS          *
**          RESTORED.                                                *
**        IN GENERAL CATALOG UPGRADE MGMT WILL NOT CHANGE INPUT      *
**          FIELDS SO THAT ITS INTERFACE IS COMPLETELY REUSABLE      *
**          AFTER A PARTICULAR CALL.                                 *
**        ERROR CODES - THE MAIN ERROR CODE IS RETURNED IN CCACD1.   *
**          THE POSSIBLE RETURN CODES ARE DESCRIBED BELOW. A         *
**          SECONDARY ERROR CODE IS RETURNED IN CCACD2. THIS FIELD   *
**          WILL CONTAIN THE RETURN CODE RECEIVED BACK FROM GET      *
**          WHEN THE CALLER'S BASE RECORD IS RESTORED. ANY NON-ZERO  *
**          CODE IN CCACD2 IS PROPAGATED TO CCACD1, PROVIDED OF      *
**          COURSE, CCACD1 IS NOT ALREADY NON-ZERO.                  *
**            ANY ERROR CODE RETURNED BY MODIFY OR EXTRACT           *
**            ANY ERROR CODE RETURNED BY THE I/O ROUTINES            *
**              (GET, PDE)                                           *
**            RCINENT - IF THE BASE RECORD ENTRY TYPE IS NOT G, C, OR*
**              D.                                                   *
**            RCINCNPM - IF THE RAB REQUIREMENTS ARE NOT MET.        *
**            RCCATBAL - IF CATALOG UPGRADE MGMT ENCOUNTERS A        *
**              STRUCTURAL ERROR.                                    *
**                                                                   *
******************************************************************** */
*                                                                  0238
*   CUMFUNC=OFF;                    /* INDICATE THIS IS DELETE TO  0238
*                                      IGGPUCOM                      */
*                                                                  0238
         L     @10,CCACUMPL(,R11)                                  0238
         NI    CUMFUNC(@10),B'11011111'                            0238
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL THE COMMON UPGRADE MANAGEMENT INITIALIZATION ROUTINE     */
*   /* (IGGPUCOM) TO PERFORM VALIDITY CHECKING AGAINST THE CALLER'S  */
*   /* INPUT PARAMETERS, TO PERFORM INITIALIZATION, AND TO READ THE  */
*   /* UPGRADE ENTRY INTO CORE. THE UPGRADE ENTRY IS NOT READ INTO   */
*   /* CORE IF THIS IS A FORCE DELETE OF THE UPGRADE ENTRY, SINCE THE*/
*   /* BASE DATA RECORD MUST BE PROCESSED FIRST.                     */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0239
*Z029UCOM:                                                         0239
*   CALL IGGPUCOM;                  /* CALL THE COMMON INITIALIZATION
*                                      ROUTINE                       */
Z029UCOM BAL   @14,IGGPUCOM                                        0239
*   IF CUMNE=ON|CCACD1^=RCS THEN    /* IF THE UPGRADE ENTRY DOES NOT
*                                      EXIST OR AN ERROR OCCURRED  0240
*                                      THEN                          */
         L     @10,CCACUMPL(,R11)                                  0240
         TM    CUMNE(@10),B'01000000'                              0240
         BO    @RT00240                                            0240
         CLI   CCACD1(R11),0                                       0240
         BE    @RF00240                                            0240
@RT00240 DS    0H                                                  0241
*Z030UEND:                                                         0241
*     CALL IGGPUEND;                /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z030UEND BAL   @14,IGGPUEND                                        0241
*   ELSE                                                           0242
*     ;                             /* THE UPGRADE ENTRY EXISTS AND
*                                      THERE WERE NO ERRORS          */
@RF00240 DS    0H                                                  0243
*   IF CUMFDEL=OFF THEN             /* IF THIS IS NOT A FORCE DELETE
*                                      OF THE UPGRADE ENTRY          */
         L     @10,CCACUMPL(,R11)                                  0243
         TM    CUMFDEL(@10),B'10000000'                            0243
         BNZ   @RF00243                                            0243
*     DO;                           /* NORMAL DELETE OF UPGRADE    0244
*                                      ASSOCIATION                   */
*                                                                  0244
*       /*************************************************************/
*       /*                                                           */
*       /* BUILD THE MODIFY INTERFACE TO DELETE THE UPGRADE          */
*       /* ASSOCIATION SPECIFIED BY THE CALLER                       */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0245
*       RFY                                                        0245
*         CTGPL BASED(CCAMCPL);     /* MAP CPL AREA                  */
*       RFY                                                        0246
*         CTGFL BASED(CTGFIELD(1)); /* MAP FIRST FPL FOR           0246
*                                      INITIALIZATION                */
*       CTGFLNG(1)=0;               /* NO DATA LENGTH                */
         SLR   @05,@05                                             0247
         L     @01,CCAMCPL(,R11)                                   0247
         L     @15,CTGFIELD(,@01)                                  0247
         ST    @05,CTGFLNG(,@15)                                   0247
*       CTGFLPT(1)=0;               /* NO DATA ADDRESS               */
         ST    @05,CTGFLPT(,@15)                                   0248
*       CUMFN1=UPGRADE;             /* INITLZ FIELD NAME FOR DELETION*/
*                                                                  0249
         MVC   CUMFN1(8,@10),@CC00066                              0249
*       /*************************************************************/
*       /*                                                           */
*       /* INITIALIZE THE TEST FPL                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0250
*                                   /* DELETE THE UPGRADE ASSOCIATION
*                                      WHICH TESTS EQUAL TO THE    0250
*                                      CALLER SPECIFIED DATA CI    0250
*                                      NUMBER                        */
*       CTGFLCHN->CTGFLNG(1)=LENGTH(CUMWCI);/* SET CI NUMBER LENGTH  */
         L     @05,CTGFLCHN(,@15)                                  0250
         MVC   CTGFLNG(4,@05),@CF00074                             0250
*       CTGFLCHN->CTGFLPT(1)=ADDR(CUMWCI);/* SET CI NUMBER ADDR      */
         LA    @15,CUMWCI(,@10)                                    0251
         ST    @15,CTGFLPT(,@05)                                   0251
*       CUMFN2=NAME;                /* INITLZ TEST FIELD NAME        */
         MVC   CUMFN2(8,@10),@CC00064                              0252
*       CUMWCI=CUMDCI;              /* INITLZ AIX CI NUMBER AS TEST
*                                      DATA                          */
*                                                                  0253
         MVC   CUMWCI(3,@10),CUMDCI(@10)                           0253
*       /*************************************************************/
*       /*                                                           */
*       /* CALL TO MODULE IGG0CLAV                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0254
*Z031MOD:                                                          0254
*       CALL IGGPMOD;               /* CALL MODIFY TO DELETE THE   0254
*                                      UPGRADE ASSOCIATION           */
Z031MOD  L     @15,@CV00037                                        0254
         BALR  @14,@15                                             0254
*       IF CCACD1^=RCS THEN         /* IF AN ERROR OCCURRED IN THE 0255
*                                      MODIFY-DELETE                 */
         CLI   CCACD1(R11),0                                       0255
         BE    @RF00255                                            0255
*Z032UEND:                                                         0256
*         CALL IGGPUEND;            /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z032UEND BAL   @14,IGGPUEND                                        0256
*       ELSE                                                       0257
*         ;                         /* NO ERROR IN THE MODIFY- DELETE*/
@RF00255 DS    0H                                                  0258
*       IF CTGFIELD(1)->CTGFLCHN->CTGFLDTS=ON THEN/* IF THE UPGRADE
*                                      ASSOCIATION DELETE DID NOT  0258
*                                      TAKE PLACE                    */
         L     @10,CCAMCPL(,R11)                                   0258
         L     @10,CTGFIELD(,@10)                                  0258
         L     @10,CTGFLCHN(,@10)                                  0258
         TM    CTGFLDTS(@10),B'00000001'                           0258
         BNO   @RF00258                                            0258
*         CUMNE=ON;                 /* INDICATE THE UPGRADE        0259
*                                      ASSOCIATION NON-EXISTENT      */
         L     @10,CCACUMPL(,R11)                                  0259
         OI    CUMNE(@10),B'01000000'                              0259
*       ELSE                                                       0260
*         ;                         /* THE UPGRADE ASSOCIATION WAS 0260
*                                      DELETED                       */
*                                                                  0260
@RF00258 DS    0H                                                  0261
*       /*************************************************************/
*       /*                                                           */
*       /* BUILD THE EXTRACT INTERFACE TO SEE IF THE UPGRADE ENTRY IS*/
*       /* EMPTY.                                                    */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0261
*       CTGFLDGC=ASSOCGC;           /* SET ASSOCIATION GROUP CODE    */
         L     @10,CCAMCPL(,R11)                                   0261
         L     @05,CTGFIELD(,@10)                                  0261
         MVI   CTGFLDGC(@05),X'02'                                 0261
*       CTGFLCHN=0;                 /* NO TEST FPLS                  */
         SLR   @15,@15                                             0262
         ST    @15,CTGFLCHN(,@05)                                  0262
*       CUMFN1=RELREPNO;            /* RETRIEVE ALL UPGRADE        0263
*                                      ASSOCIATION SEQUENCE NOS.     */
         L     @05,CCACUMPL(,R11)                                  0263
         MVC   CUMFN1(8,@05),@CC00058                              0263
*       WKALEN1=LENGTH(WKALEN1)+    /* INDICATE MINIMUM              */
*           LENGTH(WKALEN2);        /* POSSIBLE WORK AREA LENGTH TO
*                                      FORCE ERROR CODE IF UPGRADE 0264
*                                      ENTRY IS NOT EMPTY            */
*                                                                  0264
         L     @10,CTGWKA(,@10)                                    0264
         MVC   WKALEN1(2,@10),@CH00103                             0264
*       /*************************************************************/
*       /*                                                           */
*       /* CALL TO MODULE IGG0CLAZ                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0265
*Z033EXT:                                                          0265
*       CALL IGGPEXT;               /* CALL EXTRACT                  */
Z033EXT  L     @15,@CV00038                                        0265
         BALR  @14,@15                                             0265
*       IF CCACD1=RCVLSZ THEN       /* IF WORK AREA SIZE IS        0266
*                                      INSUFFICIENT, THERE EXIST   0266
*                                      UPGRADE ASSOCIATION SEQUENCE
*                                      NOS. AND THE UPGRADE ENTRY IS
*                                      NOT EMPTY                     */
         CLI   CCACD1(R11),40                                      0266
         BNE   @RF00266                                            0266
*         DO;                       /* FINISHED NORMAL DELETE        */
*                                                                  0267
*           /*********************************************************/
*           /*                                                       */
*           /* CLEAR CCAMODID, CCAREASN, AND CCARETRN                */
*           /*                                                       */
*           /*********************************************************/
*                                                                  0268
*           CCAPROB=IGNOREIT;       /* CLEAR EXPECTED RETURN CODE    */
         SLR   @10,@10                                             0268
         ST    @10,CCAPROB(,R11)                                   0268
*Z035UEND:                                                         0269
*           CALL IGGPUEND;          /* TERMINATION PROCESSING AND  0269
*                                      EXIT                          */
Z035UEND BAL   @14,IGGPUEND                                        0269
*         END;                      /* END FINISHED                  */
*       ELSE                                                       0271
*         ;                         /* NOT WORK AREA SIZE          0271
*                                      INSUFFICIENT ERROR CODE       */
@RF00266 DS    0H                                                  0272
*       IF CCACD1^=RCS THEN         /* IF ERROR                      */
         CLI   CCACD1(R11),0                                       0272
         BE    @RF00272                                            0272
*Z036UEND:                                                         0273
*         CALL IGGPUEND;            /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z036UEND BAL   @14,IGGPUEND                                        0273
*       ELSE                                                       0274
*         ;                         /* NO ERROR - UPGRADE ENTRY IS 0274
*                                      EMPTY, SO WE WILL FALL INTO 0274
*                                      THE FOLLOWING FORCE DELETE  0274
*                                      CODE                          */
@RF00272 DS    0H                                                  0275
*     END;                          /* END NORMAL DELETE             */
*   ELSE                                                           0276
*     ;                             /* FORCE DELETE                  */
*                                                                  0276
@RF00243 DS    0H                                                  0277
*   /*****************************************************************/
*   /*                                                               */
*   /* THE SUB-FUNCTION IGGPUCOM ATTEMPTS TO USE RECORD AREAS        */
*   /* ECONOMICALLY SO IT PLACES THE BASE DATA RECORD AND BASE       */
*   /* UPGRADE RECORD INTO DIFFERENT RECORD AREAS. HOWEVER, BECAUSE  */
*   /* MODIFY AND EXTRACT MAY USE ALL THREE RECORD AREAS, CHECKS MUST*/
*   /* BE MADE TO INSURE THAT THEY ARE STILL IN-CORE. IN THIS        */
*   /* SUB-FUNCTION THIS IS DONE HERE AND ALSO JUST BEFORE THE       */
*   /* UPGRADE ENTRY IS DELETED.                                     */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0277
*   RFY                                                            0277
*     CCARAB BASED(CUMRABD);        /* MAP RECORD AREA OF DATA RECORD*/
*   IF CUMDY^=RCDCINO THEN          /* IF DATA RECORD OF BASE CLUSTER
*                                      IS NOT IN-CORE                */
         L     @10,CCACUMPL(,R11)                                  0278
         L     @05,CUMRABD(,@10)                                   0278
         L     @01,CCACPE1(,@05)                                   0278
         CLC   CUMDY(3,@10),RCDCINO(@01)                           0278
         BE    @RF00278                                            0278
*     DO;                           /* RETRIEVE DATA RECORD          */
*       CCAURAB=CUMRABD;            /* SET RAB TO USE                */
         ST    @05,CCAURAB(,R11)                                   0280
*       CCASRID=CUMDY;              /* SET DATA RCD CI NO.           */
         MVC   CCASRID(3,R11),CUMDY(@10)                           0281
*       DO;                                                        0282
*         CCAF1KEY=OFF;             /* GET BY CI NO.                 */
*         CCAF1KGE=OFF;             /* NOT GET NEXT                  */
*         CCAF1UP=ON;               /* GET FOR UPD                   */
*                                                                  0285
         OI    CCAF1UP(R11),B'00000010'                            0285
         NI    CCAF1KEY(R11),B'11100111'                           0285
*         /***********************************************************/
*         /*                                                         */
*         /* CALL TO MODULE IGG0CLBI                                 */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0286
*Z038GET:                                                          0286
*         CALL IGGPGET;             /* GET RECORD                    */
Z038GET  L     @15,@CV00036                                        0286
         BALR  @14,@15                                             0286
*       END;                        /* GET DATA RECORD               */
*       IF CCACD1^=RCS THEN         /* IF ERROR IN GET               */
         CLI   CCACD1(R11),0                                       0288
         BE    @RF00288                                            0288
*Z039UEND:                                                         0289
*         CALL IGGPUEND;            /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z039UEND BAL   @14,IGGPUEND                                        0289
*       ELSE                                                       0290
*         ;                         /* NO ERROR                      */
@RF00288 DS    0H                                                  0291
*     END;                          /* END RETRIEVE                  */
*   ELSE                                                           0292
*     ;                             /* DATA RECORD OF BASE CLUSTER IS
*                                      STILL IN-CORE                 */
*                                                                  0292
@RF00278 DS    0H                                                  0293
*   /*****************************************************************/
*   /*                                                               */
*   /* BUILD THE MODIFY-DELETE INTERFACE TO DELETE THE Y ASSOCIATION */
*   /* FROM THE BASE DATA ENTRY.                                     */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0293
*   CCARABB=CUMRABD;                /* POINT TO BASE RAB FOR BASE  0293
*                                      DATA RECORD                   */
         L     @10,CCACUMPL(,R11)                                  0293
         MVC   CCARABB(4,R11),CUMRABD(@10)                         0293
*   CTGFLNG(1)=0;                   /* NO DATA LENGTH                */
         SLR   @05,@05                                             0294
         L     @01,CCAMCPL(,R11)                                   0294
         L     @15,CTGFIELD(,@01)                                  0294
         ST    @05,CTGFLNG(,@15)                                   0294
*   CTGFLPT(1)=0;                   /* NO DATA ADDRESS               */
         ST    @05,CTGFLPT(,@15)                                   0295
*   CUMFN1=NAMEDS;                  /* INITLZ FIELD NAME FOR DELETION*/
         MVC   CUMFN1(8,@10),@CC00060                              0296
*   R2=ADDR(CTGFL)+LENGTH(CTGFL)    /* ADDR OF TEST FPL              */
*       +LENGTH(CTGFLDAT);                                         0297
         LA    R2,24                                               0297
         ALR   R2,@15                                              0297
*   CTGFLCHN=R2;                    /* CHAIN TEST FPL                */
*                                                                  0298
         ST    R2,CTGFLCHN(,@15)                                   0298
*   /*****************************************************************/
*   /*                                                               */
*   /* INITIALIZE THE TEST FPL                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0299
*   RFY                                                            0299
*     CTGFL BASED(R2);              /* MAP THE TEST FPL ONE FPL    0299
*                                      LENGTH/ADDR PAIR DELETE THE 0299
*                                      NORMAL ASSOCIATION WHICH TESTS
*                                      EQUAL TO THE UPGRADE CI NUMBER*/
*   CTGFLNG(1)=LENGTH(CUMWCI);      /* SET CI NUMBER LENGTH          */
         MVC   CTGFLNG(4,R2),@CF00074                              0300
*   CTGFLPT(1)=ADDR(CUMWCI);        /* SET CI NUMBER ADDR            */
         LA    @05,CUMWCI(,@10)                                    0301
         ST    @05,CTGFLPT(,R2)                                    0301
*   CUMFN2=NAME;                    /* INITLZ TEST FIELD NAME        */
         MVC   CUMFN2(8,@10),@CC00064                              0302
*   CUMWCI=CUMSRID;                 /* INITLZ UPGRADE CI NUMBER AS 0303
*                                      TEST DATA                     */
*                                                                  0303
         MVC   CUMWCI(3,@10),CUMSRID(@10)                          0303
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLAV                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0304
*Z040MOD:                                                          0304
*   CALL IGGPMOD;                   /* CALL MODIFY TO DELETE THE   0304
*                                      ASSOCIATION IN THE BASE DATA
*                                      ENTY POINTING TO THE UPGRADE
*                                      ENTRY                         */
Z040MOD  L     @15,@CV00037                                        0304
         BALR  @14,@15                                             0304
*   IF CCACD1^=RCS THEN             /* IF AN ERROR OCCURRED IN     0305
*                                      MODIFY-DELETE                 */
         CLI   CCACD1(R11),0                                       0305
         BE    @RF00305                                            0305
*Z041UEND:                                                         0306
*     CALL IGGPUEND;                /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z041UEND BAL   @14,IGGPUEND                                        0306
*   ELSE                                                           0307
*     ;                             /* NO ERROR IN THE MODIFY- DELETE*/
@RF00305 DS    0H                                                  0308
*   RFY                                                            0308
*     CCARAB BASED(CUMRAB2);        /* MAP RAB ASSOCIATED WITH     0308
*                                      UPGRADE ENTRY                 */
*   CCAURAB=CUMRAB2;                /* SET UPGRADE RAB               */
         L     @10,CCACUMPL(,R11)                                  0309
         L     @05,CUMRAB2(,@10)                                   0309
         ST    @05,CCAURAB(,R11)                                   0309
*   IF RCDCINO^=CUMSRID THEN        /* IF UPGRADE ENTRY IS NOT     0310
*                                      IN-CORE                       */
         L     @05,CCACPE1(,@05)                                   0310
         CLC   RCDCINO(3,@05),CUMSRID(@10)                         0310
         BE    @RF00310                                            0310
*     DO;                           /* RETRIEVE UPGRADE ENTRY        */
*       CCASRID=CUMSRID;            /* SET UPGRADE CI NUMBER         */
         MVC   CCASRID(3,R11),CUMSRID(@10)                         0312
*       DO;                                                        0313
*         CCAF1KEY=OFF;             /* GET BY CI NO.                 */
*         CCAF1KGE=OFF;             /* NOT GET NEXT                  */
*         CCAF1UP=ON;               /* GET FOR UPD                   */
*                                                                  0316
         OI    CCAF1UP(R11),B'00000010'                            0316
         NI    CCAF1KEY(R11),B'11100111'                           0316
*         /***********************************************************/
*         /*                                                         */
*         /* CALL TO MODULE IGG0CLBI                                 */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0317
*Z043GET:                                                          0317
*         CALL IGGPGET;             /* GET RECORD                    */
Z043GET  L     @15,@CV00036                                        0317
         BALR  @14,@15                                             0317
*       END;                        /* GET UPGRADE ENTRY             */
*       IF CCACD1^=RCS THEN         /* IF AN ERROR OCCURRED IN GET   */
         CLI   CCACD1(R11),0                                       0319
         BE    @RF00319                                            0319
*Z044UEND:                                                         0320
*         CALL IGGPUEND;            /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z044UEND BAL   @14,IGGPUEND                                        0320
*       ELSE                                                       0321
*         ;                         /* NO ERROR IN GET               */
@RF00319 DS    0H                                                  0322
*     END;                          /* END RETRIEVE                  */
*   ELSE                                                           0323
*     ;                             /* UPGRADE ENTRY IN-CORE         */
@RF00310 DS    0H                                                  0324
*   RFY                                                            0324
*     CTGFL BASED(CTGFIELD(1));     /* MAP FIRST FPL                 */
*   CCARABB=CUMRAB2;                /* SET BASE RAB                  */
         L     @10,CCACUMPL(,R11)                                  0325
         MVC   CCARABB(4,R11),CUMRAB2(@10)                         0325
*   CUMFN1=UPGRADE;                 /* UPGRADE FIELD NAME            */
         MVC   CUMFN1(8,@10),@CC00066                              0326
*   CTGFLCHN=0;                     /* NO TEST FPL                   */
*                                                                  0327
         L     @10,CCAMCPL(,R11)                                   0327
         L     @10,CTGFIELD(,@10)                                  0327
         SLR   @05,@05                                             0327
         ST    @05,CTGFLCHN(,@10)                                  0327
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLAV                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0328
*Z045MOD:                                                          0328
*   CALL IGGPMOD;                   /* DELETE ALL UPGRADE ASSOC      */
Z045MOD  L     @15,@CV00037                                        0328
         BALR  @14,@15                                             0328
*A1:                                                               0329
*   CUMSRID=EXTGOCI;                /* SAVE EXTENSION CI NUMBER      */
A1       L     @10,CCACUMPL(,R11)                                  0329
         L     @05,CUMRAB2(,@10)                                   0329
         L     @05,CCACPE2(,@05)                                   0329
         MVC   CUMSRID(3,@10),EXTGOCI(@05)                         0329
*   DO;                                                            0330
*                                                                  0330
*     /***************************************************************/
*     /*                                                             */
*     /* CALL TO MODULE IGG0CLAG                                     */
*     /*                                                             */
*     /***************************************************************/
*                                                                  0331
*Z047PDE:                                                          0331
*     CALL IGGPPDE;                 /* PUT-DELETE RECORD             */
Z047PDE  L     @15,@CV00039                                        0331
         BALR  @14,@15                                             0331
*   END;                            /* DELETE UPGRADE ENTRY          */
*   IF CCACD1^=RCS THEN             /* IF AN ERROR OCCURRED IN     0333
*                                      PUT-DELETE                    */
         CLI   CCACD1(R11),0                                       0333
         BE    @RF00333                                            0333
*Z048UEND:                                                         0334
*     CALL IGGPUEND;                /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z048UEND BAL   @14,IGGPUEND                                        0334
*   ELSE                                                           0335
*     ;                             /* NO PUT-DELETE ERROR           */
@RF00333 DS    0H                                                  0336
*   IF CUMSRID^=0 THEN              /* IF THERE EXISTS A HORIZONTAL
*                                      EXTENSION                     */
         L     @10,CCACUMPL(,R11)                                  0336
         SLR   @05,@05                                             0336
         ICM   @05,7,CUMSRID(@10)                                  0336
         LTR   @05,@05                                             0336
         BZ    @RF00336                                            0336
*     DO;                           /* MUST DELETE EXTENSIONS        */
*       CCASRID=CUMSRID;            /* SET HORIZONTAL EXTENSION CI 0338
*                                      NUMBER                        */
         STCM  @05,7,CCASRID(R11)                                  0338
*       DO;                                                        0339
*         CCAF1KEY=OFF;             /* GET BY CI NO.                 */
*         CCAF1KGE=OFF;             /* NOT GET NEXT                  */
*         CCAF1UP=ON;               /* GET FOR UPD                   */
*                                                                  0342
         OI    CCAF1UP(R11),B'00000010'                            0342
         NI    CCAF1KEY(R11),B'11100111'                           0342
*         /***********************************************************/
*         /*                                                         */
*         /* CALL TO MODULE IGG0CLBI                                 */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0343
*Z050GET:                                                          0343
*         CALL IGGPGET;             /* GET RECORD                    */
Z050GET  L     @15,@CV00036                                        0343
         BALR  @14,@15                                             0343
*       END;                        /* GET EXTENSION                 */
*       IF CCACD1^=RCS THEN         /* IF GET ERROR                  */
         CLI   CCACD1(R11),0                                       0345
         BE    @RF00345                                            0345
*Z051UEND:                                                         0346
*         CALL IGGPUEND;            /* PERFORM TERMINATION PROCESSING
*                                      AND EXIT                      */
Z051UEND BAL   @14,IGGPUEND                                        0346
*       ELSE                                                       0347
*         ;                         /* NO GET ERROR                  */
@RF00345 DS    0H                                                  0348
*       GOTO A1;                    /* PROCESS EXTENSION DELETE      */
         B     A1                                                  0348
*     END;                          /* END MUST                      */
*   ELSE                                                           0350
*     ;                             /* NO HORIZONTAL EXTENSION       */
@RF00336 DS    0H                                                  0351
*Z052UEND:                                                         0351
*   CALL IGGPUEND;                  /* TERMINATE AND EXIT            */
Z052UEND BAL   @14,IGGPUEND                                        0351
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0352
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPUCOM');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPUCOM'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00003 DS    0H                                                  0355
@EF00003 DS    0H                                                  0355
@ER00003 BR    @14                                                 0355
@PB00003 DS    0H                                                  0356
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0357
*/********************************************************************/
*/*                                                                  */
*/*        IGGPUCOM  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0357
*IGGPUCOM:                                                         0357
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00004                                            0357
IGGPUCOM DS    0H                                                  0358
*Z053:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z053UCOM:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0358
*    (R5,                                                          0358
*     R6,                                                          0358
*     R7,                                                          0358
*     R8,                                                          0358
*     R9,                                                          0358
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z053     DS    0H                                                  0358
Z053UCOM DS    0H                                                  0359
*   RFY                                                            0359
*    (R2,                                                          0359
*     R3,                                                          0359
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA     0361
*                                      PROCEDURE                     */
         LA 13,12(13)
*/********************************************************************
**                                                                   *
**   MODULE NAME = IGGPUCOM(IGG0CLB1)                                *
**                                                                   *
**   DESCRIPTIVE NAME = CATALOG UPGRADE MGMT COMMON INITIALIZATION   *
**                       ROUTINE                                     *
**                                                                   *
**   FUNCTION =                                                      *
**      THIS SUB-FUNCTION PERFORMS INITIAL COMMON PROCESSING FOR     *
**      THE CATALOG UPGRADE ASSOCIATION ADD ROUTINE (IGGPUADD) AND   *
**      THE CATALOG UPGRADE ASSOCIATION DELETE ROUTINE (IGGPUDEL).   *
**      THE PROCESSING CONSISTS OF THE FOLLOWING:                    *
**       - INITIALIZE BASIC CATALOG UPGRADE MGMT WORK AREA POINTERS  *
**       - VALIDITY CHECK THAT THE INPUT PARAMETERS TO IGGPUADD      *
**         (IGGPUDEL) ARE CORRECT.                                   *
**       - READ IN THE UPGRADE ENTRY (RECORD TYPE =Y)                *
**                                                                   *
**   INPUT =                                                         *
**      SEE THE COMMON INPUT SECTION OF THE IGGPUADD (IGGPUDEL)      *
**        PROLOGUES                                                  *
**      CUMFDEL- INDICATES FORCE DELETE OF UPGRADE ENTRY             *
**      CUMFUNC- IF ON, INDICATES CATALOG UPGRADE ASSOCIATION ADD.   *
**        OTHERWISE, CATALOG UPGRADE ASSOCIATION DELETE.             *
**                                                                   *
**                                                                   *
**   OUTPUT =                                                        *
**      CUMWA- CATALOG UPGRADE MGMT WORK AREA ZEROED                 *
**      CUMURAB, CUMRABB, CUMBCI- USED TO SAVE CCAURAB, CCARABB,     *
**        BASE RECORD CI NUMBER                                      *
**      CUMDUP- DUPLICATE UPGRADE ASSOCIATION INDICATOR SET OFF      *
**      CUMRAB1, CUMRAB2- SET TO THE ADDR OF FREE RABS               *
**      CUMNE- SET ON, IF THE UPGRADE ENTRY IS NON-EXISTENT. THE     *
**        FOLLOWING FIELDS ARE ALSO SET:                             *
**          CUMRABD- ADDR OF RAB WHICH REFERENCES BASE D ENTRY       *
**          CUMDY- CI NUMBER OF BASE D ENTRY                         *
**          CCARABB- ADDR OF RAB WHICH REFERENCES BASE D ENTRY       *
**      CUMNE- SET OFF, IF THE UPGRADE ENTRY IS EXISTENT.            *
**        IF CUMFDEL IS ON, THE FOLLOWING FIELDS ARE ALSO SET:       *
**          CUMRABD- ADDR OF RAB WHICH REFERENCES BASE D ENTRY       *
**          CUMDY- CI NUMBER OF BASE D ENTRY                         *
**          CUMSRID- CI NUMBER OF UPGRADE ENTRY                      *
**          CCARABB- ADDR OF RAB WHICH REFERENCES BASE D ENTRY       *
**        IF CUMFDEL IS OFF, THE FOLLOWING FIELDS ARE ALSO SET:      *
**          CUMRABD- ADDR OF RAB WHICH REFERENCES BASE D ENTRY       *
**          CUMDY- CI NUMBER OF BASE D ENTRY                         *
**          CUMSRID- CI NUMBER OF UPGRADE ENTRY                      *
**          CUMRAB2- ADDR OF RAB WHICH REFERENCES UPGRADE ENTRY      *
**          CCARABB- ADDR OF RAB WHICH REFERENCES UPGRADE ENTRY      *
**                                                                   *
**      CCACD1- ANY ERROR CODE RETURNED BY MODIFY OR EXTRACT         *
**              RCINENT, IF THE BASE RECORD ENTRY TYPE IS NOT G, C,  *
**                OR D                                               *
**              RCINCNPM, IF THE RAB REQUIREMENTS ARE NOT MET        *
**              RCCATBAL, IF CATALOG UPGRADE MGMT ENCOUNTERS A       *
**                STRUCTURAL ERROR                                   *
**                                                                   *
******************************************************************** */
*                                                                  0362
*   RFY                                                            0362
*     CCARAB BASED(CCARABB);        /* MAP RAB WHICH REFERENCES    0362
*                                      CALLER'S BASE RECORD          */
*                                                                  0362
*   /*****************************************************************/
*   /*                                                               */
*   /* THE FOLLOWING CODE SAVES SOME OF THE CALLER'S INPUT PARAMETERS*/
*   /* (RESTORED BY SUB-FUNCTION IGGPUEND), INITIALIZES OUTPUT       */
*   /* PARAMETER BITS AND BASIC WORK AREA POINTERS USED BY CATALOG   */
*   /* UPGRADE MANAGEMENT.                                           */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0363
*   CUMWA=CUMWA&&CUMWA;             /* ZERO THE CATALOG UPGRADE    0363
*                                      MANAGEMENT WORK AREA          */
         L     @10,CCACUMPL(,R11)                                  0363
         XC    CUMWA(152,@10),CUMWA(@10)                           0363
*   CUMURAB=CCAURAB;                /* SAVE CALLER'S CCAURAB         */
         MVC   CUMURAB(4,@10),CCAURAB(R11)                         0364
*   CUMRABB=CCARABB;                /* SAVE CALLER'S CCARABB         */
         L     @05,CCARABB(,R11)                                   0365
         ST    @05,CUMRABB(,@10)                                   0365
*   CUMBCI=RCDCINO;                 /* SAVE CALLER'S BASE RCD CI   0366
*                                      NUMBER                        */
         L     @05,CCACPE1(,@05)                                   0366
         MVC   CUMBCI(3,@10),RCDCINO(@05)                          0366
*   CUMDUP=OFF;                     /* ASSUME NO DUPLICATE UPGRADE 0367
*                                      ASSOCIATION                   */
*   CUMNE=OFF;                      /* ASSUME UPGRADE ENTRY IS     0368
*                                      EXISTENT AND UPGRADE        0368
*                                      ASSOCIATION (FOR DELETION) IS
*                                      EXISTENT                      */
*                                                                  0368
         NI    CUMDUP(@10),B'00111111'                             0368
*   /*****************************************************************/
*   /*                                                               */
*   /* THE FOLLOWING CODE VALIDITY CHECKS THE COMMON INPUT PARAMETERS*/
*   /* SUPPLIED TO THE UPGRADE ASSOCIATION ADD SUB-FUNCTION          */
*   /* (IGGPUADD) AND THE UPGRADE ASSOCIATION DELETE SUB-FUNCTION    */
*   /* (IGGPUDEL).                                                   */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0369
*   IF CCACD1^=RCS THEN             /* CHECK WHETHER CCACD1 IS     0369
*                                      NON-ZERO                      */
         CLI   CCACD1(R11),0                                       0369
         BE    @RF00369                                            0369
*     CALL EXIT;                    /* IT IS, CALLER ERROR, EXIT     */
         BAL   @14,EXIT                                            0370
*   ELSE                                                           0371
*     ;                             /* CCACD1 IS ZERO                */
@RF00369 DS    0H                                                  0372
*   IF^(RCDID=DATAID|RCDID=CLUSID|  /* INSURE THAT THE BASE RCD      */
*       RCDID=AIXID) THEN           /* HAS THE DATA, CLUSTER, OR AIX
*                                      RECORD TYPE                   */
         L     @10,CCARABB(,R11)                                   0372
         L     @10,CCACPE1(,@10)                                   0372
         CLI   RCDID(@10),C'D'                                     0372
         BE    @RF00372                                            0372
         CLI   RCDID(@10),C'C'                                     0372
         BE    @RF00372                                            0372
         CLI   RCDID(@10),C'G'                                     0372
         BE    @RF00372                                            0372
*     DO;                           /* IF NOT, SET AN ERROR CODE     */
*                                                                  0373
*       /*************************************************************/
*       /*                                                           */
*       /* SET CCAMODID='B1', CCAREASN=RSN40, CCARETRN=RCINENT       */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0374
*       CCAPROB=IGGSEGID+(RSN40*K256)+RCINENT;/* INVALID ENTRY TYPE  */
         MVC   CCAPROB(4,R11),@CF03006                             0374
*       CALL EXIT;                  /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0375
*     END;                          /* END ERROR TYPE PROCESSING     */
*   ELSE                                                           0377
*     ;                             /* BASE RCD HAS VALID RCD TYPE   */
*                                                                  0377
@RF00372 DS    0H                                                  0378
*   /*****************************************************************/
*   /*                                                               */
*   /* INSURE THERE ARE AT LEAST 3 FREE RABS BETWEEN CCARABF AND     */
*   /* CCARABL, CCARABB IS FREE, AND LIES BETWEEN CCARABF AND        */
*   /* CCARABL.                                                      */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0378
*   R2=0;                           /* USE THIS REG TO COUNT FREE  0378
*                                      RABS                          */
         SLR   R2,R2                                               0378
*   RFY                                                            0379
*     CCARAB BASED(R3);             /* MAP EACH RAB TO CHECK ITS FREE
*                                      STATUS                        */
*   DO R3=CCARABF TO CCARABL        /* CHECK EACH RAB SUPPLIED       */
*         BY LENGTH(CCARAB);        /* BY THE CALLER                 */
         L     R3,CCARABF(,R11)                                    0380
         B     @DE00380                                            0380
@DL00380 DS    0H                                                  0381
*     IF CCARBUR=OFF THEN           /* IF RAB IS FREE                */
         TM    CCARBUR(R3),B'10000000'                             0381
         BNZ   @RF00381                                            0381
*       R2=R2+1;                    /* RAISE COUNT BY 1              */
         AH    R2,@CH00046                                         0382
*     ELSE                                                         0383
*       ;                           /* RAB IS IN-USE BY CALLER       */
@RF00381 DS    0H                                                  0384
*   END;                            /* END CHECK                     */
         AH    R3,@CH00154                                         0384
@DE00380 C     R3,CCARABL(,R11)                                    0384
         BNH   @DL00380                                            0384
*   RFY                                                            0385
*     CCARAB BASED(CCARABB);        /* MAP BASE RAB                  */
*   IF R2<MINRABS|CCARBUR=ON|       /* IF LESS THAN 3 RABS           */
*       ^(CCARABF<=CCARABB&         /* SUPPLIED BY CALLER OR         */
*       CCARABB<=CCARABL) THEN      /* BASE RAB IS NOT FREE OR BASE
*                                      RAB DOES NOT LIE BETWEEN    0386
*                                      CCARABF AND CCARABL THEN      */
         CH    R2,@CH00074                                         0386
         BL    @RT00386                                            0386
         L     @10,CCARABB(,R11)                                   0386
         TM    CCARBUR(@10),B'10000000'                            0386
         BO    @RT00386                                            0386
         C     @10,CCARABF(,R11)                                   0386
         BL    @RT00386                                            0386
         C     @10,CCARABL(,R11)                                   0386
         BNH   @RF00386                                            0386
@RT00386 DS    0H                                                  0387
*     DO;                           /* SET ERROR CODE, EXIT          */
*                                                                  0387
*       /*************************************************************/
*       /*                                                           */
*       /* SET CCAMODID='B1', CCAREASN=RSN52, CCARETRN=RCINCNPM      */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0388
*       CCAPROB=IGGSEGID+(RSN52*K256)+RCINCNPM;/* INCONSISTENT     0388
*                                      PARAMETERS                    */
         MVC   CCAPROB(4,R11),@CF03009                             0388
*       CALL EXIT;                  /* EXIT                          */
         BAL   @14,EXIT                                            0389
*     END;                          /* END SET                       */
*   ELSE                                                           0391
*     ;                             /* NO RAB VALIDITY CHECK ERR     */
*                                                                  0391
@RF00386 DS    0H                                                  0392
*   /*****************************************************************/
*   /*                                                               */
*   /* SET CUMRAB1 AND CUMRAB2 EACH TO A FREE RAB WHICH IS NOT THE   */
*   /* BASE RAB. BEGIN THE SEARCH FROM CCARABL.                      */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0392
*   RFY                                                            0392
*     CCARAB BASED(R2);             /* MAP EACH RAB FOR FREE STATUS  */
*   DO R2=CCARABL TO CCARABF        /* SCAN ALL CALLER SUPPLIED      */
*         BY-LENGTH(CCARAB);        /* RABS                          */
         L     R2,CCARABL(,R11)                                    0393
         B     @DE00393                                            0393
@DL00393 DS    0H                                                  0394
*     IF CCARBUR=OFF&               /* IF FREE RAB AND NOT BASE      */
*         R2^=CCARABB THEN          /* RAB                           */
         TM    CCARBUR(R2),B'10000000'                             0394
         BNZ   @RF00394                                            0394
         C     R2,CCARABB(,R11)                                    0394
         BE    @RF00394                                            0394
*       DO;                         /* CHECK WHETHER FREE RABS HAVE
*                                      BEEN FOUND                    */
*         IF CUMRAB2=0 THEN         /* IF FREE RAB NOT FOUND YET     */
         L     @10,CCACUMPL(,R11)                                  0396
         ICM   @05,15,CUMRAB2(@10)                                 0396
         BNZ   @RF00396                                            0396
*           CUMRAB2=R2;             /* SAVE FREE RAB                 */
         ST    R2,CUMRAB2(,@10)                                    0397
*         ELSE                      /* FREE RAB ALREADY FOUND        */
*           IF CUMRAB1=0 THEN       /* IF FREE RAB NOT FOUND YET     */
         B     @RC00396                                            0398
@RF00396 L     @10,CCACUMPL(,R11)                                  0398
         ICM   @05,15,CUMRAB1(@10)                                 0398
         BNZ   @RF00398                                            0398
*             CUMRAB1=R2;           /* SAVE FREE RAB                 */
         ST    R2,CUMRAB1(,@10)                                    0399
*           ELSE                                                   0400
*             ;                     /* FREE RAB ALREADY FOUND        */
@RF00398 DS    0H                                                  0401
*         RCDCINO=0;                /* ASSUME NONE OF THESE RECORD 0401
*                                      AREAS CONTAIN USABLE IN-CORE
*                                      RECORDS                       */
@RC00396 L     @10,CCACPE1(,R2)                                    0401
         SLR   @05,@05                                             0401
         STCM  @05,7,RCDCINO(@10)                                  0401
*       END;                        /* END CHECK                     */
*     ELSE                                                         0403
*       ;                           /* SKIP THIS RAB                 */
@RF00394 DS    0H                                                  0404
*   END;                            /* END SCAN                      */
*                                                                  0404
         AH    R2,@CH03010                                         0404
@DE00393 C     R2,CCARABF(,R11)                                    0404
         BNL   @DL00393                                            0404
*   /*****************************************************************/
*   /*                                                               */
*   /* INITIALIZE THE COMMON PORTION OF THE FIELD MGMT INTERFACE     */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0405
*   CCAXCPL=ADDR(CUMFMWA);          /* BASE FOR CPL AREA             */
         L     @10,CCACUMPL(,R11)                                  0405
         LA    @05,CUMFMWA(,@10)                                   0405
         ST    @05,CCAXCPL(,R11)                                   0405
*   CCAF2NVC=ON;                    /* DONT PERFORM ADDRESS VALIDITY
*                                      CHECKS                        */
         OI    CCAF2NVC(R11),B'01000000'                           0406
*   CCAMODPA=OFF;                   /* DONT DO PUT ADD AGAINST BASE
*                                      RCD IN MODIFY                 */
         NI    CCAMODPA(R11),B'11101111'                           0407
*   RFY                                                            0408
*     CTGPL BASED(CCAXCPL);         /* MAP FIELD MGMT CPL            */
*   CTGWKA=ADDR(CUMFMWA)+LENGTH(CTGPL)/* POINT WORK AREA TO END      */
*       +LENGTH(CTGFIELD)+NUMFPLS*(LENGTH(CTGFL)+LENGTH(CTGFLDAT));/*
*                                      OF SECOND FPL                 */
         LA    @15,80                                              0409
         ALR   @15,@05                                             0409
         ST    @15,CTGWKA(,@05)                                    0409
*   CTGNOFLD=1;                     /* ALWAYS JUST ONE FPL           */
         MVI   CTGNOFLD(@05),X'01'                                 0410
*   CTGFIELD(1)=ADDR(CUMFMWA)+LENGTH(CTGPL)/* POINT TO FIRST FPL     */
*       +LENGTH(CTGFIELD);          /* FROM CPL                      */
         LA    @14,32                                              0411
         ALR   @14,@05                                             0411
         ST    @14,CTGFIELD(,@05)                                  0411
*   RFY                                                            0412
*     CTGFL BASED(CTGFIELD(1));     /* MAP FIRST FPL                 */
*   CTGFLDNM=ADDR(CUMFN1);          /* POINT TO FIELD NAME AREA      */
         LA    @05,CUMFN1(,@10)                                    0413
         ST    @05,CTGFLDNM(,@14)                                  0413
*   CUMFN1=NAMEDS;                  /* AN ASSOCIATION IS TO BE     0414
*                                      RETRIEVED                     */
         MVC   CUMFN1(8,@10),@CC00060                              0414
*   CTGFLCHN=ADDR(CTGFL)            /*                               */
*       +LENGTH(CTGFL)+LENGTH(CTGFLDAT);/*                           */
*                                                                  0415
         LA    @05,24                                              0415
         ALR   @05,@14                                             0415
         ST    @05,CTGFLCHN(,@14)                                  0415
*   /*****************************************************************/
*   /*                                                               */
*   /* INITIALIZE TEST FPL                                           */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0416
*   CTGFLCHN->CTGFLDNM=ADDR(CUMFN2);/* POINT TO FIELD NAME AREA      */
         LA    @14,CUMFN2(,@10)                                    0416
         ST    @14,CTGFLDNM(,@05)                                  0416
*   CTGFLCHN->CTGFLDNO=1;           /* ONE FPL LENGTH/DATA PAIR      */
         MVI   CTGFLDNO(@05),X'01'                                 0417
*   CTGFLCHN->CTGFLDCD=CTGFLDEQ;    /* TEST FOR TYPE EQUAL TO CUMTYPE*/
         MVI   CTGFLDCD(@05),X'80'                                 0418
*   CUMFN2=TYPE;                    /* THE TEST IS AGAINST THE     0419
*                                      ASSOCIATION RECORD TYPE FIELD */
         MVC   CUMFN2(8,@10),@CC00062                              0419
*   CTGFLCHN->CTGFLNG(1)=LENGTH(CUMTYPE);/* LENGTH OF TYPE FIELD     */
         MVC   CTGFLNG(4,@05),@CF00046                             0420
*   CTGFLCHN->CTGFLPT(1)=ADDR(CUMTYPE);/* POINT TO TYPE FIELD        */
*                                                                  0421
         LA    @14,CUMTYPE(,@10)                                   0421
         ST    @14,CTGFLPT(,@05)                                   0421
*   /*****************************************************************/
*   /*                                                               */
*   /* INITIALIZE WORK AREA                                          */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0422
*   WKALEN1=LENGTH(WKALEN1)+LENGTH(WKALEN2)/* SET SUPPLIED           */
*       +LENGTH(CUMWASS);           /* WORK AREA LENGTH              */
         MVC   WKALEN1(2,@15),@CH00057                             0422
*   CCAREUSE=ON;                    /* ALL RECORD AREAS MAY BE REUSED
*                                      SINCE THEY HAVE ALL BEEN    0423
*                                      INITIALLY INVALIDATED (CI NO.
*                                      SET TO ZERO)                  */
*                                                                  0423
         OI    CCAREUSE(R11),B'00000100'                           0423
*   /*****************************************************************/
*   /*                                                               */
*   /* THE FOLLOWING CODE SEARCHES FOR AND READS IN THE BASE DATA    */
*   /* ENTRY AND ALSO THE UPGRADE ENTRY, IF IT EXISTS. THE UPGRADE   */
*   /* ENTRY IS NOT READ IN IF THIS IS A FORCE DELETE OF THE UPGRADE */
*   /* ENTRY SINCE THE BASE DATA ENTRY MUST BE PROCESSED FIRST.      */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0424
*   RFY                                                            0424
*     CCARAB BASED(CCARABB);        /* MAP BASE RAB                  */
*   CUMEXIT=OFF;                    /* INDICATE THAT ONE DOES NOT  0425
*                                      HAVE TO EXIT EARLY BECAUSE OF
*                                      A FORCE DELETE OF THE UPGRADE
*                                      ENTRY                         */
*                                                                  0425
         NI    CUMEXIT(@10),B'01111111'                            0425
*   /*****************************************************************/
*   /*                                                               */
*   /* SEARCH FOR THE UPGRADE ENTRY AND EXIT IF THE UPGRADE BASE     */
*   /* RECORD IS FOUND, THE BASE DATA RECORD IS FOUND AND A FORCE    */
*   /* DELETE HAS BEEN REQUESTED, THE UPGRADE ENTRY DOES NOT EXIST OR*/
*   /* AN ERROR OCCURS.                                              */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0426
*   DO WHILE(RCDID^=UPGDID&CUMEXIT=OFF/* LOOK FOR UPGRADE ENTRY      */
*         &CUMNE=OFF&CCACD1=RCS);                                  0426
*                                                                  0426
         B     @DE00426                                            0426
@DL00426 DS    0H                                                  0427
*     /***************************************************************/
*     /*                                                             */
*     /* DETERMINE NEXT ENTRY TO BE RETRIEVED (SEARCH SHOULD END WITH*/
*     /* UPGRADE ENTRY OR BASE DATA ENTRY).                          */
*     /*                                                             */
*     /***************************************************************/
*                                                                  0427
*     IF RCDID=AIXID THEN           /* IF THIS IS AN AIX ENTRY, THE
*                                      CLUSTER ASSOCIATION MUST BE 0427
*                                      GOTTEN NEXT                   */
         L     @10,CCARABB(,R11)                                   0427
         L     @10,CCACPE1(,@10)                                   0427
         CLI   RCDID(@10),C'G'                                     0427
         BNE   @RF00427                                            0427
*       CUMTYPE=CLUSID;             /* SET CLUSTER ENTRY TYPE        */
         L     @10,CCACUMPL(,R11)                                  0428
         MVI   CUMTYPE(@10),C'C'                                   0428
*     ELSE                          /* NOT AIX                       */
*       IF RCDID=CLUSID THEN        /* IF CLUSTER ENTRY,             */
         B     @RC00427                                            0429
@RF00427 L     @10,CCARABB(,R11)                                   0429
         L     @10,CCACPE1(,@10)                                   0429
         CLI   RCDID(@10),C'C'                                     0429
         BNE   @RF00429                                            0429
*         CUMTYPE=DATAID;           /* SET DATA ENTRY TYPE AS NEXT 0430
*                                      FOR RETRIEVAL                 */
         L     @10,CCACUMPL(,R11)                                  0430
         MVI   CUMTYPE(@10),C'D'                                   0430
*       ELSE                        /* NOT CLUSTER                   */
*         IF RCDID=DATAID THEN      /* IF DATA ENTRY,                */
         B     @RC00429                                            0431
@RF00429 L     @10,CCARABB(,R11)                                   0431
         L     @05,CCACPE1(,@10)                                   0431
         CLI   RCDID(@05),C'D'                                     0431
         BNE   @RF00431                                            0431
*           DO;                     /* SET NEXT ENTRY FOR RETRIEVAL
*                                      AND DO SAVES.                 */
*             CUMTYPE=UPGDID;       /* INDICATE UPGRADE ENTRY MUST BE
*                                      RETRIEVED                     */
         L     @15,CCACUMPL(,R11)                                  0433
         MVI   CUMTYPE(@15),C'Y'                                   0433
*             CUMDY=RCDCINO;        /* SAVE DATA RCD CI NO.          */
         MVC   CUMDY(3,@15),RCDCINO(@05)                           0434
*             CUMRABD=CCARABB;      /* SAVE DATA RAB ADDR            */
         ST    @10,CUMRABD(,@15)                                   0435
*           END;                    /* END SET                       */
*         ELSE                      /* NOT DATA ENTRY                */
*                                                                  0437
*           /*********************************************************/
*           /*                                                       */
*           /* SET CCAMODID='B1', CCAREASN=RSN24, CCARETRN=RCCATBAL  */
*           /*                                                       */
*           /*********************************************************/
*                                                                  0437
*           CCAPROB=IGGSEGID+(RSN24*K256)+RCCATBAL;/* INDICATE CAT 0437
*                                                            @X04SVRS
*                                      STRUCTURAL ERR        @X04SVRS*/
*                                                                  0437
         B     @RC00431                                            0437
@RF00431 MVC   CCAPROB(4,R11),@CF03013                             0437
*     /***************************************************************/
*     /*                                                             */
*     /* THE FOLLOWING CODE PLACES THE NEXT CI NUMBER TO BE READ INTO*/
*     /* CUMSRID BY EXTRACTING THE ASSOCIATION WITH RECORD TYPE      */
*     /* CUMTYPE.                                                    */
*     /*                                                             */
*     /***************************************************************/
*                                                                  0438
*     IF CCACD1^=RCS THEN                                          0438
@RC00431 DS    0H                                                  0438
@RC00429 DS    0H                                                  0438
@RC00427 CLI   CCACD1(R11),0                                       0438
         BNE   @RT00438                                            0438
*       ;                           /* IF PREVIOUS ERROR             */
*     ELSE                          /* NO PREVIOUS ERROR             */
*       DO;                         /* SEARCH FOR ASSOCIATION UNTIL
*                                      FOUND, UPGRADE ENTRY IS     0440
*                                      DETERMINED TO BE NON-EXISTENT,
*                                      OR AN ERROR OCCURS            */
*                                                                  0440
*         /***********************************************************/
*         /*                                                         */
*         /* CALL TO MODULE IGG0CLAZ                                 */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0441
*Z057EXT:                                                          0441
*         CALL IGGPEXT;             /* GET REQUIRED ASSOCIATION      */
Z057EXT  L     @15,@CV00038                                        0441
         BALR  @14,@15                                             0441
*         IF CCACD1^=RCS THEN                                      0442
         CLI   CCACD1(R11),0                                       0442
         BNE   @RT00442                                            0442
*           ;                       /* IF ERROR, QUIT                */
*         ELSE                      /* NO ERROR                      */
*           DO;                     /* CONTINUE SEARCH               */
*             RFY                                                  0445
*               CTGFL BASED(CTGFIELD(1));/* MAP FPL WHICH SHOULD   0445
*                                      POINT TO RETURNED DATA        */
*             IF CTGFLPT(1)=0 THEN  /* IF NO ASSOCIATION WAS RETURNED*/
         L     @10,CCAXCPL(,R11)                                   0446
         L     @10,CTGFIELD(,@10)                                  0446
         ICM   @05,15,CTGFLPT(@10)                                 0446
         BNZ   @RF00446                                            0446
*               IF CUMTYPE=UPGDID THEN/* IF TRYING TO GET UPGRADE  0447
*                                      ASSOCIATION                   */
         L     @10,CCACUMPL(,R11)                                  0447
         CLI   CUMTYPE(@10),C'Y'                                   0447
         BNE   @RF00447                                            0447
*                 CUMNE=ON;         /* INDICATE UPGRADE ENTRY DOES 0448
*                                      NOT EXIST                     */
         OI    CUMNE(@10),B'01000000'                              0448
*               ELSE                /* NOT TRYING TO GET UPGRADE   0449
*                                      ASSOCIATION,                  */
*                                                                  0449
*                 /***************************************************/
*                 /*                                                 */
*                 /* SET CCAMODID='B1', CCAREASN=RSN24,              */
*                 /* CCARETRN=RCCATBAL                               */
*                 /*                                                 */
*                 /***************************************************/
*                                                                  0449
*                 CCAPROB=IGGSEGID+(RSN24*K256)+RCCATBAL;/* SO STRUCT
*                                      ERR                   @X04SVRS*/
         B     @RC00447                                            0449
@RF00447 MVC   CCAPROB(4,R11),@CF03013                             0449
*             ELSE                  /* AN ASSOCIATION WAS RETURNED   */
*               DO;                 /* CONTINUE QUEST                */
         B     @RC00446                                            0450
@RF00446 DS    0H                                                  0451
*                 CUMSRID=ASSOCCI;  /* SAVE CI NO. WHICH IS NEXT TO
*                                      BE RETRIEVED                  */
         L     @10,CCACUMPL(,R11)                                  0451
         L     @05,CCAXCPL(,R11)                                   0451
         L     @05,CTGFIELD(,@05)                                  0451
         L     @05,CTGFLPT(,@05)                                   0451
         MVC   CUMSRID(3,@10),ASSOCCI(@05)                         0451
*                 IF RCDID=DATAID&CUMFDEL=ON/* IF THIS IS A FORCE    */
*                     &CUMFUNC=OFF THEN/* DELETE OF THE UPGRADE    0452
*                                      ENTRY, WE MUST EXIT NOW WHEN
*                                      THE BASE DATA ENTRY IS      0452
*                                      AVAILABLE                     */
         L     @05,CCARABB(,R11)                                   0452
         L     @05,CCACPE1(,@05)                                   0452
         CLI   RCDID(@05),C'D'                                     0452
         BNE   @RF00452                                            0452
         TM    CUMFDEL(@10),B'10000000'                            0452
         BNO   @RF00452                                            0452
         TM    CUMFUNC(@10),B'00100000'                            0452
         BNZ   @RF00452                                            0452
*                   CUMEXIT=ON;     /* INDICATE EARLY EXIT           */
         OI    CUMEXIT(@10),B'10000000'                            0453
*                 ELSE              /* NOT FORCE DELETE              */
*                   DO;             /* CONTINUE TRIP                 */
         B     @RC00452                                            0454
@RF00452 DS    0H                                                  0455
*                     IF CUMTYPE=UPGDID THEN/* IF UPGRADE ENTRY TO 0455
*                                      GET,                          */
         L     @10,CCACUMPL(,R11)                                  0455
         CLI   CUMTYPE(@10),C'Y'                                   0455
         BNE   @RF00455                                            0455
*                       CCARABB=CUMRAB2;/* MUST READ INTO SPECIFIC 0456
*                                      RAB                           */
         MVC   CCARABB(4,R11),CUMRAB2(@10)                         0456
*                     ELSE          /* NOT UPGRADE ENTRY             */
*                       CCARABB=CUMRAB1;/* USE OTHER AVAILABLE RAB   */
         B     @RC00455                                            0457
@RF00455 L     @10,CCACUMPL(,R11)                                  0457
         MVC   CCARABB(4,R11),CUMRAB1(@10)                         0457
*                     CCASRID=CUMSRID;/* SET UP FOR GET OF NEXT ENTRY*/
@RC00455 L     @10,CCACUMPL(,R11)                                  0458
         MVC   CCASRID(3,R11),CUMSRID(@10)                         0458
*                     CCAURAB=CCARABB;/* INDICATE PROPER RAB         */
         MVC   CCAURAB(4,R11),CCARABB(R11)                         0459
*                     DO;                                          0460
*                       CCAF1KEY=OFF;/* GET BY CI NO.                */
*                       CCAF1KGE=OFF;/* NOT GET NEXT                 */
*                       CCAF1UP=ON; /* GET FOR UPD                   */
*                                                                  0463
         OI    CCAF1UP(R11),B'00000010'                            0463
         NI    CCAF1KEY(R11),B'11100111'                           0463
*                       /*********************************************/
*                       /*                                           */
*                       /* CALL TO MODULE IGG0CLBI                   */
*                       /*                                           */
*                       /*********************************************/
*                                                                  0464
*Z060GET:                                                          0464
*                       CALL IGGPGET;/* GET RECORD                   */
Z060GET  L     @15,@CV00036                                        0464
         BALR  @14,@15                                             0464
*                     END;          /* READ NEXT ENTRY               */
*                   END;            /* END CONT. TRIP                */
*               END;                /* END CONT. QUEST               */
@RC00452 DS    0H                                                  0468
*           END;                    /* END CONT. SEARCH              */
@RC00446 DS    0H                                                  0469
*       END;                        /* END SEARCH                    */
@RT00442 DS    0H                                                  0470
*   END;                            /* END LOOK                      */
*                                                                  0470
@RT00438 DS    0H                                                  0470
@DE00426 L     @10,CCARABB(,R11)                                   0470
         L     @10,CCACPE1(,@10)                                   0470
         CLI   RCDID(@10),C'Y'                                     0470
         BE    @DC00426                                            0470
         L     @10,CCACUMPL(,R11)                                  0470
         TM    CUMEXIT(@10),B'10000000'                            0470
         BNZ   @DC00426                                            0470
         TM    CUMNE(@10),B'01000000'                              0470
         BNZ   @DC00426                                            0470
         CLI   CCACD1(R11),0                                       0470
         BE    @DL00426                                            0470
@DC00426 DS    0H                                                  0471
*   /*****************************************************************/
*   /*                                                               */
*   /* CHECK THAT THE BASE DATA ENTRY WAS ACTUALLY RETRIEVED         */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0471
*   IF CUMDY=0&CCACD1=RCS THEN      /* IF NO BASE DATA CI NO.        */
*                                                                  0471
         L     @10,CCACUMPL(,R11)                                  0471
         ICM   @05,7,CUMDY(@10)                                    0471
         BNZ   @RF00471                                            0471
         CLI   CCACD1(R11),0                                       0471
         BNE   @RF00471                                            0471
*     /***************************************************************/
*     /*                                                             */
*     /* SET CCAMODID='B1', CCAREASN=RSN24, CCARETRN=RCCATBAL        */
*     /*                                                             */
*     /***************************************************************/
*                                                                  0472
*     CCAPROB=IGGSEGID+(RSN24*K256)+RCCATBAL;/* STRUCT ERROR @X04SVRS*/
         MVC   CCAPROB(4,R11),@CF03013                             0472
*   ELSE                                                           0473
*     ;                             /* EVERYTHING OK                 */
@RF00471 DS    0H                                                  0474
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0474
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPUEND');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPUEND'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00004 DS    0H                                                  0477
@EF00004 DS    0H                                                  0477
@ER00004 BR    @14                                                 0477
@PB00004 DS    0H                                                  0478
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0479
*/********************************************************************/
*/*                                                                  */
*/*        IGGPUEND  -  PROCEDURE  INTERNALLY  CALLED  UNNESTED      */
*/*                                                                  */
*/********************************************************************/
*                                                                  0479
*IGGPUEND:                                                         0479
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00005                                            0479
IGGPUEND DS    0H                                                  0480
*Z062:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z062UEND:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0480
*    (R5,                                                          0480
*     R6,                                                          0480
*     R7,                                                          0480
*     R8,                                                          0480
*     R9,                                                          0480
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z062     DS    0H                                                  0480
Z062UEND DS    0H                                                  0481
*   RFY                                                            0481
*    (R2,                                                          0481
*     R3,                                                          0481
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*/* PROCEDURE                                                        */
*/********************************************************************
**                                                                   *
**   MODULE NAME = IGGPUEND(IGG0CLB1)                                *
**                                                                   *
**   DESCRIPTIVE NAME = CATALOG UPGRADE MANAGEMENT TERMINATION       *
**                                                                   *
**   FUNCTION =                                                      *
**      THIS SUB-FUNCTION PERFORMS COMMON TERMINATION PROCESSING     *
**      FOR CATALOG UPGRADE MANAGEMENT. THE PROCESSING IS AS         *
**      FOLLOWS:                                                     *
**       - RESTORE THE CALLER'S INPUT VARIABLES.                     *
**       - RETRIEVE THE CALLER'S ORIGINAL BASE RECORD INTO THE       *
**         RECORD AREA POINTED TO BY CCARABB, IF NECESSARY.          *
**                                                                   *
**   INPUT =                                                         *
**      CUMURAB - CONTENTS OF THE CALLER'S ORIGINAL CCAURAB AT ENTRY *
**        TO UPGRADE MANAGEMENT.                                     *
**      CUMRABB - CONTENTS OF THE CALLER'S ORIGINAL CCARABB AT ENTRY *
**        TO UPGRADE MANAGEMENT.                                     *
**      CUMBCI - CALLER'S ORIGINAL BASE RECORD CI NUMBER AT ENTRY    *
**        TO UPGRADE MANAGEMENT.                                     *
**      CUMNRG - IF SET ON, THE CALLER'S ORGINAL BASE RECORD IS NOT  *
**        TO BE RE-GOTTEN.                                           *
**                                                                   *
**   OUTPUT =                                                        *
**      CCAURAB - RESTORED WITH CONTENTS OF CUMURAB                  *
**      CCARABB - RESTORED WITH CONTENTS OF CUMRABB                  *
**      CCAREUSE - SET OFF                                           *
**      CCACD1 - ANY ERROR CODE RETURNED BY GET OR ANY ERROR         *
**        INITIALLY PASSED TO THIS SUB-FUNCTION.                     *
**      CCACD2 - ANY ERROR CODE RETURNED BY GET WHEN ATTEMPTING      *
**        TO RE-GET THE CALLER'S BASE RECORD                         *
**                                                                   *
******************************************************************** */
*                                                                  0482
*   /*****************************************************************/
*   /*                                                               */
*   /* THE FOLLOWING CODE RESTORES THE CALLER'S CCAURAB, CCARABB AND */
*   /* THE CALLER'S ORGINAL BASE RECORD.                             */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0482
*   RFY                                                            0482
*     CCARAB BASED(CUMRABB);        /* MAP RECORD AREA OF CALLER'S 0482
*                                      BASE RECORD                   */
*   CCACD2=RCS;                     /* CLEAR SECOND ERROR CODE       */
         MVI   CCACD2(R11),X'00'                                   0483
*   CCARABB=CUMRABB;                /* RESTORE CCARABB               */
         L     @10,CCACUMPL(,R11)                                  0484
         L     @05,CUMRABB(,@10)                                   0484
         ST    @05,CCARABB(,R11)                                   0484
*   CCAREUSE=OFF;                   /* INDICATE EXTRACT/MODIFY RECORD
*                                      AREAS ARE NOT TO BE REUSED    */
         NI    CCAREUSE(R11),B'11111011'                           0485
*   IF RCDCINO^=CUMBCI              /* IF THE CALLER'S BASE          */
*       &CUMNRG=OFF THEN            /* RECORD IS NOT IN-CORE AND THE
*                                      CALLER WISHES THE BASE RECORD
*                                      TO BE RE-GOTTEN               */
         L     @01,CCACPE1(,@05)                                   0486
         CLC   RCDCINO(3,@01),CUMBCI(@10)                          0486
         BE    @RF00486                                            0486
         TM    CUMNRG(@10),B'01000000'                             0486
         BNZ   @RF00486                                            0486
*     DO;                           /* READ BASE RECORD              */
*       CUMRC=CCAPROB;              /* SAVE MAJOR ERROR CODE         */
*                                                                  0488
         MVC   CUMRC(4,@10),CCAPROB(R11)                           0488
*       /*************************************************************/
*       /*                                                           */
*       /* CLEAR CCAMODID, CCAREASN, AND CCARETRN                    */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0489
*       CCAPROB=IGNOREIT;           /* CLEAR MAJOR ERROR CODE FIELD  */
         SLR   @15,@15                                             0489
         ST    @15,CCAPROB(,R11)                                   0489
*       CCAURAB=CUMRABB;            /* SET CCAURAB                   */
         ST    @05,CCAURAB(,R11)                                   0490
*       CCASRID=CUMBCI;             /* SET BASE RECORD CI NUMBER     */
         MVC   CCASRID(3,R11),CUMBCI(@10)                          0491
*       DO;                                                        0492
*         CCAF1KEY=OFF;             /* GET BY CI NO.                 */
*         CCAF1KGE=OFF;             /* NOT GET NEXT                  */
*         CCAF1UP=ON;               /* GET FOR UPD                   */
*                                                                  0495
         OI    CCAF1UP(R11),B'00000010'                            0495
         NI    CCAF1KEY(R11),B'11100111'                           0495
*         /***********************************************************/
*         /*                                                         */
*         /* CALL TO MODULE IGG0CLBI                                 */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0496
*Z065GET:                                                          0496
*         CALL IGGPGET;             /* GET RECORD                    */
Z065GET  L     @15,@CV00036                                        0496
         BALR  @14,@15                                             0496
*       END;                        /* GET BASE RECORD               */
*       CCACD2=CCACD1;              /* SAVE BASE RCD RE-GET ERROR    */
         MVC   CCACD2(1,R11),CCACD1(R11)                           0498
*       IF CUMRC=ZRC THEN                                          0499
         L     @10,CCACUMPL(,R11)                                  0499
         CLC   CUMRC(4,@10),@CB00044                               0499
         BE    @RT00499                                            0499
*         ;                         /* IF NO PREVIOUS MAJOR ERR      */
*       ELSE                        /* PREVIOUS MAJOR ERROR          */
*         CCAPROB=CUMRC;            /* RESTORE IT                    */
         L     @10,CCACUMPL(,R11)                                  0501
         MVC   CCAPROB(4,R11),CUMRC(@10)                           0501
*     END;                          /* END READ BASE RECORD          */
*   ELSE                                                           0503
*     ;                             /* BASE RECORD IN-CORE           */
@RF00486 DS    0H                                                  0504
*   CCAURAB=CUMURAB;                /* RESTORE CCAURAB               */
@RC00486 L     @10,CCACUMPL(,R11)                                  0504
         MVC   CCAURAB(4,R11),CUMURAB(@10)                         0504
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0505
*   END;                            /* END OF INNER PROCEDURE        */
@EL00005 DS    0H                                                  0506
@EF00005 DS    0H                                                  0506
@ER00005 BR    @14                                                 0506
@PB00005 DS    0H                                                  0507
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0508
*/********************************************************************/
*/*                                                                  */
*/*         RETURN-TO-CALLER  PROCEDURE-EXITING  SUBFUNCTION         */
*/*                                                                  */
*/********************************************************************/
*                                                                  0508
*EXIT:                                                             0508
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00006                                            0508
EXIT     DS    0H                                                  0509
*   GEN(LM 12,14,0(13));            /* RESTORE REGISTERS OF CALLER   */
         LM 12,14,0(13)
*   GEN(MVI 12(13),255);            /* MARK END OF PDL               */
         MVI 12(13),255
*   GEN(BR 14);                     /* RETURN TO CALLER              */
         BR 14
*PATCH:                                                            0512
*   GEN(DC (12+((*+139-@MAINENT)/140))A(0));/* PATCH AREA            */
PATCH    DC (12+((*+139-@MAINENT)/140))A(0)
*   END;                            /* END OF EXIT PROCEDURE         */
@EL00006 DS    0H                                                  0513
@EF00006 DS    0H                                                  0513
@ER00006 BR    @14                                                 0513
*   END                             /* END IGG0CLB1                  */
*                                                                  0514
*/* THE FOLLOWING INCLUDE STATEMENTS WERE FOUND IN THIS PROGRAM.     */
*/*%INCLUDE SYSLIB  (IGGMCDCL)                                       */
*/*%INCLUDE SYSLIB  (IGGMCTRC)                                       */
*/*%INCLUDE SYSLIB  (IGGCCA  )                                       */
*/*%INCLUDE SYSLIB  (IEZCTGPL)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGFL)                                       */
*/*%INCLUDE SYSLIB  (CVT     )                                       */
*/*%INCLUDE SYSLIB  (IKJTCB  )                                       */
*/*%INCLUDE SYSLIB  (IHAPSA  )                                       */
*/*%INCLUDE SYSLIB  (IFGACB  )                                       */
*/*%INCLUDE SYSLIB  (AMCBS   )                                       */
*/*%INCLUDE SYSLIB  (IGGCAXWA)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGFV)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGWA)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGVL)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGCV)                                       */
*/*%INCLUDE SYSLIB  (IGGMCMWA)                                       */
*                                                                  0514
*       ;                                                          0514
@EL00001 DS    0H                                                  0514
@EF00001 DS    0H                                                  0514
@ER00001 BR    @14                                                 0514
@DATA    DS    0H
@CH00097 DC    H'6'
@CH00057 DC    H'8'
@CH00154 DC    H'20'
@CH03001 DC    H'55'
@CH00070 DC    H'125'
@CH03010 DC    H'-20'
@DATD    DSECT
         DS    0F
IGG0CLB1 CSECT
         DS    0F
@CF00046 DC    F'1'
@CH00046 EQU   @CF00046+2
@CF00072 DC    F'2'
@CF00074 DC    F'3'
@CH00074 EQU   @CF00074+2
@CF00103 DC    F'4'
@CH00103 EQU   @CF00103+2
@CF00292 DC    F'10'
@CF03003 DC    F'-1024392480'
@CF03013 DC    F'-1024386956'
@CF03006 DC    F'-1024382916'
@CF03009 DC    F'-1024379764'
@DATD    DSECT
         DS    0D
         ORG   *+1-(*-@DATD)/(*-@DATD) INSURE DSECT DATA
@ENDDATD EQU   *
IGG0CLB1 CSECT
         DS    0F
@SIZDATD DC    AL1(0)
         DC    AL3(@ENDDATD-@DATD)
@CV00036 DC    V(IGGPGET)
@CV00037 DC    V(IGGPMOD)
@CV00038 DC    V(IGGPEXT)
@CV00039 DC    V(IGGPPDE)
@CV00040 DC    V(IGGPPAD)
@CV00041 DC    V(IGGPAXCI)
         DS    0D
@CC00058 DC    C'RELREPNO'
@CC00060 DC    C'NAMEDS  '
@CC00062 DC    C'TYPE    '
@CC00064 DC    C'NAME    '
@CC00066 DC    C'UPGRADE '
@CC00068 DC    C'ENTUPGD '
@CB00044 DC    X'00000000'
@00      EQU   00                      EQUATES FOR REGISTERS 0-15
@01      EQU   01
@02      EQU   02
@03      EQU   03
@04      EQU   04
@05      EQU   05
@06      EQU   06
@07      EQU   07
@08      EQU   08
@09      EQU   09
@10      EQU   10
@11      EQU   11
@12      EQU   12
@13      EQU   13
@14      EQU   14
@15      EQU   15
IGGSEGBR EQU   @12
RWA      EQU   @02
R2       EQU   @02
R3       EQU   @03
R4       EQU   @04
R5       EQU   @05
R6       EQU   @06
R7       EQU   @07
R8       EQU   @08
R9       EQU   @09
R10      EQU   @10
R11      EQU   @11
R12      EQU   @12
DCWABASE EQU   @04
RET      EQU   @14
RFLD     EQU   @04
RGO      EQU   @15
RNDX     EQU   @03
RP0      EQU   @00
RP1      EQU   @01
RSAVE    EQU   @13
RWK1     EQU   @06
RWK2     EQU   @07
RWK3     EQU   @08
RWK4     EQU   @09
RZERO    EQU   @00
R0       EQU   @00
R1       EQU   @01
R13      EQU   @13
R14      EQU   @14
R15      EQU   @15
WKR2     EQU   @02
WKR3     EQU   @03
WKR4     EQU   @04
CVTPTR   EQU   16
PSA      EQU   0
FLCRNPSW EQU   PSA
FLCROPSW EQU   PSA+8
FLCICCW2 EQU   PSA+16
PSAEEPSW EQU   PSA+132
PSAESPSW EQU   PSA+136
FLCSVILC EQU   PSAESPSW+1
PSAEPPSW EQU   PSA+140
FLCPIILC EQU   PSAEPPSW+1
FLCPICOD EQU   PSAEPPSW+2
PSAPICOD EQU   FLCPICOD+1
FLCTEA   EQU   PSAEPPSW+4
FLCPER   EQU   PSA+152
FLCMCLA  EQU   PSA+168
FLCIOEL  EQU   FLCMCLA+4
FLCIOA   EQU   FLCMCLA+16
FLCFSA   EQU   FLCMCLA+80
PSASUPER EQU   PSA+552
PSASUP1  EQU   PSASUPER
PSASUP2  EQU   PSASUPER+1
PSASUP3  EQU   PSASUPER+2
PSASUP4  EQU   PSASUPER+3
PSACLHT  EQU   PSA+640
PSALKSA  EQU   PSA+696
PSAHLHI  EQU   PSA+760
PSADSSFL EQU   PSA+836
PSADSSF1 EQU   PSADSSFL
PSADSSF3 EQU   PSADSSFL+2
PSADSSF4 EQU   PSADSSFL+3
PSARSVT  EQU   PSA+896
WKASSOC  EQU   0
ASSOCCI  EQU   WKASSOC+1
ZIT      EQU   0
IGGCCA   EQU   0
CCAPROB  EQU   IGGCCA+4
CCAERRCD EQU   CCAPROB+2
CCAREASN EQU   CCAERRCD
CCARETRN EQU   CCAERRCD+1
CCACD1   EQU   CCARETRN
CCACD2   EQU   IGGCCA+13
CCAFLG1  EQU   IGGCCA+14
CCAF1KEY EQU   CCAFLG1
CCAF1KGE EQU   CCAFLG1
CCAF1UP  EQU   CCAFLG1
CCAFLG2  EQU   IGGCCA+15
CCAF2NVC EQU   CCAFLG2
CCAF2COB EQU   CCAFLG2
CCAFLG3  EQU   IGGCCA+16
CCAFLG4  EQU   IGGCCA+17
CCAFLG5  EQU   IGGCCA+18
CCAMODPA EQU   CCAFLG5
CCAFLG6  EQU   IGGCCA+19
CCAREUSE EQU   CCAFLG6
CCATCB   EQU   IGGCCA+20
CCARB    EQU   IGGCCA+24
CCACPL   EQU   IGGCCA+28
CCAACB   EQU   IGGCCA+32
CCAURAB  EQU   IGGCCA+40
CCASRCH  EQU   IGGCCA+44
CCASRID  EQU   CCASRCH
CCARAB0  EQU   IGGCCA+88
CCAR0FLG EQU   CCARAB0
CCAR0SEG EQU   CCARAB0+8
CCARAB1  EQU   IGGCCA+108
CCAR1FLG EQU   CCARAB1
CCAR1SEG EQU   CCARAB1+8
CCARAB2  EQU   IGGCCA+128
CCAR2FLG EQU   CCARAB2
CCAR2SEG EQU   CCARAB2+8
CCARAB3  EQU   IGGCCA+148
CCAR3FLG EQU   CCARAB3
CCAR3SEG EQU   CCARAB3+8
CCARAB4  EQU   IGGCCA+168
CCAR4FLG EQU   CCARAB4
CCAR4SEG EQU   CCARAB4+8
CCARAB5  EQU   IGGCCA+188
CCAR5FLG EQU   CCARAB5
CCAR5SEG EQU   CCARAB5+8
CCATIQRN EQU   IGGCCA+212
CCADESA  EQU   IGGCCA+224
CCAEXTDE EQU   CCADESA+4
CCAEXTAD EQU   CCAEXTDE+2
CCAASCI  EQU   IGGCCA+271
CCAEQDQ  EQU   IGGCCA+280
CCAEDOPT EQU   CCAEQDQ+2
CCAMSSPL EQU   IGGCCA+300
CCACMS   EQU   IGGCCA+316
CCACMSWA EQU   CCACMS
CCACPE5  EQU   IGGCCA+324
CCACPE6  EQU   IGGCCA+340
CCACPE61 EQU   IGGCCA+344
CCACPE7  EQU   IGGCCA+348
CCAFLPT  EQU   IGGCCA+364
CCADICT  EQU   IGGCCA+372
CCAXCPL  EQU   IGGCCA+376
CCAMCPL  EQU   CCAXCPL
CCARABB  EQU   IGGCCA+380
CCARABF  EQU   IGGCCA+384
CCARABL  EQU   IGGCCA+388
CCALREL  EQU   IGGCCA+396
CCASN    EQU   IGGCCA+398
CCARBA   EQU   IGGCCA+448
CCACRAVL EQU   CCARBA
CCACRADT EQU   CCARBA+6
CCATLNG  EQU   IGGCCA+468
CCAIDPT3 EQU   IGGCCA+480
CCAGRGC  EQU   IGGCCA+493
CCAGRHI  EQU   IGGCCA+494
CCASSEQ  EQU   CCAGRHI
CCAFLG7  EQU   IGGCCA+552
CCAFLG8  EQU   IGGCCA+553
CCAFLG9  EQU   IGGCCA+554
CCASMFLG EQU   IGGCCA+562
CCASMFG1 EQU   CCASMFLG
CCASMFUC EQU   CCASMFG1
CCASMFSR EQU   CCASMFG1
CCABMINP EQU   IGGCCA+684
CCABMFLG EQU   CCABMINP+6
CCABMOUT EQU   IGGCCA+692
CCABMOFG EQU   CCABMOUT+4
CCATEMPS EQU   IGGCCA+744
CCAMNCAT EQU   IGGCCA+784
CCAMNAT  EQU   CCAMNCAT
MNATTOP  EQU   CCAMNAT
MNATFLGS EQU   CCAMNAT+240
CCAARFWA EQU   IGGCCA+1040
ARFGMLEN EQU   CCAARFWA
CCARVFG1 EQU   IGGCCA+1056
CCAREGS  EQU   IGGCCA+1060
CCABZSAV EQU   IGGCCA+1408
CCACUMPL EQU   IGGCCA+1412
CCARALSA EQU   IGGCCA+1440
CCAFLG10 EQU   IGGCCA+1444
CCASMFP  EQU   IGGCCA+1480
CCAPROBX EQU   IGGCCA+1484
CCAERCDX EQU   CCAPROBX+2
CTGPL    EQU   0
CTGOPTN1 EQU   CTGPL
CTGOPTN2 EQU   CTGPL+1
CTGEXT   EQU   CTGOPTN2
CTGERASE EQU   CTGOPTN2
CTGSMF   EQU   CTGERASE
CTGREL   EQU   CTGSMF
CTGPURG  EQU   CTGOPTN2
CTGVMNT  EQU   CTGPURG
CTGOPTN3 EQU   CTGPL+2
CTGOPTN4 EQU   CTGPL+3
CTGENT   EQU   CTGPL+4
CTGFVT   EQU   CTGENT
CTGCAT   EQU   CTGPL+8
CTGCVOL  EQU   CTGCAT
CTGWKA   EQU   CTGPL+12
CTGDSORG EQU   CTGPL+16
CTGNOFLD EQU   CTGPL+19
CTGDDNM  EQU   CTGPL+20
CTGNEWNM EQU   CTGDDNM
CTGFBFLG EQU   CTGNEWNM+2
CTGJSCB  EQU   CTGPL+24
CTGFIELD EQU   CTGPL+28
CTGFL    EQU   0
CTGFLDNO EQU   CTGFL
CTGFLDCD EQU   CTGFL+1
CTGFLDGC EQU   CTGFL+2
CTGFLDRE EQU   CTGFL+3
CTGFLDTS EQU   CTGFLDRE
CTGFLDWA EQU   CTGFL+4
CTGFLDNM EQU   CTGFL+8
CTGFLCHN EQU   CTGFL+12
CTGFLDAT EQU   CTGFL+16
CTGFLNG  EQU   CTGFLDAT
CTGFLPT  EQU   CTGFLDAT+4
USERWKA  EQU   0
WKALEN1  EQU   USERWKA
WKALEN2  EQU   USERWKA+2
WKA      EQU   USERWKA+4
ENTNAMES EQU   WKA
DICTENT  EQU   0
DICTFLGS EQU   DICTENT
DIC2ENT  EQU   0
DIC2FLGS EQU   DIC2ENT
CCARAB   EQU   0
CCARABFL EQU   CCARAB
CCARBUR  EQU   CCARABFL
CCACPE1  EQU   CCARAB+4
CCACPE2  EQU   CCARAB+8
CCACPE3  EQU   CCARAB+12
CCACPE4  EQU   CCARAB+16
CATGOPT  EQU   0
CATGOCI  EQU   CATGOPT
CATGOFLG EQU   CATGOCI
CATGOGC1 EQU   CATGOPT+3
CATGOSEQ EQU   CATGOPT+4
CATRCD   EQU   0
RCDCINO  EQU   CATRCD+1
RCDCRAVL EQU   CATRCD+5
RCDCRADT EQU   CATRCD+14
RCDID    EQU   CATRCD+44
RCDSIZ   EQU   CATRCD+45
NREPLF   EQU   CATRCD+48
REPCNTRL EQU   0
EXTRCDPT EQU   REPCNTRL
EXTGOCI  EQU   EXTRCDPT
VARFIELD EQU   0
CNTRLINF EQU   0
CVTMAP   EQU   0
CVTTCBP  EQU   CVTMAP
CVTDAR   EQU   CVTMAP+72
CVTFLGS1 EQU   CVTDAR
CVTDCB   EQU   CVTMAP+116
CVTIERLC EQU   CVTMAP+144
CVTOPTA  EQU   CVTMAP+182
CVTOPTB  EQU   CVTMAP+183
CVTGTF   EQU   CVTMAP+236
CVTGTFST EQU   CVTGTF
CVTGTFS  EQU   CVTGTFST
CVTSTATE EQU   CVTGTFST
CVTTMODE EQU   CVTGTFST
CVTFORM  EQU   CVTGTFST
CVTAQAVT EQU   CVTMAP+240
CVTTCMFG EQU   CVTAQAVT
CVTVOLM2 EQU   CVTMAP+244
CVTTATA  EQU   CVTVOLM2
CVTTSKS  EQU   CVTTATA
CVTVOLF2 EQU   CVTTSKS
CVTTAT   EQU   CVTTATA+1
CVTATER  EQU   CVTMAP+248
CVTEXT1  EQU   CVTMAP+252
CVTCBSP  EQU   CVTMAP+256
CVTPURG  EQU   CVTMAP+260
CVTAMFF  EQU   CVTMAP+264
CVTQMSG  EQU   CVTMAP+268
CVTDMSR  EQU   CVTMAP+272
CVTERPV  EQU   CVTMAP+316
CVTINTLA EQU   CVTMAP+320
CVTAPF   EQU   CVTMAP+324
CVTEXT2  EQU   CVTMAP+328
CVTHJES  EQU   CVTMAP+332
CVTPGSIA EQU   CVTMAP+348
CVTA1F1  EQU   CVTMAP+356
CVTSYSK  EQU   CVTMAP+357
CVTVOLM1 EQU   CVTMAP+380
CVTVOLF1 EQU   CVTVOLM1
CVTATMCT EQU   CVTMAP+388
CVTXTNT1 EQU   0
CVTXTNT2 EQU   0
CVTDSSV  EQU   CVTXTNT2
CVTFLGBT EQU   CVTXTNT2+5
CVTQID   EQU   CVTXTNT2+24
CVTRV400 EQU   CVTXTNT2+52
CVTRV409 EQU   CVTXTNT2+53
CVTATCVT EQU   CVTXTNT2+64
CVTRV429 EQU   CVTXTNT2+84
CVTRV438 EQU   CVTXTNT2+85
CVTRV457 EQU   CVTXTNT2+112
CVTRV466 EQU   CVTXTNT2+113
CVTFIX   EQU   0
CVTRELNO EQU   CVTFIX+252
TCBWORDS EQU   0
TCBPTR   EQU   TCBWORDS+4
TCBFIX   EQU   0
TCBFRS   EQU   TCBFIX
TCB      EQU   0
TCBPIE   EQU   TCB+4
TCBPMASK EQU   TCBPIE
TCBCMP   EQU   TCB+16
TCBCMPF  EQU   TCBCMP
TCBCPP   EQU   TCBCMPF
TCBSTCC  EQU   TCBCMPF
TCBCDBL  EQU   TCBCMPF
TCBCWTO  EQU   TCBCMPF
TCBCIND  EQU   TCBCMPF
TCBCMSG  EQU   TCBCMPF
TCBTRN   EQU   TCB+20
TCBABF   EQU   TCBTRN
TCBMSS   EQU   TCB+24
TCBPKF   EQU   TCB+28
TCBFLGS  EQU   TCB+29
TCBFLGS1 EQU   TCBFLGS
TCBFLGS2 EQU   TCBFLGS+1
TCBFLGS3 EQU   TCBFLGS+2
TCBFLGS4 EQU   TCBFLGS+3
TCBFLGS5 EQU   TCBFLGS+4
TCBABWF  EQU   TCBFLGS5
TCBJPQ   EQU   TCB+44
TCBPURGE EQU   TCBJPQ
TCBGRS   EQU   TCB+48
TCBFSA   EQU   TCB+112
TCBTME   EQU   TCB+120
TCBJSTCB EQU   TCB+124
TCBTSFLG EQU   TCB+148
TCBTSTSK EQU   TCBTSFLG
TCBSTAB  EQU   TCB+160
TCBNSTAE EQU   TCBSTAB
TCBTCT   EQU   TCB+164
TCBTCTGF EQU   TCBTCT
TCBSCNDY EQU   TCB+172
TCBNDSP  EQU   TCBSCNDY
TCBNDSP1 EQU   TCBNDSP+1
TCBNDSP2 EQU   TCBNDSP+2
TCBNDSP3 EQU   TCBNDSP+3
TCBJSCB  EQU   TCB+180
TCBRECDE EQU   TCBJSCB
TCBEXT1  EQU   TCB+196
TCBBITS  EQU   TCB+200
TCBNDSP4 EQU   TCBBITS
TCBNDSP5 EQU   TCBBITS+1
TCBFLGS6 EQU   TCBBITS+2
TCBFLGS7 EQU   TCBBITS+3
TCBDAR   EQU   TCB+204
TCBDARC  EQU   TCBDAR
TCBEXT2  EQU   TCB+208
TCBFOE   EQU   TCB+244
TCBESTAE EQU   TCB+264
TCBESTRM EQU   TCBESTAE+1
TCBFBYT1 EQU   TCB+276
TCBFBYT2 EQU   TCB+277
TCBRV124 EQU   TCB+278
TCBRV133 EQU   TCB+279
TCBXTNT2 EQU   0
TCBGTF   EQU   TCBXTNT2
TCBTFLG  EQU   TCBGTF
TCBRV146 EQU   TCBXTNT2+28
TCBRV155 EQU   TCBXTNT2+29
TCBRV164 EQU   TCBXTNT2+30
TCBRV173 EQU   TCBXTNT2+31
IFGACB   EQU   0
ACBCOMN  EQU   IFGACB
ACBLENG  EQU   ACBCOMN+2
ACBLENG2 EQU   ACBLENG
ACBAMBL  EQU   ACBCOMN+4
ACBJWA   EQU   ACBAMBL
ACBMACRF EQU   ACBCOMN+12
ACBMACR1 EQU   ACBMACRF
ACBADR   EQU   ACBMACR1
ACBCNV   EQU   ACBMACR1
ACBMACR2 EQU   ACBMACRF+1
ACBBUFPL EQU   ACBCOMN+20
ACBMACR3 EQU   ACBBUFPL
ACBRECFM EQU   ACBCOMN+24
ACBCCTYP EQU   ACBCOMN+25
ACBOPT   EQU   ACBCOMN+26
ACBDSORG EQU   ACBOPT
ACBCROPS EQU   ACBDSORG
ACBEXLST EQU   ACBCOMN+36
ACBDDNM  EQU   ACBCOMN+40
ACBAMETH EQU   ACBDDNM+3
ACBOFLGS EQU   ACBCOMN+48
ACBEXFG  EQU   ACBOFLGS
ACBIOSFG EQU   ACBOFLGS
ACBINFLG EQU   ACBCOMN+50
ACBUJFCB EQU   ACBCOMN+52
ACBBLKSZ EQU   ACBCOMN+60
ACBUAPTR EQU   ACBCOMN+64
AMCBS    EQU   0
CBSCMP   EQU   AMCBS+16
CBSVSRT  EQU   AMCBS+32
IGGCAXWA EQU   0
CAXFLGS  EQU   IGGCAXWA+8
CAXFLG2  EQU   IGGCAXWA+9
CAXFLG3  EQU   IGGCAXWA+10
CAXCCR   EQU   IGGCAXWA+32
CAXCNAM  EQU   IGGCAXWA+52
CAXOPLST EQU   IGGCAXWA+96
COPTS    EQU   CAXOPLST
CTGFV    EQU   0
CTGFVPRO EQU   CTGFV+1
CTGFVSPC EQU   CTGFV+52
CTGFVAMD EQU   CTGFV+56
CTGFVLMT EQU   CTGFV+72
CTGFVGAT EQU   CTGFV+76
CTGWA    EQU   0
CTGWAVL  EQU   CTGWA
CTGVL    EQU   0
CTGVLX   EQU   0
CTGCV    EQU   0
CTGCVDSN EQU   CTGCV+6
MWA      EQU   0
VLST     EQU   MWA+2
CVOLIND  EQU   MWA+252
DSPTBL   EQU   0
CMSWA    EQU   0
CMWFLAGS EQU   CMSWA
CMCISUSE EQU   CMSWA+2
CMWRKATR EQU   CMCISUSE
CMCPLFVT EQU   CMSWA+20
CMNRMEXT EQU   CMSWA+24
CMLDTFPL EQU   CMNRMEXT
CMSSEXTS EQU   CMSWA+28
CMHDTFPL EQU   CMSSEXTS
CMCANVOL EQU   CMSWA+32
CMHIXFPL EQU   CMCANVOL
CMRNGCTR EQU   CMSWA+40
CMVOLCTR EQU   CMSWA+42
CMSDFSPC EQU   CMSWA+48
CMVLLIST EQU   CMSWA+64
CMVLRTCD EQU   CMVLLIST+2
CMSPCATR EQU   CMSWA+76
CMWASTOR EQU   CMSWA+132
CMPOT    EQU   CMWASTOR
CMDCONV  EQU   CMPOT
CMIXFANO EQU   CMDCONV
CMPOTX   EQU   CMWASTOR+4
CMPBI    EQU   CMPOTX
CMIXFANS EQU   CMPBI
CMWABYTB EQU   CMSWA+140
CMPBIX   EQU   CMWABYTB
CMNEST   EQU   CMPBIX
CMFANT   EQU   CMNEST
CMSPCQTY EQU   CMFANT
CMWGENAM EQU   CMSWA+144
CMWDATE  EQU   CMWGENAM+12
CMWTIME  EQU   CMWGENAM+17
DCWPLSPS EQU   CMSWA+200
DCWPLSPL EQU   DCWPLSPS
DCWPLBFS EQU   CMSWA+232
DCWPLBFL EQU   DCWPLBFS
DCWPLEDS EQU   CMSWA+260
DCWPLEDL EQU   DCWPLEDS
DCWPLARS EQU   CMSWA+288
DCWPLARL EQU   DCWPLARS
DCWPLCDS EQU   CMSWA+316
DCWPLCDL EQU   DCWPLCDS
CMUCATCI EQU   CMSWA+410
CUMPLWA  EQU   0
CUMPL    EQU   CUMPLWA
CUMIN    EQU   CUMPL
CUMFDEL  EQU   CUMIN
CUMNRG   EQU   CUMIN
CUMFUNC  EQU   CUMIN
CUMOUT   EQU   CUMPL+1
CUMDUP   EQU   CUMOUT
CUMNE    EQU   CUMOUT
CUMDCI   EQU   CUMPL+2
CUMICI   EQU   CUMPL+5
CUMWA    EQU   CUMPLWA+8
CUMFLAG  EQU   CUMWA
CUMEXIT  EQU   CUMFLAG
CUMBCI   EQU   CUMWA+1
CUMURAB  EQU   CUMWA+4
CUMRABB  EQU   CUMWA+8
CUMRAB1  EQU   CUMWA+12
CUMRAB2  EQU   CUMWA+16
CUMRABD  EQU   CUMWA+20
CUMSRID  EQU   CUMWA+25
CUMDY    EQU   CUMWA+29
CUMFN1   EQU   CUMWA+32
CUMFN2   EQU   CUMWA+40
CUMWASS2 EQU   CUMWA+48
CUMWREL  EQU   CUMWASS2
CUMWASS  EQU   CUMWASS2+2
CUMWTYP  EQU   CUMWASS
CUMTYPE  EQU   CUMWTYP
CUMWCI   EQU   CUMWASS+1
CUMWTYP2 EQU   CUMWASS2+6
CUMWCI2  EQU   CUMWASS2+7
CUMFMWA  EQU   CUMWA+58
CUMRC    EQU   CUMWA+148
CATSN    EQU   0
CI       EQU   0
CTGVLDSP EQU   0
FIELDNM  EQU   0
TNAM     EQU   0
@NM00035 EQU   CTGDDNM
CTGPROB  EQU   @NM00035
CTGREASN EQU   CTGPROB+2
CVTS01   EQU   CVTPGSIA
CVTLPDIA EQU   CVTS01+12
CVTDIRST EQU   CVTLPDIA
CVTSLIDA EQU   CVTS01+24
CVTCTLFG EQU   CVTS01+50
CVTRV210 EQU   CVTS01+424
CVTRV219 EQU   CVTS01+425
CVTRV228 EQU   CVTS01+426
CVTRV237 EQU   CVTS01+427
CVTMFRTR EQU   CVTS01+452
CVTRV262 EQU   CVTS01+468
CVTRV271 EQU   CVTS01+469
CVTRV280 EQU   CVTS01+470
CVTRV289 EQU   CVTS01+471
CVTGSDA  EQU   CVTS01+600
PSARSVTE EQU   PSARSVT
MCATDSTR EQU   CVTAMFF
CVTAMF1  EQU   MCATDSTR
FLC      EQU   PSA
         AGO   .@UNREFD                START UNREFERENCED COMPONENTS
@NM00085 EQU   MCATDSTR+1
@NM00084 EQU   CVTAMF1
AMF1NONX EQU   CVTAMF1
AMF1OPEN EQU   CVTAMF1
PSARSAV  EQU   PSARSVTE+60
PSARSTK  EQU   PSARSVTE+56
PSAESAV3 EQU   PSARSVTE+52
PSAESTK3 EQU   PSARSVTE+48
PSAESAV2 EQU   PSARSVTE+44
PSAESTK2 EQU   PSARSVTE+40
PSAESAV1 EQU   PSARSVTE+36
PSAESTK1 EQU   PSARSVTE+32
PSAPSAV  EQU   PSARSVTE+28
PSAPSTK  EQU   PSARSVTE+24
PSAMSAV  EQU   PSARSVTE+20
PSAMSTK  EQU   PSARSVTE+16
PSASSAV  EQU   PSARSVTE+12
PSASSTK  EQU   PSARSVTE+8
PSANSTK  EQU   PSARSVTE+4
PSACSTK  EQU   PSARSVTE
CVTRV628 EQU   CVTS01+728
CVTRV627 EQU   CVTS01+724
CVTRV626 EQU   CVTS01+720
CVTRV625 EQU   CVTS01+716
CVTRV624 EQU   CVTS01+712
CVTRV623 EQU   CVTS01+708
CVTRV622 EQU   CVTS01+704
CVTRV621 EQU   CVTS01+700
CVTRV620 EQU   CVTS01+696
CVTRV619 EQU   CVTS01+692
CVTRV618 EQU   CVTS01+688
CVTRV617 EQU   CVTS01+684
CVTRV616 EQU   CVTS01+680
CVTRV615 EQU   CVTS01+676
CVTRV614 EQU   CVTS01+672
CVTRV613 EQU   CVTS01+668
CVTRV612 EQU   CVTS01+664
CVTRV611 EQU   CVTS01+660
CVTRV610 EQU   CVTS01+656
CVTRV609 EQU   CVTS01+652
CVTRV608 EQU   CVTS01+648
CVTRV607 EQU   CVTS01+644
CVTRV606 EQU   CVTS01+640
CVTRV605 EQU   CVTS01+636
CVTRV604 EQU   CVTS01+632
CVTEFF02 EQU   CVTS01+628
CVTCBBR  EQU   CVTS01+624
CVTSSCR  EQU   CVTS01+620
CVTEVENT EQU   CVTS01+616
CVTCRCA  EQU   CVTS01+612
CVTTPIO  EQU   CVTS01+608
CVTADV   EQU   CVTS01+604
CVTGSDAB EQU   CVTGSDA
CVTQV3   EQU   CVTS01+596
CVTQV2   EQU   CVTS01+592
CVTQV1   EQU   CVTS01+588
CVTRPT   EQU   CVTS01+584
CVTSSRB  EQU   CVTS01+580
CVTCSDRL EQU   CVTS01+576
CVTEXP1  EQU   CVTS01+572
CVTRMPMT EQU   CVTS01+568
CVTRMPTT EQU   CVTS01+564
CVTVPSA  EQU   CVTS01+560
CVTVSTOP EQU   CVTS01+556
CVTGTFR8 EQU   CVTS01+552
CVTQUIT  EQU   CVTS01+548
CVTVACR  EQU   CVTS01+544
CVTWTCB  EQU   CVTS01+540
CVTSTPRS EQU   CVTS01+536
CVT0PT02 EQU   CVTS01+532
CVTDARCM EQU   CVTS01+528
CVTIRECM EQU   CVTS01+524
CVTJRECM EQU   CVTS01+520
CVTVEMS0 EQU   CVTS01+516
CVTSPFRR EQU   CVTS01+512
CVTRLSTG EQU   CVTS01+508
CVT0TC0A EQU   CVTS01+504
CVTGMBR  EQU   CVTS01+500
CVTLFRM  EQU   CVTS01+496
CVTRMBR  EQU   CVTS01+492
CVTVIOP  EQU   CVTS01+488
CVTRV307 EQU   CVTS01+486
CVTRV306 EQU   CVTS01+484
CVTRV305 EQU   CVTS01+482
CVTRV304 EQU   CVTS01+480
CVTRV303 EQU   CVTS01+478
CVTRV302 EQU   CVTS01+476
CVTTRCA  EQU   CVTS01+472
CVTRV297 EQU   CVTRV289
CVTRV296 EQU   CVTRV289
CVTRV295 EQU   CVTRV289
CVTRV294 EQU   CVTRV289
CVTRV293 EQU   CVTRV289
CVTRV292 EQU   CVTRV289
CVTRV291 EQU   CVTRV289
CVTRV290 EQU   CVTRV289
CVTRV288 EQU   CVTRV280
CVTRV287 EQU   CVTRV280
CVTRV286 EQU   CVTRV280
CVTRV285 EQU   CVTRV280
CVTRV284 EQU   CVTRV280
CVTRV283 EQU   CVTRV280
CVTRV282 EQU   CVTRV280
CVTRV281 EQU   CVTRV280
CVTRV279 EQU   CVTRV271
CVTRV278 EQU   CVTRV271
CVTRV277 EQU   CVTRV271
CVTRV276 EQU   CVTRV271
CVTRV275 EQU   CVTRV271
CVTRV274 EQU   CVTRV271
CVTRV273 EQU   CVTRV271
CVTRV272 EQU   CVTRV271
CVTRV270 EQU   CVTRV262
CVTRV269 EQU   CVTRV262
CVTRV268 EQU   CVTRV262
CVTRV267 EQU   CVTRV262
CVTRV266 EQU   CVTRV262
CVTRV265 EQU   CVTRV262
CVTRV264 EQU   CVTRV262
CVTRV263 EQU   CVTRV262
CVTVFP   EQU   CVTS01+464
CVTVSI   EQU   CVTS01+460
CVTVPSIB EQU   CVTS01+456
CVTMFACT EQU   CVTMFRTR
CVTMFCTL EQU   CVTS01+448
CVTPVBP  EQU   CVTS01+444
CVTPWI   EQU   CVTS01+440
CVTRV254 EQU   CVTS01+438
CVTRV253 EQU   CVTS01+436
CVTRV252 EQU   CVTS01+434
CVTRV251 EQU   CVTS01+433
CVTRV250 EQU   CVTS01+432
CVTRV249 EQU   CVTS01+431
CVTRV248 EQU   CVTS01+430
CVTRV247 EQU   CVTS01+429
CVTRV246 EQU   CVTS01+428
CVTRV245 EQU   CVTRV237
CVTRV244 EQU   CVTRV237
CVTRV243 EQU   CVTRV237
CVTRV242 EQU   CVTRV237
CVTRV241 EQU   CVTRV237
CVTRV240 EQU   CVTRV237
CVTRV239 EQU   CVTRV237
CVTRV238 EQU   CVTRV237
CVTRV236 EQU   CVTRV228
CVTRV235 EQU   CVTRV228
CVTRV234 EQU   CVTRV228
CVTRV233 EQU   CVTRV228
CVTRV232 EQU   CVTRV228
CVTRV231 EQU   CVTRV228
CVTRV230 EQU   CVTRV228
CVTRV229 EQU   CVTRV228
CVTRV227 EQU   CVTRV219
CVTRV226 EQU   CVTRV219
CVTRV225 EQU   CVTRV219
CVTRV224 EQU   CVTRV219
CVTRV223 EQU   CVTRV219
CVTRV222 EQU   CVTRV219
CVTRV221 EQU   CVTRV219
CVTRV220 EQU   CVTRV219
CVTRV218 EQU   CVTRV210
CVTRV217 EQU   CVTRV210
CVTRV216 EQU   CVTRV210
CVTRV215 EQU   CVTRV210
CVTRV214 EQU   CVTRV210
CVTRV213 EQU   CVTRV210
CVTRV212 EQU   CVTRV210
CVTRV211 EQU   CVTRV210
CVTLCCAT EQU   CVTS01+420
CVTPCCAT EQU   CVTS01+416
CVTIPCRP EQU   CVTS01+412
CVTIPCRI EQU   CVTS01+408
CVTIPCDS EQU   CVTS01+404
CVTAIDVT EQU   CVTS01+400
CVTSSAP  EQU   CVTS01+396
CVTEHCIR EQU   CVTS01+392
CVTEHDEF EQU   CVTS01+388
CVTDAIR  EQU   CVTS01+384
CVTPERFM EQU   CVTS01+380
CVT044R2 EQU   CVTS01+376
CVTFETCH EQU   CVTS01+372
CVTRSTWD EQU   CVTS01+368
CVTSPOST EQU   CVTS01+364
CVTIOBP  EQU   CVTS01+360
CVTASMVT EQU   CVTS01+356
CVTRECRQ EQU   CVTS01+352
CVTWSAC  EQU   CVTS01+348
CVTRV149 EQU   CVTS01+344
CVTWSAL  EQU   CVTS01+340
CVTSPSA  EQU   CVTS01+336
CVTGLMN  EQU   CVTS01+332
CVTVEAC0 EQU   CVTS01+328
CVT062R1 EQU   CVTS01+324
CVTRPOST EQU   CVTS01+320
CVTDQIQE EQU   CVTS01+316
CVTCSD   EQU   CVTS01+312
CVTLKRMA EQU   CVTS01+308
CVTRSPIE EQU   CVTS01+304
CVTRENQ  EQU   CVTS01+300
CVTLQCB  EQU   CVTS01+296
CVTFQCB  EQU   CVTS01+292
CVTQCS01 EQU   CVTS01+288
CVTAPFT  EQU   CVTS01+284
CVTPARRL EQU   CVTS01+280
CVTVWAIT EQU   CVTS01+276
CVTGSPL  EQU   CVTS01+272
CVTLSMQ  EQU   CVTS01+268
CVTGSMQ  EQU   CVTS01+264
CVTEXPRO EQU   CVTS01+260
CVTOPCTP EQU   CVTS01+256
CVTSIC   EQU   CVTS01+252
CVTTPIOS EQU   CVTS01+248
CVTRTMS  EQU   CVTS01+244
CVTSDBF  EQU   CVTS01+240
CVTSCBP  EQU   CVTS01+236
CVTSDMP  EQU   CVTS01+232
CVTSV60  EQU   CVTS01+228
CVTRTMCT EQU   CVTS01+224
CVTASCBL EQU   CVTS01+220
CVTASCBH EQU   CVTS01+216
CVTGDA   EQU   CVTS01+212
CVTASVT  EQU   CVTS01+208
CVTVVMDI EQU   CVTS01+204
CVTAQTOP EQU   CVTS01+200
CVTIOSCS EQU   CVTS01+196
CVTSDRM  EQU   CVTS01+192
CVTOPTE  EQU   CVTS01+188
CVTSTXU  EQU   CVTS01+184
CVTQUIS  EQU   CVTS01+180
CVTPARS  EQU   CVTS01+176
CVTS1EE  EQU   CVTS01+172
CVTFRAS  EQU   CVTS01+168
CVTQSAS  EQU   CVTS01+164
CVTCRAS  EQU   CVTS01+160
CVTCRMN  EQU   CVTS01+156
CVTDELCP EQU   CVTS01+152
CVTFRECL EQU   CVTS01+148
CVTGETCL EQU   CVTS01+144
CVTBLDCP EQU   CVTS01+140
CVTAUTHL EQU   CVTS01+136
CVTSCAN  EQU   CVTS01+132
CVTRV144 EQU   CVTS01+130
CVTMAXMP EQU   CVTS01+128
CVTSTCK  EQU   CVTS01+124
CVTRV139 EQU   CVTS01+123
CVTDSSAC EQU   CVTS01+122
CVTRV513 EQU   CVTS01+121
CVTIOSPL EQU   CVTS01+120
CVTPTGT  EQU   CVTS01+116
CVTCSPIE EQU   CVTS01+112
CVTSMFEX EQU   CVTS01+108
CVTOLT0A EQU   CVTS01+104
CVTSRBRT EQU   CVTS01+100
CVTPUTL  EQU   CVTS01+96
CVTRV519 EQU   CVTS01+92
CVTRV327 EQU   CVTS01+88
CVTRV326 EQU   CVTS01+84
CVTRV325 EQU   CVTS01+80
CVTRV324 EQU   CVTS01+76
CVT0VL01 EQU   CVTS01+72
CVTSHRVM EQU   CVTS01+68
CVTRV332 EQU   CVTS01+64
CVTTAS   EQU   CVTS01+60
CVTRSCN  EQU   CVTS01+56
CVTTRAC2 EQU   CVTS01+54
CVTTRACE EQU   CVTS01+52
CVTAPG   EQU   CVTS01+51
CVTSDTRC EQU   CVTCTLFG
CVTGTRCE EQU   CVTCTLFG
CVTNOMP  EQU   CVTCTLFG
CVTRSV79 EQU   CVTCTLFG
CVTDSTAT EQU   CVTCTLFG
CVTRSV78 EQU   CVTCTLFG
CVTRV333 EQU   CVTCTLFG
CVTRV323 EQU   CVTCTLFG
CVTSPVLK EQU   CVTS01+49
CVTRSV77 EQU   CVTS01+48
CVTRV331 EQU   CVTS01+44
CVTRV330 EQU   CVTS01+40
CVTRV329 EQU   CVTS01+36
CVTRV328 EQU   CVTS01+32
CVTRV322 EQU   CVTS01+28
CVTSLID  EQU   CVTSLIDA+1
CVTSYLK  EQU   CVTSLIDA
CVTRV321 EQU   CVTS01+20
CVTRV320 EQU   CVTS01+16
CVTLPDIR EQU   CVTLPDIA+1
CVTRSV69 EQU   CVTDIRST
CVTRSV68 EQU   CVTDIRST
CVTRSV67 EQU   CVTDIRST
CVTRSV66 EQU   CVTDIRST
CVTRSV65 EQU   CVTDIRST
CVTRSV64 EQU   CVTDIRST
CVTRSV63 EQU   CVTDIRST
CVTDICOM EQU   CVTDIRST
CVTPVTP  EQU   CVTS01+8
CVTLPDSR EQU   CVTS01+4
CVTGETL  EQU   CVTS01
CTGREAS2 EQU   CTGREASN+1
CTGREAS1 EQU   CTGREASN
CTGMODID EQU   CTGPROB
@NM00107 EQU   CUMWA+146
@NM00106 EQU   CUMWA+28
@NM00105 EQU   CUMWA+24
@NM00104 EQU   CUMFLAG
@NM00103 EQU   CUMOUT
@NM00102 EQU   CUMIN
CMBYTAU  EQU   CMSWA+500
CMBYTTRK EQU   CMSWA+496
CMSCTREC EQU   CMSWA+416
@NM00101 EQU   CMSWA+413
CMDEFACI EQU   CMUCATCI
CMBLPTKS EQU   CMSWA+408
CMSDSPTB EQU   CMSWA+344
@NM00100 EQU   DCWPLCDS+27
DCWPLCDV EQU   DCWPLCDS+24
DCWPLCDA EQU   DCWPLCDL+16
DCWPLCDH EQU   DCWPLCDL
DCWPLARV EQU   DCWPLARS+24
DCWPLARA EQU   DCWPLARL+16
DCWPLARH EQU   DCWPLARL
@NM00099 EQU   DCWPLEDS+27
DCWPLEDV EQU   DCWPLEDS+24
DCWPLEDA EQU   DCWPLEDL+16
DCWPLEDH EQU   DCWPLEDL
DCWPLBFV EQU   DCWPLBFS+24
DCWPLBFA EQU   DCWPLBFL+16
DCWPLBFH EQU   DCWPLBFL
@NM00098 EQU   DCWPLSPS+31
DCWPLSPV EQU   DCWPLSPS+24
DCWPLSPA EQU   DCWPLSPL+16
DCWPLSPH EQU   DCWPLSPL
DCWFPLPT EQU   CMSWA+196
DCWAREPT EQU   CMSWA+192
@NM00097 EQU   CMSWA+190
DCWIX    EQU   CMSWA+188
CMWPAD   EQU   CMWGENAM+35
CMWTIME2 EQU   CMWTIME+11
CMWDELM2 EQU   CMWTIME+9
CMWTIME1 EQU   CMWTIME+2
CMWDELM1 EQU   CMWTIME
CMWDAY   EQU   CMWDATE+2
CMWYEAR  EQU   CMWDATE
CMWFIXID EQU   CMWGENAM
CMNCILL  EQU   CMSPCQTY
CMNCITL  EQU   CMIXFANS
CMWT     EQU   CMIXFANO
CMDEFSWA EQU   CMSWA+128
CMSRCCOR EQU   CMSWA+124
CMEXTLST EQU   CMSWA+120
WORKPROC EQU   CMSWA+118
@NM00096 EQU   CMSWA+116
CMFPLIDX EQU   CMSWA+112
CMCNVNO  EQU   CMSWA+110
CMCNVLEN EQU   CMSWA+108
CMTRKCYL EQU   CMSPCATR+12
CMTRKPCA EQU   CMSPCATR+10
CMBLKTRK EQU   CMSPCATR+8
CMSWADTY EQU   CMSPCATR+4
CMPHYBLK EQU   CMSPCATR
CMTOTVOL EQU   CMSWA+74
CMVLVOL  EQU   CMVLLIST+4
CMVLRT2  EQU   CMVLRTCD+1
CMVLRT1  EQU   CMVLRTCD
CMVLLTLN EQU   CMVLLIST
@NM00095 EQU   CMSWA+62
CMSCRADT EQU   CMSWA+58
CMSCRAVI EQU   CMSWA+52
CMSDEFCI EQU   CMSDFSPC+1
CMSWAFLG EQU   CMSDFSPC
CMSPCFVT EQU   CMSWA+44
CMRMVVOL EQU   CMVOLCTR
CMADDVOL EQU   CMRNGCTR
SPACINDC EQU   CMSWA+38
PROCESS  EQU   CMSWA+36
CMDTAFPL EQU   CMHIXFPL
CMTSTFPL EQU   CMHDTFPL
CMRNMLST EQU   CMLDTFPL
CMEXTFLS EQU   CMCPLFVT
CMASCFPL EQU   CMSWA+16
CMWFVTPT EQU   CMSWA+12
CMWACI   EQU   CMSWA+3
CMAMDRNG EQU   CMWRKATR
CMAMDORD EQU   CMWRKATR
CMAMDRPL EQU   CMWRKATR
CMAMDSDT EQU   CMWRKATR
CMAMDWCK EQU   CMWRKATR
CMAMDDST EQU   CMWRKATR
CMCISREQ EQU   CMSWA+1
CMPROC   EQU   CMWFLAGS
CMPROCM  EQU   CMWFLAGS
CMFGDLSC EQU   CMWFLAGS
DSNPHYBS EQU   DSPTBL+30
DSPHYBPT EQU   DSPTBL+22
DSMPHYBS EQU   DSPTBL+20
DSDEVCHA EQU   DSPTBL
CVLSER   EQU   MWA+259
TTR2     EQU   MWA+256
BIN0     EQU   CVOLIND+3
TTR1     EQU   CVOLIND
@NM00094 EQU   MWA+242
FSN      EQU   VLST+10
VOLSER   EQU   VLST+4
DEVTYP   EQU   VLST
VOLENTCT EQU   MWA
CTGCVCC  EQU   CTGCV+50
CTGCVDEV EQU   CTGCVDSN
CTGCVVOL EQU   CTGCV
CTGVLTTR EQU   CTGVLX+12
@NM00093 EQU   CTGVLX
CTGVLSEQ EQU   CTGVL+10
CTGVLDEV EQU   CTGVL+6
CTGVLVOL EQU   CTGVL
CTGWAGB  EQU   CTGWA+12
CTGWAGCT EQU   CTGWA+10
CTGWAUCT EQU   CTGWA+8
CTGWAVCT EQU   CTGWA+6
CTGWALV  EQU   CTGWA+4
CTGFVWKA EQU   CTGFV+88
CTGFVPWD EQU   CTGFV+84
CTGFVNAM EQU   CTGFV+80
CTGFVUPG EQU   CTGFVGAT
CTGFVEXT EQU   CTGFVLMT
CTGFVLRS EQU   CTGFV+68
CTGFVBUF EQU   CTGFV+64
CTGFVATR EQU   CTGFV+60
CTGFVFSN EQU   CTGFVAMD
CTGFVTTR EQU   CTGFVSPC
CTGFVDVT EQU   CTGFV+48
CTGFVRNG EQU   CTGFV+44
CTGFVVLT EQU   CTGFV+40
CTGFVCRE EQU   CTGFV+36
CTGFVEXP EQU   CTGFV+32
CTGFVOWN EQU   CTGFV+28
CTGFVSTY EQU   CTGFV+24
CTGFVENT EQU   CTGFV+20
CTGFVIND EQU   CTGFV+16
CTGFVVCH EQU   CTGFV+12
CTGFVICH EQU   CTGFV+8
CTGFVDCH EQU   CTGFV+4
@NM00092 EQU   CTGFV+3
CTGFVELM EQU   CTGFV+2
@NM00091 EQU   CTGFVPRO
CTGFVDRC EQU   CTGFVPRO
CTGFVNDC EQU   CTGFVPRO
CTGFVRVL EQU   CTGFVPRO
CTGFVAVL EQU   CTGFVPRO
CTGFVTYP EQU   CTGFV
CAXCRACB EQU   IGGCAXWA+112
CAXJDE   EQU   IGGCAXWA+108
CAXCCA   EQU   IGGCAXWA+104
CAXOPEWA EQU   IGGCAXWA+100
COPACB   EQU   CAXOPLST+1
@NM00090 EQU   COPTS
CENLST   EQU   COPTS
@NM00089 EQU   CAXCNAM+22
CAXRADDN EQU   CAXCNAM+14
CAXRATEP EQU   CAXCNAM+10
CAXRACTS EQU   CAXCNAM+6
CAXVOLID EQU   CAXCNAM
CAXRPL   EQU   IGGCAXWA+48
CAXRPLCT EQU   IGGCAXWA+46
CAXASID  EQU   IGGCAXWA+44
CAXFDCI  EQU   CAXCCR+9
CAXCDCI  EQU   CAXCCR+6
CAXNFCI  EQU   CAXCCR+3
CAXHACI  EQU   CAXCCR
CAXUCB   EQU   IGGCAXWA+28
CAXACB   EQU   IGGCAXWA+24
CAXDRWP  EQU   IGGCAXWA+20
CAXSCHWA EQU   IGGCAXWA+16
CAXATIOT EQU   IGGCAXWA+12
CAXACT   EQU   IGGCAXWA+11
@NM00088 EQU   CAXFLG3
CAXF3B6  EQU   CAXFLG3
CAXF3B5  EQU   CAXFLG3
@NM00087 EQU   CAXFLG3
CAXF3ANE EQU   CAXFLG3
CAXF3AT  EQU   CAXFLG3
CAXF2VTU EQU   CAXFLG2
CAXF2REC EQU   CAXFLG2
CAXF2CA  EQU   CAXFLG2
CAXF2CLR EQU   CAXFLG2
CAXF2IOE EQU   CAXFLG2
CAXF2NCR EQU   CAXFLG2
CAXF2NDD EQU   CAXFLG2
CAXF2DT  EQU   CAXFLG2
CAXSCR   EQU   CAXFLGS
CAXCMR   EQU   CAXFLGS
CAXMCT   EQU   CAXFLGS
CAXCMP   EQU   CAXFLGS
CAXEOV   EQU   CAXFLGS
CAXCLS   EQU   CAXFLGS
CAXOPN   EQU   CAXFLGS
CAXBLD   EQU   CAXFLGS
CAXCHN   EQU   IGGCAXWA+4
@NM00086 EQU   IGGCAXWA+1
CAXID    EQU   IGGCAXWA
CBSVPTR  EQU   CBSVSRT+4
CBSVUSE  EQU   CBSVSRT
CBSCRTCB EQU   AMCBS+28
CBSCRACA EQU   AMCBS+24
CBSCAXCN EQU   AMCBS+20
CBSMCUCB EQU   CBSCMP
CBSCBP   EQU   AMCBS+12
CBSACB   EQU   AMCBS+8
CBSMCSTA EQU   AMCBS+4
CBSSIZ   EQU   AMCBS+2
CBSID    EQU   AMCBS
ACBAPID  EQU   ACBCOMN+72
ACBCBMWA EQU   ACBCOMN+68
ACBLRECL EQU   ACBCOMN+62
ACBMSGLN EQU   ACBBLKSZ
ACBBUFSP EQU   ACBCOMN+56
@NM00083 EQU   ACBUJFCB+1
ACBOPTN  EQU   ACBUJFCB
ACBVVIC  EQU   ACBINFLG
ACBUCRA  EQU   ACBINFLG
ACBSCRA  EQU   ACBINFLG
ACBCAT   EQU   ACBINFLG
ACBIJRQE EQU   ACBINFLG
ACBJEPS  EQU   ACBINFLG
@NM00082 EQU   ACBINFLG
ACBERFLG EQU   ACBCOMN+49
ACBBUSY  EQU   ACBIOSFG
ACBLOCK  EQU   ACBEXFG
@NM00081 EQU   ACBOFLGS
ACBDSERR EQU   ACBOFLGS
ACBOPEN  EQU   ACBOFLGS
ACBEOV   EQU   ACBOFLGS
@NM00080 EQU   ACBOFLGS
ACBDEB   EQU   ACBDDNM+5
ACBERFL  EQU   ACBDDNM+4
ACBAM    EQU   ACBAMETH
ACBINFL  EQU   ACBDDNM+2
ACBTIOT  EQU   ACBDDNM
ACBUEL   EQU   ACBEXLST
ACBPASSW EQU   ACBCOMN+32
ACBMSGAR EQU   ACBCOMN+28
@NM00079 EQU   ACBDSORG+1
ACBDORGA EQU   ACBDSORG+1
@NM00078 EQU   ACBDSORG
ACBCRNRE EQU   ACBCROPS
ACBCRNCK EQU   ACBCROPS
ACBASA   EQU   ACBCCTYP
@NM00077 EQU   ACBCCTYP
ACBRECAF EQU   ACBRECFM
ACBJBUF  EQU   ACBBUFPL+2
ACBMACR4 EQU   ACBBUFPL+1
@NM00076 EQU   ACBMACR3
ACBNCFX  EQU   ACBMACR3
ACBSIS   EQU   ACBMACR3
ACBDFR   EQU   ACBMACR3
ACBICI   EQU   ACBMACR3
ACBGSR   EQU   ACBMACR3
ACBLSR   EQU   ACBMACR3
@NM00075 EQU   ACBMACR3
ACBBUFNI EQU   ACBCOMN+18
ACBBUFND EQU   ACBCOMN+16
ACBSTRNO EQU   ACBCOMN+15
ACBBSTNO EQU   ACBCOMN+14
ACBAIX   EQU   ACBMACR2
ACBDSN   EQU   ACBMACR2
ACBRST   EQU   ACBMACR2
ACBLOGON EQU   ACBMACR2
ACBSKP   EQU   ACBMACR2
@NM00074 EQU   ACBMACR2
ACBUBF   EQU   ACBMACR1
ACBOUT   EQU   ACBMACR1
ACBIN    EQU   ACBMACR1
ACBDIR   EQU   ACBMACR1
ACBSEQ   EQU   ACBMACR1
ACBBLK   EQU   ACBCNV
ACBADD   EQU   ACBADR
ACBKEY   EQU   ACBMACR1
ACBINRTN EQU   ACBCOMN+8
ACBIBCT  EQU   ACBJWA
ACBLEN2  EQU   ACBLENG2
ACBSTYP  EQU   ACBCOMN+1
ACBID    EQU   ACBCOMN
@NM00049 EQU   TCBXTNT2+32
TCBRV181 EQU   TCBRV173
TCBRV180 EQU   TCBRV173
TCBRV179 EQU   TCBRV173
TCBRV178 EQU   TCBRV173
TCBRV177 EQU   TCBRV173
TCBRV176 EQU   TCBRV173
TCBRV175 EQU   TCBRV173
TCBRV174 EQU   TCBRV173
TCBRV172 EQU   TCBRV164
TCBRV171 EQU   TCBRV164
TCBRV170 EQU   TCBRV164
TCBRV169 EQU   TCBRV164
TCBRV168 EQU   TCBRV164
TCBRV167 EQU   TCBRV164
TCBRV166 EQU   TCBRV164
TCBRV165 EQU   TCBRV164
TCBRV163 EQU   TCBRV155
TCBRV162 EQU   TCBRV155
TCBRV161 EQU   TCBRV155
TCBRV160 EQU   TCBRV155
TCBRV159 EQU   TCBRV155
TCBRV158 EQU   TCBRV155
TCBRV157 EQU   TCBRV155
TCBRV156 EQU   TCBRV155
TCBRV154 EQU   TCBRV146
TCBRV153 EQU   TCBRV146
TCBRV152 EQU   TCBRV146
TCBRV151 EQU   TCBRV146
TCBRV150 EQU   TCBRV146
TCBRV149 EQU   TCBRV146
TCBRV148 EQU   TCBRV146
TCBRV147 EQU   TCBRV146
TCBRV145 EQU   TCBXTNT2+26
TCBRV144 EQU   TCBXTNT2+24
TCBRV143 EQU   TCBXTNT2+20
TCBRV142 EQU   TCBXTNT2+16
TCBRSV49 EQU   TCBXTNT2+12
TCBRSV48 EQU   TCBXTNT2+8
TCBRCMP  EQU   TCBXTNT2+5
TCBRSVAB EQU   TCBXTNT2+4
TCBGTFA  EQU   TCBGTF+1
TCBRSV47 EQU   TCBTFLG
TCBRSV46 EQU   TCBTFLG
TCBRSV45 EQU   TCBTFLG
TCBRSV44 EQU   TCBTFLG
TCBRSV43 EQU   TCBTFLG
TCBDSPIT EQU   TCBTFLG
TCBERRTN EQU   TCBTFLG
TCBASYNC EQU   TCBTFLG
@NM00048 EQU   TCB+296
TCBSCBQ  EQU   TCB+292
TCBSWASA EQU   TCB+288
TCBDBTB  EQU   TCB+284
TCBRPT   EQU   TCB+280
TCBRV141 EQU   TCBRV133
TCBRV140 EQU   TCBRV133
TCBRV139 EQU   TCBRV133
TCBRV138 EQU   TCBRV133
TCBRV137 EQU   TCBRV133
TCBRV136 EQU   TCBRV133
TCBRV135 EQU   TCBRV133
TCBRV134 EQU   TCBRV133
TCBRV132 EQU   TCBRV124
TCBRV131 EQU   TCBRV124
TCBRV130 EQU   TCBRV124
TCBRV129 EQU   TCBRV124
TCBRV128 EQU   TCBRV124
TCBRV127 EQU   TCBRV124
TCBRV126 EQU   TCBRV124
TCBRV125 EQU   TCBRV124
TCBRV123 EQU   TCBFBYT2
TCBRV122 EQU   TCBFBYT2
TCBECBNV EQU   TCBFBYT2
TCBSSSYN EQU   TCBFBYT2
TCBFPRAP EQU   TCBFBYT2
TCBFDW   EQU   TCBFBYT2
TCBFMW   EQU   TCBFBYT2
TCBCNCB  EQU   TCBFBYT2
TCBLLH   EQU   TCBFBYT1
TCBRV113 EQU   TCBFBYT1
TCBEOT   EQU   TCBFBYT1
TCBRTM2  EQU   TCBFBYT1
TCBPGNLY EQU   TCBFBYT1
TCBNDIOS EQU   TCBFBYT1
TCBACTIV EQU   TCBFBYT1
TCBEOTFM EQU   TCBFBYT1
TCBAFFN  EQU   TCB+274
TCBCCPVI EQU   TCB+272
TCBUKYSP EQU   TCB+268
TCBMODE  EQU   TCBESTAE+3
TCBERTYP EQU   TCBESTAE+2
TCBRV314 EQU   TCBESTRM
TCBRV313 EQU   TCBESTRM
TCBRV312 EQU   TCBESTRM
TCBRV311 EQU   TCBESTRM
TCBRV310 EQU   TCBESTRM
TCBRV309 EQU   TCBESTRM
TCBRV308 EQU   TCBESTRM
TCBETERM EQU   TCBESTRM
TCBSCBKY EQU   TCBESTAE
TCBRTM12 EQU   TCB+260
TCBTCBID EQU   TCB+256
TCBRSV96 EQU   TCB+252
TCBSWA   EQU   TCB+248
TCBFOEA  EQU   TCBFOE+1
TCBRSV42 EQU   TCBFOE
TCBRV306 EQU   TCB+240
TCBRSV41 EQU   TCB+239
TCBTID   EQU   TCB+238
TCBRSVAA EQU   TCB+237
TCBABCUR EQU   TCB+236
TCBTMSAV EQU   TCB+232
TCBIOTIM EQU   TCB+228
TCBRTWA  EQU   TCB+224
TCBBACK  EQU   TCB+220
TCBTIRB  EQU   TCB+216
TCBAECB  EQU   TCB+212
TCBEXT2A EQU   TCBEXT2+1
TCBRSV39 EQU   TCBEXT2
TCBSTMCT EQU   TCB+207
TCBSYSCT EQU   TCB+206
TCBRSV37 EQU   TCB+205
TCBEXSVC EQU   TCBDAR
TCBRSV36 EQU   TCBDAR
TCBDARWT EQU   TCBDAR
TCBDARO  EQU   TCBDAR
TCBDARMC EQU   TCBDARC
TCBDARD  EQU   TCBDAR
TCBDARS  EQU   TCBDAR
TCBDARP  EQU   TCBDAR
TCBGTOFM EQU   TCBFLGS7
TCBADMP  EQU   TCBFLGS7
TCBRSTSK EQU   TCBFLGS7
TCBRSV35 EQU   TCBFLGS7
TCBSTACK EQU   TCBFLGS7
TCBRSV34 EQU   TCBFLGS7
TCBRSV33 EQU   TCBFLGS7
TCBGPECB EQU   TCBFLGS7
TCBNTJS  EQU   TCBFLGS6
TCBAPG   EQU   TCBFLGS6
TCBMIGR  EQU   TCBFLGS6
TCBRV303 EQU   TCBFLGS6
TCBSPVLK EQU   TCBFLGS6
TCBCPU   EQU   TCBFLGS6
TCBPIE17 EQU   TCBFLGS6
TCBRV    EQU   TCBFLGS6
TCBRSV79 EQU   TCBNDSP5
TCBRSV78 EQU   TCBNDSP5
TCBRSV77 EQU   TCBNDSP5
TCBRSV76 EQU   TCBNDSP5
TCBRSV75 EQU   TCBNDSP5
TCBRSV74 EQU   TCBNDSP5
TCBRSV95 EQU   TCBNDSP5
TCBRSV94 EQU   TCBNDSP5
TCBRSV93 EQU   TCBNDSP4
TCBRSV92 EQU   TCBNDSP4
TCBRSV91 EQU   TCBNDSP4
TCBRSV90 EQU   TCBNDSP4
TCBRSV89 EQU   TCBNDSP4
TCBRSV88 EQU   TCBNDSP4
TCBRSV87 EQU   TCBNDSP4
TCBRSV86 EQU   TCBNDSP4
TCBEXT1A EQU   TCBEXT1+1
TCBRSV32 EQU   TCBEXT1
TCBEXCPD EQU   TCB+192
TCBIOBRC EQU   TCB+188
TCBDDWTC EQU   TCB+186
TCBDDEXC EQU   TCB+184
TCBJSCBB EQU   TCBJSCB+1
@NM00047 EQU   TCBRECDE
TCBREC   EQU   TCBRECDE
TCBMDIDS EQU   TCB+176
TCBNDINT EQU   TCBNDSP3
TCBRSV27 EQU   TCBNDSP3
TCBRSV26 EQU   TCBNDSP3
TCBRSV25 EQU   TCBNDSP3
TCBRSV24 EQU   TCBNDSP3
TCBSRBND EQU   TCBNDSP3
TCBRV302 EQU   TCBNDSP3
TCBLJSND EQU   TCBNDSP3
TCBABE   EQU   TCBNDSP2
TCBDSS   EQU   TCBNDSP2
TCBOWAIT EQU   TCBNDSP2
TCBIWAIT EQU   TCBNDSP2
TCBNDTS  EQU   TCBNDSP2
TCBNDSVC EQU   TCBNDSP2
TCBSTPP  EQU   TCBNDSP2
TCBABD   EQU   TCBNDSP2
TCBRSV22 EQU   TCBNDSP1
TCBPIEND EQU   TCBNDSP1
TCBTPSP  EQU   TCBNDSP1
TCBDDRND EQU   TCBNDSP1
TCBRSPND EQU   TCBNDSP1
TCBRSTND EQU   TCBNDSP1
TCBDARPN EQU   TCBNDSP1
TCBDARTN EQU   TCBNDSP1
TCBNDSP0 EQU   TCBNDSP
TCBUSER  EQU   TCB+168
TCBTCTB  EQU   TCBTCT+1
TCBRSV9C EQU   TCBTCTGF
TCBRSV9B EQU   TCBTCTGF
TCBRSV9A EQU   TCBTCTGF
TCBRSV99 EQU   TCBTCTGF
TCBRSV98 EQU   TCBTCTGF
TCBRSV97 EQU   TCBTCTGF
TCBRSV20 EQU   TCBTCTGF
TCBSMFGF EQU   TCBTCTGF
TCBSTABB EQU   TCBSTAB+1
TCBSTCUR EQU   TCBNSTAE
TCBRV301 EQU   TCBNSTAE
TCBSYNCH EQU   TCBNSTAE
TCBHALT  EQU   TCBNSTAE
TCBPPSUP EQU   TCBNSTAE
TCB33E   EQU   TCBNSTAE
TCBQUIES EQU   TCBNSTAE
TCBSTABE EQU   TCBNSTAE
TCBAQE   EQU   TCB+156
TCBPQE   EQU   TCB+152
TCBTSDP  EQU   TCB+151
TCBTSLP  EQU   TCB+150
TCBSTPCT EQU   TCB+149
TCBCPUBN EQU   TCBTSFLG
TCBDYDSP EQU   TCBTSFLG
TCBRSV18 EQU   TCBTSFLG
TCBRSV17 EQU   TCBTSFLG
TCBTIOTG EQU   TCBTSFLG
TCBATT   EQU   TCBTSFLG
TCBSTPPR EQU   TCBTSFLG
TCBRV300 EQU   TCBTSTSK
TCBECB   EQU   TCB+144
TCBIQE   EQU   TCB+140
TCBLTC   EQU   TCB+136
TCBOTC   EQU   TCB+132
TCBNTC   EQU   TCB+128
TCBJSTCA EQU   TCBJSTCB+1
TCBRSV16 EQU   TCBJSTCB
TCBTQET  EQU   TCBTME
TCBTCB   EQU   TCB+116
TCBFSAB  EQU   TCBFSA+1
TCBQEL   EQU   TCBFSA
TCBGRS15 EQU   TCBGRS+60
TCBGRS14 EQU   TCBGRS+56
TCBGRS13 EQU   TCBGRS+52
TCBGRS12 EQU   TCBGRS+48
TCBGRS11 EQU   TCBGRS+44
TCBGRS10 EQU   TCBGRS+40
TCBGRS9  EQU   TCBGRS+36
TCBGRS8  EQU   TCBGRS+32
TCBGRS7  EQU   TCBGRS+28
TCBGRS6  EQU   TCBGRS+24
TCBGRS5  EQU   TCBGRS+20
TCBGRS4  EQU   TCBGRS+16
TCBGRS3  EQU   TCBGRS+12
TCBGRS2  EQU   TCBGRS+8
TCBGRS1  EQU   TCBGRS+4
TCBGRS0  EQU   TCBGRS
TCBJPQB  EQU   TCBJPQ+1
TCBRSV15 EQU   TCBPURGE
TCBRSV14 EQU   TCBPURGE
TCBRSV13 EQU   TCBPURGE
TCBRSV12 EQU   TCBPURGE
TCBRSV11 EQU   TCBPURGE
TCBRSV10 EQU   TCBPURGE
TCBRSV09 EQU   TCBPURGE
TCBJPQF  EQU   TCBPURGE
TCBJLB   EQU   TCB+40
TCBLLS   EQU   TCB+36
TCBDSP   EQU   TCB+35
TCBLMP   EQU   TCB+34
TCBPNDSP EQU   TCBFLGS5
TCBFCD1  EQU   TCBFLGS5
TCBSTP   EQU   TCBFLGS5
TCBSYS   EQU   TCBFLGS5
TCBANDSP EQU   TCBFLGS5
TCBPAGE  EQU   TCBFLGS5
TCBUXNDF EQU   TCBABWF
TCBFC    EQU   TCBFLGS5
TCBONDSP EQU   TCBFLGS4
TCBMPCND EQU   TCBFLGS4
TCBMPCVQ EQU   TCBFLGS4
TCBUXNDV EQU   TCBFLGS4
TCBHNDSP EQU   TCBFLGS4
TCBRQENA EQU   TCBFLGS4
TCBSER   EQU   TCBFLGS4
TCBNDUMP EQU   TCBFLGS4
TCBDWSTA EQU   TCBFLGS3
TCBRSV08 EQU   TCBFLGS3
TCBRSV07 EQU   TCBFLGS3
TCBRSV06 EQU   TCBFLGS3
TCBABGM  EQU   TCBFLGS3
TCBABTRM EQU   TCBFLGS3
TCBADINP EQU   TCBFLGS3
TCBFSM   EQU   TCBFLGS3
TCBFTS   EQU   TCBFLGS2
TCBFETXR EQU   TCBFLGS2
TCBFDSOP EQU   TCBFLGS2
TCBFJMC  EQU   TCBFLGS2
TCBFSMC  EQU   TCBFLGS2
TCBFABOP EQU   TCBFLGS2
TCBFSTI  EQU   TCBFLGS2
TCBFOINP EQU   TCBFLGS2
TCBFX    EQU   TCBFLGS1
TCBFS    EQU   TCBFLGS1
TCBFT    EQU   TCBFLGS1
TCBPDUMP EQU   TCBFLGS1
TCBRSV04 EQU   TCBFLGS1
TCBFERA  EQU   TCBFLGS1
TCBFE    EQU   TCBFLGS1
TCBFA    EQU   TCBFLGS1
TCBZERO  EQU   TCBPKF
TCBFLAG  EQU   TCBPKF
TCBMSSB  EQU   TCBMSS+1
TCBRSV03 EQU   TCBMSS
TCBTRNB  EQU   TCBTRN+1
TCBRSV02 EQU   TCBABF
TCBOLTEP EQU   TCBABF
TCBTCP   EQU   TCBABF
TCBTCPP  EQU   TCBABF
TCBRSV01 EQU   TCBABF
TCBGRPH  EQU   TCBABF
TCBNOCHK EQU   TCBABF
TCBMOD91 EQU   TCBABF
TCBCMPC  EQU   TCBCMP+1
TCBRV318 EQU   TCBCMSG
TCBRV317 EQU   TCBCIND
TCBRV316 EQU   TCBCWTO
TCBCASID EQU   TCBCDBL
TCBNOCC  EQU   TCBSTCC
TCBDMPO  EQU   TCBCPP
TCBCSTEP EQU   TCBCMPF
TCBCREQ  EQU   TCBCMPF
TCBTIO   EQU   TCB+12
TCBDEB   EQU   TCB+8
TCBPIEA  EQU   TCBPIE+1
TCBPM    EQU   TCBPMASK
@NM00046 EQU   TCBPMASK
TCBRBP   EQU   TCB
TCBFRS6  EQU   TCBFRS+24
TCBFRS4  EQU   TCBFRS+16
TCBFRS2  EQU   TCBFRS+8
TCBFRS0  EQU   TCBFRS
NEXTTCB  EQU   TCBWORDS
CVTLEVL  EQU   CVTRELNO+2
CVTNUMB  EQU   CVTRELNO
CVTMDL   EQU   CVTFIX+250
@NM00045 EQU   CVTFIX+248
@NM00044 EQU   CVTFIX
CVTRV482 EQU   CVTXTNT2+128
CVTRV481 EQU   CVTXTNT2+124
CVTRV480 EQU   CVTXTNT2+120
CVTRV479 EQU   CVTXTNT2+118
CVTRV478 EQU   CVTXTNT2+117
CVTRV477 EQU   CVTXTNT2+116
CVTRV476 EQU   CVTXTNT2+115
CVTRV475 EQU   CVTXTNT2+114
CVTRV474 EQU   CVTRV466
CVTRV473 EQU   CVTRV466
CVTRV472 EQU   CVTRV466
CVTRV471 EQU   CVTRV466
CVTRV470 EQU   CVTRV466
CVTRV469 EQU   CVTRV466
CVTRV468 EQU   CVTRV466
CVTRV467 EQU   CVTRV466
CVTRV465 EQU   CVTRV457
CVTRV464 EQU   CVTRV457
CVTRV463 EQU   CVTRV457
CVTRV462 EQU   CVTRV457
CVTRV461 EQU   CVTRV457
CVTRV460 EQU   CVTRV457
CVTRV459 EQU   CVTRV457
CVTRV458 EQU   CVTRV457
CVTRV456 EQU   CVTXTNT2+108
CVTRV455 EQU   CVTXTNT2+104
CVTRV454 EQU   CVTXTNT2+100
CVTRV453 EQU   CVTXTNT2+96
CVTRV452 EQU   CVTXTNT2+94
CVTRV451 EQU   CVTXTNT2+92
CVTRV450 EQU   CVTXTNT2+90
CVTRV449 EQU   CVTXTNT2+88
CVTRV448 EQU   CVTXTNT2+87
CVTRV447 EQU   CVTXTNT2+86
CVTRV446 EQU   CVTRV438
CVTRV445 EQU   CVTRV438
CVTRV444 EQU   CVTRV438
CVTRV443 EQU   CVTRV438
CVTRV442 EQU   CVTRV438
CVTRV441 EQU   CVTRV438
CVTRV440 EQU   CVTRV438
CVTRV439 EQU   CVTRV438
CVTRV437 EQU   CVTRV429
CVTRV436 EQU   CVTRV429
CVTRV435 EQU   CVTRV429
CVTRV434 EQU   CVTRV429
CVTRV433 EQU   CVTRV429
CVTRV432 EQU   CVTRV429
CVTRV431 EQU   CVTRV429
CVTRV430 EQU   CVTRV429
CVTRV428 EQU   CVTXTNT2+80
CVTRV427 EQU   CVTXTNT2+76
CVTRV426 EQU   CVTXTNT2+72
CVTRV425 EQU   CVTXTNT2+68
CVTATACT EQU   CVTATCVT
CVTRV423 EQU   CVTXTNT2+62
CVTRV422 EQU   CVTXTNT2+60
CVTRV421 EQU   CVTXTNT2+58
CVTRV420 EQU   CVTXTNT2+56
CVTRV419 EQU   CVTXTNT2+55
CVTRV418 EQU   CVTXTNT2+54
CVTRV417 EQU   CVTRV409
CVTRV416 EQU   CVTRV409
CVTRV415 EQU   CVTRV409
CVTRV414 EQU   CVTRV409
CVTRV413 EQU   CVTRV409
CVTRV412 EQU   CVTRV409
CVTRV411 EQU   CVTRV409
CVTRV410 EQU   CVTRV409
CVTRV408 EQU   CVTRV400
CVTRV407 EQU   CVTRV400
CVTRV406 EQU   CVTRV400
CVTRV405 EQU   CVTRV400
CVTRV404 EQU   CVTRV400
CVTRV403 EQU   CVTRV400
CVTRV402 EQU   CVTRV400
CVTRV401 EQU   CVTRV400
CVTICB   EQU   CVTXTNT2+48
CVTSKTA  EQU   CVTXTNT2+44
CVTRSV99 EQU   CVTXTNT2+40
CVTRSV98 EQU   CVTXTNT2+36
CVTRSV97 EQU   CVTXTNT2+34
CVTRSV96 EQU   CVTXTNT2+32
CVTOLTEP EQU   CVTXTNT2+28
CVTQIDA  EQU   CVTQID+1
CVTRSV95 EQU   CVTQID
CVTRSV94 EQU   CVTXTNT2+20
CVTRSV93 EQU   CVTXTNT2+16
CVTRSV92 EQU   CVTXTNT2+12
CVTDEBVR EQU   CVTXTNT2+8
CVTRSV91 EQU   CVTXTNT2+6
CVTRSV9H EQU   CVTFLGBT
CVTRSV9G EQU   CVTFLGBT
CVTRSV9F EQU   CVTFLGBT
CVTRSV9E EQU   CVTFLGBT
CVTRSV9D EQU   CVTFLGBT
CVTRSV9C EQU   CVTFLGBT
CVTVME   EQU   CVTFLGBT
CVTNPE   EQU   CVTFLGBT
CVTNUCLS EQU   CVTXTNT2+4
CVTDSSVA EQU   CVTDSSV+1
CVTRSV89 EQU   CVTDSSV
CVTRSV88 EQU   CVTXTNT1+8
CVTRSV87 EQU   CVTXTNT1+4
CVTFACHN EQU   CVTXTNT1
CVTRV488 EQU   CVTMAP+412
CVTRV487 EQU   CVTMAP+408
CVTRV486 EQU   CVTMAP+404
CVTRV485 EQU   CVTMAP+400
CVTACTAP EQU   CVTMAP+396
CVTAUTH  EQU   CVTMAP+392
CVTATMCA EQU   CVTATMCT+1
CVTATMST EQU   CVTATMCT
CVTRSV61 EQU   CVTMAP+384
CVTVOLT1 EQU   CVTVOLM1+1
CVTVOLI1 EQU   CVTVOLF1
CVTSTOA  EQU   CVTMAP+376
CVTRSV58 EQU   CVTMAP+374
CVTRSV57 EQU   CVTMAP+372
CVTDDCE  EQU   CVTMAP+368
CVTPNWFR EQU   CVTMAP+364
CVTSMF   EQU   CVTMAP+360
CVTSULK  EQU   CVTMAP+358
CVTSLKO  EQU   CVTSYSK
CVTSLKP  EQU   CVTSYSK
CVTSLKQ  EQU   CVTSYSK
CVTSLKR  EQU   CVTSYSK
CVTRSV56 EQU   CVTSYSK
CVTRSV55 EQU   CVTSYSK
CVTRSV54 EQU   CVTSYSK
CVTRSV53 EQU   CVTSYSK
CVTRSV52 EQU   CVTA1F1
CVTRSV51 EQU   CVTA1F1
CVTRSV50 EQU   CVTA1F1
CVTRSV49 EQU   CVTA1F1
CVTRSV48 EQU   CVTA1F1
CVTRSV47 EQU   CVTA1F1
CVTSRSW  EQU   CVTA1F1
CVTPFSW  EQU   CVTA1F1
CVTPCVT  EQU   CVTMAP+352
CVTRSV46 EQU   CVTMAP+344
CVTRSV45 EQU   CVTMAP+340
CVTRSV44 EQU   CVTMAP+338
CVTRSV43 EQU   CVTMAP+336
CVTHJESA EQU   CVTHJES+1
CVTRSV42 EQU   CVTHJES
CVTEXT2A EQU   CVTEXT2+1
CVTRSV41 EQU   CVTEXT2
CVTAPFA  EQU   CVTAPF+1
CVTRSV40 EQU   CVTAPF
CVTRV518 EQU   CVTINTLA
CVTRV517 EQU   CVTERPV
CVTEORM  EQU   CVTMAP+312
CVTMCHPR EQU   CVTMAP+308
CVTTZ    EQU   CVTMAP+304
CVTJEPS  EQU   CVTMAP+300
CVTJESCT EQU   CVTMAP+296
CVTMODE  EQU   CVTMAP+292
CVTPTRV  EQU   CVTMAP+288
CVTREAL  EQU   CVTMAP+284
CVTRSV39 EQU   CVTMAP+280
CVTRSV38 EQU   CVTMAP+276
CVTDMSRA EQU   CVTDMSR+1
CVTRSV37 EQU   CVTDMSR
CVTQMSGA EQU   CVTQMSG+1
CVTRSV36 EQU   CVTQMSG
CVTPURGA EQU   CVTPURG+1
CVTRSV35 EQU   CVTPURG
CVTATERA EQU   CVTATER+1
CVTSYST  EQU   CVTATER
CVTVOLT2 EQU   CVTTAT
CVTVOLI2 EQU   CVTVOLF2
CVTAQAVB EQU   CVTAQAVT+1
CVTRSV34 EQU   CVTTCMFG
CVTRSV33 EQU   CVTTCMFG
CVTRSV32 EQU   CVTTCMFG
CVTRSV31 EQU   CVTTCMFG
CVTRSV30 EQU   CVTTCMFG
CVTRSV29 EQU   CVTTCMFG
CVTRSV28 EQU   CVTTCMFG
CVTTCRDY EQU   CVTTCMFG
CVTGTFA  EQU   CVTGTF+1
CVTRSV27 EQU   CVTGTFST
CVTRNIO  EQU   CVTGTFST
CVTUSR   EQU   CVTGTFST
CVTRV318 EQU   CVTFORM
CVTRV317 EQU   CVTTMODE
CVTRV316 EQU   CVTSTATE
CVTRV315 EQU   CVTGTFS
CVTGTFAV EQU   CVTGTFS
CVT0SCR1 EQU   CVTMAP+232
CVTRV515 EQU   CVTMAP+228
CVTRMS   EQU   CVTMAP+224
CVTPATCH EQU   CVTMAP+220
CVTTSCE  EQU   CVTMAP+216
CVTLNKSC EQU   CVTMAP+214
CVTQABST EQU   CVTMAP+212
CVTMDLDS EQU   CVTMAP+208
CVTUSER  EQU   CVTMAP+204
CVTABEND EQU   CVTMAP+200
CVTSMCA  EQU   CVTMAP+196
CVTRSV18 EQU   CVTMAP+192
CVTQLPAQ EQU   CVTMAP+188
CVTQCDSR EQU   CVTMAP+184
CVTRSV17 EQU   CVTOPTB
CVTRSV16 EQU   CVTOPTB
CVTFP    EQU   CVTOPTB
CVTAPTHR EQU   CVTOPTB
CVTNLOG  EQU   CVTOPTB
CVTTOD   EQU   CVTOPTB
CVTCTIMS EQU   CVTOPTB
CVTPROT  EQU   CVTOPTB
CVTXPFP  EQU   CVTOPTA
CVTASCII EQU   CVTOPTA
CVTRSV13 EQU   CVTOPTA
CVTRSV12 EQU   CVTOPTA
CVTNIP   EQU   CVTOPTA
CVTDDR   EQU   CVTOPTA
CVTAPR   EQU   CVTOPTA
CVTCCH   EQU   CVTOPTA
CVTSNCTR EQU   CVTMAP+180
CVTQMWR  EQU   CVTMAP+176
CVTQOCR  EQU   CVTMAP+172
CVT1EF00 EQU   CVTMAP+168
CVTMZ00  EQU   CVTMAP+164
CVTHEAD  EQU   CVTMAP+160
CVTRSV11 EQU   CVTMAP+156
CVT0PT01 EQU   CVTMAP+152
CVTMSER  EQU   CVTMAP+148
CVTRV516 EQU   CVTIERLC
CVTILCH  EQU   CVTMAP+140
CVT0DS   EQU   CVTMAP+136
CVTFBOSV EQU   CVTMAP+132
CVTNUCB  EQU   CVTMAP+128
CVTIXAVL EQU   CVTMAP+124
CVTIOQET EQU   CVTMAP+120
CVTDCBA  EQU   CVTMAP+117
CVTMVS2  EQU   CVTDCB
CVT6DAT  EQU   CVTDCB
CVT4MPS  EQU   CVTDCB
CVTRSV09 EQU   CVTDCB
CVT4MS1  EQU   CVTDCB
CVT2SPS  EQU   CVTDCB
CVT1SSS  EQU   CVTDCB
CVTRSV08 EQU   CVTDCB
CVTSTB   EQU   CVTMAP+112
CVTQTD00 EQU   CVTMAP+108
CVTQTE00 EQU   CVTMAP+104
CVTCUCB  EQU   CVTMAP+100
CVTSJQ   EQU   CVTMAP+96
CVTPBLDL EQU   CVTMAP+92
CVTTPC   EQU   CVTMAP+88
CVTSVDCB EQU   CVTMAP+84
CVTBRET  EQU   CVTMAP+82
CVTEXIT  EQU   CVTMAP+80
CVT0FN00 EQU   CVTMAP+76
CVTDARA  EQU   CVTDAR+1
CVTRSV07 EQU   CVTFLGS1
CVTRSV06 EQU   CVTFLGS1
CVTRSV05 EQU   CVTFLGS1
CVTRSV04 EQU   CVTFLGS1
CVTRSV03 EQU   CVTFLGS1
CVTRSV02 EQU   CVTFLGS1
CVTRSV01 EQU   CVTFLGS1
CVTDMPLK EQU   CVTFLGS1
CVTXITP  EQU   CVTMAP+68
CVTZDTAB EQU   CVTMAP+64
CVTMSLT  EQU   CVTMAP+60
CVTDATE  EQU   CVTMAP+56
CVTBTERM EQU   CVTMAP+52
CVTSYSAD EQU   CVTMAP+48
CVTXTLER EQU   CVTMAP+44
CVTILK2  EQU   CVTMAP+40
CVTILK1  EQU   CVTMAP+36
CVTPRLTV EQU   CVTMAP+32
CVTPCNVT EQU   CVTMAP+28
CVT0VL00 EQU   CVTMAP+24
CVTXAPG  EQU   CVTMAP+20
CVTBUF   EQU   CVTMAP+16
CVTJOB   EQU   CVTMAP+12
CVTLINK  EQU   CVTMAP+8
CVT0EF00 EQU   CVTMAP+4
CVT      EQU   CVTMAP
LENFIX   EQU   CNTRLINF+1
CNTVAR   EQU   CNTRLINF
VARINFO  EQU   VARFIELD+2
VARLEN   EQU   VARFIELD
REPGOPT  EQU   REPCNTRL+6
REPCNT   EQU   REPCNTRL+5
@NM00043 EQU   EXTRCDPT+3
NREPCV   EQU   CATRCD+47
@NM00042 EQU   CATRCD+26
RCDDICTS EQU   CATRCD+22
RCDRACTS EQU   CATRCD+18
RCDCRACI EQU   CATRCD+11
RCDRELID EQU   CATRCD+4
RCDKEYTY EQU   CATRCD
CATGOGC  EQU   CATGOGC1
CATGODEL EQU   CATGOGC1
CATGOEXT EQU   CATGOGC1
CATGODSP EQU   CATGOCI+1
@NM00041 EQU   CATGOFLG
@NM00040 EQU   CCARAB+2
CCARABRI EQU   CCARAB+1
CCARBUPD EQU   CCARABFL
@NM00039 EQU   CCARABFL
CCARBPA  EQU   CCARABFL
CCARBWR  EQU   CCARABFL
CCARBU2  EQU   CCARABFL
CCARBU1  EQU   CCARABFL
DIC2GC   EQU   DIC2ENT+3
DIC2LC   EQU   DIC2ENT+2
DIC2ST   EQU   DIC2ENT+1
@NM00038 EQU   DIC2FLGS
DIC2UPGD EQU   DIC2FLGS
DIC2SCRA EQU   DIC2FLGS
DIC2MV   EQU   DIC2FLGS
DIC2FLG  EQU   DIC2FLGS
DIC2COMB EQU   DIC2FLGS
DIC2REP  EQU   DIC2FLGS
DIC2VAR  EQU   DIC2FLGS
DICTGC   EQU   DICTENT+3
DICTLC   EQU   DICTENT+2
DICTST   EQU   DICTENT+1
@NM00037 EQU   DICTFLGS
DICTUPGD EQU   DICTFLGS
DICTSCRA EQU   DICTFLGS
DICTMV   EQU   DICTFLGS
DICTFLG  EQU   DICTFLGS
DICTCOMB EQU   DICTFLGS
DICTREP  EQU   DICTFLGS
DICTVAR  EQU   DICTFLGS
ENTNAMEV EQU   ENTNAMES+1
ENTNAMET EQU   ENTNAMES
@NM00036 EQU   CTGFLDRE
CTGPSWD  EQU   CTGJSCB
@NM00034 EQU   CTGFBFLG
CTGNGDSN EQU   CTGFBFLG
CTGGDGB  EQU   CTGFBFLG
CTGKEEP  EQU   CTGFBFLG
CTGPAR   EQU   CTGFBFLG
CTGFDBK  EQU   CTGNEWNM
CTGTYPE  EQU   CTGPL+18
@NM00033 EQU   CTGDSORG
CTGOPTNS EQU   CTGDSORG
@NM00032 EQU   CTGOPTN4
CTGDOCAT EQU   CTGOPTN4
CTGLBASE EQU   CTGOPTN4
CTGAM0   EQU   CTGOPTN3
CTGNUM   EQU   CTGOPTN3
CTGSRH   EQU   CTGOPTN3
CTGGDGL  EQU   CTGOPTN3
CTGSUPLT EQU   CTGOPTN3
CTGFUNC  EQU   CTGOPTN3
@NM00031 EQU   CTGOPTN2
CTGSCR   EQU   CTGOPTN2
CTGOVRID EQU   CTGOPTN2
CTGDISC  EQU   CTGOPTN2
CTGGTNXT EQU   CTGOPTN2
CTGRCATN EQU   CTGVMNT
CTGGTALL EQU   CTGREL
CTGNSVS  EQU   CTGEXT
CTGGENLD EQU   CTGOPTN1
CTGCNAME EQU   CTGOPTN1
CTGNAME  EQU   CTGOPTN1
CTGREAD  EQU   CTGOPTN1
CTGUPD   EQU   CTGOPTN1
CTGCI    EQU   CTGOPTN1
CTGMAST  EQU   CTGOPTN1
CTGBYPSS EQU   CTGOPTN1
CCAEND   EQU   IGGCCA+1492
CCADGDGA EQU   IGGCCA+1488
CCARETRX EQU   CCAERCDX+1
CCARESNX EQU   CCAERCDX
CCAMODDX EQU   CCAPROBX
CCASMFCD EQU   CCASMFP+3
CCASMFRC EQU   CCASMFP+2
CCASMFMD EQU   CCASMFP
CCAREWKA EQU   IGGCCA+1476
CCAVTS   EQU   IGGCCA+1468
CCARAVCA EQU   IGGCCA+1460
CCARANCA EQU   IGGCCA+1452
CCADICTS EQU   IGGCCA+1448
CCASUMTT EQU   IGGCCA+1445
@NM00030 EQU   CCAFLG10
CCAPRANX EQU   CCAFLG10
CCARARTC EQU   CCAFLG10
CCARAFEV EQU   CCAFLG10
CCACATAC EQU   CCAFLG10
CCAPDMW  EQU   CCAFLG10
CCAINCPL EQU   CCAFLG10
@NM00029 EQU   CCARALSA+2
CCACRABT EQU   CCARALSA
CCARAREC EQU   IGGCCA+1436
CCARARBA EQU   IGGCCA+1432
CCARARPL EQU   IGGCCA+1428
CCARAACB EQU   IGGCCA+1424
CCACRACI EQU   IGGCCA+1420
CCASBASE EQU   IGGCCA+1417
CCATNQCT EQU   IGGCCA+1416
CCADSPWA EQU   CCABZSAV
CCAMODNM EQU   CCAREGS+4
@NM00028 EQU   CCAREGS
@NM00027 EQU   IGGCCA+1057
@NM00026 EQU   CCARVFG1
RVESBOR  EQU   CCARVFG1
RVESBO   EQU   CCARVFG1
RVCMSFG  EQU   CCARVFG1
RVARFI   EQU   CCARVFG1
RVCCAV   EQU   CCARVFG1
ARFSBECH EQU   CCAARFWA+12
ARFSBSCH EQU   CCAARFWA+8
ARFGMADR EQU   CCAARFWA+4
ARFLEN   EQU   ARFGMLEN+1
ARFGMLP  EQU   ARFGMLEN
CCAMNADR EQU   CCAMNCAT+252
CCAMNLEN EQU   CCAMNCAT+249
CCAMNLL  EQU   CCAMNCAT+248
MNATARG2 EQU   CCAMNAT+244
MNATARG1 EQU   CCAMNAT+241
MNATSCLS EQU   MNATFLGS
@NM00025 EQU   MNATFLGS
MNATVAL  EQU   MNATFLGS
@NM00024 EQU   CCAMNAT+1
@NM00023 EQU   MNATTOP
MNATFULL EQU   MNATTOP
CCABMRB2 EQU   IGGCCA+740
CCABMRB1 EQU   IGGCCA+736
CCABMWK4 EQU   IGGCCA+732
CCABMWK3 EQU   IGGCCA+728
CCABMWK2 EQU   IGGCCA+724
CCABMWK1 EQU   IGGCCA+720
CCABMSTR EQU   IGGCCA+718
CCABMBYT EQU   IGGCCA+716
CCABMBTL EQU   IGGCCA+714
CCABMBT1 EQU   IGGCCA+712
CCABMEND EQU   IGGCCA+708
CCABMPTR EQU   IGGCCA+704
CCABMGOP EQU   IGGCCA+700
CCABMPAD EQU   IGGCCA+699
@NM00022 EQU   IGGCCA+697
CCABMOST EQU   CCABMOFG
CCABMOTR EQU   CCABMOUT+2
CCABMONN EQU   CCABMOUT
@NM00021 EQU   CCABMINP+7
CCABMLST EQU   CCABMFLG
CCABMCCK EQU   CCABMFLG
CCABMSET EQU   CCABMFLG
CCABMCHK EQU   CCABMFLG
CCABMST  EQU   CCABMFLG
CCABMMIN EQU   CCABMINP+4
CCABMLIM EQU   CCABMINP+2
CCABMTRK EQU   CCABMINP
CCAR14S  EQU   IGGCCA+680
CCAXLATE EQU   IGGCCA+640
CCADEL2  EQU   IGGCCA+637
CCADEL1  EQU   IGGCCA+634
CCAVAR1  EQU   IGGCCA+614
CCAVAR   EQU   IGGCCA+594
CCADELN  EQU   IGGCCA+592
CCAVARPT EQU   IGGCCA+588
CCARBASE EQU   IGGCCA+584
CCARRAB  EQU   IGGCCA+580
CCAVARLN EQU   IGGCCA+578
CCACI3   EQU   IGGCCA+575
CCACI2   EQU   IGGCCA+572
CCACI1   EQU   IGGCCA+569
CCACHAIN EQU   IGGCCA+566
CCASMFLN EQU   IGGCCA+564
CCASMFG2 EQU   CCASMFLG+1
@NM00020 EQU   CCASMFG1
CCASMFAL EQU   CCASMFSR
CCASMFDF EQU   CCASMFUC
CCASMFCT EQU   IGGCCA+560
CCASMFRD EQU   IGGCCA+556
CCANORBA EQU   IGGCCA+555
CCARESUM EQU   CCAFLG9
CCARAICI EQU   CCAFLG9
CCARAACT EQU   CCAFLG9
CCAUCRA  EQU   CCAFLG9
CCARACR  EQU   CCAFLG9
CCARALRD EQU   CCAFLG9
CCARAEOV EQU   CCAFLG9
CCARABYC EQU   CCAFLG9
CCAAUCAT EQU   CCAFLG8
CCALEOD  EQU   CCAFLG8
CCAVRIND EQU   CCAFLG8
CCALPIND EQU   CCAFLG8
CCASCICK EQU   CCAFLG8
CCASCRA  EQU   CCAFLG8
CCAVBUFI EQU   CCAFLG8
CCADSRCL EQU   CCAFLG8
CCACRABU EQU   CCAFLG7
CCAFEOV  EQU   CCAFLG7
CCAROREQ EQU   CCAFLG7
CCAONCE  EQU   CCAFLG7
CCASMFBR EQU   CCAFLG7
CCASMFA  EQU   CCAFLG7
CCASMFEX EQU   CCAFLG7
CCALSP   EQU   CCAFLG7
CCALCPL  EQU   IGGCCA+548
CCASMFPT EQU   IGGCCA+544
CCAALPTR EQU   IGGCCA+540
CCAILNG2 EQU   IGGCCA+536
CCAILNG3 EQU   IGGCCA+532
CCASDWAP EQU   IGGCCA+528
CCACWAP  EQU   IGGCCA+524
CCACDTCT EQU   IGGCCA+522
CCADTCT  EQU   IGGCCA+520
CCACDTA  EQU   IGGCCA+516
CCADTA   EQU   IGGCCA+512
CCASVCI1 EQU   IGGCCA+509
CCASVCI  EQU   IGGCCA+506
CCADISP  EQU   IGGCCA+504
CCAREPCT EQU   IGGCCA+502
CCADIFF  EQU   IGGCCA+500
CCADLEN  EQU   IGGCCA+498
CCAIXTPL EQU   IGGCCA+496
CCAGRHI1 EQU   CCASSEQ
CCARCDID EQU   CCAGRGC
@NM00019 EQU   IGGCCA+492
CCANEFV  EQU   IGGCCA+491
CCAGVEXT EQU   IGGCCA+488
CCANEVV  EQU   IGGCCA+486
CCAGVCT  EQU   IGGCCA+484
CCARABSM EQU   CCAIDPT3
CCAIDPT2 EQU   IGGCCA+476
CCASMFIX EQU   IGGCCA+474
CCACNIX  EQU   IGGCCA+472
CCARBAL  EQU   IGGCCA+470
CCATLEN  EQU   CCATLNG
CCARBA2  EQU   CCARBA+16
CCARBA1  EQU   CCARBA+12
CCATT    EQU   CCARBA+10
CCACCHH2 EQU   CCACRADT
CCACCHH1 EQU   CCACRAVL+2
CCASS    EQU   CCACRAVL
CCATEST  EQU   IGGCCA+444
CCATNO   EQU   IGGCCA+442
CCATREL  EQU   IGGCCA+410
CCATNUM  EQU   IGGCCA+408
CCATNREL EQU   IGGCCA+406
CCAIXREL EQU   IGGCCA+404
CCAIXFPL EQU   IGGCCA+402
@NM00018 EQU   IGGCCA+400
CCASN1   EQU   CCASN
CCALREL1 EQU   CCALREL
CCAGC    EQU   IGGCCA+395
CCACBASE EQU   IGGCCA+392
CCARABPT EQU   IGGCCA+368
CCATFLPT EQU   CCAFLPT
CCAILNG  EQU   IGGCCA+360
CCASL    EQU   IGGCCA+358
CCAGOPLN EQU   IGGCCA+356
CCACPE71 EQU   IGGCCA+352
CCAIDPT  EQU   CCACPE7
CCARABSE EQU   CCACPE61
CCACPE53 EQU   IGGCCA+336
CCACPE52 EQU   IGGCCA+332
CCACPE51 EQU   IGGCCA+328
CCALUME  EQU   IGGCCA+324
CCAEXCMS EQU   CCACMS+4
CCARPRM  EQU   IGGCCA+312
@NM00017 EQU   CCAMSSPL+10
CCAMNSPL EQU   CCAMSSPL+9
@NM00016 EQU   CCAMSSPL+8
CCAMNPTR EQU   CCAMSSPL+4
CCAMNLLP EQU   CCAMSSPL
CCAMLRET EQU   IGGCCA+296
CCAEDUCB EQU   CCAEQDQ+12
CCAEDRNM EQU   CCAEQDQ+8
CCAEDQNM EQU   CCAEQDQ+4
CCAEDRCD EQU   CCAEQDQ+3
@NM00015 EQU   CCAEDOPT
CCAEDSHR EQU   CCAEDOPT
CCAEDRLN EQU   CCAEQDQ+1
CCAEDXFF EQU   CCAEQDQ
CCAASCIX EQU   IGGCCA+270
CCACRRP  EQU   IGGCCA+269
CCAASCIK EQU   IGGCCA+268
CCAEXTTH EQU   CCAEXTDE+6
CCAEXTHH EQU   CCAEXTAD+2
CCAEXTCC EQU   CCAEXTAD
CCAEXTSS EQU   CCAEXTDE
CCASSVOL EQU   CCADESA+2
CCAIXEXT EQU   CCADESA+1
CCANDEXT EQU   CCADESA
CCARPL1  EQU   IGGCCA+220
CCAQLEN  EQU   IGGCCA+219
CCASC    EQU   IGGCCA+218
CCATQDB  EQU   CCATIQRN+2
CCATASID EQU   CCATIQRN
CCARPLT  EQU   IGGCCA+211
CCARPLX  EQU   IGGCCA+210
CCARPLF  EQU   IGGCCA+209
CCARPLK  EQU   IGGCCA+208
CCACPE45 EQU   CCAR5SEG+8
CCACPE35 EQU   CCAR5SEG+4
CCACPE25 EQU   CCAR5SEG
CCAR5REC EQU   CCARAB5+4
@NM00014 EQU   CCARAB5+2
CCAR5RPL EQU   CCARAB5+1
CCAR5UPD EQU   CCAR5FLG
@NM00013 EQU   CCAR5FLG
CCAR5PA  EQU   CCAR5FLG
CCAR5WR  EQU   CCAR5FLG
CCAR5U2  EQU   CCAR5FLG
CCAR5U1  EQU   CCAR5FLG
CCAR5UR  EQU   CCAR5FLG
CCACPE44 EQU   CCAR4SEG+8
CCACPE34 EQU   CCAR4SEG+4
CCACPE24 EQU   CCAR4SEG
CCAR4REC EQU   CCARAB4+4
@NM00012 EQU   CCARAB4+2
CCAR4RPL EQU   CCARAB4+1
CCAR4UPD EQU   CCAR4FLG
@NM00011 EQU   CCAR4FLG
CCAR4PA  EQU   CCAR4FLG
CCAR4WR  EQU   CCAR4FLG
CCAR4U2  EQU   CCAR4FLG
CCAR4U1  EQU   CCAR4FLG
CCAR4UR  EQU   CCAR4FLG
CCACPE43 EQU   CCAR3SEG+8
CCACPE33 EQU   CCAR3SEG+4
CCACPE23 EQU   CCAR3SEG
CCAR3REC EQU   CCARAB3+4
@NM00010 EQU   CCARAB3+2
CCAR3RPL EQU   CCARAB3+1
CCAR3UPD EQU   CCAR3FLG
@NM00009 EQU   CCAR3FLG
CCAR3PA  EQU   CCAR3FLG
CCAR3WR  EQU   CCAR3FLG
CCAR3U2  EQU   CCAR3FLG
CCAR3U1  EQU   CCAR3FLG
CCAR3UR  EQU   CCAR3FLG
CCACPE42 EQU   CCAR2SEG+8
CCACPE32 EQU   CCAR2SEG+4
CCACPE22 EQU   CCAR2SEG
CCAR2REC EQU   CCARAB2+4
@NM00008 EQU   CCARAB2+2
CCAR2RPL EQU   CCARAB2+1
CCAR2UPD EQU   CCAR2FLG
@NM00007 EQU   CCAR2FLG
CCAR2PA  EQU   CCAR2FLG
CCAR2WR  EQU   CCAR2FLG
CCAR2U2  EQU   CCAR2FLG
CCAR2U1  EQU   CCAR2FLG
CCAR2UR  EQU   CCAR2FLG
CCACPE41 EQU   CCAR1SEG+8
CCACPE31 EQU   CCAR1SEG+4
CCACPE21 EQU   CCAR1SEG
CCAR1REC EQU   CCARAB1+4
@NM00006 EQU   CCARAB1+2
CCAR1RPL EQU   CCARAB1+1
CCAR1UPD EQU   CCAR1FLG
@NM00005 EQU   CCAR1FLG
CCAR1PA  EQU   CCAR1FLG
CCAR1WR  EQU   CCAR1FLG
CCAR1U2  EQU   CCAR1FLG
CCAR1U1  EQU   CCAR1FLG
CCAR1UR  EQU   CCAR1FLG
CCACPE40 EQU   CCAR0SEG+8
CCACPE30 EQU   CCAR0SEG+4
CCACPE20 EQU   CCAR0SEG
CCAR0REC EQU   CCARAB0+4
@NM00004 EQU   CCARAB0+2
CCAR0RPL EQU   CCARAB0+1
CCAR0UPD EQU   CCAR0FLG
@NM00003 EQU   CCAR0FLG
CCAR0PA  EQU   CCAR0FLG
CCAR0WR  EQU   CCAR0FLG
CCAR0U2  EQU   CCAR0FLG
CCAR0U1  EQU   CCAR0FLG
CCAR0UR  EQU   CCAR0FLG
@NM00002 EQU   CCASRCH+3
CCASRCIN EQU   CCASRID
CCANPCCB EQU   IGGCCA+36
CCADPL   EQU   CCARB
CCALBCYL EQU   CCATCB
CCAMOD   EQU   CCAFLG6
CCAEXT   EQU   CCAFLG6
CCASUPFD EQU   CCAFLG6
CCAINIT  EQU   CCAFLG6
CCAMNOSP EQU   CCAFLG6
CCADELP  EQU   CCAFLG6
CCAMCODR EQU   CCAFLG6
CCATEX2  EQU   CCAFLG5
CCATEX1  EQU   CCAFLG5
CCATEX   EQU   CCAFLG5
CCATHIT  EQU   CCAFLG5
CCAMEX1  EQU   CCAFLG5
CCAMEX   EQU   CCAFLG5
CCAMEX2  EQU   CCAFLG5
CCATCOMP EQU   CCAFLG4
CCAGVNE  EQU   CCAFLG4
CCAGVEX  EQU   CCAFLG4
CCAGVNBS EQU   CCAFLG4
CCAGVNF  EQU   CCAFLG4
CCAGVNC  EQU   CCAFLG4
CCAF4BYS EQU   CCAFLG4
CCAF4DRQ EQU   CCAFLG4
CCAEGREC EQU   CCAFLG3
CCALFT   EQU   CCAFLG3
CCAELC2  EQU   CCAFLG3
CCANF    EQU   CCAFLG3
CCAEXGR2 EQU   CCAFLG3
CCAGDSP  EQU   CCAFLG3
CCAGC4   EQU   CCAFLG3
CCAEXGR1 EQU   CCAFLG3
CCAF2SMO EQU   CCAFLG2
CCAF2CB  EQU   CCAF2COB
CCAF2CO  EQU   CCAF2COB
CCAF2RHS EQU   CCAFLG2
CCAF2XEQ EQU   CCAFLG2
CCAF2CCT EQU   CCAFLG2
CCAF2SYS EQU   CCAFLG2
CCAF1DK  EQU   CCAFLG1
CCAF1CR  EQU   CCAFLG1
CCAF1LRD EQU   CCAFLG1
CCAF1CNS EQU   CCAFLG1
CCAF1LPS EQU   CCAFLG1
@NM00001 EQU   IGGCCA+8
CCACDR   EQU   CCAREASN
CCAMODID EQU   CCAPROB
CCASZ    EQU   IGGCCA+2
CCAID    EQU   IGGCCA
ASSOCTYP EQU   WKASSOC
@NM00073 EQU   PSA+3668
PSASTAK  EQU   PSA+3072
@NM00072 EQU   PSA+1040
PSAUSEND EQU   PSA+1040
PSARV062 EQU   PSA+1036
PSAATCVT EQU   PSA+1032
PSAPCPSW EQU   PSA+1024
PSAPIR2  EQU   PSA+1020
PSARV059 EQU   PSA+1018
PSASVC13 EQU   PSA+1016
PSALSFCC EQU   PSA+1012
PSASFACC EQU   PSA+1008
PSASTOP  EQU   PSA+992
PSASTART EQU   PSA+976
PSARSPSW EQU   PSA+968
PSASRPSW EQU   PSA+960
PSARV045 EQU   PSA+892
PSARV044 EQU   PSA+888
PSARV043 EQU   PSA+884
PSARV042 EQU   PSA+880
PSARV041 EQU   PSA+876
PSARV040 EQU   PSA+872
PSARV025 EQU   PSA+868
PSADSSED EQU   PSA+868
PSADSSPR EQU   PSA+864
PSADSSFW EQU   PSA+860
PSADSS14 EQU   PSA+856
PSADSSPP EQU   PSA+848
PSADSSRP EQU   PSA+840
PSADSS05 EQU   PSADSSF4
PSADSS10 EQU   PSADSSF4
PSADSSVE EQU   PSADSSF4
PSADSSDE EQU   PSADSSF4
PSADSSC0 EQU   PSADSSF4
PSADSSIE EQU   PSADSSF4
PSADSS12 EQU   PSADSSF4
PSADSSRC EQU   PSADSSF4
PSARV057 EQU   PSADSSF3
PSARV056 EQU   PSADSSF3
PSARV055 EQU   PSADSSF3
PSADSSMC EQU   PSADSSF3
PSADSSRW EQU   PSADSSF3
PSADSSNM EQU   PSADSSF3
PSADSSES EQU   PSADSSF3
PSADSSGP EQU   PSADSSF3
PSADSSF2 EQU   PSADSSFL+1
PSADSSPI EQU   PSADSSF1
PSADSSOI EQU   PSADSSF1
PSADSSSP EQU   PSADSSF1
PSADSSTP EQU   PSADSSF1
PSADSSDW EQU   PSADSSF1
PSADSSDD EQU   PSADSSF1
PSADSSDM EQU   PSADSSF1
PSADSSMV EQU   PSADSSF1
PSADSSTS EQU   PSA+816
PSADSSWK EQU   PSA+812
PSADSSR3 EQU   PSA+808
PSADSSR2 EQU   PSA+804
PSADSSRS EQU   PSA+800
PSASTOR  EQU   PSA+796
PSACPUSA EQU   PSA+794
PSAVSTAP EQU   PSA+792
PSAWKVAP EQU   PSA+788
PSAWKRAP EQU   PSA+784
PSAMCHIC EQU   PSA+783
PSARV061 EQU   PSA+782
PSASYMSK EQU   PSA+781
PSAMCHFL EQU   PSA+780
PSACR0   EQU   PSA+776
PSAPSWSV EQU   PSA+768
PSALITA  EQU   PSA+764
PSACLHS  EQU   PSAHLHI
PSALKR15 EQU   PSALKSA+60
PSALKR14 EQU   PSALKSA+56
PSALKR13 EQU   PSALKSA+52
PSALKR12 EQU   PSALKSA+48
PSALKR11 EQU   PSALKSA+44
PSALKR10 EQU   PSALKSA+40
PSALKR9  EQU   PSALKSA+36
PSALKR8  EQU   PSALKSA+32
PSALKR7  EQU   PSALKSA+28
PSALKR6  EQU   PSALKSA+24
PSALKR5  EQU   PSALKSA+20
PSALKR4  EQU   PSALKSA+16
PSALKR3  EQU   PSALKSA+12
PSALKR2  EQU   PSALKSA+8
PSALKR1  EQU   PSALKSA+4
PSALKR0  EQU   PSALKSA
PSARV023 EQU   PSACLHT+52
PSALOCAL EQU   PSACLHT+48
PSACMSL  EQU   PSACLHT+44
PSAOPTL  EQU   PSACLHT+40
PSATPACL EQU   PSACLHT+36
PSATPDNL EQU   PSACLHT+32
PSATPNCL EQU   PSACLHT+28
PSAIOSLL EQU   PSACLHT+24
PSAIOSUL EQU   PSACLHT+20
PSAIOSCL EQU   PSACLHT+16
PSAIOSSL EQU   PSACLHT+12
PSASALCL EQU   PSACLHT+8
PSAASML  EQU   PSACLHT+4
PSADISPL EQU   PSACLHT
PSASRSA  EQU   PSA+636
PSARV050 EQU   PSA+634
PSADSSGO EQU   PSA+633
PSARECUR EQU   PSA+632
PSAHLHIS EQU   PSA+628
PSAIPCSA EQU   PSA+624
@NM00071 EQU   PSA+621
PSAIPCDM EQU   PSA+620
PSAIPCD  EQU   PSA+616
@NM00070 EQU   PSA+613
PSAIPCRM EQU   PSA+612
PSAIPCR  EQU   PSA+608
PSAMCHEX EQU   PSA+600
PSAMPSW  EQU   PSA+592
PSAEXPS2 EQU   PSA+584
PSAEXPS1 EQU   PSA+576
PSAPIREG EQU   PSA+572
PSARSREG EQU   PSA+568
PSAGPREG EQU   PSA+556
PSARV022 EQU   PSASUP4
PSARV021 EQU   PSASUP4
PSARV020 EQU   PSASUP4
PSARV019 EQU   PSASUP4
PSARV018 EQU   PSASUP4
PSARV017 EQU   PSASUP4
PSARV016 EQU   PSASUP4
PSARV015 EQU   PSASUP4
PSARV014 EQU   PSASUP3
PSARV013 EQU   PSASUP3
PSARV012 EQU   PSASUP3
PSARV011 EQU   PSASUP3
PSARV010 EQU   PSASUP3
PSAPSREG EQU   PSASUP3
PSAPI2   EQU   PSASUP3
PSAIOSUP EQU   PSASUP3
PSALCR   EQU   PSASUP2
PSARTM   EQU   PSASUP2
PSAACR   EQU   PSASUP2
PSAIPCE2 EQU   PSASUP2
PSAIPCES EQU   PSASUP2
PSAIPCEC EQU   PSASUP2
PSAGTF   EQU   PSASUP2
PSAIPCRI EQU   PSASUP2
PSAIPCRP EQU   PSASUP1
PSAIPCDR EQU   PSASUP1
PSADISP  EQU   PSASUP1
PSALOCK  EQU   PSASUP1
PSAPI    EQU   PSASUP1
PSAEXT   EQU   PSASUP1
PSASVC   EQU   PSASUP1
PSAIO    EQU   PSASUP1
PSAAOLD  EQU   PSA+548
PSAANEW  EQU   PSA+544
PSATOLD  EQU   PSA+540
PSATNEW  EQU   PSA+536
PSALCCAR EQU   PSA+532
PSALCCAV EQU   PSA+528
PSAPCCAR EQU   PSA+524
PSAPCCAV EQU   PSA+520
PSACPULA EQU   PSA+518
PSACPUPA EQU   PSA+516
PSAPSA   EQU   PSA+512
FLCHDEND EQU   PSA+512
FLCCRSAV EQU   FLCMCLA+280
FLCGRSAV EQU   FLCMCLA+216
FLCFPSAV EQU   FLCMCLA+184
FLCFLA   EQU   FLCMCLA+88
FLCRGNCD EQU   FLCMCLA+84
FLCFSAA  EQU   FLCFSA+1
@NM00069 EQU   FLCFSA
@NM00068 EQU   FLCMCLA+72
FLCMCIC  EQU   FLCMCLA+64
@NM00067 EQU   FLCMCLA+20
FLCIOAA  EQU   FLCIOA+1
@NM00066 EQU   FLCIOA
@NM00065 EQU   FLCMCLA+15
@NM00064 EQU   FLCMCLA+14
@NM00063 EQU   FLCMCLA+12
FLCLCL   EQU   FLCMCLA+8
FLCIOELA EQU   FLCIOEL+1
@NM00062 EQU   FLCIOEL
FLCCHNID EQU   FLCMCLA
@NM00061 EQU   PSA+160
FLCMTRCD EQU   PSA+157
@NM00060 EQU   PSA+156
FLCPERA  EQU   FLCPER+1
@NM00059 EQU   FLCPER
@NM00058 EQU   PSA+151
FLCPERCD EQU   PSA+150
FLCMCNUM EQU   PSA+149
@NM00057 EQU   PSA+148
FLCTEAA  EQU   FLCTEA+1
@NM00056 EQU   FLCTEA
PSAPIPC  EQU   PSAPICOD
PSAPIMC  EQU   PSAPICOD
PSAPIPER EQU   PSAPICOD
PSARV049 EQU   FLCPICOD
FLCPILCB EQU   FLCPIILC
@NM00055 EQU   FLCPIILC
@NM00054 EQU   PSAEPPSW
FLCSVCN  EQU   PSAESPSW+2
FLCSILCB EQU   FLCSVILC
@NM00053 EQU   FLCSVILC
@NM00052 EQU   PSAESPSW
FLCEICOD EQU   PSAEEPSW+2
PSASPAD  EQU   PSAEEPSW
@NM00051 EQU   PSA+128
FLCINPSW EQU   PSA+120
FLCMNPSW EQU   PSA+112
FLCPNPSW EQU   PSA+104
FLCSNPSW EQU   PSA+96
FLCENPSW EQU   PSA+88
FLCTRACE EQU   PSA+84
FLCTIMER EQU   PSA+80
FLCCVT2  EQU   PSA+76
FLCCAW   EQU   PSA+72
FLCCSW   EQU   PSA+64
FLCIOPSW EQU   PSA+56
FLCMOPSW EQU   PSA+48
FLCPOPSW EQU   PSA+40
FLCSOPSW EQU   PSA+32
FLCEOPSW EQU   PSA+24
@NM00050 EQU   FLCICCW2+4
FLCCVT   EQU   FLCICCW2
FLCICCW1 EQU   FLCROPSW
FLCIPPSW EQU   FLCRNPSW
.@UNREFD ANOP                          END UNREFERENCED COMPONENTS
@RC00447 EQU   @RC00446
@RT00499 EQU   @RC00486
@PB00006 EQU   @EL00001
@ENDDATA EQU   *
         END   IGG0CLB1,(C'PLS0405',0602,75084)
