         TITLE 'IGG0CLB4 - VSAM DEFINE CATALOG RECOVERY AREA           *
                        '
IGG0CLB4 CSECT ,                                                   0001
@MAINENT DS    0H                                                  0001
         USING *,@15                                               0001
         B     @PROLOG                                             0001
         DC    AL1(16)                                             0001
         DC    C'IGG0CLB4  76.057'                                 0001
         DROP  @15
@PROLOG  DS    0H                                                  0002
         USING @MAINENT,@15
         USING @DATD-(CCATEMPS-IGGCCA),R11
         DS 0F
         DC CL8'IGGPDCRA'
         EJECT
         B     @PB00002                                            0201
IGGPDCRA DS    0H                                                  0202
         ENTRY IGGPDCRA
         DROP @15
Z003     DS    0H                                                  0204
Z003DCRA DS    0H                                                  0205
         STM 12,14,12(13)
         LA 13,12(13)
         BALR 12,0
IGGSEGST DS 0H
         USING *,12
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPDCRA                                        *
**                                                                    *
**   PROCEDURE TYPE = EXTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = DEFINE CATALOG RECOVERY AREA                  *
**                                                                    *
**   FUNCTION =                                                       *
**      DEFINE A CATALOG RECOVERY AREA (CRA) ON A VOLUME OWNED BY     *
**      A CATALOG HAVING THE RECOVERABLE ATTRIBUTE:                   *
**      1.  OBTAIN MAIN STORAGE WORK SPACE AND VOLUME SPACE.          *
**      2.  PROCESS THE VOLUME'S FORMAT-4 DSCB.                       *
**      3.  CONSTRUCT THE CRA'S SELF-DESCRIBING RECORDS.              *
**      4.  OPEN THE CRA ESDS.                                        *
**      5.  WRITE THE CRA SELF-DESCRIBING RECORDS AND THE VOLUME      *
**          RECORDS TO THE CRA AND THE CATALOG.                       *
**      6.  PERFORM CLEANUP PROCESSING AND RETURN TO CALLER.          *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      VOLUME UCB ADDRESS IN CMSWA(CMSDFSPC)                         *
**                                                                    *
**   OUTPUT =                                                         *
**      IF ERROR ENCOUNTERED, 'CCAPROB' CONTAINS PROBLEM              *
**         DETERMINATION INFORMATION.                                 *
**      FORMAT-4 DSCB CONTAINS TT POINTER TO CRA.                     *
**      CRA CONTAINS SELF-DESCRIBING RECORDS AND VOLUME RECORDS.      *
**      CATALOG CONTAINS VOLUME RECORDS.                              *
**                                                                    *
**********************************************************************/
*/********************************************************************/
*/*      OBTAIN  MAIN  STORAGE  WORK  SPACE  AND  VOLUME  SPACE      */
*/********************************************************************/
*                                                                  0211
*   CCARAACB=BLANK;                 /* CLEAR CRA ACB ADDR FOR FLAG   */
         SLR   @10,@10                                             0211
         ST    @10,CCARAACB(,R11)                                  0211
*   CCACRABU=ON;                    /* TURN ON CRA BUILD FLAG        */
         OI    CCACRABU(R11),B'00000001'                           0212
*Z004STRG:                                                         0213
*   CALL IGGPSTRG;                  /* OBTAIN ALL NECESSARY SPACE    */
Z004STRG BAL   @14,IGGPSTRG                                        0213
*   IF CCACD1^=RCS THEN             /* CHECK IF GET SPACE SUCCESSFUL */
         CLI   CCACD1(R11),0                                       0214
         BE    @RF00214                                            0214
*Z005DCXT:                                                         0215
*     CALL IGGPDCXT;                /* IF NOT, CLEAN UP AND EXIT     */
Z005DCXT BAL   @14,IGGPDCXT                                        0215
*   ELSE                                                           0216
*     ;                             /* ALL REQUIRED STORAGE ACQUIRED */
@RF00214 DS    0H                                                  0217
*/********************************************************************/
*/*             PROCESS  THE  VOLUME'S  FORMAT  4  DSCB              */
*/********************************************************************/
*                                                                  0217
*Z006FMT4:                                                         0217
*   CALL IGGPFMT4;                  /* UPDATE THE FORMAT 4 DSCB      */
Z006FMT4 BAL   @14,IGGPFMT4                                        0217
*   IF CCACD1^=RCS THEN             /* CHECK IF F4DSCB UPDATE OKAY   */
         CLI   CCACD1(R11),0                                       0218
         BE    @RF00218                                            0218
*Z007DCXT:                                                         0219
*     CALL IGGPDCXT;                /* IF NOT, CLEAN UP AND EXIT     */
Z007DCXT BAL   @14,IGGPDCXT                                        0219
*   ELSE                                                           0220
*     ;                             /* FORMAT-4 DSCB UPDATE OKAY     */
@RF00218 DS    0H                                                  0221
*/********************************************************************/
*/*          CONSTRUCT  THE  CRA'S  SELF-DESCRIBING  RECORDS         */
*/********************************************************************/
*                                                                  0221
*   DCRACIPT(1)=ADDR(DCRARACI);     /* SET POINTER TO CI 0           */
         LA    @10,DCRARACI(,DCRAWAPR)                             0221
         ST    @10,DCRACIPT(,DCRAWAPR)                             0221
*   DO DCRARCTR=2 TO 6;             /* SET ADDR ARRAY SUBSCRIPT RANGE*/
         LA    DCRARCTR,2                                          0222
@DL00222 DS    0H                                                  0223
*     DCRACIPT(DCRARCTR)=           /* SET POINTER TO RECORD         */
*         DCRACIPT(DCRARCTR-INCR1)+DCRACISZ;                       0223
         LR    @10,DCRARCTR                                        0223
         SLA   @10,2                                               0223
         LA    @05,512                                             0223
         LR    @01,DCRAWAPR                                        0223
         AL    @01,@CF04659                                        0223
         AL    @05,DCRACIPT(@10,@01)                               0223
         AL    @10,@CF04660                                        0223
         ST    @05,DCRACIPT(@10,DCRAWAPR)                          0223
*   END;                                                           0224
         AH    DCRARCTR,@CH00046                                   0224
         CH    DCRARCTR,@CH00087                                   0224
         BNH   @DL00222                                            0224
*Z008CHIU:                                                         0225
*   CALL IGGPCHIU;                  /* COMPUTE HIGH USED RBA OF CRA  */
Z008CHIU BAL   @14,IGGPCHIU                                        0225
*Z009CCI0:                                                         0226
*   CALL IGGPCCI0;                  /* CONSTRUCT CRA BASE DATA RECORD*/
Z009CCI0 BAL   @14,IGGPCCI0                                        0226
*Z010CI15:                                                         0227
*   CALL IGGPCI15;                  /* CONSTRUCT CRA CI'S 1 TO 5     */
*                                                                  0227
Z010CI15 BAL   @14,IGGPCI15                                        0227
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLDA                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0228
*Z011MODI:                                                         0228
*   CALL IGGPMODI;                  /* INTERFACE TO MODIFY ROUTINE   */
Z011MODI L     @15,@CV04455                                        0228
         BALR  @14,@15                                             0228
*   IF CCACD1^=RCS THEN             /* CHECK IF MODIFY SUCCESSFUL    */
         CLI   CCACD1(R11),0                                       0229
         BE    @RF00229                                            0229
*Z012DCXT:                                                         0230
*     CALL IGGPDCXT;                /* IF NOT, CLEAN UP AND EXIT     */
Z012DCXT BAL   @14,IGGPDCXT                                        0230
*   ELSE                                                           0231
*     ;                             /* MODIFY PROCESSING SUCCESSFUL  */
@RF00229 DS    0H                                                  0232
*/********************************************************************/
*/*                CRA  'OPEN'-RELATED  PROCESSING                   */
*/********************************************************************/
*                                                                  0232
*Z013CACB:                                                         0232
*   CALL IGGPCACB;                  /* CONSTRUCT CRA ACB IN RA 1     */
Z013CACB BAL   @14,IGGPCACB                                        0232
*Z014CXWA:                                                         0233
*   CALL IGGPCXWA;                  /* CONSTRUCT CRA CAXWA IN RA 1   */
Z014CXWA BAL   @14,IGGPCXWA                                        0233
*Z015OCRA:                                                         0234
*   CALL IGGPOCRA;                  /* OPEN THE CRA ESDS             */
Z015OCRA BAL   @14,IGGPOCRA                                        0234
*   IF CCACD1^=RCS THEN             /* IF CRA OPEN FAILED,           */
         CLI   CCACD1(R11),0                                       0235
         BE    @RF00235                                            0235
*Z016DCXT:                                                         0236
*     CALL IGGPDCXT;                /* CLEAN UP AND EXIT             */
Z016DCXT BAL   @14,IGGPDCXT                                        0236
*   ELSE                                                           0237
*     ;                             /* CRA SUCCESSFULLY OPENED       */
@RF00235 DS    0H                                                  0238
*/********************************************************************/
*/*      WRITE THE CRA SELF-DESCRIBING RECORDS AND THE VOLUME        */
*/*      RECORDS TO THE CRA AND THE CATALOG.                         */
*/********************************************************************/
*                                                                  0238
*Z017CRPL:                                                         0238
*   CALL IGGPCRPL;                  /* CONSTRUCT CRA RPL IN RA 1     */
*                                                                  0238
Z017CRPL BAL   @14,IGGPCRPL                                        0238
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLDA                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0239
*Z018WCRA:                                                         0239
*   CALL IGGPWCRA;                  /* WRITE RECORDS TO THE CRA      */
Z018WCRA L     @15,@CV04458                                        0239
         BALR  @14,@15                                             0239
*   IF CCACD1^=RCS THEN             /* CHECK IF I/O SUCCESSFUL       */
         CLI   CCACD1(R11),0                                       0240
         BE    @RF00240                                            0240
*Z019DCXT:                                                         0241
*     CALL IGGPDCXT;                /* IF NOT, CLEAN UP AND EXIT     */
Z019DCXT BAL   @14,IGGPDCXT                                        0241
*   ELSE                                                           0242
*     ;                             /* CRA RECORDS WRITTEN OKAY      */
@RF00240 DS    0H                                                  0243
*   IF CCAF2CB=OFF THEN             /* CALLER IS DEFINE SPACE        */
*                                                                  0243
         TM    CCAF2CB(R11),B'00000010'                            0243
         BNZ   @RF00243                                            0243
*     /***************************************************************/
*     /*                                                             */
*     /* CALL TO MODULE IGG0CLDA                                     */
*     /*                                                             */
*     /***************************************************************/
*                                                                  0244
*Z020WCAT:                                                         0244
*     CALL IGGPWCAT;                /* WRITE VOLUME RECORDS TO CATLG */
Z020WCAT L     @15,@CV04457                                        0244
         BALR  @14,@15                                             0244
*   ELSE                                                           0245
*     ;                             /* CALLER IS DEFINE CATALOG      */
@RF00243 DS    0H                                                  0246
*Z021DCXT:                                                         0246
*   CALL IGGPDCXT;                  /* DO CLEANUP PROCESSING FOR EXIT*/
Z021DCXT BAL   @14,IGGPDCXT                                        0246
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0247
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPSTRG');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPSTRG'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00002 DS    0H                                                  0250
@EF00002 DS    0H                                                  0250
@ER00002 BR    @14                                                 0250
@PB00002 DS    0H                                                  0251
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0252
*/********************************************************************/
*/*                                                                  */
*/*        IGGPSTRG  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0252
*IGGPSTRG:                                                         0252
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00003                                            0252
IGGPSTRG DS    0H                                                  0253
*Z022:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z022STRG:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0253
*    (R5,                                                          0253
*     R6,                                                          0253
*     R7,                                                          0253
*     R8,                                                          0253
*     R9,                                                          0253
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z022     DS    0H                                                  0253
Z022STRG DS    0H                                                  0254
*   RFY                                                            0254
*    (R2,                                                          0254
*     R3,                                                          0254
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPSTRG                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = OBTAIN MAIN STORAGE AND VOLUME SPACE          *
**                                                                    *
**   FUNCTION =                                                       *
**      OBTAIN MAIN STORAGE WORK SPACE FOR THE DEFINE CRA PROCESS     *
**      AND ALSO TO ALLOCATE SPACE ON THE VOLUME FOR THE CATALOG      *
**      RECOVERY AREA ITSELF:                                         *
**      1.  OBTAIN SPACE FOR RECORD CONSTRUCTION AND CONTROL BLOCKS   *
**      2.  CLEAR NEWLY OBTAINED MAIN STORAGE WORK AREA               *
**      3.  OBTAIN ONE CYLINDER OF SPACE ON THE VOLUME FOR THE CRA    *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**                                                                    *
**   OUTPUT =                                                         *
**      IF ERROR ENCOUNTERED, 'CCAPROB' CONTAINS PROBLEM              *
**         DETERMINATION INFORMATION.                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**      ADDRESS OF DEFINE CRA WORKAREA IN CCACRACI                    *
**                                                                    *
**********************************************************************/
*/********************************************************************/
*/*     OBTAIN SPACE FOR RECORD CONSTRUCTION AND CONTROL BLOCKS      */
*/********************************************************************/
*                                                                  0257
*   CCAMNLEN=LENGTH(DCRAWKA);       /* SET REQUEST CORE SIZE         */
         LA    @10,3472                                            0257
         STCM  @10,7,CCAMNLEN(R11)                                 0257
*   IF CCAF2CB=ON THEN              /* CALLER IS DEFINE CATALOG      */
         TM    CCAF2CB(R11),B'00000010'                            0258
         BNO   @RF00258                                            0258
*     CCAMNLEN=CCAMNLEN             /* GETMAIN RAB 3 SAVE AREA ALSO  */
*         +LENGTH(DCRARABS);                                       0259
         AH    @10,@CH04638                                        0259
         STCM  @10,7,CCAMNLEN(R11)                                 0259
*   ELSE                                                           0260
*     ;                             /* CALLER IS DEFINE SPACE        */
@RF00258 DS    0H                                                  0261
*   RFY                                                            0261
*     R1 RSTD;                      /* RESTRICT R1 FOR GETMAIN MACRO */
*   R1=ADDR(CCAMSSPL);              /* POINT TO STRG MGMT P.L.       */
         LA    R1,CCAMSSPL(,R11)                                   0262
*   CCAMNSPL=USUBPOOL;              /* STORE SUBPOOL VALUE           */
*                                                                  0263
         MVI   CCAMNSPL(R11),X'FC'                                 0263
*   /*****************************************************************/
*   /*                                                               */
*   /* ALLOCATE MAIN STORAGE FROM USER SUBPOOL 252                   */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0264
*   GEN(GETMAIN LC,MF=(E,(1)));     /* GET MAIN STORAGE SPACE        */
         GETMAIN LC,MF=(E,(1))
*   GEN(LR R1,R15);                 /* PICK UP RETURN CODE           */
         LR R1,R15
*   IF R1^=NOGETMSE THEN            /* CHECK FOR ERROR RETURN        */
*                                                                  0266
         LTR   R1,R1                                               0266
         BZ    @RF00266                                            0266
*     /***************************************************************/
*     /*                                                             */
*     /* SET CCAMODID='B4', CCAREASN=RSN2, CCARETRN=RCINSSWA         */
*     /*                                                             */
*     /***************************************************************/
*                                                                  0267
*     CCAPROB=IGGSEGID+(RSN2*K256)+RCINSSWA;/* INSUFFICIENT CORE   0267
*                                      ERROR                         */
         MVC   CCAPROB(4,R11),@CF04640                             0267
*   ELSE                            /* GETMAIN SUCCESSFUL            */
*     DO;                                                          0268
         B     @RC00266                                            0268
@RF00266 DS    0H                                                  0269
*       DO;                                                        0269
*         IF MNATFULL=ON THEN       /* TRACKING TABLE FULL           */
*                                                                  0270
         TM    MNATFULL(R11),B'10000000'                           0270
         BNO   @RF00270                                            0270
*           /*********************************************************/
*           /*                                                       */
*           /* CALL TO MODULE IGG0CLAT                               */
*           /*                                                       */
*           /*********************************************************/
*                                                                  0271
*Z026ARFS:                                                         0271
*           CALL IGGPARFS;          /* YES,CALL SPILL ROUTINE        */
Z026ARFS L     @15,@CV04451                                        0271
         BALR  @14,@15                                             0271
*         GEN REFS(CCAMNAT)(MVC CCAMNAT(248,11),CCAMNAT+8(11));/*  0272
*                                      PUSH AND TRACK                */
@RF00270 DS    0H                                                  0272
         MVC CCAMNAT(248,11),CCAMNAT+8(11)
*       END;                                                       0273
*     END;                          /* GET MAIN STORAGE              */
*   RFY                                                            0275
*     R1 UNRSTD;                    /* UNRESTRICT R1'S USE           */
@RC00266 DS    0H                                                  0276
*   IF CCACD1^=RCS THEN             /* GET CORE SUCCESSFUL?          */
         CLI   CCACD1(R11),0                                       0276
         BE    @RF00276                                            0276
*     DO;                           /* IF NOT SUCCESSFUL,            */
*       DCRAWAPR=BLANK;             /* CLEAR WORK AREA ADDRESS       */
         SLR   DCRAWAPR,DCRAWAPR                                   0278
*       CALL EXIT;                  /* RETURN TO CALLER: IGGPDCRA    */
         BAL   @14,EXIT                                            0279
*     END;                                                         0280
*   ELSE                                                           0281
*     ;                             /* GET CORE SUCCESSFUL           */
@RF00276 DS    0H                                                  0282
*/********************************************************************/
*/*         CLEAR  NEWLY  OBTAINED  MAIN  STORAGE  AREA              */
*/********************************************************************/
*                                                                  0282
*   R2=CCAMNADR;                    /* GET CORE ADDRESS              */
         L     R2,CCAMNADR(,R11)                                   0282
*   R3=CCAMNLEN;                    /* GET CORE LENGTH               */
         SLR   R3,R3                                               0283
         ICM   R3,7,CCAMNLEN(R11)                                  0283
*   RFY                                                            0284
*     R5 RSTD;                      /* RESTRICT R5 FOR MVCL INSTRUC. */
*   R5=BLANK;                       /* CLEAR R5 FOR MVCL INSTRUCTION */
         SLR   R5,R5                                               0285
*   GEN(MVCL  R2,R4);               /* CLEAR MAIN STORAGE WORKAREA   */
         MVCL  R2,R4
*   RFY                                                            0287
*     R5 UNRSTD;                    /* UNRESTRICT USE OF REGISTER 5  */
*   DCRAWAPR=CCAMNADR;              /* SET CRA WORKAREA BASE REGISTER*/
         L     DCRAWAPR,CCAMNADR(,R11)                             0288
*   CCACRACI=DCRAWAPR;              /* SET PTR TO CRA CI ADDR ARRAY  */
         ST    DCRAWAPR,CCACRACI(,R11)                             0289
*   DCRAMNLN=CCAMNLEN;              /* SAVE TOTAL LENGTH OF WORKAREA */
         SLR   @10,@10                                             0290
         ICM   @10,7,CCAMNLEN(R11)                                 0290
         ST    @10,DCRAMNLN(,DCRAWAPR)                             0290
*   DCRADESA=CCADESA;               /* SAVE DADSM EXTENT INFORMATION */
         MVC   DCRADESA(44,DCRAWAPR),CCADESA(R11)                  0291
*/********************************************************************/
*/*      OBTAIN ONE CYLINDER OF SPACE ON THE VOLUME FOR THE CRA      */
*/********************************************************************/
*                                                                  0292
*   CCACRABU=OFF;                   /* TURN OFF CRA BUILD INDICATOR  */
         NI    CCACRABU(R11),B'11111110'                           0292
*   IF CCAF2CB=OFF THEN             /* CALLER IS DEFINE SPACE        */
         TM    CCAF2CB(R11),B'00000010'                            0293
         BNZ   @RF00293                                            0293
*     DO;                           /* PERFORM DEFINE SPACE CHORES   */
*       DCRATRCY=CTVNOTRK;          /* SAVE TRACKS/CYLINDER VALUE    */
         L     @10,CCAURAB(,R11)                                   0295
         L     @10,CCACPE1(,@10)                                   0295
         MVC   DCRATRCY(2,DCRAWAPR),CTVNOTRK+48(@10)               0295
*       DCRABTRK=CCACRABT;          /* SAVE BLOCKS/TRACK VALUE       */
         MVC   DCRABTRK(2,DCRAWAPR),CCACRABT(R11)                  0296
*       IF((CTGFVATR^=BLANK)&       /* DATA SET ATTR FPL EXISTS      */
*           (CTGFVATR->CTGFLPT(1)->CTGDSUNQ=ON))|/* REQUEST IS FOR 0297
*                                      DEFINE UNIQUE                 */
*           (CTGFVSPC->CTGFLPT(1)->SPCRQTYP=CMSALCAN) THEN/* REQUEST
*                                      IS FOR DEF CANDIDATE          */
*                                                                  0297
         L     @10,CCACPL(,R11)                                    0297
         L     @10,CTGFVT(,@10)                                    0297
         L     @10,CTGFVATR(,@10)                                  0297
         LTR   @10,@10                                             0297
         BZ    @GL00001                                            0297
         L     @10,CTGFLPT(,@10)                                   0297
         TM    CTGDSUNQ(@10),B'01000000'                           0297
         BO    @RT00297                                            0297
@GL00001 L     @10,CCACPL(,R11)                                    0297
         L     @10,CTGFVT(,@10)                                    0297
         L     @10,CTGFVSPC(,@10)                                  0297
         L     @10,CTGFLPT(,@10)                                   0297
         TM    SPCRQTYP(@10),B'11000000'                           0297
         BNZ   @RF00297                                            0297
@RT00297 DS    0H                                                  0298
*         /***********************************************************/
*         /*                                                         */
*         /* CALL TO MODULE IGG0CLDD                                 */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0298
*Z027RDEF:                                                         0298
*         CALL IGGPRDEF;            /* INTERFACE TO DEFINE SPACE     */
Z027RDEF BAL   @14,IGGPRDEF                                        0298
*       ELSE                                                       0299
*         ;                         /* NOT CANDIDATE OR UNIQUE     0299
*                                      REQUEST                       */
@RF00297 DS    0H                                                  0300
*     END;                          /* END 'DO' FOR DEFINE SPACE     */
*   ELSE                                                           0301
*     ;                             /* CALLER IS DEFINE CATALOG      */
@RF00293 DS    0H                                                  0302
*   IF CCACD1=RCS THEN              /* SUCCESSFUL SO FAR             */
         CLI   CCACD1(R11),0                                       0302
         BNE   @RF00302                                            0302
*Z028SBAL:                                                         0303
*     CALL IGGPSBAL;                /* INTERFACE TO SUBALLOCATE RTN  */
Z028SBAL BAL   @14,IGGPSBAL                                        0303
*   ELSE                                                           0304
*     ;                             /* AN ERROR HAS OCCURRED         */
@RF00302 DS    0H                                                  0305
*   CCACRABU=ON;                    /* TURN ON CRA BUILD FLAG AGAIN  */
         OI    CCACRABU(R11),B'00000001'                           0305
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0306
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPRDEF');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPRDEF'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00003 DS    0H                                                  0309
@EF00003 DS    0H                                                  0309
@ER00003 BR    @14                                                 0309
@PB00003 DS    0H                                                  0310
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0311
*/********************************************************************/
*/*                                                                  */
*/*        IGGPRDEF  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0311
*IGGPRDEF:                                                         0311
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00004                                            0311
IGGPRDEF DS    0H                                                  0312
*Z029:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z029RDEF:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0312
*    (R5,                                                          0312
*     R6,                                                          0312
*     R7,                                                          0312
*     R8,                                                          0312
*     R9,                                                          0312
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z029     DS    0H                                                  0312
Z029RDEF DS    0H                                                  0313
*   RFY                                                            0313
*    (R2,                                                          0313
*     R3,                                                          0313
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPRDEF                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = INTERFACE TO DEFINE SPACE (IGG0CLAQ)          *
**                                                                    *
**   FUNCTION =                                                       *
**      CONSTRUCT AN INTERFACE FOR A RECURSIVE CALL TO THE DEFINE     *
**      SPACE ROUTINE IGGPDEFS(IGG0CLAQ)                              *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      IF ERROR ENCOUNTERED, 'CCAPROB' CONTAINS PROBLEM              *
**         DETERMINATION INFORMATION.                                 *
**                                                                    *
**********************************************************************/
*                                                                  0316
*   CCADSRCL=ON;                    /* TURN ON RECURSIVE CALL FLAG   */
         OI    CCADSRCL(R11),B'10000000'                           0316
*   DCRADSWA=CCADSPWA;              /* SAVE CLAQ'S WORK AREA ADDR    */
         MVC   DCRADSWA(4,DCRAWAPR),CCADSPWA(R11)                  0317
*   DCRADSUL=CTGFVVLT;              /* SAVE CLAQ'S VOLUME LIST ADDR  */
         L     @10,CCACPL(,R11)                                    0318
         L     @10,CTGFVT(,@10)                                    0318
         MVC   DCRADSUL(4,DCRAWAPR),CTGFVVLT(@10)                  0318
*   DCRADSAF=CTGFVATR;              /* SAVE CLAQ'S DATA SET ATTR FPL
*                                      ADDRESS                       */
         MVC   DCRADSAF(4,DCRAWAPR),CTGFVATR(@10)                  0319
*   DCRAOFPL=CTGFVSPC               /* SAVE ADDRESS OF IGG0CLAQ'S    */
*       ->CTGFLPT(1);               /* SPACE ALLOCATION FPL          */
         L     @05,CTGFVSPC(,@10)                                  0320
         MVC   DCRAOFPL(4,DCRAWAPR),CTGFLPT(@05)                   0320
*   CTGFVATR=BLANK;                 /* INDICATE NON-UNIQUE SPACE     */
         SLR   @15,@15                                             0321
         ST    @15,CTGFVATR(,@10)                                  0321
*   DCRAFVEN=CTGFVENT;              /* SAV CLAQ'S ENTRY NAME @ZA05914*/
         MVC   DCRAFVEN(4,DCRAWAPR),CTGFVENT(@10)                  0322
*   CTGFVVLT=ADDR(DCRADSVL);        /* POINT TO 1-ENTRY VOLUME LIST  */
         LA    @15,DCRADSVL(,DCRAWAPR)                             0323
         ST    @15,CTGFVVLT(,@10)                                  0323
*   CTGFVSPC->CTGFLPT(1)            /* POINT TO NEW SPACE ALLOC FPL  */
*       =ADDR(DCRASPRM);                                           0324
         LA    @10,DCRASPRM(,DCRAWAPR)                             0324
         ST    @10,CTGFLPT(,@05)                                   0324
*   RFY                                                            0325
*     SPACEPRM BASED(ADDR(DCRASPRM));                              0325
*   SPCPRIMQ=DCRATRCY;              /* PRIM ALLOC: CYL NO. OF TRACKS */
         LA    @10,400                                             0326
         ALR   @10,DCRAWAPR                                        0326
         LH    @05,DCRATRCY(,DCRAWAPR)                             0326
         STCM  @05,7,SPCPRIMQ(@10)                                 0326
*   SPCSECDQ=SPCPRIMQ;              /* SEC ALLOC: CYL NO. OF TRACKS  */
         STCM  @05,7,SPCSECDQ(@10)                                 0327
*   SPCRQTYP=CMSALTRK;              /* TRACK REQUEST TYPE            */
         NI    SPCRQTYP(@10),B'10111111'                           0328
         OI    SPCRQTYP(@10),B'10000000'                           0328
*   RFY                                                            0329
*     USERLIST BASED(ADDR(DCRADSVL));/* MAP VOLUME LIST              */
*   RFY                                                            0330
*     UCBOB BASED(CMSDFSPC);        /* MAP UCB OFF OF ADDR FROM CLAQ */
*   LTLENGTH=LENGTH(LTENTRY);       /* LENGTH OF 1 VOLUME LIST ENTRY */
         LA    @10,407                                             0331
         ALR   @10,DCRAWAPR                                        0331
         MVC   LTLENGTH(2,@10),@CH00041                            0331
*   LTVOLSER(1)=UCBVOLI;            /* SET VOLUME SERIAL NUMBER      */
*                                                                  0332
         L     @05,CCACMSWA(,R11)                                  0332
         L     @05,CMSDFSPC(,@05)                                  0332
         MVC   LTVOLSER(6,@10),UCBVOLI(@05)                        0332
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLAQ                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0333
*Z030DEFS:                                                         0333
*   CALL IGGPDEFS;                  /* GET PHYSICAL SPACE            */
Z030DEFS L     @15,@CV04452                                        0333
         BALR  @14,@15                                             0333
*   DCRAWAPR=CCACRACI;              /* RESTORE DEF CRA WORKAREA ADDR */
         L     DCRAWAPR,CCACRACI(,R11)                             0334
*   CCADSPWA=DCRADSWA;              /* RESTORE CLAQ'S WORKAREA ADDR  */
         MVC   CCADSPWA(4,R11),DCRADSWA(DCRAWAPR)                  0335
*   CTGFVVLT=DCRADSUL;              /* RESTORE CLAQ'S VOL LIST ADDR  */
         L     @10,CCACPL(,R11)                                    0336
         L     @10,CTGFVT(,@10)                                    0336
         MVC   CTGFVVLT(4,@10),DCRADSUL(DCRAWAPR)                  0336
*   CTGFVATR=DCRADSAF;              /* RESTORE CLAQ'S DATA SET ATTR
*                                      FPL ADDRESS                   */
         MVC   CTGFVATR(4,@10),DCRADSAF(DCRAWAPR)                  0337
*   CTGFVENT=DCRAFVEN;              /* RESTORE ENTRY NAME ADDRESS IN
*                                      CLAQ'S SPACE FVT      @ZA05914*/
         MVC   CTGFVENT(4,@10),DCRAFVEN(DCRAWAPR)                  0338
*   CTGFVSPC->CTGFLPT(1)            /* RESTORE ADDRESS OF IGG0CLAQ'S */
*       =DCRAOFPL;                  /* SPACE ALLOCATION FPL          */
         L     @10,CTGFVSPC(,@10)                                  0339
         MVC   CTGFLPT(4,@10),DCRAOFPL(DCRAWAPR)                   0339
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0340
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPSBAL');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPSBAL'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00004 DS    0H                                                  0343
@EF00004 DS    0H                                                  0343
@ER00004 BR    @14                                                 0343
@PB00004 DS    0H                                                  0344
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0345
*/********************************************************************/
*/*                                                                  */
*/*        IGGPSBAL  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0345
*IGGPSBAL:                                                         0345
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00005                                            0345
IGGPSBAL DS    0H                                                  0346
*Z031:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z031SBAL:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0346
*    (R5,                                                          0346
*     R6,                                                          0346
*     R7,                                                          0346
*     R8,                                                          0346
*     R9,                                                          0346
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z031     DS    0H                                                  0346
Z031SBAL DS    0H                                                  0347
*   RFY                                                            0347
*    (R2,                                                          0347
*     R3,                                                          0347
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPSBAL                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = INTERFACE TO SUBALLOCATE                      *
**                                                                    *
**   FUNCTION =                                                       *
**      CONSTRUCT AN INTERFACE TO AND THEN CALL THE SUBALLOCATE       *
**      FUNCTION IGGPSALL(IGG0CLAR) TO SUBALLOCATE ONE CYLINDER       *
**      OF SPACE ON THE VOLUME FOR THE CRA.                           *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      IF ERROR ENCOUNTERED, 'CCAPROB' CONTAINS PROBLEM              *
**         DETERMINATION INFORMATION.                                 *
**                                                                    *
**********************************************************************/
*                                                                  0350
*   IF CCAF2CB=ON THEN              /* CALLER IS DEFINE CATALOG      */
         TM    CCAF2CB(R11),B'00000010'                            0350
         BNO   @RF00350                                            0350
*     DO;                           /* FETCH DEVICE CHARACTERISTICS  */
*       DCRATRCY=CMTRKCYL(1);       /* SAVE TRACKS/CYLINDER VALUE    */
         L     @10,CCACMSWA(,R11)                                  0352
         MVC   DCRATRCY(2,DCRAWAPR),CMTRKCYL(@10)                  0352
*       DCRABTRK=CMBLKTRK(1);       /* SAVE BLOCKS/TRACK VALUE       */
         MVC   DCRABTRK(2,DCRAWAPR),CMBLKTRK(@10)                  0353
*     END;                                                         0354
*   ELSE                                                           0355
*     ;                             /* CALLER IS DEFINE SPACE        */
@RF00350 DS    0H                                                  0356
*   CCAALPTR=ADDR(DCRASALP);        /* ADDR OF SUBALLOCATE PARM LIST */
         LA    @10,DCRASALP(,DCRAWAPR)                             0356
         ST    @10,CCAALPTR(,R11)                                  0356
*   SALPARL(1:22)=SALPARL(1:22)&&SALPARL(1:22);/* CLEAR PARAMETER  0357
*                                      LIST AREA                     */
         XC    SALPARL(22,@10),SALPARL(@10)                        0357
*   SALOPTNS=SATRKOPT;              /* SET TRACK REQUEST FLAG        */
         MVI   SALOPTNS(@10),X'80'                                 0358
*   SALSOURC=SARABS;                /* IND VOLUME RECORD IS IN RAB 3 */
         MVI   SALSOURC(@10),X'01'                                 0359
*   SALCALN=DCRATRCY;               /* SET CONTROL AREA LENGTH       */
         LH    @05,DCRATRCY(,DCRAWAPR)                             0360
         STH   @05,SALCALN(,@10)                                   0360
*   SALUNITS=DCRATRCY;              /* SET NUMBER OF UNITS REQUESTED */
         STH   @05,SALUNITS(,@10)                                  0361
*   SALMXEXT=NUMEXTS;               /* SET MAXIMUM NUMBER OF EXTENTS */
*                                                                  0362
         MVI   SALMXEXT(@10),X'01'                                 0362
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLAR                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0363
*Z032SALL:                                                         0363
*   CALL IGGPSALL;                  /* CALL SUBALLOCATE FUNCTION     */
Z032SALL L     @15,@CV04456                                        0363
         BALR  @14,@15                                             0363
*   DCRAWAPR=CCACRACI;              /* RESTORE DEF CRA WORKAREA ADDR */
         L     DCRAWAPR,CCACRACI(,R11)                             0364
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0365
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPFMT4');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPFMT4'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00005 DS    0H                                                  0368
@EF00005 DS    0H                                                  0368
@ER00005 BR    @14                                                 0368
@PB00005 DS    0H                                                  0369
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0370
*/********************************************************************/
*/*                                                                  */
*/*        IGGPFMT4  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0370
*IGGPFMT4:                                                         0370
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00006                                            0370
IGGPFMT4 DS    0H                                                  0371
*Z033:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z033FMT4:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0371
*    (R5,                                                          0371
*     R6,                                                          0371
*     R7,                                                          0371
*     R8,                                                          0371
*     R9,                                                          0371
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z033     DS    0H                                                  0371
Z033FMT4 DS    0H                                                  0372
*   RFY                                                            0372
*    (R2,                                                          0372
*     R3,                                                          0372
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPFMT4                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = FORMAT-4 DSCB PROCESSING                      *
**                                                                    *
**   FUNCTION =                                                       *
**      UPDATE THE FORMAT-4 DSCB ON THE VOLUME WITH THE 'TT'          *
**      LOCATION OF THE CRA.                                          *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**      VOLUME UCB ADDRESS IN CMSWA(CMSDFSPC)                         *
**                                                                    *
**   OUTPUT =                                                         *
**      IF ERROR ENCOUNTERED, 'CCAPROB' CONTAINS PROBLEM              *
**         DETERMINATION INFORMATION.                                 *
**      'CCAURAB' IS ALTERED.                                         *
**                                                                    *
**********************************************************************/
*                                                                  0375
*   RFY                                                            0375
*     IGGMF4WA BASED(CCAURAB);      /* MAP THE F4 DSCB WORK AREA     */
*   CCAURAB=ADDR(DCRARACI);         /* WORK AREA FOR IGGPF4RD        */
         LA    @10,DCRARACI(,DCRAWAPR)                             0376
         ST    @10,CCAURAB(,R11)                                   0376
*   F4WDUCB=CMSDFSPC;               /* SET VOLUME UCB ADDRESS FROM 0377
*                                      VALUE IN CMSWA SET UP BY    0377
*                                      DEFINE SPACE (IGG0CLAQ)       */
*                                                                  0377
         L     @05,CCACMSWA(,R11)                                  0377
         MVC   F4WDUCB(3,@10),CMSDFSPC+1(@05)                      0377
*   /*****************************************************************/
*   /*                                                               */
*   /* CALL TO MODULE IGG0CLBU                                       */
*   /*                                                               */
*   /*****************************************************************/
*                                                                  0378
*Z034F4RD:                                                         0378
*   CALL IGGPF4RD;                  /* READ FORMAT 4 DSCB            */
Z034F4RD L     @15,@CV04453                                        0378
         BALR  @14,@15                                             0378
*   DCRAWAPR=CCACRACI;              /* RESTORE DEF CRA WORKAREA ADDR */
         L     DCRAWAPR,CCACRACI(,R11)                             0379
*   IF CCACD1=RCS THEN              /* IF READ SUCCESSFUL, CONTINUE  */
         CLI   CCACD1(R11),0                                       0380
         BNE   @RF00380                                            0380
*     DO;                           /* ALTER THE FORMAT-4 DSCB       */
*       DS4VSCRA=CCAEXTCC(1)*       /* CONVERT CCHH TO TT            */
*           DCRATRCY+CCAEXTHH(1);                                  0382
*                                                                  0382
         L     @10,CCAURAB(,R11)                                   0382
         LH    @05,CCAEXTCC(,R11)                                  0382
         MH    @05,DCRATRCY(,DCRAWAPR)                             0382
         AH    @05,CCAEXTHH(,R11)                                  0382
         STH   @05,DS4VSCRA+120(,@10)                              0382
*       /*************************************************************/
*       /*                                                           */
*       /* CALL TO MODULE IGG0CLBU                                   */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0383
*Z035F4WR:                                                         0383
*       CALL IGGPF4WR;              /* WRITE UPDATED FORMAT 4 DSCB   */
Z035F4WR L     @15,@CV04454                                        0383
         BALR  @14,@15                                             0383
*       DCRAWAPR=CCACRACI;          /* RESTORE DEF CRA WORKAREA ADDR */
         L     DCRAWAPR,CCACRACI(,R11)                             0384
*       RFY                                                        0385
*         CATRECRD BASED(CCAURAB);  /* MAP F4 DSCB WORK AREA         */
*       CATRECRD(1)=CATRECRD(1)&&CATRECRD(1);                      0386
         L     @10,CCAURAB(,R11)                                   0386
         XC    CATRECRD(256,@10),CATRECRD(@10)                     0386
*       CATRECRD(2)=CATRECRD(2)&&CATRECRD(2);/* CLEAR WORK AREA FOR
*                                      FUTURE USE                    */
         XC    CATRECRD+256(256,@10),CATRECRD+256(@10)             0387
*     END;                          /* END 'DO' FOR FORMAT-4 ALTER   */
*   ELSE                                                           0389
*     ;                             /* FORMAT-4 READ FAILED          */
@RF00380 DS    0H                                                  0390
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0390
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPCHIU');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPCHIU'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00006 DS    0H                                                  0393
@EF00006 DS    0H                                                  0393
@ER00006 BR    @14                                                 0393
@PB00006 DS    0H                                                  0394
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0395
*/********************************************************************/
*/*                                                                  */
*/*        IGGPCHIU  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0395
*IGGPCHIU:                                                         0395
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00007                                            0395
IGGPCHIU DS    0H                                                  0396
*Z036:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z036CHIU:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0396
*    (R5,                                                          0396
*     R6,                                                          0396
*     R7,                                                          0396
*     R8,                                                          0396
*     R9,                                                          0396
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z036     DS    0H                                                  0396
Z036CHIU DS    0H                                                  0397
*   RFY                                                            0397
*    (R2,                                                          0397
*     R3,                                                          0397
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPCHIU                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = COMPUTE HIGH-USED RBA OF CRA                  *
**                                                                    *
**   FUNCTION =                                                       *
**      COMPUTE THE HIGH-USED RBA OF THE CRA FOR LATER USE IN         *
**      THE CONSTRUCTION OF THE CRA SELF-DESCRIBING RECORDS.          *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11.                                *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3).      *
**      IF CALLER IS IGG0CLAS, THEN CMSCTREC(*) CONTAINS POINTERS     *
**         TO THE VOLUME BASE AND EXTENSION RECORDS.                  *
**      IF CALLER IS IGG0CLAQ, THEN CCARAVCA(1) POINTS TO THE         *
**         FIRST RECORD IN THE CHAIN OF BUFFERED VOLUME RECORDS.      *
**                                                                    *
**   OUTPUT =                                                         *
**      'CCAURAB' POINTS TO RAB 1.                                    *
**      'DCRACHIU' CONTAINS THE VALUE OF HIGH-USED RBA OF THE CRA.    *
**                                                                    *
**********************************************************************/
*                                                                  0400
*   CCAURAB=ADDR(CCARAB1);          /* ADDRESS RECORD THROUGH RAB 1  */
         LA    @10,CCARAB1(,R11)                                   0400
         ST    @10,CCAURAB(,R11)                                   0400
*   DCRAR1PT=CCACPE1;               /* SAVE POINTER TO RECORD AREA 1 */
         MVC   DCRAR1PT(4,DCRAWAPR),CCACPE1(@10)                   0401
*   DCRACHIU=LEN10CIS;              /* LENGTH OF CI'S 0-9            */
         MVC   DCRACHIU(4,DCRAWAPR),@CF04408                       0402
*   PROCESS=GO;                     /* SET LOOP CONTROL CHARACTERS   */
         L     @10,CCACMSWA(,R11)                                  0403
         MVC   PROCESS(2,@10),@CC04442                             0403
*/********************************************************************/
*/*      COUNT  EXTENSION  RECORDS  FOR  DEFINE  CATALOG  CASE       */
*/********************************************************************/
*                                                                  0404
*   IF CCAF2CB=ON THEN              /* CALLER IS DEFINE CATALOG      */
         TM    CCAF2CB(R11),B'00000010'                            0404
         BNO   @RF00404                                            0404
*     DO DCRARCTR=11 BY 1           /* SET RECORD ADDRESS ARRAY      */
*           WHILE(PROCESS=GO);      /* SUBSCRIPT RANGE               */
         LA    DCRARCTR,11                                         0405
         B     @DE00405                                            0405
@DL00405 DS    0H                                                  0406
*       CCACPE1=CMSCTREC(DCRARCTR); /* SET POINTER TO RECORD         */
         L     @10,CCAURAB(,R11)                                   0406
         LR    @05,DCRARCTR                                        0406
         SLA   @05,2                                               0406
         L     @01,CCACMSWA(,R11)                                  0406
         L     @05,CMSCTREC-4(@05,@01)                             0406
         ST    @05,CCACPE1(,@10)                                   0406
*       IF RCDID=VOLEXTID THEN      /* COUNT IF EXTENSION RECORD     */
         CLI   RCDID(@05),C'W'                                     0407
         BNE   @RF00407                                            0407
*         DCRACHIU=DCRACHIU         /* INCREMENT USED-AREA SIZE      */
*             +DCRACISZ;                                           0408
         LA    @10,512                                             0408
         AL    @10,DCRACHIU(,DCRAWAPR)                             0408
         ST    @10,DCRACHIU(,DCRAWAPR)                             0408
*       ELSE                        /* FOR NO MORE EXTENSION RECORDS,*/
*         PROCESS=STOP;             /* SET LOOP CONTROL FLAG         */
         B     @RC00407                                            0409
@RF00407 L     @10,CCACMSWA(,R11)                                  0409
         MVC   PROCESS(2,@10),@CC04440                             0409
*     END;                          /* END 'DO-WHILE' LOOP           */
@RC00407 AH    DCRARCTR,@CH00046                                   0410
@DE00405 L     @10,CCACMSWA(,R11)                                  0410
         CLC   PROCESS(2,@10),@CC04442                             0410
         BE    @DL00405                                            0410
*/********************************************************************/
*/*       COUNT  EXTENSION  RECORDS  FOR  DEFINE  SPACE  CASE        */
*/********************************************************************/
*                                                                  0411
*   ELSE                            /* FOR DEFINE SPACE, CANDIDATE   */
*     DO;                           /* OR UNIQUE                     */
         B     @RC00404                                            0411
@RF00404 DS    0H                                                  0412
*       RFY                                                        0412
*         BUFVOLRD BASED(CCACPE1);  /* MAP BUFFERED VOL RECORD       */
*       CCACPE1=CCARAVCA(1);        /* POINT TO BASE VOLUME RECORD   */
         L     @10,CCAURAB(,R11)                                   0413
         MVC   CCACPE1(4,@10),CCARAVCA(R11)                        0413
*       DO WHILE(PROCESS=GO);       /* PROCEED UNTIL HALTED          */
         B     @DE00414                                            0414
@DL00414 DS    0H                                                  0415
*         CCACPE1=BUFNEXTB;         /* POINT TO EXTENSION RECORD     */
         L     @10,CCAURAB(,R11)                                   0415
         L     @05,CCACPE1(,@10)                                   0415
         L     @05,BUFNEXTB(,@05)                                  0415
         ST    @05,CCACPE1(,@10)                                   0415
*         IF CCACPE1^=BLANK THEN    /* NOT END OF VOL REC CHAIN      */
         LTR   @05,@05                                             0416
         BZ    @RF00416                                            0416
*           DCRACHIU=DCRACHIU       /* INCREMENT USED-AREA SIZE      */
*               +DCRACISZ;          /* BY 1 CI SIZE                  */
         LA    @10,512                                             0417
         AL    @10,DCRACHIU(,DCRAWAPR)                             0417
         ST    @10,DCRACHIU(,DCRAWAPR)                             0417
*         ELSE                      /* END OF VOLUME RECORD CHAIN    */
*           PROCESS=STOP;           /* SET STOP LOOP CONTROL FLAG    */
         B     @RC00416                                            0418
@RF00416 L     @10,CCACMSWA(,R11)                                  0418
         MVC   PROCESS(2,@10),@CC04440                             0418
*       END;                        /* END 'DO-WHILE' LOOP           */
@RC00416 DS    0H                                                  0419
@DE00414 L     @10,CCACMSWA(,R11)                                  0419
         CLC   PROCESS(2,@10),@CC04442                             0419
         BE    @DL00414                                            0419
*     END;                          /* END 'DO' FOR CANDIDATE/UNIQUE */
*   CALL EXIT;                      /* RETURN TO CALLER              */
@RC00404 BAL   @14,EXIT                                            0421
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPCCI0');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPCCI0'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00007 DS    0H                                                  0424
@EF00007 DS    0H                                                  0424
@ER00007 BR    @14                                                 0424
@PB00007 DS    0H                                                  0425
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0426
*/********************************************************************/
*/*                                                                  */
*/*        IGGPCCI0  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0426
*IGGPCCI0:                                                         0426
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00008                                            0426
IGGPCCI0 DS    0H                                                  0427
*Z037:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z037CCI0:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0427
*    (R5,                                                          0427
*     R6,                                                          0427
*     R7,                                                          0427
*     R8,                                                          0427
*     R9,                                                          0427
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z037     DS    0H                                                  0427
Z037CCI0 DS    0H                                                  0428
*   RFY                                                            0428
*    (R2,                                                          0428
*     R3,                                                          0428
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPCCI0                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = CONSTRUCT CRA SELF-DESCRIBING CI 0            *
**                                                                    *
**   FUNCTION =                                                       *
**      CONSTRUCT THE CRA BASE DATA RECORD (CI 0 - TYPE 'D') IN MAIN  *
**      STORAGE.  RAB 1 IS USED TO ADDRESS THE RECORD.                *
**      1.  SET KEY AND HEADER FIELDS                                 *
**      2.  SET FIXED INFORMATION NON-REPEATING FIELDS                *
**      3.  SET CONTROL INFORMATION FOR REPEATING FIELDS              *
**      4.  SET DELETED GROUP OCCURRENCE POINTER FIELDS               *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      CRA CI 0 PARTIALLY CONSTRUCTED IN DEFINE CRA WORKAREA.        *
**                                                                    *
**********************************************************************/
*                                                                  0431
*   CCARBUR=OFF;                    /* RAB NOT IN-USE BY MODIFY      */
         L     @10,CCAURAB(,R11)                                   0431
         NI    CCARBUR(@10),B'01111111'                            0431
*   CCACPE1=DCRACIPT(1);            /* POINT TO BASE DATA RECORD CI 0*/
         L     @05,DCRACIPT(,DCRAWAPR)                             0432
         ST    @05,CCACPE1(,@10)                                   0432
*/********************************************************************/
*/*                 SET  KEY  AND  HEADER  FIELDS                    */
*/********************************************************************/
*                                                                  0433
*   RCDRELID=CURELID;               /* SET RELEASE INDICATOR         */
         MVI   RCDRELID(@05),X'01'                                 0433
*   IF CCAF2CB=ON THEN              /* CALLER IS DEFINE CATALOG      */
         TM    CCAF2CB(R11),B'00000010'                            0434
         BNO   @RF00434                                            0434
*     RCDCRAVL=CTGFVVCH             /* SET CRA VOLUME SERIAL NUMBER  */
*         ->CTGFVVLT                /* IN THE DATA RECORD FROM THE   */
*         ->LTVOLSER(1);            /* SPACE FVT                     */
         L     @10,CCACPL(,R11)                                    0435
         L     @10,CTGFVT(,@10)                                    0435
         L     @10,CTGFVVCH(,@10)                                  0435
         L     @10,CTGFVVLT(,@10)                                  0435
         MVC   RCDCRAVL(6,@05),LTVOLSER(@10)                       0435
*   ELSE                            /* CALLER IS DEFINE SPACE        */
*     RCDCRAVL=CCAACB               /* SET CRA VOLUME SERIAL NUMBER  */
*         ->ACBUAPTR->CAXUCB        /* IN THE DATA RECORD FROM THE   */
*         ->UCBVOLI;                /* CATALOG VOLUME UCB            */
         B     @RC00434                                            0436
@RF00434 L     @10,CCAURAB(,R11)                                   0436
         L     @10,CCACPE1(,@10)                                   0436
         L     @05,CCAACB(,R11)                                    0436
         L     @05,ACBUAPTR(,@05)                                  0436
         L     @05,CAXUCB(,@05)                                    0436
         MVC   RCDCRAVL(6,@10),UCBVOLI(@05)                        0436
*   RCDRACTS=CMWASTOR(1:4);         /* SET CRA CREATION TIMESTAMP    */
@RC00434 L     @10,CCAURAB(,R11)                                   0437
         L     @10,CCACPE1(,@10)                                   0437
         L     @05,CCACMSWA(,R11)                                  0437
         MVC   RCDRACTS(4,@10),CMWASTOR(@05)                       0437
*   RCDID=CATDATID;                 /* SET RECORD AS TYPE 'D'        */
         MVI   RCDID(@10),C'D'                                     0438
*   NREPLF=LENGTH(CATRCD)           /* SET LENGTH OF FIXED FIELDS    */
*       +LENGTH(CTGRBASE);                                         0439
*                                                                  0439
         MVI   NREPLF(@10),X'8F'                                   0439
*/********************************************************************/
*/*          SET  FIXED  INFORMATION  NON-REPEATING  FIELDS          */
*/********************************************************************/
*                                                                  0440
*   IF CCAF2CB=ON THEN              /* CALLER IS DEFINE CATALOG      */
         TM    CCAF2CB(R11),B'00000010'                            0440
         BNO   @RF00440                                            0440
*     CNAME=CTGFVENT->TNAM;         /* SET CATALOG NAME VIA FVT      */
         L     @05,CCACPL(,R11)                                    0441
         L     @05,CTGFVT(,@05)                                    0441
         L     @05,CTGFVENT(,@05)                                  0441
         MVC   CNAME(44,@10),TNAM(@05)                             0441
*   ELSE                            /* CALLER IS DEFINE SPACE        */
*     CNAME=CCAACB->ACBUAPTR->CAXCNAM;/* SET CATALOG NAME FROM     0442
*                                      CATALOG'S CAXWA               */
         B     @RC00440                                            0442
@RF00440 L     @10,CCAURAB(,R11)                                   0442
         L     @10,CCACPE1(,@10)                                   0442
         L     @05,CCAACB(,R11)                                    0442
         L     @05,ACBUAPTR(,@05)                                  0442
         MVC   CNAME(44,@10),CAXCNAM(@05)                          0442
*   COWNID=NULLDWRD;                /* SET OWNER ID TO BINARY ONES   */
@RC00440 L     @10,CCAURAB(,R11)                                   0443
         L     @10,CCACPE1(,@10)                                   0443
         MVC   COWNID(8,@10),@CB04446                              0443
*   RFY                                                            0444
*     R1 RSTD;                      /* RESTRICT FOR 'TIME' MACRO     */
*   GEN(TIME);                      /* GET YYDDDF FORM OF DATE       */
         TIME
*   CCRE=R1;                        /* SET DATA-SET CREATION DATE    */
         L     @10,CCAURAB(,R11)                                   0446
         L     @05,CCACPE1(,@10)                                   0446
         STCM  R1,7,CCRE(@05)                                      0446
*   CEXP=NULLDATE;                  /* SET NULL EXPIRATION DATE      */
         MVC   CEXP(3,@05),@CB03440                                0447
*   CATTR2=SHROPTNS;                /* SET REGION/SYSTEM SHARE OPTNS */
         MVI   CATTR2(@05),X'A0'                                   0448
*   CBUF=DCRABRSZ;                  /* SET MINIMUM BUFFER SIZE (2K)  */
         MVC   CBUF(4,@05),@CF04384                                0449
*   CPRIM=DCRATRCY;                 /* PRIM ALLOC: CYL NO. OF TRACKS */
         LH    @15,DCRATRCY(,DCRAWAPR)                             0450
         STCM  @15,7,CPRIM(@05)                                    0450
*   CSEC=CPRIM;                     /* SEC ALLOC: CYL NO. OF TRACKS  */
         STCM  @15,7,CSEC(@05)                                     0451
*   CSORT=SATRKOPT;                 /* SET TRACK REQUEST FLAG        */
         MVI   CSORT(@05),X'80'                                    0452
*   CHIA=DCRACISZ                   /* SET HIGH ALLOCATED RBA OF CRA */
*       *DCRATRCY*DCRABTRK;                                        0453
         SLA   @15,9                                               0453
         MH    @15,DCRABTRK(,DCRAWAPR)                             0453
         ST    @15,CHIA(,@05)                                      0453
*   DCRACHIA=CHIA;                  /* SAVE HIGH ALLOCATED RBA OF CRA*/
         ST    @15,DCRACHIA(,DCRAWAPR)                             0454
*   CLRECL=DCRARCSZ;                /* SET AVERAGE RECORD SIZE (505) */
         MVC   CLRECL(4,@05),@CF04405                              0455
*   CEXT=NULLDWRD;                  /* SET NULL EXCEPTION EXIT NAME  */
         MVC   CEXT(8,@05),@CB04446                                0456
*/********************************************************************/
*/*        SET  CONTROL  INFORMATION  FOR  REPEATING  FIELDS         */
*/********************************************************************/
*                                                                  0457
*   CCACPE2=CCACPE1                 /* SET ADDR OF REPEATING FIELDS  */
*       +NREPLF;                    /* CONTROL INFORMATION           */
         SLR   @00,@00                                             0457
         IC    @00,NREPLF(,@05)                                    0457
         ALR   @00,@05                                             0457
         ST    @00,CCACPE2(,@10)                                   0457
*   REPCNT=NUMGOPTR;                /* COUNT OF GROUP OCCUR POINTERS */
         LR    @15,@00                                             0458
         MVI   REPCNT(@15),X'01'                                   0458
*   CCACPE3=CCACPE2                 /* SET POINTER TO FIRST GROUP    */
*       +LENGTH(REPCNTRL)           /* OCCURRENCE                    */
*       +LENGTH(REPGOPT(1));                                       0459
         LA    @15,11                                              0459
         ALR   @15,@00                                             0459
         ST    @15,CCACPE3(,@10)                                   0459
*   CCACPE4=CCACPE3;                /* SET ADDRESS OF END OF RECORD  */
         ST    @15,CCACPE4(,@10)                                   0460
*   RCDSIZ=CCACPE4-CCACPE1;         /* LENGTH OF USED INFORMATION    */
         SLR   @15,@05                                             0461
         STH   @15,RCDSIZ(,@05)                                    0461
*/********************************************************************/
*/*            SET  DELETED  GROUP  OCCURRENCE  POINTER              */
*/********************************************************************/
*                                                                  0462
*   CCACPE5=ADDR(REPGOPT(1));       /* SET ADDR OF GROUP OCCUR PTRS  */
         LR    @10,@00                                             0462
         LA    @10,REPGOPT(,@10)                                   0462
         ST    @10,CCACPE5(,R11)                                   0462
*   CATGOCI=EXTCINO;                /* SET CI NO. OF EXTENSION RECRD */
         MVC   CATGOCI(3,@10),@CF00049+1                           0463
*   CATGOGC1=CATVOLGC;              /* VOLUME OCCURRENCE GROUP CODE  */
         MVI   CATGOGC1(@10),X'03'                                 0464
*   CATGOEXT=ON;                    /* GROUP OCCURRENCE OUTSIDE RECD */
*   CATGODEL=ON;                    /* SET DELETED OCCURRENCE FLAG   */
         OI    CATGOEXT(@10),B'11000000'                           0466
*   CATGOSEQ=GOSEQNUM;              /* SET GROUP OCCUR SEQ NUMBER    */
         MVI   CATGOSEQ(@10),X'01'                                 0467
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0468
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPCI15');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPCI15'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00008 DS    0H                                                  0471
@EF00008 DS    0H                                                  0471
@ER00008 BR    @14                                                 0471
@PB00008 DS    0H                                                  0472
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0473
*/********************************************************************/
*/*                                                                  */
*/*        IGGPCI15  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0473
*IGGPCI15:                                                         0473
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00009                                            0473
IGGPCI15 DS    0H                                                  0474
*Z038:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z038CI15:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0474
*    (R5,                                                          0474
*     R6,                                                          0474
*     R7,                                                          0474
*     R8,                                                          0474
*     R9,                                                          0474
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z038     DS    0H                                                  0474
Z038CI15 DS    0H                                                  0475
*   RFY                                                            0475
*    (R2,                                                          0475
*     R3,                                                          0475
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPCI15                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = CONSTRUCT CRA CI'S 1-5                        *
**                                                                    *
**   FUNCTION =                                                       *
**      CONSTRUCT CRA SELF-DESCRIBING RECORDS (CI'S 1-5) IN MAIN      *
**      STORAGE.  RAB 2 IS USED TO ADDRESS THE RECORDS.               *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      CRA SELF-DESCRIBING CI'S 1-5 IN DEFINE CRA WORKAREA           *
**                                                                    *
**********************************************************************/
*                                                                  0478
*   CCAURAB=ADDR(CCARAB2);          /* ADDRESS RECORD THROUGH RAB 2  */
         LA    @10,CCARAB2(,R11)                                   0478
         ST    @10,CCAURAB(,R11)                                   0478
*   DCRAR2PT=CCACPE1;               /* SAVE POINTER TO RECORD AREA 2 */
         MVC   DCRAR2PT(4,DCRAWAPR),CCACPE1(@10)                   0479
*   CCARBUR=OFF;                    /* RAB NOT IN-USE BY MODIFY      */
         NI    CCARBUR(@10),B'01111111'                            0480
*/********************************************************************/
*/*         CONSTRUCT CI 1  -  TYPE 'F': EMPTY FREE RECORD           */
*/********************************************************************/
*                                                                  0481
*   CCACPE1=DCRACIPT(2);            /* POINT TO CRA CI 1             */
         L     @05,DCRACIPT+4(,DCRAWAPR)                           0481
         ST    @05,CCACPE1(,@10)                                   0481
*   RCDCINO=FREECI1;                /* SET RECORD CI NUMBER          */
         MVC   RCDCINO(3,@05),@CF00046+1                           0482
*   RCDRELID=CURELID;               /* SET RELEASE INDICATOR         */
         MVI   RCDRELID(@05),X'01'                                 0483
*   RCDID=FREERCID;                 /* SET RECORD AS TYPE 'F'        */
         MVI   RCDID(@05),C'F'                                     0484
*/********************************************************************/
*/*         CONSTRUCT CI 2  -  TYPE 'C': BASE CLUSTER RECORD         */
*/********************************************************************/
*                                                                  0485
*   CCACPE1=DCRACIPT(3);            /* POINT TO CI 2                 */
         L     @05,DCRACIPT+8(,DCRAWAPR)                           0485
         ST    @05,CCACPE1(,@10)                                   0485
*   RCDCINO=CLUSTRCI;               /* SET RECORD CI NUMBER          */
         MVC   RCDCINO(3,@05),@CF00044+1                           0486
*   RCDRELID=CURELID;               /* SET RELEASE INDICATOR         */
         MVI   RCDRELID(@05),X'01'                                 0487
*   RCDRACTS=CMWASTOR(1:4);         /* SET CRA CREATION TIMESTAMP    */
         L     @10,CCACMSWA(,R11)                                  0488
         MVC   RCDRACTS(4,@05),CMWASTOR(@10)                       0488
*   RCDID=CATCLSID;                 /* SET RECORD AS TYPE 'C'        */
         MVI   RCDID(@05),C'C'                                     0489
*   NREPLF=LENGTH(CATRCD)           /* SET LENGTH OF FIXED FIELDS    */
*       +LENGTH(CNAME)+LENGTH(COWNID)+LENGTH(CCRE)+LENGTH(CEXP)+   0490
*       LNCATTR1;                                                  0490
         MVI   NREPLF(@05),X'6C'                                   0490
*   IF CCAF2CB=ON THEN              /* CALLER IS DEFINE CATALOG      */
         TM    CCAF2CB(R11),B'00000010'                            0491
         BNO   @RF00491                                            0491
*     CNAME=CTGFVENT->TNAM;         /* SET CATALOG NAME VIA FVT      */
         L     @10,CCACPL(,R11)                                    0492
         L     @10,CTGFVT(,@10)                                    0492
         L     @10,CTGFVENT(,@10)                                  0492
         MVC   CNAME(44,@05),TNAM(@10)                             0492
*   ELSE                            /* CALLER IS DEFINE SPACE        */
*     CNAME=CCAACB->ACBUAPTR->CAXCNAM;/* SET CATALOG NAME FROM     0493
*                                      CATALOG'S CAXWA               */
         B     @RC00491                                            0493
@RF00491 L     @10,CCAURAB(,R11)                                   0493
         L     @10,CCACPE1(,@10)                                   0493
         L     @05,CCAACB(,R11)                                    0493
         L     @05,ACBUAPTR(,@05)                                  0493
         MVC   CNAME(44,@10),CAXCNAM(@05)                          0493
*   COWNID=NULLDWRD;                /* SET OWNER ID TO NULL ID VALUE */
@RC00491 L     @10,CCAURAB(,R11)                                   0494
         L     @05,CCACPE1(,@10)                                   0494
         MVC   COWNID(8,@05),@CB04446                              0494
*   CCRE=R1;                        /* SET DATA-SET CREATION DATE    */
         STCM  R1,7,CCRE(@05)                                      0495
*   RFY                                                            0496
*     R1 UNRSTD;                    /* RELEASE CREATION DATE REGISTER*/
*   CEXP=NULLDATE;                  /* SET NULL EXPIRATION DATE      */
         MVC   CEXP(3,@05),@CB03440                                0497
*   CCACPE2=CCACPE1                 /* SET ADDR OF REPEATING FIELDS  */
*       +NREPLF;                    /* CONTROL INFORMATION           */
         SLR   @15,@15                                             0498
         IC    @15,NREPLF(,@05)                                    0498
         ALR   @15,@05                                             0498
         ST    @15,CCACPE2(,@10)                                   0498
*   REPCNT=CNTGOPTR;                /* COUNT OF GROUP OCCUR POINTERS */
         MVI   REPCNT(@15),X'02'                                   0499
*   CCACPE5=ADDR(REPGOPT(1));       /* SET ADDR OF GROUP OCCUR PTRS  */
         LA    @15,REPGOPT(,@15)                                   0500
         ST    @15,CCACPE5(,R11)                                   0500
*   CATGOGC1=ASGC;                  /* SET ASSOCIATION GROUP CODE (2)*/
         MVI   CATGOGC1(@15),X'02'                                 0501
*   CATGOSEQ=GOSEQNUM;              /* SET GROUP OCCUR SEQUENCE NO.  */
         MVI   CATGOSEQ(@15),X'01'                                 0502
*   CCACPE5=CCACPE5                 /* ADDRESS NEXT GROUP OCCUR PTR  */
*       +LENGTH(CATGOPT);                                          0503
         LA    @14,5                                               0503
         ALR   @15,@14                                             0503
         ST    @15,CCACPE5(,R11)                                   0503
*   CATGOGC1=PSWDGC;                /* SET PASSWORD GROUP CODE       */
         MVI   CATGOGC1(@15),X'04'                                 0504
*   CATGODEL=ON;                    /* SET DELETED OCCURRENCE FLAG   */
         OI    CATGODEL(@15),B'01000000'                           0505
*   CATGOSEQ=GOSEQNUM;              /* SET GROUP OCCUR SEQUENCE NO.  */
         MVI   CATGOSEQ(@15),X'01'                                 0506
*   CCACPE3=CCACPE5                 /* POINT TO ASSOCIATION OCCUR    */
*       +LENGTH(CATGOPT);                                          0507
         ALR   @15,@14                                             0507
         ST    @15,CCACPE3(,@10)                                   0507
*   DCRACI2A=CI2ASSOC;              /* SET ASSOCIATION FIELDS        */
         MVC   DCRACI2A(3,@15),@CF04431+1                          0508
*   RCDSIZ=CCACPE3+CNTRLENG         /* SET LENGTH OF USED INFORMATION*/
*       -CCACPE1;                                                  0509
*                                                                  0509
         LA    @09,6                                               0509
         ALR   @15,@09                                             0509
         SLR   @15,@05                                             0509
         STH   @15,RCDSIZ(,@05)                                    0509
*/********************************************************************/
*/*         CONSTRUCT CI 3  -  TYPE 'L': CRA CONTROL RECORD          */
*/********************************************************************/
*                                                                  0510
*   CCACPE1=DCRACIPT(4);            /* POINT TO CI 3                 */
         L     @05,DCRACIPT+12(,DCRAWAPR)                          0510
         ST    @05,CCACPE1(,@10)                                   0510
*   RCDCINO=CCRCINO;                /* SET RECORD CI NUMBER          */
         MVC   RCDCINO(3,@05),@CF00061+1                           0511
*   RCDRELID=CURELID;               /* SET RELEASE INDICATOR         */
         MVI   RCDRELID(@05),X'01'                                 0512
*   RCDRACTS=CMWASTOR(1:4);         /* SET CRA CREATION TIMESTAMP    */
         L     @15,CCACMSWA(,R11)                                  0513
         MVC   RCDRACTS(4,@05),CMWASTOR(@15)                       0513
*   RCDID=CCRRECID;                 /* SET RECORD AS TYPE 'L'        */
         MVI   RCDID(@05),C'L'                                     0514
*   CCRHACI=(DCRACHIA/DCRACISZ)-INCR1;/* NO. OF THE HIGH ALLOCATED CI*/
         AH    @05,@CH00138                                        0515
         L     @08,DCRACHIA(,DCRAWAPR)                             0515
         SRL   @08,9                                               0515
         BCTR  @08,0                                               0515
         STCM  @08,7,CCRHACI(@05)                                  0515
*   CCRNFCI=DCRACHIU/DCRACISZ;      /* NO. OF THE NEXT FREE CI       */
         L     @08,DCRACHIU(,DCRAWAPR)                             0516
         SRL   @08,9                                               0516
         STCM  @08,7,CCRNFCI(@05)                                  0516
*/********************************************************************/
*/*         CONSTRUCT CI 4  -  TYPE 'F': EMPTY FREE RECORD           */
*/********************************************************************/
*                                                                  0517
*   CCACPE1=DCRACIPT(5);            /* POINT TO CI 4                 */
         L     @05,DCRACIPT+16(,DCRAWAPR)                          0517
         ST    @05,CCACPE1(,@10)                                   0517
*   RCDCINO=FREECI4;                /* SET RECORD CI NUMBER          */
         MVC   RCDCINO(3,@05),@CF00126+1                           0518
*   RCDRELID=CURELID;               /* SET RELEASE INDICATOR         */
         MVI   RCDRELID(@05),X'01'                                 0519
*   RCDID=FREERCID;                 /* SET RECORD AS TYPE 'F'        */
         MVI   RCDID(@05),C'F'                                     0520
*/********************************************************************/
*/*         CONSTRUCT CI 5  -  TYPE 'E': DATA EXTENSION RECORD       */
*/********************************************************************/
*                                                                  0521
*   CCACPE1=DCRACIPT(6);            /* POINT TO EXTENSION RECORD CI 5*/
         L     @05,DCRACIPT+20(,DCRAWAPR)                          0521
         ST    @05,CCACPE1(,@10)                                   0521
*   RCDCINO=EXTCINO;                /* SET RECORD CI NUMBER          */
         STCM  @14,7,RCDCINO(@05)                                  0522
*   RCDRELID=CURELID;               /* SET RELEASE INDICATOR         */
         MVI   RCDRELID(@05),X'01'                                 0523
*   RCDRACTS=CMWASTOR(1:4);         /* SET CRA CREATION TIMESTAMP    */
         MVC   RCDRACTS(4,@05),CMWASTOR(@15)                       0524
*   RCDID=CATEXTID;                 /* SET RECORD AS TYPE 'E'        */
         MVI   RCDID(@05),C'E'                                     0525
*   NREPLF=LENGTH(CATRCD);          /* SET LENGTH OF FIXED FIELDS    */
         LA    @08,49                                              0526
         STC   @08,NREPLF(,@05)                                    0526
*   CCACPE2=CCACPE1                 /* SET ADDRESS OF REPEATING      */
*       +NREPLF;                    /* FIELDS CONTROL INFORMATION    */
         ALR   @08,@05                                             0527
         ST    @08,CCACPE2(,@10)                                   0527
*   CCACPE3=CCACPE2                 /* SET POINTER TO FIRST GROUP    */
*       +LENGTH(REPCNTRL);          /* OCCURRENCE                    */
         ALR   @08,@09                                             0528
         ST    @08,CCACPE3(,@10)                                   0528
*   CCACPE4=CCACPE3;                /* POINT TO END OF RECORD        */
         ST    @08,CCACPE4(,@10)                                   0529
*   RCDSIZ=CCACPE4-CCACPE1;         /* SET SIZE OF RECORD INFORMATION*/
         SLR   @08,@05                                             0530
         STH   @08,RCDSIZ(,@05)                                    0530
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0531
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPCACB');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPCACB'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00009 DS    0H                                                  0534
@EF00009 DS    0H                                                  0534
@ER00009 BR    @14                                                 0534
@PB00009 DS    0H                                                  0535
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0536
*/********************************************************************/
*/*                                                                  */
*/*        IGGPCACB  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0536
*IGGPCACB:                                                         0536
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00010                                            0536
IGGPCACB DS    0H                                                  0537
*Z039:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z039CACB:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0537
*    (R5,                                                          0537
*     R6,                                                          0537
*     R7,                                                          0537
*     R8,                                                          0537
*     R9,                                                          0537
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z039     DS    0H                                                  0537
Z039CACB DS    0H                                                  0538
*   RFY                                                            0538
*    (R2,                                                          0538
*     R3,                                                          0538
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPCACB                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = CONSTRUCT ACB                                 *
**                                                                    *
**   FUNCTION =                                                       *
**      CONSTRUCT AN ACB IN ORDER TO OPEN THE CRA ESDS                *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      RECORD AREA 1 CONTAINS THE ACB                                *
**                                                                    *
**********************************************************************/
*                                                                  0541
*   RFY                                                            0541
*     IFGACB BASED(CCARAACB);       /* MAP THE ACB TO BUILD          */
*   RFY                                                            0542
*     CATRECRD BASED(CCARAACB);     /* DUMMY MAPPING TO CLEAR REC    */
*   CCARAACB=DCRAR1PT;              /* USE RECORD AREA 1             */
         L     @10,DCRAR1PT(,DCRAWAPR)                             0543
         ST    @10,CCARAACB(,R11)                                  0543
*   CATRECRD(1)=CATRECRD(1)&&CATRECRD(1);                          0544
         XC    CATRECRD(256,@10),CATRECRD(@10)                     0544
*   CATRECRD(2)=CATRECRD(2)&&CATRECRD(2);/* CLEAR ACB/RPL/CAXWA BUILD
*                                      AREA                          */
         XC    CATRECRD+256(256,@10),CATRECRD+256(@10)             0545
*/********************************************************************/
*/*           INITIALIZE  THE  ACB  TO  OPEN  THE  CRA               */
*/********************************************************************/
*                                                                  0546
*   ACBID='A0'X;                    /* SET ACB ID                    */
         MVI   ACBID(@10),X'A0'                                    0546
*   ACBLENG=LENGTH(IFGACB);         /* SET THE ACB LENGTH            */
         MVC   ACBLENG(2,@10),@CH00451                             0547
*   ACBCNV=ON;                      /* CONTROL INTERVAL PROCESSING   */
*   ACBDIR=ON;                      /* DIRECT PROCESSING             */
*   ACBOUT=ON;                      /* OUTPUT PROCESSING             */
*   ACBUBF=ON;                      /* USER SUPPLIED BUFFER SPACE    */
         OI    ACBCNV(@10),B'00101011'                             0551
*   ACBSTRNO=STRINGNO;              /* SET NUMBER OF STRINGS         */
         MVI   ACBSTRNO(@10),X'01'                                 0552
*   ACBBUFND=DATBUFNO;              /* SET NUMBER OF DATA BUFFERS    */
         MVC   ACBBUFND(2,@10),@CH00044                            0553
*   ACBDORGA=ON;                    /* SET JES ORIGIN                */
         OI    ACBDORGA(@10),B'00001000'                           0554
*   IF CCAF2CB=ON THEN              /* CALLER IS DEFINE CATALOG      */
         TM    CCAF2CB(R11),B'00000010'                            0555
         BNO   @RF00555                                            0555
*     ACBDDNM=CTGFVVCH              /* SET DD NAME IN ACB FROM       */
*         ->CTGFVIND->DCRADDNM;     /* CATALOG'S SPACE FVT           */
         L     @05,CCACPL(,R11)                                    0556
         L     @05,CTGFVT(,@05)                                    0556
         L     @05,CTGFVVCH(,@05)                                  0556
         L     @05,CTGFVIND(,@05)                                  0556
         MVC   ACBDDNM(8,@10),DCRADDNM(@05)                        0556
*   ELSE                            /* CALLER IS DEFINE SPACE        */
*     ACBDDNM=CTGFVIND              /* SET DD NAME IN ACB FROM       */
*         ->DCRADDNM;               /* JCL STATEMENT NAME ADDRESS    */
         B     @RC00555                                            0557
@RF00555 L     @10,CCARAACB(,R11)                                  0557
         L     @05,CCACPL(,R11)                                    0557
         L     @05,CTGFVT(,@05)                                    0557
         L     @05,CTGFVIND(,@05)                                  0557
         MVC   ACBDDNM(8,@10),DCRADDNM(@05)                        0557
*   ACBEXFG=ON;                     /* SET ACB BUSY FLAG             */
@RC00555 L     @10,CCARAACB(,R11)                                  0558
         OI    ACBEXFG(@10),B'00000010'                            0558
*   ACBSCRA=ON;                     /* SET SYSTEM CRA BUILD FLAG     */
         OI    ACBSCRA(@10),B'00001000'                            0559
*   ACBUAPTR=CCARAACB               /* SET CAXWA POINTER IN THE ACB  */
*       +LENGTH(IFGACB);                                           0560
         LA    @05,76                                              0560
         ALR   @05,@10                                             0560
         ST    @05,ACBUAPTR(,@10)                                  0560
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0561
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPCXWA');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPCXWA'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00010 DS    0H                                                  0564
@EF00010 DS    0H                                                  0564
@ER00010 BR    @14                                                 0564
@PB00010 DS    0H                                                  0565
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0566
*/********************************************************************/
*/*                                                                  */
*/*        IGGPCXWA  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0566
*IGGPCXWA:                                                         0566
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00011                                            0566
IGGPCXWA DS    0H                                                  0567
*Z040:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z040CXWA:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0567
*    (R5,                                                          0567
*     R6,                                                          0567
*     R7,                                                          0567
*     R8,                                                          0567
*     R9,                                                          0567
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z040     DS    0H                                                  0567
Z040CXWA DS    0H                                                  0568
*   RFY                                                            0568
*    (R2,                                                          0568
*     R3,                                                          0568
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPCXWA                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = CONSTRUCT CAXWA                               *
**                                                                    *
**   FUNCTION =                                                       *
**      CONSTRUCT A CAXWA IN ORDER TO OPEN THE CRA ESDS               *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**      VOLUME UCB ADDRESS IN CMSWA(CMSDFSPC)                         *
**      TIME STAMP IN CMSWA(CMWASTOR)                                 *
**                                                                    *
**   OUTPUT =                                                         *
**      RECORD AREA 1 CONSTAINS THE CAXWA                             *
**                                                                    *
**********************************************************************/
*                                                                  0571
*   CAXID=CAXCD;                    /* SET CAXWA ID                  */
         L     @10,CCARAACB(,R11)                                  0571
         L     @10,ACBUAPTR(,@10)                                  0571
         MVI   CAXID(@10),X'CA'                                    0571
*   CAXBLD=ON;                      /* BUILD REQUEST                 */
         OI    CAXBLD(@10),B'10000000'                             0572
*   IF CTGTYPE=CTGTMCAT THEN        /* IF MASTER CATALOG BUILD,      */
         L     @05,CCACPL(,R11)                                    0573
         CLI   CTGTYPE(@05),C'M'                                   0573
         BNE   @RF00573                                            0573
*     CAXMCT=ON;                    /* SET MASTER CATALOG BUILD FLAG */
         OI    CAXMCT(@10),B'00000100'                             0574
*   ELSE                                                           0575
*     ;                             /* NOT MASTER CATALOG BUILD      */
@RF00573 DS    0H                                                  0576
*   CAXCMR=ON;                      /* CATALOG MANAGEMENT INVOKED    */
         L     @10,CCARAACB(,R11)                                  0576
         L     @05,ACBUAPTR(,@10)                                  0576
         OI    CAXCMR(@05),B'00000010'                             0576
*   CAXACT=ACTCOUNT;                /* SET CATALOG USE COUNT         */
         MVI   CAXACT(@05),X'01'                                   0577
*   CAXATIOT=TCBTIO;                /* SET ADDRESS OF TASK I/O TABLE */
         L     @01,CVTPTR                                          0578
         L     @01,CVTTCBP(,@01)                                   0578
         L     @01,TCBPTR(,@01)                                    0578
         MVC   CAXATIOT(4,@05),TCBTIO(@01)                         0578
*   CAXACB=CCAACB;                  /* SET ADDRESS OF CATALOG'S ACB  */
         MVC   CAXACB(4,@05),CCAACB(R11)                           0579
*   CAXUCB=CMSDFSPC;                /* SET VOLUME UCB ADDRESS FROM 0580
*                                      VALUE IN CMSWA SET UP BY    0580
*                                      DEFINE SPACE (IGG0CLAQ)       */
         L     @15,CCACMSWA(,R11)                                  0580
         L     @14,CMSDFSPC(,@15)                                  0580
         ST    @14,CAXUCB(,@05)                                    0580
*   CAXVOLID=UCBVOLI;               /* SET VOLUME SERIAL NUMBER      */
         MVC   CAXVOLID(6,@05),UCBVOLI(@14)                        0581
*   CAXRACTS=CMWASTOR(1:4);         /* SET CRA CREATION TIMESTAMP    */
         MVC   CAXRACTS(4,@05),CMWASTOR(@15)                       0582
*   CENLST=ON;                      /* SET OPEN END-OF-LIST FLAG     */
         OI    CENLST(@05),B'10000000'                             0583
*   COPACB=CCARAACB;                /* SET ADDRESS OF CRA'S ACB      */
         STCM  @10,7,COPACB(@05)                                   0584
*   CAXCCA=R11;                     /* SET ADDRESS OF THE CCA        */
         ST    R11,CAXCCA(,@05)                                    0585
*   CAXCRACB=CCARAACB;              /* SET ADDRESS OF CRA'S ACB      */
         ST    @10,CAXCRACB(,@05)                                  0586
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0587
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPOCRA');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPOCRA'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00011 DS    0H                                                  0590
@EF00011 DS    0H                                                  0590
@ER00011 BR    @14                                                 0590
@PB00011 DS    0H                                                  0591
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0592
*/********************************************************************/
*/*                                                                  */
*/*        IGGPOCRA  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0592
*IGGPOCRA:                                                         0592
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00012                                            0592
IGGPOCRA DS    0H                                                  0593
*Z041:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z041OCRA:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0593
*    (R5,                                                          0593
*     R6,                                                          0593
*     R7,                                                          0593
*     R8,                                                          0593
*     R9,                                                          0593
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z041     DS    0H                                                  0593
Z041OCRA DS    0H                                                  0594
*   RFY                                                            0594
*    (R2,                                                          0594
*     R3,                                                          0594
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPOCRA                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = OPEN THE CATALOG RECOVERY AREA                *
**                                                                    *
**   FUNCTION =                                                       *
**      ISSUE 'OPEN' MACRO FOR THE CRA AND UPDATE FIELDS IN THE       *
**      VOLUME OCCURRENCE IN CRA CI 5 ('E' RECORD)                    *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      IF THE CRA CANNOT BE OPENED, 'CCAPROB' CONTAINS PROBLEM       *
**         DETERMINATION INFORMATION.                                 *
**                                                                    *
**********************************************************************/
*                                                                  0597
*   RFY                                                            0597
*     R1 RSTD;                      /* RESTRICT REG 1 FOR OPEN       */
*   R1=ADDR(CAXOPLST);              /* GET ADDRESS OF OPEN PARM LIST */
         L     @10,CCARAACB(,R11)                                  0598
         L     R1,ACBUAPTR(,@10)                                   0598
         LA    R1,CAXOPLST(,R1)                                    0598
*   GEN(OPEN  ,MF=(E,(1)));         /* OPEN THE CRA                  */
         OPEN  ,MF=(E,(1))
*   RFY                                                            0600
*     R1 UNRSTD;                    /* UNRESTRICT USE OF REG 1       */
*   DCRAWAPR=CCACRACI;              /* RESTORE DEF CRA WORKAREA ADDR */
         L     DCRAWAPR,CCACRACI(,R11)                             0601
*   IF ACBOPEN=OFF THEN             /* CHECK FOR SUCCESSFUL OPEN     */
         L     @10,CCARAACB(,R11)                                  0602
         TM    ACBOPEN(@10),B'00010000'                            0602
         BNZ   @RF00602                                            0602
*     DO;                           /* IF BAD OPEN, SET ERROR CODE   */
*                                                                  0603
*       /*************************************************************/
*       /*                                                           */
*       /* SET CCAMODID='B4', CCAREASN=RSN12, CCARETRN=RCCAT         */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0604
*       CCAPROB=IGGSEGID+(RSN12*K256)+RCCAT;/* ECD: CRA OPEN FAULT.
*                                      RCD: CANNOT OPEN CRA FOR BUILD*/
         MVC   CCAPROB(4,R11),@CF04649                             0604
*       CALL EXIT;                  /* RETURN TO IGGPDCRA            */
         BAL   @14,EXIT                                            0605
*     END;                          /* END 'DO' FOR ERROR CONDITION  */
*   ELSE                                                           0607
*     ;                             /* CRA ACB SUCCESSFULLY OPENED   */
@RF00602 DS    0H                                                  0608
*   DCRACIPT(1)->CHIU=DCRACHIU;     /* SET HI-USED RBA IN BASE DATA
*                                      RECORD (CI 0) FOLLOWING     0608
*                                      INITIAL CRA OPEN              */
         L     @10,DCRACIPT(,DCRAWAPR)                             0608
         L     @05,DCRACHIU(,DCRAWAPR)                             0608
         ST    @05,CHIU(,@10)                                      0608
*   RFY                                                            0609
*     GVO BASED(ADDR(DCRA5VGO));    /* MAP VOLUME OCCURRENCE FIELDS
*                                      IN CI 5 ('E' RECORD)          */
*   CMFLHUBA=DCRACHIU;              /* SET HIGH-USED RBA OF CRA IN 0610
*                                      VOLUME OCCURRENCE FOLLOWING 0610
*                                      INITIAL CRA OPEN              */
         LA    @10,3022                                            0610
         ALR   @10,DCRAWAPR                                        0610
         ST    @05,CMFLHUBA(,@10)                                  0610
*   CMFLNOPF=OFF;                   /* TURN OFF PREFORMATTED-SPACE 0611
*                                      INDICATOR                     */
         NI    CMFLNOPF(@10),B'10111111'                           0611
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0612
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPCRPL');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPCRPL'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00012 DS    0H                                                  0615
@EF00012 DS    0H                                                  0615
@ER00012 BR    @14                                                 0615
@PB00012 DS    0H                                                  0616
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0617
*/********************************************************************/
*/*                                                                  */
*/*        IGGPCRPL  -  PROCEDURE  INTERNALLY  CALLED  NESTED        */
*/*                                                                  */
*/********************************************************************/
*                                                                  0617
*IGGPCRPL:                                                         0617
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00013                                            0617
IGGPCRPL DS    0H                                                  0618
*Z043:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z043CRPL:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0618
*    (R5,                                                          0618
*     R6,                                                          0618
*     R7,                                                          0618
*     R8,                                                          0618
*     R9,                                                          0618
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z043     DS    0H                                                  0618
Z043CRPL DS    0H                                                  0619
*   RFY                                                            0619
*    (R2,                                                          0619
*     R3,                                                          0619
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*   GEN(STM 12,14,12(13));          /* SAVE REGISTERS OF CALLER      */
         STM 12,14,12(13)
*   GEN(LA 13,12(13));              /* POINT TO NEXT SAVE AREA       */
         LA 13,12(13)
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPCRPL                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, NESTED                       *
**                                                                    *
**   DESCRIPTIVE NAME = CONSTRUCT REQUEST PARAMETER LIST (RPL)        *
**                                                                    *
**   FUNCTION =                                                       *
**      CONSTRUCT AN RPL FOR THE CRA FOR USE BY THE I/O ROUTINE       *
**      IGGPXRIO(IGG0CLBO) WHEN WRITING THE CRA SELF-DESCRIBING       *
**      AND VOLUME RECORDS FROM MAIN STORAGE INTO THE CRA.            *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      RECORD AREA 1 CONTAINS THE RPL                                *
**                                                                    *
**********************************************************************/
*                                                                  0622
*   CCARARPL=CCARAACB               /* SET CRA'S RPL ADDRESS FOR I/O */
*       +LENGTH(IFGACB)+LENGTH(IGGCAXWA);                          0622
         L     @10,CCARAACB(,R11)                                  0622
         LA    @05,208                                             0622
         ALR   @05,@10                                             0622
         ST    @05,CCARARPL(,R11)                                  0622
*   CCARPLX=XIOPTNS;                /* SET I/O OPTION CODES FOR    0623
*                                      PUT-SEQ, USER RECORD AREA,  0623
*                                      ADDRESSED ACCESS, NEW RECORD,
*                                      CHECK I/O, NORMAL CI ENTRY    */
         MVI   CCARPLX(R11),X'09'                                  0623
*   RFY                                                            0624
*     IFGRPL BASED(CCARARPL);       /* MAP THE RPL TO BUILD          */
*   RPLREQ=RPLPUT;                  /* SET 'PUT' REQUEST TYPE        */
         MVI   RPLREQ(@05),X'01'                                   0625
*   RPLLEN=LENGTH(IFGRPL);          /* SET LENGTH OF RPL             */
         MVI   RPLLEN(@05),X'4C'                                   0626
*   RPLDACB=CCARAACB;               /* SET ADDRESS OF CRA'S ACB      */
         ST    @10,RPLDACB(,@05)                                   0627
*   RPLTCBPT=TCBPTR;                /* SET ADDRESS OF TCB            */
         L     @10,CVTPTR                                          0628
         L     @10,CVTTCBP(,@10)                                   0628
         MVC   RPLTCBPT(4,@05),TCBPTR(@10)                         0628
*   RPLARG=ADDR(CCARARBA);          /* SET SEARCH ARGUMENT POINTER   */
         LA    @10,CCARARBA(,R11)                                  0629
         ST    @10,RPLARG(,@05)                                    0629
*   RPLDIR=ON;                      /* SET DIRECT ACCESS FLAG        */
         OI    RPLDIR(@05),B'01000000'                             0630
*   RPLCNV=ON;                      /* CONTROL INTERVAL ACCESS       */
         OI    RPLCNV(@05),B'00100000'                             0631
*   RPLRLEN=DCRACISZ;               /* SET LENGTH OF RECORD          */
         MVC   RPLRLEN(4,@05),@CF00056                             0632
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0633
*   GEN(DS 0F);                     /* FULLWORD BOUNDARY ALIGNMENT   */
         DS 0F
*   GEN(DC CL8'IGGPDCXT');          /* NAME OF FOLLOWING PROCEDURE   */
         DC CL8'IGGPDCXT'
*   END;                            /* END OF INNER PROCEDURE        */
@EL00013 DS    0H                                                  0636
@EF00013 DS    0H                                                  0636
@ER00013 BR    @14                                                 0636
@PB00013 DS    0H                                                  0637
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0638
*/********************************************************************/
*/*                                                                  */
*/*        IGGPDCXT  -  PROCEDURE  INTERNALLY  CALLED  UNNESTED      */
*/*                                                                  */
*/********************************************************************/
*                                                                  0638
*IGGPDCXT:                                                         0638
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00014                                            0638
IGGPDCXT DS    0H                                                  0639
*Z044:                              /* LABEL FOR PROCEDURE TRACKING  */
*Z044DCXT:                          /* LABEL FOR PROCEDURE TRACKING  */
*   RFY                                                            0639
*    (R5,                                                          0639
*     R6,                                                          0639
*     R7,                                                          0639
*     R8,                                                          0639
*     R9,                                                          0639
*     R10) UNRSTD;                  /* PL/S COMPILER WORK REGISTERS  */
Z044     DS    0H                                                  0639
Z044DCXT DS    0H                                                  0640
*   RFY                                                            0640
*    (R2,                                                          0640
*     R3,                                                          0640
*     R4) RSTD;                     /* RESTRICT LOCAL WORK REGISTERS */
*/*********************************************************************
**                                                                    *
**   PROCEDURE NAME = IGGPDCXT                                        *
**                                                                    *
**   PROCEDURE TYPE = INTERNALLY CALLED, UNNESTED                     *
**                                                                    *
**   DESCRIPTIVE NAME = DEFINE CRA CLEANUP PROCESSING                 *
**                                                                    *
**   FUNCTION =                                                       *
**      PERFORM CLEANUP PROCESSING IN THE EVENT AN ERROR IS           *
**      ENCOUNTERED OR AFTER ALL DEFINE CRA PROCESSING IS COMPLETED,  *
**      FREEING ALL ACQUIRED MAIN STORAGE IF NECESSARY AND CLOSING    *
**      THE CRA ACB IF IT WAS SUCCESSFULLY OPENED.                    *
**                                                                    *
**   INPUT =                                                          *
**      ADDRESS OF CCA IN REGISTER 11                                 *
**      ADDRESS OF DEFINE CRA WORKAREA IN DCRAWAPR (REGISTER 3)       *
**                                                                    *
**   OUTPUT =                                                         *
**      IF THE CRA CANNOT BE CLOSED, 'CCAPROB' CONTAINS PROBLEM       *
**         DETERMINATION INFORMATION.                                 *
**      SEVERAL FIELDS AND FLAGS IN THE CCA RELATED TO DEFINE CRA     *
**         PROCESSING ARE CLEARED.                                    *
**                                                                    *
**********************************************************************/
*                                                                  0641
*   CCACRABU=OFF;                   /* TURN OFF CRA BUILD BIT IN CCA */
         NI    CCACRABU(R11),B'11111110'                           0641
*   CCADSRCL=OFF;                   /* TURN OFF RECURSIVE CALL BIT   */
*   CCAVBUFI=OFF;                   /* TURN OFF VOLUME BUFFER FLAG   */
         NI    CCADSRCL(R11),B'00111111'                           0643
*   IF DCRAWAPR^=BLANK THEN         /* CHECK IF WORK SPACE ACQUIRED  */
         SLR   @10,@10                                             0644
         CR    DCRAWAPR,@10                                        0644
         BE    @RF00644                                            0644
*     DO;                           /* YES, MAIN STG WORKAREA EXISTS */
*       CCADESA=DCRADESA;           /* RESTORE DADSM EXTENT INFO     */
         MVC   CCADESA(44,R11),DCRADESA(DCRAWAPR)                  0646
*       IF DCRAR1PT^=BLANK THEN     /* RA 1 POINTER WAS SAVED        */
         L     @05,DCRAR1PT(,DCRAWAPR)                             0647
         CR    @05,@10                                             0647
         BE    @RF00647                                            0647
*         CCAR1REC=DCRAR1PT;        /* RESTORE RA 1 PTR IN RAB 1     */
         ST    @05,CCAR1REC(,R11)                                  0648
*       ELSE                                                       0649
*         ;                         /* RECORD AREA 1 PTR NOT SAVED   */
@RF00647 DS    0H                                                  0650
*       CCAMNLEN=DCRAMNLN;          /* SET CORE LENGTH FOR FREEMAIN  */
         MVC   CCAMNLEN(3,R11),DCRAMNLN+1(DCRAWAPR)                0650
*       CCAMNADR=DCRAWAPR;          /* SET CORE ADDRESS FOR FREEMAIN */
         ST    DCRAWAPR,CCAMNADR(,R11)                             0651
*       RFY                                                        0652
*         R1 RSTD;                  /* RESTRICT R1 FOR FREEMAIN MACRO*/
*       R1=ADDR(CCAMSSPL);          /* POINT TO STRG MGMT P.L.       */
         LA    R1,CCAMSSPL(,R11)                                   0653
*       CCAMNSPL=USUBPOOL;          /* STORE SUBPOOL VALUE           */
         MVI   CCAMNSPL(R11),X'FC'                                 0654
*       DO;                                                        0655
*         IF MNATFULL=ON THEN       /* TRACKING TABLE FULL           */
*                                                                  0656
         TM    MNATFULL(R11),B'10000000'                           0656
         BNO   @RF00656                                            0656
*           /*********************************************************/
*           /*                                                       */
*           /* CALL TO MODULE IGG0CLAT                               */
*           /*                                                       */
*           /*********************************************************/
*                                                                  0657
*Z047ARFS:                                                         0657
*           CALL IGGPARFS;          /* YES,CALL SPILL ROUTINE        */
Z047ARFS L     @15,@CV04451                                        0657
         BALR  @14,@15                                             0657
*         GEN REFS(CCAMNAT)(MVC CCAMNAT(248,11),CCAMNAT+8(11));/*  0658
*                                      PUSH AND TRACK                */
@RF00656 DS    0H                                                  0658
         MVC CCAMNAT(248,11),CCAMNAT+8(11)
*       END;                                                       0659
*                                                                  0659
*       /*************************************************************/
*       /*                                                           */
*       /* RELEASE MAIN STORAGE FROM USER SUBPOOL 252                */
*       /*                                                           */
*       /*************************************************************/
*                                                                  0660
*       GEN(FREEMAIN L,MF=(E,(1))) ;/* FREE CRA WORK SPACE           */
         FREEMAIN L,MF=(E,(1))
*       RFY                                                        0661
*         R1 UNRSTD;                /* UNRESTRICT R1'S USE           */
*     END;                          /* END MAIN STORAGE PROCESSING   */
*   ELSE                                                           0663
*     ;                             /* NO WORKAREA WAS ACQUIRED      */
@RF00644 DS    0H                                                  0664
*   IF(CCARAACB^=BLANK)&            /* IF CRA ACB WAS CONSTRUCTED    */
*       (ACBOPEN=ON) THEN           /* AND THE ACB IS OPEN           */
         L     @10,CCARAACB(,R11)                                  0664
         LTR   @10,@10                                             0664
         BZ    @RF00664                                            0664
         TM    ACBOPEN(@10),B'00010000'                            0664
         BNO   @RF00664                                            0664
*     DO;                           /* CLOSE CRA ACB PROCESSING      */
*       RFY                                                        0666
*         R1 RSTD;                  /* RESTRICT REG 1 FOR CLOSE      */
*       R1=ADDR(CAXOPLST);          /* GET ADDR OF CLOSE PARM LIST   */
         L     R1,ACBUAPTR(,@10)                                   0667
         LA    R1,CAXOPLST(,R1)                                    0667
*       GEN(CLOSE  ,MF=(E,(1)));    /* CLOSE THE CRA                 */
         CLOSE  ,MF=(E,(1))
*       RFY                                                        0669
*         R1 UNRSTD;                /* UNRESTRICT USE OF REG 1       */
*       IF(ACBOPEN=ON)|             /* IF CRA STILL OPEN OR          */
*           (ACBERFLG^=RCS) THEN    /* UNSUCCESSFULLY CLOSED         */
*                                                                  0670
         L     @10,CCARAACB(,R11)                                  0670
         TM    ACBOPEN(@10),B'00010000'                            0670
         BO    @RT00670                                            0670
         CLI   ACBERFLG(@10),0                                     0670
         BE    @RF00670                                            0670
@RT00670 DS    0H                                                  0671
*         /***********************************************************/
*         /*                                                         */
*         /* SET CCAMODID='B4', CCAREASN=RSN14, CCARETRN=RCCAT       */
*         /*                                                         */
*         /***********************************************************/
*                                                                  0671
*         CCAPROB=IGGSEGID+(RSN14*K256)+RCCAT;/* ECD: CRA OPEN FAULT.
*                                      RCD: CANNOT CLOSE CRA AFTER 0671
*                                      BUILD.                        */
         MVC   CCAPROB(4,R11),@CF04653                             0671
*       ELSE                                                       0672
*         ;                         /* CRA SUCCESSFULLY CLOSED       */
@RF00670 DS    0H                                                  0673
*     END;                          /* END 'DO' FOR CRA ACB CLOSE    */
*   ELSE                                                           0674
*     ;                             /* CRA ACB IMPROPERLY PROCESSED  */
@RF00664 DS    0H                                                  0675
*   CCARAACB=BLANK;                 /* CLEAR CRA ACB ADDRESS         */
         SLR   @10,@10                                             0675
         ST    @10,CCARAACB(,R11)                                  0675
*   CCARARPL=BLANK;                 /* CLEAR CRA RPL ADDRESS         */
         ST    @10,CCARARPL(,R11)                                  0676
*   CALL EXIT;                      /* RETURN TO CALLER              */
         BAL   @14,EXIT                                            0677
*   END;                            /* END OF INNER PROCEDURE        */
@EL00014 DS    0H                                                  0678
@EF00014 DS    0H                                                  0678
@ER00014 BR    @14                                                 0678
@PB00014 DS    0H                                                  0679
*   GEN(EJECT);                     /* GO TO A NEW PRINT PAGE        */
         EJECT
*                                                                  0680
*/********************************************************************/
*/*                                                                  */
*/*         RETURN-TO-CALLER  PROCEDURE-EXITING  SUBFUNCTION         */
*/*                                                                  */
*/********************************************************************/
*                                                                  0680
*EXIT:                                                             0680
*   PROC OPTIONS(NOSAVEAREA,NOSAVE);/* SPECIFY PROCEDURE OPTIONS     */
         B     @PB00015                                            0680
EXIT     DS    0H                                                  0681
*   GEN(LM 12,14,0(13));            /* RESTORE REGISTERS OF CALLER   */
         LM 12,14,0(13)
*   GEN(MVI 12(13),255);            /* MARK END OF PDL               */
         MVI 12(13),255
*   GEN(BR 14);                     /* RETURN TO CALLER              */
         BR 14
*PATCH:                                                            0684
*   GEN(DC (12+((*+139-@MAINENT)/140))A(0));/* PATCH AREA            */
PATCH    DC (12+((*+139-@MAINENT)/140))A(0)
*   END;                            /* END OF EXIT PROCEDURE         */
@EL00015 DS    0H                                                  0685
@EF00015 DS    0H                                                  0685
@ER00015 BR    @14                                                 0685
*   END                             /* END MODULE IGG0CLB4           */
*                                                                  0686
*/* THE FOLLOWING INCLUDE STATEMENTS WERE FOUND IN THIS PROGRAM.     */
*/*%INCLUDE SYSLIB  (IGGMUPDE)                                       */
*/*%INCLUDE SYSLIB  (IGGMVEDC)                                       */
*/*%INCLUDE SYSLIB  (IGGMCDCL)                                       */
*/*%INCLUDE SYSLIB  (IGGMCTRC)                                       */
*/*%INCLUDE SYSLIB  (IGGCCA  )                                       */
*/*%INCLUDE SYSLIB  (IEZCTGPL)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGFL)                                       */
*/*%INCLUDE SYSLIB  (CVT     )                                       */
*/*%INCLUDE SYSLIB  (IKJTCB  )                                       */
*/*%INCLUDE SYSLIB  (IHAPSA  )                                       */
*/*%INCLUDE SYSLIB  (IFGACB  )                                       */
*/*%INCLUDE SYSLIB  (AMCBS   )                                       */
*/*%INCLUDE SYSLIB  (IGGCAXWA)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGFV)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGWA)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGVL)                                       */
*/*%INCLUDE SYSLIB  (IEZCTGCV)                                       */
*/*%INCLUDE SYSLIB  (ICHACHKL)                                       */
*/*%INCLUDE SYSLIB  (ICHRDDFL)                                       */
*/*%INCLUDE SYSLIB  (IGGMCMDM)                                       */
*/*%INCLUDE SYSLIB  (IDAAMDSB)                                       */
*/*%INCLUDE SYSLIB  (IGGMSAWA)                                       */
*/*%INCLUDE SYSLIB  (IGGMCMWA)                                       */
*/*%INCLUDE SYSLIB  (IGGMGVO )                                       */
*/*%INCLUDE SYSLIB  (IDAVVOLL)                                       */
*/*%INCLUDE SYSLIB  (IFGRPL  )                                       */
*/*%INCLUDE SYSLIB  (IGGMF4WA)                                       */
*/*%INCLUDE SYSLIB  (IECSDSL1)                                       */
*/*%INCLUDE SYSLIB  (IEFUCBOB)                                       */
*/*%INCLUDE SYSLIB  (IDAAMBL )                                       */
*/*%INCLUDE SYSLIB  (IDAAMB  )                                       */
*/*%INCLUDE SYSLIB  (IGGMDCRA)                                       */
*                                                                  0686
*       ;                                                          0686
@EL00001 DS    0H                                                  0686
@EF00001 DS    0H                                                  0686
@ER00001 BR    @14                                                 0686
@DATA    DS    0H
@CH00087 DC    H'6'
@CH00041 DC    H'8'
@CH00138 DC    H'44'
@CH00451 DC    H'76'
@CH04638 DC    H'532'
@DATD    DSECT
         DS    0F
IGG0CLB4 CSECT
         DS    0F
@CF00046 DC    F'1'
@CH00046 EQU   @CF00046+2
@CF00044 DC    F'2'
@CH00044 EQU   @CF00044+2
@CF00061 DC    F'3'
@CF00126 DC    F'4'
@CF00049 DC    F'5'
@CF04405 DC    F'505'
@CF00056 DC    F'512'
@CF04384 DC    F'2048'
@CF04408 DC    F'5120'
@CF04640 DC    F'-1024195932'
@CF04649 DC    F'-1024193532'
@CF04653 DC    F'-1024193020'
@CF04659 DC    F'-8'
@CF04660 DC    F'-4'
@CF04431 DC    XL4'0006C4'
@DATD    DSECT
         DS    0D
         ORG   *+1-(*-@DATD)/(*-@DATD) INSURE DSECT DATA
@ENDDATD EQU   *
IGG0CLB4 CSECT
         DS    0F
@SIZDATD DC    AL1(0)
         DC    AL3(@ENDDATD-@DATD)
@CV04451 DC    V(IGGPARFS)
@CV04452 DC    V(IGGPDEFS)
@CV04453 DC    V(IGGPF4RD)
@CV04454 DC    V(IGGPF4WR)
@CV04455 DC    V(IGGPMODI)
@CV04456 DC    V(IGGPSALL)
@CV04457 DC    V(IGGPWCAT)
@CV04458 DC    V(IGGPWCRA)
         DS    0D
@CC04440 DC    C'NG'
@CC04442 DC    C'GO'
@CB04446 DC    X'FFFFFFFFFFFFFFFF'
@CB03440 DC    X'00000F'
@00      EQU   00                      EQUATES FOR REGISTERS 0-15
@01      EQU   01
@02      EQU   02
@03      EQU   03
@04      EQU   04
@05      EQU   05
@06      EQU   06
@07      EQU   07
@08      EQU   08
@09      EQU   09
@10      EQU   10
@11      EQU   11
@12      EQU   12
@13      EQU   13
@14      EQU   14
@15      EQU   15
IGGSEGBR EQU   @12
R2       EQU   @02
R3       EQU   @03
R4       EQU   @04
RWA      EQU   @02
R1       EQU   @01
R5       EQU   @05
R6       EQU   @06
R7       EQU   @07
R8       EQU   @08
R9       EQU   @09
R10      EQU   @10
R11      EQU   @11
R12      EQU   @12
DCWABASE EQU   @04
UCBPTR   EQU   @01
DCRAWAPR EQU   @03
DCRARCTR EQU   @04
RET      EQU   @14
RFLD     EQU   @04
RGO      EQU   @15
RNDX     EQU   @03
RP0      EQU   @00
RP1      EQU   @01
RSAVE    EQU   @13
RWK1     EQU   @06
RWK2     EQU   @07
RWK3     EQU   @08
RWK4     EQU   @09
RZERO    EQU   @00
R0       EQU   @00
R13      EQU   @13
R14      EQU   @14
R15      EQU   @15
UCBDCPTR EQU   @01
WKR2     EQU   @02
WKR3     EQU   @03
WKR4     EQU   @04
CVTPTR   EQU   16
PSA      EQU   0
FLCRNPSW EQU   PSA
FLCROPSW EQU   PSA+8
FLCICCW2 EQU   PSA+16
PSAEEPSW EQU   PSA+132
PSAESPSW EQU   PSA+136
FLCSVILC EQU   PSAESPSW+1
PSAEPPSW EQU   PSA+140
FLCPIILC EQU   PSAEPPSW+1
FLCPICOD EQU   PSAEPPSW+2
PSAPICOD EQU   FLCPICOD+1
FLCTEA   EQU   PSAEPPSW+4
FLCPER   EQU   PSA+152
FLCMCLA  EQU   PSA+168
FLCIOEL  EQU   FLCMCLA+4
FLCIOA   EQU   FLCMCLA+16
FLCFSA   EQU   FLCMCLA+80
PSASUPER EQU   PSA+552
PSASUP1  EQU   PSASUPER
PSASUP2  EQU   PSASUPER+1
PSASUP3  EQU   PSASUPER+2
PSASUP4  EQU   PSASUPER+3
PSACLHT  EQU   PSA+640
PSALKSA  EQU   PSA+696
PSAHLHI  EQU   PSA+760
PSADSSFL EQU   PSA+836
PSADSSF1 EQU   PSADSSFL
PSADSSF3 EQU   PSADSSFL+2
PSADSSF4 EQU   PSADSSFL+3
PSARSVT  EQU   PSA+896
UPSVOL   EQU   0
UPLKEYPT EQU   UPSVOL+4
IDAAMDSB EQU   0
AMDATTR  EQU   IDAAMDSB+1
AMDNEST  EQU   IDAAMDSB+4
AMDHLRBA EQU   IDAAMDSB+28
AMDSSRBA EQU   IDAAMDSB+32
AMDATTR3 EQU   IDAAMDSB+40
AMDSTAT  EQU   IDAAMDSB+48
RELINFO  EQU   0
UPEXT1   EQU   0
UPKEY1   EQU   0
UPKEY    EQU   0
WKAINFO  EQU   0
WKARETN  EQU   WKAINFO+4
CTVOLRCD EQU   1
CTVDVCHR EQU   CTVOLRCD+52
CTVNOTRK EQU   CTVDVCHR+10
CTVDVFLG EQU   CTVDVCHR+17
CTVSPHDR EQU   0
CTVSPFLG EQU   CTVSPHDR+15
CTVDESGP EQU   0
CTVEXTST EQU   CTVDESGP+4
CTVSTDES EQU   CTVEXTST
CTVDSGRP EQU   0
CTVDSDFL EQU   CTVDSGRP+6
CTVDSDIR EQU   CTVDSGRP+9
CTVGOPT  EQU   0
CTVGOCI  EQU   CTVGOPT
CTVGOGCI EQU   CTVGOPT+3
CTVRCTRL EQU   0
CTVEXRPT EQU   CTVRCTRL
CTVSPMAP EQU   0
CTVSMSN  EQU   CTVSPMAP
CTVDSPAC EQU   0
CTVDSPSN EQU   CTVDSPAC
CTVDSPFG EQU   CTVDSPAC+15
CTVSPEXT EQU   CTVDSPAC+21
CTVEXTNT EQU   CTVSPEXT
CTVDIR   EQU   0
CTVDSDSN EQU   CTVDIR
CTVDEXTN EQU   0
UPREPARM EQU   0
UPNTCYL  EQU   UPREPARM
UPFLGS   EQU   UPREPARM+8
UPFLG2S  EQU   UPREPARM+9
UPREND   EQU   UPREPARM+18
UPENTVOL EQU   0
UPVOLFIX EQU   UPENTVOL
UPVLFLG  EQU   UPVOLFIX+14
UPITEX   EQU   UPVOLFIX+36
UPVOLVAR EQU   UPENTVOL+39
EXTINF1  EQU   0
EXTINFO  EQU   EXTINF1+2
EXTST    EQU   EXTINFO+2
EXTEND   EQU   EXTINFO+6
UPSPARM  EQU   0
UPSFLG   EQU   UPSPARM+4
UPMXBLK  EQU   UPSPARM+12
UPCRDT1  EQU   UPSPARM+24
UPCRDT   EQU   UPCRDT1
UPSPHDR  EQU   UPSPARM+28
UPDIRSP  EQU   UPSPARM+51
UPSPWA   EQU   UPSPARM+63
UPSPD    EQU   UPSPARM+198
IGGCCA   EQU   0
CCAPROB  EQU   IGGCCA+4
CCAERRCD EQU   CCAPROB+2
CCAREASN EQU   CCAERRCD
CCARETRN EQU   CCAERRCD+1
CCACD1   EQU   CCARETRN
CCAFLG1  EQU   IGGCCA+14
CCAFLG2  EQU   IGGCCA+15
CCAF2COB EQU   CCAFLG2
CCAF2CB  EQU   CCAF2COB
CCAFLG3  EQU   IGGCCA+16
CCAFLG4  EQU   IGGCCA+17
CCAFLG5  EQU   IGGCCA+18
CCAFLG6  EQU   IGGCCA+19
CCATCB   EQU   IGGCCA+20
CCARB    EQU   IGGCCA+24
CCACPL   EQU   IGGCCA+28
CCAACB   EQU   IGGCCA+32
CCAURAB  EQU   IGGCCA+40
CCASRCH  EQU   IGGCCA+44
CCASRID  EQU   CCASRCH
CCARAB0  EQU   IGGCCA+88
CCAR0FLG EQU   CCARAB0
CCAR0SEG EQU   CCARAB0+8
CCARAB1  EQU   IGGCCA+108
CCAR1FLG EQU   CCARAB1
CCAR1REC EQU   CCARAB1+4
CCAR1SEG EQU   CCARAB1+8
CCARAB2  EQU   IGGCCA+128
CCAR2FLG EQU   CCARAB2
CCAR2REC EQU   CCARAB2+4
CCAR2SEG EQU   CCARAB2+8
CCARAB3  EQU   IGGCCA+148
CCAR3FLG EQU   CCARAB3
CCAR3REC EQU   CCARAB3+4
CCAR3SEG EQU   CCARAB3+8
CCARAB4  EQU   IGGCCA+168
CCAR4FLG EQU   CCARAB4
CCAR4SEG EQU   CCARAB4+8
CCARAB5  EQU   IGGCCA+188
CCAR5FLG EQU   CCARAB5
CCAR5SEG EQU   CCARAB5+8
CCARPLX  EQU   IGGCCA+210
CCATIQRN EQU   IGGCCA+212
CCADESA  EQU   IGGCCA+224
CCAEXTDE EQU   CCADESA+4
CCAEXTAD EQU   CCAEXTDE+2
CCAEXTCC EQU   CCAEXTAD
CCAEXTHH EQU   CCAEXTAD+2
CCAEQDQ  EQU   IGGCCA+280
CCAEDOPT EQU   CCAEQDQ+2
CCAMSSPL EQU   IGGCCA+300
CCAMNSPL EQU   CCAMSSPL+9
CCACMS   EQU   IGGCCA+316
CCACMSWA EQU   CCACMS
CCACPE5  EQU   IGGCCA+324
CCACPE6  EQU   IGGCCA+340
CCACPE61 EQU   IGGCCA+344
CCACPE7  EQU   IGGCCA+348
CCAFLPT  EQU   IGGCCA+364
CCADICT  EQU   IGGCCA+372
CCAXCPL  EQU   IGGCCA+376
CCALREL  EQU   IGGCCA+396
CCASN    EQU   IGGCCA+398
CCARBA   EQU   IGGCCA+448
CCACRAVL EQU   CCARBA
CCACRADT EQU   CCARBA+6
CCATLNG  EQU   IGGCCA+468
CCAIDPT3 EQU   IGGCCA+480
CCAGRGC  EQU   IGGCCA+493
CCAGRHI  EQU   IGGCCA+494
CCASSEQ  EQU   CCAGRHI
CCAALPTR EQU   IGGCCA+540
CCAFLG7  EQU   IGGCCA+552
CCACRABU EQU   CCAFLG7
CCAFLG8  EQU   IGGCCA+553
CCADSRCL EQU   CCAFLG8
CCAVBUFI EQU   CCAFLG8
CCAFLG9  EQU   IGGCCA+554
CCASMFLG EQU   IGGCCA+562
CCASMFG1 EQU   CCASMFLG
CCASMFUC EQU   CCASMFG1
CCASMFSR EQU   CCASMFG1
CCABMINP EQU   IGGCCA+684
CCABMFLG EQU   CCABMINP+6
CCABMOUT EQU   IGGCCA+692
CCABMOFG EQU   CCABMOUT+4
CCATEMPS EQU   IGGCCA+744
CCAMNCAT EQU   IGGCCA+784
CCAMNAT  EQU   CCAMNCAT
MNATTOP  EQU   CCAMNAT
MNATFULL EQU   MNATTOP
MNATFLGS EQU   CCAMNAT+240
CCAMNLEN EQU   CCAMNCAT+249
CCAMNADR EQU   CCAMNCAT+252
CCAARFWA EQU   IGGCCA+1040
ARFGMLEN EQU   CCAARFWA
CCARVFG1 EQU   IGGCCA+1056
CCAREGS  EQU   IGGCCA+1060
CCABZSAV EQU   IGGCCA+1408
CCADSPWA EQU   CCABZSAV
CCACUMPL EQU   IGGCCA+1412
CCACRACI EQU   IGGCCA+1420
CCARAACB EQU   IGGCCA+1424
CCARARPL EQU   IGGCCA+1428
CCARARBA EQU   IGGCCA+1432
CCARALSA EQU   IGGCCA+1440
CCACRABT EQU   CCARALSA
CCAFLG10 EQU   IGGCCA+1444
CCARAVCA EQU   IGGCCA+1460
CCASMFP  EQU   IGGCCA+1480
CCAPROBX EQU   IGGCCA+1484
CCAERCDX EQU   CCAPROBX+2
CCAREQDQ EQU   IGGCCA+1492
CCAREDOP EQU   CCAREQDQ+2
CTGPL    EQU   0
CTGOPTN1 EQU   CTGPL
CTGOPTN2 EQU   CTGPL+1
CTGEXT   EQU   CTGOPTN2
CTGERASE EQU   CTGOPTN2
CTGSMF   EQU   CTGERASE
CTGREL   EQU   CTGSMF
CTGPURG  EQU   CTGOPTN2
CTGVMNT  EQU   CTGPURG
CTGRCATN EQU   CTGVMNT
CTGOPTN3 EQU   CTGPL+2
CTGOPTN4 EQU   CTGPL+3
CTGENT   EQU   CTGPL+4
CTGFVT   EQU   CTGENT
CTGCAT   EQU   CTGPL+8
CTGCVOL  EQU   CTGCAT
CTGWKA   EQU   CTGPL+12
CTGDSORG EQU   CTGPL+16
CTGTYPE  EQU   CTGPL+18
CTGDDNM  EQU   CTGPL+20
CTGNEWNM EQU   CTGDDNM
CTGFBFLG EQU   CTGNEWNM+2
CTGJSCB  EQU   CTGPL+24
CTGFIELD EQU   CTGPL+28
CTGFL    EQU   0
CTGFLDGC EQU   CTGFL+2
CTGFLDRE EQU   CTGFL+3
CTGFLDWA EQU   CTGFL+4
CTGFLDNM EQU   CTGFL+8
CTGFLDAT EQU   CTGFL+16
CTGFLPT  EQU   CTGFLDAT+4
USERWKA  EQU   0
WKA      EQU   USERWKA+4
ENTNAMES EQU   WKA
DICTENT  EQU   0
DICTFLGS EQU   DICTENT
DIC2ENT  EQU   0
DIC2FLGS EQU   DIC2ENT
CCARAB   EQU   0
CCARABFL EQU   CCARAB
CCARBUR  EQU   CCARABFL
CCACPE1  EQU   CCARAB+4
CCACPE2  EQU   CCARAB+8
CCACPE3  EQU   CCARAB+12
CCACPE4  EQU   CCARAB+16
CATGOPT  EQU   0
CATGOCI  EQU   CATGOPT
CATGOFLG EQU   CATGOCI
CATGOGC1 EQU   CATGOPT+3
CATGOEXT EQU   CATGOGC1
CATGODEL EQU   CATGOGC1
CATGOSEQ EQU   CATGOPT+4
CATRCD   EQU   0
RCDCINO  EQU   CATRCD+1
RCDRELID EQU   CATRCD+4
RCDCRAVL EQU   CATRCD+5
RCDRACTS EQU   CATRCD+18
RCDSECF  EQU   CATRCD+26
RCDID    EQU   CATRCD+44
RCDSIZ   EQU   CATRCD+45
NREPLF   EQU   CATRCD+48
REPCNTRL EQU   0
EXTRCDPT EQU   REPCNTRL
REPCNT   EQU   REPCNTRL+5
REPGOPT  EQU   REPCNTRL+6
VARFIELD EQU   0
CNTRLINF EQU   0
LENFIX   EQU   CNTRLINF+1
CVTMAP   EQU   0
CVTTCBP  EQU   CVTMAP
CVTDAR   EQU   CVTMAP+72
CVTFLGS1 EQU   CVTDAR
CVTDCB   EQU   CVTMAP+116
CVTIERLC EQU   CVTMAP+144
CVTOPTA  EQU   CVTMAP+182
CVTOPTB  EQU   CVTMAP+183
CVTGTF   EQU   CVTMAP+236
CVTGTFST EQU   CVTGTF
CVTGTFS  EQU   CVTGTFST
CVTSTATE EQU   CVTGTFST
CVTTMODE EQU   CVTGTFST
CVTFORM  EQU   CVTGTFST
CVTAQAVT EQU   CVTMAP+240
CVTTCMFG EQU   CVTAQAVT
CVTVOLM2 EQU   CVTMAP+244
CVTTATA  EQU   CVTVOLM2
CVTTSKS  EQU   CVTTATA
CVTVOLF2 EQU   CVTTSKS
CVTTAT   EQU   CVTTATA+1
CVTATER  EQU   CVTMAP+248
CVTEXT1  EQU   CVTMAP+252
CVTCBSP  EQU   CVTMAP+256
CVTPURG  EQU   CVTMAP+260
CVTAMFF  EQU   CVTMAP+264
CVTQMSG  EQU   CVTMAP+268
CVTDMSR  EQU   CVTMAP+272
CVTRSV37 EQU   CVTDMSR
CVTDMSRF EQU   CVTRSV37
CVTERPV  EQU   CVTMAP+316
CVTINTLA EQU   CVTMAP+320
CVTAPF   EQU   CVTMAP+324
CVTEXT2  EQU   CVTMAP+328
CVTHJES  EQU   CVTMAP+332
CVTPGSIA EQU   CVTMAP+348
CVTA1F1  EQU   CVTMAP+356
CVTSYSK  EQU   CVTMAP+357
CVTVOLM1 EQU   CVTMAP+380
CVTVOLF1 EQU   CVTVOLM1
CVTATMCT EQU   CVTMAP+388
CVTXTNT1 EQU   0
CVTXTNT2 EQU   0
CVTDSSV  EQU   CVTXTNT2
CVTFLGBT EQU   CVTXTNT2+5
CVTQID   EQU   CVTXTNT2+24
CVTRV400 EQU   CVTXTNT2+52
CVTRV409 EQU   CVTXTNT2+53
CVTATCVT EQU   CVTXTNT2+64
CVTRV429 EQU   CVTXTNT2+84
CVTRV438 EQU   CVTXTNT2+85
CVTRV457 EQU   CVTXTNT2+112
CVTRV466 EQU   CVTXTNT2+113
CVTFIX   EQU   0
CVTRELNO EQU   CVTFIX+252
TCBWORDS EQU   0
TCBPTR   EQU   TCBWORDS+4
TCBFIX   EQU   0
TCBFRS   EQU   TCBFIX
TCB      EQU   0
TCBPIE   EQU   TCB+4
TCBPMASK EQU   TCBPIE
TCBTIO   EQU   TCB+12
TCBCMP   EQU   TCB+16
TCBCMPF  EQU   TCBCMP
TCBCPP   EQU   TCBCMPF
TCBSTCC  EQU   TCBCMPF
TCBCDBL  EQU   TCBCMPF
TCBCWTO  EQU   TCBCMPF
TCBCIND  EQU   TCBCMPF
TCBCMSG  EQU   TCBCMPF
TCBTRN   EQU   TCB+20
TCBABF   EQU   TCBTRN
TCBMSS   EQU   TCB+24
TCBPKF   EQU   TCB+28
TCBFLGS  EQU   TCB+29
TCBFLGS1 EQU   TCBFLGS
TCBFLGS2 EQU   TCBFLGS+1
TCBFLGS3 EQU   TCBFLGS+2
TCBFLGS4 EQU   TCBFLGS+3
TCBFLGS5 EQU   TCBFLGS+4
TCBABWF  EQU   TCBFLGS5
TCBJPQ   EQU   TCB+44
TCBPURGE EQU   TCBJPQ
TCBGRS   EQU   TCB+48
TCBFSA   EQU   TCB+112
TCBTME   EQU   TCB+120
TCBJSTCB EQU   TCB+124
TCBTSFLG EQU   TCB+148
TCBTSTSK EQU   TCBTSFLG
TCBSTAB  EQU   TCB+160
TCBNSTAE EQU   TCBSTAB
TCBTCT   EQU   TCB+164
TCBTCTGF EQU   TCBTCT
TCBSCNDY EQU   TCB+172
TCBNDSP  EQU   TCBSCNDY
TCBNDSP1 EQU   TCBNDSP+1
TCBNDSP2 EQU   TCBNDSP+2
TCBNDSP3 EQU   TCBNDSP+3
TCBJSCB  EQU   TCB+180
TCBRECDE EQU   TCBJSCB
TCBEXT1  EQU   TCB+196
TCBBITS  EQU   TCB+200
TCBNDSP4 EQU   TCBBITS
TCBNDSP5 EQU   TCBBITS+1
TCBFLGS6 EQU   TCBBITS+2
TCBFLGS7 EQU   TCBBITS+3
TCBDAR   EQU   TCB+204
TCBDARC  EQU   TCBDAR
TCBEXT2  EQU   TCB+208
TCBXSCT  EQU   TCB+240
TCBXSCT1 EQU   TCBXSCT
TCBFOE   EQU   TCB+244
TCBESTAE EQU   TCB+264
TCBESTRM EQU   TCBESTAE+1
TCBFBYT1 EQU   TCB+276
TCBFBYT2 EQU   TCB+277
TCBRV124 EQU   TCB+278
TCBRV133 EQU   TCB+279
TCBXTNT2 EQU   0
TCBGTF   EQU   TCBXTNT2
TCBTFLG  EQU   TCBGTF
TCBRV146 EQU   TCBXTNT2+28
TCBRV155 EQU   TCBXTNT2+29
TCBRV164 EQU   TCBXTNT2+30
TCBRV173 EQU   TCBXTNT2+31
IFGACB   EQU   0
ACBCOMN  EQU   IFGACB
ACBID    EQU   ACBCOMN
ACBLENG  EQU   ACBCOMN+2
ACBLENG2 EQU   ACBLENG
ACBAMBL  EQU   ACBCOMN+4
ACBJWA   EQU   ACBAMBL
ACBMACRF EQU   ACBCOMN+12
ACBMACR1 EQU   ACBMACRF
ACBADR   EQU   ACBMACR1
ACBCNV   EQU   ACBMACR1
ACBDIR   EQU   ACBMACR1
ACBOUT   EQU   ACBMACR1
ACBUBF   EQU   ACBMACR1
ACBMACR2 EQU   ACBMACRF+1
ACBSTRNO EQU   ACBCOMN+15
ACBBUFND EQU   ACBCOMN+16
ACBBUFPL EQU   ACBCOMN+20
ACBMACR3 EQU   ACBBUFPL
ACBRECFM EQU   ACBCOMN+24
ACBCCTYP EQU   ACBCOMN+25
ACBOPT   EQU   ACBCOMN+26
ACBDSORG EQU   ACBOPT
ACBCROPS EQU   ACBDSORG
ACBDVIND EQU   ACBDSORG
ACBDORGA EQU   ACBDSORG+1
ACBEXLST EQU   ACBCOMN+36
ACBDDNM  EQU   ACBCOMN+40
ACBAMETH EQU   ACBDDNM+3
ACBOFLGS EQU   ACBCOMN+48
ACBOPEN  EQU   ACBOFLGS
ACBEXFG  EQU   ACBOFLGS
ACBIOSFG EQU   ACBOFLGS
ACBERFLG EQU   ACBCOMN+49
ACBINFLG EQU   ACBCOMN+50
ACBSCRA  EQU   ACBINFLG
ACBVVIC  EQU   ACBINFLG
ACBUJFCB EQU   ACBCOMN+52
ACBBLKSZ EQU   ACBCOMN+60
ACBUAPTR EQU   ACBCOMN+64
AMCBS    EQU   0
CBSCMP   EQU   AMCBS+16
CBSVSRT  EQU   AMCBS+32
TNAM     EQU   0
NOBASE   EQU   0
IGGCAXWA EQU   0
CAXID    EQU   IGGCAXWA
CAXFLGS  EQU   IGGCAXWA+8
CAXBLD   EQU   CAXFLGS
CAXMCT   EQU   CAXFLGS
CAXCMR   EQU   CAXFLGS
CAXFLG2  EQU   IGGCAXWA+9
CAXFLG3  EQU   IGGCAXWA+10
CAXACT   EQU   IGGCAXWA+11
CAXATIOT EQU   IGGCAXWA+12
CAXACB   EQU   IGGCAXWA+24
CAXUCB   EQU   IGGCAXWA+28
CAXCCR   EQU   IGGCAXWA+32
CAXCNAM  EQU   IGGCAXWA+52
CAXVOLID EQU   CAXCNAM
CAXRACTS EQU   CAXCNAM+6
CAXOPLST EQU   IGGCAXWA+96
COPTS    EQU   CAXOPLST
CENLST   EQU   COPTS
COPACB   EQU   CAXOPLST+1
CAXCCA   EQU   IGGCAXWA+104
CAXCRACB EQU   IGGCAXWA+112
CAXECB   EQU   IGGCAXWA+120
CAXECBHB EQU   CAXECB
CAXHRPLW EQU   IGGCAXWA+128
CTGFV    EQU   0
CTGFVPRO EQU   CTGFV+1
CTGFVVCH EQU   CTGFV+12
CTGFVIND EQU   CTGFV+16
CTGFVENT EQU   CTGFV+20
CTGFVVLT EQU   CTGFV+40
CTGFVSPC EQU   CTGFV+52
CTGFVAMD EQU   CTGFV+56
CTGFVATR EQU   CTGFV+60
CTGFVLMT EQU   CTGFV+72
CTGFVGAT EQU   CTGFV+76
CTGFVPWD EQU   CTGFV+84
CTGWA    EQU   0
CTGWAVL  EQU   CTGWA
CTGVL    EQU   0
CTGVLX   EQU   0
CTGCV    EQU   0
CTGCVDSN EQU   CTGCV+6
ACHKLIST EQU   0
ACHKINSW EQU   ACHKLIST
ACHKENTW EQU   ACHKLIST+4
ACHKFLG1 EQU   ACHKENTW
ACHKENT  EQU   ACHKENTW+1
ACHKCLNW EQU   ACHKLIST+8
ACHKFLG2 EQU   ACHKCLNW
ACHKVOLW EQU   ACHKLIST+12
ACHKFLG3 EQU   ACHKVOLW
RDDFLIST EQU   0
RDDFINSW EQU   RDDFLIST
RDDFENTW EQU   RDDFLIST+4
RDDFFLGS EQU   RDDFENTW
RDDFOVOL EQU   RDDFLIST+8
MWA      EQU   0
VLST     EQU   MWA+2
CVOLIND  EQU   MWA+252
CATRECRD EQU   0
ASGO     EQU   1
PASSGO   EQU   1
PASSWORD EQU   PASSGO
USERAREC EQU   PASSGO+50
USERLIST EQU   0
LTLENGTH EQU   USERLIST
LTENTRY  EQU   USERLIST+2
LTRETCOD EQU   LTENTRY
LTVOLSER EQU   LTENTRY+2
SPACEPRM EQU   0
SPCPRIMQ EQU   SPACEPRM
SPCSECDQ EQU   SPACEPRM+3
SPCOPTNS EQU   SPACEPRM+6
SPCRQTYP EQU   SPCOPTNS
CTGDSATR EQU   0
CTGATR1  EQU   CTGDSATR
CTGDSUNQ EQU   CTGATR1
CTGATR2  EQU   CTGDSATR+1
RGATTRFL EQU   0
RGATTR1  EQU   RGATTRFL
RGATTSG1 EQU   RGATTRFL+1
DEVCHARS EQU   0
RTDEVUCB EQU   DEVCHARS
RTBOKAB  EQU   DEVCHARS+14
RTDVFLGS EQU   DEVCHARS+17
SALPARL  EQU   0
SALOPTNS EQU   SALPARL
SALSOURC EQU   SALPARL+2
SALPARMS EQU   SALPARL+6
SALCALN  EQU   SALPARMS
SALUNITS EQU   SALPARMS+4
SALMXEXT EQU   SALPARMS+6
SALRLIST EQU   SALPARL+14
PRLIST   EQU   0
PRLENT   EQU   PRLIST+1
PRELENTF EQU   PRLENT
DSPTBL   EQU   0
CMSWA    EQU   0
CMWFLAGS EQU   CMSWA
CMCISUSE EQU   CMSWA+2
CMWRKATR EQU   CMCISUSE
CMCPLFVT EQU   CMSWA+20
CMNRMEXT EQU   CMSWA+24
CMLDTFPL EQU   CMNRMEXT
CMSSEXTS EQU   CMSWA+28
CMHDTFPL EQU   CMSSEXTS
CMCANVOL EQU   CMSWA+32
CMHIXFPL EQU   CMCANVOL
PROCESS  EQU   CMSWA+36
CMRNGCTR EQU   CMSWA+40
CMVOLCTR EQU   CMSWA+42
CMSDFSPC EQU   CMSWA+48
CMVLLIST EQU   CMSWA+64
CMVLRTCD EQU   CMVLLIST+2
CMSPCATR EQU   CMSWA+76
CMBLKTRK EQU   CMSPCATR+8
CMTRKCYL EQU   CMSPCATR+12
CMWASTOR EQU   CMSWA+132
CMPOT    EQU   CMWASTOR
CMDCONV  EQU   CMPOT
CMIXFANO EQU   CMDCONV
CMPOTX   EQU   CMWASTOR+4
CMPBI    EQU   CMPOTX
CMIXFANS EQU   CMPBI
CMWABYTB EQU   CMSWA+140
CMPBIX   EQU   CMWABYTB
CMNEST   EQU   CMPBIX
CMFANT   EQU   CMNEST
CMSPCQTY EQU   CMFANT
CMWGENAM EQU   CMSWA+144
CMWDATE  EQU   CMWGENAM+12
CMWTIME  EQU   CMWGENAM+17
DCWPLSPS EQU   CMSWA+200
DCWPLSPL EQU   DCWPLSPS
DCWPLBFS EQU   CMSWA+232
DCWPLBFL EQU   DCWPLBFS
DCWPLEDS EQU   CMSWA+260
DCWPLEDL EQU   DCWPLEDS
DCWPLARS EQU   CMSWA+288
DCWPLARL EQU   DCWPLARS
DCWPLCDS EQU   CMSWA+316
DCWPLCDL EQU   DCWPLCDS
CMUCATCI EQU   CMSWA+410
CMSCTREC EQU   CMSWA+416
CMSRAC   EQU   CMSWA+516
CMSROBJA EQU   CMSRAC+32
CMSROBJN EQU   CMSROBJA
CUMPLWA  EQU   0
CUMPL    EQU   CUMPLWA
CUMIN    EQU   CUMPL
CUMOUT   EQU   CUMPL+1
CUMWA    EQU   CUMPLWA+8
CUMFLAG  EQU   CUMWA
CUMWASS2 EQU   CUMWA+48
CUMWASS  EQU   CUMWASS2+2
CUMWTYP  EQU   CUMWASS
GVO      EQU   0
CMFLVLFL EQU   GVO+12
CMFLHUBA EQU   GVO+18
CMFLITEX EQU   GVO+34
CMFLNOPF EQU   CMFLITEX
CMFOVERL EQU   GVO+37
CMFOVLAY EQU   0
CMFXECTS EQU   0
CMFXEXST EQU   CMFXECTS+2
CMFXSTA  EQU   CMFXEXST
CMFXEXED EQU   CMFXECTS+6
CMFXEDA  EQU   CMFXEXED
IDAVVOLL EQU   0
@NM00171 EQU   IDAVVOLL
VVOLLERR EQU   IDAVVOLL+12
VVOLLENT EQU   IDAVVOLL+36
@NM00174 EQU   VVOLLENT+4
IFGRPL   EQU   0
RPLCOMN  EQU   IFGRPL
RPLIDWD  EQU   RPLCOMN
RPLREQ   EQU   RPLIDWD+2
RPLLEN   EQU   RPLIDWD+3
RPLECB   EQU   RPLCOMN+8
RPLFDBWD EQU   RPLCOMN+12
RPLFDBK  EQU   RPLFDBWD+1
RPLRTNCD EQU   RPLFDBK
RPLCNDCD EQU   RPLFDBK+1
RPLCMPON EQU   RPLCNDCD
RPLFDB2  EQU   RPLCMPON
RPLERRCD EQU   RPLCNDCD+1
RPLFDB3  EQU   RPLERRCD
RPLKEYLE EQU   RPLCOMN+16
RPLDACB  EQU   RPLCOMN+24
RPLTCBPT EQU   RPLCOMN+28
RPLARG   EQU   RPLCOMN+36
RPLOPTCD EQU   RPLCOMN+40
RPLOPT1  EQU   RPLOPTCD
RPLDIR   EQU   RPLOPT1
RPLECBSW EQU   RPLOPT1
RPLOPT2  EQU   RPLOPTCD+1
RPLADR   EQU   RPLOPT2
RPLCNV   EQU   RPLOPT2
RPLOPT3  EQU   RPLOPTCD+2
RPLNXTRP EQU   RPLCOMN+44
RPLRLEN  EQU   RPLCOMN+48
RPLOPTC2 EQU   RPLCOMN+56
RPLOPT5  EQU   RPLOPTC2
RPLWRTYP EQU   RPLOPT5
RPLOPT6  EQU   RPLOPTC2+1
RPLUNTYP EQU   RPLOPT6
RPLOPT7  EQU   RPLOPTC2+2
RPLCNOPT EQU   RPLOPT7
RPLOPT8  EQU   RPLOPTC2+3
RPLRBAR  EQU   RPLCOMN+60
RPLAIXID EQU   RPLRBAR+2
RPLEXTDS EQU   RPLCOMN+68
RPLEXTD1 EQU   RPLEXTDS
IGGMF4WA EQU   0
F4WDEB   EQU   IGGMF4WA
F4WDUCB  EQU   F4WDEB+33
F4WECB   EQU   IGGMF4WA+52
F4WIOB   EQU   IGGMF4WA+56
F4WCCW   EQU   IGGMF4WA+96
F4WBUF   EQU   IGGMF4WA+120
IECSDSF4 EQU   0
IECSDSL4 EQU   0
DS4DEVCT EQU   IECSDSL4+18
DS4DEVOV EQU   DS4DEVCT+6
DS4AMCAT EQU   IECSDSL4+40
DS4VSIND EQU   DS4AMCAT
DS4VSCRA EQU   DS4AMCAT+1
UCB      EQU   0
UCBOB    EQU   0
UCBJBNR  EQU   UCBOB
UCBFL5   EQU   UCBOB+1
UCBAF    EQU   UCBFL5
UCBSTAT  EQU   UCBOB+3
UCBCHAN  EQU   UCBOB+4
UCBSFLS  EQU   UCBOB+6
UCBFLA   EQU   UCBSFLS
UCBFL1   EQU   UCBFLA
UCBBSY   EQU   UCBFL1
UCBNRY   EQU   UCBFL1
UCBPST   EQU   UCBFL1
UCBCUB   EQU   UCBFL1
UCBFLB   EQU   UCBSFLS+1
UCBCHM   EQU   UCBOB+8
UCBCHM1  EQU   UCBCHM
UCBPTH0  EQU   UCBCHM1
UCBPTH1  EQU   UCBCHM1
UCBWGT   EQU   UCBOB+12
UCBTYP   EQU   UCBOB+16
UCBTBYT1 EQU   UCBTYP
UCB1FEA5 EQU   UCBTBYT1
UCB1FEA6 EQU   UCBTBYT1
UCBTBYT2 EQU   UCBTYP+1
UCB2OPT2 EQU   UCBTBYT2
UCBDUDN1 EQU   UCB2OPT2
UCB2OPT3 EQU   UCBTBYT2
UCBDUDN2 EQU   UCB2OPT3
UCB2OPT4 EQU   UCBTBYT2
UCBRWTAU EQU   UCB2OPT4
UCB2OPT6 EQU   UCBTBYT2
UCB2OPT7 EQU   UCBTBYT2
UCBTBYT3 EQU   UCBTYP+2
UCBDVCLS EQU   UCBTBYT3
UCBTBYT4 EQU   UCBTYP+3
UCBEXTPT EQU   UCBOB+20
UCBFLC   EQU   UCBEXTPT
UCBDEV   EQU   UCBOB+24
UCBCMEXT EQU   0
UCBATI   EQU   UCBCMEXT+3
@NM00193 EQU   UCBATI
UCBFLP1  EQU   UCBCMEXT+5
UCBMT    EQU   0
UCBOCR   EQU   0
UCB3540X EQU   0
UCBDKBYT EQU   UCB3540X+6
UCB3800X EQU   0
UCBOPTNS EQU   UCB3800X
UCBACTIV EQU   UCB3800X+3
UCBMDRBF EQU   UCB3800X+36
UCBUCS   EQU   0
UCBUCSOP EQU   UCBUCS+4
UCBFCBOP EQU   UCBUCS+5
IDAAMBL  EQU   0
AMBLEOV  EQU   IDAAMBL+12
AMBLEFLG EQU   AMBLEOV
AMBLDDNM EQU   IDAAMBL+16
AMBLIDF  EQU   AMBLDDNM
AMBLQ    EQU   AMBLIDF+7
AMBLVC   EQU   IDAAMBL+28
AMBLTYPE EQU   IDAAMBL+30
AMBLSHAR EQU   IDAAMBL+33
AMBLFLG1 EQU   IDAAMBL+35
AMBLFLG2 EQU   IDAAMBL+36
IDAAMB   EQU   0
AMBEOVR  EQU   IDAAMB+24
AMBFLG0  EQU   AMBEOVR
AMBFLG1  EQU   IDAAMB+25
AMBDSORG EQU   IDAAMB+26
AMBIOBAD EQU   IDAAMB+28
AMBINFL  EQU   IDAAMB+42
AMBDEBPT EQU   IDAAMB+44
AMBOFLGS EQU   IDAAMB+48
AMBFLG2  EQU   IDAAMB+49
AMBEOVPT EQU   IDAAMB+56
AMBCSWD1 EQU   IDAAMB+84
AMBAFLG  EQU   AMBCSWD1
DCRAWKA  EQU   0
DCRACIPT EQU   DCRAWKA
DCRADSWA EQU   DCRAWKA+24
DCRADSUL EQU   DCRAWKA+28
DCRADSAF EQU   DCRAWKA+32
DCRAOFPL EQU   DCRAWKA+36
DCRAFVEN EQU   DCRAWKA+40
DCRAR1PT EQU   DCRAWKA+44
DCRAR2PT EQU   DCRAWKA+48
DCRACHIU EQU   DCRAWKA+52
DCRACHIA EQU   DCRAWKA+56
DCRAMNLN EQU   DCRAWKA+60
DCRATRCY EQU   DCRAWKA+64
DCRABTRK EQU   DCRAWKA+66
DCRADESA EQU   DCRAWKA+68
DCRAMCPL EQU   DCRAWKA+112
DCRAMFPL EQU   DCRAWKA+152
DCRASFPL EQU   DCRAWKA+272
DCRASSOC EQU   DCRAWKA+296
DCRAVFPL EQU   DCRAWKA+300
DCRAVGO  EQU   DCRAWKA+324
DCRARACI EQU   DCRAWKA+400
DCRACI0  EQU   DCRARACI
DCRASALP EQU   DCRACI0
DCRASPRM EQU   DCRASALP
DCRADSVL EQU   DCRASALP+7
DCRACI15 EQU   DCRARACI+512
DCRACI5  EQU   DCRACI15+2048
DCRA5VGO EQU   DCRACI5+62
DCRAEND  EQU   DCRAWKA+3472
DCRARABS EQU   0
CCRCNTRL EQU   1
CCRHACI  EQU   CCRCNTRL
CCRNFCI  EQU   CCRCNTRL+3
BUFVOLRD EQU   0
BUFNEXTB EQU   BUFVOLRD+508
DCRACI2A EQU   0
DCRADDNM EQU   0
BUFSIZE  EQU   0
CATSN    EQU   0
CI       EQU   0
CLEAR    EQU   0
CLSCINO  EQU   0
CTGVLDSP EQU   0
DATEFLD  EQU   0
ENTRYNAM EQU   0
ENTVOL1  EQU   0
ENTVOL2  EQU   0
EXCPNAME EQU   0
FIELDNM  EQU   0
IDAAMDS1 EQU   0
KEYINFO  EQU   0
LRECL    EQU   0
OWNIDFLD EQU   0
UCBBGN   EQU   0
UPEVOL   EQU   0
WKAINPUT EQU   WKARETN
@NM00061 EQU   CTGDDNM
CTGPROB  EQU   @NM00061
CTGREASN EQU   CTGPROB+2
CVTS01   EQU   CVTPGSIA
CVTLPDIA EQU   CVTS01+12
CVTDIRST EQU   CVTLPDIA
CVTSLIDA EQU   CVTS01+24
CVTCTLFG EQU   CVTS01+50
CVTRV210 EQU   CVTS01+424
CVTRV219 EQU   CVTS01+425
CVTRV228 EQU   CVTS01+426
CVTRV237 EQU   CVTS01+427
CVTMFRTR EQU   CVTS01+452
CVTRV262 EQU   CVTS01+468
CVTRV271 EQU   CVTS01+469
CVTRV280 EQU   CVTS01+470
CVTRV289 EQU   CVTS01+471
CVTGSDA  EQU   CVTS01+600
PSARSVTE EQU   PSARSVT
MCATDSTR EQU   CVTAMFF
CVTAMF1  EQU   MCATDSTR
CTGRBASE EQU   NREPLF+1
CNAME    EQU   CTGRBASE
COWNID   EQU   CTGRBASE+44
CCRE     EQU   CTGRBASE+52
CEXP     EQU   CTGRBASE+55
CATTR    EQU   CTGRBASE+58
CATTR1   EQU   CATTR
CATTR2   EQU   CATTR+1
COPEN    EQU   CTGRBASE+60
CBUF     EQU   CTGRBASE+61
CSPACPRM EQU   CTGRBASE+65
CPRIM    EQU   CSPACPRM
CSEC     EQU   CSPACPRM+3
CSORT    EQU   CSPACPRM+6
CHIU     EQU   CTGRBASE+72
CHIA     EQU   CTGRBASE+76
CLRECL   EQU   CTGRBASE+80
CEXT     EQU   CTGRBASE+86
@NM00138 EQU   CATTR1
CCATTR   EQU   @NM00138
@NM00139 EQU   CCATTR
@NM00141 EQU   CATTR1
CRGATTR  EQU   @NM00141
CUPG     EQU   CRGATTR
UCBOBS01 EQU   UCBDEV
UCBVOLI  EQU   UCBOBS01+4
UCBSTAB  EQU   UCBOBS01+10
UCBBSVL  EQU   UCBSTAB
UCBPRSRS EQU   UCBSTAB
UCBDMCT  EQU   UCBOBS01+11
UCBFL4   EQU   UCBOBS01+13
UCBOBS02 EQU   UCBDEV
UCBTFL1  EQU   UCBOBS02+19
UCBXTN   EQU   UCBOBS02+20
UCBVOPT  EQU   UCBXTN
UCBOBS03 EQU   UCBDEV
UCBXTADR EQU   UCBOBS03
UCBOBS04 EQU   UCBDEV
UCBBTA   EQU   UCBOBS04+12
UCBOBS06 EQU   UCBDEV
UCBAOF   EQU   UCBOBS06
UCBAOF1  EQU   UCBAOF
UCBAOF2  EQU   UCBAOF+1
@NM00192 EQU   UCBOBS06+3
UCBIRB   EQU   UCBOBS06+4
UCBGRAF  EQU   UCBIRB
UCBLDNCA EQU   UCBOBS06+8
UCBRDYQ  EQU   UCBLDNCA
UCBIRLN  EQU   UCBRDYQ
UCBLDNCB EQU   UCBRDYQ+1
UCBCTLNK EQU   UCBOBS06+12
UCBOBS07 EQU   UCBDEV
UCBOBS08 EQU   UCBDEV
UCBCTCAD EQU   UCBOBS08
UCBCTCF1 EQU   UCBOBS08+4
UCBOBS09 EQU   UCBDEV
ACHKPROF EQU   ACHKENT
FLC      EQU   PSA
F4WDCB   EQU   F4WDEB+4
         AGO   .@UNREFD                START UNREFERENCED COMPONENTS
UCBRV066 EQU   UCBOBS09+4
UCBIOSBA EQU   UCBOBS09
UCBRV042 EQU   UCBOBS08+5
UCBRV082 EQU   UCBCTCF1
UCBRV081 EQU   UCBCTCF1
UCBRV080 EQU   UCBCTCF1
UCBRV079 EQU   UCBCTCF1
UCBRV078 EQU   UCBCTCF1
UCBRV077 EQU   UCBCTCF1
UCBRV076 EQU   UCBCTCF1
UCBCTC80 EQU   UCBCTCF1
UCBCTCAL EQU   UCBCTCAD
UCBICNCB EQU   UCBOBS07+4
UCBRV040 EQU   UCBOBS07
UCBCTLNA EQU   UCBCTLNK+1
UCBRLN   EQU   UCBCTLNK
UCBRDYQA EQU   UCBLDNCB
UCBINRLN EQU   UCBIRLN
UCBIRBA  EQU   UCBIRB+1
UCBRV039 EQU   UCBGRAF
UCBDWNR  EQU   UCBGRAF
UCBRPND  EQU   UCBGRAF
UCBUPM   EQU   UCBGRAF
UCBBTAM  EQU   UCBGRAF
UCBDRNO  EQU   UCBGRAF
UCBDRO   EQU   UCBGRAF
UCBOIP   EQU   UCBGRAF
UCBATRCD EQU   @NM00192
UCBSKPFG EQU   @NM00192
UCBRIPND EQU   @NM00192
UCBRTIAC EQU   @NM00192
UCBRSV79 EQU   @NM00192
UCBRSV78 EQU   @NM00192
UCBRSV77 EQU   @NM00192
UCBOLTEP EQU   @NM00192
UCBATNCT EQU   UCBOBS06+2
UCBRSV76 EQU   UCBAOF2
UCBRSV75 EQU   UCBAOF2
UCBRSV74 EQU   UCBAOF2
UCBRSV73 EQU   UCBAOF2
UCBRSV72 EQU   UCBAOF2
UCBRSV71 EQU   UCBAOF2
UCBRSV70 EQU   UCBAOF2
UCBRSV69 EQU   UCBAOF2
UCBRSV68 EQU   UCBAOF1
UCBRSV67 EQU   UCBAOF1
UCBRSV66 EQU   UCBAOF1
UCBRSV65 EQU   UCBAOF1
UCBRSV64 EQU   UCBAOF1
UCBOFNL  EQU   UCBAOF1
UCBOFSP  EQU   UCBAOF1
UCBOFMCR EQU   UCBAOF1
UCBBTB   EQU   UCBBTA+1
UCBDI    EQU   UCBBTA
UCBSNS   EQU   UCBOBS04+8
UCBTEB   EQU   UCBOBS04+4
UCBGCB   EQU   UCBOBS04+3
UCBOPEN  EQU   UCBOBS04+2
UCBSTART EQU   UCBOBS04
UCBXTNB  EQU   UCBXTN+1
UCBRSV22 EQU   UCBVOPT
UCBRSV21 EQU   UCBVOPT
UCBRSV20 EQU   UCBVOPT
UCBESVE  EQU   UCBVOPT
UCBERPC  EQU   UCBVOPT
UCBESVC  EQU   UCBVOPT
UCBEVA   EQU   UCBVOPT
UCBESV   EQU   UCBVOPT
UCBRV009 EQU   UCBTFL1
UCBRV008 EQU   UCBTFL1
UCBRV007 EQU   UCBTFL1
UCBRV006 EQU   UCBTFL1
UCBRV005 EQU   UCBTFL1
UCBRV038 EQU   UCBTFL1
UCBNSLTP EQU   UCBTFL1
UCBNLTP  EQU   UCBTFL1
UCBRES1B EQU   UCBOBS02+18
UCBFSER  EQU   UCBOBS02+12
@NM00191 EQU   UCBOBS02+4
UCBFSEQ  EQU   UCBOBS02+2
UCBFSCT  EQU   UCBOBS02
UCBNEXP  EQU   UCBOBS01+20
UCBBASE  EQU   UCBOBS01+16
UCBRES1A EQU   UCBOBS01+15
UCBUSER  EQU   UCBOBS01+14
UCBFL47  EQU   UCBFL4
UCBFL46  EQU   UCBFL4
UCBFL45  EQU   UCBFL4
UCBRV065 EQU   UCBFL4
UCBDSS   EQU   UCBFL4
UCBRV028 EQU   UCBFL4
UCBWDAV  EQU   UCBFL4
UCBDAVV  EQU   UCBFL4
UCBSQC   EQU   UCBOBS01+12
UCBDATP  EQU   UCBOBS01+12
UCBDMC   EQU   UCBDMCT
UCBMOUNT EQU   UCBDMCT
UCBBNUL  EQU   UCBSTAB
UCBSHAR  EQU   UCBSTAB
UCBBSTR  EQU   UCBSTAB
UCBBPUB  EQU   UCBSTAB
UCBBPRV  EQU   UCBSTAB
UCBBALB  EQU   UCBPRSRS
UCBPGFL  EQU   UCBSTAB
UCBDVSHR EQU   UCBBSVL
UCBVTOC  EQU   UCBOBS01
@NM00142 EQU   CRGATTR
CUPD     EQU   CUPG
CPGSP    EQU   @NM00139
@NM00140 EQU   @NM00139
CDOSUINF EQU   CTGRBASE+84
@NM00137 EQU   COPEN
COPENDS  EQU   COPEN
CNUS     EQU   CATTR2
@NM00136 EQU   CATTR2
CATTRSSH EQU   CATTR2
CATTRISH EQU   CATTR2
CTRKOV   EQU   CATTR1
CTEMP    EQU   CATTR1
CINHIB   EQU   CATTR1
CRECOV   EQU   CATTR1
CERASE   EQU   CATTR1
CREUSE   EQU   CATTR1
CUNIQ    EQU   CATTR1
CSP      EQU   CATTR1
@NM00113 EQU   MCATDSTR+1
@NM00112 EQU   CVTAMF1
AMF1NONX EQU   CVTAMF1
AMF1OPEN EQU   CVTAMF1
PSARSAV  EQU   PSARSVTE+60
PSARSTK  EQU   PSARSVTE+56
PSAESAV3 EQU   PSARSVTE+52
PSAESTK3 EQU   PSARSVTE+48
PSAESAV2 EQU   PSARSVTE+44
PSAESTK2 EQU   PSARSVTE+40
PSAESAV1 EQU   PSARSVTE+36
PSAESTK1 EQU   PSARSVTE+32
PSAPSAV  EQU   PSARSVTE+28
PSAPSTK  EQU   PSARSVTE+24
PSAMSAV  EQU   PSARSVTE+20
PSAMSTK  EQU   PSARSVTE+16
PSASSAV  EQU   PSARSVTE+12
PSASSTK  EQU   PSARSVTE+8
PSANSTK  EQU   PSARSVTE+4
PSACSTK  EQU   PSARSVTE
CVTRV628 EQU   CVTS01+728
CVTRV627 EQU   CVTS01+724
CVTRV626 EQU   CVTS01+720
CVTRV625 EQU   CVTS01+716
CVTRV624 EQU   CVTS01+712
CVTRV623 EQU   CVTS01+708
CVTRV622 EQU   CVTS01+704
CVTRV621 EQU   CVTS01+700
CVTIHASU EQU   CVTS01+696
CVTSUSP  EQU   CVTS01+692
CVTT6SVC EQU   CVTS01+688
CVTCDAL  EQU   CVTS01+684
CVTTCTL  EQU   CVTS01+680
CVTRSUME EQU   CVTS01+676
CVTJTERM EQU   CVTS01+672
CVTASMRM EQU   CVTS01+668
CVTTCASP EQU   CVTS01+664
CVT0PT03 EQU   CVTS01+660
CVT0PT0E EQU   CVTS01+656
CVTRV609 EQU   CVTS01+652
CVTCGK   EQU   CVTS01+648
CVTRAC   EQU   CVTS01+644
CVTRV606 EQU   CVTS01+640
CVTRV605 EQU   CVTS01+636
CVTRV604 EQU   CVTS01+632
CVTEFF02 EQU   CVTS01+628
CVTCBBR  EQU   CVTS01+624
CVTSSCR  EQU   CVTS01+620
CVTEVENT EQU   CVTS01+616
CVTCRCA  EQU   CVTS01+612
CVTTPIO  EQU   CVTS01+608
CVTADV   EQU   CVTS01+604
CVTGSDAB EQU   CVTGSDA
CVTQV3   EQU   CVTS01+596
CVTQV2   EQU   CVTS01+592
CVTQV1   EQU   CVTS01+588
CVTRPT   EQU   CVTS01+584
CVTSSRB  EQU   CVTS01+580
CVTCSDRL EQU   CVTS01+576
CVTEXP1  EQU   CVTS01+572
CVTRMPMT EQU   CVTS01+568
CVTRMPTT EQU   CVTS01+564
CVTVPSA  EQU   CVTS01+560
CVTVSTOP EQU   CVTS01+556
CVTGTFR8 EQU   CVTS01+552
CVTQUIT  EQU   CVTS01+548
CVTVACR  EQU   CVTS01+544
CVTWTCB  EQU   CVTS01+540
CVTSTPRS EQU   CVTS01+536
CVT0PT02 EQU   CVTS01+532
CVTDARCM EQU   CVTS01+528
CVTIRECM EQU   CVTS01+524
CVTJRECM EQU   CVTS01+520
CVTVEMS0 EQU   CVTS01+516
CVTSPFRR EQU   CVTS01+512
CVTRLSTG EQU   CVTS01+508
CVT0TC0A EQU   CVTS01+504
CVTGMBR  EQU   CVTS01+500
CVTLFRM  EQU   CVTS01+496
CVTRMBR  EQU   CVTS01+492
CVTVIOP  EQU   CVTS01+488
CVTRV307 EQU   CVTS01+486
CVTRV306 EQU   CVTS01+484
CVTRV305 EQU   CVTS01+482
CVTRV304 EQU   CVTS01+480
CVTRV303 EQU   CVTS01+478
CVTRV302 EQU   CVTS01+476
CVTTRCA  EQU   CVTS01+472
CVTRV297 EQU   CVTRV289
CVTRV296 EQU   CVTRV289
CVTRV295 EQU   CVTRV289
CVTRV294 EQU   CVTRV289
CVTRV293 EQU   CVTRV289
CVTRV292 EQU   CVTRV289
CVTRV291 EQU   CVTRV289
CVTRV290 EQU   CVTRV289
CVTRV288 EQU   CVTRV280
CVTRV287 EQU   CVTRV280
CVTRV286 EQU   CVTRV280
CVTRV285 EQU   CVTRV280
CVTRV284 EQU   CVTRV280
CVTRV283 EQU   CVTRV280
CVTRV282 EQU   CVTRV280
CVTRV281 EQU   CVTRV280
CVTRV279 EQU   CVTRV271
CVTRV278 EQU   CVTRV271
CVTRV277 EQU   CVTRV271
CVTRV276 EQU   CVTRV271
CVTRV275 EQU   CVTRV271
CVTRV274 EQU   CVTRV271
CVTRV273 EQU   CVTRV271
CVTRV272 EQU   CVTRV271
CVTRV270 EQU   CVTRV262
CVTRV269 EQU   CVTRV262
CVTRV268 EQU   CVTRV262
CVTRV267 EQU   CVTRV262
CVTRV266 EQU   CVTRV262
CVTRV265 EQU   CVTRV262
CVTRV264 EQU   CVTRV262
CVTRV263 EQU   CVTRV262
CVTVFP   EQU   CVTS01+464
CVTVSI   EQU   CVTS01+460
CVTVPSIB EQU   CVTS01+456
CVTMFACT EQU   CVTMFRTR
CVTMFCTL EQU   CVTS01+448
CVTPVBP  EQU   CVTS01+444
CVTPWI   EQU   CVTS01+440
CVTRV254 EQU   CVTS01+438
CVTRV253 EQU   CVTS01+436
CVTRV252 EQU   CVTS01+434
CVTRV251 EQU   CVTS01+433
CVTRV250 EQU   CVTS01+432
CVTRV249 EQU   CVTS01+431
CVTRV248 EQU   CVTS01+430
CVTRV247 EQU   CVTS01+429
CVTRV246 EQU   CVTS01+428
CVTRV245 EQU   CVTRV237
CVTRV244 EQU   CVTRV237
CVTRV243 EQU   CVTRV237
CVTRV242 EQU   CVTRV237
CVTRV241 EQU   CVTRV237
CVTRV240 EQU   CVTRV237
CVTRV239 EQU   CVTRV237
CVTRV238 EQU   CVTRV237
CVTRV236 EQU   CVTRV228
CVTRV235 EQU   CVTRV228
CVTRV234 EQU   CVTRV228
CVTRV233 EQU   CVTRV228
CVTRV232 EQU   CVTRV228
CVTRV231 EQU   CVTRV228
CVTRV230 EQU   CVTRV228
CVTRV229 EQU   CVTRV228
CVTRV227 EQU   CVTRV219
CVTRV226 EQU   CVTRV219
CVTRV225 EQU   CVTRV219
CVTRV224 EQU   CVTRV219
CVTRV223 EQU   CVTRV219
CVTRV222 EQU   CVTRV219
CVTRV221 EQU   CVTRV219
CVTRV220 EQU   CVTRV219
CVTRV218 EQU   CVTRV210
CVTRV217 EQU   CVTRV210
CVTRV216 EQU   CVTRV210
CVTRV215 EQU   CVTRV210
CVTRV214 EQU   CVTRV210
CVTRV213 EQU   CVTRV210
CVTRV212 EQU   CVTRV210
CVTRV211 EQU   CVTRV210
CVTLCCAT EQU   CVTS01+420
CVTPCCAT EQU   CVTS01+416
CVTIPCRP EQU   CVTS01+412
CVTIPCRI EQU   CVTS01+408
CVTIPCDS EQU   CVTS01+404
CVTAIDVT EQU   CVTS01+400
CVTSSAP  EQU   CVTS01+396
CVTEHCIR EQU   CVTS01+392
CVTEHDEF EQU   CVTS01+388
CVTDAIR  EQU   CVTS01+384
CVTPERFM EQU   CVTS01+380
CVT044R2 EQU   CVTS01+376
CVTFETCH EQU   CVTS01+372
CVTRSTWD EQU   CVTS01+368
CVTSPOST EQU   CVTS01+364
CVTIOBP  EQU   CVTS01+360
CVTASMVT EQU   CVTS01+356
CVTRECRQ EQU   CVTS01+352
CVTWSAC  EQU   CVTS01+348
CVTRV149 EQU   CVTS01+344
CVTWSAL  EQU   CVTS01+340
CVTSPSA  EQU   CVTS01+336
CVTGLMN  EQU   CVTS01+332
CVTVEAC0 EQU   CVTS01+328
CVT062R1 EQU   CVTS01+324
CVTRPOST EQU   CVTS01+320
CVTDQIQE EQU   CVTS01+316
CVTCSD   EQU   CVTS01+312
CVTLKRMA EQU   CVTS01+308
CVTRSPIE EQU   CVTS01+304
CVTRENQ  EQU   CVTS01+300
CVTLQCB  EQU   CVTS01+296
CVTFQCB  EQU   CVTS01+292
CVTQCS01 EQU   CVTS01+288
CVTAPFT  EQU   CVTS01+284
CVTPARRL EQU   CVTS01+280
CVTVWAIT EQU   CVTS01+276
CVTGSPL  EQU   CVTS01+272
CVTLSMQ  EQU   CVTS01+268
CVTGSMQ  EQU   CVTS01+264
CVTEXPRO EQU   CVTS01+260
CVTOPCTP EQU   CVTS01+256
CVTSIC   EQU   CVTS01+252
CVTTPIOS EQU   CVTS01+248
CVTRTMS  EQU   CVTS01+244
CVTSDBF  EQU   CVTS01+240
CVTSCBP  EQU   CVTS01+236
CVTSDMP  EQU   CVTS01+232
CVTSV60  EQU   CVTS01+228
CVTRTMCT EQU   CVTS01+224
CVTASCBL EQU   CVTS01+220
CVTASCBH EQU   CVTS01+216
CVTGDA   EQU   CVTS01+212
CVTASVT  EQU   CVTS01+208
CVTVVMDI EQU   CVTS01+204
CVTAQTOP EQU   CVTS01+200
CVTIOSCS EQU   CVTS01+196
CVTSDRM  EQU   CVTS01+192
CVTOPTE  EQU   CVTS01+188
CVTSTXU  EQU   CVTS01+184
CVTQUIS  EQU   CVTS01+180
CVTPARS  EQU   CVTS01+176
CVTS1EE  EQU   CVTS01+172
CVTFRAS  EQU   CVTS01+168
CVTQSAS  EQU   CVTS01+164
CVTCRAS  EQU   CVTS01+160
CVTCRMN  EQU   CVTS01+156
CVTDELCP EQU   CVTS01+152
CVTFRECL EQU   CVTS01+148
CVTGETCL EQU   CVTS01+144
CVTBLDCP EQU   CVTS01+140
CVTAUTHL EQU   CVTS01+136
CVTSCAN  EQU   CVTS01+132
CVTRV144 EQU   CVTS01+130
CVTMAXMP EQU   CVTS01+128
CVTSTCK  EQU   CVTS01+124
CVTRV139 EQU   CVTS01+123
CVTDSSAC EQU   CVTS01+122
CVTRV513 EQU   CVTS01+121
CVTIOSPL EQU   CVTS01+120
CVTPTGT  EQU   CVTS01+116
CVTCSPIE EQU   CVTS01+112
CVTSMFEX EQU   CVTS01+108
CVTOLT0A EQU   CVTS01+104
CVTSRBRT EQU   CVTS01+100
CVTPUTL  EQU   CVTS01+96
CVTASCRL EQU   CVTS01+92
CVTASCRF EQU   CVTS01+88
CVTRV326 EQU   CVTS01+84
CVTRV325 EQU   CVTS01+80
CVTRV324 EQU   CVTS01+76
CVT0VL01 EQU   CVTS01+72
CVTSHRVM EQU   CVTS01+68
CVTRV332 EQU   CVTS01+64
CVTTAS   EQU   CVTS01+60
CVTRSCN  EQU   CVTS01+56
CVTTRAC2 EQU   CVTS01+54
CVTTRACE EQU   CVTS01+52
CVTAPG   EQU   CVTS01+51
CVTSDTRC EQU   CVTCTLFG
CVTGTRCE EQU   CVTCTLFG
CVTNOMP  EQU   CVTCTLFG
CVTRSV79 EQU   CVTCTLFG
CVTDSTAT EQU   CVTCTLFG
CVTRSV78 EQU   CVTCTLFG
CVTRV333 EQU   CVTCTLFG
CVTRV323 EQU   CVTCTLFG
CVTSPVLK EQU   CVTS01+49
CVTRSV77 EQU   CVTS01+48
CVTRV331 EQU   CVTS01+44
CVTRV330 EQU   CVTS01+40
CVTRV329 EQU   CVTS01+36
CVTRV328 EQU   CVTS01+32
CVTRV322 EQU   CVTS01+28
CVTSLID  EQU   CVTSLIDA+1
CVTSYLK  EQU   CVTSLIDA
CVTRV321 EQU   CVTS01+20
CVTRV320 EQU   CVTS01+16
CVTLPDIR EQU   CVTLPDIA+1
CVTRSV69 EQU   CVTDIRST
CVTRSV68 EQU   CVTDIRST
CVTRSV67 EQU   CVTDIRST
CVTRSV66 EQU   CVTDIRST
CVTRSV65 EQU   CVTDIRST
CVTRSV64 EQU   CVTDIRST
CVTRSV63 EQU   CVTDIRST
CVTDICOM EQU   CVTDIRST
CVTPVTP  EQU   CVTS01+8
CVTLPDSR EQU   CVTS01+4
CVTGETL  EQU   CVTS01
CTGREAS2 EQU   CTGREASN+1
CTGREAS1 EQU   CTGREASN
CTGMODID EQU   CTGPROB
WKAICI   EQU   WKAINPUT+2
WKAIEXT  EQU   WKAINPUT
BUFRECRD EQU   BUFVOLRD
DCRARA3  EQU   DCRARABS+20
DCRARAB3 EQU   DCRARABS
@NM00231 EQU   DCRACI5+60
@NM00230 EQU   DCRACI5+55
@NM00229 EQU   DCRACI5+49
@NM00228 EQU   DCRACI5
@NM00227 EQU   DCRACI15
@NM00226 EQU   DCRADSVL+2
@NM00225 EQU   DCRADSVL
@NM00224 EQU   DCRAVGO+53
DCRAVGOM EQU   DCRAVGO+2
@NM00223 EQU   DCRAVGO
@NM00222 EQU   DCRAVFPL+16
@NM00221 EQU   DCRAVFPL
DCRANAME EQU   DCRASSOC+1
DCRATYPE EQU   DCRASSOC
@NM00220 EQU   DCRASFPL+16
@NM00219 EQU   DCRASFPL
DCRAMDSB EQU   DCRAWKA+176
@NM00218 EQU   DCRAMFPL+16
@NM00217 EQU   DCRAMFPL
@NM00216 EQU   DCRAMCPL+36
@NM00215 EQU   DCRAMCPL+32
@NM00214 EQU   DCRAMCPL+28
@NM00213 EQU   DCRAMCPL
AMBVIOT  EQU   IDAAMB+116
AMBSZCP  EQU   IDAAMB+114
AMBSZFW  EQU   IDAAMB+112
AMBSZWR  EQU   IDAAMB+110
AMBSZRD  EQU   IDAAMB+108
AMBEXEX  EQU   IDAAMB+100
AMBWSHD  EQU   IDAAMB+96
AMBCPA   EQU   IDAAMB+92
AMBBM2SH EQU   IDAAMB+88
AMBRDCNT EQU   AMBCSWD1+2
@NM00212 EQU   AMBCSWD1+1
@NM00211 EQU   AMBAFLG
AMBCFX   EQU   AMBAFLG
AMBSIS   EQU   AMBAFLG
AMBDFR   EQU   AMBAFLG
AMBICI   EQU   AMBAFLG
AMBGSR   EQU   AMBAFLG
AMBLSR   EQU   AMBAFLG
@NM00210 EQU   AMBAFLG
AMBUPLH  EQU   IDAAMB+80
AMBPAMBL EQU   IDAAMB+76
AMBPIXP  EQU   IDAAMB+72
@NM00209 EQU   IDAAMB+68
AMBIWA   EQU   IDAAMB+64
AMBWKA   EQU   IDAAMB+60
AMBAMBXN EQU   AMBEOVPT
AMBEDB   EQU   IDAAMB+52
AMBRPT   EQU   IDAAMB+50
AMBPUG   EQU   AMBFLG2
AMBBUSY  EQU   AMBOFLGS
AMBEXFG  EQU   AMBOFLGS
@NM00208 EQU   AMBOFLGS
AMBOPEN  EQU   AMBOFLGS
@NM00207 EQU   AMBOFLGS
AMBDEBAD EQU   AMBDEBPT+1
AMBIFLGS EQU   AMBDEBPT
AMBAMETH EQU   IDAAMB+43
AMBSDS   EQU   AMBINFL
AMBUPX   EQU   AMBINFL
AMBUCRA  EQU   AMBINFL
AMBSCRA  EQU   AMBINFL
AMBCAT   EQU   AMBINFL
@NM00206 EQU   AMBINFL
@NM00205 EQU   AMBINFL
AMBTIOT  EQU   IDAAMB+40
@NM00204 EQU   IDAAMB+38
AMBDDSN  EQU   IDAAMB+35
AMBCDSN  EQU   IDAAMB+32
AMBIOMB  EQU   AMBIOBAD
@NM00203 EQU   AMBDSORG+1
AMBDORGA EQU   AMBDSORG+1
@NM00202 EQU   AMBDSORG
AMBINBUF EQU   AMBFLG1
AMBJRN   EQU   AMBFLG1
AMBUBF   EQU   AMBFLG1
AMBSPEED EQU   AMBFLG1
AMBPCAT  EQU   AMBFLG1
AMBMCAT  EQU   AMBFLG1
AMBTYPE  EQU   AMBFLG1
AMBCREAT EQU   AMBFLG1
@NM00201 EQU   AMBFLG0
AMBSWSP  EQU   AMBFLG0
AMBPSDS  EQU   AMBFLG0
AMBDSB   EQU   IDAAMB+20
AMBCACB  EQU   IDAAMB+16
AMBPH    EQU   IDAAMB+12
AMBBUFC  EQU   IDAAMB+8
AMBLINK  EQU   IDAAMB+4
AMBLEN   EQU   IDAAMB+2
AMBRSC   EQU   IDAAMB+1
AMBID    EQU   IDAAMB
AMBLCMB  EQU   IDAAMBL+64
AMBLBIB  EQU   IDAAMBL+60
AMBLIX   EQU   IDAAMBL+56
AMBLDTA  EQU   IDAAMBL+52
AMBLMIDS EQU   IDAAMBL+42
AMBLNIDS EQU   IDAAMBL+41
@NM00200 EQU   IDAAMBL+40
AMBLNUM  EQU   IDAAMBL+38
AMBLNST  EQU   IDAAMBL+37
@NM00199 EQU   AMBLFLG2
AMBLSTAG EQU   AMBLFLG2
@NM00198 EQU   AMBLFLG2
AMBLDUMY EQU   AMBLFLG1
AMBLCAT  EQU   AMBLFLG1
AMBLUCRA EQU   AMBLFLG1
AMBLSCRA EQU   AMBLFLG1
AMBLVVIC EQU   AMBLFLG1
AMBLUPD  EQU   AMBLFLG1
AMBLCINV EQU   AMBLFLG1
AMBLFULL EQU   AMBLFLG1
AMBLLEN  EQU   IDAAMBL+34
@NM00197 EQU   AMBLSHAR
AMBLWRIT EQU   AMBLSHAR
AMBLCATO EQU   AMBLSHAR
AMBLPRIM EQU   AMBLSHAR
AMBLID   EQU   IDAAMBL+32
@NM00196 EQU   IDAAMBL+31
@NM00195 EQU   AMBLTYPE
AMBLFIX  EQU   AMBLTYPE
AMBLBASE EQU   AMBLTYPE
AMBLAIX  EQU   AMBLTYPE
AMBLUPGR EQU   AMBLTYPE
AMBLPATH EQU   AMBLTYPE
AMBLENO  EQU   AMBLVC+1
AMBLVRT  EQU   AMBLVC
AMBLXPT  EQU   IDAAMBL+24
AMBLDFR  EQU   AMBLQ
AMBLESDS EQU   AMBLQ
AMBLKSDS EQU   AMBLQ
AMBLUBF  EQU   AMBLQ
AMBLFSTP EQU   AMBLQ
AMBLLSR  EQU   AMBLQ
AMBLGSR  EQU   AMBLQ
AMBLDDC  EQU   AMBLQ
AMBLDCI  EQU   AMBLIDF+4
AMBLCACB EQU   AMBLIDF
@NM00194 EQU   AMBLEOV+2
AMBLCOMP EQU   AMBLEOV+1
AMBLESET EQU   AMBLEFLG
AMBLWAIT EQU   AMBLEFLG
AMBLACB  EQU   IDAAMBL+8
AMBLSCHN EQU   IDAAMBL+4
AMBLPCHN EQU   IDAAMBL
UCBERADR EQU   UCBUCS+12
UCBFCBID EQU   UCBUCS+8
UCBERCNT EQU   UCBUCS+7
UCBRSV51 EQU   UCBUCS+6
UCBRSV50 EQU   UCBFCBOP
UCBRSV49 EQU   UCBFCBOP
UCBRSV48 EQU   UCBFCBOP
UCBRSV47 EQU   UCBFCBOP
UCBRSV46 EQU   UCBFCBOP
UCBRSV45 EQU   UCBFCBOP
UCBRSV44 EQU   UCBFCBOP
UCBFCBO1 EQU   UCBFCBOP
UCBUCSPE EQU   UCBUCSOP
UCBRSV43 EQU   UCBUCSOP
UCBRSV42 EQU   UCBUCSOP
UCBRSV41 EQU   UCBUCSOP
UCBRSV40 EQU   UCBUCSOP
UCBRSV39 EQU   UCBUCSOP
UCBUCSO2 EQU   UCBUCSOP
UCBUCSO1 EQU   UCBUCSOP
UCBUCSID EQU   UCBUCS
UCBMDRBA EQU   UCBMDRBF+1
UCBRV075 EQU   UCBMDRBF
UCBRV074 EQU   UCB3800X+32
UCBIMAGE EQU   UCB3800X+28
UCBFCBNM EQU   UCB3800X+24
UCBCHAR4 EQU   UCB3800X+20
UCBCHAR3 EQU   UCB3800X+16
UCBCHAR2 EQU   UCB3800X+12
UCBCHAR1 EQU   UCB3800X+8
UCBCGMID EQU   UCB3800X+4
UCBBRSTA EQU   UCBACTIV
UCBRV063 EQU   UCBACTIV
UCBRV062 EQU   UCBACTIV
UCBRV061 EQU   UCBACTIV
UCBRV060 EQU   UCBACTIV
UCBRV059 EQU   UCBACTIV
UCBRV058 EQU   UCBACTIV
UCBRV057 EQU   UCBACTIV
UCBRV050 EQU   UCB3800X+2
UCBCGMNO EQU   UCB3800X+1
UCBRV083 EQU   UCBOPTNS
UCBBRSTR EQU   UCBOPTNS
UCBRV056 EQU   UCBOPTNS
UCBRV055 EQU   UCBOPTNS
UCBRV054 EQU   UCBOPTNS
UCBRV053 EQU   UCBOPTNS
UCBRV052 EQU   UCBOPTNS
UCBRV051 EQU   UCBOPTNS
UCBRV073 EQU   UCB3540X+7
UCBRV072 EQU   UCBDKBYT
UCBRV071 EQU   UCBDKBYT
UCBRV070 EQU   UCBDKBYT
UCBRV069 EQU   UCBDKBYT
UCBRV068 EQU   UCBDKBYT
UCBRV067 EQU   UCBDKBYT
UCBVLVER EQU   UCBDKBYT
UCBDKAMX EQU   UCBDKBYT
UCBVLSER EQU   UCB3540X
UCBRDATA EQU   UCBOCR+4
UCBFRID  EQU   UCBOCR
UCBCLN   EQU   UCBMT+14
UCBERG   EQU   UCBMT+12
UCBMS    EQU   UCBMT+11
UCBNB    EQU   UCBMT+10
UCBPW    EQU   UCBMT+9
UCBPR    EQU   UCBMT+8
UCBSIO   EQU   UCBMT+6
UCBTW    EQU   UCBMT+5
UCBTR    EQU   UCBMT+4
UCBTWT   EQU   UCBMT+3
UCBTRT   EQU   UCBMT+2
UCBCTD   EQU   UCBMT
UCBWTOID EQU   UCBCMEXT+17
UCBRV043 EQU   UCBCMEXT+16
UCBASID  EQU   UCBCMEXT+14
UCBMFCNT EQU   UCBCMEXT+12
UCBPMSK  EQU   UCBCMEXT+10
UCBCCWOF EQU   UCBCMEXT+8
UCBRV041 EQU   UCBCMEXT+6
UCBRV036 EQU   UCBFLP1
UCBRV035 EQU   UCBFLP1
UCBERLOG EQU   UCBFLP1
UCBRV033 EQU   UCBFLP1
UCBRV032 EQU   UCBFLP1
UCBRV031 EQU   UCBFLP1
UCBSHRUP EQU   UCBFLP1
UCBNSRCH EQU   UCBFLP1
UCBSNSCT EQU   UCBCMEXT+4
UCBHPDV  EQU   @NM00193
UCBHALI  EQU   @NM00193
UCBRSV09 EQU   @NM00193
UCBRSV08 EQU   @NM00193
UCBRSV07 EQU   @NM00193
UCBRSV06 EQU   @NM00193
UCBRSV05 EQU   @NM00193
UCBRSV04 EQU   @NM00193
UCBDTI   EQU   UCBCMEXT+2
UCBSTI   EQU   UCBCMEXT+1
UCBETI   EQU   UCBCMEXT
UCBEXTP  EQU   UCBEXTPT+1
UCBDDRSW EQU   UCBFLC
UCBTICBT EQU   UCBFLC
UCBIVRR  EQU   UCBFLC
UCBIVRS  EQU   UCBFLC
UCBITF   EQU   UCBFLC
UCBUDE   EQU   UCBFLC
UCBWAA   EQU   UCBFLC
UCBATTP  EQU   UCBFLC
UCBUNTYP EQU   UCBTBYT4
UCBRSV11 EQU   UCBDVCLS
UCBRSV10 EQU   UCBDVCLS
UCB3CHAR EQU   UCBDVCLS
UCB3UREC EQU   UCBDVCLS
UCB3DISP EQU   UCBDVCLS
UCB3DACC EQU   UCBDVCLS
UCB3COMM EQU   UCBDVCLS
UCB3TAPE EQU   UCBDVCLS
UCBDVPWR EQU   UCB2OPT7
UCBVLPWR EQU   UCB2OPT6
UCB2OPT5 EQU   UCBTBYT2
UCBRVDEV EQU   UCBRWTAU
UCBRPS   EQU   UCBDUDN2
UCBRR    EQU   UCBDUDN1
UCB2OPT1 EQU   UCBTBYT2
UCB2OPT0 EQU   UCBTBYT2
UCB1FEA7 EQU   UCBTBYT1
UCBD6250 EQU   UCB1FEA6
UCBD1600 EQU   UCB1FEA5
UCB1FEA4 EQU   UCBTBYT1
UCB1FEA3 EQU   UCBTBYT1
UCB1FEA2 EQU   UCBTBYT1
UCB1FEA1 EQU   UCBTBYT1
UCB1FEA0 EQU   UCBTBYT1
UCBNAME  EQU   UCBOB+13
UCBRV029 EQU   UCBWGT
UCBVHRSN EQU   UCBWGT
UCBVORSN EQU   UCBWGT
UCBMTPXP EQU   UCBWGT
UCBREW   EQU   UCBWGT
UCBPUB   EQU   UCBWGT
UCBOUT   EQU   UCBWGT
UCBIN    EQU   UCBWGT
UCBCPU   EQU   UCBOB+11
UCBLCI   EQU   UCBOB+10
UCBCNT   EQU   UCBOB+9
UCBRV017 EQU   UCBCHM1
UCBRV016 EQU   UCBCHM1
UCBRV015 EQU   UCBCHM1
UCBRV014 EQU   UCBCHM1
UCBSPB   EQU   UCBPTH1
UCBPPB   EQU   UCBPTH1
UCBSPA   EQU   UCBPTH0
UCBPPA   EQU   UCBPTH0
UCBSIGP  EQU   UCBFLB
UCBVALPH EQU   UCBFLB
UCBCRHSN EQU   UCBFLB
UCBCRHRV EQU   UCBFLB
UCBRESVH EQU   UCBFLB
UCBSPST  EQU   UCBFLB
UCBASNS  EQU   UCBFLB
UCBIORST EQU   UCBFLB
UCBQISCE EQU   UCBFL1
UCBACTV  EQU   UCBFL1
UCBSAP   EQU   UCBFL1
UCBNOTRC EQU   UCBCUB
UCBPSNS  EQU   UCBFL1
UCBUSING EQU   UCBPST
UCBNOTRD EQU   UCBNRY
UCBBUSYD EQU   UCBBSY
UCBUA    EQU   UCBCHAN+1
UCBCHA   EQU   UCBCHAN
UCBDADI  EQU   UCBSTAT
UCBSYSR  EQU   UCBSTAT
UCBPRES  EQU   UCBSTAT
UCBALOC  EQU   UCBSTAT
UCBUNLD  EQU   UCBSTAT
UCBRESV  EQU   UCBSTAT
UCBCHGS  EQU   UCBSTAT
UCBONLI  EQU   UCBSTAT
UCBID    EQU   UCBOB+2
UCBALTPH EQU   UCBFL5
UCBALTCU EQU   UCBFL5
UCBNALOC EQU   UCBFL5
UCBENVRD EQU   UCBFL5
UCBVSDR  EQU   UCBFL5
UCBSASK  EQU   UCBFL5
UCBAMV   EQU   UCBAF
UCBDCC   EQU   UCBFL5
UCBMONT  EQU   UCBJBNR
UCBRV011 EQU   UCBJBNR
UCBRSV02 EQU   UCBJBNR
UCBOLDSM EQU   UCBJBNR
UCBRV003 EQU   UCBJBNR
UCBDUC   EQU   UCBJBNR
UCBJES3  EQU   UCBJBNR
UCBVRDEV EQU   UCBJBNR
UCBCMSEG EQU   UCBOB
UCBPFXND EQU   UCB+512
UCBIOQ   EQU   UCB+508
UCBLOCK  EQU   UCB+504
UCBPXST  EQU   UCB+504
@NM00190 EQU   UCB
DS4END   EQU   IECSDSL4+96
@NM00189 EQU   IECSDSL4+71
DS4VTOCE EQU   IECSDSL4+61
DS4F6PTR EQU   IECSDSL4+56
@NM00188 EQU   IECSDSL4+51
DS4R2TIM EQU   IECSDSL4+43
@NM00187 EQU   DS4VSIND
DS4VSBAD EQU   DS4VSIND
DS4VSREF EQU   DS4VSIND
DS4AMTIM EQU   IECSDSL4+32
DS4DEVDB EQU   DS4DEVCT+13
DS4DEVDT EQU   DS4DEVCT+12
DS4DEVTL EQU   DS4DEVCT+10
DS4DEVFG EQU   DS4DEVCT+9
DS4DEVK  EQU   DS4DEVCT+8
DS4DEVL  EQU   DS4DEVOV+1
DS4DEVI  EQU   DS4DEVOV
DS4DEVTK EQU   DS4DEVCT+4
DS4DEVSZ EQU   DS4DEVCT
@NM00186 EQU   IECSDSL4+16
DS4NOEXT EQU   IECSDSL4+15
DS4VTOCI EQU   IECSDSL4+14
DS4NOATK EQU   IECSDSL4+12
DS4HCCHH EQU   IECSDSL4+8
DS4DSREC EQU   IECSDSL4+6
DS4HPCHR EQU   IECSDSL4+1
DS4IDFMT EQU   IECSDSL4
@NM00185 EQU   IECSDSF4
F4WCCW3  EQU   F4WCCW+16
F4WCCW2  EQU   F4WCCW+8
F4WCCW1  EQU   F4WCCW
F4WSEEK  EQU   IGGMF4WA+88
@NM00184 EQU   F4WIOB+24
F4WIDCB  EQU   F4WIOB+20
F4WSCP   EQU   F4WIOB+16
@NM00183 EQU   F4WIOB+8
F4WIECB  EQU   F4WIOB+4
@NM00182 EQU   F4WIOB
@NM00181 EQU   F4WECB+1
F4WECMP  EQU   F4WECB
F4WDEBP  EQU   IGGMF4WA+48
F4WNTK   EQU   F4WDEB+46
F4WECC   EQU   F4WDEB+42
F4WSCC   EQU   F4WDEB+38
F4WDBB   EQU   F4WDEB+36
F4WDMOD  EQU   F4WDEB+32
F4WDAPP  EQU   F4WDEB+29
F4WDEXS  EQU   F4WDEB+28
F4WDDCB  EQU   F4WDEB+25
F4WDID   EQU   F4WDEB+24
@NM00180 EQU   F4WDEB+17
F4WEXT   EQU   F4WDEB+16
@NM00179 EQU   F4WDEB
RPLERMSA EQU   RPLCOMN+72
RPLEMLEN EQU   RPLCOMN+70
RPLACTIV EQU   RPLCOMN+69
RPLBRANC EQU   RPLEXTD1
RPLNIB   EQU   RPLEXTD1
@NM00178 EQU   RPLEXTD1
RPLEXIT  EQU   RPLEXTD1
RPLNEXIT EQU   RPLEXTD1
RPLEXSCH EQU   RPLEXTD1
RPLDDDD  EQU   RPLRBAR+4
@NM00177 EQU   RPLRBAR+3
@NM00176 EQU   RPLAIXID
RPLAXPKP EQU   RPLAIXID
RPLAIXPC EQU   RPLRBAR
RPLRSV88 EQU   RPLOPT8
RPLUNCON EQU   RPLOPT8
RPLACTV  EQU   RPLOPT8
RPLSESS  EQU   RPLOPT8
RPLPEND  EQU   RPLOPT8
RPLODPRM EQU   RPLOPT8
RPLODACP EQU   RPLOPT8
RPLODACQ EQU   RPLOPT8
RPLRSV78 EQU   RPLOPT7
RPLRSV77 EQU   RPLOPT7
RPLRLSOP EQU   RPLOPT7
RPLTPOST EQU   RPLOPT7
RPLQOPT  EQU   RPLOPT7
RPLCNIMM EQU   RPLCNOPT
RPLCNANY EQU   RPLCNOPT
RPLCNALL EQU   RPLCNOPT
RPLRSV68 EQU   RPLOPT6
RPLRSV67 EQU   RPLOPT6
RPLLOCK  EQU   RPLOPT6
RPLNCOND EQU   RPLOPT6
RPLCOND  EQU   RPLOPT6
RPLEOT   EQU   RPLUNTYP
RPLEOM   EQU   RPLUNTYP
RPLEOB   EQU   RPLUNTYP
RPLWROPT EQU   RPLOPT5
RPLNODE  EQU   RPLOPT5
RPLERACE EQU   RPLWRTYP
RPLEAU   EQU   RPLWRTYP
RPLNERAS EQU   RPLWRTYP
RPLPSOPT EQU   RPLOPT5
RPLSSNIN EQU   RPLOPT5
RPLDLGIN EQU   RPLOPT5
RPLBUFL  EQU   RPLCOMN+52
RPLCHAIN EQU   RPLNXTRP
RPLOPT4  EQU   RPLOPTCD+3
RPLALIGN EQU   RPLOPT3
RPLFMT   EQU   RPLOPT3
RPLFLD   EQU   RPLOPT3
RPLVFY   EQU   RPLOPT3
RPLBLK   EQU   RPLOPT3
RPLSFORM EQU   RPLOPT3
RPLEODS  EQU   RPLOPT3
RPLNSP   EQU   RPLOPT2
RPLUPD   EQU   RPLOPT2
@NM00175 EQU   RPLOPT2
RPLLRD   EQU   RPLOPT2
RPLBWD   EQU   RPLOPT2
RPLADD   EQU   RPLADR
RPLKEY   EQU   RPLOPT2
RPLECBIN EQU   RPLECBSW
RPLGEN   EQU   RPLOPT1
RPLKGE   EQU   RPLOPT1
RPLASY   EQU   RPLOPT1
RPLSKP   EQU   RPLOPT1
RPLSEQ   EQU   RPLOPT1
RPLLOC   EQU   RPLOPT1
RPLDAF   EQU   RPLARG+2
RPLSAF   EQU   RPLARG
RPLAREA  EQU   RPLCOMN+32
RPLCCHAR EQU   RPLCOMN+20
RPLSTRID EQU   RPLCOMN+18
RPLKEYL  EQU   RPLKEYLE
RPLRDSOH EQU   RPLFDB3
RPLRLG   EQU   RPLFDB3
RPLLGFRC EQU   RPLFDB3
RPLREOT  EQU   RPLFDB3
RPLREOM  EQU   RPLFDB3
RPLREOB  EQU   RPLFDB3
RPLSV32  EQU   RPLFDB3
RPLUINPT EQU   RPLFDB3
RPLSTSAV EQU   RPLFDB2
RPLCUERR EQU   RPLFDB2
RPLDLGFL EQU   RPLFDB2
RPLIOERR EQU   RPLFDB2
RPLDVUNS EQU   RPLFDB2
RPLATND  EQU   RPLFDB2
RPLRVID  EQU   RPLFDB2
RPLERLK  EQU   RPLFDB2
RPLERREG EQU   RPLRTNCD
RPLSTAT  EQU   RPLFDBWD
RPLPOST  EQU   RPLECB
RPLWAIT  EQU   RPLECB
RPLPLHPT EQU   RPLCOMN+4
RPLLEN2  EQU   RPLLEN
RPLSTYP  EQU   RPLIDWD+1
RPLID    EQU   RPLIDWD
VVOLLVSR EQU   VVOLLENT+6
VVOLLVFY EQU   @NM00174
VVOLLMNT EQU   @NM00174
VVOLLUCB EQU   VVOLLENT
VVOLLDEB EQU   IDAVVOLL+32
VVOLLBIB EQU   IDAVVOLL+28
VVOLLSFI EQU   IDAVVOLL+16
VVOLLCNT EQU   VVOLLERR+3
VVOLLSFC EQU   VVOLLERR+2
VVOLLFNC EQU   VVOLLERR+1
VVOLLPDC EQU   VVOLLERR
VVOLLACB EQU   IDAVVOLL+8
VVOLLTAB EQU   IDAVVOLL+4
VVOLLNBR EQU   IDAVVOLL+2
@NM00173 EQU   IDAVVOLL+1
VVOLLSWD EQU   @NM00171
VVOLLCAT EQU   @NM00171
VVOLLEOV EQU   @NM00171
VVOLLOPE EQU   @NM00171
VVOLLTIO EQU   @NM00171
VVOLLID  EQU   @NM00171
@NM00172 EQU   @NM00171
CMFXHRBA EQU   CMFXECTS+16
CMFXLRBA EQU   CMFXECTS+12
CMFXNOTT EQU   CMFXECTS+10
CMFXEDHH EQU   CMFXEDA+2
CMFXEDCC EQU   CMFXEDA
CMFXSTHH EQU   CMFXSTA+2
CMFXSTCC EQU   CMFXSTA
CMFXEXSS EQU   CMFXECTS
CMFLEND  EQU   CMFOVLAY+14
CMFLEXLN EQU   CMFOVLAY+12
CMFLHKLN EQU   CMFOVLAY+10
CMFLLKLN EQU   CMFOVLAY+8
CMFBYTAU EQU   CMFOVLAY+4
CMFBYTRK EQU   CMFOVLAY
CMFLSSDS EQU   GVO+35
@NM00170 EQU   CMFLITEX
CMFLSSDT EQU   CMFLITEX
CMFLNTKA EQU   GVO+32
CMFLNBTK EQU   GVO+30
CMFLPHYB EQU   GVO+26
CMFLHABA EQU   GVO+22
CMFLHKBA EQU   GVO+14
CMFLNOEX EQU   GVO+13
@NM00169 EQU   CMFLVLFL
CMFLVLOV EQU   CMFLVLFL
CMFLVLCN EQU   CMFLVLFL
CMFLVLPR EQU   CMFLVLFL
CMFLFLSQ EQU   GVO+10
CMFLVOL  EQU   GVO+4
CMFLDEVT EQU   GVO
CUMRC    EQU   CUMWA+148
@NM00168 EQU   CUMWA+146
CUMFMWA  EQU   CUMWA+58
CUMWCI2  EQU   CUMWASS2+7
CUMWTYP2 EQU   CUMWASS2+6
CUMWCI   EQU   CUMWASS+1
CUMTYPE  EQU   CUMWTYP
CUMWREL  EQU   CUMWASS2
CUMFN2   EQU   CUMWA+40
CUMFN1   EQU   CUMWA+32
CUMDY    EQU   CUMWA+29
@NM00167 EQU   CUMWA+28
CUMSRID  EQU   CUMWA+25
@NM00166 EQU   CUMWA+24
CUMRABD  EQU   CUMWA+20
CUMRAB2  EQU   CUMWA+16
CUMRAB1  EQU   CUMWA+12
CUMRABB  EQU   CUMWA+8
CUMURAB  EQU   CUMWA+4
CUMBCI   EQU   CUMWA+1
@NM00165 EQU   CUMFLAG
CUMEXIT  EQU   CUMFLAG
CUMICI   EQU   CUMPL+5
CUMDCI   EQU   CUMPL+2
@NM00164 EQU   CUMOUT
CUMNE    EQU   CUMOUT
CUMDUP   EQU   CUMOUT
@NM00163 EQU   CUMIN
CUMFUNC  EQU   CUMIN
CUMNRG   EQU   CUMIN
CUMFDEL  EQU   CUMIN
CMSDYNAL EQU   CMSRAC+164
@NM00162 EQU   CMSROBJN+1
CMSROBJI EQU   CMSROBJN
CMSWRMAC EQU   CMSRAC+16
CMSWARFV EQU   CMSRAC+12
CMSRDSVL EQU   CMSRAC+8
CMSRDSNA EQU   CMSRAC+4
@NM00161 EQU   CMSRAC+2
CMSREACD EQU   CMSRAC+1
CMSRETCD EQU   CMSRAC
CMSRACBF EQU   CMSWA+504
CMBYTAU  EQU   CMSWA+500
CMBYTTRK EQU   CMSWA+496
@NM00160 EQU   CMSWA+413
CMDEFACI EQU   CMUCATCI
CMBLPTKS EQU   CMSWA+408
CMSDSPTB EQU   CMSWA+344
@NM00159 EQU   DCWPLCDS+27
DCWPLCDV EQU   DCWPLCDS+24
DCWPLCDA EQU   DCWPLCDL+16
DCWPLCDH EQU   DCWPLCDL
DCWPLARV EQU   DCWPLARS+24
DCWPLARA EQU   DCWPLARL+16
DCWPLARH EQU   DCWPLARL
@NM00158 EQU   DCWPLEDS+27
DCWPLEDV EQU   DCWPLEDS+24
DCWPLEDA EQU   DCWPLEDL+16
DCWPLEDH EQU   DCWPLEDL
DCWPLBFV EQU   DCWPLBFS+24
DCWPLBFA EQU   DCWPLBFL+16
DCWPLBFH EQU   DCWPLBFL
@NM00157 EQU   DCWPLSPS+31
DCWPLSPV EQU   DCWPLSPS+24
DCWPLSPA EQU   DCWPLSPL+16
DCWPLSPH EQU   DCWPLSPL
DCWFPLPT EQU   CMSWA+196
DCWAREPT EQU   CMSWA+192
@NM00156 EQU   CMSWA+190
DCWIX    EQU   CMSWA+188
CMWPAD   EQU   CMWGENAM+35
CMWTIME2 EQU   CMWTIME+11
CMWDELM2 EQU   CMWTIME+9
CMWTIME1 EQU   CMWTIME+2
CMWDELM1 EQU   CMWTIME
CMWDAY   EQU   CMWDATE+2
CMWYEAR  EQU   CMWDATE
CMWFIXID EQU   CMWGENAM
CMNCILL  EQU   CMSPCQTY
CMNCITL  EQU   CMIXFANS
CMWT     EQU   CMIXFANO
CMDEFSWA EQU   CMSWA+128
CMSRCCOR EQU   CMSWA+124
CMEXTLST EQU   CMSWA+120
WORKPROC EQU   CMSWA+118
@NM00155 EQU   CMSWA+116
CMFPLIDX EQU   CMSWA+112
CMCNVNO  EQU   CMSWA+110
CMCNVLEN EQU   CMSWA+108
CMTRKPCA EQU   CMSPCATR+10
CMSWADTY EQU   CMSPCATR+4
CMPHYBLK EQU   CMSPCATR
CMTOTVOL EQU   CMSWA+74
CMVLVOL  EQU   CMVLLIST+4
CMVLRT2  EQU   CMVLRTCD+1
CMVLRT1  EQU   CMVLRTCD
CMVLLTLN EQU   CMVLLIST
@NM00154 EQU   CMSWA+62
CMSCRADT EQU   CMSWA+58
CMSCRAVI EQU   CMSWA+52
CMSDEFCI EQU   CMSDFSPC+1
CMSWAFLG EQU   CMSDFSPC
CMSPCFVT EQU   CMSWA+44
CMRMVVOL EQU   CMVOLCTR
CMADDVOL EQU   CMRNGCTR
SPACINDC EQU   CMSWA+38
CMDTAFPL EQU   CMHIXFPL
CMTSTFPL EQU   CMHDTFPL
CMRNMLST EQU   CMLDTFPL
CMEXTFLS EQU   CMCPLFVT
CMASCFPL EQU   CMSWA+16
CMWFVTPT EQU   CMSWA+12
CMWACI   EQU   CMSWA+3
CMAMDRNG EQU   CMWRKATR
CMAMDORD EQU   CMWRKATR
CMAMDRPL EQU   CMWRKATR
CMAMDSDT EQU   CMWRKATR
CMAMDWCK EQU   CMWRKATR
CMAMDDST EQU   CMWRKATR
CMCISREQ EQU   CMSWA+1
CMSUALOC EQU   CMWFLAGS
CMSRACDL EQU   CMWFLAGS
CMPROC   EQU   CMWFLAGS
CMPROCM  EQU   CMWFLAGS
CMFGDLSC EQU   CMWFLAGS
DSNPHYBS EQU   DSPTBL+30
DSPHYBPT EQU   DSPTBL+22
DSMPHYBS EQU   DSPTBL+20
DSDEVCHA EQU   DSPTBL
PRLPAR2  EQU   PRELENTF+1
PRLPAR1  EQU   PRELENTF
PRLLN    EQU   PRLIST
SALAXWA  EQU   SALPARL+22
SALVLIST EQU   SALPARL+18
@NM00153 EQU   SALPARMS+7
SALBLKLN EQU   SALPARMS+2
SALDSCIN EQU   SALPARL+3
@NM00152 EQU   SALPARL+1
RTTOLFCT EQU   DEVCHARS+18
RTTOLFLG EQU   RTDVFLGS
@NM00149 EQU   RTDVFLGS
RTKABFLG EQU   RTDVFLGS
@NM00148 EQU   RTDVFLGS
RTBONK   EQU   DEVCHARS+16
RTBOKLB  EQU   RTBOKAB+1
RTBOKNLB EQU   RTBOKAB
RTTRKSIZ EQU   DEVCHARS+12
RTNOTRKS EQU   DEVCHARS+10
RTNOCYLS EQU   DEVCHARS+8
RTMXDBLK EQU   DEVCHARS+4
RTCLSTYP EQU   RTDEVUCB+2
@NM00147 EQU   RTDEVUCB
@NM00146 EQU   RGATTSG1
RGATSUP1 EQU   RGATTSG1
@NM00145 EQU   RGATTR1
RGATTUPG EQU   RGATTR1
CTGDSNUS EQU   CTGATR2
@NM00144 EQU   CTGATR2
CTGDSSSH EQU   CTGATR2
CTGDSISH EQU   CTGATR2
CTGTRKOV EQU   CTGATR1
CTGDSTPX EQU   CTGATR1
CTGDSINH EQU   CTGATR1
CTGDSREC EQU   CTGATR1
CTGDSERA EQU   CTGATR1
CTGDSREU EQU   CTGATR1
CTGDSSPD EQU   CTGATR1
SPCROUND EQU   SPCOPTNS
SPCALXRQ EQU   SPCOPTNS
SPCMXIGR EQU   SPCOPTNS
SPCCNTIG EQU   SPCOPTNS
@NM00143 EQU   SPCOPTNS
LTRETCD2 EQU   LTRETCOD+1
LTRETCD1 EQU   LTRETCOD
USERAREV EQU   USERAREC+2
USERAREL EQU   USERAREC
USVRMDUL EQU   PASSGO+42
PASSATMP EQU   PASSGO+40
PASSPRMT EQU   PASSGO+32
READPW   EQU   PASSWORD+24
UPDATEPW EQU   PASSWORD+16
CIPW     EQU   PASSWORD+8
MASTERPW EQU   PASSWORD
ASCI     EQU   ASGO+1
ASTYPE   EQU   ASGO
CVLSER   EQU   MWA+259
TTR2     EQU   MWA+256
BIN0     EQU   CVOLIND+3
TTR1     EQU   CVOLIND
@NM00135 EQU   MWA+242
FSN      EQU   VLST+10
VOLSER   EQU   VLST+4
DEVTYP   EQU   VLST
VOLENTCT EQU   MWA
RDDFVSER EQU   RDDFLIST+12
RDDFNNAM EQU   RDDFOVOL
RDDFENT  EQU   RDDFENTW+1
@NM00134 EQU   RDDFFLGS
RDDFDSTV EQU   RDDFFLGS
@NM00133 EQU   RDDFFLGS
RDDFNEWN EQU   RDDFFLGS
RDDFOLDV EQU   RDDFFLGS
RDDFTADV EQU   RDDFFLGS
RDDFTDEL EQU   RDDFFLGS
RDDFINST EQU   RDDFINSW+1
RDDFLENG EQU   RDDFINSW
ACHKVOLS EQU   ACHKVOLW+1
@NM00132 EQU   ACHKFLG3
ACHKVOL  EQU   ACHKFLG3
@NM00131 EQU   ACHKFLG3
ACHKPRF  EQU   ACHKFLG3
@NM00130 EQU   ACHKFLG3
ACHKCLN  EQU   ACHKCLNW+1
@NM00129 EQU   ACHKFLG2
ACHKTRD  EQU   ACHKFLG2
ACHKTUPD EQU   ACHKFLG2
ACHKTCTL EQU   ACHKFLG2
@NM00128 EQU   ACHKFLG2
ACHKTALT EQU   ACHKFLG2
ACHKCSA  EQU   ACHKFLG1
ACHKLOGN EQU   ACHKFLG1
ACHKLOGF EQU   ACHKFLG1
@NM00127 EQU   ACHKFLG1
ACHKDSTV EQU   ACHKFLG1
@NM00126 EQU   ACHKFLG1
@NM00125 EQU   ACHKFLG1
@NM00124 EQU   ACHKFLG1
ACHKINST EQU   ACHKINSW+1
ACHKLENG EQU   ACHKINSW
CTGCVCC  EQU   CTGCV+50
CTGCVDEV EQU   CTGCVDSN
CTGCVVOL EQU   CTGCV
CTGVLTTR EQU   CTGVLX+12
@NM00123 EQU   CTGVLX
CTGVLSEQ EQU   CTGVL+10
CTGVLDEV EQU   CTGVL+6
CTGVLVOL EQU   CTGVL
CTGWAGB  EQU   CTGWA+12
CTGWAGCT EQU   CTGWA+10
CTGWAUCT EQU   CTGWA+8
CTGWAVCT EQU   CTGWA+6
CTGWALV  EQU   CTGWA+4
CTGFVWKA EQU   CTGFV+88
CTGFVCPP EQU   CTGFVPWD
CTGFVNAM EQU   CTGFV+80
CTGFVUPG EQU   CTGFVGAT
CTGFVEXT EQU   CTGFVLMT
CTGFVLRS EQU   CTGFV+68
CTGFVBUF EQU   CTGFV+64
CTGFVFSN EQU   CTGFVAMD
CTGFVTTR EQU   CTGFVSPC
CTGFVDVT EQU   CTGFV+48
CTGFVRNG EQU   CTGFV+44
CTGFVCRE EQU   CTGFV+36
CTGFVEXP EQU   CTGFV+32
CTGFVOWN EQU   CTGFV+28
CTGFVSTY EQU   CTGFV+24
CTGFVICH EQU   CTGFV+8
CTGFVDCH EQU   CTGFV+4
@NM00122 EQU   CTGFV+3
CTGFVELM EQU   CTGFV+2
@NM00121 EQU   CTGFVPRO
CTGFVROF EQU   CTGFVPRO
CTGFVRON EQU   CTGFVPRO
CTGFVDRC EQU   CTGFVPRO
CTGFVNDC EQU   CTGFVPRO
CTGFVRVL EQU   CTGFVPRO
CTGFVAVL EQU   CTGFVPRO
CTGFVTYP EQU   CTGFV
CAXWNQID EQU   CAXHRPLW+2
CAXHRPLC EQU   CAXHRPLW
CAXASCBP EQU   IGGCAXWA+124
@NM00120 EQU   CAXECB+1
@NM00119 EQU   CAXECBHB
CAXECBPB EQU   CAXECBHB
CAXECBWB EQU   CAXECBHB
CAXRACP  EQU   IGGCAXWA+116
CAXJDE   EQU   IGGCAXWA+108
CAXPLOCK EQU   CAXCCA
CAXOPEWA EQU   IGGCAXWA+100
@NM00118 EQU   COPTS
@NM00117 EQU   CAXCNAM+22
CAXRADDN EQU   CAXCNAM+14
CAXRATEP EQU   CAXCNAM+10
CAXRPL   EQU   IGGCAXWA+48
CAXRPLCT EQU   IGGCAXWA+46
CAXASID  EQU   IGGCAXWA+44
CAXFDCI  EQU   CAXCCR+9
CAXCDCI  EQU   CAXCCR+6
CAXNFCI  EQU   CAXCCR+3
CAXHACI  EQU   CAXCCR
CAXDRWP  EQU   IGGCAXWA+20
CAXSCHWA EQU   IGGCAXWA+16
@NM00116 EQU   CAXFLG3
CAXF3B6  EQU   CAXFLG3
CAXF3B5  EQU   CAXFLG3
CAXRAC   EQU   CAXFLG3
@NM00115 EQU   CAXFLG3
CAXF3ANE EQU   CAXFLG3
CAXF3AT  EQU   CAXFLG3
CAXF2VTU EQU   CAXFLG2
CAXF2REC EQU   CAXFLG2
CAXF2CA  EQU   CAXFLG2
CAXF2CLR EQU   CAXFLG2
CAXF2IOE EQU   CAXFLG2
CAXF2NCR EQU   CAXFLG2
CAXF2NDD EQU   CAXFLG2
CAXF2DT  EQU   CAXFLG2
CAXSCR   EQU   CAXFLGS
CAXCMP   EQU   CAXFLGS
CAXEOV   EQU   CAXFLGS
CAXCLS   EQU   CAXFLGS
CAXOPN   EQU   CAXFLGS
CAXCHN   EQU   IGGCAXWA+4
@NM00114 EQU   IGGCAXWA+1
CBSVPTR  EQU   CBSVSRT+4
CBSVUSE  EQU   CBSVSRT
CBSCRTCB EQU   AMCBS+28
CBSCRACA EQU   AMCBS+24
CBSCAXCN EQU   AMCBS+20
CBSMCUCB EQU   CBSCMP
CBSCBP   EQU   AMCBS+12
CBSACB   EQU   AMCBS+8
CBSMCSTA EQU   AMCBS+4
CBSSIZ   EQU   AMCBS+2
CBSID    EQU   AMCBS
ACBAPID  EQU   ACBCOMN+72
ACBCBMWA EQU   ACBCOMN+68
ACBLRECL EQU   ACBCOMN+62
ACBMSGLN EQU   ACBBLKSZ
ACBBUFSP EQU   ACBCOMN+56
@NM00111 EQU   ACBUJFCB+1
ACBOPTN  EQU   ACBUJFCB
@NM00110 EQU   ACBINFLG+1
ACBBYPSS EQU   ACBINFLG
ACBSDS   EQU   ACBVVIC
ACBUCRA  EQU   ACBINFLG
ACBCAT   EQU   ACBINFLG
ACBIJRQE EQU   ACBINFLG
ACBJEPS  EQU   ACBINFLG
@NM00109 EQU   ACBINFLG
ACBBUSY  EQU   ACBIOSFG
ACBLOCK  EQU   ACBEXFG
@NM00108 EQU   ACBOFLGS
ACBDSERR EQU   ACBOFLGS
ACBEOV   EQU   ACBOFLGS
@NM00107 EQU   ACBOFLGS
ACBDEB   EQU   ACBDDNM+5
ACBERFL  EQU   ACBDDNM+4
ACBAM    EQU   ACBAMETH
ACBINFL  EQU   ACBDDNM+2
ACBTIOT  EQU   ACBDDNM
ACBUEL   EQU   ACBEXLST
ACBPASSW EQU   ACBCOMN+32
ACBMSGAR EQU   ACBCOMN+28
@NM00106 EQU   ACBDSORG+1
@NM00105 EQU   ACBDSORG
ACBOPTJ  EQU   ACBDVIND
ACBCRNRE EQU   ACBCROPS
ACBCRNCK EQU   ACBCROPS
ACBASA   EQU   ACBCCTYP
@NM00104 EQU   ACBCCTYP
ACBTRCID EQU   ACBCCTYP
@NM00103 EQU   ACBRECFM
ACBRECAF EQU   ACBRECFM
ACBJBUF  EQU   ACBBUFPL+2
ACBMACR4 EQU   ACBBUFPL+1
@NM00102 EQU   ACBMACR3
ACBNCFX  EQU   ACBMACR3
ACBSIS   EQU   ACBMACR3
ACBDFR   EQU   ACBMACR3
ACBICI   EQU   ACBMACR3
ACBGSR   EQU   ACBMACR3
ACBLSR   EQU   ACBMACR3
@NM00101 EQU   ACBMACR3
ACBBUFNI EQU   ACBCOMN+18
ACBBSTNO EQU   ACBCOMN+14
ACBAIX   EQU   ACBMACR2
ACBDSN   EQU   ACBMACR2
ACBRST   EQU   ACBMACR2
ACBLOGON EQU   ACBMACR2
ACBSKP   EQU   ACBMACR2
@NM00100 EQU   ACBMACR2
ACBIN    EQU   ACBMACR1
ACBSEQ   EQU   ACBMACR1
ACBBLK   EQU   ACBCNV
ACBADD   EQU   ACBADR
ACBKEY   EQU   ACBMACR1
ACBINRTN EQU   ACBCOMN+8
ACBIBCT  EQU   ACBJWA
ACBLEN2  EQU   ACBLENG2
ACBSTYP  EQU   ACBCOMN+1
@NM00075 EQU   TCBXTNT2+32
TCBRV181 EQU   TCBRV173
TCBRV180 EQU   TCBRV173
TCBRV179 EQU   TCBRV173
TCBRV178 EQU   TCBRV173
TCBRV177 EQU   TCBRV173
TCBRV176 EQU   TCBRV173
TCBRV175 EQU   TCBRV173
TCBRV174 EQU   TCBRV173
TCBRV172 EQU   TCBRV164
TCBRV171 EQU   TCBRV164
TCBRV170 EQU   TCBRV164
TCBRV169 EQU   TCBRV164
TCBRV168 EQU   TCBRV164
TCBRV167 EQU   TCBRV164
TCBRV166 EQU   TCBRV164
TCBRV165 EQU   TCBRV164
TCBRV163 EQU   TCBRV155
TCBRV162 EQU   TCBRV155
TCBRV161 EQU   TCBRV155
TCBRV160 EQU   TCBRV155
TCBRV159 EQU   TCBRV155
TCBRV158 EQU   TCBRV155
TCBRV157 EQU   TCBRV155
TCBRV156 EQU   TCBRV155
TCBRV154 EQU   TCBRV146
TCBRV153 EQU   TCBRV146
TCBRV152 EQU   TCBRV146
TCBRV151 EQU   TCBRV146
TCBRV150 EQU   TCBRV146
TCBRV149 EQU   TCBRV146
TCBRV148 EQU   TCBRV146
TCBRV147 EQU   TCBRV146
TCBRV145 EQU   TCBXTNT2+26
TCBRV144 EQU   TCBXTNT2+24
TCBRV143 EQU   TCBXTNT2+20
TCBRV142 EQU   TCBXTNT2+16
TCBRSV49 EQU   TCBXTNT2+12
TCBEVENT EQU   TCBXTNT2+8
TCBRCMP  EQU   TCBXTNT2+5
TCBRSVAB EQU   TCBXTNT2+4
TCBGTFA  EQU   TCBGTF+1
TCBRSV47 EQU   TCBTFLG
TCBRSV46 EQU   TCBTFLG
TCBRSV45 EQU   TCBTFLG
TCBRSV44 EQU   TCBTFLG
TCBRSV43 EQU   TCBTFLG
TCBDSPIT EQU   TCBTFLG
TCBERRTN EQU   TCBTFLG
TCBASYNC EQU   TCBTFLG
@NM00074 EQU   TCB+296
TCBSVCA2 EQU   TCB+292
TCBSWASA EQU   TCB+288
TCBDBTB  EQU   TCB+284
TCBRPT   EQU   TCB+280
TCBRV141 EQU   TCBRV133
TCBRV140 EQU   TCBRV133
TCBRV139 EQU   TCBRV133
TCBRV138 EQU   TCBRV133
TCBRV137 EQU   TCBRV133
TCBRV136 EQU   TCBRV133
TCBRV135 EQU   TCBRV133
TCBRV134 EQU   TCBRV133
TCBRV132 EQU   TCBRV124
TCBRV131 EQU   TCBRV124
TCBRV130 EQU   TCBRV124
TCBRV129 EQU   TCBRV124
TCBRV128 EQU   TCBRV124
TCBRV127 EQU   TCBRV124
TCBRV126 EQU   TCBRV124
TCBRV125 EQU   TCBRV124
TCBRV123 EQU   TCBFBYT2
TCBRV122 EQU   TCBFBYT2
TCBECBNV EQU   TCBFBYT2
TCBSSSYN EQU   TCBFBYT2
TCBFPRAP EQU   TCBFBYT2
TCBFDW   EQU   TCBFBYT2
TCBFMW   EQU   TCBFBYT2
TCBCNCB  EQU   TCBFBYT2
TCBLLH   EQU   TCBFBYT1
TCBRV113 EQU   TCBFBYT1
TCBEOT   EQU   TCBFBYT1
TCBRTM2  EQU   TCBFBYT1
TCBPGNLY EQU   TCBFBYT1
TCBNDIOS EQU   TCBFBYT1
TCBRV327 EQU   TCBFBYT1
TCBEOTFM EQU   TCBFBYT1
TCBAFFN  EQU   TCB+274
TCBRV326 EQU   TCB+272
TCBUKYSP EQU   TCB+268
TCBMODE  EQU   TCBESTAE+3
TCBERTYP EQU   TCBESTAE+2
TCBRV314 EQU   TCBESTRM
TCBRV313 EQU   TCBESTRM
TCBRV312 EQU   TCBESTRM
TCBRV311 EQU   TCBESTRM
TCBRV310 EQU   TCBESTRM
TCBRV309 EQU   TCBESTRM
TCBRV308 EQU   TCBESTRM
TCBETERM EQU   TCBESTRM
TCBSCBKY EQU   TCBESTAE
TCBRTM12 EQU   TCB+260
TCBTCBID EQU   TCB+256
TCBSTAWA EQU   TCB+252
TCBSWA   EQU   TCB+248
TCBFOEA  EQU   TCBFOE+1
TCBRSV42 EQU   TCBFOE
TCBCCPVI EQU   TCBXSCT+2
TCBRV325 EQU   TCBXSCT+1
TCBRV324 EQU   TCBXSCT1
TCBRV323 EQU   TCBXSCT1
TCBRV322 EQU   TCBXSCT1
TCBRV321 EQU   TCBXSCT1
TCBRV320 EQU   TCBXSCT1
TCBRV319 EQU   TCBXSCT1
TCBS3A   EQU   TCBXSCT1
TCBACTIV EQU   TCBXSCT1
TCBRSV41 EQU   TCB+239
TCBTID   EQU   TCB+238
TCBRSVAA EQU   TCB+237
TCBABCUR EQU   TCB+236
TCBTMSAV EQU   TCB+232
TCBIOTIM EQU   TCB+228
TCBRTWA  EQU   TCB+224
TCBBACK  EQU   TCB+220
TCBTIRB  EQU   TCB+216
TCBAECB  EQU   TCB+212
TCBEXT2A EQU   TCBEXT2+1
TCBRSV39 EQU   TCBEXT2
TCBSTMCT EQU   TCB+207
TCBSYSCT EQU   TCB+206
TCBRSV37 EQU   TCB+205
TCBEXSVC EQU   TCBDAR
TCBDARMS EQU   TCBDAR
TCBDARWT EQU   TCBDAR
TCBDARO  EQU   TCBDAR
TCBDARMC EQU   TCBDARC
TCBDARD  EQU   TCBDAR
TCBDARS  EQU   TCBDAR
TCBDARP  EQU   TCBDAR
TCBGTOFM EQU   TCBFLGS7
TCBADMP  EQU   TCBFLGS7
TCBRSTSK EQU   TCBFLGS7
TCBSVCS  EQU   TCBFLGS7
TCBSTACK EQU   TCBFLGS7
TCBRSV34 EQU   TCBFLGS7
TCBRSV33 EQU   TCBFLGS7
TCBGPECB EQU   TCBFLGS7
TCBNTJS  EQU   TCBFLGS6
TCBAPG   EQU   TCBFLGS6
TCBMIGR  EQU   TCBFLGS6
TCBRV303 EQU   TCBFLGS6
TCBSPVLK EQU   TCBFLGS6
TCBCPU   EQU   TCBFLGS6
TCBPIE17 EQU   TCBFLGS6
TCBRV    EQU   TCBFLGS6
TCBRSV79 EQU   TCBNDSP5
TCBRSV78 EQU   TCBNDSP5
TCBRSV77 EQU   TCBNDSP5
TCBRSV76 EQU   TCBNDSP5
TCBRSV75 EQU   TCBNDSP5
TCBRSV74 EQU   TCBNDSP5
TCBRSV95 EQU   TCBNDSP5
TCBRSV94 EQU   TCBNDSP5
TCBRSV93 EQU   TCBNDSP4
TCBRSV92 EQU   TCBNDSP4
TCBRSV91 EQU   TCBNDSP4
TCBRSV90 EQU   TCBNDSP4
TCBRSV89 EQU   TCBNDSP4
TCBRSV88 EQU   TCBNDSP4
TCBRSV87 EQU   TCBNDSP4
TCBRSV86 EQU   TCBNDSP4
TCBEXT1A EQU   TCBEXT1+1
TCBRSV32 EQU   TCBEXT1
TCBEXCPD EQU   TCB+192
TCBIOBRC EQU   TCB+188
TCBDDWTC EQU   TCB+186
TCBDDEXC EQU   TCB+184
TCBJSCBB EQU   TCBJSCB+1
@NM00073 EQU   TCBRECDE
TCBREC   EQU   TCBRECDE
TCBMDIDS EQU   TCB+176
TCBNDINT EQU   TCBNDSP3
TCBRSV27 EQU   TCBNDSP3
TCBRSV26 EQU   TCBNDSP3
TCBRSV25 EQU   TCBNDSP3
TCBRSV24 EQU   TCBNDSP3
TCBSRBND EQU   TCBNDSP3
TCBRV302 EQU   TCBNDSP3
TCBLJSND EQU   TCBNDSP3
TCBABE   EQU   TCBNDSP2
TCBDSS   EQU   TCBNDSP2
TCBOWAIT EQU   TCBNDSP2
TCBIWAIT EQU   TCBNDSP2
TCBNDTS  EQU   TCBNDSP2
TCBNDSVC EQU   TCBNDSP2
TCBSTPP  EQU   TCBNDSP2
TCBABD   EQU   TCBNDSP2
TCBRSV22 EQU   TCBNDSP1
TCBPIEND EQU   TCBNDSP1
TCBTPSP  EQU   TCBNDSP1
TCBDDRND EQU   TCBNDSP1
TCBRSPND EQU   TCBNDSP1
TCBRSTND EQU   TCBNDSP1
TCBDARPN EQU   TCBNDSP1
TCBDARTN EQU   TCBNDSP1
TCBNDSP0 EQU   TCBNDSP
TCBUSER  EQU   TCB+168
TCBTCTB  EQU   TCBTCT+1
TCBRSV9C EQU   TCBTCTGF
TCBRSV9B EQU   TCBTCTGF
TCBRSV9A EQU   TCBTCTGF
TCBRSV99 EQU   TCBTCTGF
TCBRSV98 EQU   TCBTCTGF
TCBRSV97 EQU   TCBTCTGF
TCBRSV20 EQU   TCBTCTGF
TCBSMFGF EQU   TCBTCTGF
TCBSTABB EQU   TCBSTAB+1
TCBSTCUR EQU   TCBNSTAE
TCBRV301 EQU   TCBNSTAE
TCBSYNCH EQU   TCBNSTAE
TCBHALT  EQU   TCBNSTAE
TCBPPSUP EQU   TCBNSTAE
TCB33E   EQU   TCBNSTAE
TCBQUIES EQU   TCBNSTAE
TCBSTABE EQU   TCBNSTAE
TCBAQE   EQU   TCB+156
TCBPQE   EQU   TCB+152
TCBTSDP  EQU   TCB+151
TCBTSLP  EQU   TCB+150
TCBSTPCT EQU   TCB+149
TCBCPUBN EQU   TCBTSFLG
TCBDYDSP EQU   TCBTSFLG
TCBRSV18 EQU   TCBTSFLG
TCBRSV17 EQU   TCBTSFLG
TCBTIOTG EQU   TCBTSFLG
TCBATT   EQU   TCBTSFLG
TCBSTPPR EQU   TCBTSFLG
TCBRV300 EQU   TCBTSTSK
TCBECB   EQU   TCB+144
TCBIQE   EQU   TCB+140
TCBLTC   EQU   TCB+136
TCBOTC   EQU   TCB+132
TCBNTC   EQU   TCB+128
TCBJSTCA EQU   TCBJSTCB+1
TCBRSV16 EQU   TCBJSTCB
TCBTQET  EQU   TCBTME
TCBTCB   EQU   TCB+116
TCBFSAB  EQU   TCBFSA+1
TCBQEL   EQU   TCBFSA
TCBGRS15 EQU   TCBGRS+60
TCBGRS14 EQU   TCBGRS+56
TCBGRS13 EQU   TCBGRS+52
TCBGRS12 EQU   TCBGRS+48
TCBGRS11 EQU   TCBGRS+44
TCBGRS10 EQU   TCBGRS+40
TCBGRS9  EQU   TCBGRS+36
TCBGRS8  EQU   TCBGRS+32
TCBGRS7  EQU   TCBGRS+28
TCBGRS6  EQU   TCBGRS+24
TCBGRS5  EQU   TCBGRS+20
TCBGRS4  EQU   TCBGRS+16
TCBGRS3  EQU   TCBGRS+12
TCBGRS2  EQU   TCBGRS+8
TCBGRS1  EQU   TCBGRS+4
TCBGRS0  EQU   TCBGRS
TCBJPQB  EQU   TCBJPQ+1
TCBRSV15 EQU   TCBPURGE
TCBRSV14 EQU   TCBPURGE
TCBRSV13 EQU   TCBPURGE
TCBRSV12 EQU   TCBPURGE
TCBRSV11 EQU   TCBPURGE
TCBRSV10 EQU   TCBPURGE
TCBRSV09 EQU   TCBPURGE
TCBJPQF  EQU   TCBPURGE
TCBJLB   EQU   TCB+40
TCBLLS   EQU   TCB+36
TCBDSP   EQU   TCB+35
TCBLMP   EQU   TCB+34
TCBPNDSP EQU   TCBFLGS5
TCBFCD1  EQU   TCBFLGS5
TCBSTP   EQU   TCBFLGS5
TCBSYS   EQU   TCBFLGS5
TCBANDSP EQU   TCBFLGS5
TCBPAGE  EQU   TCBFLGS5
TCBUXNDF EQU   TCBABWF
TCBFC    EQU   TCBFLGS5
TCBONDSP EQU   TCBFLGS4
TCBMPCND EQU   TCBFLGS4
TCBMPCVQ EQU   TCBFLGS4
TCBUXNDV EQU   TCBFLGS4
TCBHNDSP EQU   TCBFLGS4
TCBRQENA EQU   TCBFLGS4
TCBSER   EQU   TCBFLGS4
TCBNDUMP EQU   TCBFLGS4
TCBDWSTA EQU   TCBFLGS3
TCBRSV08 EQU   TCBFLGS3
TCBRSV07 EQU   TCBFLGS3
TCBRSV06 EQU   TCBFLGS3
TCBABGM  EQU   TCBFLGS3
TCBABTRM EQU   TCBFLGS3
TCBADINP EQU   TCBFLGS3
TCBFSM   EQU   TCBFLGS3
TCBFTS   EQU   TCBFLGS2
TCBFETXR EQU   TCBFLGS2
TCBFDSOP EQU   TCBFLGS2
TCBFJMC  EQU   TCBFLGS2
TCBFSMC  EQU   TCBFLGS2
TCBFABOP EQU   TCBFLGS2
TCBFSTI  EQU   TCBFLGS2
TCBFOINP EQU   TCBFLGS2
TCBFX    EQU   TCBFLGS1
TCBFS    EQU   TCBFLGS1
TCBFT    EQU   TCBFLGS1
TCBPDUMP EQU   TCBFLGS1
TCBNONPR EQU   TCBFLGS1
TCBFERA  EQU   TCBFLGS1
TCBFE    EQU   TCBFLGS1
TCBFA    EQU   TCBFLGS1
TCBZERO  EQU   TCBPKF
TCBFLAG  EQU   TCBPKF
TCBMSSB  EQU   TCBMSS+1
TCBRSV03 EQU   TCBMSS
TCBTRNB  EQU   TCBTRN+1
TCBRSV02 EQU   TCBABF
TCBOLTEP EQU   TCBABF
TCBTCP   EQU   TCBABF
TCBTCPP  EQU   TCBABF
TCBRSV01 EQU   TCBABF
TCBGRPH  EQU   TCBABF
TCBNOCHK EQU   TCBABF
TCBMOD91 EQU   TCBABF
TCBCMPC  EQU   TCBCMP+1
TCBRV318 EQU   TCBCMSG
TCBRV317 EQU   TCBCIND
TCBRV316 EQU   TCBCWTO
TCBCASID EQU   TCBCDBL
TCBNOCC  EQU   TCBSTCC
TCBDMPO  EQU   TCBCPP
TCBCSTEP EQU   TCBCMPF
TCBCREQ  EQU   TCBCMPF
TCBDEB   EQU   TCB+8
TCBPIEA  EQU   TCBPIE+1
TCBPM    EQU   TCBPMASK
@NM00072 EQU   TCBPMASK
TCBRBP   EQU   TCB
TCBFRS6  EQU   TCBFRS+24
TCBFRS4  EQU   TCBFRS+16
TCBFRS2  EQU   TCBFRS+8
TCBFRS0  EQU   TCBFRS
NEXTTCB  EQU   TCBWORDS
CVTLEVL  EQU   CVTRELNO+2
CVTNUMB  EQU   CVTRELNO
CVTMDL   EQU   CVTFIX+250
@NM00071 EQU   CVTFIX+248
@NM00070 EQU   CVTFIX
CVTRV482 EQU   CVTXTNT2+128
CVTRV481 EQU   CVTXTNT2+124
CVTRV480 EQU   CVTXTNT2+120
CVTRV479 EQU   CVTXTNT2+118
CVTRV478 EQU   CVTXTNT2+117
CVTRV477 EQU   CVTXTNT2+116
CVTRV476 EQU   CVTXTNT2+115
CVTRV475 EQU   CVTXTNT2+114
CVTRV474 EQU   CVTRV466
CVTRV473 EQU   CVTRV466
CVTRV472 EQU   CVTRV466
CVTRV471 EQU   CVTRV466
CVTRV470 EQU   CVTRV466
CVTRV469 EQU   CVTRV466
CVTRV468 EQU   CVTRV466
CVTRV467 EQU   CVTRV466
CVTRV465 EQU   CVTRV457
CVTRV464 EQU   CVTRV457
CVTRV463 EQU   CVTRV457
CVTRV462 EQU   CVTRV457
CVTRV461 EQU   CVTRV457
CVTRV460 EQU   CVTRV457
CVTRV459 EQU   CVTRV457
CVTRV458 EQU   CVTRV457
CVTRV456 EQU   CVTXTNT2+108
CVTRV455 EQU   CVTXTNT2+104
CVTRV454 EQU   CVTXTNT2+100
CVTRV453 EQU   CVTXTNT2+96
CVTRV452 EQU   CVTXTNT2+94
CVTRV451 EQU   CVTXTNT2+92
CVTRV450 EQU   CVTXTNT2+90
CVTRV449 EQU   CVTXTNT2+88
CVTRV448 EQU   CVTXTNT2+87
CVTRV447 EQU   CVTXTNT2+86
CVTRV446 EQU   CVTRV438
CVTRV445 EQU   CVTRV438
CVTRV444 EQU   CVTRV438
CVTRV443 EQU   CVTRV438
CVTRV442 EQU   CVTRV438
CVTRV441 EQU   CVTRV438
CVTRV440 EQU   CVTRV438
CVTRV439 EQU   CVTRV438
CVTRV437 EQU   CVTRV429
CVTRV436 EQU   CVTRV429
CVTRV435 EQU   CVTRV429
CVTRV434 EQU   CVTRV429
CVTRV433 EQU   CVTRV429
CVTRV432 EQU   CVTRV429
CVTRV431 EQU   CVTRV429
CVTRV430 EQU   CVTRV429
CVTRV428 EQU   CVTXTNT2+80
CVTRV427 EQU   CVTXTNT2+76
CVTRV426 EQU   CVTXTNT2+72
CVTRV425 EQU   CVTXTNT2+68
CVTATACT EQU   CVTATCVT
CVTRV423 EQU   CVTXTNT2+62
CVTRV422 EQU   CVTXTNT2+60
CVTRV421 EQU   CVTXTNT2+58
CVTRV420 EQU   CVTXTNT2+56
CVTRV419 EQU   CVTXTNT2+55
CVTRV418 EQU   CVTXTNT2+54
CVTRV417 EQU   CVTRV409
CVTRV416 EQU   CVTRV409
CVTRV415 EQU   CVTRV409
CVTRV414 EQU   CVTRV409
CVTRV413 EQU   CVTRV409
CVTRV412 EQU   CVTRV409
CVTRV411 EQU   CVTRV409
CVTRV410 EQU   CVTRV409
CVTRV408 EQU   CVTRV400
CVTRV407 EQU   CVTRV400
CVTRV406 EQU   CVTRV400
CVTRV405 EQU   CVTRV400
CVTRV404 EQU   CVTRV400
CVTRV403 EQU   CVTRV400
CVTRV402 EQU   CVTRV400
CVTRV401 EQU   CVTRV400
CVTICB   EQU   CVTXTNT2+48
CVTSKTA  EQU   CVTXTNT2+44
CVTRSV99 EQU   CVTXTNT2+40
CVTRSV98 EQU   CVTXTNT2+36
CVTRSV97 EQU   CVTXTNT2+34
CVTRSV96 EQU   CVTXTNT2+32
CVTOLTEP EQU   CVTXTNT2+28
CVTQIDA  EQU   CVTQID+1
CVTRSV95 EQU   CVTQID
CVTRSV94 EQU   CVTXTNT2+20
CVTRSV93 EQU   CVTXTNT2+16
CVTRSV92 EQU   CVTXTNT2+12
CVTDEBVR EQU   CVTXTNT2+8
CVTRSV91 EQU   CVTXTNT2+6
CVTRSV9H EQU   CVTFLGBT
CVTRSV9G EQU   CVTFLGBT
CVTRSV9F EQU   CVTFLGBT
CVTRSV9E EQU   CVTFLGBT
CVTRSV9D EQU   CVTFLGBT
CVTRSV9C EQU   CVTFLGBT
CVTVME   EQU   CVTFLGBT
CVTNPE   EQU   CVTFLGBT
CVTNUCLS EQU   CVTXTNT2+4
CVTDSSVA EQU   CVTDSSV+1
CVTRSV89 EQU   CVTDSSV
CVTRSV88 EQU   CVTXTNT1+8
CVTRSV87 EQU   CVTXTNT1+4
CVTFACHN EQU   CVTXTNT1
CVTRV488 EQU   CVTMAP+412
CVTRV487 EQU   CVTMAP+408
CVTRV486 EQU   CVTMAP+404
CVTRV485 EQU   CVTMAP+400
CVTACTAP EQU   CVTMAP+396
CVTAUTH  EQU   CVTMAP+392
CVTATMCA EQU   CVTATMCT+1
CVTATMST EQU   CVTATMCT
CVTRSV61 EQU   CVTMAP+384
CVTVOLT1 EQU   CVTVOLM1+1
CVTVOLI1 EQU   CVTVOLF1
CVTSTOA  EQU   CVTMAP+376
CVTRSV58 EQU   CVTMAP+374
CVTRSV57 EQU   CVTMAP+372
CVTDDCE  EQU   CVTMAP+368
CVTPNWFR EQU   CVTMAP+364
CVTSMF   EQU   CVTMAP+360
CVTSULK  EQU   CVTMAP+358
CVTSLKO  EQU   CVTSYSK
CVTSLKP  EQU   CVTSYSK
CVTSLKQ  EQU   CVTSYSK
CVTSLKR  EQU   CVTSYSK
CVTRSV56 EQU   CVTSYSK
CVTRSV55 EQU   CVTSYSK
CVTRSV54 EQU   CVTSYSK
CVTRSV53 EQU   CVTSYSK
CVTRSV52 EQU   CVTA1F1
CVTRSV51 EQU   CVTA1F1
CVTRSV50 EQU   CVTA1F1
CVTRSV49 EQU   CVTA1F1
CVTRSV48 EQU   CVTA1F1
CVTRSV47 EQU   CVTA1F1
CVTSRSW  EQU   CVTA1F1
CVTPFSW  EQU   CVTA1F1
CVTPCVT  EQU   CVTMAP+352
CVTRSV46 EQU   CVTMAP+344
CVTRSV45 EQU   CVTMAP+340
CVTRSV44 EQU   CVTMAP+338
CVTRSV43 EQU   CVTMAP+336
CVTHJESA EQU   CVTHJES+1
CVTRSV42 EQU   CVTHJES
CVTEXT2A EQU   CVTEXT2+1
CVTRSV41 EQU   CVTEXT2
CVTAPFA  EQU   CVTAPF+1
CVTRSV40 EQU   CVTAPF
CVTRV518 EQU   CVTINTLA
CVTRV517 EQU   CVTERPV
CVTEORM  EQU   CVTMAP+312
CVTMCHPR EQU   CVTMAP+308
CVTTZ    EQU   CVTMAP+304
CVTJEPS  EQU   CVTMAP+300
CVTJESCT EQU   CVTMAP+296
CVTMODE  EQU   CVTMAP+292
CVTPTRV  EQU   CVTMAP+288
CVTREAL  EQU   CVTMAP+284
CVTRSV39 EQU   CVTMAP+280
CVTRSV38 EQU   CVTMAP+276
CVTDMSRA EQU   CVTDMSR+1
CVTRV634 EQU   CVTDMSRF
CVTRV633 EQU   CVTDMSRF
CVTRV632 EQU   CVTDMSRF
CVTRV631 EQU   CVTDMSRF
CVTRV630 EQU   CVTDMSRF
CVTRV629 EQU   CVTDMSRF
CVTUDUMP EQU   CVTDMSRF
CVTSDUMP EQU   CVTDMSRF
CVTQMSGA EQU   CVTQMSG+1
CVTRSV36 EQU   CVTQMSG
CVTPURGA EQU   CVTPURG+1
CVTRSV35 EQU   CVTPURG
CVTATERA EQU   CVTATER+1
CVTSYST  EQU   CVTATER
CVTVOLT2 EQU   CVTTAT
CVTVOLI2 EQU   CVTVOLF2
CVTAQAVB EQU   CVTAQAVT+1
CVTRSV34 EQU   CVTTCMFG
CVTRSV33 EQU   CVTTCMFG
CVTRSV32 EQU   CVTTCMFG
CVTRSV31 EQU   CVTTCMFG
CVTRSV30 EQU   CVTTCMFG
CVTRSV29 EQU   CVTTCMFG
CVTRSV28 EQU   CVTTCMFG
CVTTCRDY EQU   CVTTCMFG
CVTGTFA  EQU   CVTGTF+1
CVTRSV27 EQU   CVTGTFST
CVTRNIO  EQU   CVTGTFST
CVTUSR   EQU   CVTGTFST
CVTRV318 EQU   CVTFORM
CVTRV317 EQU   CVTTMODE
CVTRV316 EQU   CVTSTATE
CVTRV315 EQU   CVTGTFS
CVTGTFAV EQU   CVTGTFS
CVT0SCR1 EQU   CVTMAP+232
CVTRV515 EQU   CVTMAP+228
CVTRMS   EQU   CVTMAP+224
CVTPATCH EQU   CVTMAP+220
CVTTSCE  EQU   CVTMAP+216
CVTLNKSC EQU   CVTMAP+214
CVTQABST EQU   CVTMAP+212
CVTMDLDS EQU   CVTMAP+208
CVTUSER  EQU   CVTMAP+204
CVTABEND EQU   CVTMAP+200
CVTSMCA  EQU   CVTMAP+196
CVTRSV18 EQU   CVTMAP+192
CVTQLPAQ EQU   CVTMAP+188
CVTQCDSR EQU   CVTMAP+184
CVTRSV17 EQU   CVTOPTB
CVTRSV16 EQU   CVTOPTB
CVTFP    EQU   CVTOPTB
CVTAPTHR EQU   CVTOPTB
CVTNLOG  EQU   CVTOPTB
CVTTOD   EQU   CVTOPTB
CVTCTIMS EQU   CVTOPTB
CVTPROT  EQU   CVTOPTB
CVTXPFP  EQU   CVTOPTA
CVTASCII EQU   CVTOPTA
CVTRSV13 EQU   CVTOPTA
CVTRSV12 EQU   CVTOPTA
CVTNIP   EQU   CVTOPTA
CVTDDR   EQU   CVTOPTA
CVTAPR   EQU   CVTOPTA
CVTCCH   EQU   CVTOPTA
CVTSNCTR EQU   CVTMAP+180
CVTQMWR  EQU   CVTMAP+176
CVTQOCR  EQU   CVTMAP+172
CVT1EF00 EQU   CVTMAP+168
CVTMZ00  EQU   CVTMAP+164
CVTHEAD  EQU   CVTMAP+160
CVTRSV11 EQU   CVTMAP+156
CVT0PT01 EQU   CVTMAP+152
CVTMSER  EQU   CVTMAP+148
CVTRV516 EQU   CVTIERLC
CVTILCH  EQU   CVTMAP+140
CVT0DS   EQU   CVTMAP+136
CVTFBOSV EQU   CVTMAP+132
CVTNUCB  EQU   CVTMAP+128
CVTIXAVL EQU   CVTMAP+124
CVTIOQET EQU   CVTMAP+120
CVTDCBA  EQU   CVTMAP+117
CVTMVS2  EQU   CVTDCB
CVT6DAT  EQU   CVTDCB
CVT4MPS  EQU   CVTDCB
CVTRSV09 EQU   CVTDCB
CVT4MS1  EQU   CVTDCB
CVT2SPS  EQU   CVTDCB
CVT1SSS  EQU   CVTDCB
CVTRSV08 EQU   CVTDCB
CVTSTB   EQU   CVTMAP+112
CVTQTD00 EQU   CVTMAP+108
CVTQTE00 EQU   CVTMAP+104
CVTCUCB  EQU   CVTMAP+100
CVTSJQ   EQU   CVTMAP+96
CVTPBLDL EQU   CVTMAP+92
CVTTPC   EQU   CVTMAP+88
CVTSVDCB EQU   CVTMAP+84
CVTBRET  EQU   CVTMAP+82
CVTEXIT  EQU   CVTMAP+80
CVT0FN00 EQU   CVTMAP+76
CVTDARA  EQU   CVTDAR+1
CVTRSV07 EQU   CVTFLGS1
CVTRSV06 EQU   CVTFLGS1
CVTRSV05 EQU   CVTFLGS1
CVTRSV04 EQU   CVTFLGS1
CVTRSV03 EQU   CVTFLGS1
CVTRSV02 EQU   CVTFLGS1
CVTRSV01 EQU   CVTFLGS1
CVTDMPLK EQU   CVTFLGS1
CVTXITP  EQU   CVTMAP+68
CVTZDTAB EQU   CVTMAP+64
CVTMSLT  EQU   CVTMAP+60
CVTDATE  EQU   CVTMAP+56
CVTBTERM EQU   CVTMAP+52
CVTSYSAD EQU   CVTMAP+48
CVTXTLER EQU   CVTMAP+44
CVTILK2  EQU   CVTMAP+40
CVTILK1  EQU   CVTMAP+36
CVTPRLTV EQU   CVTMAP+32
CVTPCNVT EQU   CVTMAP+28
CVT0VL00 EQU   CVTMAP+24
CVTXAPG  EQU   CVTMAP+20
CVTBUF   EQU   CVTMAP+16
CVTJOB   EQU   CVTMAP+12
CVTLINK  EQU   CVTMAP+8
CVT0EF00 EQU   CVTMAP+4
CVT      EQU   CVTMAP
CNTVAR   EQU   CNTRLINF
VARINFO  EQU   VARFIELD+2
VARLEN   EQU   VARFIELD
@NM00069 EQU   EXTRCDPT+3
EXTGOCI  EQU   EXTRCDPT
NREPCV   EQU   CATRCD+47
@NM00068 EQU   CATRCD+27
RCDEXSEC EQU   RCDSECF
RCDDICTS EQU   CATRCD+22
RCDCRADT EQU   CATRCD+14
RCDCRACI EQU   CATRCD+11
RCDKEYTY EQU   CATRCD
CATGOGC  EQU   CATGOGC1
CATGODSP EQU   CATGOCI+1
@NM00067 EQU   CATGOFLG
@NM00066 EQU   CCARAB+2
CCARABRI EQU   CCARAB+1
CCARBUPD EQU   CCARABFL
@NM00065 EQU   CCARABFL
CCARBPA  EQU   CCARABFL
CCARBWR  EQU   CCARABFL
CCARBU2  EQU   CCARABFL
CCARBU1  EQU   CCARABFL
DIC2GC   EQU   DIC2ENT+3
DIC2LC   EQU   DIC2ENT+2
DIC2ST   EQU   DIC2ENT+1
@NM00064 EQU   DIC2FLGS
DIC2UPGD EQU   DIC2FLGS
DIC2SCRA EQU   DIC2FLGS
DIC2MV   EQU   DIC2FLGS
DIC2FLG  EQU   DIC2FLGS
DIC2COMB EQU   DIC2FLGS
DIC2REP  EQU   DIC2FLGS
DIC2VAR  EQU   DIC2FLGS
DICTGC   EQU   DICTENT+3
DICTLC   EQU   DICTENT+2
DICTST   EQU   DICTENT+1
@NM00063 EQU   DICTFLGS
DICTUPGD EQU   DICTFLGS
DICTSCRA EQU   DICTFLGS
DICTMV   EQU   DICTFLGS
DICTFLG  EQU   DICTFLGS
DICTCOMB EQU   DICTFLGS
DICTREP  EQU   DICTFLGS
DICTVAR  EQU   DICTFLGS
ENTNAMEV EQU   ENTNAMES+1
ENTNAMET EQU   ENTNAMES
WKALEN2  EQU   USERWKA+2
WKALEN1  EQU   USERWKA
CTGFLNG  EQU   CTGFLDAT
CTGFLCHN EQU   CTGFL+12
CTGFLDTS EQU   CTGFLDRE
@NM00062 EQU   CTGFLDRE
CTGFLDCD EQU   CTGFL+1
CTGFLDNO EQU   CTGFL
CTGPSWD  EQU   CTGJSCB
@NM00060 EQU   CTGFBFLG
CTGNGDSN EQU   CTGFBFLG
CTGGDGB  EQU   CTGFBFLG
CTGKEEP  EQU   CTGFBFLG
CTGPAR   EQU   CTGFBFLG
CTGFDBK  EQU   CTGNEWNM
CTGNOFLD EQU   CTGPL+19
@NM00059 EQU   CTGDSORG
CTGOPTNS EQU   CTGDSORG
@NM00058 EQU   CTGOPTN4
CTGBYPMT EQU   CTGOPTN4
@NM00057 EQU   CTGOPTN4
CTGNPROF EQU   CTGOPTN4
CTGDOCAT EQU   CTGOPTN4
CTGLBASE EQU   CTGOPTN4
CTGAM0   EQU   CTGOPTN3
CTGNUM   EQU   CTGOPTN3
CTGSRH   EQU   CTGOPTN3
CTGGDGL  EQU   CTGOPTN3
CTGSUPLT EQU   CTGOPTN3
CTGFUNC  EQU   CTGOPTN3
@NM00056 EQU   CTGOPTN2
CTGSCR   EQU   CTGOPTN2
CTGOVRID EQU   CTGOPTN2
CTGDISC  EQU   CTGOPTN2
CTGGTNXT EQU   CTGOPTN2
CTGSWAP  EQU   CTGRCATN
CTGGTALL EQU   CTGREL
CTGNSVS  EQU   CTGEXT
CTGGENLD EQU   CTGOPTN1
CTGCNAME EQU   CTGOPTN1
CTGNAME  EQU   CTGOPTN1
CTGREAD  EQU   CTGOPTN1
CTGUPD   EQU   CTGOPTN1
CTGCI    EQU   CTGOPTN1
CTGMAST  EQU   CTGOPTN1
CTGBYPSS EQU   CTGOPTN1
CCAEND   EQU   IGGCCA+1508
@NM00055 EQU   IGGCCA+1504
CCARMINN EQU   CCAREQDQ+8
CCARMAJN EQU   CCAREQDQ+4
CCAREDRC EQU   CCAREQDQ+3
@NM00054 EQU   CCAREDOP
CCAREDSH EQU   CCAREDOP
CCAREDLN EQU   CCAREQDQ+1
CCAREDFF EQU   CCAREQDQ
CCADGDGA EQU   IGGCCA+1488
CCARETRX EQU   CCAERCDX+1
CCARESNX EQU   CCAERCDX
CCAMODDX EQU   CCAPROBX
CCASMFCD EQU   CCASMFP+3
CCASMFRC EQU   CCASMFP+2
CCASMFMD EQU   CCASMFP
CCAREWKA EQU   IGGCCA+1476
CCAVTS   EQU   IGGCCA+1468
CCARANCA EQU   IGGCCA+1452
CCADICTS EQU   IGGCCA+1448
CCASUMTT EQU   IGGCCA+1445
@NM00053 EQU   CCAFLG10
CCAPRANX EQU   CCAFLG10
CCARARTC EQU   CCAFLG10
CCARAFEV EQU   CCAFLG10
CCACATAC EQU   CCAFLG10
CCAPDMW  EQU   CCAFLG10
CCAINCPL EQU   CCAFLG10
@NM00052 EQU   CCARALSA+2
CCARAREC EQU   IGGCCA+1436
CCASBASE EQU   IGGCCA+1417
CCATNQCT EQU   IGGCCA+1416
CCAMODNM EQU   CCAREGS+4
@NM00051 EQU   CCAREGS
@NM00050 EQU   IGGCCA+1057
@NM00049 EQU   CCARVFG1
RVWG     EQU   CCARVFG1
RVRPLMFG EQU   CCARVFG1
RVESBOR  EQU   CCARVFG1
RVESBO   EQU   CCARVFG1
RVCMSFG  EQU   CCARVFG1
RVARFI   EQU   CCARVFG1
RVCCAV   EQU   CCARVFG1
ARFSBECH EQU   CCAARFWA+12
ARFSBSCH EQU   CCAARFWA+8
ARFGMADR EQU   CCAARFWA+4
ARFLEN   EQU   ARFGMLEN+1
ARFGMLP  EQU   ARFGMLEN
CCAMNLL  EQU   CCAMNCAT+248
MNATARG2 EQU   CCAMNAT+244
MNATARG1 EQU   CCAMNAT+241
MNATSCLS EQU   MNATFLGS
@NM00048 EQU   MNATFLGS
MNATVAL  EQU   MNATFLGS
@NM00047 EQU   CCAMNAT+1
@NM00046 EQU   MNATTOP
CCABMRB2 EQU   IGGCCA+740
CCABMRB1 EQU   IGGCCA+736
CCABMWK4 EQU   IGGCCA+732
CCABMWK3 EQU   IGGCCA+728
CCABMWK2 EQU   IGGCCA+724
CCABMWK1 EQU   IGGCCA+720
CCABMSTR EQU   IGGCCA+718
CCABMBYT EQU   IGGCCA+716
CCABMBTL EQU   IGGCCA+714
CCABMBT1 EQU   IGGCCA+712
CCABMEND EQU   IGGCCA+708
CCABMPTR EQU   IGGCCA+704
CCABMGOP EQU   IGGCCA+700
CCABMPAD EQU   IGGCCA+699
@NM00045 EQU   IGGCCA+697
CCABMOST EQU   CCABMOFG
CCABMOTR EQU   CCABMOUT+2
CCABMONN EQU   CCABMOUT
@NM00044 EQU   CCABMINP+7
CCABMLST EQU   CCABMFLG
CCABMCCK EQU   CCABMFLG
CCABMSET EQU   CCABMFLG
CCABMCHK EQU   CCABMFLG
CCABMST  EQU   CCABMFLG
CCABMMIN EQU   CCABMINP+4
CCABMLIM EQU   CCABMINP+2
CCABMTRK EQU   CCABMINP
CCAR14S  EQU   IGGCCA+680
CCAXLATE EQU   IGGCCA+640
CCADEL2  EQU   IGGCCA+637
CCADEL1  EQU   IGGCCA+634
CCAVAR1  EQU   IGGCCA+614
CCAVAR   EQU   IGGCCA+594
CCADELN  EQU   IGGCCA+592
CCAVARPT EQU   IGGCCA+588
CCARBASE EQU   IGGCCA+584
CCARRAB  EQU   IGGCCA+580
CCAVARLN EQU   IGGCCA+578
CCACI3   EQU   IGGCCA+575
CCACI2   EQU   IGGCCA+572
CCACI1   EQU   IGGCCA+569
CCACHAIN EQU   IGGCCA+566
CCASMFLN EQU   IGGCCA+564
CCASMFG2 EQU   CCASMFLG+1
@NM00043 EQU   CCASMFG1
CCASMFAL EQU   CCASMFSR
CCASMFDF EQU   CCASMFUC
CCASMFCT EQU   IGGCCA+560
CCASMFRD EQU   IGGCCA+556
CCANORBA EQU   IGGCCA+555
CCARESUM EQU   CCAFLG9
CCARAICI EQU   CCAFLG9
CCARAACT EQU   CCAFLG9
CCAUCRA  EQU   CCAFLG9
CCARACR  EQU   CCAFLG9
CCARALRD EQU   CCAFLG9
CCARAEOV EQU   CCAFLG9
CCARABYC EQU   CCAFLG9
CCAAUCAT EQU   CCAFLG8
CCALEOD  EQU   CCAFLG8
CCAVRIND EQU   CCAFLG8
CCALPIND EQU   CCAFLG8
CCASCICK EQU   CCAFLG8
CCASCRA  EQU   CCAFLG8
CCAFEOV  EQU   CCAFLG7
CCAROREQ EQU   CCAFLG7
CCAONCE  EQU   CCAFLG7
CCASMFBR EQU   CCAFLG7
CCASMFA  EQU   CCAFLG7
CCASMFEX EQU   CCAFLG7
CCALSP   EQU   CCAFLG7
CCALCPL  EQU   IGGCCA+548
CCASMFPT EQU   IGGCCA+544
CCAILNG2 EQU   IGGCCA+536
CCAILNG3 EQU   IGGCCA+532
CCASDWAP EQU   IGGCCA+528
CCACWAP  EQU   IGGCCA+524
CCACDTCT EQU   IGGCCA+522
CCADTCT  EQU   IGGCCA+520
CCACDTA  EQU   IGGCCA+516
CCADTA   EQU   IGGCCA+512
CCASVCI1 EQU   IGGCCA+509
CCASVCI  EQU   IGGCCA+506
CCADISP  EQU   IGGCCA+504
CCAREPCT EQU   IGGCCA+502
CCADIFF  EQU   IGGCCA+500
CCADLEN  EQU   IGGCCA+498
CCAIXTPL EQU   IGGCCA+496
CCAGRHI1 EQU   CCASSEQ
CCARCDID EQU   CCAGRGC
@NM00042 EQU   IGGCCA+492
CCANEFV  EQU   IGGCCA+491
CCAGVEXT EQU   IGGCCA+488
CCANEVV  EQU   IGGCCA+486
CCAGVCT  EQU   IGGCCA+484
CCARABSM EQU   CCAIDPT3
CCAIDPT2 EQU   IGGCCA+476
CCASMFIX EQU   IGGCCA+474
CCACNIX  EQU   IGGCCA+472
CCARBAL  EQU   IGGCCA+470
CCATLEN  EQU   CCATLNG
CCARBA2  EQU   CCARBA+16
CCARBA1  EQU   CCARBA+12
CCATT    EQU   CCARBA+10
CCACCHH2 EQU   CCACRADT
CCACCHH1 EQU   CCACRAVL+2
CCASS    EQU   CCACRAVL
CCATEST  EQU   IGGCCA+444
CCATNO   EQU   IGGCCA+442
CCATREL  EQU   IGGCCA+410
CCATNUM  EQU   IGGCCA+408
CCATNREL EQU   IGGCCA+406
CCAIXREL EQU   IGGCCA+404
CCAIXFPL EQU   IGGCCA+402
@NM00041 EQU   IGGCCA+400
CCASN1   EQU   CCASN
CCALREL1 EQU   CCALREL
CCAGC    EQU   IGGCCA+395
CCACBASE EQU   IGGCCA+392
CCARABL  EQU   IGGCCA+388
CCARABF  EQU   IGGCCA+384
CCARABB  EQU   IGGCCA+380
CCAMCPL  EQU   CCAXCPL
CCARABPT EQU   IGGCCA+368
CCATFLPT EQU   CCAFLPT
CCAILNG  EQU   IGGCCA+360
CCASL    EQU   IGGCCA+358
CCAGOPLN EQU   IGGCCA+356
CCACPE71 EQU   IGGCCA+352
CCAIDPT  EQU   CCACPE7
CCARABSE EQU   CCACPE61
CCACPE53 EQU   IGGCCA+336
CCACPE52 EQU   IGGCCA+332
CCACPE51 EQU   IGGCCA+328
CCALUME  EQU   IGGCCA+324
CCAEXCMS EQU   CCACMS+4
CCARPRM  EQU   IGGCCA+312
@NM00040 EQU   CCAMSSPL+10
@NM00039 EQU   CCAMSSPL+8
CCAMNPTR EQU   CCAMSSPL+4
CCAMNLLP EQU   CCAMSSPL
CCAMLRET EQU   IGGCCA+296
CCAEDUCB EQU   CCAEQDQ+12
CCAEDRNM EQU   CCAEQDQ+8
CCAEDQNM EQU   CCAEQDQ+4
CCAEDRCD EQU   CCAEQDQ+3
@NM00038 EQU   CCAEDOPT
CCAEDSHR EQU   CCAEDOPT
CCAEDRLN EQU   CCAEQDQ+1
CCAEDXFF EQU   CCAEQDQ
CCAASCI  EQU   IGGCCA+271
CCAASCIX EQU   IGGCCA+270
CCACRRP  EQU   IGGCCA+269
CCAASCIK EQU   IGGCCA+268
CCAEXTTH EQU   CCAEXTDE+6
CCAEXTSS EQU   CCAEXTDE
CCASSVOL EQU   CCADESA+2
CCAIXEXT EQU   CCADESA+1
CCANDEXT EQU   CCADESA
CCARPL1  EQU   IGGCCA+220
CCAQLEN  EQU   IGGCCA+219
CCASC    EQU   IGGCCA+218
CCATQDB  EQU   CCATIQRN+2
CCATASID EQU   CCATIQRN
CCARPLT  EQU   IGGCCA+211
CCARPLF  EQU   IGGCCA+209
CCARPLK  EQU   IGGCCA+208
CCACPE45 EQU   CCAR5SEG+8
CCACPE35 EQU   CCAR5SEG+4
CCACPE25 EQU   CCAR5SEG
CCAR5REC EQU   CCARAB5+4
@NM00037 EQU   CCARAB5+2
CCAR5RPL EQU   CCARAB5+1
CCAR5UPD EQU   CCAR5FLG
@NM00036 EQU   CCAR5FLG
CCAR5PA  EQU   CCAR5FLG
CCAR5WR  EQU   CCAR5FLG
CCAR5U2  EQU   CCAR5FLG
CCAR5U1  EQU   CCAR5FLG
CCAR5UR  EQU   CCAR5FLG
CCACPE44 EQU   CCAR4SEG+8
CCACPE34 EQU   CCAR4SEG+4
CCACPE24 EQU   CCAR4SEG
CCAR4REC EQU   CCARAB4+4
@NM00035 EQU   CCARAB4+2
CCAR4RPL EQU   CCARAB4+1
CCAR4UPD EQU   CCAR4FLG
@NM00034 EQU   CCAR4FLG
CCAR4PA  EQU   CCAR4FLG
CCAR4WR  EQU   CCAR4FLG
CCAR4U2  EQU   CCAR4FLG
CCAR4U1  EQU   CCAR4FLG
CCAR4UR  EQU   CCAR4FLG
CCACPE43 EQU   CCAR3SEG+8
CCACPE33 EQU   CCAR3SEG+4
CCACPE23 EQU   CCAR3SEG
@NM00033 EQU   CCARAB3+2
CCAR3RPL EQU   CCARAB3+1
CCAR3UPD EQU   CCAR3FLG
@NM00032 EQU   CCAR3FLG
CCAR3PA  EQU   CCAR3FLG
CCAR3WR  EQU   CCAR3FLG
CCAR3U2  EQU   CCAR3FLG
CCAR3U1  EQU   CCAR3FLG
CCAR3UR  EQU   CCAR3FLG
CCACPE42 EQU   CCAR2SEG+8
CCACPE32 EQU   CCAR2SEG+4
CCACPE22 EQU   CCAR2SEG
@NM00031 EQU   CCARAB2+2
CCAR2RPL EQU   CCARAB2+1
CCAR2UPD EQU   CCAR2FLG
@NM00030 EQU   CCAR2FLG
CCAR2PA  EQU   CCAR2FLG
CCAR2WR  EQU   CCAR2FLG
CCAR2U2  EQU   CCAR2FLG
CCAR2U1  EQU   CCAR2FLG
CCAR2UR  EQU   CCAR2FLG
CCACPE41 EQU   CCAR1SEG+8
CCACPE31 EQU   CCAR1SEG+4
CCACPE21 EQU   CCAR1SEG
@NM00029 EQU   CCARAB1+2
CCAR1RPL EQU   CCARAB1+1
CCAR1UPD EQU   CCAR1FLG
@NM00028 EQU   CCAR1FLG
CCAR1PA  EQU   CCAR1FLG
CCAR1WR  EQU   CCAR1FLG
CCAR1U2  EQU   CCAR1FLG
CCAR1U1  EQU   CCAR1FLG
CCAR1UR  EQU   CCAR1FLG
CCACPE40 EQU   CCAR0SEG+8
CCACPE30 EQU   CCAR0SEG+4
CCACPE20 EQU   CCAR0SEG
CCAR0REC EQU   CCARAB0+4
@NM00027 EQU   CCARAB0+2
CCAR0RPL EQU   CCARAB0+1
CCAR0UPD EQU   CCAR0FLG
@NM00026 EQU   CCAR0FLG
CCAR0PA  EQU   CCAR0FLG
CCAR0WR  EQU   CCAR0FLG
CCAR0U2  EQU   CCAR0FLG
CCAR0U1  EQU   CCAR0FLG
CCAR0UR  EQU   CCAR0FLG
@NM00025 EQU   CCASRCH+3
CCASRCIN EQU   CCASRID
CCANPCCB EQU   IGGCCA+36
CCADPL   EQU   CCARB
CCALBCYL EQU   CCATCB
CCAMOD   EQU   CCAFLG6
CCAEXT   EQU   CCAFLG6
CCAREUSE EQU   CCAFLG6
CCASUPFD EQU   CCAFLG6
CCAINIT  EQU   CCAFLG6
CCAMNOSP EQU   CCAFLG6
CCADELP  EQU   CCAFLG6
CCAMCODR EQU   CCAFLG6
CCATEX2  EQU   CCAFLG5
CCATEX1  EQU   CCAFLG5
CCATEX   EQU   CCAFLG5
CCATHIT  EQU   CCAFLG5
CCAMODPA EQU   CCAFLG5
CCAMEX1  EQU   CCAFLG5
CCAMEX   EQU   CCAFLG5
CCAMEX2  EQU   CCAFLG5
CCATCOMP EQU   CCAFLG4
CCAGVNE  EQU   CCAFLG4
CCAGVEX  EQU   CCAFLG4
CCAGVNBS EQU   CCAFLG4
CCAGVNF  EQU   CCAFLG4
CCAGVNC  EQU   CCAFLG4
CCAF4BYS EQU   CCAFLG4
CCAF4DRQ EQU   CCAFLG4
CCAEGREC EQU   CCAFLG3
CCALFT   EQU   CCAFLG3
CCAELC2  EQU   CCAFLG3
CCANF    EQU   CCAFLG3
CCAEXGR2 EQU   CCAFLG3
CCAGDSP  EQU   CCAFLG3
CCAGC4   EQU   CCAFLG3
CCAEXGR1 EQU   CCAFLG3
CCAF2SMO EQU   CCAFLG2
CCAF2CO  EQU   CCAF2COB
CCAF2RHS EQU   CCAFLG2
CCAF2XEQ EQU   CCAFLG2
CCAF2CCT EQU   CCAFLG2
CCAF2NVC EQU   CCAFLG2
CCAF2SYS EQU   CCAFLG2
CCAF1DK  EQU   CCAFLG1
CCAF1UP  EQU   CCAFLG1
CCAF1CR  EQU   CCAFLG1
CCAF1KGE EQU   CCAFLG1
CCAF1KEY EQU   CCAFLG1
CCAF1LRD EQU   CCAFLG1
CCAF1CNS EQU   CCAFLG1
CCAF1LPS EQU   CCAFLG1
CCACD2   EQU   IGGCCA+13
@NM00024 EQU   IGGCCA+8
CCACDR   EQU   CCAREASN
CCAMODID EQU   CCAPROB
CCASZ    EQU   IGGCCA+2
CCAID    EQU   IGGCCA
@NM00023 EQU   UPSPD+2
UPSDREL  EQU   UPSPD
UPSTIME  EQU   UPSPWA
@NM00022 EQU   UPDIRSP+2
UPDDREL  EQU   UPDIRSP
@NM00021 EQU   UPSPHDR+2
UPSHREL  EQU   UPSPHDR
UPCRDTF  EQU   UPCRDT1+3
UPCRDTDD EQU   UPCRDT+1
UPCRDTY  EQU   UPCRDT
UPSOPT1  EQU   UPSPARM+23
UPSPACE  EQU   UPSPARM+20
UPNODSP  EQU   UPSPARM+18
UPREL    EQU   UPSPARM+16
UPCNT    EQU   UPSPARM+14
UPUCB    EQU   UPMXBLK
UPUDSNPT EQU   UPSPARM+8
UPDSDIR  EQU   UPSPARM+5
@NM00020 EQU   UPSFLG
UPSEXIT  EQU   UPSFLG
UPSUNIQ  EQU   UPSFLG
UPSPQTY  EQU   UPSPARM+1
UPSOPT   EQU   UPSPARM
EXTDDDD2 EQU   EXTINFO+16
EXTDDDD1 EQU   EXTINFO+12
EXTTT    EQU   EXTINFO+10
EXTHH2   EQU   EXTEND+2
EXTCC2   EQU   EXTEND
EXTHH1   EQU   EXTST+2
EXTCC1   EQU   EXTST
EXTSS    EQU   EXTINFO
EXTILEN  EQU   EXTINF1
UPEND    EQU   UPENTVOL+176
@NM00019 EQU   UPVOLVAR+134
UPEXLN   EQU   UPVOLVAR+132
@NM00018 EQU   UPVOLVAR+68
UPHKLN   EQU   UPVOLVAR+66
@NM00017 EQU   UPVOLVAR+2
UPLKLN   EQU   UPVOLVAR
UPSSDS   EQU   UPVOLFIX+37
@NM00016 EQU   UPITEX
UPEXTNP  EQU   UPITEX
UPSSDT   EQU   UPITEX
UPNTKA   EQU   UPVOLFIX+34
UPNBTK   EQU   UPVOLFIX+32
UPPHYB   EQU   UPVOLFIX+28
UPHABA   EQU   UPVOLFIX+24
UPHUBA   EQU   UPVOLFIX+20
UPHKBA   EQU   UPVOLFIX+16
UPNOEXT  EQU   UPVOLFIX+15
@NM00015 EQU   UPVLFLG
UPVLOV   EQU   UPVLFLG
UPVLCN   EQU   UPVLFLG
UPVLPR   EQU   UPVLFLG
UPFLSQ   EQU   UPVOLFIX+12
UPVOL    EQU   UPVOLFIX+6
UPDEVT   EQU   UPVOLFIX+2
UPRLRP   EQU   UPVOLFIX
UPEXTLN  EQU   UPREPARM+16
UPCHIA   EQU   UPREPARM+12
@NM00014 EQU   UPREPARM+11
UPSMFL   EQU   UPFLG2S
UPEXIT   EQU   UPFLG2S
UPKSET   EQU   UPFLG2S
UPSRCH   EQU   UPFLG2S
UPKREQ   EQU   UPFLG2S
UPUSSWD  EQU   UPFLG2S
@NM00013 EQU   UPFLGS
@NM00012 EQU   UPFLGS
UPKRAN   EQU   UPFLGS
@NM00011 EQU   UPFLGS
UPREPL   EQU   UPFLGS
UPSSWD   EQU   UPFLGS
@NM00010 EQU   UPFLGS
UPKEYDS  EQU   UPFLGS
UPVCINO  EQU   UPREPARM+5
UPICINO  EQU   UPREPARM+2
UPNIEXT  EQU   UPNTCYL
CTVDRSTT EQU   CTVDEXTN+4
CTVDIRTS EQU   CTVDEXTN
CTVDSDCI EQU   CTVDIR+2
CTVDSDLF EQU   CTVDSDSN+1
CTVDSDCV EQU   CTVDSDSN
CTVEXTNN EQU   CTVEXTNT+2
CTVEXTT  EQU   CTVEXTNT
CTVSQT   EQU   CTVDSPAC+18
CTVSPOPT EQU   CTVDSPAC+17
CTVNOEXT EQU   CTVDSPAC+16
@NM00009 EQU   CTVDSPFG
CTVCATDF EQU   CTVDSPFG
@NM00008 EQU   CTVDSPFG
CTVAUTO  EQU   CTVDSPFG
CTVUNIQ  EQU   CTVDSPFG
CTVF1PTR EQU   CTVDSPAC+10
CTVF1TME EQU   CTVDSPAC+2
CTVDSPLF EQU   CTVDSPSN+1
CTVDSPCV EQU   CTVDSPSN
CTVSMBIT EQU   CTVSPMAP+4
CTVSMLEN EQU   CTVSPMAP+2
CTVSMLF  EQU   CTVSMSN+1
CTVSMCV  EQU   CTVSMSN
CTVEGOPT EQU   CTVRCTRL+6
CTVEXRCT EQU   CTVRCTRL+5
@NM00007 EQU   CTVEXRPT+3
CTVEXGCI EQU   CTVEXRPT
CTVGOSEQ EQU   CTVGOPT+4
CTVGOGC  EQU   CTVGOGCI
CTVGODEL EQU   CTVGOGCI
CTVGOEXT EQU   CTVGOGCI
CTVGODSP EQU   CTVGOCI+1
CTVGOFLG EQU   CTVGOCI
CTVSPSXC EQU   CTVDSDIR+2
CTVSPSSN EQU   CTVDSDIR
CTVDSXCT EQU   CTVDSGRP+7
@NM00006 EQU   CTVDSDFL
CTVDSDCN EQU   CTVDSDFL
CTVDSCID EQU   CTVDSGRP+3
CTVDSNEX EQU   CTVDSGRP+2
CTVDSGLF EQU   CTVDSGRP+1
CTVDSGCV EQU   CTVDSGRP
CTVMAP   EQU   CTVDESGP+14
CTVMAPLN EQU   CTVDESGP+12
CTVDESHD EQU   CTVDESGP+10
CTVDESTK EQU   CTVDESGP+8
CTVSTHH  EQU   CTVSTDES+2
CTVSTCC  EQU   CTVSTDES
CTVDESUU EQU   CTVDESGP+2
CTVDESLF EQU   CTVDESGP+1
CTVDESCV EQU   CTVDESGP
CTVSPSQT EQU   CTVSPHDR+20
CTVSPSPO EQU   CTVSPHDR+19
CTVSPNEX EQU   CTVSPHDR+18
CTVSPNDS EQU   CTVSPHDR+16
@NM00005 EQU   CTVSPFLG
CTVSPCDF EQU   CTVSPFLG
CTVSPMCT EQU   CTVSPFLG
CTVSPUCT EQU   CTVSPFLG
CTVSPAUT EQU   CTVSPFLG
CTVSPUNQ EQU   CTVSPFLG
CTVF1ADR EQU   CTVSPHDR+10
CTVF1TIM EQU   CTVSPHDR+2
CTVSPHLF EQU   CTVSPHDR+1
CTVSPHCV EQU   CTVSPHDR
@NM00004 EQU   CTVOLRCD+74
CTVSYEDS EQU   CTVOLRCD+73
CTVOLFLG EQU   CTVOLRCD+72
CTVTLFAC EQU   CTVDVCHR+18
CTVTLFLG EQU   CTVDVFLG
@NM00003 EQU   CTVDVFLG
CTVBONK  EQU   CTVDVCHR+16
CTVBKLB  EQU   CTVDVCHR+15
CTVBKNLB EQU   CTVDVCHR+14
CTVTRKSZ EQU   CTVDVCHR+12
CTVNOCYL EQU   CTVDVCHR+8
CTVMXBLK EQU   CTVDVCHR+4
CTVDEVTY EQU   CTVDVCHR
CTVOLTIM EQU   CTVOLRCD+44
CTVOLNME EQU   CTVOLRCD
WKAEXTSS EQU   WKARETN+6
WKAEXT   EQU   WKARETN+4
WKARELSS EQU   WKARETN+2
WKAREL   EQU   WKARETN
@NM00002 EQU   WKAINFO
UPKEYD   EQU   UPKEY+2
UPKEYLN  EQU   UPKEY
UPKEYD1  EQU   UPKEY1
UPEXTD2  EQU   UPEXT1+2
UPEXTLN1 EQU   UPEXT1
@NM00001 EQU   RELINFO+2
RELREP   EQU   RELINFO
AMDEXCP  EQU   AMDSTAT+44
AMDNCAS  EQU   AMDSTAT+40
AMDNCIS  EQU   AMDSTAT+36
AMDASPA  EQU   AMDSTAT+32
AMDRETR  EQU   AMDSTAT+28
AMDUPR   EQU   AMDSTAT+24
AMDIREC  EQU   AMDSTAT+20
AMDDELR  EQU   AMDSTAT+16
AMDNLR   EQU   AMDSTAT+12
AMDNEXT  EQU   AMDSTAT+10
AMDNIL   EQU   AMDSTAT+8
AMDSTSP  EQU   AMDSTAT
@NM00151 EQU   IDAAMDSB+41
@NM00150 EQU   AMDATTR3
AMDLM    EQU   AMDATTR3
AMDWAIT  EQU   AMDATTR3
AMDBIND  EQU   AMDATTR3
AMDFAULT EQU   AMDATTR3
AMDUNQ   EQU   AMDATTR3
AMDPARDB EQU   IDAAMDSB+36
AMDMAXRR EQU   AMDSSRBA
AMDNSLOT EQU   AMDHLRBA
AMDLRECL EQU   IDAAMDSB+24
AMDCINV  EQU   IDAAMDSB+20
AMDFSCI  EQU   IDAAMDSB+16
AMDFSCA  EQU   IDAAMDSB+14
AMDCIPCA EQU   IDAAMDSB+12
AMDPCTCI EQU   IDAAMDSB+11
AMDPCTCA EQU   IDAAMDSB+10
AMDKEYLN EQU   IDAAMDSB+8
AMDRKP   EQU   IDAAMDSB+6
AMDAXRKP EQU   AMDNEST
AMDLEN   EQU   IDAAMDSB+2
AMDSPAN  EQU   AMDATTR
AMDRRDS  EQU   AMDATTR
AMDRANGE EQU   AMDATTR
AMDORDER EQU   AMDATTR
AMDREPL  EQU   AMDATTR
AMDSDT   EQU   AMDATTR
AMDWCK   EQU   AMDATTR
AMDDST   EQU   AMDATTR
AMDSBID  EQU   IDAAMDSB
UPSEND   EQU   UPSVOL+22
UPSCNT   EQU   UPSVOL+20
UPSREL   EQU   UPSVOL+18
UPVOLSER EQU   UPSVOL+12
UPHKEYPT EQU   UPSVOL+8
UPHKEYLN EQU   UPSVOL+2
UPLKEYLN EQU   UPSVOL
@NM00099 EQU   PSA+3668
PSASTAK  EQU   PSA+3072
@NM00098 EQU   PSA+1048
PSAUSEND EQU   PSA+1048
PSARV062 EQU   PSA+1044
PSACDAL  EQU   PSA+1040
PSAWTCOD EQU   PSA+1036
PSAATCVT EQU   PSA+1032
PSAPCPSW EQU   PSA+1024
PSAPIR2  EQU   PSA+1020
PSARV059 EQU   PSA+1018
PSASVC13 EQU   PSA+1016
PSALSFCC EQU   PSA+1012
PSASFACC EQU   PSA+1008
PSASTOP  EQU   PSA+992
PSASTART EQU   PSA+976
PSARSPSW EQU   PSA+968
PSASRPSW EQU   PSA+960
PSARV045 EQU   PSA+892
PSARV044 EQU   PSA+888
PSARV043 EQU   PSA+884
PSARV042 EQU   PSA+880
PSARV041 EQU   PSA+876
PSARV040 EQU   PSA+872
PSARV025 EQU   PSA+868
PSADSSED EQU   PSA+868
PSADSSPR EQU   PSA+864
PSADSSFW EQU   PSA+860
PSADSS14 EQU   PSA+856
PSADSSPP EQU   PSA+848
PSADSSRP EQU   PSA+840
PSADSS05 EQU   PSADSSF4
PSADSS10 EQU   PSADSSF4
PSADSSVE EQU   PSADSSF4
PSADSSDE EQU   PSADSSF4
PSADSSC0 EQU   PSADSSF4
PSADSSIE EQU   PSADSSF4
PSADSS12 EQU   PSADSSF4
PSADSSRC EQU   PSADSSF4
PSARV057 EQU   PSADSSF3
PSARV056 EQU   PSADSSF3
PSARV055 EQU   PSADSSF3
PSADSSMC EQU   PSADSSF3
PSADSSRW EQU   PSADSSF3
PSADSSNM EQU   PSADSSF3
PSADSSES EQU   PSADSSF3
PSADSSGP EQU   PSADSSF3
PSADSSF2 EQU   PSADSSFL+1
PSADSSPI EQU   PSADSSF1
PSADSSOI EQU   PSADSSF1
PSADSSSP EQU   PSADSSF1
PSADSSTP EQU   PSADSSF1
PSADSSDW EQU   PSADSSF1
PSADSSDD EQU   PSADSSF1
PSADSSDM EQU   PSADSSF1
PSADSSMV EQU   PSADSSF1
PSADSSTS EQU   PSA+816
PSADSSWK EQU   PSA+812
PSADSSR3 EQU   PSA+808
PSADSSR2 EQU   PSA+804
PSADSSRS EQU   PSA+800
PSASTOR  EQU   PSA+796
PSACPUSA EQU   PSA+794
PSAVSTAP EQU   PSA+792
PSAWKVAP EQU   PSA+788
PSAWKRAP EQU   PSA+784
PSAMCHIC EQU   PSA+783
PSARV061 EQU   PSA+782
PSASYMSK EQU   PSA+781
PSAMCHFL EQU   PSA+780
PSACR0   EQU   PSA+776
PSAPSWSV EQU   PSA+768
PSALITA  EQU   PSA+764
PSACLHS  EQU   PSAHLHI
PSALKR15 EQU   PSALKSA+60
PSALKR14 EQU   PSALKSA+56
PSALKR13 EQU   PSALKSA+52
PSALKR12 EQU   PSALKSA+48
PSALKR11 EQU   PSALKSA+44
PSALKR10 EQU   PSALKSA+40
PSALKR9  EQU   PSALKSA+36
PSALKR8  EQU   PSALKSA+32
PSALKR7  EQU   PSALKSA+28
PSALKR6  EQU   PSALKSA+24
PSALKR5  EQU   PSALKSA+20
PSALKR4  EQU   PSALKSA+16
PSALKR3  EQU   PSALKSA+12
PSALKR2  EQU   PSALKSA+8
PSALKR1  EQU   PSALKSA+4
PSALKR0  EQU   PSALKSA
PSARV023 EQU   PSACLHT+52
PSALOCAL EQU   PSACLHT+48
PSACMSL  EQU   PSACLHT+44
PSAOPTL  EQU   PSACLHT+40
PSATPACL EQU   PSACLHT+36
PSATPDNL EQU   PSACLHT+32
PSATPNCL EQU   PSACLHT+28
PSAIOSLL EQU   PSACLHT+24
PSAIOSUL EQU   PSACLHT+20
PSAIOSCL EQU   PSACLHT+16
PSAIOSSL EQU   PSACLHT+12
PSASALCL EQU   PSACLHT+8
PSAASML  EQU   PSACLHT+4
PSADISPL EQU   PSACLHT
PSASRSA  EQU   PSA+636
PSARV050 EQU   PSA+634
PSADSSGO EQU   PSA+633
PSARECUR EQU   PSA+632
PSAHLHIS EQU   PSA+628
PSAIPCSA EQU   PSA+624
@NM00097 EQU   PSA+621
PSAIPCDM EQU   PSA+620
PSAIPCD  EQU   PSA+616
@NM00096 EQU   PSA+613
PSAIPCRM EQU   PSA+612
PSAIPCR  EQU   PSA+608
PSAMCHEX EQU   PSA+600
PSAMPSW  EQU   PSA+592
PSAEXPS2 EQU   PSA+584
PSAEXPS1 EQU   PSA+576
PSAPIREG EQU   PSA+572
PSARSREG EQU   PSA+568
PSAGPREG EQU   PSA+556
PSARV022 EQU   PSASUP4
PSARV021 EQU   PSASUP4
PSARV020 EQU   PSASUP4
PSARV019 EQU   PSASUP4
PSARV018 EQU   PSASUP4
PSARV017 EQU   PSASUP4
PSARV016 EQU   PSASUP4
PSARV015 EQU   PSASUP4
PSARV014 EQU   PSASUP3
PSARV013 EQU   PSASUP3
PSARV012 EQU   PSASUP3
PSAESTA  EQU   PSASUP3
PSASPR   EQU   PSASUP3
PSAPSREG EQU   PSASUP3
PSAPI2   EQU   PSASUP3
PSAIOSUP EQU   PSASUP3
PSALCR   EQU   PSASUP2
PSARTM   EQU   PSASUP2
PSAACR   EQU   PSASUP2
PSAIPCE2 EQU   PSASUP2
PSAIPCES EQU   PSASUP2
PSAIPCEC EQU   PSASUP2
PSAGTF   EQU   PSASUP2
PSAIPCRI EQU   PSASUP2
PSATYPE6 EQU   PSASUP1
PSATCTL  EQU   PSASUP1
PSADISP  EQU   PSASUP1
PSALOCK  EQU   PSASUP1
PSAPI    EQU   PSASUP1
PSAEXT   EQU   PSASUP1
PSASVC   EQU   PSASUP1
PSAIO    EQU   PSASUP1
PSAAOLD  EQU   PSA+548
PSAANEW  EQU   PSA+544
PSATOLD  EQU   PSA+540
PSATNEW  EQU   PSA+536
PSALCCAR EQU   PSA+532
PSALCCAV EQU   PSA+528
PSAPCCAR EQU   PSA+524
PSAPCCAV EQU   PSA+520
PSACPULA EQU   PSA+518
PSACPUPA EQU   PSA+516
PSAPSA   EQU   PSA+512
FLCHDEND EQU   PSA+512
FLCCRSAV EQU   FLCMCLA+280
FLCGRSAV EQU   FLCMCLA+216
FLCFPSAV EQU   FLCMCLA+184
FLCFLA   EQU   FLCMCLA+88
FLCRGNCD EQU   FLCMCLA+84
FLCFSAA  EQU   FLCFSA+1
@NM00095 EQU   FLCFSA
@NM00094 EQU   FLCMCLA+72
FLCMCIC  EQU   FLCMCLA+64
@NM00093 EQU   FLCMCLA+20
FLCIOAA  EQU   FLCIOA+1
@NM00092 EQU   FLCIOA
@NM00091 EQU   FLCMCLA+15
@NM00090 EQU   FLCMCLA+14
@NM00089 EQU   FLCMCLA+12
FLCLCL   EQU   FLCMCLA+8
FLCIOELA EQU   FLCIOEL+1
@NM00088 EQU   FLCIOEL
FLCCHNID EQU   FLCMCLA
@NM00087 EQU   PSA+160
FLCMTRCD EQU   PSA+157
@NM00086 EQU   PSA+156
FLCPERA  EQU   FLCPER+1
@NM00085 EQU   FLCPER
@NM00084 EQU   PSA+151
FLCPERCD EQU   PSA+150
FLCMCNUM EQU   PSA+149
@NM00083 EQU   PSA+148
FLCTEAA  EQU   FLCTEA+1
@NM00082 EQU   FLCTEA
PSAPIPC  EQU   PSAPICOD
PSAPIMC  EQU   PSAPICOD
PSAPIPER EQU   PSAPICOD
PSARV049 EQU   FLCPICOD
FLCPILCB EQU   FLCPIILC
@NM00081 EQU   FLCPIILC
@NM00080 EQU   PSAEPPSW
FLCSVCN  EQU   PSAESPSW+2
FLCSILCB EQU   FLCSVILC
@NM00079 EQU   FLCSVILC
@NM00078 EQU   PSAESPSW
FLCEICOD EQU   PSAEEPSW+2
PSASPAD  EQU   PSAEEPSW
@NM00077 EQU   PSA+128
FLCINPSW EQU   PSA+120
FLCMNPSW EQU   PSA+112
FLCPNPSW EQU   PSA+104
FLCSNPSW EQU   PSA+96
FLCENPSW EQU   PSA+88
FLCTRACE EQU   PSA+84
FLCTIMER EQU   PSA+80
FLCCVT2  EQU   PSA+76
FLCCAW   EQU   PSA+72
FLCCSW   EQU   PSA+64
FLCIOPSW EQU   PSA+56
FLCMOPSW EQU   PSA+48
FLCPOPSW EQU   PSA+40
FLCSOPSW EQU   PSA+32
FLCEOPSW EQU   PSA+24
@NM00076 EQU   FLCICCW2+4
FLCCVT   EQU   FLCICCW2
FLCICCW1 EQU   FLCROPSW
FLCIPPSW EQU   FLCRNPSW
.@UNREFD ANOP                          END UNREFERENCED COMPONENTS
@PB00015 EQU   @EL00001
@ENDDATA EQU   *
         END   IGG0CLB4,(C'PLS1915',0701,76057)
