       TITLE 'VPSS SUBSYSTEM INITIALIZATION TABLE'
**********************************************************************
*                                                                    *
*        IGT0VPSS - VPSS SUBSYSTEM INITIALIZATION TABLE              *
*                                                                    *
*        MVS/VS1    09/13/76    LEVEL 01.1                           *
*                                                                    *
*     METHOD OF ACCESS - IGT0VPSS IS ASSEMBLED AND LINK-EDITED INTO  *
*                        SYS1.LOADVPSS AS IGT0VPSS. SUBSYSTEM        *
*                        INITIALIZATIION ISSUES A LOAD OF            *
*                        IGT0VPSS.                                   *
*                                                                    *
*     FUNCTION - IGT0VPSS CONTAINS SEVERAL TABLES ORGANIZED AS       *
*                FOLLOWS:                                            *
*                                                                    *
*                  (1) APOT - VPSS SUBSYSTEM OPTIONS TABLE           *
*                             THIS PORTION OF THE APOT CONTAINS      *
*                             POINTERS TO THE OTHER APOT TABLES -    *
*                             APOTNM,APOTVT,APOTAD                   *
*                  (2) APCT - VPSS SUBSYSTEM CONTROL TABLE           *
*                             THE APCT CONTAINS THE ADDRESSES OF     *
*                             VPSS PROCESSING ROUTINES, POINTERS TO  *
*                             VARIOUS VPSS TABLES AND INSTALLATION-  *
*                             DEFINED PROCESSING PARAMETERS.         *
*                  (3) ARPT - APCP ROUTINE POINTER TABLE
*                             THIS TABLE CONTAINS ADDRESSES OF       *
*                             ROUTINES WHICH PERFORM USER CALL       *
*                             PROCESSING AS DEFINED IN THE APCP.     *
*                  (4) APCP/APAL - VPSS CALL INTERFACE PROCESSOR     *
*                             PARAMETERS/VPSS ALGORITHM BUILD MASKS  *
*                  (5) APOTNM - VPSS MODULE NAME TABLE               *
*                             THIS TABLE CONTAINS THE NAMES OF VPSS  *
*                             MODULES TO BE LOADED AND THE APCT      *
*                             OFFSET WHERE THE LOADED MODULE'S       *
*                             ADDRESS IS TO BE PLACED.               *
*                  (6) APOTVT - VPSS SUBSYSTEM VECTOR TABLE          *
*                             SSVT INITIALIZATION PARAMETERS.        *
*                  (7) APOTAD - APCT RELOCATION TABLE                *
*                             USED TO MODIFY THE APCT INTERNAL       *
*                             POINTERS.                              *
*                                                                    *
*      THE FUNCTION AND METHOD OF CHANGE FOR EACH TABLE IS EXPLAINED *
*     IN THE PROLOGUE TO EACH MACRO DEFINITION.                      *
*                                                                    *
**********************************************************************
         EJECT
**********************************************************************
*                                                                    *
*                   IGT0VPSS INTERNAL MACROS.                        *
*                                                                    *
*        THE FOLLOWING MACROS ARE USED INTERNALLY TO GENERATE        *
*        PORTIONS OF IGT0VPSS. THE EXPLANATION PRECEDING EACH        *
*        MACRO DEFINITION INDICATES THE FUNCTION AND USE OF THE      *
*        MACRO.                                                      *
*                                                                    *
**********************************************************************
         EJECT
**********************************************************************
*                                                                    *
*                       IGT0APCT MACRO                               *
*                                                                    *
*         FUNCTION - THIS MACRO IS USED TO CHANGE VALUES IN THE      *
*                    APCT. INVOCATIONS OF THIS MACRO MUST FOLLOW     *
*                    THE GENERATION OF THE APCT BY THE VPSS MACRO    *
*                    IGTXAPCT. THIS MACRO ORGS TO THE SPECIFIED      *
*                    APCT LABEL AND REDEFINES THE DATA AT THAT       *
*                    LABEL.                                          *
*        MACRO INVOCATION -                                          *
*                   APCTLBL  IGT0APCT APCTNVAL                       *
*                   WHERE APCTLBL IS THE LAST 1-4 CHARACTERS(FOLLOW- *
*                   THE CHARACTERS APCT) OF THE LABEL IN APCT FOR    *
*                   WHICH THE CHANGE IS TO BE MADE                   *
*                   AND APCTNVAL IS THE REPLACEMENT VALUE.           *
*                   THIS VALUE MUST HAVE THE SAME LENGTH ATTRIBUTE   *
*                   AS THE VALUE IT REPLACES.                        *
*        EXAMPLE OF USE -                                            *
*                   TO CHANGE THE SUBSYSTEM NAME TO XXXX, THE        *
*                   FOLLOWING MACRO INVOCATION WOULD FOLLOW THE      *
*                   GENERATION OF THE APCT:                          *
*                   SSID   IGT0APCT   CL4'XXXX'                      *
*                                                                    *
**********************************************************************
         SPACE 5
         MACRO                     IGT0APCT
&APCTLBL IGT0APCT &APCTVAL         APCT CHANGE MACR0
.*
.** THIS MACRO IS USED TO CHANGE VALUES IN THE APCT. THIS IS
.** ACCOMPLISHED BY 'ORG'ING TO THE SPECIFIED APCT LABEL AND
.**'DC'ING A NEW VALUE AT THAT LABEL.
.** INPUT ---
.**          &APCTLBL  - THE LAST 1-4 CHARACTERS(FOLLOWING THE
.**                      CHARACTERS 'APCT') OF THE APCT LABEL FOR WHICH
.**                      THE CHANGE IS TO BE MADE.
.**          &APCTVAL  - THE NEW VALUE TO BE INSERTED AT THE LOCATION
.**                      SPECIFIED BY &APCTLBL.
.*
         LCLA  &LNP                HOLDS LENGTH OF &APCTLBL PARAMETER
&LNP     SETA  K'&APCTLBL          GET NUMBER OF CHARACTERS IN
.*                                 &APCTLBL PARAMETER
         AIF   (&LNP LT 1 OR &LNP GT 4).BADLBL
.*                                 .BADLBL IF NUMBER OF CHARACTERS IS
.*                                 ZERO OR GT 5
         ORG   APCT&APCTLBL        ORG TO DESIRED APCT LOCATION
         DC    &APCTVAL            DEFINE NEW APCT VALUE
         AGO   .APCTOVR            CHANGE IS MADE - GO TO .APCTOVR
.BADLBL  ANOP  ,                   &APCTLBL IS BAD - ISSUE ERROR MSG.
         IGTXMERR ERRN=1,KEYWORD=APCTLBL,ERRLVL=8,VALUE=&APCTLBL
.APCTOVR ANOP  ,
         ORG   ,
         MEND
         EJECT
***********************************************************************
*                                                                     *
*                          IGT0MNT MACRO                              *
*                                                                     *
*           FUNCTION - THIS MACRO IS USED TO GENERATE APOTNM ENTRIES  *
*                      WHICH TELL VPSS WHICH MODULES TO LOAD.         *
*                      SEE THE APOTNM DSECT WITHIN IGTXAPOT FOR       *
*                      APOTNM FORMAT.                                 *
*                                                                     *
*           MACRO INVOCATION -                                        *
*                                                                     *
*           LABEL   IGT0MNT SUFFIX,TYPE,SUNBR,LOAD,SYST=,FIX=,NAME=   *
*                                                                     *
*                      WHERE:                                         *
*                           LABEL - LABEL DESIRED FOR THIS APOTNM     *
*                                   ENTRY. IF OMITTED, A LABEL OF THE *
*                                   FORM IGTT(TYPE)(SUFFIX) IS        *
*                           SUFFIX- MODULE SUFFIX(REQUIRED PARAMETER) *
*                           TYPE  - MODULE TYPE(REQUIRED PARAMETER)   *
*                                   = Z - VPSS ROUTINE.               *
*                                   = @ - USER EXIT ROUTINE.          *
*                           SUNBR - SU NUMBER FOR THIS MODULE(IF      *
*                                   APPLICABLE)                       *
*                           LOAD  - MODULE LOAD FLAG(REQUIRED ONLY IF *
*                                   SUNBR APPLIES.                    *
*                                   = YES - IF SUNBR APPLID - LOAD    *
*                                   THIS MODULE.                      *
*                                   = NO  - IF SUNBR APPLIED - DO NOT *
*                                   LOAD THIS MODULE.                 *
*                           SYST= - KEYWORD PARAMETER USED TO SPECIFY *
*                                   FOR WHICH SYSTEM MODULE APPLIES.  *
*                                   = VS1 - LOAD THIS MODULE FOR VS1  *
*                                           ONLY.                     *
*                                   = MVS - LOAD THIS MODULE FOR MVS  *
*                                           ONLY.                     *
*                                   = ALL(DEFAULT) - LOAD THIS MODULE *
*                                           FOR MVS AND VS1.          *
*                           FIX=  - KEYWORD PARAMETER USED TO SPECIFY *
*                                   IF A MODULE IS TO BE PERMANENTLY  *
*                                   FIXED.                            *
*                                   = YES - FIX MODULE.               *
*                                   = NO(DEFAULT) - DO NOT FIX MODULE.*
*                           NAME= - KEYWORD PARAMETER USED TO         *
*                                   GENERATE MODULE NAME IF STAN-     *
*                                   DARD NAME OF IGT(SUFFIX) IS NOT   *
*                                   TO BE USED.                       *
*           EXAMPLE OF USE   - TO GENERATE AN APOTNM ENTRY FOR USER   *
*                              EXIT MODULE IGTXXX, APPLIED FOR SU     *
*                              NUMBER 99 WITH LOAD = YES, APPLICABLE  *
*                              TO MVS AND VS1 AND LABELED LABXXX, THE *
*                              FOLLOWING MACRO STATEMENT WOULD BE     *
*                              USED:                                  *
*                                                                     *
*           LABXXX  IGT0MNT XXX,@,99,YES                              *
*                                                                     *
***********************************************************************
         SPACE 5
         MACRO                          IGT0MNT
&NMLBL   IGT0MNT &SUFFIX,&TYPE,&SUNBR,&LOAD,&SYST=ALL,&FIX=NO,&NAME=
.*                                      GENERATE APOTNMT ENTRY
.*
.** THIS MACRO IS USED TO GENERATE APOTNMT ENTRIES. FIRST, THE SYST
.** KEYWORD IS CHECKED AGAINST THE SYSTEM BEING USED(FROM IGTXSYSA).
.** IF THE SYSTEM DOES NOT MATCH, THIS MACRO EXITS. OTHERWISE, AN 8
.** CHARACTER NAME IS DEFINED VIA 'DC'. THE NAME IS OF THE FORM
.** IGT(SUFFIX) UNLESS A MODULE NAME HAS BEEN SUPPLIED VIA THE NAME
.** KEYWORD. NEXT, USING THE TYPE PARAMETER, THE APCT OFFSET IS
.** DETERMINED AND PLACED INTO A HALFWORD LOCATION. THE NEXT TWO
.** BYTES ARE CONFIGURED ACCORDING TO THE LOAD PARAMETER. AND THE
.** LAST HALFWORD IS FILLED WITH THE APPLICABLE SU NUMBER, IF ANY.
.** THE ENTRY LABEL IS THE USER'S IF SUPPLIED. OTHERWISE, THIS MACRO
.** GENERATES A LABEL OF THE FORM IGTT(TYPE)(SUFFIX).
.** INPUT ---
.**          &NMLBL    - DESIRED ENTRY LABEL OR BLANK IF THIS MACRO IS
.**                      TO GENERATE LABEL.
.**          &SUFFIX   - SUFFIX OF MODULE TO BE LOADED.(MODULE NAME
.**                      WITHOUT CHARACTERS'IGT').
.**          &TYPE     - MODULE TYPE.
.**                      = Z - VPSS ROUTINE.
.**                      = @ - APCP ROUTINE POINTER.
.**          &SUNBR    - SU NUMBER FOR THIS MODULE, IF APPLICABLE.
.**          &LOAD     - MODULE LOAD FLAG(APPLICABLE ONLY IF SUNBR
.**                      USED).
.**                      = YES - IF SUNBR APPLIED - LOAD
.**                        THIS MODULE.
.**                      = NO  - IF SUNBR APPLIED - DO NOT
.**                        LOAD THIS MODULE.
.**          &SYST    - SYSTEM FLAG.
.**                     = VS1 - VS1 ONLY MODULE.
.**                     = MVS - MVS ONLY MODULE.
.**                     = ALL(DEFAULT) - MVS AND VS1 MODULE.
.**          &FIX     - MODULE FIX FLAG.
.**                     = YES - FIX MODULE.
.**                     = NO  - DO NOT FIX MODULE.
.**          &NAME    - MODULE NAME. USED IF MODULE NAME TO
.**                     BE OTHER THAN THE STANDARD FORM OF
.**                     IGT(SUFFIX).
.*
         COPY  IGTXSYSA                 GET SYSTEM NAME/LEVEL
         AIF   ('&SYST' EQ 'ALL').DONM  .DONM IF FOR MVS AND VS1
         AIF   ('&SYST' EQ 'VS1').CKVS1 .CKVS1 IF SYST=VS1
         AIF   ('&SYST' NE 'MVS').SYSER .SYSER IF SYST NE MVS/VS1/ALL
.*
.**                                     MODULE IS FOR MVS ONLY
.*
         AIF   ('&VPSSSYS' EQ 'MVS').DONM  .DONM IF THIS IS MVS SYSTEM
         MEXIT                          IGNORE THIS REQUEST IF NOT MVS
.CKVS1   AIF   ('&VPSSSYS' EQ 'VS1').DONM  .DONM IF THIS IS VS1 SYSTEM
         MEXIT                          IGNORE THIS REQUEST IF NOT VS1
.DONM    AIF   ('&SUFFIX' EQ ' ').NOSUF .NOSUF IF SUFFIX OMITTED
         AIF   ('&TYPE' EQ '').NOTYPE   .NOTYPE IF TYPE OMITTED
         AIF   ('&FIX' EQ 'YES' OR '&FIX' EQ 'NO').FXOK
.*                                      VALIDATE FIX PARAMETER
         IGTXMERR ERRN=1,ERRLVL=12,KEYWORD=FIX,VALUE=&FIX,             *
               RANGE=(YES,NO)           ISSUE SEVERE ERROR MSG IF FIX
         MEXIT                          PARM BAD AND EXIT
.FXOK    AIF   ('&SUNBR' EQ '' AND '&LOAD' NE '').NOSUNB
.*                                      .NOSUNB IF LOAD SPECIFIED BUT
.*                                      SUNBR NOT SPECIFIED
         AIF   ('&LOAD' EQ '' AND '&SUNBR' NE '').NOLOAD
.*                                      .NOLOAD IF SUNBR SPECIFIED BUT
.*                                      LOAD NOT SPECIFIED
         AIF   ('&LOAD' EQ '').LDCK     .LDCK IF LOAD DOES NOT NEED
.*                                      VALIDITY CHECK
         AIF   ('&LOAD' EQ 'YES' OR '&LOAD' EQ 'NO').LDCK
.*                                      .LDCK IF LOAD OK
         IGTXMERR ERRN=1,ERRLVL=12,KEYWORD=LOAD,VALUE=&LOAD,           *
               RANGE=(YES,NO)           ISSUE SEVERE ERROR MESSAGE
         MEXIT                          AND EXIT
.LDCK    ANOP  ,                        ALL PARAMETERS CHECKED
.* DEFINE THE MODULE NAME
         AIF   ('&NMLBL' NE  '').LBLGVN .LBLGVN IF LABEL SUPPLIED
         AIF   ('&NAME' EQ '').NONMLB   .NONMLB IF NAME NOT SUPPLIED
IGTT&TYPE&SUFFIX DC CL8'&NAME'          APOTNMLD
         AGO   .OFFSET                  .OFFSET W/ MY LABEL,USER NAME
.NONMLB  ANOP  ,                        NO USER LABEL OR NAME
IGTT&TYPE&SUFFIX DC CL8'IGT&SUFFIX' APOTNMLD
         AGO   .OFFSET                  GIVE IT A LABEL - GO TO .OFFSE
.LBLGVN  AIF   ('&NAME' EQ '').LBNONM   .LBNONM IF NAME NOT SUPPLIED
&NMLBL   DC CL8'&NAME'                  APOTNMLD
         AGO   .OFFSET                  .OFFSET MY NAME,MY LABEL
.LBNONM  ANOP  ,                        NO USER LABEL,USER NAME
&NMLBL   DC    CL8'IGT&SUFFIX'          APOTNMLD
.OFFSET  ANOP  ,                        MODULE NAME DEFINED
         AIF   ('&TYPE' EQ 'Z').TYPEZ   .TYPEZ IF TYPE = Z
         AIF   ('&TYPE' EQ '@').TYPE@   .TYPE@ IF TYPE = @
.* DEFINED APCT OFFSET FOR NON-Z OR NON-@ TYPE ROUTINES
         DC    Y(APCT&TYPE&SUFFIX-APCT) APOTNMOF
         AGO   .FLAGS                   OFFSET DEFINED
.TYPEZ   ANOP  ,
.* DEFINE Z-TYPE APCT OFFSET
         DC    Y(APCTZ&SUFFIX-APCT)     APOTNMOF
         AGO   .FLAGS                   Z-TYPE OFFSET DEFINED
.TYPE@   ANOP  ,
.* DEFINE @-TYPE APCT OFFSET
         DC    Y(@&SUFFIX-APCT)         APOTNMLD
.FLAGS   ANOP  ,
         AIF   ('&SUNBR' EQ '').NOSULD  .NOSULD IF NO SUNBR
         AIF   ('&LOAD' EQ 'YES').YESL  .YESL IF LOAD = YES
         AIF   ('&FIX' EQ 'YES').NOLDFX .NOLDFX IF FIX=YES
         DC    X'20'                    APOTNMF0
         AGO   .SU                      .SU WITH LOAD=NO,FIX=NO
.NOLDFX  DC    X'A0'                    APOTNMF0
         AGO   .SU                      .SU WITH LOAD=NO,FIX=YES
.YESL    AIF   ('&FIX' EQ 'YES').LDFX   .LDFX IF FIX=YES
         DC    X'40'                    APOTNMF0
         AGO   .SU                      .SU WITH LOAD=YES,FIX=NO
.LDFX    DC    X'C0'                    APOTNMF0
.SU      DC    AL1(&SUNBR)              APOTNMSU
         AGO   .NMDONE
.NOSULD  AIF   ('&FIX' EQ 'YES').FX     .FX IF FIX=YES
         DC    X'00'                    APOTNMF0
         AGO   .DONMSU                  .DONMSU WITH FIX=NO
.FX      DC    X'80'                    APOTNMF0
.DONMSU  ANOP  ,                        FINISH NO SU PATH
         DC    AL1(0)                   APOTNMSU
         AGO   .NMDONE
.NOSUF   IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=SUFFIX
.*                                      SUFFIX MISSING - ISSUE ERR MSG
         MEXIT                          AND EXIT
.NOTYPE  IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=TYPE
.*                                      TYPE MISSING - ISSUE ERR MSG
         MEXIT                          AND EXIT
.NOSUNB  IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=SUNBR
.*                                      SUNBR MISSING - ISSUE ERR MSG
         MEXIT                          AND EXIT
.NOLOAD  IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=LOAD,RANGE=(YES,NO)
.*                                      LOAD MISSING - ISSUE ERR MSG
.*                                      AND EXIT
.NMDONE  MEXIT
.SYSER   IGTXMERR ERRN=1,ERRLVL=12,KEYWORD=SYST,VALUE=&SYST
.*                                      ISSUE SEVERE ERROR MESSAGE
         MEXIT                          AND EXIT
         MEND
       EJECT
***********************************************************************
*                                                                     *
*                        IGT0CTRT MACRO                               *
*                                                                     *
*            FUNCTION - THIS MACRO IS USED TO GENERATE THE SKELETON   *
*                       APCT RELOCATION TABLE - APOTAD.               *
*            METHOD OF INVOCATION -                                   *
*                                                                     *
*                   LABEL    IGT0CTRT &SUFFIX,&CTRTLN                 *
*            WHERE:                                                   *
*                   &SUFFIX    - APCT SUFFIX OF RELOCATION TABLE      *
*                   &CTRTLN    - NUMBER OF ENTRIES TO GENERATE        *
*            EXAMPLE OF USE - TO GENERATE 100 APOTAD ENTRIES AT APCT  *
*                       RELOCATION TABLE U@@@, THE FOLLOWING MACRO    *
*                       STATEMENT WOULD BE USED:
*                                                                     *
*            LABEL   IGT0CTRT U@@@,100
*                                                                     *
***********************************************************************
       SPACE 5
       MACRO                        IGT0CTRT
&CTRTLBL IGT0CTRT &SUFFIX,&CTRTLN   GENERATE THE APCT RELOCATION TABLE
.*
.** THIS MACRO IS USED TO GENERATE THE APOTAD TABLE. CTRTLN HALFWORD
.** APOTAD ENTRIES ARE GENERATED FOR APCT(SUFFIX).
.*
       LCLA  &CTR                   HOLDS &CTRTLN FOR LOOP CONTROL
       LCLA  &OFF                   HOLDS APCT OFFSETS
       AIF   ('&SUFFIX' EQ '').NOSUF
.*                                  .NOSUF IF APCT SUFFIX OMITTED
       AIF   ('&CTRTLN' EQ '').NOCTL
.*                                  .NOCTL IF CTRTLN OMITTED
&CTR   SETA  &CTRTLN                GET NUMBER OF ENTRIES TO GENERATE
&OFF   SETA  0                      START AT OFFSET ZERO
.* GENERATE AN APOTAD ENTRY
.ADLOOP DC    AL2(APCT&SUFFIX+&OFF-APCT)  APOTADOF
&CTR   SETA  &CTR-1                 DECREMENT COUNTER
&OFF   SETA  &OFF+4                 BUMP APCT OFFSET BY 4 BYTES
       AIF   (&CTR GT 0).ADLOOP     LOOP TILL ALL DONE
       AGO   .ADDONE                APOTAD GENERATED
.* HANDLE IGT0CTRT ERRORS. ISSUE SEVERE ERROR MESSAGE AND EXIT.
.NOSUF IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=SUFFIX
.*                                  SUFFIX OMITTED
       MEXIT                        EXIT - NO APOTAD ENTRIES CREATED
.NOCTL IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=CTRTLN
.*                                  CTRTLN OMITTED
       MEXIT                        EXIT - NO APOTAD ENTRIES CREATED
.ADDONE MEND                        END OF IGT0CTRT PROCESSING
       EJECT
***********************************************************************
*                                                                     *
*                           IGT0SSVT MACRO                            *
*                                                                     *
*            FUNCTION - THIS MACRO IS USED TO GENERATE APOTVT         *
*  ENTRIES IN THE FORMAT DESCRIBED IN THE APOTVT DSECT WITHIN         *
*  IGTXAPOT.                                                          *
*                                                                     *
*            METHOD OF INVOCATION -                                   *
*            SSVTLBL IGT0SSVT &MFB,&ACTIVAT,&SUFFIX,&SYSTM=           *
*            WHERE:                                                   *
*                 SSVTLBL  - LABEL DESIRED AT GENERATED ENTRY         *
*                 &MFB     - MATRIX FUNCTION BYTE                     *
*                            = 0   THE FUNCTION SPECIFIED IN THE SSOB *
*                                  IS NOT SUPPORTED BY THIS SYSTEM    *
*                            ^ 0   THE VALUE (FUNCTION BYTE -1)*4     *
*                                  IS ADDED TO THE ADDRESS OF THE     *
*                                  SSVTFRTN TO OBTAIN THE ADDRESS     *
*                                  OF THE WORD CONTAININING THE       *
*                                  FUNCTION ROUTINE POINTER FOR THIS  *
*                                  REQUEST.                           *
*                 &ACTIVAT -                                          *
*                            = S   FUNCTION IS TO BE ACTIVATED BY     *
*                                  SUBSYSTEM STARTUP.                 *
*                            = M   FUNCTION IS TO BE ACTIVATED BY     *
*                                  MONITOR INITIALIZATION.            *
*                 &SUFFIX  - MODULE SUFFIX - USED TO DETERMINE OFFSET *
*                            WITHIN APCT OF THE REQUIRED MODULE.      *
*                  &SYSTM=  - KEYWORD PARAMETER USED TO SPECIFY FOR   *
*                             WHICH SYSTEM(MVS,VS1 OR BOTH) THE SSVT  *
*                             ENTRY IS TO BE GENERATED.               *
*                             = MVS - GENERATE FOR MVS ONLY.          *
*                             = VS1 - GENERATE FOR VS1 ONLY.          *
*                             = ALL(DEFAULT) - GENERATE FOR VS1 & MVS *
*                                                                     *
***********************************************************************
       SPACE 5
       MACRO                                   IGT0SSVT
&SSVTLBL IGT0SSVT &MFB,&ACTIVAT,&SUFFIX,&SYSTM=ALL   GENERATE APOTVT
.*                                                   ENTRY
         COPY  IGTXSYSA                 GET SYSTEM NAME/LEVEL
         AIF   ('&SYSTM' EQ 'ALL').DOVT .DOVT IF FOR VS1 AND MVS
         AIF   ('&SYSTM' EQ 'VS1').CKVS1 .CKVS1 IF SYSTM=VS1
         AIF   ('&SYSTM' NE 'MVS').BADSYS .BADSYS IF ILLEGAL SYSTM VAL
.*
.**                                     SSVT ENTRY TO BE GENERATED FOR
.**                                     MVS ONLY
.*
         AIF   ('&VPSSSYS' EQ 'MVS').DOVT .DOVT IF THIS IS MVS SYSTEM
         MEXIT                          NO SSVT ENTRY IF THIS IS VS1
.CKVS1   ANOP  ,                        GENERATE SSVT IF THIS IS VS1
         AIF   ('&VPSSSYS' EQ 'VS1').DOVT .DOVT IF THIS IS VS1 SYSTEM
         MEXIT                          NO SSVT ENTRY IF THIS IS MVS
.DOVT    AIF   ('&MFB' EQ ' ').NOMFB   .NOMFB IF MATRIX FUNCTION
.*                                        BYTE OMITTED
       AIF   ('&ACTIVAT' EQ '').NOACT     .NOACT IF ACTIVATE FLAG
.*                                        OMITTED
       AIF   ('&SUFFIX' EQ '').NOSUF      .NOSUF IF SUFFIX OMITTED
       AIF   ('&ACTIVAT' EQ 'S').GDACT    .GDACT IF ACTIVAT=S
       AIF   ('&ACTIVAT' NE 'M').BADACT   .BADACT IF ACTIVAT INVALID
.GDACT ANOP  ,                            ACTIVAT VALID HERE
.* DEFINE THE MATRIX FUNCTION BYTE
&SSVTLBL DC    AL1(&MFB)                  APOTVTCD
       AIF   ('&ACTIVAT' EQ 'S').STARTUP  .STARTUP IF ACTIVAT=S
.* DEFINE THE ACTIVATE FLAG AS MONITOR.
       DC    X'40'                        APOTVTIT
       AGO   .DOOF                        .DOOF
.STARTUP DC  X'80'                        APOTVTIT
.* USE SUFFIX TO DEFINE OFFSET WITHIN APCT OF ADDRESS OF
.* REQUIRED MODULE.
.DOOF  DC    Y(APCTZ&SUFFIX-APCT)         APOTVTOF
       AGO   .SSVTDUN                     THAT'S ALL
.* HANDLE IGT0SSVT ERRORS. ISSUE SEVERE ERROR MESSAGE AND EXIT.
.NOMFB IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=MFB
.*                                        MFB OMITTED
       MEXIT
.NOACT IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=ACTIVAT
.*                                        ACTIVAT OMITTED
       MEXIT
.NOSUF IGTXMERR ERRN=2,ERRLVL=12,KEYWORD=SUFFIX
.*                                        SUFFIX OMITTED
       MEXIT
.BADACT IGTXMERR ERRN=1,ERRLVL=12,KEYWORD=ACTIVAT,VALUE=&ACTIVAT,      *
               RANGE=(S,M)                ACTIVAT INVALID
       MEXIT
.BADSYS  IGTXMERR ERRN=1,ERRLVL=12,KEYWORD=SYSTM,VALUE=&SYSTM
.*                                      ISSUE SEVERE ERROR MESSAGE
         MEXIT                          AND EXIT
.SSVTDUN MEND                             END IGT0SSVT PROCESSING
       EJECT
***********************************************************************
*                   END OF IGT0VPSS INTERNAL MACROS                   *
***********************************************************************
       EJECT
***********************************************************************
*                            IGT0VPSS                                 *
*                                                                     *
*                BEGIN GENERATION OF IGT0VPSS TABLES.                 *
*                                                                     *
***********************************************************************
       EJECT
***********************************************************************
*                                                                     *
*                         IGTXAPOT                                    *
* LIST THE DSECT APOT. THIS WILL BE FOLLOWED BY THE ACTUAL VALUES FOR *
* THE APOT.                                                           *
*                                                                     *
***********************************************************************
       SPACE 5
       IGTXAPOT LIST=YES               LIST THE APOT DSECT
       SPACE 5
***********************************************************************
*                                                                     *
*                       APOT DATA TABLE                               *
*                                                                     *
*      THE FOLLOWING TABLE IS THE ACTUAL BEGINNING OF THE IGT0VPSS    *
* DATA SET AND CONTAINS APOT DATA IN THE FORMAT DESCRIBED IN THE      *
* APOT DSECT WITHIN IGTXAPOT. THIS PORTION OF THE TABLE CONTAINS      *
* POINTERS TO THE OTHER APOT TABLES. REFER TO THE APOT DSECT WHEN IN- *
* TERPRETING THIS TABLE. THE COMMENTS FIELD OF EACH FOLLOWING APOT    *
* DATA DEFINITION INDICATES THE IGTXAPOT LABEL FOR THAT VALUE.        *
* THIS SECTION OF APOT SHOULD NOT BE CHANGED UNLESS THE IGTXAPOT IS   *
* REDEFINED.                                                          *
*      NOTE THAT VALUES IN THIS TABLE ARE ONLY FOR THE APOT DSECT     *
* WITHIN IGTXAPOT. VALUES FOR THE DSECTS APOTNM, APOTVT AND APOTAD    *
* ARE GIVEN LATER ON WITHIN IGT0VPSS.                                 *
*                                                                     *
***********************************************************************
       SPACE 5
IGT0VPSS CSECT                         VPSS OPTIONS TABLE
       DS    0F                        APOT
       DC    CL4'APOT'                 APOTID
       DC    A(*-*)                    APOTUSER
       DC    A(APCT)                   APOTCTAD - ADDRESS OF APCT
       DC    A(APCTEND-APCT)           APOTCTLN - LENGTH OF APCT
       DC    A(MNT)                    APOTNMAD - ADDRESS OF APOTNM -
*                                                 MODULE NAME TABLE
       DC    A(MNT#)                   APOTNMCT - NUMBER OF APOTNM
*                                                 ENTRIES
       DC    A(SSVTFT)                 APOTVTAD - ADDRESS OF APOTVT -
*                                                 SUBSYSTEM VECTOR
*                                                 TABLE(SSVT)
       DC    A(SSVT#)                  APOTVTCT - NUMBER OF SSVT
*                                                 ENTRIES
       DC    A(APCTRT)                 APOTADAD - ADDRESS OF APCT
*                                                 RELOCATION TABLE
       DC    A(APCTRT#)                APOTADCT - NUMBER OF APCT
*                                      RELOCATION TABLE ENTRIES
       DC    A(APCTBR14-APCT)          APOTBR14
       DC    A(CAPCP)                  APOTCPAD
       DC    A(2+(CAPCPZ-CAPCP-APCPLN0)/APCPLN1) APOTCPCT
       DC    A(APCPLN0)                APOTCPPL
       DC    A(APCPLN1)                APOTCPLN
       DS    0D
***********************************************************************
*                                                                     *
*                     END OF APOT DATA TABLE                          *
*                                                                     *
***********************************************************************
       EJECT
***********************************************************************
*                                                                     *
*                          APCT DATA TABLE                            *
*                                                                     *
*      THE APCT DATA TABLE IS GENERATED BY USING THE CSECT OPTION     *
* (DSECT=NO) ON THE MACRO INVOCATION IGTXAPCT. REFER TO THE COMMENTS  *
* IN THE EXPANDED IGTXAPCT FOR INTERPRETATION OF THE APCT. IF A       *
* CHANGE IS DESIRED TO THE APCT, USE THE INTERNAL MACRO IGT0APCT      *
* TO EFFECT CHANGES. REFER TO THE IGT0APCT MACRO FOR AN EXPLANATION   *
* OF ITS USE. CHANGES MUST BE INSERTED AFTER THE GENERATION OF THE    *
* APCT AT THE SECTION TITLED 'CHANGES TO THE APCT'.                   *
*                                                                     *
***********************************************************************
         SPACE 5
         IGTXAPCT LIST=YES,DSECT=NO    GENERATE THE APCT
ENDAPCT  EQU   *
         EJECT
***********************************************************************
*                                                                     *
*                      CHANGES TO THE APCT                            *
*                                                                     *
*      CHANGES TO THE APCT ARE TO BE MADE HERE. THE INTERNAL MACRO    *
* IGT0APCT MUST BE USED TO EFFECT APCT CHANGES. REFER TO THE IGT0APCT *
* MACRO FOR INSTRUCTIONS ON ITS USE.                                  *
*                                                                     *
***********************************************************************
         SPACE 5
CRTN     IGT0APCT   A(CAPCPRTN-APCT)     DEFINE ADDR OF APCP CALL TABLE
         SPACE 5
***********************************************************************
*                                                                     *
*                      END OF APCT CHANGES                            *
*                                                                     *
***********************************************************************
         SPACE 5
***********************************************************************
*                                                                     *
*                      END OF APCT                                    *
*                                                                     *
***********************************************************************
         EJECT
***********************************************************************
*                                                                     *
*                    APCP ROUTINE POINTER TABLE                       *
*                                                                     *
*      THIS TABLE CONTAINS POINTERS TO ROUTINES USED IN CALL INTER-   *
* FACE PROCESSING. ENTRIES ARE REFERRED TO BY LABEL IN THE APCP       *
* DATA TABLE. AN ENTRY IN THE APCP MUST REFER TO ONE OF THES ENTRIES  *
* TO DETERMINE WHICH ROUTINE WILL PROCESS THE USER CALL. ENTRIES MUST *
* BE MADE IN THE SAME FORMAT. ENTRIES ARE CHANGED ONLY IF THE NAME OF *
* PROCESSING ROUTINE CHANGES OR A NEW ROUTINE IS ADDED TO HANDLE THE  *
* SAME CALL.                                                          *
* NOTE: THIS TABLE MUST FOLLOW THE APCT.
*                                                                     *
***********************************************************************
         SPACE 5
CAPCPRTN DS    0A                  APCP ROUTINE POINTER TABLE
@MJB     DC    V(IGTMJB)           GENERATE ACB
@MJC     DC    V(IGTMJC)           OPEN ACB
@MJD     DC    V(IGTMJD)           GENERATE AND OPEN ACB
@MJE     DC    V(IGTMJE)           MODIFY ACB
@MJF     DC    V(IGTMJF)           CLOSE ACB
@MJG     DC    V(IGTMJG)           FREE ACB
@MLA     DC    V(IGTMLA)           INITIAL BUILD REQUEST
@MFA     DC    V(IGTMFA)           LOGIC OPERATION CONSTRUCTION
@MDA     DC    V(IGTMDA)           ALGORITHM CONSTRUCTION
@MDB     DC    V(IGTMDB)           CHANNEL TRANSFER CONSTRUCTION
@MDC     DC    V(IGTMDC)           NTH ZERO CROSSING CONSTRUCTION
@MGA     DC    V(IGTMGA)           TRANSLATE
@MHA     DC    V(IGTMHA)           RELEASE
@MCA     DC    V(IGTMCA)           EXCW/EXC/WAIT ERROR PROCESSING
APCTEND  EQU   *                   END OF APCT FOR IGT0VPSS PURPOSES
***********************************************************************
*                                                                     *
*                  END OF APCP ROUTINE POINTER TABLE                  *
*                                                                     *
***********************************************************************
         EJECT
***********************************************************************
*                                                                     *
*                        IGTXAPCP DSECT                               *
*                                                                     *
*      LIST THE DSECT IGTXAPCP USED TO MAP VALUES IN THE APCP         *
* PORTIONS OF THE APCP/APAL DATA TABLE. REFER TO THIS DSECT WHEN      *
* INTERPRETING THE APCP ENTRIES.                                      *
*                                                                     *
***********************************************************************
         SPACE 5
         IGTXAPCP LIST=YES            LIST THE APCP DSECT
         EJECT
***********************************************************************
*                                                                     *
*                        IGTXAPAL DSECT                               *
*                                                                     *
*      LIST THE DSECT IGTXAPAL USED TO MAP VALUES IN THE APAL         *
* PORTIONS OF THE APCP/APAL DATA TABLE. REFER TO THIS DSECT WHEN      *
* INTERPRETING THE APAL ENTRIES.                                      *
*                                                                     *
***********************************************************************
         SPACE 5
         IGTXAPAL LIST=YES
         EJECT
***********************************************************************
*                                                                     *
*                        APCP/APAL DATA TABLE                         *
*                                                                     *
*      THE FOLLOWING TABLE CONTAINS APCP AND APAL DATA IN THE FORMAT  *
* DESCRIBED BY THE PRECEDING DSECTS IGTXAPCP AND IGTXAPAL. AN ENTRY   *
* EXISTS FOR EACH VPSS CALL. IF THE CALL IS NOT A 3838 OPERATION(VPSS *
* ONLY), CONSTANTS ARE DEFINED ONLY FOR IGTXAPCP. IF THE CALL IS A    *
* 3838 OPERATION, CONSTANTS ARE DEFINED FOR IGTXAPCP FOLLOWED BY      *
* THE CONSTANTS FOR IGTXAPAL. REFER TO THE IGTXAPCP AND IGTXAPAL      *
* DSECTS WHEN INTERPRETING THIS TABLE. THE COMMENTS FIELD OF EACH     *
* TABLE ENTRY INDICATES THE CORRESPONDING IGTXAPCP OR IGTXAPAL LABEL. *
* TO CHANGE A CONSTANT IN THIS TABLE, FIRST DETERMINE DESIRED NEW     *
* VALUE BY REFERRING TO THE APPROPRIATE DSECT. NEXT, FIND THE ENTRY   *
* IN THE APCP/APAL DATA TABLE FOR WHICH THE CHANGE IS TO BE IMPLE-    *
* MENTED. LOCATE THE CONSTANT WITHIN THE ENTRY USING THE LABEL NAMES  *
* IN THE COMMENTS FIELD OF THE ENTRY. FINALLY, MAKE THE DESIRED       *
* CHANGE, REASSEMBLE IGT0VPSS AND LINK-EDIT INTO SYS1.LOADVPSS AS     *
* IGT0VPSS. SINCE APCP/APAL DATA IS POSITIONALLY MAPPED VIA DSECT,    *
* CARE MUST BE EXERCISED IN MAINTAINING THE POSITIONAL CHARACTERIS-   *
* TICS OF THE ENTRY. A VALUE CANNOT BE DELETED - IT MUST BE CHANGED.  *
* THE LENGTHS OF THE FIELDS MAY NOT CHANGE.                           *
*                                                                     *
***********************************************************************
         SPACE 5
IGT0VPSS CSECT                         APCP/APAL DATA TABLE
*
***********************************************************************
*                                                                     *
*                             APCP HEADER                             *
*                                                                     *
***********************************************************************
CAPCP    DC    CL4'APCP'           APCPID
         DC    F'0'                APCPUSER
         DC    A(CAPCPZ-CAPCP)     APCPLAST
         DC    F'0'                RESERVED
         EJECT
***********************************************************************
*                                                                     *
*                        BEGIN APCT/APAL ENTRIES                      *
*                                                                     *
***********************************************************************
         EJECT
***********************************************************************
*                                                                     *
*                 APCP ENTRY FOR EXCW                                 *
*                  'EXECUTE AND WAIT'                                 *
*                                                                     *
***********************************************************************
         DC    CL4'EXCW'           APCPAPFN
         DC    A(APOPEXCW)         APCPAPOP
         DC    A(@MCA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'10000000'       APCPSAR
         DC    BL1'10000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'10000000'       APCPSRCL
         DC    BL1'10000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    XL48'00'            APAL
         EJECT
***********************************************************************
*                                                                     *
*                 APCP ENTRY FOR EXC                                  *
*                     'EXECUTE'                                       *
*                                                                     *
***********************************************************************
         DC    CL4'EXC '           APCPAPFN
         DC    A(APOPEXC)          APCPAPOP
         DC    A(@MCA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'10000000'       APCPSAR
         DC    BL1'10000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'10000000'       APCPSRCL
         DC    BL1'10000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    XL48'00'            APAL
         EJECT
***********************************************************************
*                                                                     *
*                 APCP ENTRY FOR WAIT                                 *
*                       'WAIT'                                        *
*                                                                     *
***********************************************************************
         DC    CL4'WAIT'           APCPAPFN
         DC    A(APOPWAIT)         APCPAPOP
         DC    A(@MCA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'10000000'       APCPSAR
         DC    BL1'10000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'10000000'       APCPSRCL
         DC    BL1'10000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    XL48'00'            APAL
         EJECT
***********************************************************************
*                                                                     *
*                      APCP ENTRY FOR GEN                             *
*                        'GENERATE ACB'                               *
*                                                                     *
***********************************************************************
         DC    CL4'GEN '           APCPAPFN
         DC    A(APOPGEN)          APCPAPOP
         DC    A(@MJB-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'99'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'00100000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'00100000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'00000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'00100000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    XL48'00'            APAL
         EJECT
***********************************************************************
*                                                                     *
*                      APCP ENTRY FOR GENO                            *
*                    'GENERATE AND OPEN ACB'                          *
*                                                                     *
***********************************************************************
         DC    CL4'GENO'           APCPAPFN
         DC    A(APOPGENO)         APCPAPOP
         DC    A(@MJD-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'99'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'00100000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'00100000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'00000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'00100000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    XL48'00'            APAL
         EJECT
***********************************************************************
*                                                                     *
*                    APCP ENTRY FOR OPEN                              *
*                       'OPEN ACB'                                    *
*                                                                     *
***********************************************************************
         DC    CL4'OPEN'           APCPAPFN
         DC    A(APOPOPEN)         APCPAPOP
         DC    A(@MJC-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'00110000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'00100000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         EJECT
***********************************************************************
*                                                                     *
*                       APCP ENTRY FOR MOD                            *
*                         'MODIFY ACB'                                *
*                                                                     *
***********************************************************************
         DC    CL4'MOD '           APCPAPFN
         DC    A(APOPMOD)          APCPAPOP
         DC    A(@MJE-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'99'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'01000000'       APCPSBLD
         DC    BL1'01000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'00100000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         EJECT
***********************************************************************
*                                                                     *
*                       APCP ENTRY FOR CLSE                           *
*                          'CLOSE ACB'                                *
*                                                                     *
***********************************************************************
         DC    CL4'CLSE'           APCPAPFN
         DC    A(APOPCLSE)         APCPAPOP
         DC    A(@MJF-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10010000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'10010000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'00000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'00100000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         EJECT
***********************************************************************
*                                                                     *
*                       APCP ENTRY FOR BLD                            *
*                    'INITIAL BUILD REQUEST'                          *
*                                                                     *
***********************************************************************
         DC    CL4'BLD '           APCPAPFN
         DC    A(APOPBLD)          APCPAPOP
         DC    A(@MLA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'07'               APCPMAXP
         DC    H'07'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         EJECT
***********************************************************************
*                                                                     *
*                       APCP ENTRY FOR XLTE                           *
*                       'TRANSLATE AND FIX'                           *
*                                                                     *
***********************************************************************
         DC    CL4'XLTE'           APCPAPFN
         DC    A(APOPXLTE)         APCPAPOP
         DC    A(@MGA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         EJECT
***********************************************************************
*                                                                     *
*                       APCP ENTRY FOR RLSE                           *
*                             'RELEASE'                               *
*                                                                     *
***********************************************************************
         DC    CL4'RLSE'           APCPAPFN
         DC    A(APOPRLSE)         APCPAPOP
         DC    A(@MHA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'10000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR ZMV                          *
*                         'ZERO MOVE'                                 *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'ZMV '           APCPAPFN
         DC    A(APOPZMV)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'10'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'12'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'10'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'41'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'00'             APALXFSW
         DC    XL1'00'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'00'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VMC                          *
*                     'VECTOR MOVE/CONVERT'                           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VMC '           APCPAPFN
         DC    A(APOPVMC)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'10'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'6C'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VMV                          *
*                        'VECTOR MOVE'                                *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VMV '           APCPAPFN
         DC    A(APOPVMV)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'15'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'10'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SMV                          *
*                        'SCALAR MOVE'                                *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SMV '           APCPAPFN
         DC    A(APOPSMV)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'10'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'84'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'01'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR CEM                          *
*                      'COMPLEX MULTIPLY'                             *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'CEM '           APCPAPFN
         DC    A(APOPCEM)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'14'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'41'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'8C'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8C'             APALUFSW
         DC    XL1'40'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'40'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SCEM                         *
*                   'SCALAR COMPLEX MULTIPLY'                         *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SCEM'           APCPAPFN
         DC    A(APOPSCEM)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'14'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8C'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8C'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR CMCC                         *
*              'COMPLEX MULTIPLY - COMPLEX CONJUGATE'                 *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'CMCC'           APCPAPFN
         DC    A(APOPCMCC)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'11'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'84'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR CMCO                         *
*              'COMPLEX MULTIPLY - CONJUGATE OUTPUT'                  *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'CMCO'           APCPAPFN
         DC    A(APOPCMCO)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'14'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'41'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'8C'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8C'             APALUFSW
         DC    XL1'40'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'40'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SCMC                         *
*           'SCALAR COMPLEX MULTIPLY - COMPLEX CONJUGATE'             *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SCMC'           APCPAPFN
         DC    A(APOPSCMC)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'14'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8C'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8C'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VPUT                         *
*                        'VECTOR WRITE'                               *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VPUT'           APCPAPFN
         DC    A(APOPVPUT)         APCPAPOP
         DC    A(@MDB-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'80'             APALIOTP
         DC    XL1'40'             APALIOCH
         DC    XL6'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VGET                         *
*                        'VECTOR READ'                                *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VGET'           APCPAPFN
         DC    A(APOPVGET)         APCPAPOP
         DC    A(@MDB-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'80'             APALIOTP
         DC    XL1'80'             APALIOCH
         DC    XL6'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XWR                          *
*                         'INDEX WRITE'                               *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XWR '           APCPAPFN
         DC    A(APOPXWR)          APCPAPOP
         DC    A(@MDB-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'05'               APCPMAXP
         DC    H'05'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'04'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'40'             APALIOTP
         DC    XL1'40'             APALIOCH
         DC    XL6'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XRD                          *
*                         'INDEX READ'                                *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XRD '           APCPAPFN
         DC    A(APOPXRD)          APCPAPOP
         DC    A(@MDB-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'05'               APCPMAXP
         DC    H'05'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'04'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'40'             APALIOTP
         DC    XL1'80'             APALIOCH
         DC    XL6'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR FFTR                         *
*              'FAST FOURIER TRANSFORM(FORWARD REAL)'                 *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'FFTR'           APCPAPFN
         DC    A(APOPFFTR)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'86'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR FFTC                         *
*             'FAST FOURIER TRANSFORM(FORWARD COMPLEX)'               *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'FFTC'           APCPAPFN
         DC    A(APOPFFTC)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'11'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'84'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR IFTR                         *
*                'FAST FOURIER TRANSFORM(INVERSE REAL)'               *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'IFTR'           APCPAPFN
         DC    A(APOPIFTR)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'86'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR IFTC                         *
*              'FAST FOURIER TRANSFORM(INVERSE COMPLEX)'              *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'IFTC'           APCPAPFN
         DC    A(APOPIFTC)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'86'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VFX                          *
*            'VECTOR FLOATING-POINT TO FIXED-POINT CONVERT'           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VFX '           APCPAPFN
         DC    A(APOPVFX)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'16'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'38'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'66'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'84'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0002'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR CVM                          *
*                     'CONVOLVING MULTIPLY'                           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'CVM '           APCPAPFN
         DC    A(APOPCVM)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'24'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8F'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8F'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'80'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR INT                          *
*                   'QUADRATIC INTERPOLATION'                         *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'INT '           APCPAPFN
         DC    A(APOPINT)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'24'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'87'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'87'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'80'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR DUMP                         *
*                            'DUMP'                                   *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'DUMP'           APCPAPFN
         DC    A(APOPDUMP)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'02'               APCPMAXP
         DC    H'02'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'02'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'08'               APALLTYP
         DC    XL1'00'             APALLI1F
         DC    XL1'00'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR DUMP - MSG ENTRY             *
*             THIS ENTRY ALLOWS THE CALL ROUTER MESSAGE FORMATTER     *
*             TO CORRECTLY IDENTIFY A DUMP OPERATION.                 *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'DUMP'           APCPAPFN
         DC    A(0)                APCPAPOP - OP CODE OF ZEROS
         DC    A(0)                APCPRTN  - CALL NOT PROCESSED
         DC    F'0'                APCPUSRX
         DC    H'02'               APCPMAXP
         DC    H'02'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    XL48'00'            APAL
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR BRK                          *
*                    'BREAK 3838 CIT CHAIN'                           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'BRK '           APCPAPFN
         DC    A(APOPBRK)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'02'               APCPMAXP
         DC    H'02'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'02'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'09'               APALLTYP
         DC    XL1'00'             APALLI1F
         DC    XL1'00'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VES                          *
*                'VECTOR ELEMENT-BY-ELEMENT SUM'                      *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VES '           APCPAPFN
         DC    A(APOPVES)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'20'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'41'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8D'             APALUFSW
         DC    XL1'40'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'40'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SSUM                         *
*                          'SCALAR SUM'                               *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SSUM'           APCPAPFN
         DC    A(APOPSSUM)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'18'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8C'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VEM                          *
*              'VECTOR ELEMENT-BY-ELEMENT MULTIPLY'                   *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VEM '           APCPAPFN
         DC    A(APOPVEM)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'20'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'41'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8D'             APALUFSW
         DC    XL1'40'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'40'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SMY                          *
*                       'SCALAR MULTIPLY'                             *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SMY '           APCPAPFN
         DC    A(APOPSMY)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'18'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8C'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SSQ                          *
*                        'SUM OF SQUARES'                             *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SSQ '           APCPAPFN
         DC    A(APOPSSQ)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'70'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'84'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'01'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'87'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'41'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SSA                          *
*                    'SIGNED SQUARED ARRAY'                           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SSA '           APCPAPFN
         DC    A(APOPSSA)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'15'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'41'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SVE                          *
*                   'SUM OF VECTOR ELEMENTS'                          *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SVE '           APCPAPFN
         DC    A(APOPSVE)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'70'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'84'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'01'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'8F'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR DIV                          *
*                           'DIVIDE'                                  *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'DIV '           APCPAPFN
         DC    A(APOPDIV)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'20'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'34'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8D'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SDIV                         *
*                       'SCALAR DIVIDE'                               *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SDIV'           APCPAPFN
         DC    A(APOPSDIV)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'18'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'34'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8C'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR VIP                          *
*                    'VECTOR INNER PRODUCT'                           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'VIP '           APCPAPFN
         DC    A(APOPVIP)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'18'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'70'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'84'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'01'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALYCNT
         DC    XL1'8F'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8D'             APALUFSW
         DC    XL1'40'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'40'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR MAX                          *
*                      'SCAN FOR MAXIMUM'                             *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'MAX '           APCPAPFN
         DC    A(APOPMAX)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'13'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'00'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'84'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'01'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0002'             APALYCNT
         DC    XL1'8F'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR SQRT                         *
*                          'SQUARE ROOT'                              *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'SQRT'           APCPAPFN
         DC    A(APOPSQRT)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'15'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'14'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR LIM                          *
*                             'LIMIT'                                 *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'LIM '           APCPAPFN
         DC    A(APOPLIM)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'18'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'00'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'84'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0002'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR WLEV                         *
*                    'WIENER-LEVINSON FILTER'                         *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'WLEV'           APCPAPFN
         DC    A(APOPWLEV)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'18'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'34'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'85'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'85'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XMVX                         *
*                   'MOVE BULK STORE TO INDEX'                        *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XMVX'           APCPAPFN
         DC    A(APOPXMVX)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'01'               APALLTYP
         DC    XL1'00'             APALLI1F
         DC    XL1'00'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XMVS                         *
*                   'MOVE INDEX TO BULK STORE'                        *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XMVS'           APCPAPFN
         DC    A(APOPXMVS)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'01'               APALLTYP
         DC    XL1'00'             APALLI1F
         DC    XL1'00'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XMV                          *
*                     'MOVE INDEX TO INDEX'                           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XMV '           APCPAPFN
         DC    A(APOPXMV)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'04'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'02'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XMVI                         *
*                   'MOVE IMMEDIATE TO INDEX'                         *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XMVI'           APCPAPFN
         DC    A(APOPXMVI)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'03'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XAD                          *
*                     'ADD INDEX TO INDEX'                            *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XAD '           APCPAPFN
         DC    A(APOPXAD)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'04'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'02'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XADI                         *
*                    'ADD IMMEDIATE TO INDEX'                         *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XADI'           APCPAPFN
         DC    A(APOPXADI)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'03'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XSB                          *
*                  'SUBTRACT INDEX FROM INDEX'                        *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XSB '           APCPAPFN
         DC    A(APOPXSB)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'04'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'02'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XSBI                         *
*                'SUBTRACT IMMEDIATE FROM INDEX'                      *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XSBI'           APCPAPFN
         DC    A(APOPXSBI)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'03'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XML                          *
*                   'MULTIPLY INDEX BY INDEX'                         *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XML '           APCPAPFN
         DC    A(APOPXML)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'04'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'02'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XMLI                         *
*                 'MULTIPLY INDEX BY IMMEDIATE'                       *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XMLI'           APCPAPFN
         DC    A(APOPXMLI)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'03'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XDV                          *
*                    'DIVIDE INDEX BY INDEX'                          *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XDV '           APCPAPFN
         DC    A(APOPXDV)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'04'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'02'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XDVI                         *
*                  'DIVIDE INDEX BY IMMEDIATE'                        *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XDVI'           APCPAPFN
         DC    A(APOPXDVI)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'03'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XDIR                         *
*                  'DIVIDE IMMEDIATE BY INDEX'                        *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XDIR'           APCPAPFN
         DC    A(APOPXDIR)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'06'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'03'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XID                          *
*                      'LABEL DEFINITION'                             *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XID '           APCPAPFN
         DC    A(APOPXID)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
** NOTE: ALTHOUGH A VPSS FUNCTION ONLY, APAL CONSTANTS ARE DEFINED   **
**       TO PROVIDE AN INDEX OPERATION TYPE FOR MODULE IGTMFA.       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'00'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'07'               APALLTYP
         DC    XL1'00'             APALLI1F
         DC    XL1'00'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XC                           *
*                    'COMPARE INDEX:INDEX'                            *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XC  '           APCPAPFN
         DC    A(APOPXC)           APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'04'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XCI                          *
*                   'COMPARE INDEX:IMMEDIATE'                         *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XCI '           APCPAPFN
         DC    A(APOPXCI)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'04'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XDC                          *
*               'DECREMENT AND COMPARE INDEX:INDEX'                   *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XDC '           APCPAPFN
         DC    A(APOPXDC)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'04'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XDCI                         *
*            'DECREMENT AND COMPARE INDEX:IMMEDIATE'                  *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XDCI'           APCPAPFN
         DC    A(APOPXDCI)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'04'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XCF                          *
*             'FLOATING-POINT COMPARE INDEX:INDEX'                    *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XCF '           APCPAPFN
         DC    A(APOPXCF)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'04'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'80'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XCFI                         *
*            'FLOATING-POINT COMPARE INDEX:IMMEDIATE'                 *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XCFI'           APCPAPFN
         DC    A(APOPXCFI)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'06'               APCPMAXP
         DC    H'06'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'04'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'40'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XBCT                         *
*                    'BRANCH AND COUNT LOOP'                          *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XBCT'           APCPAPFN
         DC    A(APOPXBCT)         APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'04'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'06'               APALLTYP
         DC    XL1'80'             APALLI1F
         DC    XL1'00'             APALLI2F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR XGO                          *
*                    'UNCONDITIONAL BRANCH'                           *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'XGO '           APCPAPFN
         DC    A(APOPXGO)          APCPAPOP
         DC    A(@MFA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'08'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    H'05'               APALLTYP
         DC    XL1'00'             APALLI1F
         DC    XL1'00'             LI0F
         DC    XL4'00'             RESERVED
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR NZCP                         *
*               'NTH ZERO CROSSING(FORWARD SCAN)'                     *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'NZCP'           APCPAPFN
         DC    A(APOPNZCP)         APCPAPOP
         DC    A(@MDC-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'12'               APCPMAXP
         DC    H'09'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'00'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'00'             APALYFSW
         DC    XL1'00'             APALYVS
         DC    XL1'00'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'00'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'86'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'84'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR NZCN                         *
*               'NTH ZERO CROSSING(REVERSE SCAN)'                     *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'NZCN'           APCPAPFN
         DC    A(APOPNZCN)         APCPAPOP
         DC    A(@MDC-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'12'               APCPMAXP
         DC    H'09'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'00'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'00'             APALYFSW
         DC    XL1'00'             APALYVS
         DC    XL1'00'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'00'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'86'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'80'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'84'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'01'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0001'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR EXP                          *
*                         'EXPONENTIAL'                               *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'EXP '           APCPAPFN
         DC    A(APOPEXP)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'15'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'85'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR TAN                          *
*                           'TANGENT'                                 *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'TAN '           APCPAPFN
         DC    A(APOPTAN)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'15'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'85'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR LOG                          *
*                          'LOGARITHM'                                *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'LOG '           APCPAPFN
         DC    A(APOPLOG)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'11'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'86'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'00'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'84'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'00'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR ATAN                         *
*                         'ARCTANGENT'                                *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'ATAN'           APCPAPFN
         DC    A(APOPATAN)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'15'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'85'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR REC                          *
*                     'RECURSIVE FILTER'                              *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'REC '           APCPAPFN
         DC    A(APOPREC)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'20'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'8D'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'8D'             APALUFSW
         DC    XL1'40'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'40'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR POLY                         *
*                     'POLYNOMIAL EXPANSION'                          *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'POLY'           APCPAPFN
         DC    A(APOPPOLY)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'20'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'85'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'86'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'80'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR ATN2                         *
*                  'ARCTANGENT (QUOTIENT FORM)'                       *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'ATN2'           APCPAPFN
         DC    A(APOPATN2)         APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'20'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVE
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'28'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'30'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'80'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'85'             APALXFSW
         DC    XL1'80'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'80'             APALXDL
         DC    XL1'80'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'85'             APALUFSW
         DC    XL1'80'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'40'             APALUDL
         DC    XL1'80'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
*                                                                     *
*                    APCP/APAL ENTRY FOR REV                          *
*                        'VECTOR REVERSE'                             *
*                                                                     *
***********************************************************************
**                       DEFINE APCP CONSTANTS                       **
         DC    CL4'REV '           APCPAPFN
         DC    A(APOPREV)          APCPAPOP
         DC    A(@MDA-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'15'               APCPMAXP
         DC    H'04'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'10000000'       APCPSBLD
         DC    BL1'10000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'01000000'       APCPSWX
         DC    BL1'10000000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
**                       DEFINE APAL CONSTANTS                       **
         DC    01XL48'00'          APAL
         ORG   *-48
         DC    XL4'00000000'       APALUSER
         DC    H'20'               APALCISZ
         DC    XL2'0000'           RESERVED
         DC    XL1'10'             APALSTAT
         DC    XL3'000000'         RESERVED
         DC    XL1'87'             APALYFSW
         DC    XL1'80'             APALYVS
         DC    XL1'80'             APALYCT
         DC    XL1'40'             APALYDL
         DC    XL1'80'             APALYFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALYCNT
         DC    XL1'85'             APALXFSW
         DC    XL1'40'             APALXVS
         DC    XL1'00'             APALXCT
         DC    XL1'40'             APALXDL
         DC    XL1'40'             APALXFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALXCNT
         DC    XL1'00'             APALUFSW
         DC    XL1'00'             APALUVS
         DC    XL1'00'             APALUCT
         DC    XL1'00'             APALUDL
         DC    XL1'00'             APALUFMT
         DC    XL5'0000000000'     RESERVED
         DC    H'0000'             APALUCNT
         ORG   ,
         EJECT
***********************************************************************
**                        START OF LAST VALID ENTRY                  **
***********************************************************************
CAPCPZ   EQU   *                   LAST ENTRY
***********************************************************************
*                                                                     *
*                 APCP ENTRY FOR FREE                                 *
*                      'FREE ACB'                                     *
*                                                                     *
***********************************************************************
         DC    CL4'FREE'           APCPAPFN
         DC    A(APOPFREE)         APCPAPOP
         DC    A(@MJG-CAPCPRTN)    APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'03'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10010000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'00100000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'00000000'       APCPSWX
         DC    BL1'10010000'       APCPSFN
         DC    BL1'00100000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         EJECT
***********************************************************************
*                                                                     *
*                         DUMMY APCP ENTRY                            *
*                                                                     *
***********************************************************************
         DC    CL4'UNKN'           APCPAPFN
         DC    A(0)                APCPAPOP
         DC    A(CAPCPRTN-CAPCPRTN) APCPRTN
         DC    F'0'                APCPUSRX
         DC    H'99'               APCPMAXP
         DC    H'03'               APCPMINP
         DC    XL4'00000000'       RESERVED
         DC    XL8'0000000000000000' RESERVED
         DC    00XL16'00'          APCPSWS
         DC    BL1'10000000'       APCPSACB
         DC    BL1'00000000'       APCPSBLD
         DC    BL1'00000000'       APCPSOP
         DC    BL1'00000000'       APCPSAR
         DC    BL1'00000000'       APCPSIO
         DC    BL4'00000000'       RESERVED
         DC    BL1'00000000'       APCPSWX
         DC    BL1'00100000'       APCPSFN
         DC    BL1'01000000'       APCPSRCL
         DC    BL1'01000000'       APCPSEML
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    BL1'00000000'       RESERVED
         DC    01XL48'00'          APAL
         SPACE 5
***********************************************************************
*                                                                     *
*                  END OF APCP/APAL DATA TABLE                        *
*                                                                     *
***********************************************************************
         EJECT
***********************************************************************
*                                                                     *
*                       IGTXAPOP DSECT                                *
*                                                                     *
*      INCLUDE THE IGTXAPOP DSECT TO RESOLVE REFERENCES TO IGTXAPOP   *
* IN THE APCP/APAL DATA TABLE. THIS CAUSES OPERATION IDS FROM         *
* IGTXAPOP TO BE ASSEMBLED INTO AN APCP ENTRY.                        *
*                                                                     *
***********************************************************************
         SPACE 5
         IGTXDSCT IGTXAPOP
         EJECT
***********************************************************************
*                                                                     *
*                     APOTNM - VPSS MODULE NAME TABLE                 *
*                                                                     *
*      THE MODULE NAME TABLE CONTAINS THE NAMES OF VPSS MODULES TO BE *
* LOADED AND THE APCT OFFSET WHERE THE MODULE'S ADDRESS IS TO BE      *
* SAVED. REFER TO THE DSECT APOTNM WITHIN IGTXAPOT TO INTERPRET THIS  *
* TABLE. ENTRIES TO THIS TABLE ARE GENERATED VIA THE INTERNAL MACRO   *
* IGT0MNT. REFER TO THE PROLOGUE TO IGT0MNT FOR AN EXPLANATION OF THE *
* USE OF IGT0MNT.                                                     *
*                                                                     *
***********************************************************************
         SPACE 5
IGT0VPSS CSECT ,                       RESUME CSECT PROCESSING
MNT    DS    0D                        APOTNM
         IGT0MNT UAA,Z
         IGT0MNT UAB,Z
         IGT0MNT UAC,Z
         IGT0MNT UAD,Z
         IGT0MNT UAE,Z
         IGT0MNT UBB,Z
         IGT0MNT D10,Z
         IGT0MNT D20,Z
         IGT0MNT D40,Z
         IGT0MNT D41,Z
         IGT0MNT D42,Z
         IGT0MNT D43,Z
         IGT0MNT D44,Z
         IGT0MNT D45,Z
         IGT0MNT D46,Z
         IGT0MNT D47,Z
         IGT0MNT D48,Z
         IGT0MNT D49,Z
         IGT0MNT D60,Z
         IGT0MNT D80,Z
         IGT0MNT D81,Z,SYST=VS1
         IGT0MNT D82,Z,SYST=VS1
         IGT0MNT D4E,Z
         IGT0MNT MBA,Z
         IGT0MNT MBD,Z
         IGT0MNT MBG,Z
         IGT0MNT MBR,Z
         IGT0MNT DX1,Z
         IGT0MNT DX2,Z
         IGT0MNT DX3,Z
         IGT0MNT DX4,Z
         IGT0MNT MBH,Z
         IGT0MNT MBK,Z
         IGT0MNT MBM,Z
         IGT0MNT MJB,Z
         IGT0MNT MJC,Z
         IGT0MNT MJH,Z
         IGT0MNT MMA,Z
         IGT0MNT DID,Z
         IGT0MNT DRI,Z
         IGT0MNT MJI,Z
         IGT0MNT DWT,Z
         IGT0MNT DMM,Z
         IGT0MNT DAL,Z
         IGT0MNT DBK,Z
         IGT0MNT DIP,Z
         IGT0MNT DDA,Z
         IGT0MNT DUL,Z
         IGT0MNT DEQ,Z
         IGT0MNT DDQ,Z
         IGT0MNT DUA,Z
         IGT0MNT UAF,Z
         IGT0MNT UAG,Z
         IGT0MNT DIO,Z
         IGT0MNT UHA,Z
         IGT0MNT UHB,Z
         IGT0MNT JFA,Z
         IGT0MNT JAA,Z
         IGT0MNT JAD,Z
         IGT0MNT JAG,Z
         IGT0MNT JAJ,Z
         IGT0MNT JAM,Z
         IGT0MNT JAP,Z
         IGT0MNT JAX,Z,SYST=VS1
         IGT0MNT JAY,Z
         IGT0MNT UFA,Z
         IGT0MNT UJA,Z
         IGT0MNT DAP,Z
         IGT0MNT KAA,Z,SYST=MVS
         IGT0MNT MIB,Z
         IGT0MNT MIC,Z
         IGT0MNT MIF,Z
         IGT0MNT JH0,Z
         IGT0MNT JI0,Z
         IGT0MNT JI3,Z
         IGT0MNT JJ0,Z
         IGT0MNT JBA,Z
         IGT0MNT JBD,Z
         IGT0MNT JBG,Z
         IGT0MNT JBJ,Z
         IGT0MNT JBM,Z
         IGT0MNT KBA,Z,SYST=MVS
         IGT0MNT KCA,Z,SYST=MVS
         IGT0MNT KDA,Z,SYST=MVS
         IGT0MNT KBB,Z,SYST=MVS
         IGT0MNT KBC,Z,SYST=MVS
         IGT0MNT JDA,Z,SYST=MVS
         IGT0MNT JDD,Z
         IGT0MNT JDG,Z,SYST=MVS
         IGT0MNT JDJ,Z
         IGT0MNT JDM,Z
         IGT0MNT JH6,Z
         IGT0MNT JHE,Z
         IGT0MNT JHF,Z
         IGT0MNT JHT,Z
         IGT0MNT JHR,Z
         IGT0MNT JHW,Z
         IGT0MNT JHO,Z
         IGT0MNT JHC,Z
         IGT0MNT KBD,Z,SYST=MVS
         IGT0MNT KBE,Z,SYST=MVS
         IGT0MNT KBF,Z,SYST=MVS
         IGT0MNT UKA,Z
         IGT0MNT LCA,Z,SYST=MVS
         IGT0MNT LCB,Z,SYST=MVS
         IGT0MNT DWI,Z
         IGT0MNT DQR,Z,SYST=VS1
         IGT0MNT JHQ,Z,SYST=VS1
         IGT0MNT KGA,Z,SYST=VS1,FIX=YES IGTKGA MUST BE FIXED IN VS1
         IGT0MNT KGB,Z,SYST=VS1,FIX=YES IGTKGB MUST BE FIXED IN VS1
         IGT0MNT KGC,Z,SYST=VS1,FIX=YES IGTKGC MUST BE FIXED IN VS1
         IGT0MNT KGD,Z,SYST=VS1,FIX=YES IGTKGD MUST BE FIXED IN VS1
*                                  END OF THE VPSS ROUTINES AND START
*                                  OF THE USER EXIT ROUTINES.
         IGT0MNT MJB,@
         IGT0MNT MJC,@
         IGT0MNT MJD,@
         IGT0MNT MJE,@
         IGT0MNT MJF,@
         IGT0MNT MJG,@
         IGT0MNT MLA,@
         IGT0MNT MFA,@
         IGT0MNT MDA,@
         IGT0MNT MDB,@
         IGT0MNT MDC,@
         IGT0MNT MGA,@
         IGT0MNT MHA,@
         IGT0MNT MCA,@
*
** THE FOLLOWING ENTRIES ARE DUMMY ENTRIES FOR THE INSTALLATION EXITS.
** THEY ARE ADDED SO THAT INSTALLATION EXITS MAY BE ZAPPED IN.
*
         IGT0MNT MAB,1,NAME=IGTUBR14    APUT INITIALIZED
         IGT0MNT MBD,1,NAME=IGTUBR14    CALL ROUTER(BEFORE PROCESSING)
         IGT0MNT MBD,2,NAME=IGTUBR14    CALL ROUTER(AFTER PROCESSING)
         IGT0MNT CMD,1,NAME=IGTUBR14    COMMAND VERIFICATION
         IGT0MNT MBM,1,NAME=IGTUBR14    CALL EXLST ENTRY FOR ERROR EXIT
         IGT0MNT JAA,1,NAME=IGTUBR14    SUBSYSTEM OPEN EXIT(OPEN DONE)
         IGT0MNT JHC,1,NAME=IGTUBR14    OK SHARED PORT ALLOCATION
         IGT0MNT JHC,2,NAME=IGTUBR14    OK EXCLUSIVE PORT ALLOCATION
         IGT0MNT JHT,1,NAME=IGTUBR14    OK ALLOCATION OPERATOR DIALOG
         IGT0MNT JHL,1,NAME=IGTUBR14    AFTER OPTIONAL CRITERION 1
         IGT0MNT JHL,2,NAME=IGTUBR14    AFTER OPTIONAL CRITERION 2
         IGT0MNT JBJ,1,NAME=IGTUBR14    VPSS SUBSYSTEM CLOSE UPDATE SMF
MNT#   EQU   (*-MNT)/12                DETERMINE NAME TABLE LENGTH
       DC    120X'00'                  RESERVE ZAP SPACE   - CHANGE
*                                      MNT#!
       SPACE 5
***********************************************************************
*                                                                     *
*                          END OF APOTNM                              *
*                                                                     *
***********************************************************************
         EJECT
***********************************************************************
*                                                                     *
*                       APOTVT DSECT - SSVT                           *
*                                                                     *
*      LIST THE SSVT FORMAT - ALSO CORRESPONDS TO DSECT APOTVT        *
*  WITHIN IGTXAPOT. REFER TO THIS DSECT WHEN INTERPRETING VALUES IN   *
*  THE APOTVT TABLE.                                                  *
*                                                                     *
***********************************************************************
         SPACE 5
         IEFJSSVT ,                SSVT FORMAT
         EJECT
***********************************************************************
*                                                                     *
*                   APOTVT - VPSS SUBSYSTEM VECTOR TABLE              *
*                                                                     *
*      THE FOLLOWING TABLE GIVES VALUES FOR THE VPSS SUBSYSTEM VECTOR *
* TABLE - MAPPED BY DSECTS APOTVT WITHIN IGTXAPOT AND THE PRECEDING   *
* SSVT DSECT. REFER TO THE DSECTS WHEN INTERPRETING THE APOTVT.       *
*      ENTRIES IN THIS TABLE ARE GENERATED VIA THE IGT0VPSS INTERNAL  *
* MACRO IGT0SSVT. USE THIS MACRO TO ADD OR CHANGE AN APOTVT ENTRY.    *
* EXPLANATION OF THE USE OF IGT0SSVT IS CONTAINED IN THE PROLOGUE TO  *
* THE IGT0VPSS MACRO.                                                 *
*                                                                     *
***********************************************************************
       SPACE 5
IGT0VPSS CSECT                         RESUME CSECT PROCESSING
SSVTFT DS    0D                        APOTVT
       IGT0SSVT    4,M,JDJ             END OF TASK - IGTJDJ
       IGT0SSVT    7,M,JJ0             UNALLOCATION - IGTJJ0
       IGT0SSVT    8,M,JDA,SYSTM=MVS   END OF ADDR SPACE - IGTJDA
       IGT0SSVT    10,M,JFA,SYSTM=MVS  SVC CMD PROCESSOR - IGTJFA
       IGT0SSVT    16,M,JAA            OPEN - IGTJAA
       IGT0SSVT    17,M,JBA            CLOSE - IGTJBA
       IGT0SSVT    38,S,JI0            CONVERTER - IGTJI0
         IGT0SSVT    39,S,JH0,SYSTM=MVS  ALLOCATION - IGTJH0
         IGT0SSVT    6,S,JH0,SYSTM=VS1   ALLOCATION - IGTJH0
SSVT#  EQU   (*-SSVTFT)/4
       DC    10XL4'00'                 RESERVE ZAP SPACE - CHANGE SSFT#
       SPACE 5
***********************************************************************
*                                                                     *
*                          END OF APOTVT                              *
*                                                                     *
***********************************************************************
       EJECT
***********************************************************************
*                                                                     *
*                   APOTAD - VPSS APCT RELOCATION TABLE               *
*                                                                     *
*      THIS TABLE COMPRISES THE APCT RELOCATION TABLE USED TO MODIFY  *
* THE APCT INTERNAL POINTERS. REFER TO THE DSECT APOTAD WITHIN        *
* IGTXAPOT WHEN INTERPRETING THIS TABLE. THIS TABLE IS GENERTATED     *
* VIA THE IGT0VPSS INTERNAL MACRO IGT0CTRT. USE OF THIS MACRO IS      *
* EXPLAINED IN THE PROLOGUE TO THE IGT0CTRT MACRO.                    *
*                                                                     *
***********************************************************************
       SPACE 5
APCTRT  DS    0D                       APOTAD
        IGT0CTRT U@@@,100              GENERATE THE RELOCATION TABLE
        DC    Y(APCTABR-APCT)          POINTER TO BR14
        DC    Y(APCTCRTN-APCT)         CALL INTERFACE ROUTINES
APCTRT# EQU   (*-APCTRT)/2             DETERMINE NUMBER OF APCTRT
*                                      ENTRIES
        DC    10A(0)                   ZAP SPACE
        SPACE 5
***********************************************************************
*                                                                     *
*                            END OF APOTAD                            *
*                                                                     *
***********************************************************************
        EJECT
IGT0VPSS CSECT
        IGTXEPLG LIST=YES              GET EPILOGUE
        EJECT
***********************************************************************
*                                                                     *
*                         END OF IGT0VPSS                             *
*                                                                     *
***********************************************************************
        END   IGT0VPSS
