         TITLE 'IRARMWLM -- SYSTEM RESOURCES MANAGER WORKLOAD MANAGER  *
                        '
IRARMWLM CSECT ,                                                01S0001
@PROLOG  BALR  @12,0                                            01S0001
@PSTART  DS    0H                                               01S0001
         USING @PSTART,@12                                      01S0001
*;                                                              01S0028
*/* SERVICE CALCULATOR ROUTINE                                       */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWM1                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER SERVICE CALCULATOR        */
*/*                       ROUTINE                                    */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWM1 ROUTINE CALCULATES THE AMOUNT OF      */
*/*          SERVICE PROVIDED TO AN USER SINCE THE BEGINNING OF      */
*/*          THE CURRENT WORKLOAD MANAGER MEASUREMENT INTERVAL       */
*/*          FOR THAT USER. SERVICE IS CALCULATED USING THE          */
*/*          FOLLOWING EQUATION:                                     */
*/*                                                                  */
*/*          SERVICE = ((MP)/K)+((CT)/K)+EI  WHERE :                 */
*/*                                                                  */
*/*          T = THE JOB STEP TIME ELAPSED IN THE CURRENT INTERVAL.  */
*/*          K = THE TIME REQUIRED TO EXECUTE 10,000 INSTRUCTIONS.   */
*/*              ( DEPENDENT ON THE CPU MODEL)                       */
*/*          M = THE MSO SERVICE COEFFICIENT SCALED BY 1/50.         */
*/*          P = THE NUMBER OF PAGE-SECONDS USED BY THE USER         */
*/*          C = THE CPU SERVICE COEFFICIENT.                        */
*/*          E = THE EXCP COUNT FOR THIS INTERVAL.                   */
*/*          I = THE I/O SERVICE COEFFICIENT.                        */
*/*                                                                  */
*/*          THIS ROUTINE CALCULATES EACH OF THE THREE SERVICE       */
*/*          FACTORS AND THE TOTAL SERVICE FOR THE USER FOR THE      */
*/*          INTERVAL.                                               */
*/*                                                                  */
*/*    CALLED BY - IRARME38 - USER SERVICE REQST SYSEVENT RTNE       */
*/*                IRARMWM3 - USER WORKLOAD EVALUATION SBRTINE       */
*/*                IRARMWMI - USER MEASUREMENT INTERVL SBRTINE       */
*/*                IRARMWMK - USER APPLICABL ALGORITHM SBRTNE        */
*/*                IRARMWM2 - ANALYSIS USER REORDERING ROUTINE       */
*/*                IRARMWAR - WORKLOAD ACTIVITY REPORT ROUTINE       */
*/*                                                                  */
*/*    NOTE  - THE IRARMWM1 ROUTINE DOES NOT REQUIRE SERIALIZATN     */
*/*            AGAINST OTHER SYSTEM RESOURCES MANAGER PROCESSES,     */
*/*            WHICH COULD BE EXAMINING THE SAME OUCB FOR WHICH      */
*/*            SERVICE IS BEING CALCULATED BY IRARMWM1.  SINCE       */
*/*            THE RESULT OF IRARMWM1 IS A MORE CORRECT VALUE OF     */
*/*            THE SERVICE ACCUMULATED BY A USER, IT IS ACCEPT-      */
*/*            ABLE FOR SUCH UPDATING TO BE PERFORMED ON THE FLY.    */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHOSE INTERVAL SERVICE AMOUNTS ARE TO BE CALCULATED.  */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - RMCTADJC CONTAINS THE TIME IN MICROSECONDS TO EXE-    */
*/*            CUTE 10,000 INSTRUCTIONS.  A HEXADECIMAL POINT IS     */
*/*            ASSUMED ONE PLACE TO THE LEFT IN ITS BINARY VALUE.    */
*/*          - WMSTCPU CONTAINS THE CPU SERVICE COEFFICIENT.         */
*/*          - WMSTIOC CONTAINS THE I/O SERVICE COEFFICIENT.         */
*/*          - WMSTMSO CONTAINS THE MSO SERVICE COEFFICIENT. THIS    */
*/*            VALUE IS SCALED BY 1/50.                              */
*/*            (NOTE: THE SERVICE COEFFICIENTS HAVE HEXADECIMAL      */
*/*            POINTS THREE PLACES TO THE LEFT                       */
*/*            OF THE BINARY VALUES IN THESE FIELDS.)                */
*/*          - ASCBEJST CONTAINS THE CUMULATIVE USER JOB STEP TIME.  */
*/*            A HEXADECIMAL POINT IS ASSUMED 3 PLACES TO THE LEFT.  */
*/*          - OUCBIOSM CONTAINS THE CUMULATIVE USER I/O REQST CNT.
*                                                          @ZA10808*/
*/*          - OUCBPSS CONTAINS THE CURRENT USER PAGE-SECONDS. A     */
*/*            HEXADECIMAL POINT IS ASSUMED 0 PLACES TO THE LEFT.    */
*/*          - OUXBCPS CONTAINS THE JOB STEP TIME VALUE FROM THE     */
*/*            BEGINNING OF THE INTERVAL. A HEXADECIMAL POINT IS     */
*/*            ASSUMED 3 PLACES TO THE LEFT.                         */
*/*          - OUXBIOS CONTAINS THE I/O REQUEST COUNT FROM THE       */
*/*            BEGINNING OF THE INTERVAL.                            */
*/*          - OUXBMSS CONTAINS THE PAGE-SECONDS COUNT FROM THE      */
*/*            BEFINNING OF THE INTERVAL.                            */
*/*                                                                  */
*/*    OUTPT - OUCBWMS CONTAINS THE ACCUMULATED INTERVAL SERVICE.    */
*/*          - OUCBMSO CONTAINS THE REAL PAGE FRAMES OCCUPIED TIMES  */
*/*            THE CPU SERVICE UNITS FOR THE INTERVAL.               */
*/*          - OUCBCPU CONTAINS THE CPU SERVICE UNITS FOR THE        */
*/*            INTERVAL.                                             */
*/*          - OUCBIOC CONTAINS THE I/O SERVICE UNITS FOR THE        */
*/*            INTERVAL.                                             */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WM1) */                                    02S0029
*  DCL IRARMWM1 ENTRY INT;                                      02S0029
*  GEN CODE REFS(LCCACPUS,PSALCCAV,WSACOPTM)                    02S0030
*        DEFS(IRARMWM1) NOEXIT SEQFLOW;                         02S0030
*                                  /*
         ENTRY IRARMWM1
IRARMWM1 DS    0H
         L     @15,PSALCCAV            LCCA ADDRESS
         L     @15,LCCACPUS-LCCA(,@15) CPU SAVE AREA VECTOR TABLE
         L     @15,WSACOPTM-WSAC(,@15) SRM SAVE AREA START ADDRESS
         STM   @00,@14,128+000(@15)    SAVE INVOKING RTNE REGS
         LR    @13,@15                 ROUTINE SAVE AREA ADDRESS
         BALR  @15,0                   SETUP ENTRY ADDRESSABILITY
         LA    @12,*-@PSTART           DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*    WMSTPTR = RMCTWMST;            /* PICK WLM SPECIFICATION TABLE  */
         L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0031
*    ASCBPTR = OUCBASCB;            /* PICK ADDRS SPAC CONTROL BLOCK */
         L     ASCBPTR,OUCBASCB(,OUCBPTR)                       02S0032
*    OUXBPTR = ASCBOUXB;            /* PICK USER CTRL BLCK EXTENSION */
         L     OUXBPTR,ASCBOUXB(,ASCBPTR)                       02S0033
*    GEN CODE REFS(ASCBEJST,OUCBIOSM,RMCTADJC,OUXBMSS,          02S0034
*          OUXBCPS,OUXBIOS,WMSTMSO,WMSTCPU,WMSTIOC,OUCBWMS,     02S0034
*  OUCBMSO,OUCBCPU,OUCBIOC,OUXBIOSM,ASCBIOSM);     /*@ZA10808*/ 02S0034
*                                  /*
         LM    WKRE,WKRO,ASCBEJST(ASCBPTR) CURR JOB STEP TIME, IN
*                                      TIMER  UNITS . PICTURE IS
*                                      XXXXXXXXXXXXX.XXX
         SL    WKRO,OUXBCPS+4(,OUXBPTR)  SUB THE LOW ORDER WORD OF
*                                      THE PREVIOUS JOB STEP TIME.
         BC    @03,RMWM1B1             BRANCH ON CARRY
         BCTR  WKRE,@00                PROPOGATE A BORROW.
RMWM1B1  SL    WKRE,OUXBCPS(,OUXBPTR)  SUBTRACT HI ORDER WORD
*                                      WRKE/WRKO NOW CONTAIN T.
         SLDL  WKRE,@01                INCLUDE HI ORDER BIT OF
*                                      DIVIDEND IN COMPARE  @ZA16890
         CL    WKRE,RMCTADJC(,RMCTPTR) WOULD A DIVIDE EXCEPTION OCCUR?
         SRDL  WKRE,@01                RESTORE DIVIDEND    @ZA16890
         BL    RMWM1B2                 BRANCH IF NOT
         SLR   WKRE,WKRE               PREVENT DIVIDE EXCEPTION
RMWM1B2  D     WKRE,RMCTADJC(,RMCTPTR) DIVIDE ELAPSED TIME THIS
*                                      INTERVAL BY TIME REQUIRED
*                                      TO EXECUTE 10K INSTRUCTIONS
*                                      RMCTADJC HAS ONE HEX PLACE.
*        THIS RESULT IN WRKO IS T/K TO 2 HEX PLACES.
*
         M     WKRE,WMSTCPU(,WMSTPTR)  THIS RESULT, IN WKRE AND WKRO,
*                                      IS (CT)/K TO 5 HEX PLACES.
         SRDA  WKRE,20                 ELIMINATE MISLEADING HEX PLACES
         ST    WKRO,OUCBCPU(,OUCBPTR)  STORE (CT)/K
*
         LM    R14,R15,OUCBPSS(OUCBPTR) MSO PAGE-SECONDS
*                                      0 HEX PLACES
         SL    R15,OUXBMSS+4(,OUXBPTR) SUBTRACT THE LOW ORDER WORD OF
*                                      THE PREVIOUS PAGE-SECOND COUNT
         BC    @03,RMWM1B3             BRANCH ON CARRY
         BCTR  R14,0                   PROPOGATE A BORROW
RMWM1B3  SL    R14,OUXBMSS(,OUXBPTR)   SUBTRACT HI ORDER WORD
*                                      R14/R15 NOW CONTAIN P.
         SLDL  R14,10                  SCALE FOR DIVIDE
*        THIS RESULT IS P TO 0 HEX PLACES IN MICRO-SECONDS
*
         D     R14,RMCTADJC(,RMCTPTR)  DIVIDE BY TIME REQUIRED TO
*                                      EXECUTE 10,000 INSTRUCTIONS.
*                                      RMCTADJC HAS ONE HEX PLACE
*        THIS RESULT IS P/K TO -1 HEX PLACES.
*
         M     R14,WMSTMSO(,WMSTPTR)   MULTIPLY BY MSO SERVICE COEF.
*        THIS RESULT IS MP/K TO 2 HEX PLACES.
*
         SRDA  R14,8                   ELIMINATED MISLEADING HEX
*                                      WORD ARITHMETIC.
*
         ST    R15,OUCBMSO(,OUCBPTR)   STORE (MP)/K
*
         ALR   WKRO,R15                (MP/K)+(CT/K) IS THIS RESULT.
*
         LH    R15,ASCBIOSM(,ASCBPTR) CURR JES EXCP COUNT @ZA10808
         N     R15,HWORDMSK            CLEAR EXTRANEOUS HWD @ZA08717
         LH    R14,OUXBIOS(,OUXBPTR) PREV JES I/O REQ COUNT @ZA10808
         N     R14,HWORDMSK            CLEAR EXTRANEOUS HWD @ZA08717
         SLR   R15,R14         JES SERV I/O REQUEST  COUNT @ZA10808
         BNM   RMWM1B4                 NO WRAPPING          @ZA10808
         SR    R15,R15                 CLEAR IF WRAP        @ZA10808
RMWM1B4  AL    R15,OUCBIOSM(,OUCBPTR) ADD SMF EXCP COUNT   @ZA10808
         SL    R15,OUXBIOSM(,OUXBPTR)  SUB. PREV SMF COUNT  @ZA10808
* R15 CONTAINS TOTAL SERVICE I/O REQUEST COUNT              @ZA10808
         M     R14,WMSTIOC(,WMSTPTR)   RESULT, IN R14 AND R15, IS EI.
         SRDA  R14,12                  ELIMINATE EXTRANEOUS HEX PLACES
*
         ST    R15,OUCBIOC(,OUCBPTR)   STORE (EI)
*
         ALR   WKRO,R15                FINAL RESULT, IN WKRO, IS
*                                      (MP/K)+(CT/K)+(EI)
*
         ST    WKRO,OUCBWMS(,OUCBPTR)  SAVE CALCULATED SERVICE
*  /* ? IRACTLEP ENN(WM1) */                                    02S0035
*  GEN CODE EXIT NOSEQFLOW;                                     02S0035
*                                  /*
         LM    @00,@14,128+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0036
*/* SWAPPABLE USER EVALUATION ROUTINE                                */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWM2                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - SWAPPABLE USER EVALUATION ROUTINE          */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWM2 ROUTINE SCANS THE IN-CORE QUEUE AND   */
*/*          THE OUT-OF-CORE-BUT-READY QUEUE, AND EVALUATES EACH     */
*/*          SWAPPABLE USER ASSIGNING  EACH HIS CURRENT WORKLOAD LVL */
*/*                                                                  */
*/*    CALLED BY - IRARMCEL - TIMED ALOGORITHM SCHEDULING ROUTINE    */
*/*                                                                  */
*/*    NOTE  - THE IRARMWM2 ROUTINE IS RUN UNDER THE CONTROL OF      */
*/*            THE SYSTEM RESOURCES MANAGER ANALYSIS LOCK.           */
*/*                                                                  */
*/*    INPUT - THE OUCB QUEUES LOCATED BY RMCTINQE AND RMCTOTQE      */
*/*            CONTAIN THE USERS TO WHOM SWAP ANALYSIS APPLIES.      */
*/*                                                                  */
*/*    OUTPT - THE OUCBS ON THE SWAP ANALYSIS QUEUES HAVE BEEN       */
*/*            ASSIGNED A WORKLOAD MANAGER RECOMMENDATION VALUE.     */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WM2) */                                    02S0036
*  DCL IRARMWM2 ENTRY INT;                                      02S0036
*  GEN CODE REFS(RRPAEPA)                                       02S0037
*        DEFS(IRARMWM2) NOEXIT SEQFLOW;                         02S0037
*                                  /*
         ENTRY IRARMWM2
IRARMWM2 DS    0H
         STM   @00,@14,000+000(@13)    SAVE INVOKING RTNE REGS
         ST    @01,RRPAEPA(,RRPAPTR)   INDICATE NEW CURRENT RMEP
         LA    @12,IRARMWM2-@PSTART    DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*    STPR = SVEREG;                 /* SAVE INPUT SAVEAREA ADDRESS   */
         LR    STPR,SVEREG                                      02S0038
*    /* THE FOLLOWING CODE STEPS THROUGH THE IN-QUEUE AND THE OUT-   */
*    /* QUEUE.  THE WAIT-QUE IS NOT SCANNED                          */
*    /* BECAUSE ANALYSIS IS NOT APPLICABLE TO USERS ON THAT QUEUE.   */
*    DO PMRE = ADDR(RMCTOTQE) TO ADDR(RMCTINQE) BY LENGTH(RMCTOTQE);
         LA    PMRE,RMCTOTQE(,RMCTPTR)                          02S0039
         B     @DE00039                                         02S0039
@DL00039 DS    0H                                               02S0040
*    OUCBPTR = PMRE -> IRARMPTR -> OUCBFWD; /* PICK 1ST USER ON QUE */
         L     @09,IRARMPTR(,PMRE)                              02S0040
         L     OUCBPTR,OUCBFWD(,@09)                            02S0040
*    WKRE = OUCBFWD;            /* SAVE PTR FOR QUEUE ORDER  @ZA11855*/
         L     WKRE,OUCBFWD(,OUCBPTR)                           02S0041
*      /* SCAN CONTENTS OF THE INDIVIDUAL SWAP ANALYSIS QUEUE.  THE  */
*      /* USERS CAN BE IN VARIOUS STATES, SO END IS GIVN BY BLK NAME */
*      DO WHILE (OUCBNAME = 'OUCB');                            02S0042
         B     @DE00042                                         02S0042
@DL00042 DS    0H                                               02S0043
*      IF OUCBGOO = '0'B & OUCBGOI = '0'B & /* USER NOT BEING SWAPPD */
*            OUCBGOB = '0'B & OUCBINV = '0'B & /* USER IS QUEUED OK  */
*            OUCBNSW = '0'B & OUCBPVL = '0'B THEN /* USER SWAPPABLE  */
         TM    OUCBGOO(OUCBPTR),B'11100000'                     02S0043
         BNZ   @RF00043                                         02S0043
         TM    OUCBINV(OUCBPTR),B'10010100'                     02S0043
         BNZ   @RF00043                                         02S0043
*        /* BYPASS RE-ANALYSIS OF USER RECENTLY EVALUATED ON QUEUE   */
*           IF RRPATOD-OUCBTMA>RMPTWMET/* USER NOT RECENTLY ANALYZED */
*             &OUCBTMW ^= ZERO THEN /* TRANSACTION INTERVAL EXISTS   */
         L     @09,RRPATOD(,RRPAPTR)                            02S0044
         SL    @09,OUCBTMA(,OUCBPTR)                            02S0044
         CL    @09,RMPTWMET(,RMCTPTR)                           02S0044
         BNH   @RF00044                                         02S0044
         L     @09,OUCBTMW(,OUCBPTR)                            02S0044
         LTR   @09,@09                                          02S0044
         BZ    @RF00044                                         02S0044
*             DO;                                               02S0045
*             WMSTPTR = RMCTWMST; /* ADDRESS OF WMST        @ZM43084*/
         L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0046
*             WPGDPTR = WMSTPGDT+OUCBWMG; /* ADDR OF PERFORMANCE
*                                          GROUP DESCRIPTOR @ZM43084*/
         LH    WPGDPTR,OUCBWMG(,OUCBPTR)                        02S0047
         AL    WPGDPTR,WMSTPGDT(,WMSTPTR)                       02S0047
*             WPGPPTR = WPGDPTR+OUCBPGP;  /* ADDRESS OF CURRENT 02S0048
*                                         PERIOD DESCRIPTOR @ZM43084*/
         LR    WPGPPTR,WPGDPTR                                  02S0048
         SLR   @09,@09                                          02S0048
         IC    @09,OUCBPGP(,OUCBPTR)                            02S0048
         ALR   WPGPPTR,@09                                      02S0048
*             IF WPGPUNT = '1'B  |  /* DUR IS REAL TIME     @ZM43084*/
*                  OUCBOUT='0'B THEN /* OR USER IN (UNT=S)  @ZA11855*/
         TM    WPGPUNT(WPGPPTR),B'01000000'                     02S0049
         BO    @RT00049                                         02S0049
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0049
         BNZ   @RF00049                                         02S0049
@RT00049 DS    0H                                               02S0050
*          /* DETERMINE USER NORMALIZED WORKLOAD LEVEL               */
*          DO;                                                  02S0050
*          IF OUCBOUT = '0'B THEN   /* USER IS NOW RECEIVING SERVICE */
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0051
         BNZ   @RF00051                                         02S0051
*            /* ? IRACTLCL EPN(WM1) */                          02S0052
*            CALL RMSBWM1 -> IRARMRTN ;         /* INVOKE WM1 SUBRTN */
         L     @15,RMSBWM1(,RMCTPTR)                            02S0052
         BALR  @14,@15                                          02S0052
*          CALL IRARMWM5;           /* CHECK PERFORMANCE GRP PERIOD  */
@RF00051 LA    @15,IRARMWM5                                     02S0053
         BALR  @14,@15                                          02S0053
*          CALL IRARMWM7;           /* OBTAIN USER NORMALIZED LEVEL  */
         BAL   @14,IRARMWM7                                     02S0054
*          END;                                                 02S0055
*         END;                      /*                      @ZM43084*/
@RF00049 DS    0H                                               02S0057
*      OUCBPTR = WKRE;              /* OUCB AS SAVED ...WM5 MAY      */
@RF00044 DS    0H                                               02S0057
@RF00043 LR    OUCBPTR,WKRE                                     02S0057
*                                   /* CHANGE QUEUE POSITION.@ZA11855*/
*      WKRE=OUCBFWD;                /* SAVE NEXT FWD PTR.    @ZA11855*/
         L     WKRE,OUCBFWD(,OUCBPTR)                           02S0058
*      END;                                                     02S0059
@DE00042 CLC   OUCBNAME(4,OUCBPTR),@CC02243                     02S0059
         BE    @DL00042                                         02S0059
*    END;                                                       02S0060
         AL    PMRE,@CF00036                                    02S0060
@DE00039 LA    @09,RMCTINQE(,RMCTPTR)                           02S0060
         CLR   PMRE,@09                                         02S0060
         BNH   @DL00039                                         02S0060
*    SVEREG = STPR;                 /* RESTORE INPUT SAVEAR ADDRESS  */
         LR    SVEREG,STPR                                      02S0061
*  /* ? IRACTLEP ENN(WM2) */                                    02S0062
*  GEN CODE EXIT NOSEQFLOW;                                     02S0062
*                                  /*
         LM    @00,@14,000+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0063
*/* WORKLOAD MANAGER USER EVALUATION ROUTINE                         */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWM3                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - INDIVIDUAL USER EVALUATION ROUTINE         */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWM3 ROUTINE EVALUATES SWAPPBLE USERS      */
*/*               ON THE IN AND OUT QUEUE, ASSIGNING EACH A          */
*/*               CURRENT RECOMMENDATION VALUE.                      */
*/*                                                                  */
*/*    CALLED BY - IRARMEQ1 - ENQ STATUS CHANGE ROUTINE              */
*/*              - IRARMASM - AUX STORAGE MANAGEMENT ROUTINE         */
*/*              - IRARMMS2 - MAIN STORAGE OCCUPANCY ROUTINE         */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHOSE WORKLOAD LEVEL IS TO BE EVALUATED.              */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWM3 RTNE ADDRESS.            */
*/*                                                                  */
*/*    OUTPT - OUCBWMR CONTAINS THE USER'S RECOMMENDATION VALUE.     */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WM3) */                                    02S0063
*  DCL IRARMWM3 ENTRY INT;                                      02S0063
*  GEN CODE DEFS(IRARMWM3) NOEXIT SEQFLOW;                      02S0064
*                                  /*
         ENTRY IRARMWM3
IRARMWM3 DS    0H
         STM   @00,@14,128+000(@13)    SAVE INVOKING RTNE REGS
         LA    @12,IRARMWM3-@PSTART    DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*IF OUCBGOO = '0'B & OUCBGOI = '0'B & /* USER NOT TRANSITIONING      */
*   OUCBGOB = '0'B & OUCBINV = '0'B & /* USER IS QUEUED OK           */
*   OUCBNSW = '0'B &             /* USER IS SWAPPABLE                */
*   OUCBPVL = '0'B THEN          /* USER IS NOT PRIVILEDGED          */
         TM    OUCBGOO(OUCBPTR),B'11100000'                     02S0065
         BNZ   @RF00065                                         02S0065
         TM    OUCBINV(OUCBPTR),B'10010100'                     02S0065
         BNZ   @RF00065                                         02S0065
* DO ;                                                          02S0066
* IF OUCBTMW ^= ZERO THEN        /* TRANSACTION INTERVAL EXISTS      */
         L     @09,OUCBTMW(,OUCBPTR)                            02S0067
         LTR   @09,@09                                          02S0067
         BZ    @RF00067                                         02S0067
*   DO ;                                                        02S0068
*   IF OUCBOUT = '0'B THEN       /* USER RECIEVING SERVICE           */
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0069
         BNZ   @RF00069                                         02S0069
*       /* ? IRACTLCL EPN(WM1) */                               02S0070
*       CALL RMSBWM1 -> IRARMRTN ;              /* INVOKE WM1 SUBRTN */
         L     @15,RMSBWM1(,RMCTPTR)                            02S0070
         BALR  @14,@15                                          02S0070
*   CALL IRARMWM5 ;              /* VERIFY CURRENT PERF GROUP PERIOD */
@RF00069 LA    @15,IRARMWM5                                     02S0071
         BALR  @14,@15                                          02S0071
*   CALL IRARMWM7 ;              /* OBTAIN USER RECOMMENDATION VALUE */
         BAL   @14,IRARMWM7                                     02S0072
*   END ;                                                       02S0073
* END ;                                                         02S0074
@RF00067 DS    0H                                               02S0075
*  /* ? IRACTLEP ENN(WM3) */                                    02S0075
*  GEN CODE EXIT NOSEQFLOW;                                     02S0075
@RF00065 DS    0H                                               02S0075
*                                  /*
         LM    @00,@14,128+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0076
*/* USER NORMALIZED LEVEL CALCULATION SUBROUTINE                     */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWM4                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER NORMALIZED LEVEL          */
*/*                       CALCULATOR SUBROUTINE                      */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWM4 ROUTINE ACCEPTS A SERVICE RATE        */
*/*          AND A PERFORMANCE OBJECTIVE, AND CALCULATES THE         */
*/*          CORRESPONDING NORMALIZED WORKLOAD LEVEL.                */
*/*                                                                  */
*/*    CALLED BY - IRARMWM7 - USER WORKLOAD EVALUATION SBRTINE       */
*/*                IRARMHIT - USER READY SYSEVENT SWAP-IN RTNE       */
*/*                IRARMWAR - WORKLOAD ACTIVITY REPORT ROUTINE       */
*/*                IRARMWMO - WORKLOAD MANAGER TRANS STOP ROUTINE    */
*/*                                                                  */
*/*    INPUT - REG 0 CONTAINS THE SERVICE RATE FOR WHICH THE         */
*/*            EQUIVALENT WORKLOAD IS TO BE CALCULATED.              */
*/*          - REG 1 CONTAINS THE ADDRESS OF THE PERFORMANCE         */
*/*            OBJECTIVE TO BE USED TO TRANSFORM THE SERVICE         */
*/*            RATE INTO THE WORKLOAD LEVEL.                         */
*/*          - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE        */
*/*            USER WHOSE WORKLOAD IS BEING EVALUATED.               */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*                                                                  */
*/*    OUTPT - REG 0 CONTAINS THE WORKLOAD LEVEL CALCULATED          */
*/*            FOR THE GIVEN SERVICE RATE.  A HEXADECIMAL            */
*/*            POINT IS ASSUMED TWO HEX PLACES TO THE LEFT           */
*/*            OF THE REG 0 BINARY OUTPUT VALUE.  TO OBTAIN          */
*/*            THE EXTERNALLY SPECIFIED WORKLOAD LEVEL, THE          */
*/*            VALUE IN REG 0 SHOULD BE MULTIPLIED BY THE            */
*/*            CONTENTS OF -WMSTWLLO-.                               */
*/*          - REG 1 CONTAINS THE PERFORMANCE OBJECTIVE POINT        */
*/*            DESCRIPTION USED TO CALCULATE THE WRKLD LEVEL.        */
*/*            IF -WPOSSRV- AT THIS POINT IS NOT NEGATIVE, AND       */
*/*            IF -WPOPSRV- MINUS -WPOSSRV- IS MORE THAN 0, THEN     */
*/*            THE CALCULATED WORKLOAD LEVEL IS UNIQUE TO THE        */
*/*            SERVICE RATE SPECIFIED.  OTHERWISE, THE SERVICE       */
*/*            RATE FALLS ON A 'PLATEAU', ON WHICH MULTIPLE          */
*/*            WORKLOAD LEVELS HAVE BEEN SPEC'D FOR THIS RATE.       */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WM4)SAVE(2,14) */                          02S0076
*  DCL IRARMWM4 ENTRY INT;                                      02S0076
*  GEN CODE REFS(LCCACPUS,PSALCCAV,WSACOPTM)                    02S0077
*        DEFS(IRARMWM4) NOEXIT SEQFLOW;                         02S0077
*                                  /*
         ENTRY IRARMWM4
IRARMWM4 DS    0H
         L     @15,PSALCCAV            LCCA ADDRESS
         L     @15,LCCACPUS-LCCA(,@15) CPU SAVE AREA VECTOR TABLE
         L     @15,WSACOPTM-WSAC(,@15) SRM SAVE AREA START ADDRESS
         STM   @02,@14,128+008(@15)    SAVE INVOKING RTNE REGS
         LR    @13,@15                 ROUTINE SAVE AREA ADDRESS
         BALR  @15,0                   SETUP ENTRY ADDRESSABILITY
         LA    @12,*-@PSTART           DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*    RFY (ASCBPTR,OUXBPTR,WPGDPTR,STPR,PMRE,PMRO,SVRE,SVRO) UNRSTD;
*    R0 = R0 & HWORDMSK;            /* ASSURE SERVICE RATE BOUNDED   */
         N     R0,HWORDMSK                                      02S0079
*    /* THE FOLLOWING CODE SERIALLY SCANS THE PERFORMANCE OBJECTIVE  */
*    /* DESCRIPTION UNTIL IT LOCATES A POINT HAVING A SMALLER SERVIC */
*    /* RATE THAN THE INPUT.  TO INSURE THAT THIS SEARCH WILL ALWAYS */
*    /* BE SUCCESSFUL, EACH OBJECTIVE IS TERMINATED BY A DUMMY POINT */
*    /* CONTAINING A NEGATIVE SERVICE RATE.                          */
*    DO WHILE (R0 < WPOSSRV | R0 = WPOXSRV);        /*       @Z30WIWP*/
         B     @DE00080                                         02S0080
@DL00080 DS    0H                                               02S0081
*    WPODPTR = WPODPTR + LENGTH(WPOS); /* STEP TO NEXT WPOD ENTRY    */
         AL    WPODPTR,@CF00036                                 02S0081
*    END;                                                       02S0082
@DE00080 LH    @05,WPOSSRV(,WPODPTR)                            02S0082
         CR    R0,@05                                           02S0082
         BL    @DL00080                                         02S0082
         CH    R0,WPOXSRV(,WPODPTR)                             02S0082
         BE    @DL00080                                         02S0082
*    WKRE = (WPOPSRV - WPOSSRV);    /* SRV RATE CHNG FRM PREV POINT  */
         LH    WKRE,WPOPSRV(,WPODPTR)                           02S0083
         SLR   WKRE,@05                                         02S0083
*    IF R0 > ZERO & WKRE > ZERO THEN /* WKLD LVL NUMB UNIQUE @Z30WIWP*/
         SLR   @09,@09                                          02S0084
         CR    R0,@09                                           02S0084
         BNH   @RF00084                                         02S0084
         CLR   WKRE,@09                                         02S0084
         BNH   @RF00084                                         02S0084
*      /* CALC NORMALIZED LEVEL CORRESPONDING TO SERVICE OBTAINED    */
*      DO;                                                      02S0085
*      WKRO = WPOSWKL - WPOPWKL;    /* WORKLOAD CHANG FM PREV POINT  */
         LH    @09,WPOSWKL(,WPODPTR)                            02S0086
         N     @09,@CF02217                                     02S0086
         LR    WKRO,@09                                         02S0086
         LH    @08,WPOPWKL(,WPODPTR)                            02S0086
         N     @08,@CF02217                                     02S0086
         SLR   WKRO,@08                                         02S0086
*      WKRO = WKRO * (R0 - WPOSSRV) / WKRE; /* SRV CHANG FROM POINT  */
         LR    @07,@05                                          02S0087
         LCR   @07,@07                                          02S0087
         ALR   @07,R0                                           02S0087
         MR    @06,WKRO                                         02S0087
         DR    @06,WKRE                                         02S0087
         LR    WKRO,@07                                         02S0087
*      R0 = WPOSWKL - WKRO;         /* WORKLOAD LEVEL AT CALC POINT  */
         SLR   @09,WKRO                                         02S0088
         LR    R0,@09                                           02S0088
*      END;                                                     02S0089
*    ELSE                           /* MULTIPLE LEVELS AT THIS SRVC  */
*      /* PICK LO LEVEL FOR SWAPIN,HIGH FOR SWAPOUT          @Z30WIWP*/
*      DO;                                                 /*@Z30WIWP*/
         B     @RC00084                                         02S0090
@RF00084 DS    0H                                               02S0091
*      IF R0 = ZERO THEN            /* ZERO RATE SPECIFIED   @Z30WIWP*/
         LTR   R0,R0                                            02S0091
         BNZ   @RF00091                                         02S0091
*       R0 = WPOSWKL ; /* GIVE WLL LEVEL WHERE OBJ = 0 */       02S0092
         LH    R0,WPOSWKL(,WPODPTR)                             02S0092
         N     R0,@CF02217                                      02S0092
*     ELSE                                                      02S0093
*      IF RRPAWAR = '1'B |          /* EVALUATE WORKLOAD ACTIVITY    */
*            OUCBGOO = '1'B |       /* EVALUATE USR BEING QUIESCD    */
*            OUCBOUT = '1'B THEN    /* EVALUATE USER FOR SWAP-IN     */
         B     @RC00091                                         02S0093
@RF00091 TM    RRPAWAR(RRPAPTR),B'01000000'                     02S0093
         BO    @RT00093                                         02S0093
         TM    OUCBGOO(OUCBPTR),B'10000100'                     02S0093
         BZ    @RF00093                                         02S0093
@RT00093 DS    0H                                               02S0094
*        R0 = WPOPWKL;              /* WKLD FROM AHEAD POINT @Z30WIWP*/
         LH    R0,WPOPWKL(,WPODPTR)                             02S0094
         N     R0,@CF02217                                      02S0094
*      ELSE                         /* EVALUATE USER FOR SWAP-OUT    */
*        R0 = WPOSWKL;              /* WKLD FROM BEHIND PT   @Z30WIWP*/
         B     @RC00093                                         02S0095
@RF00093 LH    R0,WPOSWKL(,WPODPTR)                             02S0095
         N     R0,@CF02217                                      02S0095
*      END;                                                     02S0096
@RC00093 DS    0H                                               02S0096
@RC00091 DS    0H                                               02S0097
*    RFY (ASCBPTR,OUXBPTR,WPGDPTR,STPR,PMRE,PMRO,SVRE,SVRO) RSTD;
@RC00084 DS    0H                                               02S0098
*  /* ? IRACTLEP ENN(WM4) */                                    02S0098
*  GEN CODE EXIT NOSEQFLOW;                                     02S0098
*                                  /*
         LM    @02,@14,128+008(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0099
*/* PERFORMANCE GROUP PERIOD CHANGE SUBROUTINE                       */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWM5                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER UPDATE PERFORMANCE        */
*/*                       GROUP PERIOD SUBROUTINE                    */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWM5 SUBROUTINE TESTS WHETHER AN USER      */
*/*          HAS ACCUMULATED ENOUGH SERVICE/TIME TO BE ASSIGNED      */
*/*          TO A NEW PERFORMANCE GROUP PERIOD.  IF SO, IRARMWM5     */
*/*          ADJUSTS THE POINTERS WHICH INDICATE THE PERFORMANCE     */
*/*          GROUP PERIOD , PERF OBJECTIVE AND DOMAIN APPLICABLE TO  */
*/*          THE TRANSACTION CURRENT FOR THE USER.  NOTE THAT THE    */
*/*          FREQUENCY (RESOLUTION) AT WHICH THE TEST FOR PERIOD     */
*/*          END IS MADE DEPENDS ON HOW OFTEN IRARMWM5 IS CALLED     */
*/*          FOR ANY GIVEN USER.                                     */
*/*                                                                  */
*/*    CALLED BY - IRARMWMO - USER TRANSACTION SUSPEND ROUTINE       */
*/*                IRARMWMQ - USER TRANSACTION QUIESCE ROUTINE       */
*/*                IRARMWMR - USER TRANSACTION RESUMPT ROUTINE       */
*/*                IRARMWMK - USER APPLICABL ALGORITHM SBRTINE       */
*/*                IRARMWM2 - ANALYSIS USER REORDERING ROUTINE       */
*/*                IRARMCVL - SWAP ANALYSIS USER EVALUATION ROUTINE  */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHO IS TO BE TESTED FOR PERIOD END.                   */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - OUCBWMG CONTAINS THE OFFSET TO THE CURRENT            */
*/*            PERFORMANCE GROUP DESCRIPTOR FOR THE USER.            */
*/*                                                                  */
*/*    OUTPT - OUCBPGP CONTAINS THE OFFSET TO THE CURRENT PERIOD     */
*/*            WITHIN THE PERFORMANCE GROUP DESCRIPTION.             */
*/*          - OUCBDMN CONTAINS THE DOMAIN NUMBER FOR THE USER       */
*/*          - OUCBDMO CONTAINS THE OFFSET INTO THE DOMAIN TABLE     */
*/*            BASED ON THE USERS DOMAIN SPECIFICATION               */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WM5) */                                    02S0099
*  DCL IRARMWM5 ENTRY INT;                                      02S0099
*  GEN CODE REFS(LCCACPUS,PSALCCAV,WSACOPTM)                    02S0100
*        DEFS(IRARMWM5) NOEXIT SEQFLOW;                         02S0100
*                                  /*
         ENTRY IRARMWM5
IRARMWM5 DS    0H
         L     @15,PSALCCAV            LCCA ADDRESS
         L     @15,LCCACPUS-LCCA(,@15) CPU SAVE AREA VECTOR TABLE
         L     @15,WSACOPTM-WSAC(,@15) SRM SAVE AREA START ADDRESS
         STM   @00,@14,128+000(@15)    SAVE INVOKING RTNE REGS
         LR    @13,@15                 ROUTINE SAVE AREA ADDRESS
         BALR  @15,0                   SETUP ENTRY ADDRESSABILITY
         LA    @12,*-@PSTART           DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*  /* DETERMINE IF PERFORMANCE GROUP PERIOD DURATION AVAILABLE       */
*  NTRREG = WM5RTRN;                /* RETURN TO REQUESTOR OF WM5    */
         L     NTRREG,WM5RTRN(,SVEREG)                          02S0101
*  IF OUCBTMW ^= ZERO THEN          /* HAVE CURRENT WLM INTERVAL     */
         L     @09,OUCBTMW(,OUCBPTR)                            02S0102
         LTR   @09,@09                                          02S0102
         BZ    @RF00102                                         02S0102
*    /* DETERMINE IF ANOTHER PERFORMANCE GROUP PERIOD FOLLOWS        */
*    DO;                                                        02S0103
*    WMSTPTR = RMCTWMST;            /* PICK WLM SPECIFICATION TABLE  */
         L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0104
*    ASCBPTR = OUCBASCB;            /* PICK ADDRS SPAC CONTROL BLOCK */
         L     ASCBPTR,OUCBASCB(,OUCBPTR)                       02S0105
*    OUXBPTR = ASCBOUXB;            /* PICK USER CTRL BLCK EXTENSION */
         L     OUXBPTR,ASCBOUXB(,ASCBPTR)                       02S0106
*    WPGDPTR = WMSTPGDT + OUCBWMG;  /* PICK CURR PERF GRP DESCRIPTOR */
         LH    WPGDPTR,OUCBWMG(,OUCBPTR)                        02S0107
         AL    WPGDPTR,WMSTPGDT(,WMSTPTR)                       02S0107
*    WKRE = OUXBTRS;                /* CURRENT TRANSACTION SERVICE   */
         L     WKRE,OUXBTRS(,OUXBPTR)                           02S0108
*    IF OUCBRTR ^= '1'B &           /* TRANSACTION NOT SUSPENDED     */
*          OUCBTMS ^= RRPATOD THEN  /* TRANSACTION NOT JUST UPDATED  */
         TM    OUCBRTR(OUCBPTR),B'00010000'                     02S0109
         BO    @RF00109                                         02S0109
         CLC   OUCBTMS(4,OUCBPTR),RRPATOD(RRPAPTR)              02S0109
         BE    @RF00109                                         02S0109
*      WKRE = WKRE + OUCBWMS;       /* ADD CURRENT INTERVL SERVICE   */
         AL    WKRE,OUCBWMS(,OUCBPTR)                           02S0110
*    WPGPPTR = WPGDPTR + OUCBPGP;   /* PICK CURR PERF GROUP PERIOD   */
@RF00109 LR    WPGPPTR,WPGDPTR                                  02S0111
         SLR   @09,@09                                          02S0111
         IC    @09,OUCBPGP(,OUCBPTR)                            02S0111
         ALR   WPGPPTR,@09                                      02S0111
*    IF WPGPFIN ^= '1'B & OUCBNSW = '0'B & /* CAN GO TO NEXT PERIOD  */
*          OUCBINC = '1'B &         /* SESSION NOT BETWEEN STEPS     */
*          (OUCBOUT = '0'B |        /* ALWAYS CHCK PERIOD IF INCORE  */
*          WPGPUNT = '1'B) THEN     /* CHK ONLY REAL TME OUT PERIOD  */
         TM    WPGPFIN(WPGPPTR),B'10000000'                     02S0112
         BO    @RF00112                                         02S0112
         TM    OUCBNSW(OUCBPTR),B'10000000'                     02S0112
         BNZ   @RF00112                                         02S0112
         TM    OUCBINC(OUCBPTR),B'00000001'                     02S0112
         BNO   @RF00112                                         02S0112
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0112
         BZ    @RT00112                                         02S0112
         TM    WPGPUNT(WPGPPTR),B'01000000'                     02S0112
         BNO   @RF00112                                         02S0112
@RT00112 DS    0H                                               02S0113
*      /* CALCULATE HOW MUCH OF CURRENT PERIOD HAS BEEN USED UP      */
*      DO;                                                      02S0113
*      WPGLPTR = WPGDPTR + WPGPDURP; /* ADDR PERF GRP PERIOD DURATN  */
         LR    WPGLPTR,WPGDPTR                                  02S0114
         SLR   @09,@09                                          02S0114
         IC    @09,WPGPDURP(,WPGPPTR)                           02S0114
         ALR   WPGLPTR,@09                                      02S0114
*      IF WPGPUNT = '1'B THEN       /* PERIOD DURATION IN REAL TIME  */
         TM    WPGPUNT(WPGPPTR),B'01000000'                     02S0115
         BNO   @RF00115                                         02S0115
*        /* WILL COMPARE CURR TIME TO PERIOD START PLUS DURATION     */
*        DO;                                                    02S0116
*        WKRE = RRPATOD;            /* CURRENT TIME OF DAY           */
         L     WKRE,RRPATOD(,RRPAPTR)                           02S0117
*        RTNREG = OUCBTMP + WPGLDUR; /* PERIOD ENDING TIME           */
         L     RTNREG,OUCBTMP(,OUCBPTR)                         02S0118
         AL    RTNREG,WPGLDUR(,WPGLPTR)                         02S0118
*        END;                                                   02S0119
*      ELSE                         /* PERIOD DUR IN SERVICE UNITS   */
*        /* WILL COMPARE CURR SERVICE TO PERIOD START PLUS DURATION  */
*        DO;                                                    02S0120
         B     @RC00115                                         02S0120
@RF00115 DS    0H                                               02S0121
*                                   /* CURRENT SERVICE CALC ABOVE    */
*        RTNREG = OUXBPRS + WPGLDUR; /* PERIOD ENDING SERVICE        */
         L     RTNREG,OUXBPRS(,OUXBPTR)                         02S0121
         AL    RTNREG,WPGLDUR(,WPGLPTR)                         02S0121
*        END;                                                   02S0122
*      NTRREG = WM5RTRN;            /* RETURN TO REQUESTOR OF WM5    */
@RC00115 L     NTRREG,WM5RTRN(,SVEREG)                          02S0123
*      /* DETERMINE IF CURRENT PERIOD DURATION HAS BEEN EXCEEDED     */
*      IF WKRE ^< RTNREG THEN       /* PERIOD END HAS BEEN REACHED   */
         CLR   WKRE,RTNREG                                      02S0124
         BL    @RF00124                                         02S0124
*        DO;                                                    02S0125
*          IF WPGPUNT = '1'B THEN   /* PERIOD DURATION IN REAL TIME  */
         TM    WPGPUNT(WPGPPTR),B'01000000'                     02S0126
         BNO   @RF00126                                         02S0126
*            OUCBTMP = RTNREG;      /* STEP NXT PERIOD STARTING TIME */
         ST    RTNREG,OUCBTMP(,OUCBPTR)                         02S0127
*          ELSE                     /* PERIOD DURATN IN SERVIC UNITS */
*            OUCBTMP = RRPATOD;     /* SAVE NEW PERIOD STARTING TIME */
         B     @RC00126                                         02S0128
@RF00126 L     @09,RRPATOD(,RRPAPTR)                            02S0128
         ST    @09,OUCBTMP(,OUCBPTR)                            02S0128
*          IF OUCBOUT = '0'B THEN   /* USER FIELDS ARE AVAILABLE NOW */
@RC00126 TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0129
         BNZ   @RF00129                                         02S0129
*            /* UPDATE BEGINNING SERVICE VALUE FOR NEW PERIOD        */
*            DO;                                                02S0130
*            OUCBPCH = '0'B;        /* RESET PERIOD CHNG OUTSTND FLG */
         NI    OUCBPCH(OUCBPTR),B'11110111'                     02S0131
*            OUXBPRS = OUXBTRS;     /* SAVE NEW PERIOD START SERVICE */
         L     @09,OUXBTRS(,OUXBPTR)                            02S0132
         ST    @09,OUXBPRS(,OUXBPTR)                            02S0132
*            IF OUCBRTR ^= '1'B &   /* TRANSACT NOT IN SUSPEND STATE */
*                  OUCBTMS ^= RRPATOD THEN /* XACTION NOT YET UPDATD */
         TM    OUCBRTR(OUCBPTR),B'00010000'                     02S0133
         BO    @RF00133                                         02S0133
         CLC   OUCBTMS(4,OUCBPTR),RRPATOD(RRPAPTR)              02S0133
         BE    @RF00133                                         02S0133
*              OUXBPRS = OUXBPRS + OUCBWMS; /* ADD CURR NTRVL SERVIC */
         AL    @09,OUCBWMS(,OUCBPTR)                            02S0134
         ST    @09,OUXBPRS(,OUXBPTR)                            02S0134
*            END;                                               02S0135
*          ELSE                     /* UPDATE WHEN USER SWAPPED IN   */
*            OUCBPCH = '1'B;        /* INDICAT PERIOD CHNG INCOMPLET */
         B     @RC00129                                         02S0136
@RF00129 OI    OUCBPCH(OUCBPTR),B'00001000'                     02S0136
*          IF RMCTMFA = '1'B &      /* WRKLD ACTIVITY BEING MEASURED */
*                OUCBATR = '1'B &   /* HAVE TRANSACTION FOR THIS USR */
*                (OUCBRDY = '1'B | OUCBOUT = '0'B) & /* XACTN ACTIVE */
*                OUCBTMS ^= RRPATOD THEN /* SERVICE ALREADY RECORDED */
@RC00129 TM    RMCTMFA(RMCTPTR),B'10000000'                     02S0137
         BNO   @RF00137                                         02S0137
         TM    OUCBATR(OUCBPTR),B'10000000'                     02S0137
         BNO   @RF00137                                         02S0137
         TM    OUCBRDY(OUCBPTR),B'10000000'                     02S0137
         BO    @GL00014                                         02S0137
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0137
         BNZ   @RF00137                                         02S0137
@GL00014 CLC   OUCBTMS(4,OUCBPTR),RRPATOD(RRPAPTR)              02S0137
         BE    @RF00137                                         02S0137
*            /* INVOKE MEASUREMENT ROUTINE TO COLLECT PERIOD SERVICE */
*            DO;                                                02S0138
*            RRPAWAR = '1'B;        /* INSERT WKLD ACTIVTY RTNE FLAG */
         OI    RRPAWAR(RRPAPTR),B'01000000'                     02S0139
*            /* ? IRACTLCL EPN(WR4) */                          02S0140
*            CALL RMSBWR4 -> IRARMRTN ;         /* INVOKE WR4 SUBRTN */
         L     @15,RMSBWR4(,RMCTPTR)                            02S0140
         BALR  @14,@15                                          02S0140
*            OUCBPGP = OUCBPGP + LENGTH(WPGP); /* STEPP PERFM PERIOD */
         LA    @09,8                                            02S0141
         SLR   @08,@08                                          02S0141
         IC    @08,OUCBPGP(,OUCBPTR)                            02S0141
         ALR   @09,@08                                          02S0141
         STC   @09,OUCBPGP(,OUCBPTR)                            02S0141
*            OUCBMAR = '1'B;        /* SPECFY NEGATIV PERIOD ACCUMUL */
         OI    OUCBMAR(OUCBPTR),B'00000100'                     02S0142
*            /* ? IRACTLCL EPN(WR4) */                          02S0143
*            CALL RMSBWR4 -> IRARMRTN ;         /* INVOKE WR4 SUBRTN */
         L     @15,RMSBWR4(,RMCTPTR)                            02S0143
         BALR  @14,@15                                          02S0143
*            OUCBMAR = '0'B;        /* RESET NEGATIVE PERIOD ACCUMUL */
         NI    OUCBMAR(OUCBPTR),B'11111011'                     02S0144
*            RRPAWAR = '0'B;        /* DELETE WKLD ACTIVTY RTNE FLAG */
         NI    RRPAWAR(RRPAPTR),B'10111111'                     02S0145
*            END;                                               02S0146
*          ELSE                     /* NO NEED TO REPORT PERIOD CHNG */
*            OUCBPGP = OUCBPGP + LENGTH(WPGP); /* STEPP PERFM PERIOD */
         B     @RC00137                                         02S0147
@RF00137 LA    @09,8                                            02S0147
         SLR   @08,@08                                          02S0147
         IC    @08,OUCBPGP(,OUCBPTR)                            02S0147
         ALR   @09,@08                                          02S0147
         STC   @09,OUCBPGP(,OUCBPTR)                            02S0147
*          WPGPPTR = WPGDPTR + OUCBPGP; /* PICK CURR PERF GRP PERIOD */
@RC00137 LR    WPGPPTR,WPGDPTR                                  02S0148
         SLR   @09,@09                                          02S0148
         IC    @09,OUCBPGP(,OUCBPTR)                            02S0148
         ALR   WPGPPTR,@09                                      02S0148
*          IF OUCBDMN ^= WPGPDMN THEN   /* SWITCHING DOMAINS ??      */
         CLC   OUCBDMN(1,OUCBPTR),WPGPDMN(WPGPPTR)              02S0149
         BE    @RF00149                                         02S0149
*            DO;                        /* YES                       */
*            OUCBDMN = WPGPDMN ;    /* SAVE THE NEW DOMAIN NUMBER    */
         SLR   @09,@09                                          02S0151
         IC    @09,WPGPDMN(,WPGPPTR)                            02S0151
         STC   @09,OUCBDMN(,OUCBPTR)                            02S0151
*            OUCBDMO = WDMVTENT;    /* OFFSET INTO THE DOMAIN TABLE  */
         ALR   @09,@09                                          02S0152
         AL    @09,WMSTDMVT(,WMSTPTR)                           02S0152
         LH    @09,WDMVTENT(,@09)                               02S0152
         STH   @09,OUCBDMO(,OUCBPTR)                            02S0152
*            /* RE-POSITION USER AT END OF QUEUE                     */
*            OUCBGOB = '1'B;        /* QUEUE POSITION BEING CHANGED  */
         OI    OUCBGOB(OUCBPTR),B'00100000'                     02S0153
*              /* ? IRACTLCL EPN(RPS) */                        02S0154
*              DO;                                              02S0154
*              R1 = ADDR(RMEPBRPS);          /* RPS REQUEST IS PARAM */
         LA    R1,RMEPBRPS(,RMCTPTR)                            02S0155
*              CALL RMSBCRN -> IRARMRTN ;       /* ACTION INVK FUNCT */
         L     @15,RMSBCRN(,RMCTPTR)                            02S0156
         BALR  @14,@15                                          02S0156
*              END;                                             02S0157
*            END;                                               02S0158
*          RFY (NTRREG,WPGLPTR,R15) UNRSTD;   /*             @Z40BPEB*/
@RF00149 DS    0H                                               02S0160
*          IF OUCBAPG='1'B          /* IF USER IN APG        @Z40BPEB*/
*           THEN IF WPGPAPG = '1'B &   /* PERIOD APG         @Z40BPEB*/
         TM    OUCBAPG(OUCBPTR),B'01000000'                     02S0160
         BNO   @RF00160                                         02S0160
*             (WPGPAPGV > CCCAPGDP-CCCAPMDP)  /*SPECIFIES DP @Z40BPEB*/
*            THEN                  /* ABOVE MEAN TIME WAIT   @Z40BPEB*/
         TM    WPGPAPG(WPGPPTR),B'00100000'                     02S0161
         BNO   @RF00161                                         02S0161
         SLR   @15,@15                                          02S0161
         IC    @15,WPGPAPGV(,WPGPPTR)                           02S0161
         LH    @09,CCCAPMDP(,RMCTPTR)                           02S0161
         LH    @08,CCCAPGDP(,RMCTPTR)                           02S0161
         SLR   @08,@09                                          02S0161
         CR    @15,@08                                          02S0161
         BNH   @RF00161                                         02S0161
*              OUCBNDP=CCCAPMDP+WPGPAPGV; /*REQUESTED DP     @Z40BPEB*/
         ALR   @09,@15                                          02S0162
         STC   @09,OUCBNDP(,OUCBPTR)                            02S0162
*            ELSE                   /* IN MEAN TIME TO WAIT  @Z40BPEB*/
*              IF OUCBNDP > CCCAPGDP  /* LAST PERIOD WAS     @Z40BPEB*/
*                THEN                 /* ABOVE MTW           @Z40BPEB*/
         B     @RC00161                                         02S0163
@RF00161 LH    @15,CCCAPGDP(,RMCTPTR)                           02S0163
         SLR   @09,@09                                          02S0163
         IC    @09,OUCBNDP(,OUCBPTR)                            02S0163
         CR    @15,@09                                          02S0163
         BNL   @RF00163                                         02S0163
*                  OUCBNDP=CCCAPGDP;  /*BEGIN MTW DP.        @Z40BPEB*/
         STC   @15,OUCBNDP(,OUCBPTR)                            02S0164
*          RFY (NTRREG,WPGLPTR,R15) RSTD;     /*HOLD R15    @Z40BPEB*/
@RF00163 DS    0H                                               02S0165
@RC00161 DS    0H                                               02S0165
@RF00160 DS    0H                                               02S0166
*          IF WPGPUNT = '1'B &      /* PERIOD DURATION IN REAL TIME  */
*                OUCBOFF = '1'B THEN /* MULTI-CHECK ONLY WAIT USERS  */
         TM    WPGPUNT(WPGPPTR),B'01000000'                     02S0166
         BNO   @RF00166                                         02S0166
         TM    OUCBOFF(OUCBPTR),B'00001000'                     02S0166
         BNO   @RF00166                                         02S0166
*            SVRE = RMSBWM5;        /* WILL RECURSE THROUGH WM5 RTN  */
         L     SVRE,RMSBWM5(,RMCTPTR)                           02S0167
*          ELSE                     /* PERIOD DURATN IN SERVC UNITS  */
*            SVRE = WM5RTRN;        /* WILL RETURN TO REQSTR OF WM5  */
         B     @RC00166                                         02S0168
@RF00166 L     SVRE,WM5RTRN(,SVEREG)                            02S0168
*          CALL IRARMWMJ;           /* SET APPLICABLE ALGORITHM INDS */
@RC00166 BAL   @14,IRARMWMJ                                     02S0169
*          NTRREG = SVRE;           /* RESTORE WM5 EXIT REGISTER     */
         LR    NTRREG,SVRE                                      02S0170
*          END;                                                 02S0171
*      END;                                                     02S0172
@RF00124 DS    0H                                               02S0173
*    END;                                                       02S0173
@RF00112 DS    0H                                               02S0174
*  /* ? IRACTLEP ENN(WM5) */                                    02S0174
*  GEN CODE EXIT NOSEQFLOW;                                     02S0174
@RF00102 DS    0H                                               02S0174
*                                  /*
         LM    @00,@14,128+000(@13)    RELOAD INVOKING RTNE REGS
         BR    NTRREG                  EXIT TO REQUESTOR/RECURSE
*                                                               01S0175
*/* USER RECOMMENDATION VALUE CALCULATION ROUTINE                    */
*/* ******************************************************************/
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWM7                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WLM RECOMMENDATION CALCULATION ROUTINE     */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWM7 ROUTINE CALCULATES A WORKLOAD         */
*/*               MANAGER RECOMMENDATION VALUE FOR A USER BASED ON   */
*/*               THE SERVICE WHICH WAS RECEIVED AND ON THE          */
*/*               PERFORMANCE OBJECTIVE CURRENTLY ASSOCIATED WITH    */
*/*               THE USER. USERS WHICH HAVE NOT YET RECEIVIED AN    */
*/*               AMOUNT OF SERVICE EQUAL TO THEIR INTERVAL SERVICE  */
*/*               VALUE (ISV) SPECIFICATION WHILE IN CORE ARE GIVEN  */
*/*               A RECOMMENDATION VALUE BOOST. THE BOOST GIVES      */
*/*               PREFERENTIAL TREATMENT TO USERS IN THEIR ISV AS    */
*/*               COMPARED TO USERS NOT IN THEIR ISV AND USERS       */
*/*               BETWEEN JOB STEPS.                                 */
*/*                                                                  */
*/*    CALLED BY - IRARMWM3 - INDIVIDUAL USER EVALUATION SUBROUTINE  */
*/*              - IRARMWM2 - WLM SWAPPABLE USER EVALUATION ROUTINE  */
*/*              - IRARMWMO - WLM TRANSACTION STOP ROUTINE           */
*/*              - IRARMWMQ - WLM QUIESCE COMPLETE PROCESSING ROUTIN */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHOSE RECOMMENDATION VALUE IS TO BE DETERMINED.       */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWM7 ROUTINE ADDRESS.         */
*/*                                                                  */
*/*    OUTPT - OUCBWMR CONTAINS THE USER RECOMMENDATION VALUE.       */
*/*                                                                  */
*/*    EXIT - BRANCH ON REGISTER 14.                                 */
*/*         - THERE IS NO ERROR EXIT OR INDICATION.                  */
*/*                                                                  */
*/* ******************************************************************/
*IRARMWM7: PROC OPTIONS(NOSAVE,NOSAVEAREA,NOENTREG,RETREG,NOPARMREG);
IRARMWM7 DS    0H                                               02S0176
*    IF OUCBNSW = '0'B &           /* USER IS SWAPPABLE     */  02S0176
*       OUCBGOO = '0'B & OUCBGOI = '0'B & /* NOT TRANSITIONING       */
*       OUCBGOB = '0'B THEN                                     02S0176
         TM    OUCBNSW(OUCBPTR),B'10000000'                     02S0176
         BNZ   @RF00176                                         02S0176
         TM    OUCBGOO(OUCBPTR),B'11100000'                     02S0176
         BNZ   @RF00176                                         02S0176
*      DO;                                                      02S0177
*      /* PICK UP PERFORMANCE GROUP PERIOD                           */
*      WMSTPTR = RMCTWMST ;         /* PICK WLM SPECIFICATION TABLE  */
         L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0178
*      WPGDPTR = WMSTPGDT + OUCBWMG ;  /* PICK PERF GROUP DESCRIPTOR */
         LH    WPGDPTR,OUCBWMG(,OUCBPTR)                        02S0179
         AL    WPGDPTR,WMSTPGDT(,WMSTPTR)                       02S0179
*      WPGPPTR = WPGDPTR + OUCBPGP; /* PICK PERF GROUP PERIOD        */
         LR    WPGPPTR,WPGDPTR                                  02S0180
         SLR   @09,@09                                          02S0180
         IC    @09,OUCBPGP(,OUCBPTR)                            02S0180
         ALR   WPGPPTR,@09                                      02S0180
*      /* SET PERFORMANCE OBJECTIVE APPLICABLE TO USER               */
*      R1 = WMSTPODT + WPOVTENT;    /* PTR TO PERF PERIOD OBJECTIVE  */
         SLR   @09,@09                                          02S0181
         IC    @09,WPGPOBJN(,WPGPPTR)                           02S0181
         ALR   @09,@09                                          02S0181
         AL    @09,WMSTPOVT(,WMSTPTR)                           02S0181
         LH    @09,WPOVTENT(,@09)                               02S0181
         AL    @09,WMSTPODT(,WMSTPTR)                           02S0181
         LR    R1,@09                                           02S0181
*      WPGLPTR = WPGDPTR + WPGPISVP; /* PTR TO ISV VALUE             */
         LR    WPGLPTR,WPGDPTR                                  02S0182
         SLR   @09,@09                                          02S0182
         IC    @09,WPGPISVP(,WPGPPTR)                           02S0182
         ALR   WPGLPTR,@09                                      02S0182
*      SVRO = ZERO;                 /* INITIALIZE ISV BOOST          */
         SLR   SVRO,SVRO                                        02S0183
*      /* TEST IF SERVICE RECIEVED LESS THAN ISV AMOUNT OR IF USER   */
*      /* IS BETWEEN JOB STEPS. IF SO, ASSIGN RECOMMENDATION VALUE   */
*      /* WHERE OBJECTIVE GOES TO ZERO POINT.                        */
*      IF OUCBWMS < WPGLISV |       /* SERVICE LESS THAN ISV AMOUNT  */
*        OUCBINC = '0'B THEN        /* OR JOB IS BETWEEN STEPS       */
         L     @09,OUCBWMS(,OUCBPTR)                            02S0184
         C     @09,WPGLISV(,WPGLPTR)                            02S0184
         BL    @RT00184                                         02S0184
         TM    OUCBINC(OUCBPTR),B'00000001'                     02S0184
         BNZ   @RF00184                                         02S0184
@RT00184 DS    0H                                               02S0185
*        DO;                                                    02S0185
*        R0 = ZERO;                 /* PEG NORMLZ LEVEL AT ZERO RATE */
         SLR   R0,R0                                            02S0186
*        IF OUCBPVL = '0'B &        /* USER NOT PRIVILEDGED          */
*           OUCBOUT = '0'B &        /* AND IN CORE                   */
*           OUCBTMW ^= ZERO THEN    /* AND HAS TRANSACTION INTERVAL  */
         TM    OUCBPVL(OUCBPTR),B'00000100'                     02S0187
         BNZ   @RF00187                                         02S0187
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0187
         BNZ   @RF00187                                         02S0187
         L     @09,OUCBTMW(,OUCBPTR)                            02S0187
         LTR   @09,@09                                          02S0187
         BZ    @RF00187                                         02S0187
*          SVRO = RMCAISV;          /* GIVE USER ISV BOOST           */
         LH    SVRO,RMCAISV(,RMCTPTR)                           02S0188
         N     SVRO,@CF02217                                    02S0188
*        END;                                                   02S0189
*      ELSE                                                     02S0190
*        /* CALCULATE RATE - DIVIDE AMOUNT OF SERVICE BY TIME NTRVL  */
*        DO;                                                    02S0190
         B     @RC00184                                         02S0190
@RF00184 DS    0H                                               02S0191
*        WKRO = RRPATOD - OUCBTMW;  /* SET WORKLOAD INTERVAL DURATN  */
         L     WKRO,RRPATOD(,RRPAPTR)                           02S0191
         SL    WKRO,OUCBTMW(,OUCBPTR)                           02S0191
*        VLRO = MAX(VLRO,ONE);      /* PROTECT FRM DIVIDE EXCEPTION  */
         L     @09,ONE                                          02S0192
         CR    VLRO,@09                                         02S0192
         BNL   *+6
         LR    VLRO,@09                                         02S0192
*        R0 = OUCBWMS * RMTMFCT / WKRO; /* SERVICE RATE FOR INTERVL  */
         LH    @09,RMTMFCT                                      02S0193
         M     @08,OUCBWMS(,OUCBPTR)                            02S0193
         DR    @08,WKRO                                         02S0193
         LR    R0,@09                                           02S0193
*        END;                                                   02S0194
*      WKRO = RTNREG;               /* SAVE RETURN REG CONTENTS      */
@RC00184 LR    WKRO,RTNREG                                      02S0195
*       /* ? IRACTLCL EPN(WM4) */                               02S0196
*       CALL RMSBWM4 -> IRARMRTN ;              /* INVOKE WM4 SUBRTN */
         L     @15,RMSBWM4(,RMCTPTR)                            02S0196
         BALR  @14,@15                                          02S0196
*        /* CALC USER NORM WKL LEVEL      */                    02S0197
*      OUCBWMR = R0+SVRO ;          /* GIVE USER NORMALIZED LEVEL    */
         LR    @09,R0                                           02S0197
         ALR   @09,SVRO                                         02S0197
         ST    @09,OUCBWMR(,OUCBPTR)                            02S0197
*      OUCBTMA=RRPATOD ;            /* INDICATE USER RECENTLY ANALZD */
         L     @09,RRPATOD(,RRPAPTR)                            02S0198
         ST    @09,OUCBTMA(,OUCBPTR)                            02S0198
*      RTNREG = WKRO;               /* RESTORE RETURN REG CONTENTS   */
         LR    RTNREG,WKRO                                      02S0199
*      END;                                                     02S0200
*    END IRARMWM7;                                              02S0201
@EL00002 DS    0H                                               02S0201
@EF00002 DS    0H                                               02S0201
@ER00002 BR    @14                                              02S0201
*                                                               01S0202
*/* USERRDY SYSEVENT PROCESSOR                                       */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMHIT                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER USER READY SYSEVENT       */
*/*                       SWAP-IN SCHEDULING ROUTINE                 */
*/*                                                                  */
*/*    FUNCTION - THE IRARMHIT ROUTINE RECEIVES CONTROL AS THE       */
*/*          RESULT OF A DECISION TO APPLY SWAPIN PROCESSING         */
*/*          TO A NEWLY AWAKENED USER.                               */
*/*                                                                  */
*/*    CALLED BY - IRARME04 - USER READY SYSEVENT PROCESS RTNE       */
*/*                IRARME15 - SWAP OUT COMPLETE SYSEVENT RTNE        */
*/*                IRARME44 - BRINGIN SYSEVENT RTNE                  */
*/*                                                                  */
*/*    INPUT - REG4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER    */
*/*                                                                  */
*/*    OUTPT - SEE FUNCTION.                                         */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/*    CHANGE ACTIVITY - XXXXXXX                                    */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(HIT) */                                    02S0202
*  DCL IRARMHIT ENTRY INT;                                      02S0202
*  GEN CODE REFS(RMCTRMSA,RRPAEPA)                              02S0203
*        DEFS(IRARMHIT) NOEXIT SEQFLOW;                         02S0203
*                                  /*
         ENTRY IRARMHIT
IRARMHIT DS    0H
         L     @15,RMCTRMSA(,RMCTPTR)  SRM SAVE AREA START ADDRESS
         STM   @00,@14,064+000(@15)    SAVE INVOKING RTNE REGS
         MVC   064+060(4,@15),RRPAEPA(RRPAPTR) SAVE PREV RMEP
         ST    @01,RRPAEPA(,RRPAPTR)   INDICATE NEW CURRENT RMEP
         LR    @13,@15                 ROUTINE SAVE AREA ADDRESS
         BALR  @15,0                   SETUP ENTRY ADDRESSABILITY
         LA    @12,*-@PSTART           DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*  IF OUCBOUT = '1'B & OUCBGOB = '0'B & /* USR AVAILABLE FOR SWAP-IN */
*        OUCBGOO = '0'B & OUCBGOI = '0'B THEN /* USER NOT TRANSITION */
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0204
         BNO   @RF00204                                         02S0204
         TM    OUCBGOB(OUCBPTR),B'11100000'                     02S0204
         BNZ   @RF00204                                         02S0204
*    /* PREPARE SWAP-IN CANDIDACY OF READIED USER                    */
*    DO;                                                        02S0205
*    RFY (ASCBPTR,OUXBPTR,STPR,PMRE,SVRE) UNRSTD;               02S0206
*  IF OUCBOFF = '1'B & OUCBATR = '1'B & /* TRNSACTION SUSPENDED */
*     OUCBINC = '1'B THEN /* SESSION IS NOT  BETWEEN STEPS */   02S0207
         TM    OUCBOFF(OUCBPTR),B'00001000'                     02S0207
         BNO   @RF00207                                         02S0207
         TM    OUCBATR(OUCBPTR),B'10000001'                     02S0207
         BNO   @RF00207                                         02S0207
*  /* TEST FOR PERFORMANCE GROUP PERIOD CHANGE */               02S0208
*    /* ? IRACTLCL EPN(WM5) */                                  02S0208
*    CALL RMSBWM5 -> IRARMRTN ;                 /* INVOKE WM5 SUBRTN */
         L     @15,RMSBWM5(,RMCTPTR)                            02S0208
         BALR  @14,@15                                          02S0208
*    WMSTPTR = RMCTWMST;            /* PICK WLM SPECIFICATION TABLE  */
@RF00207 L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0209
*    WPGDPTR = WMSTPGDT + WPGVTENT; /* PICK PERFORM GROUP DESCRIPTR  */
         SLR   @05,@05                                          02S0210
         IC    @05,OUCBNPG(,OUCBPTR)                            02S0210
         ALR   @05,@05                                          02S0210
         AL    @05,WMSTPGVT(,WMSTPTR)                           02S0210
         LH    WPGDPTR,WPGVTENT(,@05)                           02S0210
         AL    WPGDPTR,WMSTPGDT(,WMSTPTR)                       02S0210
* /* TEST IF EXISTING TRANSACTION IS TO BE RESUMED */           02S0211
*   IF OUCBPVL = '1'B |   /* PRIVILEDGE USER - NO TRANS */      02S0211
*      OUCBENQ = '1'B |   /*  USER HOLDING WANTED RESOURCE           */
*      (OUCBOFF ='1'B &   /* USER AWAKENED FORM LONG WAIT            */
*      (OUCBSTR = '1'B |  /* STARTING NEW TRANS */              02S0211
*      OUCBATR = '0'B)) THEN /* NO TRANSACTION EXISTS/               */
         TM    OUCBPVL(OUCBPTR),B'00000110'                     02S0211
         BNZ   @RT00211                                         02S0211
         TM    OUCBOFF(OUCBPTR),B'00001000'                     02S0211
         BNO   @RF00211                                         02S0211
         TM    OUCBSTR(OUCBPTR),B'01000000'                     02S0211
         BNZ   @RT00211                                         02S0211
         TM    OUCBATR(OUCBPTR),B'10000000'                     02S0211
         BO    @RF00211                                         02S0211
@RT00211 DS    0H                                               02S0212
*    DO ;                                                       02S0212
*    OUCBTMS = RRPATOD ;            /* INIT INTERVAL BEGIN TIME      */
         L     @06,RRPATOD(,RRPAPTR)                            02S0213
         ST    @06,OUCBTMS(,OUCBPTR)                            02S0213
*    OUCBOFF = '0'B ;               /* INDICATE USER STATUS NOT WAIT */
         NI    OUCBOFF(OUCBPTR),B'11110111'                     02S0214
*      IF OUCBTRM = '1'B & OUCBCIM = '0'B &    /* TERMINL WAIT USER  */
*         OUCBINC = '1'B THEN       /* AND NOT BETWEEN JOB STEPS     */
         TM    OUCBTRM(OUCBPTR),B'01000000'                     02S0215
         BNO   @RF00215                                         02S0215
         TM    OUCBCIM(OUCBPTR),B'00010000'                     02S0215
         BNZ   @RF00215                                         02S0215
         TM    OUCBINC(OUCBPTR),B'00000001'                     02S0215
         BNO   @RF00215                                         02S0215
*        /* USER READY AFTER TERMINAL WAIT STARTS NEW TRANSACTION    */
*        DO;                                                    02S0216
*        OUCBNTR = '1'B;            /* FLAG XACTN PERFM GROUP RESET  */
*        OUCBSTR = '1'B;            /* FLAG XACTN START REQST PENDNG */
         OI    OUCBNTR(OUCBPTR),B'01100000'                     02S0218
*        OUCBWMG = WPGDPTR - WMSTPGDT; /* PERF GROUP DESCRIPT OFFSET */
         LR    @05,WPGDPTR                                      02S0219
         SL    @05,WMSTPGDT(,WMSTPTR)                           02S0219
         STH   @05,OUCBWMG(,OUCBPTR)                            02S0219
*        OUCBPGP = WPGPOFST;        /* PERF GRP OFFSET TO 1ST PERIOD */
         LA    @05,12                                           02S0220
         STC   @05,OUCBPGP(,OUCBPTR)                            02S0220
*        WPGPPTR = WPGDPTR + OUCBPGP; /* PICK PERF GROUP PERIOD      */
         ALR   @05,WPGDPTR                                      02S0221
         LR    WPGPPTR,@05                                      02S0221
*        OUCBDMN = WPGPDMN;         /* PERF GROUP PERIOD DOMAIN NO.  */
         SLR   @05,@05                                          02S0222
         IC    @05,WPGPDMN(,WPGPPTR)                            02S0222
         STC   @05,OUCBDMN(,OUCBPTR)                            02S0222
*        OUCBDMO = WDMVTENT ;       /* OFFSET INTO THE DOMAIN TABLE  */
         ALR   @05,@05                                          02S0223
         AL    @05,WMSTDMVT(,WMSTPTR)                           02S0223
         LH    @05,WDMVTENT(,@05)                               02S0223
         STH   @05,OUCBDMO(,OUCBPTR)                            02S0223
*        IF OUCBPVL = '0'B THEN     /* USER SUBJECT TO WKLD ANALYSIS */
         TM    OUCBPVL(OUCBPTR),B'00000100'                     02S0224
         BNZ   @RF00224                                         02S0224
*          OUCBTMW = RRPATOD;       /* INDICAT INTERVAL WAS ANALYZED */
         ST    @06,OUCBTMW(,OUCBPTR)                            02S0225
*        OUCBTMP = RRPATOD;         /* INITL PERFM PERIOD START TIME */
@RF00224 L     @06,RRPATOD(,RRPAPTR)                            02S0226
         ST    @06,OUCBTMP(,OUCBPTR)                            02S0226
*        END;                                                   02S0227
*        WPGPPTR = WPGDPTR + OUCBPGP; /* PICK CURR PERF GROUP PERIOD */
@RF00215 LR    WPGPPTR,WPGDPTR                                  02S0228
         SLR   @06,@06                                          02S0228
         IC    @06,OUCBPGP(,OUCBPTR)                            02S0228
         ALR   WPGPPTR,@06                                      02S0228
*          /* CALCULATE WKLD LEVL APPROPRIATE TO NEWLY STARTED XACTN */
*          R0 = ZERO;               /* GOTTN XACTN SRVC RATE IS ZERO */
         SLR   R0,R0                                            02S0229
*          R1 = WMSTPODT + WPOVTENT; /* PERF OBJECTIVE APPLIC TO USR */
         SLR   @06,@06                                          02S0230
         IC    @06,WPGPOBJN(,WPGPPTR)                           02S0230
         ALR   @06,@06                                          02S0230
         AL    @06,WMSTPOVT(,WMSTPTR)                           02S0230
         LH    @06,WPOVTENT(,@06)                               02S0230
         AL    @06,WMSTPODT(,WMSTPTR)                           02S0230
         LR    R1,@06                                           02S0230
*          /* ? IRACTLCL EPN(WM4) */                            02S0231
*          CALL RMSBWM4 -> IRARMRTN ;           /* INVOKE WM4 SUBRTN */
         L     @15,RMSBWM4(,RMCTPTR)                            02S0231
         BALR  @14,@15                                          02S0231
*          OUCBWMR = R0;            /* SAVE CURRENT NORMALIZED LEVEL */
         ST    R0,OUCBWMR(,OUCBPTR)                             02S0232
*          OUCBTMA = RRPATOD ;  /* INDICATE USER RECENTLY ANALYZED */
         L     @06,RRPATOD(,RRPAPTR)                            02S0233
         ST    @06,OUCBTMA(,OUCBPTR)                            02S0233
*     END ;                                                     02S0234
* ELSE                                                          02S0235
* /* PUT AWAKENED USER BACK INTO NORMAL SWAPPING CATEGORY */    02S0235
*  DO ;                                                         02S0235
         B     @RC00211                                         02S0235
@RF00211 DS    0H                                               02S0236
*    IF OUCBINC = '1'B & OUCBRTR = '1'B THEN /* RESUME TRANS */ 02S0236
         TM    OUCBINC(OUCBPTR),B'00010001'                     02S0236
         BNO   @RF00236                                         02S0236
*     /* SETUP WLM INTERVAL AS IF SWAP-OUT HAD JUST ENDED */    02S0237
*      DO ;                                                     02S0237
*       OUCBTMW = OUCBTMW + RRPATOD-OUCBTMS ; /* ADJUST WLM TIME */
         L     @06,OUCBTMW(,OUCBPTR)                            02S0238
         AL    @06,RRPATOD(,RRPAPTR)                            02S0238
         SL    @06,OUCBTMS(,OUCBPTR)                            02S0238
         ST    @06,OUCBTMW(,OUCBPTR)                            02S0238
*      IF OUCBTMW = ZERO THEN  /* DISALLOW  0 FOR INTERVAL TIME */
         LTR   @06,@06                                          02S0239
         BNZ   @RF00239                                         02S0239
*         OUCBTMW = ONE ; /* INSERT NONZERO STARTING TIME */    02S0240
         L     @06,ONE                                          02S0240
         ST    @06,OUCBTMW(,OUCBPTR)                            02S0240
*      OUCBTMS = RRPATOD ; /* LAST SWAP ACTION TIME OF READY */ 02S0241
@RF00239 L     @06,RRPATOD(,RRPAPTR)                            02S0241
         ST    @06,OUCBTMS(,OUCBPTR)                            02S0241
*    END ;                                                      02S0242
*  END ;                                                        02S0243
@RF00236 DS    0H                                               02S0244
*/* POSITION USER ON OUT QUEUE   */                             02S0244
*  OUCBGOB = '1'B ;                 /* QUEUE POSITION BEING CHANGED  */
@RC00211 OI    OUCBGOB(OUCBPTR),B'00100000'                     02S0244
*  /* ? IRACTLCL EPN(RPS) */                                    02S0245
*  DO;                                                          02S0245
*  R1 = ADDR(RMEPBRPS);                      /* RPS REQUEST IS PARAM */
         LA    R1,RMEPBRPS(,RMCTPTR)                            02S0246
*  CALL RMSBCRN -> IRARMRTN ;                   /* ACTION INVK FUNCT */
         L     @15,RMSBCRN(,RMCTPTR)                            02S0247
         BALR  @14,@15                                          02S0247
*  END;                                                         02S0248
*/* REQUEST ANALYSYS IN ORDER TO EXPEDITITE THE SWAP-IN              */
*  /* ? IRACTLCL EPN(CAP) */                                    02S0249
*  DO;                                                          02S0249
*  R1 = ADDR(RMEPBCAP);                      /* CAP REQUEST IS PARAM */
         LA    R1,RMEPBCAP(,RMCTPTR)                            02S0250
*  CALL RMSBCRL -> IRARMRTN ;                   /* ALGRTHM SCH FUNCT */
         L     @15,RMSBCRL(,RMCTPTR)                            02S0251
         BALR  @14,@15                                          02S0251
*  END;                                                         02S0252
*  RFY   (ASCBPTR,OUXBPTR,STPR,PMRE,SVRE) RSTD;                 02S0253
*END ;                                                          02S0254
*  /* ? IRACTLEP ENN(HIT) */                                    02S0255
*  GEN CODE EXIT NOSEQFLOW;                                     02S0255
@RF00204 DS    0H                                               02S0255
*                                  /*
         MVC   RRPAEPA(4,RRPAPTR),064+060(@13) RESTORE PREV RMEP
         LM    @00,@14,064+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0256
*/* TRANSACTION START ROUTINE                                        */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMN                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER TRANSACTION START RTNE    */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMN ROUTINE RECEIVES CONTROL AS THE       */
*/*          RESULT OF A SYSEVENT THAT HAS BEEN DEFINED BY THE       */
*/*          WORKLOAD MANAGER TO SIGNIFY THAT A NEW TRANSACTION      */
*/*          SHOULD BE STARTED FOR THAT USER.  IF THE USER IS        */
*/*          NOT IN CORE, A FLAG IS SET TO CAUSE THE IRARMWMN        */
*/*          ROUTINE TO BE RE-ENTERED DURING THE SWAP-IN OF THE      */
*/*          USER.  OTHERWISE, ANY EXISTING XACTN IS STOPPED BY      */
*/*          CALLING IRARMWMO, AND THE USER TRANSACTION FIELDS       */
*/*          ARE RESET TO REFLECT THE NEW XACTN BEING STARTED.       */
*/*                                                                  */
*/*    CALLED BY - IRARME10 - INITIATOR ATTCH SYSEVENT ROUTINE       */
*/*                IRARME31 - PERFM GRP RESET SYSEVENT ROUTINE       */
*/*                IRARME32 - NEW PERFM SPECF SYSEVENT ROUTINE       */
*/*                IRARME34 - TERMINAL GET/PT SYSEVENT ROUTINE       */
*/*                IRARMWMR - USER TRANSACTION RESUMPT ROUTINE       */
*/*                IRARMSET - SYS IPS CHANGE PROCESSNG ROUTINE       */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            FOR WHOM A TRANSACTION IS TO BE STARTED.              */
*/*          - REG 5 CONTAINS THE USER ASCB ADDRESS.                 */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWMN RTNE ADDRESS.            */
*/*          - OUCBNPG IS SET TO THE APPLICABLE PERF GRP NUMBER.     */
*/*          - OUCBATR IS SET IF HAVE A XACTN CURR FOR THE USER.     */
*/*          - RMCTMFA IS SET IF WKLD ACTIVITY IS BEING MEASURD.     */
*/*          - OUCBOUT INDICATES WHETHER THE USER IS NOT IN CORE.    */
*/*          - OUCBTMW INDICATES WHETHER USER ANALYZED AT SWAPIN.    */
*/*                                                                  */
*/*    OUTPT - OUCBWMG IS SET TO THE PERF GRP DESCRIPT TO BE USED.   */
*/*          - OUCBPGP IS SET TO THE 1ST PERIOD IN THE PERF GROUP.   */
*/*          - OUXBTRS,TRT,TMO,PRS,TMP VALUES ARE RESET.             */
*/*          - OUCBSTR,NTR,RTR ARE SET OFF AT BEGIN OF TRANSACTN.    */
*/*          - OUCBATR IS SET ON TO INDICATE HAV ACTIVE TRANSACTN.   */
*/*          - OUCBSTR IS SET ON IF USER CURRENTLY SWAPPED OUT.      */
*/*          - OUCBSWC IS SET TO ZERO .                              */
*/*          - OUCBTMO IS SET TO THE TRANSACTION STARTING TIME.      */
*/*          - OUXBTRC IS AUGUMENTED TO SHOW ONE MORE TRANSACTION.   */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WMN) */                                    02S0256
*  DCL IRARMWMN ENTRY INT;                                      02S0256
*  GEN CODE DEFS(IRARMWMN) NOEXIT SEQFLOW;                      02S0257
*                                  /*
         ENTRY IRARMWMN
IRARMWMN DS    0H
         STM   @00,@14,000+000(@13)    SAVE INVOKING RTNE REGS
         LA    @12,IRARMWMN-@PSTART    DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*    RFY (WKRE) UNRSTD;                                         02S0258
*IRARMWMB: /* POINT AT WHICH PERFORM TRANSACTION START PROCESSING    */
*    OUXBPTR = ASCBOUXB;            /* PICK USER CTRL BLCK EXTENSION */
IRARMWMB L     OUXBPTR,ASCBOUXB(,ASCBPTR)                       02S0259
*    WMSTPTR = RMCTWMST;            /* PICK WLM SPECIFICATION TABLE  */
         L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0260
*    WPGDPTR = WMSTPGDT + WPGVTENT; /* PICK USER PERF GRP DESCRIPTOR */
         SLR   @10,@10                                          02S0261
         IC    @10,OUCBNPG(,OUCBPTR)                            02S0261
         ALR   @10,@10                                          02S0261
         AL    @10,WMSTPGVT(,WMSTPTR)                           02S0261
         LH    WPGDPTR,WPGVTENT(,@10)                           02S0261
         AL    WPGDPTR,WMSTPGDT(,WMSTPTR)                       02S0261
*    IF OUCBINP = '1'B THEN         /* JOB SELECT SYSEVENT RECEIVED  */
         TM    OUCBINP(OUCBPTR),B'00000010'                     02S0262
         BNO   @RF00262                                         02S0262
*      /* WORKLOAD MANAGER TRANSACTION PROCESSING APPLICABLE TO USER */
*      DO;                                                      02S0263
*      OUCBSTR = '1'B;              /* FLAG XACTN START REQST PENDNG */
         OI    OUCBSTR(OUCBPTR),B'01000000'                     02S0264
*      IF OUCBINC = '1'B &          /* USER IS SUBJECT TO WLM CONTRL */
*            OUCBOUT = '0'B THEN    /* USER FIELDS ARE AVAILABLE NOW */
         TM    OUCBINC(OUCBPTR),B'00000001'                     02S0265
         BNO   @RF00265                                         02S0265
         TM    OUCBOUT(OUCBPTR),B'00000100'                     02S0265
         BNZ   @RF00265                                         02S0265
*        /* INITIALIZE PROPR USER FIELDS TO REFLECT NEW STARTD XACTN */
*        DO;                                                    02S0266
*        IF OUCBATR = '1'B THEN     /* PREVIOUS XACTN IN EXISTENCE   */
         TM    OUCBATR(OUCBPTR),B'10000000'                     02S0267
         BNO   @RF00267                                         02S0267
*          /* INVOKE ROUTINE TO CLEAN UP THE PREVIOUS TRANSACTION    */
*          /* ? IRACTLCL EPN(WMO) */                            02S0268
*          CALL RMSBWMO -> IRARMRTN ;           /* INVOKE WMO SUBRTN */
         L     @15,RMSBWMO(,RMCTPTR)                            02S0268
         BALR  @14,@15                                          02S0268
*        OUCBWMG = WPGDPTR - WMSTPGDT; /* PERF GRP DESCRIPTOR OFFSET */
@RF00267 LR    @10,WPGDPTR                                      02S0269
         SL    @10,WMSTPGDT(,WMSTPTR)                           02S0269
         STH   @10,OUCBWMG(,OUCBPTR)                            02S0269
*        WPGPPTR = WPGDPTR + OUCBPGP ; /* ADDR OF PERF GROUP PERIOD  */
         LR    WPGPPTR,WPGDPTR                                  02S0270
         SLR   @10,@10                                          02S0270
         IC    @10,OUCBPGP(,OUCBPTR)                            02S0270
         ALR   WPGPPTR,@10                                      02S0270
*        OUCBATR = '1'B;            /* INDICATE HAVE ACTIVE TRANSACN */
*        OUCBSTR = '0'B;            /* RESET XACTN START REQUEST FLG */
*        OUCBNTR = '0'B;            /* RESET XACTN TERMINATE REQ FLG */
*        OUCBRTR = '0'B;            /* RESET XACTN RESUM REQUEST FLG */
*        OUCBPCH = '0'B;            /* RESET PERIOD CHNG OUTSTND FLG */
         OI    OUCBATR(OUCBPTR),B'10000000'                     02S0275
         NI    OUCBSTR(OUCBPTR),B'10000111'                     02S0275
*        OUXBTRC = OUXBTRC + ONE;   /* INDICATE HAV ONE MOR TRANSACN */
         L     @10,OUXBTRC(,OUXBPTR)                            02S0276
         AL    @10,ONE                                          02S0276
         ST    @10,OUXBTRC(,OUXBPTR)                            02S0276
*        OUCBSWC = ZERO;            /* RESET USER SWAP COUNT        */
         SLR   @10,@10                                          02S0277
         STH   @10,OUCBSWC(,OUCBPTR)                            02S0277
*        OUXBTRS = ZERO;            /* RESET XACTN SERVIC UNIT ACCUM */
         ST    @10,OUXBTRS(,OUXBPTR)                            02S0278
*        OUXBTRT = ZERO;            /* RESET XACTN ACTIVE TIME ACCUM */
         ST    @10,OUXBTRT(,OUXBPTR)                            02S0279
*        OUXBTRR = ZERO;            /* RESET XACTN RESDNT TIME ACCUM */
         ST    @10,OUXBTRR(,OUXBPTR)                            02S0280
*        OUXBPRS = ZERO;            /* RESET PERIOD SERVC UNIT FIELD */
         ST    @10,OUXBPRS(,OUXBPTR)                            02S0281
*        OUCBWMS = ZERO;            /* RESET WLM INTERVAL CURR SERVC */
         ST    @10,OUCBWMS(,OUCBPTR)                            02S0282
*        OUCBCPU = ZERO ;           /* RESET CPU SERVICE             */
         ST    @10,OUCBCPU(,OUCBPTR)                            02S0283
*        OUCBIOC = ZERO ;           /* RESET IO  SERVICE             */
         ST    @10,OUCBIOC(,OUCBPTR)                            02S0284
*        OUCBMSO = ZERO ;           /* RESET MSO SERVICE             */
         ST    @10,OUCBMSO(,OUCBPTR)                            02S0285
*        IF RRPACOD ^= RSTORCMP THEN /* XACTN NOT STARTD BY USER RDY */
         CLI   RRPACOD(RRPAPTR),19                              02S0286
         BE    @RF00286                                         02S0286
*          /* IMPROVE RESPONSE OF INITIAL TSO PROGRAM INVOCATION     */
*          OUCBTMS = RRPATOD;       /* INSERT DUMMY LAST ACTION TIME */
         L     @10,RRPATOD(,RRPAPTR)                            02S0287
         ST    @10,OUCBTMS(,OUCBPTR)                            02S0287
*        ELSE                       /* XACTN WAS STARTED BY USER RDY */
*          /* RECORD THE AMOUNT OF TIME SPENT OUT OF CORE BY XACTN   */
*          DO;                                                  02S0288
         B     @RC00286                                         02S0288
@RF00286 DS    0H                                               02S0289
*          OUXBTRT = RRPATOD - OUCBTMS; /* KEEP TRANSACTION OUT TIME */
         L     @10,RRPATOD(,RRPAPTR)                            02S0289
         SL    @10,OUCBTMS(,OUCBPTR)                            02S0289
         ST    @10,OUXBTRT(,OUXBPTR)                            02S0289
*          IF RMCTMFA = '1'B THEN   /* WKLOD ACTIVITY BEING MEASURED */
         TM    RMCTMFA(RMCTPTR),B'10000000'                     02S0290
         BNO   @RF00290                                         02S0290
*            /* INVOKE MEASUREMENT ROUTINE TO COLLECT XACTN OUT TIME */
*            DO;                                                02S0291
*            RRPAWAR = '1'B;        /* INSERT WKLD ACTIVTY RTNE FLAG */
         OI    RRPAWAR(RRPAPTR),B'01000000'                     02S0292
*            /* ? IRACTLCL EPN(WR4) */                          02S0293
*            CALL RMSBWR4 -> IRARMRTN ;         /* INVOKE WR4 SUBRTN */
         L     @15,RMSBWR4(,RMCTPTR)                            02S0293
         BALR  @14,@15                                          02S0293
*            RRPAWAR = '0'B;        /* DELETE WKLD ACTIVTY RTNE FLAG */
         NI    RRPAWAR(RRPAPTR),B'10111111'                     02S0294
*            END;                                               02S0295
*          END;                                                 02S0296
@RF00290 DS    0H                                               02S0297
*        OUCBTMP = OUCBTMS;         /* RESET PERIOD START TIME FIELD */
@RC00286 L     @10,OUCBTMS(,OUCBPTR)                            02S0297
         ST    @10,OUCBTMP(,OUCBPTR)                            02S0297
*        OUCBTMO = OUCBTMS;         /* RESET TRANSACN BEGINNING TIME */
         ST    @10,OUCBTMO(,OUCBPTR)                            02S0298
*        CALL IRARMWMH;             /* RESET INTERVAL BEG MEASURMNTS */
         BAL   @14,IRARMWMH                                     02S0299
*        CALL IRARMWMJ;             /* SET APPLICABLE ALGORITHM INDS */
         BAL   @14,IRARMWMJ                                     02S0300
*        END;                                                   02S0301
*      ELSE                         /* USER IS CURRENTLY SWAPPED OUT */
*        /* IRARMWMN ROUTINE WILL BE REINVOKED WHEN USER IS SWAPD IN */
*        DO;                                                    02S0302
         B     @RC00265                                         02S0302
@RF00265 DS    0H                                               02S0303
*        IF OUCBATR = '1'B THEN     /* PREVIOUS XACTN IN EXISTENCE   */
         TM    OUCBATR(OUCBPTR),B'10000000'                     02S0303
         BNO   @RF00303                                         02S0303
*          /* INVOKE MEASUREMENT ROUTINE TO COLLECT XACTN END TIME   */
*          DO;                                                  02S0304
*          IF RMCTMFA = '1'B &      /* WKLD ACTIVITY BEING MEASURD   */
*                OUCBNTR = '0'B THEN /* TRANSACN STOP NOT COLLECTD   */
         TM    RMCTMFA(RMCTPTR),B'10000000'                     02S0305
         BNO   @RF00305                                         02S0305
         TM    OUCBNTR(OUCBPTR),B'00100000'                     02S0305
         BNZ   @RF00305                                         02S0305
*            /* UPDATE WORKLOAD ACTIVITY TRANSACTION MEASUREMENTS    */
*            DO;                                                02S0306
*            RRPAWAR = '1'B;        /* INSERT WKLD ACTIVTY RTNE FLAG */
         OI    RRPAWAR(RRPAPTR),B'01000000'                     02S0307
*            /* ? IRACTLCL EPN(WR6) */                          02S0308
*            CALL RMSBWR6 -> IRARMRTN ;         /* INVOKE WR6 SUBRTN */
         L     @15,RMSBWR6(,RMCTPTR)                            02S0308
         BALR  @14,@15                                          02S0308
*            IF OUCBINC = '1'B & OUCBRDY = '1'B THEN /* NOT SUSPEND
*                                                            @YM05631*/
         TM    OUCBINC(OUCBPTR),B'00000001'                     02S0309
         BNO   @RF00309                                         02S0309
         TM    OUCBRDY(OUCBPTR),B'10000000'                     02S0309
         BNO   @RF00309                                         02S0309
*              /* UPDATE WORKLOAD ACTIVITY PERIOD MEASUREMENTS       */
*              DO;                                              02S0310
*              /* ? IRACTLCL EPN(WR4) */                        02S0311
*              CALL RMSBWR4 -> IRARMRTN ;       /* INVOKE WR4 SUBRTN */
         L     @15,RMSBWR4(,RMCTPTR)                            02S0311
         BALR  @14,@15                                          02S0311
*              OUCBWMG = WPGDPTR-WMSTPGDT;/* OFFSET TO NEW PERF 02S0312
*                                       GRP DESCRIPTOR       @ZA20350*/
         LR    @10,WPGDPTR                                      02S0312
         SL    @10,WMSTPGDT(,WMSTPTR)                           02S0312
         STH   @10,OUCBWMG(,OUCBPTR)                            02S0312
*              OUCBPGP = WPGPOFST;  /* PERF GRP OFFSET TO 1ST PERIOD */
         MVI   OUCBPGP(OUCBPTR),X'0C'                           02S0313
*              OUCBMAR = '1'B;      /* SPECFY NEGATIV PERIOD ACCUMUL */
         OI    OUCBMAR(OUCBPTR),B'00000100'                     02S0314
*              /* ? IRACTLCL EPN(WR4) */                        02S0315
*              CALL RMSBWR4 -> IRARMRTN ;       /* INVOKE WR4 SUBRTN */
         L     @15,RMSBWR4(,RMCTPTR)                            02S0315
         BALR  @14,@15                                          02S0315
*              OUCBMAR = '0'B;      /* RESET NEGATIVE PERIOD ACCUMUL */
         NI    OUCBMAR(OUCBPTR),B'11111011'                     02S0316
*              END;                                             02S0317
*            RRPAWAR = '0'B;        /* DELETE WKLD ACTIVTY RTNE FLAG */
@RF00309 NI    RRPAWAR(RRPAPTR),B'10111111'                     02S0318
*            END;                                               02S0319
*          OUCBNTR = '1'B;          /* INDICATE XACTN STOP COLLECTED */
@RF00305 OI    OUCBNTR(OUCBPTR),B'00100000'                     02S0320
*          END;                     /* END COLLECTING XACN      02S0321
*                                      MEASUREMENTS          @ZA20350*/
*          OUCBWMG = WPGDPTR-WMSTPGDT;/* OFFST TO NEW PERF GRP  02S0322
*                                         DESCRIPTOR         @ZA20350*/
@RF00303 LR    @10,WPGDPTR                                      02S0322
         SL    @10,WMSTPGDT(,WMSTPTR)                           02S0322
         STH   @10,OUCBWMG(,OUCBPTR)                            02S0322
*          OUCBPGP = WPGPOFST;      /* PERF GRP OFFSET TO 1ST PERIOD */
         LA    @10,12                                           02S0323
         STC   @10,OUCBPGP(,OUCBPTR)                            02S0323
*          WPGPPTR = WPGDPTR + OUCBPGP ; /* PICK USER PERF GROUP PERI*/
         ALR   @10,WPGDPTR                                      02S0324
         LR    WPGPPTR,@10                                      02S0324
*          OUCBDMN = WPGPDMN ;    /* PERF GROUP PERIOD DOMAIN        */
         SLR   @10,@10                                          02S0325
         IC    @10,WPGPDMN(,WPGPPTR)                            02S0325
         STC   @10,OUCBDMN(,OUCBPTR)                            02S0325
*          OUCBDMO = WDMVTENT ;     /* OFFSET INTO THE DOMAIN TABLE  */
         ALR   @10,@10                                          02S0326
         AL    @10,WMSTDMVT(,WMSTPTR)                           02S0326
         LH    @10,WDMVTENT(,@10)                               02S0326
         STH   @10,OUCBDMO(,OUCBPTR)                            02S0326
*        END;                                                   02S0327
*      END;                                                     02S0328
@RC00265 DS    0H                                               02S0329
*    RFY (WKRE) RSTD;                                           02S0329
@RF00262 DS    0H                                               02S0330
*  /* ? IRACTLEP ENN(WMN) */                                    02S0330
*  GEN CODE EXIT NOSEQFLOW;                                     02S0330
*                                  /*
         LM    @00,@14,000+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0331
*/* TRANSACTION STOP ROUTINE                                         */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMO                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER TRANSACTION STOP RTNE     */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMO ROUTINE RECEIVES CONTROL AS THE       */
*/*          RESULT OF A SYSEVENT THAT HAS BEEN SPECIFIED BY THE     */
*/*          WORKLOAD MANAGER AS DEFINING THE END OF ANY CURRENT     */
*/*          USER TRANSACTION.  IF A NEW TRANSACTION WILL NEXT       */
*/*          BE CREATED FOR THE USER, IRARMWMO INDICATES THE END     */
*/*          OF THE CURRENT TRANSACTION.  IF THE NEXT USER EVENT     */
*/*          IS NOT KNOWN, IRARMWMO LEAVES THE TRANSACTION ACCUM     */
*/*          VALUES FOR LATER POSSIBLE RESUMPTION OF THE XACTN.      */
*/*          IN ANY CASE, IRARMWMO CAUSES THE PRECEDING TIME AND     */
*/*          SERVICE TO BE PROPERLY RECORDED TO THE CURR XACTN.      */
*/*                                                                  */
*/*    CALLED BY - IRARME08 - USER JOB SELECT SYSEVENT ROUTINE       */
*/*                IRARME09 - USER JOB DELETE SYSEVENT ROUTINE       */
*/*                IRARME11 - INITIATOR DETCH SYSEVENT ROUTINE       */
*/*                IRARMWMN - USER TRANSACTION STARTUP ROUTINE       */
*/*                IRARMWMQ - USER TRANSACTION SUSPEND ROUTINE       */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHOSE TRANSACTION IS TO BE STOPPED.                   */
*/*          - REG 5 CONTAINS THE USER ASCB ADDRESS.                 */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWMO RTNE ADDRESS.            */
*/*          - OUCBATR IS SET IF HAVE A XACTN CURR FOR THE USER.     */
*/*          - OUCBSTR IS SET IF A NEW XACTN WILL NEXT BE STARTED.   */
*/*          - OUCBNTR IS SET IF THE OLD XACTN IS TO BE STOPPED.     */
*/*          - OUCBTMS IS SET TO THE BEG TIME OF THE CURR NTRVL.     */
*/*          - OUCBSWC INDICATES IF XACTN HAD ONLY ONE SWAP-IN.      */
*/*                                                                  */
*/*    OUTPT - IRARMWMO INVOKES IRARMWMI, WHICH UPDATES THE USER     */
*/*            TRANSACTION ACCUMULATOR FIELDS.                       */
*/*          - OUCBATR IS SET OFF IF CURR XACTN WAS ENDED.           */
*/*          - OUCBRTR SET ON IF CURR XACTN WAS SUSPENDED.           */
*/*          - OUCBSWC IS UPDATED IF CURR XACTN SUSPENDED.           */
*/*          - OUCBTMW IS ZEROED  IF CURR XACTN WAS ENDED.           */
*/*          - OUXBTRC IS UPDATED IF CURR XACTN WAS ENDED.           */
*/*          - OUXBJBS IS UPDATED IF CURR XACTN WAS ENDED.           */
*/*          - OUXBJBT IS UPDATED IF CURR XACTN WAS ENDED.           */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WMO) */                                    02S0331
*  DCL IRARMWMO ENTRY INT;                                      02S0331
*  GEN CODE DEFS(IRARMWMO) NOEXIT SEQFLOW;                      02S0332
*                                  /*
         ENTRY IRARMWMO
IRARMWMO DS    0H
         STM   @00,@14,128+000(@13)    SAVE INVOKING RTNE REGS
         LA    @12,IRARMWMO-@PSTART    DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*IRARMWME: /* POINT AT WHICH TERMINATE OR SUSPEND CURRENT TRANSACTN  */
*    WMSTPTR = RMCTWMST;            /* PICK WLM SPECIFICATION TABLE  */
IRARMWME L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0333
*    IF OUCBATR = '1'B THEN         /* TRANSACTION CURRENTLY ACTIVE  */
         TM    OUCBATR(OUCBPTR),B'10000000'                     02S0334
         BNO   @RF00334                                         02S0334
*      /* DETERMINE WHETHER TRANSACTION HAS BEEN ENDED OR SUSPENDED  */
*      DO;                                                      02S0335
*      OUXBPTR = ASCBOUXB;          /* PICK USER CTRL BLOCK EXTENSN  */
         L     OUXBPTR,ASCBOUXB(,ASCBPTR)                       02S0336
*      IF OUCBSTR = '1'B |          /* NEW XACTN TO BE STARTED  02S0337
*                                                            @YM05631*/
*            OUCBNTR = '1'B |       /* OLD XACTN TO BE STOPPED       */
*            (OUCBLOG = '1'B &      /* TSO USER TRANSACTION          */
*            (RRPACOD = INITDET |   /* STEP DETACH SYSEVENT          */
*            (OUCBTRM = '1'B & OUCBCIM = '0'B))) THEN /* TERMNL WAIT */
         TM    OUCBSTR(OUCBPTR),B'01100000'                     02S0337
         BNZ   @RT00337                                         02S0337
         TM    OUCBLOG(OUCBPTR),B'00100000'                     02S0337
         BNO   @RF00337                                         02S0337
         CLI   RRPACOD(RRPAPTR),11                              02S0337
         BE    @RT00337                                         02S0337
         TM    OUCBTRM(OUCBPTR),B'01000000'                     02S0337
         BNO   @RF00337                                         02S0337
         TM    OUCBCIM(OUCBPTR),B'00010000'                     02S0337
         BNZ   @RF00337                                         02S0337
@RT00337 DS    0H                                               02S0338
*        /* ACCUMULATE TRANSACTION STATISTICS TO USER SESSION/JOB    */
*        DO;                                                    02S0338
*        IF OUCBRTR = '0'B &        /* TRANSACTION WAS NOT SUSPENDED */
*              OUCBNTR = '0'B THEN  /* XACTN PERFORM GROUP NOT RESET */
         TM    OUCBRTR(OUCBPTR),B'00110000'                     02S0339
         BNZ   @RF00339                                         02S0339
*          CALL IRARMWMI;           /* UPDATE TRANSACTION STATISTICS */
         BAL   @14,IRARMWMI                                     02S0340
*        ELSE                       /* XACTN ENDED WHILE SWAPPED OUT */
*          IF OUCBSTR = '0'B THEN   /* NEW TRANSACTION NOT YET START */
         B     @RC00339                                         02S0341
@RF00339 TM    OUCBSTR(OUCBPTR),B'01000000'                     02S0341
         BNZ   @RF00341                                         02S0341
*            CALL IRARMWMH;         /* RESET TRANSACTN NTRVL MEASURM */
         BAL   @14,IRARMWMH                                     02S0342
*        IF OUCBNTR = '0'B &        /* TRANSACTN STOP NOT COLLECTED  */
*              RMCTMFA = '1'B THEN  /* WKLD ACTIVITY BEING MEASURED  */
@RF00341 DS    0H                                               02S0343
@RC00339 TM    OUCBNTR(OUCBPTR),B'00100000'                     02S0343
         BNZ   @RF00343                                         02S0343
         TM    RMCTMFA(RMCTPTR),B'10000000'                     02S0343
         BNO   @RF00343                                         02S0343
*          /* UPDATE WORKLOAD ACTIVITY TRANSACTION MEASUREMENTS      */
*          DO;                                                  02S0344
*          RRPAWAR = '1'B;          /* INSERT WKLD ACTIVTY RTNE FLAG */
         OI    RRPAWAR(RRPAPTR),B'01000000'                     02S0345
*          /* ? IRACTLCL EPN(WR6) */                            02S0346
*          CALL RMSBWR6 -> IRARMRTN ;           /* INVOKE WR6 SUBRTN */
         L     @15,RMSBWR6(,RMCTPTR)                            02S0346
         BALR  @14,@15                                          02S0346
*          RRPAWAR = '0'B;          /* DELETE WKLD ACTIVTY RTNE FLAG */
         NI    RRPAWAR(RRPAPTR),B'10111111'                     02S0347
*          END;                                                 02S0348
*        OUXBJBS = OUXBJBS + OUXBTRS; /* ADD TO SESSION XACTN SERV   */
@RF00343 L     @09,OUXBJBS(,OUXBPTR)                            02S0349
         AL    @09,OUXBTRS(,OUXBPTR)                            02S0349
         ST    @09,OUXBJBS(,OUXBPTR)                            02S0349
*        OUXBJBT = OUXBJBT + OUXBTRT; /* ADD TO SESSION XACTN TIME   */
         L     @09,OUXBJBT(,OUXBPTR)                            02S0350
         AL    @09,OUXBTRT(,OUXBPTR)                            02S0350
         ST    @09,OUXBJBT(,OUXBPTR)                            02S0350
*        OUXBJBR = OUXBJBR + OUXBTRR; /* ADD TO SESSION RESID TIME   */
         L     @09,OUXBJBR(,OUXBPTR)                            02S0351
         AL    @09,OUXBTRR(,OUXBPTR)                            02S0351
         ST    @09,OUXBJBR(,OUXBPTR)                            02S0351
*        OUCBATR = '0'B;            /* NO LONGER HAVE ACTIVE XACTN   */
*        OUCBSTR = '0'B;            /* SET OFF XACTN START REQUEST   */
*        OUCBNTR = '0'B;            /* SET OFF XACTN END INDICATOR   */
*        OUCBRTR = '0'B;            /* SET OFF XACTN SUSPENSE FLAG   */
*        OUCBPCH = '0'B;            /* SET OFF PER CHG OUTSTD FLAG   */
         NI    OUCBATR(OUCBPTR),B'00000111'                     02S0356
*        OUCBTMW = ZERO;            /* RESET WLM INTERVAL BEG TIME   */
         SLR   @09,@09                                          02S0357
         ST    @09,OUCBTMW(,OUCBPTR)                            02S0357
*        OUCBWMS = ZERO;            /* RESET WLM INTERVL CURR SRVC   */
         ST    @09,OUCBWMS(,OUCBPTR)                            02S0358
*        OUCBCPU = ZERO ;           /* RESET CPU SERVICE             */
         ST    @09,OUCBCPU(,OUCBPTR)                            02S0359
*        OUCBIOC = ZERO ;           /* RESET IO SERVICE              */
         ST    @09,OUCBIOC(,OUCBPTR)                            02S0360
*        OUCBMSO = ZERO ;           /* RESET MSO SERVICE             */
         ST    @09,OUCBMSO(,OUCBPTR)                            02S0361
*        OUCBWMR = ZERO;            /* RESET WLM CURR NORMLZD LEVL   */
         ST    @09,OUCBWMR(,OUCBPTR)                            02S0362
*        OUCBSWC = ZERO;            /* RESET TRANSACTN SWAPOUT CNT   */
         STH   @09,OUCBSWC(,OUCBPTR)                            02S0363
*        OUXBTRS = ZERO;            /* RESET XACTN SERV ACCUMULATR   */
         ST    @09,OUXBTRS(,OUXBPTR)                            02S0364
*        OUXBTRT = ZERO;            /* RESET XACTN TIME ACCUMULATR   */
         ST    @09,OUXBTRT(,OUXBPTR)                            02S0365
*        OUXBTRR = ZERO;            /* RESET RESID TIME ACCUMULATR   */
         ST    @09,OUXBTRR(,OUXBPTR)                            02S0366
*        OUCBPGP = WPGPOFST;        /* PERF GRP OFST TO 1ST PERIOD   */
         LA    @09,12                                           02S0367
         STC   @09,OUCBPGP(,OUCBPTR)                            02S0367
*        WPGPPTR = WMSTPGDT + OUCBWMG+OUCBPGP ;/*PERF GROUP DESC */
         LH    WPGPPTR,OUCBWMG(,OUCBPTR)                        02S0368
         AL    WPGPPTR,WMSTPGDT(,WMSTPTR)                       02S0368
         ALR   WPGPPTR,@09                                      02S0368
*        R1=WMSTPODT+WPOVTENT; /* USER PERF OBJ */              02S0369
         SLR   @09,@09                                          02S0369
         IC    @09,WPGPOBJN(,WPGPPTR)                           02S0369
         ALR   @09,@09                                          02S0369
         AL    @09,WMSTPOVT(,WMSTPTR)                           02S0369
         LH    @09,WPOVTENT(,@09)                               02S0369
         AL    @09,WMSTPODT(,WMSTPTR)                           02S0369
         LR    R1,@09                                           02S0369
*        R0 = ZERO ; /* INDICATED ZERO SERVICE */               02S0370
         SLR   R0,R0                                            02S0370
*        WKRO = RTNREG ; /* SAVE RETURN REG */                  02S0371
         LR    WKRO,RTNREG                                      02S0371
*        /* CALC USER NORMALIZED WORK LEVEL */                  02S0372
*        /* ? IRACTLCL EPN(WM4) */                              02S0372
*        CALL RMSBWM4 -> IRARMRTN ;             /* INVOKE WM4 SUBRTN */
         L     @15,RMSBWM4(,RMCTPTR)                            02S0372
         BALR  @14,@15                                          02S0372
*        OUCBWMR = R0 ; /* SAVE WORK LOAD LEVEL */              02S0373
         ST    R0,OUCBWMR(,OUCBPTR)                             02S0373
*        OUCBTMA = RRPATOD ; /* INDICATED USER ANALYZED */      02S0374
         L     @09,RRPATOD(,RRPAPTR)                            02S0374
         ST    @09,OUCBTMA(,OUCBPTR)                            02S0374
*        RTNREG = WKRO ; /* RESTORE RETURN REG CONTENTS */      02S0375
         LR    RTNREG,WKRO                                      02S0375
*        END;                                                   02S0376
*      ELSE                         /* SAME XACN COULD RESUME LATER  */
*        /* INSURE TRANSACTION STATISTICS ARE UP TO DATE             */
*        DO;                                                    02S0377
         B     @RC00337                                         02S0377
@RF00337 DS    0H                                               02S0378
*        IF RRPACOD = QSCECMP THEN  /* QUIESCE COMPLETE FOR SWAPOUT  */
         CLI   RRPACOD(RRPAPTR),13                              02S0378
         BNE   @RF00378                                         02S0378
*          OUCBSWC = OUCBSWC + ONE; /* INCREMENT XACTN SWAP-OUT CNT  */
         LH    @09,OUCBSWC(,OUCBPTR)                            02S0379
         AL    @09,ONE                                          02S0379
         STH   @09,OUCBSWC(,OUCBPTR)                            02S0379
*        CALL IRARMWMI;             /* UPDAT TRANSACTION STATISTICS  */
@RF00378 BAL   @14,IRARMWMI                                     02S0380
*        CALL IRARMWM5;             /* CHECK TRANSACTION PERIOD END  */
         LA    @15,IRARMWM5                                     02S0381
         BALR  @14,@15                                          02S0381
*        CALL IRARMWM7;             /* OBTAIN USER NORMALIZED LEVEL  */
         BAL   @14,IRARMWM7                                     02S0382
*        OUCBRTR = '1'B;            /* INDICAT CURR XACTN SUSPENDED  */
         OI    OUCBRTR(OUCBPTR),B'00010000'                     02S0383
*        IF RRPACOD = INITDET THEN  /* INITIATOR DETACH OF JOB STEP
*                                                            @YM05631*/
         CLI   RRPACOD(RRPAPTR),11                              02S0384
         BNE   @RF00384                                         02S0384
*          OUCBTMW = ZERO;          /* RESET WLM INTERVAL INDICATOR
*                                                            @YM05631*/
         SLR   @09,@09                                          02S0385
         ST    @09,OUCBTMW(,OUCBPTR)                            02S0385
*        END;                                                   02S0386
@RF00384 DS    0H                                               02S0387
*      CALL IRARMWMJ;               /* SET APPLICABLE ALGORITHM INDS */
@RC00337 BAL   @14,IRARMWMJ                                     02S0387
*      END;                                                     02S0388
*  /* ? IRACTLEP ENN(WMO) */                                    02S0389
*  GEN CODE EXIT NOSEQFLOW;                                     02S0389
@RF00334 DS    0H                                               02S0389
*                                  /*
         LM    @00,@14,128+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0390
*/* QSCECMP SYSEVENT WORKLOAD ROUTINE                                */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMQ                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER QUIESCE COMPLETED         */
*/*                       SYSEVENT PROCESSING ROUTINE                */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMQ ROUTINE RECEIVES CONTROL AT THE       */
*/*          TIME THAT AN USER HAS STOPPED EXECUTING, AND IS         */
*/*          BEING SWAPPED OUT, SO THAT THE WORKLOAD MANAGER         */
*/*          MAY RECORD THE SERVICE GIVEN THAT USER WHILE HE         */
*/*          WAS IN CORE.  THE WORKLOAD MANAGER DETERMINES IF        */
*/*          AN USER EVENT CAUSED THE SWAP-OUT, AND FLAGS THE        */
*/*          USER TO INDICATE WHETHER SUCH PREVIOUS SERVICE IS       */
*/*          TO BE CONSIDERED WHEN THE USER IS NEXT SWAPPED-IN.      */
*/*                                                                  */
*/*    CALLED BY - IRARME13 - QUIESCE COMPLET SYSEVENT ROUTINE       */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHO IS BEING SWAPPED OUT.                             */
*/*          - REG 5 CONTAINS THE USER ASCB ADDRESS.                 */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWMQ RTNE ADDRESS.            */
*/*          - OUCBRDY IS ZERO IF THE USER ENTERED WAIT STATUS.      */
*/*                                                                  */
*/*    OUTPT - DEPENDING ON USER STATUS, IRARMWMQ ROUTES CONTROL     */
*/*            TO THE IRARMWMO, IRARMWMI, IRARMWM5 ROUTINES, WHICH   */
*/*            PERFORM THE ACTUAL UPDATING FOR THAT USER.            */
*/*          - OUCBSWC IS UPDATED TO SHOW XACTN WAS SWAPPED.         */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WMQ) */                                    02S0390
*  DCL IRARMWMQ ENTRY INT;                                      02S0390
*  GEN CODE DEFS(IRARMWMQ) NOEXIT SEQFLOW;                      02S0391
*                                  /*
         ENTRY IRARMWMQ
IRARMWMQ DS    0H
         STM   @00,@14,128+000(@13)    SAVE INVOKING RTNE REGS
         LA    @12,IRARMWMQ-@PSTART    DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*    IF OUCBRDY = '0'B | OUCBATR = '0'B THEN /* LONG- OR TERM- WAIT  */
         TM    OUCBRDY(OUCBPTR),B'10000000'                     02S0392
         BZ    @RT00392                                         02S0392
         TM    OUCBATR(OUCBPTR),B'10000000'                     02S0392
         BZ    @RT00392                                         02S0392
*      /* 'XCTL' TO WMO SBRTNE WHICH USES SAME REG SAVE AREA AS WMQ  */
*      GOTO IRARMWME;               /* DISPOSE OF XACN USER GAVE UP  */
*    ELSE                           /* SWAPOUT CAUSED BY SYSTM ACTN  */
*      /* RECORD SERVICE PROVIDED USER WHILE IN-CORE                 */
*      DO;                                                      02S0394
*      OUXBPTR = ASCBOUXB;          /* PICK USER CTRL BLOCK EXTENSN  */
         L     OUXBPTR,ASCBOUXB(,ASCBPTR)                       02S0395
*      OUCBSWC = OUCBSWC + ONE;     /* INCREMENT XACTN SWAP-OUT CNT  */
         LH    @09,OUCBSWC(,OUCBPTR)                            02S0396
         AL    @09,ONE                                          02S0396
         STH   @09,OUCBSWC(,OUCBPTR)                            02S0396
*      CALL IRARMWMI;               /* UPDAT TRANSACTION STATISTICS  */
         BAL   @14,IRARMWMI                                     02S0397
*      CALL IRARMWM5;               /* CHECK TRANSACTION PERIOD END  */
         LA    @15,IRARMWM5                                     02S0398
         BALR  @14,@15                                          02S0398
*      CALL IRARMWM7;               /* OBTAIN USER NORMALIZED LEVEL  */
         BAL   @14,IRARMWM7                                     02S0399
*      END;                                                     02S0400
*  /* ? IRACTLEP ENN(WMQ) */                                    02S0401
*  GEN CODE EXIT NOSEQFLOW;                                     02S0401
*                                  /*
         LM    @00,@14,128+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0402
*/* RSTORCMP SYSEVENT WORKLOAD ROUTINE                               */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMR                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER RESTORE COMPLETED         */
*/*                       SYSEVENT PROCESSING ROUTINE                */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMR ROUTINE RECEIVES CONTROL AT THE       */
*/*          TIME THAT AN USER HAS BEEN SWAPPED IN, AND IS READY     */
*/*          TO BEGIN EXECUTING, SO THAT THE WORKLOAD MANAGER        */
*/*          CAN SET UP THE FIELDS USED TO CALCULATE THE SERVICE     */
*/*          RATE RECEIVED BY THE USER DURING THE FORTHCOMING        */
*/*          IN-CORE RESIDENCY PERIOD.                               */
*/*                                                                  */
*/*    CALLED BY - IRARME10 - INITIATOR ATTCH SYSEVENT ROUTINE       */
*/*                IRARME19 - RESTORE COMPLET SYSEVENT ROUTINE       */
*/*                                                                  */
*/*    NOTE  - IRARMWMR MAY BE CALLED MULTIPLE TIMES DUE TO THE      */
*/*            RESTORE COMPLETE SYSEVENT BEING ISSUED MORE THAN      */
*/*            ONCE DURING RECOVERY FROM A FAILURE IN QUIESCE.       */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHOSE SWAP-IN IS NOW COMPLETE.                        */
*/*          - REG 5 CONTAINS THE USER ASCB ADDRESS.                 */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWMR RTNE ADDRESS.            */
*/*          - OUCBSTR IS SET IF A NEW TRANSACTION IS TO BE STARTED  */
*/*            FOR THIS USER.                                        */
*/*          - OUCBRTR IS SET IF THE OUT-OF-CORE PERIOD IS NOT TO    */
*/*            BE COUNTED TOWARDS THE CURRENT TRANSACTION.           */
*/*          - OUCBTMS IS SET TO THE BEG TIME WHEN XACTN AWAKE.      */
*/*                                                                  */
*/*    OUTPT - DEPENDING ON USER STATUS, IRARMWMR ROUTES CONTROL     */
*/*            TO THE IRARMWMN, IRARMWM5, IRARMWMH ROUTINES, WHICH   */
*/*            PERFORM THE UPDATING OF THE USER.                     */
*/*          - OUCBWMS IS RESET TO SHOW NO SERVICE THIS INTERVAL.    */
*/*          - OUCBTMS IS RESET TO SHOW THE INTERVAL START TIME.     */
*/*          - OUXBTRT IS UPDATED WITH THE XACTN OUT-OF-CORE TIME.   */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*  /* ? IRACTLEP EPN(WMR) */                                    02S0402
*  DCL IRARMWMR ENTRY INT;                                      02S0402
*  GEN CODE DEFS(IRARMWMR) NOEXIT SEQFLOW;                      02S0403
*                                  /*
         ENTRY IRARMWMR
IRARMWMR DS    0H
         STM   @00,@14,000+000(@13)    SAVE INVOKING RTNE REGS
         LA    @12,IRARMWMR-@PSTART    DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*    OUCBWMS = ZERO;                /* RESET INTERVAL SERVICE ACCUM  */
         SLR   @09,@09                                          02S0404
         ST    @09,OUCBWMS(,OUCBPTR)                            02S0404
*    OUCBCPU = ZERO ;                /* RESET CPU SERVICE VALUE      */
         ST    @09,OUCBCPU(,OUCBPTR)                            02S0405
*    OUCBIOC = ZERO ;                /* RESET IO SERVICE VALUE       */
         ST    @09,OUCBIOC(,OUCBPTR)                            02S0406
*    OUCBMSO = ZERO ;                /* RESET MSO SERVICE VALUE      */
         ST    @09,OUCBMSO(,OUCBPTR)                            02S0407
*    IF OUCBSTR = '1'B | OUCBATR = '0'B THEN /* XACTN START PENDING  */
         TM    OUCBSTR(OUCBPTR),B'01000000'                     02S0408
         BNZ   @RT00408                                         02S0408
         TM    OUCBATR(OUCBPTR),B'10000000'                     02S0408
         BNO   @RT00408                                         02S0408
*      /* 'XCTL' TO WMN SBRTNE WHICH USES SAME REG SAVE AREA AS WMR  */
*      GOTO IRARMWMB;               /* INITIATE A NEW XACTN FOR USR  */
*    ELSE                           /* CURR TRANSACTN WILL CONTINUE  */
*      /* SETUP USER'S WORKLOAD MNGR FIELDS FOR NEW IN-CORE INTERVAL */
*      DO;                                                      02S0410
*      OUXBPTR = ASCBOUXB;          /* OBTAIN OUXB ADDRESS   @ZA09481*/
         L     OUXBPTR,ASCBOUXB(,ASCBPTR)                       02S0411
*      OUCBRTR = '0'B;              /* RESET XACTN RESUME REQST FLAG */
         NI    OUCBRTR(OUCBPTR),B'11101111'                     02S0412
*      IF OUCBPCH = '1'B THEN       /* PERF PERIOD CHANGED WHILE OUT */
         TM    OUCBPCH(OUCBPTR),B'00001000'                     02S0413
         BNO   @RF00413                                         02S0413
*        /* UPDATE BEGINNING SERVICE VALUE FOR NEW PERF GROUP PERIOD */
*        DO;                                                    02S0414
*        OUCBPCH = '0'B;            /* RESET PERIOD CHNG OUTSTND FLG */
         NI    OUCBPCH(OUCBPTR),B'11110111'                     02S0415
*        OUXBPRS = OUXBTRS;         /* SAVE NEW PERIOD START SERVICE */
         L     @09,OUXBTRS(,OUXBPTR)                            02S0416
         ST    @09,OUXBPRS(,OUXBPTR)                            02S0416
*        END;                                                   02S0417
*      IF RRPACOD ^= INITATT THEN   /* IGNORE TIME PRIOR TO ATTACH   */
@RF00413 CLI   RRPACOD(RRPAPTR),10                              02S0418
         BE    @RF00418                                         02S0418
*        /* RECORD THE AMOUNT OF TIME SPENT OUT OF CORE BY TRANSACTN */
*        DO;                                                    02S0419
*        OUXBTRT = OUXBTRT + RRPATOD - OUCBTMS; /* ADD OUTCORE TIME  */
         L     @09,OUXBTRT(,OUXBPTR)                            02S0420
         AL    @09,RRPATOD(,RRPAPTR)                            02S0420
         SL    @09,OUCBTMS(,OUCBPTR)                            02S0420
         ST    @09,OUXBTRT(,OUXBPTR)                            02S0420
*        IF RMCTMFA = '1'B THEN     /* WKLD ACTIVITY BEING MEASURED  */
         TM    RMCTMFA(RMCTPTR),B'10000000'                     02S0421
         BNO   @RF00421                                         02S0421
*          /* INVOKE MEASUREMENT RTNE TO COLLECT XACTN OUTCORE TIME  */
*          DO;                                                  02S0422
*          RRPAWAR = '1'B;          /* INSERT WKLD ACTIVTY RTNE FLAG */
         OI    RRPAWAR(RRPAPTR),B'01000000'                     02S0423
*          /* ? IRACTLCL EPN(WR4) */                            02S0424
*          CALL RMSBWR4 -> IRARMRTN ;           /* INVOKE WR4 SUBRTN */
         L     @15,RMSBWR4(,RMCTPTR)                            02S0424
         BALR  @14,@15                                          02S0424
*          RRPAWAR = '0'B;          /* DELETE WKLD ACTIVTY RTNE FLAG */
         NI    RRPAWAR(RRPAPTR),B'10111111'                     02S0425
*          END;                                                 02S0426
*        END;                                                   02S0427
@RF00421 DS    0H                                               02S0428
*      CALL IRARMWMH;               /* RESET INTERVL STRT MEASURMNTS */
@RF00418 BAL   @14,IRARMWMH                                     02S0428
*      CALL IRARMWM5;               /* CHECK FOR PERF PERIOD CHANGE  */
         LA    @15,IRARMWM5                                     02S0429
         BALR  @14,@15                                          02S0429
*      CALL IRARMWMJ;               /* SET APPLICABLE ALGORITHM INDS */
         BAL   @14,IRARMWMJ                                     02S0430
*      END;                                                     02S0431
*  /* ? IRACTLEP ENN(WMR) */                                    02S0432
*  GEN CODE EXIT NOSEQFLOW;                                     02S0432
*                                  /*
         LM    @00,@14,000+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0433
*/* IN-CORE INTERVAL CHANGE SUBROUTINE                               */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMI                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER IN-CORE INTERVAL          */
*/*                       CHANGE SUBROUTINE                          */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMI SUBROUTINE UPDATES THE TRANSACTION    */
*/*          ACCUMULATORS WITH THE SERVICE AND THE TIME RECEIVED     */
*/*          BY THE USER DURING THE PRECEDING IN-CORE INTERVAL.      */
*/*                                                                  */
*/*    CALLED BY - IRARMWMO - USER TRANSACTION COMPLET ROUTINE       */
*/*                IRARMWMQ - USER TRANSACTION QUIESCE ROUTINE       */
*/*                                                                  */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMH                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WORKLOAD MANAGER MEASUREMENT INTERVAL      */
*/*                       RESET SUBROUTINE                           */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMH SUBROUTINE RESETS THE WORKLOAD        */
*/*          MANAGER MEASUREMENT INTERVAL START VALUE FIELDS TO      */
*/*          BEGIN A NEW WORKLOAD MANAGER MEASUREMENT INTERVAL.      */
*/*                                                                  */
*/*    CALLED BY - IRARMWMN - USER TRANSACTION STARTUP ROUTINE       */
*/*                IRARMWMR - USER TRANSACTION RESUMPT ROUTINE       */
*/*                                                                  */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE USER   */
*/*            WHOSE MEASUREMENT INTERVAL IS TO BE CHANGED/RESET.    */
*/*          - REG 5 CONTAINS THE USER ASCB BLOCK ADDRESS.           */
*/*          - REG 6 CONTAINS THE USER OUXB BLOCK ADDRESS.           */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - OUCBTMS CONTAINS THE BEG TIME OF THE CURR NTRVL.      */
*/*                                                                  */
*/*    OUTPT - SEE FUNCTION.                                         */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*IRARMWMI: PROC OPTIONS(NOSAVE,NOSAVEAREA,NOENTREG,RETREG,NOPARMREG);
IRARMWMI DS    0H                                               02S0434
*    PMRO = RTNREG;                 /* SAVE RETURN REG CONTENTS      */
         LR    PMRO,RTNREG                                      02S0434
*    /* ? IRACTLCL EPN(WM1) */                                  02S0435
*    CALL RMSBWM1 -> IRARMRTN ;                 /* INVOKE WM1 SUBRTN */
         L     @15,RMSBWM1(,RMCTPTR)                            02S0435
         BALR  @14,@15                                          02S0435
*    OUXBTRS = OUXBTRS + OUCBWMS;   /* ADD SERVICE TO TRANSACTION    */
         L     @09,OUXBTRS(,OUXBPTR)                            02S0436
         AL    @09,OUCBWMS(,OUCBPTR)                            02S0436
         ST    @09,OUXBTRS(,OUXBPTR)                            02S0436
*    OUXBTRT = OUXBTRT + (RRPATOD - OUCBTMS); /* ADD TIME TO XACN    */
         L     @09,RRPATOD(,RRPAPTR)                            02S0437
         SL    @09,OUCBTMS(,OUCBPTR)                            02S0437
         L     @08,OUXBTRT(,OUXBPTR)                            02S0437
         ALR   @08,@09                                          02S0437
         ST    @08,OUXBTRT(,OUXBPTR)                            02S0437
*    OUXBTRR = OUXBTRR + (RRPATOD - OUCBTMS); /* ADD RESIDNT TIME    */
         AL    @09,OUXBTRR(,OUXBPTR)                            02S0438
         ST    @09,OUXBTRR(,OUXBPTR)                            02S0438
*    IF RMCTMFA = '1'B THEN         /* WORKLOAD ACTIVITY MEASURED    */
         TM    RMCTMFA(RMCTPTR),B'10000000'                     02S0439
         BNO   @RF00439                                         02S0439
*      /* UPDATE WORKLOAD ACTIVITY MEASUREMENT REPORT STATISTICS     */
*      DO;                                                      02S0440
*      RRPAWAR = '1'B;              /* INSERT WKLD ACTIVTY RTNE FLAG */
         OI    RRPAWAR(RRPAPTR),B'01000000'                     02S0441
*      /* ? IRACTLCL EPN(WR4) */                                02S0442
*      CALL RMSBWR4 -> IRARMRTN ;               /* INVOKE WR4 SUBRTN */
         L     @15,RMSBWR4(,RMCTPTR)                            02S0442
         BALR  @14,@15                                          02S0442
*      RRPAWAR = '0'B;              /* DELETE WKLD ACTIVTY RTNE FLAG */
         NI    RRPAWAR(RRPAPTR),B'10111111'                     02S0443
*      END;                                                     02S0444
*    RTNREG = PMRO;                 /* RESTORE RETURN REG CONTENTS   */
@RF00439 LR    RTNREG,PMRO                                      02S0445
*/*    ENTRY WHEN DONT NEED TO ACCUM DATA, JUST START INTERVAL ANEW  */
*IRARMWMH: /* SUBROUTINE TO RESET WORKLOAD MANAGER INTERVAL          */
*      /* SAVE EXTRA CALL BY CODING AS ENTRY INSTEAD OF SEPARAT PROC */
*      ENTRY;                                                   02S0446
         B     @EC00446                                         02S0446
IRARMWMH DS    0H                                               02S0446
@EC00446 DS    0H                                               02S0447
*    WMSTPTR = RMCTWMST;            /* PICK WLM SPECIFICATION TABLE  */
         L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0447
*    WPGDPTR = WMSTPGDT + OUCBWMG;  /* PICK CURR PERF GRP DESCRIPTOR */
         LH    WPGDPTR,OUCBWMG(,OUCBPTR)                        02S0448
         AL    WPGDPTR,WMSTPGDT(,WMSTPTR)                       02S0448
*    IF OUCBPVL = '0'B THEN         /* USER SUBJECT TO WLM ANALYSIS  */
         TM    OUCBPVL(OUCBPTR),B'00000100'                     02S0449
         BNZ   @RF00449                                         02S0449
*      OUCBTMW = MAX(ONE,OUCBTMS);  /* INSERT NONZERO STARTING TIME  */
         L     @09,ONE                                          02S0450
         L     @08,OUCBTMS(,OUCBPTR)                            02S0450
         CLR   @09,@08                                          02S0450
         BNL   *+6
         LR    @09,@08                                          02S0450
         ST    @09,OUCBTMW(,OUCBPTR)                            02S0450
*    OUCBTMS = RRPATOD;             /* SAVE ACTUAL LAST ACTION TIME  */
@RF00449 L     @09,RRPATOD(,RRPAPTR)                            02S0451
         ST    @09,OUCBTMS(,OUCBPTR)                            02S0451
*    OUXBCPS = ASCBEJST;            /* RESET CPU SERVICE BASE VALUE  */
         MVC   OUXBCPS(8,OUXBPTR),ASCBEJST(ASCBPTR)             02S0452
*    OUXBIOS = ASCBIOSM;            /* RESET IOM SERVICE BASE VALUE  */
         LH    @09,ASCBIOSM(,ASCBPTR)                           02S0453
         STH   @09,OUXBIOS(,OUXBPTR)                            02S0453
*    OUXBIOSM = OUCBIOSM;  /*RESET SMF BASE EXCP COUNT       @ZA10808*/
         L     @09,OUCBIOSM(,OUCBPTR)                           02S0454
         ST    @09,OUXBIOSM(,OUXBPTR)                           02S0454
*    OUXBMSS = OUCBPSS;             /* RESET MSO BASE SERVICE VALUE  */
         MVC   OUXBMSS(8,OUXBPTR),OUCBPSS(OUCBPTR)              02S0455
*    END IRARMWMI;                                              02S0456
@EL00003 DS    0H                                               02S0456
@EF00003 DS    0H                                               02S0456
@ER00003 BR    @14                                              02S0456
*                                                               01S0457
*/* ALGORITHM APPLICABILITY DETERMINATION SUBROUTINE                 */
*/* **************************************************************** */
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMJ                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - ROUTINE TO DETERMINE THE SCOPE OF          */
*/*          APPLICABILITY OF ANALYSIS PROCESSING TO A USER          */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMJ ROUTINE EXAMINES THE CURRENT          */
*/*          SWAPPABILITY STATUS AND THE PERFORMANCE SPECI-          */
*/*          FICATION FOR AN USER.  IT INDICATES IF THE RESOURCE     */
*/*          MANAGER ALGORITHMS ARE APPLICABLE TO THIS USER.         */
*/*                                                                  */
*/*    CALLED BY - IRARMWMK - WLM DONTSWAP/OKSWAP SERVICE UPDT RTNE  */
*/*                IRARMWMN - USER TRANSACTION STARTUP ROUTINE       */
*/*                IRARMWMO - USER TRANSACTION SUSPEND ROUTINE       */
*/*                IRARMWMR - USER TRANSACTION RESUMPT ROUTINE       */
*/*                IRARMWM5 - PERFORMANCE PERIOD CHECK SBRTINE       */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THE ADDRESS OF THE OUCB FOR THE        */
*/*            USER WHOSE ANALYSIS APPLIC IS BEING DETERMINED.       */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWMK RTNE ADDRESS.            */
*/*          - OUCBNSW INDICATES IF THE USER IS NON-SWAPPABLE.       */
*/*          - OUCBNPG INDICATES THE PERF GROUP FOR THE USER.        */
*/*          - OUCBPGP INDICATES THE PERF PERIOD FOR THE USER.       */
*/*          - WPGDRTB INDICATES THE RESPONSE/THROUGHPUT BIAS.       */
*/*                                                                  */
*/*    OUTPT - OUCBRMA IS SET ON IF RESOURCE ALGORITHM RECOM-        */
*/*            MENDATIONS ARE TO BE SOLICITED FOR THIS USER.         */
*/*                                                                  */
*/*    EXIT  - BRANCH ON REGISTER 14.                                */
*/*          - THERE IS NO ERROR EXIT OR INDICATION.                 */
*/*                                                                  */
*/* **************************************************************** */
*IRARMWMJ: PROC OPTIONS(NOSAVE,NOSAVEAREA,NOENTREG,RETREG,NOPARMREG);
IRARMWMJ DS    0H                                               02S0458
*    RFY (WKRE) UNRSTD;                                         02S0458
*    STPR = RTNREG;                 /* SAVE RETURN REG CONTENTS      */
         LR    STPR,RTNREG                                      02S0459
*    /* NON-SWAPPABLE USERS ARE EXCLUDED FROM SWAP ANALYSIS          */
*    OUCBRMA = '0'B;                /* RESET RESOURCE ANALYSIS FLG   */
         NI    OUCBRMA(OUCBPTR),B'11011111'                     02S0460
*    IF OUCBNSW = '0'B THEN /* USER  WKLD CONTROLLED */         02S0461
         TM    OUCBNSW(OUCBPTR),B'10000000'                     02S0461
         BNZ   @RF00461                                         02S0461
*      DO;                                                      02S0462
*      WMSTPTR = RMCTWMST;          /* PICK WLM SPECIFICATION TABLE  */
         L     WMSTPTR,RMCTWMST(,RMCTPTR)                       02S0463
*      WPGDPTR = WMSTPGDT + WPGVTENT; /* PICK PERF GROUP DESCRIPTOR  */
         SLR   @10,@10                                          02S0464
         IC    @10,OUCBNPG(,OUCBPTR)                            02S0464
         ALR   @10,@10                                          02S0464
         AL    @10,WMSTPGVT(,WMSTPTR)                           02S0464
         LH    WPGDPTR,WPGVTENT(,@10)                           02S0464
         AL    WPGDPTR,WMSTPGDT(,WMSTPTR)                       02S0464
*      WPGPPTR = WPGDPTR + OUCBPGP; /* PICK CURR PERF GRP PERIOD   */
         LR    WPGPPTR,WPGDPTR                                  02S0465
         SLR   @10,@10                                          02S0465
         IC    @10,OUCBPGP(,OUCBPTR)                            02S0465
         ALR   WPGPPTR,@10                                      02S0465
*      IF WPGPRTB = '1'B THEN     /* THROUGHPUT BIAS INDICATED     */
         TM    WPGPRTB(WPGPPTR),B'00000001'                     02S0466
         BNO   @RF00466                                         02S0466
*        OUCBRMA = '1'B;          /* RESOURCE ALGORITHMS APPLIC    */
         OI    OUCBRMA(OUCBPTR),B'00100000'                     02S0467
*      ELSE                       /* IF RTB=0 BE SURE THAT  @ZA26908*/
*        DO;                      /* LOAD BALANCER RECOMMEN @ZA26908*/
         B     @RC00466                                         02S0468
@RF00466 DS    0H                                               02S0469
*        OUCBCRV=ZERO;            /* DATION VALUES ARE      @ZA26908*/
         SLR   @10,@10                                          02S0469
         STH   @10,OUCBCRV(,OUCBPTR)                            02S0469
*        OUCBIRV=ZERO;            /* ZERO                   @ZA26908*/
         STH   @10,OUCBIRV(,OUCBPTR)                            02S0470
*        END;                     /*                        @ZA26908*/
*      END;                                                     02S0472
*    ELSE                         /* IF NONSWAPPABLE MEMORY @ZA26908*/
*      DO;                        /* LOAD BALANCER RECOMEN  @ZA26908*/
         B     @RC00461                                         02S0473
@RF00461 DS    0H                                               02S0474
*      OUCBCRV=ZERO;              /* DATION VALUES SHOULD   @ZA26908*/
         SLR   @10,@10                                          02S0474
         STH   @10,OUCBCRV(,OUCBPTR)                            02S0474
*      OUCBIRV=ZERO;              /* BE ZERO                @ZA26908*/
         STH   @10,OUCBIRV(,OUCBPTR)                            02S0475
*      END;                       /*                        @ZA26908*/
*    RTNREG = STPR;                 /* RESTORE RETURN REG CONTENTS   */
@RC00461 LR    RTNREG,STPR                                      02S0477
*    RFY (WKRE) RSTD;                                           02S0478
*    END IRARMWMJ;                                              02S0479
@EL00004 DS    0H                                               02S0479
@EF00004 DS    0H                                               02S0479
@ER00004 BR    @14                                              02S0479
*                                                               01S0480
*/* WLM DONTSWAP/OKSWAP USER ANALYSIS ROUTINE                        */
*/* ******************************************************************/
*/*                                                                  */
*/*    ROUTINE NAME - IRARMWMK                                       */
*/*                                                                  */
*/*    DESCRIPTIVE NAME - WLM DONTSWAP/OKSWAP USER ANALYSY ROUTINE   */
*/*                                                                  */
*/*    FUNCTION - THE IRARMWMK  ROUTINE CALCULATES THE CURRENT       */
*/*               SERVICE AND INSURES THAT THE USER IS IN THE        */
*/*               CORRECT PERFORMANCE GROUP PERIOD. APPLICABLE       */
*/*               ALGORITHM INDICATORS ARE SET BASED ON THE NEW      */
*/*               SWAPPABILITY STATUS OF THE USER.                   */
*/*    CALLED BY - IRARME41 - USER DON'T SWAP SYSEVENT ROUTINE       */
*/*                IRARME42 - USER OK TO SWAP SYSEVENT ROUTINE       */
*/*                                                                  */
*/*    INPUT - REG 4 CONTAINS THA ADDRESS OF THE OUCB FOR THE USER   */
*/*            TO BE EVALUATED.                                      */
*/*          - REG 14 CONTAINS THE RETURN ADDRESS.                   */
*/*          - REG 15 CONTAINS THE IRARMWMK ROUTINE ADDRESS          */
*/*                                                                  */
*/*    OUTPUT - SEE FUNCTION                                         */
*/*                                                                  */
*/*    EXIT - BRANCH ON REGISTER 14.                                 */
*/*         - THERE IS NO ERROR EXIT OR INDICATION.                  */
*/*                                                                  */
*/* ******************************************************************/
*  /* ? IRACTLEP EPN(WMK) */                                    02S0480
*  DCL IRARMWMK ENTRY INT;                                      02S0480
*  GEN CODE DEFS(IRARMWMK) NOEXIT SEQFLOW;                      02S0481
*                                  /*
         ENTRY IRARMWMK
IRARMWMK DS    0H
         STM   @00,@14,128+000(@13)    SAVE INVOKING RTNE REGS
         LA    @12,IRARMWMK-@PSTART    DISPLACEMENT TO MODULE BASE
         LCR   @12,@12                 NEGATIVE FOR SUBTRACT
         ALR   @12,@15                 SETUP MODULE ADDRESSABILITY
*IF OUCBTMW ^= ZERO THEN            /* TRANSACTION INTERVAL EXISTS   */
         L     @09,OUCBTMW(,OUCBPTR)                            02S0482
         LTR   @09,@09                                          02S0482
         BZ    @RF00482                                         02S0482
*  DO ;                                                         02S0483
*  IF OUCBOUT = '0'B & OUCBGOB = '0'B & /* USER RECIEVING SERVICE    */
*     OUCBGOO = '0'B & OUCBGOI = '0'B THEN /* USER NOT TRANSITIONING */
         TM    OUCBOUT(OUCBPTR),B'11100100'                     02S0484
         BNZ   @RF00484                                         02S0484
*    /* OBTAIN USER SERVICE                                          */
*      /* ? IRACTLCL EPN(WM1) */                                02S0485
*      CALL RMSBWM1 -> IRARMRTN ;               /* INVOKE WM1 SUBRTN */
         L     @15,RMSBWM1(,RMCTPTR)                            02S0485
         BALR  @14,@15                                          02S0485
*  /* VERIFY CURRENT PERFORMANCE GROUP PERIOD                        */
*  CALL IRARMWM5 ;                                              02S0486
@RF00484 LA    @15,IRARMWM5                                     02S0486
         BALR  @14,@15                                          02S0486
*  END ;                                                        02S0487
*/* SET APPLICABLE ALGORITHM INDICATORS                              */
*CALL IRARMWMJ ;                                                02S0488
@RF00482 BAL   @14,IRARMWMJ                                     02S0488
*  /* ? IRACTLEP ENN(WMK) */                                    02S0489
*  GEN CODE EXIT NOSEQFLOW;                                     02S0489
*                                  /*
         LM    @00,@14,128+000(@13)    RELOAD INVOKING RTNE REGS
         BR    RTNREG                  RETURN TO INVOKER
*                                                               01S0490
*        END IRARMWLM                                           01S0490
*/* THE FOLLOWING INCLUDE STATEMENTS WERE FOUND IN THIS PROGRAM.      *
*/*%INCLUDE SYSUT5  (IRAWLMDT)                                        *
*/*%INCLUDE SYSLIB  (IRACNSMP)                                        *
*/*%INCLUDE SYSLIB  (IRARMCT )                                        *
*/*%INCLUDE SYSLIB  (IRACCT  )                                        *
*/*%INCLUDE SYSLIB  (IRAICT  )                                        *
*/*%INCLUDE SYSLIB  (IRAMCT  )                                        *
*/*%INCLUDE SYSLIB  (IRARCT  )                                        *
*/*%INCLUDE SYSLIB  (IRARMPT )                                        *
*/*%INCLUDE SYSLIB  (IRARMCA )                                        *
*/*%INCLUDE SYSLIB  (IRARMEX )                                        *
*/*%INCLUDE SYSLIB  (IRARMSB )                                        *
*/*%INCLUDE SYSLIB  (IRAEPAT )                                        *
*/*%INCLUDE SYSLIB  (IRAEPDT )                                        *
*/*%INCLUDE SYSLIB  (IRAEPST )                                        *
*/*%INCLUDE SYSLIB  (IRARRPA )                                        *
*/*%INCLUDE SYSLIB  (IHAASCB )                                        *
*/*%INCLUDE SYSLIB  (IRAOUCB )                                        *
*/*%INCLUDE SYSLIB  (IHAOUXB )                                        *
*/*%INCLUDE SYSLIB  (IRADMDT )                                        *
*/*%INCLUDE SYSLIB  (IRAWMST )                                        *
*/*%INCLUDE SYSLIB  (IRAWPGDT)                                        *
*/*%INCLUDE SYSLIB  (IRAWPODT)                                        *
*/*%INCLUDE SYSLIB  (IHAWSAVT)                                        *
*/*%INCLUDE SYSLIB  (IHAPSA  )                                        *
*/*%INCLUDE SYSLIB  (IHALCCA )                                        *
*/*%INCLUDE SYSLIB  (CVT     )                                        *
*/*%INCLUDE SYSLIB  (IHAPVT  )                                        *
*/*%INCLUDE SYSLIB  (IRAREGS )                                        *
*/*%INCLUDE SYSUT5  (IRARMWM1)                                        *
*/*%INCLUDE SYSUT5  (IRARMWM2)                                        *
*/*%INCLUDE SYSUT5  (IRARMWM3)                                        *
*/*%INCLUDE SYSUT5  (IRARMWM4)                                        *
*/*%INCLUDE SYSUT5  (IRARMWM5)                                        *
*/*%INCLUDE SYSUT5  (IRARMWM7)                                        *
*/*%INCLUDE SYSUT5  (IRARMHIT)                                        *
*/*%INCLUDE SYSUT5  (IRARMWMN)                                        *
*/*%INCLUDE SYSUT5  (IRARMWMO)                                        *
*/*%INCLUDE SYSUT5  (IRARMWMQ)                                        *
*/*%INCLUDE SYSUT5  (IRARMWMR)                                        *
*/*%INCLUDE SYSUT5  (IRARMWMI)                                        *
*/*%INCLUDE SYSUT5  (IRARMWMJ)                                        *
*/*%INCLUDE SYSUT5  (IRARMWMK)                                        *
*;                                                              01S0490
@DATA    DS    0H
@DATD    DSECT
         DS    0F
IRARMWLM CSECT
         DS    0F
@CF00036 DC    F'4'
@CF02217 DC    XL4'0000FFFF'
@DATD    DSECT
         DS    0D
         ORG   *+1-(*-@DATD)/(*-@DATD) INSURE DSECT DATA
@ENDDATD EQU   *
IRARMWLM CSECT
         DS    0F
@SIZDATD DC    AL1(0)
         DC    AL3(@ENDDATD-@DATD)
         DS    0D
HWORDMSK DC    XL4'0000FFFF'
ONE      DC    F'1'
RMTMFCT  DC    H'976'
@CC02243 DC    C'OUCB'
@00      EQU   00                      EQUATES FOR REGISTERS 0-15
@01      EQU   01
@02      EQU   02
@03      EQU   03
@04      EQU   04
@05      EQU   05
@06      EQU   06
@07      EQU   07
@08      EQU   08
@09      EQU   09
@10      EQU   10
@11      EQU   11
@12      EQU   12
@13      EQU   13
@14      EQU   14
@15      EQU   15
RMCTPTR  EQU   @02
RRPAPTR  EQU   @03
ASCBPTR  EQU   @05
OUCBPTR  EQU   @04
OUXBPTR  EQU   @06
WMSTPTR  EQU   @01
WPGDPTR  EQU   @07
WPGPPTR  EQU   @11
WPGLPTR  EQU   @15
WPODPTR  EQU   @01
R0       EQU   @00
R1       EQU   @01
SVEREG   EQU   @13
RTNREG   EQU   @14
NTRREG   EQU   @15
RTNCOD   EQU   @15
STPR     EQU   @05
PMRE     EQU   @06
PMRO     EQU   @07
SVRE     EQU   @06
SVRO     EQU   @07
WKRE     EQU   @10
WKRO     EQU   @11
VLRO     EQU   @11
R14      EQU   @14
R15      EQU   @15
VLRE     EQU   @10
PSA      EQU   0
FLCRNPSW EQU   PSA
FLCROPSW EQU   PSA+8
FLCICCW2 EQU   PSA+16
FLCCVT   EQU   FLCICCW2
PSAEEPSW EQU   PSA+132
PSAESPSW EQU   PSA+136
FLCSVILC EQU   PSAESPSW+1
PSAEPPSW EQU   PSA+140
FLCPIILC EQU   PSAEPPSW+1
FLCPICOD EQU   PSAEPPSW+2
PSAPICOD EQU   FLCPICOD+1
FLCTEA   EQU   PSAEPPSW+4
FLCPER   EQU   PSA+152
FLCMCLA  EQU   PSA+168
FLCIOEL  EQU   FLCMCLA+4
FLCIOA   EQU   FLCMCLA+16
FLCFSA   EQU   FLCMCLA+80
PSALCCAV EQU   PSA+528
PSASUPER EQU   PSA+552
PSASUP1  EQU   PSASUPER
PSASUP2  EQU   PSASUPER+1
PSASUP3  EQU   PSASUPER+2
PSASUP4  EQU   PSASUPER+3
PSACLHT  EQU   PSA+640
PSALKSA  EQU   PSA+696
PSAHLHI  EQU   PSA+760
PSADSSFL EQU   PSA+836
PSADSSF1 EQU   PSADSSFL
PSADSSF3 EQU   PSADSSFL+2
PSADSSF4 EQU   PSADSSFL+3
PSARSVT  EQU   PSA+896
RMCT     EQU   0
RMCTWMST EQU   RMCT+24
RMCTRMSA EQU   RMCT+32
RMCTADJC EQU   RMCT+64
RMCTOTQE EQU   RMCT+108
RMCTINQE EQU   RMCT+112
RMCTFLGS EQU   RMCT+148
RMCTMFA  EQU   RMCTFLGS
RMCTDMDT EQU   RMCT+180
CCT      EQU   RMCT+192
CCCAPMDP EQU   CCT+40
CCCAPGDP EQU   CCT+42
CCTFLG1  EQU   CCT+124
ICT      EQU   RMCT+320
ICTFLAGS EQU   ICT+84
MCT      EQU   RMCT+408
MCTSFLGS EQU   MCT+64
MCTOFLGS EQU   MCT+65
RCT      EQU   RMCT+504
RMPT     EQU   RMCT+592
RMPTWMET EQU   RMPT+48
RMCA     EQU   RMCT+664
RMCAFLGS EQU   RMCA+36
RMCAISV  EQU   RMCA+70
RMCASRC  EQU   RMCA+124
RMEX     EQU   RMCT+832
RMEPBEVT EQU   RMEX
RMEPBCTL EQU   RMEX+16
RMSB     EQU   RMCT+896
RMSBCRL  EQU   RMSB+36
RMSBCRN  EQU   RMSB+40
RMSBWM5  EQU   RMSB+48
RMSBWM1  EQU   RMSB+52
RMSBWM4  EQU   RMSB+56
RMSBWMO  EQU   RMSB+68
RMSBWR4  EQU   RMSB+88
RMSBWR6  EQU   RMSB+92
EPAT     EQU   RMCT+1016
RMEPBCAP EQU   EPAT
EPDT     EQU   RMCT+1416
RMEPBRPS EQU   EPDT+80
EPST     EQU   RMCT+1512
RMEPBWM3 EQU   EPST
RMEPBIL3 EQU   EPST+16
RMEPBCL3 EQU   EPST+32
RMEPBCSO EQU   EPST+48
RMEPBCSI EQU   EPST+64
RRPA     EQU   0
RRPAINC  EQU   RRPA
RRPAVPGN EQU   RRPAINC+2
RRPAFLG0 EQU   RRPAVPGN
RRPACOD  EQU   RRPAINC+3
RRPAINP  EQU   RRPA+4
RRPAFLG  EQU   RRPA+8
RRPAWAR  EQU   RRPAFLG+1
RRPATOD  EQU   RRPA+12
RRPAINS  EQU   RRPA+16
RRPAEPA  EQU   RRPA+20
ASCB     EQU   0
ASCBSUPC EQU   ASCB+16
ASCBIOSM EQU   ASCB+40
ASCBRSM  EQU   ASCB+52
ASCBRSMF EQU   ASCBRSM
ASCBEJST EQU   ASCB+64
ASCBFW1  EQU   ASCB+100
ASCBRCTF EQU   ASCBFW1+2
ASCBFLG1 EQU   ASCBFW1+3
ASCBDSP1 EQU   ASCB+114
ASCBFLG2 EQU   ASCB+115
ASCBLKGP EQU   ASCB+128
ASCBOUXB EQU   ASCB+148
ASCBSRQ  EQU   ASCB+180
ASCBSRQ1 EQU   ASCBSRQ
ASCBSRQ2 EQU   ASCBSRQ+1
ASCBSRQ3 EQU   ASCBSRQ+2
ASCBSRQ4 EQU   ASCBSRQ+3
OUCB     EQU   0
OUCBNAME EQU   OUCB
OUCBFWD  EQU   OUCB+4
OUCBTMA  EQU   OUCB+12
OUCBQFL  EQU   OUCB+16
OUCBGOO  EQU   OUCBQFL
OUCBGOI  EQU   OUCBQFL
OUCBGOB  EQU   OUCBQFL
OUCBOFF  EQU   OUCBQFL
OUCBOUT  EQU   OUCBQFL
OUCBSFL  EQU   OUCB+17
OUCBNSW  EQU   OUCBSFL
OUCBINV  EQU   OUCBSFL
OUCBPVL  EQU   OUCBSFL
OUCBENQ  EQU   OUCBSFL
OUCBYFL  EQU   OUCB+18
OUCBLOG  EQU   OUCBYFL
OUCBAFL  EQU   OUCB+19
OUCBAPG  EQU   OUCBAFL
OUCBRMA  EQU   OUCBAFL
OUCBTFL  EQU   OUCB+20
OUCBATR  EQU   OUCBTFL
OUCBSTR  EQU   OUCBTFL
OUCBNTR  EQU   OUCBTFL
OUCBRTR  EQU   OUCBTFL
OUCBPCH  EQU   OUCBTFL
OUCBMAR  EQU   OUCBTFL
OUCBINP  EQU   OUCBTFL
OUCBINC  EQU   OUCBTFL
OUCBEFL  EQU   OUCB+21
OUCBTRM  EQU   OUCBEFL
OUCBCIM  EQU   OUCBEFL
OUCBUFL  EQU   OUCB+23
OUCBNPG  EQU   OUCB+24
OUCBNDP  EQU   OUCB+26
OUCBPGP  EQU   OUCB+31
OUCBWMG  EQU   OUCB+32
OUCBDMO  EQU   OUCB+34
OUCBDMN  EQU   OUCB+36
OUCBSWC  EQU   OUCB+38
OUCBASCB EQU   OUCB+40
OUCBTMW  EQU   OUCB+48
OUCBWMS  EQU   OUCB+52
OUCBCPU  EQU   OUCB+56
OUCBIOC  EQU   OUCB+60
OUCBMSO  EQU   OUCB+64
OUCBTMS  EQU   OUCB+68
OUCBTMO  EQU   OUCB+72
OUCBCSW  EQU   OUCB+84
OUCBCFL  EQU   OUCBCSW+2
OUCBRDY  EQU   OUCBCFL
OUCBWMR  EQU   OUCB+92
OUCBIRV  EQU   OUCB+96
OUCBCRV  EQU   OUCB+98
OUCBTMP  EQU   OUCB+104
OUCBIOSM EQU   OUCB+108
OUCBPSS  EQU   OUCB+112
OUXB     EQU   0
OUXBIOS  EQU   OUXB+52
OUXBIOSM EQU   OUXB+56
OUXBTRC  EQU   OUXB+64
OUXBJBS  EQU   OUXB+68
OUXBJBT  EQU   OUXB+72
OUXBTRS  EQU   OUXB+76
OUXBTRT  EQU   OUXB+80
OUXBJBR  EQU   OUXB+84
OUXBTRR  EQU   OUXB+88
OUXBCPS  EQU   OUXB+96
OUXBMSS  EQU   OUXB+104
OUXBPRS  EQU   OUXB+120
DMDT     EQU   0
WMST     EQU   0
WMSTPGVT EQU   WMST+8
WMSTPGDT EQU   WMST+16
WMSTPOVT EQU   WMST+24
WMSTPODT EQU   WMST+32
WMSTDMVT EQU   WMST+48
WMSTCPU  EQU   WMST+68
WMSTIOC  EQU   WMST+72
WMSTMSO  EQU   WMST+76
WPGD     EQU   0
WPGDDFL  EQU   WPGD+3
WPGDPGP  EQU   WPGD+12
WPGDINDS EQU   WPGDPGP+5
WPGP     EQU   0
WPGPOBJN EQU   WPGP
WPGPDMN  EQU   WPGP+1
WPGPAPGV EQU   WPGP+4
WPGPINDS EQU   WPGP+5
WPGPFIN  EQU   WPGPINDS
WPGPUNT  EQU   WPGPINDS
WPGPAPG  EQU   WPGPINDS
WPGPRTB  EQU   WPGPINDS
WPGPISVP EQU   WPGP+6
WPGPDURP EQU   WPGP+7
WPGL     EQU   0
WPGLISV  EQU   WPGL
WPGLDUR  EQU   WPGLISV
WPOD     EQU   0
WPOP     EQU   WPOD
WPOPSRV  EQU   WPOP
WPOPWKL  EQU   WPOP+2
WPOS     EQU   WPOD+4
WPOSSRV  EQU   WPOS
WPOSWKL  EQU   WPOS+2
WSAC     EQU   0
WSACOPTM EQU   WSAC+8
LCCA     EQU   0
LCCAIHRC EQU   LCCA+520
LCCAIHR1 EQU   LCCAIHRC
LCCAIHR2 EQU   LCCAIHRC+1
LCCAIHR3 EQU   LCCAIHRC+2
LCCAIHR4 EQU   LCCAIHRC+3
LCCASPIN EQU   LCCA+524
LCCASPN1 EQU   LCCASPIN
LCCASPN2 EQU   LCCASPIN+1
LCCASPN3 EQU   LCCASPIN+2
LCCASPN4 EQU   LCCASPIN+3
LCCACPUS EQU   LCCA+536
LCCADSF1 EQU   LCCA+540
LCCADSF2 EQU   LCCA+541
LCCASPSA EQU   LCCA+544
LCCACRFL EQU   LCCA+692
LCCACREX EQU   LCCA+693
LCCALKFG EQU   LCCA+694
LCCASRBF EQU   LCCA+720
CVTMAP   EQU   0
CVTDAR   EQU   CVTMAP+72
CVTFLGS1 EQU   CVTDAR
CVTDCB   EQU   CVTMAP+116
CVTIOQET EQU   CVTMAP+120
CVTIERLC EQU   CVTMAP+144
CVTHEAD  EQU   CVTMAP+160
CVTSV76C EQU   CVTHEAD
CVTOPTA  EQU   CVTMAP+182
CVTOPTB  EQU   CVTMAP+183
CVTGTF   EQU   CVTMAP+236
CVTGTFST EQU   CVTGTF
CVTGTFS  EQU   CVTGTFST
CVTSTATE EQU   CVTGTFST
CVTTMODE EQU   CVTGTFST
CVTFORM  EQU   CVTGTFST
CVTAQAVT EQU   CVTMAP+240
CVTTCMFG EQU   CVTAQAVT
CVTVOLM2 EQU   CVTMAP+244
CVTTATA  EQU   CVTVOLM2
CVTTSKS  EQU   CVTTATA
CVTVOLF2 EQU   CVTTSKS
CVTTAT   EQU   CVTTATA+1
CVTATER  EQU   CVTMAP+248
CVTEXT1  EQU   CVTMAP+252
CVTPURG  EQU   CVTMAP+260
CVTQMSG  EQU   CVTMAP+268
CVTDMSR  EQU   CVTMAP+272
CVTRSV37 EQU   CVTDMSR
CVTDMSRF EQU   CVTRSV37
CVTERPV  EQU   CVTMAP+316
CVTINTLA EQU   CVTMAP+320
CVTAPF   EQU   CVTMAP+324
CVTEXT2  EQU   CVTMAP+328
CVTHJES  EQU   CVTMAP+332
CVTPGSIA EQU   CVTMAP+348
CVTA1F1  EQU   CVTMAP+356
CVTSYSK  EQU   CVTMAP+357
CVTVOLM1 EQU   CVTMAP+380
CVTVOLF1 EQU   CVTVOLM1
CVTATMCT EQU   CVTMAP+388
CVTXTNT1 EQU   0
CVTXTNT2 EQU   0
CVTDSSV  EQU   CVTXTNT2
CVTFLGBT EQU   CVTXTNT2+5
CVTQID   EQU   CVTXTNT2+24
CVTRV400 EQU   CVTXTNT2+52
CVTRV409 EQU   CVTXTNT2+53
CVTATCVT EQU   CVTXTNT2+64
CVTRV429 EQU   CVTXTNT2+84
CVTRV438 EQU   CVTXTNT2+85
CVTRV457 EQU   CVTXTNT2+112
CVTRV466 EQU   CVTXTNT2+113
CVTFIX   EQU   0
CVTRELNO EQU   CVTFIX+252
PVT      EQU   0
PVTFLAG1 EQU   PVT
PVTFLAG2 EQU   PVT+1
PVTQS    EQU   PVT+304
PVTWSAX  EQU   PVT+344
IRARMRTN EQU   0
IRARMPTR EQU   0
WM5RTRN  EQU   184
WPGVTENT EQU   0
WPOVTENT EQU   0
WDMVTENT EQU   0
IRARMNTR EQU   0
IRASERPS EQU   4
DMDTPTR  EQU   RMCTDMDT
PSARSVTE EQU   PSARSVT
LCCAPTR  EQU   PSALCCAV
CVTPTR   EQU   FLCCVT
CVTS01   EQU   CVTPGSIA
CVTPVTP  EQU   CVTS01+8
CVTLPDIA EQU   CVTS01+12
CVTDIRST EQU   CVTLPDIA
CVTSLIDA EQU   CVTS01+24
CVTCTLFG EQU   CVTS01+50
CVTRV210 EQU   CVTS01+424
CVTRV219 EQU   CVTS01+425
CVTRV228 EQU   CVTS01+426
CVTRV237 EQU   CVTS01+427
CVTMFRTR EQU   CVTS01+452
CVTRV262 EQU   CVTS01+468
CVTRV271 EQU   CVTS01+469
CVTRV280 EQU   CVTS01+470
CVTRV289 EQU   CVTS01+471
CVTGSDA  EQU   CVTS01+600
PVTPTR   EQU   CVTPVTP
EPATRPS  EQU   RMEPBRPS
WPOXSRV  EQU   WPOD+8
FLC      EQU   PSA
         AGO   .@UNREFD                START UNREFERENCED COMPONENTS
CVTRV628 EQU   CVTS01+728
CVTRV627 EQU   CVTS01+724
CVTRV626 EQU   CVTS01+720
CVTRV625 EQU   CVTS01+716
CVTRV624 EQU   CVTS01+712
CVTRV623 EQU   CVTS01+708
CVTRV622 EQU   CVTS01+704
CVTRV621 EQU   CVTS01+700
CVTIHASU EQU   CVTS01+696
CVTSUSP  EQU   CVTS01+692
CVTT6SVC EQU   CVTS01+688
CVTCDAL  EQU   CVTS01+684
CVTTCTL  EQU   CVTS01+680
CVTRSUME EQU   CVTS01+676
CVTJTERM EQU   CVTS01+672
CVTASMRM EQU   CVTS01+668
CVTTCASP EQU   CVTS01+664
CVT0PT03 EQU   CVTS01+660
CVT0PT0E EQU   CVTS01+656
CVTRV609 EQU   CVTS01+652
CVTCGK   EQU   CVTS01+648
CVTRAC   EQU   CVTS01+644
CVTHSM   EQU   CVTS01+640
CVTRV605 EQU   CVTS01+636
CVTRV604 EQU   CVTS01+632
CVTEFF02 EQU   CVTS01+628
CVTCBBR  EQU   CVTS01+624
CVTSSCR  EQU   CVTS01+620
CVTEVENT EQU   CVTS01+616
CVTCRCA  EQU   CVTS01+612
CVTTPIO  EQU   CVTS01+608
CVTADV   EQU   CVTS01+604
CVTGSDAB EQU   CVTGSDA
CVTQV3   EQU   CVTS01+596
CVTQV2   EQU   CVTS01+592
CVTQV1   EQU   CVTS01+588
CVTRPT   EQU   CVTS01+584
CVTSSRB  EQU   CVTS01+580
CVTCSDRL EQU   CVTS01+576
CVTEXP1  EQU   CVTS01+572
CVTRMPMT EQU   CVTS01+568
CVTRMPTT EQU   CVTS01+564
CVTVPSA  EQU   CVTS01+560
CVTVSTOP EQU   CVTS01+556
CVTGTFR8 EQU   CVTS01+552
CVTQUIT  EQU   CVTS01+548
CVTVACR  EQU   CVTS01+544
CVTWTCB  EQU   CVTS01+540
CVTSTPRS EQU   CVTS01+536
CVT0PT02 EQU   CVTS01+532
CVTDARCM EQU   CVTS01+528
CVTIRECM EQU   CVTS01+524
CVTJRECM EQU   CVTS01+520
CVTVEMS0 EQU   CVTS01+516
CVTSPFRR EQU   CVTS01+512
CVTRLSTG EQU   CVTS01+508
CVT0TC0A EQU   CVTS01+504
CVTGMBR  EQU   CVTS01+500
CVTLFRM  EQU   CVTS01+496
CVTRMBR  EQU   CVTS01+492
CVTVIOP  EQU   CVTS01+488
CVTRV307 EQU   CVTS01+486
CVTRV306 EQU   CVTS01+484
CVTRV305 EQU   CVTS01+482
CVTRV304 EQU   CVTS01+480
CVTRV303 EQU   CVTS01+478
CVTRV302 EQU   CVTS01+476
CVTTRCA  EQU   CVTS01+472
CVTRV297 EQU   CVTRV289
CVTRV296 EQU   CVTRV289
CVTRV295 EQU   CVTRV289
CVTRV294 EQU   CVTRV289
CVTRV293 EQU   CVTRV289
CVTRV292 EQU   CVTRV289
CVTRV291 EQU   CVTRV289
CVTRV290 EQU   CVTRV289
CVTRV288 EQU   CVTRV280
CVTRV287 EQU   CVTRV280
CVTRV286 EQU   CVTRV280
CVTRV285 EQU   CVTRV280
CVTRV284 EQU   CVTRV280
CVTRV283 EQU   CVTRV280
CVTRV282 EQU   CVTRV280
CVTRV281 EQU   CVTRV280
CVTRV279 EQU   CVTRV271
CVTRV278 EQU   CVTRV271
CVTRV277 EQU   CVTRV271
CVTRV276 EQU   CVTRV271
CVTRV275 EQU   CVTRV271
CVTRV274 EQU   CVTRV271
CVTRV273 EQU   CVTRV271
CVTRV272 EQU   CVTRV271
CVTRV270 EQU   CVTRV262
CVTRV269 EQU   CVTRV262
CVTRV268 EQU   CVTRV262
CVTRV267 EQU   CVTRV262
CVTRV266 EQU   CVTRV262
CVTRV265 EQU   CVTRV262
CVTRV264 EQU   CVTRV262
CVTRV263 EQU   CVTRV262
CVTVFP   EQU   CVTS01+464
CVTVSI   EQU   CVTS01+460
CVTVPSIB EQU   CVTS01+456
CVTMFACT EQU   CVTMFRTR
CVTMFCTL EQU   CVTS01+448
CVTPVBP  EQU   CVTS01+444
CVTPWI   EQU   CVTS01+440
CVTRV254 EQU   CVTS01+438
CVTRV253 EQU   CVTS01+436
CVTRV252 EQU   CVTS01+434
CVTRV251 EQU   CVTS01+433
CVTRV250 EQU   CVTS01+432
CVTRV249 EQU   CVTS01+431
CVTRV248 EQU   CVTS01+430
CVTRV247 EQU   CVTS01+429
CVTRV246 EQU   CVTS01+428
CVTRV245 EQU   CVTRV237
CVTRV244 EQU   CVTRV237
CVTRV243 EQU   CVTRV237
CVTRV242 EQU   CVTRV237
CVTRV241 EQU   CVTRV237
CVTRV240 EQU   CVTRV237
CVTRV239 EQU   CVTRV237
CVTRV238 EQU   CVTRV237
CVTRV236 EQU   CVTRV228
CVTRV235 EQU   CVTRV228
CVTRV234 EQU   CVTRV228
CVTRV233 EQU   CVTRV228
CVTRV232 EQU   CVTRV228
CVTRV231 EQU   CVTRV228
CVTRV230 EQU   CVTRV228
CVTRV229 EQU   CVTRV228
CVTRV227 EQU   CVTRV219
CVTRV226 EQU   CVTRV219
CVTRV225 EQU   CVTRV219
CVTRV224 EQU   CVTRV219
CVTRV223 EQU   CVTRV219
CVTRV222 EQU   CVTRV219
CVTRV221 EQU   CVTRV219
CVTRV220 EQU   CVTRV219
CVTRV218 EQU   CVTRV210
CVTRV217 EQU   CVTRV210
CVTRV216 EQU   CVTRV210
CVTRV215 EQU   CVTRV210
CVTRV214 EQU   CVTRV210
CVTRV213 EQU   CVTRV210
CVTRV212 EQU   CVTRV210
CVTRV211 EQU   CVTRV210
CVTLCCAT EQU   CVTS01+420
CVTPCCAT EQU   CVTS01+416
CVTIPCRP EQU   CVTS01+412
CVTIPCRI EQU   CVTS01+408
CVTIPCDS EQU   CVTS01+404
CVTAIDVT EQU   CVTS01+400
CVTSSAP  EQU   CVTS01+396
CVTEHCIR EQU   CVTS01+392
CVTEHDEF EQU   CVTS01+388
CVTDAIR  EQU   CVTS01+384
CVTPERFM EQU   CVTS01+380
CVT044R2 EQU   CVTS01+376
CVTFETCH EQU   CVTS01+372
CVTRSTWD EQU   CVTS01+368
CVTSPOST EQU   CVTS01+364
CVTIOBP  EQU   CVTS01+360
CVTASMVT EQU   CVTS01+356
CVTRECRQ EQU   CVTS01+352
CVTWSAC  EQU   CVTS01+348
CVTRV149 EQU   CVTS01+344
CVTWSAL  EQU   CVTS01+340
CVTSPSA  EQU   CVTS01+336
CVTGLMN  EQU   CVTS01+332
CVTVEAC0 EQU   CVTS01+328
CVT062R1 EQU   CVTS01+324
CVTRPOST EQU   CVTS01+320
CVTDQIQE EQU   CVTS01+316
CVTCSD   EQU   CVTS01+312
CVTLKRMA EQU   CVTS01+308
CVTRSPIE EQU   CVTS01+304
CVTRENQ  EQU   CVTS01+300
CVTLQCB  EQU   CVTS01+296
CVTFQCB  EQU   CVTS01+292
CVTQCS01 EQU   CVTS01+288
CVTAPFT  EQU   CVTS01+284
CVTPARRL EQU   CVTS01+280
CVTVWAIT EQU   CVTS01+276
CVTGSPL  EQU   CVTS01+272
CVTLSMQ  EQU   CVTS01+268
CVTGSMQ  EQU   CVTS01+264
CVTEXPRO EQU   CVTS01+260
CVTOPCTP EQU   CVTS01+256
CVTSIC   EQU   CVTS01+252
CVTTPIOS EQU   CVTS01+248
CVTRTMS  EQU   CVTS01+244
CVTSDBF  EQU   CVTS01+240
CVTSCBP  EQU   CVTS01+236
CVTSDMP  EQU   CVTS01+232
CVTSV60  EQU   CVTS01+228
CVTRTMCT EQU   CVTS01+224
CVTASCBL EQU   CVTS01+220
CVTASCBH EQU   CVTS01+216
CVTGDA   EQU   CVTS01+212
CVTASVT  EQU   CVTS01+208
CVTVVMDI EQU   CVTS01+204
CVTAQTOP EQU   CVTS01+200
CVTIOSCS EQU   CVTS01+196
CVTSDRM  EQU   CVTS01+192
CVTOPTE  EQU   CVTS01+188
CVTSTXU  EQU   CVTS01+184
CVTQUIS  EQU   CVTS01+180
CVTPARS  EQU   CVTS01+176
CVTS1EE  EQU   CVTS01+172
CVTFRAS  EQU   CVTS01+168
CVTQSAS  EQU   CVTS01+164
CVTCRAS  EQU   CVTS01+160
CVTCRMN  EQU   CVTS01+156
CVTDELCP EQU   CVTS01+152
CVTFRECL EQU   CVTS01+148
CVTGETCL EQU   CVTS01+144
CVTBLDCP EQU   CVTS01+140
CVTAUTHL EQU   CVTS01+136
CVTSCAN  EQU   CVTS01+132
CVTRV144 EQU   CVTS01+130
CVTMAXMP EQU   CVTS01+128
CVTSTCK  EQU   CVTS01+124
CVTRV139 EQU   CVTS01+123
CVTDSSAC EQU   CVTS01+122
CVTRV513 EQU   CVTS01+121
CVTIOSPL EQU   CVTS01+120
CVTPTGT  EQU   CVTS01+116
CVTCSPIE EQU   CVTS01+112
CVTSMFEX EQU   CVTS01+108
CVTOLT0A EQU   CVTS01+104
CVTSRBRT EQU   CVTS01+100
CVTPUTL  EQU   CVTS01+96
CVTASCRL EQU   CVTS01+92
CVTASCRF EQU   CVTS01+88
CVTRV326 EQU   CVTS01+84
CVTRV325 EQU   CVTS01+80
CVTRV324 EQU   CVTS01+76
CVT0VL01 EQU   CVTS01+72
CVTSHRVM EQU   CVTS01+68
CVTRV332 EQU   CVTS01+64
CVTTAS   EQU   CVTS01+60
CVTRSCN  EQU   CVTS01+56
CVTTRAC2 EQU   CVTS01+54
CVTTRACE EQU   CVTS01+52
CVTAPG   EQU   CVTS01+51
CVTSDTRC EQU   CVTCTLFG
CVTGTRCE EQU   CVTCTLFG
CVTNOMP  EQU   CVTCTLFG
CVTRSV79 EQU   CVTCTLFG
CVTDSTAT EQU   CVTCTLFG
CVTRSV78 EQU   CVTCTLFG
CVTRV333 EQU   CVTCTLFG
CVTRV323 EQU   CVTCTLFG
CVTSPVLK EQU   CVTS01+49
CVTRSV77 EQU   CVTS01+48
CVTRV331 EQU   CVTS01+44
CVTRV330 EQU   CVTS01+40
CVTRV329 EQU   CVTS01+36
CVTRV328 EQU   CVTS01+32
CVTRV322 EQU   CVTS01+28
CVTSLID  EQU   CVTSLIDA+1
CVTSYLK  EQU   CVTSLIDA
CVTRV321 EQU   CVTS01+20
CVTRV320 EQU   CVTS01+16
CVTLPDIR EQU   CVTLPDIA+1
CVTRSV69 EQU   CVTDIRST
CVTRSV68 EQU   CVTDIRST
CVTRSV67 EQU   CVTDIRST
CVTRSV66 EQU   CVTDIRST
CVTRSV65 EQU   CVTDIRST
CVTRSV64 EQU   CVTDIRST
CVTRSV63 EQU   CVTDIRST
CVTDICOM EQU   CVTDIRST
CVTLPDSR EQU   CVTS01+4
CVTGETL  EQU   CVTS01
PSARSAV  EQU   PSARSVTE+60
PSARSTK  EQU   PSARSVTE+56
PSAESAV3 EQU   PSARSVTE+52
PSAESTK3 EQU   PSARSVTE+48
PSAESAV2 EQU   PSARSVTE+44
PSAESTK2 EQU   PSARSVTE+40
PSAESAV1 EQU   PSARSVTE+36
PSAESTK1 EQU   PSARSVTE+32
PSAPSAV  EQU   PSARSVTE+28
PSAPSTK  EQU   PSARSVTE+24
PSAMSAV  EQU   PSARSVTE+20
PSAMSTK  EQU   PSARSVTE+16
PSASSAV  EQU   PSARSVTE+12
PSASSTK  EQU   PSARSVTE+8
PSANSTK  EQU   PSARSVTE+4
PSACSTK  EQU   PSARSVTE
PVTEND   EQU   PVT+1848
PVTTASID EQU   PVT+1846
PVTXXXXX EQU   PVT+1844
PVTMVCLC EQU   PVT+1840
PVTSPARE EQU   PVT+1838
PVTCHUIC EQU   PVT+1836
PVTCOMFX EQU   PVT+1834
PVTSQAFX EQU   PVT+1832
PVTCNTFX EQU   PVT+1830
PVTDEFFX EQU   PVT+1828
PVTFIXOK EQU   PVT+1826
PVTPEROK EQU   PVT+1824
PVTMAXFX EQU   PVT+1822
PVTPERFX EQU   PVT+1820
PVTRSRB  EQU   PVT+1776
PVTRESV5 EQU   PVT+1773
PVTSRBS  EQU   PVT+1772
PVTSSRB  EQU   PVT+1768
PVTACA   EQU   PVTWSAX+1400
PVTSAVE  EQU   PVTWSAX+1328
PVTWSA18 EQU   PVTWSAX+1256
PVTWSA17 EQU   PVTWSAX+1184
PVTWSA16 EQU   PVTWSAX+1112
PVTWSA15 EQU   PVTWSAX+1040
PVTWSA14 EQU   PVTWSAX+968
PVTWSA13 EQU   PVTWSAX+896
PVTWSA12 EQU   PVTWSAX+824
PVTWSA11 EQU   PVTWSAX+752
PVTWSA10 EQU   PVTWSAX+672
PVTWSA9  EQU   PVTWSAX+592
PVTWSA8  EQU   PVTWSAX+504
PVTWSA7  EQU   PVTWSAX+432
PVTWSA6  EQU   PVTWSAX+360
PVTWSA5  EQU   PVTWSAX+288
PVTWSA4  EQU   PVTWSAX+216
PVTWSA3  EQU   PVTWSAX+144
PVTWSA2  EQU   PVTWSAX+72
PVTWSA1  EQU   PVTWSAX
PVTCIOQL EQU   PVTQS+36
PVTCIOQF EQU   PVTQS+32
PVTGFADL EQU   PVTQS+28
PVTGFADF EQU   PVTQS+24
PVTFPCBL EQU   PVTQS+20
PVTFPCBF EQU   PVTQS+16
PVTSQAQL EQU   PVTQS+14
PVTSQAQF EQU   PVTQS+12
PVTCFQL  EQU   PVTQS+10
PVTCFQF  EQU   PVTQS+8
PVTRSRVL EQU   PVTQS+6
PVTRSRVF EQU   PVTQS+4
PVTAFQL  EQU   PVTQS+2
PVTAFQF  EQU   PVTQS
PVTSPFR  EQU   PVT+302
PVTCFMCT EQU   PVT+300
PVTSPREC EQU   PVT+296
PVTCAREC EQU   PVT+292
PVTCAOUT EQU   PVT+288
PVTCAIN  EQU   PVT+284
PVTNSWPS EQU   PVT+280
PVTNPREC EQU   PVT+276
PVTSPOUT EQU   PVT+272
PVTSPIN  EQU   PVT+268
PVTVAMR  EQU   PVT+264
PVTVAMO  EQU   PVT+260
PVTVAMI  EQU   PVT+256
PVTNPOUT EQU   PVT+252
PVTNPIN  EQU   PVT+248
PVTPRSET EQU   PVT+244
PVTFRSLT EQU   PVT+240
PVTPAGIO EQU   PVT+236
PVTSWPIO EQU   PVT+232
PVTPSWPP EQU   PVT+228
PVTPPREF EQU   PVT+224
PVTPOPBR EQU   PVT+220
PVTPRELF EQU   PVT+216
PVTPRELV EQU   PVT+212
PVTPFREE EQU   PVT+208
PVTPRELS EQU   PVT+204
PVTPOUT  EQU   PVT+200
PVTPFXLD EQU   PVT+196
PVTPITAS EQU   PVT+192
PVTPSRBP EQU   PVT+188
PVTPDSEG EQU   PVT+184
PVTPCSGB EQU   PVT+180
PVTPCSEG EQU   PVT+176
PVTPINV  EQU   PVT+172
PVTPSWPC EQU   PVT+168
PVTPSOUT EQU   PVT+164
PVTPSWIN EQU   PVT+160
PVTPRCV  EQU   PVT+156
PVTPRCFI EQU   PVT+152
PVTPRCF  EQU   PVT+148
PVTPVRC  EQU   PVT+144
PVTPVRIN EQU   PVT+140
PVTPVRLS EQU   PVT+136
PVTPVEQR EQU   PVT+132
PVTPPIOI EQU   PVT+128
PVTPVRPO EQU   PVT+124
PVTPRFR  EQU   PVT+120
PVTPFP2  EQU   PVT+116
PVTPFP   EQU   PVT+112
PVTPFTE  EQU   PVT+108
PVTPCB   EQU   PVT+104
PVTREP2  EQU   PVT+100
PVTPIOCP EQU   PVT+96
PVTPIOP  EQU   PVT+92
PVTPGFAD EQU   PVT+88
PVTPGFA  EQU   PVT+84
PVTPSQA  EQU   PVT+80
PVTPSINT EQU   PVT+76
PVTPSIB  EQU   PVT+72
PVTSRBID EQU   PVT+68
PVTPCIWA EQU   PVT+64
PVTHQSA  EQU   PVT+62
PVTLQSA  EQU   PVT+60
PVTOROOT EQU   PVT+56
PVTPRCA  EQU   PVT+52
PVTREUS  EQU   PVT+48
PVTSQNPC EQU   PVT+47
PVTSQVRC EQU   PVT+46
PVTRSUS  EQU   PVT+44
PVTVROOT EQU   PVT+40
PVTPCBLO EQU   PVT+38
PVTPCBCT EQU   PVT+36
PVTDFRS  EQU   PVT+34
PVTRSQA  EQU   PVT+32
PVTPTLB  EQU   PVT+31
PVTPCBNO EQU   PVT+30
PVTSRBNO EQU   PVT+29
PVTSQDC  EQU   PVT+28
PVTLCSA  EQU   PVT+26
PVTLPRIV EQU   PVT+24
PVTLVR   EQU   PVT+22
PVTFVR   EQU   PVT+20
PVTLPFN  EQU   PVT+18
PVTFPFN  EQU   PVT+16
PVTPFTP  EQU   PVT+12
PVTSSPIN EQU   PVT+11
PVTPCBS  EQU   PVT+10
PVTPOOL  EQU   PVT+8
PVTAFCOK EQU   PVT+6
PVTAFCLO EQU   PVT+4
PVTAFC   EQU   PVT+2
@NM00031 EQU   PVTFLAG2
PVTRSMGM EQU   PVTFLAG2
PVTDUMP  EQU   PVTFLAG1
PVTSIT   EQU   PVTFLAG1
PVTLSI   EQU   PVTFLAG1
PVTAPREF EQU   PVTFLAG1
PVTPCBLT EQU   PVTFLAG1
PVTSRBIU EQU   PVTFLAG1
PVTBGMS  EQU   PVTFLAG1
PVTPMSG  EQU   PVTFLAG1
CVTLEVL  EQU   CVTRELNO+2
CVTNUMB  EQU   CVTRELNO
CVTMDL   EQU   CVTFIX+250
@NM00030 EQU   CVTFIX+248
@NM00029 EQU   CVTFIX
CVTRV482 EQU   CVTXTNT2+128
CVTRV481 EQU   CVTXTNT2+124
CVTRV480 EQU   CVTXTNT2+120
CVTRV479 EQU   CVTXTNT2+118
CVTRV478 EQU   CVTXTNT2+117
CVTRV477 EQU   CVTXTNT2+116
CVTRV476 EQU   CVTXTNT2+115
CVTRV475 EQU   CVTXTNT2+114
CVTRV474 EQU   CVTRV466
CVTRV473 EQU   CVTRV466
CVTRV472 EQU   CVTRV466
CVTRV471 EQU   CVTRV466
CVTRV470 EQU   CVTRV466
CVTRV469 EQU   CVTRV466
CVTRV468 EQU   CVTRV466
CVTRV467 EQU   CVTRV466
CVTRV465 EQU   CVTRV457
CVTRV464 EQU   CVTRV457
CVTRV463 EQU   CVTRV457
CVTRV462 EQU   CVTRV457
CVTRV461 EQU   CVTRV457
CVTRV460 EQU   CVTRV457
CVTRV459 EQU   CVTRV457
CVTRV458 EQU   CVTRV457
CVTRV456 EQU   CVTXTNT2+108
CVTRV455 EQU   CVTXTNT2+104
CVTRV454 EQU   CVTXTNT2+100
CVTRV453 EQU   CVTXTNT2+96
CVTRV452 EQU   CVTXTNT2+94
CVTRV451 EQU   CVTXTNT2+92
CVTRV450 EQU   CVTXTNT2+90
CVTRV449 EQU   CVTXTNT2+88
CVTRV448 EQU   CVTXTNT2+87
CVTRV447 EQU   CVTXTNT2+86
CVTRV446 EQU   CVTRV438
CVTRV445 EQU   CVTRV438
CVTRV444 EQU   CVTRV438
CVTRV443 EQU   CVTRV438
CVTRV442 EQU   CVTRV438
CVTRV441 EQU   CVTRV438
CVTRV440 EQU   CVTRV438
CVTRV439 EQU   CVTRV438
CVTRV437 EQU   CVTRV429
CVTRV436 EQU   CVTRV429
CVTRV435 EQU   CVTRV429
CVTRV434 EQU   CVTRV429
CVTRV433 EQU   CVTRV429
CVTRV432 EQU   CVTRV429
CVTRV431 EQU   CVTRV429
CVTRV430 EQU   CVTRV429
CVTRV428 EQU   CVTXTNT2+80
CVTRV427 EQU   CVTXTNT2+76
CVTRV426 EQU   CVTXTNT2+72
CVTRV425 EQU   CVTXTNT2+68
CVTATACT EQU   CVTATCVT
CVTRV423 EQU   CVTXTNT2+62
CVTRV422 EQU   CVTXTNT2+60
CVTRV421 EQU   CVTXTNT2+58
CVTRV420 EQU   CVTXTNT2+56
CVTRV419 EQU   CVTXTNT2+55
CVTRV418 EQU   CVTXTNT2+54
CVTRV417 EQU   CVTRV409
CVTRV416 EQU   CVTRV409
CVTRV415 EQU   CVTRV409
CVTRV414 EQU   CVTRV409
CVTRV413 EQU   CVTRV409
CVTRV412 EQU   CVTRV409
CVTRV411 EQU   CVTRV409
CVTRV410 EQU   CVTRV409
CVTRV408 EQU   CVTRV400
CVTRV407 EQU   CVTRV400
CVTRV406 EQU   CVTRV400
CVTRV405 EQU   CVTRV400
CVTRV404 EQU   CVTRV400
CVTRV403 EQU   CVTRV400
CVTRV402 EQU   CVTRV400
CVTRV401 EQU   CVTRV400
CVTICB   EQU   CVTXTNT2+48
CVTSKTA  EQU   CVTXTNT2+44
CVTCCVT  EQU   CVTXTNT2+40
CVTRSV98 EQU   CVTXTNT2+36
CVTRSV97 EQU   CVTXTNT2+34
CVTRSV96 EQU   CVTXTNT2+32
CVTOLTEP EQU   CVTXTNT2+28
CVTQIDA  EQU   CVTQID+1
CVTRSV95 EQU   CVTQID
CVTRSV94 EQU   CVTXTNT2+20
CVTRSV93 EQU   CVTXTNT2+16
CVTRSV92 EQU   CVTXTNT2+12
CVTDEBVR EQU   CVTXTNT2+8
CVTRSV91 EQU   CVTXTNT2+6
CVTRSV9H EQU   CVTFLGBT
CVTRSV9G EQU   CVTFLGBT
CVTRSV9F EQU   CVTFLGBT
CVTRSV9E EQU   CVTFLGBT
CVTRSV9D EQU   CVTFLGBT
CVTRSV9C EQU   CVTFLGBT
CVTVME   EQU   CVTFLGBT
CVTNPE   EQU   CVTFLGBT
CVTNUCLS EQU   CVTXTNT2+4
CVTDSSVA EQU   CVTDSSV+1
CVTRSV89 EQU   CVTDSSV
CVTRSV88 EQU   CVTXTNT1+8
CVTRSV87 EQU   CVTXTNT1+4
CVTFACHN EQU   CVTXTNT1
CVTRV488 EQU   CVTMAP+412
CVTRV487 EQU   CVTMAP+408
CVTRV486 EQU   CVTMAP+404
CVTRV485 EQU   CVTMAP+400
CVTACTAP EQU   CVTMAP+396
CVTAUTH  EQU   CVTMAP+392
CVTATMCA EQU   CVTATMCT+1
CVTATMST EQU   CVTATMCT
CVTRSV61 EQU   CVTMAP+384
CVTVOLT1 EQU   CVTVOLM1+1
CVTVOLI1 EQU   CVTVOLF1
CVTSTOA  EQU   CVTMAP+376
CVTRSV58 EQU   CVTMAP+374
CVTRSV57 EQU   CVTMAP+372
CVTDDCE  EQU   CVTMAP+368
CVTPNWFR EQU   CVTMAP+364
CVTSMF   EQU   CVTMAP+360
CVTSULK  EQU   CVTMAP+358
CVTSLKO  EQU   CVTSYSK
CVTSLKP  EQU   CVTSYSK
CVTSLKQ  EQU   CVTSYSK
CVTSLKR  EQU   CVTSYSK
CVTRSV56 EQU   CVTSYSK
CVTRSV55 EQU   CVTSYSK
CVTRSV54 EQU   CVTSYSK
CVTRSV53 EQU   CVTSYSK
CVTRSV52 EQU   CVTA1F1
CVTRSV51 EQU   CVTA1F1
CVTRSV50 EQU   CVTA1F1
CVTRSV49 EQU   CVTA1F1
CVTRSV48 EQU   CVTA1F1
CVTRSV47 EQU   CVTA1F1
CVTSRSW  EQU   CVTA1F1
CVTPFSW  EQU   CVTA1F1
CVTPCVT  EQU   CVTMAP+352
CVTRSV46 EQU   CVTMAP+344
CVTRSV45 EQU   CVTMAP+340
CVTRSV44 EQU   CVTMAP+338
CVTRSV43 EQU   CVTMAP+336
CVTHJESA EQU   CVTHJES+1
CVTRSV42 EQU   CVTHJES
CVTEXT2A EQU   CVTEXT2+1
CVTRSV41 EQU   CVTEXT2
CVTAPFA  EQU   CVTAPF+1
CVTRSV40 EQU   CVTAPF
CVTRV518 EQU   CVTINTLA
CVTRV517 EQU   CVTERPV
CVTEORM  EQU   CVTMAP+312
CVTMCHPR EQU   CVTMAP+308
CVTTZ    EQU   CVTMAP+304
CVTJEPS  EQU   CVTMAP+300
CVTJESCT EQU   CVTMAP+296
CVTMODE  EQU   CVTMAP+292
CVTPTRV  EQU   CVTMAP+288
CVTREAL  EQU   CVTMAP+284
CVTRSV39 EQU   CVTMAP+280
CVTRSV38 EQU   CVTMAP+276
CVTDMSRA EQU   CVTDMSR+1
CVTRV634 EQU   CVTDMSRF
CVTRV633 EQU   CVTDMSRF
CVTRV632 EQU   CVTDMSRF
CVTRV631 EQU   CVTDMSRF
CVTRV630 EQU   CVTDMSRF
CVTRV629 EQU   CVTDMSRF
CVTUDUMP EQU   CVTDMSRF
CVTSDUMP EQU   CVTDMSRF
CVTQMSGA EQU   CVTQMSG+1
CVTRSV36 EQU   CVTQMSG
CVTAMFF  EQU   CVTMAP+264
CVTPURGA EQU   CVTPURG+1
CVTRSV35 EQU   CVTPURG
CVTCBSP  EQU   CVTMAP+256
CVTATERA EQU   CVTATER+1
CVTSYST  EQU   CVTATER
CVTVOLT2 EQU   CVTTAT
CVTVOLI2 EQU   CVTVOLF2
CVTAQAVB EQU   CVTAQAVT+1
CVTRSV34 EQU   CVTTCMFG
CVTRSV33 EQU   CVTTCMFG
CVTRSV32 EQU   CVTTCMFG
CVTRSV31 EQU   CVTTCMFG
CVTRSV30 EQU   CVTTCMFG
CVTRSV29 EQU   CVTTCMFG
CVTLDEV  EQU   CVTTCMFG
CVTTCRDY EQU   CVTTCMFG
CVTGTFA  EQU   CVTGTF+1
CVTRSV27 EQU   CVTGTFST
CVTRNIO  EQU   CVTGTFST
CVTUSR   EQU   CVTGTFST
CVTRV318 EQU   CVTFORM
CVTRV317 EQU   CVTTMODE
CVTRV316 EQU   CVTSTATE
CVTRV315 EQU   CVTGTFS
CVTGTFAV EQU   CVTGTFS
CVT0SCR1 EQU   CVTMAP+232
CVTRV515 EQU   CVTMAP+228
CVTRMS   EQU   CVTMAP+224
CVTPATCH EQU   CVTMAP+220
CVTTSCE  EQU   CVTMAP+216
CVTLNKSC EQU   CVTMAP+214
CVTQABST EQU   CVTMAP+212
CVTMDLDS EQU   CVTMAP+208
CVTUSER  EQU   CVTMAP+204
CVTABEND EQU   CVTMAP+200
CVTSMCA  EQU   CVTMAP+196
CVTRSV18 EQU   CVTMAP+192
CVTQLPAQ EQU   CVTMAP+188
CVTQCDSR EQU   CVTMAP+184
CVTRSV17 EQU   CVTOPTB
CVTRSV16 EQU   CVTOPTB
CVTFP    EQU   CVTOPTB
CVTAPTHR EQU   CVTOPTB
CVTNLOG  EQU   CVTOPTB
CVTTOD   EQU   CVTOPTB
CVTCTIMS EQU   CVTOPTB
CVTPROT  EQU   CVTOPTB
CVTXPFP  EQU   CVTOPTA
CVTASCII EQU   CVTOPTA
CVTRSV13 EQU   CVTOPTA
CVTRSV12 EQU   CVTOPTA
CVTNIP   EQU   CVTOPTA
CVTDDR   EQU   CVTOPTA
CVTAPR   EQU   CVTOPTA
CVTCCH   EQU   CVTOPTA
CVTSNCTR EQU   CVTMAP+180
CVTQMWR  EQU   CVTMAP+176
CVTQOCR  EQU   CVTMAP+172
CVT1EF00 EQU   CVTMAP+168
CVTMZ00  EQU   CVTMAP+164
CVTSV76Q EQU   CVTSV76C
CVTRSV11 EQU   CVTMAP+156
CVT0PT01 EQU   CVTMAP+152
CVTMSER  EQU   CVTMAP+148
CVTRV516 EQU   CVTIERLC
CVTILCH  EQU   CVTMAP+140
CVT0DS   EQU   CVTMAP+136
CVTFBOSV EQU   CVTMAP+132
CVTNUCB  EQU   CVTMAP+128
CVTIXAVL EQU   CVTMAP+124
CVTSV76M EQU   CVTIOQET
CVTDCBA  EQU   CVTMAP+117
CVTMVS2  EQU   CVTDCB
CVT6DAT  EQU   CVTDCB
CVT4MPS  EQU   CVTDCB
CVTRSV09 EQU   CVTDCB
CVT4MS1  EQU   CVTDCB
CVT2SPS  EQU   CVTDCB
CVT1SSS  EQU   CVTDCB
CVTRSV08 EQU   CVTDCB
CVTSTB   EQU   CVTMAP+112
CVTQTD00 EQU   CVTMAP+108
CVTQTE00 EQU   CVTMAP+104
CVTCUCB  EQU   CVTMAP+100
CVTSJQ   EQU   CVTMAP+96
CVTPBLDL EQU   CVTMAP+92
CVTTPC   EQU   CVTMAP+88
CVTSVDCB EQU   CVTMAP+84
CVTBRET  EQU   CVTMAP+82
CVTEXIT  EQU   CVTMAP+80
CVT0FN00 EQU   CVTMAP+76
CVTDARA  EQU   CVTDAR+1
CVTRSV07 EQU   CVTFLGS1
CVTRSV06 EQU   CVTFLGS1
CVTRSV05 EQU   CVTFLGS1
CVTRSV04 EQU   CVTFLGS1
CVTRSV03 EQU   CVTFLGS1
CVTRSV02 EQU   CVTFLGS1
CVTRSV01 EQU   CVTFLGS1
CVTDMPLK EQU   CVTFLGS1
CVTXITP  EQU   CVTMAP+68
CVTZDTAB EQU   CVTMAP+64
CVTMSLT  EQU   CVTMAP+60
CVTDATE  EQU   CVTMAP+56
CVTBTERM EQU   CVTMAP+52
CVTSYSAD EQU   CVTMAP+48
CVTXTLER EQU   CVTMAP+44
CVTILK2  EQU   CVTMAP+40
CVTILK1  EQU   CVTMAP+36
CVTPRLTV EQU   CVTMAP+32
CVTPCNVT EQU   CVTMAP+28
CVT0VL00 EQU   CVTMAP+24
CVTXAPG  EQU   CVTMAP+20
CVTBUF   EQU   CVTMAP+16
CVTJOB   EQU   CVTMAP+12
CVTLINK  EQU   CVTMAP+8
CVT0EF00 EQU   CVTMAP+4
CVTTCBP  EQU   CVTMAP
CVT      EQU   CVTMAP
LCCAR125 EQU   LCCA+964
LCCAR124 EQU   LCCA+960
LCCASGPR EQU   LCCA+896
LCCADRT2 EQU   LCCA+888
LCCADRT1 EQU   LCCA+880
LCCAR103 EQU   LCCA+876
LCCAESS2 EQU   LCCA+872
LCCASPLJ EQU   LCCA+868
LCCASMQJ EQU   LCCA+864
LCCAIRT  EQU   LCCA+736
LCCAASID EQU   LCCA+732
LCCARV89 EQU   LCCA+728
LCCAPGTA EQU   LCCASRBF+2
LCCASAFN EQU   LCCASRBF
LCCAECSA EQU   LCCA+716
LCCAICR0 EQU   LCCA+712
LCCALWTM EQU   LCCA+704
LCCARV78 EQU   LCCA+700
LCCAPINV EQU   LCCA+696
LCCARV88 EQU   LCCA+695
LCCARV87 EQU   LCCALKFG
LCCARV86 EQU   LCCALKFG
LCCARV85 EQU   LCCALKFG
LCCARV84 EQU   LCCALKFG
LCCALKRD EQU   LCCALKFG
LCCALKAQ EQU   LCCALKFG
LCCALKSA EQU   LCCALKFG
LCCALKDP EQU   LCCALKFG
LCCACRST EQU   LCCACREX
LCCACRDP EQU   LCCACREX
LCCACRLM EQU   LCCACREX
LCCACRIN EQU   LCCACREX
LCCACRRT EQU   LCCACREX
LCCACRLE EQU   LCCACREX
LCCACRRM EQU   LCCACREX
LCCACREF EQU   LCCACREX
LCCAVARY EQU   LCCACRFL
LCCARV73 EQU   LCCACRFL
LCCARV72 EQU   LCCACRFL
LCCARV71 EQU   LCCACRFL
LCCARV70 EQU   LCCACRFL
LCCARV69 EQU   LCCACRFL
LCCACLMS EQU   LCCACRFL
LCCACRTM EQU   LCCACRFL
LCCALCR0 EQU   LCCA+688
LCCACRLC EQU   LCCA+684
LCCARCPU EQU   LCCA+680
LCCADCPU EQU   LCCA+676
LCCASRBJ EQU   LCCA+672
LCCADSSR EQU   LCCA+668
LCCADSSC EQU   LCCA+660
LCCADSS3 EQU   LCCA+648
LCCADSS2 EQU   LCCA+636
LCCADSS1 EQU   LCCA+624
LCCAWTIM EQU   LCCA+616
LCCAITOD EQU   LCCA+608
LCCADTOD EQU   LCCA+600
LCCASTOD EQU   LCCA+592
LCCARIR5 EQU   LCCASPSA+44
LCCARIR4 EQU   LCCASPSA+40
LCCARIR3 EQU   LCCASPSA+36
LCCARIR2 EQU   LCCASPSA+32
LCCARPR5 EQU   LCCASPSA+28
LCCARPR4 EQU   LCCASPSA+24
LCCARPR3 EQU   LCCASPSA+20
LCCARPR2 EQU   LCCASPSA+16
LCCADSR5 EQU   LCCASPSA+12
LCCADSR4 EQU   LCCASPSA+8
LCCADSR3 EQU   LCCASPSA+4
LCCADSR2 EQU   LCCASPSA
LCCARV68 EQU   LCCA+543
LCCAPSMK EQU   LCCA+542
LCCARV67 EQU   LCCADSF2
LCCARV66 EQU   LCCADSF2
LCCARV65 EQU   LCCADSF2
LCCARV64 EQU   LCCADSF2
LCCADSRW EQU   LCCADSF2
LCCADSPL EQU   LCCADSF2
LCCAGSRB EQU   LCCADSF2
LCCASRBM EQU   LCCADSF2
LCCARV61 EQU   LCCADSF1
LCCARV60 EQU   LCCADSF1
LCCARV59 EQU   LCCADSF1
LCCARV58 EQU   LCCADSF1
LCCATIMR EQU   LCCADSF1
LCCADSS  EQU   LCCADSF1
LCCAVCPU EQU   LCCADSF1
LCCAACR  EQU   LCCADSF1
LCCAASCP EQU   LCCA+532
LCCAESSA EQU   LCCA+528
LCCARV55 EQU   LCCASPN4
LCCARV54 EQU   LCCASPN4
LCCARV53 EQU   LCCASPN4
LCCARV52 EQU   LCCASPN4
LCCARV51 EQU   LCCASPN4
LCCARV50 EQU   LCCASPN4
LCCARV49 EQU   LCCASPN4
LCCARV48 EQU   LCCASPN4
LCCARV47 EQU   LCCASPN3
LCCARV46 EQU   LCCASPN3
LCCARV45 EQU   LCCASPN3
LCCARV44 EQU   LCCASPN3
LCCARV43 EQU   LCCASPN3
LCCARV42 EQU   LCCASPN3
LCCARV41 EQU   LCCASPN3
LCCARV40 EQU   LCCASPN3
LCCARV39 EQU   LCCASPN2
LCCARV38 EQU   LCCASPN2
LCCARV37 EQU   LCCASPN2
LCCARV36 EQU   LCCASPN2
LCCARV35 EQU   LCCASPN2
LCCARV34 EQU   LCCASPN2
LCCARV33 EQU   LCCASPN2
LCCARV32 EQU   LCCASPN2
LCCARV31 EQU   LCCASPN1
LCCARV30 EQU   LCCASPN1
LCCAMFIO EQU   LCCASPN1
LCCARSTR EQU   LCCASPN1
LCCATSPN EQU   LCCASPN1
LCCALOCK EQU   LCCASPN1
LCCASIGP EQU   LCCASPN1
LCCAPTLB EQU   LCCASPN1
LCCARV27 EQU   LCCAIHR4
LCCARV26 EQU   LCCAIHR4
LCCARV25 EQU   LCCAIHR4
LCCARV24 EQU   LCCAIHR4
LCCARV23 EQU   LCCAIHR4
LCCARV22 EQU   LCCAIHR4
LCCARV21 EQU   LCCAIHR4
LCCARV20 EQU   LCCAIHR4
LCCARV19 EQU   LCCAIHR3
LCCARV18 EQU   LCCAIHR3
LCCARV17 EQU   LCCAIHR3
LCCARV16 EQU   LCCAIHR3
LCCARV15 EQU   LCCAIHR3
LCCARV14 EQU   LCCAIHR3
LCCARV13 EQU   LCCAIHR3
LCCARV12 EQU   LCCAIHR3
LCCARV11 EQU   LCCAIHR2
LCCARV10 EQU   LCCAIHR2
LCCARV09 EQU   LCCAIHR2
LCCARV08 EQU   LCCAIHR2
LCCARV07 EQU   LCCAIHR2
LCCARV06 EQU   LCCAIHR2
LCCARV05 EQU   LCCAIHR2
LCCARV04 EQU   LCCAIHR2
LCCARV03 EQU   LCCAIHR1
LCCARV02 EQU   LCCAIHR1
LCCARV01 EQU   LCCAIHR1
LCCAPPIE EQU   LCCAIHR1
LCCAPSG1 EQU   LCCAIHR1
LCCAPDAT EQU   LCCAIHR1
LCCAXRC2 EQU   LCCAIHR1
LCCAXRC1 EQU   LCCAIHR1
LCCAIOPS EQU   LCCA+512
LCCAGPGR EQU   LCCA+448
LCCAR133 EQU   LCCA+444
LCCAR132 EQU   LCCA+440
LCCAR131 EQU   LCCA+436
LCCAR130 EQU   LCCA+432
LCCAR129 EQU   LCCA+428
LCCAR128 EQU   LCCA+424
LCCAR127 EQU   LCCA+420
LCCAR126 EQU   LCCA+416
LCCARSGR EQU   LCCA+352
LCCAXGR3 EQU   LCCA+288
LCCAXGR2 EQU   LCCA+224
LCCAXGR1 EQU   LCCA+160
LCCACR0  EQU   LCCA+156
LCCAMCR1 EQU   LCCA+152
LCCAPVAD EQU   LCCA+148
LCCAPINT EQU   LCCA+144
LCCAPPSW EQU   LCCA+136
LCCAPGR2 EQU   LCCA+72
LCCAPGR1 EQU   LCCA+8
LCCARV77 EQU   LCCA+6
LCCACPUA EQU   LCCA+4
LCCALCCA EQU   LCCA
WSACASMS EQU   WSAC+60
WSACASMD EQU   WSAC+56
WSACCCH  EQU   WSAC+52
WSACRRSA EQU   WSAC+48
WSACREST EQU   WSAC+44
WSACRSTI EQU   WSAC+40
WSACABTM EQU   WSAC+36
WSACMF1  EQU   WSAC+32
WSACEDS0 EQU   WSAC+28
WSACIOS  EQU   WSAC+24
WSACRTMK EQU   WSAC+20
WSACACR  EQU   WSAC+16
WSACTIME EQU   WSAC+12
WSACGTF  EQU   WSAC+4
WSACCWSA EQU   WSAC
WPOSEND  EQU   WPOD+8
WPODEND  EQU   WPOD+4
WPGLEND  EQU   WPGL+4
WPGPEND  EQU   WPGP+8
WPGPRB2  EQU   WPGPINDS
@NM00004 EQU   WPGP+2
WPGDEND  EQU   WPGD+20
WPGDPGL  EQU   WPGD+20
WPGDDURP EQU   WPGDPGP+7
WPGDISVP EQU   WPGDPGP+6
WPGDRTB  EQU   WPGDINDS
WPGDRB2  EQU   WPGDINDS
WPGDAPG  EQU   WPGDINDS
WPGDUNT  EQU   WPGDINDS
WPGDFIN  EQU   WPGDINDS
WPGDAPGV EQU   WPGDPGP+4
@NM00003 EQU   WPGDPGP+2
WPGDDMN  EQU   WPGDPGP+1
WPGDOBJN EQU   WPGDPGP
WPGDISV  EQU   WPGD+8
WPGDATM  EQU   WPGD+4
WPGDRD1  EQU   WPGDDFL
WPGDVLD  EQU   WPGDDFL
WPGDR02  EQU   WPGD+2
WPGDSIZ  EQU   WPGD+1
WPGDPGN  EQU   WPGD
WMSTEND  EQU   WMST+112
WMSTIPM  EQU   WMST+108
WMSTIPI  EQU   WMST+104
WMSTIPC  EQU   WMST+100
WMSTSECB EQU   WMST+96
WMSTNWST EQU   WMST+92
WMSTSTCB EQU   WMST+88
WMSTSET  EQU   WMST+84
WMSTRSVD EQU   WMST+81
WMSTREAL EQU   WMST+80
WMSTDMNC EQU   WMST+66
WMSTPGPC EQU   WMST+64
WMSTPGHI EQU   WMST+62
WMSTWLHI EQU   WMST+60
WMSTDMDE EQU   WMST+56
WMSTDMVS EQU   WMST+52
WMSTDMDS EQU   WMST+44
WMSTDMDT EQU   WMST+40
WMSTPODS EQU   WMST+36
WMSTPOVS EQU   WMST+28
WMSTPGDS EQU   WMST+20
WMSTPGVS EQU   WMST+12
WMSTWLLO EQU   WMST+6
WMSTID   EQU   WMST+4
WMSTNAME EQU   WMST
DMDTEND  EQU   DMDT+20
DMDTRUC  EQU   DMDT+16
DMDTINCU EQU   DMDT+14
DMDTOUTU EQU   DMDT+12
DMDTCMPL EQU   DMDT+10
DMDTGOOU EQU   DMDT+8
DMDTRUA  EQU   DMDT+6
DMDTMPLT EQU   DMDT+4
DMDTWT   EQU   DMDT+3
DMDTHI   EQU   DMDT+2
DMDTLO   EQU   DMDT+1
DMDTNO   EQU   DMDT
OUXBEND  EQU   OUXB+144
OUXBTSW  EQU   OUXB+140
OUXBFIXC EQU   OUXB+138
OUXBNVC  EQU   OUXB+136
OUXBUIC  EQU   OUXB+134
OUXBVSC  EQU   OUXB+132
OUXBILS  EQU   OUXB+128
OUXBRSV1 EQU   OUXB+126
OUXBWCT  EQU   OUXB+124
OUXBSTD  EQU   OUXB+116
OUXBITD  EQU   OUXB+112
OUXBAET  EQU   OUXB+92
OUXBNQT  EQU   OUXB+60
OUXBSTC  EQU   OUXB+54
OUXBSTCT EQU   OUXB+48
OUXBCAPR EQU   OUXB+44
OUXBCAPI EQU   OUXB+40
OUXBVAMR EQU   OUXB+36
OUXBVAMO EQU   OUXB+32
OUXBVAMI EQU   OUXB+28
OUXBPREC EQU   OUXB+24
OUXBPOUT EQU   OUXB+20
OUXBPIN  EQU   OUXB+16
OUXBRSW  EQU   OUXB+12
OUXBMET  EQU   OUXB+8
OUXBPET  EQU   OUXB+4
OUXBNAME EQU   OUXB
OUCBEND  EQU   OUCB+136
OUCBNTSP EQU   OUCB+134
OUCBNDS  EQU   OUCB+132
OUCBTIO  EQU   OUCB+128
OUCBTCP  EQU   OUCB+124
OUCBPST  EQU   OUCB+120
OUCBPS2  EQU   OUCBPSS+4
OUCBPS1  EQU   OUCBPSS
OUCBR03  EQU   OUCB+102
OUCBIOR  EQU   OUCB+100
OUCBCMRV EQU   OUCB+88
OUCBCF2  EQU   OUCBCFL
OUCBRSM  EQU   OUCBCFL
OUCBACN  EQU   OUCBCSW
OUCBACT  EQU   OUCB+80
OUCBWSS  EQU   OUCB+78
OUCBPSO  EQU   OUCB+76
OUCBIMCB EQU   OUCB+44
OUCBSRC  EQU   OUCB+37
OUCBR40  EQU   OUCB+30
OUCBR30  EQU   OUCB+29
OUCBSPG  EQU   OUCB+28
OUCBRDP  EQU   OUCB+27
OUCBRPG  EQU   OUCB+25
OUCBUF7  EQU   OUCBUFL
OUCBUF6  EQU   OUCBUFL
OUCBUF5  EQU   OUCBUFL
OUCBTSWC EQU   OUCBUFL
OUCBTSWP EQU   OUCBUFL
OUCBRSWP EQU   OUCBUFL
OUCBJSAS EQU   OUCBUFL
OUCBJSFS EQU   OUCBUFL
OUCBNQC  EQU   OUCB+22
OUCBMWT  EQU   OUCBEFL
OUCBQSC  EQU   OUCBEFL
OUCBQSS  EQU   OUCBEFL
OUCBNQF  EQU   OUCBEFL
OUCBOWT  EQU   OUCBEFL
OUCBLWT  EQU   OUCBEFL
OUCBASW  EQU   OUCBAFL
OUCBNWT  EQU   OUCBAFL
OUCBR02  EQU   OUCBAFL
OUCBJSR  EQU   OUCBAFL
OUCBCPL  EQU   OUCBAFL
OUCBAF0  EQU   OUCBAFL
OUCBFXS  EQU   OUCBYFL
OUCBDTA  EQU   OUCBYFL
OUCBAXS  EQU   OUCBYFL
OUCBPSTR EQU   OUCBYFL
OUCBMNT  EQU   OUCBYFL
OUCBSTT  EQU   OUCBYFL
OUCBPSTE EQU   OUCBYFL
OUCBSCN  EQU   OUCBSFL
OUCBNSWI EQU   OUCBSFL
OUCBBIB  EQU   OUCBSFL
OUCBCTI  EQU   OUCBSFL
OUCBQF7  EQU   OUCBQFL
OUCBQF6  EQU   OUCBQFL
OUCBQF3  EQU   OUCBQFL
OUCBBCK  EQU   OUCB+8
ASCBEND  EQU   ASCB+208
ASCBSRBT EQU   ASCB+200
ASCBSWTL EQU   ASCB+196
ASCBRS14 EQU   ASCB+195
ASCBSMCT EQU   ASCB+194
ASCBRS12 EQU   ASCB+192
ASCBPCTT EQU   ASCB+188
ASCBVGTT EQU   ASCB+184
ASCBRV43 EQU   ASCBSRQ4
ASCBRV42 EQU   ASCBSRQ4
ASCBRV41 EQU   ASCBSRQ4
ASCBRV40 EQU   ASCBSRQ4
ASCBRV39 EQU   ASCBSRQ4
ASCBRV38 EQU   ASCBSRQ4
ASCBRV37 EQU   ASCBSRQ4
ASCBRV36 EQU   ASCBSRQ4
ASCBRV35 EQU   ASCBSRQ3
ASCBRV34 EQU   ASCBSRQ3
ASCBRV33 EQU   ASCBSRQ3
ASCBRV32 EQU   ASCBSRQ3
ASCBRV31 EQU   ASCBSRQ3
ASCBRV30 EQU   ASCBSRQ3
ASCBRV29 EQU   ASCBSRQ3
ASCBRV28 EQU   ASCBSRQ3
ASCBRV27 EQU   ASCBSRQ2
ASCBRV26 EQU   ASCBSRQ2
ASCBRV25 EQU   ASCBSRQ2
ASCBRV24 EQU   ASCBSRQ2
ASCBRV23 EQU   ASCBSRQ2
ASCBRV22 EQU   ASCBSRQ2
ASCBRV21 EQU   ASCBSRQ2
ASCBRV20 EQU   ASCBSRQ2
ASCBRV19 EQU   ASCBSRQ1
ASCBRV18 EQU   ASCBSRQ1
ASCBRV17 EQU   ASCBSRQ1
ASCBRV16 EQU   ASCBSRQ1
ASCBRV15 EQU   ASCBSRQ1
ASCBRV14 EQU   ASCBSRQ1
ASCBRV13 EQU   ASCBSRQ1
ASCBSTA  EQU   ASCBSRQ1
ASCBJBNS EQU   ASCB+176
ASCBJBNI EQU   ASCB+172
ASCBMCC  EQU   ASCB+168
ASCBRTWA EQU   ASCB+164
ASCBIQEA EQU   ASCB+160
ASCBXMPQ EQU   ASCB+156
ASCBRS01 EQU   ASCB+154
ASCBFMCT EQU   ASCB+152
ASCBOUCB EQU   ASCB+144
ASCBMECB EQU   ASCB+140
ASCBQECB EQU   ASCB+136
ASCBLSQH EQU   ASCBLKGP+4
ASCBLOCK EQU   ASCBLKGP
ASCBTCBS EQU   ASCB+124
ASCBNVSC EQU   ASCB+122
ASCBVSC  EQU   ASCB+120
ASCBSRBS EQU   ASCB+118
ASCBSSRB EQU   ASCB+116
ASCBRV06 EQU   ASCBFLG2
ASCBRV05 EQU   ASCBFLG2
ASCBRV04 EQU   ASCBFLG2
ASCBSNQS EQU   ASCBFLG2
ASCBS2S  EQU   ASCBFLG2
ASCBCEXT EQU   ASCBFLG2
ASCBPXMT EQU   ASCBFLG2
ASCBXMPT EQU   ASCBFLG2
ASCBRF07 EQU   ASCBDSP1
ASCBRF06 EQU   ASCBDSP1
ASCBRF05 EQU   ASCBDSP1
ASCBRF04 EQU   ASCBDSP1
ASCBRF03 EQU   ASCBDSP1
ASCBRF02 EQU   ASCBDSP1
ASCBFAIL EQU   ASCBDSP1
ASCBNOQ  EQU   ASCBDSP1
ASCBSWCT EQU   ASCB+112
ASCBASXB EQU   ASCB+108
ASCBTMCH EQU   ASCB+104
ASCBNSWP EQU   ASCBFLG1
ASCBTYP1 EQU   ASCBFLG1
ASCBSTND EQU   ASCBFLG1
ASCBABNT EQU   ASCBFLG1
ASCBTERM EQU   ASCBFLG1
ASCBS3S  EQU   ASCBFLG1
ASCBCMSH EQU   ASCBFLG1
ASCBTOFF EQU   ASCBFLG1
ASCBRF01 EQU   ASCBRCTF
ASCBTMLW EQU   ASCBRCTF
ASCBOUT  EQU   ASCBRCTF
ASCBWAIT EQU   ASCBRCTF
ASCBRV08 EQU   ASCBRCTF
ASCBFQU  EQU   ASCBRCTF
ASCBFRS  EQU   ASCBRCTF
ASCBTMNO EQU   ASCBRCTF
ASCBAFFN EQU   ASCBFW1
ASCBDUMP EQU   ASCB+96
ASCBRV44 EQU   ASCB+92
ASCBUBET EQU   ASCB+88
ASCBECB  EQU   ASCB+84
ASCBJSTL EQU   ASCB+80
ASCBEWST EQU   ASCB+72
ASCBTSB  EQU   ASCB+60
ASCBCSCB EQU   ASCB+56
ASCBRSMA EQU   ASCBRSM+1
ASCBRV54 EQU   ASCBRSMF
ASCBRV53 EQU   ASCBRSMF
ASCBRV52 EQU   ASCBRSMF
ASCBRV51 EQU   ASCBRSMF
ASCBVEQR EQU   ASCBRSMF
ASCBN2LP EQU   ASCBRSMF
ASCB1LPU EQU   ASCBRSMF
ASCB2LPU EQU   ASCBRSMF
ASCBLDA  EQU   ASCB+48
ASCBSTOR EQU   ASCB+44
ASCBDP   EQU   ASCB+43
ASCBRV07 EQU   ASCB+42
ASCBSEQN EQU   ASCB+38
ASCBASID EQU   ASCB+36
ASCBCPUS EQU   ASCB+32
ASCBSPL  EQU   ASCB+28
ASCBIOSP EQU   ASCB+24
ASCBSYNC EQU   ASCBSUPC+4
ASCBSVRB EQU   ASCBSUPC
ASCBCMSF EQU   ASCB+12
ASCBBWDP EQU   ASCB+8
ASCBFWDP EQU   ASCB+4
ASCBASCB EQU   ASCB
ASCBEGIN EQU   ASCB
RRPAEND  EQU   RRPA+24
RRPARTN  EQU   RRPAINS
RRPARTC  EQU   RRPA+11
RRPAKEY  EQU   RRPA+10
RRPARCI  EQU   RRPAFLG+1
RRPARFI  EQU   RRPAFLG+1
RRPARTI  EQU   RRPAFLG+1
RRPACTL  EQU   RRPAFLG+1
RRPAF11  EQU   RRPAFLG+1
RRPAIPS  EQU   RRPAFLG+1
RRPAF07  EQU   RRPAFLG
RRPAOPT  EQU   RRPAFLG
RRPAF05  EQU   RRPAFLG
RRPAF04  EQU   RRPAFLG
RRPADSP  EQU   RRPAFLG
RRPASVC  EQU   RRPAFLG
RRPAENB  EQU   RRPAFLG
RRPADSB  EQU   RRPAFLG
RRPAVLU  EQU   RRPAINP
@NM00002 EQU   RRPAFLG0
RRPANSWI EQU   RRPAFLG0
RRPAASD  EQU   RRPAINC
EPSTEND  EQU   EPST+80
EPSTCSI  EQU   RMEPBCSI
EPSTCSO  EQU   RMEPBCSO
EPSTCL3  EQU   RMEPBCL3
EPSTIL3  EQU   RMEPBIL3
EPSTWM3  EQU   RMEPBWM3
EPDTEND  EQU   EPDT+96
RMEPBHIT EQU   EPDT+64
RMEPBUXB EQU   EPDT+48
RMEPBIL4 EQU   EPDT+32
RMEPBDEL EQU   EPDT+16
RMEPBIPS EQU   EPDT
EPATEND  EQU   EPAT+400
RMEPBPR1 EQU   EPAT+368
RMEPBEQ1 EQU   EPAT+336
RMEPBAP1 EQU   EPAT+304
RMEPBCL1 EQU   EPAT+272
RMEPBWM2 EQU   EPAT+240
RMEPBIL1 EQU   EPAT+208
RMEPBASM EQU   EPAT+176
RMEPBMS2 EQU   EPAT+160
RMEPBPR5 EQU   EPAT+144
RMEPBMS6 EQU   EPAT+112
RMEPBSQA EQU   EPAT+96
RMEPBRM2 EQU   EPAT+64
RMEPBRM1 EQU   EPAT+32
RMSBEND  EQU   RMSB+120
RMSBRA9  EQU   RMSB+116
RMSBIL0  EQU   RMSB+112
RMSBCL0  EQU   RMSB+108
RMSBRA6  EQU   RMSB+104
RMSBNOP  EQU   RMSB+100
RMSBSET  EQU   RMSB+96
RMSBTRC  EQU   RMSB+84
RMSBWMY  EQU   RMSB+80
RMSBWMR  EQU   RMSB+76
RMSBWMQ  EQU   RMSB+72
RMSBWMN  EQU   RMSB+64
RMSBWMK  EQU   RMSB+60
RMSBCRY  EQU   RMSB+44
RMSBI10  EQU   RMSB+32
RMSBI09  EQU   RMSB+28
RMSBR24  EQU   RMSB+24
RMSBI07  EQU   RMSB+20
RMSBI06  EQU   RMSB+16
RMSBI05  EQU   RMSB+12
RMSBI04  EQU   RMSB+8
RMSBI03  EQU   RMSB+4
RMSBI02  EQU   RMSB
RMEXEND  EQU   RMEX+64
RMEXSRE  EQU   RMEX+60
RMEXXPE  EQU   RMEX+56
RMEXRR2  EQU   RMEX+52
RMEXRR1  EQU   RMEX+48
RMEXI48  EQU   RMEX+44
RMEXCET  EQU   RMEX+40
RMEXCXX  EQU   RMEX+36
RMEXI01  EQU   RMEX+32
RMEXCTL  EQU   RMEPBCTL
RMEXEVT  EQU   RMEPBEVT
RMCAEND  EQU   RMCA+168
RMCAR90  EQU   RMCA+164
RMCAUSSC EQU   RMCASRC+36
RMCAEXSC EQU   RMCASRC+32
RMCANQSC EQU   RMCASRC+28
RMCARQSC EQU   RMCASRC+24
RMCADWSC EQU   RMCASRC+20
RMCARSSC EQU   RMCASRC+16
RMCAXSSC EQU   RMCASRC+12
RMCALWSC EQU   RMCASRC+8
RMCATISC EQU   RMCASRC+4
RMCATOSC EQU   RMCASRC
RMCACPW1 EQU   RMCA+120
RMCACIUS EQU   RMCA+118
RMCAR14  EQU   RMCA+116
RMCAR13  EQU   RMCA+112
RMCAWKA  EQU   RMCA+80
RMCAMAS  EQU   RMCA+76
RMCAR20  EQU   RMCA+74
RMCABCPG EQU   RMCA+73
RMCALGPG EQU   RMCA+72
RMCAR11  EQU   RMCA+68
RMCAINV  EQU   RMCA+64
RMCAR10  EQU   RMCA+60
RMCARSV9 EQU   RMCA+56
RMCACHU  EQU   RMCA+52
RMCACHP  EQU   RMCA+48
RMCARSV8 EQU   RMCA+44
RMCARSV7 EQU   RMCA+40
RMCARSVF EQU   RMCAFLGS
RMCASAAF EQU   RMCAFLGS
RMCARSV5 EQU   RMCA+32
RMCARSV4 EQU   RMCA+28
RMCARSV3 EQU   RMCA+24
RMCATOI  EQU   RMCA+20
RMCATRS  EQU   RMCA+16
RMCATQS  EQU   RMCA+12
RMCARSV2 EQU   RMCA+10
RMCARSV1 EQU   RMCA+8
RMCAINUS EQU   RMCA+6
RMCAR04  EQU   RMCA+4
RMCANAME EQU   RMCA
RMPTEND  EQU   RMPT+72
RMPTOPE  EQU   RMPT+66
RMPTOPI  EQU   RMPT+63
RMPTOPC  EQU   RMPT+60
RMPTIOET EQU   RMPT+56
RMPTCPET EQU   RMPT+52
RMPTSAET EQU   RMPT+44
RMPTXCHT EQU   RMPT+40
RMPTRSV3 EQU   RMPT+38
RMPTRSV2 EQU   RMPT+36
RMPTRSV1 EQU   RMPT+32
RMPTTOL  EQU   RMPT+28
RMPTTOM  EQU   RMPT+24
RMPTTCS  EQU   RMPT+20
RMPTIMN  EQU   RMPT+16
RMPTERV  EQU   RMPT+12
RMPTIOC  EQU   RMPT+8
RMPTCPU  EQU   RMPT+4
RMPTNAME EQU   RMPT
RCTEND   EQU   RCT+88
RCVRSVF3 EQU   RCT+84
RCVRSVF2 EQU   RCT+80
RCVTAPAD EQU   RCT+76
RCVRSVF1 EQU   RCT+74
RCVTOTDF EQU   RCT+72
RCVBPTTM EQU   RCT+68
RCVBPTCT EQU   RCT+64
RCVASMQ  EQU   RCT+60
RCVAVQP  EQU   RCT+56
RCVCPUC  EQU   RCT+52
RCVUICC  EQU   RCT+48
RCVPTR   EQU   RCT+46
RCVASMQA EQU   RCT+44
RCVAVQC  EQU   RCT+42
RCVCPUA  EQU   RCT+40
RCVUICA  EQU   RCT+38
RCVCTMC  EQU   RCT+36
RCCRSVF3 EQU   RCT+32
RCCRSVF2 EQU   RCT+28
RCCRSVF1 EQU   RCT+26
RCCLCHRR EQU   RCT+24
RCCLCHUT EQU   RCT+22
RCCTOTUT EQU   RCT+20
RCCASMTH EQU   RCT+18
RCCASMTL EQU   RCT+16
RCCPTRTH EQU   RCT+14
RCCPTRTL EQU   RCT+12
RCCCPUTH EQU   RCT+10
RCCCPUTL EQU   RCT+8
RCCUICTH EQU   RCT+6
RCCUICTL EQU   RCT+4
RCTRCT   EQU   RCT
MCTEND   EQU   MCT+96
MCVRSVF3 EQU   MCT+92
MCVRSVF2 EQU   MCT+88
MCVRSVF1 EQU   MCT+84
MCCINTMX EQU   MCT+80
MCCPR1IN EQU   MCT+76
MCCUICIN EQU   MCT+72
MCCUICBD EQU   MCT+68
MCTRSVB1 EQU   MCT+67
MCTXRSVD EQU   MCT+66
MCTFX2   EQU   MCTOFLGS
MCTFX1   EQU   MCTOFLGS
MCTSMS2  EQU   MCTOFLGS
MCTSMS1  EQU   MCTOFLGS
MCTAMS2  EQU   MCTOFLGS
@NM00001 EQU   MCTOFLGS
MCTASM2  EQU   MCTOFLGS
MCTASM1  EQU   MCTOFLGS
MCTSF04  EQU   MCTSFLGS
MCTFAVQ  EQU   MCTSFLGS
MCTAVQ1  EQU   MCTSFLGS
MCTSQA2  EQU   MCTSFLGS
MCTSQA1  EQU   MCTSFLGS
MCVAVQC  EQU   MCT+60
MCVFRCNT EQU   MCT+58
MCVSTCRI EQU   MCT+56
MCCEND   EQU   MCT+56
MCCASMT2 EQU   MCT+54
MCCASMT1 EQU   MCT+52
MCCRSVH1 EQU   MCT+50
MCCPLUS  EQU   MCT+48
MCCMS403 EQU   MCT+44
MCCMS402 EQU   MCT+40
MCCMS401 EQU   MCT+36
MCCMS400 EQU   MCT+32
MCCMS203 EQU   MCT+28
MCCMS202 EQU   MCT+24
MCCMS201 EQU   MCT+20
MCCMS200 EQU   MCT+16
MCCMS102 EQU   MCT+12
MCCMS101 EQU   MCT+8
MCCMS100 EQU   MCT+4
MCTMCT   EQU   MCT
ICTEND   EQU   ICT+88
ICTRSVB3 EQU   ICT+87
ICTRSVB2 EQU   ICT+86
ICTRSVB1 EQU   ICT+85
ICT003   EQU   ICTFLAGS
ICTIOOT  EQU   ICTFLAGS
ICTIOL   EQU   ICTFLAGS
ICTDRSV4 EQU   ICTFLAGS
ICVIRSV3 EQU   ICT+80
ICVULCBT EQU   ICT+76
ICVOLCBT EQU   ICT+72
ICVLCBPT EQU   ICT+68
ICVLUTBT EQU   ICT+64
ICCEND   EQU   ICT+64
ICCRSV01 EQU   ICT+62
ICCEDSUT EQU   ICT+60
ICCSRSV2 EQU   ICT+58
ICCSIGUP EQU   ICT+56
ICCINDAT EQU   ICT+52
ICCINLOT EQU   ICT+48
ICCINHIT EQU   ICT+44
ICCDAUTH EQU   ICT+42
ICCLOUTH EQU   ICT+40
ICCHIUTH EQU   ICT+38
ICCDASF2 EQU   ICT+36
ICCDASF1 EQU   ICT+34
ICCMINRV EQU   ICT+32
ICCMAXRV EQU   ICT+30
ICCRVSCF EQU   ICT+28
ICCMNIOR EQU   ICT+26
ICCLCHN  EQU   ICT+24
ICCLCLST EQU   ICT+20
ICCMNSWP EQU   ICT+16
ICCMNUIN EQU   ICT+12
ICCMXICT EQU   ICT+8
ICCMRSV1 EQU   ICT+4
ICTICT   EQU   ICT
CCVEND   EQU   CCT+128
CCTFLG2  EQU   CCT+125
CCTCPUOT EQU   CCTFLG1
CCTALL   EQU   CCTFLG1
CCVCPUCT EQU   CCT+122
CCVENQCT EQU   CCT+120
CCVLGFMC EQU   CCT+118
CCVLGUTL EQU   CCT+116
CCVR07   EQU   CCT+114
CCVR06   EQU   CCT+112
CCVUTILP EQU   CCT+110
CCVR05   EQU   CCT+108
CCVIMBBT EQU   CCT+104
CCVEJST  EQU   CCT+100
CCVPGCPF EQU   CCT+96
CCVRVSWF EQU   CCT+92
CCVRBSTD EQU   CCT+88
CCVR04   EQU   CCT+84
CCVRBSWT EQU   CCT+80
CCVR03   EQU   CCT+76
CCCEND   EQU   CCT+76
CCCRSVH1 EQU   CCT+74
CCCSCFC2 EQU   CCT+72
CCCSCFC1 EQU   CCT+70
CCCR02   EQU   CCT+68
CCCSIGUR EQU   CCT+66
CCCSIGDP EQU   CCT+64
CCCSCFAC EQU   CCT+62
CCCMINRV EQU   CCT+60
CCCMAXRV EQU   CCT+58
CCCLOSCF EQU   CCT+56
CCCHISCF EQU   CCT+54
CCCUTLOT EQU   CCT+52
CCCUTHIT EQU   CCT+50
CCCAPRHT EQU   CCT+48
CCCAPRLT EQU   CCT+46
CCCR01   EQU   CCT+44
CCCAPLDP EQU   CCT+38
CCCAPBDP EQU   CCT+36
CCCAPROT EQU   CCT+34
CCCAPDIV EQU   CCT+32
CCCRSVF1 EQU   CCT+28
CCCMNSIN EQU   CCT+24
CCCMNUIN EQU   CCT+20
CCCAPDEL EQU   CCT+16
CCCAPMAX EQU   CCT+12
CCCAPMIN EQU   CCT+8
CCCAPMET EQU   CCT+4
CCTCCT   EQU   CCT
RMCTEND  EQU   RMCT+192
RMCTR50  EQU   RMCT+190
RMCTDMNC EQU   RMCT+188
RMCTDMDE EQU   RMCT+184
RMCTSRBT EQU   RMCT+176
RMCTOUXB EQU   RMCT+172
RMCTOUCB EQU   RMCT+168
RMCTTOCI EQU   RMCT+160
RMCTCPID EQU   RMCT+156
RMCTTELM EQU   RMCT+152
RMCTF09  EQU   RMCTFLGS+1
RMCTMFS  EQU   RMCTFLGS+1
RMCTF07  EQU   RMCTFLGS+1
RMCTF06  EQU   RMCTFLGS
RMCTRSV2 EQU   RMCTFLGS
RMCTSTW  EQU   RMCTFLGS
RMCTRSV1 EQU   RMCTFLGS
RMCTINIT EQU   RMCTFLGS
RMCTF03  EQU   RMCTFLGS
RMCTCPS1 EQU   RMCTFLGS
RMCTRQSV EQU   RMCT+144
RMCTALR  EQU   RMCT+140
RMCTALA  EQU   RMCT+136
RMCTTOC  EQU   RMCT+128
RMCTTOD  EQU   RMCT+124
RMCTTBS  EQU   RMCT+120
RMCTR10  EQU   RMCT+116
RMCTWTQE EQU   RMCT+104
RMCTAQHD EQU   RMCT+100
RMCTAQCT EQU   RMCT+96
RMCTTMQE EQU   RMCT+92
RMCTWAMT EQU   RMCT+88
RMCTWAST EQU   RMCT+84
RMCTEPPR EQU   RMCT+80
RMCTFLTM EQU   RMCT+76
RMCTEPET EQU   RMCT+72
RMCTITT  EQU   RMCT+68
RMCTEPBG EQU   RMCT+60
RMCTEPAT EQU   RMCT+56
RMCTEPDT EQU   RMCT+52
RMCTEPPA EQU   RMCT+48
RMCTRMSB EQU   RMCT+44
RMCTRMEX EQU   RMCT+40
RMCTRMPD EQU   RMCT+36
RMCTRLCT EQU   RMCT+28
RMCTRMCA EQU   RMCT+20
RMCTRMPT EQU   RMCT+16
RMCTMCT  EQU   RMCT+12
RMCTICT  EQU   RMCT+8
RMCTCCT  EQU   RMCT+4
RMCTNAME EQU   RMCT
@NM00028 EQU   PSA+3668
PSASTAK  EQU   PSA+3072
@NM00027 EQU   PSA+1048
PSAUSEND EQU   PSA+1048
PSARV062 EQU   PSA+1044
PSACDAL  EQU   PSA+1040
PSAWTCOD EQU   PSA+1036
PSAATCVT EQU   PSA+1032
PSAPCPSW EQU   PSA+1024
PSAPIR2  EQU   PSA+1020
PSARV059 EQU   PSA+1018
PSASVC13 EQU   PSA+1016
PSALSFCC EQU   PSA+1012
PSASFACC EQU   PSA+1008
PSASTOP  EQU   PSA+992
PSASTART EQU   PSA+976
PSARSPSW EQU   PSA+968
PSASRPSW EQU   PSA+960
PSARV045 EQU   PSA+892
PSARV044 EQU   PSA+888
PSARV043 EQU   PSA+884
PSARV042 EQU   PSA+880
PSARV041 EQU   PSA+876
PSARV040 EQU   PSA+872
PSARV025 EQU   PSA+868
PSADSSED EQU   PSA+868
PSADSSPR EQU   PSA+864
PSADSSFW EQU   PSA+860
PSADSS14 EQU   PSA+856
PSADSSPP EQU   PSA+848
PSADSSRP EQU   PSA+840
PSADSS05 EQU   PSADSSF4
PSADSS10 EQU   PSADSSF4
PSADSSVE EQU   PSADSSF4
PSADSSDE EQU   PSADSSF4
PSADSSC0 EQU   PSADSSF4
PSADSSIE EQU   PSADSSF4
PSADSS12 EQU   PSADSSF4
PSADSSRC EQU   PSADSSF4
PSARV057 EQU   PSADSSF3
PSARV056 EQU   PSADSSF3
PSARV055 EQU   PSADSSF3
PSADSSMC EQU   PSADSSF3
PSADSSRW EQU   PSADSSF3
PSADSSNM EQU   PSADSSF3
PSADSSES EQU   PSADSSF3
PSADSSGP EQU   PSADSSF3
PSADSSF2 EQU   PSADSSFL+1
PSADSSPI EQU   PSADSSF1
PSADSSOI EQU   PSADSSF1
PSADSSSP EQU   PSADSSF1
PSADSSTP EQU   PSADSSF1
PSADSSDW EQU   PSADSSF1
PSADSSDD EQU   PSADSSF1
PSADSSDM EQU   PSADSSF1
PSADSSMV EQU   PSADSSF1
PSADSSTS EQU   PSA+816
PSADSSWK EQU   PSA+812
PSADSSR3 EQU   PSA+808
PSADSSR2 EQU   PSA+804
PSADSSRS EQU   PSA+800
PSASTOR  EQU   PSA+796
PSACPUSA EQU   PSA+794
PSAVSTAP EQU   PSA+792
PSAWKVAP EQU   PSA+788
PSAWKRAP EQU   PSA+784
PSAMCHIC EQU   PSA+783
PSARV061 EQU   PSA+782
PSASYMSK EQU   PSA+781
PSAMCHFL EQU   PSA+780
PSACR0   EQU   PSA+776
PSAPSWSV EQU   PSA+768
PSALITA  EQU   PSA+764
PSACLHS  EQU   PSAHLHI
PSALKR15 EQU   PSALKSA+60
PSALKR14 EQU   PSALKSA+56
PSALKR13 EQU   PSALKSA+52
PSALKR12 EQU   PSALKSA+48
PSALKR11 EQU   PSALKSA+44
PSALKR10 EQU   PSALKSA+40
PSALKR9  EQU   PSALKSA+36
PSALKR8  EQU   PSALKSA+32
PSALKR7  EQU   PSALKSA+28
PSALKR6  EQU   PSALKSA+24
PSALKR5  EQU   PSALKSA+20
PSALKR4  EQU   PSALKSA+16
PSALKR3  EQU   PSALKSA+12
PSALKR2  EQU   PSALKSA+8
PSALKR1  EQU   PSALKSA+4
PSALKR0  EQU   PSALKSA
PSARV023 EQU   PSACLHT+52
PSALOCAL EQU   PSACLHT+48
PSACMSL  EQU   PSACLHT+44
PSAOPTL  EQU   PSACLHT+40
PSATPACL EQU   PSACLHT+36
PSATPDNL EQU   PSACLHT+32
PSATPNCL EQU   PSACLHT+28
PSAIOSLL EQU   PSACLHT+24
PSAIOSUL EQU   PSACLHT+20
PSAIOSCL EQU   PSACLHT+16
PSAIOSSL EQU   PSACLHT+12
PSASALCL EQU   PSACLHT+8
PSAASML  EQU   PSACLHT+4
PSADISPL EQU   PSACLHT
PSASRSA  EQU   PSA+636
PSARV050 EQU   PSA+634
PSADSSGO EQU   PSA+633
PSARECUR EQU   PSA+632
PSAHLHIS EQU   PSA+628
PSAIPCSA EQU   PSA+624
@NM00026 EQU   PSA+621
PSAIPCDM EQU   PSA+620
PSAIPCD  EQU   PSA+616
@NM00025 EQU   PSA+613
PSAIPCRM EQU   PSA+612
PSAIPCR  EQU   PSA+608
PSAMCHEX EQU   PSA+600
PSAMPSW  EQU   PSA+592
PSAEXPS2 EQU   PSA+584
PSAEXPS1 EQU   PSA+576
PSAPIREG EQU   PSA+572
PSARSREG EQU   PSA+568
PSAGPREG EQU   PSA+556
PSARV022 EQU   PSASUP4
PSARV021 EQU   PSASUP4
PSARV020 EQU   PSASUP4
PSARV019 EQU   PSASUP4
PSARV018 EQU   PSASUP4
PSARV017 EQU   PSASUP4
PSARV016 EQU   PSASUP4
PSARV015 EQU   PSASUP4
PSARV014 EQU   PSASUP3
PSAULCMS EQU   PSASUP3
PSARV012 EQU   PSASUP3
PSAESTA  EQU   PSASUP3
PSASPR   EQU   PSASUP3
PSAPSREG EQU   PSASUP3
PSAPI2   EQU   PSASUP3
PSAIOSUP EQU   PSASUP3
PSALCR   EQU   PSASUP2
PSARTM   EQU   PSASUP2
PSAACR   EQU   PSASUP2
PSAIPCE2 EQU   PSASUP2
PSAIPCES EQU   PSASUP2
PSAIPCEC EQU   PSASUP2
PSAGTF   EQU   PSASUP2
PSAIPCRI EQU   PSASUP2
PSATYPE6 EQU   PSASUP1
PSATCTL  EQU   PSASUP1
PSADISP  EQU   PSASUP1
PSALOCK  EQU   PSASUP1
PSAPI    EQU   PSASUP1
PSAEXT   EQU   PSASUP1
PSASVC   EQU   PSASUP1
PSAIO    EQU   PSASUP1
PSAAOLD  EQU   PSA+548
PSAANEW  EQU   PSA+544
PSATOLD  EQU   PSA+540
PSATNEW  EQU   PSA+536
PSALCCAR EQU   PSA+532
PSAPCCAR EQU   PSA+524
PSAPCCAV EQU   PSA+520
PSACPULA EQU   PSA+518
PSACPUPA EQU   PSA+516
PSAPSA   EQU   PSA+512
FLCHDEND EQU   PSA+512
FLCCRSAV EQU   FLCMCLA+280
FLCGRSAV EQU   FLCMCLA+216
FLCFPSAV EQU   FLCMCLA+184
FLCFLA   EQU   FLCMCLA+88
FLCRGNCD EQU   FLCMCLA+84
FLCFSAA  EQU   FLCFSA+1
@NM00024 EQU   FLCFSA
@NM00023 EQU   FLCMCLA+72
FLCMCIC  EQU   FLCMCLA+64
@NM00022 EQU   FLCMCLA+20
FLCIOAA  EQU   FLCIOA+1
@NM00021 EQU   FLCIOA
@NM00020 EQU   FLCMCLA+15
@NM00019 EQU   FLCMCLA+14
@NM00018 EQU   FLCMCLA+12
FLCLCL   EQU   FLCMCLA+8
FLCIOELA EQU   FLCIOEL+1
@NM00017 EQU   FLCIOEL
FLCCHNID EQU   FLCMCLA
@NM00016 EQU   PSA+160
FLCMTRCD EQU   PSA+157
@NM00015 EQU   PSA+156
FLCPERA  EQU   FLCPER+1
@NM00014 EQU   FLCPER
@NM00013 EQU   PSA+151
FLCPERCD EQU   PSA+150
FLCMCNUM EQU   PSA+149
@NM00012 EQU   PSA+148
FLCTEAA  EQU   FLCTEA+1
@NM00011 EQU   FLCTEA
PSAPIPC  EQU   PSAPICOD
PSAPIMC  EQU   PSAPICOD
PSAPIPER EQU   PSAPICOD
PSARV049 EQU   FLCPICOD
FLCPILCB EQU   FLCPIILC
@NM00010 EQU   FLCPIILC
@NM00009 EQU   PSAEPPSW
FLCSVCN  EQU   PSAESPSW+2
FLCSILCB EQU   FLCSVILC
@NM00008 EQU   FLCSVILC
@NM00007 EQU   PSAESPSW
FLCEICOD EQU   PSAEEPSW+2
PSASPAD  EQU   PSAEEPSW
@NM00006 EQU   PSA+128
FLCINPSW EQU   PSA+120
FLCMNPSW EQU   PSA+112
FLCPNPSW EQU   PSA+104
FLCSNPSW EQU   PSA+96
FLCENPSW EQU   PSA+88
FLCTRACE EQU   PSA+84
FLCTIMER EQU   PSA+80
FLCCVT2  EQU   PSA+76
FLCCAW   EQU   PSA+72
FLCCSW   EQU   PSA+64
FLCIOPSW EQU   PSA+56
FLCMOPSW EQU   PSA+48
FLCPOPSW EQU   PSA+40
FLCSOPSW EQU   PSA+32
FLCEOPSW EQU   PSA+24
@NM00005 EQU   FLCICCW2+4
FLCICCW1 EQU   FLCROPSW
FLCIPPSW EQU   FLCRNPSW
.@UNREFD ANOP                          END UNREFERENCED COMPONENTS
@RF00133 EQU   @RC00129
@RF00187 EQU   @RC00184
@RF00176 EQU   @EL00002
@RT00392 EQU   IRARMWME
@RT00408 EQU   IRARMWMB
@RC00466 EQU   @RC00461
@ENDDATA EQU   *
         END   IRARMWLM,(C'PLS2213',0701,78024)
