         TITLE 'ISTRACTB---BUFFER TRACE                                *
                        '
ISTRACTB CSECT ,                                                   0001
@MAINENT DS    0H                                                  0001
         USING *,@15                                               0001
         B     @PROLOG                                             0001
         DC    AL1(8)                                              0001
         DC    C'ISTRACTB'                                         0001
         DROP  @15
@PROLOG  BALR  @12,0                                               0001
@PSTART  DS    0H                                                  0001
         USING @PSTART,@12                                         0001
*/********************************************************************/
*/*                                                                  */
*/*  ROUTINE NAME : TBMAINLN - ISTRACTB MAIN LINE PROCEDURE          */
*/*                                                                  */
*/*  FUNCTION : TO PROVIDE MAIN LINE CONTROL FOR BUFFER TRACE. CALLS */
*/*             A ROUTINE TO BUILD A TRACE RECORD (ISTTRAC) AND SETUP*/
*/*             THE TRAC'S HEADER. IF THE DATA IS CONFIDENTIAL THE   */
*/*             RECORD IS WRITTEN WITHOUT FILLING IN THE BUFFER.     */
*/*             OTHERWISE, ROUTINES ARE INVOKED TO FILL THE TRAC     */
*/*             BUFFER WITH CONTROL LAYER OR TPIOS DATA (TBSRPL OR   */
*/*             TBSFDB RESPECTIVELY). AFTER THE BUFFER IS FILLED THE */
*/*             HEADER INFORMATION IS COMPLETED AND THE TRACE RECORD */
*/*             IS WRITTEN. THE WRITTING OF THE RECORD DEPENDS ON THE*/
*/*             SYSTEM. IN DOS, RECORDING IS ACCOMPLISHED BY TPQUEING*/
*/*             THE TRACE RECORD TO THE TRACE WRITER PAB. IN VS1 AND */
*/*             VS2 THE GTRACE MACRO IS ISSUED, GTF TRACE COPIES THE */
*/*             TRACE RECORD AND RETURNS CONTROL.                    */
*/*                                                                  */
*/*  INPUT : R6-BYTE 0 = X'00' - RPL OUTBOUND                        */
*/*                      X'04' - RPL INBOUND                         */
*/*                      X'20' - FDB INPUT                           */
*/*             BYTES1-3 = ADDRESS OF RPL OR FDB DEPENDING ON BYTE 0 */
*/*                                                                  */
*/*  OUTPUT : NONE                                                   */
*/*                                                                  */
*/*  ROUTINES : TBTRAC - GETS CORE FOR TRACE RECORD. SETS UP HEADER. */
*/*                      IF CONFIDENTIAL DATA, COMPLETES HEADER SO   */
*/*                      THE RECORD IS READY TO WRITE.               */
*/*             TBSFDB - CALLED TO HANDLE TPIOS INBOUND NON-         */
*/*                      CONFIDENTIAL DATA. FILLS THE TRAC BUFFER    */
*/*                      WITH FDB DATA                               */
*/*             TBSRPL - CALLED TO HANDLE CONTROL/LAYER NON-         */
*/*                      CONFIDENTIAL DATA. CALLS ROUTINES THAT FILL */
*/*                      THE TRAC BUFFER WITH RPL DATA.              */
*/*                                                                  */
*/*  MACROS : VERSION - TO PROVIDE DIFFERENT PROCESSING DEPENDING ON */
*/*                     THE SYSTEM.                                  */
*/*           CIDCTL (FIND) - TO OBTAIN THE DAFLOCK AND NCB ADDRESS. */
*/*           CIDCTL (FINISH) - TO RELEASE THE DAFLOCK.              */
*/*           RELSTORE - TO RELEASE THE TRAC WHEN AN ERROR OCCURS. IN*/
*/*                     VS1 AND VS2 TO RELEASE THE TRAC AFTER GTRACE */
*/*                     HAS COPIED IT.                               */
*/*           TPQUE - USED IN DOS TO QUEUE THE TRACE RECORD TO THE   */
*/*                   TRACE WRITER (ISTRAETW) PAB FOR RECORDING.     */
*/*           GTRACE - USED IN VS1 AND VS2 TO RECORD TRACE RECORDS.  */
*/*                                                                  */
*/********************************************************************/
*TBMAINLN:                                                         0456
* R14SAVE=R14;                                    /*SAVE REG14       */
TBMAINLN LR    R14SAVE,R14                                         0456
*  IF (SYSCVTPT->CVTGTFAV) = '0'B     /* IS GTRACE ACTIVE    @VS41640*/
*    THEN;                                        /*  NO-EXIT        */
         L     @11,SYSCVTPT                                        0457
         TM    CVTGTFAV(@11),B'10000000'                           0457
         BZ    @RT00457                                            0457
*    ELSE DO;                                     /*  YES-PROCEED    */
*/********************************************************************/
*/*                                                                  */
*/*  ROUTINE NAME : TBTRAC - GET TRAC ROUTINE                        */
*/*                                                                  */
*/*  FUNCTION : TO OBTAIN CORE FOR THE TRACE RECORD. INITIALIZE THE  */
*/*             HEADER. IF CONFIDENTIAL DATA IS INDICATED IN THE FMCB*/
*/*             THE ENTIRE HEADER IS FILLED IN. IN THE CASE OF NORMAL*/
*/*             DATA,THE LOST RECORD CNT AND THE DESTINATION NODE    */
*/*             NAME AND THE APPLICATION NAME ARE NOT SET, RATHER    */
*/*             THE CID IS STORED IN THE HEADER FOR LATER USE BY     */
*/*             TBMAINLN.                                            */
*/*                                                                  */
*/*  INPUT : R6 - INPUT PARAMETERS AS DEFINED IN PROLOGUE.           */
*/*                                                                  */
*/*  OUTPUT : RTNCODE = 0 - TRACE RECORD BUILT SUCCESSFULLY.         */
*/*                  ^= 0 - TRACE RECORD NOT BUILT BECAUSE THE CID   */
*/*                         WAS INVALID OR BECAUSE NO STORAGE WAS    */
*/*                         AVAILABLE.                               */
*/*                                                                  */
*/*  ROUTINES : NONE                                                 */
*/*                                                                  */
*/*  MACROS : CIDCTL - (FIND & FINISH) TO OBTAIN THE FMCB & DNCB     */
*/*                    ADDRESSES,AND TO LOCK AND UNLOCK THE          */
*/*                    DAFLOCK.                                      */
*/*           REQSTORE - CALLED TO OBTAIN STORAGE FOR THE TRACE      */
*/*                      RECORD                                      */
*/*                                                                  */
*/********************************************************************/
*  RFY (R2,FMCBR2,R14) RSTD;                                       0460
*  RTNCODE = 0;                             /* INIT RETRN CODE */  0461
         SLR   RTNCODE,RTNCODE                                     0461
*  IF (R6INPUT&INPTMSK)=FDBINPUT            /*FDB INPUT?             */
*    THEN DO;                               /* YES-OBTAIN CID FROM   */
         LR    @11,R6INPUT                                         0462
         N     @11,INPTMSK                                         0462
         CL    @11,FDBINPUT                                        0462
         BNE   @RF00462                                            0462
*      RFY ISTFDB BASED(R6INPUT);           /*  FSB(R6-> TO 1ST FDB) */
*      R14=R6INPUT                          /*    (ISTFSB PRECEDES   */
*          -LENGTH(ISTFSB)                  /*    1ST FDB,FSB LENGTH */
*          +LENGTH(ISTFDB);                 /*    INCLUDES 1ST FDB)  */
         LR    R14,R6INPUT                                         0465
         AL    R14,@CF04445                                        0465
*      R2=FSBCID;                                                  0466
         L     R2,FSBCID(,R14)                                     0466
*  RFY CIDSAVR5 RSTD;                       /*SAVE CID TO DO CID     */
*  CIDSAVR5=R2;                             /*FINISH.                */
*                                                                  0468
         LR    CIDSAVR5,R2                                         0468
*   DO; /* CIDCTL(FIND) LOCK(XCLUSIVE) */                          0469
*                                                                  0469
*   RFY (EMCR2,EMCR3) RSTD;                                        0470
*     GEN SETS(RPHWORK)                                            0471
*        (STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH);      0471
         STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH
*     EMCR3 = '00'X; /*CLEAR EXTRA BITS*/                          0472
         SLR   EMCR3,EMCR3                                         0472
*     EMCR3 = '02'X; /*INDICATE EXCLUSIVE LOCK*/                   0473
         LA    EMCR3,2                                             0473
*     CALL ATCLCPTR->ATCCDFND->ISTSDCCF; /*PROCESS REQUEST*/       0474
*                                                                  0474
         L     @11,ATCLCPTR                                        0474
         L     @15,ATCCDFND(,@11)                                  0474
         BALR  @14,@15                                             0474
*   RFY (EMCR2,EMCR3) UNRSTD;                                      0475
*   END; /*CIDCTL*/           /*GET FMCB&NCB ADDRS &   */          0476
*     END;                                                         0477
*    ELSE DO;                            /*  NO,LOAD DNCB @ FROM FMCB *
         B     @RC00462                                            0478
@RF00462 DS    0H                                                  0479
*      RFY (R0,FMCBSVR5) RSTD;                                     0479
*      R0 = FMCDNCBA;                    /* GET DNCB PTR */        0480
         L     R0,FMCDNCBA(,FMCBR2)                                0480
*      FMCBSVR5 = FMCBR2;                /* SAVE FMCB PTR */       0481
         LR    FMCBSVR5,FMCBR2                                     0481
*     END;                                                         0482
*  RFY NCBPTR RSTD;                                                0483
@RC00462 DS    0H                                                  0484
*  IF RTNCODE=ZERO                          /* DAF LOCK OBTAINED */
*    THEN DO;                               /* YES-GET CORE FOR TRAC */
         LTR   RTNCODE,RTNCODE                                     0484
         BNZ   @RF00484                                            0484
*      NCBPTR=R0;                                                  0486
         LR    NCBPTR,R0                                           0486
*      RFY R0 UNRSTD;                                              0487
*       /*REQSTORE CBID(FMCB)CLEAR(YES)SAVE(YES)*/                 0488
*        DO;                                                       0488
*/* ATBUILD (ISTDYPAB,ISTATCVT,ISTRPH)  */                         0489
*/* END OF ATBUILD */                                              0489
*                                                                  0489
*       RFY (GPR02P,GPR03P,GPR04P,GPR15P) RSTD;                    0489
*                                                                  0489
*       RFY (GPR01P,GPR13P) RSTD;                                  0490
*          GENERATE REFS(GPR01P,GPR02P,GPR13P) SETS(RPHWORK);      0491
*                                                                  0491
           STM GPR02P,GPR13P,RPHWORK(GPR01P)
*       RFY (GPR01P,GPR13P) UNRSTD;                                0492
*                                                                  0492
*       DO; /* LISATCVT CVTBASE(PAGEABLE,(GPR15P)) */              0493
*         GPR15P = ATCLCPTR; /*SET UP ATCVT POINTER*/              0494
         L     GPR15P,ATCLCPTR                                     0494
*       END; /*LISATCVT*/                                          0495
*          GPR02P=GPR15P->ATCBPDA | '80000000'X;                   0496
         L     GPR02P,ATCBPDA(,GPR15P)                             0496
         O     GPR02P,@CF01359                                     0496
*          DCL 1 FMZA0010 FIXED(31) STATIC LOCAL,                  0497
*              2 *        FIXED(8) INIT(152),                      0497
*              2 *        PTR(24) INIT(0);                         0497
*          GPR03F=FMZA0010;                                        0498
         L     GPR03F,FMZA0010                                     0498
*          GPR04F=25;                                              0499
         LA    GPR04F,25                                           0499
*          GPR15P=GPR15P->ATCSMRQ;                                 0500
         L     GPR15P,ATCSMRQ(,GPR15P)                             0500
*          DCL FMZL0010 BASED(GPR15P) LABEL VALUERANGE(*);         0501
*          CALL FMZL0010;                                          0502
*                                                                  0502
         BALR  @14,@15                                             0502
*       RFY (GPR13P) RSTD;                                         0503
*          GENERATE REFS(GPR01P,RPHWORK) SETS(GPR02P,GPR13P);      0504
*                                                                  0504
           LM GPR02P,GPR13P,RPHWORK(GPR01P)
*       RFY (GPR13P) UNRSTD;                                       0505
*                                                                  0505
*       RFY (GPR02P,GPR03P,GPR04P,GPR15P) UNRSTD;                  0506
*       END ;                                                      0507
*          /********* END OF REQSTORE EXPANSION **********/        0508
*      IF RTNCODE<EIGHT                     /*  CORE AVAILABLE?      */
*        THEN DO;                           /*   YES-SETUP HEADER    */
         LA    @04,8                                               0508
         CR    RTNCODE,@04                                         0508
         BNL   @RF00508                                            0508
*          RTNCODE=ZERO;                    /*    SET GOOD RTN CODE  */
         SLR   @03,@03                                             0510
         LR    RTNCODE,@03                                         0510
*          TRACPTR=RPHSRPRM;                /*    SETUP TRAC BASE    */
         L     TRACPTR,RPHSRPRM(,RPHPTR)                           0511
*          IF PROCFTX='1'B                  /*    CONFIDENTIAL DATA? */
*            THEN DO;                       /*     YES-SETUP ENTIRE  */
         TM    PROCFTX+48(FMCBR2),B'00000001'                      0512
         BNO   @RF00512                                            0512
*              RFY ISTRPRE BASED(NCBRDTE);  /*      TRAC HEADER.     */
*              TRALRC=NCBLTRC;              /*      LOST RECORD CNT  */
         LH    @00,NCBLTRC(,NCBPTR)                                0515
         STH   @00,TRALRC(,TRACPTR)                                0515
*              NCBLTRC=ZERO;                /*      REINITIALIZE     */
         STH   @03,NCBLTRC(,NCBPTR)                                0516
*              TRADNODE=RPRNAME;            /*      DEST. NODE NAME  */
         L     @03,NCBRDTE(,NCBPTR)                                0517
         MVC   TRADNODE(8,TRACPTR),RPRNAME(@03)                    0517
*              TRADNODF='1'B;               /*      DEST NODENAME FLG*/
         OI    TRADNODF(TRACPTR),B'00000100'                       0518
*              RFY ISTRPRE BASED(ACDRDTE);  /*      FMCB->ACD->RDTE  */
*              TRAANODE=RPRNAME;            /*      APPLICATION NAME */
         L     @03,FMCDEBA(,FMCBR2)                                0520
         L     @03,ACDRDTE(,@03)                                   0520
         MVC   TRAANODE(8,TRACPTR),RPRNAME(@03)                    0520
*              TRAANODF='1'B;               /*      APPL NAME FLAG   */
         OI    TRAANODF(TRACPTR),B'00000010'                       0521
*              TRADATCT=LENGTH(TRAANODE);   /*      BUFFER LENGTH    */
         STH   @04,TRADATCT(,TRACPTR)                              0522
*              TRACXTF='1'B;                /*      CONFIDENTIAL DATA*/
         OI    TRACXTF(TRACPTR),B'00100000'                        0523
*             END;                          /* NO-SAVE CID/FMCB IN */
*            ELSE                           /*      TRAC FOR LATER   */
*              TRACID=R5;                   /*      TO FIND DNCB    */
         B     @RC00512                                            0525
@RF00512 ST    R5,TRACID(,TRACPTR)                                 0525
*          TRABF='1'B;                      /*    BUFFER TRACE       */
@RC00512 DS    0H                                                  0527
*          TRAFRST='1'B;                    /*    FIRST BUFFER       */
         OI    TRABF(TRACPTR),B'01001000'                          0527
*          TRATYPE=TRTYPE;                  /*    C/B TYPE = TRAC    */
         MVI   TRATYPE(TRACPTR),X'13'                              0528
*          TRALNG=LENGTH(ISTTRAC);          /*    LENGTH             */
         MVI   TRALNG(TRACPTR),X'20'                               0529
*                                           /*    SET I/O DIRECTION  */
*          IF (R6INPUT&INPTMSK)=FDBINPUT    /*    BASED ON R6 INPUT: */
*            THEN                           /*     TPIOS FDB CALL    */
         LR    @04,R6INPUT                                         0530
         N     @04,INPTMSK                                         0530
         CL    @04,FDBINPUT                                        0530
         BNE   @RF00530                                            0530
*              TRADIO='1'B;                 /*     IS ALWAYS INBOUND */
         OI    TRADIO(TRACPTR),B'00001000'                         0531
*            ELSE DO;                       /*    CONTROL LAYER INPUT*/
         B     @RC00530                                            0532
@RF00530 DS    0H                                                  0533
*              IF (R6INPUT&INPTMSK)=ZERO    /*     OUTBOUND?         */
*                THEN;                      /*      YES-FLG=0(OUTBND)*/
         LR    @04,R6INPUT                                         0533
         N     @04,INPTMSK                                         0533
         LTR   @04,@04                                             0533
         BZ    @RT00533                                            0533
*                ELSE                       /*      NO-FLG=1(INBOUND)*/
*                  TRADIO='1'B;                                    0535
         OI    TRADIO(TRACPTR),B'00001000'                         0535
*             END;                                                 0536
*         END;                                                     0537
*        ELSE                               /*   REQSTORE FAILURE-   */
*          NCBLTRC=NCBLTRC+1;               /*   INCR. LOST RCD CNT  */
         B     @RC00508                                            0538
@RF00508 LA    @04,1                                               0538
         AH    @04,NCBLTRC(,NCBPTR)                                0538
         STH   @04,NCBLTRC(,NCBPTR)                                0538
*      IF (R6INPUT&INPTMSK) = FDBINPUT                             0539
*        THEN DO;                           /* INPUT IS A FDB */   0539
@RC00508 LR    @04,R6INPUT                                         0539
         N     @04,INPTMSK                                         0539
         CL    @04,FDBINPUT                                        0539
         BNE   @RF00539                                            0539
*      R2=CIDSAVR5;                         /*  GET CID              */
         LR    R2,CIDSAVR5                                         0541
*      RFY SAVRCR5 RSTD;                    /*  SAVE REQSTORE RTNCODE*/
*      SAVRCR5=RTNCODE;                     /*  ACROSS CIDCTL.       */
*                                                                  0543
         LR    SAVRCR5,RTNCODE                                     0543
*       DO; /* CIDCTL(FINISH)  */                                  0544
*                                                                  0544
*       RFY (EMCR2,EMCR3) RSTD;                                    0545
*         GEN SETS(RPHWORK)                                        0546
*        (STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH);      0546
         STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH
*         EMCR3 = '00'X; /*CLEAR EXTRA BITS*/                      0547
         SLR   EMCR3,EMCR3                                         0547
*         CALL ATCLCPTR->ATCCDFIN->ISTSDCCR; /*PROCESS REQUEST*/   0548
*                                                                  0548
         L     @04,ATCLCPTR                                        0548
         L     @15,ATCCDFIN(,@04)                                  0548
         BALR  @14,@15                                             0548
*       RFY (EMCR2,EMCR3) UNRSTD;                                  0549
*       END; /*CIDCTL*/                    /*  UNLOCK THE DAFLOCK.  */
*      RTNCODE=SAVRCR5;                     /*  RESTORE RETURN CODE  */
         LR    RTNCODE,SAVRCR5                                     0551
*             END;                                                 0552
*        ELSE;                            /* DAF LOCK WAS NOT HELD */
@RF00539 DS    0H                                                  0554
*     END;                                                         0554
*    ELSE;                                  /* ERROR ON CID FIND     */
@RF00484 DS    0H                                                  0556
*  RFY (FMCBR2,R2,NCBPTR,CIDSAVR5,SAVRCR5,FMCBSVR5) UNRSTD;        0556
*                                                                  0556
*/*******************************************************************/
*      IF RTNCODE=ZERO                            /*SUCCESSFUL?      */
*        THEN DO;                                 /* YES-PROCEED     */
         LTR   RTNCODE,RTNCODE                                     0557
         BNZ   @RF00557                                            0557
*          IF TRACXTF='0'B                        /*CONFIDENTIAL DATA*/
*            THEN DO;                             /* NO-PROCEED      */
         TM    TRACXTF(TRACPTR),B'00100000'                        0559
         BNZ   @RF00559                                            0559
*              IF (R6INPUT&INPTMSK)=FDBINPUT      /*IS CALLER TPIOS? */
*                THEN DO;                         /* YES-FDB INPUT   */
         LR    @11,R6INPUT                                         0561
         N     @11,INPTMSK                                         0561
         CL    @11,FDBINPUT                                        0561
         BNE   @RF00561                                            0561
*/********************************************************************/
*/*                                                                  */
*/*  ROUTINE NAME : TBSFDB - TPIOS FDB INPUT PROCESSOR (DOS ONLY)    */
*/*                                                                  */
*/*  FUNCTION : TO PROCESS TPIOS INBOUND INPUT. MOVES DATA FROM      */
*/*             FDBS UNTIL THE TRAC BUFFER IS FILLED OR UNTIL ALL    */
*/*             FDB DATA IS MOVED, WHICHEVER OCCURS FIRST.           */
*/*                                                                  */
*/*  INPUT : R6INPUT-ADDRESS OF FIRST FDB. FDBS ARE CHAINED, AND THE */
*/*                  END OF THE CHAIN IS ZERO.                       */
*/*                                                                  */
*/*  OUTPUT : A COMPLETED TRACE BUFFER.                              */
*/*           TRADATCT=LENGTH OF DATA IN THE BUFFER.                 */
*/*                                                                  */
*/*  ROUTINES : NONE                                                 */
*/*                                                                  */
*/*  MACROS : NONE                                                   */
*/*                                                                  */
*/********************************************************************/
*  RFY (BUFNX,LENR4) RSTD;                                         0563
*  RFY FDBR2 RSTD;                      /*SETUP FDB BASE-FIRST FDB   */
*  RFY ISTFDB BASED(FDBR2);             /*ADDRESS IS AN INPUT        */
*  FDBR2=R6INPUT;                       /*PARAMETER.                 */
         LR    FDBR2,R6INPUT                                       0566
*  DO WHILE (TRADATCT<BUFCT) & (FDBR2^=ZERO); /* FILL BUFFER UNTIL   */
         B     @DE00567                                            0567
@DL00567 DS    0H                                                  0568
*                                       /*FULL OR NO MORE INPUT      */
*    IF FDBDATCT>(BUFCT-TRADATCT)       /*DATA TO MOVE>BUFFER LEFT?  */
*      THEN                             /* YES-SET LENGTH TO MOVE =  */
         L     @11,BUFCT                                           0568
         SH    @11,TRADATCT(,TRACPTR)                              0568
         CH    @11,FDBDATCT(,FDBR2)                                0568
         BNL   @RF00568                                            0568
*        LENR4=(BUFCT-TRADATCT);        /*  SIZE OF REMAINING BUFFER */
         LR    LENR4,@11                                           0569
*      ELSE                             /* NO-SET LENGTH TO MOVE = TO*/
*        LENR4=FDBDATCT;                /*  DATA SIZE IN FDB         */
         B     @RC00568                                            0570
@RF00568 LH    LENR4,FDBDATCT(,FDBR2)                              0570
*    BUFNX=ADDR(TRABFFER)+TRADATCT;     /*ADDRESS TO MOVE TO IN TRAC */
@RC00568 LA    BUFNX,TRABFFER(,TRACPTR)                            0571
         LH    @11,TRADATCT(,TRACPTR)                              0571
         ALR   BUFNX,@11                                           0571
*    BUFNX->BUFDATA(1:LENR4)=FDBDATAA->BUFDATA(1:LENR4);  /*MOVE DATA*/
         LR    @03,LENR4                                           0572
         BCTR  @03,0                                               0572
         L     @09,FDBDATAA(,FDBR2)                                0572
         EX    @03,@SM04453                                        0572
*    FDBR2=FDBFDBA;                     /*GET ADDR OF NEXT FDB       */
         L     FDBR2,FDBFDBA(,FDBR2)                               0573
*    TRADATCT=LENR4+TRADATCT;           /*INCR.DATA COUNT BY AMT MOVD*/
         ALR   @11,LENR4                                           0574
         STH   @11,TRADATCT(,TRACPTR)                              0574
*  END;                                 /*  E N D  L O O P           */
@DE00567 LH    @11,TRADATCT(,TRACPTR)                              0575
         C     @11,BUFCT                                           0575
         BNL   @DC00567                                            0575
         LTR   FDBR2,FDBR2                                         0575
         BNZ   @DL00567                                            0575
@DC00567 DS    0H                                                  0576
*  RFY (LENR4,FDBR2,BUFNX) UNRSTD;                                 0576
*                                                                  0576
*/********************************************************************/
*                 END;                                             0577
*                ELSE DO;                         /* NO-RPL INPUT    */
         B     @RC00561                                            0578
@RF00561 DS    0H                                                  0579
*/********************************************************************/
*/*                                                                  */
*/*  ROUTINE NAME : TBSRPL - CONTROL LAYER RPL INPUT PROCESSOR       */
*/*                                                                  */
*/*  FUNCTION : TO CHANGE THE PROTECTION KEY AND TO OBTAIN THE LOCAL */
*/*             MEMORY LOCK NECESSARY TO PROCESS USER DATA. TO       */
*/*             INVOKE THE PROPER ROUTINE DEPENDING ON THE RPL       */
*/*             COMMAND CODE.                                        */
*/*                                                                  */
*/*  INPUT : R6INPUT - ADDRESS OF RPL                                */
*/*                                                                  */
*/*  OUTPUT : RTNCODE = 0 NO ERRORS                                  */
*/*                  ^= 0 WHEN AN ERROR OCCURS IN A VALCHECK ISSUED  */
*/*                       BY TBSRWSR OR TBSLDO. OR TBSLDO FOUND AN   */
*/*                       ERROR IN LDO JUMP COMMANDS.                */
*/*  ROUTINES : TBSRWSR-CALLED TO HANDLE READ/WRITE/SEND/RECEIVE     */
*/*                     INPUT.                                       */
*/*             TBSLDO-CALLED TO HANDLE LDO INPUT.                   */
*/*  MACROS : VMDESET-TO SET AND RESET THE PROTECTION KEY FROM VTAM  */
*/*                   PROTECTION KEY TO SUPERVISOR. NO-OP IN DOS.    */
*/*           VSETLCK-TO SET AND RESET THE LOCAL MEMORY LOCK IN VS2. */
*/*                   NO-OP IN DOS AND VS1.                          */
*/*                                                                  */
*/********************************************************************/
*  RFY IFGRPL BASED(R6INPUT);           /*RPL ADDRESS IS INPUT PARM. */
*  RFY R13 RSTD;                        /*SAVE PROTECT KEY IN REG2 . */
*/*VMDESET  ((RTNCODE),ZERO,,)*/                                   0581
*   DO;                                  /* INVOKE MODESET MACRO    */
*     GEN;                                                         0582
         MODESET EXTKEY=ZERO,WORKREG=2
*     RTNCODE=0;                         /*PUT RETURN CODE IN RCREG*/
         SLR   RTNCODE,RTNCODE                                     0583
*   END;  /* VMDESET */         /*SET PROTECT KEY TO 0 (VS2).*/    0584
*  R13=ADDR(RPHSAVE1);                  /*5 WORD SAVEAREA FOR VSETLCK*/
         LA    R13,RPHSAVE1(,RPHPTR)                               0585
*  TBLOCK:                              /*SET LOCAL MEMORY LOCK (VS2)*/
*                                                                  0586
*/*VSETLCK  (RTNCODE,OBTAIN,LOCAL,(LOCAL,ISTRACTB(TBUNLOCK)),)*/   0586
*    DO;                                                           0586
TBLOCK   DS    0H                                                  0587
*                                        /*DECLARE AND RESTRICT REGS */
*                                        /*   2,10,11,12,13,14, AND 15
**/                                                                0587
*    RFY (GPR02P,GPR10P,GPR11P,GPR12P,GPR13P,GPR14P,GPR15P) RSTD;  0587
*                                        /*SAVE REGS 2 AND 10 THRU 14*/
*        GEN REFS(GPR02F,GPR10F,GPR11F,GPR12F,GPR14F,GPR13P);      0588
*                                                                  0588
         STM   GPR10F,GPR12F,0(GPR13P)
         ST    GPR14F,12(GPR13P)
         ST    GPR02F,16(GPR13P)
*                                        /*INVOKE MODESET TO CHANGE  */
*                                        /*PSW PROTECTION KEY TO 0   */
*        GEN SETS(2);                                              0589
*                                                                  0589
         MODESET EXTKEY=ZERO,SAVEKEY=(2),WORKREG=2
*      GPR10P=GPR13P;                    /*PUT SAVE AREA ADDR IN R 10*/
         LR    GPR10P,GPR13P                                       0590
*                                        /*INVOKE OS/VS SETLOCK MACRO*/
*      GEN REFS(GPR13P,PSALITA,FLC) SETS(GPR12F);                  0591
         SETLOCK OBTAIN,TYPE=LOCAL,MODE=UNCOND,                        *
               RELATED=(LOCAL,ISTRACTB(TBUNLOCK))
*      GPR15P=GPR13P;                    /*PUT RETURN CODE IN R 15 */
         LR    GPR15P,GPR13P                                       0592
*                                        /*INVOKE MODESET MACRO TO   */
*        DO;                             /*RESTORE PSW PROTECTION KEY*/
*        GEN SETS(2);                                              0594
         MODESET KEYADDR=(2),WORKREG=2
*        END;                            /* RESTORE REGISTERS        */
*      GEN REFS(GPR10F,GPR13P) SETS(GPR10F,GPR11F,                 0596
*      GPR12F,GPR13F,GPR14F,GPR02F);                               0596
         LR    GPR13F,GPR10F
         LM    GPR10F,GPR12F,0(GPR13P)
         L     GPR14F,12(GPR13P)
         L     GPR02F,16(GPR13P)
*      IF GPR15F=4 THEN GPR15F=0;                                  0597
         C     GPR15F,@CF00037                                     0597
         BNE   @RF00597                                            0597
         SLR   GPR15F,GPR15F                                       0598
*      RTNCODE=GPR15F;                   /*PUT RETURN CODE IN RCREG*/
*                                                                  0599
@RF00597 DS    0H                                                  0600
*    RFY (GPR02P,GPR10P,GPR11P,GPR12P,GPR13P,GPR14P,GPR15P) UNRSTD;
*    END;  /*VSETLCK */                                            0601
*  IF RPLREQ='13'X                      /*RPL COMMAND CODE=LDO?      */
*    THEN DO;                           /* YES-PROCESS LOGICAL DEVICE*/
         CLI   RPLREQ(R6INPUT),19                                  0602
         BNE   @RF00602                                            0602
*/********************************************************************/
*/*                                                                  */
*/*  ROUTINE NAME : TBSLDO - CONTROL LAYER LDO INPUT PROCESSOR       */
*/*                                                                  */
*/*  FUNCTION : TO FILL THE TRACE BUFFER WITH DATA READ OR WRITTEN BY*/
*/*             LDOS.                                                */
*/*                                                                  */
*/*  INPUT : R6INPUT - RPL POINTER                                   */
*/*                                                                  */
*/*  OUTPUT : R15 = 0 SUCCESSFUL FILLING OF TRACE DATA BUFFER. NO    */
*/*                   ERRORS OCCURED IN VALCHEK OF LDOS OR LDO DATA  */
*/*                   ADDRESSES, AND NO ERROR IN LDO JUMP COMMANDS.  */
*/*                   ERROR CHECKING IS DONE ONLY UNTIL THE TRACE    */
*/*                   BUFFER IS FILLED.                              */
*/*           R15^= 0 AN LDO ADDRESS OR AN LDO DATA ADDRESS WAS NOT A*/
*/*                   VALID USER AREA, OR AN LDO JUMP POINTED TO     */
*/*                   ANOTHER JUMP COMMAND  OR NO DATA COPIED INTO   */
*/*                   THE TRAC BECAUSE  NO VALID LDO WAS FOUND.      */
*/*  ROUTINES : NONE                                                 */
*/*                                                                  */
*/*  MACROS : VALCHEK - TO VALIDATE A USER LDO ADDRESS OR AN LDO'S   */
*/*                     DATA ADDRESS                                 */
*/*  NOTES ON LDOS :                                                 */
*/*    -'LDOAREA' POINTS TO I/O DATA AREA UNLESS LDOCMD=LDOJUMP,     */
*/*     THEN LDOAREA POINTS TO NEXT LDO.                             */
*/*    -LDOS ARE BUILT ONE AFTER ANOTHER UNTIL LDOCMD=LDOJUMP,THEN   */
*/*     NEXT LDO IS POINTED TO BY LDOAREA.                           */
*/*    -EITHER LDOACHN=1 OR LDOCMHN=1 MEANS THERE IS ANOTHER LDO     */
*/*     FOLLOWING. IF BOTH=0, THERE ARE NO LDOS FOLLOWING. THESE BITS*/
*/*     ARE NOT APPLICABLE TO LDOS WITH LDOJUMP CMD.                 */
*/*    -IF LDOACHN=1(DATA-CHAINED), THE APPROPRIATE LDOCMD WILL BE   */
*/*     FOUND ONLY IN THE FIRST LDO OF THE CHAIN.                    */
*/*                                                                  */
*/********************************************************************/
*  RFY (LENR4,DATAR9,R11FLG,BUFNX,LDOR8,RBUFCT) RSTD;              0604
*  DATAR9=ZERO;                                                    0605
         SLR   @03,@03                                             0605
         LR    DATAR9,@03                                          0605
*  LDOR8=RPLAREA;                     /*FIRST LDO ADDRESS            */
         L     LDOR8,RPLAREA(,R6INPUT)                             0606
*  R11FLG=JUMP;                       /*SET JUMP FLG TO INSURE 1ST   */
         LA    R11FLG,4                                            0607
*  RTNCODE=ZERO;                      /*LDO NOT A JUMP. INITIALIZE RC*/
         LR    RTNCODE,@03                                         0608
*  RBUFCT=BUFCT;                      /*INITIALIZE TOTAL-DATA-TO-MOVE*/
         L     RBUFCT,BUFCT                                        0609
*  IF TRADIO='1'B                     /*INBOUND I/O?                 */
*    THEN                             /* YES-ACTUAL DATA READ IS IN  */
         TM    TRADIO(TRACPTR),B'00001000'                         0610
         BNO   @RF00610                                            0610
*      IF RBUFCT<RPLRLEN              /*  RPL. IF<TOTAL BUFFER SIZE  */
*        THEN;                        /*  SET TOTAL-DATA-TO-MOVE TO  */
         C     RBUFCT,RPLRLEN(,R6INPUT)                            0611
         BL    @RT00611                                            0611
*        ELSE RBUFCT=RPLRLEN;         /*  RPL VALUE.                 */
         L     RBUFCT,RPLRLEN(,R6INPUT)                            0613
*    ELSE;                                                         0614
@RF00610 DS    0H                                                  0615
*  DO WHILE (RTNCODE=ZERO)            /*LOOP TO FILL BUFFER WHILE NO */
*      & (TRADATCT<RBUFCT);           /*ERRORS, BUFFR NOT FULL       */
@RC00610 B     @DE00615                                            0615
@DL00615 DS    0H                                                  0616
*/*VALCHEK  (RTNCODE,RPHSAVE1,(LDOR8),UA,LENGLDO)  RANGE*/         0616
*    DO;                                                           0616
*                                                                  0616
*    RFY (GPR01P,GPR02P,GPR04P,GPR06P,GPR13P,GPR14P,GPR15P) RSTD;  0617
*                                        /*DECLARE AND RESTRICT REGS*/
*                                        /*  1, 2, 4, 6, 13,14, AND 15
*/     RTNCODE=0;                       /*SET RETURN CODE TO NO ERROR*/
         SLR   RTNCODE,RTNCODE                                     0618
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY SAVEAREA OPERAND */
*                                        /*AND PUT IT IN REG 13    */
*                                                                  0619
*/*XREGSET  PID('SAVEAREA')PPARM(RPHSAVE1)RREG(13,GPR13P)*/        0619
*    DO;                                                           0619
*      GPR13P=ADDR(RPHSAVE1);            /*SET RREG SYMBOL TO ADDRESS*/
         LA    GPR13P,RPHSAVE1(,RPHPTR)                            0620
*                                        /*OF PPARM OPERAND          */
*    END;  /* XREGSET */                                           0621
*      DCL 1 SAV0037 CHAR(20) BASED(GPR13P),                       0622
*                                        /*ESTABLISH SAVE AREA MAP   */
*            2 SAV10037 CHAR(4),         /*TO SAVE REGS 1,2,4,6, AND 14
*/           2 SAV20037 CHAR(4),                                   0622
*            2 SAV40037 CHAR(4),                                   0622
*            2 SAV60037 CHAR(4),                                   0622
*            2 SAVE0037 CHAR(4);                                   0622
*      SAV10037=GPR01F;                  /* SAVE REGS ONE,     */  0623
         STCM  GPR01F,15,SAV10037(GPR13P)                          0623
*      SAV20037=GPR02F;                  /*           TWO,     */  0624
         STCM  GPR02F,15,SAV20037(GPR13P)                          0624
*      SAV40037=GPR04F;                  /*          FOUR,     */  0625
         STCM  GPR04F,15,SAV40037(GPR13P)                          0625
*      SAV60037=GPR06F;                  /*          SIX,      */  0626
         STCM  GPR06F,15,SAV60037(GPR13P)                          0626
*      SAVE0037=GPR14F;                  /*     AND FOURTEEN   */  0627
         STCM  GPR14F,15,SAVE0037(GPR13P)                          0627
*      GPR04P=GPR01P->RPHTSKID;          /*PUT ADDR OF PROCESS SCHED-*/
         L     GPR04P,RPHTSKID(,GPR01P)                            0628
*                                        /*ULING TABLE IN REG 4      */
*                                                                  0629
*/* ATBUILD (ISTPST)  */                                           0629
*/* ***** PROCESS SCHEDULING TABLE FOR VS/2       ****** */        0629
*/*  INITIAL LEVEL                                      001 04/09/75 */
*DCL  1  ISTPST BDY(DWORD) BASED,                                  0629
*        2  PSTTYPE  CHAR(1),          /* CONTROL BLOCK TYPE         */
*        2  PSTLNGTH PTR(8),           /* LENGTH IN BYTES            */
*        2  *        CHAR(2),          /* RESERVED                   */
*        2  PSTCHAIN PTR(31),          /* ADDRESS OF NEXT PST        */
*        2  PSTRQFGA CHAR(8) BDY(DWORD),  /* READY QUEUE AND FLAGS   */
*           3  PSTRQUE PTR(31),        /* READY QUEUE                */
*              4  PSTRQFLG BIT(1),     /* READY QUEUE GATE           */
*              4  PSTRSV01 BIT(1),     /* REVERVED                   */
*              4  PSTRSV02 BIT(1),     /* REVERVED                   */
*              4  PSTRSV03 BIT(1),     /* REVERVED                   */
*              4  PSTRSV04 BIT(1),     /* REVERVED                   */
*              4  PSTRSV05 BIT(1),     /* REVERVED                   */
*              4  PSTRSV06 BIT(1),     /* REVERVED                   */
*              4  PSTRSV07 BIT(1),     /* REVERVED                   */
*              4  PSTRQUEA PTR(24),    /* READY QUEUE                */
*           3  PSTFLGA PTR(32),        /* FLAG FIELD                 */
*              4  PSTSSRN BIT(1),      /* IRB RUNNING SYSTEM SERVICE
*                                         TYPE REQUESTS              */
*              4  PSTUERN BIT(1),      /* IRB RUNNING USER EXIT TYPE
*                                         REQUESTS                   */
*              4  PSTIRBU1 BIT(1),     /* IRB # 1 IN USE             */
*              4  PSTIRBU2 BIT(1),     /* IRB # 2 IN USE             */
*              4  PSTSMQR BIT(1),      /* SMS RPH QUEUED FOR STORAGE */
*              4  PSTSMCU BIT(1),      /* SMS CRA IN USE             */
*              4  PSTAUTH BIT(1),      /* THE TASK THIS PST IS OBTAINED
*                                         FOR IS AUTHORIZED          */
*              4  PSTABINP BIT(1),     /* ABEND IN PROGRESS ON THIS
*                                         TASK                       */
*                 5 PSTABEND BIT(1),   /* TASK ABEND IN PROGRESS     */
*              4  PSTDMPC BIT(1),      /* DUMP HAS BEEN REQUESTED    */
*              4  PSTRSV08 BIT(23),    /* RESERVED                   */
*        2  PSTTCBA PTR(31),           /* TCB POINTER                */
*        2  PSTMPSTP PTR(31),          /* MPST POINTER - PST 6TH WORD*/
*        2  PSTACBCT FIXED(31),        /* COUNT OF OPEN ACB'S USING
*                                         THIS PST                   */
*        2  PSTPSSQ PTR(31),           /* POSTED SYSTEM SERVICE QUEUE*/
*        2  PSTNSSQ PTR(31),           /* NORMAL SYSTEM SERVICE QUEUE*/
*        2  PSTUEQ  PTR(31),           /* USER EXIT QUEUE            */
*        2  PSTIQE11 PTR(31),          /* INACTIVE IQE FOR FIRST IRB */
*        2  PSTIQE12 PTR(31),          /* ACTIVE IQE FOR FIRST IRB   */
*        2  PSTIQE21 PTR(31),          /* INACTIVE IQE FOR 2ND IRB   */
*        2  PSTIQE22 PTR(31),          /* ACTIVE IQE FOR 2ND IRB     */
*        2  PSTCRR1 PTR(31),           /* PREALLOCATED CRA           */
*        2  PSTCRRS PTR(31),           /* SMS CRA                    */
*        2  PSTSMDYP CHAR(LENGTH(ISTDYPAB)) BDY(DWORD), /* SMS PAB   */
*        2  PSTSAVE CHAR(32),          /* SAVE AREA                  */
*        2  PSTDEBP  PTR(31),          /* POINTER TO DEB CHAIN       */
*                                      /* WHEN TASK ABENDS           */
*        2  PSTDMPS  FIXED(31),        /* DUMMY MPST                 */
*           3 PSTDGATE BIT(1),         /* DUMMY MPST GATE BIT        */
*           3 *        BIT(7),         /* NOT TO BE USED             */
*           3 PSTDRDYQ PTR(24);        /* DUMMY MPST READY QUEUE     */
*DCL  PSTSSRN0 BIT(32) CONSTANT('7FFFFFFF'X); /* TURN PSTSSRN OFF    */
*DCL  PSTSSRN1 BIT(32) CONSTANT('80000000'X); /* TURN PSTSSRN ON     */
*DCL  PSTUERN0 BIT(32) CONSTANT('BFFFFFFF'X); /* TURN PSTUERN OFF    */
*DCL  PSTUERN1 BIT(32) CONSTANT('40000000'X); /* TURN PSTUERN ON     */
*DCL  PSTGATE0 BIT(32) CONSTANT('7FFFFFFF'X); /* TURN PST READY QUEUE
*                                                GATE OFF            */
*DCL  PSTGATE1 BIT(32) CONSTANT('80000000'X); /* TURN PST READY QUEUE
*                                                GATE ON             */
*DCL  PSTIRB10 BIT(32) CONSTANT('DFFFFFFF'X); /* TURN PSTIRBU1 OFF   */
*DCL  PSTIRB11 BIT(32) CONSTANT('20000000'X); /* TURN PSTIRBU1 ON    */
*DCL  PSTIRB20 BIT(32) CONSTANT('EFFFFFFF'X); /* TURN PSTIRBU2 OFF   */
*DCL  PSTIRB21 BIT(32) CONSTANT('10000000'X); /* TURN PSTIRBU2 ON    */
*DCL  PSTSMQR0 BIT(32) CONSTANT('F7FFFFFF'X); /* TURN PSTSMQR OFF    */
*DCL  PSTSMQR1 BIT(32) CONSTANT('08000000'X); /* TURN PSTSMQR ON     */
*DCL  PSTSMCU0 BIT(32) CONSTANT('FBFFFFFF'X); /* TURN PSTSMCU OFF    */
*DCL  PSTSMCU1 BIT(32) CONSTANT('04000000'X); /* TURN PSTSMCU ON     */
*DCL  PSTABIP1 BIT(32) CONSTANT('01000000'X); /* TURN PSTABINP ON    */
*DCL  PSTABIP0 BIT(32) CONSTANT('FEFFFFFF'X); /* TURN OFF PSTABINP   */
*DCL  PSTYPE   BIT(8) CONSTANT('61'X);        /* TYPE CODE FOR PST   */
*/* END OF ATBUILD */                    /*INCLUDE PROCESS SCHEDULING *
*                                        /*   TABLE (ISTPST)         */
*      GPR15P=GPR04P->PSTTCBA;           /*PUT TASK CONTROL BLOCK    */
         L     GPR15P,PSTTCBA(,GPR04P)                             0647
*                                        /*ADDRESS IN REGISTER 15    */
*      GPR04F=SAV40037;                  /*RESTORE REG 4*/         0648
         ICM   @03,15,SAV40037(GPR13P)                             0648
         LR    GPR04F,@03                                          0648
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY USER AREA OPERAND*/
*                                        /*AND PUT ADDR IN REG 1   */
*                                                                  0649
*/*XREGSET  PID('USER AREA')PPARM((LDOR8))RREG(,GPR01P)*/          0649
*    DO;                                                           0649
*      GPR01P=LDOR8;                     /*SET RREG SYMBOL TO VALUE*/
         LR    GPR01P,LDOR8                                        0650
*                                        /*OF PPARM OPERAND        */
*    END;  /* XREGSET */                                           0651
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY LENGTH OPERAND   */
*                                        /*AND PUT VALUE IN REG 14 */
*                                                                  0652
*/*XREGSET  PID('LENGTH')PPARM(LENGLDO)RREG(,GPR14F)OUTPUT(PARM)*/ 0652
*    DO;                                                           0652
*      GPR14F=LENGLDO;                   /*SET RREG SYMBOL TO VALUE*/
         L     GPR14F,LENGLDO                                      0653
*                                        /*OF PPARM OPERAND        */
*    END;  /* XREGSET */                                           0654
*        GEN SETS(2);                                              0655
         MODESET EXTKEY=ZERO,SAVEKEY=(2),WORKREG=2
*      GPR06P=GPR02P;                                              0656
         LR    GPR06P,GPR02P                                       0656
*      GPR04P=GPR15P;                    /*RESTORE TCB ADDR*/      0657
         LR    GPR04P,GPR15P                                       0657
*      GPR02F=GPR01F+GPR14F-1;           /*PUT END ADDRESS IN REG 2  */
         LR    GPR02F,GPR01F                                       0658
         ALR   GPR02F,GPR14F                                       0658
         BCTR  GPR02F,0                                            0658
*      DCL CZMP0037 PTR(31) LOCATION(16);/*ESTABLISH POINTER TO CVT  */
*      CALL CZMP0037->CVT0VL00;          /*INVOKE TASK SUPERVISOR    */
         L     @03,CZMP0037                                        0660
         L     @15,CVT0VL00(,@03)                                  0660
         BALR  @14,@15                                             0660
*                                        /*VALIDITY CHECKING ROUTINE */
*                                        /*UPON RETURN, CHECK PSW    */
*                                        /*CONDITION CODE SET BY THE */
*                                        /*VALIDITY CHECKING ROUTINE */
*      BC (8,OKAY0037);                  /*IF ADDR VALID, DON'T ALTER*/
         BC    8,OKAY0037                                          0661
*                                        /*RETURN CODE VALUE FROM 0  */
*      RTNCODE=4;                        /*SET RETURN CODE TO ERROR  */
         LA    RTNCODE,4                                           0662
*      GO TO VC30037;                                              0663
         B     VC30037                                             0663
*OKAY0037:                                                         0664
*      RTNCODE=0;                        /*PUT RETURN CODE IN RCREG  */
OKAY0037 SLR   RTNCODE,RTNCODE                                     0664
*VC30037:                                                          0665
*      GPR02P=GPR06P;                                              0665
VC30037  LR    GPR02P,GPR06P                                       0665
*        GEN SETS(2);                                              0666
         MODESET KEYADDR=(2),WORKREG=2
*      GPR01F=SAV10037;                  /*RESTORE REGS ONE,         */
         ICM   GPR01F,15,SAV10037(GPR13P)                          0667
*      GPR02F=SAV20037;                  /*            TWO,         */
         ICM   GPR02F,15,SAV20037(GPR13P)                          0668
*      GPR04F=SAV40037;                  /*           FOUR,         */
         ICM   GPR04F,15,SAV40037(GPR13P)                          0669
*      GPR06F=SAV60037;                  /*           SIX,          */
         ICM   GPR06F,15,SAV60037(GPR13P)                          0670
*      GPR14F=SAVE0037;                  /*    AND FOURTEEN         */
*                                                                  0671
         ICM   GPR14F,15,SAVE0037(GPR13P)                          0671
*    RFY (GPR01P,GPR02P,GPR04P,GPR06P,GPR13P,GPR14P,GPR15P) UNRSTD;
*                                        /*UNRESTRICT REGISTERS    */
*    END;  /* VALCHEK */                                      /*@D32CKZ
**/                                                                0673
*   IF RTNCODE=ZERO                                                0674
*     THEN DO;                        /* VALID-                      */
         LTR   RTNCODE,RTNCODE                                     0674
         BNZ   @RF00674                                            0674
*       IF LDOCMD=LDOJUMP             /*  COMMAND JUMP?              */
*         THEN DO;                    /*   YES-                      */
         CLI   LDOCMD(LDOR8),X'08'                                 0676
         BNE   @RF00676                                            0676
*           IF R11FLG=JUMP            /*    PREVIOUS CMD=JUMP        */
*             THEN RTNCODE=EIGHT;     /*     YES-ERROR EXIT          */
         C     R11FLG,@CF00037                                     0678
         BNE   @RF00678                                            0678
         LA    RTNCODE,8                                           0679
*             ELSE DO;                /*     NO-                     */
         B     @RC00678                                            0680
@RF00678 DS    0H                                                  0681
*               LDOR8=LDOADDR;        /*      STEP TO NEXT LDO       */
         L     LDOR8,LDOADDR(,LDOR8)                               0681
*               R11FLG=JUMP;          /*      SET JUMP FLG           */
         LA    R11FLG,4                                            0682
*              END;                                                0683
*          END;                                                    0684
*         ELSE DO;                    /*   NO-NOT A JUMP COMMAND     */
         B     @RC00676                                            0685
@RF00676 DS    0H                                                  0686
*           R11FLG=ZERO;              /*    TURN OFF JUMP FLG        */
         SLR   @03,@03                                             0686
         LR    R11FLG,@03                                          0686
*           IF ((DATAR9=ZERO)&(TRADIO='1'B)&(LDOCMD=LDOREAD)) /*I/O  */
*             |((DATAR9=ZERO)&(TRADIO='0'B)&(LDOCMD=LDOWRITE)) /*DIR-*/
*             |((TRADIO='1'B)&(DATAR9=READ))       /*ECTION=LDOCMD OR*/
*             |((TRADIO='0'B)&(DATAR9=WRITE))      /*DATA CHND COMMND*/
*             THEN DO;                  /**YES-COPY DATA**************/
         CR    DATAR9,@03                                          0687
         BNE   @GL00004                                            0687
         TM    TRADIO(TRACPTR),B'00001000'                         0687
         BNO   @GL00004                                            0687
         CLI   LDOCMD(LDOR8),X'02'                                 0687
         BE    @RT00687                                            0687
@GL00004 LTR   DATAR9,DATAR9                                       0687
         BNZ   @GL00003                                            0687
         TM    TRADIO(TRACPTR),B'00001000'                         0687
         BNZ   @GL00003                                            0687
         CLI   LDOCMD(LDOR8),X'05'                                 0687
         BE    @RT00687                                            0687
@GL00003 TM    TRADIO(TRACPTR),B'00001000'                         0687
         BNO   @GL00002                                            0687
         C     DATAR9,@CF00037                                     0687
         BE    @RT00687                                            0687
@GL00002 TM    TRADIO(TRACPTR),B'00001000'                         0687
         BNZ   @RF00687                                            0687
         C     DATAR9,@CF00039                                     0687
         BNE   @RF00687                                            0687
@RT00687 DS    0H                                                  0688
*               LENR4=LDOLEN;             /*  LENGTH FOR VALCHEK     */
*                                                                  0689
         LH    LENR4,LDOLEN(,LDOR8)                                0689
         N     LENR4,@CF04455                                      0689
*/*VALCHEK  (RTNCODE,RPHSAVE1,LDOADDR,UA,LENR4)  RANGE*/           0690
*                DO;                                               0690
*                                                                  0690
*                RFY (GPR01P,GPR02P,GPR04P,GPR06P,GPR13P,GPR14P,GPR15P)
*RSTD;                                   /*DECLARE AND RESTRICT REGS*/
*                                        /*  1, 2, 4, 6, 13,14, AND 15
*/                 RTNCODE=0;           /*SET RETURN CODE TO NO ERROR*/
         SLR   RTNCODE,RTNCODE                                     0692
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY SAVEAREA OPERAND */
*                                        /*AND PUT IT IN REG 13    */
*                                                                  0693
*/*XREGSET  PID('SAVEAREA')PPARM(RPHSAVE1)RREG(13,GPR13P)*/        0693
*                DO;                                               0693
*                  GPR13P=ADDR(RPHSAVE1);/*SET RREG SYMBOL TO ADDRESS*/
         LA    GPR13P,RPHSAVE1(,RPHPTR)                            0694
*                                        /*OF PPARM OPERAND          */
*                END;  /* XREGSET */                               0695
*                  DCL 1 SAV0044 CHAR(20) BASED(GPR13P),           0696
*                                        /*ESTABLISH SAVE AREA MAP   */
*                        2 SAV10044 CHAR(4),                       0696
*                                        /*TO SAVE REGS 1,2,4,6, AND 14
*/                       2 SAV20044 CHAR(4),                       0696
*                        2 SAV40044 CHAR(4),                       0696
*                        2 SAV60044 CHAR(4),                       0696
*                        2 SAVE0044 CHAR(4);                       0696
*                  SAV10044=GPR01F;      /* SAVE REGS ONE,     */  0697
         STCM  GPR01F,15,SAV10044(GPR13P)                          0697
*                  SAV20044=GPR02F;      /*           TWO,     */  0698
         STCM  GPR02F,15,SAV20044(GPR13P)                          0698
*                  SAV40044=GPR04F;      /*          FOUR,     */  0699
         STCM  GPR04F,15,SAV40044(GPR13P)                          0699
*                  SAV60044=GPR06F;      /*          SIX,      */  0700
         STCM  GPR06F,15,SAV60044(GPR13P)                          0700
*                  SAVE0044=GPR14F;      /*     AND FOURTEEN   */  0701
         STCM  GPR14F,15,SAVE0044(GPR13P)                          0701
*                  GPR04P=GPR01P->RPHTSKID;                        0702
         L     GPR04P,RPHTSKID(,GPR01P)                            0702
*                                        /*PUT ADDR OF PROCESS SCHED-*/
*                                        /*ULING TABLE IN REG 4      */
*                                                                  0703
*/* ATBUILD (ISTPST)  */                                           0703
*/* END OF ATBUILD */                    /*INCLUDE PROCESS SCHEDULING *
*                                        /*   TABLE (ISTPST)         */
*                  GPR15P=GPR04P->PSTTCBA;                         0703
         L     GPR15P,PSTTCBA(,GPR04P)                             0703
*                                        /*PUT TASK CONTROL BLOCK    */
*                                        /*ADDRESS IN REGISTER 15    */
*                  GPR04F=SAV40044;      /*RESTORE REG 4*/         0704
         ICM   @03,15,SAV40044(GPR13P)                             0704
         LR    GPR04F,@03                                          0704
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY USER AREA OPERAND*/
*                                        /*AND PUT ADDR IN REG 1   */
*                                                                  0705
*/*XREGSET  PID('USER AREA')PPARM(LDOADDR)RREG(,GPR01P)*/          0705
*                DO;                                               0705
*                  GPR01P=ADDR(LDOADDR); /*SET RREG SYMBOL TO ADDRESS*/
         LA    GPR01P,LDOADDR(,LDOR8)                              0706
*                                        /*OF PPARM OPERAND          */
*                END;  /* XREGSET */                               0707
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY LENGTH OPERAND   */
*                                        /*AND PUT VALUE IN REG 14 */
*                                                                  0708
*/*XREGSET  PID('LENGTH')PPARM(LENR4)RREG(,GPR14F)OUTPUT(PARM)*/   0708
*                DO;                                               0708
*                  GPR14F=LENR4;         /*SET RREG SYMBOL TO VALUE*/
         LR    GPR14F,LENR4                                        0709
*                                        /*OF PPARM OPERAND        */
*                END;  /* XREGSET */                               0710
*        GEN SETS(2);                                              0711
         MODESET EXTKEY=ZERO,SAVEKEY=(2),WORKREG=2
*                  GPR06P=GPR02P;                                  0712
         LR    GPR06P,GPR02P                                       0712
*                  GPR04P=GPR15P;        /*RESTORE TCB ADDR*/      0713
         LR    GPR04P,GPR15P                                       0713
*                  GPR02F=GPR01F+GPR14F-1;                         0714
         LR    GPR02F,GPR01F                                       0714
         ALR   GPR02F,GPR14F                                       0714
         BCTR  GPR02F,0                                            0714
*                                        /*PUT END ADDRESS IN REG 2  */
*                  DCL CZMP0044 PTR(31) LOCATION(16);              0715
*                                        /*ESTABLISH POINTER TO CVT  */
*                  CALL CZMP0044->CVT0VL00;                        0716
         L     @03,CZMP0044                                        0716
         L     @15,CVT0VL00(,@03)                                  0716
         BALR  @14,@15                                             0716
*                                        /*INVOKE TASK SUPERVISOR    */
*                                        /*VALIDITY CHECKING ROUTINE */
*                                        /*UPON RETURN, CHECK PSW    */
*                                        /*CONDITION CODE SET BY THE */
*                                        /*VALIDITY CHECKING ROUTINE */
*                  BC (8,OKAY0044);      /*IF ADDR VALID, DON'T ALTER*/
         BC    8,OKAY0044                                          0717
*                                        /*RETURN CODE VALUE FROM 0  */
*                  RTNCODE=4;            /*SET RETURN CODE TO ERROR  */
         LA    RTNCODE,4                                           0718
*                  GO TO VC30044;                                  0719
         B     VC30044                                             0719
*OKAY0044:         RTNCODE=0;            /*PUT RETURN CODE IN RCREG  */
OKAY0044 SLR   RTNCODE,RTNCODE                                     0720
*VC30044:          GPR02P=GPR06P;                                  0721
VC30044  LR    GPR02P,GPR06P                                       0721
*        GEN SETS(2);                                              0722
         MODESET KEYADDR=(2),WORKREG=2
*                  GPR01F=SAV10044;      /*RESTORE REGS ONE,         */
         ICM   GPR01F,15,SAV10044(GPR13P)                          0723
*                  GPR02F=SAV20044;      /*            TWO,         */
         ICM   GPR02F,15,SAV20044(GPR13P)                          0724
*                  GPR04F=SAV40044;      /*           FOUR,         */
         ICM   GPR04F,15,SAV40044(GPR13P)                          0725
*                  GPR06F=SAV60044;      /*           SIX,          */
         ICM   GPR06F,15,SAV60044(GPR13P)                          0726
*                  GPR14F=SAVE0044;      /*    AND FOURTEEN         */
*                                                                  0727
         ICM   GPR14F,15,SAVE0044(GPR13P)                          0727
*                RFY (GPR01P,GPR02P,GPR04P,GPR06P,GPR13P,GPR14P,GPR15P)
*UNRSTD;                                 /*UNRESTRICT REGISTERS    */
*                END;  /* VALCHEK */                             /*@D32
*HZA*/                                                             0729
*                                                          /*@XM08642*/
*               IF RTNCODE^=ZERO          /*  USER DATA ADDR VALID?  */
*                 THEN;                   /*   NO-ERROR EXIT         */
         LTR   RTNCODE,RTNCODE                                     0730
         BNZ   @RT00730                                            0730
*                 ELSE DO;                /*   YES-SET LNG-TO-MOVE TO*/
*                   LENR4=RBUFCT-TRADATCT;  /*  BUFFER REMAINING.    */
         LR    LENR4,RBUFCT                                        0733
         SH    LENR4,TRADATCT(,TRACPTR)                            0733
*                   IF LENR4<LDOLEN       /*    IF LDO CMD LENGTH IS */
*                     THEN;               /*    LESS, USE LDO CMD    */
         LH    @03,LDOLEN(,LDOR8)                                  0734
         N     @03,@CF04455                                        0734
         CR    LENR4,@03                                           0734
         BL    @RT00734                                            0734
*                     ELSE LENR4=LDOLEN;  /*    LENGTH.              */
         LH    LENR4,LDOLEN(,LDOR8)                                0736
         N     LENR4,@CF04455                                      0736
*                   BUFNX=ADDR(TRABFFER)+TRADATCT; /*MOVE-TO-ADDRESS */
@RT00734 LA    BUFNX,TRABFFER(,TRACPTR)                            0737
         LH    @03,TRADATCT(,TRACPTR)                              0737
         ALR   BUFNX,@03                                           0737
*                   BUFNX->BUFDATA(1:LENR4)=LDOADDR->BUFDATA(1:LENR4);
         LR    @02,LENR4                                           0738
         BCTR  @02,0                                               0738
         L     @03,LDOADDR(,LDOR8)                                 0738
         EX    @02,@SM04456                                        0738
*                   TRADATCT=TRADATCT+LENR4; /* UPDATE TRAC DATA CNT */
         LH    @03,TRADATCT(,TRACPTR)                              0739
         ALR   @03,LENR4                                           0739
         STH   @03,TRADATCT(,TRACPTR)                              0739
*                  END;                                            0740
*              END;                       /***DATA COPY COMPLETE******/
*             ELSE;                       /*  NO-I/O DIRECTION NOT = */
@RF00687 DS    0H                                                  0743
*           IF LDOACHN='1'B               /* DATA CHAINED?           */
*             THEN IF DATAR9=ZERO         /*  YES-1ST LDO OF DATA CHN*/
@RC00687 TM    LDOACHN(LDOR8),B'10000000'                          0743
         BNO   @RF00743                                            0743
*                    THEN IF LDOCMD=LDOREAD   /* YES-SET DATA CHN FLG*/
         LTR   DATAR9,DATAR9                                       0744
         BNZ   @RF00744                                            0744
*                           THEN DATAR9=READ; /*   ACCORDING TO      */
         CLI   LDOCMD(LDOR8),X'02'                                 0745
         BNE   @RF00745                                            0745
         LA    DATAR9,4                                            0746
*                           ELSE              /*   I/O OPERATION     */
*                             IF LDOCMD=LDOWRITE                   0747
*                               THEN DATAR9=WRITE;                 0747
         B     @RC00745                                            0747
@RF00745 CLI   LDOCMD(LDOR8),X'05'                                 0747
         BNE   @RF00747                                            0747
         LA    DATAR9,8                                            0748
*                               ELSE DATAR9=OTHER;                 0749
         B     @RC00747                                            0749
@RF00747 LA    DATAR9,12                                           0749
*                    ELSE;                     /* NO-FLG ALREADY SET */
@RF00744 DS    0H                                                  0751
*             ELSE DATAR9=ZERO;            /* NOT DATA CHAINED.      */
         B     @RC00743                                            0751
@RF00743 SLR   DATAR9,DATAR9                                       0751
*           IF LDOACHN='1'B | LDOCMHN='1'B /*DATA|CMD CHAINED?       */
*             THEN LDOR8=LDOR8+LENGTH(ISTLDO); /*YES-STEP TO NEXT LDO*/
@RC00743 TM    LDOACHN(LDOR8),B'11000000'                          0752
         BZ    @RF00752                                            0752
         AL    LDOR8,@CF00039                                      0753
*             ELSE RBUFCT=TRADATCT;            /*NO-STOP LOOP        */
         B     @RC00752                                            0754
@RF00752 LH    RBUFCT,TRADATCT(,TRACPTR)                           0754
*          END;                       /* END OF NON-JUMP PROCESSING  */
*       END;                          /*END OF VALID LDO PROCESSING  */
*      ELSE;                          /*LDO INVALID USER ADDRESS     */
@RF00674 DS    0H                                                  0758
*  END;                               /*END DO LOOP                  */
@RC00674 DS    0H                                                  0758
@DE00615 LTR   RTNCODE,RTNCODE                                     0758
         BNZ   @DC00615                                            0758
         CH    RBUFCT,TRADATCT(,TRACPTR)                           0758
         BH    @DL00615                                            0758
@DC00615 DS    0H                                                  0759
*  RFY (LENR4,DATAR9,R11FLG,BUFNX,LDOR8,RBUFCT) UNRSTD;            0759
*  IF TRADATCT = 0                     /* IF NO DATA COPIED  @XM07842*/
*    THEN RTNCODE = 4;                 /* SET RTRN CODE TO   @XM07842
*                                         BYPASS GTRACE      @XM07842*/
         LH    @11,TRADATCT(,TRACPTR)                              0760
         LTR   @11,@11                                             0760
         BNZ   @RF00760                                            0760
         LA    RTNCODE,4                                           0761
*    ELSE;                             /* CONTINUE           @XM07842*/
*                                                                  0762
@RF00760 DS    0H                                                  0763
*/********************************************************************/
*     END;                                                         0763
*    ELSE DO;                           /* NO-PROCESS READ/WRITE/SEND*/
         B     @RC00602                                            0764
@RF00602 DS    0H                                                  0765
*/********************************************************************/
*/*                                                                  */
*/*  ROUTINE NAME : TBSRWSR - CONTROL LAYER'S READ/WRITE/SEND/RECEIVE*/
*/*                           INPUT HANDLER                          */
*/*  FUNCTION : TO FILL THE TRACE BUFFER WITH DATA TRANSFERRED BY    */
*/*             READS,WRITES,SENDS OR RECEIVES.                      */
*/*  INPUT : R6INPUT - RPL POINTER                                   */
*/*          TRADIO - I/O DIRECTION FLAG SET BY TBTRAC.              */
*/*  OUTPUT : R15 = 0 SUCCESSFUL                                     */
*/*           R15^= 0 VALCHEK ERROR ON OUTBOUND.                     */
*/*  ROUTINES : NONE                                                 */
*/*                                                                  */
*/*  MACROS : VALCHEK - TO VALIDATE THE USER DATA ADDRESS. ON        */
*/*                      OUTBOUND DATA ISTRACTB IS CALLED BEFORE     */
*/*                      CONTROL LAYER HAS PERFORMED A VALCHECK.     */
*/********************************************************************/
*RFY FROMR5   RSTD;                   /*DATA ADDRESS SPECIFIED IN RPL*/
*RFY LENR4    RSTD;                   /*LENGTH OF DATA TO MOVE       */
*RTNCODE=ZERO;                        /*INIT GOOD RC         @XM08901*/
         SLR   RTNCODE,RTNCODE                                     0767
*LENR4=BUFCT;                         /*INITIALIZE TO FULL BUFFR SIZE*/
         L     LENR4,BUFCT                                         0768
*IF ((RPLREQ='22'X)&(RPLDATA='0'B))    /*CNTRL COMMAND AND   @XM08901*/
*  | ((RPLREQ='23'X)&(RPLDATA='0'B))    /*NO DATA TRANSFD?   @XM08901*/
* THEN RTNCODE=EIGHT;                 /*    YES-DO NOT TRACE @XM08901*/
         CLI   RPLREQ(R6INPUT),34                                  0769
         BNE   @GL00012                                            0769
         TM    RPLDATA+16(R6INPUT),B'10000000'                     0769
         BZ    @RT00769                                            0769
@GL00012 CLI   RPLREQ(R6INPUT),35                                  0769
         BNE   @RF00769                                            0769
         TM    RPLDATA+16(R6INPUT),B'10000000'                     0769
         BNZ   @RF00769                                            0769
@RT00769 DS    0H                                                  0770
         LA    RTNCODE,8                                           0770
* ELSE DO;                            /*    NO-TRACE THE DATA@XM08901*/
         B     @RC00769                                            0771
@RF00769 DS    0H                                                  0772
*  IF TRADIO='1'B                     /*I/O DIRECTION INBOUND?       */
*    THEN DO;                         /* YES (INBOUND)               */
         TM    TRADIO(TRACPTR),B'00001000'                         0772
         BNO   @RF00772                                            0772
*      IF RPLREQ='11'X                /*  IS COMMAND WRITE           */
*        |RPLREQ='22'X                /*   OR SEND?                  */
*        THEN DO;                     /*   YES-ACTUAL DATA LENGTH IS */
         CLI   RPLREQ(R6INPUT),17                                  0774
         BE    @RT00774                                            0774
         CLI   RPLREQ(R6INPUT),34                                  0774
         BNE   @RF00774                                            0774
@RT00774 DS    0H                                                  0775
*          IF LENR4<RPLARCLN          /*    IN RPL.IF BUFFR SMALLER  */
*            THEN;                    /*    THAN ACTUAL DATA,MOVE LEN*/
         C     LENR4,RPLARCLN(,R6INPUT)                            0776
         BL    @RT00776                                            0776
*            ELSE                     /*    OF BUFFER,ELSE MOVE FOR  */
*              LENR4=RPLARCLN;        /*    ACTUAL LEN OF DATA TRANS-*/
         L     LENR4,RPLARCLN(,R6INPUT)                            0778
*          FROMR5=RPLAAREA;           /*    FERRED. PTR TO DATA.     */
@RT00776 L     FROMR5,RPLAAREA(,R6INPUT)                           0779
*         END;                                                     0780
*        ELSE DO;                     /*    NO-(READ/RECEIVE)DATA CMD*/
         B     @RC00774                                            0781
@RF00774 DS    0H                                                  0782
*          IF LENR4<RPLRLEN           /*     LENGTH IS IN RPL. IF >  */
*            THEN;                    /*     BUFFR,MOVE LEN OF BUFFR,*/
         C     LENR4,RPLRLEN(,R6INPUT)                             0782
         BL    @RT00782                                            0782
*            ELSE                     /*     ELSE MOVE FOR LEN OF    */
*              LENR4=RPLRLEN;         /*     DATA.                   */
         L     LENR4,RPLRLEN(,R6INPUT)                             0784
*          FROMR5=RPLAREA;            /*     POINTER TO DATA.        */
@RT00782 L     FROMR5,RPLAREA(,R6INPUT)                            0785
*          IF RPLREQ='23'X            /*     RECEIVE?                */
*            THEN                     /*      YES-DATA LEN ACTUALLY  */
         CLI   RPLREQ(R6INPUT),35                                  0786
         BNE   @RF00786                                            0786
*              IF LENR4<RPLBUFL       /*       TRANSFERRED   @XM08670*/
*                THEN;                /*       IN RPLBUFL. IF<CMD LEN*/
         C     LENR4,RPLBUFL(,R6INPUT)                             0787
         BL    @RT00787                                            0787
*                ELSE                 /*       OR BUFFR LEN, MOVE FOR*/
*                  LENR4=RPLBUFL;     /*       LEN OF DATA   @XM08670*/
         L     LENR4,RPLBUFL(,R6INPUT)                             0789
*            ELSE;                    /*        TRANSFERRED (READ)   */
@RF00786 DS    0H                                                  0791
*         END;                                                     0791
*     END;                            /*  END OF INBOUND             */
*    ELSE DO;                         /* NO-(OUTBOUND) ACTUAL DATA   */
         B     @RC00772                                            0793
@RF00772 DS    0H                                                  0794
*     IF (RPLREQ='11'X)&(RPLEAU='1'B)    /* LOCAL CNTRL CMD? @XM08901*/
*       THEN RTNCODE=EIGHT;           /*  YES-DONOT TRACE    @XM08901*/
         CLI   RPLREQ(R6INPUT),17                                  0794
         BNE   @RF00794                                            0794
         TM    RPLEAU(R6INPUT),B'00001000'                         0794
         BNO   @RF00794                                            0794
         LA    RTNCODE,8                                           0795
*       ELSE DO;                      /*  NO-TRACE:          @XM08091*/
         B     @RC00794                                            0796
@RF00794 DS    0H                                                  0797
*        IF LENR4<RPLRLEN             /*  LENGTH IS IN RPL. IF>BUFFER*/
*          THEN;                      /*  MOVE SET FOR BUFFER SIZE,  */
         C     LENR4,RPLRLEN(,R6INPUT)                             0797
         BL    @RT00797                                            0797
*          ELSE                       /*  ELSE SET MOVE FOR ACTUAL   */
*            LENR4=RPLRLEN;           /*  DATA SIZE.                 */
         L     LENR4,RPLRLEN(,R6INPUT)                             0799
*        FROMR5=RPLAREA;              /*  POINTER TO DATA. VALIDATE  */
@RT00797 L     @11,RPLAREA(,R6INPUT)                               0800
         LR    FROMR5,@11                                          0800
*        R14=RPLRLEN;                 /*  USER AREA                  */
*                                                                  0801
         L     R14,RPLRLEN(,R6INPUT)                               0801
*/*VALCHEK  (RTNCODE,RPHSAVE1,(RPLAREA),UA,R14)  RANGE*/           0802
*         DO;                                                      0802
*                                                                  0802
*         RFY (GPR01P,GPR02P,GPR04P,GPR06P,GPR13P,GPR14P,GPR15P) RSTD;
*                                        /*DECLARE AND RESTRICT REGS*/
*                                        /*  1, 2, 4, 6, 13,14, AND 15
*/          RTNCODE=0;                  /*SET RETURN CODE TO NO ERROR*/
         SLR   RTNCODE,RTNCODE                                     0804
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY SAVEAREA OPERAND */
*                                        /*AND PUT IT IN REG 13    */
*                                                                  0805
*/*XREGSET  PID('SAVEAREA')PPARM(RPHSAVE1)RREG(13,GPR13P)*/        0805
*         DO;                                                      0805
*           GPR13P=ADDR(RPHSAVE1);       /*SET RREG SYMBOL TO ADDRESS*/
         LA    GPR13P,RPHSAVE1(,RPHPTR)                            0806
*                                        /*OF PPARM OPERAND          */
*         END;  /* XREGSET */                                      0807
*           DCL 1 SAV0051 CHAR(20) BASED(GPR13P),                  0808
*                                        /*ESTABLISH SAVE AREA MAP   */
*                 2 SAV10051 CHAR(4),    /*TO SAVE REGS 1,2,4,6, AND 14
*/                2 SAV20051 CHAR(4),                              0808
*                 2 SAV40051 CHAR(4),                              0808
*                 2 SAV60051 CHAR(4),                              0808
*                 2 SAVE0051 CHAR(4);                              0808
*           SAV10051=GPR01F;             /* SAVE REGS ONE,     */  0809
         STCM  GPR01F,15,SAV10051(GPR13P)                          0809
*           SAV20051=GPR02F;             /*           TWO,     */  0810
         STCM  GPR02F,15,SAV20051(GPR13P)                          0810
*           SAV40051=GPR04F;             /*          FOUR,     */  0811
         STCM  GPR04F,15,SAV40051(GPR13P)                          0811
*           SAV60051=GPR06F;             /*          SIX,      */  0812
         STCM  GPR06F,15,SAV60051(GPR13P)                          0812
*           SAVE0051=GPR14F;             /*     AND FOURTEEN   */  0813
         STCM  GPR14F,15,SAVE0051(GPR13P)                          0813
*           GPR04P=GPR01P->RPHTSKID;     /*PUT ADDR OF PROCESS SCHED-*/
         L     GPR04P,RPHTSKID(,GPR01P)                            0814
*                                        /*ULING TABLE IN REG 4      */
*                                                                  0815
*/* ATBUILD (ISTPST)  */                                           0815
*/* END OF ATBUILD */                    /*INCLUDE PROCESS SCHEDULING *
*                                        /*   TABLE (ISTPST)         */
*           GPR15P=GPR04P->PSTTCBA;      /*PUT TASK CONTROL BLOCK    */
         L     GPR15P,PSTTCBA(,GPR04P)                             0815
*                                        /*ADDRESS IN REGISTER 15    */
*           GPR04F=SAV40051;             /*RESTORE REG 4*/         0816
         ICM   @03,15,SAV40051(GPR13P)                             0816
         LR    GPR04F,@03                                          0816
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY USER AREA OPERAND*/
*                                        /*AND PUT ADDR IN REG 1   */
*                                                                  0817
*/*XREGSET  PID('USER AREA')PPARM((RPLAREA))RREG(,GPR01P)*/        0817
*         DO;                                                      0817
*           GPR01P=RPLAREA;              /*SET RREG SYMBOL TO VALUE*/
         LR    GPR01P,@11                                          0818
*                                        /*OF PPARM OPERAND        */
*         END;  /* XREGSET */                                      0819
*                                        /*INVOKE XREGSET MACRO TO */
*                                        /*VERIFY LENGTH OPERAND   */
*                                        /*AND PUT VALUE IN REG 14 */
*                                                                  0820
*/*XREGSET  PID('LENGTH')PPARM(R14)RREG(,GPR14F)OUTPUT(PARM)*/     0820
*         DO;                                                      0820
*           GPR14F=R14;                  /*SET RREG SYMBOL TO VALUE*/
*                                        /*OF PPARM OPERAND        */
*         END;  /* XREGSET */                                      0822
*        GEN SETS(2);                                              0823
         MODESET EXTKEY=ZERO,SAVEKEY=(2),WORKREG=2
*           GPR06P=GPR02P;                                         0824
         LR    GPR06P,GPR02P                                       0824
*           GPR04P=GPR15P;               /*RESTORE TCB ADDR*/      0825
         LR    GPR04P,GPR15P                                       0825
*           GPR02F=GPR01F+GPR14F-1;      /*PUT END ADDRESS IN REG 2  */
         LR    GPR02F,GPR01F                                       0826
         ALR   GPR02F,GPR14F                                       0826
         BCTR  GPR02F,0                                            0826
*           DCL CZMP0051 PTR(31) LOCATION(16);                     0827
*                                        /*ESTABLISH POINTER TO CVT  */
*           CALL CZMP0051->CVT0VL00;     /*INVOKE TASK SUPERVISOR    */
         L     @11,CZMP0051                                        0828
         L     @15,CVT0VL00(,@11)                                  0828
         BALR  @14,@15                                             0828
*                                        /*VALIDITY CHECKING ROUTINE */
*                                        /*UPON RETURN, CHECK PSW    */
*                                        /*CONDITION CODE SET BY THE */
*                                        /*VALIDITY CHECKING ROUTINE */
*           BC (8,OKAY0051);             /*IF ADDR VALID, DON'T ALTER*/
         BC    8,OKAY0051                                          0829
*                                        /*RETURN CODE VALUE FROM 0  */
*           RTNCODE=4;                   /*SET RETURN CODE TO ERROR  */
         LA    RTNCODE,4                                           0830
*           GO TO VC30051;                                         0831
         B     VC30051                                             0831
*OKAY0051:  RTNCODE=0;                   /*PUT RETURN CODE IN RCREG  */
OKAY0051 SLR   RTNCODE,RTNCODE                                     0832
*VC30051:   GPR02P=GPR06P;                                         0833
VC30051  LR    GPR02P,GPR06P                                       0833
*        GEN SETS(2);                                              0834
         MODESET KEYADDR=(2),WORKREG=2
*           GPR01F=SAV10051;             /*RESTORE REGS ONE,         */
         ICM   GPR01F,15,SAV10051(GPR13P)                          0835
*           GPR02F=SAV20051;             /*            TWO,         */
         ICM   GPR02F,15,SAV20051(GPR13P)                          0836
*           GPR04F=SAV40051;             /*           FOUR,         */
         ICM   GPR04F,15,SAV40051(GPR13P)                          0837
*           GPR06F=SAV60051;             /*           SIX,          */
         ICM   GPR06F,15,SAV60051(GPR13P)                          0838
*           GPR14F=SAVE0051;             /*    AND FOURTEEN         */
*                                                                  0839
         ICM   GPR14F,15,SAVE0051(GPR13P)                          0839
*         RFY (GPR01P,GPR02P,GPR04P,GPR06P,GPR13P,GPR14P,GPR15P) UNRSTD
*                                        /*UNRESTRICT REGISTERS    */
*         END;  /* VALCHEK */                               /*@D32CKZA*
*       END;                                              /* @XM08901*/
*     END;                                                         0843
@RC00794 DS    0H                                                  0844
*  END;                                                   /* @XM08901*/
*                                                                  0844
@RC00772 DS    0H                                                  0845
*IF RTNCODE=ZERO                      /*VALCHEK ERROR OR NO DATA     */
*  THEN DO;                           /* NO-IS DATA TRANSFERRED      */
@RC00769 SLR   @11,@11                                             0845
         CR    RTNCODE,@11                                         0845
         BNE   @RF00845                                            0845
*    IF LENR4>ZERO                    /*  LENGTH > 0?                */
*      THEN DO;                       /*   YES-MOVE THE DATA.        */
         CR    LENR4,@11                                           0847
         BNH   @RF00847                                            0847
*        TRABFFER(1:LENR4)=FROMR5->BUFDATA(1:LENR4);               0849
         LR    @11,LENR4                                           0849
         BCTR  @11,0                                               0849
         EX    @11,@SM04458                                        0849
*        TRADATCT=LENR4;              /*    SETUP DATA LENGTH IN TRAC*/
         STH   LENR4,TRADATCT(,TRACPTR)                            0850
*       END;                                                       0851
*      ELSE;                          /*   NO-LNG 0,SO NO MOVE       */
@RF00847 DS    0H                                                  0853
*   END;                                                           0853
*  ELSE;                              /* YES-VALCHEK ERROR,SO EXIT.  */
@RF00845 DS    0H                                                  0855
*RFY (FROMR5,LENR4) UNRSTD;                                        0855
*                                                                  0855
*/********************************************************************/
*     END;                                                         0856
*  RFY SAVRCR5 RSTD;                    /*VSETLCK DESTROYS RETURN    */
@RC00602 DS    0H                                                  0858
*  SAVRCR5=RTNCODE;                     /*CODE FROM TBSLDO/TBSRWSR,SO*/
         LR    SAVRCR5,RTNCODE                                     0858
*  TBUNLOCK:                            /*SAVE IT. RELEASE LOCAL LOCK*/
*                                                                  0859
*/*VSETLCK  (RTNCODE,RELEASE,LOCAL,(LOCAL,ISTRACTB(TBLOCK)),)*/    0859
*    DO;                                                           0859
TBUNLOCK DS    0H                                                  0860
*                                        /*DECLARE AND RESTRICT REGS */
*                                        /*   2,10,11,12,13,14, AND 15
**/                                                                0860
*    RFY (GPR02P,GPR10P,GPR11P,GPR12P,GPR13P,GPR14P,GPR15P) RSTD;  0860
*                                        /*SAVE REGS 2 AND 10 THRU 14*/
*        GEN REFS(GPR02F,GPR10F,GPR11F,GPR12F,GPR14F,GPR13P);      0861
*                                                                  0861
         STM   GPR10F,GPR12F,0(GPR13P)
         ST    GPR14F,12(GPR13P)
         ST    GPR02F,16(GPR13P)
*                                        /*INVOKE MODESET TO CHANGE  */
*                                        /*PSW PROTECTION KEY TO 0   */
*        GEN SETS(2);                                              0862
*                                                                  0862
         MODESET EXTKEY=ZERO,SAVEKEY=(2),WORKREG=2
*      GPR10P=GPR13P;                    /*PUT SAVE AREA ADDR IN R 10*/
         LR    GPR10P,GPR13P                                       0863
*                                        /*INVOKE OS/VS SETLOCK MACRO*/
*      GEN REFS(GPR13P,PSALITA,FLC) SETS(GPR12F);                  0864
         SETLOCK RELEASE,TYPE=LOCAL,                                   *
               RELATED=(LOCAL,ISTRACTB(TBLOCK))
*      GPR15P=GPR13P;                    /*PUT RETURN CODE IN R 15 */
         LR    GPR15P,GPR13P                                       0865
*                                        /*INVOKE MODESET MACRO TO   */
*        DO;                             /*RESTORE PSW PROTECTION KEY*/
*        GEN SETS(2);                                              0867
         MODESET KEYADDR=(2),WORKREG=2
*        END;                            /* RESTORE REGISTERS        */
*      GEN REFS(GPR10F,GPR13P) SETS(GPR10F,GPR11F,                 0869
*      GPR12F,GPR13F,GPR14F,GPR02F);                               0869
         LR    GPR13F,GPR10F
         LM    GPR10F,GPR12F,0(GPR13P)
         L     GPR14F,12(GPR13P)
         L     GPR02F,16(GPR13P)
*      IF GPR15F=4 THEN GPR15F=0;                                  0870
         C     GPR15F,@CF00037                                     0870
         BNE   @RF00870                                            0870
         SLR   GPR15F,GPR15F                                       0871
*      RTNCODE=GPR15F;                   /*PUT RETURN CODE IN RCREG*/
*                                                                  0872
@RF00870 DS    0H                                                  0873
*    RFY (GPR02P,GPR10P,GPR11P,GPR12P,GPR13P,GPR14P,GPR15P) UNRSTD;
*    END;  /*VSETLCK */                                            0874
*                                                                  0874
*/*VMDESET  ((RTNCODE),VTAM,,)*/                                   0875
*   DO;                                  /* INVOKE MODESET MACRO    */
*     GEN;                                                         0876
         MODESET EXTKEY=VTAM,WORKREG=2
*     RTNCODE=0;                         /*PUT RETURN CODE IN RCREG*/
         SLR   RTNCODE,RTNCODE                                     0877
*   END;  /* VMDESET */        /*RESET PROTECT KEY TO VTAM. */     0878
*  RTNCODE=SAVRCR5;                     /*PUT RC FROM TBSLDO/TBSRWSR */
         LR    RTNCODE,SAVRCR5                                     0879
*  RFY (R13,SAVRCR5) UNRSTD;            /*BACK IN RTNCODE REG.       */
*                                                                  0880
*/********************************************************************/
*                 END;                                             0881
*              IF RTNCODE=ZERO                    /*ERRORS           */
*                THEN DO;                         /* NO-SETUP REMAIN-*/
@RC00561 LTR   RTNCODE,RTNCODE                                     0882
         BNZ   @RF00882                                            0882
*                  RFY (R2,NCBPTR,R0,CIDSAVR5) RSTD; /* ING HDR INFO:*/
*                  RFY FMCBR2 RSTD;                                0885
*                  IF (R6INPUT&INPTMSK) = FDBINPUT                 0886
*                    THEN DO;             /* INPUT IS A FDB */     0886
         LR    @04,R6INPUT                                         0886
         N     @04,INPTMSK                                         0886
         CL    @04,FDBINPUT                                        0886
         BNE   @RF00886                                            0886
*                  R2=TRACID;             /* GET SAVED CID */      0888
*                                                                  0888
         L     R2,TRACID(,TRACPTR)                                 0888
*                   DO; /* CIDCTL(FIND) LOCK(XCLUSIVE) */          0889
*                                                                  0889
*                   RFY (EMCR2,EMCR3) RSTD;                        0890
*                     GEN SETS(RPHWORK)                            0891
*        (STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH);      0891
         STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH
*                     EMCR3 = '00'X; /*CLEAR EXTRA BITS*/          0892
         SLR   EMCR3,EMCR3                                         0892
*                     EMCR3 = '02'X; /*INDICATE EXCLUSIVE LOCK*/   0893
         LA    EMCR3,2                                             0893
*                     CALL ATCLCPTR->ATCCDFND->ISTSDCCF; /*PROCESS REQU
*ST*/                                                              0894
         L     @04,ATCLCPTR                                        0894
         L     @15,ATCCDFND(,@04)                                  0894
         BALR  @14,@15                                             0894
*                   RFY (EMCR2,EMCR3) UNRSTD;                      0895
*                   END; /*CIDCTL*/  /* OBTAIN DAF LOCK */         0896
*                  IF RTNCODE=ZERO                /* CID VALID?      */
*                    THEN DO;                     /* YES-PROCEED     */
         LTR   RTNCODE,RTNCODE                                     0897
         BNZ   @RF00897                                            0897
*                      CIDSAVR5=TRACID;           /* SAVE CID        */
         L     CIDSAVR5,TRACID(,TRACPTR)                           0899
*                      NCBPTR=R0;                 /* SET NCB BASE    */
         LR    NCBPTR,R0                                           0900
*                         END;                                     0901
*                    ELSE;              /* CID ERROR */            0902
@RF00897 DS    0H                                                  0903
*                         END;                                     0903
*                    ELSE DO;                                      0904
         B     @RC00886                                            0904
@RF00886 DS    0H                                                  0905
*                         FMCBR2 = TRACID;   /* GET SAVED FMCB @ */
         L     FMCBR2,TRACID(,TRACPTR)                             0905
*                         NCBPTR = FMCDNCBA; /* GET DNCB PTR */    0906
         L     NCBPTR,FMCDNCBA(,FMCBR2)                            0906
*                        END;                                      0907
*                  IF RTNCODE = 0          /* IF NO CID ERROR */   0908
*                    THEN DO;                                      0908
@RC00886 SLR   @04,@04                                             0908
         CR    RTNCODE,@04                                         0908
         BNE   @RF00908                                            0908
*                      TRALRC=NCBLTRC;            /* GET LOST RCD CNT*/
         LH    @03,NCBLTRC(,NCBPTR)                                0910
         STH   @03,TRALRC(,TRACPTR)                                0910
*                      NCBLTRC=ZERO;              /* RE-INITILIZE CNT*/
         STH   @04,NCBLTRC(,NCBPTR)                                0911
*                      RFY ISTRPRE BASED(NCBRDTE);                 0912
*                      TRADNODE=RPRNAME;          /* GET NODE NAME   */
         L     @04,NCBRDTE(,NCBPTR)                                0913
         MVC   TRADNODE(8,TRACPTR),RPRNAME(@04)                    0913
*                      TRADNODF='1'B;             /* SET NODENAME FLG*/
         OI    TRADNODF(TRACPTR),B'00000100'                       0914
*                      RFY ISTRPRE BASED(ACDRDTE); /* FMCB->ACD->RDTE*/
*                      TRAANODE=RPRNAME;          /* APPLICATION NAME*/
         L     @04,FMCDEBA(,FMCBR2)                                0916
         L     @04,ACDRDTE(,@04)                                   0916
         MVC   TRAANODE(8,TRACPTR),RPRNAME(@04)                    0916
*                      TRAANODF='1'B;             /* APPL NAME FLAG  */
         OI    TRAANODF(TRACPTR),B'00000010'                       0917
*                      TRADATCT=TRADATCT+LENGTH(TRAANODE); /*DATA CNT*/
         LA    @04,8                                               0918
         AH    @04,TRADATCT(,TRACPTR)                              0918
         STH   @04,TRADATCT(,TRACPTR)                              0918
*                      IF (R6INPUT&INPTMSK) = FDBINPUT             0919
*                        THEN DO;          /* INPUT IS A FDB */    0919
         LR    @04,R6INPUT                                         0919
         N     @04,INPTMSK                                         0919
         CL    @04,FDBINPUT                                        0919
         BNE   @RF00919                                            0919
*                      R2=CIDSAVR5;               /* CID IN REG 2    */
*                                                                  0921
         LR    R2,CIDSAVR5                                         0921
*                       DO; /* CIDCTL(FINISH)  */                  0922
*                                                                  0922
*                       RFY (EMCR2,EMCR3) RSTD;                    0923
*                         GEN SETS(RPHWORK)                        0924
*        (STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH);      0924
         STM EMCR0,EMCR15,RPHWORK(RPHPTR) SAVE REGS IN RPH
*                         EMCR3 = '00'X; /*CLEAR EXTRA BITS*/      0925
         SLR   EMCR3,EMCR3                                         0925
*                         CALL ATCLCPTR->ATCCDFIN->ISTSDCCR; /*PROCESS
*EQUEST*/                                                          0926
         L     @04,ATCLCPTR                                        0926
         L     @15,ATCCDFIN(,@04)                                  0926
         BALR  @14,@15                                             0926
*                       RFY (EMCR2,EMCR3) UNRSTD;                  0927
*                       END; /*CIDCTL*/          /* UNLOCK DAFLOCK  */
*                             END;                                 0929
*                        ELSE;                                     0930
@RF00919 DS    0H                                                  0931
*                      RFY (R2,NCBPTR,CIDSAVR5,FMCBR2) UNRSTD;     0931
*                     END;                                         0932
*                   ELSE;                         /*CID ERROR        */
@RF00908 DS    0H                                                  0934
*                 END;                                             0934
*                ELSE;                       /*ERROR IN TBSFDB/TBSRPL*/
@RF00882 DS    0H                                                  0936
*             END;                                                 0936
*            ELSE;                           /*CONFIDENTIAL DATA     */
@RF00559 DS    0H                                                  0938
*          IF RTNCODE^=ZERO                       /* ANY ERROR SO FAR*/
*            THEN DO;                             /*  YES-RELEASE THE*/
         LTR   RTNCODE,RTNCODE                                     0938
         BZ    @RF00938                                            0938
*               /*RELSTORE BUFADDR(TRACPTR)*/                      0940
*                DO;                                               0940
*/* ATBUILD (ISTATCVT,ISTRPH)  */                                  0941
*/* END OF ATBUILD */                                              0941
*               RFY (GPR02P,GPR03P,GPR04P,GPR15P) RSTD;            0941
*         RFY (GPR01P,GPR13P) RSTD;                                0942
*         GENERATE REFS(GPR01P,GPR02P,GPR13P) SETS(RPHWORK);       0943
*                                                                  0943
          STM GPR02P,GPR13P,RPHWORK(GPR01P)
*         RFY GPR13P UNRSTD;                                       0944
*         GPR04P=TRACPTR;                                          0945
         LR    GPR04P,TRACPTR                                      0945
*         DCL 1 FMZA0070 FIXED(31) STATIC LOCAL,                   0946
*             2 * FIXED(8) INIT(64),                               0946
*             2 * PTR(24) INIT(0);                                 0946
*         GPR03F = FMZA0070;                                       0947
*                                                                  0947
         L     GPR03F,FMZA0070                                     0947
*               DO; /* LISATCVT CVTBASE(PAGEABLE,(GPR15P)) */      0948
*                 GPR15P = ATCLCPTR; /*SET UP ATCVT POINTER*/      0949
         L     GPR15P,ATCLCPTR                                     0949
*               END; /*LISATCVT*/                                  0950
*         GPR02P = GPR15P->ATCBPDA;                                0951
         L     GPR02P,ATCBPDA(,GPR15P)                             0951
*         GPR15P=GPR15P->ATCSMRS;                                  0952
         L     GPR15P,ATCSMRS(,GPR15P)                             0952
*         DCL FMZL0070 BASED(GPR15P) LABEL VALUERANGE(*);          0953
*         CALL FMZL0070;                                           0954
         BALR  @14,@15                                             0954
*         RFY GPR13P RSTD;                                         0955
*         GENERATE REFS(GPR01P,RPHWORK) SETS(GPR02P,GPR13P);       0956
*                                                                  0956
          LM GPR02P,GPR13P,RPHWORK(GPR01P)
*         RFY (GPR01P,GPR13P) UNRSTD;                              0957
*               RFY (GPR02P,GPR03P,GPR04P,GPR15P) UNRSTD;          0958
*               END ;/**********END OF RELSTORE EXPANSION**********/
*    /*      TRACE RCD  */                                         0960
*             END;                                                 0960
*            ELSE DO;                             /*  NO-RECORD TRAC */
         B     @RC00938                                            0961
@RF00938 DS    0H                                                  0962
*              RFY R5RPHSAV RSTD;                 /*     VIA GTRACE  */
*              R5RPHSAV=RPHPTR;                   /*      SAVE R1    */
         LR    R5RPHSAV,RPHPTR                                     0963
*              TRA1WORD(1:2)=TRADATCT+LENGTH(ISTTRAC); /*    @XM06246*/
         LA    @11,32                                              0964
         AH    @11,TRADATCT(,TRACPTR)                              0964
         STCM  @11,3,TRA1WORD(TRACPTR)                             0964
*              TRARSV01=GTFID;                    /*     USE 1ST 2WDS*/
         LH    @11,GTFID                                           0965
         STH   @11,TRARSV01(,TRACPTR)                              0965
*              TRACHAIN=TRACPTR;                  /*     OF RCD FOR  */
         ST    TRACPTR,TRACHAIN(,TRACPTR)                          0966
*                                                 /*     PARM LIST.  */
*              GEN REFS(TRACPTR);                                  0967
               GTRACE ID=ISTCLEID,MF=(E,(TRACPTR))
*              RPHPTR=R5RPHSAV;                   /*     RESTORE R1  */
         LR    RPHPTR,R5RPHSAV                                     0968
*              RFY R5RPHSAV UNRSTD;                                0969
*               /*RELSTORE BUFADDR(TRACPTR)*/                      0970
*                DO;                                               0970
*/* ATBUILD (ISTATCVT,ISTRPH)  */                                  0971
*/* END OF ATBUILD */                                              0971
*               RFY (GPR02P,GPR03P,GPR04P,GPR15P) RSTD;            0971
*         RFY (GPR01P,GPR13P) RSTD;                                0972
*         GENERATE REFS(GPR01P,GPR02P,GPR13P) SETS(RPHWORK);       0973
*                                                                  0973
          STM GPR02P,GPR13P,RPHWORK(GPR01P)
*         RFY GPR13P UNRSTD;                                       0974
*         GPR04P=TRACPTR;                                          0975
         LR    GPR04P,TRACPTR                                      0975
*         DCL 1 FMZA0086 FIXED(31) STATIC LOCAL,                   0976
*             2 * FIXED(8) INIT(64),                               0976
*             2 * PTR(24) INIT(0);                                 0976
*         GPR03F = FMZA0086;                                       0977
*                                                                  0977
         L     GPR03F,FMZA0086                                     0977
*               DO; /* LISATCVT CVTBASE(PAGEABLE,(GPR15P)) */      0978
*                 GPR15P = ATCLCPTR; /*SET UP ATCVT POINTER*/      0979
         L     GPR15P,ATCLCPTR                                     0979
*               END; /*LISATCVT*/                                  0980
*         GPR02P = GPR15P->ATCBPDA;                                0981
         L     GPR02P,ATCBPDA(,GPR15P)                             0981
*         GPR15P=GPR15P->ATCSMRS;                                  0982
         L     GPR15P,ATCSMRS(,GPR15P)                             0982
*         DCL FMZL0086 BASED(GPR15P) LABEL VALUERANGE(*);          0983
*         CALL FMZL0086;                                           0984
         BALR  @14,@15                                             0984
*         RFY GPR13P RSTD;                                         0985
*         GENERATE REFS(GPR01P,RPHWORK) SETS(GPR02P,GPR13P);       0986
*                                                                  0986
          LM GPR02P,GPR13P,RPHWORK(GPR01P)
*         RFY (GPR01P,GPR13P) UNRSTD;                              0987
*               RFY (GPR02P,GPR03P,GPR04P,GPR15P) UNRSTD;          0988
*               END ;/**********END OF RELSTORE EXPANSION**********/
*    /*     RELEASE TRAC*/                                         0990
*             END;                                                 0990
*         END;                                                     0991
*        ELSE;                                    /*ERROR IN TBTRAC  */
@RF00557 DS    0H                                                  0993
*     END;                                                         0993
@RC00557 DS    0H                                                  0994
* R14=R14SAVE;                                    /*RESTORE CALLERS14*/
*                                                                  0994
@RT00457 LR    R14,R14SAVE                                         0994
*/* TPEPILOG   */                                                  0995
* GEN (PRINT OFF);                                                 0995
*                                                                  0995
*                                                                  0995
         PRINT OFF
* DECLARE                                                          0996
*   EMCR0 PTR REG(0),                                              0996
*   EMCR1 PTR REG(1),                                              0996
*   EMCR2 PTR REG(2),                                              0996
*   EMCR3 PTR REG(3),                                              0996
*   EMCR4 PTR REG(4),                                              0996
*   EMCR5 PTR REG(5),                                              0996
*   EMCR6 PTR REG(6),                                              0996
*   EMCR7 PTR REG(7),                                              0996
*   EMCR8 PTR REG(8),                                              0996
*   EMCR9 PTR REG(9),                                              0996
*   EMCR10 PTR REG(10),                                            0996
*   EMCR11 PTR REG(11),                                            0996
*   EMCR12 PTR REG(12),                                            0996
*   EMCR13 PTR REG(13),                                            0996
*   EMCR14 PTR REG(14),                                            0996
*   EMCR15 PTR REG(15);                                            0996
*                                                                  0996
* DECLARE                                                          0997
*   ISTSDCCA ENTRY BASED(ATCCDADD) VALRG(*), /*CIDCTL ADD ROUTINE*/
*   ISTSDCCD ENTRY BASED(ATCCDDEL) VALRG(*), /*CIDCTL DELETE ROUTINE*/
*   ISTSDCCF ENTRY BASED(ATCCDFND) VALRG(*), /*CIDCTL FIND ROUTINE*/
*   ISTSDCCN ENTRY BASED(ATCCDFDN) VALRG(*), /*CIDCTL FIND NCB ROUTINE*
*   ISTSDCCR ENTRY BASED(ATCCDFIN) VALRG(*); /*CIDCTL FINISH ROUTINE*/
*   DCL CMZCIDM FIXED(15) DEFINED(ATCCIDM); /*CIDMASK ALIAS*/      0998
* GEN (PRINT ON);                                                  0999
         PRINT ON
*/* END OF TPEPILOG */                                             1000
*END ISTRACTB                                     /* AND RETURN      */
*/* THE FOLLOWING INCLUDE STATEMENTS WERE FOUND IN THIS PROGRAM.      *
*/*%INCLUDE SYSLIB  (IFGRPL  )                                        *
*/*%INCLUDE SYSLIB  (IFGRPLVT)                                        *
*/*%INCLUDE SYSLIB  (ISTRPLFB)                                        *
*/*%INCLUDE SYSLIB  (CVT     )                                        *
*/*%INCLUDE SYSLIB  (IHAPSA  )                                        *
*/*%INCLUDE SYSUT5  (TBTRAC  )                                        *
*/*%INCLUDE SYSUT5  (TBSFDB  )                                        *
*/*%INCLUDE SYSUT5  (TBSRPL  )                                        *
*/*%INCLUDE SYSUT5  (TBSLDO  )                                        *
*/*%INCLUDE SYSUT5  (TBSRWSR )                                        *
*;                                                                 1000
@EL00001 DS    0H                                                  1000
@EF00001 DS    0H                                                  1000
@ER00001 BR    @14                                                 1000
@DATA    DS    0H
@SM04453 MVC   BUFDATA(0,BUFNX),BUFDATA(@09)
@SM04456 MVC   BUFDATA(0,BUFNX),BUFDATA(@03)
@SM04458 MVC   TRABFFER(0,TRACPTR),BUFDATA(FROMR5)
@DATD    DSECT
         DS    0F
ISTRACTB CSECT
         DS    0F
@CF00037 DC    F'4'
@CF00039 DC    F'8'
@CF04445 DC    F'-32'
@CF01359 DC    XL4'80000000'
@CF04455 DC    XL4'0000FFFF'
@DATD    DSECT
         DS    0D
CVTPTR   DS    A
FSBPTR   DS    A
FMCBPTR  DS    A
BDUPTR   DS    A
         ORG   *+1-(*-@DATD)/(*-@DATD) INSURE DSECT DATA
@ENDDATD EQU   *
ISTRACTB CSECT
         DS    0F
@SIZDATD DC    AL1(0)
         DC    AL3(@ENDDATD-@DATD)
         DS    0D
FDBINPUT DC    XL4'20000000'
INPTMSK  DC    XL4'FF000000'
LENGLDO  DC    AL4(8)
BUFCT    DC    AL4(256-32-8)
FMZA0010 DS    FL4
         ORG   FMZA0010
@NM00091 DC    AL1(152)
@NM00092 DC    AL3(0)
         ORG   FMZA0010+4
FMZA0070 DS    FL4
         ORG   FMZA0070
@NM00095 DC    AL1(64)
@NM00096 DC    AL3(0)
         ORG   FMZA0070+4
FMZA0086 DS    FL4
         ORG   FMZA0086
@NM00097 DC    AL1(64)
@NM00098 DC    AL3(0)
         ORG   FMZA0086+4
GTFID    DC    XL2'00FD'
LNG256   DC    XL2'0100'
CMAINT   DC    50H'0'
@00      EQU   00                      EQUATES FOR REGISTERS 0-15
@01      EQU   01
@02      EQU   02
@03      EQU   03
@04      EQU   04
@05      EQU   05
@06      EQU   06
@07      EQU   07
@08      EQU   08
@09      EQU   09
@10      EQU   10
@11      EQU   11
@12      EQU   12
@13      EQU   13
@14      EQU   14
@15      EQU   15
GPR01F   EQU   @01
GPR01P   EQU   @01
GPR02F   EQU   @02
GPR02P   EQU   @02
GPR03F   EQU   @03
GPR03P   EQU   @03
GPR04F   EQU   @04
GPR04P   EQU   @04
GPR06F   EQU   @06
GPR06P   EQU   @06
GPR10F   EQU   @10
GPR10P   EQU   @10
GPR11F   EQU   @11
GPR11P   EQU   @11
GPR12F   EQU   @12
GPR12P   EQU   @12
GPR13F   EQU   @13
GPR13P   EQU   @13
GPR14F   EQU   @14
GPR14P   EQU   @14
GPR15F   EQU   @15
GPR15P   EQU   @15
RPHPTR   EQU   @01
R0       EQU   @00
RBUFCT   EQU   @00
R2       EQU   @02
FMCBR2   EQU   @02
FDBR2    EQU   @02
LENR4    EQU   @04
R5       EQU   @05
TBCVTPTR EQU   @05
CIDSAVR5 EQU   @05
FMCBSVR5 EQU   @05
FROMR5   EQU   @05
BUFNX    EQU   @05
SAVRCR5  EQU   @05
R5RPHSAV EQU   @05
R6INPUT  EQU   @06
TRACPTR  EQU   @07
LDOR8    EQU   @08
DATAR9   EQU   @09
R14SAVE  EQU   @10
NCBPTR   EQU   @11
R11FLG   EQU   @11
R13      EQU   @13
R14      EQU   @14
RTNCODE  EQU   @15
EMCR2    EQU   @02
EMCR3    EQU   @03
EMCR0    EQU   @00
EMCR1    EQU   @01
EMCR10   EQU   @10
EMCR11   EQU   @11
EMCR12   EQU   @12
EMCR13   EQU   @13
EMCR14   EQU   @14
EMCR15   EQU   @15
EMCR4    EQU   @04
EMCR5    EQU   @05
EMCR6    EQU   @06
EMCR7    EQU   @07
EMCR8    EQU   @08
EMCR9    EQU   @09
GPR00F   EQU   @00
GPR00P   EQU   @00
GPR05F   EQU   @05
GPR05P   EQU   @05
GPR07F   EQU   @07
GPR07P   EQU   @07
GPR08F   EQU   @08
GPR08P   EQU   @08
GPR09F   EQU   @09
GPR09P   EQU   @09
PSA      EQU   0
FLCRNPSW EQU   PSA
FLCROPSW EQU   PSA+8
FLCICCW2 EQU   PSA+16
PSAEEPSW EQU   PSA+132
PSAESPSW EQU   PSA+136
FLCSVILC EQU   PSAESPSW+1
PSAEPPSW EQU   PSA+140
FLCPIILC EQU   PSAEPPSW+1
FLCPICOD EQU   PSAEPPSW+2
PSAPICOD EQU   FLCPICOD+1
FLCTEA   EQU   PSAEPPSW+4
FLCPER   EQU   PSA+152
FLCMCLA  EQU   PSA+168
FLCIOEL  EQU   FLCMCLA+4
FLCIOA   EQU   FLCMCLA+16
FLCFSA   EQU   FLCMCLA+80
PSASUPER EQU   PSA+552
PSASUP1  EQU   PSASUPER
PSASUP2  EQU   PSASUPER+1
PSASUP3  EQU   PSASUPER+2
PSASUP4  EQU   PSASUPER+3
PSACLHT  EQU   PSA+640
PSALKSA  EQU   PSA+696
PSAHLHI  EQU   PSA+760
PSALITA  EQU   PSA+764
PSADSSFL EQU   PSA+836
PSADSSF1 EQU   PSADSSFL
PSADSSF3 EQU   PSADSSFL+2
PSADSSF4 EQU   PSADSSFL+3
PSARSVT  EQU   PSA+896
ATCLCPTR EQU   1032
SYSCVTPT EQU   16
CZMP0037 EQU   16
CZMP0044 EQU   16
CZMP0051 EQU   16
IFGRPL   EQU   0
RPLCOMN  EQU   IFGRPL
RPLIDWD  EQU   RPLCOMN
RPLREQ   EQU   RPLIDWD+2
RPLLEN   EQU   RPLIDWD+3
RPLECB   EQU   RPLCOMN+8
RPLFDBWD EQU   RPLCOMN+12
RPLFDBK  EQU   RPLFDBWD+1
RPLRTNCD EQU   RPLFDBK
RPLCNDCD EQU   RPLFDBK+1
RPLCMPON EQU   RPLCNDCD
RPLFDB2  EQU   RPLCMPON
RPLERRCD EQU   RPLCNDCD+1
RPLFDB3  EQU   RPLERRCD
RPLKEYLE EQU   RPLCOMN+16
RPLAREA  EQU   RPLCOMN+32
RPLARG   EQU   RPLCOMN+36
RPLOPTCD EQU   RPLCOMN+40
RPLOPT1  EQU   RPLOPTCD
RPLECBSW EQU   RPLOPT1
RPLOPT2  EQU   RPLOPTCD+1
RPLADR   EQU   RPLOPT2
RPLOPT3  EQU   RPLOPTCD+2
RPLOPT4  EQU   RPLOPTCD+3
RPLNXTRP EQU   RPLCOMN+44
RPLRLEN  EQU   RPLCOMN+48
RPLBUFL  EQU   RPLCOMN+52
RPLOPTC2 EQU   RPLCOMN+56
RPLOPT5  EQU   RPLOPTC2
RPLWRTYP EQU   RPLOPT5
RPLEAU   EQU   RPLWRTYP
RPLOPT6  EQU   RPLOPTC2+1
RPLUNTYP EQU   RPLOPT6
RPLOPT7  EQU   RPLOPTC2+2
RPLCNOPT EQU   RPLOPT7
RPLOPT8  EQU   RPLOPTC2+3
RPLRBAR  EQU   RPLCOMN+60
RPLAIXID EQU   RPLRBAR+2
RPLEXTDS EQU   RPLCOMN+68
RPLEXTD1 EQU   RPLEXTDS
RPLVTEXT EQU   IFGRPL+76
RPLAAREA EQU   RPLVTEXT
RPLARCLN EQU   RPLVTEXT+8
RPLFDBK2 EQU   RPLVTEXT+12
RPLSSNSI EQU   RPLFDBK2
RPLDSB   EQU   RPLSSNSI
RPLDSB1  EQU   RPLDSB
RPLSSEI  EQU   RPLDSB1
RPLDSB2  EQU   RPLDSB+1
RPLUSNSI EQU   RPLFDBK2+2
RPLOPTC3 EQU   RPLVTEXT+20
RPLOPT9  EQU   RPLOPTC3
RPLOPT10 EQU   RPLOPTC3+1
RPLOPT11 EQU   RPLOPTC3+2
RPLOPT12 EQU   RPLOPTC3+3
RPLOSENS EQU   RPLVTEXT+24
RPLSSNSO EQU   RPLOSENS
RPLSSEO  EQU   RPLSSNSO
RPLOF16  EQU   0
RPLRH3   EQU   RPLOF16
RPLSRTYP EQU   RPLOF16+1
RPLSTYPE EQU   RPLSRTYP
RPLRTYPE EQU   RPLSRTYP
RPLCHN   EQU   RPLOF16+2
RPLVTFL1 EQU   RPLOF16+3
RPLVTFL2 EQU   RPLOF16+4
RPLPOSTV EQU   RPLVTFL2
RPLRESP  EQU   RPLVTFL2
RPLCNTRL EQU   RPLOF16+5
RPLCNTDF EQU   RPLCNTRL
RPLDATA  EQU   RPLCNTDF
RPLCNTDC EQU   RPLCNTRL+1
RPLCNTSC EQU   RPLCNTRL+2
RPLOF60  EQU   0
RPLOBSQ  EQU   RPLOF60+4
RPLIBSQ  EQU   RPLOF60+5
CVTMAP   EQU   0
CVT0VL00 EQU   CVTMAP+24
CVTDAR   EQU   CVTMAP+72
CVTFLGS1 EQU   CVTDAR
CVTDCB   EQU   CVTMAP+116
CVTIERLC EQU   CVTMAP+144
CVTOPTA  EQU   CVTMAP+182
CVTOPTB  EQU   CVTMAP+183
CVTGTF   EQU   CVTMAP+236
CVTGTFST EQU   CVTGTF
CVTGTFS  EQU   CVTGTFST
CVTGTFAV EQU   CVTGTFS
CVTSTATE EQU   CVTGTFST
CVTTMODE EQU   CVTGTFST
CVTFORM  EQU   CVTGTFST
CVTAQAVT EQU   CVTMAP+240
CVTTCMFG EQU   CVTAQAVT
CVTVOLM2 EQU   CVTMAP+244
CVTTATA  EQU   CVTVOLM2
CVTTSKS  EQU   CVTTATA
CVTVOLF2 EQU   CVTTSKS
CVTTAT   EQU   CVTTATA+1
CVTATER  EQU   CVTMAP+248
CVTEXT1  EQU   CVTMAP+252
CVTPURG  EQU   CVTMAP+260
CVTQMSG  EQU   CVTMAP+268
CVTDMSR  EQU   CVTMAP+272
CVTERPV  EQU   CVTMAP+316
CVTINTLA EQU   CVTMAP+320
CVTAPF   EQU   CVTMAP+324
CVTEXT2  EQU   CVTMAP+328
CVTHJES  EQU   CVTMAP+332
CVTPGSIA EQU   CVTMAP+348
CVTA1F1  EQU   CVTMAP+356
CVTSYSK  EQU   CVTMAP+357
CVTVOLM1 EQU   CVTMAP+380
CVTVOLF1 EQU   CVTVOLM1
CVTATMCT EQU   CVTMAP+388
CVTXTNT1 EQU   0
CVTXTNT2 EQU   0
CVTDSSV  EQU   CVTXTNT2
CVTFLGBT EQU   CVTXTNT2+5
CVTQID   EQU   CVTXTNT2+24
CVTSHS   EQU   CVTXTNT2+40
CVTRV400 EQU   CVTXTNT2+52
CVTRV409 EQU   CVTXTNT2+53
CVTATCVT EQU   CVTXTNT2+64
CVTRV429 EQU   CVTXTNT2+84
CVTRV438 EQU   CVTXTNT2+85
CVTRV457 EQU   CVTXTNT2+112
CVTRV466 EQU   CVTXTNT2+113
CVTFIX   EQU   0
CVTRELNO EQU   CVTFIX+252
ISTPAB   EQU   0
PABWQCHN EQU   ISTPAB
PABWEQA  EQU   PABWQCHN
PABCHAIN EQU   PABWQCHN+4
PABRPHFG EQU   ISTPAB+12
PABFLAGS EQU   PABRPHFG
PABERLCK EQU   PABFLAGS
PABERRLK EQU   PABERLCK
ISTLOK   EQU   0
LOKCHN   EQU   ISTLOK+1
LOKCHNG  EQU   LOKCHN
@NM00054 EQU   LOKCHNG+2
ISTDYPAB EQU   0
ISTRPH   EQU   0
RPHCNTL  EQU   ISTRPH
RPHFLAGS EQU   RPHCNTL+2
RPHAPTYP EQU   RPHFLAGS
RPHFSTLC EQU   RPHFLAGS
RPHFLGB  EQU   RPHCNTL+3
RPHRPHA  EQU   ISTRPH+4
RPHTSKID EQU   ISTRPH+8
RPHRESMA EQU   ISTRPH+16
RPHWPFLG EQU   RPHRESMA
RPHWEA   EQU   ISTRPH+24
RPHCSPA  EQU   RPHWEA
RPHSRPRM EQU   ISTRPH+28
RPHPABQA EQU   ISTRPH+36
RPHWORK  EQU   ISTRPH+40
RPHSAVE1 EQU   RPHWORK
RPHSBITS EQU   RPHSAVE1+1
ISTATCVT EQU   0
ATCCOM   EQU   ISTATCVT
ATCSTAT  EQU   ATCCOM
ATCSTAT1 EQU   ATCSTAT
ATCSTFLG EQU   ATCSTAT+1
ATCPOFLG EQU   ATCSTAT+2
ATCSTAT4 EQU   ATCSTAT+3
ATCBPDA  EQU   ATCCOM+128
ATCIOTRC EQU   ATCCOM+140
ATCTHTRC EQU   ATCIOTRC
ATCSMRQ  EQU   ATCCOM+180
ATCSMRS  EQU   ATCCOM+188
ATCCIDM  EQU   ATCCOM+266
ATCCDADD EQU   ISTATCVT+276
ATCCDDEL EQU   ISTATCVT+280
ATCCDFND EQU   ISTATCVT+284
ATCCSMR  EQU   ISTATCVT+296
ATCSHRFG EQU   ISTATCVT+316
ATCHPGM  EQU   ISTATCVT+384
ATCCDFIN EQU   ISTATCVT+460
ATCCDFDN EQU   ISTATCVT+464
ATCVLCNT EQU   ISTATCVT+772
ATCRNO   EQU   ISTATCVT+788
ATCSOPD  EQU   ISTATCVT+792
ATCSIPD  EQU   ISTATCVT+800
ATCSECST EQU   ISTATCVT+812
ATCSTAT2 EQU   ISTATCVT+818
@NM00065 EQU   ISTATCVT+911
ATCAOSM  EQU   ISTATCVT+912
ATCGSRBQ EQU   ATCAOSM+4
ATCGSRBG EQU   ATCGSRBQ
ATCICAPP EQU   ATCAOSM+100
ATCLDAPP EQU   ATCAOSM+120
ATCDAPVT EQU   ATCAOSM+140
ATCVTFLG EQU   ATCAOSM+392
ATCZFLAG EQU   ATCAOSM+588
ISTFDB   EQU   0
FDBFDBA  EQU   ISTFDB
FDBDATAA EQU   ISTFDB+4
FDBDATCT EQU   ISTFDB+8
ISTFSB   EQU   0
FSBFSB   EQU   ISTFSB
FSBMNRCD EQU   FSBFSB+2
FSBSFLGS EQU   FSBFSB+3
FSBFSBA  EQU   FSBFSB+4
FSBBTH   EQU   FSBFSB+8
FSBCID   EQU   FSBBTH
FSBRQTAG EQU   FSBBTH+4
FSBSEQID EQU   FSBRQTAG
FSBFLAGS EQU   FSBSEQID
FSBSEQ   EQU   FSBSEQID+1
FSBCSWST EQU   FSBBTH+6
FSBTHFLD EQU   FSBCSWST
FSBTH1   EQU   FSBTHFLD
FSBSYSRS EQU   FSBTH1
FSBTH2   EQU   FSBTHFLD+1
FSBEXTRS EQU   FSBTH2
FSBXFSTS EQU   FSBEXTRS
FSBXCODE EQU   FSBEXTRS
FSBNETRS EQU   FSBFSB+16
FSBSRCAT EQU   FSBNETRS
FSBERC   EQU   FSBNETRS+1
FSBFBMI  EQU   FSBERC
FSBDMGRS EQU   FSBFSB+18
FSBSS1   EQU   FSBDMGRS
FSBSS2   EQU   FSBDMGRS+1
FSBLCPB  EQU   FSBFSB+20
FSBMSGID EQU   FSBLCPB
FSBLCCWA EQU   FSBFSB+24
FSBRLSTS EQU   FSBLCCWA
ISTDEVCH EQU   0
DEVSHCH  EQU   ISTDEVCH
DEVCHAR  EQU   DEVSHCH
DEVTCODE EQU   ISTDEVCH+1
DEVFLAGS EQU   ISTDEVCH+3
DEVFCCTL EQU   DEVFLAGS
DEVCHAR3 EQU   DEVFLAGS
ISTPROCD EQU   0
PROPROC  EQU   ISTPROCD
PROPROC1 EQU   PROPROC
PROPROC2 EQU   PROPROC+1
PROCFTX  EQU   PROPROC2
PROPROC3 EQU   PROPROC+2
PROPROC4 EQU   PROPROC+3
PROPROCS EQU   0
PROSETMS EQU   PROPROCS+1
PROSETM2 EQU   PROSETMS
PROSETM3 EQU   PROSETMS+1
PROSETM4 EQU   PROSETMS+2
ISTFMCB  EQU   0
FMCDNCBA EQU   ISTFMCB+16
FMCSTAT  EQU   ISTFMCB+28
FMCSTAT1 EQU   ISTFMCB+29
FMCCMDFL EQU   ISTFMCB+31
FMCDEBA  EQU   ISTFMCB+32
FMCCMPS1 EQU   ISTFMCB+36
FMCPROCD EQU   ISTFMCB+48
FMCSIPA  EQU   ISTFMCB+148
FMCNXOSQ EQU   ISTFMCB+152
FMCNXCFM EQU   FMCNXOSQ
FMCNXOSC EQU   FMCNXOSQ+2
FMCSSTAT EQU   ISTFMCB+159
FMCPACE  EQU   ISTFMCB+160
FMCPACEN EQU   FMCPACE
FMCPACEM EQU   FMCPACE+1
FMCPACPC EQU   FMCPACE+2
FMCCVAL  EQU   FMCPACE+3
FMCOBCE  EQU   FMCCVAL
FMCCLEXT EQU   ISTFMCB+164
FMCRPLA  EQU   FMCCLEXT
FMCFBA   EQU   FMCCLEXT+4
FMCCMPS2 EQU   FMCCLEXT+16
FMCFLGA  EQU   FMCCMPS2
FMCFLGCL EQU   FMCCMPS2+1
FMCPSCMD EQU   FMCCMPS2+2
FMCPSTAT EQU   FMCCMPS2+3
FMCSTAT2 EQU   ISTFMCB+186
FMCMODEB EQU   ISTFMCB+189
FMCCLSEQ EQU   ISTFMCB+190
FMCRRESP EQU   ISTFMCB+195
FMCSSEGQ EQU   ISTFMCB+200
FMCFBAS  EQU   ISTFMCB+216
FMCZV002 EQU   FMCFBAS
FMCSNCTL EQU   ISTFMCB+224
FMCSCFLG EQU   FMCSNCTL
FMCVS1   EQU   ISTFMCB+232
FMCZFLAG EQU   FMCVS1
FMCZF006 EQU   FMCZFLAG
FMCLBXM  EQU   FMCVS1+12
FMCSHEAQ EQU   FMCLBXM
FMCLCPBQ EQU   FMCVS1+16
FMCLCPAQ EQU   FMCVS1+20
FMCSEQ1  EQU   FMCVS1+34
FMCZFLG2 EQU   FMCVS1+35
FMCCVFLG EQU   FMCVS1+44
FMCFLAG  EQU   0
FMCFLGB  EQU   FMCFLAG
ISTLDO   EQU   0
LDOCMD   EQU   ISTLDO
LDOFLAGS EQU   ISTLDO+1
LDOACHN  EQU   LDOFLAGS
LDOCMHN  EQU   LDOFLAGS
LDOLEN   EQU   ISTLDO+2
LDOADDR  EQU   ISTLDO+4
ISTNCB   EQU   0
NCBRDTE  EQU   ISTNCB+4
NCBFLAGS EQU   ISTNCB+24
NCBFRAS  EQU   NCBFLAGS+1
NCBFTRIO EQU   NCBFRAS
NCBFLAG1 EQU   NCBFLAGS+2
NCBLTRC  EQU   ISTNCB+30
ISTRH    EQU   0
RHFLAGS  EQU   ISTRH
RH1      EQU   RHFLAGS
RH2      EQU   RHFLAGS+1
RHRETRY  EQU   RH2
RHRSVD1  EQU   RH2
RH3      EQU   RHFLAGS+2
ISTRU    EQU   0
RUDATA   EQU   ISTRU
ISTRUB   EQU   0
NSRUNA5  EQU   0
ISTBDU   EQU   0
BDUMODIF EQU   ISTBDU+1
BDUMOD1  EQU   BDUMODIF
BDUDAF   EQU   ISTBDU+2
BDUFUNC  EQU   BDUDAF
BDUCHECK EQU   BDUFUNC
BDUFLAGS EQU   BDUDAF+1
ISTTRAC  EQU   0
TRA1WORD EQU   ISTTRAC
TRATYPE  EQU   TRA1WORD
TRALNG   EQU   TRA1WORD+1
TRARSV01 EQU   TRA1WORD+2
TRACHAIN EQU   ISTTRAC+4
TRATRCB  EQU   ISTTRAC+8
TRABDW   EQU   TRATRCB
TRAFLG   EQU   ISTTRAC+12
TRAFRST  EQU   TRAFLG
TRABF    EQU   TRAFLG
TRADNODF EQU   TRAFLG
TRAANODF EQU   TRAFLG
TRAFLG2  EQU   ISTTRAC+13
TRACXTF  EQU   TRAFLG2
TRADIO   EQU   TRAFLG2
TRADATCT EQU   ISTTRAC+14
TRADNODE EQU   ISTTRAC+16
TRACID   EQU   TRADNODE
TRALRC   EQU   ISTTRAC+28
TRADATA  EQU   ISTTRAC+32
ISTACDEB EQU   0
ACDVTPRX EQU   ISTACDEB
ACDDBPFX EQU   ISTACDEB+16
ACDTBLOF EQU   ACDDBPFX+6
ACDBASIC EQU   ISTACDEB+24
ACDTCBAD EQU   ACDBASIC
ACDNMSUB EQU   ACDTCBAD
ACDDEBAD EQU   ACDBASIC+4
ACDUSRPG EQU   ACDBASIC+16
ACDACBAD EQU   ACDBASIC+24
ACDAPPAD EQU   ACDBASIC+28
ACDVTAM  EQU   ISTACDEB+56
ACDRDTE  EQU   ACDVTAM+4
ACDSSFLG EQU   ACDVTAM+80
ACDAPDAT EQU   ACDVTAM+83
ISTRPRE  EQU   0
RPRNAME  EQU   ISTRPRE
RPRCOMID EQU   ISTRPRE+8
RPRFLAGA EQU   ISTRPRE+21
RPRFLAGB EQU   ISTRPRE+22
RPRCAUSE EQU   ISTRPRE+23
RPRFLAGD EQU   ISTRPRE+61
RPRFLAGC EQU   ISTRPRE+62
ISTRCPRE EQU   0
RCPPMODE EQU   ISTRCPRE+91
RCPLGNIN EQU   ISTRCPRE+92
ISTRDT   EQU   0
RDTPLEN  EQU   ISTRDT+72
RDTPRIOR EQU   RDTPLEN
RDTFLAGB EQU   ISTRDT+91
RDTVYRPH EQU   ISTRDT+108
RDTSTCNT EQU   ISTRDT+112
ISTRAP   EQU   0
RAPFLG   EQU   ISTRAP+108
RAPFLGB  EQU   ISTRAP+109
RAPFLGC  EQU   ISTRAP+110
ISTRLC   EQU   0
RLCADR   EQU   ISTRLC+72
RLCFLAG  EQU   RLCADR
ISTRRN   EQU   0
RRNADR   EQU   ISTRRN+136
RRNFLAG  EQU   RRNADR
RRNUCBAD EQU   RRNADR+1
RRNRDF   EQU   ISTRRN+172
RRNRDFA  EQU   RRNRDF
RRNRDFB  EQU   RRNRDF+1
RRNCMF   EQU   ISTRRN+176
RRNCMFA  EQU   RRNCMF
RRNCMFB  EQU   RRNCMF+1
RRNCMFC  EQU   RRNCMF+2
RRNCMFD  EQU   RRNCMF+3
RRNNMDPD EQU   ISTRRN+192
RRNHPRE  EQU   ISTRRN+216
ISTRGP   EQU   0
RGPFLG   EQU   ISTRGP+72
ISTRLN   EQU   0
RLNADR   EQU   ISTRLN+72
RLNFLAG  EQU   RLNADR
RLNUCBAD EQU   RLNADR+1
RLNSTAT1 EQU   ISTRLN+86
RLNSTAT2 EQU   ISTRLN+87
RLNRDF   EQU   ISTRLN+88
RLNRDFA  EQU   RLNRDF
RLNRACNT EQU   RLNRDFA
RLNRDFB  EQU   RLNRDF+1
RLNCRFLG EQU   ISTRLN+102
RLNLSCMD EQU   RLNCRFLG
ISTRPT   EQU   0
RPTLNCID EQU   ISTRPT+116
RPTDITT  EQU   ISTRPT+128
RPTFLAG1 EQU   ISTRPT+140
ISTRCL   EQU   0
RCLADR   EQU   ISTRCL+72
RCLFLAG  EQU   RCLADR
ISTRTR   EQU   0
RTRADR   EQU   ISTRTR+124
RTRFLAG  EQU   RTRADR
RTRRDFA  EQU   ISTRTR+136
RTRRAALL EQU   RTRRDFA
RTRRDFB  EQU   ISTRTR+137
RTRRDFC  EQU   ISTRTR+138
RTRRDFD  EQU   ISTRTR+139
RTRRDFE  EQU   ISTRTR+140
RTRRDFF  EQU   ISTRTR+141
RTRCRFLG EQU   ISTRTR+143
RTRLSCMD EQU   RTRCRFLG
RTRBSCID EQU   ISTRTR+188
ISTRIN   EQU   0
RINDFD   EQU   ISTRIN+124
RINREMPO EQU   RINDFD
RINNODEF EQU   RINDFD
RINLDISC EQU   RINDFD
RINNODID EQU   ISTRIN+125
RINRLDAF EQU   ISTRIN+126
RINREMOT EQU   ISTRIN+128
RINVPAB  EQU   ISTRIN+132
RINRPHCN EQU   ISTRIN+136
RINNCSPL EQU   ISTRIN+140
RINCNCNT EQU   ISTRIN+144
ISTRDS   EQU   0
ISTDOT   EQU   0
ISTRCC   EQU   0
RCCFLAG  EQU   ISTRCC+73
RCCFLAGB EQU   ISTRCC+74
RCCSTAID EQU   ISTRCC+78
RCCID48  EQU   RCCSTAID+2
RCCCRFLG EQU   ISTRCC+108
ISTRLU   EQU   0
RLUFLAG  EQU   ISTRLU+125
RLUCRFLG EQU   ISTRLU+126
RLUPACE  EQU   ISTRLU+132
RLUNCPPC EQU   ISTRLU+134
RLUUSSTB EQU   ISTRLU+140
RLUMODET EQU   ISTRLU+152
ISTRPX   EQU   0
ISTRLX   EQU   0
RLXFLAGA EQU   ISTRLX+76
ISTRSW   EQU   0
ISTRDA   EQU   0
RDAADR   EQU   ISTRDA+72
RDAFLAG  EQU   RDAADR
RDAUCBAD EQU   RDAADR+1
ISTRLS   EQU   0
ISTDNT   EQU   0
ISTGNT   EQU   0
ISTPAT   EQU   0
PATFLAGA EQU   ISTPAT+7
BUFFER   EQU   0
BUFDATA  EQU   BUFFER
ISTSDCCF EQU   0
FMZL0010 EQU   0
ISTSDCCR EQU   0
SAV0037  EQU   0
SAV10037 EQU   SAV0037
SAV20037 EQU   SAV0037+4
SAV40037 EQU   SAV0037+8
SAV60037 EQU   SAV0037+12
SAVE0037 EQU   SAV0037+16
ISTPST   EQU   0
PSTRQFGA EQU   ISTPST+8
PSTRQUE  EQU   PSTRQFGA
PSTFLGA  EQU   PSTRQFGA+4
PSTABINP EQU   PSTFLGA
PSTTCBA  EQU   ISTPST+16
PSTDMPS  EQU   ISTPST+132
SAV0044  EQU   0
SAV10044 EQU   SAV0044
SAV20044 EQU   SAV0044+4
SAV40044 EQU   SAV0044+8
SAV60044 EQU   SAV0044+12
SAVE0044 EQU   SAV0044+16
SAV0051  EQU   0
SAV10051 EQU   SAV0051
SAV20051 EQU   SAV0051+4
SAV40051 EQU   SAV0051+8
SAV60051 EQU   SAV0051+12
SAVE0051 EQU   SAV0051+16
FMZL0070 EQU   0
FMZL0086 EQU   0
ISTRPL   EQU   0
ISTSDCCA EQU   0
ISTSDCCD EQU   0
ISTSDCCN EQU   0
CVTS01   EQU   CVTPGSIA
CVTLPDIA EQU   CVTS01+12
CVTDIRST EQU   CVTLPDIA
CVTSLIDA EQU   CVTS01+24
CVTCTLFG EQU   CVTS01+50
CVTRV210 EQU   CVTS01+424
CVTRV219 EQU   CVTS01+425
CVTRV228 EQU   CVTS01+426
CVTRV237 EQU   CVTS01+427
CVTMFRTR EQU   CVTS01+452
CVTRV262 EQU   CVTS01+468
CVTRV271 EQU   CVTS01+469
CVTRV280 EQU   CVTS01+470
CVTRV289 EQU   CVTS01+471
CVTGSDA  EQU   CVTS01+600
FLC      EQU   PSA
PSARSVTE EQU   PSARSVT
FSBCOMBN EQU   FSBNETRS
FSBRHFLD EQU   FSBCOMBN+1
FSBRH3   EQU   FSBRHFLD+2
FSBLDFLG EQU   FSBRH3
@NM00071 EQU   FSBRQTAG
ISTSENRU EQU   RUDATA
SRUSENSE EQU   ISTSENRU
SRUSENS1 EQU   SRUSENSE
SRUSENS2 EQU   SRUSENSE+2
ISTSRU   EQU   ISTRUB
SRUDATA  EQU   ISTSRU+1
SRUTYPE  EQU   SRUDATA
SRULDID  EQU   SRUDATA+1
SRUSPSN  EQU   SRULDID
ISTNSRU  EQU   ISTRUB
NSRUNETS EQU   ISTNSRU
NSRUDATA EQU   ISTNSRU+3
NSRUHOOK EQU   NSRUDATA
NSSSVRU  EQU   NSRUDATA
NSSSVNOD EQU   NSSSVRU+3
NSSSVT3  EQU   NSSSVNOD
NSSSVT3C EQU   NSSSVT3+2
NSSSVT3G EQU   NSSSVT3+5
NSSSVT4  EQU   NSSSVNOD
NSSSVT5  EQU   NSSSVNOD
NSRUDAT1 EQU   NSRUDATA
NSRUDAT2 EQU   NSRUDAT1+2
ISTTRRU  EQU   NSRUDAT2
ISTCTDRU EQU   NSRUDAT2
ISTINORU EQU   NSRUDAT2
SRUDAT1  EQU   SRUDATA
SRUDAT1L EQU   SRUDAT1+1
SRUDAT2  EQU   SRUDATA
ISTMRU   EQU   ISTRUB
SRUDATA3 EQU   SRUDATA
SRUDATA4 EQU   SRUDATA
NSRUDAT3 EQU   NSRUDAT2
NSRUDAT4 EQU   NSRUDAT2
NSRUDAT5 EQU   NSRUDAT2
NSRUDAT6 EQU   NSRUDAT2
NSRUDAT7 EQU   NSRUDAT2
NSRUDIL  EQU   NSRUDAT2
NSRUDILB EQU   NSRUDIL+1
NSRUNA   EQU   NSRUDAT2
NSRUNA2  EQU   NSRUNA+1
NSRUCLSP EQU   NSRUDAT2
NSRUOH1  EQU   NSRUDAT2
NSRUDAT8 EQU   NSRUDATA
ISTUNBRU EQU   SRUTYPE
TRAIODT  EQU   TRADATA
TRATH    EQU   TRAIODT+22
TRABDUD  EQU   TRAIODT+35
TRABUFFR EQU   TRADATA
TRAANODE EQU   TRABUFFR
TRABFFER EQU   TRABUFFR+8
CMZCIDM  EQU   ATCCIDM
RTRDNCBA EQU   ISTRTR+40
         AGO   .@UNREFD                START UNREFERENCED COMPONENTS
@NM00084 EQU   TRABDUD+4
TRASEND  EQU   TRABDUD
TRARH    EQU   TRAIODT+32
TRATHDCF EQU   TRATH+8
TRATHSNF EQU   TRATH+6
TRATHOAF EQU   TRATH+4
TRATHDAF EQU   TRATH+2
TRATHHD  EQU   TRATH
TRARSV06 EQU   TRAIODT+21
TRAISEN  EQU   TRAIODT+20
TRAICPA  EQU   TRAIODT+16
TRAIPSW  EQU   TRAIODT+8
TRAICSW  EQU   TRAIODT
UNBTYPE  EQU   ISTUNBRU
NSRUDA8T EQU   NSRUDAT8
NSRUOHID EQU   NSRUOH1
NSRULSV  EQU   NSRUCLSP
NSRUNA4  EQU   NSRUNA+2
NSRUNA3  EQU   NSRUNA2
NSRUNA1  EQU   NSRUNA
NSRUDILF EQU   NSRUDIL+4
NSRUDILE EQU   NSRUDIL+3
NSRUDILD EQU   NSRUDIL+2
NSRUDILC EQU   NSRUDILB
@NM00083 EQU   NSRUDILB
NSRUDILA EQU   NSRUDIL
NSRUDA7T EQU   NSRUDAT7
NSRUDA6L EQU   NSRUDAT6+4
NSRUDA6S EQU   NSRUDAT6
NSRUDA5E EQU   NSRUDAT5
NSRUDA4T EQU   NSRUDAT4
NSRUDA3S EQU   NSRUDAT3
SRUDAT4D EQU   SRUDATA4
SRUDAT3I EQU   SRUDATA3
MRUDATA  EQU   ISTMRU+9
MRURESP  EQU   ISTMRU+7
MRUMODIF EQU   ISTMRU+6
MRUCMND  EQU   ISTMRU+5
MRUDAF   EQU   ISTMRU+3
MRUREQ   EQU   ISTMRU+2
MRUPREF  EQU   ISTMRU
SRUDAT2N EQU   SRUDAT2+1
SRUDAT2T EQU   SRUDAT2
SRUDAT1I EQU   SRUDAT1+2
SRUTSPRO EQU   SRUDAT1L
SRUFMPRO EQU   SRUDAT1L
SRUDAT1T EQU   SRUDAT1
INORUET  EQU   ISTINORU
CTDRUSTA EQU   ISTCTDRU
TRRDATA  EQU   ISTTRRU+4
TRRSTAT  EQU   ISTTRRU+3
TRRPEP   EQU   ISTTRRU+2
TRRTIME  EQU   ISTTRRU+1
TRRTYPE  EQU   ISTTRRU
NSRUDAF  EQU   NSRUDAT1
NSSSVT5A EQU   NSSSVT5
NSSSVT4D EQU   NSSSVT4+3
NSSSVT4C EQU   NSSSVT4+2
NSSSVT4B EQU   NSSSVT4+1
NSSSVT4A EQU   NSSSVT4
NSSSVT3K EQU   NSSSVT3+8
NSSSVT3J EQU   NSSSVT3+7
NSSSVT3I EQU   NSSSVT3+6
@NM00082 EQU   NSSSVT3G
NSSSVT3H EQU   NSSSVT3G
NSSSVT3F EQU   NSSSVT3+4
NSSSVT3E EQU   NSSSVT3+3
@NM00081 EQU   NSSSVT3C
NSSSVT3D EQU   NSSSVT3C
NSSSVT3B EQU   NSSSVT3+1
NSSSVT3A EQU   NSSSVT3
NSSSVT   EQU   NSSSVRU+2
NSSSVDAF EQU   NSSSVRU
NSRUHKTY EQU   NSRUHOOK
NSRURCD  EQU   NSRUNETS+2
NSRUSUB  EQU   NSRUNETS+1
NSRUID   EQU   NSRUNETS
SRURSV01 EQU   SRULDID+4
SRUPSSN  EQU   SRULDID+2
SRUDFCLV EQU   SRUSPSN
@NM00080 EQU   SRUTYPE
SRUPSAC  EQU   SRUTYPE
SRUSPAC  EQU   SRUTYPE
SRUQSREQ EQU   ISTSRU
SENRU    EQU   ISTSENRU+4
SRUSNS4  EQU   SRUSENS2+1
SRUSNS3  EQU   SRUSENS2
SRUSNST  EQU   SRUSENS1+1
SRUSNS1  EQU   SRUSENS1
FSBRT0B  EQU   @NM00071+1
FSBRT0A  EQU   @NM00071+1
FSBRT09  EQU   @NM00071+1
FSBRT08  EQU   @NM00071+1
FSBRT07  EQU   @NM00071
FSBRT06  EQU   @NM00071
FSBRT05  EQU   @NM00071
FSBRT04  EQU   @NM00071
FSBRT03  EQU   @NM00071
FSBRT02  EQU   @NM00071
FSBRT01  EQU   @NM00071
FSBRT00  EQU   @NM00071
FSBLDCUF EQU   FSBLDFLG
FSBLDSIF EQU   FSBLDFLG
FSBLDSOF EQU   FSBLDFLG
FSBLDEBF EQU   FSBLDFLG
FSBLDBBF EQU   FSBLDFLG
FSBLDFME EQU   FSBLDFLG
FSBLDSCF EQU   FSBLDFLG
FSBLDRMF EQU   FSBLDFLG
FSBRH2   EQU   FSBRHFLD+1
FSBRH1   EQU   FSBRHFLD
FSBALIGN EQU   FSBCOMBN
PSARSAV  EQU   PSARSVTE+60
PSARSTK  EQU   PSARSVTE+56
PSAESAV3 EQU   PSARSVTE+52
PSAESTK3 EQU   PSARSVTE+48
PSAESAV2 EQU   PSARSVTE+44
PSAESTK2 EQU   PSARSVTE+40
PSAESAV1 EQU   PSARSVTE+36
PSAESTK1 EQU   PSARSVTE+32
PSAPSAV  EQU   PSARSVTE+28
PSAPSTK  EQU   PSARSVTE+24
PSAMSAV  EQU   PSARSVTE+20
PSAMSTK  EQU   PSARSVTE+16
PSASSAV  EQU   PSARSVTE+12
PSASSTK  EQU   PSARSVTE+8
PSANSTK  EQU   PSARSVTE+4
PSACSTK  EQU   PSARSVTE
CVTRV628 EQU   CVTS01+728
CVTRV627 EQU   CVTS01+724
CVTRV626 EQU   CVTS01+720
CVTRV625 EQU   CVTS01+716
CVTRV624 EQU   CVTS01+712
CVTRV623 EQU   CVTS01+708
CVTRV622 EQU   CVTS01+704
CVTRV621 EQU   CVTS01+700
CVTRV620 EQU   CVTS01+696
CVTSUSP  EQU   CVTS01+692
CVTT6SVC EQU   CVTS01+688
CVTCDAL  EQU   CVTS01+684
CVTTCTL  EQU   CVTS01+680
CVTRSUME EQU   CVTS01+676
CVTJTERM EQU   CVTS01+672
CVTASMRM EQU   CVTS01+668
CVTTCASP EQU   CVTS01+664
CVT0PT03 EQU   CVTS01+660
CVT0PT0E EQU   CVTS01+656
CVTIRM   EQU   CVTS01+652
CVTCGK   EQU   CVTS01+648
CVTRAC   EQU   CVTS01+644
CVTRV606 EQU   CVTS01+640
CVTRV605 EQU   CVTS01+636
CVTRV604 EQU   CVTS01+632
CVTEFF02 EQU   CVTS01+628
CVTCBBR  EQU   CVTS01+624
CVTSSCR  EQU   CVTS01+620
CVTEVENT EQU   CVTS01+616
CVTCRCA  EQU   CVTS01+612
CVTTPIO  EQU   CVTS01+608
CVTADV   EQU   CVTS01+604
CVTGSDAB EQU   CVTGSDA
CVTQV3   EQU   CVTS01+596
CVTQV2   EQU   CVTS01+592
CVTQV1   EQU   CVTS01+588
CVTRPT   EQU   CVTS01+584
CVTSSRB  EQU   CVTS01+580
CVTCSDRL EQU   CVTS01+576
CVTEXP1  EQU   CVTS01+572
CVTRMPMT EQU   CVTS01+568
CVTRMPTT EQU   CVTS01+564
CVTVPSA  EQU   CVTS01+560
CVTVSTOP EQU   CVTS01+556
CVTGTFR8 EQU   CVTS01+552
CVTQUIT  EQU   CVTS01+548
CVTVACR  EQU   CVTS01+544
CVTWTCB  EQU   CVTS01+540
CVTSTPRS EQU   CVTS01+536
CVT0PT02 EQU   CVTS01+532
CVTDARCM EQU   CVTS01+528
CVTIRECM EQU   CVTS01+524
CVTJRECM EQU   CVTS01+520
CVTVEMS0 EQU   CVTS01+516
CVTSPFRR EQU   CVTS01+512
CVTRLSTG EQU   CVTS01+508
CVT0TC0A EQU   CVTS01+504
CVTGMBR  EQU   CVTS01+500
CVTLFRM  EQU   CVTS01+496
CVTRMBR  EQU   CVTS01+492
CVTVIOP  EQU   CVTS01+488
CVTRV307 EQU   CVTS01+486
CVTRV306 EQU   CVTS01+484
CVTRV305 EQU   CVTS01+482
CVTRV304 EQU   CVTS01+480
CVTRV303 EQU   CVTS01+478
CVTRV302 EQU   CVTS01+476
CVTTRCA  EQU   CVTS01+472
CVTRV297 EQU   CVTRV289
CVTRV296 EQU   CVTRV289
CVTRV295 EQU   CVTRV289
CVTRV294 EQU   CVTRV289
CVTRV293 EQU   CVTRV289
CVTRV292 EQU   CVTRV289
CVTRV291 EQU   CVTRV289
CVTRV290 EQU   CVTRV289
CVTRV288 EQU   CVTRV280
CVTRV287 EQU   CVTRV280
CVTRV286 EQU   CVTRV280
CVTRV285 EQU   CVTRV280
CVTRV284 EQU   CVTRV280
CVTRV283 EQU   CVTRV280
CVTRV282 EQU   CVTRV280
CVTRV281 EQU   CVTRV280
CVTRV279 EQU   CVTRV271
CVTRV278 EQU   CVTRV271
CVTRV277 EQU   CVTRV271
CVTRV276 EQU   CVTRV271
CVTRV275 EQU   CVTRV271
CVTRV274 EQU   CVTRV271
CVTRV273 EQU   CVTRV271
CVTRV272 EQU   CVTRV271
CVTRV270 EQU   CVTRV262
CVTRV269 EQU   CVTRV262
CVTRV268 EQU   CVTRV262
CVTRV267 EQU   CVTRV262
CVTRV266 EQU   CVTRV262
CVTRV265 EQU   CVTRV262
CVTRV264 EQU   CVTRV262
CVTRV263 EQU   CVTRV262
CVTVFP   EQU   CVTS01+464
CVTVSI   EQU   CVTS01+460
CVTVPSIB EQU   CVTS01+456
CVTMFACT EQU   CVTMFRTR
CVTMFCTL EQU   CVTS01+448
CVTPVBP  EQU   CVTS01+444
CVTPWI   EQU   CVTS01+440
CVTRV254 EQU   CVTS01+438
CVTRV253 EQU   CVTS01+436
CVTRV252 EQU   CVTS01+434
CVTRV251 EQU   CVTS01+433
CVTRV250 EQU   CVTS01+432
CVTRV249 EQU   CVTS01+431
CVTRV248 EQU   CVTS01+430
CVTRV247 EQU   CVTS01+429
CVTRV246 EQU   CVTS01+428
CVTRV245 EQU   CVTRV237
CVTRV244 EQU   CVTRV237
CVTRV243 EQU   CVTRV237
CVTRV242 EQU   CVTRV237
CVTRV241 EQU   CVTRV237
CVTRV240 EQU   CVTRV237
CVTRV239 EQU   CVTRV237
CVTRV238 EQU   CVTRV237
CVTRV236 EQU   CVTRV228
CVTRV235 EQU   CVTRV228
CVTRV234 EQU   CVTRV228
CVTRV233 EQU   CVTRV228
CVTRV232 EQU   CVTRV228
CVTRV231 EQU   CVTRV228
CVTRV230 EQU   CVTRV228
CVTRV229 EQU   CVTRV228
CVTRV227 EQU   CVTRV219
CVTRV226 EQU   CVTRV219
CVTRV225 EQU   CVTRV219
CVTRV224 EQU   CVTRV219
CVTRV223 EQU   CVTRV219
CVTRV222 EQU   CVTRV219
CVTRV221 EQU   CVTRV219
CVTRV220 EQU   CVTRV219
CVTRV218 EQU   CVTRV210
CVTRV217 EQU   CVTRV210
CVTRV216 EQU   CVTRV210
CVTRV215 EQU   CVTRV210
CVTRV214 EQU   CVTRV210
CVTRV213 EQU   CVTRV210
CVTRV212 EQU   CVTRV210
CVTRV211 EQU   CVTRV210
CVTLCCAT EQU   CVTS01+420
CVTPCCAT EQU   CVTS01+416
CVTIPCRP EQU   CVTS01+412
CVTIPCRI EQU   CVTS01+408
CVTIPCDS EQU   CVTS01+404
CVTAIDVT EQU   CVTS01+400
CVTSSAP  EQU   CVTS01+396
CVTEHCIR EQU   CVTS01+392
CVTEHDEF EQU   CVTS01+388
CVTDAIR  EQU   CVTS01+384
CVTPERFM EQU   CVTS01+380
CVT044R2 EQU   CVTS01+376
CVTFETCH EQU   CVTS01+372
CVTRSTWD EQU   CVTS01+368
CVTSPOST EQU   CVTS01+364
CVTIOBP  EQU   CVTS01+360
CVTASMVT EQU   CVTS01+356
CVTRECRQ EQU   CVTS01+352
CVTWSAC  EQU   CVTS01+348
CVTRV149 EQU   CVTS01+344
CVTWSAL  EQU   CVTS01+340
CVTSPSA  EQU   CVTS01+336
CVTGLMN  EQU   CVTS01+332
CVTVEAC0 EQU   CVTS01+328
CVT062R1 EQU   CVTS01+324
CVTRPOST EQU   CVTS01+320
CVTDQIQE EQU   CVTS01+316
CVTCSD   EQU   CVTS01+312
CVTLKRMA EQU   CVTS01+308
CVTRSPIE EQU   CVTS01+304
CVTRENQ  EQU   CVTS01+300
CVTLQCB  EQU   CVTS01+296
CVTFQCB  EQU   CVTS01+292
CVTQCS01 EQU   CVTS01+288
CVTAPFT  EQU   CVTS01+284
CVTPARRL EQU   CVTS01+280
CVTVWAIT EQU   CVTS01+276
CVTGSPL  EQU   CVTS01+272
CVTLSMQ  EQU   CVTS01+268
CVTGSMQ  EQU   CVTS01+264
CVTEXPRO EQU   CVTS01+260
CVTOPCTP EQU   CVTS01+256
CVTSIC   EQU   CVTS01+252
CVTTPIOS EQU   CVTS01+248
CVTRTMS  EQU   CVTS01+244
CVTSDBF  EQU   CVTS01+240
CVTSCBP  EQU   CVTS01+236
CVTSDMP  EQU   CVTS01+232
CVTSV60  EQU   CVTS01+228
CVTRTMCT EQU   CVTS01+224
CVTASCBL EQU   CVTS01+220
CVTASCBH EQU   CVTS01+216
CVTGDA   EQU   CVTS01+212
CVTASVT  EQU   CVTS01+208
CVTVVMDI EQU   CVTS01+204
CVTAQTOP EQU   CVTS01+200
CVTIOSCS EQU   CVTS01+196
CVTSDRM  EQU   CVTS01+192
CVTOPTE  EQU   CVTS01+188
CVTSTXU  EQU   CVTS01+184
CVTQUIS  EQU   CVTS01+180
CVTPARS  EQU   CVTS01+176
CVTS1EE  EQU   CVTS01+172
CVTFRAS  EQU   CVTS01+168
CVTQSAS  EQU   CVTS01+164
CVTCRAS  EQU   CVTS01+160
CVTCRMN  EQU   CVTS01+156
CVTDELCP EQU   CVTS01+152
CVTFRECL EQU   CVTS01+148
CVTGETCL EQU   CVTS01+144
CVTBLDCP EQU   CVTS01+140
CVTAUTHL EQU   CVTS01+136
CVTSCAN  EQU   CVTS01+132
CVTRV144 EQU   CVTS01+130
CVTMAXMP EQU   CVTS01+128
CVTSTCK  EQU   CVTS01+124
CVTRV139 EQU   CVTS01+123
CVTDSSAC EQU   CVTS01+122
CVTRV513 EQU   CVTS01+121
CVTIOSPL EQU   CVTS01+120
CVTPTGT  EQU   CVTS01+116
CVTCSPIE EQU   CVTS01+112
CVTSMFEX EQU   CVTS01+108
CVTOLT0A EQU   CVTS01+104
CVTSRBRT EQU   CVTS01+100
CVTPUTL  EQU   CVTS01+96
CVTASCRL EQU   CVTS01+92
CVTASCRF EQU   CVTS01+88
CVTRV326 EQU   CVTS01+84
CVTRV325 EQU   CVTS01+80
CVTRV324 EQU   CVTS01+76
CVT0VL01 EQU   CVTS01+72
CVTSHRVM EQU   CVTS01+68
CVTRV332 EQU   CVTS01+64
CVTTAS   EQU   CVTS01+60
CVTRSCN  EQU   CVTS01+56
CVTTRAC2 EQU   CVTS01+54
CVTTRACE EQU   CVTS01+52
CVTAPG   EQU   CVTS01+51
CVTSDTRC EQU   CVTCTLFG
CVTGTRCE EQU   CVTCTLFG
CVTNOMP  EQU   CVTCTLFG
CVTRSV79 EQU   CVTCTLFG
CVTDSTAT EQU   CVTCTLFG
CVTRSV78 EQU   CVTCTLFG
CVTRV333 EQU   CVTCTLFG
CVTRV323 EQU   CVTCTLFG
CVTSPVLK EQU   CVTS01+49
CVTRSV77 EQU   CVTS01+48
CVTRV331 EQU   CVTS01+44
CVTRV330 EQU   CVTS01+40
CVTRV329 EQU   CVTS01+36
CVTRV328 EQU   CVTS01+32
CVTRV322 EQU   CVTS01+28
CVTSLID  EQU   CVTSLIDA+1
CVTSYLK  EQU   CVTSLIDA
CVTRV321 EQU   CVTS01+20
CVTRV320 EQU   CVTS01+16
CVTLPDIR EQU   CVTLPDIA+1
CVTRSV69 EQU   CVTDIRST
CVTRSV68 EQU   CVTDIRST
CVTRSV67 EQU   CVTDIRST
CVTRSV66 EQU   CVTDIRST
CVTRSV65 EQU   CVTDIRST
CVTRSV64 EQU   CVTDIRST
CVTRSV63 EQU   CVTDIRST
CVTDICOM EQU   CVTDIRST
CVTPVTP  EQU   CVTS01+8
CVTLPDSR EQU   CVTS01+4
CVTGETL  EQU   CVTS01
PSTDRDYQ EQU   PSTDMPS+1
@NM00094 EQU   PSTDMPS
PSTDGATE EQU   PSTDMPS
PSTDEBP  EQU   ISTPST+128
PSTSAVE  EQU   ISTPST+96
PSTSMDYP EQU   ISTPST+64
PSTCRRS  EQU   ISTPST+60
PSTCRR1  EQU   ISTPST+56
PSTIQE22 EQU   ISTPST+52
PSTIQE21 EQU   ISTPST+48
PSTIQE12 EQU   ISTPST+44
PSTIQE11 EQU   ISTPST+40
PSTUEQ   EQU   ISTPST+36
PSTNSSQ  EQU   ISTPST+32
PSTPSSQ  EQU   ISTPST+28
PSTACBCT EQU   ISTPST+24
PSTMPSTP EQU   ISTPST+20
PSTRSV08 EQU   PSTFLGA+1
PSTDMPC  EQU   PSTFLGA+1
PSTABEND EQU   PSTABINP
PSTAUTH  EQU   PSTFLGA
PSTSMCU  EQU   PSTFLGA
PSTSMQR  EQU   PSTFLGA
PSTIRBU2 EQU   PSTFLGA
PSTIRBU1 EQU   PSTFLGA
PSTUERN  EQU   PSTFLGA
PSTSSRN  EQU   PSTFLGA
PSTRQUEA EQU   PSTRQUE+1
PSTRSV07 EQU   PSTRQUE
PSTRSV06 EQU   PSTRQUE
PSTRSV05 EQU   PSTRQUE
PSTRSV04 EQU   PSTRQUE
PSTRSV03 EQU   PSTRQUE
PSTRSV02 EQU   PSTRQUE
PSTRSV01 EQU   PSTRQUE
PSTRQFLG EQU   PSTRQUE
PSTCHAIN EQU   ISTPST+4
@NM00093 EQU   ISTPST+2
PSTLNGTH EQU   ISTPST+1
PSTTYPE  EQU   ISTPST
PATRSV01 EQU   PATFLAGA
PATUSEFG EQU   PATFLAGA
PATRDCNT EQU   ISTPAT+6
PATGRPID EQU   ISTPAT+5
PATPTHID EQU   ISTPAT+4
PATDINDX EQU   ISTPAT+2
PATGINDX EQU   ISTPAT
GNTGRPNM EQU   ISTGNT
DNTTELNO EQU   ISTDNT+1
DNTCTDIG EQU   ISTDNT
RLSRSV02 EQU   ISTRLS+140
RLSRSV01 EQU   ISTRLS+138
RLSCTELU EQU   ISTRLS+136
RLSPRE   EQU   ISTRLS
RDARSVPT EQU   ISTRDA+84
RDAPUBI  EQU   ISTRDA+83
RDADFCUA EQU   ISTRDA+80
RDACTPCC EQU   ISTRDA+79
RDADACUA EQU   ISTRDA+76
RDALUBA  EQU   RDAUCBAD+1
RDACHQI  EQU   RDAUCBAD
RDARSV01 EQU   RDAFLAG
RDARBISA EQU   RDAFLAG
RDAPRE   EQU   ISTRDA
RSWRSV01 EQU   ISTRSW+152
RSWRSV02 EQU   ISTRSW+150
RSWCTELU EQU   ISTRSW+148
RSWGRUPZ EQU   ISTRSW+146
RSWDIALZ EQU   ISTRSW+144
RSWGRUPT EQU   ISTRSW+140
RSWDIALT EQU   ISTRSW+136
RSWPRE   EQU   ISTRSW
RLXRSV03 EQU   ISTRLX+80
RLXRSV02 EQU   ISTRLX+77
RLXRSV01 EQU   RLXFLAGA
RLXINUSE EQU   RLXFLAGA
RLXNLUX  EQU   ISTRLX+72
RLXPRE   EQU   ISTRLX
RPXRSV01 EQU   ISTRPX+80
RPXRSV02 EQU   ISTRPX+77
RPXMAXLU EQU   ISTRPX+76
RPXFLUX  EQU   ISTRPX+72
RPXPRE   EQU   ISTRPX
RLURSV02 EQU   ISTRLU+172
RLULGMOD EQU   ISTRLU+164
RLUMODEP EQU   RLUMODET+8
RLUMODEN EQU   RLUMODET
RLUUSSPT EQU   RLUUSSTB+8
RLUUSSNM EQU   RLUUSSTB
RLURSV01 EQU   ISTRLU+137
RLULOCAD EQU   ISTRLU+136
RLUNCPPM EQU   RLUNCPPC+1
RLUNCPPN EQU   RLUNCPPC
RLUPACEM EQU   RLUPACE+1
RLUPACEN EQU   RLUPACE
RLUINTA  EQU   ISTRLU+128
RLUSSCFM EQU   ISTRLU+127
RLUCRRSV EQU   RLUCRFLG
RLUACTLU EQU   RLUCRFLG
RLURSV00 EQU   RLUFLAG
RLUGSTAT EQU   RLUFLAG
RLUBATCH EQU   RLUFLAG
RLURBISA EQU   RLUFLAG
RLUTYPE  EQU   ISTRLU+124
RLUCRPR  EQU   ISTRLU
RCCRVPT2 EQU   ISTRCC+144
RCCXTOSW EQU   ISTRCC+140
RCCSWTOX EQU   ISTRCC+136
RCCRSPT2 EQU   ISTRCC+133
RCCADDR  EQU   ISTRCC+132
RCCPASSL EQU   ISTRCC+131
RCCMXOUT EQU   ISTRCC+130
RCCMXDAT EQU   ISTRCC+128
RCCINDEX EQU   ISTRCC+126
RCCPATHZ EQU   ISTRCC+124
RCCPATHT EQU   ISTRCC+120
RCCDPQ   EQU   ISTRCC+116
RCCTPAB  EQU   ISTRCC+112
RCCRSV02 EQU   ISTRCC+109
RCCCRRSV EQU   RCCCRFLG
RCCCTCFL EQU   RCCCRFLG
RCCACTPU EQU   RCCCRFLG
RCCNCSPL EQU   ISTRCC+104
RCCRPHCN EQU   ISTRCC+100
RCCEPAB  EQU   ISTRCC+96
RCCDPAB  EQU   ISTRCC+92
RCCVPAB  EQU   ISTRCC+88
@NM00090 EQU   ISTRCC+87
RCCSESCT EQU   ISTRCC+86
RCCIDNUM EQU   RCCID48+3
RCCIDBLK EQU   RCCID48+2
RCCIDPA2 EQU   RCCID48+1
RCCIDPUT EQU   RCCID48
RCCIDPA1 EQU   RCCSTAID
RCCCTELU EQU   ISTRCC+76
RCCCNCNT EQU   ISTRCC+75
RCCRTGOH EQU   RCCFLAGB
RCCBRKCN EQU   RCCFLAGB
RCCRESET EQU   RCCFLAGB
RCCFINAL EQU   RCCFLAGB
RCCBNNSP EQU   RCCFLAGB
RCCIRETR EQU   RCCFLAGB
RCCDIALP EQU   RCCFLAGB
RCCDSTCT EQU   RCCFLAGB
RCCCTPGD EQU   RCCFLAG
RCCPRGDN EQU   RCCFLAG
RCCOUERP EQU   RCCFLAG
RCCOHIMM EQU   RCCFLAG
RCCLDISC EQU   RCCFLAG
RCCERPIP EQU   RCCFLAG
RCCCUDIE EQU   RCCFLAG
RCCRBISA EQU   RCCFLAG
RCCTYPE  EQU   ISTRCC+72
RCCPRE   EQU   ISTRCC
DOTADTRM EQU   ISTDOT
RDSADDOT EQU   ISTRDS+76
RDSCTATR EQU   ISTRDS+74
RDSCTTRM EQU   ISTRDS+72
RDSPRE   EQU   ISTRDS
RINRSV05 EQU   ISTRIN+156
RINRSV04 EQU   ISTRIN+152
RINLOCRN EQU   ISTRIN+148
RINRSV02 EQU   ISTRIN+145
RTRCNCNT EQU   RINCNCNT
RTRNCSPL EQU   RINNCSPL
RTRRPHCN EQU   RINRPHCN
RTRVPAB  EQU   RINVPAB
RTRREMOT EQU   RINREMOT
RTRRLDAF EQU   RINRLDAF
RTRNODID EQU   RINNODID
RINRSV01 EQU   RINDFD
RTRLDISC EQU   RINLDISC
RTRNODEF EQU   RINNODEF
RTRREMPO EQU   RINREMPO
RINCRPRE EQU   ISTRIN
RTRRSV04 EQU   ISTRTR+210
RTRLCTI  EQU   ISTRTR+209
RTRIDCAS EQU   RTRBSCID+1
RTRIDLEN EQU   RTRBSCID
RTRRSVDD EQU   ISTRTR+181
RTRRDFG  EQU   ISTRTR+180
RTRDIAA  EQU   ISTRTR+176
RTRPOLA  EQU   ISTRTR+172
RTRCADDR EQU   ISTRTR+164
RTRCTADR EQU   ISTRTR+162
RTRCTPOL EQU   ISTRTR+160
RTRCTDIA EQU   ISTRTR+158
RTRRSV03 EQU   ISTRTR+156
RTRRVPT1 EQU   ISTRTR+152
RTRBHSN1 EQU   ISTRTR+144
RTRCRSV2 EQU   RTRCRFLG
RTRDTRLM EQU   RTRLSCMD
RTRCRSV1 EQU   RTRCRFLG
RTRTRMAX EQU   ISTRTR+142
RTRRSV24 EQU   RTRRDFF
RTRRFDEK EQU   RTRRDFF
RTRRFANK EQU   RTRRDFF
RTRRFPFK EQU   RTRRDFF
RTRRFPEN EQU   RTRRDFF
RTRRFXCT EQU   RTRRDFF
RTRRESCT EQU   RTRRDFE
RTRRESUP EQU   RTRRDFE
RTRREATT EQU   RTRRDFE
RTRRECHK EQU   RTRRDFE
RTRREACR EQU   RTRRDFE
RTRRSV0A EQU   RTRRDFE
RTRREBRK EQU   RTRRDFE
RTRRSV11 EQU   RTRRDFD
RTRRSV10 EQU   RTRRDFD
RTRRDBHI EQU   RTRRDFD
RTRRDBHD EQU   RTRRDFD
RTRRDBHN EQU   RTRRDFD
RTRRSV23 EQU   RTRRDFC
RTRRCXMS EQU   RTRRDFC
RTRRCMSG EQU   RTRRDFC
RTRRCBLK EQU   RTRRDFC
RTRRCPT3 EQU   RTRRDFC
RTRRCIT2 EQU   RTRRDFC
RTRRCIT1 EQU   RTRRDFC
RTRRSV32 EQU   RTRRDFB
RTRRBEND EQU   RTRRDFB
RTRRBECT EQU   RTRRDFB
RTRRSV02 EQU   RTRRDFA
RTRRACTR EQU   RTRRDFA
RTRRSV60 EQU   RTRRDFA
RTRRACDT EQU   RTRRDFA
RTRRAPT3 EQU   RTRRAALL
RTRRAPT2 EQU   RTRRAALL
RTRRAPT1 EQU   RTRRAALL
RTRRAATN EQU   RTRRDFA
RTRRSV45 EQU   ISTRTR+132
RTRRSV01 EQU   ISTRTR+130
RTRCTECM EQU   ISTRTR+128
RTRADRPT EQU   RTRADR+1
RTRRSV22 EQU   RTRFLAG
RTRVTERM EQU   RTRFLAG
RTRRSV51 EQU   RTRFLAG
RTRRBISA EQU   RTRFLAG
RTRRSV21 EQU   RTRFLAG
RTRCRPR  EQU   ISTRTR
RCLRVPT2 EQU   ISTRCL+92
RCLRVPT1 EQU   ISTRCL+88
RCLGPOLA EQU   ISTRCL+84
RCLCTNGP EQU   ISTRCL+82
RCLRSV01 EQU   ISTRCL+80
RCLCTECM EQU   ISTRCL+78
RCLCTETR EQU   ISTRCL+76
RCLGPADR EQU   RCLADR+1
RCLRSV33 EQU   RCLFLAG
RCLBATCH EQU   RCLFLAG
RCLRBISA EQU   RCLFLAG
RCLRSV22 EQU   RCLFLAG
RCLPRE   EQU   ISTRCL
RPTRSV03 EQU   ISTRPT+156
RPTRSV02 EQU   ISTRPT+152
RPTNSNSV EQU   ISTRPT+148
RPTPTHCT EQU   ISTRPT+146
RPTPEPCT EQU   ISTRPT+144
RPTDAFSV EQU   ISTRPT+142
RPTRSV01 EQU   ISTRPT+141
RPTISTAT EQU   RPTFLAG1
RPTDYNAM EQU   RPTFLAG1
RPTVYIMM EQU   RPTFLAG1
RPTVIODN EQU   RPTFLAG1
RPTONLIN EQU   RPTFLAG1
RPTHLVPD EQU   RPTFLAG1
RPTVRYPD EQU   RPTFLAG1
RPTASLEP EQU   RPTFLAG1
RPTUTERM EQU   ISTRPT+136
RPTATERM EQU   ISTRPT+132
RPTDIVT  EQU   RPTDITT
RPTCTNM  EQU   ISTRPT+120
RPTLNDAF EQU   RPTLNCID+2
RPTLNSAF EQU   RPTLNCID
RPTLN    EQU   ISTRPT
RLNRVPT2 EQU   ISTRLN+112
RLNNCSPL EQU   ISTRLN+108
RLNX2SW  EQU   ISTRLN+104
@NM00089 EQU   ISTRLN+103
RLNCRRSV EQU   RLNCRFLG
RLNSWEP  EQU   RLNCRFLG
RLNSVSKP EQU   RLNLSCMD
RLNSESLM EQU   RLNLSCMD
RLNNPRLM EQU   RLNLSCMD
RLNSWNCP EQU   RLNCRFLG
RLNACTLN EQU   RLNCRFLG
RLNPAUSE EQU   ISTRLN+101
RLNPOLMT EQU   ISTRLN+100
RLNACTSN EQU   ISTRLN+98
RLNRESV1 EQU   ISTRLN+97
RLNACTRM EQU   ISTRLN+96
RLNCTSES EQU   ISTRLN+95
RLNRDSLM EQU   ISTRLN+94
RLNAUTOA EQU   ISTRLN+92
RLNRDFD  EQU   RLNRDF+3
RLNRDFC  EQU   RLNRDF+2
RLNMULTP EQU   RLNRDFB
RLNRATRN EQU   RLNRDFB
RLNRAPOL EQU   RLNRDFB
RLNTTV   EQU   RLNRDFB
RLNRSV0B EQU   RLNRDFB
RLNAUTOD EQU   RLNRDFB
RLNANSWR EQU   RLNRDFB
RLNRBUSE EQU   RLNRDFB
RLNRAPEP EQU   RLNRDFA
RLNRANCP EQU   RLNRDFA
RLNRAEP  EQU   RLNRDFA
RLNIDV   EQU   RLNRDFA
RLNPORT  EQU   RLNRDFA
RLNGANSW EQU   RLNRDFA
RLNRACAN EQU   RLNRACNT
RLNRACAT EQU   RLNRACNT
RLNRSV04 EQU   RLNSTAT2
RLNINUSE EQU   RLNSTAT2
RLNOHIN  EQU   RLNSTAT2
RLNRSV03 EQU   RLNSTAT1
RLNVD    EQU   RLNSTAT1
RLNINOPP EQU   RLNSTAT1
RLNINOPL EQU   RLNSTAT1
RLNLRST  EQU   RLNSTAT1
RLNTRAP  EQU   RLNSTAT1
RLNACTCT EQU   ISTRLN+84
RLNCTECM EQU   ISTRLN+82
RLNCTEIN EQU   ISTRLN+80
RLNCTETR EQU   ISTRLN+78
RLNCTECL EQU   ISTRLN+76
RLNCUA   EQU   RLNUCBAD
RLNRSV01 EQU   RLNFLAG
RLNRBISA EQU   RLNFLAG
RLNEAM   EQU   RLNFLAG
RLNPRE   EQU   ISTRLN
RGPRVPT2 EQU   ISTRGP+88
RGPRVPT1 EQU   ISTRGP+84
RGPCTECM EQU   ISTRGP+82
RGPCTEIN EQU   ISTRGP+80
RGPCTETR EQU   ISTRGP+78
RGPCTECL EQU   ISTRGP+76
RGPCTELN EQU   ISTRGP+74
RGPRSV03 EQU   ISTRGP+73
RGPRSV02 EQU   RGPFLG
RGPASDLC EQU   RGPFLG
RGPABSC  EQU   RGPFLG
RGPASS   EQU   RGPFLG
RGPADIL  EQU   RGPFLG
RGPPRE   EQU   ISTRGP
RRNRINNM EQU   ISTRRN+248
@NM00088 EQU   ISTRRN+247
RRNRDCUA EQU   ISTRRN+244
RRNLASTE EQU   ISTRRN+240
RRNNCSPL EQU   ISTRRN+236
RRNRPLPT EQU   ISTRRN+232
RRNMSIZE EQU   ISTRRN+228
RRNNCPNM EQU   ISTRRN+221
RRNCTPCC EQU   ISTRRN+220
RRNHBUFS EQU   ISTRRN+218
RRNHLENT EQU   RRNHPRE+1
RRNHLENH EQU   RRNHPRE
RRNNMNCP EQU   ISTRRN+208
RRNDELAY EQU   ISTRRN+206
RRNRSV04 EQU   ISTRRN+200
RRNRSV88 EQU   RRNNMDPD+7
RRNPUBI  EQU   RRNNMDPD+6
RRNDPLUB EQU   RRNNMDPD
RRNEPADR EQU   ISTRRN+190
RRNCPLUB EQU   ISTRRN+184
RRNFLUX  EQU   ISTRRN+180
RRNRSV05 EQU   RRNCMFD
RRNREMPO EQU   RRNCMFD
RRNCDREC EQU   RRNCMFD
RRNCDDSI EQU   RRNCMFD
RRNCDBCK EQU   RRNCMFD
RRNCDDTL EQU   RRNCMFD
RRNCDSTD EQU   RRNCMFD
RRNCDCSP EQU   RRNCMFD
RRNCCCSL EQU   RRNCMFC
RRNCCSIM EQU   RRNCMFC
RRNCCRCM EQU   RRNCMFC
RRNCCRIM EQU   RRNCMFC
RRNCCCRC EQU   RRNCMFC
RRNCCRST EQU   RRNCMFC
RRNCCRDQ EQU   RRNCMFC
RRNCCRCN EQU   RRNCMFC
RRNCBALT EQU   RRNCMFB
RRNCBMOD EQU   RRNCMFB
RRNCBNPL EQU   RRNCMFB
RRNCBRLS EQU   RRNCMFB
RRNCBRDS EQU   RRNCMFB
RRNCBDLR EQU   RRNCMFB
RRNCBDLH EQU   RRNCMFB
RRNCBDLO EQU   RRNCMFB
RRNCADGO EQU   RRNCMFA
RRNCADTD EQU   RRNCMFA
RRNCACDT EQU   RRNCMFA
RRNCAMBH EQU   RRNCMFA
RRNCAATI EQU   RRNCMFA
RRNCAATG EQU   RRNCMFA
RRNCAATD EQU   RRNCMFA
RRNCAALL EQU   RRNCMFA
RRNRSV55 EQU   RRNRDF+3
RRNPTASL EQU   RRNRDFB+1
RRNRSV01 EQU   RRNRDFB+1
RRNVYDON EQU   RRNRDFB+1
RRNRBBUP EQU   RRNRDFB+1
RRNRSV03 EQU   RRNRDFB
RRNRBCT2 EQU   RRNRDFB
RRNRBCT1 EQU   RRNRDFB
RRNRBSEC EQU   RRNRDFB
RRNRBCCT EQU   RRNRDFB
RRNRBCCL EQU   RRNRDFB
RRNRSV02 EQU   RRNRDFB
RRNRBMDL EQU   RRNRDFB
RRNRAPEP EQU   RRNRDFA
RRNRANCP EQU   RRNRDFA
RRNRBREM EQU   RRNRDFA
RRNRAOLT EQU   RRNRDFA
RRNRASYN EQU   RRNRDFA
RRNRADMP EQU   RRNRDFA
RRNRAIPL EQU   RRNRDFA
RRNRASHT EQU   RRNRDFA
RRNCTENT EQU   ISTRRN+170
RRNCTECM EQU   ISTRRN+168
RRNCTEIN EQU   ISTRRN+166
RRNCTETR EQU   ISTRRN+164
RRNCTECL EQU   ISTRRN+162
RRNCTELI EQU   ISTRRN+160
RRNCTEGP EQU   ISTRRN+158
RRNCTBHS EQU   ISTRRN+156
RRNBHSET EQU   ISTRRN+152
RRNTRENT EQU   ISTRRN+148
RRNRSV22 EQU   ISTRRN+147
RRNRNCUA EQU   ISTRRN+144
RRNGRPA  EQU   ISTRRN+140
RRNLUBA  EQU   RRNUCBAD+1
RRNCHQI  EQU   RRNUCBAD
RRNRSV11 EQU   RRNFLAG
RRNDSTCT EQU   RRNFLAG
RRNERPIP EQU   RRNFLAG
RRNLCDIP EQU   RRNFLAG
RRNLCDIE EQU   RRNFLAG
RRNLCSIP EQU   RRNFLAG
RRNRBISA EQU   RRNFLAG
RRNRSV89 EQU   RRNFLAG
RRNPRE   EQU   ISTRRN
RLCRVPT2 EQU   ISTRLC+80
RLCRVPT1 EQU   ISTRLC+79
RLCLCCUA EQU   ISTRLC+76
RLCUCBAD EQU   RLCADR+1
RLCRSVDE EQU   RLCFLAG
RLCACNT  EQU   RLCFLAG
RLCRSV14 EQU   RLCFLAG
RLCRBISA EQU   RLCFLAG
RLCRSV22 EQU   RLCFLAG
RLCPRE   EQU   ISTRLC
RAPRVPT2 EQU   ISTRAP+116
RAPRVPT1 EQU   ISTRAP+112
RAPBFACT EQU   ISTRAP+111
RAPRSV17 EQU   RAPFLGC
RAPRSV16 EQU   RAPFLGC
RAPRSV15 EQU   RAPFLGC
RAPRSV14 EQU   RAPFLGC
RAPRSV13 EQU   RAPFLGC
RAPRSV12 EQU   RAPFLGC
RAPRSV11 EQU   RAPFLGC
RAPRSV10 EQU   RAPFLGC
RAPRSV09 EQU   RAPFLGB
RAPASPO  EQU   RAPFLGB
RAPAPPO  EQU   RAPFLGB
RAPAACQ  EQU   RAPFLGB
RAPAPRMT EQU   RAPFLGB
RAPAPASS EQU   RAPFLGB
RAPABLCK EQU   RAPFLGB
RAPALLF  EQU   RAPFLGB
RAPRSV06 EQU   RAPFLG
RAPRSV05 EQU   RAPFLG
RAPRSV04 EQU   RAPFLG
RAPRSV03 EQU   RAPFLG
RAPATLGN EQU   RAPFLG
RAPASLGI EQU   RAPFLG
RAPAQUI  EQU   RAPFLG
RAPABLGN EQU   RAPFLG
RAPACEQD EQU   ISTRAP+106
RAPTRPRE EQU   ISTRAP+104
RAPACEAP EQU   ISTRAP+100
RAPICEAQ EQU   ISTRAP+96
RAPICEAC EQU   ISTRAP+92
RAPICELG EQU   ISTRAP+88
RAPAPPWD EQU   ISTRAP+80
RAPACDEB EQU   ISTRAP+76
RAPACBA  EQU   ISTRAP+72
RAPPRE   EQU   ISTRAP
RDTVSRPL EQU   ISTRDT+132
RDTCRSPW EQU   ISTRDT+124
RDTCRSDS EQU   ISTRDT+116
@NM00087 EQU   ISTRDT+113
RRNSTCNT EQU   RDTSTCNT
RRNVYRPH EQU   RDTVYRPH
RDTTPAB  EQU   ISTRDT+104
RDTDPAB  EQU   ISTRDT+100
RDTEPAB  EQU   ISTRDT+96
RDTVPAB  EQU   ISTRDT+92
RDTRSV01 EQU   RDTFLAGB
RDTBHSET EQU   RDTFLAGB
RDTCTCFL EQU   RDTFLAGB
RDTCTPGD EQU   RDTFLAGB
RDTPRGDN EQU   RDTFLAGB
RDTOUERP EQU   RDTFLAGB
RDTLGCT  EQU   ISTRDT+90
RDTNODID EQU   ISTRDT+89
RDTMAXID EQU   ISTRDT+88
RDTCTSNT EQU   ISTRDT+86
RDTFLAGS EQU   ISTRDT+84
RDTBACK  EQU   ISTRDT+80
RDTFORW  EQU   ISTRDT+76
RDTLEN   EQU   RDTPLEN+1
RDTPRILS EQU   RDTPRIOR
RDTPRISW EQU   RDTPRIOR
RDTPRIRN EQU   RDTPRIOR
RDTPRILC EQU   RDTPRIOR
RDTPRIAP EQU   RDTPRIOR
@NM00086 EQU   RDTPRIOR
RDTPRE   EQU   ISTRDT
RCPRSV05 EQU   ISTRCPRE+120
RCPRSV04 EQU   ISTRCPRE+116
RCPRSV03 EQU   ISTRCPRE+112
RCPINTA  EQU   RCPLGNIN+16
RCPLGNCT EQU   RCPLGNIN+8
RCPLGNAP EQU   RCPLGNIN
@NM00085 EQU   RCPPMODE
RCPRRECD EQU   RCPPMODE
RCPRBASC EQU   RCPPMODE
RCPBFLIM EQU   ISTRCPRE+90
RCPBHSI2 EQU   ISTRCPRE+89
RCPBHSI1 EQU   ISTRCPRE+88
RCPACETR EQU   ISTRCPRE+84
RCPICEAQ EQU   ISTRCPRE+80
RCPLNAPA EQU   ISTRCPRE+76
RCPICELN EQU   ISTRCPRE+72
RCPPRE   EQU   ISTRCPRE
RPRCONID EQU   ISTRPRE+63
RPRCLVPD EQU   RPRFLAGC
RPRINIMM EQU   RPRFLAGD
RPRRSV05 EQU   RPRFLAGD
RPRCONP  EQU   RPRFLAGD
RPRACTP  EQU   RPRFLAGD
RPRRSON  EQU   RPRFLAGD
RPRCONCT EQU   RPRFLAGD
RPRLTASN EQU   RPRFLAGD
RPRPTHCT EQU   ISTRPRE+60
RPRSWDAF EQU   ISTRPRE+58
RPRSTPCT EQU   ISTRPRE+56
RPRRVPT1 EQU   ISTRPRE+52
RPRDEVCH EQU   ISTRPRE+44
RPRNCBA  EQU   ISTRPRE+40
RPRNSNEA EQU   ISTRPRE+36
RPRLGONB EQU   ISTRPRE+32
RPRLGONF EQU   ISTRPRE+28
RPRHDTYP EQU   ISTRPRE+27
RPRTIOCT EQU   ISTRPRE+26
RPRPEPCT EQU   ISTRPRE+24
RPRRSV03 EQU   RPRCAUSE
RPRRSYNC EQU   RPRCAUSE
RPRCRCPT EQU   RPRCAUSE
RPRNOSRT EQU   RPRCAUSE
RPRINTST EQU   RPRFLAGB
RPRVYIMM EQU   RPRFLAGB
RPRHLVPD EQU   RPRFLAGB
RPRONLIN EQU   RPRFLAGB
RPRVIODN EQU   RPRFLAGB
RPRAOPN  EQU   RPRFLAGB
RPRENTVT EQU   RPRFLAGB
RPRENTVH EQU   RPRFLAGB
RPRLIVE  EQU   RPRFLAGA
RPRVRYPD EQU   RPRFLAGA
RPRENTED EQU   RPRFLAGA
RPRENTIO EQU   RPRFLAGA
RPRENTBF EQU   RPRFLAGA
RPRTIED  EQU   RPRFLAGA
RPRINV   EQU   RPRFLAGA
RPRACT   EQU   RPRFLAGA
RPRENTRY EQU   ISTRPRE+20
RPRPELEN EQU   ISTRPRE+18
RPRELEN  EQU   ISTRPRE+16
RPRRLOCK EQU   ISTRPRE+12
RPRDAF   EQU   RPRCOMID+2
RPRSAF   EQU   RPRCOMID
ACDREFQ  EQU   ISTACDEB+204
ACDASFQ  EQU   ISTACDEB+200
ACDSOP   EQU   ISTACDEB+184
ACDSIP   EQU   ISTACDEB+168
ACDRSV24 EQU   ACDVTAM+108
ACDRSV23 EQU   ACDVTAM+104
ACDRSV22 EQU   ACDVTAM+100
ACDOCWAD EQU   ACDVTAM+96
ACDRSV10 EQU   ACDVTAM+92
ACDAPDDT EQU   ACDAPDAT+1
ACDAPDLN EQU   ACDAPDAT
ACDRSV07 EQU   ACDVTAM+82
ACDRSV06 EQU   ACDVTAM+81
ACDRSV09 EQU   ACDSSFLG
ACDAICLS EQU   ACDSSFLG
ACDABEND EQU   ACDSSFLG
ACDCACB  EQU   ACDSSFLG
ACDRSV21 EQU   ACDVTAM+64
ACDSSPAB EQU   ACDVTAM+48
ACDNEPAB EQU   ACDVTAM+32
ACDRSV20 EQU   ACDVTAM+20
ACDRARQ  EQU   ACDVTAM+16
ACDRAFQ  EQU   ACDVTAM+12
ACDFMCB  EQU   ACDVTAM+8
ACDLOCK  EQU   ACDVTAM
ACDAPP   EQU   ACDAPPAD+1
ACDEXSCL EQU   ACDAPPAD
ACDACB   EQU   ACDACBAD+1
ACDDEBID EQU   ACDACBAD
ACDPROTG EQU   ACDACBAD
ACDECBAD EQU   ACDBASIC+20
ACDUSRPB EQU   ACDUSRPG+1
ACDNMEXT EQU   ACDUSRPG
ACDPRLEN EQU   ACDBASIC+15
ACDFLGS1 EQU   ACDBASIC+14
ACDQSCNT EQU   ACDBASIC+13
ACDOPATB EQU   ACDBASIC+12
ACDIRBAD EQU   ACDBASIC+8
ACDDEB   EQU   ACDDEBAD+1
ACDAMLNG EQU   ACDDEBAD
ACDTCB   EQU   ACDTCBAD+1
ACDOPNAD EQU   ACDNMSUB
ACDRSV03 EQU   ACDTBLOF+1
ACDRSV02 EQU   ACDTBLOF
ACDAMTYP EQU   ACDDBPFX+5
ACDLENG  EQU   ACDDBPFX+4
ACDDEBX  EQU   ACDDBPFX
ACDRSV08 EQU   ACDVTPRX+12
ACDTSKID EQU   ACDVTPRX+8
ACDCHN   EQU   ACDVTPRX+4
ACDSAF   EQU   ACDVTPRX+2
ACDLNGTH EQU   ACDVTPRX+1
ACDTYPE  EQU   ACDVTPRX
TRARSV05 EQU   ISTTRAC+30
TRATIME  EQU   ISTTRAC+24
TRARSV04 EQU   TRADNODE+4
TRADAF   EQU   TRACID+2
TRASAF   EQU   TRACID
TRASEN   EQU   TRAFLG2
TRAMPX   EQU   TRAFLG2
TRADEV   EQU   TRAFLG2
TRABDU   EQU   TRAFLG2
TRAESTAT EQU   TRAFLG2
TRAPIU   EQU   TRAFLG2
TRARSV02 EQU   TRAFLG
TRAIO    EQU   TRAFLG
TRAPRTU  EQU   TRAFLG
TRACONT  EQU   TRAFLG
TRABDWR  EQU   TRABDW+2
TRABDWL  EQU   TRABDW
BDURSV04 EQU   BDUFLAGS
BDURTOE  EQU   BDUFLAGS
BDUSTPR  EQU   BDUFLAGS
BDURSV03 EQU   BDUFLAGS
BDUALT   EQU   BDUFUNC
BDUNACK  EQU   BDUFUNC
BDUACK   EQU   BDUFUNC
BDUTRPM  EQU   BDUFUNC
BDUFBMI  EQU   BDUFUNC
BDULDGPH EQU   BDUFUNC
BDUHEADP EQU   BDUFUNC
BDUSOH   EQU   BDUCHECK
BDURSV01 EQU   BDUMODIF
BDUMOD0  EQU   BDUMOD1
BDUCOMND EQU   ISTBDU
NSRUNA6  EQU   NSRUNA5
RUBDATA  EQU   ISTRUB
RHRSVD5  EQU   RH3
RHRSVD4  EQU   RH3
RHLOG    EQU   RH3
RHRSVD3  EQU   RH3
RHRCDIR  EQU   RH3
RHCDIR   EQU   RH3
RHEB     EQU   RH3
RHBB     EQU   RH3
RHPACE   EQU   RH2
RHRSVD2  EQU   RH2
RHINOP   EQU   RHRSVD1
RHBUSY   EQU   RHRETRY
RHEXCEPT EQU   RH2
RHRRN    EQU   RH2
RHPE     EQU   RH2
RHFME    EQU   RH2
RHCHAIN  EQU   RH1
RHSENSE  EQU   RH1
RHFORMAT EQU   RH1
RHDIRI   EQU   RH1
RHSCI    EQU   RH1
RHTYPE   EQU   RH1
RHQS     EQU   RH1
NCBRSV01 EQU   ISTNCB+46
NCBSWDAF EQU   ISTNCB+44
NCBDEVCH EQU   ISTNCB+36
NCBFMCBP EQU   ISTNCB+32
NCBRVPT1 EQU   ISTNCB+28
NCBNMLLN EQU   NCBFLAGS+3
@NM00079 EQU   NCBFLAG1
NCBFORCE EQU   NCBFLAG1
NCBTTF   EQU   NCBFLAG1
NCBRSYNC EQU   NCBFLAG1
NCBRSTRT EQU   NCBFLAG1
NCBCONF  EQU   NCBFLAG1
NCBSNAI  EQU   NCBFLAG1
NCBFLTIO EQU   NCBFTRIO
NCBCSTAT EQU   NCBFLAGS
NCBNCSL  EQU   ISTNCB+22
NCBNCBA  EQU   ISTNCB+20
NCBFMCBA EQU   ISTNCB+16
NCBNSNA  EQU   ISTNCB+12
NCBTSKID EQU   ISTNCB+8
NCBDAF   EQU   ISTNCB+2
NCBLNGTH EQU   ISTNCB+1
NCBTYPE  EQU   ISTNCB
FMCFLRSV EQU   FMCFLGB
FMCFLGA8 EQU   FMCFLGB
FMCCLRPH EQU   FMCVS1+45
FMCRSV0X EQU   FMCCVFLG
FMCVYRPH EQU   FMCCVFLG
FMCZV004 EQU   FMCVS1+40
FMCLOCK2 EQU   FMCVS1+36
@NM00078 EQU   FMCZFLG2
FMCZF027 EQU   FMCZFLG2
FMCZF026 EQU   FMCZFLG2
FMCSEQ1B EQU   FMCSEQ1
FMCSEQ1T EQU   FMCSEQ1
FMCZX006 EQU   FMCVS1+33
FMCZX005 EQU   FMCVS1+32
FMCZX004 EQU   FMCVS1+31
FMCZX003 EQU   FMCVS1+30
FMCZX002 EQU   FMCVS1+29
FMCZX001 EQU   FMCVS1+28
FMCZV001 EQU   FMCVS1+24
FMCAWATQ EQU   FMCLCPAQ+1
@NM00077 EQU   FMCLCPAQ
FMCLCPB2 EQU   FMCLCPBQ+1
FMCLCPB1 EQU   FMCLCPBQ
FMCASCDQ EQU   FMCSHEAQ+1
@NM00076 EQU   FMCSHEAQ
FMCSHEDQ EQU   FMCVS1+8
FMCZFCSQ EQU   FMCVS1+6
FMCZH001 EQU   FMCVS1+4
FMCZF025 EQU   FMCZFLAG+3
FMCZF024 EQU   FMCZFLAG+3
FMCZF023 EQU   FMCZFLAG+3
FMCZF022 EQU   FMCZFLAG+3
FMCZF021 EQU   FMCZFLAG+3
FMCZF020 EQU   FMCZFLAG+3
FMCZF019 EQU   FMCZFLAG+3
FMCZF018 EQU   FMCZFLAG+3
FMCZF017 EQU   FMCZFLAG+2
FMCZF016 EQU   FMCZFLAG+2
FMCZF015 EQU   FMCZFLAG+2
FMCZF014 EQU   FMCZFLAG+2
FMCZF013 EQU   FMCZFLAG+2
FMCZF012 EQU   FMCZFLAG+2
FMCZF011 EQU   FMCZFLAG+2
FMCZF010 EQU   FMCZFLAG+2
FMCZF00F EQU   FMCZFLAG+1
FMCZF00E EQU   FMCZFLAG+1
FMCZF00D EQU   FMCZFLAG+1
FMCZF00C EQU   FMCZFLAG+1
FMCZF00B EQU   FMCZFLAG+1
FMCRSV10 EQU   FMCZFLAG+1
FMCZF009 EQU   FMCZFLAG+1
FMCZFD06 EQU   FMCZF006+1
FMCZFC06 EQU   FMCZF006
FMCZFB06 EQU   FMCZF006
FMCZFA06 EQU   FMCZF006
FMCZF005 EQU   FMCZFLAG
FMCZF004 EQU   FMCZFLAG
FMCZF003 EQU   FMCZFLAG
FMCZF002 EQU   FMCZFLAG
FMCZF001 EQU   FMCZFLAG
FMCFRESA EQU   ISTFMCB+228
FMCSCSQN EQU   FMCSNCTL+2
FMCSCCTL EQU   FMCSNCTL+1
FMCSTCTL EQU   FMCSCFLG
FMCSCCLO EQU   FMCSCFLG
FMCAPRO  EQU   FMCSCFLG
FMCLURO  EQU   FMCSCFLG
FMCOBSAC EQU   FMCSCFLG
FMCIBSAC EQU   FMCSCFLG
FMCFBRE  EQU   ISTFMCB+220
FMCFBXM  EQU   FMCZV002
FMCASEGQ EQU   ISTFMCB+212
FMCRDTPT EQU   ISTFMCB+208
FMCLOCK  EQU   ISTFMCB+204
FMCSEGD  EQU   FMCSSEGQ
FMCRSPLM EQU   ISTFMCB+198
FMCPSTRS EQU   ISTFMCB+196
@NM00075 EQU   FMCRRESP
FMCTSLHA EQU   FMCRRESP
FMCFRRN  EQU   FMCRRESP
FMCFME   EQU   FMCRRESP
FMCFEX   EQU   FMCRRESP
FMCFPE   EQU   FMCRRESP
FMCCTLB  EQU   ISTFMCB+194
FMCBSQNO EQU   FMCCLSEQ+2
FMCTSQNO EQU   FMCCLSEQ
FMCOBPRG EQU   FMCMODEB
FMCIBPRG EQU   FMCMODEB
FMCRESOT EQU   FMCMODEB
FMCRSTSR EQU   FMCMODEB
FMCLUQ   EQU   FMCMODEB
FMCAPQ   EQU   FMCMODEB
FMCCLEAR EQU   FMCMODEB
FMCHOLD  EQU   FMCMODEB
FMCIPSEQ EQU   ISTFMCB+188
FMCLUSTR EQU   ISTFMCB+187
FMCRSV87 EQU   FMCSTAT2
FMCSEQI  EQU   FMCSTAT2
FMCCPIP  EQU   FMCSTAT2
FMCALIPR EQU   FMCSTAT2
FMCIAIO  EQU   FMCSTAT2
FMCZF00A EQU   FMCSTAT2
FMCVPIP  EQU   FMCSTAT2
FMCBHSET EQU   ISTFMCB+185
FMCPRMRC EQU   FMCCLEXT+20
FMCRSV89 EQU   FMCPSTAT
FMCPCLSD EQU   FMCPSTAT
FMCDDOCD EQU   FMCPSTAT
FMCIDVFD EQU   FMCPSTAT
FMCELKST EQU   FMCPSTAT
FMCRSV88 EQU   FMCPSCMD
FMCRDBLK EQU   FMCPSCMD
FMCDSEOC EQU   FMCPSCMD
FMCPURGE EQU   FMCPSCMD
FMCRESTI EQU   FMCPSCMD
FMCCNTCT EQU   FMCPSCMD
FMCINVT  EQU   FMCPSCMD
FMCFLGB7 EQU   FMCFLGCL
FMCFLGB6 EQU   FMCFLGCL
FMCFLGB5 EQU   FMCFLGCL
FMCFLGB4 EQU   FMCFLGCL
FMCFLGB3 EQU   FMCFLGCL
FMCFLGB2 EQU   FMCFLGCL
FMCFLGB1 EQU   FMCFLGCL
FMCFLGB0 EQU   FMCFLGCL
FMCFLGA7 EQU   FMCFLGA
FMCFLGA6 EQU   FMCFLGA
FMCFLGA5 EQU   FMCFLGA
FMCFLGA4 EQU   FMCFLGA
FMCFLGA3 EQU   FMCFLGA
FMCFLGA2 EQU   FMCFLGA
FMCFLGA1 EQU   FMCFLGA
FMCFLGA0 EQU   FMCFLGA
FMCDCLOK EQU   FMCCLEXT+12
FMCRAFM  EQU   FMCCLEXT+8
FMCFBPTR EQU   FMCFBA+1
FMCFBRSV EQU   FMCFBA
FMCRPLPT EQU   FMCRPLA+1
FMCRPLRS EQU   FMCRPLA
FMCPRCLS EQU   FMCCVAL
FMCIBCE  EQU   FMCCVAL
FMCONLY  EQU   FMCOBCE
FMCLAST  EQU   FMCOBCE
FMCMIDLE EQU   FMCOBCE
FMCFIRST EQU   FMCOBCE
FMCCPCNT EQU   FMCPACPC
FMCMPCNT EQU   FMCPACEM
FMCNPCNT EQU   FMCPACEN
FMCCONF  EQU   FMCSSTAT
FMCPURCH EQU   FMCSSTAT
FMCNSALU EQU   FMCSSTAT
FMCBIP   EQU   FMCSSTAT
FMCSDTR  EQU   FMCSSTAT
FMCUIPI  EQU   FMCSSTAT
FMCCIPI  EQU   FMCSSTAT
FMCSNBI  EQU   FMCSSTAT
FMCRVD01 EQU   ISTFMCB+158
FMCISSN  EQU   ISTFMCB+156
FMCOASN  EQU   FMCNXOSC
FMCOSSN  EQU   FMCNXCFM
FMCSIP   EQU   FMCSIPA
FMCEXLST EQU   ISTFMCB+144
FMCPAB5  EQU   ISTFMCB+128
FMCPAB4  EQU   ISTFMCB+112
FMCPAB3  EQU   ISTFMCB+96
FMCPAB2  EQU   ISTFMCB+80
FMCPAB1  EQU   ISTFMCB+64
FMCFASYA EQU   ISTFMCB+60
FMCMODE  EQU   ISTFMCB+52
FMCVWT   EQU   ISTFMCB+44
FMCRPLH  EQU   ISTFMCB+40
FMCBUFNB EQU   FMCCMPS1+2
FMCBUFLM EQU   FMCCMPS1
@NM00074 EQU   FMCCMDFL
FMCSNA   EQU   FMCCMDFL
FMCVSNA  EQU   FMCCMDFL
FMCCMDR  EQU   FMCCMDFL
@NM00073 EQU   ISTFMCB+30
@NM00072 EQU   FMCSTAT1
FMCHPURG EQU   FMCSTAT1
FMCVSP2  EQU   FMCSTAT1
FMCVSP1  EQU   FMCSTAT
FMCLTEAS EQU   FMCSTAT
FMCZF007 EQU   FMCSTAT
FMCDEDSC EQU   FMCSTAT
FMCDEVOF EQU   FMCSTAT
FMCDAPT  EQU   FMCSTAT
FMCPRMTE EQU   FMCSTAT
FMCPRMTR EQU   FMCSTAT
FMCUSFLD EQU   ISTFMCB+24
FMCFMCBA EQU   ISTFMCB+20
FMCDVTA  EQU   ISTFMCB+12
FMCTSKID EQU   ISTFMCB+8
FMCNXTCB EQU   ISTFMCB+4
FMCSAF   EQU   ISTFMCB+2
FMCLNGTH EQU   ISTFMCB+1
FMCTYPE  EQU   ISTFMCB
PROSRDEF EQU   PROSETM4
PRORSV47 EQU   PROSETM4
PROSCON  EQU   PROSETM4
PRORSV46 EQU   PROSETM3
PROSMONT EQU   PROSETM3
PRORSV45 EQU   PROSETM3
PROSNTO  EQU   PROSETM3
PRORSV44 EQU   PROSETM3
PRORSV43 EQU   PROSETM2
PROSNTFL EQU   PROSETM2
PRORSV42 EQU   PROSETM2
PRORSV41 EQU   PROPROCS
PROMODC  EQU   PROPROC4
PROMODT  EQU   PROPROC4
PROMODM  EQU   PROPROC4
PROMODB  EQU   PROPROC4
PRORSV14 EQU   PROPROC4
PRORSV13 EQU   PROPROC4
PRORSV12 EQU   PROPROC4
PROEIB   EQU   PROPROC4
PRORSV11 EQU   PROPROC3
PROMONT2 EQU   PROPROC3
PROMONIT EQU   PROPROC3
PROSUBLK EQU   PROPROC3
PRONTO   EQU   PROPROC3
PROLGIN  EQU   PROPROC3
PROERPI  EQU   PROPROC3
PRORSV07 EQU   PROPROC3
PROEMLC  EQU   PROPROC2
PRONTFL  EQU   PROPROC2
PRORSV05 EQU   PROPROC2
PRORSV04 EQU   PROPROC2
PROLGOT  EQU   PROPROC2
PROERPO  EQU   PROPROC2
PRORSV03 EQU   PROPROC2
PRORPLC  EQU   PROPROC1
PROCS    EQU   PROPROC1
PROCA    EQU   PROPROC1
PRORESPX EQU   PROPROC1
PRODFASY EQU   PROPROC1
PROXPOPT EQU   PROPROC1
PROTRUNC EQU   PROPROC1
PROASYIP EQU   PROPROC1
DEVRSV03 EQU   ISTDEVCH+5
DEVPHYSA EQU   ISTDEVCH+4
DEVCSLPN EQU   DEVCHAR3
DEVCSTCL EQU   DEVCHAR3
DEVCCHEK EQU   DEVCHAR3
DEVCATTN EQU   DEVCHAR3
DEVCSWL  EQU   DEVFCCTL
DEVCRVB  EQU   DEVFCCTL
DEVCSSL  EQU   DEVFCCTL
DEVCBSC  EQU   DEVFCCTL
DEVMCODE EQU   ISTDEVCH+2
DEVCHAR2 EQU   DEVTCODE
DEVRSV01 EQU   DEVCHAR
DEVCCTL  EQU   DEVCHAR
DEVNNSPT EQU   DEVCHAR
DEVSPS   EQU   DEVCHAR
DEVSUBND EQU   DEVCHAR
DEVCONVR EQU   DEVCHAR
DEVOTPUT EQU   DEVCHAR
DEVINPUT EQU   DEVCHAR
FSBFDB   EQU   ISTFSB+32
FSBRSV01 EQU   FSBFSB+30
FSBTHCNT EQU   FSBFSB+28
FSBRLRSV EQU   FSBRLSTS
FSBCFTX  EQU   FSBRLSTS
FSBICNCB EQU   FSBMSGID
FSBSSOC  EQU   FSBSS2
FSBSSCC  EQU   FSBSS2
FSBSSDC  EQU   FSBSS2
FSBSSEC  EQU   FSBSS2
FSBSSIR  EQU   FSBSS2
FSBSSCR  EQU   FSBSS2
@NM00070 EQU   FSBSS2
@NM00069 EQU   FSBSS1
FSBSSDE  EQU   FSBSS1
FSBSSUS  EQU   FSBSS1
@NM00068 EQU   FSBSS1
FSBSOHI  EQU   FSBERC
FSBLGI   EQU   FSBERC
FSBLMRI  EQU   FSBFBMI
FSBETI   EQU   FSBERC
FSBEMI   EQU   FSBERC
FSBEBI   EQU   FSBERC
FSBECI   EQU   FSBERC
FSBELI   EQU   FSBERC
FSBMJRCD EQU   FSBSRCAT
FSBXLCI  EQU   FSBXCODE
FSBXFINS EQU   FSBXCODE
FSBXNORM EQU   FSBXFSTS
FSBSCODE EQU   FSBSYSRS
FSBSPHSE EQU   FSBSYSRS
FSBSERR  EQU   FSBSYSRS
FSBCMD   EQU   FSBSEQ
FSBDTRCE EQU   FSBFLAGS
FSBASBTU EQU   FSBFLAGS
FSBINPET EQU   FSBFLAGS
FSBNONCB EQU   FSBFLAGS
FSBLBIND EQU   FSBFLAGS
FSBFFIND EQU   FSBFLAGS
FSBACIND EQU   FSBFLAGS
FSBRFIND EQU   FSBFLAGS
FSBDAF   EQU   FSBCID+2
FSBSAF   EQU   FSBCID
FSBGATE  EQU   FSBFSBA
FSBRSV02 EQU   FSBSFLGS
FSBPGBUF EQU   FSBSFLGS
FSBMNSDA EQU   FSBMNRCD
FSBMNOA  EQU   FSBMNRCD
FSBMNDIF EQU   FSBMNRCD
FSBMNEOP EQU   FSBMNRCD
FSBMNDNU EQU   FSBMNRCD
FSBMNATN EQU   FSBMNRCD
FSBMNRVI EQU   FSBMNRCD
FSBMNELI EQU   FSBMNRCD
FSBLNGTH EQU   FSBFSB+1
FSBTYPE  EQU   FSBFSB
FDBOFSET EQU   ISTFDB+10
ATCAOS   EQU   ISTATCVT+1584
ATCPOPAB EQU   ATCAOSM+640
ATCVIOPM EQU   ATCAOSM+636
ATCTPMMI EQU   ATCAOSM+632
ATCTPMPB EQU   ATCAOSM+600
ATCRSV88 EQU   ATCAOSM+596
ATCRPLCT EQU   ATCAOSM+594
ATCCRACT EQU   ATCAOSM+592
ATCSRBCT EQU   ATCAOSM+590
ATCRSV75 EQU   ATCAOSM+589
ATCRSV66 EQU   ATCZFLAG
ATCZF001 EQU   ATCZFLAG
ATCTPCP1 EQU   ATCAOSM+584
ATCTPCP0 EQU   ATCAOSM+580
ATCAICTN EQU   ATCAOSM+576
ATCZR01  EQU   ATCAOSM+572
ATCMPLTA EQU   ATCAOSM+568
ATCASCB  EQU   ATCAOSM+564
ATCAP86  EQU   ATCAOSM+560
ATCAP85  EQU   ATCAOSM+556
ATCAP84  EQU   ATCAOSM+552
ATCAP83  EQU   ATCAOSM+548
ATCAP82  EQU   ATCAOSM+544
ATCAP69  EQU   ATCAOSM+540
ATCAP66  EQU   ATCAOSM+536
ATCAP62  EQU   ATCAOSM+532
ATCAPC59 EQU   ATCAOSM+528
ATCAP56  EQU   ATCAOSM+524
ATCAP55  EQU   ATCAOSM+520
ATCRCFY0 EQU   ATCAOSM+516
ATCAP41  EQU   ATCAOSM+512
ATCAPC25 EQU   ATCAOSM+508
ATCAP23  EQU   ATCAOSM+504
ATCTPSPB EQU   ATCAOSM+472
ATCTRMPB EQU   ATCAOSM+440
ATCAPBE  EQU   ATCAOSM+436
ATCPSTA  EQU   ATCAOSM+432
ATCSMABF EQU   ATCAOSM+428
ATCSMAPR EQU   ATCAOSM+424
ATCTPWQA EQU   ATCAOSM+420
ATCSMAID EQU   ATCAOSM+416
ATCSMARA EQU   ATCAOSM+412
ATCCACXA EQU   ATCAOSM+408
ATCZLAAB EQU   ATCAOSM+404
ATCATA10 EQU   ATCAOSM+400
ATCHLTCB EQU   ATCAOSM+396
@NM00067 EQU   ATCVTFLG
ATCVTACT EQU   ATCVTFLG
ATCRPT40 EQU   ATCAOSM+388
ATCRPT39 EQU   ATCAOSM+384
ATCRPT38 EQU   ATCAOSM+380
ATCRPT37 EQU   ATCAOSM+376
ATCRPT36 EQU   ATCAOSM+372
ATCRPT35 EQU   ATCAOSM+368
ATCRPT34 EQU   ATCAOSM+364
ATCRPT33 EQU   ATCAOSM+360
ATCRPT32 EQU   ATCAOSM+356
ATCRPT31 EQU   ATCAOSM+352
ATCRPT30 EQU   ATCAOSM+348
ATCRPT29 EQU   ATCAOSM+344
ATCRPT28 EQU   ATCAOSM+340
ATCRPT27 EQU   ATCAOSM+336
ATCRPT26 EQU   ATCAOSM+332
ATCRPT25 EQU   ATCAOSM+328
ATCRPT24 EQU   ATCAOSM+324
ATCRPT23 EQU   ATCAOSM+320
ATCRPT22 EQU   ATCAOSM+316
ATCRPT21 EQU   ATCAOSM+312
ATCRPT20 EQU   ATCAOSM+308
ATCRPT19 EQU   ATCAOSM+304
ATCRPT18 EQU   ATCAOSM+300
ATCRPT17 EQU   ATCAOSM+296
ATCRPT16 EQU   ATCAOSM+292
ATCRPT15 EQU   ATCAOSM+288
ATCRPT14 EQU   ATCAOSM+284
ATCRPT13 EQU   ATCAOSM+280
ATCRPT12 EQU   ATCAOSM+276
ATCDMAPP EQU   ATCAOSM+272
ATCAP38  EQU   ATCAOSM+268
ATCAP37  EQU   ATCAOSM+264
ATCZDFJ0 EQU   ATCAOSM+260
ATCAP51  EQU   ATCAOSM+256
ATCZPAB2 EQU   ATCAOSM+224
ATCZPAB1 EQU   ATCAOSM+192
ATCZLFVT EQU   ATCAOSM+188
ATCZLPVT EQU   ATCAOSM+184
ATCZDVTX EQU   ATCAOSM+180
ATCZDVTA EQU   ATCAOSM+176
ATCZDVT9 EQU   ATCAOSM+172
ATCZDVT8 EQU   ATCAOSM+168
ATCZDVT7 EQU   ATCAOSM+164
ATCRSV56 EQU   ATCDAPVT+22
ATCDAPDG EQU   ATCDAPVT+20
ATCDXCEA EQU   ATCDAPVT+16
ATCDCEA  EQU   ATCDAPVT+12
ATCDPCIA EQU   ATCDAPVT+8
ATCDSIOA EQU   ATCDAPVT+4
ATCDEOEA EQU   ATCDAPVT
ATCZLAEA EQU   ATCLDAPP+16
ATCZLCEA EQU   ATCLDAPP+12
ATCRSV17 EQU   ATCLDAPP+8
ATCSMFRR EQU   ATCLDAPP+4
ATCCDTID EQU   ATCLDAPP
ATCZAEA  EQU   ATCICAPP+16
ATCZCEA  EQU   ATCICAPP+12
ATCCDTCB EQU   ATCICAPP+8
ATCZSIOA EQU   ATCICAPP+4
ATCCDRPH EQU   ATCICAPP
ATCAP65  EQU   ATCAOSM+96
ATCZDVT6 EQU   ATCAOSM+92
ATCZDVT4 EQU   ATCAOSM+88
ATCZDVT3 EQU   ATCAOSM+84
ATCZDVT1 EQU   ATCAOSM+80
ATCICNCB EQU   ATCAOSM+76
ATCZPSVT EQU   ATCAOSM+72
ATCZFSVT EQU   ATCAOSM+68
ATCAP64  EQU   ATCAOSM+64
ATCAP63  EQU   ATCAOSM+60
ATCAP61  EQU   ATCAOSM+56
ATCAP58  EQU   ATCAOSM+52
ATCAP57  EQU   ATCAOSM+48
ATCAP54  EQU   ATCAOSM+44
ATCAP53  EQU   ATCAOSM+40
ATCAP52  EQU   ATCAOSM+36
ATCAP34  EQU   ATCAOSM+32
ATCAP32  EQU   ATCAOSM+28
ATCAP31  EQU   ATCAOSM+24
ATCAP12  EQU   ATCAOSM+20
ATCAP11  EQU   ATCAOSM+16
ATCCMTCB EQU   ATCAOSM+12
ATCPRPH  EQU   ATCAOSM+8
ATCGSRBF EQU   ATCGSRBG
ATCSRBRT EQU   ATCAOSM
@NM00066 EQU   @NM00065
ATCVTMAB EQU   @NM00065
@NM00064 EQU   ISTATCVT+908
ATCVDLOK EQU   ISTATCVT+904
ATCVFPAB EQU   ISTATCVT+872
ATCVDPAB EQU   ISTATCVT+840
ATCIOECB EQU   ISTATCVT+836
ATCINCW3 EQU   ISTATCVT+832
ATCMODTB EQU   ISTATCVT+828
ATCUSSPT EQU   ISTATCVT+824
ATCSRTAB EQU   ISTATCVT+820
@NM00063 EQU   ISTATCVT+819
@NM00062 EQU   ATCSTAT2
ATCTRSMS EQU   ATCSTAT2
ATCTRLOK EQU   ATCSTAT2
ATCTRPSS EQU   ATCSTAT2
ATCTRESC EQU   ATCSTAT2
ATCTRAPI EQU   ATCSTAT2
ATCNMCTR EQU   ISTATCVT+816
ATCSEC51 EQU   ATCSECST
ATCSEC40 EQU   ATCSIPD+8
ATCSEC30 EQU   ATCSIPD+4
ATCSEC21 EQU   ATCSIPD
ATCSEC10 EQU   ATCSOPD+4
ATCSEC01 EQU   ATCSOPD
ATCPOIA  EQU   ATCRNO+1
@NM00061 EQU   ATCRNO
ATCPODVT EQU   ISTATCVT+784
ATCS49XL EQU   ISTATCVT+780
ATCS49XI EQU   ISTATCVT+776
ATCLCLCT EQU   ATCVLCNT+2
ATCRNCNT EQU   ATCVLCNT
ATCCFEAD EQU   ISTATCVT+768
ATCPTR00 EQU   ISTATCVT+736
ATCRSV10 EQU   ISTATCVT+704
ATCTACB  EQU   ISTATCVT+700
ATCVTINM EQU   ISTATCVT+696
ATCTCLIM EQU   ISTATCVT+692
ATCTCLOM EQU   ISTATCVT+688
ATCVTIPB EQU   ISTATCVT+656
ATCTCIPB EQU   ISTATCVT+624
ATCTIPAB EQU   ISTATCVT+592
ATCOCCCB EQU   ISTATCVT+588
ATCOCCOB EQU   ISTATCVT+584
ATCCDLAD EQU   ISTATCVT+580
ATCCCLAD EQU   ISTATCVT+576
ATCDLRPB EQU   ISTATCVT+544
ATCPGPAB EQU   ISTATCVT+512
ATCVPAB1 EQU   ISTATCVT+480
ATCRSV97 EQU   ISTATCVT+476
ATCRDTLK EQU   ISTATCVT+472
ATCVOCLK EQU   ISTATCVT+468
ATCDVTLK EQU   ISTATCVT+456
ATCAP36  EQU   ISTATCVT+452
ATCAP35  EQU   ISTATCVT+448
ATCAP33  EQU   ISTATCVT+444
ATCRSV99 EQU   ISTATCVT+440
ATCONFIG EQU   ISTATCVT+432
ATCNODDS EQU   ISTATCVT+428
ATCNODEL EQU   ISTATCVT+420
ATCRSV69 EQU   ISTATCVT+416
ATCAPCR5 EQU   ISTATCVT+412
ATCLDECB EQU   ISTATCVT+408
ATCLDNCS EQU   ISTATCVT+404
ATCCDPTR EQU   ISTATCVT+400
ATCADEL  EQU   ISTATCVT+396
ATCTODVT EQU   ISTATCVT+392
ATCDEBCH EQU   ISTATCVT+388
ATCHBSIZ EQU   ATCHPGM+2
ATCHBFNO EQU   ATCHPGM
ATCCRME  EQU   ISTATCVT+380
ATCRVCHS EQU   ISTATCVT+372
ATCECTLP EQU   ISTATCVT+368
ATCZDVTB EQU   ISTATCVT+364
ATCECPRT EQU   ISTATCVT+360
ATCVTLOD EQU   ISTATCVT+356
ATCTRPAB EQU   ISTATCVT+352
ATCMSGM  EQU   ISTATCVT+348
ATCMSGP  EQU   ISTATCVT+344
ATCEPT   EQU   ISTATCVT+340
ATCDVT   EQU   ISTATCVT+336
ATCTRCPT EQU   ISTATCVT+332
ATCDVLOD EQU   ISTATCVT+328
ATCALERT EQU   ISTATCVT+324
ATCEPA   EQU   ISTATCVT+320
ATCSAF   EQU   ISTATCVT+318
ATCXRANG EQU   ISTATCVT+317
ATCLAST  EQU   ATCSHRFG
ATCSHRTN EQU   ISTATCVT+308
ATCUEP   EQU   ISTATCVT+304
ATCOCRT  EQU   ISTATCVT+300
ATCCSMA  EQU   ATCCSMR
ATCESC02 EQU   ISTATCVT+292
ATCESC01 EQU   ISTATCVT+288
ATCMNT   EQU   ISTATCVT+272
ATCRSV16 EQU   ISTATCVT+270
ATCMAXID EQU   ISTATCVT+269
ATCMSGSP EQU   ATCCOM+268
ATCACTRM EQU   ATCCOM+264
ATCNERAP EQU   ATCCOM+260
ATCNESAL EQU   ATCCOM+256
ATCRCC65 EQU   ATCCOM+252
ATCRCC26 EQU   ATCCOM+248
ATCRCC63 EQU   ATCCOM+244
ATCRSV11 EQU   ATCCOM+242
ATCVTMID EQU   ATCCOM+240
ATCAPOST EQU   ATCCOM+236
ATCNERFN EQU   ATCCOM+232
ATCNERNE EQU   ATCCOM+228
ATCNERCV EQU   ATCCOM+224
ATCNERST EQU   ATCCOM+220
ATCDCC61 EQU   ATCCOM+216
ATCDCC60 EQU   ATCCOM+212
ATCSRTDF EQU   ATCCOM+208
ATCSRTAD EQU   ATCCOM+204
ATCFIRA  EQU   ATCCOM+200
ATCSMBQ  EQU   ATCCOM+196
ATCSMRC  EQU   ATCCOM+192
ATCSMQU  EQU   ATCCOM+184
ATCACDA  EQU   ATCCOM+176
ATCSTMA  EQU   ATCCOM+172
ATCFRSTR EQU   ATCCOM+168
ATCGTSTR EQU   ATCCOM+164
ATCREADA EQU   ATCCOM+160
ATCALOAD EQU   ATCCOM+156
ATCBLDLA EQU   ATCCOM+152
ATCFBRPH EQU   ATCCOM+148
ATCBFTRC EQU   ATCCOM+144
ATCTPBUF EQU   ATCTHTRC
ATCREMOV EQU   ATCCOM+136
ATCADD   EQU   ATCCOM+132
ATCDVTPT EQU   ATCCOM+124
ATCFEPT  EQU   ATCCOM+120
ATCFDVT  EQU   ATCCOM+116
ATCCONFT EQU   ATCCOM+112
ATCPTCHA EQU   ATCCOM+108
ATCAUTHA EQU   ATCCOM+104
ATCACCTA EQU   ATCCOM+100
ATCRDTH  EQU   ATCCOM+96
ATCFRSCH EQU   ATCCOM+92
ATCFSB   EQU   ATCCOM+88
ATCHLTMS EQU   ATCCOM+84
ATCECPRM EQU   ATCCOM+80
ATCECNET EQU   ATCCOM+76
ATCECOPC EQU   ATCCOM+72
ATCVPARM EQU   ATCCOM+68
ATCECVRQ EQU   ATCCOM+64
ATCECSES EQU   ATCCOM+60
ATCOCHA  EQU   ATCCOM+56
ATCDSPLQ EQU   ATCCOM+52
ATCECDSP EQU   ATCCOM+48
ATCVARYQ EQU   ATCCOM+44
ATCECVRY EQU   ATCCOM+40
ATCHALTQ EQU   ATCCOM+36
ATCECHLT EQU   ATCCOM+32
ATCMODQ  EQU   ATCCOM+28
ATCECMOD EQU   ATCCOM+24
ATCASCHD EQU   ATCCOM+20
ATCSRT   EQU   ATCCOM+16
ATCRDT   EQU   ATCCOM+12
ATCDCFRR EQU   ATCCOM+8
ATCAPI   EQU   ATCCOM+4
ATCRSV04 EQU   ATCSTAT4
ATCVTTRC EQU   ATCSTAT4
ATCNCSD  EQU   ATCSTAT4
ATCMTST  EQU   ATCSTAT4
ATCTPRT  EQU   ATCSTAT4
ATCPRTAT EQU   ATCSTAT4
@NM00060 EQU   ATCPOFLG
ATCPOUNS EQU   ATCPOFLG
ATCHSDMF EQU   ATCSTFLG
@NM00059 EQU   ATCSTAT1
ATCMDHLT EQU   ATCSTAT1
ATCINHLT EQU   ATCSTAT1
ATCNETSL EQU   ATCSTAT1
ATCQKHLT EQU   ATCSTAT1
ATCSNHLT EQU   ATCSTAT1
ATCACTIV EQU   ATCSTAT1
ATCSTART EQU   ATCSTAT1
RPHNEXPO EQU   ISTRPH+104
RPHSAV16 EQU   RPHWORK+60
RPHSAV15 EQU   RPHWORK+56
RPHSAV14 EQU   RPHWORK+52
RPHSAV13 EQU   RPHWORK+48
RPHSAV12 EQU   RPHWORK+44
RPHSAV11 EQU   RPHWORK+40
RPHSAV10 EQU   RPHWORK+36
RPHSAVE9 EQU   RPHWORK+32
RPHSAVE8 EQU   RPHWORK+28
RPHSAVE7 EQU   RPHWORK+24
RPHSAVE6 EQU   RPHWORK+20
RPHSAVE5 EQU   RPHWORK+16
RPHSAVE4 EQU   RPHWORK+12
RPHSAVE3 EQU   RPHWORK+8
RPHSAVE2 EQU   RPHWORK+4
RPHSHALF EQU   RPHSAVE1+2
RPHSBIT8 EQU   RPHSBITS
RPHSBIT7 EQU   RPHSBITS
RPHSBIT6 EQU   RPHSBITS
RPHSBIT5 EQU   RPHSBITS
RPHSBIT4 EQU   RPHSBITS
RPHSBIT3 EQU   RPHSBITS
RPHSBIT2 EQU   RPHSBITS
RPHSBIT1 EQU   RPHSBITS
RPHSBYTE EQU   RPHSAVE1
RPHPABQP EQU   RPHPABQA+1
RPHPABFG EQU   RPHPABQA
RPHCRR   EQU   ISTRPH+32
RPHSRP34 EQU   RPHSRPRM+2
RPHSRP12 EQU   RPHSRPRM
RPHWEGT  EQU   RPHCSPA
RPHMAJCB EQU   ISTRPH+21
RPHPABOF EQU   ISTRPH+20
RPHRESUM EQU   RPHRESMA+1
RPHRSV01 EQU   RPHWPFLG
RPHPURGE EQU   RPHWPFLG
RPHPT    EQU   RPHWPFLG
RPHWT    EQU   RPHWPFLG
RPHRSKEY EQU   RPHWPFLG
RPHDVTA  EQU   ISTRPH+12
RPHTIK   EQU   RPHTSKID+3
@NM00058 EQU   RPHTSKID
RPHRPHAP EQU   RPHRPHA+1
@NM00057 EQU   RPHRPHA
RPHGATE  EQU   RPHRPHA
@NM00056 EQU   RPHFLGB
RPHRLCRA EQU   RPHFLGB
RPHNRSAV EQU   RPHFLGB
RPHPGCMP EQU   RPHFLGB
RPHFBAPS EQU   RPHFLGB
RPHBSSP  EQU   RPHFLGB
RPHLOCK  EQU   RPHFLGB
RPHFNFLG EQU   RPHFSTLC
RPHMLTCP EQU   RPHFLAGS
RPHSMCLR EQU   RPHFLAGS
RPHSMTYP EQU   RPHFLAGS
RPHSMQ   EQU   RPHFLAGS
RPHAUTEX EQU   RPHAPTYP
RPHSPGIN EQU   RPHFLAGS
RPHOGIND EQU   RPHFLAGS
RPHLNGTH EQU   RPHCNTL+1
RPHTYPE  EQU   RPHCNTL
DYPPAB   EQU   ISTDYPAB+16
DYPRSV01 EQU   ISTDYPAB+12
DYPTSKID EQU   ISTDYPAB+8
DYPCHAIN EQU   ISTDYPAB+4
DYPFLGS  EQU   ISTDYPAB+2
DYPLNGTH EQU   ISTDYPAB+1
DYPTYPE  EQU   ISTDYPAB
LOKXCL   EQU   @NM00054
LOKPRY   EQU   @NM00054
@NM00055 EQU   @NM00054
@NM00053 EQU   LOKCHNG
LOKCT    EQU   ISTLOK
PABRPHA  EQU   PABRPHFG+1
PABNODQ  EQU   PABFLAGS
PABINHBT EQU   PABFLAGS
PABERLOK EQU   PABERRLK
PABSSN   EQU   PABFLAGS
PABDYNAM EQU   PABFLAGS
PABNORPR EQU   PABFLAGS
PABPRIOR EQU   PABFLAGS
PABAPYP  EQU   PABFLAGS
PABDVTA  EQU   ISTPAB+9
PABOFFST EQU   ISTPAB+8
PABCHNGP EQU   PABCHAIN+1
PABRSV02 EQU   PABCHAIN
PABRUNNG EQU   PABCHAIN
PABRESET EQU   PABCHAIN
PABUNCON EQU   PABCHAIN
PABCDP   EQU   PABCHAIN
PABRESCH EQU   PABCHAIN
PABCHNG  EQU   PABCHAIN
PABWEQP  EQU   PABWEQA+1
PABRSV01 EQU   PABWEQA
PABWEQG  EQU   PABWEQA
CVTLEVL  EQU   CVTRELNO+2
CVTNUMB  EQU   CVTRELNO
CVTMDL   EQU   CVTFIX+250
@NM00028 EQU   CVTFIX+248
@NM00027 EQU   CVTFIX
CVTRV482 EQU   CVTXTNT2+128
CVTRV481 EQU   CVTXTNT2+124
CVTRV480 EQU   CVTXTNT2+120
CVTRV479 EQU   CVTXTNT2+118
CVTRV478 EQU   CVTXTNT2+117
CVTRV477 EQU   CVTXTNT2+116
CVTRV476 EQU   CVTXTNT2+115
CVTRV475 EQU   CVTXTNT2+114
CVTRV474 EQU   CVTRV466
CVTRV473 EQU   CVTRV466
CVTRV472 EQU   CVTRV466
CVTRV471 EQU   CVTRV466
CVTRV470 EQU   CVTRV466
CVTRV469 EQU   CVTRV466
CVTRV468 EQU   CVTRV466
CVTRV467 EQU   CVTRV466
CVTRV465 EQU   CVTRV457
CVTRV464 EQU   CVTRV457
CVTRV463 EQU   CVTRV457
CVTRV462 EQU   CVTRV457
CVTRV461 EQU   CVTRV457
CVTRV460 EQU   CVTRV457
CVTRV459 EQU   CVTRV457
CVTRV458 EQU   CVTRV457
CVTRV456 EQU   CVTXTNT2+108
CVTRV455 EQU   CVTXTNT2+104
CVTRV454 EQU   CVTXTNT2+100
CVTRV453 EQU   CVTXTNT2+96
CVTRV452 EQU   CVTXTNT2+94
CVTRV451 EQU   CVTXTNT2+92
CVTRV450 EQU   CVTXTNT2+90
CVTRV449 EQU   CVTXTNT2+88
CVTRV448 EQU   CVTXTNT2+87
CVTRV447 EQU   CVTXTNT2+86
CVTRV446 EQU   CVTRV438
CVTRV445 EQU   CVTRV438
CVTRV444 EQU   CVTRV438
CVTRV443 EQU   CVTRV438
CVTRV442 EQU   CVTRV438
CVTRV441 EQU   CVTRV438
CVTRV440 EQU   CVTRV438
CVTRV439 EQU   CVTRV438
CVTRV437 EQU   CVTRV429
CVTRV436 EQU   CVTRV429
CVTRV435 EQU   CVTRV429
CVTRV434 EQU   CVTRV429
CVTRV433 EQU   CVTRV429
CVTRV432 EQU   CVTRV429
CVTRV431 EQU   CVTRV429
CVTRV430 EQU   CVTRV429
CVTRV428 EQU   CVTXTNT2+80
CVTRV427 EQU   CVTXTNT2+76
CVTRV426 EQU   CVTXTNT2+72
CVTRV425 EQU   CVTXTNT2+68
CVTATACT EQU   CVTATCVT
CVTRV423 EQU   CVTXTNT2+62
CVTRV422 EQU   CVTXTNT2+60
CVTRV421 EQU   CVTXTNT2+58
CVTRV420 EQU   CVTXTNT2+56
CVTRV419 EQU   CVTXTNT2+55
CVTRV418 EQU   CVTXTNT2+54
CVTRV417 EQU   CVTRV409
CVTRV416 EQU   CVTRV409
CVTRV415 EQU   CVTRV409
CVTRV414 EQU   CVTRV409
CVTRV413 EQU   CVTRV409
CVTRV412 EQU   CVTRV409
CVTRV411 EQU   CVTRV409
CVTRV410 EQU   CVTRV409
CVTRV408 EQU   CVTRV400
CVTRV407 EQU   CVTRV400
CVTRV406 EQU   CVTRV400
CVTRV405 EQU   CVTRV400
CVTRV404 EQU   CVTRV400
CVTRV403 EQU   CVTRV400
CVTRV402 EQU   CVTRV400
CVTRV401 EQU   CVTRV400
CVTICB   EQU   CVTXTNT2+48
CVTSKTA  EQU   CVTXTNT2+44
CVTSHSB  EQU   CVTSHS
CVTRSV98 EQU   CVTXTNT2+36
CVTRSV97 EQU   CVTXTNT2+34
CVTRSV96 EQU   CVTXTNT2+32
CVTOLTEP EQU   CVTXTNT2+28
CVTQIDA  EQU   CVTQID+1
CVTRSV95 EQU   CVTQID
CVTRSV94 EQU   CVTXTNT2+20
CVTRSV93 EQU   CVTXTNT2+16
CVTRSV92 EQU   CVTXTNT2+12
CVTDEBVR EQU   CVTXTNT2+8
CVTRSV91 EQU   CVTXTNT2+6
CVTRSV9H EQU   CVTFLGBT
CVTRSV9G EQU   CVTFLGBT
CVTRSV9F EQU   CVTFLGBT
CVTRSV9E EQU   CVTFLGBT
CVTRSV9D EQU   CVTFLGBT
CVTRSV9C EQU   CVTFLGBT
CVTVME   EQU   CVTFLGBT
CVTNPE   EQU   CVTFLGBT
CVTNUCLS EQU   CVTXTNT2+4
CVTDSSVA EQU   CVTDSSV+1
CVTRSV89 EQU   CVTDSSV
CVTRSV88 EQU   CVTXTNT1+8
CVTRSV87 EQU   CVTXTNT1+4
CVTFACHN EQU   CVTXTNT1
CVTRV488 EQU   CVTMAP+412
CVTRV487 EQU   CVTMAP+408
CVTRV486 EQU   CVTMAP+404
CVTRV485 EQU   CVTMAP+400
CVTACTAP EQU   CVTMAP+396
CVTAUTH  EQU   CVTMAP+392
CVTATMCA EQU   CVTATMCT+1
CVTATMST EQU   CVTATMCT
CVTRSV61 EQU   CVTMAP+384
CVTVOLT1 EQU   CVTVOLM1+1
CVTVOLI1 EQU   CVTVOLF1
CVTSTOA  EQU   CVTMAP+376
CVTRSV58 EQU   CVTMAP+374
CVTRSV57 EQU   CVTMAP+372
CVTDDCE  EQU   CVTMAP+368
CVTPNWFR EQU   CVTMAP+364
CVTSMF   EQU   CVTMAP+360
CVTSULK  EQU   CVTMAP+358
CVTSLKO  EQU   CVTSYSK
CVTSLKP  EQU   CVTSYSK
CVTSLKQ  EQU   CVTSYSK
CVTSLKR  EQU   CVTSYSK
CVTRSV56 EQU   CVTSYSK
CVTRSV55 EQU   CVTSYSK
CVTRSV54 EQU   CVTSYSK
CVTRSV53 EQU   CVTSYSK
CVTRSV52 EQU   CVTA1F1
CVTRSV51 EQU   CVTA1F1
CVTRSV50 EQU   CVTA1F1
CVTRSV49 EQU   CVTA1F1
CVTRSV48 EQU   CVTA1F1
CVTRSV47 EQU   CVTA1F1
CVTSRSW  EQU   CVTA1F1
CVTPFSW  EQU   CVTA1F1
CVTPCVT  EQU   CVTMAP+352
CVTRSV46 EQU   CVTMAP+344
CVTRSV45 EQU   CVTMAP+340
CVTRSV44 EQU   CVTMAP+338
CVTRSV43 EQU   CVTMAP+336
CVTHJESA EQU   CVTHJES+1
CVTRSV42 EQU   CVTHJES
CVTEXT2A EQU   CVTEXT2+1
CVTRSV41 EQU   CVTEXT2
CVTAPFA  EQU   CVTAPF+1
CVTRSV40 EQU   CVTAPF
CVTRV518 EQU   CVTINTLA
CVTRV517 EQU   CVTERPV
CVTEORM  EQU   CVTMAP+312
CVTMCHPR EQU   CVTMAP+308
CVTTZ    EQU   CVTMAP+304
CVTJEPS  EQU   CVTMAP+300
CVTJESCT EQU   CVTMAP+296
CVTMODE  EQU   CVTMAP+292
CVTPTRV  EQU   CVTMAP+288
CVTREAL  EQU   CVTMAP+284
CVTRSV39 EQU   CVTMAP+280
CVTRSV38 EQU   CVTMAP+276
CVTDMSRA EQU   CVTDMSR+1
CVTRSV37 EQU   CVTDMSR
CVTQMSGA EQU   CVTQMSG+1
CVTRSV36 EQU   CVTQMSG
CVTAMFF  EQU   CVTMAP+264
CVTPURGA EQU   CVTPURG+1
CVTRSV35 EQU   CVTPURG
CVTCBSP  EQU   CVTMAP+256
CVTATERA EQU   CVTATER+1
CVTSYST  EQU   CVTATER
CVTVOLT2 EQU   CVTTAT
CVTVOLI2 EQU   CVTVOLF2
CVTAQAVB EQU   CVTAQAVT+1
CVTRSV34 EQU   CVTTCMFG
CVTRSV33 EQU   CVTTCMFG
CVTRSV32 EQU   CVTTCMFG
CVTRSV31 EQU   CVTTCMFG
CVTRSV30 EQU   CVTTCMFG
CVTRSV29 EQU   CVTTCMFG
CVTRSV28 EQU   CVTTCMFG
CVTTCRDY EQU   CVTTCMFG
CVTGTFA  EQU   CVTGTF+1
CVTRSV27 EQU   CVTGTFST
CVTRNIO  EQU   CVTGTFST
CVTUSR   EQU   CVTGTFST
CVTRV318 EQU   CVTFORM
CVTRV317 EQU   CVTTMODE
CVTRV316 EQU   CVTSTATE
CVTRV315 EQU   CVTGTFS
CVT0SCR1 EQU   CVTMAP+232
CVTRV515 EQU   CVTMAP+228
CVTRMS   EQU   CVTMAP+224
CVTPATCH EQU   CVTMAP+220
CVTTSCE  EQU   CVTMAP+216
CVTLNKSC EQU   CVTMAP+214
CVTQABST EQU   CVTMAP+212
CVTMDLDS EQU   CVTMAP+208
CVTUSER  EQU   CVTMAP+204
CVTABEND EQU   CVTMAP+200
CVTSMCA  EQU   CVTMAP+196
CVTRSV18 EQU   CVTMAP+192
CVTQLPAQ EQU   CVTMAP+188
CVTQCDSR EQU   CVTMAP+184
CVTRSV17 EQU   CVTOPTB
CVTRSV16 EQU   CVTOPTB
CVTFP    EQU   CVTOPTB
CVTAPTHR EQU   CVTOPTB
CVTNLOG  EQU   CVTOPTB
CVTTOD   EQU   CVTOPTB
CVTCTIMS EQU   CVTOPTB
CVTPROT  EQU   CVTOPTB
CVTXPFP  EQU   CVTOPTA
CVTASCII EQU   CVTOPTA
CVTRSV13 EQU   CVTOPTA
CVTRSV12 EQU   CVTOPTA
CVTNIP   EQU   CVTOPTA
CVTDDR   EQU   CVTOPTA
CVTAPR   EQU   CVTOPTA
CVTCCH   EQU   CVTOPTA
CVTSNCTR EQU   CVTMAP+180
CVTQMWR  EQU   CVTMAP+176
CVTQOCR  EQU   CVTMAP+172
CVT1EF00 EQU   CVTMAP+168
CVTMZ00  EQU   CVTMAP+164
CVTHEAD  EQU   CVTMAP+160
CVTRSV11 EQU   CVTMAP+156
CVT0PT01 EQU   CVTMAP+152
CVTMSER  EQU   CVTMAP+148
CVTRV516 EQU   CVTIERLC
CVTILCH  EQU   CVTMAP+140
CVT0DS   EQU   CVTMAP+136
CVTFBOSV EQU   CVTMAP+132
CVTNUCB  EQU   CVTMAP+128
CVTIXAVL EQU   CVTMAP+124
CVTIOQET EQU   CVTMAP+120
CVTDCBA  EQU   CVTMAP+117
CVTMVS2  EQU   CVTDCB
CVT6DAT  EQU   CVTDCB
CVT4MPS  EQU   CVTDCB
CVTRSV09 EQU   CVTDCB
CVT4MS1  EQU   CVTDCB
CVT2SPS  EQU   CVTDCB
CVT1SSS  EQU   CVTDCB
CVTRSV08 EQU   CVTDCB
CVTSTB   EQU   CVTMAP+112
CVTQTD00 EQU   CVTMAP+108
CVTQTE00 EQU   CVTMAP+104
CVTCUCB  EQU   CVTMAP+100
CVTSJQ   EQU   CVTMAP+96
CVTPBLDL EQU   CVTMAP+92
CVTTPC   EQU   CVTMAP+88
CVTSVDCB EQU   CVTMAP+84
CVTBRET  EQU   CVTMAP+82
CVTEXIT  EQU   CVTMAP+80
CVT0FN00 EQU   CVTMAP+76
CVTDARA  EQU   CVTDAR+1
CVTRSV07 EQU   CVTFLGS1
CVTRSV06 EQU   CVTFLGS1
CVTRSV05 EQU   CVTFLGS1
CVTRSV04 EQU   CVTFLGS1
CVTRSV03 EQU   CVTFLGS1
CVTRSV02 EQU   CVTFLGS1
CVTRSV01 EQU   CVTFLGS1
CVTDMPLK EQU   CVTFLGS1
CVTXITP  EQU   CVTMAP+68
CVTZDTAB EQU   CVTMAP+64
CVTMSLT  EQU   CVTMAP+60
CVTDATE  EQU   CVTMAP+56
CVTBTERM EQU   CVTMAP+52
CVTSYSAD EQU   CVTMAP+48
CVTXTLER EQU   CVTMAP+44
CVTILK2  EQU   CVTMAP+40
CVTILK1  EQU   CVTMAP+36
CVTPRLTV EQU   CVTMAP+32
CVTPCNVT EQU   CVTMAP+28
CVTXAPG  EQU   CVTMAP+20
CVTBUF   EQU   CVTMAP+16
CVTJOB   EQU   CVTMAP+12
CVTLINK  EQU   CVTMAP+8
CVT0EF00 EQU   CVTMAP+4
CVTTCBP  EQU   CVTMAP
CVT      EQU   CVTMAP
RPLSEQNO EQU   RPLOF60+6
@NM00026 EQU   RPLIBSQ
RPLIINV  EQU   RPLIBSQ
RPLINEG  EQU   RPLIBSQ
RPLIPOS  EQU   RPLIBSQ
RPLIIGN  EQU   RPLIBSQ
RPLIRSET EQU   RPLIBSQ
RPLITST  EQU   RPLIBSQ
RPLISET  EQU   RPLIBSQ
@NM00025 EQU   RPLOBSQ
RPLOINV  EQU   RPLOBSQ
RPLONEG  EQU   RPLOBSQ
RPLOPOS  EQU   RPLOBSQ
RPLOIGN  EQU   RPLOBSQ
RPLORSET EQU   RPLOBSQ
RPLOTST  EQU   RPLOBSQ
RPLOSET  EQU   RPLOBSQ
RPLIBSQV EQU   RPLOF60+2
RPLOBSQV EQU   RPLOF60
@NM00024 EQU   RPLCNTSC
RPLRSHUT EQU   RPLCNTSC
RPLRQR   EQU   RPLCNTSC
RPLSHUTC EQU   RPLCNTSC
RPLSHUTD EQU   RPLCNTSC
RPLSTSN  EQU   RPLCNTSC
RPLCLEAR EQU   RPLCNTSC
RPLSDT   EQU   RPLCNTSC
@NM00023 EQU   RPLCNTDC
RPLSIGNL EQU   RPLCNTDC
RPLLUS   EQU   RPLCNTDC
RPLRTR   EQU   RPLCNTDC
RPLBID   EQU   RPLCNTDC
@NM00022 EQU   RPLCNTDF
RPLQI    EQU   RPLCNTDF
RPLRELQ  EQU   RPLCNTDF
RPLCHASE EQU   RPLCNTDF
RPLQEC   EQU   RPLCNTDF
RPLQC    EQU   RPLCNTDF
RPLCNCEL EQU   RPLCNTDF
RPLRRN   EQU   RPLRESP
RPLNFME  EQU   RPLRESP
RPLEX    EQU   RPLRESP
@NM00021 EQU   RPLRESP
@NM00020 EQU   RPLPOSTV
RPLSCHED EQU   RPLPOSTV
@NM00019 EQU   RPLVTFL1
RPLAUTUS EQU   RPLVTFL1
RPLVTUSE EQU   RPLVTFL1
@NM00018 EQU   RPLVTFL1
@NM00017 EQU   RPLCHN
RPLONLY  EQU   RPLCHN
RPLLAST  EQU   RPLCHN
RPLMIDLE EQU   RPLCHN
RPLFIRST EQU   RPLCHN
@NM00016 EQU   RPLRTYPE
RPLDFASY EQU   RPLRTYPE
RPLNFSYN EQU   RPLRTYPE
RPLRRESP EQU   RPLRTYPE
@NM00015 EQU   RPLSTYPE
RPLSRESP EQU   RPLSTYPE
@NM00014 EQU   RPLRH3
RPLCSI   EQU   RPLRH3
RPLCHREQ EQU   RPLRH3
RPLCMD   EQU   RPLRH3
RPLEB    EQU   RPLRH3
RPLBB    EQU   RPLRH3
RPLSIGDA EQU   RPLVTEXT+32
RPLSAV13 EQU   RPLVTEXT+28
RPLUSNSO EQU   RPLOSENS+2
RPLSSMO  EQU   RPLSSNSO+1
@NM00013 EQU   RPLSSEO
RPLRRO   EQU   RPLSSEO
RPLFIO   EQU   RPLSSEO
RPLSTATO EQU   RPLSSEO
RPLCPMO  EQU   RPLSSEO
@NM00012 EQU   RPLSSEO
RPLFMHDR EQU   RPLOPT12
@NM00011 EQU   RPLOPT12
@NM00010 EQU   RPLOPT12
@NM00009 EQU   RPLOPT12
RPLNIBTK EQU   RPLOPT12
RPLTRUNC EQU   RPLOPT12
RPLKEEP  EQU   RPLOPT12
@NM00008 EQU   RPLOPT12
RPLRSVE8 EQU   RPLOPT11
RPLRSVE7 EQU   RPLOPT11
RPLRSVE6 EQU   RPLOPT11
RPLRSVE5 EQU   RPLOPT11
RPLRSVE4 EQU   RPLOPT11
RPLSTOP  EQU   RPLOPT11
RPLSTART EQU   RPLOPT11
RPLQUIES EQU   RPLOPT11
RPLRSVI6 EQU   RPLOPT10
RPLRSVI5 EQU   RPLOPT10
RPLRSVI4 EQU   RPLOPT10
RPLRSVI3 EQU   RPLOPT10
RPLRSVI2 EQU   RPLOPT10
RPLSPARM EQU   RPLOPT10
RPLDSPLY EQU   RPLOPT10
RPLBSCID EQU   RPLOPT10
RPLTOPL  EQU   RPLOPT9
RPLCIDE  EQU   RPLOPT9
RPLRNNM  EQU   RPLOPT9
RPLAPPST EQU   RPLOPT9
RPLCOUNT EQU   RPLOPT9
RPLTERMS EQU   RPLOPT9
RPLDEVCH EQU   RPLOPT9
RPLLOGON EQU   RPLOPT9
RPLUSFLD EQU   RPLVTEXT+16
RPLESR2  EQU   RPLUSNSI+1
RPLESR1  EQU   RPLUSNSI
RPLSSMI  EQU   RPLDSB2
@NM00007 EQU   RPLSSEI
RPLRRI   EQU   RPLSSEI
RPLFII   EQU   RPLSSEI
RPLSTATI EQU   RPLSSEI
RPLCPMI  EQU   RPLSSEI
RPLPATHI EQU   RPLSSEI
RPLAARLN EQU   RPLVTEXT+4
RPLERMSA EQU   RPLCOMN+72
RPLEMLEN EQU   RPLCOMN+70
RPLACTIV EQU   RPLCOMN+69
RPLBRANC EQU   RPLEXTD1
RPLNIB   EQU   RPLEXTD1
@NM00006 EQU   RPLEXTD1
RPLEXIT  EQU   RPLEXTD1
RPLNEXIT EQU   RPLEXTD1
RPLEXSCH EQU   RPLEXTD1
RPLDDDD  EQU   RPLRBAR+4
@NM00005 EQU   RPLRBAR+3
@NM00004 EQU   RPLAIXID
RPLAXPKP EQU   RPLAIXID
RPLAIXPC EQU   RPLRBAR
RPLRSV88 EQU   RPLOPT8
RPLUNCON EQU   RPLOPT8
RPLACTV  EQU   RPLOPT8
RPLSESS  EQU   RPLOPT8
RPLPEND  EQU   RPLOPT8
RPLODPRM EQU   RPLOPT8
RPLODACP EQU   RPLOPT8
RPLODACQ EQU   RPLOPT8
RPLRSV78 EQU   RPLOPT7
RPLTCRNO EQU   RPLOPT7
RPLRLSOP EQU   RPLOPT7
RPLTPOST EQU   RPLOPT7
RPLQOPT  EQU   RPLOPT7
RPLCNIMM EQU   RPLCNOPT
RPLCNANY EQU   RPLCNOPT
RPLCNALL EQU   RPLCNOPT
RPLRSV68 EQU   RPLOPT6
RPLRSV67 EQU   RPLOPT6
RPLLOCK  EQU   RPLOPT6
RPLNCOND EQU   RPLOPT6
RPLCOND  EQU   RPLOPT6
RPLEOT   EQU   RPLUNTYP
RPLEOM   EQU   RPLUNTYP
RPLEOB   EQU   RPLUNTYP
RPLWROPT EQU   RPLOPT5
RPLNODE  EQU   RPLOPT5
RPLERACE EQU   RPLWRTYP
RPLNERAS EQU   RPLWRTYP
RPLPSOPT EQU   RPLOPT5
@NM00003 EQU   RPLOPT5
RPLDLGIN EQU   RPLOPT5
RPLCHAIN EQU   RPLNXTRP
RPLBOP   EQU   RPLOPT4
@NM00002 EQU   RPLOPT4
RPLMKFRM EQU   RPLOPT4
RPLENDTR EQU   RPLOPT4
RPLALIGN EQU   RPLOPT3
RPLFMT   EQU   RPLOPT3
RPLFLD   EQU   RPLOPT3
RPLVFY   EQU   RPLOPT3
RPLBLK   EQU   RPLOPT3
RPLSFORM EQU   RPLOPT3
RPLEODS  EQU   RPLOPT3
RPLNSP   EQU   RPLOPT2
RPLUPD   EQU   RPLOPT2
@NM00001 EQU   RPLOPT2
RPLLRD   EQU   RPLOPT2
RPLBWD   EQU   RPLOPT2
RPLCNV   EQU   RPLOPT2
RPLADD   EQU   RPLADR
RPLKEY   EQU   RPLOPT2
RPLECBIN EQU   RPLECBSW
RPLGEN   EQU   RPLOPT1
RPLKGE   EQU   RPLOPT1
RPLASY   EQU   RPLOPT1
RPLSKP   EQU   RPLOPT1
RPLSEQ   EQU   RPLOPT1
RPLDIR   EQU   RPLOPT1
RPLLOC   EQU   RPLOPT1
RPLDAF   EQU   RPLARG+2
RPLSAF   EQU   RPLARG
RPLTCBPT EQU   RPLCOMN+28
RPLDACB  EQU   RPLCOMN+24
RPLCCHAR EQU   RPLCOMN+20
RPLSTRID EQU   RPLCOMN+18
RPLKEYL  EQU   RPLKEYLE
RPLRDSOH EQU   RPLFDB3
RPLRLG   EQU   RPLFDB3
RPLLGFRC EQU   RPLFDB3
RPLREOT  EQU   RPLFDB3
RPLREOM  EQU   RPLFDB3
RPLREOB  EQU   RPLFDB3
RPLSV32  EQU   RPLFDB3
RPLUINPT EQU   RPLFDB3
RPLSTSAV EQU   RPLFDB2
RPLCUERR EQU   RPLFDB2
RPLDLGFL EQU   RPLFDB2
RPLIOERR EQU   RPLFDB2
RPLDVUNS EQU   RPLFDB2
RPLATND  EQU   RPLFDB2
RPLRVID  EQU   RPLFDB2
RPLERLK  EQU   RPLFDB2
RPLERREG EQU   RPLRTNCD
RPLSTAT  EQU   RPLFDBWD
RPLPOST  EQU   RPLECB
RPLWAIT  EQU   RPLECB
RPLPLHPT EQU   RPLCOMN+4
RPLLEN2  EQU   RPLLEN
RPLSTYP  EQU   RPLIDWD+1
RPLID    EQU   RPLIDWD
@NM00052 EQU   PSA+3668
PSASTAK  EQU   PSA+3072
@NM00051 EQU   PSA+1048
PSAUSEND EQU   PSA+1048
PSARV062 EQU   PSA+1044
PSACDAL  EQU   PSA+1040
PSAWTCOD EQU   PSA+1036
PSAATCVT EQU   PSA+1032
PSAPCPSW EQU   PSA+1024
PSAPIR2  EQU   PSA+1020
PSARV059 EQU   PSA+1018
PSASVC13 EQU   PSA+1016
PSALSFCC EQU   PSA+1012
PSASFACC EQU   PSA+1008
PSASTOP  EQU   PSA+992
PSASTART EQU   PSA+976
PSARSPSW EQU   PSA+968
PSASRPSW EQU   PSA+960
PSARV045 EQU   PSA+892
PSARV044 EQU   PSA+888
PSARV043 EQU   PSA+884
PSARV042 EQU   PSA+880
PSARV041 EQU   PSA+876
PSARV040 EQU   PSA+872
PSARV025 EQU   PSA+868
PSADSSED EQU   PSA+868
PSADSSPR EQU   PSA+864
PSADSSFW EQU   PSA+860
PSADSS14 EQU   PSA+856
PSADSSPP EQU   PSA+848
PSADSSRP EQU   PSA+840
PSADSS05 EQU   PSADSSF4
PSADSS10 EQU   PSADSSF4
PSADSSVE EQU   PSADSSF4
PSADSSDE EQU   PSADSSF4
PSADSSC0 EQU   PSADSSF4
PSADSSIE EQU   PSADSSF4
PSADSS12 EQU   PSADSSF4
PSADSSRC EQU   PSADSSF4
PSARV057 EQU   PSADSSF3
PSARV056 EQU   PSADSSF3
PSARV055 EQU   PSADSSF3
PSADSSMC EQU   PSADSSF3
PSADSSRW EQU   PSADSSF3
PSADSSNM EQU   PSADSSF3
PSADSSES EQU   PSADSSF3
PSADSSGP EQU   PSADSSF3
PSADSSF2 EQU   PSADSSFL+1
PSADSSPI EQU   PSADSSF1
PSADSSOI EQU   PSADSSF1
PSADSSSP EQU   PSADSSF1
PSADSSTP EQU   PSADSSF1
PSADSSDW EQU   PSADSSF1
PSADSSDD EQU   PSADSSF1
PSADSSDM EQU   PSADSSF1
PSADSSMV EQU   PSADSSF1
PSADSSTS EQU   PSA+816
PSADSSWK EQU   PSA+812
PSADSSR3 EQU   PSA+808
PSADSSR2 EQU   PSA+804
PSADSSRS EQU   PSA+800
PSASTOR  EQU   PSA+796
PSACPUSA EQU   PSA+794
PSAVSTAP EQU   PSA+792
PSAWKVAP EQU   PSA+788
PSAWKRAP EQU   PSA+784
PSAMCHIC EQU   PSA+783
PSARV061 EQU   PSA+782
PSASYMSK EQU   PSA+781
PSAMCHFL EQU   PSA+780
PSACR0   EQU   PSA+776
PSAPSWSV EQU   PSA+768
PSACLHS  EQU   PSAHLHI
PSALKR15 EQU   PSALKSA+60
PSALKR14 EQU   PSALKSA+56
PSALKR13 EQU   PSALKSA+52
PSALKR12 EQU   PSALKSA+48
PSALKR11 EQU   PSALKSA+44
PSALKR10 EQU   PSALKSA+40
PSALKR9  EQU   PSALKSA+36
PSALKR8  EQU   PSALKSA+32
PSALKR7  EQU   PSALKSA+28
PSALKR6  EQU   PSALKSA+24
PSALKR5  EQU   PSALKSA+20
PSALKR4  EQU   PSALKSA+16
PSALKR3  EQU   PSALKSA+12
PSALKR2  EQU   PSALKSA+8
PSALKR1  EQU   PSALKSA+4
PSALKR0  EQU   PSALKSA
PSARV023 EQU   PSACLHT+52
PSALOCAL EQU   PSACLHT+48
PSACMSL  EQU   PSACLHT+44
PSAOPTL  EQU   PSACLHT+40
PSATPACL EQU   PSACLHT+36
PSATPDNL EQU   PSACLHT+32
PSATPNCL EQU   PSACLHT+28
PSAIOSLL EQU   PSACLHT+24
PSAIOSUL EQU   PSACLHT+20
PSAIOSCL EQU   PSACLHT+16
PSAIOSSL EQU   PSACLHT+12
PSASALCL EQU   PSACLHT+8
PSAASML  EQU   PSACLHT+4
PSADISPL EQU   PSACLHT
PSASRSA  EQU   PSA+636
PSARV050 EQU   PSA+634
PSADSSGO EQU   PSA+633
PSARECUR EQU   PSA+632
PSAHLHIS EQU   PSA+628
PSAIPCSA EQU   PSA+624
@NM00050 EQU   PSA+621
PSAIPCDM EQU   PSA+620
PSAIPCD  EQU   PSA+616
@NM00049 EQU   PSA+613
PSAIPCRM EQU   PSA+612
PSAIPCR  EQU   PSA+608
PSAMCHEX EQU   PSA+600
PSAMPSW  EQU   PSA+592
PSAEXPS2 EQU   PSA+584
PSAEXPS1 EQU   PSA+576
PSAPIREG EQU   PSA+572
PSARSREG EQU   PSA+568
PSAGPREG EQU   PSA+556
PSARV022 EQU   PSASUP4
PSARV021 EQU   PSASUP4
PSARV020 EQU   PSASUP4
PSARV019 EQU   PSASUP4
PSARV018 EQU   PSASUP4
PSARV017 EQU   PSASUP4
PSARV016 EQU   PSASUP4
PSARV015 EQU   PSASUP4
PSARV014 EQU   PSASUP3
PSARV013 EQU   PSASUP3
PSARV012 EQU   PSASUP3
PSARV011 EQU   PSASUP3
PSASPR   EQU   PSASUP3
PSAPSREG EQU   PSASUP3
PSAPI2   EQU   PSASUP3
PSAIOSUP EQU   PSASUP3
PSALCR   EQU   PSASUP2
PSARTM   EQU   PSASUP2
PSAACR   EQU   PSASUP2
PSAIPCE2 EQU   PSASUP2
PSAIPCES EQU   PSASUP2
PSAIPCEC EQU   PSASUP2
PSAGTF   EQU   PSASUP2
PSAIPCRI EQU   PSASUP2
PSARV063 EQU   PSASUP1
PSATCTL  EQU   PSASUP1
PSADISP  EQU   PSASUP1
PSALOCK  EQU   PSASUP1
PSAPI    EQU   PSASUP1
PSAEXT   EQU   PSASUP1
PSASVC   EQU   PSASUP1
PSAIO    EQU   PSASUP1
PSAAOLD  EQU   PSA+548
PSAANEW  EQU   PSA+544
PSATOLD  EQU   PSA+540
PSATNEW  EQU   PSA+536
PSALCCAR EQU   PSA+532
PSALCCAV EQU   PSA+528
PSAPCCAR EQU   PSA+524
PSAPCCAV EQU   PSA+520
PSACPULA EQU   PSA+518
PSACPUPA EQU   PSA+516
PSAPSA   EQU   PSA+512
FLCHDEND EQU   PSA+512
FLCCRSAV EQU   FLCMCLA+280
FLCGRSAV EQU   FLCMCLA+216
FLCFPSAV EQU   FLCMCLA+184
FLCFLA   EQU   FLCMCLA+88
FLCRGNCD EQU   FLCMCLA+84
FLCFSAA  EQU   FLCFSA+1
@NM00048 EQU   FLCFSA
@NM00047 EQU   FLCMCLA+72
FLCMCIC  EQU   FLCMCLA+64
@NM00046 EQU   FLCMCLA+20
FLCIOAA  EQU   FLCIOA+1
@NM00045 EQU   FLCIOA
@NM00044 EQU   FLCMCLA+15
@NM00043 EQU   FLCMCLA+14
@NM00042 EQU   FLCMCLA+12
FLCLCL   EQU   FLCMCLA+8
FLCIOELA EQU   FLCIOEL+1
@NM00041 EQU   FLCIOEL
FLCCHNID EQU   FLCMCLA
@NM00040 EQU   PSA+160
FLCMTRCD EQU   PSA+157
@NM00039 EQU   PSA+156
FLCPERA  EQU   FLCPER+1
@NM00038 EQU   FLCPER
@NM00037 EQU   PSA+151
FLCPERCD EQU   PSA+150
FLCMCNUM EQU   PSA+149
@NM00036 EQU   PSA+148
FLCTEAA  EQU   FLCTEA+1
@NM00035 EQU   FLCTEA
PSAPIPC  EQU   PSAPICOD
PSAPIMC  EQU   PSAPICOD
PSAPIPER EQU   PSAPICOD
PSARV049 EQU   FLCPICOD
FLCPILCB EQU   FLCPIILC
@NM00034 EQU   FLCPIILC
@NM00033 EQU   PSAEPPSW
FLCSVCN  EQU   PSAESPSW+2
FLCSILCB EQU   FLCSVILC
@NM00032 EQU   FLCSVILC
@NM00031 EQU   PSAESPSW
FLCEICOD EQU   PSAEEPSW+2
PSASPAD  EQU   PSAEEPSW
@NM00030 EQU   PSA+128
FLCINPSW EQU   PSA+120
FLCMNPSW EQU   PSA+112
FLCPNPSW EQU   PSA+104
FLCSNPSW EQU   PSA+96
FLCENPSW EQU   PSA+88
FLCTRACE EQU   PSA+84
FLCTIMER EQU   PSA+80
FLCCVT2  EQU   PSA+76
FLCCAW   EQU   PSA+72
FLCCSW   EQU   PSA+64
FLCIOPSW EQU   PSA+56
FLCMOPSW EQU   PSA+48
FLCPOPSW EQU   PSA+40
FLCSOPSW EQU   PSA+32
FLCEOPSW EQU   PSA+24
@NM00029 EQU   FLCICCW2+4
FLCCVT   EQU   FLCICCW2
FLCICCW1 EQU   FLCROPSW
FLCIPPSW EQU   FLCRNPSW
.@UNREFD ANOP                          END UNREFERENCED COMPONENTS
@RC00530 EQU   @RC00508
@RT00533 EQU   @RC00508
@RT00611 EQU   @RC00610
@RT00730 EQU   @RC00687
@RC00744 EQU   @RC00743
@RC00676 EQU   @RC00674
@RC00752 EQU   @RC00674
@RC00774 EQU   @RC00772
@RC00786 EQU   @RC00772
@RC00938 EQU   @RC00557
@RC00678 EQU   @RC00676
@RC00747 EQU   @RC00744
@RC00745 EQU   @RC00744
@RT00787 EQU   @RC00786
@ENDDATA EQU   *
         END   ISTRACTB,(C'PLS1842',0603,75275)
